using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.errordisplay;
using com.plumtree.uiinfrastructure.form;
namespace com.plumtree.portalpages.common.helpers
{
  /// <author>
  /// adolfoc
  /// This class is for use in collecting and validating information sent to models.
  /// Although designed for Editor models, most of these methods will work with any model.
  /// </author>
  public class ModelHelper
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_COMMON),
     "com.plumtree.portalpages.common.helpers.ModelHelper");
    
    /// <summary>
    /// Gets a boolean Form Element from a hashtable of form data - returns the
    /// default value if the form element was not found or is empty.
    /// NOTE: This assumes that booleans are defined in forms as follows.
    /// "0" == false
    /// Any non-zero integer (e.g. "1") == true
    /// </summary>
    /// <param name="_htFormData"></param>
    /// <param name="strFormElement"></param>
    /// <param name="bDefault">TODO bug 45991: Remove this unused parameter, since a default of "true" makes no sense for a checkbox. If the checkbox value is absent from the form post, it was not checked.</param>
    /// <returns>boolean</returns>
    public static bool GetFormElementBool(XPHashtable _htFormData, String strFormElement,
     bool bDefault)
    {
      String[] data = (String[]) _htFormData.GetElement(strFormElement);
      // Absence of a checkbox form element means it's not checked
      if (data == null || IsBlankStringArray(data))
      {
        return false;
      }
      int elementValue = 0;
      try
      {
        elementValue = XPConvert.ToInteger(data[0]);
      }
      catch (Exception e)
      {
        log.Debug(e, "ModelHelper::GetFormElementInt: Unknown error while retrieving form element (" + strFormElement + "). ");
        }
      // Non-zero value indicates the checkbox was checked.
      return elementValue != 0;
    }
    
    /// <summary>
    /// Gets a boolean Form Element from a hashtable of form data - returns the
    /// default value if the form element was not found or is empty. NOTE: Unlike
    /// the other GetForm... methods defined in this class, the default value
    /// cannot be null. ptdDefault is used to retrieve the default MASK value
    /// (among other things).
    /// </summary>
    /// <param name="_htFormData"></param>
    /// <param name="strFormElementDurationInput"></param>
    /// <param name="strFormElementDurationUnitsSelect"></param>
    /// <param name="ptdDefault"></param>
    /// <returns>PTDuration</returns>
    public static PTDuration GetFormElementDuration(XPHashtable _htFormData,
     String strFormElementDurationInput, String strFormElementDurationUnitsSelect,
     PTDuration ptdDefault)
    {
      // AGC- If we get one of the following two form elements and not the other, strange behavior will result. However,
      // this is unlikely since both of these elements are part of the same htmlconstruct.
      long lTimeout = ModelHelper.GetFormElementLong(_htFormData, strFormElementDurationInput,
       -1L);
      int nTimeoutUnits = ModelHelper.GetFormElementInt(_htFormData, strFormElementDurationUnitsSelect,
       -1);
      if ((-1L == lTimeout) || (-1 == nTimeoutUnits))
      {
        log.Warn("ModelHelper::GetFormElementDuration: Null or empty form element found while retrieving form elements for Duration. Returning Default Duration.");
        return ptdDefault;
      }
      PTDuration ptdResult = new PTDuration(lTimeout, nTimeoutUnits, ptdDefault.GetSupportedUnitsMask());
      return ptdResult;
    }
    
    /// <summary>
    /// Checks to see if the string array passed in is null or
    /// of zero length
    /// </summary>
    /// <param name="data"></param>
    /// <returns></returns>
    public static bool IsEmptyStringArray(String[] arsData)
    {
      if (null == arsData)
      {
        return true;
      }
      if (arsData.Length <= 0)
      {
        return true;
      }
      return false;
    }
    
    /// <summary>
    /// Checks to see if the string array passed in is null or
    /// of zero length OR the first element is a blank string.
    /// </summary>
    /// <param name="data"></param>
    /// <returns></returns>
    public static bool IsBlankStringArray(String[] arsData)
    {
      if (IsEmptyStringArray(arsData))
      {
        return true;
      }
      if (arsData[0].Length <= 0)
      {
        return true;
      }
      return false;
    }
    
    /// <summary>
    /// Checks to see if the string array passed in is null or
    /// of zero length
    /// </summary>
    /// <param name="data"></param>
    /// <returns></returns>
    public static bool IsBlankString(String sData)
    {
      if (null == sData)
      {
        return true;
      }
      if (sData.Length <= 0)
      {
        return true;
      }
      return false;
    }
    
    /// <summary>
    /// Gets an integer Form Element from a hashtable of form data - returns the
    /// default value if the form element was not found or is empty.
    /// </summary>
    /// <param name="_htFormData"></param>
    /// <param name="strFormElement"></param>
    /// <param name="nDefault"></param>
    /// <returns>String</returns>
    public static int GetFormElementInt(XPHashtable _htFormData, String strFormElement,
     int nDefault)
    {
      String[] data = (String[]) _htFormData.GetElement(strFormElement);
      if (!IsBlankStringArray(data))
      {
        try
        {
          return XPConvert.ToInteger(data[0]);
        }
        catch (Exception e)
        {
          log.Warn(e, "ModelHelper::GetFormElementInt: Unknown error while retrieving form element (" + strFormElement + "). ");
          return nDefault;
        }
      } else
      {
        log.Warn("ModelHelper::GetFormElementInt: Null or empty form element found while retrieving form element (" + strFormElement + "). ");
        return nDefault;
      }
    }
    
    /// <summary>
    /// Gets a long Form Element from a hashtable of form data - returns the
    /// default value if the form element was not found or is empty.
    /// </summary>
    /// <param name="_htFormData"></param>
    /// <param name="strFormElement"></param>
    /// <param name="lDefault"></param>
    /// <returns>String</returns>
    public static long GetFormElementLong(XPHashtable _htFormData, String strFormElement,
     long lDefault)
    {
      String[] data = (String[]) _htFormData.GetElement(strFormElement);
      if (!IsBlankStringArray(data))
      {
        try
        {
          return XPConvert.ToLong(data[0]);
        }
        catch (Exception e)
        {
          log.Warn(e, "ModelHelper::GetFormElementLong: Unknown error while retrieving form element (" + strFormElement + "). ");
          return lDefault;
        }
      } else
      {
        log.Warn("ModelHelper::GetFormElementLong: Null or empty form element found while retrieving form element (" + strFormElement + "). ");
        return lDefault;
      }
    }
    
    /// <summary>
    /// Gets a string Form Element from a hashtable of form data - returns the
    /// default value if the form element was not found or is empty.
    /// </summary>
    /// <param name="_htFormData"></param>
    /// <param name="strFormElement"></param>
    /// <param name="strDefault"></param>
    /// <returns>String</returns>
    public static String GetFormElementString(XPHashtable _htFormData, String strFormElement,
     String strDefault)
    {
      String[] data = (String[]) _htFormData.GetElement(strFormElement);
      if (!IsEmptyStringArray(data))
      {
        return data[0];
      } else
      {
        log.Warn("ModelHelper::GetFormElementString: Null or empty form element found while retrieving form element (" + strFormElement + "). ");
        return strDefault;
      }
    }
    
    /// <summary>
    /// Translates an Object array into an XPArrayList.
    /// Returns an empty XPArrayList if arObj is null.
    /// </summary>
    /// <param name="arObj"></param>
    /// <returns>XPArrayList</returns>
    public static XPArrayList ObjectArrayToXPArray(Object[] arObj)
    {
      XPArrayList xpalResult = new XPArrayList();
      if (null != arObj)
      {
        for (int nIndex = 0;nIndex < arObj.Length;nIndex++)
        {
          xpalResult.Add(arObj[nIndex]);
        }
      }
      return xpalResult;
    }
    
    /// <summary>
    /// Returns the value of objValue or bDefault if objValue is null or
    /// the conversion fails.
    /// </summary>
    public static bool ObjectToBool(Object objValue, bool bDefault)
    {
      if (null == objValue)
      {
        return bDefault;
      }
      int nValue = bDefault ? 1 : 0;
      try
      {
        nValue = XPConvert.ToInteger(objValue);
      }
      catch (Exception e)
      {
        log.Debug(e, "ModelHelper::ObjectToBool: Error - Returning Default value.");
        nValue = bDefault ? 1 : 0;
      }
      if (0 == nValue)
      {
        return false;
      } else
      {
        return true;
      }
    }
    
    /// <summary>
    /// Returns the value of objValue or nDefault if objValue is null or
    /// the conversion fails.
    /// </summary>
    public static int ObjectToInt(Object objValue, int nDefault)
    {
      if (null == objValue)
      {
        return nDefault;
      }
      int nValue = nDefault;
      try
      {
        nValue = XPConvert.ToInteger(objValue);
      }
      catch (Exception e)
      {
        log.Debug(e, "ModelHelper::ObjectToInt: Error - Returning Default value.");
        nValue = nDefault;
      }
      return nValue;
    }
    
    /// <summary>
    /// Reads a boolean value from a property bag. Returns bDefault if the
    /// value was not found or an error occurred.
    /// </summary>
    /// <param name="sPropID"></param>
    /// <param name="ptPropBag"></param>
    /// <param name="bDefault"></param>
    /// <returns>boolean</returns>
    public static bool PBReadBool(IXPPropertyBag ptPropBag, String sPropID,
     bool bDefault)
    {
      try
      {
        return ObjectToBool(ptPropBag.ReadAsObject(sPropID), bDefault);
      }
      catch (Exception e)
      {
        log.Warn(e, "ModelHelper::PBReadBool: Unable to Read Bool from property bag, returning default (" + sPropID + ").");
        return bDefault;
      }
    }
    
    /// <summary>
    /// Reads an int value from a property bag. Returns nDefault if the value
    /// was not found or an error occurred.
    /// </summary>
    /// <param name="sPropID"></param>
    /// <param name="ptPropBag"></param>
    /// <param name="nDefault"></param>
    /// <returns>int</returns>
    public static int PBReadInt(IXPPropertyBag ptPropBag, String sPropID,
     int nDefault)
    {
      try
      {
        return ObjectToInt(ptPropBag.ReadAsObject(sPropID), nDefault);
      }
      catch (Exception e)
      {
        log.Warn(e, "ModelHelper::PBReadInt: Unable to Read Int from property bag, returning default (" + sPropID + ").");
        return nDefault;
      }
    }
    
    /// <summary>
    /// Reads a String value from a property bag. Returns strDefault if the value
    /// was not found or an error occurred.
    /// </summary>
    /// <param name="sPropID"></param>
    /// <param name="ptPropBag"></param>
    /// <param name="strDefault"></param>
    /// <returns>String</returns>
    public static String PBReadString(IXPPropertyBag ptPropBag, String sPropID,
     String strDefault)
    {
      String strReturn = strDefault;
      try
      {
        strReturn = ptPropBag.ReadAsString(sPropID);
        if (null == strReturn)
        {
          return strDefault;
        }
        return strReturn;
      }
      catch (Exception e)
      {
        log.Warn(e, "ModelHelper::PBReadString: Unable to Read String from property bag, returning default (" + sPropID + ").");
        return strDefault;
      }
    }
    
    /// <summary>
    /// Writes a value to a property bag.
    /// </summary>
    /// <param name="sPropID"></param>
    /// <param name="ptPropBag"></param>
    /// <param name="strValue"></param>
    public static void PBWrite(IXPPropertyBag ptPropBag, String sPropID, String strValue)
    {
      try
      {
        ptPropBag.Write(sPropID, strValue);
      }
      catch (Exception e)
      {
        log.Warn(e, "WSModeHelper::PBWrite (String): Unknown Error storing property bag value: (" + sPropID + ")(" + strValue + ")");
        }
    }
    
    /// <summary>
    /// Writes a value to a property bag.
    /// </summary>
    /// <param name="sPropID"></param>
    /// <param name="ptPropBag"></param>
    /// <param name="nValue"></param>
    public static void PBWrite(IXPPropertyBag ptPropBag, String sPropID, int nValue)
    {
      try
      {
        ptPropBag.Write(sPropID, nValue);
      }
      catch (Exception e)
      {
        log.Warn(e, "WSModeHelper::PBWrite (Int): Unknown Error storing property bag value: (" + sPropID + ")(" + XPConvert.ToString(nValue) + ")");
        }
    }
    
    /// <summary>
    /// Writes a value to a property bag.
    /// </summary>
    /// <param name="sPropID"></param>
    /// <param name="ptPropBag"></param>
    /// <param name="lValue"></param>
    public static void PBWrite(IXPPropertyBag ptPropBag, String sPropID, long lValue)
    {
      try
      {
        ptPropBag.Write(sPropID, (int) lValue);
      }
      catch (Exception e)
      {
        log.Warn(e, "WSModeHelper::PBWrite (Long): Unknown Error storing property bag value: (" + sPropID + ")(" + XPConvert.ToString(lValue) + ")");
        }
    }
    
    /// <summary>
    /// Writes a value to a property bag.
    /// </summary>
    /// <param name="sPropID"></param>
    /// <param name="ptPropBag"></param>
    /// <param name="bValue"></param>
    public static void PBWrite(IXPPropertyBag ptPropBag, String sPropID, bool bValue)
    {
      if (bValue)
      {
        PBWrite(ptPropBag, sPropID, 1);
      } else
      {
        PBWrite(ptPropBag, sPropID, 0);
      }
    }
    
    /// <summary>
    /// Writes a value to a property bag.
    /// </summary>
    /// <param name="sPropID"></param>
    /// <param name="ptPropBag"></param>
    /// <param name="bValue"></param>
    public static void PBWrite(IXPPropertyBag ptPropBag, String sPropID, XPArrayList xpalValue)
    {
      try
      {
        ptPropBag.Write(sPropID, xpalValue.ToArray());
      }
      catch (Exception e)
      {
        log.Warn(e, "WSModeHelper::PBWrite (XPArrayList): Unknown Error storing property bag value: (" + sPropID + ")");
        }
    }
    
    /// <summary>
    /// Validates that a specified Duration falls within the specified duration
    /// range (specified in milliseconds).
    /// </summary>
    /// <param name="ptdIn"></param>
    /// <param name="asOwner"></param>
    /// <param name="strError"></param>
    /// <param name="nLastError"></param>
    /// <param name="lMinMilli"></param>
    /// <param name="lMaxMilli"></param>
    /// <returns>int</returns>
    public static int ValidateDurationRange(PTDuration ptdIn, AActivitySpace asOwner,
     String strError, int nLastError, long lMinMilli, long lMaxMilli)
    {
      long lMilliIn;
      bool bInvalidDuration = false;
      if (ptdIn == null)
      {
        bInvalidDuration = true;
        lMilliIn = -1L;
      } else
      {
        lMilliIn = ptdIn.GetMilliseconds();
      }
      if (!bInvalidDuration && (lMilliIn >= lMinMilli) && (lMilliIn <= lMaxMilli))
      {
        return nLastError;
      } else
      {
        log.Warn("ModelHelper::ValidateDurationRange: Input out of range. (" + XPConvert.ToString(lMinMilli) + ">" + XPConvert.ToString(lMilliIn) + ">" + XPConvert.ToString(lMaxMilli));
        UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,
         strError, strError);
        UIErrorHandler.RaiseError(myError, asOwner.GetSubSession(), asOwner.GetApplication());
        return RepostControl.PAGE_STATUS_INVALID;
      }
    }
    
    /// <summary>
    /// Validates an arbitrary boolean. If bError is true it errors, else
    /// it does not.
    /// </summary>
    /// <param name="ptdIn"></param>
    /// <param name="asOwner"></param>
    /// <param name="strError"></param>
    /// <param name="nLastError"></param>
    /// <param name="lMinMilli"></param>
    /// <param name="lMaxMilli"></param>
    /// <returns>int</returns>
    public static int ValidateBool(bool bValid, AActivitySpace asOwner, String strError,
     int nLastError)
    {
      if (bValid)
      {
        return nLastError;
      } else
      {
        log.Warn("ModelHelper::ValidateBool: Validate Failed.");
        UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,
         strError, strError);
        UIErrorHandler.RaiseError(myError, asOwner.GetSubSession(), asOwner.GetApplication());
        return RepostControl.PAGE_STATUS_INVALID;
      }
    }
    
    /// <summary>
    /// Validates that an integer is within the specified range (nMin to nMax).
    /// Raises a UIError with descriptiong strError if the integer is invalid
    /// (and returns PAGE_STATUS_INVALID). Returns nLastError otherwise.
    /// Note: If nMax is less than nMin, this function only validates that
    /// nIn is greater than nMin (nMax is ignored).
    /// Note: If nMax == nMin, then this function validates that nIn == nMax ==
    /// nMin.
    /// </summary>
    /// <param name="nIn"></param>
    /// <param name="asOwner"></param>
    /// <param name="strError"></param>
    /// <param name="nLastError"></param>
    /// <param name="nMin"></param>
    /// <param name="nMax"></param>
    /// <returns>int</returns>
    public static int ValidateIntegerRange(int nIn, AActivitySpace asOwner,
     String strError, int nLastError, int nMin, int nMax)
    {
      if (nMin <= nMax)
      {
        if ((nIn >= nMin) && (nIn <= nMax))
        {
          return nLastError;
        }
      } else
      {
        if (nIn >= nMin)
        {
          return nLastError;
        }
      }
      // AGC- If we haven't returned by now, there's an error.
      log.Warn("ModelHelper::ValidateIntegerRange: Input out of range. (" + XPConvert.ToString(nMin) + ">" + XPConvert.ToString(nIn) + ">" + XPConvert.ToString(nMax));
      UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,
       strError, strError);
      UIErrorHandler.RaiseError(myError, asOwner.GetSubSession(), asOwner.GetApplication());
      return RepostControl.PAGE_STATUS_INVALID;
    }
    
    /// <summary>
    /// Checks that a string starts with http:// or https://.
    /// </summary>
    /// <param name="strIn"></param>
    /// <returns>boolean</returns>
    public static bool CheckIsURLString(String strIn)
    {
      String strToCheck = XPStringUtility.Trim(XPStringUtility.ToLowerCase(strIn));
      strToCheck = TrimAllTrailingSlashes(strToCheck);
      if ((null != strToCheck) && !(0 == strToCheck.Length))
      {
        if ((XPStringUtility.StartsWith(strToCheck, "http://")) || (XPStringUtility.StartsWith(strToCheck,
         "https://")))
        {
          return true;
        }
      }
      return false;
    }
    
    /// <summary>
    /// Validates that a string starts with http:// or https://. Raises a UIError
    /// with descriptiong strError if the string is invalid (and returns
    /// PAGE_STATUS_INVALID). Returns nLastError otherwise.
    /// </summary>
    /// <param name="strIn"></param>
    /// <param name="asOwner"></param>
    /// <param name="strError"></param>
    /// <param name="nLastError"></param>
    /// <returns>int</returns>
    public static int ValidateIsURLString(String strIn, AActivitySpace asOwner,
     String strError, int nLastError)
    {
      if (CheckIsURLString(strIn))
      {
        return nLastError;
      }
      log.Warn("ModelHelper::ValidateIsURLString: non-URL string found. ");
      UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,
       strError, strError);
      UIErrorHandler.RaiseError(myError, asOwner.GetSubSession(), asOwner.GetApplication());
      return RepostControl.PAGE_STATUS_INVALID;
    }
    public static bool HasTrailingSlash(String strIn)
    {
      if ((XPStringUtility.EndsWith(strIn, "\\")) || (XPStringUtility.EndsWith(strIn,
       "/")))
      {
        return true;
      } else
      {
        return false;
      }
    }
    public static String TrimTrailingSlash(String strIn)
    {
      String strResult = strIn;
      if (HasTrailingSlash(strResult))
      {
        strResult = XPStringUtility.Substring(strResult, 0, strResult.Length - 1);
        }
      return strResult;
    }
    public static String TrimAllTrailingSlashes(String strIn)
    {
      String strResult = strIn;
      // nMaxCount is just a safety variable. It doesn't strictly need to be here, but helps
      // just in case XPStringUtility.Substring is broken or misused (I have trust issues OK?).
      int nMaxCount = strIn.Length;
      while ((strResult.Length > 0) && HasTrailingSlash(strResult) && (nMaxCount > 0))
      {
        strResult = XPStringUtility.Substring(strResult, 0, strResult.Length - 1);
        nMaxCount--;
      }
      return strResult;
    }
    
    /// <summary>
    /// Validates that a string is non-empty. Raises a UIError with descriptiong
    /// strError if the string is empty or null (and returns
    /// PAGE_STATUS_INVALID). Returns nLastError otherwise.
    /// </summary>
    /// <param name="strIn"></param>
    /// <param name="asOwner"></param>
    /// <param name="strError"></param>
    /// <param name="nLastError"></param>
    /// <returns>int</returns>
    public static int ValidateNonEmptyString(String strIn, AActivitySpace asOwner,
     String strError, int nLastError)
    {
      if ((null != strIn) && !(0 == strIn.Length))
      {
        return nLastError;
      } else
      {
        log.Warn("ModelHelper::ValidateNotEmptyString: Null or empty string found. ");
        UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,
         strError, strError);
        UIErrorHandler.RaiseError(myError, asOwner.GetSubSession(), asOwner.GetApplication());
        return RepostControl.PAGE_STATUS_INVALID;
      }
    }
    
    /// <summary>
    /// Validates that both a login and password were set.
    /// </summary>
    /// <param name="strLogin"></param>
    /// <param name="strPassword"></param>
    /// <param name="strNULLPASSWORD"></param>
    /// <param name="asOwner"></param>
    /// <param name="strError"></param>
    /// <param name="nLastError"></param>
    /// <returns>int</returns>
    public static int ValidateLoginPassword(String strLogin, String strPassword,
     String strNULLPASSWORD, AActivitySpace asOwner, String strError, int nLastError)
    {
      // AGC- Right now, the Password is not checked, but it may be in the future, if we decide to force
      // the password to be reset every time the editor is accessed.
      if (0 >= XPStringUtility.Trim(strLogin).Length)
      {
        log.Warn("ModelHelper::ValidateNotEmptyString: Null or empty string found. ");
        UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,
         strError, strError);
        UIErrorHandler.RaiseError(myError, asOwner.GetSubSession(), asOwner.GetApplication());
        return RepostControl.PAGE_STATUS_INVALID;
      } else
      {
        return nLastError;
      }
    }
    public static XPArrayList StringArrayToXPArray(String[] arsIn)
    {
      XPArrayList xpalResult = new XPArrayList();
      if (null != arsIn)
      {
        for (int nIndex = 0;nIndex < arsIn.Length;nIndex++)
        {
          xpalResult.Add(arsIn[nIndex]);
        }
      }
      return xpalResult;
    }
  }
}