using System;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.login;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.server.portalpagesperf;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.login;
using com.plumtree.uiinfrastructure.pei;
using com.plumtree.uiinfrastructure.web;
//Put explicit import for Interpreter (to avoid ambiguity of using the deprecated class)
using Interpreter=com.plumtree.uiinfrastructure.interpreter.Interpreter;
namespace com.plumtree.portalpages.pei
{
  /// <author>
  /// Ben Geislinger
  /// This class is automatically called during logins and caches
  /// the user's personal settings from the SessionInfo object.
  /// </author>
  public class PTLoginActions : ILoginActions
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_COMMON),
     "com.plumtree.portalpages.pei.PTLoginActions");
    
    /// <summary>
    /// This function will take a given userSession and grab all of the users
    /// personal settings and take care of caching them.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.login.ILoginActions#OnAfterLogin(Object,ApplicationData)"></seealso>
    public virtual Redirect OnAfterLogin(Object _oUserSession, ApplicationData _appData)
    {
      IPTSession ptSession = (IPTSession) _oUserSession;
      IApplication application = _appData.GetApplication();
      ISessionManager sessionManager = _appData.GetSubSession();
      ISessionManager persistentSessionManager = _appData.GetPersistentSubSession();
      XPResourceManager rm = (XPResourceManager) application.GetCachingManager().GetEntry("ResourceManager");
      // This will cache the users settings on the http session for us.  The
      // locale may come back as UseBrowser, which requires us to determine
      // the proper locale from the browser settings.
      PTPersonalSettingsHelper.CachePersonalSettings(sessionManager, application,
       ptSession, rm, _appData);
      // Check if the default user locale string is UseBrowser
      if (0 == XPStringUtility.CompareIgnoreCase((String) PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.LOCALE,
       sessionManager, application, ptSession), PTPersonalSettingsHelper.LOCALE_USE_BROWSER_SETTING))
      {
        // We need to set the language setting from the request header.
        // This needs to be done after CachePersonalSettings, otherwise we
        // will always get the default language.  CachePersonalSettings can
        // return UserBrowser either because the user has explicitly chosen
        // it, or if the user did not choose anything.
        SetBrowserLocaleSetting(_appData, ptSession, sessionManager, persistentSessionManager,
         application, rm);
      }
      if (!LoginHelper.INSTANCE.GetIsGuestUser(ptSession))
      {
        // Only track non-guest logins.
        PerfCounters.INSTANCE.IncrementCounter(PORTALPAGESPERF_COUNTERS.PORTALPAGESPERF_LOGINS_SUCCESSFUL);
        }
      // JF- Set the content-language in the header so it can be used by braille devices
      String sContentLanguageHeader = (String) PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.LOCALE,
       sessionManager, application, ptSession);
      _appData.SetContentLanguage(sContentLanguageHeader);
      // JF- Also set the content language in the Session so it can be set again 
      // in the header next request by the interpreter.
      sessionManager.SetAttribute(Interpreter.USERLOCALE, sContentLanguageHeader);
      return null;
    }
    public virtual Redirect OnBeforeLogin(ApplicationData _appData, String _strUserName)
    {
      // Note that the user name will be "Guest" for the guest user and blank
      // for auto-login users.
      // This method cannot redirect internally for the guest user, only for
      // users that have already been logged in as guest.
      return null;
    }
    public virtual Redirect OnBeforeLogout(Object _oUserSession, ApplicationData _appData)
    {
      // TODO: This should get automatically cleared now
      // mj: clear the SSOInfo object off of the session
      ISessionManager sessionManager = _appData.GetSubSession();
      sessionManager.RemoveAttribute(PlumtreeExtensibility.PT_SSOINFO_ON_STATE);
      return null;
    }
    public virtual String OnFailedLogin(ApplicationData _appData, String _strUserName,
     XPException _xpError)
    {
      PerfCounters.INSTANCE.IncrementCounter(PORTALPAGESPERF_COUNTERS.PORTALPAGESPERF_LOGINS_FAILURE);
      return "";
    }
    
    /// <summary>
    /// This method gets a supported locale from the browser settings, and
    /// sets it on the http session and as a runtime setting.
    /// </summary>
    /// <param name="_appData"></param>
    /// <param name="_oUserSession"></param>
    /// <param name="_sessionManager"></param>
    /// <param name="_application"></param>
    /// <param name="_rm"></param>
    protected virtual void SetBrowserLocaleSetting(ApplicationData _appData,
     IPTSession _ptSession, ISessionManager _sessionManager, ISessionManager _persistentSessionManager,
     IApplication _application, XPResourceManager _rm)
    {
      String sBrowserLocaleList = _appData.GetHeader("accept-language");
      String strLocale = InternalGetBrowserLocale(sBrowserLocaleList, _application,
       _rm);
      if (strLocale != null)
      {
        log.Info("Setting user language based on supported locale of " + strLocale + " from accept-language header " + sBrowserLocaleList);
        InternalSetBrowserLocaleSetting(strLocale, _ptSession, _sessionManager,
         _persistentSessionManager, _application, _rm);
      }
    }
    
    /// <summary>
    /// This method gets a supported locale from the browser settings, but does
    /// not set it anywhere.
    /// </summary>
    /// <param name="_asOwner"></param>
    /// <returns>String The browser locale.</returns>
    public static String GetBrowserLocaleSetting(AActivitySpace _asOwner)
    {
      String sBrowserLocaleList = _asOwner.GetCurrentHTTPRequest().GetHeader("accept-language");
      return InternalGetBrowserLocale(sBrowserLocaleList, _asOwner.GetApplication(),
       _asOwner.GetResourceManager());
    }
    
    /// <summary>
    /// This method gets a supported locale from the browser settings, and
    /// sets it on the http session and as a runtime setting.
    /// </summary>
    /// <param name="_appData"></param>
    /// <param name="_oUserSession"></param>
    /// <param name="_sessionManager"></param>
    /// <param name="_application"></param>
    /// <param name="_rm"></param>
    protected static String InternalGetBrowserLocale(String sBrowserLocaleList,
     IApplication _application, XPResourceManager _rm)
    {
      String sBrowserLocale = null;
      if ((sBrowserLocaleList != null) && (0 != sBrowserLocaleList.Length))
      {
        try
        {
          // This method maps the accept header to a supported culture non-neutral locale
          // for us (non-neutral for .NET data formatting).
          sBrowserLocale = _rm.MapLocaleToUISupportedLocaleID(sBrowserLocaleList,
           PTPersonalSettingsHelper.GetDefaultLocale(_application));
        }
        catch (Exception e)
        {
          log.Error(e, "Unable to map locale to ui supported locale from accept-language header " + sBrowserLocaleList + ".");
          sBrowserLocale = null;
        }
        if ((null == sBrowserLocale) || (0 == sBrowserLocale.Length))
        {
          // We should never get here, because MapLocaleToUISupportedLocaleID
          // should always at least return the default locale, but just in case ...
          // This is worth considering, because if the locale is NOT set to a legit
          // value, the user is REALLY hosed- can't see anything.  Hence, even though 
          // we can "never" get here, it's STILL worth making triple sure we set the 
          // locale to something legit.
          log.Warn("Unable to map locale to ui supported locale from accept-language header " + sBrowserLocaleList + ". Using default system locale.");
          sBrowserLocale = PTPersonalSettingsHelper.GetDefaultLocale(_application);
          }
      } else
      {
        log.Warn("Unable to find browser locale list to map locale to ui supported locale from accept-language header " + sBrowserLocaleList + ". Using default system locale.");
        sBrowserLocale = PTPersonalSettingsHelper.GetDefaultLocale(_application);
        }
      return sBrowserLocale;
    }
    
    /// <summary>
    /// This method gets a supported locale from the browser settings, and
    /// sets it on the http session and as a runtime setting.
    /// </summary>
    /// <param name="_appData"></param>
    /// <param name="_oUserSession"></param>
    /// <param name="_sessionManager"></param>
    /// <param name="_application"></param>
    /// <param name="_rm"></param>
    protected static void InternalSetBrowserLocaleSetting(String sLocale,
     IPTSession _ptSession, ISessionManager _sessionManager, ISessionManager _persistentSessionManager,
     IApplication _application, XPResourceManager _rm)
    {
      // Some general rules/guidelines regarding locale settings
      //
      // 1) "Use Default", the zero length string, and not set should all act equivalently.
      // 2) If a non-guest/non-default profile user logs in with USE_DEFAULT or USE_BROWSER
      // set for it's locale, an appropriate locale will be determined, and then promoted
      // and stored in the DB as the user's new locale setting.
      // 3) There is a strict hierarchy of settings (i.e. it uses the user setting, but if
      // that's default, then it uses the XML setting, but if that's empty it uses the browser.
      // 4) guest user and default profile user locale settings never get promoted.
      // Do not store the locale setting the DB for the guest user or default profile users.
      if (LoginHelper.INSTANCE.GetIsGuestUser(_ptSession) || PlumtreeHelpers.IsEditingDefaultProfile(_persistentSessionManager))
      {
        // This will cache the locale for the UI and as a runtime setting, but not
        // save it to the DB.
        PTPersonalSettingsHelper.SetPersonalSetting(PersonalSettingNames.LOCALE,
         sLocale, _ptSession, _sessionManager, _application, _rm);
      } else
      {
        // This will cache the locale and language for the UI and as a
        // runtime setting, as well as store it in the DB.
        PTPersonalSettingsHelper.SetAndCachePersonalSetting(PersonalSettingNames.LOCALE,
         sLocale, _ptSession, _sessionManager, _application, _rm);
      }
    }
  }
}