using System;
using com.plumtree.openfoundation.reflect;
using com.plumtree.openfoundation.util;
using com.plumtree.portaluiinfrastructure.ptlink;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
namespace com.plumtree.taskapi.portalui
{
  /// <summary>
  /// This interface describes the various helper methods for Communities
  /// in the Task API.
  /// </summary>
  public interface ITaskAPIUICommunity
  {
    
    /// <summary>
    /// Gets the name of the specified Community.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="nCommunityID">Object ID of the Community to retrieve the name for.</param>
    /// <returns>	Name of the specified Community.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    String GetCommunityName(AActivitySpace owner, int nCommunityID);
    
    /// <summary>
    /// Gets the Subcommunities for the specified Community as an integer array.
    /// Subcommunites are Communities directly below a Community in the administrative folder hierarchy.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="nCommunityID">Object ID for the Community to retrieve Subcommunities from.</param>
    /// <returns>	Array of integers with Object IDs of the Subcommunities in the specified Community. </returns>
    /// <throws>
    /// XPException
    /// 									If the Community could not be accessed, for example if the Object ID
    /// is not valid.
    /// </throws>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    int[] GetSubCommununityIDs(AActivitySpace owner, int nCommunityID);
    
    /// <summary>
    /// Gets the Related Communities for the specified Community as an integer array.
    /// Related Communities are Communities in the same administrative folder.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="nCommunityID">Object ID for the Community to retrieve Related Communities from.</param>
    /// <returns>	Array of integers with Object IDs of the Related Communities  in the specified Community. </returns>
    /// <throws>
    /// XPException
    /// 									If the Community could not be accessed, for example if the Object ID
    /// is not valid.
    /// </throws>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    int[] GetRelatedCommununityIDs(AActivitySpace owner, int nCommunityID);
    
    /// <summary>
    /// Get Community Pages in the specified Community as a list of <c>IPTURL</c>s.
    /// The <c>PTURLConstants.KEY_CURRENTMARKER</c> flag is set on an URL entry if is leading to the current
    /// Community Page. 
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="nCommunityID">Object Id of the Communitu to list pages of.</param>
    /// <returns>A list with Community Pages as <c>IPTURL</c> objects.</returns>
    /// <throws>
    /// XPIllegalArgumentException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    XPArrayList GetCommunityPagesAsURLs(AActivitySpace owner, int nCommunityID);
    
    /// <summary>
    /// Gets to URL to the Community Knowledge Directory f0r the current Community.
    /// The <c>PTURLConstants.KEY_CURRENTMARKER</c> flag is set on the URL if the current page is a Community Knowledge
    /// Directory page.
    /// </summary>
    /// <param name="owner">Current Activityspace</param>
    /// <param name="nCommunityID">Object ID of the Community to get pages from.</param>
    /// <returns><c>IPTURL</c> with the URL to the Community Knowledge Directory for the specified Community.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    /// <seealso cref="PTURLConstants"></seealso>
    IPTURL GetURLForCommunityKnowledgeDirectory(AActivitySpace owner, int nCommunityID);
    
    /// <summary>
    /// Gets URLs to Related Communities for the specified Community in a list of <c>IPTURL</c>s.
    /// Related Communities are Communities in the same administrative folder.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="nCommunityID">Object ID for the Community to retrieve Related Communities from.</param>
    /// <returns><c>XPArrayList</c> with <c>IPTURL</c>s to Related Communities or an empty <c>XPArrayList</c> if none exists.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    XPArrayList GetRelatedCommununitiesAsURLs(AActivitySpace owner, int nCommunityID);
    
    /// <summary>
    /// Gets the Community Name of the current Community.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>	Community Name of the current community or <c>null</c> value if not currently on a Community Page. </returns>
    /// <throws>
    /// XPIllegalArgumentException
    /// 		If the provided <c>AActivityspace</c> is <c>null</c>.
    /// </throws>
    String GetCurrentCommunityName(AActivitySpace owner);
    
    /// <summary>
    /// Gets the Community Page Name of the current Community Page. If current
    /// page is not a Community related page, i.e., Community portlet, Sitemap
    /// or Community Gatewaypage page, a null value is returned.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>	Community Page Name of the current community pageor null value if not currently on a Community Page. </returns>
    /// <throws>
    /// XPIllegalArgumentException
    /// 		If the provided <c>AActivityspace</c> is <c>null</c>
    /// </throws>
    String GetCurrentCommunityPageName(AActivitySpace owner);
    
    /// <summary>
    /// Gets the Community ID of the current Community. If current page is not a Community page,
    /// i.e., Community portlet, Sitemap or Community Gatewaypage page, <c>TaskAPIUICommon.INVALID_OBJECTID</c> is returned.
    /// </summary>
    /// <param name="owner">Current Activityspace</param>
    /// <returns>	Community ID of the current community or <c>TaskAPIUICommon.INVALID_OBJECTID</c> if not in a Community.</returns>
    /// <throws>
    /// XPIllegalArgumentException
    /// 		If the provided <c>AActivityspace</c> is <c>null</c>
    /// </throws>
    int GetCurrentCommunityID(AActivitySpace owner);
    
    /// <summary>
    /// Gets the current Community Page ID, If current page is not a Community page,
    /// the <c>TaskAPIUICommon.INVALID_OBJECTID</c> value is returned.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>	Current Community Page ID or <c>TaskAPIUICommon.INVALID_OBJECTID</c> if not in on Community Page.</returns>
    /// <throws>
    /// XPIllegalArgumentException
    /// 		If the provided <c>AActivityspace</c> is <c>null</c>.
    /// </throws>
    int GetCurrentCommunityPageID(AActivitySpace owner);
    
    /// <summary>
    /// Gets the current Page as an object of type<c>IPTURL</c>. Returns  
    /// </summary>
    /// <param name="owner">Current Activityspace</param>
    /// <returns>IPTURL to the current community page or null if the current page is not a community page. </returns>
    /// <throws>XPIllegalArgumentException</throws>
    IPTURL GetCurrentPageAsURL(AActivitySpace owner);
    
    /// <summary>
    /// Gets Subcommunities in the current Community as an <c>XPArrayList</c> with <c>IPTURL</c> entries.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// Array list with <c>IPTURL</c> of Subcommunities.  An empty list is returned if there are 
    /// no subcommunities exist or not in a Community.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    XPArrayList GetCurrentSubCommunitiesAsURLs(AActivitySpace owner);
    
    /// <summary>
    /// Gets the URL to the Unsubscribe from Communities Editor.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// 		<c>IPTURL</c> with URL to the Unsubscribe from Communities Editor, or <c>null</c> if the URL is not
    /// valid in the current context.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    IPTURL GetURLForUnsubscribeCommunitiesEditor(AActivitySpace owner);
    
    /// <summary>
    /// Gets the URL to the Community Editor for the current Community.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// 		<c>IPTURL</c> with URL to the Editor for the current Community, or <c>null</c> if the URL is not
    /// valid in the current context.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    IPTURL GetURLForCurrentCommunityEditor(AActivitySpace owner);
    
    /// <summary>
    /// Gets an XPArrayList with a single entry for the URL for the current Community.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// 		<c>XPArrayList</c> list containing a single URL to the current Community, 
    /// or <c>null</c> if the URL is not valid in the current context.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    XPArrayList GetURLForCurrentCommunity(AActivitySpace owner);
    
    /// <summary>
    /// Gets the URL to the Community Add Portlets Editor.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// 		<c>IPTURL</c> with URL to the Community Add Portlets Editor, or <c>null</c> if the URL is not
    /// valid in the current context.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    IPTURL GetURLForAddCommunityPortletsEditor(AActivitySpace owner);
    
    /// <summary>
    /// Gets the URL to the Community Add Portlets Flyout.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// 		<c>IPTURL</c> with URL to the Community Add Portlets Flyout, or <c>null</c> if the URL is not
    /// valid in the current context.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    IPTURL GetURLForAddCommunityPortletsFlyout(AActivitySpace owner);
    
    /// <summary>
    /// Gets the URL to the Create Community page in the Community Editor.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// 		<c>IPTURL</c> with URL to Create Community Page in the Community Editor, 
    /// or <c>null</c> if the URL is not valid in the current context.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    IPTURL GetURLForCreateCommunityPage(AActivitySpace owner);
    
    /// <summary>
    /// Gets the URL to the Create Community page in the Community Editor.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// 		<c>IPTURL</c> with URL to Create Community Page in the Community Editor, 
    /// or <c>null</c> if the URL is not valid in the current context.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    IPTURL GetURLForCreateCommunityPageOneClick(AActivitySpace owner, bool bInheritTemplate,
     int nPageTemplate);
    
    /// <summary>
    /// Gets the URL for the Join Community Editor where users add Communities to their My Communities list.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// 		<c>IPTURL</c> with URL to the Join Communities Editor, or <c>null</c> if the URL is not
    /// valid in the current context.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    IPTURL GetURLForJoinCommunitiesEditor(AActivitySpace owner);
    
    /// <summary>
    /// Gets the URL to the Community Edit Portlet Preferences Editor.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// 		<c>IPTURL</c> with URL to the Community Edit Portlet Preferences Editor, or <c>null</c> if the URL is not
    /// valid in the current context.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    IPTURL GetURLForEditPortletPreferencesEditor(AActivitySpace owner);
    
    /// <summary>
    /// Checks if the current user is already subscribed to the current Community, i.e. if the current Community
    /// is already on the user's My Community list.
    /// </summary>
    /// <param name="owner">Current activityspace.</param>
    /// <returns>
    /// <c>true</c> if current Community is already subscribed by the user, <c>false</c> if not member, or on User profile community
    /// or  not or if not currently in a Community.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    bool IsUserMemberOfCurrentCommunity(AActivitySpace owner);
    
    /// <summary>
    /// Checks if the current page is a Community Page.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>	<c>true</c> if the current page is a Community Page, <c>false</c> otherwise.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    bool IsCurrentPageCommunityPage(AActivitySpace owner);
    
    /// <summary>
    /// Checks if the current Community is the User Profile Community.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>	<c>true</c> if the current Community is the User Profile Community, <c>false</c> otherwise.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    bool IsCurrentCommunityUserProfileCommunity(AActivitySpace owner);
    
    /// <summary>
    /// Checks if the current user has access (requires EDIT access) to use the Community Editor on the current
    /// Community Page.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>	<c>true</c> if the current user has at least EDIT access to the current Community Page, <c>false</c> otherwise.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    bool IsCurrentUserAbleToEditCurrentCommunityPage(AActivitySpace owner);
    
    /// <summary>
    /// Checks if the current user is the owner of the current Community.
    /// The check only works on Community pages or Community Knowledge directory pages, not in Community editors.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>	<c>true</c> if the current user has at least EDIT access to the current Community, <c>false</c> otherwise.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    bool IsCurrentUserCommunityOwner(AActivitySpace owner);
    
    /// <summary>
    /// Checks if the current user has at least SELECT access to the current Community.
    /// Only works on Community pages or Community Knowledge directory pages, not in Community Editors.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>	<c>true</c> if the current user has at least SELECT access to the current Community, <c>false</c> otherwise.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    bool IsUserAbleToJoinCurrentCommunity(AActivitySpace owner);
    
    /// <summary>
    /// Checks if the current user has at least READ access to the current Community.
    /// Only works on Community pages or Community Knowledge directory pages, not in Community editors.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>	<c>true</c> if the current user has at least READ access to the current Community, <c>false</c> otherwise.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    bool IsUserAbleToAccessCurrentCommunity(AActivitySpace owner);
    
    /// <summary>
    /// Checks if the current page is a Myportal page, i.e. either a MyPage or Community Page.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>	<c>true</c> if the current page is either a MyPage or Community Page, <c>false</c> otherwise.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    bool IsCurrentPageMyPortalPage(AActivitySpace owner);
    
    /// <summary>
    /// Gets Community Actions as an XPArrayList with IPTURL entries.
    /// Most Community actions only display when in an Community and only if a user has access 
    /// to perform the actions.  
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>	An <c>XPArrayList</c> with <c>IPTURL</c> entries with Community Actions. </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    XPArrayList GetCommunityActionsAsURLs(AActivitySpace owner);
    
    /// <summary>
    /// Gets Community Actions as an XPArrayList with IPTURL entries.
    /// Most Community actions only display when in an Community and only if a user has access 
    /// to perform the actions.  
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="includeOneClick">Flag to include the "one-click page creation" link</param>
    /// <returns>	An <c>XPArrayList</c> with <c>IPTURL</c> entries with Community Actions. </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    XPArrayList GetCommunityActionsAsURLs(AActivitySpace owner, bool bIncludeOneClick,
     bool bInheritTemplate, int nPageTemplate);
    
    /// <summary>
    /// Helper function to create an URL to a specified Community. 
    /// The <c>PTURLConstants.KEY_CURRENTMARKER</c> flag is set on the URL if is leading to the current
    /// Community.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="nCommunityID">Object ID of the Community to create an URL to.</param>
    /// <param name="strCommunityName">Name of the URL, normally the name of the Community.</param>
    /// <returns>	<c>IPTURL</c> with an URL to the specified Community.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    IPTURL CreateCommunityCommonOpenerURL(AActivitySpace owner, int nCommunityID,
     String strCommunityName);
    
    /// <summary>
    /// Helper function to create an URL to a specified Community Page. 
    /// The <c>PTURLConstants.KEY_CURRENTMARKER</c> flag is set on the URL if is leading to the current
    /// Community Page. In case the page is a profile page, the link generated will be of the viewed profile,
    /// not the current user's.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="nCommunityID">Object ID of the Community where the Page is located in.</param>
    /// <param name="nCommunityPageID">Object ID of the Page to create a URL to.</param>
    /// <param name="strCommunityPageName">Title of the URL, normally the name of the Page.</param>
    /// <returns><c>IPTURL</c> with an URL to the specified Community Page.</returns>
    /// <throws>
    /// XPIllegalArgumentException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    IPTURL CreateCommunityPageCommonOpenerURL(AActivitySpace owner, int nCommunityID,
     int nCommunityPageID, String strCommunityPageName);
    
    /// <summary>
    /// Gets an URL to a UserProfile page for the specified user.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="nProfileUserID">The Object ID of the user to open the User Profile page for.</param>
    /// <param name="sProfileUserName">The name of the user to open the User Profile page for.  Also used for the link title.</param>
    /// <returns>	<c>IPTURL</c> with the URL to the user profile page for the specified user.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    IPTURL CreateUserProfilePageCommonOpenerURL(AActivitySpace owner, int nProfileUserID,
     String sProfileUserName);
    
    /// <summary>
    /// Gets an URL to a UserProfile page for the specified user.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="nProfileUserID">The Object ID of the user to open the User Profile page for.</param>
    /// <param name="sProfileUserName">The name of the user to open the User Profile page for.</param>
    /// <param name="strURLTitle">The title of the URL.</param>
    /// <returns>	<c>IPTURL</c> with the URL to the user profile page for the specified user.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    IPTURL CreateUserProfilePageCommonOpenerURL(AActivitySpace owner, int nProfileUserID,
     String sProfileUserName, String strURLTitle);
    
    /// <summary>
    /// Gets an URL to a UserProfile page for the specified user.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="nProfileUserID">The Object ID of the user to open the User Profile page for.</param>
    /// <param name="sProfileUserName">The name of the user to open the User Profile page for.</param>
    /// <param name="strURLTitle">The title of the URL.</param>
    /// <returns>	<c>IPTURL</c> with the URL to the user profile page for the specified user.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    IPTURL CreateUserProfilePageCommonOpenerURL(AActivitySpace owner, int nProfileUserID,
     String sProfileUserName, String strURLTitle, int nPageId, String strPageName);
    
    /// <summary>
    /// Check the if the current user has at least the accesslevel specified to the current
    /// community 
    /// </summary>
    /// <param name="AccessLevel">to check for. Example: PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_READ or PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT</param>
    /// <param name="nCommunityID">Object ID of the Community to check security of</param>
    /// <returns>boolean			True if the current user has at least the specified accesslevel, false if the user does not have access or there is no current community.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    /// <seealso cref="PT_ACCESS_LEVELS"></seealso>
    bool CheckUserAccessLevelToCommunity(AActivitySpace owner, int nAccessLevel,
     int nCommunityID);
    
    /// <summary>
    /// Check the if the current user has at least the accesslevel specified to the current
    /// community 
    /// </summary>
    /// <param name="AccessLevel">to check for. Example: PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_READ or PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT</param>
    /// <returns>boolean			True if the current user has at least the specified accesslevel, false if the user does not have access or there is no current community.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    /// <seealso cref="PT_ACCESS_LEVELS"></seealso>
    bool CheckUserAccessLevelToCurrentCommunity(AActivitySpace owner, int nAccessLevel);
    
    /// <summary>
    /// Check the if the current user has at least the accesslevel specified to the current
    /// community page
    /// </summary>
    /// <param name="AccessLevel">to check for. Example: PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_READ or PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT</param>
    /// <returns>boolean			True if the current user has at least the specified accesslevel, false if the user does not have access or there is no current community page.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    /// <seealso cref="PT_ACCESS_LEVELS"></seealso>
    bool CheckUserAccessLevelToCurrentCommunityPage(AActivitySpace owner,
     int nAccessLevel);
    
    /// <summary>
    /// Gets the Object ID of the parent Community in the specified Community.  (the Community where the current Community is located in the administrative hierarchy).
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="nCommunityID">Object ID of the Community to get the parent Community from.</param>
    /// <returns>Object ID of current parent Community ID, or <c>0 /code>  if non-existant.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    int GetParentCommunityID(AActivitySpace owner, int nCommunityID);
    
    /// <summary>
    /// Gets the Object ID of the parent Community in the current Community. 
    /// Parent Community is the Community where the current Community is located in the administrative hierarchy.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// Object ID of current parent Community ID, or <c>0</c> if not currently in a Community or
    /// the current Community does not have a parent Community.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    int GetCurrentParentCommunityID(AActivitySpace owner);
    
    /// <summary>
    /// Gets Related Communities (a.k.a. peer Communities) of the current Community as an integer array.
    /// Related Communities are Communities located in one Community in the administrative hierarchy.  
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// Array with Object IDs of Related Community to the current Community.  
    /// Empty array if the current Community has no Related communities or if not in a Community.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    int[] GetCurrentRelatedCommununityIDs(AActivitySpace owner);
    
    /// <summary>
    /// Gets the URL to Join Current Community Action.
    /// The URL goes to a page that subscribes the user to the current Community. This URL is only visible
    /// on Communities a user is not already member of.  
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns><c>IPTURL</c> to join current community or <c>null</c> if the current user is already a member or does not have access to join the current community.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    IPTURL GetURLForJoinCurrentCommunityAction(AActivitySpace owner);
    
    /// <summary>
    /// Gets the URL to Unsubscribe Current Community Action.
    /// The URL goes to a page that unsubscribes the user to the current Community. This URL is only visible
    /// on Communities a user is not already member of.  
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns><c>IPTURL</c> to unsubscribe current community or <c>null</c> if the current user is already not a member or does not have access to unsubscribe the current community.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    IPTURL GetURLForUnsubscribeCurrentCommunityAction(AActivitySpace owner);
    
    /// <summary>
    /// Gets the Subcommunities of the current Community as a integer array.
    /// Subcommunties are Communities located in another Community in the adminstrative hierarchy.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// An integer array with Object IDs to Subcommunity.  
    /// Returns an empty array if the current Community has no Subcommunities or not currently in a Community.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    int[] GetCurrentSubCommunityIDs(AActivitySpace owner);
    
    /// <summary>
    /// Gets the Community Pages of the current Community as an <c>XPArraylist</c> with <c>IPTURL</c> entries.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>An <c>XPArrayList</c> with <c>IPTURL</c> to CommunityPages, or empty <c>XPArrayList</c> if the current page is not a Community Page or valid Page.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    XPArrayList GetCurrentCommunityPagesAsURLs(AActivitySpace owner);
    
    /// <summary>
    /// Gets URLs to Related Communities for for the current Community in a list of <c>IPTURL</c>s.
    /// Related Communities are Communities in the same administrative folder.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns><c>XPArrayList</c> with <c>IPTURL</c>s to Related Communities or an empty <c>XPArrayList</c> if none exists.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    XPArrayList GetCurrentRelatedCommunitiesAsURLs(AActivitySpace owner);
    
    /// <summary>
    /// Gets the URL to the parent Community of the current Community. 
    /// Parent Community is the Community where the current Community is located in the administrative hierarchy.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// IPTURL to current Parent Community  or <c>null</c> if no parent Community exists, or if not currently
    /// in a Community.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    IPTURL GetCurrentParentCommunityAsURL(AActivitySpace owner);
    
    /// <summary>
    /// Gets the URL to the parent Community of the specified Community. 
    /// Parent Community is the Community where the current Community is located in the administrative hierarchy.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="nCommunityID"></param>
    /// <returns>
    ///  IPTURL to current Parent Community  or <c>null</c> if no parent Community exists, or if the 
    /// specified Community is invalid.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    IPTURL GetParentCommunityAsURL(AActivitySpace owner, int nCommunityID);
    
    /// <summary>
    /// Gets the Subcommunities for the specified Community as a list with <c>IPTURL</c> entries.
    /// Subcommunities are Community located below another Community in the administrative hierarchy. 
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="nCommunityID">Object ID for the Community to get Subcommunities from.</param>
    /// <returns>
    /// 	An <c>XPArrayList</c> with <c>IPTURL</c> entries to Subcommunities, or
    /// empty array if the specified Community does no thave any Subcommunities or it is invalid.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    XPArrayList GetSubCommununitiesAsURLs(AActivitySpace owner, int nCommunityID);
    
    /// <summary>
    /// Gets the default Community ID, the first Community in a user's My Community list.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// 	Object ID of the first Community in the current user's My Community list, or <c>-1</c> if
    /// the user is not a member of any communities.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    int GetDefaultCommunityID(AActivitySpace owner);
    
    /// <summary>
    /// Gets a user's My Communities list as an <c>XPArrayList</c> with <c>IPTURL</c> entries.
    /// The <c>PTURLConstants.KEY_CURRENTMARKER</c> flag is set on an URL entry if is leading to the current
    /// Community. 
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="bUseCachedData">
    /// Get the current user's mandatory Communities with Tabs from a cache that contains at 
    /// most 15 minutes old data if true. Will query the database for the mandatory communities if false.
    /// </param>
    /// <returns>
    /// 	<c>XPArrayList</c> with <c>IPTURL</c>s to the current user's My Communities ,
    /// or <c>null</c> if Communities features is disabled in the current Experience Definition.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    XPArrayList GetMyCommunitiesAsURLs(AActivitySpace owner, bool bUseCachedData);
    
    /// <summary>
    /// Gets a user's Mandatory with Tab Communities as an <c>XPArrayList</c>  with <c>IPTURL</c> entries. 
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="bUseCachedData">
    /// Get the current user's mandatory Communities with Tabs from a cache that contains at most 15 minutes old data if true. Will query the database 
    /// for the mandatory communities if false.
    /// </param>
    /// <returns>
    /// <c>XPArrayList</c> with <c>IPTURL</c>s to the current user's mandatory Communities with Tabs,
    /// or <c>null</c> if Communities features is disabled in the current Experience Definition.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    XPArrayList GetMyMandatoryCommunitiesWithTabsAsURLs(AActivitySpace owner,
     bool bUseCachedData);
    
    /// <summary>
    /// Gets the Community ID from an IPTURL with an URL to a Community or Community Page.
    /// </summary>
    /// <param name="ptURL"><c>IPTURL</c> to a Community or Community Page.</param>
    /// <returns>The Community ID, or <c>0</c> if none is found in the URL.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    int GetCommunityIDFromPTURL(IPTURL ptURL);
    
    /// <summary>
    /// Gets the Community Page ID from an IPTURL with an URL to a Community Page.
    /// </summary>
    /// <param name="ptURL"><c>IPTURL</c> to a Community Page.</param>
    /// <returns>The Community Page ID, or <c>0</c> if none is found in the URL.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    int GetCommunityPageIDFromPTURL(IPTURL ptURL);
    
    /// <summary>
    /// Checks whether or not the current user is a member of the specified community.
    /// </summary>
    /// <param name="_asOwner">the current Activity Space</param>
    /// <param name="_nCommunityID">the community to check membership for.</param>
    /// <returns>true if the current user is a member of the specified community.</returns>
    bool IsCommunityMember(AActivitySpace _asOwner, int _nCommunityID);
    
    /// <summary>
    /// This method causes the current user to join the specified community.
    /// This method will check community membership before joining the community.
    /// </summary>
    /// <param name="_asOwner">the current Activity Space</param>
    /// <param name="_nCommunityID">the community to join.</param>
    /// <returns>
    /// True means the user has joined the community, false means the user
    /// was already a member.
    /// </returns>
    /// <throws>XPException if there was an error.</throws>
    bool JoinCommunity(AActivitySpace _asOwner, int _nCommunityID);
    
    /// <summary>
    /// This method causes the current user to leave the specified community.
    /// This method will check community membership before leaving the community.
    /// </summary>
    /// <param name="_asOwner">the current Activity Space</param>
    /// <param name="_nCommunityID">the community to leave.</param>
    /// <returns>
    /// True means the user has left the community, false means the user
    /// was not a member of the community.
    /// </returns>
    /// <throws>XPException if there was an error.</throws>
    bool LeaveCommunity(AActivitySpace _asOwner, int _nCommunityID);
  }
}