using System;
using junit.framework;
using com.plumtree.openfoundation.util;
using com.plumtree.portalpages.rest.utils;
namespace com.plumtree.portalpages.test.rest.utils
{
  /// <summary>
  /// This tests the simple JSON parser that's used for the
  /// KD REST API calls.
  /// It is very limited, and does not handle arrays, or non-string
  /// object types.  It also only handles escaped quotes and slashes.
  /// </summary>
  public class JSONParserTest : TestCase
  {
    public virtual void testParseError()
    {
      String strErrorJSON = "blah blah blah";
      IJSONParser parser = JSONParser.GetInstance();
      bool bException = false;
      try
      {
        parser.ParseJSONObject(strErrorJSON);
      }
      catch (XPFormatException xpfe)
      {
        bException = true;
      }
      if (!bException)
      {
        fail("Parser should fail when parsing non-JSON object.");
      }
    }
    public virtual void testMissingCommasParseError()
    {
      String strErrorJSON = "{ \"document\": { \"name\": \"test\" \"description\": \"test test\" } }";
      IJSONParser parser = JSONParser.GetInstance();
      bool bException = false;
      try
      {
        parser.ParseJSONObject(strErrorJSON);
      }
      catch (XPFormatException xpfe)
      {
        bException = true;
      }
      if (!bException)
      {
        fail("Parser should fail when parsing non-JSON object.");
      }
    }
    public virtual void testSimpleObject()
    {
      String strDocumentJSON = "{ \"document\": { \"name\": \"test\", \"description\": \"test test\" } }";
      IJSONParser parser = JSONParser.GetInstance();
      parser.ParseJSONObject(strDocumentJSON);
      assertEquals("The parsed object name was incorrect.", "document", parser.GetObjectName());
      assertEquals("The parsed name attribute was incorrect.", "test", parser.GetMember("name"));
      assertEquals("The parsed description attribute was incorrect.", "test test",
       parser.GetMember("description"));
    }
    public virtual void testSimpleObjectCommentFiltered()
    {
      // Comment filtering is often used to thwart JSON security attacks.  We wrap the text in /* */.
      String strDocumentJSON = "/* { \"document\": { \"name\": \"test\", \"description\": \"test test\" } } */";
      IJSONParser parser = JSONParser.GetInstance();
      parser.ParseJSONObject(strDocumentJSON);
      assertEquals("The parsed object name was incorrect.", "document", parser.GetObjectName());
      assertEquals("The parsed name attribute was incorrect.", "test", parser.GetMember("name"));
      assertEquals("The parsed description attribute was incorrect.", "test test",
       parser.GetMember("description"));
    }
    public virtual void testSimpleObjectCommentFilteredWithWhitespace()
    {
      // Comment filtering is often used to thwart JSON security attacks.  We wrap the text in /* */.
      String strDocumentJSON = " /* { \"document\": { \"name\": \"test\", \"description\": \"test test\" } } */ ";
      IJSONParser parser = JSONParser.GetInstance();
      parser.ParseJSONObject(strDocumentJSON);
      assertEquals("The parsed object name was incorrect.", "document", parser.GetObjectName());
      assertEquals("The parsed name attribute was incorrect.", "test", parser.GetMember("name"));
      assertEquals("The parsed description attribute was incorrect.", "test test",
       parser.GetMember("description"));
    }
    public virtual void testSimpleObjectWithWhiteSpace()
    {
      String strDocumentJSON = "{     \"error\":   {   \"text\":  \"test\",   \"code\":" + "\"123\"  }      }";
      IJSONParser parser = JSONParser.GetInstance();
      parser.ParseJSONObject(strDocumentJSON);
      assertEquals("The parsed object name was incorrect.", "error", parser.GetObjectName());
      assertEquals("The parsed text attribute was incorrect.", "test", parser.GetMember("text"));
      assertEquals("The parsed code attribute was incorrect.", "123", parser.GetMember("code"));
      }
    public virtual void testSimpleObjectWithNoWhiteSpace()
    {
      String strDocumentJSON = "{\"error\":{\"text\":\"test\",\"code\":\"123\"}}";
      IJSONParser parser = JSONParser.GetInstance();
      parser.ParseJSONObject(strDocumentJSON);
      assertEquals("The parsed object name was incorrect.", "error", parser.GetObjectName());
      assertEquals("The parsed text attribute was incorrect.", "test", parser.GetMember("text"));
      assertEquals("The parsed code attribute was incorrect.", "123", parser.GetMember("code"));
      }
    public virtual void testSimpleObjectWithEscapedQuotes()
    {
      String strDocumentJSON = "{\"error\":{\"text\":\"test \\\"me\\\" and a slash like \\\\\",\"code\":\"123\"}}";
      IJSONParser parser = JSONParser.GetInstance();
      parser.ParseJSONObject(strDocumentJSON);
      assertEquals("The parsed object name was incorrect.", "error", parser.GetObjectName());
      assertEquals("The parsed text attribute with escaped quotes was incorrect.",
       "test \"me\" and a slash like \\", parser.GetMember("text"));
      assertEquals("The parsed code attribute was incorrect.", "123", parser.GetMember("code"));
      }
  }
}