package com.plumtree.portalpages.admin.editors.group;


// Plumtree Server
import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.common.constants.*;
import com.plumtree.portaluiinfrastructure.application.varpacks.*;
import com.plumtree.portaluiinfrastructure.editor.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.migrationhistory.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.objectsecurity.*;
import com.plumtree.portaluiinfrastructure.expandablelist.selectptobject.*;
import com.plumtree.portaluiinfrastructure.filter.*;
import com.plumtree.portaluiinfrastructure.helpers.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.server.impl.core.*;
import com.plumtree.server.impl.users.*;
import com.plumtree.uiinfrastructure.arraywrapper.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.tree.*;
import com.plumtree.xpshared.htmlconstructs.*;

/**
 * This class handles all the data for the Group Editor. Methods are called by
 * the control when the user switches between display pages, and also when the
 * user finishes.
 *
 * @author RickyW
 */
public class GroupModel extends ObjEditorModel implements IGroupModelRO
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
		"com.plumtree.portalpages.admin.editors.group.GroupModel");

	/** class name */
	public static final String STR_MVC_CLASS_NAME = "Group";
	
	/** storage container for parent groups when first loaded */
	private XPArrayList m_xpalOriginalParentGroups;

	/** storage container for non direct parent groups */
	private XPArrayList m_xpalNonDirectParentGroups;
	
	/** boolean indicating if this is a remote group of not */
	private boolean m_bIsRemoteGroup;
	
	/** boolean indicating update to dynamic members on finish */
	private boolean m_bUpdateDynamicMembersOnFinish;
	
	/**
	 * Object array of valid property ids to show the user.
	 */
	Object[] m_oArNonRefPropIDs;
	
	/** dynamic rules chooser mvars */
	private IPTFilter m_Filter;
	private FilterIterator m_FilterIterator;

	/**
	 * Number of users belonging to this group
	 */
	private int m_nNumUsers	= 0;
		
	/**
	 * Setting that indicates max number of users before we stop displaying them. 
	 */
	// Bug 43496 - Trying to view a very large group (1,000,000 members) causes the portal to crash
	private int m_nMaxResultsToDisplay = 500;
	
	/**
	 * Flag to indicate whether we are displaying users or too many users to display.
	 */
	// Bug 43496 - Trying to view a very large group (1,000,000 members) causes the portal to crash
	private boolean m_bDisplayUsers = true;
	
	public GroupModel()
	{
		super();

		m_xpalNonDirectParentGroups = new XPArrayList();
		m_xpalOriginalParentGroups = new XPArrayList();
		
		// aj: default to intrinsic group
		m_bIsRemoteGroup = false;
		
		// aj: default is false
		m_bUpdateDynamicMembersOnFinish = false;
		
		// aj: default is null
		m_oArNonRefPropIDs = null;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new GroupModel();
	}

	/**
	 * Code executed after the object is stored. Groups must exist
	 * before information can be added to them.  Therefore, all of
	 * the group's data is saved here.
	 */
	public void DoTaskAfterStore()
	{
		// get group, and group's server context
		IPTUserGroup ptGroup  = (IPTUserGroup) m_ptObject;
		IPTServerContext ptSC = (IPTServerContext) ptGroup;
		PTGrowableSortedArrayWrapper saData;

		//The try/catch blocks have been refactored in this file to provide more detailed error handling
		IPTServerContext ptSCParent = null;

		// lock group for editing
		try
		{
			ptSC.LockObject();
		}
		catch (Exception ex)
		{

			log.Error(ex, "GroupModel::DoTaskAfterStore Unable to lock the Group.");

			throw new XPException(m_asOwner.GetString(419, "ptmsgs_portaladminmsgs"), ex);
		}

		try
		{
			// The Everyone group contains all the users in the portal.  The memberships cannot be edited.
			if (this.m_nObjectID !=	PT_INTRINSICS.PT_USERGROUP_EVERYONE)
			{
				// aj: only save child members if this is a native group, remote
				//     groups are RO for this...
				// Bug 43496 - Trying to view a very large group (1,000,000 members) causes the portal to crash
				// WA: only save child members if we are actually displaying the members 
				//     (i.e., not too many users to display)
				if(!IsRemoteGroup() && m_bDisplayUsers){
					// ******************
					// ** SAVE MEMBERS **
					// ******************
					// aj: bug 42886 - perform membership updates using deltas as opposed
					//     to wiping the slate clean each time someone hits finish.  this 
					//     requires us to populate the following four array lists and
					//     make the appropriate API calls
					XPArrayList xpalUsersToAdd  = new XPArrayList();
					XPArrayList xpalUsersToDelete = new XPArrayList();
					XPArrayList xpalGroupsToAdd = new XPArrayList();
					XPArrayList xpalGroupsToDelete = new XPArrayList();
					CalculateMembershipDeltas(xpalUsersToAdd, xpalUsersToDelete, xpalGroupsToAdd, xpalGroupsToDelete);

					// aj: we'll put members into this static array for addition/deletion
					int[] arChildMembers;

					// aj: add users to the group
					if(!xpalUsersToAdd.IsEmpty())
					{
						arChildMembers = ConvertArrListToIntArr(xpalUsersToAdd);
						ptGroup.AddUsers(arChildMembers);
					}

					// aj: delete users from the group
					if(!xpalUsersToDelete.IsEmpty())
					{
						arChildMembers = ConvertArrListToIntArr(xpalUsersToDelete);
						ptGroup.RemoveUsers(arChildMembers);
					}

					// aj: add child groups to the group
					if(!xpalGroupsToAdd.IsEmpty())
					{
						arChildMembers = ConvertArrListToIntArr(xpalGroupsToAdd);
						ptGroup.AddChildGroups(arChildMembers);
					}

					// aj: delete child groups from the group
					if(!xpalGroupsToDelete.IsEmpty())
					{
						arChildMembers = ConvertArrListToIntArr(xpalGroupsToDelete);
						ptGroup.RemoveChildGroups(arChildMembers);
					}					
				}

				// ************************
				// ** SAVE PARENT GROUPS **
				// ************************

				// aj: all groups (native and remote) have editable parents

				IPTUserGroup ptParentGroup;
				PTGrowableSortedArrayWrapper saSelectedParentGroups;

				// get selected parent groups
				saSelectedParentGroups = ((SelectPTObjectModel) m_asOwner.
						GetModel(GroupEditorAS.strStorageParentGroupSelectPTObject)).GetItems();

				// transfer to array list for easy manipulation
				XPArrayList xpalSelectedParentGroups = new XPArrayList();
				for (int ii = 0; ii < saSelectedParentGroups.GetCount(); ii ++)
				{
					xpalSelectedParentGroups.Add(XPConvert.ToString(saSelectedParentGroups.
							GetItem(ii, SelectPTObjectModel.EXPLIST_COL_OBJECTID)));
				}	

				// add new parent groups
				for (int ii = 0; ii < xpalSelectedParentGroups.GetSize(); ii ++)
				{
					// this was newly selected
					if (!m_xpalOriginalParentGroups.Contains(xpalSelectedParentGroups.GetElement(ii)))
					{
						// get group, and group's server context
						ptParentGroup = (IPTUserGroup) m_Session.
						GetObjectManagers(PT_CLASSIDS.PT_USERGROUP_ID).
						Open(XPConvert.ToInteger(xpalSelectedParentGroups.GetElement(ii)), false);				
						ptSCParent = (IPTServerContext) ptParentGroup;
						ptSCParent.LockObject();

						// add this group as a child group in the parent group
						ptParentGroup.AddChildGroups(this.m_nObjectID);

						// save
						ptSCParent.Store();
						ptSCParent.UnlockObject();
					}
				}

				// remove deleted parent groups
				for (int ii = 0; ii < m_xpalOriginalParentGroups.GetSize(); ii ++)
				{
					// this was newly deleted
					if (!xpalSelectedParentGroups.Contains(m_xpalOriginalParentGroups.GetElement(ii)))
					{
						// get group, and group's server context
						ptParentGroup = (IPTUserGroup) m_Session.
						GetObjectManagers(PT_CLASSIDS.PT_USERGROUP_ID).
						Open(XPConvert.ToInteger(m_xpalOriginalParentGroups.GetElement(ii)), false);				
						ptSCParent = (IPTServerContext) ptParentGroup;
						ptSCParent.LockObject();

						// remove this group as a child group in the parent group
						ptParentGroup.RemoveChildGroups(this.m_nObjectID);

						// save
						ptSCParent.Store();
						ptSCParent.UnlockObject();
					}
				}
			}
		}
		catch (Exception ex)
		{				
			log.Error(ex, "GroupModel::DoTaskAfterStore:  Unable to add or remove the Group.");

			// unlock the object on error
			try
			{
				//we're unlocking the group here b/c we're going to immediately throw an exception and leave this method
				ptSC.UnlockObject();

				//if we were adding items to the parent group and an error occurred, we have to make sure that
				//all objects are unlocked.  This issue only occurs if we add more than 1 group at a time.
				if (null != ptSCParent)
				{
					ptSCParent.UnlockObject();
				}
			}
			catch (Exception unlockEx)
			{
				//do nothing, if we can't unlock object, it must already be unlocked.
				log.Debug("GroupModel::DoTaskAfterStore:  An unlock exception occurred, ignore, object was already unlocked.");
			}

			throw new XPException(m_asOwner.GetString(421, "ptmsgs_portaladminmsgs"), ex);
		}

		try 
		{
			// **************************
			// ** SAVE ACTIVITY RIGHTS **
			// **************************

			// now we have to deal with activity rights.  we can't just
			// remove and then add because some rights are needed for 
			// adding.  thus we need to only apply the deltas.  first
			// we'll retrieve the rights the group has, then retrieve
			// the rights we want it to have from the editor, and 
			// finally add, and then remove the differences.
			boolean bFound;
			int nSelectedRightsID;
			XPArrayList xpalRights		 = new XPArrayList();
			XPArrayList xpalRemoveRights = new XPArrayList();

			// retrieving existing activity rights
			IPTQueryResult ptQRRights = ptGroup.QueryAllActivityRights();
			int[] saRights = null;

			if (null != ptQRRights)
			{
				saRights = new int[ptQRRights.RowCount()];

				for (int ii = 0; ii < ptQRRights.RowCount(); ii++)
				{
					saRights[ii] = ptQRRights.ItemAsInt(ii,
							PT_PROPIDS.PT_PROPID_OBJECTID);
				}
			}

			// retriving newly selected activity rights
			saData = ((SelectActivityRightsModel) m_asOwner.GetModel(GroupEditorAS.strStorageSelectActivityRights)).GetItems();

			// adding new activity rights.  for each right that's
			// on our newly selected activity rights that's not on
			// our existing activity rights, add.
			if (null != saData)
			{
				for (int ii = 0; ii < saData.GetCount(); ii++)
				{
					nSelectedRightsID     = XPConvert.ToInteger(saData.GetItem(ii,
							SelectPTObjectModel.EXPLIST_COL_OBJECTID));
					bFound				  = false;

					for (int jj = 0; jj < saRights.length; jj++)
					{
						if (nSelectedRightsID == saRights[jj])
						{
							bFound = true;
						}
					}

					// add new rights to helper object
					if (bFound == false)
					{
						xpalRights.Add(new Integer(nSelectedRightsID));
					}
				}

				if (xpalRights.GetSize() > 0)
				{
					// add rights
					ptGroup.GrantActivityRights(xpalRights.ToArray());
				}
			}

			// revoking old activity rights.  for each right that's
			// on our existing rights that's not on our existing
			// activity rights, remove.
			if (null != saRights)
			{
				for (int ii = 0; ii < saRights.length; ii++)
				{
					nSelectedRightsID     = saRights[ii];
					bFound				  = false;

					for (int jj = 0; jj < saData.GetCount(); jj++)
					{
						if (nSelectedRightsID == XPConvert.ToInteger(saData.GetItem(
								jj,
								SelectPTObjectModel.EXPLIST_COL_OBJECTID)))
						{
							bFound = true;
						}
					}

					// mark for removal
					if (bFound == false)
					{
						xpalRemoveRights.Add(new Integer(nSelectedRightsID));
					}
				}

				if (xpalRemoveRights.GetSize() > 0)
				{
					// revoke rights
					ptGroup.RevokeActivityRights(xpalRemoveRights.ToArray());
				}
			}

			// save the group, unlock, and update editor object
			try
			{
				ptSC.Store();
				ptSC.UnlockObject();
				m_ptObject = ptGroup;
			}
			catch (Exception ex)
			{

				log.Error(ex, "GroupModel::DoTaskAfterStore Unable to store or unlock group.");

				throw new XPException(m_asOwner.GetString(422, "ptmsgs_portaladminmsgs"), ex);

			}
		}
		catch (Exception ex)
		{

			log.Error(ex,"GroupModel::DoTaskAfterStore Errors saving the group.");

			throw new XPException(m_asOwner.GetString(423, "ptmsgs_portaladminmsgs"), ex);

		}
		
		// aj: added this (outside try/catch above) to update group memberships
		//     when appropriate.  this should remain outside the try/catch
		//     above since it should not affect the execution of that code in
		//     any way.  also logging should be different for the update on error.
		try
		{
			if(ShouldUpdateDynamicMembersOnFinish())
			{
				IPTUserGroup pGroup;
				pGroup = (IPTUserGroup)m_ptObject;
				
				pGroup.UpdateDynamicMembers();
			}
		} catch (Exception e)
		{
			log.Error(e, "GroupModel::DoTaskAfterStore Errors updating the dynamic members of the group.");
			
			throw new XPException(m_asOwner.GetString(3175,
			ResourceConstants.PORTALADMINMSGS), e);
		}
	}
	
	/**
	 * Figures out what memberships have been added and deleted for both child
	 * users and child groups.  Does this by comparing current memberships
	 * on server to those reported by the UI.
	 * 
	 * @param xpalUsersToAdd - XPArrayList of users to add.  Will be populated by this function.
	 * @param xpalUsersToDelete - XPArrayList of users to delete.  Will be populated by this function.
	 * @param xpalGroupsToAdd - XPArrayList of groups to add.  Will be populated by this function.
	 * @param xpalGroupsToDelete - XPArrayList of groups to delete.  Will be populated by this function.
	 */
	private void CalculateMembershipDeltas(
			XPArrayList xpalUsersToAdd, 
			XPArrayList xpalUsersToDelete, 
			XPArrayList xpalGroupsToAdd, 
			XPArrayList xpalGroupsToDelete)
	{
		// aj: Note: we don't care if any of our callers have locked the 
		//     ptobject, this is of little consequence for membership manipulation...
		
		// aj: our strategy is to compare what is in the UI to what is on 
		//     the server.  this requires us to load both the direct users
		//     and direct child groups from the server here.  
		IPTUserGroup pUserGroup = (IPTUserGroup) m_ptObject;
		int nObjectID = ((IPTServerContext) m_ptObject).GetObjectID();
		IPTQueryResult rsChildGroups = pUserGroup.QueryDirectChildGroups();
		XPHashtable htChildGroupsOnServer = ConvertQRToHT(rsChildGroups);
		log.Debug("CalculateMembershipDeltas found {0} child groups on server for group {1}", htChildGroupsOnServer.GetSize(), nObjectID);
		rsChildGroups = null;
		
		IPTQueryResult rsChildUsers = pUserGroup.QueryUsers();
		XPHashtable htChildUsersOnServer = ConvertQRToHT(rsChildUsers);
		log.Debug("CalculateMembershipDeltas found {0} child users on server for group {1}", htChildUsersOnServer.GetSize(), nObjectID);
		rsChildUsers = null;
		
		// aj: now iterate through the UI's data and calculate deltas
		int nClassID;
		PTGrowableSortedArrayWrapper saData = 
			((SelectPTObjectModel) m_asOwner.GetModel(GroupEditorAS.strStorageMemberSelectPTObject)).GetItems();
		log.Debug("CalculateMemebershipDeltas found a total of {0} child user and group memberships for group {1}", saData.GetCount(), nObjectID);
		
		// aj: in the UI there is only one set of data for both child users and groups.
		//     Here we sift through that, and check the server hashtables to identify
		//     users that exist in the UI, but not the server.  These users will be added
		//     to the Add hashtables we were passed.  We will remove users from the server
		//     hashtables that we see in both the UI and the Server.  This leaves the server
		//     hashtables with the set of users/groups to delete...
		for (int ii = 0; ii < saData.GetCount(); ii++)
		{
			// retrieve the class ID
			nClassID = XPConvert.ToInteger(saData.GetItem(ii,
					SelectPTObjectModel.EXPLIST_COL_CLASSID));
			Integer iObjectID = (Integer) saData.GetItem(ii,
					SelectPTObjectModel.EXPLIST_COL_OBJECTID);

			if (nClassID == PT_CLASSIDS.PT_USER_ID)
			{
				// aj: we've got a user - check to see if the user is in
				//     the server as well
				if(htChildUsersOnServer.ContainsKey(iObjectID))
				{
					// aj: user is on server as well as UI - remove it
					//     from the server HT
					htChildUsersOnServer.RemoveElement(iObjectID);
				} else {
					// aj: user is NOT on the server, but is in UI - add it
					//     to the add list
					xpalUsersToAdd.Add(iObjectID);
				}
			}
			else if (nClassID == PT_CLASSIDS.PT_USERGROUP_ID)
			{
				// aj: we've got a group - check to see if the group is in
				//     the server as well
				if(htChildGroupsOnServer.ContainsKey(iObjectID))
				{
					// aj: group is on server as well as UI - remove it
					//     from the server HT
					htChildGroupsOnServer.RemoveElement(iObjectID);
				} else {
					// aj: group is NOT on the server, but is in UI - add it
					//     to the add list
					xpalGroupsToAdd.Add(iObjectID);
				}
			}
		}
		log.Info("CalculateMembershipDeltas identified {0} user memberships to add to group {1}", xpalUsersToAdd.GetSize(), nObjectID);
		log.Info("CalculateMembershipDeltas identified {0} group memberships to add to group {1}", xpalGroupsToAdd.GetSize(), nObjectID);
		
		// aj: we've now got a bunch of users/groups left in the
		//     child hashtables for the server.  these memberships
		//     should be deleted since they are only on the server...
		for(IXPEnumerator xpenum = htChildUsersOnServer.GetKeys(); xpenum.MoveNext(); )
		{
			xpalUsersToDelete.Add(xpenum.GetCurrent());
		}
		log.Info("CalculateMembershipDeltas identified {0} user memberships to delete from group {1}", xpalUsersToDelete.GetSize(), nObjectID);
		
		for(IXPEnumerator xpenum = htChildGroupsOnServer.GetKeys(); xpenum.MoveNext(); )
		{
			xpalGroupsToDelete.Add(xpenum.GetCurrent());
		}
		log.Info("CalculateMembershipDeltas identified {0} group memberships to delete from group {1}", xpalGroupsToDelete.GetSize(), nObjectID);
	}
	
	/**
	 * Converts an XPArrayList to an int[] Array.
	 * 
	 * @param xpList - list to convert
	 * @return int[] of data from XPArrayList
	 */
	private int[] ConvertArrListToIntArr(XPArrayList xpList)
	{
		// aj: base-case, empty
		if(xpList == null || xpList.GetSize() == 0)
		{
			return (new int[0]);
		}
		
		// aj: otherwise build up the array
		int[] nArResult = new int[xpList.GetSize()];
		int nPos = 0;
		for(IXPEnumerator xpenum = xpList.GetEnumerator(); xpenum.MoveNext(); )
		{
			nArResult[nPos++] = XPConvert.ToInteger(xpenum.GetCurrent());
		}
		
		return nArResult;
	}
	
	/**
	 * Converts an IPTQueryResult pQR to an XPHashtable.
	 * 
	 * @param pQR - the IPTQueryResult to convert.
	 * @return an XPHashtable with the ObjectID column value as the key and element
	 */
	private XPHashtable ConvertQRToHT(IPTQueryResult pQR)
	{
		// aj: base-case empty
		if(pQR == null || pQR.RowCount() == 0)
		{
			return new XPHashtable();
		}
		
		// aj: otherwise populate the hashtable
		int nRows = pQR.RowCount();
		XPHashtable xpHT = new XPHashtable(nRows);
		for(int i=0; i<nRows; i++)
		{
			// aj: object id should be the key and element
			int nObjectID = pQR.ItemAsInt(i, PT_PROPIDS.PT_PROPID_OBJECTID);
			Integer iObjectID = new Integer(nObjectID);
			xpHT.PutElement(iObjectID, iObjectID);
		}
		
		return xpHT;
	}

	/**
	 * Code executed before the object is stored.
	 */
	public void DoTaskBeforeStore()
	{
		// The Everyone group contains all the users in the portal.  The memberships cannot be edited.
		if (this.m_nObjectID !=	PT_INTRINSICS.PT_USERGROUP_EVERYONE)
		{
			// aj: get the group
			IPTUserGroup ptGroup  = (IPTUserGroup) m_ptObject;
			
			// aj: only save child members if this is a native group, remote
			//     groups are RO for this...
			if(!IsRemoteGroup()){
				// ***********************************
				// ** SAVE Dynamic Membership Rules **
				// ***********************************
				
				// aj: also only done to non-remote, non-everyone groups
				ptGroup.SetDynamicMembershipRule(m_Filter);
				
				// aj: do not trigger a group update here, the update agent
				//     will do that...
			}
		}
	}

	/**
	 * Code executed when the Editor is starting.
	 *
	 * @param _htFormData
	 */
	public void DoTaskOnStartEditor(XPHashtable _htFormData)
	{		
		//SE: logging
		log.FunctionBegin("GroupModel::DoTaskOnStartEditor");
		// get the group
		IPTUserGroup ptGroup = (IPTUserGroup) m_ptObject;
		
		// aj: set the remote flag immediately
		// aj: bug 46983, make this a greater than comparison so authsource id = 0,1 are
		//     treated as editable.  the everyone and administrator's groups are actually
		//     on authsourceid 0 (native) while intrinsice groups are on 1.
		if(ptGroup.GetAuthenticationSourceID() > PT_INTRINSICS.PT_AUTHSOURCE_PLUMTREE)
		{
			m_bIsRemoteGroup = true;
		}
		
		// aj: the everyone and administrator's groups should not have dynamic memberships.
		//     also, synchronized (remote) groups should not have dynamic memberships.
		if(m_bIsRemoteGroup ||
				ptGroup.GetObjectID() == PT_INTRINSICS.PT_USERGROUP_EVERYONE ||
				ptGroup.GetObjectID() == PT_INTRINSICS.PT_USERGROUP_SYSADMINS)
		{
			// aj: hide the dynamic membership rules link
			((EditorAS)m_asOwner).GetLeftMenu().HideLink(true, GroupDynamicRulesDP.STR_MVC_CLASS_NAME);
		} 
		
		// Bug 43496 - Trying to view a very large group (1,000,000 members) causes the portal to crash
		if (EditorStartControl.EDITOR_START_FLAG_EDIT == GetEditorType())
		{
			CheckForTooManyUsersToDisplay();
		}

		// initialize expandable lists		
		StartSelectMemberGroups();
		StartDynamicMemberListRO();
		StartSelectParentGroups();
		StartDynamicRulesChooser();
		StartSelectDynMemberPreviewPopup();
		StartSelectedRightsList();
		StartInheritedRightsList();

		// The Everyone group contains all the users in the portal.  The memberships cannot be edited.
		if (this.m_nObjectID ==	PT_INTRINSICS.PT_USERGROUP_EVERYONE)
		{
			((SelectPTObjectModel) m_asOwner.GetModel(GroupEditorAS.strStorageMemberSelectPTObject)).SetIsListReadOnly(true);
			((SelectPTObjectModel) m_asOwner.GetModel(GroupEditorAS.strStorageDynMemberSelectPTObject)).SetIsListReadOnly(true);
			((SelectPTObjectModel) m_asOwner.GetModel(GroupEditorAS.strStorageParentGroupSelectPTObject)).SetIsListReadOnly(true);
		}
			
		// aj: bug 45124, should set the hidden parent groups in all editor modes (create/edit)
		SetHiddenParentGroups();
		
		if (EditorStartControl.EDITOR_START_FLAG_EDIT == GetEditorType())
		{
			// since we're editing, load info
			LoadGroupInfo();
			SetHiddenMemberGroups();
		}
		else if (EditorStartControl.EDITOR_START_FLAG_CREATE == GetEditorType())
		{
			// no security page when creating
			((EditorAS) m_asOwner).GetLeftMenu().HideLink(true,
				ObjectSecurityDP.STR_MVC_CLASS_NAME);
			((EditorAS) m_asOwner).GetLeftMenu().HideLink(true,
				MigrationHistoryPluginDP.STR_MVC_CLASS_NAME);
		}
		//SE: logging
		log.FunctionEnd("GroupModel::DoTaskOnStartEditor");
	}

	/*
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}
	
	public boolean IsRemoteGroup()
	{
		return m_bIsRemoteGroup;
	}
	
	/*
	 * This function returns 'true' if the dynamic memberships
	 * for this group should be refreshed upon editor exit.
	 * Otherwise it returns 'false'.  The later is the
	 * default behavior. 
	 */
	protected boolean ShouldUpdateDynamicMembersOnFinish()
	{
		if(log.IsDebugEnabled())
		{
			log.Debug("GroupModel.ShouldUpdateDynamicMembersOnFinish(): value is \'{0}\' for group {1}.",
					XPConvert.ToString(m_bUpdateDynamicMembersOnFinish), m_nObjectID);
		}
		return m_bUpdateDynamicMembersOnFinish;
	}
	
	/*
	 * This function set a flag indicating that the dynamic
	 * memberships of this group should be updated upon
	 * editor exit.  This is only necessary when the rules
	 * defining the group have changed in some way. 
	 */
	protected void SetUpdateDynamicMembersOnFinish(boolean b)
	{
		m_bUpdateDynamicMembersOnFinish = b;
		
		if(log.IsDebugEnabled())
		{
			log.Debug("GroupModel.SetUpdateDynamicMembersOnFinish(): value was set to \'{0}\' for group {1}.",
					XPConvert.ToString(m_bUpdateDynamicMembersOnFinish), m_nObjectID);
		}
	}
	
	/**
	 *  Returns the current filter iterator.
	 *  @return the current filter iterator.
	 */
	public IFilterIteratorRO GetFilterIterator()
	{
		return m_FilterIterator;
	}

	public Object[][] LoadMembers() 
	{
		return LoadMembers(Global.ALL_DB_ROWS, 0);
	}

	public Object[][] LoadMembers(int nMaxRows, int nRowsToSkip) {	
		// get the group
		IPTUserGroup ptGroup = (IPTUserGroup) m_ptObject;
		
		// aj: get direct groups
		IPTQueryResult ptQRGroups = null; 
		int nNumUsers			  = 0;
		int nNumGroups			  = 0;

		// retrieve users and sub groups
		// aj: only include the manual members here
		IPTQueryResult ptQRUsers = ptGroup.QueryUsers(false, true, nMaxRows, nRowsToSkip);	

		if (null != ptQRUsers)
		{
			nNumUsers = ptQRUsers.RowCount();
		}

		// If number of users is greater than nMaxRows, no need to query group
		// for current paged data
		if ((nNumUsers < nMaxRows) || (Global.ALL_DB_ROWS == nMaxRows))
		{
			ptQRGroups = ptGroup.QueryDirectChildGroups();
		}
		
		if (null != ptQRGroups) 
		{
			nNumGroups = ptQRGroups.RowCount();			
		}

		// initialize building of data for SelectPTObject
		int nUserID;
		int nGroupID;
		String sUserName;
		String sGroupName;
		Object[][] arMembers = new Object[nNumUsers + nNumGroups][SelectPTObjectModel.EXPLIST_COL_MAX];

		// add users to data
		for (int ii = 0; ii < nNumUsers; ii++)
		{
			// retrieve from query results
			nUserID		  = ptQRUsers.ItemAsInt(ii,
					PT_PROPIDS.PT_PROPID_OBJECTID);
			sUserName     = ptQRUsers.ItemAsString(ii,
					PT_PROPIDS.PT_PROPID_NAME);
					
			// add login name, since that's what tree control returns now (bug 15574)
			sUserName = sUserName + " (" + ptQRUsers.ItemAsString(ii, PT_PROPIDS.PT_PROPID_USER_LOGINNAME) + ")";

			// build array for SelectPTObject
			arMembers[ii][SelectPTObjectModel.EXPLIST_COL_CLASSID]	    = new Integer(PT_CLASSIDS.PT_USER_ID);
			arMembers[ii][SelectPTObjectModel.EXPLIST_COL_OBJECTID]     = new Integer(nUserID);
			arMembers[ii][SelectPTObjectModel.EXPLIST_COL_STRING]	    = sUserName;
		}

		// add groups to data
		for (int ii = 0; ii < nNumGroups; ii++)
		{
			// retrieve from query results
			nGroupID	   = ptQRGroups.ItemAsInt(ii,
					PT_PROPIDS.PT_PROPID_OBJECTID);
			sGroupName     = ptQRGroups.ItemAsString(ii,
					PT_PROPIDS.PT_PROPID_NAME);

			// build array for SelectPTObject (SAVE AFTER users to prevent overwriting)
			arMembers[ii + nNumUsers][SelectPTObjectModel.EXPLIST_COL_CLASSID]	    = new Integer(PT_CLASSIDS.PT_USERGROUP_ID);
			arMembers[ii + nNumUsers][SelectPTObjectModel.EXPLIST_COL_OBJECTID]     = new Integer(nGroupID);
			arMembers[ii + nNumUsers][SelectPTObjectModel.EXPLIST_COL_STRING]	    = sGroupName;
		}
		
		return arMembers;
	}
	
	protected void CheckForTooManyUsersToDisplay() 
	{
		// Get MaxResultsToDisplay setting from portal config, if any
		String strMaxResults = PTConfigService.getPTInstance().GetConfigVariableAsString(
				PTConfigConstants.ADMINISTRATION_SECTION, 
				PTConfigConstants.MAX_RESULTS_TO_DISPLAY_SUBSECTION,
				PTConfigConstants.VALUE_ATTRIBUTE, m_asOwner);
		
		if (null != strMaxResults)
		{
			try {
				m_nMaxResultsToDisplay = XPConvert.ToInteger(strMaxResults);
			} catch (Exception e) {
				log.Info("Invalid setting value found in portal configuration: " + PTConfigConstants.MAX_RESULTS_TO_DISPLAY_SUBSECTION);
			}
		}
		
		// Check if we have too many users
		IPTUserGroup ptGroup = (IPTUserGroup) m_ptObject;
		
		if (null != ptGroup) 
		{
			m_nNumUsers = ptGroup.QueryUsersCount(false, true);
			
			if (m_nNumUsers <= m_nMaxResultsToDisplay) 
			{
				m_bDisplayUsers = true;
			} else {
				// Too many users to display
				m_bDisplayUsers = false;
				log.Info("Group being edited contains too many users to display.");
			}
		}
	}
	
	/**
	 * If we are editing an exisiting group, we need to load the group's data.
	 */
	public void LoadGroupInfo()
	{
		try
		{
			// SE: logging
			log.FunctionBegin("GroupModel::LoadGroupInfo");
			
			// get the group
			IPTUserGroup ptGroup = (IPTUserGroup) m_ptObject;

			// *************************
			// ** LOAD MANUAL MEMBERS **
			// *************************

			// retrieve users and sub groups
			// initialize building of data for SelectPTObject	

			// load data into SelectPTObject storage
			Object[][] arMembers = null;
			
			if (m_bDisplayUsers) 
			{
				arMembers = LoadMembers();
			}
			
			if (null != arMembers)
			{
				((SelectPTObjectModel) m_asOwner.GetModel(GroupEditorAS.strStorageMemberSelectPTObject)).SetDefaultObjects(arMembers);
			}
			
			// **************************
			// ** LOAD DYNAMIC MEMBERS **
			// **************************
			// retrieve users and sub groups
			// aj: only include the manual members here
			IPTQueryResult ptQRDynUsers  = ptGroup.QueryUsers(true, false);
			int nNumDynUsers			  = 0;

			if (null != ptQRDynUsers)
			{
				nNumDynUsers = ptQRDynUsers.RowCount();
			}

			// initialize building of data for SelectPTObject
			int nDynUserID;
			String sDynUserName;
			Object[][] arDynMembers = new Object[nNumDynUsers][SelectPTObjectModel.EXPLIST_COL_MAX];

			// add dynamic users to data
			for (int ii = 0; ii < nNumDynUsers; ii++)
			{
				// retrieve from query results
				nDynUserID		  = ptQRDynUsers.ItemAsInt(ii,
						PT_PROPIDS.PT_PROPID_OBJECTID);
				sDynUserName     = ptQRDynUsers.ItemAsString(ii,
						PT_PROPIDS.PT_PROPID_NAME);
						
				// add login name, since that's what tree control returns now (bug 15574)
				sDynUserName = sDynUserName + " (" + ptQRDynUsers.ItemAsString(ii, PT_PROPIDS.PT_PROPID_USER_LOGINNAME) + ")";

				// build array for SelectPTObject
				arDynMembers[ii][SelectPTObjectModel.EXPLIST_COL_CLASSID]	    = new Integer(PT_CLASSIDS.PT_USER_ID);
				arDynMembers[ii][SelectPTObjectModel.EXPLIST_COL_OBJECTID]     = new Integer(nDynUserID);
				arDynMembers[ii][SelectPTObjectModel.EXPLIST_COL_STRING]	    = sDynUserName;
			}

			// load data into SelectPTObject storage
			((SelectPTObjectModel) m_asOwner.GetModel(GroupEditorAS.strStorageDynMemberSelectPTObject)).SetDefaultObjects(arDynMembers);
			
			// ****************************************************
			// ** LOAD PARENT GROUPS AND NONDIRECT PARENT GROUPS **
			// ****************************************************

			// retrieve direct parent groups
			IPTQueryResult ptQRParentGroups = ptGroup.QueryDirectParentGroups();
			int nNumParentGroups = 0;

			if (null != ptQRParentGroups)
			{
				nNumParentGroups = ptQRParentGroups.RowCount();
			}

			// initialize building of data for SelectPTObject
			int nParentGroupID;
			String sParentGroupName;
			Object[][] arParentGroups = new Object[nNumParentGroups][SelectPTObjectModel.EXPLIST_COL_MAX];

			// add parent groups to data
			for (int ii = 0; ii < nNumParentGroups; ii ++)
			{
				// retrieve from query results
				nParentGroupID = ptQRParentGroups.ItemAsInt(ii, PT_PROPIDS.PT_PROPID_OBJECTID);
				sParentGroupName = ptQRParentGroups.ItemAsString(ii, PT_PROPIDS.PT_PROPID_NAME);

				// build array for SelectPTObject
				arParentGroups[ii][SelectPTObjectModel.EXPLIST_COL_CLASSID] = new Integer(PT_CLASSIDS.PT_USERGROUP_ID);
				arParentGroups[ii][SelectPTObjectModel.EXPLIST_COL_OBJECTID] = new Integer(nParentGroupID);
				arParentGroups[ii][SelectPTObjectModel.EXPLIST_COL_STRING] = sParentGroupName;
				
				// add to array list
				m_xpalOriginalParentGroups.Add(XPConvert.ToString(nParentGroupID));				
			}

			// load data into SelectPTObject storage
			((SelectPTObjectModel) m_asOwner.GetModel(GroupEditorAS.strStorageParentGroupSelectPTObject)).
				SetDefaultObjects(arParentGroups);

			// retrieve flattened parent groups
			IPTQueryResult ptQRFlattenedParentGroups = ptGroup.QueryFlattenedParentGroups();
			int nNumFlattenedParentGroups = 0;

			if (null != ptQRFlattenedParentGroups)
			{
				nNumFlattenedParentGroups = ptQRFlattenedParentGroups.RowCount();
			}

			// initialize building of data for SelectPTObject
			int nFlattenedParentGroupID;
			
			// add parent groups to data
			for (int ii = 0; ii < nNumFlattenedParentGroups; ii ++)
			{
				// retrieve from query results
				nFlattenedParentGroupID = ptQRFlattenedParentGroups.ItemAsInt(ii, PT_PROPIDS.PT_PROPID_OBJECTID);

				// add to our array lists
				m_xpalNonDirectParentGroups.Add(XPConvert.ToString(nFlattenedParentGroupID));
			}
			
			// remove each direct parent group to create nondirect parent groups.
			int removeGroupID;
			for (int ii = 0; ii < nNumParentGroups; ii ++)
			{
				removeGroupID = XPConvert.ToInteger(arParentGroups[ii][SelectPTObjectModel.EXPLIST_COL_OBJECTID]);
				m_xpalNonDirectParentGroups.Remove(XPConvert.ToString(removeGroupID));
			}

			// **************************
			// ** LOAD ACTIVITY RIGHTS **
			// **************************

			// retrieve activity rights
			IPTQueryResult ptQRRights = ptGroup.QueryAllActivityRights();
			int nNumRights			  = 0;

			if (null != ptQRRights)
			{
				nNumRights = ptQRRights.RowCount();
			}

			// initialize building of data for SelectList
			int nActivityID;
			String sRightName;
			Object[][] arRights = new Object[nNumRights][SelectActivityRightsModel.EXPLIST_COL_MAX];

			// add rights to data
			for (int ii = 0; ii < nNumRights; ii++)
			{
				// retrieve from query results
				nActivityID     = ptQRRights.ItemAsInt(ii,
						PT_PROPIDS.PT_PROPID_OBJECTID);
				sRightName	    = ptQRRights.ItemAsString(ii,
						PT_PROPIDS.PT_PROPID_NAME);

				// build array for SelectActivityRights
				arRights[ii][SelectActivityRightsModel.EXPLIST_COL_CLASSID]		 = new Integer(PT_CLASSIDS.PT_ACTIVITY_ID);
				arRights[ii][SelectActivityRightsModel.EXPLIST_COL_OBJECTID]     = new Integer(nActivityID);
				arRights[ii][SelectActivityRightsModel.EXPLIST_COL_STRING]		 = sRightName;
			}

			// load data into SelectActivityRights storage
			((SelectActivityRightsModel) m_asOwner.GetModel(GroupEditorAS.strStorageSelectActivityRights)).SetDefaultObjects(arRights);
			
			// retrieve inherited rights
			IPTQueryResult ptQRInheritedRights = ptGroup.QueryAllInheritedActivityRights(); 
			int nNumInheritedRights			  = 0;

			if (null != ptQRInheritedRights)
			{
				nNumInheritedRights = ptQRInheritedRights.RowCount();
			}

			// initialize building of data for SelectList
			Object[][] arInheritedRights = new Object[nNumInheritedRights][SelectActivityRightsModel.EXPLIST_COL_MAX];

			// add rights to data
			for (int ii = 0; ii < nNumInheritedRights; ii++)
			{
				// retrieve from query results
				nActivityID     = ptQRInheritedRights.ItemAsInt(ii,
						PT_PROPIDS.PT_PROPID_OBJECTID);
				sRightName	    = ptQRInheritedRights.ItemAsString(ii,
						PT_PROPIDS.PT_PROPID_NAME);

				// build array for SelectActivityRights
				arInheritedRights[ii][SelectActivityRightsModel.EXPLIST_COL_CLASSID]		 = new Integer(PT_CLASSIDS.PT_ACTIVITY_ID);
				arInheritedRights[ii][SelectActivityRightsModel.EXPLIST_COL_OBJECTID]     = new Integer(nActivityID);
				arInheritedRights[ii][SelectActivityRightsModel.EXPLIST_COL_STRING]		 = sRightName;
			}

			// load data into SelectActivityRights storage
			((SelectActivityRightsModel) m_asOwner.GetModel(GroupEditorAS.strStorageSelectInheritedRights)).SetDefaultObjects(arInheritedRights);			
			
			// SE: logging
			log.FunctionEnd("GroupModel::LoadGroupInfo");
		}
		catch (Exception ex)
		{
			
				log.Error("GroupModel::LoadGroupInfo Errors loading the group.  The following exception occured: " +
					ex);
			

			throw new XPException(m_asOwner.GetString(424,
					"ptmsgs_portaladminmsgs"), ex);
		}
	}

	/**
	 * Calls the right save function according to the page name passed in
	 * argument.
	 *
	 * @param _sPageName
	 * @param _htFormData
	 *
	 * @return int Page status for server side validation
	 */
	public int SavePage(String _sPageName, XPHashtable _htFormData)
	{
		// server validation
		int nPageStatus = super.SavePage(_sPageName, _htFormData);

		if (_sPageName.equals(GroupMainSettingsDP.STR_MVC_CLASS_NAME))
		{
			// save and return result
			nPageStatus = SaveMainSettingsPage(_htFormData);
		}
		else if (_sPageName.equals(GroupActivityRightsDP.STR_MVC_CLASS_NAME))
		{
			// save and return result
			nPageStatus = SaveActivityRightsPage(_htFormData);
		}
		else if (_sPageName.equals(GroupDynamicRulesDP.STR_MVC_CLASS_NAME))
		{
			// save and return result
			nPageStatus = SaveDynamicRulesPage(_htFormData);
		}

		return nPageStatus;
	}

	/**
	 * Helper method to start the Select List.
	 */
	public void StartSelectedRightsList()
	{
		SelectActivityRightsModel myActivityRightsModel = (SelectActivityRightsModel) m_asOwner.
			GetModel(GroupEditorAS.strStorageSelectActivityRights);

		// Initialize the Select Activity Rights list.
		//      _sSectionTitle: Main title.
		//      _sSectionSubTitle: Sub title.
		//      _imgAddIcon: Icon for the add action.
		//      _sAddLabel: Text to display next to the add icon.
		//      leave empty -- hard coded for activity rights
		//      _nListKey: In case you have multiple Expandable Lists on
		//                 the same DP, they should have a different key
		//                 (otherwise, just set it to 1).
		//		_bIsInherited: Are we dealing with inherited rights?
		myActivityRightsModel.InitializeSelectList(
			m_asOwner.GetString(415, "ptmsgs_portaladminmsgs"), 
			m_asOwner.GetString(416, "ptmsgs_portaladminmsgs"),
			PTImageType.ICON_ACTIVITY_RIGHT,
			m_asOwner.GetString(414, "ptmsgs_portaladminmsgs"), 
			new int[] {  }, 
			1,
			false);

		// message and instructure to display when list is empty
		myActivityRightsModel.SetEmptyListMessages(
			m_asOwner.GetString(417, "ptmsgs_portaladminmsgs"),
			m_asOwner.GetString(418, "ptmsgs_portaladminmsgs"));


		// bug 15921 - if the user doesn't have delegate rights, then
		// he cannot modify this list, so it should be read only for him.
		if (!m_Session.GetSessionInfo().CheckForActivityRight(
			PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_DELEGATE_RIGHTS))
		{
			myActivityRightsModel.SetIsListReadOnly(true);
			myActivityRightsModel.DisableAddButton();
		}
		
		// if we are editing the admin group then disable 
		// ability to edit activity rights.
		if (this.m_nObjectID == PT_INTRINSICS.PT_USERGROUP_SYSADMINS)
		{
			myActivityRightsModel.SetIsListReadOnly(true);
			myActivityRightsModel.DisableAddButton();
		} 		 
		
		// icon for activity rights		
		myActivityRightsModel.SetRowIcon(PTImageType.ICON_ACTIVITY_RIGHT);
	}

	/**
	 * Helper method to start the inherited rights select list.
	 */
	public void StartInheritedRightsList()
	{
		SelectActivityRightsModel myInheritedRightsModel = (SelectActivityRightsModel) m_asOwner.
			GetModel(GroupEditorAS.strStorageSelectInheritedRights);

		// Initialize the Select Activity Rights list.
		//      _sSectionTitle: Main title.
		//      _sSectionSubTitle: Sub title.
		//      _imgAddIcon: Icon for the add action.
		//      _sAddLabel: Text to display next to the add icon.
		//      leave empty -- hard coded for activity rights
		//      _nListKey: In case you have multiple Expandable Lists on
		//                 the same DP, they should have a different key
		//                 (otherwise, just set it to 1).
		//		_bIsInherited: Are we dealing with inherited rights?
		myInheritedRightsModel.InitializeSelectList(
			m_asOwner.GetString(2165, "ptmsgs_portaladminmsgs"), 
			m_asOwner.GetString(416, "ptmsgs_portaladminmsgs"),
			PTImageType.ICON_ACTIVITY_RIGHT,
			"", 
			new int[] {  }, 
			2,
			true);
			
		// inherited rights cannot be modified
		myInheritedRightsModel.SetIsListReadOnly(true);
				
		// icon for activity rights		
		myInheritedRightsModel.SetRowIcon(PTImageType.ICON_ACTIVITY_RIGHT);
	}

	/**
	 * Helper method to start the SelectPTObject List.
	 */
	public void StartSelectMemberGroups()
	{
		SelectPTObjectModel myMemberSelectPTObjectModel = (SelectPTObjectModel) m_asOwner.
			GetModel(GroupEditorAS.strStorageMemberSelectPTObject);

		// aj: if this is a remote group, this should be a RO object
		if(IsRemoteGroup())
		{
			// Initialize the Select PTObject list.
			//		 _sSectionTitle Main title.
			//		 _sSectionSubTitle Sub title
			//		 _arrObjectClassIDs Class ID of the Objects displayed on the list.
			//		 _nListKey In case you have multiple Expandable Lists on
			myMemberSelectPTObjectModel.InitializeReadOnlySelectPTObject(
				m_asOwner.GetString(410, "ptmsgs_portaladminmsgs"),
				m_asOwner.GetString(411, "ptmsgs_portaladminmsgs"),
				new int[] { PT_CLASSIDS.PT_USERGROUP_ID, PT_CLASSIDS.PT_USER_ID }, 
				3);
			
			// set the message and instructions to display when the list is empty.
			myMemberSelectPTObjectModel.SetEmptyListMessages(
				m_asOwner.GetString(412, "ptmsgs_portaladminmsgs"),
				// aj: indicate that they can't add new groups/users bc is remote
				m_asOwner.GetString(2707, "ptmsgs_portaladminmsgs"));
			
		} else {
			if (!m_bDisplayUsers) 
			{ 	// Too many users to display
				// Bug 43496 - Trying to view a very large group (1,000,000 members) causes the portal to crash
				myMemberSelectPTObjectModel.InitializeReadOnlySelectPTObject(
					m_asOwner.GetString(410, "ptmsgs_portaladminmsgs"),
					m_asOwner.GetString(411, "ptmsgs_portaladminmsgs"),
					new int[] { PT_CLASSIDS.PT_USERGROUP_ID, PT_CLASSIDS.PT_USER_ID }, 
					3);

				String sMsg = XPFormat.Format(m_asOwner.GetString(3220, "ptmsgs_portaladminmsgs"),
						new Object[]{XPConvert.ToString(m_nNumUsers)});
				
				// set the message and instructions to display when the list is empty.
				myMemberSelectPTObjectModel.SetEmptyListMessages(
					m_asOwner.GetString(3219, "ptmsgs_portaladminmsgs"), 
					sMsg);				
			} 
			else 
			{
				// Initialize the Select PTObject list.
				//      _sSectionTitle: Main title.
				//      _sSectionSubTitle: Sub title.
				//      _imgAddIcon: Icon for the add action.
				//      _sAddLabel: Text to display next to the add icon.
				//      _arrObjectClassIDs Class ID of the Objects displayed on the list.
				//      _nListKey: In case you have multiple Expandable Lists on
				//                 the same DP, they should have a different key
				//                 (otherwise, just set it to 1).
				myMemberSelectPTObjectModel.InitializeSelectPTObject(
					m_asOwner.GetString(410, "ptmsgs_portaladminmsgs"),
					m_asOwner.GetString(411, "ptmsgs_portaladminmsgs"),
					PTImageType.ICON_GROUP,
					m_asOwner.GetString(409, "ptmsgs_portaladminmsgs"),
					new int[] { PT_CLASSIDS.PT_USERGROUP_ID, PT_CLASSIDS.PT_USER_ID }, 
					3);
				
				// set the message and instructions to display when the list is empty.
				myMemberSelectPTObjectModel.SetEmptyListMessages(
					m_asOwner.GetString(412, "ptmsgs_portaladminmsgs"),
					// aj: provide instructions for adding new users/groups
					m_asOwner.GetString(413, "ptmsgs_portaladminmsgs"));
			}
		}
		
		// set title and subtitles
		myMemberSelectPTObjectModel.SetTreeTitle(m_asOwner.GetString(1830, "ptmsgs_portaladminmsgs"));
		myMemberSelectPTObjectModel.SetTreeSubTitle(m_asOwner.GetString(1831, "ptmsgs_portaladminmsgs"));
		
		// set icon for users
		myMemberSelectPTObjectModel.SetRowIconByClassID(PT_CLASSIDS.PT_USER_ID,
			PTImageType.ICON_USER);
		// set icon for groups
		myMemberSelectPTObjectModel.SetRowIconByClassID(PT_CLASSIDS.PT_USERGROUP_ID,
			PTImageType.ICON_GROUP);
			
		myMemberSelectPTObjectModel.EnablePagination(GroupEditorAS.strStorageMemberSelectPTObject);
	}
	
	/**
	 * Helper method to start the dynamic members list.
	 */
	public void StartDynamicMemberListRO()
	{
		SelectPTObjectModel myMemberSelectPTObjectModel = (SelectPTObjectModel) m_asOwner.
			GetModel(GroupEditorAS.strStorageDynMemberSelectPTObject);
		
		// aj: if this is a remote group, we should indicate it will not contain anything.
		//     we should also indicate this for the administrators and everyone groups...
		if(IsRemoteGroup() ||
				m_ptObject.GetObjectID() == PT_INTRINSICS.PT_USERGROUP_EVERYONE ||
				m_ptObject.GetObjectID() == PT_INTRINSICS.PT_USERGROUP_SYSADMINS)
		{
			// Initialize the Select PTObject list.
			//		 _sSectionTitle Main title.
			//		 _sSectionSubTitle Sub title
			//		 _arrObjectClassIDs Class ID of the Objects displayed on the list.
			//		 _nListKey In case you have multiple Expandable Lists on
			myMemberSelectPTObjectModel.InitializeReadOnlySelectPTObject(
				m_asOwner.GetString(2704, "ptmsgs_portaladminmsgs"),
				m_asOwner.GetString(411, "ptmsgs_portaladminmsgs"),
				new int[] { PT_CLASSIDS.PT_USERGROUP_ID, PT_CLASSIDS.PT_USER_ID }, 
				5);
		
			// set the message and instructions to display when the list is empty.
			myMemberSelectPTObjectModel.SetEmptyListMessages(
				m_asOwner.GetString(2705, "ptmsgs_portaladminmsgs"),
				m_asOwner.GetString(2706, "ptmsgs_portaladminmsgs"));
		} else {
			// Initialize the Select PTObject list.
			//		 _sSectionTitle Main title.
			//		 _sSectionSubTitle Sub title
			//		 _arrObjectClassIDs Class ID of the Objects displayed on the list.
			//		 _nListKey In case you have multiple Expandable Lists on
			myMemberSelectPTObjectModel.InitializeReadOnlySelectPTObject(
				m_asOwner.GetString(2704, "ptmsgs_portaladminmsgs"),
				m_asOwner.GetString(411, "ptmsgs_portaladminmsgs"),
				new int[] { PT_CLASSIDS.PT_USERGROUP_ID, PT_CLASSIDS.PT_USER_ID }, 
				5);
		
			// set the message and instructions to display when the list is empty.
			myMemberSelectPTObjectModel.SetEmptyListMessages(
				m_asOwner.GetString(2705, "ptmsgs_portaladminmsgs"),
				m_asOwner.GetString(2708, "ptmsgs_portaladminmsgs"));
		}

		// set icon for users
		myMemberSelectPTObjectModel.SetRowIconByClassID(
				PT_CLASSIDS.PT_USER_ID, PTImageType.ICON_USER);
			
		myMemberSelectPTObjectModel.EnablePagination(GroupEditorAS.strStorageDynMemberSelectPTObject);
	}

	/**
	 * Helper method to start the Select Parents Groups Expandable List.
	 */
	public void StartSelectParentGroups()
	{
		SelectPTObjectModel myParentGroupSelectPTObjectModel = (SelectPTObjectModel) m_asOwner.
			GetModel(GroupEditorAS.strStorageParentGroupSelectPTObject);

		// Initialize the Select PTObject list.
		//      _sSectionTitle: Main title.
		//      _sSectionSubTitle: Sub title.
		//      _imgAddIcon: Icon for the add action.
		//      _sAddLabel: Text to display next to the add icon.
		//      _arrObjectClassIDs Class ID of the Objects displayed on the list.
		//      _nListKey: In case you have multiple Expandable Lists on
		//                 the same DP, they should have a different key
		//                 (otherwise, just set it to 1).
		myParentGroupSelectPTObjectModel.InitializeSelectPTObject(
			m_asOwner.GetString(2429, "ptmsgs_portaladminmsgs"),
			m_asOwner.GetString(2430, "ptmsgs_portaladminmsgs"),
			PTImageType.ICON_GROUP,
			m_asOwner.GetString(2428, "ptmsgs_portaladminmsgs"),
			new int[] { PT_CLASSIDS.PT_USERGROUP_ID }, 
			4);

		// set the message and instructions to display when the list is empty.
		myParentGroupSelectPTObjectModel.SetEmptyListMessages(
			m_asOwner.GetString(2431, "ptmsgs_portaladminmsgs"),
			m_asOwner.GetString(2432, "ptmsgs_portaladminmsgs"));
		
		// set title and subtitles
		myParentGroupSelectPTObjectModel.SetTreeTitle(m_asOwner.GetString(2433, "ptmsgs_portaladminmsgs"));
		myParentGroupSelectPTObjectModel.SetTreeSubTitle(m_asOwner.GetString(2434, "ptmsgs_portaladminmsgs"));
		
		// set default icon
		myParentGroupSelectPTObjectModel.SetDefaultRowIcon(PTImageType.ICON_GROUP);
		
		myParentGroupSelectPTObjectModel.EnablePagination(GroupEditorAS.strStorageParentGroupSelectPTObject);
	}
	
	/**
	 * Helper method to start the dynamic members list.
	 */
	public void StartDynamicRulesChooser()
	{
		// aj: note - if this is a remote group, we won't need to init/display this object.
		//     same for everyone group and administrators group.
		if(IsRemoteGroup() ||
				m_ptObject.GetObjectID() == PT_INTRINSICS.PT_USERGROUP_EVERYONE ||
				m_ptObject.GetObjectID() == PT_INTRINSICS.PT_USERGROUP_SYSADMINS)
		{
			// aj: just return
			return;
		}

		// aj: otherwise load the filter
		m_Filter = ((IPTUserGroup)m_ptObject).GetDynamicMembershipRule();
		
		// aj: not all groups will have a filter, just create one if it doesn't exist
		if(m_Filter == null)
		{
			m_Filter = PortalObjectsFactory.CreateSearchFilter();	
		}
		UpdateFilterIterator();
	}
	
	private void UpdateFilterIterator()
	{
		// aj: create the iterator
		m_FilterIterator = new FilterIterator(m_Session, m_Filter, m_asOwner);
		
		// aj: ensure the iterator only presents user property IDs...
		m_FilterIterator.SetAllowablePropertyIDs(GetUserNonReferenceProperties(m_Session));
		m_FilterIterator.SetIncludeGroupMembership(true);
	}
	
	public Object[] GetUserNonReferenceProperties(IPTSession ptSession)
	{
		// aj: check if we've already cached the set of allowable properties
		if(m_oArNonRefPropIDs != null)
		{
			return m_oArNonRefPropIDs;
		}
		
		// aj: bug 42493, property ACLs and dgroup filters.  As per JohnK2, 
		//     we should show all properties that a user has SELECT access
		//     to as well as what is currently part of the group.  If a user
		//     has been granted admin access to the group - then they need
		//     to be able to see what the group does in its entirety, not
		//     just the limited view they are given...
		
		try
		{
			IPTObjectPropertyMap ptPropMap = (IPTObjectPropertyMap)
				ptSession.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_OBJECTPROPERTYMAP, false);

			IPTQueryResult ptQR =
				ptPropMap.QueryClassPropertyMappings(PT_CLASSIDS.PT_USER_ID, 
						PT_PROPIDS.PT_PROPID_OBJECTID | PT_PROPIDS.PT_PROPID_PROP_VALUETYPE);

			int nRowCount = ptQR.RowCount();
			
			// aj: put the valid props into our XPArrayList.  this includes:
			//	a) dates
			//	b) doubles
			//	c) longs
			//	d) membership
			//	e) string
			//	types only.  Encrypted, invalid and reference properties are not allowed.
			// note: as per bug 42493, this should be limited to the properties the
			//       user has SELECT access to and those that are already on the 
			//       dynamic group...
			XPArrayList xpAlUserValidProperties = new XPArrayList(nRowCount);
			for (int i = 0; i < nRowCount; i++)
			{
				int nPropType = ptQR.ItemAsInt(i, PT_PROPIDS.PT_PROPID_PROP_VALUETYPE);
				int nPropID = ptQR.ItemAsInt(i, PT_PROPIDS.PT_PROPID_OBJECTID);
				if((nPropType == PT_PROPERTY_TYPES.PT_PROPTYPE_DATE
						|| nPropType == PT_PROPERTY_TYPES.PT_PROPTYPE_DOUBLE
						|| nPropType == PT_PROPERTY_TYPES.PT_PROPTYPE_LONG
						|| nPropType == PT_PROPERTY_TYPES.PT_PROPTYPE_MEMBERSHIP
						|| nPropType == PT_PROPERTY_TYPES.PT_PROPTYPE_STRING)
						&& ptSession.GetSessionInfo().CheckPropertySearchRight(nPropID)
						)
				{
					xpAlUserValidProperties.Add(new Integer(nPropID));
				}
			}
			
			// aj: bug 42493, here we add the things that aren't already in the arraylist
			//     but are on the dynamic group
			FlattenedFilter ff = new FlattenedFilter(m_Filter);
			int nNumItems = ff.GetNumItems();
			for(int i=0; i<nNumItems; i++)
			{
				// aj: only look at statements
				if(ff.GetItem(i).GetType() == PT_FILTER_ITEM_TYPES.PT_FILTER_ITEM_STATEMENT)
				{
					IPTPropertyFilterStatement statement = (IPTPropertyFilterStatement) ff.GetItem(i);
					
					// aj: get the property id, ignore it if it's either invalid or the 
					//     member of property id
					int nPropID = statement.GetOperand();
					if(nPropID == PTUserGroup.MEMBER_OF_PROP_ID || nPropID < 0)
					{
						break;
					}
					
					// aj: now see if it's in the arraylist already
					int nNumCurrentProps = xpAlUserValidProperties.GetSize();
					boolean bFoundMatch = false;
					for(int j=0; j<nNumCurrentProps; j++)
					{
						int nCurrPropID = XPConvert.ToInteger(xpAlUserValidProperties.GetElement(j));
						if(nPropID == nCurrPropID)
						{
							bFoundMatch = true;
							break;
						}
					}
					
					// aj: if we didn't find the property in the list, add it
					if(!bFoundMatch)
					{
						xpAlUserValidProperties.Add(new Integer(nPropID));
					}
				}
			}
			
			
			// aj: we allocate one extra slot which we will use for the "Member Of" property
			Object[] oArUserProperties = new Object[xpAlUserValidProperties.GetSize() + 1];

			int nPos = 0;
			for (IXPEnumerator xpEnum = xpAlUserValidProperties.GetEnumerator(); xpEnum.MoveNext();)
			{
				oArUserProperties[nPos++] = xpEnum.GetCurrent();
			}
			
			// aj: insert our "Member Of" property ID
			oArUserProperties[xpAlUserValidProperties.GetSize()] = 
				new Integer(PTUserGroup.MEMBER_OF_PROP_ID);

			
			// aj: populate the cached copy and return it
			m_oArNonRefPropIDs = oArUserProperties;
			return m_oArNonRefPropIDs;

		}
		catch (Exception e)
		{
			
				log.Error(e, "An error occurred when trying get the list of valid user properties.");
			
		}

		return null;
	}
	
	/**
	 * Helper method to start the dynamic members list.
	 */
	public void StartSelectDynMemberPreviewPopup()
	{
		SelectPTObjectModel myMemberSelectPTObjectModel = (SelectPTObjectModel) m_asOwner.
			GetModel(GroupEditorAS.strStorageDynMemberPreviewPopup);

		// aj: initialize the select pt object...
		if(IsRemoteGroup() ||
				m_ptObject.GetObjectID() == PT_INTRINSICS.PT_USERGROUP_EVERYONE ||
				m_ptObject.GetObjectID() == PT_INTRINSICS.PT_USERGROUP_SYSADMINS)
		{
			// aj: if this is a remote group, do nothing since we can't get called.
			//     same for everyone and administrators group
		} else {
			// Initialize the Select PTObject list.
			//		 _sSectionTitle Main title.
			//		 _sSectionSubTitle Sub title
			//		 _arrObjectClassIDs Class ID of the Objects displayed on the list.
			//		 _nListKey In case you have multiple Expandable Lists on
			myMemberSelectPTObjectModel.InitializeReadOnlySelectPTObject(
				m_asOwner.GetString(2714, "ptmsgs_portaladminmsgs"),
				m_asOwner.GetString(411, "ptmsgs_portaladminmsgs"),
				new int[] { PT_CLASSIDS.PT_USERGROUP_ID, PT_CLASSIDS.PT_USER_ID }, 
				5);
		
			// set the message and instructions to display when the list is empty.
			myMemberSelectPTObjectModel.SetEmptyListMessages(
				m_asOwner.GetString(2716, "ptmsgs_portaladminmsgs"),
				m_asOwner.GetString(2717, "ptmsgs_portaladminmsgs"));
		}

		// set icon for users
		myMemberSelectPTObjectModel.SetRowIconByClassID(
				PT_CLASSIDS.PT_USER_ID, PTImageType.ICON_USER);
			
		myMemberSelectPTObjectModel.EnablePagination(GroupEditorAS.strStorageDynMemberPreviewPopup);
	}
	
	/**
	 * Set which groups we don't want to be able to add.
	 */
	public void SetHiddenMemberGroups()
	{
		// SE: logging
		log.FunctionBegin("GroupModel::SetHiddenMemberGroups");
		
		SelectPTObjectModel myMemberSelectPTObjectModel = (SelectPTObjectModel) m_asOwner.
			GetModel(GroupEditorAS.strStorageMemberSelectPTObject);
		
		// aj: dynamic group members do not contain subgroups, so no need to supress them here...

		PTGrowableSortedArrayWrapper saData;
		XPArrayList xpalParentGroups = new XPArrayList();

		// get selected parent group members
		saData = ((SelectPTObjectModel) m_asOwner.
			GetModel(GroupEditorAS.strStorageParentGroupSelectPTObject)).GetItems();

		// add them to our array
		for (int ii = 0; ii < saData.GetCount(); ii++)
		{
			xpalParentGroups.Add(saData.GetItem(ii,
				SelectPTObjectModel.EXPLIST_COL_OBJECTID));
		}

		// create the hidden groups array to be passed later
		// the size will be the number of selected parent groups,
		// the number of parent groups, and this group itself.
		int totalSize = xpalParentGroups.GetSize() + 
						m_xpalNonDirectParentGroups.GetSize() + 
						1;
		int[][] aHiddenGroups = new int[totalSize][SelectPTObjectModel.HIDE_COL_MAX];

		// cannot add selected parent groups
		for (int ii = 0; ii < xpalParentGroups.GetSize(); ii ++)
		{
			aHiddenGroups[ii][SelectPTObjectModel.HIDE_COL_CLASSID]	= PT_CLASSIDS.PT_USERGROUP_ID;
			aHiddenGroups[ii][SelectPTObjectModel.HIDE_COL_OBJECTID] = XPConvert.ToInteger(xpalParentGroups.GetElement(ii));
		}
		
		// cannot add flattened parent groups
		for (int ii = 0; ii < m_xpalNonDirectParentGroups.GetSize(); ii ++)
		{
			aHiddenGroups[ii + xpalParentGroups.GetSize()]
				[SelectPTObjectModel.HIDE_COL_CLASSID]	= PT_CLASSIDS.PT_USERGROUP_ID;
			aHiddenGroups[ii + xpalParentGroups.GetSize()]
				[SelectPTObjectModel.HIDE_COL_OBJECTID] = XPConvert.ToInteger(m_xpalNonDirectParentGroups.GetElement(ii));
		}

		// cannot add yourself
		IPTUserGroup ptGroup  = (IPTUserGroup) m_ptObject;
		IPTServerContext ptSC = (IPTServerContext) ptGroup;
		aHiddenGroups[totalSize - 1][SelectPTObjectModel.HIDE_COL_CLASSID]	= PT_CLASSIDS.PT_USERGROUP_ID;
		aHiddenGroups[totalSize - 1][SelectPTObjectModel.HIDE_COL_OBJECTID] = ptSC.GetObjectID();

		myMemberSelectPTObjectModel.ClearHiddenObjects();
		myMemberSelectPTObjectModel.SetHiddenObjects(aHiddenGroups);

//		XPSystem.Out.WriteLine("HIDDEN_MEMBER_GROUPS:" + myMemberSelectPTObjectModel.GetHiddenObjects());

		// SE: logging
		log.FunctionEnd("GroupModel::SetHiddenMemberGroups");
	}

	/**
	 * Set which groups we don't want to be able to add.
	 */
	public void SetHiddenParentGroups()
	{
		SelectPTObjectModel myParentGroupSelectPTObjectModel = (SelectPTObjectModel) m_asOwner.
			GetModel(GroupEditorAS.strStorageParentGroupSelectPTObject);

		int nClassID;
		PTGrowableSortedArrayWrapper saData;
		XPArrayList xpalGroups = new XPArrayList();

		// get selected group members
		saData = ((SelectPTObjectModel) m_asOwner.
			GetModel(GroupEditorAS.strStorageMemberSelectPTObject)).GetItems();
		
		// aj: dynamic group members do not contain subgroups, so no need to supress them here...

		// add just the selected groups to our array
		for (int ii = 0; ii < saData.GetCount(); ii++)
		{
			nClassID = XPConvert.ToInteger(saData.GetItem(ii,
				SelectPTObjectModel.EXPLIST_COL_CLASSID));
	
			if (nClassID == PT_CLASSIDS.PT_USERGROUP_ID)
			{
				// this stores groups, to be used as argument for adding groups
				xpalGroups.Add(saData.GetItem(ii,
					SelectPTObjectModel.EXPLIST_COL_OBJECTID));
			}
		}

		// create the hidden groups array to be passed later
		// the size will be the number of selected groups,
		// the group itself, and read only groups.
		int totalSize = xpalGroups.GetSize() + 
						1 +
						1;
		int[][] aHiddenGroups = new int[totalSize][SelectPTObjectModel.HIDE_COL_MAX];

		// cannot add selected groups
		for (int ii = 0; ii < xpalGroups.GetSize(); ii ++)
		{
			aHiddenGroups[ii][SelectPTObjectModel.HIDE_COL_CLASSID]	= PT_CLASSIDS.PT_USERGROUP_ID;
			aHiddenGroups[ii][SelectPTObjectModel.HIDE_COL_OBJECTID] = XPConvert.ToInteger(xpalGroups.GetElement(ii));
		}

		// cannot add yourself
		IPTUserGroup ptGroup  = (IPTUserGroup) m_ptObject;
		IPTServerContext ptSC = (IPTServerContext) ptGroup;
		aHiddenGroups[totalSize - 2][SelectPTObjectModel.HIDE_COL_CLASSID] = PT_CLASSIDS.PT_USERGROUP_ID;
		aHiddenGroups[totalSize - 2][SelectPTObjectModel.HIDE_COL_OBJECTID] = ptSC.GetObjectID();

		// cannot add everyone group (read-only)			
		aHiddenGroups[totalSize - 1][SelectPTObjectModel.HIDE_COL_CLASSID]	= PT_CLASSIDS.PT_USERGROUP_ID;
		aHiddenGroups[totalSize - 1][SelectPTObjectModel.HIDE_COL_OBJECTID] = PT_INTRINSICS.PT_USERGROUP_EVERYONE;

		myParentGroupSelectPTObjectModel.ClearHiddenObjects();
		myParentGroupSelectPTObjectModel.SetHiddenObjects(aHiddenGroups);				

//		XPSystem.Out.WriteLine("HIDDEN_PARENT_GROUPS:" + myParentGroupSelectPTObjectModel.GetHiddenObjects());
	}
	
	/**
	 * Saves data on Activity Rights page.
	 *
	 * @param _htFormData
	 *
	 * @return int Page status for server side validation
	 */
	protected int SaveActivityRightsPage(XPHashtable _htFormData)
	{
		// server validation
		int nPageStatus = RepostControl.PAGE_STATUS_VALID;

		((SelectActivityRightsModel) m_asOwner.GetModel(GroupEditorAS.strStorageSelectActivityRights)).SaveSection(_htFormData);

		return nPageStatus;
	}

	/**
	 * Saves data on Main Settings page.
	 *
	 * @param _htFormData
	 *
	 * @return int Page status for server side validation
	 */
	protected int SaveMainSettingsPage(XPHashtable _htFormData)
	{
		// server validation
		int nPageStatus = RepostControl.PAGE_STATUS_VALID;

		// aj: no need to save the dynamic groups expandable list, it is rule generated...
		
		// aj: if we are remote, don't save the member select part, it is RO
		if(!IsRemoteGroup())
		{
			((SelectPTObjectModel) m_asOwner.GetModel(GroupEditorAS.strStorageMemberSelectPTObject)).SaveSection(_htFormData);
		}
		((SelectPTObjectModel) m_asOwner.GetModel(GroupEditorAS.strStorageParentGroupSelectPTObject)).SaveSection(_htFormData);
		
		return nPageStatus;
	}
	
	/**
	 * Saves data on Dynamic Membership Rules page.
	 *
	 * @param _htFormData
	 *
	 * @return int Page status for server side validation
	 */
	protected int SaveDynamicRulesPage(XPHashtable _htFormData)
	{
		// server validation
		int nPageStatus = RepostControl.PAGE_STATUS_VALID;

		if(IsRemoteGroup() ||
				m_ptObject.GetObjectID() == PT_INTRINSICS.PT_USERGROUP_EVERYONE ||
				m_ptObject.GetObjectID() == PT_INTRINSICS.PT_USERGROUP_SYSADMINS)
		{
			// aj: if we are remote, there is nothing to save.  if native
			//     deal with the filter. same for everyone and administrators
			//     group - no data to save here.
		} else {
			// aj: rebulid the filter from our form
			IPTFilter filter = FilterForm.BuildFilterFromForm(m_asOwner, _htFormData);

			// If the form contained a filter
			if (null != filter)
			{
				// aj bug 45737: should check the number of clauses in the filter and make
				//    sure it doesn't exceed 10 total...
				FlattenedFilter ff = new FlattenedFilter(filter);
				if(CountNumStatements(ff) > 10)
				{
					// aj: one or the other must exist in our string, throw an error
					UIError myError = new UIError(UIErrorLevel.FRIENDLY, 
					        UIErrorScope.CURRENT_USER, null, 
							m_asOwner.GetString(3174, ResourceConstants.PORTALADMINMSGS), 
					        m_asOwner.GetString(3174, ResourceConstants.PORTALADMINMSGS)); 
					UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), 
					       m_asOwner.GetApplication());
					
					// aj: set error - but continue so the user doesn't lose
					//     the rules they just set up...
					nPageStatus = RepostControl.PAGE_STATUS_INVALID;
				}
				
				// replace our current filter with the new one
				m_Filter = filter;
			}

			// Make sure our FilterIterator is in sync with our filter
			m_FilterIterator.InitFromFilter(m_Filter);
			
			// aj: indicate that we've changed the filter.  this means,
			//     we need to update the dynamic members on finish...
			SetUpdateDynamicMembersOnFinish(true);
		}

		return nPageStatus;
	}
	
	private int CountNumStatements(FlattenedFilter ff)
	{
		int nNumStatements = ff.GetNumItems();
		for(int n=0; n<ff.GetNumItems(); n++)
		{
			// aj: decrement the number of statements each time we find a clause...
			if(ff.GetItem(n).GetType() == PT_FILTER_ITEM_TYPES.PT_FILTER_ITEM_CLAUSES)
			{
				nNumStatements--;
			}
		}
		
		return nNumStatements;
	}
	
	public void PreviewMembershipFilter()
	{
		// SE: logging
		log.FunctionBegin("GroupModel::PreviewMembershipFilter");
		log.Debug("Space name: " + m_asOwner.GetName() + ", Space ID: " + m_asOwner.GetSpaceID() + ", Parent Space Name: " + m_asOwner.GetParentSpaceName() + ", Parent Space ID: " + m_asOwner.GetParentSpaceID());
		
		IPTUserGroup ptGroup = (IPTUserGroup)m_ptObject;
		
		// aj: call preview with out current filter...
		IPTQueryResult ptQRDynUsers  = ptGroup.PreviewDynamicMembershipRule(m_Filter);
		int nNumDynUsers			  = 0;

		if (null != ptQRDynUsers)
		{
			nNumDynUsers = ptQRDynUsers.RowCount();
			log.Debug("Number of dynamic users from query: " + nNumDynUsers);
		}

		// initialize building of data for SelectPTObject
		int nDynUserID;
		String sDynUserName;
		Object[][] arDynMembers = new Object[nNumDynUsers][SelectPTObjectModel.EXPLIST_COL_MAX];

		// add dynamic users to data
		for (int ii = 0; ii < nNumDynUsers; ii++)
		{
			// retrieve from query results
			nDynUserID		  = ptQRDynUsers.ItemAsInt(ii,
					PT_PROPIDS.PT_PROPID_OBJECTID);
			sDynUserName     = ptQRDynUsers.ItemAsString(ii,
					PT_PROPIDS.PT_PROPID_NAME);
					
			// add login name, since that's what tree control returns now (bug 15574)
			sDynUserName = sDynUserName + " (" + ptQRDynUsers.ItemAsString(ii, PT_PROPIDS.PT_PROPID_USER_LOGINNAME) + ")";
			log.Debug("Processing result: " + sDynUserName + " to model");

			// build array for SelectPTObject
			arDynMembers[ii][SelectPTObjectModel.EXPLIST_COL_CLASSID]	    = new Integer(PT_CLASSIDS.PT_USER_ID);
			arDynMembers[ii][SelectPTObjectModel.EXPLIST_COL_OBJECTID]     = new Integer(nDynUserID);
			arDynMembers[ii][SelectPTObjectModel.EXPLIST_COL_STRING]	    = sDynUserName;
		}
		
		// aj: we may be on a re-preview, so we need to
		//     clear the items in the list first...
		((SelectPTObjectModel) m_asOwner.GetModel(GroupEditorAS.strStorageDynMemberPreviewPopup)).GetItems().ClearArray();
		
		// aj: reset the pagination
		int nListKey = ((SelectPTObjectModel) m_asOwner.GetModel(GroupEditorAS.strStorageDynMemberPreviewPopup)).GetListKey();
		((SelectPTObjectModel) m_asOwner.GetModel(GroupEditorAS.strStorageDynMemberPreviewPopup)).SetPagination(nListKey, -1, -1);

		// load data into SelectPTObject storage
		((SelectPTObjectModel) m_asOwner.GetModel(GroupEditorAS.strStorageDynMemberPreviewPopup)).SetDefaultObjects(arDynMembers);
		
		// SE: logging
		log.FunctionEnd("GroupModel::PreviewMembershipFilter");
	}
	
	/**
	 * Action for modifying a member of row on the Dynamic Membership Rules page.
	 *
	 * @param _htFormData Form data
	 */
	public void ModifyMemberOfRow(XPHashtable _htFormData)
	{
		// Get the hidden input with the row to modify
		String sDataKey = QueryCondition.HTMLHIDDEN_ROWINDEX;

		if (_htFormData.ContainsKey(sDataKey))
		{
			String[] aDataValue = (String[]) _htFormData.GetElement(sDataKey);

			if ((null != aDataValue) && (aDataValue.length > 0))
			{
				// aj: the row ID should have a pipe to split group
				//     from statement id.  need to split it up here and
				//     make sure we have at least one character on each side
				//     to convert.
				int nPipeLoc = XPStringUtility.IndexOf(aDataValue[0], '|');
				if(nPipeLoc < 1 || nPipeLoc > (aDataValue[0].length() - 2)){
					return;
				}
				
				String sGroupIndex = XPStringUtility.Substring(aDataValue[0], 0, nPipeLoc);
				int nGroupIndex = XPConvert.ToInteger(sGroupIndex);
				String sStatementIndex = XPStringUtility.Substring(aDataValue[0], nPipeLoc + 1);
				int nStatementIndex = XPConvert.ToInteger(sStatementIndex);

				// aj: now that we know what statement got altered, we need to
				//     figure out what value it got set to...
				ATreeAS asTree;
				if (m_asOwner.GetSubSpaceExists(QueryCondition.SUBSPACE_KEY_GROUP_CHOOSER))
				{
					asTree = (ATreeAS) m_asOwner.GetSubSpace(QueryCondition.SUBSPACE_KEY_GROUP_CHOOSER);

					ITreeModelRO myTree = (ITreeModelRO) asTree.GetModel(ATreeAS.TREEMODEL);
					XPArrayList mySelectedItems = myTree.GetSelectedItems();

					if (0 < mySelectedItems.GetSize())
					{
						// Get the first selected item
						String strGroupObjectName = ((TreeLeafNode) mySelectedItems.GetElement(0)).GetName();
						int nGroupObjectID = ((TreeLeafNode) mySelectedItems.GetElement(0)).m_nObjectID;

						// convert int to object
						Integer iGroupObjectID = new Integer(nGroupObjectID);

						// aj: update the appropriate row...s
						UpdateGroupMembershipRow(nGroupIndex, nStatementIndex, iGroupObjectID, strGroupObjectName);
					} else {
						// aj: should be able to set ID to null...
						UpdateGroupMembershipRow(nGroupIndex, nStatementIndex, null, null);
					}
				}
			}
		}
	}
	
	/**
	 * Updates the Filter and FilterIterator associated with this model.  Specifically
	 * updates the statement in the group specified to have a group membership id
	 * and string associated with it.
	 * 
	 * @param nGroupIndex
	 * @param nStatementIndex
	 * @param oGroupID
	 * @param sGroupName
	 */
	public void UpdateGroupMembershipRow(int nGroupIndex, int nStatementIndex, Object oGroupID, String sGroupName)
	{
		// aj: we 're going to update the filter in place, and then regenerate the filter iterator...
		//     first step: find the right group
		if(m_Filter == null)
		{
			return;
		}
		
		IPTPropertyFilterItem pPropFilter = m_Filter.GetPropertyFilter();
		
		// aj: stuff the propfilter and count into a wrapper obj
		IPTPropertyFilterItem pGroupPropFilter = FindNthClause(nGroupIndex, null, pPropFilter);
		if(pGroupPropFilter == null)
		{
			// aj: invalid request just return, don't update
			return;
		}
		
		// aj: now that we have the clause, we need to iterate to the correct statement
		if(pGroupPropFilter.GetType() == PT_FILTER_ITEM_TYPES.PT_FILTER_ITEM_CLAUSES)
		{
			IPTPropertyFilterClauses pGroupPropFilterClauses = (IPTPropertyFilterClauses) pGroupPropFilter;
			int nClauseCount = pGroupPropFilterClauses.GetCount();
			if(nStatementIndex > nClauseCount)
			{
				// aj: invalid request just return, don't update
				return;
			}
			
			IPTPropertyFilterItem pSubFilter = pGroupPropFilterClauses.GetItem(nStatementIndex-1);
			if(pSubFilter.GetType() == PT_FILTER_ITEM_TYPES.PT_FILTER_ITEM_STATEMENT)
			{
				IPTPropertyFilterStatement pStatement = (IPTPropertyFilterStatement) pSubFilter;
				pStatement.SetValue(oGroupID);
				
				// aj: now that we've changed the filter, lets update the filter iterator
				UpdateFilterIterator();
			}
		}
	}

	private IPTPropertyFilterItem FindNthClause(int nClauseIndex, RecursiveFilterIndexWrapper pIndexWrapper, IPTPropertyFilterItem pPropFilter)
	{
		// aj: first time through, init the wrapper for our curr index
		if(pIndexWrapper == null)
		{
			pIndexWrapper = new RecursiveFilterIndexWrapper();
		}

		// aj: exit condition
		if(nClauseIndex == pIndexWrapper.nFilterIndex)
		{
			return pPropFilter;
		}
		
		int nPropFilterType = pPropFilter.GetType();
		if(nPropFilterType == PT_FILTER_ITEM_TYPES.PT_FILTER_ITEM_CLAUSES)
		{
			IPTPropertyFilterClauses pPropFilterClauses = (IPTPropertyFilterClauses) pPropFilter;
			
			// aj: since we have a set of clauses - find the nGroupIndex'd one
			//     by doing a left to right crawl...
			for(int i=0; i<pPropFilterClauses.GetCount(); i++)
			{
				IPTPropertyFilterItem pSubItem = pPropFilterClauses.GetItem(i);
				if(pSubItem.GetType() == PT_FILTER_ITEM_TYPES.PT_FILTER_ITEM_CLAUSES)
				{
					pIndexWrapper.nFilterIndex++;
					
					// aj: get the result, see if it matches our clause index.  if it does,
					//     pass it back up the recursive call chain...
					IPTPropertyFilterItem pCandidate = FindNthClause(nClauseIndex, pIndexWrapper, pSubItem);
					if(nClauseIndex == pIndexWrapper.nFilterIndex)
					{
						return pCandidate;
					}
				}
			}
		}
		
		// aj: if we've got a statement, and it didn't match the clause index
		//     piece above, then it's a no match, so just follow default case
		//     below.
		
		// aj: assume we couldn't find it...
		return null;
	}
}

class RecursiveFilterIndexWrapper
{
	int nFilterIndex = 1;
}
