package com.plumtree.portalpages.admin.objcommon;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.admin.objmgr.*;
import com.plumtree.portalpages.browsing.search.results.*;
import com.plumtree.portalpages.common.constants.*;
import com.plumtree.portalpages.pei.*;
import com.plumtree.portaluiinfrastructure.classtypedescriptors.*;
import com.plumtree.portaluiinfrastructure.login.*;
import com.plumtree.portaluiinfrastructure.resultwrapper.*;
import com.plumtree.portaluiinfrastructure.search.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.tree.*;
import com.plumtree.uiinfrastructure.web.*;

/**
 * @author Matt Fryer
 *
 * Helper class that ObjMgrModel and AdminSearchModel use.
 */
public class AdminModelHelper
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
		"com.plumtree.portalpages.admin.objcommon.AdminModelHelper");

	public static final String STR_MVC_CLASS_NAME = "AdminSearch";

	private AActivitySpace m_asOwner = null;

	private IPTAdminCatalog m_ptCatalog = null;

	private static final int m_RootFolderID = 1;

	/**
	 * pt query filters cannot take more than 255 objects in the IN clause
	 * of a query filter.
	 */
	private static final int MAX_OBJECTS_IN_FILTER = 250;
	
	/** map of class IDs to Obj Mgrs */
	private XPHashtable m_xpmObjMgrs;
	
	/** Object array of all ObjectActions PEIs */
	private Object[] m_oObjectActions = null;
	
	private String m_strExpandedObjectTypes = null;
	
	private int[] m_nObjectIDPairsForDeleteCheck = null;
	
	private boolean m_bFoldersForDeleteCheck = false;

	/** list of job IDs for ABO deletes. */
	private int[] m_nABOJobIDs;

	public AdminModelHelper()
	{
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#CleanupTempData()
	 */
	public void CleanupTempData()
	{
		m_nObjectIDPairsForDeleteCheck     = null;

	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#CleanupAllData()
 	*/	
	public void CleanupAllData()
	{
	}

	/**
	 * This method copies folders to a new folder.
	 *
	 * @param int nTargetFolderID the ID of the folder to copy the folders to.
	 * @param int[] nFolders an array of folder IDs.
	 */
	public void CopyFolders(int nTargetFolderID, int[] nFolders)
	{
		Object oObjectAction;
		IObjectActions iAction;
		
		if (nFolders.length <= 0)
		{
			return;
		}

		// JF- Error messages from all PEIs for all folders
		String sPEIErrorMsgs = null;

		String strErrors = null;

		for (int i = 0; i < nFolders.length; i++)
		{
			try
			{	
				//PEI : IObjectActions - BEGIN
				//BrianH: Implementation of the OnCopyAdminFolder function of the IObjectActions
				//For each admin folder that is being copied the functions will be called to determine if the folder
				//requested should be copied to the target folder. This will only be done if there
				//are implemented IObjectActions functions otherwise the normal processing will occur.
				//The normal processing will also occur if no errors are returned.
				if (m_oObjectActions != null)
				{
					//JF- Error messages from all PEIs for the current folder
					String sObjectErrorMsgs = null;

					for (int x = 0; x < m_oObjectActions.length; x++)
					{
						oObjectAction = m_oObjectActions[x];

						log.Debug("AdminSearchModel.CopyFolders - About to call OnCopyAdminFolder on an: " + oObjectAction);

						if(oObjectAction != null)
						{
							String sError;

							iAction = (IObjectActions) oObjectAction;
							sError = iAction.OnBeforeCopyAdminFolder(m_asOwner, 
								(IPTSession) m_asOwner.GetUserSession(), nFolders[i], nTargetFolderID);

							sObjectErrorMsgs = ConcatenateErrors(sObjectErrorMsgs, sError);
						}
					}

					if ((null != sObjectErrorMsgs) && (sObjectErrorMsgs.length() > 0))
					{
						sPEIErrorMsgs = ConcatenateErrors(sPEIErrorMsgs, sObjectErrorMsgs);
					}
					else
					{
						m_ptCatalog.CopyAdminFolder(nFolders[i], nTargetFolderID);
					}
				}
				else
				{
					m_ptCatalog.CopyAdminFolder(nFolders[i], nTargetFolderID);
				}
				//PEI : OnCopyAdminFolder END
			}
			catch (Exception e)
			{
				String strFullError = ConcatenateErrors(
					GetFolderErrorString(1256, nFolders[i]),
					ObjMgrModel.GetLocalizedErrorString(e, m_asOwner));

				log.Error(e, strFullError);

				strErrors = ConcatenateErrors(strErrors, strFullError);
			}
		}

		strErrors = ConcatenateErrors(strErrors, sPEIErrorMsgs);
		
		if ((null != strErrors) && (!"".equals(strErrors)))
		{
			RaiseError(strErrors);
		}
	}

	/**
	 * This method copies objects between folders.
	 *
	 * @param int nFolderID the ID of the folder to copy the objects to.
	 * @param XPHashtable a map of Integer Class IDs to int[] arrays of object IDs.
	 * @param int nCurrentFolderID -1 means all folders
	 */
	public void CopyObjectsBetweenFolders(int nFolderID, XPHashtable xpmObjects, int nCurrentFolderID)
	{
		if (xpmObjects.GetSize() == 0)
		{
			return;
		}

		if (nFolderID == PT_INTRINSICS.PT_ADMIN_FOLDER_ROOTFOLDER)
		{
			// Cannot copy objects to root folder
			RaiseError(m_asOwner.GetString(2158, "ptmsgs_portaladminmsgs"));
			return;
		}

		int nTargetFolderID = nFolderID;
		Object[][] ptqf		    = new Object[3][2];

		// Object IDs to look up
		ptqf[0][0]     = new Integer(PT_PROPIDS.PT_PROPID_OBJECTID);
		ptqf[1][0]     = new Integer(PT_FILTEROPS.PT_FILTEROP_IN);

		// ptqf[2][0] is filled in for each object manager with an array of object IDs
		// Security level

		ptqf[0][1]     = new Integer(PT_PROPIDS.PT_PROPID_ACCESSLEVEL);
		ptqf[1][1]     = new Integer(PT_FILTEROPS.PT_FILTEROP_GTE);

		// Assume that the user has select level access to the items that they want
		// to copy.
		// PAI - changed this to READ in ObjMgrModel because SELECT gave inconsistent behavior 
		//(didn't show errors with READ, but did show them with SELECT, when both aren't allowed)
		ptqf[2][1] = new Integer(PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_READ);

		ObjectManagerHolder curMgrHolder;
		int nCurClass;
		PTObjectManagerQueryWrapper ptomqr;
		IASQueryResult ptqr;
		Object nLocked;
		IPTAdminFolder targetFolder;
		
		Object oObjectAction;
		IObjectActions iAction;

		// Check if we are in a community
		boolean bMoveToCommunity = false;

		try
		{
			targetFolder = m_ptCatalog.OpenAdminFolder(nFolderID, false);
		}
		catch (Exception e)
		{
			String strErrorMessage = XPResourceManager.FormatMessage(
				m_asOwner.GetString(1254, "ptmsgs_portaladminmsgs"),
				new Integer(nFolderID));

			log.Error(e, strErrorMessage);

			RaiseError(strErrorMessage);

			return;
		}

		if (targetFolder.GetFolderType() == PT_ADMIN_FOLDER_TYPES.PT_ADMIN_FOLDER_TYPE_COMMUNITYFOLDER)
		{
			bMoveToCommunity = true;
		}

		boolean bUserError		  = false;
		boolean bSubPortalError  = false;

		// Display errors to the user
		String strErrors = null;

		IXPEnumerator classIDEnum = xpmObjects.GetKeys();
		IXPEnumerator objectEnum  = xpmObjects.GetElements();
		
		while (objectEnum.MoveNext() && classIDEnum.MoveNext())
		{
			nCurClass = XPConvert.ToInteger(classIDEnum.GetCurrent());

			if (nCurClass == PT_CLASSIDS.PT_USER_ID)
			{
				// We do not allow users to be copied because it doesn't
				// make any sense to copy users.  ErickS says there may
				// also be a security hole if we allow it.
				log.Error(m_asOwner.GetString(1257, "ptmsgs_portaladminmsgs"));
				bUserError = true;
			}
			else if (!bMoveToCommunity ||
					(nCurClass == PT_CLASSIDS.PT_USERGROUP_ID) ||
					(nCurClass == PT_CLASSIDS.PT_GADGET_ID) ||
					(nCurClass == PT_CLASSIDS.PT_PAGE_ID))
			{
				curMgrHolder     = GetObjMgrHolder(nCurClass);

				int[] objectIDs = (int[]) objectEnum.GetCurrent();
				int nCopiedObjects = 0;
				int nObjectsToCopy = objectIDs.length - nCopiedObjects;

				while (nObjectsToCopy > 0)
				{
					// Since we can only have 255 object IDs in an IN query filter clause,
					// we need to query separately for each group of approximately 255
					// objects.
					if (nObjectsToCopy > MAX_OBJECTS_IN_FILTER)
					{
						nObjectsToCopy = MAX_OBJECTS_IN_FILTER;
					}

					// Create a temporary array if we have too many objects to copy
					if (nObjectsToCopy != objectIDs.length)
					{
						int[] objectsToCopy = new int[nObjectsToCopy];
						int nOriginalObjectsIndex = nCopiedObjects;

						for (int i = 0; i < objectsToCopy.length; i++)
						{
							objectsToCopy[i] = objectIDs[nOriginalObjectsIndex];
							nOriginalObjectsIndex++;
						}

						ptqf[2][0] = objectsToCopy;
					} else
					{
						ptqf[2][0] = objectIDs;
					}
	
					ptomqr		   = new PTObjectManagerQueryWrapper(curMgrHolder.m_ptObjMgr);
	
					// TODO: is all this needed in the mask, or will the filter handle it?
					ptomqr.Query( /*PT_PROPIDS.PT_PROPID_ACCESSLEVEL | */
						PT_PROPIDS.PT_PROPID_LOCKED /*| PT_PROPIDS.PT_PROPID_OBJECTID*/,
						nCurrentFolderID, null, ptqf);
					ptqr = ptomqr;
	
					try
					{
						// JF- Error messages from all PEIs for all objects
						String sPEIErrorMsgs = null;
									  
						for (int i = 0; i < ptqr.GetCount(); i++)
						{
							nLocked = ptqr.GetFields(i,
									PT_PROPIDS.PT_PROPID_LOCKED);
		
							// If the object is unlocked (not sure if this is right, but it sometimes returns null.
							// Don't know if it will ever return 0 or not.  Just in case.
							if ((nLocked == null) || (XPConvert.ToInteger(nLocked) == 0))
							{
								//PEI : IObjectActions - BEGIN
								//BrianH: Implementation of the OnCopyObject function of the IObjectActions
								//For each class of objects the functions will be called to determine if the objects
								//requested should be copied to the target folder. This will only be done if there
								//are implemented IObjectActions functions otherwise the normal processing will occur.
								//The normal processing will also occur if no errors are returned.
								if (m_oObjectActions != null)
								{
									//JF- Error messages from all PEIs for the current object
									String sObjectErrorMsgs = null;
									
									for (int x = 0; x < m_oObjectActions.length; x++)
									{
										oObjectAction = m_oObjectActions[x];

										log.Debug("AdminSearchModel.CopyObjectsBetweenFolders - About to call OnCopyObject on an: " + oObjectAction);

										if(oObjectAction != null)
										{
											String sError;
		
											iAction = (IObjectActions) oObjectAction;
											sError = iAction.OnBeforeCopyObject(m_asOwner, (IPTSession) m_asOwner.GetUserSession(), 
												nCurClass, XPConvert.ToInteger(ptqr.GetFields(i, PT_PROPIDS.PT_PROPID_OBJECTID)), 
												nTargetFolderID);

											sObjectErrorMsgs = ConcatenateErrors(sObjectErrorMsgs, sError);
										 }
									}
									
									if ((null != sObjectErrorMsgs) && (sObjectErrorMsgs.length() > 0))
									{
										sPEIErrorMsgs = ConcatenateErrors(sPEIErrorMsgs, sObjectErrorMsgs);
									}
									else
									{
										// clone the object (returns the object ID, or potentially an error)
										curMgrHolder.m_ptObjMgr.Clone(XPConvert.ToInteger(ptqr.GetFields(
											i, PT_PROPIDS.PT_PROPID_OBJECTID)),
											nTargetFolderID);
									}
								}
								else
								{
									// clone the object (returns the object ID, or potentially an error)
									curMgrHolder.m_ptObjMgr.Clone(XPConvert.ToInteger(ptqr.GetFields(
										i, PT_PROPIDS.PT_PROPID_OBJECTID)),
										nTargetFolderID);
								}
								//PEI : ObjectActions : OnCopyObject END
							} else
							{
								// Object is locked, can't copy it.
								strErrors = ConcatenateErrors(strErrors,
									XPResourceManager.FormatMessage(
									m_asOwner.GetString(2370, "ptmsgs_portaladminmsgs"),
									XPConvert.ToString(ptqr.GetFields(i, PT_PROPIDS.PT_PROPID_NAME))));
							}
						}

						strErrors = ConcatenateErrors(strErrors, sPEIErrorMsgs);
					} catch (Exception e)
					{
						String strFullError = ConcatenateErrors(
							m_asOwner.GetString(2155, "ptmsgs_portaladminmsgs"),
							ObjMgrModel.GetLocalizedErrorString(e, m_asOwner));

						log.Error(e, strFullError);

						strErrors = ConcatenateErrors(strErrors, strFullError);
					}
	
					// Figure out how many objects there are left to copy
					nCopiedObjects += nObjectsToCopy;
					nObjectsToCopy = objectIDs.length - nCopiedObjects;
				}
			}
			else
			{
				String strErrorMessage = GetFolderErrorString(1253, nFolderID);

				log.Error(strErrorMessage);

				strErrors = ConcatenateErrors(strErrors, strErrorMessage);
			}
		}

		if (bUserError)
		{
			strErrors = ConcatenateErrors(strErrors,
				m_asOwner.GetString(1257, "ptmsgs_portaladminmsgs"));
		}

		if (bSubPortalError)
		{
			strErrors = ConcatenateErrors(strErrors,
				m_asOwner.GetString(2528, "ptmsgs_portaladminmsgs"));
		}

		// display objects that could not be copied or had errors
		if ((null != strErrors) && (!"".equals(strErrors)))
		{
			RaiseError(strErrors);
		}
	}

	/**
	 *
	 */
	public void DeleteCards(int[] cards, int[] folders)
	{
		IPTCatalog cat = ((IPTSession)m_asOwner.GetUserSession()).GetCatalog();
		int[] arg = new int[1];
		try
		{
			for(int i = 0; i < cards.length; i++)
			{
				arg[0] = cards[i];
				if(folders[i] == GroupedResultsViewHelper.MULTIFOLDER_CODE)
				{
					int[] multiFolder = GetAllFoldersForCard(cards[i]);
					for(int k = 0; k < multiFolder.length; k++)
					{
						cat.RemoveCards(multiFolder[k], arg);
					}
				}
				else
				{
					cat.RemoveCards(folders[i], arg);
				}
			}
		}
		catch (XPException xpe)
		{
			throw new XPException(STR_MVC_CLASS_NAME +
				"DeleteCards Error when deleting cards" + xpe.GetMessage(), xpe);
		}
	}
	
	protected int[] GetAllFoldersForCard(int cardId)
	{
		IPTCatalog cat	= ((IPTSession) m_asOwner.GetUserSession()).GetCatalog();
		IPTQueryResult res = cat.QueryCardPaths(cardId, true, true);
		int count = res.RowCount();
		int[] retval = new int[count];
		for (int i = 0; i < count; i++)
		{
			retval[i] = XPConvert.ToInteger(res.ItemAsObject(i, PT_PROPIDS.PT_PROPID_CARD_PARENTFOLDERID));
		}
		return retval;
	}
	
	/**
	 *
	 */
	public void DeleteDocumentFolders(int[] documentFolders)
	{
		try
		{
			int nNumFolders = documentFolders.length;

			for (int i = 0; i < nNumFolders; i++)
			{
				int nFolderID = documentFolders[i];
				((IPTSession)m_asOwner.GetUserSession()).GetCatalog().DeleteFolder(nFolderID, true); // recursive
			}
		}
		catch (Exception e)
		{
			throw new XPException(STR_MVC_CLASS_NAME +
				"DeleteDocumentFolders Error when deleting folders" +
				e.getMessage(), e);
		}
	}

	/**
	 * This method deletes folders.
	 *
	 * @param int[] the folder IDs to delete
	 */
	public void DeleteFolders(int[] nFolders)
	{
		String strErrors = null;
		
		// JF- Error messages from all PEIs for all folders
		String sPEIErrorMsgs = null;
		
		Object oObjectAction;
		IObjectActions iAction;

		for (int i = 0; i < nFolders.length; i++)
		{
			try
			{
				if (nFolders[i] <= PT_SERVERCONSTANTS.PT_MIN_OBJECTID)
				{
					if (nFolders[i] == PT_INTRINSICS.PT_ADMIN_FOLDER_INTRINSICOPS)
					{
						// special case message - can move but not delete
						strErrors = ConcatenateErrors(strErrors, m_asOwner.GetString(3125, ResourceConstants.PORTALADMINMSGS));
					} else if (nFolders[i] == PT_INTRINSICS.PT_ADMIN_FOLDER_DEFAULTSUBPORTAL)
					{
						// special case message - can move but not delete
						strErrors = ConcatenateErrors(strErrors, m_asOwner.GetString(3126, ResourceConstants.PORTALADMINMSGS));
					} else
					{
						// ignore intrinsic objects -- they can't be deleted
						strErrors = ConcatenateErrors(strErrors, m_asOwner.GetString(1980, ResourceConstants.PORTALADMINMSGS));
					}
				} else
				{
					//PEI : IObjectActions - BEGIN
					//BrianH: Implementation of the OnDeleteAdminFolder function of the IObjectActions
					//For each admin folder being deleted, the OnDeleteAdminFolder function will be processed
					//to see if the folder and its corresponding objects should be deleted. This will
					//only be done if there are implemented IObjectActions functions otherwise the normal
					//processing will occur. The normal processing will also occur if no errors are returned.
					if (m_oObjectActions != null)
					{
						// JF- Error messages from all PEIs for the current folder
						String sObjectErrorMsgs = null;
													  
						for (int x = 0; x < m_oObjectActions.length; x++)
						{
							oObjectAction = m_oObjectActions[x];

							log.Debug("AdminSearchModel.DeleteFolders - About to call OnDeleteAdminFolder on an: " + oObjectAction);

							if(oObjectAction != null)
							{
								String sError;

								iAction = (IObjectActions) oObjectAction;
								sError = iAction.OnBeforeDeleteAdminFolder(m_asOwner, 
									(IPTSession) m_asOwner.GetUserSession(), nFolders[i]);

								sObjectErrorMsgs = ConcatenateErrors(sObjectErrorMsgs, sError);
							 }
						}
						
						if ((null != sObjectErrorMsgs) && (sObjectErrorMsgs.length() > 0))
						{
							sPEIErrorMsgs = ConcatenateErrors(sPEIErrorMsgs, sObjectErrorMsgs);
						}
						else
						{
							m_ptCatalog.DeleteAdminFolder(nFolders[i]);
						}
					}
					else
					{
						m_ptCatalog.DeleteAdminFolder(nFolders[i]);
					}
					//PEI : ObjectActions : OnDeleteAdminFolder END	
				}
			}
			catch (Exception e)
			{
				String strFullError = ConcatenateErrors(
					GetFolderErrorString(1260, nFolders[i]),
					ObjMgrModel.GetLocalizedErrorString(e, m_asOwner));

				log.Error(e, strFullError);

				strErrors = ConcatenateErrors(strErrors, strFullError);
			}
		}

		strErrors = ConcatenateErrors(strErrors, sPEIErrorMsgs);

		if ((null != strErrors) && (!"".equals(strErrors)))
		{
			RaiseError(strErrors);
		}
	}

	/**
	 * @see com.plumtree.portalpages.admin.objmgr.IObjMgrModelRO#ABOJobIDsToDisplay()
	 */
	public boolean GetABOJobIDsToDisplay()
	{
		if (m_nABOJobIDs.length > 0)
		{
			for (int i = 0; i < m_nABOJobIDs.length; i++)
			{
				if (m_nABOJobIDs[i] != -1)
				{
					return true;
				}
			}
		}

		return false;
	}

	/**
	 * @see com.plumtree.portalpages.admin.objmgr.IObjMgrModelRO#GetABOJobIDs()
	 */
	public String[] GetABOJobNames(int[] nABOJobIDs)
	{
		String[] result = new String[nABOJobIDs.length];

		IPTObjectManager jobManager = ((IPTSession) m_asOwner.GetUserSession()).
			GetObjectManagers(PT_CLASSIDS.PT_JOB_ID);

		for (int i = 0; i < nABOJobIDs.length; i++)
		{
			if (nABOJobIDs[i] != -1)
			{
				// get the job's localized name
				IPTJob myTempJob = (IPTJob) jobManager.
					Open(nABOJobIDs[i], true);
				result[i] = myTempJob.GetLocalizedName();
			}
		}

		return result;
	}

	/**
	 * @see com.plumtree.portalpages.admin.objmgr.IObjMgrModelRO#GetABOJobIDs()
	 */
	public int[] GetABOJobIDs()
	{
		int[] result = m_nABOJobIDs;

		m_nABOJobIDs = new int[0];

		return result;
	}

	/**
	 * This method creates a job to delete folders using Asynchronous
	 * Browser Operations (ABO).
	 *
	 * @param int[] the folder IDs to delete
	 */
	public void ABODeleteFolders(int[] nFolders)
	{
		String strErrors = null;

		int[] nABOTempJobIDs = new int[m_nABOJobIDs.length + nFolders.length];
		
		// JF- When we create the array above, each cell contains the value 0 
		// and apparently we assume (see GetABOJobIDsToDisplay) that the default value is -1.
		// Without the loop below, if the PEI returns an error, the ABO confirmation pop-up 
		// will still be displayed. 
		for (int i = 0; i < nABOTempJobIDs.length; i ++)
		{
			nABOTempJobIDs[i] = -1;
		}
		
		for (int i = 0; i < m_nABOJobIDs.length; i++)
		{
			nABOTempJobIDs[i] = m_nABOJobIDs[i];
		}
		
		// JF- Error messages from all PEIs for all folders
		String sPEIErrorMsgs = null;

		for (int i = 0; i < nFolders.length; i++)
		{
			try
			{
				if (nFolders[i] <= PT_SERVERCONSTANTS.PT_MIN_OBJECTID)
				{
					// ignore intrinsic objects -- they can't be deleted
					strErrors = ConcatenateErrors(strErrors, m_asOwner.GetString(1980, "ptmsgs_portaladminmsgs"));
				} 
				else
				{
					//PEI : IObjectActions : OnBeforeCreateABOJob - BEGIN
					//JF- Implementation of the OnBeforeCreateABOJob function of the IObjectActions
					//For each admin folder being scheduled to be deleted, the OnBeforeCreateABOJob function 
					//will be processed to see if the folder and its corresponding objects should be deleted. 
					//This will only be done if there are implemented IObjectActions functions otherwise the normal
					//processing will occur. The normal processing will also occur if no errors are returned.
					if (m_oObjectActions != null)
					{
						//JF- Error messages from all PEIs for the current folder
						String sObjectErrorMsgs = null;
						
						Object oObjectAction;
						IObjectActions iAction;
								
						for (int x = 0; x < m_oObjectActions.length; x++)
						{
							oObjectAction = m_oObjectActions[x];

							log.Debug("AdminModelHelper.OnBeforeCreateABOJob - About to call OnBeforeCreateABOJob on an: " + oObjectAction);

							if(oObjectAction != null)
							{
								String sError;

								iAction = (IObjectActions) oObjectAction;
								sError = iAction.OnBeforeCreateABOJob(m_asOwner, 
									(IPTSession) m_asOwner.GetUserSession(), PT_CLASSIDS.PT_ADMIN_FOLDER_ID, nFolders[i]);

								sObjectErrorMsgs = ConcatenateErrors(sObjectErrorMsgs, sError);
							}
						}
						
						if ((null != sObjectErrorMsgs) && (sObjectErrorMsgs.length() > 0))
						{
							sPEIErrorMsgs = ConcatenateErrors(sPEIErrorMsgs, sObjectErrorMsgs);
						}
						else
						{
							nABOTempJobIDs[i] = CreateABOJobToDeleteFolder(nFolders[i]);
							
							//PEI : IObjectActions : OnAfterCreateABOJob - BEGIN
							for (int z = 0; z < m_oObjectActions.length; z++)
							{
								oObjectAction = m_oObjectActions[z];

								log.Debug("AdminModelHelper.OnAfterCreateABOJob - About to call OnAfterCreateABOJob on an: " + oObjectAction);

								if(oObjectAction != null)
								{
									iAction = (IObjectActions) oObjectAction;
									iAction.OnAfterCreateABOJob(m_asOwner, (IPTSession) m_asOwner.GetUserSession(),
										nABOTempJobIDs[i], PT_CLASSIDS.PT_ADMIN_FOLDER_ID, nFolders[i]);
								}
							}
							//PEI : IObjectActions : OnAfterCreateABOJob - END
						}
					}
					//PEI : IObjectActions : OnBeforeCreateABOJob - END
					else
					{
						nABOTempJobIDs[i] = CreateABOJobToDeleteFolder(nFolders[i]);
					}
				}
			} 
			catch (Exception e)
			{
				log.Error(e, "Unable to check PEIs for ABO folder delete.");
				nABOTempJobIDs[i] = -1;
			}

			if (nABOTempJobIDs[i] == -1)
			{
				String strErrorMessage = GetFolderErrorString(1261, nFolders[i]);

				log.Error(strErrorMessage);

				strErrors = ConcatenateErrors(strErrors, strErrorMessage);
			}
		}

		strErrors = ConcatenateErrors(strErrors, sPEIErrorMsgs);

		if ((null != strErrors) && (!"".equals(strErrors)))
		{
			RaiseError(strErrors);
		}

		m_nABOJobIDs = nABOTempJobIDs;
	}
	
	/**
	 * Helper method to create the ABO Job for the Folder.
	 * 
	 * @param _nFolderID
	 * @return int - ABO Job ID
	 */
	private int CreateABOJobToDeleteFolder(int _nFolderID)
	{			
		IPTAdminFolder ptFolder = m_ptCatalog.OpenAdminFolder(_nFolderID, true);
		ptFolder.SetJobType(PT_ADMIN_FOLDER_JOB_TYPES.PT_ADMIN_FOLDER_JOB_DELETEFOLDER);
		// Store the object on the server (before you schedule the job)
		IPTServerContext myContext = (IPTServerContext) ptFolder.GetInterfaces("IPTServerContext");
		myContext.Store();
		myContext.UnlockObject();
		IPTScheduledOperation ptOp = (IPTScheduledOperation) ptFolder.GetInterfaces("IPTScheduledOperation");
		
		return ptOp.ScheduleImmediateJob();
	}
	
	public void ABODeletePageTemplates(int[] nPageTemplates)
	{
		ObjectManagerHolder curMgrHolder = (ObjectManagerHolder) m_xpmObjMgrs.GetElement(new Integer(PT_CLASSIDS.PT_PAGETEMPLATE_ID));

		String strErrors = null;

		int[] nABOTempJobIDs = new int[m_nABOJobIDs.length + nPageTemplates.length];
		
		// JF- When we create the array above, each cell contains the value 0 
		// and apparently we assume (see GetABOJobIDsToDisplay) that the default value is -1.
		// Without the loop below, if the PEI returns an error, the ABO confirmation pop-up 
		// will still be displayed. 
		for (int i = 0; i < nABOTempJobIDs.length; i ++)
		{
			nABOTempJobIDs[i] = -1;
		}
			 
		for (int i = 0; i < m_nABOJobIDs.length; i++)
		{
			nABOTempJobIDs[i] = m_nABOJobIDs[i];
		}

		// JF- Error messages from all PEIs for all folders
		String sPEIErrorMsgs = null;

		for (int i = 0; i < nPageTemplates.length; i++)
		{
			try
			{
				if (nPageTemplates[i] <= PT_SERVERCONSTANTS.PT_MIN_OBJECTID)
				{
					// ignore intrinsic objects -- they can't be deleted
					strErrors = ConcatenateErrors(strErrors, m_asOwner.GetString(1980, "ptmsgs_portaladminmsgs"));
				} else
				{	
					//PEI : IObjectActions : OnBeforeCreateABOJob - BEGIN
					//JF- Implementation of the OnBeforeCreateABOJob function of the IObjectActions
					//For each page template being scheduled to be deleted, the OnBeforeCreateABOJob function 
					//will be processed to see if the folder and its corresponding objects should be deleted. 
					//This will only be done if there are implemented IObjectActions functions otherwise the normal
					//processing will occur. The normal processing will also occur if no errors are returned.
					if (m_oObjectActions != null)
					{
						// JF- Error messages from all PEIs for the current folder
						String sObjectErrorMsgs = null;
											  
						Object oObjectAction;
						IObjectActions iAction;
								
						for (int x = 0; x < m_oObjectActions.length; x++)
						{
							oObjectAction = m_oObjectActions[x];

							log.Debug("AdminModelHelper.OnBeforeCreateABOJob - About to call OnBeforeCreateABOJob on an: " + oObjectAction);

							if(oObjectAction != null)
							{
								String sError;

								iAction = (IObjectActions) oObjectAction;
								sError = iAction.OnBeforeCreateABOJob(m_asOwner, (IPTSession) m_asOwner.GetUserSession(),
									PT_CLASSIDS.PT_PAGETEMPLATE_ID, nPageTemplates[i]);

								sObjectErrorMsgs = ConcatenateErrors(sObjectErrorMsgs, sError);
							}
						}
						
						if ((null != sObjectErrorMsgs) && (sObjectErrorMsgs.length() > 0))
						{
							sPEIErrorMsgs = ConcatenateErrors(sPEIErrorMsgs, sObjectErrorMsgs);
						}
						else
						{
							nABOTempJobIDs[i] = CreateABOJobToDeletePageTemplate(curMgrHolder, nPageTemplates[i]);
							
							//PEI : IObjectActions : OnAfterCreateABOJob - BEGIN
							for (int z = 0; z < m_oObjectActions.length; z++)
							{
								oObjectAction = m_oObjectActions[z];

								log.Debug("AdminModelHelper.OnAfterCreateABOJob - About to call OnAfterCreateABOJob on an: " + oObjectAction);

								if(oObjectAction != null)
								{
									iAction = (IObjectActions) oObjectAction;
									iAction.OnAfterCreateABOJob(m_asOwner, (IPTSession) m_asOwner.GetUserSession(), 
										nABOTempJobIDs[i], PT_CLASSIDS.PT_PAGETEMPLATE_ID, nPageTemplates[i]);
								}
							}
							//PEI : IObjectActions : OnAfterCreateABOJob - END
						}
					}
					//PEI : IObjectActions : OnBeforeCreateABOJob - END
					else
					{
						nABOTempJobIDs[i] = CreateABOJobToDeletePageTemplate(curMgrHolder, nPageTemplates[i]);
					}
				}
			} catch (Exception e)
			{
				log.Error(e, "Error ABO deleting page templates.");
				nABOTempJobIDs[i] = -1;
			}

			if (nABOTempJobIDs[i] == -1)
			{
				String strError = XPResourceManager.FormatMessage(m_asOwner.GetString(2169, "ptmsgs_portaladminmsgs"),
					XPConvert.ToString(nPageTemplates[i]));

				log.Error(strError);

				strErrors = ConcatenateErrors(strErrors, strError);
			}
		}

		strErrors = ConcatenateErrors(strErrors, sPEIErrorMsgs);

		if ((null != strErrors) && (!"".equals(strErrors)))
		{
			RaiseError(strErrors);
		}

		m_nABOJobIDs = nABOTempJobIDs;
	}
	
	/**
	 * JF- Helper method to create the ABO Job for the Page Template.
	 * 
	 * @param _curMgrHolder
	 * @param _nPageTempID
	 * @return int - ABO Job ID
	 */
	private int CreateABOJobToDeletePageTemplate(ObjectManagerHolder _curMgrHolder, int _nPageTempID)
	{
		IPTPageTemplate ptPageTemplate = (IPTPageTemplate)_curMgrHolder.m_ptObjMgr.Open(_nPageTempID, true);
		ptPageTemplate.SetJobType(PT_PAGE_TEMPLATE_JOB_TYPES.PT_PAGE_TEMPLATE_JOB_DELETE_PAGETEMPLATE);
		// Store the object on the server (before you schedule the job)
		IPTServerContext myContext = (IPTServerContext) ptPageTemplate.GetInterfaces(
			"IPTServerContext");
		myContext.Store();
		myContext.UnlockObject();
		IPTScheduledOperation ptOp = (IPTScheduledOperation) ptPageTemplate.GetInterfaces(
			"IPTScheduledOperation");
		return ptOp.ScheduleImmediateJob();
	}

	/**
	 * This method takes a class type and adds it and it's dependent class types
	 * to a list, if they are present in the hashtable.  The list will be sorted
	 * with dependent class types (e.g. portlets) coming before the class types
	 * they depend on (e.g. Portlet Web Services).
	 * 
	 * This method only takes simple dependencies into account and cannot handle
	 * dependency loops.  It shouldn't go into an infinite loops, but the resulting
	 * order is undefined.
	 * 
	 * @param _nCurrentType The class type to check for dependent class types.
	 * @param _xpSortedTypes The List of class types sorted by dependency.
	 * @param _xpTypesToDelete The hashtable of types to delete.
	 * @param _xpInProcessTypes The List of types that are currently being processed.
	 * This is used to avoid infinite loops.  This can be null.
	 */
	public void AddDependentTypes(int _nCurrentType, XPArrayList _xpSortedTypes,
		XPHashtable _xpTypesToDelete, XPArrayList _xpInProcessTypes)
	{
		ClassTypeDescManager ptClassTypeDescManager = PTConfigHelper.GetPTClassTypeDescrManager();

		int nDependentIDs[] = ptClassTypeDescManager.GetDependentClassTypeIDs(_nCurrentType);

		if (null != nDependentIDs)
		{
			for (int i = 0; i < nDependentIDs.length; i++)
			{
				Integer nDependentID = new Integer(nDependentIDs[i]);
				// Only add the dependent type if we are going to delete objects of that type.
				if (_xpTypesToDelete.ContainsKey(nDependentID))
				{
					// If we haven't already processed this type
					if (!_xpSortedTypes.Contains(nDependentID))
					{
						if (null == _xpInProcessTypes)
						{
							_xpInProcessTypes = new XPArrayList();
						}

						// Make sure we're not in an infinite loop
						if (!_xpInProcessTypes.Contains(nDependentID))
						{
							// Check if there are any sub-dependent types
							// and add this dependent type.
							AddDependentTypes(nDependentIDs[i], _xpSortedTypes,
								_xpTypesToDelete, _xpInProcessTypes);
						}
					}
				}
			}
		}

		Integer nCurrentID = new Integer(_nCurrentType);

		// If we haven't already processed this type
		if (!_xpSortedTypes.Contains(nCurrentID))
		{
			// Add the current type to the dependent list
			_xpSortedTypes.Add(nCurrentID);
		}
	}

	/**
	 * This method deletes objects.
	 *
	 * @param XPHashtable a map of Integer Class IDs to int[] arrays of object IDs.
	 */
	public void DeleteObjects(XPHashtable xpmObjects, boolean bABODeleteObjects)
	{
		// In order to resolve problems with deleting objects with dependencies,
		// we first put the objects into a list sorted so as not to delete
		// dependent object types before the object types that they can depend on.

		XPArrayList sortedClassIDsToDelete = new XPArrayList(xpmObjects.GetSize());
		XPArrayList sortedObjectIDsToDelete = new XPArrayList(xpmObjects.GetSize());

		IXPEnumerator classIDEnum = xpmObjects.GetKeys();

		int nClassID;

		// Put the class ids into a sorted list
		while (classIDEnum.MoveNext())
		{
			nClassID = XPConvert.ToInteger(classIDEnum.GetCurrent());
			AddDependentTypes(nClassID, sortedClassIDsToDelete, xpmObjects, null);
		}

		// Transfer the object ID lists into a new list in the same sorted order
		for (int i = 0; i < sortedClassIDsToDelete.GetSize(); i++)
		{
			sortedObjectIDsToDelete.Add(xpmObjects.GetElement(sortedClassIDsToDelete.GetElement(i)));
		}

		String strErrors = null;

		ObjectManagerHolder curMgrHolder;
		int nCurClass;
		int[] nIDs;
		int[] nTempIDs;
		int nCount;

		Object oObjectAction;
		IObjectActions iAction;
		
		// JF- Error messages from all PEIs for all objects
		String sPEIErrorMsgs = null;

		for (int nDeleteIndex = 0; nDeleteIndex < sortedClassIDsToDelete.GetSize(); nDeleteIndex++)
		{
			nCurClass	= XPConvert.ToInteger(sortedClassIDsToDelete.GetElement(nDeleteIndex));
			nIDs	   	= (int[]) sortedObjectIDsToDelete.GetElement(nDeleteIndex);

			if (!bABODeleteObjects || (PT_CLASSIDS.PT_PAGETEMPLATE_ID != nCurClass))
			{
				curMgrHolder = GetObjMgrHolder(nCurClass);
				nCount = 0;
	
				for (int i = 0; i < nIDs.length; i++)
				{
					if (nIDs[i] >= PT_SERVERCONSTANTS.PT_MIN_OBJECTID)
					{
						nCount++;
					}
					else
					{
						// ignore intrinsic objects -- they can't be deleted
						strErrors = ConcatenateErrors(strErrors, m_asOwner.GetString(1980, "ptmsgs_portaladminmsgs"));
					}
				}
	
				if (nCount < nIDs.length)
				{
					nTempIDs     = new int[nCount];
					nCount		 = 0;
	
					for (int i = 0; i < nIDs.length; i++)
					{
						if ((nIDs[i] > PT_SERVERCONSTANTS.PT_MIN_OBJECTID) &&
								(nCount < nTempIDs.length))
						{
							nTempIDs[nCount] = nIDs[i];
							nCount++;
						}
					}
					nIDs = nTempIDs;
				}
				
				// Rob: Temporary overdrive fix for bug #25675
				// Need to delete a subportal's mandatory link sitemapfolder object too (if one exists)
				if ( nCurClass == PT_CLASSIDS.PT_SUBPORTAL_ID ) 
				{
					for ( int i = 0; i < nIDs.length; i++ ) {
						IPTSession ptSession = ((IPTSession)m_asOwner.GetUserSession());
						IPTSubPortal ptPortal = (IPTSubPortal)ptSession.GetSubPortals().Open(nIDs[i], false);
						int nSiteMapFolderID = ptPortal.GetMandatoryLinksID();
						if ( nSiteMapFolderID > PT_INTRINSICS.PT_NULL_OBJECTID )
						{
							try {
								log.Debug("DeleteObjects() - About to delete SiteMapFolder with ID: " + nSiteMapFolderID+" for SubPortal with ID: "+nIDs[i]);
								ptSession.GetSiteMapFolders().Delete(nSiteMapFolderID);
							} catch (Exception e) {
								// Ignore any errors with sitemapfolder delete, probably deleted already.
							}
						}
						
						// WilliamA - need to delete associated Guest Login object of this
						// subportal as well
						IPTObjectManager ptObjMgr = ptSession.GetGuestLogins();
						IPTGuestLoginManager ptGLMgr = (IPTGuestLoginManager)ptObjMgr;
						if (null != ptGLMgr)
						{
							int nGuestLoginID = ptGLMgr.GetGuestLoginIDFromSubPortalID(nIDs[i]);
							if (PT_INTRINSICS.PT_NULL_OBJECTID != nGuestLoginID)
							{
								ptObjMgr.Delete(nGuestLoginID);
							}
						}
					}
				}
	
				try
				{
					//PEI : IObjectActions - BEGIN
					//BrianH: Implementation of the OnBeforeDeleteObject function of the IObjectActions
					//For each class of objects and their corresponding integer array of object ids
					//the functions will be called to see if the objects should be deleted. This will
					//only be done if there are implemented IObjectActions functions otherwise the normal
					//processing will occur. The normal processing will also occur if no errors are returned.
					
					if (m_oObjectActions != null)
					{
						// JF- Array of objects that will actually be deleted 
						// (the OnBeforeDeleteObject PEI can prevent any object 
						// from being deleted by just returning a non empty String)
						XPArrayList alObjectIDsToDelete = new XPArrayList();
					
						for (int k = 0; k < nIDs.length; k ++)
						{
							// JF- Current Object ID
							int nObjectID = nIDs[k];
																 
							// JF- Error messages from all PEIs for the current object
							String sObjectErrorMsgs = null;
														  
							for (int x = 0; x < m_oObjectActions.length; x++)
							{
								oObjectAction = m_oObjectActions[x];

								log.Debug("DeleteObjects() - About to call OnBeforeDeleteObject on an: " + oObjectAction);

								if(oObjectAction != null)
								{
									String sError;
		
									iAction = (IObjectActions) oObjectAction;
									sError = iAction.OnBeforeDeleteObject(m_asOwner, 
										(IPTSession) m_asOwner.GetUserSession(), nCurClass, nObjectID);

									sObjectErrorMsgs = ConcatenateErrors(sObjectErrorMsgs, sError);
								}
							}
		
							if ((null != sObjectErrorMsgs) && (sObjectErrorMsgs.length() > 0))
							{
								sPEIErrorMsgs = ConcatenateErrors(sPEIErrorMsgs, sObjectErrorMsgs);
							}
							else
							{
								// JF- We got noe error message from the PEIs,
								// we add the object ID to the array of objects to delete.
								alObjectIDsToDelete.Add(new Integer(nObjectID));
							}
						}
						
						// JF- Number of objects to delete
						int nObjsToDeleteCount = alObjectIDsToDelete.GetSize();
						
						if (0 != nObjsToDeleteCount)
						{
							// JF- Copy the XPArrayList to an int array
							int[] arrObjectIDsToDelete = new int[nObjsToDeleteCount];
						
							for (int k = 0; k < nObjsToDeleteCount; k ++)
							{
								arrObjectIDsToDelete[k] = XPConvert.ToInteger(alObjectIDsToDelete.GetElement(k));
							}
						
							// JF- Delete the Objects
							Object[] deleteResults = curMgrHolder.m_ptObjMgr.DeleteObjects(arrObjectIDsToDelete);

							// Check the return code for all objects
							for (int i = 0; i < deleteResults.length; i++)
							{
								if (XPConvert.ToInteger(deleteResults[i]) != PT_RESULTCODES.PT_S_OK)
								{
									int nErrorCode = XPConvert.ToInteger(deleteResults[i]);

									String strGenericError;
									try
									{
										ClassTypeDescManager ptClassTypeDescManager = PTConfigHelper.GetPTClassTypeDescrManager();
										String strLangID = (String) PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingsHelper.LANGUAGE, m_asOwner);

										String strObjectType = ptClassTypeDescManager.GetClassTypeLocalizedName(
											nCurClass, strLangID);

										IPTQueryResult failedObject = curMgrHolder.m_ptObjMgr.QuerySingleObject(arrObjectIDsToDelete[i]);
										String strObjectName = failedObject.ItemAsString(0, PT_PROPIDS.PT_PROPID_NAME);

										strGenericError =
											XPResourceManager.FormatMessage(m_asOwner.GetString(3142, ResourceConstants.PORTALADMINMSGS),
											strObjectType, strObjectName);
									} catch (Exception e)
									{
										log.Error(e, "Unable to open object to get object name for delete error message.");
										strGenericError = m_asOwner.GetString(2154, "ptmsgs_portaladminmsgs");
									}

									String strSpecificError = ObjMgrModel.GetLocalizedErrorString(nErrorCode, m_asOwner);

									String strFullError = ConcatenateErrors(strGenericError, strSpecificError);

									strErrors = ConcatenateErrors(strErrors, strFullError);
								}
							}
						}
						//PEI : ObjectActions : OnBeforeDeleteObject END
					}
					else
					{
						curMgrHolder.m_ptObjMgr.DeleteObjects(nIDs);
					}
				}
				catch (Exception e)
				{
					String strFullError = ConcatenateErrors(m_asOwner.GetString(2154, "ptmsgs_portaladminmsgs"),
						ObjMgrModel.GetLocalizedErrorString(e, m_asOwner));

					log.Error(e, strFullError);

					strErrors = ConcatenateErrors(strErrors, strFullError);
				}
			}
		}

		strErrors = ConcatenateErrors(strErrors, sPEIErrorMsgs);

		// display objects that could not be deleted or had errors
		if ((null != strErrors) && (!"".equals(strErrors)))
		{
			RaiseError(strErrors);
		}
	}

	/**
	 * This method enables and disables users and web services.
	 */
	public void EnableObjects(XPHashtable xpmObjects, String sComment, boolean bEnable)
	{
		ObjectManagerHolder curObjMgrHolder;
		int nCurClass;
		int[] nIDs = null;

		nCurClass     = PT_CLASSIDS.PT_USER_ID;
		nIDs		  = (int[]) xpmObjects.GetElement(new Integer(nCurClass));

		if (nIDs != null)
		{
			curObjMgrHolder = GetObjMgrHolder(nCurClass);

			if (curObjMgrHolder != null)
			{
				IPTUser curUser;

				for (int i = 0; i < nIDs.length; i++)
				{
					// false = open object unlocked
					curUser = (IPTUser) curObjMgrHolder.m_ptObjMgr.Open(nIDs[i], false);
					
					// Can't lock self or guest or if not edit access to folder
					// WilliamA - change guest user detection (for multiple-guest-user support)
					// -- (nIDs[i] != PT_INTRINSICS.PT_USER_GUEST)
					if ( (PT_USER_TYPES.PT_USER_TYPE_GUEST != curUser.GetUserType()) &&
							(nIDs[i] != ((IPTSession) m_asOwner.GetUserSession()).GetSessionInfo().GetCurrentUserID()) && 
							(m_ptCatalog.IsAdminFolderAccessible(curUser.GetAdminFolderID(), PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_EDIT)) )
					{
						curUser.SetLockedStatus(!bEnable);
					}
					else
					{
						log.Warn("Guest and self cannot be locked.");
						RaiseError(m_asOwner.GetString(1263, "ptmsgs_portaladminmsgs"));
					}
				}
			}
		}

		nCurClass     = PT_CLASSIDS.PT_WEBSERVICE_ID;
		nIDs		  = (int[]) xpmObjects.GetElement(new Integer(nCurClass));

		if (nIDs != null)
		{
			curObjMgrHolder = GetObjMgrHolder(nCurClass);

			if (curObjMgrHolder != null)
			{
				IPTWebService curWebService;
				IPTServerContext myContext;

				for (int i = 0; i < nIDs.length; i++)
				{
					try {
						// true = open object locked
						curWebService = (IPTWebService) curObjMgrHolder.m_ptObjMgr.Open(nIDs[i], true);
						curWebService.SetEnabled(bEnable);
	
						if (!bEnable)
						{
							if ((sComment != null) && (!"".equals(sComment)))
							{
								// Comment cannot be null or empty.
								curWebService.SetDisabledMessage(sComment);
							}
						}					
	
						// Store the object on the server
						myContext = (IPTServerContext) curWebService.GetInterfaces(
							"IPTServerContext");
						myContext.Store();
						myContext.UnlockObject();
					} catch (Exception e) {
						log.Error(e, "AdminModelHelper:EnableObjects Could not enable or disable web service with Id: " + nIDs[i]);
						RaiseError(m_asOwner.GetString(1263, "ptmsgs_portaladminmsgs"));
					}
				}
			}
		}
	}

	/**
	 * @see com.plumtree.portalpages.admin.objmgr.IObjMgrModelRO#GetCommunityID()
	 */
	public int GetCommunityID(int nFolderID, int defaultFolderID)
	{
		// -1 means use root folder
		if (nFolderID == -1)
		{
			nFolderID = defaultFolderID;
		}

		ObjectManagerHolder objMgrHolder = GetObjMgrHolder(
					PT_CLASSIDS.PT_COMMUNITY_ID);

		if (objMgrHolder == null)
		{
			log.Error("Unable to access community object manager.");

			return -1;
		}

		PTObjectManagerQueryWrapper communityQuery = new PTObjectManagerQueryWrapper(objMgrHolder.m_ptObjMgr);
		communityQuery.SimpleQuery(nFolderID, PT_PROPIDS.PT_PROPID_NAME);

		int nCount = communityQuery.GetCount();

		if (nCount < 1)
		{
			log.Error("Unable to access community object.");

			return -1;
		}
		else if (nCount > 1)
		{
			log.Warn("More than one community object found in folder " +
				nFolderID);
		}

		int nCommunityID;

		try
		{
			nCommunityID = XPConvert.ToInteger(communityQuery.GetFields(0,
							PT_PROPIDS.PT_PROPID_OBJECTID));
		}
		catch (Exception e)
		{
			log.Error(e, "Unable to retrieve community id.");

			nCommunityID = -1;
		}

		return nCommunityID;
	}

	/**
	 * @see com.plumtree.portalpages.admin.objmgr.IObjMgrModelRO#GetCurrentFolderIsCommunity()
	 */
	public boolean GetFolderIsCommunity(IPTAdminFolder folder)
	{
		return (folder.GetFolderType() == PT_ADMIN_FOLDER_TYPES.PT_ADMIN_FOLDER_TYPE_COMMUNITYFOLDER);
	}

	/**
	 * @see com.plumtree.portalpages.admin.objmgr.IObjMgrModelRO#GetCurrentFolderIsSubportal()
	 */
	public boolean GetFolderIsSubportal(IPTAdminFolder folder)
	{
		if (folder.GetFolderType() == PT_ADMIN_FOLDER_TYPES.PT_ADMIN_FOLDER_TYPE_REGULARFOLDER)
		{
			return false;
		} else if (folder.GetFolderType() == PT_ADMIN_FOLDER_TYPES.PT_ADMIN_FOLDER_TYPE_COMMUNITYFOLDER)
		{
			return false;
		}

		// Everything else must be a subportal
		return true;
	}

	/**
	 * This method gets the selected folder from the tree sub space.
	 *
	 * @return int the folder ID (-1 implies error)
	 */
	public int GetFolderIDFromTree(String subspaceKey)
	{
		AActivitySpace asTree  = m_asOwner.GetSubSpace(subspaceKey);
		ITreeModelRO treeModel = (ITreeModelRO) asTree.GetModel(ATreeAS.TREEMODEL);
		XPArrayList itemList   = treeModel.GetSelectedItems();

		if (itemList.GetSize() != 1)
		{
			log.Error("Tree selected wrong number of items.  Expecting 1, got " +
				itemList.GetSize());

			return -1;
		}

		TreeLeafNode item = (TreeLeafNode) itemList.GetElement(0);

		if (item == null)
		{
			log.Error("Tree selected item null");

			return -1;
		}

		return item.m_nObjectID;
	}
	/**
	 * @see com.plumtree.portalpages.admin.objmgr.IObjMgrModelRO#GetFolderTreeID()
	 */
	public String GetFolderTreeID()
	{
		// If we haven't set this yet
		if (m_asOwner.GetSubSpaceExists(ObjMgrAS.FOLDER_TREE_SUB_SPACE_KEY))
		{
			try
			{
				return m_asOwner.GetSubSpace(ObjMgrAS.FOLDER_TREE_SUB_SPACE_KEY)
					.GetSpaceID();
			} catch (Exception e)
			{
				log.Error(e, "Unable to get folder tree ID.");
			}
		}

		return null;
	}
	
	public int GetRootFolderID()
	{
		return m_RootFolderID;
	}
	
	public String GetAllObjectTypesString()
	{
		if (m_strExpandedObjectTypes == null)
		{
			
			int[] nExpandedObjectTypes = GetAllObjectTypesArray();
			m_strExpandedObjectTypes = GeneralHelpers.ConvertToString(nExpandedObjectTypes);
		}

		return m_strExpandedObjectTypes;
	}

	public int[] GetAllObjectTypesArray()
	{
		ClassTypeDescManager ptClassTypeDescManager = PTConfigHelper.GetPTClassTypeDescrManager();
		return ptClassTypeDescManager.GetOrderedListOfClassIDs("en");
	}

	/**
	 * Get the SpaceID of the parent Activity Space
	 *
	 * @return int m_asOwner.GetSpaceID();
	 */
	public String GetSpaceID()
	{
		return m_asOwner.GetSpaceID();
	}

	/**
	 * @see com.plumtree.xpshared.activityspace.IModel#Init()
	 */
	public void Init(AActivitySpace parent)
	{
		m_asOwner			  = parent;
		m_ptCatalog			  = ((IPTSession) m_asOwner.GetUserSession()).GetAdminCatalog();

		// Set the help topic
		m_asOwner.SetHelpTopic("PT_Administration");

		m_xpmObjMgrs     = new XPHashtable();

		//PEI : ObjectActions
		//BrianH: Initialization of all ObjectActions PEIs
		IApplication application = m_asOwner.GetApplication();
		m_oObjectActions = (Object[]) application.GetCachingManager().GetEntry("ObjectActions");

		m_nABOJobIDs = new int[0];
	}

	/**
	 * This method adds folders into the migration queue.
	 */
	public void MigrateFolders(int[] nFolderIDs, String sComment)
	{
		IPTMigrationManager ptMM = (IPTMigrationManager) ((IPTSession) m_asOwner.GetUserSession()).
			OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_MIGRATION_MANAGER, false);

		IPTAdminFolder currentFolder;
		String strErrors = null;
		int nCommunities = 0;
		int[] nCommunityIDs = new int[nFolderIDs.length];

		for (int i = 0; i < nFolderIDs.length; i++)
		{
			try
			{
				currentFolder = m_ptCatalog.OpenAdminFolder(nFolderIDs[i], false);
			}
			catch (Exception e)
			{
				String strErrorMessage = XPResourceManager.FormatMessage(
					m_asOwner.GetString(1254, "ptmsgs_portaladminmsgs"),
					new Integer(nFolderIDs[i]));

				currentFolder = null;

				log.Error(e, strErrorMessage);

				strErrors = ConcatenateErrors(strErrors, strErrorMessage);
			}

			// Check for community folders
			if ((currentFolder != null) &&
				(currentFolder.GetFolderType() == PT_ADMIN_FOLDER_TYPES.PT_ADMIN_FOLDER_TYPE_COMMUNITYFOLDER))
			{
				// TODO: THis won't work if we add in ObjMgrModel methods.
				nCommunityIDs[nCommunities] = GetCommunityID(nFolderIDs[i], 1);
				if (nCommunityIDs[nCommunities] > 0)
				{
					// Got the community ID.  Move on.
					nCommunities++;
					nFolderIDs[i] = -1;
				}
			}
		}

		if (nCommunities > 0)
		{
			int nNewCommunityIDs[] = new int[nCommunities];
			for (int i = 0; i < nCommunities; i++)
			{
				nNewCommunityIDs[i] = nCommunityIDs[i];
			}

			// Migrate communities
			try
			{
				ptMM.BulkSetMigrationStatus(PT_CLASSIDS.PT_COMMUNITY_ID,
					nNewCommunityIDs,
					PT_MIGRATION_STATUS_TYPES.PT_MIGRATION_STATUS_APPROVAL_REQUESTED,
					sComment);
			}
			catch (Exception e)
			{
				String strErrorMessage = m_asOwner.GetString(2513, "ptmsgs_portaladminmsgs"); 

				log.Error(e, strErrorMessage);

				strErrors = ConcatenateErrors(strErrors, strErrorMessage);
			}

			// Remove communities from folder array
			int[] nNewFolderIDs = new int[nFolderIDs.length - nCommunities];
			int nFolderIndex = 0;
			for (int i = 0; i < nFolderIDs.length; i++)
			{
				if (nFolderIDs[i] != -1)
				{
					nNewFolderIDs[nFolderIndex] = nFolderIDs[i];
					nFolderIndex++;
				}
			}
			nFolderIDs = nNewFolderIDs;
		}

		// migrate folders
		if ((nFolderIDs != null) && (nFolderIDs.length > 0))
		{
			try
			{
				ptMM.BulkSetMigrationStatus(PT_CLASSIDS.PT_ADMIN_FOLDER_ID,
					nFolderIDs,
					PT_MIGRATION_STATUS_TYPES.PT_MIGRATION_STATUS_APPROVAL_REQUESTED,
					sComment);
			}
			catch (Exception e)
			{
				String strErrorMessage = m_asOwner.GetString(2513, "ptmsgs_portaladminmsgs");

				log.Error(e, strErrorMessage);

				strErrors = ConcatenateErrors(strErrors, strErrorMessage);
			}
		}

		if ((null != strErrors) && (!"".equals(strErrors)))
		{
			RaiseError(strErrors);
		}
	}

	/**
	 * This method adds objects into the migration queue.
	 */
	public void MigrateObjects(XPHashtable xpmObjects, String sComment)
	{
		String strErrors = null;

		Object nCurClass;
		int nCurClassID;
		int[] nIDs;

		IPTMigrationManager ptMM = (IPTMigrationManager) ((IPTSession) m_asOwner.GetUserSession()).OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_MIGRATION_MANAGER,
				false);

		IXPEnumerator classIDEnum = xpmObjects.GetKeys();
		IXPEnumerator objectEnum  = xpmObjects.GetElements();
		
		Object oObjectAction;
		IObjectActions iAction;
		
		// JF- Error messages from all PEIs for all objects
		String sPEIErrorMsgs = null;

		while (objectEnum.MoveNext() && classIDEnum.MoveNext())
		{
			nCurClass     = classIDEnum.GetCurrent();
			nCurClassID = XPConvert.ToInteger(nCurClass);
			nIDs		  = (int[]) objectEnum.GetCurrent();

			if ((nCurClass != null) && (nIDs != null) && (nIDs.length > 0))
			{
				try
				{
					//PEI : IObjectActions - BEGIN
					//BrianH: Implementation of the OnMigrateObject function of the IObjectActions
					//For each class of objects and their corresponding integer array of object ids
					//the functions will be called to see if the objects should be migrated. This will
					//only be done if there are implemented IObjectActions functions otherwise the normal
					//processing will occur. The normal processing will also occur if no errors are returned.
					if (m_oObjectActions != null)
					{
						// JF- Error messages from all PEIs for the current object
						String sObjectErrorMsgs = null;
													  
						for (int x = 0; x < m_oObjectActions.length; x++)
						{
							oObjectAction = m_oObjectActions[x];

							log.Debug("AdminSearchModel.MigrateObjects - About to call OnMigrateObject on an: " + oObjectAction);

							if(oObjectAction != null)
							{
								String sError;

								iAction = (IObjectActions) oObjectAction;
								sError = iAction.OnBeforeMigrateObject(m_asOwner, 
									(IPTSession) m_asOwner.GetUserSession(), nCurClassID, nIDs);

								sObjectErrorMsgs = ConcatenateErrors(sObjectErrorMsgs, sError);
							 }
						}
						
						if ((null != sObjectErrorMsgs) && (sObjectErrorMsgs.length() > 0))
						{
							sPEIErrorMsgs = ConcatenateErrors(sPEIErrorMsgs, sObjectErrorMsgs);
						}
						else
						{
							ptMM.BulkSetMigrationStatus(XPConvert.ToInteger(nCurClass), nIDs,
								PT_MIGRATION_STATUS_TYPES.PT_MIGRATION_STATUS_APPROVAL_REQUESTED,
								sComment);
						}
					}
					else
					{
						ptMM.BulkSetMigrationStatus(XPConvert.ToInteger(nCurClass), nIDs,
							PT_MIGRATION_STATUS_TYPES.PT_MIGRATION_STATUS_APPROVAL_REQUESTED,
							sComment);
					}
					//PEI : ObjectActions : OnMigrateObject END
				}
				catch (Exception e)
				{
					ClassTypeDescManager ptClassTypeDescManager = PTConfigHelper.GetPTClassTypeDescrManager();
					String strLangID = (String) PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingsHelper.LANGUAGE, m_asOwner);

					String strErrorMessage = XPResourceManager.FormatMessage
						(m_asOwner.GetString(2514, "ptmsgs_portaladminmsgs"),
						ptClassTypeDescManager.GetClassTypeLocalizedName(nCurClassID, strLangID));

					log.Error(e, strErrorMessage);

					strErrors = ConcatenateErrors(strErrors, strErrorMessage);
				}
			}
		}

		strErrors = ConcatenateErrors(strErrors, sPEIErrorMsgs);

		if ((null != strErrors) && (!"".equals(strErrors)))
		{
			RaiseError(strErrors);
		}
	}
	/**
	 * This method adds objects into the migration queue.
	 */
	public void MigrateObjects(int nClassID, int[] nObjects, String sComment)
	{
		IPTMigrationManager ptMM = (IPTMigrationManager) ((IPTSession)m_asOwner.GetUserSession()).OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_MIGRATION_MANAGER, false);

		if ((nObjects != null) && (nObjects.length > 0))
		{
			try
			{
				ptMM.BulkSetMigrationStatus(nClassID,
					nObjects,
					PT_MIGRATION_STATUS_TYPES.PT_MIGRATION_STATUS_APPROVAL_REQUESTED,
					sComment);
			}
			catch (Exception e)
			{
				log.Error(e, m_asOwner.GetString(1219, "ptmsgs_portalbrowsingmsgs"));

				throw new XPException(STR_MVC_CLASS_NAME +
					"::MigrateFolders Error when migrating folders" +
					e.getMessage(), e);
			}
		}
	}
	/**
	 * This method moves folders to a new folder.
	 *
	 * @param int nTargetFolderID the ID of the folder to move the folders to.
	 * @param int[] nFolders an array of folder IDs.
	 */
	public void MoveFolders(int nTargetFolderID, int[] nFolders)
	{
		if (nFolders.length <= 0)
		{
			return;
		}

		boolean bCommunity		   = CheckIsCommunityFolder(nTargetFolderID);
		IPTAdminFolder movedFolder = null;
		String strErrors		   = null;

		for (int i = 0; i < nFolders.length; i++)
		{
			if ((nFolders[i] <= PT_SERVERCONSTANTS.PT_MIN_OBJECTID) &&
				(nFolders[i] != PT_INTRINSICS.PT_ADMIN_FOLDER_DEFAULTSUBPORTAL) &&
				(nFolders[i] != PT_INTRINSICS.PT_ADMIN_FOLDER_INTRINSICOPS))
			{
				// ignore intrinsic objects -- they can't be moved
				// Make an exception for default subportal and intrinsic ops folders.
				// They can be moved, but not deleted.
				strErrors += m_asOwner.GetString(1980, "ptmsgs_portaladminmsgs");
			} else
			{
				if (bCommunity)
				{
					try
					{
						movedFolder = m_ptCatalog.OpenAdminFolder(nFolders[i], false);
					}
					catch (Exception e)
					{
						movedFolder = null;

						String strErrorMessage = XPResourceManager.FormatMessage(
							m_asOwner.GetString(1254, "ptmsgs_portaladminmsgs"),
							new Integer(nFolders[i]));

						log.Error(e, strErrorMessage);

						strErrors = ConcatenateErrors(strErrors, strErrorMessage);
					}
				}
	
				// Check that we are only moving community folders into a community
				if ((movedFolder != null) &&
						(movedFolder.GetFolderType() != PT_ADMIN_FOLDER_TYPES.PT_ADMIN_FOLDER_TYPE_COMMUNITYFOLDER))
				{
					String strErrorMessage = GetFolderErrorString(1252, nFolders[i]);

					log.Warn(strErrorMessage);

					strErrors = ConcatenateErrors(strErrors, strErrorMessage);
				}
				else
				{
					try
					{
						m_ptCatalog.MoveAdminFolder(nFolders[i], nTargetFolderID);

					}
					catch (Exception e)
					{
						String strErrorMessage = GetFolderErrorString(1258, nFolders[i]);

						log.Error(e, strErrorMessage);

						strErrors = ConcatenateErrors(strErrors, strErrorMessage);
					}
				}
			}
		}

		if ((null != strErrors) && (!"".equals(strErrors)))
		{
			RaiseError(strErrors);
		}

		// TODO: check results -- but results are always OK, right?
		// Should I get the list of objs you can't move?
	}

	/**
	 * This method moves objects between folders.
	 *
	 * @param int nFolderID the ID of the folder to move the objects to.
	 * @param Object[][] nObjects an (2 x n) array of class IDs / Object IDs
	 */
	public void MoveObjectsBetweenFolders(int nFolderID, Object[][] nObjects, Object[][] nFolders)
	{

		if (nObjects[0].length == 0)
		{
			return;
		}

		if (nFolderID == PT_INTRINSICS.PT_ADMIN_FOLDER_ROOTFOLDER)
		{
			// Cannot move objects to the root folder.
			RaiseError(m_asOwner.GetString(2158, "ptmsgs_portaladminmsgs"));
			return;
		}

		// Should I get the list of objs you can't move?
		boolean bNonCommunityObjects = false;
		int nID;
		
		Object oObjectAction;
		IObjectActions iAction;

		for (int i = 0; i < nObjects[0].length; i++)
		{
			nID = XPConvert.ToInteger(nObjects[0][i]);

			if ((nID != PT_CLASSIDS.PT_USERGROUP_ID) &&
					(nID != PT_CLASSIDS.PT_GADGET_ID) &&
					(nID != PT_CLASSIDS.PT_PAGE_ID))
			{
				bNonCommunityObjects = true;

				break;
			}
		}

		if (bNonCommunityObjects)
		{
			IPTAdminFolder targetFolder;

			// Check if we are in a community
			try
			{
				targetFolder = m_ptCatalog.OpenAdminFolder(nFolderID, false);
			}
			catch (Exception e)
			{
				targetFolder = null;

				String strErrorMessage = XPResourceManager.FormatMessage(
					m_asOwner.GetString(1254, "ptmsgs_portaladminmsgs"),
					new Integer(nFolderID));

				log.Error(e, strErrorMessage);

				RaiseError(strErrorMessage);

				return;
			}

			if (targetFolder.GetFolderType() == PT_ADMIN_FOLDER_TYPES.PT_ADMIN_FOLDER_TYPE_COMMUNITYFOLDER)
			{
				String strErrorMessage = XPResourceManager.FormatMessage(m_asOwner.GetString(1253, "ptmsgs_portaladminmsgs"),
					targetFolder.GetLocalizedName());

				log.Error(strErrorMessage);

				RaiseError(strErrorMessage);

				// TODO: Shouldn't this still move the community objects?
				return;
			}
		}

		try
		{
			//PEI : IObjectActions - BEGIN
			//BrianH: Implementation of the OnMoveObject function of the IObjectActions
			//For each class of objects the functions will be called to determine if the objects
			//requested should be moved to the target folder. This will only be done if there
			//are implemented IObjectActions functions otherwise the normal processing will occur.
			//The normal processing will also occur if no errors are returned.
			int nClassID;
			int nObjectID;
			
			if (m_oObjectActions != null)
			{
				String sErrorMsgs = null;
				
				for (int x = 0; x < m_oObjectActions.length; x++)
				{
					oObjectAction = m_oObjectActions[x];

					log.Debug("AdminSearchModel.MoveObjectsBetweenFolders - About to call OnMoveObject on an: " + oObjectAction);

					if(oObjectAction != null)
					{
						iAction = (IObjectActions) oObjectAction;
						for (int y = 0; y < nObjects[0].length; y++)
						{
							String sError;
							
							nClassID = XPConvert.ToInteger(nObjects[0][y]);
							nObjectID = XPConvert.ToInteger(nObjects[1][y]);
							
							sError = iAction.OnBeforeMoveObject(m_asOwner, 
								(IPTSession) m_asOwner.GetUserSession(), nClassID, nObjectID, nFolderID);

							sErrorMsgs = ConcatenateErrors(sErrorMsgs, sError);
						}
					}
				}
				
				if ((null != sErrorMsgs) && (sErrorMsgs.length() > 0))
				{
					RaiseError(sErrorMsgs);
					return;
				}
				else
				{
					DoMove(nObjects, nFolders, nFolderID);
				}
			}
			else
			{			
				DoMove(nObjects, nFolders, nFolderID);
			}
			//PEI : ObjectActions : OnMoveObject END
		}
		catch (Exception e)
		{
			String strErrorMessage = m_asOwner.GetString(1259, "ptmsgs_portaladminmsgs");

			log.Error(e, strErrorMessage);

			RaiseError(strErrorMessage);
		}
	}

	//TODO: This may need to be an abstract class when we add in ObjMgrModel.
	public void RefreshResults(String key)
	{
		SearchResultModel model = (SearchResultModel)m_asOwner.GetModel(SearchResultModel.STR_MVC_CLASS_NAME);
		model.RefreshResults(key);
	}
	
	public void DoMove(Object[][] nObjects, Object[][] nFolders, int nFolderID)
	{
		Object[] moveResults;
		Object[][] arg = new Object[2][1];
		for (int i = 0; i < nObjects[0].length; i++)
		{	
			// These two should be Integers
			Object classID = nObjects[0][i];
			Object objectID = nObjects[1][i];
			int folderID = XPConvert.ToInteger(nFolders[1][i]);

			arg[0][0] = classID;
			arg[1][0] = objectID;

			int classIDint = XPConvert.ToInteger(classID);
			if( classIDint == PT_CLASSIDS.PT_CATALOGCARD_ID )
			{
				int[] cardArray = new int[1];
				cardArray[0] = XPConvert.ToInteger(objectID);
				MoveCards(cardArray,folderID,nFolderID);
			}
			else
			{
				moveResults = m_ptCatalog.MoveObjectsBetweenAdminFolders(arg, folderID, nFolderID);
				// Each object has it's own return code.
				for (int l = 0; l < moveResults.length; l++)
				{
					if (XPConvert.ToInteger(moveResults[l]) != PT_RESULTCODES.PT_S_OK)
					{
						RaiseError(m_asOwner.GetString(1259, "ptmsgs_portaladminmsgs"));
					}
				}
			}
		}
	}

	public void MoveCards(int[] arCardIDs, int nFromFolder, int nTargetFolder)
	{
		try
		{
			((IPTSession)m_asOwner.GetUserSession()).GetCatalog().MoveCards(nFromFolder, arCardIDs,
				nTargetFolder, PT_CARDREF_STATES.PT_APPROVED, false); // just like 4.5ws, move as approved and bAddOnlyAtSpecifiedApprovalState = false
		}
		catch (XPException xpe)
		{
			throw new XPException(STR_MVC_CLASS_NAME +
				"::MoveCards Error when moving cards" + xpe.GetMessage(), xpe);
		}
	}

	/**
	 * This method raises a UI Error.
	 *
	 * @param strError 
	 */
	public void RaiseError(String strError)
	{
		UIError myError = new UIError(UIErrorLevel.FRIENDLY,
				UIErrorScope.CURRENT_USER, strError, strError);

		myError.SetActivitySpace(m_asOwner.GetName());
		myError.SetDisplayOnSeparatePage(false);
		myError.SetClearWithActivitySpaceChange(true);
		myError.SetImmediateClear(true);

		UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(),
			m_asOwner.GetApplication());
	}

	/**
	 * This method concatenates two error messages into a single message
	 * with proper internationalized spacing between the messages.
	 * 
	 * This method will check both error messages for null and empty
	 * string values.
	 * 
	 * @param _strFirstError The first error.  May be null or empty string.
	 * @param _strSecondError The second error.  May be null or empty string.
	 * @return The combined error string.  Will be either null or a valid error
	 * string.
	 */
	public String ConcatenateErrors(String _strFirstError, String _strSecondError)
	{
		String strFullError;

		if ((null != _strFirstError) && (!"".equals(_strFirstError)))
		{
			if ((null != _strSecondError) && (!"".equals(_strSecondError)))
			{
				// Concatenate 2 strings (with a space between for English, etc...
				// and no space for languages like Chinese).
				strFullError = XPResourceManager.FormatMessage(m_asOwner.GetString(3143, ResourceConstants.PORTALADMINMSGS),
					_strFirstError, _strSecondError);
			} else
			{
				strFullError = _strFirstError;
			}
		} else
		{
			strFullError = _strSecondError;
		}

		if ((null == strFullError) || ("".equals(strFullError)))
		{
			return null;
		}

		return strFullError;
	}

	/////////////////////////////////
	// Write methods for IRepostModel
	/////////////////////////////////

	/**
	 * This method sets the selected items in a Class ID
	 *
	 * @param int nClassID the class ID of the selected items
	 * @param int[] nSelectedObjectIDs the selected object IDs
	 */
	public void SetSelectedObjects(int nClassID, int[] nSelectedObjectIDs)
	{
		ObjectManagerHolder objMgrHolder = GetObjMgrHolder(nClassID);

		if (objMgrHolder != null)
		{
			objMgrHolder.m_nSelectedObjectIDs = nSelectedObjectIDs;
		}
	}

	/**
	 * 
	 *
	 * @param nFolderID 
	 *
	 * @return 
	 */
	public boolean CheckIsCommunityFolder(int nFolderID)
	{
		// Check if we are in a community
		IPTAdminFolder targetFolder = m_ptCatalog.OpenAdminFolder(nFolderID, false);

		if (targetFolder == null)
		{
			log.Error("Unable to locate target folder: " + nFolderID);

			return false;
		}

		return (targetFolder.GetFolderType() == PT_ADMIN_FOLDER_TYPES.PT_ADMIN_FOLDER_TYPE_COMMUNITYFOLDER);
	}

	/**
	 * This method is used to get and format an error message where the only arg
	 * in the message is a folder name.
	 * 
	 * @param int nErrorID the id in the xml file
	 * @param int nCurrentFolderID the id of the folder to use the name of
	 * @return String the error message
	 */
	public String GetFolderErrorString(int nErrorID, int nCurrentFolderID)
	{
		String strFolderName;

		try {
			IPTAdminFolder tempFolder = m_ptCatalog.OpenAdminFolder(nCurrentFolderID, false);
			strFolderName = tempFolder.GetLocalizedName();
		} catch (Exception e)
		{
			log.Error("Unable to get name for folder " + nCurrentFolderID);

			strFolderName = "";
		}

		return XPResourceManager.FormatMessage(m_asOwner.GetString(nErrorID, "ptmsgs_portaladminmsgs"),
			strFolderName);
	}

	/**
	 * This method returns the requested object manager holder, instantiating it if
	 * it was not cached.
	 *
	 * @return ObjectManagerHolder the obj mgr holder -- null on failure
	 */
	public ObjectManagerHolder GetObjMgrHolder(int nClassID)
	{
		ObjectManagerHolder resultHolder = (ObjectManagerHolder) m_xpmObjMgrs.GetElement(new Integer(nClassID));

		if ((resultHolder == null) || (resultHolder.m_ptObjMgr == null))
		{
			resultHolder = new ObjectManagerHolder(m_ptCatalog.GetObjectManager(
						nClassID));

			if (resultHolder.m_ptObjMgr == null)
			{
				log.Error("Unable to get object manager for ID " + nClassID);

				return null;
			}

			m_xpmObjMgrs.PutElement(new Integer(nClassID), resultHolder);
		}

		return resultHolder;
	}

	public boolean CheckAuthSourceForDelete(int nCurrentFolderID) 
	{				
		if (m_nObjectIDPairsForDeleteCheck == null)
		{
			return false;
		}

		int nUserCount  = 0;
		int nGroupCount = 0;

		for (int i = 0; (i + 1) < m_nObjectIDPairsForDeleteCheck.length;
				i += 2)
		{
			// If we are a non-intrinsic user
			if ((m_nObjectIDPairsForDeleteCheck[i + 1] > PT_SERVERCONSTANTS.PT_MIN_OBJECTID) &&
					(m_nObjectIDPairsForDeleteCheck[i] == PT_CLASSIDS.PT_USER_ID))
			{
				nUserCount++;
			}

			// If we are a non-intrinsic group
			else if ((m_nObjectIDPairsForDeleteCheck[i + 1] > PT_SERVERCONSTANTS.PT_MIN_OBJECTID) &&
					(m_nObjectIDPairsForDeleteCheck[i] == PT_CLASSIDS.PT_USERGROUP_ID))
			{
				nGroupCount++;
			}
		}

		if ((nUserCount == 0) && (nGroupCount == 0))
		{
			return false;
		}

		if (CheckObjectsFromExternalAuthSource(nUserCount, nCurrentFolderID,
			PT_CLASSIDS.PT_USER_ID, PT_PROPIDS.PT_PROPID_USER_AUTHSOURCE))
		{
			// We found users from an external auth source
			return true;
		}

		if (CheckObjectsFromExternalAuthSource(nGroupCount, nCurrentFolderID,
			PT_CLASSIDS.PT_USERGROUP_ID, PT_PROPIDS.PT_PROPID_USERGROUP_AUTHSOURCE))
		{
			// We found groups from an external auth source
			return true;
		}

		return false;
   }

	/**
	 * This helper method checks whether any of the objects
	 * for deletion are from an external authentication source.
	 * 
	 * @param nObjectCount
	 * @param nCurrentFolderID
	 * @param nClassID
	 * @param nAuthSourcePropID
	 * @return true implies there were objects from an external auth source.
	 */
	protected boolean CheckObjectsFromExternalAuthSource(int nObjectCount,
		int nCurrentFolderID, int nClassID, int nAuthSourcePropID)
	{
		Object[][] ptqf = new Object[3][2];

		// Object IDs to look up
		ptqf[0][0]	= new Integer(PT_PROPIDS.PT_PROPID_OBJECTID);
		ptqf[1][0]	= new Integer(PT_FILTEROPS.PT_FILTEROP_IN);
		// ptqf[2][0] is filled in later

		// ! Plumtree authsource
		ptqf[0][1]	= new Integer(nAuthSourcePropID);
		ptqf[1][1]	= new Integer(PT_FILTEROPS.PT_FILTEROP_NE);
		ptqf[2][1]	= new Integer(PT_INTRINSICS.PT_AUTHSOURCE_PLUMTREE);

		if (nObjectCount > 0)
		{
			int nObjectsToQuery = nObjectCount;
			if (nObjectsToQuery > MAX_OBJECTS_IN_FILTER)
			{
				nObjectsToQuery = MAX_OBJECTS_IN_FILTER;
			}

			int[] nObjectIDs;

			int nObjIDPairsIndex = 0;
			int nObjectsCounted = 0;

			// Since we can only have 255 object IDs in an IN query filter clause,
			// we need to query separately for each group of approximately 255
			// objects.  Once we find a match, we can stop.
			while (((nObjIDPairsIndex + 1) < m_nObjectIDPairsForDeleteCheck.length) &&
				(nObjectsToQuery > 0))
			{
				nObjectIDs = new int[nObjectsToQuery];
				int nObjectIndex = 0;

				while (((nObjIDPairsIndex + 1) < m_nObjectIDPairsForDeleteCheck.length) &&
					(nObjectIndex < nObjectIDs.length))
				{
					// If we are a non-intrinsic object of type nClassID
					if ((m_nObjectIDPairsForDeleteCheck[nObjIDPairsIndex + 1] > PT_SERVERCONSTANTS.PT_MIN_OBJECTID) &&
						(m_nObjectIDPairsForDeleteCheck[nObjIDPairsIndex] == nClassID))
					{
						nObjectIDs[nObjectIndex] = m_nObjectIDPairsForDeleteCheck[nObjIDPairsIndex + 1];
						nObjectIndex++;
					}

					nObjIDPairsIndex += 2;
				}
	
				ptqf[2][0] = nObjectIDs;
	
				ObjectManagerHolder objMgrHolder = GetObjMgrHolder(nClassID);

				if (objMgrHolder.m_ptObjMgr.QueryCount(nCurrentFolderID, ptqf) > 0)
				{
					// Shortcut execution of method - we have found objects from
					// an external auth source
					return true;
				}

				nObjectsCounted += nObjectsToQuery;

				if ((nObjectCount - nObjectsCounted) < MAX_OBJECTS_IN_FILTER)
				{
					nObjectsToQuery = nObjectCount - nObjectsCounted;
				}
			}
		}

		return false;
	}

	/**
	 * This method checks whether or not the user is trying to delete any page templates.
	 *
	 * @return true implies page templates are being deleted
	 */
	public boolean CheckPageTemplatesForDelete() 
	{ 		
		if (m_nObjectIDPairsForDeleteCheck == null)
		{
			return false;
		}

		for (int i = 0; (i + 1) < m_nObjectIDPairsForDeleteCheck.length;
				i += 2)
		{
			// If we are a non-intrinsic user
			if ((m_nObjectIDPairsForDeleteCheck[i + 1] > PT_SERVERCONSTANTS.PT_MIN_OBJECTID) &&
					(m_nObjectIDPairsForDeleteCheck[i] == PT_CLASSIDS.PT_PAGETEMPLATE_ID))
			{
				return true;
			}
		}
		
		return false;
	}
	/**
	 * This method checks whether or not the user is trying to delete any folders.
	 *
	 * @return true implies folders are being deleted
	 */
	public boolean CheckFoldersForDelete() 
	{ 
		return m_bFoldersForDeleteCheck; 
	}
	
	/**
	 * This method is used to set the list of folder IDs (temporary)
	 * to be checked for delete.
	 *
	 * @param int[] of object class id / object id pairs.
	 */

	public void SetObjectsForDeleteCheck(int[] nObjectIDPairs)
	{
		m_nObjectIDPairsForDeleteCheck = nObjectIDPairs;
	}
	
	public void SetFoldersForDeleteCheck(boolean tf)
	{
		m_bFoldersForDeleteCheck = tf;
	}

	public String GetAdminFolderPath(int folderID)
	{
		  try
		  {
			  IPTAdminFolder folder = m_ptCatalog.OpenAdminFolder(folderID, false);
			  String retval = folder.GetPath();
			  return retval;
		  }
		  catch(Exception e)
		  {
		  	log.Warn(e, "Unable to get admin folder path for folder id " + folderID);
		  }
		  return null;
	}
		
	public void MoveDocumentFolders(int[] foldersToMove,int targetFolder)
	{
		try
		{
			IPTCatalog cat = ((IPTSession)m_asOwner.GetUserSession()).GetCatalog();
			for(int i = 0; i < foldersToMove.length; i++)
			{
				cat.MoveFolder(foldersToMove[i],targetFolder);
			}
			
		}
		catch (XPException xpe)
		{
			throw new XPException(STR_MVC_CLASS_NAME +
				"MoveDocumentFolders Error when moving cards" + xpe.GetMessage(), xpe);
		}

	}
	
	public void MoveCards(int fromFolder, int cardId, int toFolder)	
	{
		try
		{
			IPTCatalog cat = ((IPTSession)m_asOwner.GetUserSession()).GetCatalog();
			cat.MoveCards(fromFolder, cardId, toFolder, PT_CARDREF_STATES.PT_APPROVED, false);
		}
		catch (XPException xpe)
		{
			throw new XPException(STR_MVC_CLASS_NAME +
				"MoveCards Error when moving cards" + xpe.GetMessage(), xpe);
		}
	}

	/**
	 * 
	 * @param cardIds
	 * @param toFolder
	 */
	public void CopyCards(int[] cardIds, int toFolder)
	{
		try
		{
			IPTCatalog cat = ((IPTSession)m_asOwner.GetUserSession()).GetCatalog();
			cat.AddMultipleCardsToFolder(cardIds, toFolder, PT_CARDREF_STATES.PT_APPROVED, false);

		}
		catch (XPException xpe)
		{
			log.Error(xpe, "During  copy  got exception");

			throw new XPException(STR_MVC_CLASS_NAME +
				"CopyCards Error when copying cards" + xpe.GetMessage(), xpe);
		}
	}

	/**
	 * 
	 * @param folderIDs
	 * @param toFolder
	 */
	public void CopyDocumentFolders(int[] folderIDs, int toFolder)
	{
		try
		{
			IPTCatalog cat = ((IPTSession)m_asOwner.GetUserSession()).GetCatalog();
			for (int i = 0; i < folderIDs.length; i++)
			{
				int nFolderID = folderIDs[i];
				cat.CopyFolder(nFolderID, toFolder, true, true);
			}
		}
		catch (XPException xpe)
		{
			throw new XPException(STR_MVC_CLASS_NAME +
				"CopyDocumentFolders Error when copying folders" + xpe.GetMessage(), xpe);
		}
	}

	public XPHashtable GetObjMgrs()
	{
		return m_xpmObjMgrs;
	}
}