package com.plumtree.portalpages.admin.objmgr;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.admin.objcommon.*;
import com.plumtree.portalpages.common.constants.*;
import com.plumtree.portalpages.common.folder.*;
import com.plumtree.portalpages.common.objects.*;
import com.plumtree.portalpages.pei.*;
import com.plumtree.portaluiinfrastructure.classtypedescriptors.*;
import com.plumtree.portaluiinfrastructure.login.*;
import com.plumtree.portaluiinfrastructure.resultwrapper.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.server.marshalers.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.arraywrapper.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.form.pagination.*;
import com.plumtree.uiinfrastructure.form.sort.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.web.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * @author Don Hayler
 *
 * This implements the admin page (obj mgr).
 */
public class ObjMgrModel implements IObjMgrModelRO, IFolderModel,
	IFolderModelRO, IObjectModelRO, IRepostModel, ISortModel, IPaginationModel, IObjMgrDeleteRO
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
		"com.plumtree.portalpages.admin.objmgr.ObjMgrModel");

	/** Name of this object */
	public static final String STR_MVC_CLASS_NAME = "ObjMgr";

	public static final int MAX_NAME_LENGTH = 63;
	public static final int NUM_SUB_SUB_FOLDERS = 6;

	public static final int MAX_DISPLAYED_USERS = 20;
	public static final int MAX_PRE_EXPANDED_OBJECTS = 50;

	///////////////////
	// Header constants
	///////////////////

	private static final int HEADER_CHECKBOX_INDEX = 0;
	private static final int HEADER_NAME_INDEX     = 1;
	private static final int HEADER_MODIFIED_INDEX = 2;
	private static final int HEADER_CREATED_INDEX  = 3;

	// Job Specific constants

	private static final int HEADER_LAST_RUN_INDEX = 4;
	private static final int HEADER_NEXT_RUN_INDEX = 5;
	private static final int HEADER_JOB_STATUS_INDEX	= 6;

	/////////////////////
	// internal variables
	/////////////////////

	/** Stores common class used to display one folder */
	public IFolderViewHelper m_folderViewHelper;

	/** Owning AS */
	private AActivitySpace m_asOwner = null;

	/** the admin catalog object */
	private IPTAdminCatalog m_ptCatalog = null;

	////////////////
	// temporary variables -- deleted at the end of each request
	////////////////

	/** query for the object count in this folder */
	private IASQueryResult m_asqrObjectCount = null;

	/** query for the current objects (deleted between requests) */
	private IASQueryResult m_asqrObjects = null;

	/** query for the current subfolders (deleted between requests) */
	private IASQueryResult m_asqrSubfolders = null;

	/**
	 * This variable should not be used directly.
	 * Use GetCurrentFolder() instead (it creates
	 * it if it's not there).
	 */
	private IPTAdminFolder m_ptCurrentFolder = null;

	/**
	 * A string of comma separated Class IDs of the expanded object types
	 * for use in URLS
	 */
	private String m_strExpandedObjectTypes;

	////////////////
	// temporary variables -- reset at the end of each request
	////////////////

	/** sort inputs displayed */
	private boolean m_bSortInputsDisplayed = false;

	/** pagination inputs displayed */
	private boolean m_bPaginationInputsDisplayed = false;

	/** The object type that is being opened in this request, if any.  -1 if unused. */
	private int m_nCurrentlyOpeningType = -1;

	/** An array of the Class IDs of the object types to be expanded */
	private int[] m_nExpandedObjectTypes;

	/** True implies all object types should be expanded. */
	private boolean m_bExpandAllObjectTypes = false;

	/** temporary variables -- good between each calling of MoveNext() */

	/** the headers for the object view */
	private String[] m_astrHeaders = null;

	/** the headers for the job object view */
	private String[] m_astrJobHeaders = null;

	/** is this object type collapsed? */
	private boolean m_bIsCollapsed;

	/** The folder ID of the current folder */
	private int m_nCurrentFolderID = 0;

	/**
	 * The folder ID of the previous folder (used to provide a
	 * back link from things like the subportal manager)
	 */
	private int m_nPreviousFolderID = 0;


	private int m_nFirstItemIndex = 0;


	private int m_nLastItemIndex = 3;

	/** the class ID of the current object type */
	private int m_nObjectClassID;

	/** 
	 * the number of objects of the current type in the current pagination page 
	 * in this folder 
	 */
	private int m_nObjectCount;
	
	/** the total number of objects of the current type in this folder */
	private int m_nTotalObjectCount;

	/** index into m_asqrObjects */
	private int m_nObjectCountIndex = 0;

	/** Object array of all ObjectActions PEIs */
	private Object[] m_oObjectActions = null;
	
	private AdminModelHelper m_aoHelper = null;

	public ObjMgrModel()
	{
		m_folderViewHelper = new ObjMgrFolderViewHelper();
	}

	/**
	 * This method checks the array of folder IDs
	 * until it finds a non-community folder.
	 *
	 * @param int[] nFolders the folder IDs to check
	 * @return int the index of the first non-community folder.
	 */
	public int GetFirstFolderIndex(int[] nFolders)
	{
		int nResult = -1;
		IPTAdminFolder ptFolder;
		for (int i = 0; i < nFolders.length; i++)
		{
			ptFolder = m_ptCatalog.OpenAdminFolder(nFolders[i], false);
			if (ptFolder.GetFolderType() != PT_ADMIN_FOLDER_TYPES.PT_ADMIN_FOLDER_TYPE_COMMUNITYFOLDER)
			{
				nResult = i;
				break;
			}
		}

		if (nResult < 0)
		{
			RaiseError(m_asOwner.GetString(1981, "ptmsgs_portaladminmsgs"));
		}

		return nResult;
	}

	/**
	 * This method creates a job to delete folders using Asynchronous
	 * Browser Operations (ABO).
	 *
	 * @param int[] the folder IDs to delete
	 */
	public void ABODeleteFolders(int[] nFolders)
	{
		m_aoHelper.ABODeleteFolders(nFolders);
	}

	/**
	 * @param XPHashtable a map of Integer Class IDs to int[] arrays of object IDs.
	 */
	public void ABODeletePageTemplates(XPHashtable xpmSelectedObjects)
	{
		Object nPageTemplatesKey = new Integer(PT_CLASSIDS.PT_PAGETEMPLATE_ID);
		if (xpmSelectedObjects.ContainsKey(nPageTemplatesKey))
		{
			m_aoHelper.ABODeletePageTemplates((int[]) xpmSelectedObjects.GetElement(nPageTemplatesKey));
		}
	}

	/**
	 * @see com.plumtree.portalpages.admin.objmgr.IObjMgrModelRO#ABOJobIDsToDisplay()
	 */
	public boolean ABOJobsToDisplay()
	{
		return m_aoHelper.GetABOJobIDsToDisplay();
	}

	/**
	 * @see com.plumtree.portalpages.admin.objmgr.IObjMgrModelRO#CheckAuthSourceForDelete()
	 */
	public boolean CheckAuthSourceForDelete()
	{
		return m_aoHelper.CheckAuthSourceForDelete(m_nCurrentFolderID);
	}

	/**
	 * @see com.plumtree.portalpages.admin.objmgr.IObjMgrModelRO#CheckPageTemplatesForDelete()
	 */
	public boolean CheckPageTemplatesForDelete()
	{
		return m_aoHelper.CheckPageTemplatesForDelete();
	}

	/**
	 * @see com.plumtree.portalpages.admin.objmgr.IObjMgrModelRO#CheckFoldersForDelete()
	 */
	public boolean CheckFoldersForDelete()
	{
		return m_aoHelper.CheckFoldersForDelete();
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#CleanupTempData()
	 */
	public void CleanupTempData()
	{
		m_ptCurrentFolder = null;

		// cleanup variables
		m_asqrObjectCount     = null;
		m_asqrObjects		  = null;
		m_asqrSubfolders	  = null;
		m_strExpandedObjectTypes = null;


		// reset variables
		m_bSortInputsDisplayed			 = false;
		m_bPaginationInputsDisplayed     = false;

		m_nCurrentlyOpeningType = -1;
		
		m_aoHelper.CleanupTempData();
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#CleanupAllData()
	 */
	public void CleanupAllData()
	{
		m_aoHelper.CleanupAllData();
	}

	/**
	 * This method copies folders to a new folder.
	 *
	 * @param int nTargetFolderID the ID of the folder to copy the folders to.
	 * @param int[] nFolders an array of folder IDs.
	 */
	public void CopyFolders(int nTargetFolderID, int[] nFolders)
	{
		m_aoHelper.CopyFolders(nTargetFolderID, nFolders);
	}

	/**
	 * This method copies objects between folders.
	 *
	 * @param int nFolderID the ID of the folder to copy the objects to.
	 * @param XPHashtable a map of Integer Class IDs to int[] arrays of object IDs.
	 */
	public void CopyObjectsBetweenFolders(int nFolderID, XPHashtable xpmObjects)
	{
		m_aoHelper.CopyObjectsBetweenFolders(nFolderID, xpmObjects, m_nCurrentFolderID);
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new ObjMgrModel();
	}

	/////////////////
	// Public methods
	/////////////////

	public void CreateFolder(String name, String description)
	{
		if (GetCurrentFolderIsCommunity())
		{
			
				log.Warn("Folders cannot be created in communities.  Could not create Folder " + name);
			

			// TODO: oops, this error will not display because it needs to be
			// set on the main page.

			// This error is not very specific, but this should be okay, since this
			// error should only occur when people are actively trying to mess up
			// the admin page.  To get here, you have to open the create folder popup
			// and then navigate into a community, and then finish the popup.
			RaiseError(XPResourceManager.FormatMessage(m_asOwner.GetString(1264, "ptmsgs_portaladminmsgs"),
				name));

			return;
		}

		Object oObjectAction;
		IObjectActions iAction;
		boolean bValidName = false;

		if (name != null)
		{
			int nNameLength = name.length();

			if ((nNameLength >= 1) && (nNameLength <= MAX_NAME_LENGTH))
			{
				bValidName = true;
			}
		}

		if (!bValidName)
		{
			
				log.Error(m_asOwner.GetString(1265, "ptmsgs_portaladminmsgs"));
			

			// TODO: oops, this error will not display because it needs to be
			// set on the main page.
			RaiseError(m_asOwner.GetString(1265, "ptmsgs_portaladminmsgs"));

			return;
		}

		if (m_ptCatalog != null)
		{
			try
			{
				//PEI : IObjectActions
				//BrianH: Implementation of the OnCreateAdminFolder function of the IObjectActions
				//For each admin folder being created, the OnCreateAdminFolder function will be processed
				//to see if the settings being used are valid. This will only be done if there are implemented
				//IObjectActions functions otherwise the normal processing will occur.
				if (m_oObjectActions != null)
				{
					for (int x = 0; x < m_oObjectActions.length; x++)
					{
						oObjectAction = m_oObjectActions[x];

						
							log.Debug("ObjMgrModel.CreateFolder - About to call OnCreateAdminFolder on an: " + oObjectAction);
						

						if(oObjectAction != null)
						{
							String results[];

							iAction = (IObjectActions) oObjectAction;
							results = iAction.OnBeforeCreateAdminFolder(m_asOwner, (IPTSession) m_asOwner.GetUserSession(), 
								name, description);
							if (results != null)
							{
								if (results[0] != null && !("").equals(results[0]))
								{
									name = results[0];
								}
								if (results[1] != null && !("").equals(results[1]))
								{
									description = results[1];
								}
							}
						 }
					}
				}
				//PEI : OnCreateAdminFolder : END

				IPTAdminFolder myFolder = m_ptCatalog.CreateAdminFolder(m_nCurrentFolderID, PT_ADMIN_FOLDER_TYPES.PT_ADMIN_FOLDER_TYPE_REGULARFOLDER);
				myFolder.SetName(name);
				myFolder.SetDescription(description);

				// Store the object on the server
				IPTServerContext myContext = (IPTServerContext) myFolder.GetInterfaces(
						"IPTServerContext");
				myContext.Store();
				myContext.UnlockObject();
			}
			catch (Exception e)
			{
				
					log.Error(e, "Could not create Folder " + name);
				

				// TODO: oops, this error will not display because it needs to be
				// set on the main page.
				RaiseError(XPResourceManager.FormatMessage(m_asOwner.GetString(1264, "ptmsgs_portaladminmsgs"),
					name));
			}
		}
	}

	/**
	 * This method deletes folders.
	 *
	 * @param int[] the folder IDs to delete
	 */
	public void DeleteFolders(int[] nFolders)
	{
		m_aoHelper.DeleteFolders(nFolders);
	}

	/**
	 * This method deletes objects.
	 *
	 * @param XPHashtable a map of Integer Class IDs to int[] arrays of object IDs.
	 * @param boolean true implies ABO delete has been selected (so page templates
	 * will not be deleted through this method).
	 */
	public void DeleteObjects(XPHashtable xpmObjects, boolean bABODeleteObjects)
	{
		m_aoHelper.DeleteObjects(xpmObjects, bABODeleteObjects);
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#DoNotShowObject(int)
	 */
	public boolean DoNotShowObject(int nIndex)
	{
		return false;
	}

	/**
	 * This method enables and disables users and web services.
	 */
	public void EnableObjects(XPHashtable xpmObjects, String sComment, boolean bEnable)
	{
		m_aoHelper.EnableObjects(xpmObjects, sComment, bEnable);
	}

	/**
	 * This method expands an object type.
	 *
	 * @param int the class ID to expand
	 * @param boolean true implies expand, false implies collapse
	 */
	public void ExpandObjectType(int nClassID, boolean bExpand)
	{
		int[] nNewExpandedObjectTypes;

		if (bExpand)
		{
			// Add the new type
			nNewExpandedObjectTypes = new int[m_nExpandedObjectTypes.length +
				1];

			for (int i = 0; i < m_nExpandedObjectTypes.length; i++)
			{
				nNewExpandedObjectTypes[i] = m_nExpandedObjectTypes[i];
			}

			nNewExpandedObjectTypes[nNewExpandedObjectTypes.length - 1] = nClassID;
		}
		else
		{
			// remove the old type
			nNewExpandedObjectTypes = new int[m_nExpandedObjectTypes.length -
				1];

			m_bExpandAllObjectTypes = false;

			int nNewIndex = 0;

			for (int i = 0; i < m_nExpandedObjectTypes.length; i++)
			{
				if ((m_nExpandedObjectTypes[i] != nClassID) &&
						(nNewIndex < nNewExpandedObjectTypes.length))
				{
					nNewExpandedObjectTypes[nNewIndex] = m_nExpandedObjectTypes[i];
					nNewIndex++;
				}
			}
		}

		m_nExpandedObjectTypes = nNewExpandedObjectTypes;

		m_strExpandedObjectTypes = GeneralHelpers.ConvertToString(m_nExpandedObjectTypes);

		m_nCurrentlyOpeningType = nClassID;
	}

	/**
	 * This method returns the object type that is in the process of being
	 * expanded, if any.
	 *
	 * @return int the class ID that is being expanded, -1 if none are being expanded
	 */
	public int GetCurrentlyOpeningObjectType()
	{
		return m_nCurrentlyOpeningType;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.form.IRepostModel#Finish()
	 */
	public void Finish()
	{
	}

	/**
	 * @see com.plumtree.portalpages.admin.objmgr.IObjMgrModelRO#GetABOJobNames()
	 */
	public String[] GetABOJobNames(int[] nABOJobIDs)
	{
		return m_aoHelper.GetABOJobNames(nABOJobIDs);
	}

	/**
	 * @see com.plumtree.portalpages.admin.objmgr.IObjMgrModelRO#GetABOJobIDs()
	 */
	public int[] GetABOJobIDs()
	{
		return m_aoHelper.GetABOJobIDs();
	}

	/**
	 * @see com.plumtree.portalpages.admin.objmgr.IObjMgrModelRO#GetABOJobFolderName()
	 */
	public String GetABOJobFolderName()
	{
		// get the folder that the job was created in's localized name
		IPTAdminFolder myTempAdminFolder = m_ptCatalog.
			OpenAdminFolder(PT_INTRINSICS.PT_ADMIN_FOLDER_INTRINSICOPS, false);
		return myTempAdminFolder.GetLocalizedName();
	}

	/**
	 * @see com.plumtree.portalpages.admin.objmgr.IObjMgrModelRO#GetCommunityID()
	 */
	public int GetCommunityID(int nFolderID)
	{
		return m_aoHelper.GetCommunityID(nFolderID, m_nCurrentFolderID);
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetCurrentFolderID()
	 */
	public int GetCurrentFolderID()
	{
		return m_nCurrentFolderID;
	}

	/**
	 * This method returns the folder ID that we were previously at.
	 * This is used to return from places like the subportal manager.
	 *
	 * @return int the previous folder ID.
	 */
	public int GetPreviousFolderID()
	{
		return m_nPreviousFolderID;
	}

	/**
	 * @see com.plumtree.portalpages.admin.objmgr.IObjMgrModelRO#GetBreadCrumbPath()
	 */
	public IASQueryResult GetBreadCrumbPath()
	{
		return new ASQueryResultWrapper(GetCurrentFolder().GetPathAsQueryResult());
	}

	/**
	 * @see com.plumtree.portalpages.admin.objmgr.IObjMgrModelRO#GetCurrentFolderIsCommunity()
	 */
	public boolean GetCurrentFolderIsCommunity()
	{
		return (GetCurrentFolder().GetFolderType() == PT_ADMIN_FOLDER_TYPES.PT_ADMIN_FOLDER_TYPE_COMMUNITYFOLDER);
	}

	/**
	 * @see com.plumtree.portalpages.admin.objmgr.IObjMgrModelRO#GetCurrentFolderIsSubportal()
	 */
	public boolean GetCurrentFolderIsSubportal()
	{
		if (GetCurrentFolder().GetFolderType() == PT_ADMIN_FOLDER_TYPES.PT_ADMIN_FOLDER_TYPE_REGULARFOLDER)
		{
			return false;
		} else if (GetCurrentFolder().GetFolderType() == PT_ADMIN_FOLDER_TYPES.PT_ADMIN_FOLDER_TYPE_COMMUNITYFOLDER)
		{
			return false;
		}

		// Everything else must be a subportal
		return true;
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetCurrentFolderLevel()
	 */
	public int GetCurrentFolderLevel()
	{
		return GetCurrentFolder().GetFolderLevel();
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetCurrentFolderName()
	 */
	public String GetCurrentFolderName()
	{
		return GetCurrentFolder().GetLocalizedName();
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IPaginationModelRO#GetCurrentLowerObjectIndex()
	 */
	public int GetCurrentLowerObjectCount()
	{
		// NOTE: The ObjMgrHolder will contain the "real" pagination counts, which may be different
		// from the index into the query result.  This is because we can only query up to 1000 objects
		// at a time.  Therefore, when we want to query for the 1005th object, we have to start a new
		// query and translate 1005 into the index into the query (5th result -- i=4).  This method
		// should return the "real" object index, rather than the index into the query result.

		// Code outside of the model should only see "real" counts, whereas code inside should use
		// query indices.

		// Default start at beginning
		int nResult = 1;

		ObjectManagerHolder temp = GetObjMgrHolder(m_nObjectClassID);

		if (temp != null)
		{
			if (temp.GetFirstObjectCount() != -1)
			{
				// use the pagination setting if it has been set.
				nResult = (temp.GetFirstObjectCount() >= 1) ? temp.GetFirstObjectCount() : 1;

				// Pagination includes skipped rows (i.e. real numbers of objects, not necessarily
				// the number of objects in the current query since we only query for 1000 objects)

				// If for some reason our low count is lower than the first object, or greater than
				// the number of objects we have, set it to the first object.
				if ((nResult <= temp.GetSkippedRows()) ||
					(nResult > m_nTotalObjectCount))
				{
					// NOTE: This should never happen.
					nResult = temp.GetSkippedRows() + 1;
				}
			}
		}
		else
		{
			
				log.Error("Unable to get object manager for pagination.");
			
		}

		return nResult;
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IPaginationModelRO#GetCurrentUpperObjectIndex()
	 */
	public int GetCurrentUpperObjectCount()
	{
		// NOTE: The ObjMgrHolder will contain the "real" pagination counts, which may be different
		// from the index into the query result.  This is because we can only query up to 1000 objects
		// at a time.  Therefore, when we want to query for the 1005th object, we have to start a new
		// query and translate 1005 into the index into the query (5th result -- i=4).  This method
		// should return the "real" object index, rather than the index into the query result.

		// Code outside of the model should only see "real" counts, whereas code inside should use
		// query indices.

		// Default use the object count or the user's pagination preference
		int nResult = (m_nObjectCount <= GetPaginationSize()) ? m_nObjectCount : GetPaginationSize();

		ObjectManagerHolder temp = GetObjMgrHolder(m_nObjectClassID);

		if (temp != null)
		{
			if (temp.GetLastObjectCount() != -1)
			{
				// use the pagination setting if it has been set.
				nResult = (temp.GetLastObjectCount() >= 1) ? temp.GetLastObjectCount() : 1;

				// Pagination includes skipped rows (i.e. real numbers of objects, not necessarily
				// the number of objects in the current query since we only query for 1000 objects)

				// If for some reason our max count is too high or too low, set it to the last object we have.
				if ((nResult > m_nTotalObjectCount) ||
					(nResult <= temp.GetSkippedRows()))
				{
					// NOTE: This should never happen.
					nResult = m_nTotalObjectCount;
				}
			}
		}
		else
		{
			
				log.Error("Unable to get object manager for pagination.");
			
		}

		return nResult;
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetDisplayPaginationHiddenInputs()
	 */
	public boolean GetDisplayPaginationHiddenInputs()
	{
		// return true for the first one, then suppress after (per request)
		boolean bResult = !m_bPaginationInputsDisplayed;

		if (bResult == true)
		{
			m_bPaginationInputsDisplayed = true;
		}

		return bResult;
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetDisplaySortHiddenInputs()
	 */
	public boolean GetDisplaySortHiddenInputs()
	{
		// return true for the first one, then suppress after (per request)
		boolean bResult = !m_bSortInputsDisplayed;

		if (bResult == true)
		{
			m_bSortInputsDisplayed = true;
		}

		return bResult;
	}

	/**
	 * @see com.plumtree.portalpages.admin.objmgr.IObjMgrModelRO#GetExpandedObjectTypesArray()
	 */
	public int[] GetExpandedObjectTypesArray()
	{
	    if (m_bExpandAllObjectTypes && m_aoHelper.GetObjMgrs().GetSize() > m_nExpandedObjectTypes.length) {
	        ExpandAllClasses();
	    }
		return m_nExpandedObjectTypes;
	}

	///////////////////////////////////
	// Read methods from IObjMgrModelRO
	///////////////////////////////////

	/**
	 * @see com.plumtree.portalpages.admin.objmgr.IObjMgrModelRO#GetExpandedObjectTypesString()
	 */
	public String GetExpandedObjectTypesString()
	{
		if (m_strExpandedObjectTypes == null)
		{
			m_strExpandedObjectTypes = GeneralHelpers.ConvertToString(m_nExpandedObjectTypes);
		}

		return m_strExpandedObjectTypes;
	}

	/**
	 * This method is used to determine whether to display the 'expand all'
	 * button or the 'collapse all' button.
	 *
	 * @return boolean true implies all object expanded button has
	 * been clicked.
	 */
	public boolean GetAreAllObjectsExpanded()
	{
		return m_bExpandAllObjectTypes;
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetFirstFolderIndex()
	 */
	public int GetFirstFolderIndex()
	{
		return m_nFirstItemIndex;
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetFolderColumnCount()
	 */
	public int GetFolderColumnCount()
	{
		//root folder has two columns
		if (InRootFolder())
		{
			return 2;
		}

		// Otherwise return three columns
		return 3;
	}

	/**
	 * This method gets the selected folder from the tree sub space.
	 *
	 * @return int the folder ID (-1 implies error)
	 */
	public int GetFolderIDFromTree()
	{
		return m_aoHelper.GetFolderIDFromTree(ObjMgrAS.FOLDER_TREE_SUB_SPACE_KEY);
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetFolderObjectCount()
	 */
	public IASQueryResult GetFolderObjectCount()
	{
		ASQueryResultWrapper result = null;

		try
		{
			result = new ASQueryResultWrapper(GetCurrentFolder().QueryObjectsCount(false));
		}
		catch (Exception e)
		{
		}

		return result;
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetFolderData()
	 */
	public IASQueryResult GetFolderQueryResult()
	{
		// Get the query of the current subgolders
		if (m_asqrSubfolders == null)
		{
			// 0 means just this level, 1 means also one level down
			// In the root folder we also display sub-folders
			int nLevel = InRootFolder() ? 1 : 0;

			try
			{
				m_asqrSubfolders = new ASQueryResultWrapper(GetCurrentFolder().QuerySubfolders(PT_PROPIDS.PT_PROPID_OBJECTID |
							PT_PROPIDS.PT_PROPID_NAME |
							PT_PROPIDS.PT_PROPID_FOLDER_LEVEL |
							PT_PROPIDS.PT_PROPID_FOLDER_FOLDERTYPE, nLevel,
							new Integer(PT_PROPIDS.PT_PROPID_NAME), 0, -1, null));
			}
			catch (Exception e)
			{
				
					log.Error(e, "Unable to query sub folders.");
				

				m_asqrSubfolders = null;
			}
		}

		return m_asqrSubfolders;
	}

	/**
	 * @see com.plumtree.portalpages.admin.objmgr.IObjMgrModelRO#GetFolderTreeID()
	 */
	public String GetFolderTreeID()
	{
		return m_aoHelper.GetFolderTreeID();
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetFolderViewHelper()
	 */
	public IFolderViewHelper GetFolderViewHelper()
	{
		return m_folderViewHelper;
	}

	/**
	 * This method checks whether or not the current page has any objects that
	 * will be displayed for the requested classID.
	 *
	 * @param int nClassID the class ID to check for
	 */
	public boolean GetHasDisplayedObjects(int nClassID)
	{
		boolean bResult = false;

		if (m_bExpandAllObjectTypes)
		{
			bResult = true;
		} else
		{
			// Check if this class is even expanded
			for (int i = 0; i < m_nExpandedObjectTypes.length; i++)
			{
				if (m_nExpandedObjectTypes[i] == nClassID)
				{
					bResult = true;
				}
			}
		}

		if (bResult == true)
		{
			bResult = false;

			// Check if there are any objects of this class ID.
			// Get the query of the number and types of objects in this folder
			InitializeObjectCount();

			for (int i = 0; i < m_asqrObjectCount.GetCount(); i++)
			{
				if (  XPConvert.ToInteger(m_asqrObjectCount.GetFields(
									i, PT_PROPIDS.PT_PROPID_CLASSID)) == nClassID)
				{
					if ( XPConvert.ToInteger(m_asqrObjectCount.GetFields(i,
										PT_PROPIDS.PT_PROPID_OBJECTCOUNT)) > 0)
					{
						return true;
					}
				}
			}
		}

		return bResult;
	}

	/**
	 * This method checks approximately how many objects of
	 * the requested classID are displayed on the page.  This
	 * number doesn't take security into account, so it can
	 * return a higher than actual value.
	 *
	 * @param int nClassID the class ID to check for
	 */
	public int GetRoughDisplayedObjectCount(int nClassID)
	{
		if (!m_bExpandAllObjectTypes)
		{
			boolean bDisplayed = false;

			// Check if this class is even expanded
			for (int i = 0; i < m_nExpandedObjectTypes.length; i++)
			{
				if (m_nExpandedObjectTypes[i] == nClassID)
				{
					bDisplayed = true;
				}
			}

			if (!bDisplayed)
			{
				return 0;
			}
		}

		int nNumObjects = 0;

		// Check if there are any objects of this class ID.
		// Get the query of the number and types of objects in this folder
		InitializeObjectCount();

		for (int i = 0; i < m_asqrObjectCount.GetCount(); i++)
		{
			if (  XPConvert.ToInteger(m_asqrObjectCount.GetFields(
								i, PT_PROPIDS.PT_PROPID_CLASSID)) == nClassID)
			{
				nNumObjects = XPConvert.ToInteger(m_asqrObjectCount.GetFields(i,
					PT_PROPIDS.PT_PROPID_OBJECTCOUNT));
			}
		}

		return nNumObjects;
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetHeaderCount()
	 */
	public int GetHeaderCount()
	{
		if (m_nObjectClassID == PT_CLASSIDS.PT_JOB_ID)
		{
			if (null == m_astrJobHeaders)
			{
				return 0;
			}

			return m_astrJobHeaders.length;
		}

		if (null == m_astrHeaders)
		{
			return 0;
		}

		return m_astrHeaders.length;
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetHeaderHasCheckbox()
	 */
	public boolean GetHeaderHasCheckbox()
	{
		return true;
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetHeaderRow()
	 */
	public HTMLTableRow GetHeaderRow()
	{
		return null;
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetHeaderText(int)
	 */
	public String GetHeaderText(int nHeaderIndex)
	{
		if (m_nObjectClassID == PT_CLASSIDS.PT_JOB_ID)
		{
			return m_astrJobHeaders[nHeaderIndex];
		}

		return m_astrHeaders[nHeaderIndex];
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetIsCollapsable()
	 */
	public boolean GetIsCollapsable()
	{
		return true;
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetShowCheckbox()
	 */
	public boolean GetIsFolderSelected(int nFolderID)
	{
		// We currently do not track which folders are selected.  This means that
		// if the page is refreshed, the folders will no longer be selected.
		return false;
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetIsObjectSelected(int)
	 */
	public boolean GetIsObjectSelected(int nObjectID)
	{
		ObjectManagerHolder objMgrHolder = GetObjMgrHolder(m_nObjectClassID);

		if (objMgrHolder != null)
		{
			return objMgrHolder.IsObjectSelected(nObjectID);
		}

		return false;
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetLastFolderIndex()
	 */
	public int GetLastFolderIndex()
	{
		return m_nLastItemIndex;
	}

	//////////////////////////////////
	// Read only methods from IModelRO
	//////////////////////////////////

	/**
	 * @see com.plumtree.xpshared.activityspace.IModel#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	///////////////////////////////////////////
	// Read only method from IPaginationModelRO
	///////////////////////////////////////////

	/**
	 * @see com.plumtree.portalpages.common.objects.IPaginationModelRO#GetNumObjects()
	 */
	public int GetNumObjects()
	{
		return m_nObjectCount;
	}


	public int GetPaginationSize()
	{
		return PTConfigHelper.GetConfigVariableAsInt("Administration",
			"AdminObjectsPerPage", ConfigConstants.VALUE_ATTRIBUTE, m_asOwner);
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetNumSubSubFolders()
	 */
	public int GetNumSubSubFoldersToDisplay()
	{
		// Display 6 sub sub folders in the root folder
		return NUM_SUB_SUB_FOLDERS;
	}

	////////////////////////////////////////////
	// Read only methods from IObjectModelRO
	////////////////////////////////////////////

	/**
	 * @see com.plumtree.portalpages.browsing.objects.IObjectModelRO#GetObjectBannerTitle()
	 */
	public String GetObjectBannerTitle()
	{
		String strLangID = (String) PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingsHelper.LANGUAGE, m_asOwner);
		ClassTypeDescManager ptClassTypeDescManager = PTConfigHelper.GetPTClassTypeDescrManager();
		String strClassTypeName					    = ptClassTypeDescManager.GetClassTypeLocalizedName(m_nObjectClassID, strLangID);

		if (strClassTypeName != null)
		{
			return strClassTypeName;
		}
		else
		{
			
				log.Error("Unable to find name for this class ID: " +
					m_nObjectClassID);
			

			return m_asOwner.GetString(2421, "ptmsgs_portaladminmsgs"); //"Object"
		}
	}

	/**
	 * @see com.plumtree.portalpages.browsing.objects.IObjectModelRO#GetObjectColumnCount()
	 */
	public int GetObjectColumnCount()
	{
		return 1;
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetObjectCount()
	 */
	public int GetObjectCount()
	{
		// NOTE: The ObjMgrHolder will contain the "real" pagination counts, which may be different
		// from the index into the query result.  This is because we can only query up to 1000 objects
		// at a time.  Therefore, when we want to query for the 1005th object, we have to start a new
		// query and translate 1005 into the index into the query (5th result -- i=4).  This method
		// should return the "real" object count, rather than the index into the query result.

		// Code outside of the model should only see "real" counts, whereas code inside should use
		// query indices.

		// Bug 66279 - error when attempting to display more than the default objects of a type in the Admin UI.
		/*
		ObjectManagerHolder objMgrHolder = GetObjMgrHolder(m_nObjectClassID);

		// Translate from query count into "real" count.
		int nRealCount = m_nObjectCount + objMgrHolder.GetSkippedRows();

		return nRealCount;
		*/
		
		return m_nTotalObjectCount;
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetObjectFields(int,
	 *      int)
	 */
	public Object GetObjectFields(int nIndex, int ptPropID)
	{
		// NOTE: The ObjMgrHolder will contain the "real" pagination counts, which may be different
		// from the index into the query result.  This is because we can only query up to 1000 objects
		// at a time.  Therefore, when we want to query for the 1005th object, we have to start a new
		// query and translate 1005 into the index into the query (5th result -- i=4).  This method
		// should get passed the "real" object index, rather than the index into the query result.

		// Code outside of the model should only see "real" counts, whereas code inside should use
		// query indices.

		ObjectManagerHolder objMgrHolder = GetObjMgrHolder(m_nObjectClassID);

		// Translate from "real" index into query index.
		int nQueryIndex = nIndex - objMgrHolder.GetSkippedRows();

		return m_asqrObjects.GetFields(nQueryIndex, ptPropID);
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetObjectType()
	 */
	public int GetObjectType()
	{
		return m_nObjectClassID;
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetObjectSubType()
	 */
	public int GetObjectSubType(int nIndex) 
	{
		int nSubType = -1;

		if (PT_CLASSIDS.PT_USER_ID == m_nObjectClassID)
		{
			try 
			{
				Object temp = GetObjectFields
					(nIndex, PT_PROPIDS.PT_PROPID_USER_USERTYPE);
			
				if (null != temp) 
				{
					nSubType = XPConvert.ToInteger(temp);
				}
			
				// log.Debug("Object sub type: " + nSubType);
			} catch (Exception e)
			{
				log.Error(e, "ObjMgrViewHelper:DetermineSubType cannot determine user type.");
			}
		}
				
		return nSubType;		
	}
	
	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetObjectViewIsCollapsed()
	 */
	public boolean GetObjectViewIsCollapsed()
	{
		return m_bIsCollapsed;
	}

	////////////////////////////////////////////
	// Read only methods from IFolderModelRO
	////////////////////////////////////////////

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetParentFolderID()
	 */
	public int GetParentFolderID()
	{
		return GetCurrentFolder().GetParentFolderID();
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetParentFolderIDs()
	 */
	public Object[] GetParentFolderIDs()
	{
		// Not used by ObjMgrBreadCrumbView
		return null;
		// return (Object[]) GetCurrentFolder().GetParentIDs();
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetPath()
	 */
	public String GetPath()
	{
		// Not used by ObjMgrBreadCrumbView
		return null;
		// return GetCurrentFolder().GetPath();
	}

	/**
	 * @see com.plumtree.uiinfrastructure.form.IRepostModel#GetRedirectAfterFinish()
	 */
	public Redirect GetRedirectAfterFinish()
	{
		return null;
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetShowCheckbox()
	 */
	public boolean GetShowCheckbox()
	{
		return true;
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetShowBanner()
	 */
	public boolean GetShowFolderBanner()
	{
		//        if (InRootFolder() && m_asOwner.GetCurrentPage().GetName().equals(ObjMgrRootDP.STR_MVC_CLASS_NAME) ) {
		return false;

		//        }
		//        return true;
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetShowPagination()
	 */
	public boolean GetShowPagination()
	{
		return true;
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.ISortModelRO#GetSortDirection()
	 */
	public int GetSortDirection()
	{
		ObjectManagerHolder temp = GetObjMgrHolder(m_nObjectClassID);

		if (temp != null)
		{
			return temp.GetSortDirection();
		}
		else
		{
			
				log.Error("Unable to get object manager for sort direction.");
			
		}

		// Default direction
		return PTSortedArrayWrapper.PTSAW_SORT_DESCENDING;
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetSortable()
	 */
	public boolean GetSortable()
	{
		return true;
	}

	////////////////////////////////////////////
	// Read only methods from ISortModelRO
	////////////////////////////////////////////

	/**
	 * @see com.plumtree.portalpages.common.objects.ISortModelRO#GetSortedHeader()
	 */
	public int GetSortedHeader()
	{
		ObjectManagerHolder temp = GetObjMgrHolder(m_nObjectClassID);

		if (temp != null)
		{
			int nPropID = temp.GetSortPropID();

			switch (nPropID)
			{
			case PT_PROPIDS.PT_PROPID_NAME:
				return HEADER_NAME_INDEX;

			case PT_PROPIDS.PT_PROPID_LASTMODIFIED:
				return HEADER_MODIFIED_INDEX;

			case PT_PROPIDS.PT_PROPID_CREATED:
				return HEADER_CREATED_INDEX;

			default:

				
					log.Error("Sorting on unknown header " + nPropID);
				

				break;
			}
		}
		else
		{
			
				log.Error("Unable to get object manager for sort header.");
			
		}

		// Default name
		return HEADER_NAME_INDEX;
	}

	/**
	 * Get the SpaceID of the parent Activity Space
	 *
	 * @return int m_asOwner.GetSpaceID();
	 */
	public String GetSpaceID()
	{
		return m_asOwner.GetSpaceID();
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetFolderCount()
	 * Get all subfolders in query at the appropriate level and above.
	 */
	public int GetSubFolderCount(int nLevel)
	{
		if (GetCurrentFolderLevel() == nLevel)
		{
			// This will not return any subfolders below the current subfolders
			return GetCurrentFolder().QuerySubfoldersCount();
		} else if (nLevel > GetCurrentFolderLevel())
		{
			// We can return this, because we never query for more than
			// one level past the current level.
			return GetSubFolderCount();
		}

		// If we are looking for folders above our current level, don't
		// return any.
		return 0;
	}
	
	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetFolderCount()
	 * Get the number of folders at all levels in the current query
	 */
	public int GetSubFolderCount()
	{
		return GetFolderQueryResult().GetCount();
	}

	/**
	 * JF- Helper method to determine if the current user should see the create 
	 * object menu. (He should see it only if he has at least edit rights to 
	 * the current admin folder.)
	 * 
	 * @return boolean
	 */
	public boolean HasAccessToCreateMenu()
	{
		return m_ptCatalog.IsAdminFolderAccessible(m_nCurrentFolderID,
			PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_EDIT);
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#InRootFolder()
	 */
	public boolean InRootFolder()
	{
		return (m_nCurrentFolderID == PT_INTRINSICS.PT_ADMIN_FOLDER_ROOTFOLDER);
	}

	////////////////////////////
	// Write methods from IModel
	////////////////////////////

	/**
	 * @see com.plumtree.xpshared.activityspace.IModel#Init()
	 */
	public void Init(AActivitySpace parent)
	{
		m_asOwner			  = parent;
		m_ptCatalog			  = ((IPTSession) m_asOwner.GetUserSession()).GetAdminCatalog();
		m_ptCurrentFolder     = m_ptCatalog.GetRootAdminFolder();

		// Set the help topic
		m_asOwner.SetHelpTopic("PT_Administration");

		m_astrHeaders    = new String[4];
		m_astrJobHeaders = new String[7];

		// We need to manually specify the root folder ID because Folder.GetID
		// doesn't seem to work right for the root folder.
		m_nCurrentFolderID		 = PT_INTRINSICS.PT_ADMIN_FOLDER_ROOTFOLDER;
		m_nPreviousFolderID		 = PT_INTRINSICS.PT_ADMIN_FOLDER_ROOTFOLDER;

		m_nExpandedObjectTypes = new int[0];
		m_bExpandAllObjectTypes = false;

		SetObjMgrSettings(m_nCurrentFolderID, m_nExpandedObjectTypes);

		// the first header is a checkbox, and needs to be set for each object type
		m_astrHeaders[HEADER_CHECKBOX_INDEX]     = ""; // Will be replaced with in_cb_XXX during MoveNext()
		m_astrHeaders[HEADER_NAME_INDEX]		 = m_asOwner.GetString(2137,
			"ptmsgs_portaladminmsgs"); //"Name";
		m_astrHeaders[HEADER_MODIFIED_INDEX]     = m_asOwner.GetString(2138,
			"ptmsgs_portaladminmsgs"); //"Last Modified";
		m_astrHeaders[HEADER_CREATED_INDEX]		 = m_asOwner.GetString(2139,
			"ptmsgs_portaladminmsgs"); //"Created";

		// Job specific headers
		m_astrJobHeaders[HEADER_CHECKBOX_INDEX]     = ObjMgrViewHelper.HTML_OBJECT_CHECKBOX_PREFIX +
			PT_CLASSIDS.PT_JOB_ID + ObjMgrViewHelper.HTML_OBJECT_CHECKBOX_SUFFIX;
		m_astrJobHeaders[HEADER_NAME_INDEX]		 	= m_asOwner.GetString(2137,
			"ptmsgs_portaladminmsgs"); //"Name";
		m_astrJobHeaders[HEADER_MODIFIED_INDEX]     = m_asOwner.GetString(2138,
			"ptmsgs_portaladminmsgs"); //"Last Modified";
		m_astrJobHeaders[HEADER_CREATED_INDEX]		= m_asOwner.GetString(2139,
			"ptmsgs_portaladminmsgs"); //"Created";
		m_astrJobHeaders[HEADER_LAST_RUN_INDEX]		= m_asOwner.GetString(2478,
			"ptmsgs_portaladminmsgs"); //"Last Run";
		m_astrJobHeaders[HEADER_NEXT_RUN_INDEX]		= m_asOwner.GetString(2479,
			"ptmsgs_portaladminmsgs"); //"Next Run";
		m_astrJobHeaders[HEADER_JOB_STATUS_INDEX]	= m_asOwner.GetString(2480,
			"ptmsgs_portaladminmsgs"); //"Status";

		//PEI : ObjectActions
		//BrianH: Initialization of all ObjectActions PEIs
		IApplication application = m_asOwner.GetApplication();
		m_oObjectActions = (Object[]) application.GetCachingManager().GetEntry("ObjectActions");
		
		m_aoHelper 	= new AdminModelHelper();
		m_aoHelper.Init(parent);

	}

	/**
	 * This method adds folders into the migration queue.
	 */
	public void MigrateFolders(int[] nFolderIDs, String sComment)
	{
		m_aoHelper.MigrateFolders(nFolderIDs, sComment);	
	}

	/**
	 * This method adds objects into the migration queue.
	 */
	public void MigrateObjects(XPHashtable xpmObjects, String sComment)
	{
		String strErrors = "";

		Object nCurClass;
		int nCurClassID;
		int[] nIDs;

		IPTMigrationManager ptMM = (IPTMigrationManager) ((IPTSession) m_asOwner.GetUserSession()).OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_MIGRATION_MANAGER,
				false);

		IXPEnumerator classIDEnum = xpmObjects.GetKeys();
		IXPEnumerator objectEnum  = xpmObjects.GetElements();

		Object oObjectAction;
		IObjectActions iAction;
		
		// JF- Error messages from all PEIs for all objects
		String sPEIErrorMsgs = "";

		while (objectEnum.MoveNext() && classIDEnum.MoveNext())
		{
			nCurClass     = classIDEnum.GetCurrent();
			nCurClassID = XPConvert.ToInteger(nCurClass);
			nIDs		  = (int[]) objectEnum.GetCurrent();

			if ((nCurClass != null) && (nIDs != null) && (nIDs.length > 0))
			{
				try
				{
					//PEI : IObjectActions - BEGIN
					//BrianH: Implementation of the OnMigrateObject function of the IObjectActions
					//For each class of objects and their corresponding integer array of object ids
					//the functions will be called to see if the objects should be migrated. This will
					//only be done if there are implemented IObjectActions functions otherwise the normal
					//processing will occur. The normal processing will also occur if no errors are returned.
					if (m_oObjectActions != null)
					{
						// JF- Error messages from all PEIs for the current object
						String sObjectErrorMsgs = "";
											 
						for (int x = 0; x < m_oObjectActions.length; x++)
						{
							oObjectAction = m_oObjectActions[x];

							
								log.Debug("ObjMgrModel.MigrateObjects - About to call OnMigrateObject on an: " + oObjectAction);
							

							if(oObjectAction != null)
							{
								String sError;

								iAction = (IObjectActions) oObjectAction;
								sError = iAction.OnBeforeMigrateObject(m_asOwner, 
									(IPTSession) m_asOwner.GetUserSession(), nCurClassID, nIDs);
								
								if ((null != sError) && (sError.length() > 0))
								{
									sObjectErrorMsgs += sError + " ";
								}
							 }
						}
						
						if ((null != sObjectErrorMsgs) && (sObjectErrorMsgs.length() > 0))
						{
							sPEIErrorMsgs += sObjectErrorMsgs + " ";
						}
						else
						{
							ptMM.BulkSetMigrationStatus(XPConvert.ToInteger(nCurClass), nIDs,
								PT_MIGRATION_STATUS_TYPES.PT_MIGRATION_STATUS_APPROVAL_REQUESTED,
								sComment);
						}
					}
					else
					{
						ptMM.BulkSetMigrationStatus(XPConvert.ToInteger(nCurClass), nIDs,
							PT_MIGRATION_STATUS_TYPES.PT_MIGRATION_STATUS_APPROVAL_REQUESTED,
							sComment);
					}
					//PEI : ObjectActions : OnMigrateObject END
				}
				catch (Exception e)
				{
					ClassTypeDescManager ptClassTypeDescManager = PTConfigHelper.GetPTClassTypeDescrManager();
					String strLangID = (String) PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingsHelper.LANGUAGE, m_asOwner);

					String strErrorMessage = XPResourceManager.FormatMessage
						(m_asOwner.GetString(2514, "ptmsgs_portaladminmsgs"),
						ptClassTypeDescManager.GetClassTypeLocalizedName(nCurClassID, strLangID));

					
						log.Error(e, strErrorMessage);
					

					strErrors += strErrorMessage + " ";
				}
			}
		}
		
		if ((null != sPEIErrorMsgs) && (sPEIErrorMsgs.length() > 0))
		{
			strErrors += sPEIErrorMsgs;
		}

		if (!"".equals(strErrors))
		{
			RaiseError(strErrors);
		}
	}

	/**
	 * This method moves folders to a new folder.
	 *
	 * @param int nTargetFolderID the ID of the folder to move the folders to.
	 * @param int[] nFolders an array of folder IDs.
	 */
	public void MoveFolders(int nTargetFolderID, int[] nFolders)
	{
		m_aoHelper.MoveFolders(nTargetFolderID, nFolders);
	}

	/**
	 * This method moves to the next object type to display.
	 *
	 * This can only be called until it returns false, and then can't be
	 * called until the next request.
	 *
	 * @return boolean true implies success,
	 *             false implies failure, no more object types to display
	 */
	public boolean MoveNext()
	{
		// If this is the first time we are calling MoveNext() in this
		// request, initialize the query of the estimated number and
		// types of objects in this folder
		InitializeObjectCount();

		if (m_asqrObjectCount.GetCount() == 0)
		{
			return false;
		}

		// Move to the next object
		m_nObjectCountIndex++;

		int nEstimatedObjectCount;

		if (m_asqrObjectCount.GetCount() > m_nObjectCountIndex)
		{
			m_nObjectClassID     = XPConvert.ToInteger(
						m_asqrObjectCount.GetFields(m_nObjectCountIndex,
							PT_PROPIDS.PT_PROPID_CLASSID));
			// This is the count of all objects of this type in the current
			// folder, without checking whether or not the current user
			// actually has security to see all those objects.
			nEstimatedObjectCount = XPConvert.ToInteger(
						m_asqrObjectCount.GetFields(m_nObjectCountIndex,
							PT_PROPIDS.PT_PROPID_OBJECTCOUNT));

			if (IsSuppressedClass(m_nObjectClassID))
			{
				// move to the next object type
				return MoveNext();
			}

			// Figure out whether this object type is expanded or collapsed
			if (m_bExpandAllObjectTypes == true)
			{
				m_bIsCollapsed = false;
			} else
			{
				m_bIsCollapsed = true;

				for (int i = 0; i < m_nExpandedObjectTypes.length; i++)
				{
					if (m_nExpandedObjectTypes[i] == m_nObjectClassID)
					{
						m_bIsCollapsed = false;
					}
				}
			}

			// Set the object count to the most accurate count we have
			if (!m_bIsCollapsed)
			{
				ObjectManagerHolder curMgrHolder = GetObjMgrHolder(
							m_nObjectClassID);

				if (curMgrHolder == null)
				{
					// Failure -- move to the next object type
					return MoveNext();
				}

				try
				{
					curMgrHolder.SetDefaultPaginationSize(GetPaginationSize());
					m_asqrObjects = curMgrHolder.QueryObjects(m_nCurrentFolderID);
				}
				catch (Exception e)
				{
					
						log.Error(e, "Unable to query objects of type " +
							curMgrHolder.m_ptObjMgr.GetManagedClassID());
					

					m_asqrObjects = null;
				}

				if (m_asqrObjects == null)
				{
					// Failure -- move to the next object type
					return MoveNext();
				}

				// This is the count of objects returned in the query that the
				// user actually has access to see.  This query, however, will
				// always be truncated at the max rows count (1000), so there
				// may be more objects.
				m_nObjectCount = m_asqrObjects.GetCount(); 
				m_nTotalObjectCount = curMgrHolder.m_ptObjMgr.QueryCount(m_nCurrentFolderID, null);

				// If we estimated there to be more than the max number of queryable
				// objects, and the query returned the max number, then there may be
				// more objects than were actually returned in the query.
				// Comment out below (bug 27566 and bug 21969)
				/*
				if ((nEstimatedObjectCount > APTQueryWrapper.PTTQRW_DEFAULT_MAX_ROWS) &&
					(m_nObjectCount >= APTQueryWrapper.PTTQRW_DEFAULT_MAX_ROWS))
				{
					// The true count may be larger than the query size, so we
					// must re-query for the count (no query filter) on the object
					// manager, which should give us a more accurate count than our
					// original estimate.
					// Note that this means that m_nObjectCount is greater than the actual
					// count of objects in the query.  This means that the model and view
					// must be careful not to ask for more than the max number of items
					// (which should be the same or less than the actual number of items in
					// the query)
					m_nObjectCount = curMgrHolder.m_ptObjMgr.QueryCount(m_nCurrentFolderID, null);
				}
				*/

				m_astrHeaders[HEADER_CHECKBOX_INDEX] = ObjMgrViewHelper.HTML_OBJECT_CHECKBOX_PREFIX +
					m_nObjectClassID + ObjMgrViewHelper.HTML_OBJECT_CHECKBOX_SUFFIX;
			} else
			{
				// Since this object type is not expanded, just use the estimated count.
				m_nObjectCount = nEstimatedObjectCount;
				m_nTotalObjectCount = nEstimatedObjectCount;
			}

			return true;
		}

		// Otherwise we are at the end of the object types -- fail
		return false;
	}

	/** 
	 * Helper method to initialize this query at the beginning of every
	 * request.
	 */
	private void InitializeObjectCount()
	{
		if (m_asqrObjectCount == null)
		{
			// This query returns the total number of objects of all types
			// in this folder, but does not check if the user has access
			// to see them all.  Therefore, this count may be higher than
			// the true count of objects that the user can actually see.
			// This query is used because it is more performant than one
			// that checks security.
			m_asqrObjectCount = new ASQueryResultWrapper(GetCurrentFolder().QueryObjectsCount(false));

			// Since this is a new query, we reset the object index.
			// This is the index into the query and allows us to iterate
			// through all of the different object types that are in the
			// current folder.
			m_nObjectCountIndex = -1;
		}
	}
	/**
	 * This method moves objects between folders.
	 *
	 * @param int nFolderID the ID of the folder to move the objects to.
	 * @param Object[][] nObjects an (2 x n) array of class IDs / Object IDs
	 */
	public void MoveObjectsBetweenFolders(int nFolderID, Object[][] nObjects)
	{
		if (nObjects[0].length == 0)
		{
			return;
		}

		if (nFolderID == PT_INTRINSICS.PT_ADMIN_FOLDER_ROOTFOLDER)
		{
			// Cannot move objects to the root folder.
			RaiseError(m_asOwner.GetString(2158, "ptmsgs_portaladminmsgs"));
			return;
		}

		// Should I get the list of objs you can't move?
		boolean bNonCommunityObjects = false;
		int nID;

		Object oObjectAction;
		IObjectActions iAction;

		for (int i = 0; i < nObjects[0].length; i++)
		{
			nID = XPConvert.ToInteger(nObjects[0][i]);

			if ((nID != PT_CLASSIDS.PT_USERGROUP_ID) &&
					(nID != PT_CLASSIDS.PT_GADGET_ID) &&
					(nID != PT_CLASSIDS.PT_PAGE_ID))
			{
				bNonCommunityObjects = true;

				break;
			}
		}

		if (bNonCommunityObjects)
		{
			IPTAdminFolder targetFolder;

			// Check if we are in a community
			try
			{
				targetFolder = m_ptCatalog.OpenAdminFolder(nFolderID, false);
			}
			catch (Exception e)
			{
				targetFolder = null;

				String strErrorMessage = XPResourceManager.FormatMessage(
					m_asOwner.GetString(1254, "ptmsgs_portaladminmsgs"),
					new Integer(nFolderID));

				
					log.Error(e, strErrorMessage);
				

				RaiseError(strErrorMessage);

				return;
			}

			if (targetFolder.GetFolderType() == PT_ADMIN_FOLDER_TYPES.PT_ADMIN_FOLDER_TYPE_COMMUNITYFOLDER)
			{
				
					log.Error(XPResourceManager.FormatMessage(m_asOwner.GetString(1253, "ptmsgs_portaladminmsgs"),
						targetFolder.GetLocalizedName()));
				

				RaiseError(XPResourceManager.FormatMessage(m_asOwner.GetString(1253, "ptmsgs_portaladminmsgs"),
					targetFolder.GetLocalizedName()));

				// TODO: Shouldn't this still move the community objects?
				return;
			}
		}

		String sErrorMsgs = "";

		Object[][] arrObjectsToMove = nObjects;

		try
		{
			//PEI : IObjectActions - BEGIN
			//BrianH: Implementation of the OnMoveObject function of the IObjectActions
			//For each class of objects the functions will be called to determine if the objects
			//requested should be moved to the target folder. This will only be done if there
			//are implemented IObjectActions functions otherwise the normal processing will occur.
			//The normal processing will also occur if no errors are returned.
			if (m_oObjectActions != null)
			{
				int nClassID;
				int nObjectID;

				XPArrayList alApprovedClassIDs = new XPArrayList();
				XPArrayList alApprovedObjectIDs = new XPArrayList();

				for (int nObjectIndex = 0; nObjectIndex < nObjects[0].length; nObjectIndex++)
				{
					String sCurrentError;
					String sObjectError = "";
					
					nClassID = XPConvert.ToInteger(nObjects[0][nObjectIndex]);
					nObjectID = XPConvert.ToInteger(nObjects[1][nObjectIndex]);

					for (int x = 0; x < m_oObjectActions.length; x++)
					{
						oObjectAction = m_oObjectActions[x];

						
							log.Debug("ObjMgrModel.MoveObjectsBetweenFolders - About to call OnBeforeMoveObject on an: " + oObjectAction);
						
	
						if(oObjectAction != null)
						{
							iAction = (IObjectActions) oObjectAction;
							
							sCurrentError = iAction.OnBeforeMoveObject(m_asOwner, 
								(IPTSession) m_asOwner.GetUserSession(), nClassID, nObjectID, nFolderID);
							
							if ((null != sCurrentError) && (sCurrentError.length() > 0))
							{
								sObjectError += sCurrentError + " ";
							}
						}
					}

					if ((null != sObjectError) && (sObjectError.length() > 0))
					{
						sErrorMsgs += sObjectError + " ";
					} else
					{
						// Add this object to the list of objects which will get moved.
						alApprovedClassIDs.Add(nObjects[0][nObjectIndex]);
						alApprovedObjectIDs.Add(nObjects[1][nObjectIndex]);
					}
				}

				// Create new array of objects that were not failed by the PEI.
				// JF- Number of objects to move
				int nObjsToMoveCount = alApprovedClassIDs.GetSize();
				
				if (0 != nObjsToMoveCount)
				{
					// JF- Copy the XPArrayLists to an object array[][]
					arrObjectsToMove = new Object[2][nObjsToMoveCount];
				
					for (int k = 0; k < nObjsToMoveCount; k ++)
					{
						arrObjectsToMove[0][k] = alApprovedClassIDs.GetElement(k);
						arrObjectsToMove[1][k] = alApprovedObjectIDs.GetElement(k);
					}
				}
			}

			// move the objects
			Object[] moveResults = m_ptCatalog.MoveObjectsBetweenAdminFolders(arrObjectsToMove,
				m_nCurrentFolderID, nFolderID);

			// Each object has it's own return code.
			for (int i = 0; i < moveResults.length; i++)
			{
				if (XPConvert.ToInteger(moveResults[i]) != PT_RESULTCODES.PT_S_OK)
				{
					sErrorMsgs += m_asOwner.GetString(1259, "ptmsgs_portaladminmsgs") + " ";
				}
			}

			if ((null != sErrorMsgs) && (sErrorMsgs.length() > 0))
			{
				RaiseError(sErrorMsgs);
			}

			//PEI : ObjectActions : OnMoveObject END
		}
		catch (Exception e)
		{
			
				log.Error(e, m_asOwner.GetString(1259, "ptmsgs_portaladminmsgs"));
			

			sErrorMsgs += m_asOwner.GetString(1259, "ptmsgs_portaladminmsgs");

			RaiseError(sErrorMsgs);
		}
	}

	/////////////////////////////////////////
	// Write methods from IFolderModel
	/////////////////////////////////////////

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModel#OpenSubFolder(int)
	 *
	 * This is just here to satisfy the interface.  It should not get used because
	 * the obj mgr space does not use the opensubfoldercontrol.
	 */
	public void OpenSubFolder(int nID)
	{
	}

	/**
	 *
	 *
	 * @param strError
	 */
	public void RaiseError(String strError)
	{
		m_aoHelper.RaiseError(strError);
	}

	/////////////////////////////////
	// Write methods for IRepostModel
	/////////////////////////////////

	/**
	 * @see com.plumtree.uiinfrastructure.form.IRepostModel#SavePage(String, XPHashtable)
	 *
	 * @return int Page status for server side validation
	 */
	public int SavePage(String _sPageName, XPHashtable _htFormData)
	{
		return RepostControl.PAGE_STATUS_VALID;
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModel#SetFirstItemIndex(int)
	 */
	public void SetFirstItemIndex(int nFirstItemIndex)
	{
		m_nFirstItemIndex = nFirstItemIndex;
	}

	/**
	 * This method is used to set whether or not folders will be deleted.
	 *
	 * @param boolean true implies folders will be deleted.
	 */
	public void SetFoldersForDeleteCheck(boolean bFolders)
	{
		m_aoHelper.SetFoldersForDeleteCheck(bFolders);
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModel#SetLastItemIndex(int)
	 */
	public void SetLastItemIndex(int nLastItemIndex)
	{
		m_nLastItemIndex = nLastItemIndex;
	}

	/////////////////////////////////
	// Write methods from ObjMgrModel
	/////////////////////////////////

	/**
	 * This method sets the basic settings on the object manager model.
	 *
	 * @param int the current folder ID
	 * @param int[] the list of expanded object types
	 */
	public void SetObjMgrSettings(int nFolderID, int[] nExpandedObjectTypes)
	{
		if (nFolderID == PT_INTRINSICS.PT_ADMIN_FOLDER_PROFILELAYOUT)
		{
			// Do not allow users to browse to this folder.
			// Ignore this.

			String strError = m_asOwner.GetString(3172, ResourceConstants.PORTALADMINMSGS);
			log.Info(strError);

			RaiseError(strError);

			// Setting the new folder ID equal to the current ID
			// will cause the folder ID to be ignored.
			nFolderID = m_nCurrentFolderID;
		}

		// Used to auto-open the object if there's only one type in the folder.
		boolean bAutoExpand = false;
		int nSingleClassID = -1;

		if (m_nCurrentFolderID != nFolderID)
		{
			IPTAdminFolder temp = null;

			try
			{
				temp = m_ptCatalog.OpenAdminFolder(nFolderID, false);
			}
			catch (Exception e)
			{
				// Security	exception if you try to open folder ID 0
				log.Info(e, "Error Opening SubFolder " + nFolderID);

				RaiseError(XPResourceManager.FormatMessage(
					m_asOwner.GetString(3173, ResourceConstants.PORTALADMINMSGS),
					XPConvert.ToString(nFolderID)));

				temp = null;
			}

			if (temp != null)
			{
				m_ptCurrentFolder		 = temp;
				m_nPreviousFolderID		 = m_nCurrentFolderID;
				m_nCurrentFolderID		 = nFolderID;

				log.Info("Opened SubFolder " + nFolderID + " called " +
					m_ptCurrentFolder.GetLocalizedName());

				// Check if there is only one object type in this new folder, and if
				// it is, auto-expand that type so it can be seen.

				// Initialize the basic object query - this does not check security,
				// but should be sufficient for our purposes.
				InitializeObjectCount();

				// We don't want to pre-expand anything if there are too many items.
				// Some installs have 1000's or more of users in a single folder.  We
				// should let the user decide if they want to expand that, because the
				// DB query could take a while.
				if (m_asqrObjectCount.GetCount() < MAX_PRE_EXPANDED_OBJECTS)
				{
					// See if there is more than one objec type in this folder.
					for (int i = 0; i < m_asqrObjectCount.GetCount(); i++)
					{
						int nClassID = XPConvert.ToInteger(m_asqrObjectCount.GetFields(i, PT_PROPIDS.PT_PROPID_CLASSID));
						if (nSingleClassID == -1)
						{
							nSingleClassID = nClassID;
							bAutoExpand = true;
						}
						if (nClassID != nSingleClassID)
						{
							bAutoExpand = false;
							break;
						}
					}
				}
					
				// Set the help topic
				switch (m_nCurrentFolderID) {
					case PT_INTRINSICS.PT_ADMIN_FOLDER_DEFAULTPROFILES:
						m_asOwner.SetHelpTopic("PT_EditDefaultProfile");

						break;

					default:
						m_asOwner.SetHelpTopic("PT_Administration");

						break;
				}
			}
			else 
				log.Info("Could not open SubFolder " + nFolderID);
			

			// Clear selected objects in ObjMgr map
			IXPEnumerator objMgrs = m_aoHelper.GetObjMgrs().GetElements();

			while (objMgrs.MoveNext())
			{
				((ObjectManagerHolder) objMgrs.GetCurrent()).Reset();
			}

			if (m_bExpandAllObjectTypes)
			{
				ExpandAllClasses();
			}
		}

		if (((!m_bExpandAllObjectTypes) ||
			(nExpandedObjectTypes.length != m_nExpandedObjectTypes.length)) &&
			(nExpandedObjectTypes != null) &&
			(nExpandedObjectTypes.length > 0))
		{
			m_bExpandAllObjectTypes = false;
			m_nExpandedObjectTypes = nExpandedObjectTypes;
			m_strExpandedObjectTypes = GeneralHelpers.ConvertToString(m_nExpandedObjectTypes);
		}

		if ((bAutoExpand) && !m_bExpandAllObjectTypes)
		{
			// Expand all classes if we just switched to a new folder and
			// there's only one object type in there.

			// NOTE: If we don't clear this, then we will keep expanding more and more
			// objects as we browse the admin UI.  This is not necessarily bad, but
			// we didn't pre-expand everything in the first place because we were worried
			// about too many DB queries slowing down performance.

			// Clear all previously expanded classes
			m_nExpandedObjectTypes = new int[0];
			m_strExpandedObjectTypes= "";

			ExpandObjectType(nSingleClassID, true);
		}
	}

	/**
	 * This method is used to expand and collapse all objects in an
	 * admin folder.
	 *
	 * @param boolean true implies expand all objects
	 */
	public void ExpandAllObjects(boolean bExpand)
	{
		m_bExpandAllObjectTypes = bExpand;

		if (bExpand)
		{
			// Expand all classes
			ExpandAllClasses();
		} else
		{
			// Clear all expanded classes
			m_nExpandedObjectTypes = new int[0];
			m_strExpandedObjectTypes= "";
		}
	}

	/**
	 * This method is used to set the list of folder IDs (temporary)
	 * to be checked for delete.
	 *
	 * @param int[] of object class id / object id pairs.
	 */
	public void SetObjectsForDeleteCheck(int[] nObjectIDPairs)
	{
		m_aoHelper.SetObjectsForDeleteCheck(nObjectIDPairs);
	}

	/////////////////////////////////////
	// Write methods for IPaginationModel
	/////////////////////////////////////

	/**
	 * @see com.plumtree.uiinfrastructure.form.pagination#SetPagination(int, int, int)
	 */
	public void SetPagination(int nPaginationID, int nPaginationLowerBound,
		int nPaginationUpperBound)
	{
		ObjectManagerHolder temp = GetObjMgrHolder(nPaginationID);

		if (temp != null)
		{
			// Check that we are not trying to display more objects than we can query for
			if ((nPaginationUpperBound - nPaginationLowerBound) >=
				APTQueryWrapper.PTTQRW_DEFAULT_MAX_ROWS)
			{
				nPaginationUpperBound = nPaginationLowerBound + APTQueryWrapper.PTTQRW_DEFAULT_MAX_ROWS - 1;
				RaiseError(m_asOwner.GetString(2536, "ptmsgs_portaladminmsgs"));
			}
			temp.SetPagination(nPaginationLowerBound, nPaginationUpperBound);
		}
		else
		{
			
				log.Error("Unable to find object manager for pagination: " +
					nPaginationID);
			
		}
		
	}

	/**
	 * This method sets the selected items in a Class ID
	 *
	 * @param int nClassID the class ID of the selected items
	 * @param int[] nSelectedObjectIDs the selected object IDs
	 */
	public void SetSelectedObjects(int nClassID, int[] nSelectedObjectIDs)
	{
		ObjectManagerHolder objMgrHolder = GetObjMgrHolder(nClassID);

		if (objMgrHolder != null)
		{
			objMgrHolder.m_nSelectedObjectIDs = nSelectedObjectIDs;
		}
	}

	/**
	 * This method clears all of the selected items.  This should be
	 * called after a command has been completed (move, copy, etc...).
	 */
	public void ClearAllSelectedObjects()
	{
		// Clear selected objects in ObjMgr map
		IXPEnumerator objMgrs = m_aoHelper.GetObjMgrs().GetElements();

		while (objMgrs.MoveNext())
		{
			((ObjectManagerHolder) objMgrs.GetCurrent()).m_nSelectedObjectIDs = null;
		}
	}

	///////////////////////////////
	// Write methods for ISortModel
	///////////////////////////////

	/**
	 * @see com.plumtree.portalpages.common.objects.ISortModel#SetSort(int, int, int)
	 */
	public void SetSort(int nSortID, int nSortHeader, int nSortDirection)
	{
		int nPropID = PT_PROPIDS.PT_PROPID_NAME;

		switch (nSortHeader)
		{
		case HEADER_NAME_INDEX:
			nPropID = PT_PROPIDS.PT_PROPID_NAME;

			break;

		case HEADER_MODIFIED_INDEX:
			nPropID = PT_PROPIDS.PT_PROPID_LASTMODIFIED;

			break;

		case HEADER_CREATED_INDEX:
			nPropID = PT_PROPIDS.PT_PROPID_CREATED;

			break;

		default:

			
				log.Error("Sorting on unknown header " + nSortHeader);
			

			break;
		}

		ObjectManagerHolder temp = GetObjMgrHolder(nSortID);

		if (temp != null)
		{
			temp.SetSortOrder(nPropID, nSortDirection);
		}
		else
		{
			
				log.Error("Unable to find object manager to sort on: " + nSortID);
			
		}
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#ShowObjectBanner()
	 */
	public boolean ShowObjectBanner()
	{
		return true;
	}

	/////////////////
	// Helper methods
	/////////////////

	/**
	 * TODO: Move this to a central location
	 * 
	 * This method is used to get a localized version of a server exception.
	 * It only works for PTExceptions, but it will return null for all other
	 * Exceptions.
	 *
	 * @param Throwable e
	 * @return String the localized String, null if couldn't get the message
	 */
	public static String GetLocalizedErrorString(Throwable e, AActivitySpace m_asOwner)
	{
		String result = null;

		if( e instanceof com.plumtree.server.marshalers.PTException )
		{
			PTException ptE = (PTException) e;

			int nErrorCode = ptE.GetErrorCode();

			result = GetLocalizedErrorString(nErrorCode, m_asOwner);

			if (result == null)
			{
				
					log.Error(e, "Unable to find localized string for error code " + nErrorCode + ". ");
				
			}
		}

		return result;
	}

	/**
	 * TODO: Move this to a central location
	 * 
	 * This method is used to get a localized version of a server error
	 * code.
	 *
	 * This works for error codes from PTExceptions as well as error codes
	 * from result values.
	 *
	 * @param int nErrorCode the error code to translate
	 * @return String the localized String, null if couldn't get the message
	 */
	public static String GetLocalizedErrorString(int nErrorCode, AActivitySpace m_asOwner)
	{
		String result = "";

		switch (nErrorCode) {
			case PT_RESULTCODES.PT_E_ACCESSDENIED:
			case PT_RESULTCODES.PT_E_INSUFFICIENTROLEACCESS:
				result = m_asOwner.GetString(2184, "ptmsgs_portaladminmsgs");
				break;

			case PT_RESULTCODES.PT_E_REMOTESERVER_IN_USE_WEBSRVC:
				result = m_asOwner.GetString(2185, "ptmsgs_portaladminmsgs");
				break;

			case PT_RESULTCODES.PT_E_WEBSERVICE_IN_USE_AUTHSRC:
				result = m_asOwner.GetString(2290, "ptmsgs_portaladminmsgs");
				break;

			case PT_RESULTCODES.PT_E_WEBSERVICE_IN_USE_DATASOURCE:
				result = m_asOwner.GetString(2291, "ptmsgs_portaladminmsgs");
				break;

			case PT_RESULTCODES.PT_E_WEBSERVICE_IN_USE_FEDSRCH:
				result = m_asOwner.GetString(2292, "ptmsgs_portaladminmsgs");
				break;

			case PT_RESULTCODES.PT_E_WEBSERVICE_IN_USE_PORTLET:
				result = m_asOwner.GetString(2293, "ptmsgs_portaladminmsgs");
				break;

			case PT_RESULTCODES.PT_E_WEBSERVICE_IN_USE_PROFILESRC:
				result = m_asOwner.GetString(2294, "ptmsgs_portaladminmsgs");
				break;

			case PT_RESULTCODES.PT_E_OBJECTLOCKED:
				result = m_asOwner.GetString(2186, "ptmsgs_portaladminmsgs");
				break;

			case PT_RESULTCODES.PT_E_CANTDELETEAUTHSOURCE:
				result = m_asOwner.GetString(2288, "ptmsgs_portaladminmsgs");
				break;

			case PT_RESULTCODES.PT_E_CANTDELETEDATASOURCE:
				result = m_asOwner.GetString(2289, "ptmsgs_portaladminmsgs");
				break;

			case PT_RESULTCODES.PT_E_CANTDELETEDATASOURCE_DOCS:
				result = m_asOwner.GetString(2668, "ptmsgs_portaladminmsgs");
				break;
				
			case PT_RESULTCODES.PT_E_PROPERTY_IN_USE_DGROUP:
				result = m_asOwner.GetString(3124, ResourceConstants.PORTALADMINMSGS);
				break;
				
			case PT_RESULTCODES.PT_E_COMM_IN_USE_COMMUNITYTEMPLATE:
				result = m_asOwner.GetString(3151, ResourceConstants.PORTALADMINMSGS);
				break;

			default:
				
					log.Error("Unable to find localized string for error code " + nErrorCode + ". ");
				

				break;
		}

		return result;
	}

	/**
	 *
	 *
	 * @param nFolderID
	 *
	 * @return
	 */
	protected boolean CheckIsCommunityFolder(int nFolderID)
	{
		return m_aoHelper.CheckIsCommunityFolder(nFolderID);
	}

	/**
	 * This method is used to set the expanded object types
	 * index and string to include all currently visible
	 * classes.
	 */
	protected void ExpandAllClasses()
	{
		int nSize = 0;
		int nClassID;

		IXPEnumerator objMgrs = m_aoHelper.GetObjMgrs().GetElements();
		while (objMgrs.MoveNext())
		{
			nClassID = ((ObjectManagerHolder) objMgrs.GetCurrent()).m_ptObjMgr.GetManagedClassID();
			if (!IsSuppressedClass(nClassID))
			{
				nSize++;
			}
		}

		m_nExpandedObjectTypes = new int[nSize];
		if (nSize > 0)
		{
			int nIndex = 0;
			objMgrs = m_aoHelper.GetObjMgrs().GetElements();
			while (objMgrs.MoveNext())
			{
				nClassID = ((ObjectManagerHolder) objMgrs.GetCurrent()).m_ptObjMgr.GetManagedClassID();
				if (!IsSuppressedClass(nClassID))
				{
					if (nIndex < nSize)
					{
						m_nExpandedObjectTypes[nIndex] = nClassID;
						nIndex++;
					}
				}
			}
		}

		m_strExpandedObjectTypes = GeneralHelpers.ConvertToString(m_nExpandedObjectTypes);
	}

	/**
	 * This method allows us to suppress certain object types.
	 *
	 * @param int nObjectClassID
	 * @return true implies the class is suppressed.
	 */
	protected boolean IsSuppressedClass(int nObjectClassID)
	{
		// Skip communities, user interface objects, and taxonomists -- always.
		if ((nObjectClassID == PT_CLASSIDS.PT_COMMUNITY_ID) ||
			(nObjectClassID == PT_CLASSIDS.PT_USERINTERFACE_ID) ||
			(nObjectClassID == PT_CLASSIDS.PT_TAXONOMIST_ID))
		{
			return true;
		}

		// Skip sitemaps when in community folders.
		if ((nObjectClassID == PT_CLASSIDS.PT_SITEMAPFOLDER_ID) &&
				(GetCurrentFolderIsCommunity()))
		{
			return true;
		}

		return false;
	}

	/** This method is used to wrap the current folder and get it if it's not set */
	private IPTAdminFolder GetCurrentFolder()
	{
		if (m_ptCurrentFolder == null)
		{
			try
			{
				m_ptCurrentFolder = m_ptCatalog.OpenAdminFolder(m_nCurrentFolderID, false);
			} catch (Exception e)
			{
				if (m_nCurrentFolderID != PT_INTRINSICS.PT_ADMIN_FOLDER_ROOTFOLDER)
				{
					String strError = XPResourceManager.FormatMessage(m_asOwner.GetString(3216, "ptmsgs_portaladminmsgs"), XPConvert.ToString(m_nCurrentFolderID));

					log.Warn(e, strError);

					RaiseError(strError);

					m_nCurrentFolderID = PT_INTRINSICS.PT_ADMIN_FOLDER_ROOTFOLDER;

					// Try again with the root folder.
					return GetCurrentFolder();
				}

				throw new XPException("Unable to open admin folder with id " + m_nCurrentFolderID +
					".  Ignoring.", e);
			}
		}

		return m_ptCurrentFolder;
	}

	/**
	 * This method is used to get and format an error message where the only arg
	 * in the message is a folder name.
	 *
	 * @param int nErrorID the id in the xml file
	 * @param int nCurrentFolderID the id of the folder to use the name of
	 * @return String the error message
	 */
	private String GetFolderErrorString(int nErrorID, int nCurrentFolderID)
	{
		return m_aoHelper.GetFolderErrorString(nErrorID, nCurrentFolderID);
	}

	/**
	 * This method returns the requested object manager holder, instantiating it if
	 * it was not cached.
	 *
	 * @return ObjectManagerHolder the obj mgr holder -- null on failure
	 */
	private ObjectManagerHolder GetObjMgrHolder(int nClassID)
	{
		return m_aoHelper.GetObjMgrHolder(nClassID);
	}
}

