
package com.plumtree.portalpages.admin.ptutilities.objectlockmanager;


// Framework Imports
import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.resultwrapper.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.server.impl.admin.PTLockManager;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.arraywrapper.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.form.*;

/**
 * This class keeps an up-to-date state of all the settings
 * for the object lock manager.
 *
 * It contains the set and get methods for those settings.
 *
 * It contains the start, save and finish methods.
 *
 * @author John Hogan
 */
public class ObjectLockManagerModel extends EditorModel
	implements IObjectLockManagerModelRO
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
		"com.plumtree.portalpages.admin.ptutilities.objectlockmanager.ObjectLockManagerModel");

	/** The unique identifier for the class */
	public static final String STR_MVC_CLASS_NAME = "ObjectLockManagerModel";

	/** Server object: Plumtree Session */
	private IPTSession m_Session;

	// member variables
	private PTGrowableSortedArrayWrapper m_arrLockedItems;
    private boolean m_sortRequest = false;
    private boolean m_DeleteRequest = false;
	
    private int m_intClassID    = -1;
	private int m_intSortColumn;
	private int m_intSortDirection;
	private XPHashtable m_hUnlockedItems;
    private XPHashtable m_hLockedClassTypes;
    private XPArrayList m_SelectedObjects = null;
	
	

	// END ***************** repost action methods *****************

	public ObjectLockManagerModel() {
		super();
		m_intSortColumn = PT_PROPIDS.PT_PROPID_NAME;
		m_intSortDirection = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;
	}

	/**
	 * Sets the classID.
	 * @param classID The classID to set
	 */
	public void SetClassID(int classID) {
		m_intClassID = classID;
	}

	/**
	 * Returns the classID.
	 * @return int
	 */
	public int GetClassID() {
		return m_intClassID;
	}

	/**
	 * Sets the sortColumn.
	 * @param sortColumn The sortColumn to set
	 */
	public void SetSortColumn(int sortColumn) {
		m_intSortColumn = sortColumn;
	}

	/**
	 * Returns the sortColumn.
	 * @return int
	 */
	public int GetSortColumn() {
		return m_intSortColumn;
	}

	/**
	 * Sets the sortDirection.
	 * @param sortDirection The sortDirection to set
	 */
	public void SetSortDirection(int sortDirection) {
		m_intSortDirection = sortDirection;
	}

	/**
	 * Returns the sortDirection.
	 * @return int
	 */
	public int GetSortDirection() {
		return m_intSortDirection;
	}

	/**
	 * Sets whether we are just sorting the list of locked items, or whether we
	 * should retrieve new data from the database.
	 * @param m_bSortOnly The m_bSortOnly to set
	 */
	public void SetSortOnly(boolean m_bSortOnly) {
		m_sortRequest = m_bSortOnly;
	}

	/**
	 * Returns whether we are just sorting the list of locked items, or whether
	 * we should retrieve new data from the database.
	 * @return boolean
	 */
	public boolean GetSortOnly() {
		return m_sortRequest;
	}
	
	/**
	 * DoTaskOnChangePage()
	 * 
	 * Changes the classID to the ID of the page to which the user is entering.
	 * Next initializes the growable array of locks in the hashtable of locked
	 * objects
	 *
	 * @param sNewSpaceName
	 * @param sNewPageName
	 */
	
	public void DoTaskOnChangePage(String _sNewSpaceName, String _sNewPageName)
	{
		
		SetClassID(((MainDP) m_asOwner.GetPage(_sNewPageName)).GetChildClassID());
//      Set Locks is being called by the view already, so there is no need to call it again!
		SetLocks();
	}

	/**
	 * Changes the sort of the locked items array.
	 *
	 * @param intColumn
	 * @param intDirection
	 */
	public void ChangeSort() {
		//change the sort on the locked array
        m_sortRequest = true; //Set this so that the data is not retrieved again

	}
    
    private void ReverseSort(PTGrowableSortedArrayWrapper itemList) {
        if ((itemList != null) && (itemList.GetCount() > 0)) {
            int[][] arrSort = new int[1][2];
            arrSort[0][0]     = m_intSortColumn;
            arrSort[0][1]     = m_intSortDirection;
            itemList.Sort(arrSort);
        }
        
        if (m_intSortDirection == PTSortedArrayWrapper.PTSAW_SORT_ASCENDING) {
            m_intSortDirection = PTSortedArrayWrapper.PTSAW_SORT_DESCENDING;
        } else {
            m_intSortDirection = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;
        }
    }

	/**
	 * Implements some actions to perform on start.
	 *
	 * @param _htFormData
	 *
	 * @return boolean Is true if the current user has the rights to access this Editor.
	 */
	public boolean CheckEditorAccessAndStart(XPHashtable _htFormData)
	{
		if (!((IPTSession) m_asOwner.GetUserSession()).GetSessionInfo().CheckForActivityRight(
						PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_ACCESS_UTILITIES))
		{
			return false;
		}

		IPTLockManager ptLockMan;
		ptLockMan = (IPTLockManager) m_Session.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_LOCKMANAGER,
				false);

		//make sure we're showing an up-to-date view of the system's locks
		ptLockMan.ClearAllExpiredLocks();
		
		//SS - Initialize hashtables that store all locked items and all items that will be unlocked based
		//on the UI interaction (to be committed in the Finish() method)
		m_hUnlockedItems = new XPHashtable();
        m_SelectedObjects = new XPArrayList();
        m_hLockedClassTypes = new XPHashtable();
		
		//Set the classID to which page the user enters by default
		SetClassID(((MainDP) m_asOwner.GetCurrentPage()).GetChildClassID());
        
        //This is beging called from the view anyways, so don't call it from here!
		//SetLocks();

		return true;
	}
	
	/**
	 * SetLocks()
	 * 
	 * Initializes hashtable of locked items for the current page - called from View to ensure that
	 * the correct class of objects is set.
	 * 
	 */
	
	public void SetLocks() {
		m_arrLockedItems = GetLockedItems(m_intClassID);
	}
	

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create() {
		return new ObjectLockManagerModel();
	}

	/**
	 * Implement some code to execute when the user clicks on "Finish".
	 *
	 */
	public void Finish() {
		
		IPTLockManager ptLockMan;
		ptLockMan = (IPTLockManager) m_Session.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_LOCKMANAGER,
				false);
				
		//SS - Get enumeration of unlocked objects from hashtable that stores the objects that
		//the user has chosen to unlock
		IXPEnumerator eKeys = m_hUnlockedItems.GetKeys();
        int classID = -1;
        int objectID = -1;
		
		//SS - Iterate through the enumeration of objects and unlock each one
		while (eKeys.MoveNext()) {
			classID = XPConvert.ToInteger(eKeys.GetCurrent());
			XPHashtable aUnlockedClassItems = (XPHashtable) m_hUnlockedItems.GetElement(new Integer(classID));
            if ((aUnlockedClassItems != null) && (aUnlockedClassItems.GetSize() > 0)) {
                IXPEnumerator eData = aUnlockedClassItems.GetKeys();
                try {
                    while (eData.MoveNext()) {
                        objectID = XPConvert.ToInteger(eData.GetCurrent());
                        ptLockMan.UnlockObject(classID,objectID);
                    }
                } catch (Exception e) {
                        log.Error("ObjectLockManagerModel::UnlockObject Unable to unlock the object." +
                            " The following exception occured: " + e);
                    throw new XPException(m_asOwner.GetString(1536,
                            "ptmsgs_portaladminmsgs"), e);
                }
            }
		}
	}

	/**
	 * Retrieves the list of locked items from the database, places those items
	 * into a PTSortedArrayWrapper member variable, m_arrLockedItems, then
	 * returns that member variable.  This function is called by the view class,
	 * ObjectLockManagerView each time the page is repainted.  The only reason
	 * to store the list at all is for sorting purposes.  If the user resorts
	 * the list of locks, the sort column is changed, but the data is not
	 * retrieved again.
	 *
	 * @return PTGrowableSortedArrayWrapper
	 */
	public PTGrowableSortedArrayWrapper GetLockedItems(int intClassID)
	{
		PTGrowableSortedArrayWrapper myArray			   = null;
		
        PTLockManager ptLockMan = (PTLockManager) m_Session.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_LOCKMANAGER, false);
		PTObjectManagerQueryWrapper myQueryWrapper = new PTObjectManagerQueryWrapper(m_Session.GetUsers());
		int intLockedCount = 0;
		ASQueryResultWrapper ptqrLockedItems = new ASQueryResultWrapper(ptLockMan.QueryLockedObjects(
					intClassID));
		intLockedCount = ptqrLockedItems.GetCount();
        Object objectID = null;
        XPHashtable currentUnlockedList = null;
        currentUnlockedList = (XPHashtable)(m_hUnlockedItems.GetElement(new Integer(intClassID)));
        
        if (m_DeleteRequest) {
            m_DeleteRequest = false;
            if ((m_SelectedObjects != null) && (m_SelectedObjects.GetSize() > 0)) {
                if (currentUnlockedList == null) {
                    currentUnlockedList = new XPHashtable();
                    m_hUnlockedItems.PutElement(new Integer(intClassID), currentUnlockedList);
                }
                for (int i = 0; i < m_SelectedObjects.GetSize(); i++) {
                    currentUnlockedList.PutElement(m_SelectedObjects.GetElement(i), new Integer(1));
                }
            }
        }

		if (intLockedCount > 0) {
			myArray = new PTGrowableSortedArrayWrapper();

			int[] arrIndexes   = new int[]
				{
					PT_PROPIDS.PT_PROPID_OBJECTID,
					PT_PROPIDS.PT_PROPID_NAME,
					PT_PROPIDS.PT_PROPID_OBJLOCK_FROMMACHINENAME,
					PT_PROPIDS.PT_PROPID_OBJLOCK_ACQUIREDAT,
					PT_PROPIDS.PT_PROPID_OBJLOCK_LOCKINGUSERID
				};
            Object[] arrData = new Object[]{new XPArrayList(),new XPArrayList(), new XPArrayList(), new XPArrayList(), new XPArrayList()};
			int[][] arrSort    = new int[1][2];
			arrSort[0][0]	   = m_intSortColumn;
			arrSort[0][1]	   = m_intSortDirection;

			for (int i = 0; i < intLockedCount; i++) {
                objectID = ptqrLockedItems.GetFields(i, PT_PROPIDS.PT_PROPID_OBJECTID);
                if ((currentUnlockedList != null) && (currentUnlockedList.GetElement(objectID) != null)) {
                    continue;
                }

                ((XPArrayList)arrData[0]).Add(objectID);

                String strName    = "";
				if (PT_CLASSIDS.PT_GLOBALOBJECT_ID == XPConvert.ToInteger(ptqrLockedItems.GetFields(i, PT_PROPIDS.PT_PROPID_CLASSID))) {
					//If it is a global object, get the name from the GetAgentName helper function
					strName = GetAgentName(XPConvert.ToInteger(ptqrLockedItems.GetFields(
								i, PT_PROPIDS.PT_PROPID_OBJECTID)));
				} else {
					strName = (String) ptqrLockedItems.GetFields(i, PT_PROPIDS.PT_PROPID_NAME);
				}

                ((XPArrayList)arrData[1]).Add(strName);
                ((XPArrayList)arrData[2]).Add(ptqrLockedItems.GetFields(i, PT_PROPIDS.PT_PROPID_OBJLOCK_FROMMACHINENAME));

                ((XPArrayList)arrData[3]).Add((XPDateTime) ptqrLockedItems.GetFields(i, PT_PROPIDS.PT_PROPID_OBJLOCK_ACQUIREDAT));

                boolean bSuccess  = myQueryWrapper.QuerySingleObject(XPConvert.ToInteger(ptqrLockedItems.GetFields(
							i, PT_PROPIDS.PT_PROPID_OBJLOCK_LOCKINGUSERID)));

				if (bSuccess) {
                    ((XPArrayList)arrData[4]).Add(myQueryWrapper.GetItem(0,PT_PROPIDS.PT_PROPID_NAME));
				} else {
                    ((XPArrayList)arrData[4]).Add(m_asOwner.GetString(1537, "ptmsgs_portaladminmsgs"));
				}
			}
            Object[][] finalData = new Object[5][0];
            for (int i = 0; i < 5; i++) {
                finalData[i] = ((XPArrayList)arrData[i]).ToArray();
            }
			myArray.Initialize(arrIndexes, finalData, arrSort, true);
		}
        
        if (m_sortRequest) {
            m_sortRequest = false;
            ReverseSort(myArray);
        }
        
        myQueryWrapper = new PTObjectManagerQueryWrapper(m_Session.GetUsers());
        int lockedClassCount = 0;
        ASQueryResultWrapper ptqrLockedClassTypes = new ASQueryResultWrapper(ptLockMan.QueryLockedClassTypes());
        lockedClassCount = ptqrLockedClassTypes.GetCount();
        m_hLockedClassTypes.Clear();

        if (lockedClassCount > 0) {
            for (int i = 0; i < lockedClassCount; i++) {
                m_hLockedClassTypes.PutElement(ptqrLockedClassTypes.GetFields(i, PT_PROPIDS.PT_PROPID_CLASSID), 
                                            ptqrLockedClassTypes.GetFields(i, PT_PROPIDS.PT_PROPID_OBJECTCOUNT));
            }
        }

        return myArray;
	}
    
    public int GetLockedCount(int classID) {
        int count = -1;
        Object obj = null;

        obj = m_hLockedClassTypes.GetElement(new Integer(classID)); 
        if (obj != null) {
            count = XPConvert.ToInteger(obj);
        }
        
        XPHashtable unlockedItems = (XPHashtable) m_hUnlockedItems.GetElement(new Integer(classID));
        if (null != unlockedItems) {
            count -= unlockedItems.GetSize();
        }
        return count;
    }

	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	// END ***************** helper methods *****************

	/**
	 * Inits the Model with the IPTSession.
	 *
	 * @param parent
	 */
	public void Init(AActivitySpace parent)
	{
		super.Init(parent);

		// JF- Get the Session object from the associated AS.
		m_Session = (IPTSession) parent.GetUserSession();
	}

	/**
	 * Calls the right save function according to the page name passed in argument.
	 *
	 * @param _sPageName
	 * @param _htFormData
	 * @return int Page status for server side validation
	 */
	public int SavePage(String _sPageName, XPHashtable _htFormData)
	{
		SaveValues(_htFormData);

		return RepostControl.PAGE_STATUS_VALID;
	}

	/**
	 * Stores the object IDs of the objects selected to be unlocked in the
	 * m_arrItemsToUnlock member variable.
	 *
	 * @param _htFormData
	 */
	public void SaveValues(XPHashtable _htFormData) {
		String[] data;
		data = (String[]) _htFormData.GetElement(ObjectLockManagerView.HTMLCHECKBOX_LOCKED);
        
        m_SelectedObjects.Clear();

		if (null != data) {
			for (int i = 0; i < data.length; i++) {
                m_SelectedObjects.Add(m_arrLockedItems.GetItem(XPConvert.ToInteger(data[i]), PT_PROPIDS.PT_PROPID_OBJECTID));
			}
		}
	}

	/**
	 * 
	 * UnlockObjects()
	 * 
	 * Maintains the state of the hashtables that store lock/unlock object information.  Does not commit to the
	 * database until the Finish().  This method is called through the RepostControl when the user selects
	 * the "Unlock" button in the UI
	 * 
	 */

	public void UnlockObjects() {
        m_DeleteRequest = true;
	}
	
	/**
	 * 
	 * GetLockedItemsFromHashtable()
	 * 
	 * Returns the growable array of locked items for the classID of the particular view.  Used to display
	 * the objects in the UI
	 * 
	 */
	
	public PTGrowableSortedArrayWrapper GetLockedItemsFromHashtable(int _intClassID) {
		return m_arrLockedItems;
	}
    
    // BEGIN ***************** helper methods *****************

    /**
     * Static method to return system agent names based on their objectids
     *
     * @param intObjectID
     * @return strName
     */
    public String GetAgentName(int intObjectID) {
        switch (intObjectID)
        {
        case PT_GLOBALOBJECTS.PT_GLOBAL_CARD_REFRESH_AGENT:

            return m_asOwner.GetString(1539, "ptmsgs_portaladminmsgs");

        case PT_GLOBALOBJECTS.PT_GLOBAL_CATALOG_SEARCH_UPDATE_AGENT:

            return m_asOwner.GetString(1538, "ptmsgs_portaladminmsgs");

        case PT_GLOBALOBJECTS.PT_GLOBAL_SYNC_NTUSERS_AGENT:

            return m_asOwner.GetString(1540, "ptmsgs_portaladminmsgs");

        case PT_GLOBALOBJECTS.PT_GLOBAL_FILE_UPLOAD_AGENT:

            return m_asOwner.GetString(1541, "ptmsgs_portaladminmsgs");

        case PT_GLOBALOBJECTS.PT_GLOBAL_WEEKLY_HOUSEKEEPING_AGENT:

            return m_asOwner.GetString(1542, "ptmsgs_portaladminmsgs");

        case PT_GLOBALOBJECTS.PT_GLOBAL_FEDERATED_SEARCH_AGENT:

            return m_asOwner.GetString(1543, "ptmsgs_portaladminmsgs");

        case PT_GLOBALOBJECTS.PT_GLOBAL_DOCUMENTTYPEMAP:

            return m_asOwner.GetString(1545, "ptmsgs_portaladminmsgs");

        case PT_GLOBALOBJECTS.PT_GLOBAL_PROPERTYMAP:

            return m_asOwner.GetString(1546, "ptmsgs_portaladminmsgs");

        case PT_GLOBALOBJECTS.PT_GLOBAL_ACLSYNCMAP:

            return m_asOwner.GetString(1547, "ptmsgs_portaladminmsgs");

        case PT_GLOBALOBJECTS.PT_GLOBAL_ACTIVITY_SECURITY_MANAGER:

            return m_asOwner.GetString(1548, "ptmsgs_portaladminmsgs");

        case PT_GLOBALOBJECTS.PT_GLOBAL_MIGRATION_MANAGER:

            return m_asOwner.GetString(1549, "ptmsgs_portaladminmsgs");

        case PT_GLOBALOBJECTS.PT_GLOBAL_STRING_LOCALIZATION_MANAGER:

            return m_asOwner.GetString(1550, "ptmsgs_portaladminmsgs");

        case PT_GLOBALOBJECTS.PT_GLOBAL_PROFILE_MANAGER:

            return m_asOwner.GetString(1551, "ptmsgs_portaladminmsgs");

        default:
                log.Error("JobModel::GetAgentName... ObjectID does not translate into a known agent. Returning Unknown");
            

            return m_asOwner.GetString(1544, "ptmsgs_portaladminmsgs");
        }
    }
}

