package com.plumtree.portalpages.rest.utils;

import com.plumtree.openfoundation.util.*;

/**
 * This JSON parser handles simple objects only.
 * It is very limited, and does not handle arrays, or non-string
 * object types.  It also only handles escaped quotes and slashes,
 * and only allows double quotes around values, not single quotes.
 */
public class JSONParser implements IJSONParser
{
	/**
	 * @return an instance of a IJSONParser.
	 */
	public static IJSONParser GetInstance()
	{
		// If necessary, this method could be adapted to include
		// a mock JSON parser.
		return new JSONParser();
	}

	// Constants used in parsing the JSON.

	protected static final char OPEN_CURLY_BRACKET = '{';
	protected static final char CLOSE_CURLY_BRACKET = '}';
	protected static final char QUOTE = '\"';
	protected static final char SEMI_COLON = ':';
	protected static final char COMMA = ',';
	protected static final char ESCAPE = '\\';

	protected String m_strObjectName = null;
	protected XPHashtable m_xpMembers = new XPHashtable();

	/**
	 * Private constructor so it can only be accessed by the GetInstance method.
	 */
	private JSONParser() {}

	/* (non-Javadoc)
	 * @see com.plumtree.portalpages.browsing.directory.rest.utils.IJSONParser#ParseJSONObject(Java.lang.String)
	 */
	public void ParseJSONObject(String _strJSON)
	{
		// Comment filtering is often used to thwart JSON security attacks.
		// The text should be wrapped in /* */.  While we don't need to parse
		// comment filtered JSON, since comment filtering should really only
		// be generated on the server side, it doesn't hurt to handle it.

		// Remove any leading/trailing whitespace to help checking for comment filtering.
		_strJSON = XPStringUtility.Trim(_strJSON);

		if (XPStringUtility.StartsWith(_strJSON, "/*") &&
			XPStringUtility.EndsWith(_strJSON, "*/"))
		{
			// Remove the comment filtering.
			_strJSON = XPStringUtility.Substring(_strJSON, 2, _strJSON.length() - 2);
		}
		// TODO: This should also handle single quotes.
	
		// NOTE: This JSON "parser" only handles single objects with string member pairs.
		// It does not handle arrays or non-string values. 
		// We need to tokenize the whitespace because we need it sometimes.
		XPStringTokenizer tokenizer = new XPStringTokenizer(_strJSON, "{\"\\:,} \t\n\r\f", true);

		if (!tokenizer.HasMoreTokens())
		{
			throw new XPFormatException("JSON text cannot be empty. [" + _strJSON + "]");
		}

		// Check for initial open curly bracket.
		SkipToken(tokenizer, OPEN_CURLY_BRACKET);

		// Check for quote at beginning of object name.
		SkipToken(tokenizer, QUOTE);

		// Get the unquoted object name.  This unescapes it and reads the end quote.
		m_strObjectName = GetEscapedTokenAndEndQuote(tokenizer);

		// Check for semi-colon, and open curly bracket at end of object name.
		SkipToken(tokenizer, SEMI_COLON);
		SkipToken(tokenizer, OPEN_CURLY_BRACKET);

		// Parse the object members
		String strToken;
		do
		{
			// Get the start of the next object member.
			strToken = GetToken(tokenizer);

			if (strToken.charAt(0) != QUOTE)
			{
				throw new XPFormatException("Expecting token [" + QUOTE + "] when parsing object members, but found [" + strToken + "]");
			}

			// Get the unquoted name.  This unescapes it and reads the end quote.
			String strName = GetEscapedTokenAndEndQuote(tokenizer);

			SkipToken(tokenizer, SEMI_COLON);
			// TODO: This should support numbers, true, and false
			SkipToken(tokenizer, QUOTE);
			// Get the unquoted value.  This unescapes it and reads the end quote.
			String strValue = GetEscapedTokenAndEndQuote(tokenizer);

			m_xpMembers.PutElement(strName, strValue);

			strToken = GetToken(tokenizer);

			// If the object member name/value pair ends with a comma, then we need to
			// read the next member.
		} while (strToken.charAt(0) == COMMA);

		// The end of the JSON object member list.
		if (strToken.charAt(0) != CLOSE_CURLY_BRACKET)
		{
			throw new XPFormatException("Expecting token [" + CLOSE_CURLY_BRACKET + "] at end of object members list, but found [" + strToken + "]");
		}

		// The end of the JSON object.
		SkipToken(tokenizer, CLOSE_CURLY_BRACKET);
	}

	/**
	 * @param _tokenizer the JSON tokenizer
	 * @return the next token without whitespace.
	 */
	public String GetToken(XPStringTokenizer _tokenizer)
	{
		String strToken = _tokenizer.GetNextToken();

		// Skip whitespace
		while (IsWhiteSpace(strToken))
		{
			strToken = GetToken(_tokenizer);
		}

		return strToken;
	}

	/**
	 * Skip the next token if it matches the expected token.
	 * 
	 * @param _tokenizer the JSON tokenizer
	 * @param _token the expected token
	 * @throws XPFormatException if the next token doesn't match the expected token.
	 */
	public void SkipToken(XPStringTokenizer _tokenizer, char _token)
	{
		if (!_tokenizer.HasMoreTokens())
		{
			throw new XPFormatException("Expecting token [" + _token + "], but reached end of JSON text.");			
		}

		String strToken = GetToken(_tokenizer);

		if ((strToken.length() != 1) ||
			(strToken.charAt(0) != _token))
		{
			throw new XPFormatException("Expecting token [" + _token + "], but found [" + strToken + "]");			
		}
	}

	/**
	 * Checks if a token is a single piece of whitespace.
	 * 
	 * @param _strToken
	 * @return boolean true means that the token is whitespace.
	 */
	public boolean IsWhiteSpace(String _strToken)
	{
		if (_strToken.length() == 1)
		{
			char cToken = _strToken.charAt(0);
			switch (cToken) {
			case ' ':
			case '\t':
			case '\n':
			case '\r':
			case '\f':
				return true;

			default:
				break;
			}
		}

		return false;
	}

	/**
	 * This gets an escaped quoted sequence, starting immediately after
	 * the opening quote.  It reads the end quote, but does not return it.
	 * @param _tokenizer
	 * @return the escaped string token.  The closing quote has been read.
	 * @throws XPFormatException
	 */
	public String GetEscapedTokenAndEndQuote(XPStringTokenizer _tokenizer)
	{
		// Get the next token, even if it's whitespace
		String _strToken = _tokenizer.GetNextToken();

		XPStringBuilder sbEscapedValue = new XPStringBuilder(); 

		while ((_strToken.charAt(0) != QUOTE))
		{
			// Check for parsed escape sequence
			if ((_strToken.length() == 1) && (_strToken.charAt(0) == ESCAPE))
			{
				_strToken = GetToken(_tokenizer);
				if (_strToken.length() != 1)
				{
					throw new XPFormatException("Read escape char [\\], expecting either [\\] or [\"] token next, but found [" + _strToken + "].");			
				}

				// Put in the unescaped value
				char cEscaped = _strToken.charAt(0);
				switch (cEscaped) {
				case QUOTE:
					sbEscapedValue.Append('"');
					break;

				case ESCAPE:
					sbEscapedValue.Append('\\');
					break;

				default:
					throw new XPFormatException("Read escape char [\\], expecting either [\\] or [\"] token next, but found [" + cEscaped + "].");			
				}
			} else
			{
				sbEscapedValue.Append(_strToken);
			}

			// Get the next token, even if it's whitespace
			_strToken = _tokenizer.GetNextToken();
		}

		return sbEscapedValue.ToString();
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portalpages.browsing.directory.rest.utils.IJSONParser#GetObjectName()
	 */
	public String GetObjectName() {
		return m_strObjectName;
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portalpages.browsing.directory.rest.utils.IJSONParser#GetMember(Java.lang.String)
	 */
	public String GetMember(String _strName) {
		return (String) m_xpMembers.GetElement(_strName);
	}
}