package com.plumtree.taskapi.portalui;

import com.plumtree.openfoundation.reflect.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.ptlink.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;

/**
 * This interface describes the various helper methods for Communities
 * in the Task API.
 */
public interface ITaskAPIUICommunity
{
    /**
     * Gets the name of the specified Community.
     * 
     * @param owner		Current Activityspace.
     * @param nCommunityID	Object ID of the Community to retrieve the name for.
     * @return	Name of the specified Community.
     * 
     *  @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
     */
    public String GetCommunityName(AActivitySpace owner, int nCommunityID);

    /**
     * Gets the Subcommunities for the specified Community as an integer array.
     * Subcommunites are Communities directly below a Community in the administrative folder hierarchy.
     * 
     * @param owner		Current Activityspace.
     * @param nCommunityID	Object ID for the Community to retrieve Subcommunities from.
     * @return	Array of integers with Object IDs of the Subcommunities in the specified Community. 
     * 
     * @throws XPException									If the Community could not be accessed, for example if the Object ID
     * is not valid.
	 * @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
     */
    public int[] GetSubCommununityIDs(AActivitySpace owner, int nCommunityID);

    /**
     * Gets the Related Communities for the specified Community as an integer array.
     * Related Communities are Communities in the same administrative folder.
     *
     * @param owner		Current Activityspace.
     * @param nCommunityID	Object ID for the Community to retrieve Related Communities from.
     * @return	Array of integers with Object IDs of the Related Communities  in the specified Community. 
     * 
     * @throws XPException									If the Community could not be accessed, for example if the Object ID
     * is not valid.
	 * @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
     */
    public int[] GetRelatedCommununityIDs(AActivitySpace owner, int nCommunityID);

    /**
     * 
     * Get Community Pages in the specified Community as a list of <code>IPTURL</code>s.
     * The <code>PTURLConstants.KEY_CURRENTMARKER</code> flag is set on an URL entry if is leading to the current
     * Community Page. 
     * 
     * @param owner		Current Activityspace.
     * @param nCommunityID	Object Id of the Communitu to list pages of.
     * @return A list with Community Pages as <code>IPTURL</code> objects.
     * 
     *  @throws XPIllegalArgumentException			If the user does not have sufficient access to the Community.
     */
    public XPArrayList GetCommunityPagesAsURLs(AActivitySpace owner, int nCommunityID);

	/**
	 * Gets to URL to the Community Knowledge Directory f0r the current Community.
	 * The <code>PTURLConstants.KEY_CURRENTMARKER</code> flag is set on the URL if the current page is a Community Knowledge
	 * Directory page.
	 *  
	 * @param owner		Current Activityspace
	 * @param nCommunityID	Object ID of the Community to get pages from.
	 * @return <code>IPTURL</code> with the URL to the Community Knowledge Directory for the specified Community.
	 *  @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
	 *  
	 * @see PTURLConstants
	 */
    public IPTURL GetURLForCommunityKnowledgeDirectory(AActivitySpace owner, int nCommunityID);

    /**
     * Gets URLs to Related Communities for the specified Community in a list of <code>IPTURL</code>s.
     * Related Communities are Communities in the same administrative folder.
     * 
     * @param owner		Current Activityspace.
     * @param nCommunityID	Object ID for the Community to retrieve Related Communities from.
     * @return <code>XPArrayList</code> with <code>IPTURL</code>s to Related Communities or an empty <code>XPArrayList</code> if none exists.
     *  @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
     */
    public XPArrayList GetRelatedCommununitiesAsURLs(AActivitySpace owner, int nCommunityID);
    
     /**
     * Gets the Community Name of the current Community.
     * 
     * @param owner		Current Activityspace.
     * @return	Community Name of the current community or <code>null</code> value if not currently on a Community Page. 
     * 
     * @throws XPIllegalArgumentException		If the provided <code>AActivityspace</code> is <code>null</code>.
     */
    public String GetCurrentCommunityName(AActivitySpace owner) throws XPIllegalArgumentException;

	/**
	 * Gets the Community Page Name of the current Community Page. If current
	 * page is not a Community related page, i.e., Community portlet, Sitemap
	 * or Community Gatewaypage page, a null value is returned.
	 * 
	 * @param owner		Current Activityspace.
	 * @return	Community Page Name of the current community pageor null value if not currently on a Community Page. 
	 * 
	 * @throws XPIllegalArgumentException		If the provided <code>AActivityspace</code> is <code>null</code>
	 */
	public String GetCurrentCommunityPageName(AActivitySpace owner) throws XPIllegalArgumentException;

    /**
     * Gets the Community ID of the current Community. If current page is not a Community page,
     * i.e., Community portlet, Sitemap or Community Gatewaypage page, <code>TaskAPIUICommon.INVALID_OBJECTID</code> is returned.
     * 
     * @param owner		Current Activityspace
     * @return	Community ID of the current community or <code>TaskAPIUICommon.INVALID_OBJECTID</code> if not in a Community.
     * 
     * @throws XPIllegalArgumentException		If the provided <code>AActivityspace</code> is <code>null</code>
     */
    public int GetCurrentCommunityID(AActivitySpace owner);

    /**
     * Gets the current Community Page ID, If current page is not a Community page,
     * the <code>TaskAPIUICommon.INVALID_OBJECTID</code> value is returned.
     * 
     * @param owner Current Activityspace.
     * @return	Current Community Page ID or <code>TaskAPIUICommon.INVALID_OBJECTID</code> if not in on Community Page.
     * 
     * @throws XPIllegalArgumentException		If the provided <code>AActivityspace</code> is <code>null</code>.
     */
    public int GetCurrentCommunityPageID(AActivitySpace owner) throws XPIllegalArgumentException;

    /**
     * Gets the current Page as an object of type<code>IPTURL</code>. Returns  
     * 
     * @param owner Current Activityspace
     * @return IPTURL to the current community page or null if the current page is not a community page. 
     * @throws XPIllegalArgumentException
     */
    public IPTURL GetCurrentPageAsURL(AActivitySpace owner) throws XPIllegalArgumentException;

    /**
     * Gets Subcommunities in the current Community as an <code>XPArrayList</code> with <code>IPTURL</code> entries.
     * 
     * @param owner	Current Activityspace.
     * @return Array list with <code>IPTURL</code> of Subcommunities.  An empty list is returned if there are 
     * no subcommunities exist or not in a Community.
     * 
     *  @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
     */
     public XPArrayList GetCurrentSubCommunitiesAsURLs(AActivitySpace owner);
    
	/**
     * Gets the URL to the Unsubscribe from Communities Editor.
     * 
     * @param owner		Current Activityspace.
     * @return		<code>IPTURL</code> with URL to the Unsubscribe from Communities Editor, or <code>null</code> if the URL is not
     * valid in the current context.
     * 
     *  @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
     */
	public IPTURL GetURLForUnsubscribeCommunitiesEditor(AActivitySpace owner);

	/**
     * Gets the URL to the Community Editor for the current Community.
     * 
     * @param owner		Current Activityspace.
     * @return		<code>IPTURL</code> with URL to the Editor for the current Community, or <code>null</code> if the URL is not
     * valid in the current context.
     * 
     *  @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
     */
	public IPTURL GetURLForCurrentCommunityEditor(AActivitySpace owner);

   /**
    * Gets an XPArrayList with a single entry for the URL for the current Community.
    * 
    * @param owner		Current Activityspace.
    * @return		<code>XPArrayList</code> list containing a single URL to the current Community, 
    * or <code>null</code> if the URL is not valid in the current context.
    * 
    *  @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
    */
	public XPArrayList GetURLForCurrentCommunity(AActivitySpace owner);

	/**
	 * Gets the URL to the Community Add Portlets Editor.
	 * 
	 * @param owner		Current Activityspace.
	 * @return		<code>IPTURL</code> with URL to the Community Add Portlets Editor, or <code>null</code> if the URL is not
     * valid in the current context.
     * 
     *  @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
	 */
	public IPTURL GetURLForAddCommunityPortletsEditor(AActivitySpace owner);

	/**
	 * Gets the URL to the Community Add Portlets Flyout.
	 * 
	 * @param owner		Current Activityspace.
	 * @return		<code>IPTURL</code> with URL to the Community Add Portlets Flyout, or <code>null</code> if the URL is not
     * valid in the current context.
     * 
     *  @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
	 */
	public IPTURL GetURLForAddCommunityPortletsFlyout(AActivitySpace owner);

	/**
	 * Gets the URL to the Create Community page in the Community Editor.
	 * 
	 * @param owner		Current Activityspace.
	 * @return		<code>IPTURL</code> with URL to Create Community Page in the Community Editor, 
	 * or <code>null</code> if the URL is not valid in the current context.
	 * 
	 *  @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
	 */
	public IPTURL GetURLForCreateCommunityPage(AActivitySpace owner);
	
	/**
	 * Gets the URL to the Create Community page in the Community Editor.
	 * 
	 * @param owner		Current Activityspace.
	 * @return		<code>IPTURL</code> with URL to Create Community Page in the Community Editor, 
	 * or <code>null</code> if the URL is not valid in the current context.
	 * 
	 *  @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
	 */
	
	public IPTURL GetURLForCreateCommunityPageOneClick(AActivitySpace owner, boolean bInheritTemplate, int nPageTemplate);

	/**
     * Gets the URL for the Join Community Editor where users add Communities to their My Communities list.
     * 
     * @param owner		Current Activityspace.
     * @return		<code>IPTURL</code> with URL to the Join Communities Editor, or <code>null</code> if the URL is not
     * valid in the current context.
     * 
     *  @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
     */
	public IPTURL GetURLForJoinCommunitiesEditor(AActivitySpace owner);

	/**
	 * Gets the URL to the Community Edit Portlet Preferences Editor.
	 * 
	 * @param owner		Current Activityspace.
	 * @return		<code>IPTURL</code> with URL to the Community Edit Portlet Preferences Editor, or <code>null</code> if the URL is not
     * valid in the current context.
     * 
     *  @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
	 */
	public IPTURL GetURLForEditPortletPreferencesEditor(AActivitySpace owner);

    /**
     * Checks if the current user is already subscribed to the current Community, i.e. if the current Community
     * is already on the user's My Community list.
     * 
     * @param owner		Current activityspace.
     * @return <code>true</code> if current Community is already subscribed by the user, <code>false</code> if not member, or on User profile community
     * or  not or if not currently in a Community.
     * 
     *  @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
     */
    public boolean IsUserMemberOfCurrentCommunity(AActivitySpace owner);

    /**
     * Checks if the current page is a Community Page.
     * 
     * @param owner		Current Activityspace.
     * @return	<code>true</code> if the current page is a Community Page, <code>false</code> otherwise.
     * 
     *  @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
     */
    public boolean IsCurrentPageCommunityPage(AActivitySpace owner);

    /**
     * Checks if the current Community is the User Profile Community.
     * 
     * @param owner		Current Activityspace.
     * @return	<code>true</code> if the current Community is the User Profile Community, <code>false</code> otherwise.
     * 
     *  @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
     */
    public boolean IsCurrentCommunityUserProfileCommunity(AActivitySpace owner);

    /**
	 * Checks if the current user has access (requires EDIT access) to use the Community Editor on the current
	 * Community Page.
	 * 
	 * @param owner		Current Activityspace.
	 * @return	<code>true</code> if the current user has at least EDIT access to the current Community Page, <code>false</code> otherwise.
	 * 
	 *  @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
	 */
    public boolean IsCurrentUserAbleToEditCurrentCommunityPage(AActivitySpace owner);

    /**
     * Checks if the current user is the owner of the current Community.
     * The check only works on Community pages or Community Knowledge directory pages, not in Community editors.
     * 
     * @param owner		Current Activityspace.
     * @return	<code>true</code> if the current user has at least EDIT access to the current Community, <code>false</code> otherwise.
     * 
     *  @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
     */
    public boolean IsCurrentUserCommunityOwner(AActivitySpace owner);

    /**
     * Checks if the current user has at least SELECT access to the current Community.
     * Only works on Community pages or Community Knowledge directory pages, not in Community Editors.
     * 
     * @param owner		Current Activityspace.
     * @return	<code>true</code> if the current user has at least SELECT access to the current Community, <code>false</code> otherwise.
     * 
     *  @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
     */
    public boolean IsUserAbleToJoinCurrentCommunity(AActivitySpace owner);

	/**
	 * Checks if the current user has at least READ access to the current Community.
	 * Only works on Community pages or Community Knowledge directory pages, not in Community editors.
	 * 
	 * @param owner		Current Activityspace.
	 * @return	<code>true</code> if the current user has at least READ access to the current Community, <code>false</code> otherwise.
	 * 
	 *  @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
	 */
	public boolean IsUserAbleToAccessCurrentCommunity(AActivitySpace owner);

    /**
     * Checks if the current page is a Myportal page, i.e. either a MyPage or Community Page.
     * 
     * @param owner		Current Activityspace.
     * @return	<code>true</code> if the current page is either a MyPage or Community Page, <code>false</code> otherwise.
     * 
     *  @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
     */
    public boolean IsCurrentPageMyPortalPage(AActivitySpace owner);

    /**
     * Gets Community Actions as an XPArrayList with IPTURL entries.
     * Most Community actions only display when in an Community and only if a user has access 
     * to perform the actions.  
     * 
     * @param owner		Current Activityspace.
     * @return	An <code>XPArrayList</code> with <code>IPTURL</code> entries with Community Actions. 
     * 
     *  @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
     */
    public XPArrayList GetCommunityActionsAsURLs(AActivitySpace owner);

    /**
     * Gets Community Actions as an XPArrayList with IPTURL entries.
     * Most Community actions only display when in an Community and only if a user has access 
     * to perform the actions.  
     * 
     * @param owner		Current Activityspace.
     * @param includeOneClick		Flag to include the "one-click page creation" link
     * @return	An <code>XPArrayList</code> with <code>IPTURL</code> entries with Community Actions. 
     * 
     *  @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
     */
    public XPArrayList GetCommunityActionsAsURLs(AActivitySpace owner, boolean bIncludeOneClick, boolean bInheritTemplate, int nPageTemplate);

    /**
     * Helper function to create an URL to a specified Community. 
     * The <code>PTURLConstants.KEY_CURRENTMARKER</code> flag is set on the URL if is leading to the current
     * Community.
     * 
     * @param owner		Current Activityspace.
     * @param nCommunityID		Object ID of the Community to create an URL to.
     * @param strCommunityName	Name of the URL, normally the name of the Community.
     * @return	<code>IPTURL</code> with an URL to the specified Community.
     * 
     *  @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
     */
    public IPTURL CreateCommunityCommonOpenerURL(AActivitySpace owner, int nCommunityID, String strCommunityName);

    /**
     * Helper function to create an URL to a specified Community Page. 
     * The <code>PTURLConstants.KEY_CURRENTMARKER</code> flag is set on the URL if is leading to the current
     * Community Page. In case the page is a profile page, the link generated will be of the viewed profile,
     * not the current user's.
     * 
     * @param owner		Current Activityspace.
     * @param nCommunityID		Object ID of the Community where the Page is located in.
     * @param nCommunityPageID	Object ID of the Page to create a URL to.
     * @param strCommunityPageName	Title of the URL, normally the name of the Page.
     * @return <code>IPTURL</code> with an URL to the specified Community Page.
     * 
     *  @throws XPIllegalArgumentException			If the user does not have sufficient access to the Community.
     */
    public IPTURL CreateCommunityPageCommonOpenerURL(AActivitySpace owner, int nCommunityID, int nCommunityPageID, String strCommunityPageName);

    /**
     *Gets an URL to a UserProfile page for the specified user.
     * 
     * @param owner		Current Activityspace.
     * @param nProfileUserID	The Object ID of the user to open the User Profile page for.
     * @param sProfileUserName	The name of the user to open the User Profile page for.  Also used for the link title.
     * @return	<code>IPTURL</code> with the URL to the user profile page for the specified user.
     * 
     * @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
     */
   public IPTURL CreateUserProfilePageCommonOpenerURL(AActivitySpace owner, int nProfileUserID, String sProfileUserName);

     /**
      *Gets an URL to a UserProfile page for the specified user.
      * 
      * @param owner		Current Activityspace.
      * @param nProfileUserID	The Object ID of the user to open the User Profile page for.
      * @param sProfileUserName	The name of the user to open the User Profile page for.
      * @param strURLTitle		The title of the URL.
      * @return	<code>IPTURL</code> with the URL to the user profile page for the specified user.
      * 
      * @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
      */
    public IPTURL CreateUserProfilePageCommonOpenerURL(AActivitySpace owner, int nProfileUserID, String sProfileUserName, String strURLTitle);
    
    /**
     *Gets an URL to a UserProfile page for the specified user.
     * 
     * @param owner		Current Activityspace.
     * @param nProfileUserID	The Object ID of the user to open the User Profile page for.
     * @param sProfileUserName	The name of the user to open the User Profile page for.
     * @param strURLTitle		The title of the URL.
     * @return	<code>IPTURL</code> with the URL to the user profile page for the specified user.
     * 
     * @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
     */
   public IPTURL CreateUserProfilePageCommonOpenerURL(AActivitySpace owner, int nProfileUserID, String sProfileUserName,
	   String strURLTitle, int nPageId, String strPageName);

	  /**
	 * Check the if the current user has at least the accesslevel specified to the current
	 * community 
	 * 
	 * @param 							AccessLevel to check for. Example: PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_READ or PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT
	 * @param							nCommunityID Object ID of the Community to check security of
	 * @return boolean			True if the current user has at least the specified accesslevel, false if the user does not have access or there is no current community.
	 * 
	 *  @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
	 * @see PT_ACCESS_LEVELS
	 */
	public boolean CheckUserAccessLevelToCommunity(AActivitySpace owner, int nAccessLevel, int nCommunityID);

     /**
	 * Check the if the current user has at least the accesslevel specified to the current
	 * community 
	 * 
	 * @param 							AccessLevel to check for. Example: PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_READ or PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT
	 * @return boolean			True if the current user has at least the specified accesslevel, false if the user does not have access or there is no current community.
	 * 
	 * @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
	 * @see PT_ACCESS_LEVELS
	 */
	public boolean CheckUserAccessLevelToCurrentCommunity(AActivitySpace owner, int nAccessLevel);
	
	/**
	* Check the if the current user has at least the accesslevel specified to the current
	* community page
	* 
	* @param 							AccessLevel to check for. Example: PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_READ or PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT
	* @return boolean			True if the current user has at least the specified accesslevel, false if the user does not have access or there is no current community page.
	* 
	* @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
	* @see PT_ACCESS_LEVELS
	*/
	public boolean CheckUserAccessLevelToCurrentCommunityPage(AActivitySpace owner, int nAccessLevel);

	/**
	 * Gets the Object ID of the parent Community in the specified Community.  (the Community where the current Community is located in the administrative hierarchy).
	 * 
	 * @param owner		Current Activityspace.
	 * @param nCommunityID	Object ID of the Community to get the parent Community from.
	 * @return Object ID of current parent Community ID, or <code>0 /code>  if non-existant.
	 * 
	 * @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
	 */
	public int GetParentCommunityID(AActivitySpace owner, int nCommunityID);

    /**
     * Gets the Object ID of the parent Community in the current Community. 
     * Parent Community is the Community where the current Community is located in the administrative hierarchy.
     * 
     * @param owner		Current Activityspace.
     * @return Object ID of current parent Community ID, or <code>0</code> if not currently in a Community or
     * the current Community does not have a parent Community.
     * 
     * @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
     */
    public int GetCurrentParentCommunityID(AActivitySpace owner);

    /**
     * Gets Related Communities (a.k.a. peer Communities) of the current Community as an integer array.
     * Related Communities are Communities located in one Community in the administrative hierarchy.  
     * 
     * @param owner		Current Activityspace.
     * @return Array with Object IDs of Related Community to the current Community.  
     * Empty array if the current Community has no Related communities or if not in a Community.
     * 
     * @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
     */
    public int[] GetCurrentRelatedCommununityIDs(AActivitySpace owner);

    /**
     * Gets the URL to Join Current Community Action.
     * The URL goes to a page that subscribes the user to the current Community. This URL is only visible
     * on Communities a user is not already member of.  
     * 
     * @param owner		Current Activityspace.
     * @return <code>IPTURL</code> to join current community or <code>null</code> if the current user is already a member or does not have access to join the current community.
     * 
     * @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
     */
    public IPTURL GetURLForJoinCurrentCommunityAction(AActivitySpace owner);

    /**
     * Gets the URL to Unsubscribe Current Community Action.
     * The URL goes to a page that unsubscribes the user to the current Community. This URL is only visible
     * on Communities a user is not already member of.  
     * 
     * @param owner		Current Activityspace.
     * @return <code>IPTURL</code> to unsubscribe current community or <code>null</code> if the current user is already not a member or does not have access to unsubscribe the current community.
     * 
     * @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
     */
    public IPTURL GetURLForUnsubscribeCurrentCommunityAction(AActivitySpace owner);

    /**
     * Gets the Subcommunities of the current Community as a integer array.
     * Subcommunties are Communities located in another Community in the adminstrative hierarchy.
     * 
     * @param owner		Current Activityspace.
     * @return An integer array with Object IDs to Subcommunity.  
     * Returns an empty array if the current Community has no Subcommunities or not currently in a Community.
     * 
     * @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
     */
    public int[] GetCurrentSubCommunityIDs(AActivitySpace owner);

     /**
     * Gets the Community Pages of the current Community as an <code>XPArraylist</code> with <code>IPTURL</code> entries.
     * 
     * @param owner		Current Activityspace.
     * @return An <code>XPArrayList</code> with <code>IPTURL</code> to CommunityPages, or empty <code>XPArrayList</code> if the current page is not a Community Page or valid Page.
     * 
     * @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
     */
    public XPArrayList GetCurrentCommunityPagesAsURLs(AActivitySpace owner);

    /**
     * Gets URLs to Related Communities for for the current Community in a list of <code>IPTURL</code>s.
     * Related Communities are Communities in the same administrative folder.
     * 
     * @param owner		Current Activityspace.
     * @return <code>XPArrayList</code> with <code>IPTURL</code>s to Related Communities or an empty <code>XPArrayList</code> if none exists.
     * 
     * @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
     */
    public XPArrayList GetCurrentRelatedCommunitiesAsURLs(AActivitySpace owner);

	/**
	 * Gets the URL to the parent Community of the current Community. 
     * Parent Community is the Community where the current Community is located in the administrative hierarchy.
	 * 
	 * @param owner		Current Activityspace.
	 * @return IPTURL to current Parent Community  or <code>null</code> if no parent Community exists, or if not currently
	 * in a Community.
	 * 
	 * @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
	 */
	public IPTURL GetCurrentParentCommunityAsURL(AActivitySpace owner);

	 /**
     * Gets the URL to the parent Community of the specified Community. 
     * Parent Community is the Community where the current Community is located in the administrative hierarchy.
     * 
     * @param owner		Current Activityspace.
     * @param nCommunityID
     * @return  IPTURL to current Parent Community  or <code>null</code> if no parent Community exists, or if the 
     * specified Community is invalid.
     * 
     * @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
     */
    public IPTURL GetParentCommunityAsURL(AActivitySpace owner, int nCommunityID);

    /**
     * Gets the Subcommunities for the specified Community as a list with <code>IPTURL</code> entries.
     * Subcommunities are Community located below another Community in the administrative hierarchy. 
     * 
     * @param owner		Current Activityspace.
     * @param nCommunityID	Object ID for the Community to get Subcommunities from.
     * @return	An <code>XPArrayList</code> with <code>IPTURL</code> entries to Subcommunities, or
     * empty array if the specified Community does no thave any Subcommunities or it is invalid.
     * 
     * @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
     */
    public XPArrayList GetSubCommununitiesAsURLs(AActivitySpace owner, int nCommunityID);

    /**
     * Gets the default Community ID, the first Community in a user's My Community list.
     * 
     * @param owner		Current Activityspace.
     * @return	Object ID of the first Community in the current user's My Community list, or <code>-1</code> if
     * the user is not a member of any communities.
     * 
     * @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
     */
    public int GetDefaultCommunityID(AActivitySpace owner);

    /**
     * Gets a user's My Communities list as an <code>XPArrayList</code> with <code>IPTURL</code> entries.
     * The <code>PTURLConstants.KEY_CURRENTMARKER</code> flag is set on an URL entry if is leading to the current
     * Community. 
     * 
     * @param owner		Current Activityspace.	
     * @param bUseCachedData 	Get the current user's mandatory Communities with Tabs from a cache that contains at 
     * most 15 minutes old data if true. Will query the database for the mandatory communities if false.
     * @return	<code>XPArrayList</code> with <code>IPTURL</code>s to the current user's My Communities ,
     *  or <code>null</code> if Communities features is disabled in the current Experience Definition.
     *  
     *  @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
     */
    public XPArrayList GetMyCommunitiesAsURLs(AActivitySpace owner, boolean bUseCachedData);

    /**
     * Gets a user's Mandatory with Tab Communities as an <code>XPArrayList</code>  with <code>IPTURL</code> entries. 
     * 
     * @param owner 		Current Activityspace.
     * @param bUseCachedData Get the current user's mandatory Communities with Tabs from a cache that contains at most 15 minutes old data if true. Will query the database 
     *  for the mandatory communities if false.
     * 
     * @return <code>XPArrayList</code> with <code>IPTURL</code>s to the current user's mandatory Communities with Tabs,
     *  or <code>null</code> if Communities features is disabled in the current Experience Definition.
     *  
     *  @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
     */
    public XPArrayList GetMyMandatoryCommunitiesWithTabsAsURLs(AActivitySpace owner, boolean bUseCachedData);
	
	/**
	 * Gets the Community ID from an IPTURL with an URL to a Community or Community Page.
	 * 
	 * @param ptURL	<code>IPTURL</code> to a Community or Community Page.
	 * @return The Community ID, or <code>0</code> if none is found in the URL.
	 * 
	 * @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
	 */
	public int GetCommunityIDFromPTURL(IPTURL ptURL);
	
	/**
	 * Gets the Community Page ID from an IPTURL with an URL to a Community Page.
	 * 
	 * @param ptURL		<code>IPTURL</code> to a Community Page.
	 * @return The Community Page ID, or <code>0</code> if none is found in the URL.
	 * 
	 * @throws XPIllegalAccessException			If the user does not have sufficient access to the Community.
	 */
	public int GetCommunityPageIDFromPTURL(IPTURL ptURL);

	/**
	 * Checks whether or not the current user is a member of the specified community.
	 * 
	 * @param _asOwner the current Activity Space
	 * @param _nCommunityID the community to check membership for.
	 * @return true if the current user is a member of the specified community.
	 */
	public boolean IsCommunityMember(AActivitySpace _asOwner, int _nCommunityID);

	/**
	 * This method causes the current user to join the specified community.
	 * 
	 * This method will check community membership before joining the community.
	 * 
	 * @param _asOwner the current Activity Space
	 * @param _nCommunityID the community to join.
	 * @return True means the user has joined the community, false means the user
	 * was already a member.
	 * @throws XPException if there was an error.
	 */
	public boolean JoinCommunity(AActivitySpace _asOwner, int _nCommunityID) throws XPException;

	/**
	 * This method causes the current user to leave the specified community.
	 * 
	 * This method will check community membership before leaving the community.
	 * 
	 * @param _asOwner the current Activity Space
	 * @param _nCommunityID the community to leave.
	 * @return True means the user has left the community, false means the user
	 * was not a member of the community.
	 * @throws XPException if there was an error.
	 */
	public boolean LeaveCommunity(AActivitySpace _asOwner, int _nCommunityID) throws XPException;
}