package com.plumtree.taskapi.portalui;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.admin.editors.page.*;
import com.plumtree.portalpages.browsing.editors.editportletprefs.*;
import com.plumtree.portalpages.browsing.myportal.*;
import com.plumtree.portalpages.browsing.myportal.editpage.*;
import com.plumtree.portalpages.browsing.myportal.mypages.*;
import com.plumtree.portalpages.browsing.objectselection.*;
import com.plumtree.portalpages.browsing.objectselection.flyout.*;
import com.plumtree.portalpages.browsing.objectselection.portlets.*;
import com.plumtree.portalpages.common.editpage.*;
import com.plumtree.portaluiinfrastructure.classtypedescriptors.classtypes.*;
import com.plumtree.portaluiinfrastructure.opener.*;
import com.plumtree.portaluiinfrastructure.ptlink.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.taskapi.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.xpshared.htmlconstructs.*;

/**
 * MyPages related TaskAPI functions.
 * 
 */
public class TaskAPIUIMyPages
{
	private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
		"com.plumtree.taskapi.portalui.TaskAPIUIMyPages");
	
	
	 /**
      * Checks if MyPages feature is enabled in the current Experience Definition.
      * 
      * @param owner		Current Activityspace.
      * @return true if the MyPages feature is enabled in the current Experience Definition, false otherwise.
      * 
      * @throws <code>XPIllegalArgumentException</code> if the provided <code>AActivityspace</code> is <code>null</code> or not valid.
      */
     static boolean IsMyPageFeatureEnabledInSubPortal(AActivitySpace owner)
     {
     	TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
     	IPTSession ptSession = TaskAPIUICommon.GetPTSession(owner);
     	
     	return TaskAPIServerSubPortal.IsSubPortalFeatureEnabled(ptSession, PT_SUBPORTAL_ACCESS_TYPES.PT_SUBPORTAL_MYPAGES_ACCESS);
     }
	
	 /**
     * Gets an <code>IPTURL</code> to the specified MyPage.
     * 
     * @param owner		Current Activityspace.
     * @param nMyPageID Object ID of the MyPage to create an URL to.
     * @param strURLTitle Title of the URL, i.e. what a link using the URL will display. 
     * Most often, the MyPage name should be used.
     * @return <code>IPTURL</code> with an URL to the specified MyPage.
     * 
     * @throws <code>XPIllegalArgumentException</code> if the provided <code>AActivityspace</code> is <code>null</code> or not valid.
     */
    public static IPTURL CreateMyPageCommonOpenerURL(AActivitySpace owner, int nMyPageID, String strURLTitle)
    {
    	TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
    	String strAppBaseURL = TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
    	int nUserID = TaskAPIUICommon.GetPTSession(owner).GetSessionInfo().GetCurrentUserID();
    	String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
		
    	IPTURL url = null;
    	if (TaskAPIUICommon.UseFriendlyURLS(PT_CLASSIDS.PT_MYPAGESCONTAINER_ID)) {
        	url = new CommonOpenerFriendlyURL(strURLTitle, PT_CLASSIDS.PT_MYPAGESCONTAINER_ID, nMyPageID, owner );
    	} else {
	    	url = new CommonOpenerOpenObjIDURL(strURLTitle, strAppBaseURL, PT_CLASSIDS.PT_MYPAGESCONTAINER_ID, nUserID, IPTOpenerConstants.OPENER_MODE_VIEW, nUserID);
	    	url.AddControlParameter(PTMyPages.STR_PAGE_ID, nMyPageID);
    	}
    	((ACommonOpenerURL) url).SetURLImageType(PTImageType.ICON_MYPAGE, strImageServerURL);
    	if ( nMyPageID == TaskAPIUIMyPages.GetCurrentMyPageID(owner) ) {
    		url.SetURLData(PTURLConstants.KEY_CURRENTMARKER, "");
    	}
    	
        return url;
    }
	
	
	/**
	 * Gets the list of the current user's MyPages
	 * 
	 * @param owner		Current Activityspace.
	 * @return <code>XPArrayList</code> with <code>IPTURL</code> entries to the current user's MyPages, or <code>null</code>
	 * when MyPages should not be displayed, i.e. if MyPages feature is disabled in the current Experience Definition or on Preview
	 * pages.
	 * 
	 * @throws <code>XPIllegalArgumentException</code> if the provided <code>AActivityspace</code> is <code>null</code> or not valid.
	 */
	public static XPArrayList GetMyPagesAsURLs(AActivitySpace owner)
    {
        TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
        IPTSession ptSession = TaskAPIUICommon.GetPTSession(owner);
        XPArrayList list = null;

        if ( IsMyPageFeatureEnabledInSubPortal(owner) && !TaskAPIUICommon.IsCurrentPageInPreviewMode(owner) )
        {
        	IPTQueryResult qrMyPages =  TaskAPIServerMyPages.GetMyPages(ptSession);
        	list = new XPArrayList();
        	
        	for ( int i = 0; i < qrMyPages.RowCount(); i++ )
        	{
        		int nPageID    				= qrMyPages.ItemAsInt(i,PT_PROPIDS.PT_PROPID_OBJECTID);
				String strPageName 	= qrMyPages.ItemAsString(i, PT_PROPIDS.PT_PROPID_NAME);
				
				IPTURL url = CreateMyPageCommonOpenerURL(owner, nPageID, strPageName);
				list.Add(url);
        	}
        }
        	
        return list;
    }
        
	/**
	 * Gets the list of MyPages actions for the current user.
	 * 
	 * @param owner		Current Activityspace.
	 * @return <code>XPArrayList</code> with <code>IPTURL</code> entries to the current user's MyPage Actions.
	 * Actions Edit MyPage Editor, Add MyPage Portlets Editor, and Edit MyPage Portlet Preferences Editor,
	 * and Delete Current MyPage are only displayed when on a MyPage.  
	 * 
	 * @throws <code>XPIllegalArgumentException</code> if the provided <code>AActivityspace</code> is <code>null</code> or not valid.
	 */
    public static XPArrayList GetMyPageActionsAsURLs(AActivitySpace owner)
    {
    	return GetMyPageActionsAsURLs(owner, false);
    }
    
	/**
	 * Gets the list of MyPages actions for the current user.
	 * 
	 * @param owner		Current Activityspace.
	 * @param bIncludeOneClick	indicates if the URL for one-click page creation should be included in the list.  default is false.
	 * @return <code>XPArrayList</code> with <code>IPTURL</code> entries to the current user's MyPage Actions.
	 * Actions Edit MyPage Editor, Add MyPage Portlets Editor, and Edit MyPage Portlet Preferences Editor,
	 * and Delete Current MyPage are only displayed when on a MyPage.  
	 * 
	 * @throws <code>XPIllegalArgumentException</code> if the provided <code>AActivityspace</code> is <code>null</code> or not valid.
	 */
    public static XPArrayList GetMyPageActionsAsURLs(AActivitySpace owner, boolean bIncludeOneClick)
    {
        TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
        XPArrayList listURLs = new XPArrayList();
        
        IPTURL url = null;
        
        if (bIncludeOneClick)
        {
	        url = GetURLForCreateMyPageOneClick(owner);
	        if ( null != url ) listURLs.Add(url);
        }
        
        url = GetURLForCreateMyPageEditor(owner);
        if ( null != url ) listURLs.Add(url);
        url = GetURLForEditMyPageEditor(owner);
        if ( null != url ) listURLs.Add(url);
        url = GetURLForEditLayoutEditor(owner);
        if ( null != url ) listURLs.Add(url);
        url = GetURLForAddMyPagePortletsEditor(owner);
        if ( null != url ) listURLs.Add(url);
        url = GetURLForEditMyPagePortletPreferencesEditor(owner);
        if ( null != url ) listURLs.Add(url);
        url = GetURLForDeleteCurrentMyPageAction(owner);
        if ( null != url ) listURLs.Add(url);
        
        return listURLs;
        
    }
     
     /**
      * Checks if the current user already has created the maximum number of MyPages (defined in
      * <code>PT_SERVERCONSTANTS.PT_MAX_MYPORTAL_PAGES</code>.
      *  
      * @param owner		Current Activityspace
      * @return <code>true</code> if the maximum number of MyPages has been created for the current user, <code>false</code> otherwise.
      * 
      * @throws <code>XPIllegalArgumentException</code> if the provided <code>AActivityspace</code> is <code>null</code> or not valid.
      */
    public static boolean IsMaxMyPageLimitReached(AActivitySpace owner)
    {
    	TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
    	IPTSession ptSession = TaskAPIUICommon.GetPTSession(owner);
    	IPTQueryResult qrMyPages = TaskAPIServerMyPages.GetMyPages(ptSession);
		
		return ( PT_SERVERCONSTANTS.PT_MAX_MYPORTAL_PAGES < qrMyPages.RowCount() ); 
    }
    
    /**
     * Checks if the current page is a MyPage
     * 
     * @param owner		Current Activityspace
     * @return <code>true</code> if current page is a MyPage, false otherwise.
     * 
     * @throws <code>XPIllegalArgumentException</code> if the provided AActivityspace is null
     */
    public static boolean IsCurrentPageMyPage(AActivitySpace owner)
    {
    	TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
    	String strASName = TaskAPIUICommon.GetActivitySpaceName(owner);
	    return MyPageAS.STR_MVC_CLASS_NAME.equals(  strASName );
    }

	/**
	 * Gets the current MyPage name. If current page is not a MyPage, a null value is returned.
	 * 
	 * @param owner
	 * @return	MyPage Name of the current MyPage or <code>null</code> value if not currently on a MyPage. 
	 * 
	 *@throws <code>XPIllegalArgumentException</code> if the provided <code>AActivityspace</code> is <code>null</code> or not valid.
	 */
	public static String GetCurrentMyPageName(AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);

		if (owner.GetModelExists(MyPortalModel.STR_MVC_CLASS_NAME))
		{
			MyPortalModel model = (MyPortalModel) owner.GetModel(MyPortalModel.STR_MVC_CLASS_NAME);
			return model.GetPageName();
		}
		else if ( FlyoutPortletSelectionAS.STR_MVC_CLASS_NAME.equals(TaskAPIUICommon.GetActivitySpaceName(owner)) )
		{
			// rob: The flyout AS is always preceded by community or mypage AS, so call the same function with the parent space  
			return GetCurrentMyPageName(owner.GetParentSpace());
		}

		return null;
	}

    /**
     * Gets the URL for the Create MyPage Editor.
     * 
     * @param owner		Current Activityspace
     * @return	<code>IPTURL</code> with the URL to the Create MyPage Editor, or <code>null</code> if it URL is not valid
     * in the current context. For example when MyPages feature is disabled in the current Experience Definition Editor, or
     * the maximum number of MyPages has been reached.
     * 
     * @throws <code>XPIllegalArgumentException</code> if the provided <code>AActivityspace</code> is <code>null</code> or not valid.
     */
    public static IPTURL GetURLForCreateMyPageEditor(AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		// This will get the Parent activity space if the current space is not MyPageAS
		// We need to pass in the correct space for the checks below.
		AActivitySpace myPageAS = TaskAPIUICommon.GetMyPageOrCommunityAActivitySpace(owner);
		// not on a mypage or community as, don't return anything
		if ( myPageAS == null) { return null; }
			
			
		CommonOpenerOpenASURL url = null;
		// Special editor in Edit User Profile Mode
		if ( TaskAPIUICommon.IsCurrentPageInEditDefaultProfileMode(myPageAS) )
		{
			String strURLTitle 			= myPageAS.GetString(302, "ptmsgs_portalcommonmsgs");
			// Use current AActivitySpace for the following calls
			String strApplBaseURL   =  TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
			String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
			String strASName 				= PageEditorAS.STR_MVC_CLASS_NAME;
			int nUserID 							= TaskAPIUIUser.GetCurrentUserID(myPageAS);
		
			url = new CommonOpenerOpenASURL(strURLTitle, strApplBaseURL,  strASName ,nUserID);
			
			url.AddControlParameter(ASConstants.CONTROL , EditorStartControl.STR_MVC_CLASS_NAME);
			url.AddControlParameter(EditorStartControl.QS_EDITOR_TYPE, EditorStartControl.EDITOR_START_FLAG_CREATE);
			
			
			url.AddControlParameter(PageEditorModel.EDITOR_QS_INT_QS_FOLDER_ID, -nUserID );
			url.AddControlParameter(PageEditorModel.EDITOR_QS_INT_CLASS_ID, PT_CLASSIDS.PT_PAGE_ID );
			url.AddControlParameter(PTPages.PAGE_EDITOR_QS_INT_DEFAULT_PROFILE_ID, nUserID );
			
			url.AddControlParameter(ASConstants.PARENTSPACEID, myPageAS.GetSpaceID());
			url.AddControlParameter(ASConstants.PARENTSPACENAME, myPageAS.GetName());

			// Set image
			url.SetURLImageType(PTImageType.ACTION_MYPAGE_NEW, strImageServerURL);
			
		} 
		else 
		{
		
			if ( IsMyPageActionsDisplayable(myPageAS, false) )
			{
				
				if (! IsMaxMyPageLimitReached(myPageAS) )
				{
				
					String strURLTitle 			= myPageAS.GetString(302, "ptmsgs_portalcommonmsgs");
					String strApplBaseURL   =  TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
					String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
					String strASName 				= EditMyPortalPageAS.STR_MVC_CLASS_NAME;
					int nUserID 							= TaskAPIUIUser.GetCurrentUserID(myPageAS);
					
					url = new CommonOpenerOpenASURL(strURLTitle, strApplBaseURL,  strASName ,nUserID);
					url.AddControlParameter(ASConstants.CONTROL , EditorStartControl.STR_MVC_CLASS_NAME);
					url.AddControlParameter(EditPageQueryArguments.HTMLINPUT_EDITPAGE_CREATEMYPAGE, XPConvert.ToString(true));
					url.AddControlParameter(EditorStartControl.QS_EDITOR_TYPE, EditorStartControl.EDITOR_START_FLAG_MISC);
					
					url.AddControlParameter(ASConstants.PARENTSPACEID, myPageAS.GetSpaceID());
					url.AddControlParameter(ASConstants.PARENTSPACENAME, myPageAS.GetName());
	
					// Set image
					url.SetURLImageType(PTImageType.ACTION_MYPAGE_NEW, strImageServerURL);
				}
			}
		}
		
		return url;
		
	}
     
    /**
     * Gets the URL for the one-click MyPage creation
     * 
     * @param owner		Current Activityspace
     * @return	<code>IPTURL</code> with the URL to the one-click MyPage control, or <code>null</code> if it URL is not valid
     * in the current context. For example when MyPages feature is disabled in the current Experience Definition Editor, or
     * the maximum number of MyPages has been reached.
     * 
     * @throws <code>XPIllegalArgumentException</code> if the provided <code>AActivityspace</code> is <code>null</code> or not valid.
     */
    public static IPTURL GetURLForCreateMyPageOneClick(AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		// This will get the Parent activity space if the current space is not MyPageAS
		// We need to pass in the correct space for the checks below.
		AActivitySpace myPageAS = TaskAPIUICommon.GetMyPageOrCommunityAActivitySpace(owner);
		if ( myPageAS == null) { return null; }
		
		CommonOpenerOpenASURL url = null;

		if ( IsMyPageActionsDisplayable(myPageAS, false) )
		{
			
			if (! IsMaxMyPageLimitReached(myPageAS) )
			{
			
				String strURLTitle 			= myPageAS.GetString(802, "ptmsgs_portalcommonmsgs");
				String strApplBaseURL   =  TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
				String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
				String strASName 				= EditMyPortalPageAS.STR_MVC_CLASS_NAME;
				int nUserID 							= TaskAPIUIUser.GetCurrentUserID(myPageAS);
				
				url = new CommonOpenerOpenASURL(strURLTitle, strApplBaseURL,  strASName ,nUserID);
				url.AddControlParameter(ASConstants.CONTROL , EditPageCreateDefaultControl.STR_MVC_CLASS_NAME);
				url.AddControlParameter(EditPageQueryArguments.HTMLINPUT_EDITPAGE_CREATEMYPAGE, XPConvert.ToString(true));
				
				url.AddControlParameter(ASConstants.PARENTSPACEID, myPageAS.GetSpaceID());
				url.AddControlParameter(ASConstants.PARENTSPACENAME, myPageAS.GetName());

				// Set image
				url.SetURLImageType(PTImageType.ACTION_MYPAGE_NEW, strImageServerURL);
			}
		}
		
		return url;
	}    
    
	 /**
	  * Gets the URL to the Edit current MyPage Editor for the Standard Access Style mode. 
	  * 
	  * @param owner		Current Activityspace.
	  * @return <code>IPTURL</code> with the URL to the current MyPage Editor, or <code>null</code>if the URL is invalid 
	  * in the current context. For example when MyPages feature is disabled in the current Experience Definition Editor.
	  * 
	  * @throws <code>XPIllegalArgumentException</code> if the provided <code>AActivityspace</code> is <code>null</code> or not valid.
	  */
    public static IPTURL GetURLForEditMyPageEditor(AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		// This will get the Parent activity space if the current space is not MyPageAS
		// We need to pass in the correct space for the checks below.
		AActivitySpace myPageAS = TaskAPIUICommon.GetMyPageOrCommunityAActivitySpace(owner);
		if ( myPageAS == null) { return null; }
		
		CommonOpenerOpenASURL url = null;
		// Special editor in Edit User Profile Mode
		if ( TaskAPIUICommon.IsCurrentPageInEditDefaultProfileMode(myPageAS) )
		{
			String strURLTitle 			= myPageAS.GetString(301, "ptmsgs_portalcommonmsgs");
			// Use current AActivitySpace for the following calls
			String strApplBaseURL   =  TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
			String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
			String strASName 				= PageEditorAS.STR_MVC_CLASS_NAME;
			int nUserID 							= TaskAPIUIUser.GetCurrentUserID(myPageAS);
			int nPageID							= GetCurrentMyPageID(myPageAS);
		
			url = new CommonOpenerOpenASURL(strURLTitle, strApplBaseURL,  strASName ,nUserID);
			
			url.AddControlParameter(ASConstants.CONTROL , EditorStartControl.STR_MVC_CLASS_NAME);
			url.AddControlParameter(EditorStartControl.QS_EDITOR_TYPE, EditorStartControl.EDITOR_START_FLAG_EDIT);
			
			
			url.AddControlParameter(PageEditorModel.EDITOR_QS_INT_QS_FOLDER_ID, -nUserID );
			url.AddControlParameter(PageEditorModel.EDITOR_QS_INT_QS_OBJECT_ID, nPageID );
			url.AddControlParameter(PageEditorModel.EDITOR_QS_INT_CLASS_ID, PT_CLASSIDS.PT_PAGE_ID );
			url.AddControlParameter(PTPages.PAGE_EDITOR_QS_INT_DEFAULT_PROFILE_ID, nUserID );
			
			url.AddControlParameter(ASConstants.PARENTSPACEID, myPageAS.GetSpaceID());
			url.AddControlParameter(ASConstants.PARENTSPACENAME, myPageAS.GetName());

			// Set image
			url.SetURLImageType(PTImageType.ACTION_MYPAGE_NEW, strImageServerURL);
			
		}
		else
		{
	
			if ( IsMyPageActionsDisplayable(myPageAS, true) && TaskAPIUIUser.IsAccessStyleStandard(myPageAS) )
			{
				
					String strURLTitle 			= myPageAS.GetString(301, "ptmsgs_portalcommonmsgs");
					String strApplBaseURL   =  TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
					String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
					String strASName 				= EditMyPortalPageAS.STR_MVC_CLASS_NAME;
					int nUserID 							= TaskAPIUIUser.GetCurrentUserID(myPageAS);
					int nPageID							= GetCurrentMyPageID(myPageAS);
					
					url = new CommonOpenerOpenASURL(strURLTitle, strApplBaseURL,  strASName ,nUserID);
					url.AddControlParameter(ASConstants.CONTROL , EditorStartControl.STR_MVC_CLASS_NAME);
					url.AddControlParameter(EditPageQueryArguments.HTMLINPUT_EDITPAGE_OPENERPAGEID, nPageID);
					url.AddControlParameter(EditorStartControl.QS_EDITOR_TYPE, EditorStartControl.EDITOR_START_FLAG_MISC);
					
					url.AddControlParameter(ASConstants.PARENTSPACEID, myPageAS.GetSpaceID());
					url.AddControlParameter(ASConstants.PARENTSPACENAME, myPageAS.GetName());
					
					// Set image
					url.SetURLImageType(PTImageType.ICON_EDIT_MENU, strImageServerURL);
			}
			
		}
		
		return url;
		
	}
     
    /**
	  * Gets the URL to the Edit current MyPage Editor for the Low-Bandwidth and Accessibility Style mode. 
	  * 
	  * 
	  * @param owner		Current Activityspace.
	  * @return <code>IPTURL</code> with the URL to the current MyPage Editor, or <code>null</code>if the URL is invalid 
	  * in the current context. For example when MyPages feature is disabled in the current Experience Definition Editor.
	  * 
	  * @throws <code>XPIllegalArgumentException</code> if the provided <code>AActivityspace</code> is <code>null</code> or not valid.
	  * */
    public static IPTURL GetURLForEditLayoutEditor(AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		// This will get the Parent activity space if the current space is not MyPageAS
		// We need to pass in the correct space for the checks below.
		AActivitySpace myPageAS = TaskAPIUICommon.GetMyPageOrCommunityAActivitySpace(owner);
		if ( myPageAS == null) { return null; }
		
		CommonOpenerOpenASURL url = null;
		if ( IsMyPageActionsDisplayable(myPageAS, true) )
		{
			if ( !TaskAPIUIUser.IsAccessStyleStandard(myPageAS) )
			{
				String strURLTitle 			= myPageAS.GetString(301, "ptmsgs_portalcommonmsgs");
				// Use current AActivitySpace for the following calls
				String strApplBaseURL   =  TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
				String strImgSvrURL   		=  TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
				String strASName 				= EditMyPagePortletLayoutAS_D.STR_MVC_CLASS_NAME;
				int nUserID 							= TaskAPIUIUser.GetCurrentUserID(myPageAS);
				int nPageID							= GetCurrentMyPageID(myPageAS);
				
				url = new CommonOpenerOpenASURL(strURLTitle, strApplBaseURL,  strASName, nUserID);
				url.AddControlParameter(ASConstants.CONTROL , EditorStartControl.STR_MVC_CLASS_NAME);
				url.AddControlParameter(EditMyPagePortletLayoutModel_D.QS_PAGEID, nPageID);
				url.AddControlParameter(EditorStartControl.QS_EDITOR_TYPE, EditorStartControl.EDITOR_START_FLAG_MISC);
				
				url.AddControlParameter(ASConstants.PARENTSPACEID, myPageAS.GetSpaceID());
				url.AddControlParameter(ASConstants.PARENTSPACENAME, myPageAS.GetName());

				// Set image
				url.SetURLImageType(PTImageType.ICON_EDIT_MENU, strImgSvrURL);
			}
		}
		return url;
		
	}

     /**
      * INTERNAL HELPER FUNCTION
      * Performs the common checks whether a MyPage actions should be displayed or not.  
      * 
      * @param owner		Current Activityspace.
      * @return <code>true</code> if MyPage actions should be displayed in the current context, <code>false</code> if not. 
      * 
      * @throws <code>XPIllegalArgumentException</code> if the provided <code>AActivityspace</code> is <code>null</code> or not valid.
      */
     static boolean IsMyPageActionsDisplayable(AActivitySpace owner, boolean bRequireMyPage)
     {
     	// Don't display mypage actions for guest user - except when editing default user profile
     	if ( TaskAPIUIUser.IsCurrentUserGuestUser(owner) && !TaskAPIUICommon.IsCurrentPageInEditDefaultProfileMode(owner))
     		return false;
     	
     	if ( IsMyPageFeatureEnabledInSubPortal(owner) && !TaskAPIUICommon.IsCurrentPageInPreviewMode(owner) 
     		)
     	{
     		if ( bRequireMyPage ) {
     			return IsCurrentPageMyPage( owner);
     		} else {
     			return true;
     		}
     	}
     	
     	return false;
     		
     }
     
     /**
      * Gets the URL to the MyPage Add Portlets Editor.
      * 
      * @param owner		Current Activityspace.
      * @return <code>IPTURL</code> with the URL to the MyPage App Portlets Editor, or <code>null</code>if the URL is invalid 
	  * in the current context. For example when MyPages feature is disabled in the current Experience Definition Editor.
      * 
      * @throws <code>XPIllegalArgumentException</code> if the provided <code>AActivityspace</code> is <code>null</code> or not valid.
      */
    public static IPTURL GetURLForAddMyPagePortletsEditor(AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		// This will get the Parent activity space if the current space is not MyPageAS
		// We need to pass in the correct space for the checks below.
		AActivitySpace myPageAS = TaskAPIUICommon.GetMyPageOrCommunityAActivitySpace(owner);
		if ( myPageAS == null) { return null; }
		
		CommonOpenerOpenASURL url = null;
		if ( IsMyPageActionsDisplayable(myPageAS, true) )
		{
				String strURLTitle 			= myPageAS.GetString(300, "ptmsgs_portalcommonmsgs");
				// Use current AActivitySpace for the following calls
				String strApplBaseURL   =  TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
				String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
				String strASName 				= PortletSelectionEditorAS.STR_MVC_CLASS_NAME;
				int nUserID 							= TaskAPIUIUser.GetCurrentUserID(myPageAS);
				int nPageID							= GetCurrentMyPageID(myPageAS);
				
				url = new CommonOpenerOpenASURL(strURLTitle, strApplBaseURL,  strASName ,nUserID);
				url.AddControlParameter(ASConstants.CONTROL , EditorStartControl.STR_MVC_CLASS_NAME);
				url.AddControlParameter(ObjSelQueryArguments.HTMLINPUT_OBJECTSELECTION_SELECTOROBJECTID, nPageID);
				url.AddControlParameter(EditorStartControl.QS_EDITOR_TYPE, EditorStartControl.EDITOR_START_FLAG_MISC);
				
				url.AddControlParameter(ASConstants.PARENTSPACEID, myPageAS.GetSpaceID());
				url.AddControlParameter(ASConstants.PARENTSPACENAME, myPageAS.GetName());
				// Set image
				url.SetURLImageType(PTImageType.ACTION_PORTLET_ADD, strImageServerURL);
		}
		return url;
		
	}
     
     /**
      * Gets the URL to the MyPage Add Portlets Flyout.
      * 
      * @param owner		Current Activityspace.
      * @return <code>IPTURL</code> with the URL to the MyPage App Portlets Flyout, or <code>null</code>if the URL is invalid 
	  * in the current context. For example when MyPages feature is disabled in the current Experience Definition Editor.
      * 
      * @throws <code>XPIllegalArgumentException</code> if the provided <code>AActivityspace</code> is <code>null</code> or not valid.
      */
    public static IPTURL GetURLForAddMyPagePortletsFlyout(AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		
		CommonOpenerOpenASURL url = null;
		if ( IsMyPageActionsDisplayable(owner, true) )
		{
			String strURLTitle 			= owner.GetString(300, "ptmsgs_portalcommonmsgs");
			String strApplBaseURL   =  TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
			String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
			String strASName 				= FlyoutPortletSelectionAS.STR_MVC_CLASS_NAME;
			int nUserID 						= TaskAPIUIUser.GetCurrentUserID(owner);
			int nPageID							= GetCurrentMyPageID(owner);
			
			url = new CommonOpenerOpenASURL(strURLTitle, strApplBaseURL, strASName, nUserID);
			url.AddControlParameter(ASConstants.CONTROL , EditorStartControl.STR_MVC_CLASS_NAME);
			url.AddControlParameter(ObjSelQueryArguments.HTMLINPUT_OBJECTSELECTION_SELECTOROBJECTID, nPageID);
			url.AddControlParameter(EditorStartControl.QS_EDITOR_TYPE, EditorStartControl.EDITOR_START_FLAG_MISC);
			url.AddControlParameter(ASConstants.PARENTSPACEID, owner.GetSpaceID());
			url.AddControlParameter(ASConstants.PARENTSPACENAME, owner.GetName());
			// Set image
			url.SetURLImageType(PTImageType.ACTION_PORTLET_ADD, strImageServerURL);
		}
		return url;
		
	} 
     
    /**
      * Gets the URL to the MyPage Edit Portlet Preference Editor.
      * 
      * @param owner		Current Activityspace.
      * @return <code>IPTURL</code> with the URL to the MyPage Edit Portlet Preferences Editor, or <code>null</code>if the URL is invalid 
	  * in the current context. For example when MyPages feature is disabled in the current Experience Definition Editor.
      * 
      * @throws <code>XPIllegalArgumentException</code> if the provided <code>AActivityspace</code> is <code>null</code> or not valid.
      */
    public static IPTURL GetURLForEditMyPagePortletPreferencesEditor(AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		// This will get the Parent activity space if the current space is not MyPageAS
		// We need to pass in the correct space for the checks below.
		AActivitySpace myPageAS = TaskAPIUICommon.GetMyPageOrCommunityAActivitySpace(owner);
		if ( myPageAS == null) { return null; }
		
		CommonOpenerOpenASURL url = null;
		if ( IsMyPageActionsDisplayable(myPageAS, true) )
		{
			String strURLTitle 			= owner.GetString(306, "ptmsgs_portalcommonmsgs");
			// Use current AActivitySpace for the following calls
			String strApplBaseURL   =  TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
			String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
			String strASName 				= EditPortletPrefsAS.STR_MVC_CLASS_NAME;
			int nUserID 							= TaskAPIUIUser.GetCurrentUserID(myPageAS);
			int nPageID							= GetCurrentMyPageID(myPageAS);
			
			url = new CommonOpenerOpenASURL(strURLTitle, strApplBaseURL,  strASName ,nUserID);
			url.AddControlParameter(ASConstants.CONTROL , EditorStartControl.STR_MVC_CLASS_NAME);
			url.AddControlParameter(EditPortletPrefsAS.QS_PARAM_PAGEID, nPageID);
			url.AddControlParameter(EditorStartControl.QS_EDITOR_TYPE, EditorStartControl.EDITOR_START_FLAG_MISC);
			
			url.AddControlParameter(ASConstants.PARENTSPACEID, myPageAS.GetSpaceID());
			url.AddControlParameter(ASConstants.PARENTSPACENAME, myPageAS.GetName());

			// Set image
			url.SetURLImageType(PTImageType.ACTION_PORTLET_PREF, strImageServerURL);
		}
		return url;
		
	}     
    
	/**
      * Gets the URL for Deleting the current MyPage.
      * 
      * @param owner		Current Activityspace.
      * @return <code>IPTURL</code> with the URL to delete the current MyPage, or <code>null</code>if the URL is invalid 
	  * in the current context. For example when MyPages feature is disabled in the current Experience Definition Editor, or
	  * if the current MyPage is the only MyPage.
      * 
      * @throws <code>XPIllegalArgumentException</code> if the provided <code>AActivityspace</code> is <code>null</code> or not valid.
      */
    public static IPTURL GetURLForDeleteCurrentMyPageAction(AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		// This will get the Parent activity space if the current space is not MyPageAS
		// We need to pass in the correct space for the checks below.
		AActivitySpace myPageAS = TaskAPIUICommon.GetMyPageOrCommunityAActivitySpace(owner);
		if ( myPageAS == null) { return null; }
		
		CommonOpenerOpenASURL url = null;
		if ( IsMyPageActionsDisplayable(myPageAS, true) )
		{
//			mrc: fixed bug where delete link would should up even if there was only 1 page left			
//			if ( GetCurrentMyPageID(myPageAS) > 0)
	    	IPTQueryResult qrMyPages = TaskAPIServerMyPages.GetMyPages(TaskAPIUICommon.GetPTSession(owner));
			if (qrMyPages.RowCount() > 1)
			{
				String strURLTitle 			= myPageAS.GetString(303, "ptmsgs_portalcommonmsgs");
				// Use current AActivitySpace for the following calls
				String strApplBaseURL   =  TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
				String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
				String strASName 				= MyPageAS.STR_MVC_CLASS_NAME;
				int nUserID 							= TaskAPIUIUser.GetCurrentUserID(myPageAS);
				int nPageID							= GetCurrentMyPageID(myPageAS);
				
				url = new CommonOpenerOpenASURL(strURLTitle, strApplBaseURL,  strASName ,nUserID);
				url.AddControlParameter(ASConstants.CONTROL , DeletePageControl.STR_MVC_CLASS_NAME);
				url.AddControlParameter(DeletePageControl.STR_PAGE_ID, nPageID);
				
				url.AddControlParameter(ASConstants.PARENTSPACEID, myPageAS.GetSpaceID());
				url.AddControlParameter(ASConstants.PARENTSPACENAME, myPageAS.GetName());
				
				// Set image
				url.SetURLImageType(PTImageType.ACTION_DELETE_MENU, strImageServerURL);
				
			}
		}
		return url;
		
	}     
    
    /**
     * Gets the object ID of the current MyPage.
     * 
     * @param owner		Current Activityspace.
     * @return Object ID of the MyPage, or 0 if not currently on a MyPage.
     * 
     * @throws <code>XPIllegalArgumentException</code> if the provided <code>AActivityspace</code> is <code>null</code> or not valid.
     */
    public static int GetCurrentMyPageID(AActivitySpace owner)
    {
    	TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
    	int nCurrPageID = 0;
    	if ( TaskAPIUICommon.GetMyPortalPageModelExists(owner) ) 
    	{
    		MyPortalModel ptModel = TaskAPIUICommon.GetMyPortalPageModel(owner);
    		nCurrPageID = ptModel.GetPageID();
    	}
    	else if ( FlyoutPortletSelectionAS.STR_MVC_CLASS_NAME.equals(TaskAPIUICommon.GetActivitySpaceName(owner)) )
		{
			// rob: The flyout AS is always preceded by community or mypage AS, so call the same function with the parent space  
			nCurrPageID = GetCurrentMyPageID(owner.GetParentSpace());
		}
    	
    	
    	return nCurrPageID;
    }
    
    /**
	 * Helper function to retrieve the MyPage Object ID from a MyPage <code>CommonOpenerOpenObjIDURL</code> to a MyPage.
	 * 
	 * @param ptURL		An <code>IPTURL</code> to a MyPage
	 * @return The MyPage ID as an <code>int</code> or 0 if the provided URL is not a 
	 * <code>CommonOpenerOpenObjIDURL</code>or MyPage URL.
	 * 
	 * @throws <code>XPIllegalArgumentException</code> if the provided <code>AActivityspace</code> is <code>null</code> or not valid.
	 */
	public static int GetMyPageIDFromPTURL(IPTURL ptURL)
	{
		if ( null == ptURL ) {
			return 0;
		}
		int nPageID = 0;
		String strPageID = ptURL.GetData(PTMyPages.STR_PAGE_ID);
		
		if ( null != strPageID )
		{
			try {
				nPageID = XPConvert.ToInteger(strPageID);
			} catch (XPFormatException e) {
				// Don't do anything, we will just return the default page id
			}
		}
		return nPageID;
		
	}
     
}
