using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.application.varpacks;
using com.plumtree.portaluiinfrastructure.sso.integrations;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.activityspace.utils;
using com.plumtree.uiinfrastructure.interpreter;
using com.plumtree.uiinfrastructure.login;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.portaluiinfrastructure.sso
{
  /// <summary>
  /// Helper class which handles integration with SSO products.
  /// </summary>
  /// <author>Mike Jahr</author>
  public sealed class SSOIntegrator : ISSOHelper
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.sso.SSOIntegrator");
    
    /// <summary>
    /// Config.xml SSOVENDOR constant: SSO disabled. 
    /// </summary>
    public const int SSOVENDOR_NONE = 0;
    
    /// <summary>
    /// Config.xml SSOVENDOR constant: Netegrity Siteminder 
    /// </summary>
    public const int SSOVENDOR_NETEGRITY = 2;
    
    /// <summary>
    /// Config.xml SSOVENDOR constant: Oblix NetPoint 
    /// </summary>
    public const int SSOVENDOR_OBLIX = 3;
    
    /// <summary>
    /// Config.xml SSOVENDOR constant: Windows Integrated Auth (aka NTLM) 
    /// </summary>
    public const int SSOVENDOR_WINDOWS = 5;
    
    /// <summary>
    /// Config.xml SSOVENDOR constant: Basic SSO 
    /// </summary>
    public const int SSOVENDOR_BASIC = 50;
    
    /// <summary>
    /// Config.xml SSOVENDOR constant: custom integration 
    /// </summary>
    public const int SSOVENDOR_CUSTOM = 100;
    
    /// <summary>
    /// This is the name of the session attribute that holds the GW URL that was 
    /// accessed before the session was established. -- moose
    /// </summary>
    public const String REQUEST_URL_BEFORE_SSO_LOGIN = "REQUESTURLBEFORESSO";
    // mj: static members
    private static bool s_bSSOEnabled = false;
    private static ISSOIntegration s_masterSSO = null;
    private static int s_nSSOVendor = SSOVENDOR_NONE;
    private static String s_sCustomClass = null;
    private static String s_sCustomAssembly = null;
    // mj: cookie settings
    private static String s_sCookiePath;
    private static String s_sCookieDomain;
    private static bool s_bCookieIsSecure;
    // mj: capture basic auth setting
    private static bool s_bCaptureBasicAuth = false;
    // mj: array of secure header names
    private static String[] s_asHeaders = null;
    // mj: the default auth source prefix.
    private static String s_sDefaultPrefix = null;
    // mj: a map containing the names of all secure cookies, in uppercase
    private static XPHashtable s_mapCookies = null;
    
    /// <summary>
    /// Returns login info from the given request.
    /// </summary>
    /// <param name="request">an <c>IXPRequest</c> value</param>
    /// <returns>the default prefix</returns>
    public static SSOLoginInfo GetLoginInfo(IXPRequest request)
    {
      for (IXPEnumerator e = request.GetHeaderNames();e.MoveNext();)
      {
        String sHeader = (String) e.GetCurrent();
        log.Debug("SSO sees the header " + sHeader + ": " + request.GetHeader(sHeader));
        }
      SSOLoginInfo loginInfo = s_masterSSO.GetLoginInfo(request);
      return loginInfo;
    }
    
    /// <summary>
    /// Returns a <c>PTSSOInfo</c> object with the correct values.
    /// </summary>
    /// <param name="request">an <c>IXPRequest</c> value</param>
    /// <param name="info">a <c>SSOLoginInfo</c> value</param>
    /// <returns>an <c>IPTSSOInfo</c> value</returns>
    public static IPTSSOInfo GetSSOInfo(IXPRequest request, SSOLoginInfo info)
    {
      IPTSSOInfo ptSSOInfo = PortalObjectsFactory.CreateSSOInfo();
      if (ptSSOInfo == null)
      {
        log.Error("Unable to create a new IPTSSOInfo object. SSO is not operating correctly.");
        return null;
      }
      // mj: add secure headers
      if (s_asHeaders != null)
      {
        for (int i = 0;i < s_asHeaders.Length;i++)
        {
          ptSSOInfo.AddOneSecureHeader(s_asHeaders[i]);
        }
      }
      // mj: iterate over all cookies, and add the sso cookies
      XPCookie[] aCookies = request.GetCookies();
      bool bExpiresSet = false;
      XPDateTime ptExpires = new XPDateTime();
      if (aCookies != null && s_mapCookies != null)
      {
        for (int i = 0;i < aCookies.Length;i++)
        {
          if (s_mapCookies.ContainsKey(XPStringUtility.ToUpperCase(aCookies[i].GetName())))
          {
            log.Debug("SSO is adding the cookie " + aCookies[i].GetName());
            // mj: by default, make the expiration date 20 minutes in the
            // future.	Since we will pull the most recent cookie value out
            // of the headers on each request for a MyPage (and thus this
            // expiration value doesn't really correspond with when we
            // actually get a new cookie, since we get it on every request),
            // we just need to make this value larger than the amount of
            // time it takes to load a MyPage.
            if (!bExpiresSet)
            {
              ptExpires = new XPDateTime();
              ptExpires = new XPDateTime(ptExpires.GetTimeInMilliSeconds() + 20 * 1000 * 60);
              bExpiresSet = true;
            }
            // The path, domain, and IsSecure values are set in x_config.xml.
            // The Plumtree web server will forward the secure cookies to
            // any remote gadget servers whose URLs match the path and
            // domain specified in x_config.xml.	If IsSecure is TRUE, then
            // the Plumtree web server will send the cookie only over an SSL
            // connection.	If it is FALSE, then SSL is not required.
            ptSSOInfo.AddSecureSessionCookie(aCookies[i].GetName(), aCookies[i].GetValue(),
             ptExpires, s_sCookiePath, s_sCookieDomain, s_bCookieIsSecure);
            }
        }
      }
      // mj: add basic auth name/password
      if (s_bCaptureBasicAuth)
      {
        // mj: extract only the username portion
        String sName = info.GetPrefixAndName();
        ptSSOInfo.SetBasicAuthName(XPStringUtility.Substring(sName, XPStringUtility.IndexOf(sName,
         '\\') + 1));
        ptSSOInfo.SetBasicAuthPassword(info.GetPassword());
        log.Debug("SSO is setting the basic auth username to \'" + XPStringUtility.Substring(sName,
         XPStringUtility.IndexOf(sName, '\\') + 1) + "\'");
      }
      return ptSSOInfo;
    }
    
    /// <summary>
    /// Returns the default auth source prefix.
    /// </summary>
    /// <returns>the default prefix</returns>
    public static String GetDefaultPrefix()
    {
      return s_sDefaultPrefix;
    }
    
    /// <summary>
    /// Attribute on the subsession indicating whether sso login has been attempted.
    /// </summary>
    public const String SSOSTATEATTRIBUTE = "nSSOStatus";
    
    /// <summary>
    /// <c>SSOSTATEATTRIBUTE</c> constant indicating that no sso user is
    /// logged in or attempting to log in.
    /// </summary>
    public const int SSOSTATE_LOGGEDOUT = 0;
    
    /// <summary>
    /// <c>SSOSTATEATTRIBUTE</c> constant indicating that an sso user is
    /// attempting to log in.
    /// </summary>
    public const int SSOSTATE_ATTEMPTING = 1;
    
    /// <summary>
    /// <c>SSOSTATEATTRIBUTE</c> constant indicating that an sso user is
    /// logged in.
    /// </summary>
    public const int SSOSTATE_LOGGEDIN = 2;
    
    /// <summary>
    /// Calls the SSO logout code and performs an SSO logout redirect if necessary.
    /// </summary>
    /// <param name="tempData">a <c>RequestData</c> value</param>
    /// <returns>a <c>boolean</c> value</returns>
    public bool DoLogoutRedirect(RequestData tempData)
    {
      Object oAttribute = tempData.m_SessionManager.GetAttribute(SSOSTATEATTRIBUTE);
      if (null == oAttribute || SSOSTATE_LOGGEDOUT != XPConvert.ToInteger(oAttribute))
      {
        // mj: if we're not a guest, this means the user is logging out.
        // mj: create an url so we know where to return to afterwards.
        XPStringBuilder sb = new XPStringBuilder(tempData.m_xpRequest.GetRequestURL());
        String sUserName = XPHttpUtility.UrlEncode(((IPTSession) tempData.GetUserSession()).GetSessionInfo().GetCurrentUserName(),
         LinkConstants.UTF_8_ENCODING);
        // mj: add a parameter to make the login control display the sso logout page.
        sb.Append('?').Append(LoginHelper.PARAM_SSOLOGOUT).Append('=').Append(sUserName);
        // mj: add the rest of the original query string
        String sQS = tempData.m_xpRequest.GetQueryString();
        if (sQS != null)
        {
          int indStr = XPStringUtility.IndexOf(sQS, "doLogout=&");
          if (indStr != -1 && sQS.Length >= indStr + 10)
          {
            String strBegin = XPStringUtility.Substring(sQS, 0, indStr);
            String strEnd = XPStringUtility.Substring(sQS, indStr + 10);
            XPStringBuilder sbQS = new XPStringBuilder(strBegin);
            sbQS.Append(strEnd);
            sb.Append('&').Append(sbQS.ToString());
          }
        }
        // mj: this will be our return url
        String sReturnURL = sb.ToString();
        // mj: set the sso state attribute so we don't get in an infinite loop
        tempData.m_SessionManager.SetAttribute(SSOSTATEATTRIBUTE, SSOSTATE_LOGGEDOUT);
        // mj: do the sso logout
        if (s_masterSSO.OnLogout(tempData.m_xpResponse, sReturnURL))
        {
          log.Info("Redirecting for SSO logout; return url is " + sReturnURL);
          // mj: invoke logout actions
          Redirect r = LoginHelper.INSTANCE.DoTasksBeforeLogout(tempData.GetUserSession(),
           tempData.m_xpRequest, tempData.m_wdPageData, tempData.m_Application,
           tempData.m_smSandbox, tempData.m_smPersistentSandbox);
          if (r != null && log.IsWarnEnabled())
          {
            log.Warn("SSO logout is pre-empting the Logout PEI redirect to " + (r.GetIsExternalURL() ? r.GetExternalURL() : r.GetQueryString()));
            }
          // mj: logout code goes here.
          LoginHandlers.DoLogout(tempData);
          // mj: if a redirect is required, exit the method.
          return true;
        }
      }
      return false;
    }
    
    /// <summary>
    /// Performs a redirect to the SSO login page, unless SSO login has previously failed.
    /// </summary>
    /// <param name="tempData">a <c>RequestData</c> value</param>
    /// <param name="sQueryString">a <c>String</c> value</param>
    /// <returns>boolean true if we are redirecting, false otherwise</returns>
    public Redirect DoLoginRedirect(RequestData tempData, String sQueryString)
    {
      // CR DLH 10/7/05: The way this method passes the original URL to the
      // SSO servlet seems unnecessarily complicated.  I think this may just
      // be because we originally only passed the query string, but now we
      // need to pass the server as well (since you can have multiple server
      // URLs with URL mapping) and we just left the query string section as
      // is, rather than integrating them into one consistent scheme.
      if (!SSOAuthenticator.GetInstance().UseThisAuth(tempData))
      {
        return null;
      }
      // mj: if sso has already failed, don't try it again.
      Object oAttribute = tempData.m_SessionManager.GetAttribute(SSOSTATEATTRIBUTE);
      if (tempData.m_nSSOState == SSOSTATE_ATTEMPTING || (null != oAttribute && SSOSTATE_ATTEMPTING == XPConvert.ToInteger(oAttribute)))
      {
        // mj: reset sso state attribute so the user can try to log in again later
        tempData.m_SessionManager.SetAttribute(SSOSTATEATTRIBUTE, SSOSTATE_LOGGEDOUT);
        tempData.m_nSSOState = SSOSTATE_ATTEMPTING;
        // due to session fixation protection with sso in dotnet where session is lost during redirect to 
        // sso login page, the redirect url is passed in cookie instead of session
        // should really pass redirect in querystring but there is some old and complex handling of querystring
        // in ssologin page so less risky to pass redirect in cookie for nightrain release
        XPCookie cookie = new XPCookie(REQUEST_URL_BEFORE_SSO_LOGIN, "");
        cookie.SetPath("/");
        tempData.m_xpResponse.AddCookie(cookie);
        //			tempData.m_SessionManager.RemoveAttribute(REQUEST_URL_BEFORE_SSO_LOGIN);
        log.Info("SSO login previously attempted; not performing SSO redirect.");
        return null;
      } else
      {
        // mj: set the sso state to attempting so we'll know if we fail.
        //	   otherwise we could get in an infinite loop.
        tempData.m_SessionManager.SetAttribute(SSOSTATEATTRIBUTE, SSOSTATE_ATTEMPTING);
        String strRedirectURL = tempData.GetRequestFullURL();
        if (XPStringUtility.CompareIgnoreCase("POST", tempData.m_xpRequest.GetMethod()) == 0)
        {
          String strReferer = tempData.m_xpRequest.GetHeader("Referer");
          // This ensures that we have a valid referer string that starts with http
          if ((null != strReferer) && (XPStringUtility.StartsWith(strReferer,
           HttpUtils.HTTP_SCHEME)))
          {
            // We have to use the referer string for POST requests because the
            // SSO servlet will try to do a 302 redirect to the original URL, and
            // 302 redirects can't do POST requests, only GETS.  This means the
            // original POST data will be lost, but hopefully the user will get
            // redirected back to the original form that they can then resubmit.
            // This seems to be the best possible scenario given the http
            // limitations, although it can definitely lead to strange behavior sometimes.
            strRedirectURL = strReferer;
            // The SSO Servlet recombines the stored base URL with the query string
            // to come up with a full url.  Since we are using the Referer URL instead
            // of the original URL, the original query string is no longer relevant.
            // NOTE: most POST requests should not have query strings, but some may.			
            sQueryString = null;
            // If the referer URL has a query string, we need to use this instead so
            // that the SSOLoginPage can recombine this query string with the referer
            // host name stored on the session.
            int nQueryStringIndex = XPStringUtility.IndexOf(strReferer, "?");
            if ((nQueryStringIndex > 0) && (strReferer.Length > nQueryStringIndex))
            {
              sQueryString = XPStringUtility.Substring(strReferer, nQueryStringIndex + 1);
              }
          }
        }
        // mj: set the original request url on the session so we'll know where to return to
        // rob: store as a cookie since in dotnet the session is lost in the redirect due to
        // session fixation protection
        // @TODO Should really pass redirect in querystring but there is some old and complex handling 
        // of querystring in ssologin page so less risky to pass redirect in cookie for nightrain release
        //tempData.m_SessionManager.SetAttribute(REQUEST_URL_BEFORE_SSO_LOGIN, strRedirectURL);
        // Replace semi-colon with colon since semi-colon is a delimiter in cookies, need to replace back when
        // reading cookie value again
        String cookieEncodedRedirectUrl = strRedirectURL.Replace(';', ':');
        log.Debug("Storing cookie value with SSO Login Redirect URL: " + cookieEncodedRedirectUrl);
        XPCookie cookie = new XPCookie(REQUEST_URL_BEFORE_SSO_LOGIN, cookieEncodedRedirectUrl);
        cookie.SetPath("/");
        tempData.m_xpResponse.AddCookie(cookie);
        // The URLs being retrieved here cannot be cached since these methods are called through an static singleton that is not thread safe.
        // GetSSOBaseURL returns a relative URL that includes the SSO virtual directory, e.g. /portal/SSOServlet
        // 43555: change RelativeURL to BaseURL
        String ssoRedirectURL = PTConfigHelper.GetSSOBaseURL();
        // mel: Fix for 36883, Should have SSOServlet on https when the portal is running in security mode 1
        // In security mode 1 all login traffic should be secure. We force the use of SecureApplicationURL here.
        // In security mode 2, all traffic is through https, so there's no need to use an absolute URL.
        if (ConfigHelper.GetSecurityMode() == 1)
        {
          // GetSSOAbsoluteURL returns an absolute URL, e.g. https://hostname:8080/portal/SSOServlet
          ssoRedirectURL = SecurityConfigHelper.GetInstance().GetSSOAbsoluteURL();
          }
        // mj: redirect to sso servlet, passing the query string
        String sRedirect = ssoRedirectURL + ((sQueryString == null) ? "" : sQueryString);
        tempData.m_xpResponse.Redirect(sRedirect);
        log.Info("SSO login redirecting to: " + sRedirect);
        Redirect redirect = new Redirect();
        redirect.SetIsHTTPRedirect(true);
        redirect.SetLinkToExternalURL(sRedirect);
        return redirect;
      }
    }
    
    /// <summary>
    /// Indicates whether SSO is enabled at the portal level.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.login.ISSOHelper#IsSSOEnabled()"></seealso>
    /// <returns>a <c>boolean</c> value</returns>
    public bool IsSSOEnabled()
    {
      return s_bSSOEnabled;
    }
    
    /// <summary>
    /// Indicates whether capture basic auth is enabled.
    /// </summary>
    /// <returns>a <c>boolean</c> value</returns>
    public static bool IsCaptureBasicAuthEnabled()
    {
      return s_bCaptureBasicAuth;
    }
    
    /// <summary>
    /// Initializes global SSO settings using the given settings from x_config.xml.
    /// Note that only StaticInitSSO actually enables SSO.
    /// </summary>
    /// <param name="vpConfig">a <c>PTConfigVarPack</c> value</param>
    public static void StaticInitConfig(PTConfigVarPack vpConfig)
    {
      s_nSSOVendor = 0;
      // mj: get the sso integrator class from the config settings
      try
      {
        // mj: get the capture basic auth setting
        try
        {
          s_bCaptureBasicAuth = vpConfig.GetConfigValueAsString("Authentication",
           "CaptureBasicAuthenticationForPortlets", "value").Equals("1");
          }
        catch (Exception e)
        {
          log.Warn("Unable to retrieve CaptureBasicAuthenticationForPortlets setting, defaulting to 0. " + "Check the CaptureBasicAuthenticationForPortlets setting in the Authentication section " + "of PTConfig.xml; it must be an integer.");
          }
        s_nSSOVendor = vpConfig.GetConfigValueAsInt("Authentication", "SSOVendor",
         "value");
        if (s_nSSOVendor >= SSOVENDOR_CUSTOM)
        {
          // mj: if custom sso is enabled, get the class and assembly names.
          s_sCustomClass = vpConfig.GetConfigValueAsString("Authentication",
           "CustomSSOClass", "value");
          s_sCustomAssembly = vpConfig.GetConfigValueAsString("Authentication",
           "CustomSSOAssembly", "value");
        }
        if (s_nSSOVendor != SSOVENDOR_NONE)
        {
          // mj: get the default auth source prefix
          s_sDefaultPrefix = vpConfig.GetConfigValueAsString("Authentication",
           "DefaultAuthSourcePrefix", "value");
          if ((s_sDefaultPrefix == null) || (s_sDefaultPrefix.Length == 0))
          {
            s_sDefaultPrefix = null;
            log.Info("SSO default auth source prefix is empty.");
          } else
          {
            log.Info("SSO default auth source prefix is \'" + s_sDefaultPrefix + "\'.");
            }
          log.Info("CaptureBasicAuthenticationForPortlets is " + XPConvert.ToString(s_bCaptureBasicAuth));
          // mj: get the cookie path and domain
          try
          {
            s_sCookiePath = vpConfig.GetConfigValueAsString("Authentication",
             "CookiePath", "value");
            s_sCookieDomain = vpConfig.GetConfigValueAsString("Authentication",
             "CookieDomain", "value");
          }
          catch (Exception e)
          {
            log.Warn("Unable to retrieve SSO cookie path and domain. " + "Check the CookiePath and CookieDomain settings in the Authentication section of PTConfig.xml.");
            }
          try
          {
            // TODO: Write GetConfigValueAsBoolean
            s_bCookieIsSecure = (1 == vpConfig.GetConfigValueAsInt("Authentication",
             "SSOCookieIsSecure", "value"));
          }
          catch (Exception e)
          {
            log.Warn("SSOCookieIsSecure setting is not an integer. Defaulting to 0. " + "Check the SSOCookieIsSecure setting in the Authentication section of PTConfig.xml; " + "it must be an integer.");
            }
          log.Info("SSO cookie path is " + s_sCookiePath);
          log.Info("SSO cookie domain is " + s_sCookieDomain);
          log.Info("SSO CookieIsSecure is " + XPConvert.ToString(s_bCookieIsSecure));
          }
      }
      catch (Exception e)
      {
        log.Error(e, "Unable to get the SSO config settings.  " + "Check the settings in the Authentication section of PTConfig.xml.");
        log.Error("SSO is disabled.");
      }
    }
    
    /// <summary>
    /// Initializes SSO settings using the config settings.
    /// Can only be called once globally.
    /// </summary>
    /// <param name="vpSSO">a <c>PTSSOVarPack</c> value</param>
    public static void StaticInitSSO(PTSSOVarPack vpSSO)
    {
      ISSOIntegration ssoInt = null;
      try
      {
        switch (s_nSSOVendor){
        case SSOVENDOR_NONE:
        // mj: sso is disabled; return
        log.Info("SSO is disabled.");
        s_masterSSO = null;
        s_bSSOEnabled = false;
        return;
        goto case SSOVENDOR_NETEGRITY;
        case SSOVENDOR_NETEGRITY:
        ssoInt = new Netegrity();
        break;
        case SSOVENDOR_OBLIX:
        ssoInt = new Oblix();
        break;
        case SSOVENDOR_WINDOWS:
        ssoInt = new WindowsIntegratedAuth();
        break;
        case SSOVENDOR_BASIC:
        ssoInt = new BasicSSO();
        break;
        default:
        if (s_nSSOVendor < SSOVENDOR_CUSTOM)
        {
          log.Error("Unknown SSO vendor " + s_nSSOVendor + "; SSO is disabled.");
          return;
        }
        // mj: we must have a class name; assembly is optional.
        if ((s_sCustomClass == null) || (s_sCustomClass.Length == 0))
        {
          log.Error("Unable to read the custom SSO class config string; SSO is disabled.");
          return;
        }
        log.Info("Attempting to instantiate the custom SSO class " + s_sCustomClass);
        // mj: try to instantiate the custom class
        try
        {
          ssoInt = (ISSOIntegration) XPDynamicDiscovery.GetObject(s_sCustomClass,
           s_sCustomAssembly);
        }
        catch (Exception e)
        {
          log.Error(e, "Exception while instantiating the custom SSO class " + s_sCustomClass + ", assembly " + s_sCustomAssembly);
          log.Error("SSO is disabled.");
          return;
        }
        break;
        }
      }
      catch (Exception e)
      {
        log.Error(e, "Exception while instantiating the SSO integration class.");
        log.Error("SSO is disabled.");
        return;
      }
      try
      {
        log.Info("Attempting to initialize SSO; vendor product name is " + ssoInt.GetSSOProductName());
        // mj: init the sso integration
        if (!ssoInt.Initialize(vpSSO.GetVendorSettings()))
        {
          log.Error("Initialization of the SSO integration class failed. SSO is disabled.");
          return;
        }
        // mj: init static headers array
        String[] asHeaders = ssoInt.GetSecureHeaders();
        if (asHeaders != null)
        {
          log.Info("Got " + asHeaders.Length + " secure SSO headers.");
          for (int i = 0;i < asHeaders.Length;i++)
          {
            log.Debug("SSO header [" + i + "]: " + asHeaders[i]);
          }
        }
        // mj: init static cookies map
        String[] asCookies = ssoInt.GetSecureCookies();
        if (asCookies != null)
        {
          log.Info("Got " + asCookies.Length + " secure SSO cookies.");
          for (int i = 0;i < asCookies.Length;i++)
          {
            log.Debug("SSO cookie [" + i + "]: " + asCookies[i]);
          }
        }
        XPHashtable mapCookies = null;
        if (asCookies != null)
        {
          mapCookies = new XPHashtable();
          for (int i = 0;i < asCookies.Length;i++)
          {
            if (asCookies[i] != null)
            {
              mapCookies.PutElement(XPStringUtility.ToUpperCase(asCookies[i]),
               asCookies[i]);
            } else
            {
              log.Warn("SSO integration returned a null cookie name.");
            }
          }
        }
        // mj: we managed to instantiate and initialize an ISSOIntegration. init the static members.
        s_masterSSO = ssoInt;
        s_asHeaders = asHeaders;
        s_mapCookies = mapCookies;
        s_bSSOEnabled = true;
        log.Info("SSO is enabled.");
      }
      catch (Exception e)
      {
        log.Error(e, "SSO initialization failed. SSO is disabled.");
      }
    }
    public static void UpdateSSOCookieHeader(AActivitySpace _asOwner)
    {
      RequestData reqData = new RequestData(_asOwner.GetCurrentHTTPRequest(),
       _asOwner.GetCurrentHTTPResponse(), _asOwner.GetSubSession(), _asOwner.GetApplication());
      if (!SSOAuthenticator.GetInstance().UseThisAuth(reqData))
      {
        return;
      }
      //Get the IPTSSOInfo off of the sandbox session
      ISessionManager sessionManager = _asOwner.GetSubSession();
      Object ssoInfo = sessionManager.GetAttribute(PlumtreeExtensibility.PT_SSOINFO_ON_STATE);
      if (null != ssoInfo)
      {
        // Look for the SSO cookie
        int nSSOCookies = ((IPTSSOInfo) ssoInfo).GetTotalNumCookies();
        for (int indSSOCookie = 0;indSSOCookie < nSSOCookies;indSSOCookie++)
        {
          IPTCookieInfo cookieInfo = ((IPTSSOInfo) ssoInfo).GetCookieOfIndex(indSSOCookie);
          String strSSOCookieName = cookieInfo.GetCookieName();
          String cookieNameUppercase = XPStringUtility.ToUpperCase(strSSOCookieName);
          // mel: Fix for 45749, Basic SSO: Cookie(s) not being sent to every page.
          // The cookie name as stored in cookieInfo may not match our s_mapCookies
          // values that are case sensitive.
          // As a result, we should be using the uppercase key names
          // in our s_mapCookies to determine if the cookie value should be updated.
          // This bug was only noticeable by having mixed case cookie
          // names and appeared as failure to refresh
          // cookie values sent to a remote portlet.
          // TODO Enforce uppercase only access to s_mapCookies so that subtle bugs like forgetting to uppercase can be prevented. i.e. provide a small wrapper that enforces the cookie case rules.
          if (s_mapCookies.ContainsKey(cookieNameUppercase))
          {
            // Set the new expiration to 20 minutes
            XPDateTime ptExpires = new XPDateTime();
            ptExpires = new XPDateTime(ptExpires.GetTimeInMilliSeconds() + 20 * 1000 * 60);
            // Look up the cookie value from the request in case it has been updated.
            // If we don't find it, then it will default to the original value.
            String strCookieVal = cookieInfo.GetCookieValue();
            XPCookie[] reqCookies = _asOwner.GetCurrentHTTPRequest().GetCookies();
            for (int indReqCookie = 0;indReqCookie < reqCookies.Length;indReqCookie++)
            {
              if (reqCookies[indReqCookie].GetName().Equals(strSSOCookieName))
              {
                strCookieVal = reqCookies[indReqCookie].GetValue();
                break;
              }
            }
            // Overwrite the old cookie
            cookieInfo.PutCookieInfo(strSSOCookieName, strCookieVal, ptExpires,
             cookieInfo.GetCookiePath(), cookieInfo.GetCookieDomain(), cookieInfo.IsSecureCookie());
            }
        }
      }
    }
  }
}
//	/**
//	 * Extracts specified cookies from the given <code>IXPRequest</code>.  Note
//	 * that only the keys of <code>mapNames</code> are used; the values are
//	 * ignored.
//	 *
//	 * @param mapNames an <code>XPHashtable</code> containing the cookie names
//	 * @param request an <code>IXPRequest</code> value
//	 * @return a <code>XPCookie[]</code> value
//	 */
//	private static XPArrayList GetCookieValues(XPHashtable mapNames, IXPRequest request)
//	{
//		// mj: if mapNames is null, return null
//		if (mapNames == null)
//		{
//			return null;
//		}
//
//		// mj: iterate over all the cookies in the request, picking out the
//		// names we were given
//		XPArrayList alOut	= new XPArrayList(mapNames.GetSize());
//		XPCookie[] aCookies = request.GetCookies();
//
//		for (int i = 0; i < aCookies.length; i++)
//		{
//			if (mapNames.ContainsKey(aCookies[i].GetName()))
//			{
//				
//					log.Debug(//						"SSO is processing the cookie " + aCookies[i].GetName());
//				
//
//				alOut.Add(aCookies[i]);
//			}
//		}
//		
//		return alOut;
//	}
