using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.statichelpers;
namespace com.plumtree.portaluiinfrastructure.sso
{
  /// <summary>
  /// Simple class which contains a username and password.
  /// </summary>
  /// <author>Mike Jahr</author>
  public class SSOLoginInfo
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.sso.SSOLoginInfo");
    private String m_sPassword;
    private String m_sPrefixAndName;
    
    /// <summary>
    /// Constructs a <c>SSOLoginInfo</c> with the given information.  The
    /// given prefix and username are conca  If no password is available, <c>null</c>
    /// should be passed, or the simplest constructor can be used.
    /// </summary>
    /// <param name="sPrefix">the user prefix</param>
    /// <param name="sName">the username without the prefix</param>
    /// <param name="sPassword">the password of the user</param>
    public SSOLoginInfo(String sPrefix, String sName, String sPassword)
    {
      m_sPassword = sPassword;
      if (sPrefix != null && sPrefix.Length > 0)
      {
        m_sPrefixAndName = sPrefix + '\\' + sName;
      } else
      {
        SetPrefixAndName(sName);
      }
    }
    
    /// <summary>
    /// Constructs a <c>SSOLoginInfo</c> with the given information.  If
    /// the prefix of the user is unavailable, sPrefixAndName should contain only
    /// the username without a "\\"; in this case the default prefix from
    /// x_config.xml will be added.  If no password is available, <c>null</c>
    /// should be passed, or the simplest constructor can be used.
    /// </summary>
    /// <param name="sPrefixAndName">the username in the form <c>"prefix\\username"</c></param>
    /// <param name="sPassword">the password of the user</param>
    public SSOLoginInfo(String sPrefixAndName, String sPassword)
    {
      m_sPassword = sPassword;
      SetPrefixAndName(sPrefixAndName);
    }
    
    /// <summary>
    /// Constructs a <c>SSOLoginInfo</c> with the given username and an
    /// empty password.\u0009Identical to 
    /// <c>SSOLoginInfo(sPrefixAndName, null)</c>.
    /// </summary>
    /// <param name="sPrefixAndName">the username in the form <c>"prefix\\username"</c></param>
    public SSOLoginInfo(String sPrefixAndName) : this(sPrefixAndName, null)
    {
    }
    
    /// <summary>
    /// Returns the user's password.\u0009 Only necessary if it is to be passed to
    /// portlets in basic auth headers.
    /// </summary>
    /// <returns>the password</returns>
    public virtual String GetPassword()
    {
      return m_sPassword;
    }
    
    /// <summary>
    /// Returns the prefix and name of this user in the form
    /// <c>"prefix\\username"</c>.
    /// </summary>
    /// <returns>the qualified username</returns>
    public virtual String GetPrefixAndName()
    {
      return m_sPrefixAndName;
    }
    
    /// <summary>
    /// Sets the password;
    /// </summary>
    /// <param name="sPassword">the user's password</param>
    public virtual void SetPassword(String sPassword)
    {
      m_sPassword = sPassword;
    }
    
    /// <summary>
    /// Sets the username in the form <c>"prefix\\username"</c>.  If the
    /// prefix of the user is unavailable, sPrefixAndName should contain only the
    /// username without a "\\"; in this case the default prefix from x_config.xml
    /// will be added.
    /// </summary>
    /// <param name="sName">the username in the form <c>"prefix\\username"</c></param>
    public virtual void SetPrefixAndName(String sName)
    {
      if ((sName == null) || (sName.Length == 0))
      {
        m_sPrefixAndName = null;
      } else
      if (XPStringUtility.IndexOf(sName, '\\') == -1)
      {
        // mj: if m_sPrefixAndName doesn't contain the domain, we must add it
        String sDomain = SSOIntegrator.GetDefaultPrefix();
        if ((sDomain != null) && (sDomain.Length > 0))
        {
          log.Info("SSOLoginInfo: Found default prefix: {0}", sDomain);
          m_sPrefixAndName = new XPStringBuilder(sDomain.Length + sName.Length + 1).Append(sDomain).Append('\\').Append(sName).ToString();
          } else
        {
          log.Info("SSOLoginInfo: No domain or default prefix was found.");
          m_sPrefixAndName = sName;
        }
      } else
      {
        m_sPrefixAndName = sName;
      }
      log.Info("SSOLoginInfo username: {0}", m_sPrefixAndName);
    }
  }
}