using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.portaluiinfrastructure.tags;
using com.plumtree.portaluiinfrastructure.tags.helper;
using com.plumtree.portaluiinfrastructure.tags.manager;
using com.plumtree.portaluiinfrastructure.tags.metadata;
using com.plumtree.portaluiinfrastructure.tags.tasks;
using com.plumtree.server;
using ConfigHelper=com.plumtree.uiinfrastructure.statichelpers.ConfigHelper;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portaluiinfrastructure.tags.displaytree
{
  /// <summary>
  /// This class displays a Tag.
  /// On construction, it merely stores the Tag data,
  /// and waits until execution time to create the tag.
  /// This is a framework only class and should not be
  /// used by external developers.
  /// </summary>
  /// <author>donh</author>
  public class TagDisplayElement : IDisplayElement
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADAPTIVETAGS),
     "com.plumtree.portaluiinfrastructure.tags.displaytree.TagDisplayElement");
    
    /// <summary>
    /// This is the default tag library name used when a tag does
    /// not specify a library name.  This library is used for the
    /// legacy 5.0 Transformer Tags.
    /// </summary>
    public const String DEFAULT_TAG_LIB_NAME = "standard";
    
    /// <summary>
    /// This is the separator used between library and tag names.
    /// </summary>
    public const char LIBRARY_SEPARATOR = '.';
    protected Object[][] m_ObjXML;
    protected ATag m_Tag = null;
    protected String m_strTagName = null;
    protected String m_strLibraryName = null;
    protected CompositeDisplayElement m_ParentContainer = null;
    protected CompositeDisplayElement m_ChildElements = null;
    protected String m_strErrorMessage = null;
    
    /// <summary>
    /// Construct the Display Element, but do not construct
    /// the Tag until it is actually required.
    /// </summary>
    /// <param name="Object[][]">_ObjXML The tag data.</param>
    /// <param name="CompositeDisplayElement">
    /// The parent container element
    /// that this tag element is stored in.
    /// </param>
    public TagDisplayElement(Object[][] _objXML, CompositeDisplayElement _parentContainer)
    {
      m_ObjXML = _objXML;
      m_ParentContainer = _parentContainer;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.displaytree.IDisplayElement#Display(TagVisitor)"></seealso>
    public virtual HTMLElement Display(TagVisitor _tagVisitor)
    {
      HTMLElement tagHTML;
      // Get the initial debug state
      bool bDebug = TagStateTasks.GetIsDebugMode(_tagVisitor.GetState());
      // Get the display stack trace state (Bug 64494)
      bool bDisplayStackTrace = ConfigHelper.GetPerformanceStacktracesEnabled();
      // Locate the tag
      try
      {
        // Get the tag name if it has not already been determined.
        CalculateTagName();
        // Pass through all tags from unknown tag libraries.
        if (TagLibManager.getInstance().GetLibraries().GetElement(m_strLibraryName) == null)
        {
          // Convert tag to HTML and display as is.
          return DisplayAsHTML(_tagVisitor);
        }
        // Get the tag if it has not already been retrieved.
        if (null == m_Tag)
        {
          m_Tag = TagLibManager.getInstance().GetTag(m_strTagName, m_strLibraryName);
          }
        if (null == m_Tag)
        {
          String strError = "Could not find tag " + m_strTagName + " in library " + m_strLibraryName + ".  Tag and Library names are case sensitive. " + "The TagMetaData class automatically converts tag " + "names to lowercase, so a tag created with uppercase " + "letters in the name can only be accessed using all " + "lowercase letters.";
          log.Error(strError);
          return new HTMLComment(strError);
        }
      }
      catch (Exception e)
      {
        String strError = "Error locating tag " + m_strTagName + " in library " + m_strLibraryName + ".";
        log.Error(e, strError);
        XPException xpe = XPException.GetInstance(e);
        if (bDisplayStackTrace)
        {
          tagHTML = new HTMLComment(strError + " " + xpe.GetMessage() + ": " + xpe.GetStackTrace());
          } else
        {
          tagHTML = new HTMLComment("An Exception occurred displaying this tag. See log for details.");
          }
      }
      // Display the tag
      try
      {
        InitializeTag(_tagVisitor);
        String strValidateError = m_Tag.GetTagHelper().ValidateContext();
        if (null == strValidateError)
        {
          tagHTML = m_Tag.GetTagHelper().DisplayTag();
        } else
        {
          // This tag is not being used in the proper context
          // and should not be displayed.
          String strError = "Tag " + m_strTagName + " in library " + m_strLibraryName + " used incorrectly: " + strValidateError;
          log.Error(strError);
          tagHTML = new HTMLComment(strError);
        }
      }
      catch (Exception e)
      {
        String strError = "Error displaying tag " + m_strTagName + " in library " + m_strLibraryName + ".";
        log.Error(e, strError);
        XPException xpe = XPException.GetInstance(e);
        if (bDisplayStackTrace)
        {
          tagHTML = new HTMLComment(strError + " " + XPHttpUtility.HtmlEncode(xpe.GetMessage()) + ": " + xpe.GetStackTrace());
          } else
        {
          tagHTML = new HTMLComment("An Exception occurred displaying this tag. See log for details.");
          }
      }
      finally
      {
        // Make sure that even if DisplayTag throws an exception,
        // we clean up the TagState properly so we don't get
        // into a bad state.
        CleanupTag(_tagVisitor);
      }
      HTMLElement result;
      // Get the current debug state.  Any tag that starts or ends in
      // debug mode should display the debug comments.
      bDebug = bDebug || TagStateTasks.GetIsDebugMode(_tagVisitor.GetState());
      if (bDebug)
      {
        result = new HTMLElementCollection();
        // Debug mode comments
        HTMLElement startComment = GetStartComment();
        HTMLElement endComment = GetEndComment();
        result.AddInnerHTMLElement(startComment);
        result.AddInnerHTMLElement(tagHTML);
        result.AddInnerHTMLElement(endComment);
      } else
      {
        result = tagHTML;
      }
      // Display the tag error message, if there is one.
      if (null != m_strErrorMessage)
      {
        HTMLElementCollection errorResult = new HTMLElementCollection();
        // Display the message before the tag, so it is easier to locate.
        errorResult.AddInnerHTMLElement(new HTMLComment(m_strErrorMessage));
        errorResult.AddInnerHTMLElement(result);
        result = errorResult;
      }
      // If this tag has a body, but is not supposed to
      // have a body, add a warning.
      if ((m_Tag.GetTagType() == TagType.NO_BODY) && (null != m_ChildElements))
      {
        HTMLElementCollection warnResult = new HTMLElementCollection();
        String strWarning = "Tag " + m_strTagName + " in library " + m_strLibraryName + " does not display inner HTML, but this tag has inner HTML.  The inner HTML " + "will be ignored.  NO_BODY tags should always be used as singletons (i.e. <pt:lib.tag/>).";
        log.Warn(strWarning);
        // Display the message before the tag, so it is easier to locate.
        warnResult.AddInnerHTMLElement(new HTMLComment(strWarning));
        warnResult.AddInnerHTMLElement(result);
        result = warnResult;
      }
      return result;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.displaytree.IDisplayElement#Release()"></seealso>
    public virtual void Release()
    {
      if (null != m_Tag)
      {
        // This allows tag authors to execute special release code before
        // the tag passes out of scope.
        m_Tag.Release();
      }
    }
    
    /// <summary>
    /// This calculates the tag and library name, if they have
    /// not already been calculated.
    /// </summary>
    protected virtual void CalculateTagName()
    {
      if (null != m_strLibraryName)
      {
        // We have already figured out the library name and tag name.
        return;
      }
      String strName = (String) m_ObjXML[PT_XML_ARRAY_COLS.PT_XML_ARRAY_COLS_NAME][0];
      // The tag name should be in the form library.tag, so we need to parse
      // out the library name and the tag name.  Legacy tags may not have a library
      // name or separator.
      int nSeparatorIndex = XPStringUtility.IndexOf(strName, LIBRARY_SEPARATOR);
      if (-1 != nSeparatorIndex)
      {
        if (nSeparatorIndex > 0)
        {
          m_strLibraryName = XPStringUtility.Substring(strName, 0, nSeparatorIndex);
          } else
        {
          // Assume that tags with a separator but no library name
          // are Legacy tags and are in the default tag library.
          m_strLibraryName = DEFAULT_TAG_LIB_NAME;
        }
        if ((nSeparatorIndex + 1) < strName.Length)
        {
          m_strTagName = XPStringUtility.Substring(strName, nSeparatorIndex + 1,
           strName.Length);
        } else
        {
          // No tag name.
          log.Error("Tag " + strName + " does not have a tag name.");
          return;
        }
      } else
      {
        // Legacy tag in default tag library.
        m_strLibraryName = DEFAULT_TAG_LIB_NAME;
        m_strTagName = strName;
      }
    }
    
    /// <summary>
    /// This initializes the Tag with the data it needs
    /// to execute.
    /// </summary>
    /// <param name="_tagVisitor">The Tag Visitor object</param>
    protected virtual void InitializeTag(TagVisitor _tagVisitor)
    {
      // TODO: We can probably reuse parts of the taghelper if
      // a tag is in a loop and is being called multiple times
      // user supplied attributes will not change, although
      // we will have to reset the processed attributes that
      // used variables (i.e. $var).  This is a possible
      // optimization.
      TagHelper tagHelper = new TagHelper(m_ObjXML, m_ChildElements, m_Tag);
      m_Tag.InitForDisplay(tagHelper);
      _tagVisitor.SetCurrentTag(this);
      // We need to set the tag visitor before we validate or display
      m_Tag.GetTagHelper().SetTagVisitor(_tagVisitor);
      _tagVisitor.GetState().EnterTag(m_Tag);
    }
    
    /// <summary>
    /// This cleans up the tag to make sure it is in the
    /// proper state.
    /// </summary>
    protected virtual void CleanupTag(TagVisitor _tagVisitor)
    {
      // Make sure that if DisplayTag throws an exception,
      // we clean up the TagState properly so we don't get
      // into a bad state.
      _tagVisitor.GetState().ExitTag();
      _tagVisitor.SetCurrentTag(null);
      m_Tag.GetTagHelper().ReleaseTagVisitor();
    }
    
    /// <summary>
    /// This returns a CompositeDisplayElement to be used to
    /// contain the children of this tag.
    /// </summary>
    /// <param name="_parent">The parent of the current tag</param>
    /// <returns>
    /// CompositeDisplayElement Container for the
    /// child elements of this Tag.
    /// </returns>
    public virtual CompositeDisplayElement GetChildElements(CompositeDisplayElement _parent)
    {
      // Lazy initialization of the Child Element array
      // Tags with no body will not need this.
      if (null == m_ChildElements)
      {
        m_ChildElements = new CompositeDisplayElement(this, _parent);
      }
      return m_ChildElements;
    }
    
    /// <summary>
    /// Generate the start comment for debug mode.
    /// </summary>
    /// <returns>
    /// HTMLElement The start comment for
    /// debug mode
    /// </returns>
    protected virtual HTMLElement GetStartComment()
    {
      HTMLElement result = null;
      try
      {
        result = new HTMLComment("Start of " + m_strTagName + " HTML Tag in " + m_strLibraryName + " library.");
        }
      catch (Exception e)
      {
        log.Warn(e, "Unable to add debug mode comment for " + m_strTagName + " tag in " + m_strLibraryName + " library.");
        result = null;
      }
      return result;
    }
    
    /// <summary>
    /// Generate the end comment for debug mode.
    /// </summary>
    /// <returns>
    /// HTMLElement The end comment for
    /// debug mode
    /// </returns>
    protected virtual HTMLElement GetEndComment()
    {
      HTMLElement result = null;
      try
      {
        result = new HTMLComment("End of " + m_strTagName + " HTML Tag in " + m_strLibraryName + " library.");
        }
      catch (Exception e)
      {
        log.Warn(e, "Unable to add debug mode comment for " + m_strTagName + " tag in " + m_strLibraryName + " library.");
        result = null;
      }
      return result;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.displaytree.IDisplayElement#HasChildElements(ITagMetaData[],Object[],TagLibraryMetaData,int)"></seealso>
    public virtual bool HasChildElements(ITagMetaData[] _childTags, Object[] _results,
     TagLibraryMetaData _library, int _nCurrentDepth)
    {
      // Get the tag name if it has not already been determined.
      // Since we are searching down through the tree, this may
      // be the first time we have visited this tag.  There is no
      // need to instantiate the actual tag yet, as we may not
      // process this tag if there is an error higher up in the tree.
      CalculateTagName();
      // Increment the depth for this current PT Transformer Tag.
      _nCurrentDepth++;
      // Only seach the tags if we're even in the same library
      if (0 == XPStringUtility.Compare(m_strLibraryName, _library.GetName()))
      {
        bool bCompleted = true;
        for (int i = 0;i < _childTags.Length;i++)
        {
          if (0 == XPStringUtility.Compare(m_strTagName, _childTags[i].GetName()))
          {
            _results[i] = _nCurrentDepth;
          } else
          if (null == _results[i])
          {
            bCompleted = false;
          }
        }
        if (bCompleted)
        {
          // All of the child tags have been located, so we can
          // finish the search.
          return true;
        }
      }
      if (null != m_ChildElements)
      {
        return m_ChildElements.HasChildElements(_childTags, _results, _library,
         _nCurrentDepth);
      }
      // Search not completed.  Do not return false.
      return false;
    }
    
    /// <summary>
    /// This method checks if the current tag has the requested tags as child tags.
    /// This does not check if they match the current tag, unlike HasChildElements.
    /// </summary>
    /// <param name="_childTags">
    /// An array of ITagMetaData objects for each child
    /// tag to search for.
    /// </param>
    /// <param name="_results">
    /// An array of null Objects.  After execution, child
    /// tags that have been found that will be replaced with Integer objects
    /// containing the positive depth of the child tag in the same location
    /// the tag is in the _ChildTags array.
    /// </param>
    /// <param name="_library">The Tag Library of the tags to look for.</param>
    /// <param name="_nCurrentDepth">The depth of the current tag.</param>
    /// <returns>boolean implies that all of the Child Tags have been found.</returns>
    public virtual bool HasChildTags(ITagMetaData[] _childTags, Object[] _results,
     TagLibraryMetaData _library, int _nCurrentDepth)
    {
      // Increment the depth for this current PT Transformer Tag.
      _nCurrentDepth++;
      if (null != m_ChildElements)
      {
        return m_ChildElements.HasChildElements(_childTags, _results, _library,
         _nCurrentDepth);
      }
      // Search not completed.  Do not return false.
      return false;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.displaytree.IDisplayElement#HasParentElement(ITagMetaData,TagLibraryMetaData)"></seealso>
    public virtual int HasParentElement(ITagMetaData _parentTag, TagLibraryMetaData _library)
    {
      // Get the tag name if it has not already been determined,
      // although we should always have the tag name here.
      CalculateTagName();
      // Make sure both the tag name and library match.
      if ((0 == XPStringUtility.Compare(m_strLibraryName, _library.GetName())) && (0 == XPStringUtility.Compare(m_strTagName,
       _parentTag.GetName())))
      {
        return 1;
      }
      if (null == m_ParentContainer)
      {
        return 0;
      }
      int nResult = m_ParentContainer.HasParentElement(_parentTag, _library);
      if (nResult > 0)
      {
        // Add one to the level for the current tag.
        nResult++;
        return nResult;
      } else
      {
        return 0;
      }
    }
    
    /// <summary>
    /// This method checks if the current tag has the requested tag as a parent tag.
    /// This does not check if it matches the current tag, unlike HasParentElement.
    /// </summary>
    /// <param name="_ParentTag">The ITagMetaData of the tag to look for.</param>
    /// <param name="_library">The Tag Library of the tag to look for.</param>
    /// <returns>
    /// int 0 implies the parent tag was not found and a
    /// positive integer implies that the parent tag was found n
    /// levels above the current tag.
    /// </returns>
    public virtual int HasParentTag(ITagMetaData _parentTag, TagLibraryMetaData _library)
    {
      if (null == m_ParentContainer)
      {
        return 0;
      }
      int nResult = m_ParentContainer.HasParentElement(_parentTag, _library);
      if (nResult > 0)
      {
        // Add one to the level for the current tag.
        nResult++;
        return nResult;
      } else
      {
        return 0;
      }
    }
    
    /// <summary>
    /// This method gets the name of this tag.
    /// Note that this initializes the tag holder
    /// and may cause unnecessary work if this tag
    /// is never displayed.
    /// </summary>
    /// <returns>String The name of this tag.</returns>
    public virtual String GetTagName()
    {
      CalculateTagName();
      return m_strTagName;
    }
    
    /// <summary>
    /// This method gets the name of the library
    /// for this tag.
    /// Note that this initializes the tag holder
    /// and may cause unnecessary work if this tag
    /// is never displayed.
    /// </summary>
    /// <returns>String The name of the library for this tag.</returns>
    public virtual String GetTagLibraryName()
    {
      CalculateTagName();
      return m_strLibraryName;
    }
    
    /// <summary>
    /// This method sets an error to be displayed in an HTML Comment
    /// before the tag. This is used in case there is a problem
    /// parsing the tag HTML (i.e. an unclosed tag).
    /// </summary>
    /// <param name="_strError">
    /// The error message to be displayed.  This does
    /// not need to have the HTML comment marks (i.e. <!-- x -->).
    /// </param>
    public virtual void SetErrorMessage(String _strError)
    {
      m_strErrorMessage = _strError;
    }
    
    /// <summary>
    /// Output the current tag as HTML. This allows it to be handled by
    /// another processing engine if necessary.
    /// </summary>
    /// <param name="_tagVisitor">The tag visitor.</param>
    /// <returns>
    /// HTML for the current tag, plus the processed results
    /// of any child tags.
    /// </returns>
    protected virtual HTMLElement DisplayAsHTML(TagVisitor _tagVisitor)
    {
      HTMLElementCollection result = new HTMLElementCollection();
      bool bIsSingleton = false;
      // The first element has to be the tag name.
      String strName = (String) m_ObjXML[PT_XML_ARRAY_COLS.PT_XML_ARRAY_COLS_NAME][0];
      // Output the current tag as HTML
      XPStringBuilder sb = new XPStringBuilder();
      sb.Append("<pt:").Append(strName);
      // skip the first element, because that's the name
      for (int i = 1;i < m_ObjXML[PT_XML_ARRAY_COLS.PT_XML_ARRAY_COLS_TYPE].Length;
      i++)
      {
        int type = XPConvert.ToInteger(m_ObjXML[PT_XML_ARRAY_COLS.PT_XML_ARRAY_COLS_TYPE][i]);
        if (PT_XML_TYPES.PT_XML_TYPES_PTUI_ATT == type)
        {
          sb.Append(" pt:").Append(m_ObjXML[PT_XML_ARRAY_COLS.PT_XML_ARRAY_COLS_NAME][i]).Append("=\"").Append(m_ObjXML[PT_XML_ARRAY_COLS.PT_XML_ARRAY_COLS_VALUE][i]).Append("\"");
          } else
        if (PT_XML_TYPES.PT_XML_TYPES_XHTML_ATT == type)
        {
          sb.Append(" ").Append(m_ObjXML[PT_XML_ARRAY_COLS.PT_XML_ARRAY_COLS_NAME][i]).Append("=\"").Append(m_ObjXML[PT_XML_ARRAY_COLS.PT_XML_ARRAY_COLS_VALUE][i]).Append("\"");
          } else
        if (PT_XML_TYPES.PT_XML_TYPES_PTUI_CLOSE == type)
        {
          // This is a close element
          bIsSingleton = true;
        }
      }
      if (bIsSingleton)
      {
        sb.Append("/>");
        result.AddInnerHTMLString(sb.ToString());
      } else
      {
        sb.Append(">");
        result.AddInnerHTMLString(sb.ToString());
        result.AddInnerHTMLElement(m_ChildElements.Display(_tagVisitor));
        result.AddInnerHTMLString("</pt:" + strName + ">");
      }
      return result;
    }
  }
}