using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.portaluiinfrastructure.tags;
using IDataObject=com.plumtree.portaluiinfrastructure.tags.data.IDataObject;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.web;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portaluiinfrastructure.tags.helper
{
  /// <summary>
  /// This class manages variable storage and retrieval for
  /// tags.  It allows you to get and set variables for
  /// various contexts (session, request, tag, etc...).
  /// </summary>
  public class TagState : ITagState
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.tags.helper.TagState");
    
    /// <summary>
    /// This key is used to store an http request level list that
    /// stores whether or not a particular class of tags has been
    /// displayed on the page yet.
    /// </summary>
    public const String SHARED_JS_VAR_LIST = "DisplayedTagClasses";
    
    /// <summary>
    /// This key is used to store an http request level HTMLElementCollection
    /// that contains all of the shared javascript for the tags in the HTTP
    /// request.
    /// </summary>
    public const String SHARED_JS_COLLECTION = "SharedJSCollection";
    
    /// <summary>
    /// This key is used to store an http request level Boolean that marks
    /// whether or not the current shared javascript collection for this HTTP
    /// request has been retrieved for displayed or not.
    /// </summary>
    public const String SHARED_JS_COLLECTION_DISPLAYED = "SharedJSCollectionDisplayed";
    
    /// <summary>
    /// The separator between library and tag name in the combined key.
    /// </summary>
    protected const String COMBINED_KEY_SEPARATOR = ".";
    
    /// <summary>
    /// This key is used to store HTTP request data collections on
    /// the session using the request as a key.
    /// </summary>
    public const String HTTP_REQUEST_DATA_KEY = "httprequestdata";
    
    /// <summary>
    /// The Tag Stack.  This keeps track of which Tags
    /// have been entered. It is a list of ATags.
    /// </summary>
    protected XPArrayList m_TagStack;
    
    /// <summary>
    /// The Tag Var Stack.  This keeps track of what
    /// Data has been stored by which tag in tag scope.
    /// It is a list of XPArrayLists.
    /// </summary>
    protected XPArrayList m_TagVarStack;
    
    /// <summary>
    /// The index into the top of the Tag Stack.
    /// </summary>
    protected int m_nTagStackTop;
    
    /// <summary>
    /// The Tag Data container.  This cannot be re-used
    /// between portlet requests and must be reset.
    /// </summary>
    protected XPHashtable m_TagScopeData;
    
    /// <summary>
    /// The Portlet Request Data container.  This
    /// cannot be re-used between portlet requests
    /// and must be reset.
    /// </summary>
    protected XPHashtable m_PortletRequestScopeData;
    
    /// <summary>
    /// The HTTP Request Data container.  This hashtable
    /// is stored in the session using the request as a
    /// key and is cached in the TagState object.
    /// </summary>
    protected XPHashtable m_HTTPRequestScopeData;
    
    /// <summary>
    /// The HTTP Application.
    /// </summary>
    protected IApplication m_Application;
    
    /// <summary>
    /// The sub-session.
    /// </summary>
    protected ISessionManager m_Session;
    
    /// <summary>
    /// The persistent sub-session.
    /// </summary>
    protected ISessionManager m_PersistentSession;
    
    /// <summary>
    /// The current tag.
    /// </summary>
    protected ATag m_CurrentTag = null;
    
    /// <summary>
    /// This static variable can be used to supply a pre-populated
    /// hashtable for the portlet request data.  This will be used
    /// during TagState construction only.  This should only be used
    /// in unit tests and not in production code.
    /// </summary>
    public static XPHashtable TEST_PORTLET_REQUEST_DATA;
    
    /// <summary>
    /// Creates a TagState object.
    /// The TagState object contains portlet request
    /// specific data and must either be reset between
    /// requests or re-created.
    /// </summary>
    /// <param name="_application">The HTTP Application</param>
    /// <param name="_session">The sub-session</param>
    /// <param name="_persistentSession">The persistent sub-session</param>
    public TagState(IApplication _application, ISessionManager _session, ISessionManager _persistentSession,
     XPHashtable _httpRequestData)
    {
      m_TagStack = new XPArrayList();
      m_TagVarStack = new XPArrayList();
      m_nTagStackTop = -1;
      m_TagScopeData = new XPHashtable();
      // If we are in unit test mode, use the supplied hash table if it exists
      if (TagState.TEST_PORTLET_REQUEST_DATA != null)
      {
        m_PortletRequestScopeData = TagState.TEST_PORTLET_REQUEST_DATA;
        // Clear this so unit tests won't overwrite each other.
        TagState.TEST_PORTLET_REQUEST_DATA = null;
      } else
      {
        m_PortletRequestScopeData = new XPHashtable();
      }
      m_HTTPRequestScopeData = _httpRequestData;
      m_Application = _application;
      m_Session = _session;
      m_PersistentSession = _persistentSession;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ITagState#SetVariable(String,Object,Scope)"></seealso>
    public virtual void SetVariable(String _strKey, Object _oValue, Scope _scope)
    {
      CheckValidVariableName(_strKey);
      String strLibName = m_CurrentTag.GetTagMetaDataHelper().GetTagLibraryMetaData().GetName();
      // This stores the variable with a key composed of the lib name
      // and key, meaning that this var is unique for this library.
      // Another library could set the same variable and they would
      // not conflict.
      InternalSetVariable(strLibName, _strKey, _oValue, _scope);
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ITagState#SetSharedVariable(String,Object,Scope,boolean)"></seealso>
    public virtual void SetSharedVariable(String _strKey, Object _oValue,
     Scope _scope, bool bOwnerEditOnly)
    {
      CheckValidVariableName(_strKey);
      String strLibName = m_CurrentTag.GetTagMetaDataHelper().GetTagLibraryMetaData().GetName();
      // Check for appropriate edit access
      SharedStateVariable sharedVariable = (SharedStateVariable) InternalGetVariable(null,
       _strKey, _scope);
      if (null != sharedVariable)
      {
        // This will throw an exception if this library does not
        // have edit access.
        sharedVariable.CheckHasEditAccess(strLibName, _strKey, m_CurrentTag.GetName());
        // Update the pre-existing variable with the new value.
        sharedVariable.SetValue(_oValue);
      } else
      {
        // Create a new variable.
        if (bOwnerEditOnly)
        {
          sharedVariable = new SharedStateVariable(strLibName, _oValue);
        } else
        {
          sharedVariable = new SharedStateVariable(_oValue);
        }
      }
      if (null == _oValue)
      {
        // Setting a variable to null means that we should remove the
        // variable from the appropriate collection.
        sharedVariable = null;
      }
      // This stores the variable with a key composed of just the
      // key, meaning that this var is shared and can be accessed
      // by other libraries.
      InternalSetVariable(null, _strKey, sharedVariable, _scope);
    }
    
    /// <summary>
    /// This method checks that the variable name does not contain
    /// any reserved characters (".").
    /// </summary>
    /// <param name="String">_strVariableName The variable name to be checked.</param>
    /// <throws>
    /// XPIllegalArgumentException if the variable name contains
    /// illegal characters (".").
    /// </throws>
    protected virtual void CheckValidVariableName(String _strVariableName)
    {
      // NOTE: This method is called approximately 2500 times for
      // an empty mypage, so it's pretty expensive.
      if (XPStringUtility.IndexOf(_strVariableName, TagHelper.ATTRIBUTE_VARIABLE_SEPARATOR) >= 0)
      {
        throw new XPIllegalArgumentException("Illegal variable name " + _strVariableName + ".  Contains illegal character \".\".");
        }
    }
    
    /// <summary>
    /// Helper method to set variables in memory.
    /// </summary>
    /// <param name="_strLibrary">The current library.  May be null.</param>
    /// <param name="_strKey">The key used to store the variable.</param>
    /// <param name="_oValue">The variable.</param>
    /// <param name="_scope">The scope used to store the variable.</param>
    protected virtual void InternalSetVariable(String _strLibrary, String _strKey,
     Object _oValue, Scope _scope)
    {
      if (null == _strKey)
      {
        throw new XPIllegalArgumentException("Null key argument in tag " + GetTagName() + " while setting variable to value " + _oValue + ".  Scope: " + _scope);
        }
      if (null == _scope)
      {
        throw new XPIllegalArgumentException("Null scope argument in tag " + GetTagName() + " while setting variable " + _strKey + " to value " + _oValue + ".");
        }
      String strCombinedKey;
      if (null != _strLibrary)
      {
        strCombinedKey = _strLibrary + COMBINED_KEY_SEPARATOR + _strKey;
      } else
      {
        strCombinedKey = COMBINED_KEY_SEPARATOR + _strKey;
      }
      if (null == _oValue)
      {
        // Setting a variable to null means that we should remove the
        // variable from the appropriate collection.
        InternalRemoveVariable(strCombinedKey, _scope);
        return;
      }
      if (Scope.TAG == _scope)
      {
        if (!m_TagScopeData.ContainsKey(strCombinedKey))
        {
          // TODO: check if m_nTagStackTop >= m_TagStack.GetSize() or -1?  (Both invalid)
          XPArrayList varList = (XPArrayList) m_TagVarStack.GetElement(m_nTagStackTop);
          if (null == varList)
          {
            // Create a new list if this is the first variable
            // associated with this tag
            varList = new XPArrayList();
            m_TagVarStack.SetElement(m_nTagStackTop, varList);
          }
          // Associate the new tag scope variable with the current tag 
          varList.Add(strCombinedKey);
        }
        m_TagScopeData.PutElement(strCombinedKey, _oValue);
      } else
      if (Scope.PORTLET_REQUEST == _scope)
      {
        m_PortletRequestScopeData.PutElement(strCombinedKey, _oValue);
      } else
      if (Scope.HTTP_REQUEST == _scope)
      {
        m_HTTPRequestScopeData.PutElement(strCombinedKey, _oValue);
      } else
      if (Scope.SESSION == _scope)
      {
        m_Session.SetAttribute(strCombinedKey, _oValue);
      } else
      if (Scope.PERSISTENT_SESSION == _scope)
      {
        m_PersistentSession.SetAttribute(strCombinedKey, _oValue);
      } else
      if (Scope.APPLICATION == _scope)
      {
        m_Application.GetCachingManager().SetEntry(strCombinedKey, _oValue);
        } else
      {
        // Unknown scope constant
        throw new XPIllegalArgumentException("Unknown scope argument in tag " + GetTagName() + " while setting variable " + _strKey + " to value " + _oValue + ".  Scope: " + _scope);
        }
    }
    
    /// <summary>
    /// This method removes a variable from the underlying collection.
    /// </summary>
    /// <param name="_strCombinedKey">The key used to access the variable.</param>
    /// <param name="_scope">The scope used to store the variable. Assumed to be non-null.</param>
    protected virtual void InternalRemoveVariable(String _strCombinedKey,
     Scope _scope)
    {
      if (Scope.TAG == _scope)
      {
        if (!m_TagScopeData.ContainsKey(_strCombinedKey))
        {
          // TODO: check if m_nTagStackTop >= m_TagStack.GetSize() or -1?  (Both invalid)
          XPArrayList varList = (XPArrayList) m_TagVarStack.GetElement(m_nTagStackTop);
          if ((null == varList) || (!varList.Contains(_strCombinedKey)))
          {
          } else
          // Ignore - variable not present, so does not need to be removed.
          {
            // Remove the variable
            varList.Remove(_strCombinedKey);
          }
        }
        m_TagScopeData.RemoveElement(_strCombinedKey);
      } else
      if (Scope.PORTLET_REQUEST == _scope)
      {
        m_PortletRequestScopeData.RemoveElement(_strCombinedKey);
      } else
      if (Scope.HTTP_REQUEST == _scope)
      {
        m_HTTPRequestScopeData.RemoveElement(_strCombinedKey);
      } else
      if (Scope.SESSION == _scope)
      {
        m_Session.RemoveAttribute(_strCombinedKey);
      } else
      if (Scope.PERSISTENT_SESSION == _scope)
      {
        m_PersistentSession.RemoveAttribute(_strCombinedKey);
      } else
      if (Scope.APPLICATION == _scope)
      {
        m_Application.GetCachingManager().RemoveEntry(_strCombinedKey);
      } else
      {
        // Unknown scope constant
        throw new XPIllegalArgumentException("Unknown scope argument in tag " + GetTagName() + " while attempting to remove variable " + _strCombinedKey + ".  Scope: " + _scope);
        }
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ITagState#GetVariable(String,Scope)"></seealso>
    public virtual Object GetVariable(String _strKey, Scope _scope)
    {
      String strLibName = m_CurrentTag.GetTagMetaDataHelper().GetTagLibraryMetaData().GetName();
      return InternalGetVariable(strLibName, _strKey, _scope);
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ITagState#GetSharedVariable(String,Scope)"></seealso>
    public virtual Object GetSharedVariable(String _strKey, Scope _scope)
    {
      Object oSharedResult = InternalGetVariable(null, _strKey, _scope);
      SharedStateVariable sharedResult;
      try
      {
        sharedResult = (SharedStateVariable) oSharedResult;
      }
      catch (Exception e)
      {
        throw new XPException("Unable to convert value stored in memory to SharedStateVariable object.",
         e);
      }
      if (null == sharedResult)
      {
        return null;
      }
      return sharedResult.GetValue();
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ITagState#GetAnyVariableFromAllScopes(String,Scope)"></seealso>
    public virtual Object GetAnyVariable(String _strKey)
    {
      int nIndexOfDot = XPStringUtility.IndexOf(_strKey, '.');
      if (nIndexOfDot > 0)
      {
        String strObjectContainer = XPStringUtility.Substring(_strKey, 0,
         nIndexOfDot);
        String strRestOfVariable = XPStringUtility.Substring(_strKey, nIndexOfDot + 1);
        IDataObject dataObject = (IDataObject) InternalGetAnyVariable(strObjectContainer);
        if (dataObject != null)
        {
          return dataObject.GetData(strRestOfVariable);
        }
      } else
      {
        Object oResult = null;
        oResult = InternalGetAnyVariable(_strKey);
        if (null != oResult)
        {
          return oResult;
        }
      }
      return null;
    }
    
    /// <summary>
    /// Helper method to get variables in memory from any scope.
    /// </summary>
    /// <param name="_strKey">The key used to store the variable.</param>
    /// <returns>Object the requested variable</returns>
    private Object InternalGetAnyVariable(String _strKey)
    {
      Object oResult;
      Scope[] scopeSearchOrder = Scope.GetScopeSearchOrder();
      // Check variables
      for (int i = 0;i < scopeSearchOrder.Length;i++)
      {
        oResult = GetVariable(_strKey, scopeSearchOrder[i]);
        if (null != oResult)
        {
          return oResult;
        }
      }
      // Check shared variables
      for (int i = 0;i < scopeSearchOrder.Length;i++)
      {
        oResult = GetSharedVariable(_strKey, scopeSearchOrder[i]);
        if (null != oResult)
        {
          return oResult;
        }
      }
      return null;
    }
    
    /// <summary>
    /// Helper method to get variables in memory.
    /// </summary>
    /// <param name="_strLibrary">The current library.  May be null.</param>
    /// <param name="_strKey">The key used to store the variable.</param>
    /// <param name="_scope">The scope used to store the variable.</param>
    /// <returns>
    /// Object the requested variable (may be a SharedStateVariable object that
    /// needs to be processed before being returned to the tag.
    /// </returns>
    public virtual Object InternalGetVariable(String _strLibrary, String _strKey,
     Scope _scope)
    {
      CheckValidVariableName(_strKey);
      if (null == _strKey)
      {
        throw new XPIllegalArgumentException("Null key argument in tag " + GetTagName() + " while getting variable.  Scope: " + _scope);
        }
      if (null == _scope)
      {
        throw new XPIllegalArgumentException("Null scope argument in tag " + GetTagName() + " while getting variable " + _strKey + ".");
        }
      String strCombinedKey;
      if (null != _strLibrary)
      {
        strCombinedKey = _strLibrary + COMBINED_KEY_SEPARATOR + _strKey;
      } else
      {
        strCombinedKey = COMBINED_KEY_SEPARATOR + _strKey;
      }
      Object oResult;
      if (Scope.TAG == _scope)
      {
        oResult = m_TagScopeData.GetElement(strCombinedKey);
      } else
      if (Scope.PORTLET_REQUEST == _scope)
      {
        oResult = m_PortletRequestScopeData.GetElement(strCombinedKey);
      } else
      if (Scope.HTTP_REQUEST == _scope)
      {
        oResult = m_HTTPRequestScopeData.GetElement(strCombinedKey);
      } else
      if (Scope.SESSION == _scope)
      {
        oResult = m_Session.GetAttribute(strCombinedKey);
      } else
      if (Scope.PERSISTENT_SESSION == _scope)
      {
        oResult = m_PersistentSession.GetAttribute(strCombinedKey);
      } else
      if (Scope.APPLICATION == _scope)
      {
        oResult = m_Application.GetCachingManager().GetEntry(strCombinedKey);
        } else
      {
        // Unknown scope constant
        throw new XPIllegalArgumentException("Unknown scope argument in tag " + GetTagName() + " while getting variable " + _strKey + ".  Scope: " + _scope);
        }
      return oResult;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ITagState#GetApplication()"></seealso>
    public virtual IApplication GetApplication()
    {
      return m_Application;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ITagState#GetSubSession()"></seealso>
    public virtual ISessionManager GetSubSession()
    {
      return m_Session;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ITagState#GetPersistentSubSession()"></seealso>
    public virtual ISessionManager GetPersistentSubSession()
    {
      return m_PersistentSession;
    }
    
    /// <summary>
    /// The tag container calls this method every time a
    /// new tag is displayed.
    /// </summary>
    public virtual void EnterTag(ATag _currentTag)
    {
      m_CurrentTag = _currentTag;
      m_TagStack.Add(_currentTag);
      m_nTagStackTop++;
      m_TagVarStack.Add(null);
    }
    
    /// <summary>
    /// The tag container calls this method when a tag is
    /// finished displaying.  This will clean up data
    /// stored in the tag scope.
    /// </summary>
    public virtual void ExitTag()
    {
      XPArrayList varList = (XPArrayList) m_TagVarStack.GetElement(m_nTagStackTop);
      if (null != varList)
      {
        IXPEnumerator vars = varList.GetEnumerator();
        while (vars.MoveNext())
        {
          m_TagScopeData.RemoveElement(vars.GetCurrent());
        }
      }
      // Error checking
      if (m_nTagStackTop != (m_TagVarStack.GetSize() - 1))
      {
        log.Error("Invalid TagState while calling ExitTag().  TagStackTop " + m_nTagStackTop + " not equal to top of stack " + (m_TagVarStack.GetSize() - 1));
        // Reset to top of stack.
        m_nTagStackTop = m_TagVarStack.GetSize() - 1;
      }
      m_TagVarStack.RemoveAt(m_nTagStackTop);
      m_TagStack.RemoveAt(m_nTagStackTop);
      m_nTagStackTop--;
      // Restore the current tag variable to the previous tag
      if (m_nTagStackTop >= 0)
      {
        m_CurrentTag = (ATag) m_TagStack.GetElement(m_nTagStackTop);
      } else
      {
        m_CurrentTag = null;
      }
    }
    
    /// <summary>
    /// This method is called at the end of every HTTP request to
    /// reset the page request data.  This is a framework method
    /// and should not be called by tag developers.
    /// </summary>
    /// <param name="ISessionManager">the current session manager.</param>
    public static void ResetPageRequestData(AActivitySpace _aSpace)
    {
      // SE: Bug 61632 - Fix for Samsung. Backport of DonH's fix - CL 258931.
      // TODO: This method shouldn't be static.
      // TODO: Bug#61730 This method isn't really relevant anymore,
      // except possibly for unit tests. Maybe it should be removed.
      IXPRequest request = _aSpace.GetCurrentHTTPRequest();
      // This can be null if the page display operation times out.
      if (request != null)
      {
        // Remove the current request from the HTTP request data set
        request.SetAttribute(HTTP_REQUEST_DATA_KEY, null);
      }
    }
    
    /// <summary>
    /// The tag container calls this method to reset the
    /// portlet request data for each portlet on a page
    /// and to verify that the TagState is in the correct
    /// state after having finished displaying all of the
    /// tags in the current portlet request.
    /// </summary>
    public virtual void ResetPortletRequestData()
    {
      m_PortletRequestScopeData = new XPHashtable();
      // This hashtable should be empty.
      if (m_TagScopeData.GetSize() > 0)
      {
        log.Error("Tag Scope Data not empty at end of portlet request.");
        // Clear it if it is not empty
        m_TagScopeData.Clear();
      }
      if ((-1 != m_nTagStackTop) || (0 != m_TagStack.GetSize()) || (0 != m_TagVarStack.GetSize()))
      {
        log.Error("Tag Stack not empty at end of portlet request.");
        m_nTagStackTop = -1;
        m_TagStack.Clear();
        m_TagVarStack.Clear();
      }
      m_CurrentTag = null;
    }
    
    /// <summary>
    /// This method extracts the HTTP request data from the
    /// Activity Space.
    /// </summary>
    /// <param name="aSpace">The current AActivitySpace.</param>
    /// <returns>the current HTTP Request data object.</returns>
    public static XPHashtable GetHTTPRequestData(AActivitySpace aSpace)
    {
      // SE: Bug 61632 - Fix for Samsung. Backport of DonH's fix - CL 258931.
      // TODO: Bug#61730 This no longer requires the session manager.
      // Refactor this to remove it from here, and other places where it
      // can be removed, like the GetSharedJavascriptCollection method.
      // Get the HTTP Request Scope Data
      Object oRequestDataStorage = aSpace.GetCurrentHTTPRequest().GetAttribute(HTTP_REQUEST_DATA_KEY);
      if (null == oRequestDataStorage)
      {
        aSpace.GetCurrentHTTPRequest().SetAttribute(HTTP_REQUEST_DATA_KEY,
         new XPHashtable());
        return GetHTTPRequestData(aSpace);
      }
      return (XPHashtable) oRequestDataStorage;
    }
    
    /// <summary>
    /// This method gets the shared JavaScript collection for this HTTP request,
    /// and initializes it if it has not been initialized yet.
    /// </summary>
    /// <param name="asOwner">The current Activity Space.</param>
    /// <returns>The shared JavaScript collection for this HTTP request.</returns>
    public static HTMLElementCollection GetSharedJavascriptCollection(AActivitySpace asOwner)
    {
      XPHashtable HTTPRequestData = TagState.GetHTTPRequestData(asOwner);
      // Check and set if SharedJSCollection has been called before
      // NOTE: What happens if a custom tag overwrites this variable or someone stores
      // some random request data here using the variable tag?
      Object oSharedJSCollection = HTTPRequestData.GetElement(TagState.SHARED_JS_COLLECTION);
      if (null == oSharedJSCollection)
      {
        HTTPRequestData.PutElement(TagState.SHARED_JS_COLLECTION, new HTMLElementCollection());
        return GetSharedJavascriptCollection(asOwner);
      }
      return (HTMLElementCollection) oSharedJSCollection;
    }
    
    /// <summary>
    /// This method marks the shared JavaScript collection for this HTTP request as
    /// retrieved for display.
    /// </summary>
    /// <param name="asOwner">The current Activity Space.</param>
    /// <throws>XPException if this method is called more than once in an HTTP request.</throws>
    public static void SetSharedJavascriptCollectionAsDisplayed(AActivitySpace asOwner)
    {
      XPHashtable HTTPRequestData = TagState.GetHTTPRequestData(asOwner);
      // Check and set if SharedJSCollection has been called before
      // NOTE: What happens if a custom tag overwrites this variable or someone stores
      // some random request data here using the variable tag?
      Object oSharedJSCollectionDisplayed = HTTPRequestData.GetElement(TagState.SHARED_JS_COLLECTION_DISPLAYED);
      if (null != oSharedJSCollectionDisplayed)
      {
        throw new XPException("Tag Markup Shared JavaScript Collection already marked for display.  " + "The collection should only be displayed once per HTTP request.");
        }
      HTTPRequestData.PutElement(TagState.SHARED_JS_COLLECTION, true);
      return;
    }
    
    /// <summary>
    /// Get the name of the current tag.
    /// </summary>
    /// <returns>String the name of the current tag</returns>
    protected virtual String GetTagName()
    {
      if (null != m_CurrentTag)
      {
        return m_CurrentTag.GetName();
      }
      return "";
    }
  }
}