using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.resultwrapper;
using com.plumtree.portaluiinfrastructure.search;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.tree;
namespace com.plumtree.portaluiinfrastructure.tree
{
  /// <author>
  /// Don Hayler
  /// This model browses the directory catalog folders and cards.
  /// Originally written by OlegS.
  /// Usage example in SmartSortEditorMainForm. -- olegs
  /// This model handles generating a tree for document folders.
  /// </author>
  public class DirectoryTreeModel : AObjectTreeModel
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.tree.DirectoryTreeModel");
    
    /// <summary>
    /// Browsing Tree object name 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "BrowsingTree";
    
    /// <summary>
    /// Browse Mode 
    /// </summary>
    public const int BROWSE_MODE = 1;
    
    /// <summary>
    /// Edit Mode 
    /// </summary>
    public const int EDIT_MODE = 2;
    
    /// <summary>
    /// Is Edit Directory Mode? 
    /// </summary>
    private bool m_bEditMode;
    // Plumtree session
    protected IPTSession m_ptSession;
    //    private Object[][] m_ptqfCards = null;
    protected int[] m_nDisplayedClassIDs;
    // Browsing specific member variables
    private IPTCatalog m_ptCatalog = null;
    private int[] m_nHiddenCardIDs = null;
    private bool m_bShowCards = false;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new DirectoryTreeModel();
    }
    
    /// <summary>
    /// Expand or collapse a node in the tree
    /// </summary>
    /// <param name="bExpand"></param>
    /// <param name="nNodeID"></param>
    public override void ExpandNode(bool bExpand, int nNodeID)
    {
      base.ExpandNode(bExpand, nNodeID);
      TreeCompositeNode node = (TreeCompositeNode) m_Root.GetNode(nNodeID);
      if (node == null)
      {
        log.Error("Could not find node ID " + nNodeID);
        return;
      }
      node.m_bExpanded = bExpand;
      if ((node.m_bChildrenLoaded == false) && bExpand)
      {
        IPTFolder nodeFolder = null;
        ASQueryResultWrapper query = null;
        // We need to do a special query for selectable folders if we
        // are selecting folders and we have a special folder select query filter
        // Note: directory does not allow query filters unless the level is edit (jhq-actually, it's select) or above.
        bool bUseFolderSelectQuery = ((m_ptFolderSelectqf != null) && (m_nAccessLevel >= PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT) && ((m_nSelectMode == TreeConstants.N_COMPOSITESELECT) || (m_nSelectMode == TreeConstants.N_LEAFCOMPOSITESELECT)));
        // 0 means just this level.
        int nLevel = 0;
        try
        {
          // 0 = false = open object unlocked
          nodeFolder = m_ptCatalog.OpenFolder(node.m_nObjectID, false);
          // Use a null query filter because we always want to display all subfolders,
          // even if they're not selectable
          query = new ASQueryResultWrapper(nodeFolder.QuerySubfolders(PT_PROPIDS.PT_PROPID_OBJECTID | PT_PROPIDS.PT_PROPID_NAME | PT_PROPIDS.PT_PROPID_DESCRIPTION,
           nLevel, PT_PROPIDS.PT_PROPID_NAME, 0, -1, null));
        }
        catch (Exception e)
        {
          log.Error(e, "Failed getting subfolders of folder # " + nNodeID + ".");
          return;
        }
        // Add composite node children
        TreeCompositeNode myFolder;
        int nCount = query.GetCount();
        int nFolderID;
        for (int i = 0;i < nCount;i++)
        {
          nFolderID = XPConvert.ToInteger(query.GetFields(i, PT_PROPIDS.PT_PROPID_OBJECTID));
          myFolder = GetCompositeIfAlreadySelected(PT_CLASSIDS.PT_CATALOGFOLDER_ID + "," + nFolderID);
          if (myFolder == null)
          {
            myFolder = new TreeCompositeNode(GetNextUniqueID());
          }
          myFolder.SetName(XPConvert.ToString(query.GetFields(i, PT_PROPIDS.PT_PROPID_NAME)));
          myFolder.SetDescription(XPConvert.ToString(query.GetFields(i, PT_PROPIDS.PT_PROPID_DESCRIPTION)));
          myFolder.m_nObjectID = nFolderID;
          myFolder.m_nClassID = PT_CLASSIDS.PT_CATALOGFOLDER_ID;
          // This should use the GetName because it should be unencoded
          // when the end user of the tree displays it, and the tree
          // never displays it directly.
          myFolder.m_strLocation = node.m_strLocation + "\\" + myFolder.GetName();
          if (bUseFolderSelectQuery)
          {
            // We will set selectable folders below
            myFolder.m_bBrowsableOnly = true;
          } else
          {
            // All folders are browsable and selectable
            // (if folders are selectable)
            myFolder.m_bBrowsableOnly = false;
          }
          SetCompositeImage(myFolder, m_asOwner);
          node.AddCompositeChild(myFolder);
          log.Info("In node " + nNodeID + ", loaded composite child " + myFolder.GetEncodedName());
          }
        if (bUseFolderSelectQuery)
        {
          // Get selectable folders
          // NOTE: The previous query should really return the security levels so
          // I can selectively set browsing only folders but
          // PTAdminFolder.QuerySubfolders() doesn't allow you to add
          // the access level to the query for some reason.
          try
          {
            // 0 = false = open object unlocked
            query = new ASQueryResultWrapper(nodeFolder.QuerySubfolders(PT_PROPIDS.PT_PROPID_OBJECTID | PT_PROPIDS.PT_PROPID_NAME | PT_PROPIDS.PT_PROPID_DESCRIPTION,
             nLevel, PT_PROPIDS.PT_PROPID_NAME, 0, -1, m_ptFolderSelectqf));
            }
          catch (Exception e)
          {
            log.Error(e, "Failed getting selectable subfolders of folder # " + nNodeID);
            return;
          }
          nCount = query.GetCount();
          // set selectable composite node children
          for (int i = 0;i < nCount;i++)
          {
            nFolderID = XPConvert.ToInteger(query.GetFields(i, PT_PROPIDS.PT_PROPID_OBJECTID));
            myFolder = (TreeCompositeNode) node.GetNode(PT_CLASSIDS.PT_CATALOGFOLDER_ID,
             nFolderID);
            if (myFolder != null)
            {
              myFolder.m_bBrowsableOnly = false;
            }
            log.Debug("In node " + nNodeID + ", set selectable composite child " + myFolder.GetEncodedName());
            }
        }
        if (m_bShowCards)
        {
          // Add leaf node children
          IPTFolder ptFolder = (node.m_nObjectID < 0) ? m_ptSession.GetCatalog().GetRootFolder() : m_ptSession.GetCatalog().OpenFolder(node.m_nObjectID,
           false);
          ASQueryResultWrapper queryCards = new ASQueryResultWrapper(ptFolder.QueryCards(PT_PROPIDS.PT_PROPID_ALL,
           m_bEditMode, null, 0, -1, null));
          if (queryCards.GetCount() > 0)
          {
            TreeLeafNode myLeaf;
            int nCardID;
            nCount = queryCards.GetCount();
            for (int i = 0;i < nCount;i++)
            {
              nCardID = XPConvert.ToInteger(queryCards.GetFields(i, PT_PROPIDS.PT_PROPID_OBJECTID));
              if (!HiddenCard(nCardID))
              {
                myLeaf = GetLeafIfAlreadySelected(PT_CLASSIDS.PT_CATALOGCARD_ID + "," + nCardID);
                if (myLeaf == null)
                {
                  myLeaf = new TreeLeafNode(GetNextUniqueID());
                }
                myLeaf.SetName(XPConvert.ToString(queryCards.GetFields(i,
                 PT_PROPIDS.PT_PROPID_NAME)));
                myLeaf.SetDescription(XPConvert.ToString(queryCards.GetFields(i,
                 PT_PROPIDS.PT_PROPID_DESCRIPTION)));
                myLeaf.m_nObjectID = nCardID;
                myLeaf.m_nClassID = PT_CLASSIDS.PT_CATALOGCARD_ID;
                SetLeafImage(myLeaf, m_asOwner);
                node.AddLeafChild(myLeaf);
                log.Info("In node " + nNodeID + ", loaded leaf child " + myLeaf.GetEncodedName());
                }
            }
          } else
          {
            log.Error("Unable to query for cards in folder " + node.m_nObjectID);
            }
        }
        node.m_bChildrenLoaded = true;
        log.Info("Loaded children of folder # " + nNodeID);
      }
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tree.IObjectTreeModel#SelectNode(int,boolean)"></seealso>
    public override void SelectNode(int nID, bool bSelect)
    {
      // The super implementation does not add the location to the composite nodes,
      // so we have to do that later in certain cases.
      // If this method is being called from a search page, the search items will
      // get added to the tree for the first time by the super implementation.
      // Since the super implementation doesn't add the location correctly (because
      // it doesn't know about document directory protocols), we need to add it here.
      base.SelectNode(nID, bSelect);
      TreeLeafNode selectedNode = null;
      // Find the node and add the correct location to it.
      // This is only necessary if we are selecting a search node for the first time.
      if (m_bDisplayingSearchResults && bSelect)
      {
        SearchResultModel searchModel = (SearchResultModel) m_asOwner.GetModel(SearchResultModel.STR_MVC_CLASS_NAME);
        if (nID != -1)
        {
          int nClassID = searchModel.GetResultClassID(nID);
          int nObjectID = searchModel.GetResultObjectID(nID);
          selectedNode = (TreeLeafNode) m_xpmSelectedItems.GetElement(nClassID + "," + nObjectID);
          // Only do this for folders that don't already have the location set
          if ((null != selectedNode) && (nClassID == PT_CLASSIDS.PT_CATALOGFOLDER_ID) && (null == selectedNode.m_strLocation))
          {
            try
            {
              // 0 = false = open object unlocked
              IPTFolder nodeFolder = m_ptCatalog.OpenFolder(nObjectID, false);
              // The path of the folder, including it's name.
              selectedNode.m_strLocation = nodeFolder.GetPath();
            }
            catch (Exception e)
            {
              log.Error(e, "Unable to determine path of document folder " + nObjectID);
              }
          }
        }
      }
    }
    //////////////////////////////////////
    // Read Methods for IObjectTreeModelRO
    //////////////////////////////////////
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tree.IObjectTreeModelRO#GetDisplayedClassIDs()"></seealso>
    public override int[] GetDisplayedClassIDs()
    {
      return m_nDisplayedClassIDs;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tree.IObjectTreeModelRO#GetFolderClassID()"></seealso>
    public override int GetFolderClassID()
    {
      return PT_CLASSIDS.PT_CATALOGFOLDER_ID;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tree.IObjectTreeModelRO#GetItemDisplayed(int,int)"></seealso>
    public override bool GetItemDisplayed(int nClassID, int nObjectID)
    {
      if (nClassID == PT_CLASSIDS.PT_CATALOGCARD_ID)
      {
        return !HiddenCard(nObjectID);
      }
      return true;
    }
    ///////////////////////////
    // RO Methods from IModelRO
    ///////////////////////////
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    /////////////////
    // Helper methods
    /////////////////
    public virtual bool HiddenCard(int nID)
    {
      for (int i = 0;(i + 1) < m_nHiddenCardIDs.Length;i+=2)
      {
        if ((m_nHiddenCardIDs[i] == PT_CLASSIDS.PT_CATALOGCARD_ID) && (nID == m_nHiddenCardIDs[i + 1]))
        {
          return true;
        }
      }
      return false;
    }
    ////////////////////////////
    // Write Methods from IModel
    ////////////////////////////
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IModel#Init(AActivitySpace)"></seealso>
    public override void Init(AActivitySpace parent)
    {
      base.Init(parent);
      m_ptSession = (IPTSession) m_asOwner.GetUserSession();
      if (m_ptSession == null)
      {
        log.Error("Failed getting IPTSession.");
      }
      m_ptCatalog = m_ptSession.GetCatalog();
      if (m_ptCatalog == null)
      {
        log.Error("Failed getting catalog.");
      }
      m_bEditMode = true;
    }
    /////////////////////////////////////
    // Write Methods for IObjectTreeModel
    /////////////////////////////////////
    public override void ResetTree()
    {
      base.ResetTree();
      m_bShowCards = false;
      m_nHiddenCardIDs = new int[0];
      //		m_ptqfCards = null;
      m_nDisplayedClassIDs = new int[0];
    }
    ////////////////////////////////
    // Write Methods from ITreeModel
    ////////////////////////////////
    
    /// <param name="nIDs"></param>
    public override void SetClassIDsToDisplay(int[] nIDs)
    {
      for (int i = 0;i < nIDs.Length;i++)
      {
        if (nIDs[i] == PT_CLASSIDS.PT_CATALOGCARD_ID)
        {
          m_bShowCards = true;
        } else
        // We have to allow display of catalog folders because that's how
        // directory vs. admin folders is set.
        if (nIDs[i] != PT_CLASSIDS.PT_CATALOGFOLDER_ID)
        {
          log.Warn("Attempting to display unsupported class: " + nIDs[i]);
          }
      }
      if (m_bShowCards)
      {
        m_nDisplayedClassIDs = new int[1];
        m_nDisplayedClassIDs[0] = PT_CLASSIDS.PT_CATALOGCARD_ID;
      }
    }
    public virtual void SetDirectoryMode(int _nDirMode)
    {
      if (BROWSE_MODE == _nDirMode)
      {
        m_bEditMode = false;
      } else
      {
        m_bEditMode = true;
      }
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.tree.ITreeModel#SetObjectIDsToDisplay(int[])"></seealso>
    public override void SetObjectIDsToDisplay(int[] nIDs)
    {
    }
    // Not currently necessary
    
    /// <seealso cref="com.plumtree.uiinfrastructure.tree.ITreeModel#SetObjectIDsToHide(int[])"></seealso>
    public override void SetObjectIDsToHide(int[] nIDs)
    {
      m_nHiddenCardIDs = nIDs;
    }
    
    /// <param name="nIDs"></param>
    public override void SetObjectIDsToSelect(int[] nIDs)
    {
      if (m_Root != null)
      {
        // we are refreshing selected items, so we clear all old items.
        // clear all selected items
        m_Root.ClearSelected(false, true);
        m_xpmSelectedItems.Clear();
      }
      TreeLeafNode node;
      IPTFolder folder;
      for (int i = 0;(i + 1) < nIDs.Length;i+=2)
      {
        if (nIDs[i] == PT_CLASSIDS.PT_CATALOGFOLDER_ID)
        {
          node = null;
          if (m_Root != null)
          {
            // since we are refreshing, the folder may already exist
            // Check if the folder is already in the tree
            node = m_Root.GetNode(nIDs[i], nIDs[i + 1]);
          }
          if (node == null)
          {
            // 0 = false = open object unlocked
            folder = m_ptCatalog.OpenFolder(nIDs[i + 1], false);
            if (folder != null)
            {
              node = new TreeCompositeNode(-1);
              node.m_nClassID = PT_CLASSIDS.PT_CATALOGFOLDER_ID;
              node.m_nObjectID = nIDs[i + 1];
              node.SetName(folder.GetLocalizedName());
              node.SetDescription(folder.GetLocalizedDescription());
              SetCompositeImage((TreeCompositeNode) node, m_asOwner);
            } else
            {
              log.Error("Unable to open directory folder: " + nIDs[i + 1]);
              }
          }
          if (node != null)
          {
            node.m_bSelected = true;
            m_xpmSelectedItems.PutElement(GetKeyFromNode(node), node);
          }
        } else
        if (nIDs[i] == PT_CLASSIDS.PT_CATALOGCARD_ID)
        {
          node = null;
          if (m_Root != null)
          {
            // Since we are refreshing, the object may already exist
            // Check if the object is already in the tree
            node = m_Root.GetNode(nIDs[i], nIDs[i + 1]);
          }
          if (node == null)
          {
            IPTObject objDocument = m_ptCatalog.OpenCard(nIDs[i + 1], false);
            if (objDocument != null)
            {
              node = new TreeLeafNode(-1);
              node.m_nClassID = PT_CLASSIDS.PT_CATALOGCARD_ID;
              node.m_nObjectID = nIDs[i + 1];
              node.SetName(objDocument.GetLocalizedName());
              node.SetDescription(objDocument.GetLocalizedDescription());
              SetLeafImage(node, m_asOwner);
            }
          }
          if (node != null)
          {
            node.m_bSelected = true;
            m_xpmSelectedItems.PutElement(GetKeyFromNode(node), node);
          } else
          {
            log.Error("Unable to find document (ObjectID " + nIDs[i + 1] + ").");
            }
        }
      }
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tree.IObjectTreeModel#SetQueryFilter(int,int,int[])">This must be called after SetAccessLevel() and SetClassIDsToDisplay().</seealso>
    public override void SetQueryFilter(int nClassID, int nPropID, int[] nValues)
    {
      log.Error("Attempting to set query filter on class other than gadgets: " + nClassID);
      return;
    }
    /*
     if (nClassID != PT_CLASSIDS.PT_CATALOGCARD_ID)
     {
     if ( log.IsErrorEnabled() )
     {
     log.Error("Attempting to set query filter on class other than catalogcards: " + nClassID);
     }
     return;
     }
     
     m_ptqfCards = new Object[3][2];
     
     // Copy the security filter over
     m_ptqfCards[0][0] = m_ptqf[0][0];
     m_ptqfCards[1][0] = m_ptqf[1][0];
     m_ptqfCards[2][0] = m_ptqf[2][0];
     
     // Add new query filter
     m_ptqfCards[0][1] = nPropID;
     m_ptqfCards[1][1] = new Integer(PT_FILTEROPS.PT_FILTEROP_IN);
     m_ptqfCards[2][1] = nValues;
     */
     
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tree.IObjectTreeModel#SetQueryFilter(int,int,String[])">This must be called after SetAccessLevel() and SetClassIDsToDisplay().</seealso>
    public override void SetQueryFilter(int nClassID, int nPropID, String[] sValues)
    {
      log.Error("Attempting to set query filter on class other than gadgets: " + nClassID);
      return;
    }
    
    /// <param name="nMode"></param>
    /// <param name="nFolderID"></param>
    public override void SetCommunityMode(int nMode, int nFolderID)
    {
    }
    // Not used in directory
    
    /// <param name="nID"></param>
    public override void SetRootFolderID(int nID)
    {
      // 0 = false = open object unlocked
      IPTFolder root = m_ptCatalog.OpenFolder(nID, false);
      m_Root = GetCompositeIfAlreadySelected(PT_CLASSIDS.PT_CATALOGFOLDER_ID + "," + nID);
      if (null == m_Root)
      {
        m_Root = new TreeCompositeNode(GetNextUniqueID());
      }
      m_Root.SetName(root.GetLocalizedName());
      m_Root.SetDescription(root.GetLocalizedDescription());
      m_Root.m_nClassID = PT_CLASSIDS.PT_CATALOGFOLDER_ID;
      m_Root.m_nObjectID = nID;
      m_Root.m_strLocation = root.GetPath();
      SetCompositeImage(m_Root, m_asOwner);
      // If we are displaying folders, and we are using access levels,
      // we may need to display this folder as read only (not selectable).
      if ((m_ptFolderSelectqf != null) && (m_nAccessLevel > PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT) && ((m_nSelectMode == TreeConstants.N_COMPOSITESELECT) || (m_nSelectMode == TreeConstants.N_LEAFCOMPOSITESELECT)))
      {
        if (!m_ptCatalog.IsCatalogObjectAccessible(PT_CLASSIDS.PT_CATALOGFOLDER_ID,
         nID, m_nAccessLevel))
        {
          // Make the root node non-selectable
          m_Root.m_bBrowsableOnly = true;
        }
      }
      if (!m_bShowRoot)
      {
        ExpandNode(true, m_Root.m_nTreeNodeID);
      }
      // Allow pre-expanded multi-selects in the directory tree.
      // Could be too expensive if used poorly.
      ExpandToSelectedItem();
    }
    
    /// <summary>
    /// This method expands the tree to display the
    /// pre-selected item.
    /// NOTE: expanding too many items could cause performance
    /// issues.
    /// </summary>
    protected virtual void ExpandToSelectedItem()
    {
      IXPEnumerator tempEnum = m_xpmSelectedItems.GetElements();
      if (tempEnum.MoveNext())
      {
        TreeLeafNode selectedItem = (TreeLeafNode) tempEnum.GetCurrent();
        int nClassID = selectedItem.m_nClassID;
        int nObjectID = selectedItem.m_nObjectID;
        int nParentFolderID;
        // Get the ID of the parent folder
        if (nClassID == PT_CLASSIDS.PT_CATALOGFOLDER_ID)
        {
          /* root folder, no parent */
          if (PT_INTRINSICS.PT_FOLDER_ROOTFOLDER == nObjectID)
          {
            return;
          }
          // 0 = false = open object unlocked
          IPTFolder selectedFolder = m_ptCatalog.OpenFolder(nObjectID, false);
          nParentFolderID = selectedFolder.GetParentFolderID();
        } else
        if (nClassID == PT_CLASSIDS.PT_CATALOGCARD_ID)
        {
          // 0 = false = open object unlocked
          IPTQueryResult qrParentFolders = m_ptCatalog.QueryCardPaths(nObjectID,
           false, false);
          // DCA - just try to get the first one
          if (qrParentFolders.RowCount() > 0)
          {
            nParentFolderID = qrParentFolders.ItemAsInt(0, PT_PROPIDS.PT_PROPID_CARD_PARENTFOLDERID);
            } else
          {
            // set parent id to root document folder
            nParentFolderID = PT_INTRINSICS.PT_FOLDER_ROOTFOLDER;
          }
        } else
        {
          // We don't know how to handle the objects, so skip them.
          return;
        }
        // 0 = false = open object unlocked
        IPTFolder parentFolder = m_ptCatalog.OpenFolder(nParentFolderID, false);
        int[] nIDs;
        // GetPath() does not return the extra 0 folder at the beginning (like
        // IPTAdminFolder.GetParentIDs(), so we don't need to check for it.
        try
        {
          IPTQueryResult qrFolderPath = parentFolder.GetPathAsQueryResult();
          nIDs = new int[qrFolderPath.RowCount()];
          for (int i = 0;i < qrFolderPath.RowCount();i++)
          {
            nIDs[i] = qrFolderPath.ItemAsInt(i, PT_PROPIDS.PT_PROPID_OBJECTID);
            }
        }
        catch (Exception e)
        {
          log.Error(e, "Pre-expand selected item could not get path.");
          return;
        }
        int nTreeNodeID;
        // Open the folders up to the parent folder
        for (int i = 0;i < nIDs.Length;i++)
        {
          TreeLeafNode node = m_Root.GetNode(PT_CLASSIDS.PT_CATALOGFOLDER_ID,
           nIDs[i]);
          if (node != null)
          {
            ExpandNode(true, node.m_nTreeNodeID);
          }
        }
      }
    }
    
    /// <summary>
    /// This method calculates the read only query filter.
    /// </summary>
    /// <param name="int">the access level required by the tree</param>
    /// <returns>the qf.</returns>
    protected override Object[][] CalculateFolderSelectQueryFilter(int nAccessLevel)
    {
      // document folders require edit (jhq-actually, it's select) or greater access, and use of the GTE
      // operator
      if (nAccessLevel >= PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT)
      {
        return PlumtreeHelpers.GetQueryFilter(PT_PROPIDS.PT_PROPID_ACCESSLEVEL,
         PT_FILTEROPS.PT_FILTEROP_GTE, nAccessLevel);
      }
      return null;
    }
  }
}