package com.plumtree.portaluiinfrastructure.compoundlist;

import com.plumtree.openfoundation.util.*;
import com.plumtree.xpshared.htmlconstructs.*;

/**
 * List to store different types of URLs, primarily URLs to various
 * portal areas.
 * 
 * @author robertz
 */
public class ASCompoundList implements ICPListEntryIterator
{
	/**
	 * Constant values of the different types of URLs the list stores.
	 */
	public final static int ENTRYTYPE_STRINGLABEL  = 100;
	public final static int ENTRYTYPE_FULLURL	   = 101;
	public final static int ENTRYTYPE_COMMPTEMPL   = 102;
	public final static int ENTRYTYPE_MYPAGETEMPL  = 103;
	public final static int ENTRYTYPE_DIRTEMPL     = 104;
	public final static int ENTRYTYPE_EDITORTEMPL  = 105;
	public final static int ENTRYTYPE_PREFSTEMPL   = 106;
	public final static int ENTRYTYPE_GENERICTEMPL = 107;
	public final static int ENTRYTYPE_DOCTEMPL  = 108;
	public final static int ENTRYTYPE_USERTEMPL = 109;
	public final static int ENTRYTYPE_SITEMAPROOTTEMPL = 110;
	
	public final static int ENTRYTYPE_FULLINTERNALURL = 111;
	/**
	 * Constant values of the different types of getting
	 * the Activity Space.
	 */
	public final static int NEWASTYPE_CREATENEW   = 0;
	public final static int NEWASTYPE_GETCACHED   = 1;
	public final static int NEWASTYPE_GETIFCACHED = 2;

	private ListEntry m_curEntry;
	private XPArrayList m_list;

	/* Iterator variables */
	private int m_curListItemIndex;
	private int m_nTotalCount;

	/**
	 * Creates a new ASCompoundList object.
	 */
	public ASCompoundList()
	{
		m_nTotalCount     = 0;
		m_list			  = new XPArrayList();
		m_curEntry		  = null;
	}

	/**
	 * Adds a Community page URL
	 *
	 * @param label 	String to display
	 * @param nCommID 	Community ID
	 * @param nPageID 	Community Page ID
	 */
	public void AddCommPageURL(String label, int nCommID, int nPageID)
	{
		if (nPageID < 0)
			nPageID = 0;
			
		AddCommPageURL(label, nCommID, nPageID, "", -1);
	}

	/**
	 * Adds a User Profile Community Page URL 
	 *
	 * @param label 	String to display
	 * @param nCommID 	Community ID
	 * @param nPageID 	Community Page ID
	 * @param strDescription Description for the user profile
	 * @param nProfileID Profile User ID
	 */
	public void AddCommPageURL(String label, int nCommID, int nPageID, String strDescription, int nProfileID)
	{
		ListEntry entry = new ListEntry(ENTRYTYPE_COMMPTEMPL);
		entry.SetLabel(label);
		entry.SetCommunityID(nCommID);
		entry.SetDescription(strDescription);
		entry.SetPageID(nPageID);
		entry.SetProfileID(nProfileID);
		m_list.Add(entry);
		m_nTotalCount++;
	}

	/**
	 * Adds a Community Preview URL
	 *
	 * @param label 	String to display
	 * @param nCommID 	Community ID
	 * @param nPageID 	Communiyt Page ID
	 * @param strDescription Description for the Community
	 */
	public void AddCommPreviewPageURL(String label, int nCommID, int nPageID,
		String strDescription)
	{
		ListEntry entry = new ListEntry(ENTRYTYPE_COMMPTEMPL);
		String [] saParams = new String[1];
		String [] values = new String[1];
		entry.SetLabel(label);
		entry.SetCommunityID(nCommID);
		entry.SetPageID(nPageID);
		entry.SetDescription(strDescription);
		saParams[0] = "preview";
		values[0] = "y";
		entry.SetExtraParams(saParams, values);
		m_list.Add(entry);
		m_nTotalCount++;
	}
	
	/**
	 * Adds a Community Knowledge Directory Root URL
	 * 
	 * @param label 		String to display
	 * @param nCommID 		Community ID
	 * @param nMode 		Browse or Edit Mode
	 * @param bIsPreview	Is this page in preview mode
	 */
	public void AddSiteMapRootURL(String label, int nCommID, int nMode, boolean bIsPreview)
	{
		ListEntry entry = new ListEntry(ENTRYTYPE_SITEMAPROOTTEMPL);
		entry.SetLabel(label);
		entry.SetCommunityID(nCommID);
		entry.SetMode(nMode);
		entry.SetIsPreview(bIsPreview);
		m_list.Add(entry);
		m_nTotalCount++;	
	}
	
	/**
	 * Adds the content of another ASCompoundList
	 *
	 * @param list 		ASCompoundList, does nothing if null
	 */
	public void AddCompoundList(ICPListEntryIterator list)
	{
		if (null != list)
		{
			list.GetIterator();

			while (list.Next())
			{
				m_list.Add(list.GetEntry());
				m_nTotalCount++;
			}
		}
	}

	/**
	 * Adds a Directory URL
	 *
	 * @param label 		String to display
	 * @param nMode 		Browse or Edit Mode
	 * @param nFolderID 	ID of Folder to display
	 */
	public void AddDirectoryURL(String label, int nMode, int nFolderID)
	{
		AddDirectoryURL(label, nMode, nFolderID, "");
	}

	/**
	 * Adds a Directory URL with description
	 *
	 * @param label 		String to display
	 * @param nMode 		Browse or Edit Mode
	 * @param nFolderID 	ID of Folder to display
	 * @param strDescription Description of folder
	 */
	public void AddDirectoryURL(String label, int nMode, int nFolderID,
		String strDescription)
	{
		ListEntry entry = new ListEntry(ENTRYTYPE_DIRTEMPL);
		entry.SetLabel(label);
		entry.SetMode(nMode);
		entry.SetFolderID(nFolderID);
		entry.SetDescription(strDescription);
		m_list.Add(entry);
		m_nTotalCount++;
	}

	/**
	 * Adds an URL to a directory document
	 *
	 * @param label 			String to display	
	 * @param nDocID 			ID of document
	 */
	public void AddDocumentURL(String label, int nDocID)
	{
		AddDocumentURL(label, nDocID, "");
	}

	/**
	 * Adds an URL to a directory document with description
	 *
	 * @param label 			String to display
	 * @param nDocID 			ID of document
	 * @param strDescription 	Document description 
	 */
	public void AddDocumentURL(String label, int nDocID, String strDescription)
	{
		ListEntry entry = new ListEntry(ENTRYTYPE_DOCTEMPL);
		entry.SetLabel(label);
		entry.SetPageID(nDocID);
		entry.SetDescription(strDescription);
		m_list.Add(entry);
		m_nTotalCount++;
	}

	/**
	 * Adds an URL to an editor
	 *
	 * @param label 			String to display
	 * @param editor 			Editor Activityspace ID
	 * @param control 			Editor Control ID
	 * @param nEditorType 		Editor type constant
	 */
	public void AddEditorPageURL(String label, String editor, String control,
		int nEditorType)
	{
		ListEntry entry = new ListEntry(ENTRYTYPE_EDITORTEMPL);
		entry.SetLabel(label);
		entry.SetControl(control);
		entry.SetEditorAS(editor);
		entry.SetEditorType(nEditorType);
		m_list.Add(entry);
		m_nTotalCount++;
	}



	/**
	 * Adds an URL to an editor with additional parameters.
	 *
	 * @param label 			String to display
	 * @param editor 			Editor Activityspace ID
	 * @param control 			Editor Control ID
	 * @param page 				Editor Page ID
	 * @param paramNames 		String array of parameter names, each name element must
	 * 							have a corresponding value element in paramValues
	 * @param paramValues 		String array of parameter values, each value element must
	 * 							have a corresponding name element in paramNames
	 */
	public void AddEditorPageURL(String label, String editor, String control,
		String page, String[] paramNames, String[] paramValues)
	{
		ListEntry entry = new ListEntry(ENTRYTYPE_EDITORTEMPL);
		entry.SetLabel(label);
		entry.SetEditorAS(editor);
		entry.SetControl(control);
		entry.SetPage(page);
		entry.SetExtraParams(paramNames, paramValues);
		m_list.Add(entry);
		m_nTotalCount++;
	}

	/**
	 * 
	 * Adds a full URL for a portal page
	 * Use this URL type instead of AddFullURL for portal pages since ENTRYTYPE_FULLURL 
	 * URL type will open in a popup if displayoptions specify external URL to open in a pop.  
	 * 
	 * @param label
	 * @param url
	 */
	public void AddInternalFullURL(String label, String url)
	{
		ListEntry entry = new ListEntry(ENTRYTYPE_FULLINTERNALURL);
		entry.SetLabel(label);
		entry.SetFullURL(url);
		
		m_list.Add(entry);
		m_nTotalCount++;
	}
	
	
	/**
	 * Adds an external URL
	 *
	 * @param label 			String to display
	 * @param url 				The full url to an external site (including http://)
	 */
	public void AddFullURL(String label, String url)
	{
		AddFullURL(label, url, "");
	}

	/**
	 * Adds an external URL with description
	 *
	 * @param label 			String to display
	 * @param url 				The full url to an external site (including http://)
	 * @param strDescription 	Description of external URL
	 */
	public void AddFullURL(String label, String url, String strDescription)
	{
		ListEntry entry = new ListEntry(ENTRYTYPE_FULLURL);
		entry.SetLabel(label);
		entry.SetFullURL(url);
		entry.SetDescription(strDescription);
		m_list.Add(entry);
		m_nTotalCount++;
	}

	/**
	 * Adds a generic portal URL
	 *
	 * @param label 			String to display	
	 * @param strAS 			Activityspace ID
	 * @param paramNames 		String array of parameter names, each name element must
	 * 							have a corresponding value element in paramValues
	 * @param paramValues 		String array of parameter values, each value element must
	 * 							have a corresponding name element in paramNames
	 * @param nNewASType		How to retrieve the activityspace, use NEWSASTYPE_ constant 
	 */
	public void AddGenericURL(String label, String strAS, String[] paramNames,
		String[] paramValues, int nNewASType)
	{
		ListEntry entry = new ListEntry(ENTRYTYPE_GENERICTEMPL);
		entry.SetLabel(label);
		entry.SetEditorAS(strAS);
		entry.SetExtraParams(paramNames, paramValues);
		entry.SetNewASType(nNewASType);
		m_list.Add(entry);
		m_nTotalCount++;
	}

	/**
	 * Adds just a label
	 *
	 * @param label 			String to display
	 */
	public void AddLabel(String label)
	{
		ListEntry entry = new ListEntry(ENTRYTYPE_STRINGLABEL);
		entry.SetLabel(label);
		m_list.Add(entry);
		m_nTotalCount++;
	}

	/**
	 * Adds a Mypage URL
	 *
	 * @param label 			String to display
	 * @param nPageID 			Mypage ID
	 */
	public void AddMyPageURL(String label, int nPageID)
	{
		ListEntry entry = new ListEntry(ENTRYTYPE_MYPAGETEMPL);
		entry.SetLabel(label);
		entry.SetPageID(nPageID);
		m_list.Add(entry);
		m_nTotalCount++;
	}

	/**
	 * Adds a User Preference Page URL 
	 *
	 * @param label 			String to display
	 * @param strPrefsPage 		Preference Page ID
	 */
	public void AddPreferencesURL(String label, String strPrefsPage)
	{
		ListEntry entry = new ListEntry(ENTRYTYPE_PREFSTEMPL);
		entry.SetLabel(label);
		entry.SetPage(strPrefsPage);
		m_list.Add(entry);
		m_nTotalCount++;
	}

	/**
	 * Adds a User Profile URL
	 *
	 * @param label 			String to display
	 * @param nUserID 			The ID of the user
	 */
	public void AddUserProfileURL(String label, int nUserID)
	{
		AddUserProfileURL(label, nUserID, "");
	}

	/**
	 * Adds a User Profile URL with description
	 *
	 * @param label 			String to display
	 * @param nUserID 			The ID of the user
	 * @param strDescription 	Description of the user
	 */
	public void AddUserProfileURL(String label, int nUserID,
		String strDescription)
	{
		ListEntry entry = new ListEntry(ENTRYTYPE_USERTEMPL);
		entry.SetLabel(label);
		entry.SetPageID(nUserID);
		entry.SetDescription(strDescription);
		m_list.Add(entry);
		m_nTotalCount++;
	}

	/**
	 * Gets the current entry
	 * 
	 * ResetIterator and GetNextEntry will get the
	 * first entry in the list.
	 *
	 * @return					The current entry in the list
	 */
	public ListEntry GetEntry()
	{
		return (ListEntry) m_list.GetElement(m_curListItemIndex);
	}

	/**
	 * Gets an entry at a specific entry
	 *
	 * @param nIndex 			Index of the entry
	 *
	 * @return 					Entry at the specified entry
	 */
	public ListEntry GetEntryAtIndex(int nIndex)
	{
		return (ListEntry) m_list.GetElement(nIndex);
	}

	/**
	 * Reset the Iterator
	 * 
	 */
	public void GetIterator()
	{
		m_curListItemIndex = -1;
	}

	/**
	 * Get the number of entries in the list
	 *
	 * @return 					Count of entries in the list
	 */
	public int GetTotalCount()
	{
		return m_nTotalCount;
	}

	/**
	 * Advances the iterator to next entry
	 *
	 * @return 					False if no more exist, otherwise true
	 */
	public boolean Next()
	{
		// TODO CR DONH: Why doesn't this behave the same as normal Java &
		// C# iterators?  Normally you say while movenext, getcurrent, but
		// here that will miss the first item.  Why is this non-standard?
		m_curListItemIndex++;

		return (m_curListItemIndex < (m_list.GetSize()));
	}

	// bjf -- need a version that doesn't require passing in icon height & width, too hard
	// to keep it in sync with what is hard-coded in html constructs.
	
	/**
	 * Assigns an image to the entry last added to the list
	 * 
	 * @param image				Image construct for the image to add
	 * @param strImageAltText	Alt String for the image
	 */
	public void SetImageForLastAddedURL(PTImageType image,
		String strImageAltText)
	{
		ListEntry entry = (ListEntry) m_list.GetElement(m_nTotalCount - 1);
		entry.SetImage(image);
		entry.SetImageAltText(strImageAltText);
	}
	
	/**
	 * Assigns a two part image to the entry last added to the list
	 * 
	 * @param image				Image construct for the first image
	 * @param image2			Image construct for the second image
	 * @param strImageAltText	Alt string for both images
	 */
	public void SetImageForLastAddedURL(PTImageType image, PTImageType image2, String strImageAltText)
	{
		ListEntry entry = (ListEntry) m_list.GetElement(m_nTotalCount - 1);
		entry.SetImage(image);
		entry.SetImage2(image2);
		entry.SetImageAltText(strImageAltText);
	}
	
	/**
	 * Not for general use
	 * Only added for converting IPTURLs to ListEntry URLs in NavigationModel. 
	 * Only supports icons for Community Actions, My Page actions and Mandatory link entry types
	 * 
	 * @param strImagePath
	 * @param strImageAltText
	 */
	public void SetImageForLastAddedURL(String strImagePath, String strImageAltText)
	{
		ListEntry entry = (ListEntry) m_list.GetElement(m_nTotalCount - 1);
		entry.SetImageAltText(strImageAltText);
		
		// This is a poor way to do this but there is no way to map image names or paths to PTImageTypes 
		
		// Community Actions
		if ( strImagePath.endsWith(PTImageType.ACTION_COMMUNITIES_JOIN.GetFileName())  )
		{
			entry.SetImage(PTImageType.ACTION_COMMUNITIES_JOIN);
		} 
		else if ( strImagePath.endsWith(PTImageType.ACTION_COMMUNITIES_UNSUB.GetFileName()) )
		{
			entry.SetImage(PTImageType.ACTION_COMMUNITIES_UNSUB);
		}
		else if (strImagePath.endsWith(PTImageType.ACTION_MYPAGE_NEW.GetFileName()) )
		{
			entry.SetImage(PTImageType.ACTION_MYPAGE_NEW);
		}
		else if (strImagePath.endsWith(PTImageType.ACTION_COMMUNITIES_EDIT.GetFileName()) )
		{
			entry.SetImage(PTImageType.ACTION_COMMUNITIES_EDIT);
		}
		else if (strImagePath.endsWith(PTImageType.ACTION_PORTLET_ADD.GetFileName()) )
		{
			entry.SetImage(PTImageType.ACTION_PORTLET_ADD);
		}
		else if (strImagePath.endsWith(PTImageType.ACTION_PORTLET_PREF.GetFileName()) )
		{
			entry.SetImage(PTImageType.ACTION_PORTLET_PREF);
		}
		// MyPage Actions
		else if (strImagePath.endsWith(PTImageType.ACTION_MYPAGE_NEW.GetFileName()) )
		{
			entry.SetImage(PTImageType.ACTION_MYPAGE_NEW);
		}
		else if (strImagePath.endsWith(PTImageType.ICON_EDIT_MENU.GetFileName()) )
		{
			entry.SetImage(PTImageType.ICON_EDIT_MENU);
		}
		else if (strImagePath.endsWith(PTImageType.ACTION_PORTLET_ADD.GetFileName()) )
		{
			entry.SetImage(PTImageType.ACTION_PORTLET_ADD);
		}
		else if (strImagePath.endsWith(PTImageType.ACTION_DELETE_MENU.GetFileName()) )
		{
			entry.SetImage(PTImageType.ACTION_DELETE_MENU);
		}
		// Mandatory Links entries
		else if (strImagePath.endsWith(PTImageType.ICON_URL.GetFileName()) )
		{
			entry.SetImage(PTImageType.ICON_URL);
		}
		else if (strImagePath.endsWith(PTImageType.ICON_COMMUNITY.GetFileName()) )
		{
			entry.SetImage(PTImageType.ICON_COMMUNITY);
		}
		else if (strImagePath.endsWith(PTImageType.ICON_USER.GetFileName()) )
		{
			entry.SetImage(PTImageType.ICON_USER);
		}
		else if (strImagePath. endsWith(PTImageType.ICON_FOLDER.GetFileName()) )
		{
			entry.SetImage(PTImageType.ICON_FOLDER);
		}
		else if (strImagePath.endsWith(PTImageType.ICON_DOCUMENT.GetFileName()) )
		{
			entry.SetImage(PTImageType.ICON_DOCUMENT);
		}
		
		
	}
	
} 

