package com.plumtree.portaluiinfrastructure.expandablelist.properties;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.login.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.arraywrapper.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.expandablelist.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.pagecontrols.datepicker.*;
import com.plumtree.uiinfrastructure.pagecontrols.framework.*;
import com.plumtree.uiinfrastructure.pagecontrols.jsdatepicker.*;
import com.plumtree.uiinfrastructure.pagecontrols.jsdatetimepicker.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.jsutil.*;

/**
 * Expandable list model for properties
 *
 * @author josephc
 */
public abstract class ExpListPropModel extends ExpListModel implements IExpListPropModelRO
{
	/** Prefix for unique ID of row containing datetime picker */
	public static final String STR_ROW_PROPERTY_DATE_PICKER_UNIQUE_ID_PREFIX = "RowPropertyDP";

	/** Expandable list column for property ID */
	public static final int EXPLIST_COL_PROPID = 0;

	/** Expandable list column for property value */
	public static final int EXPLIST_COL_VALUE = 1;

	/** Expandable list column for property name */
	public static final int EXPLIST_COL_NAME = 2;

	/** Expandable list column for property type */
	public static final int EXPLIST_COL_TYPE = 3;

	/** Expandable list column for row status (no change, added, deleted, edited) */
	public static final int EXPLIST_COL_STATUS = 4;

	/** Status of unchanged row */
	public static final int ROW_STATUS_NO_CHANGE = 0;

	/** Status of newly added row */
	public static final int ROW_STATUS_ADDED = 1;

	/** Status of modified row */
	public static final int ROW_STATUS_MODIFIED = 2;

	/** DateTime picker control */
	protected JSDateTimePickerMgr m_ptJSDateTimePickerMgr;

	/** Page manager */
	protected PageMgr m_ptPageMgr;

	/**
	 * Inits the Model with the Activity Space.  Initializes date/time picker and page manager
	 * for date/time picker.
	 *
	 * @param parent
	 */
	public void Init(AActivitySpace parent)
	{
		super.Init(parent);

		// Deal with the page manager
		String strImageServerBaseURL = ImgSvrURLHelper.GetInstance().GetRootURL(parent.GetIsSecuredSpace());
		m_ptPageMgr = new PageMgr(m_asOwner.GetAccessStyle(), PTPersonalSettingsHelper.GetPersonalSettingValue_Locale(m_asOwner), PTPersonalSettingsHelper.GetPersonalSettingValue_TimeZone(m_asOwner), strImageServerBaseURL, ConfigHelper.GetPortalImageServerConnectionURL(m_asOwner));

		JSRegistry ptJSRegistry = JSRegistry.GetRegistry(strImageServerBaseURL, ConfigHelper.GetPortalImageServerConnectionURL(m_asOwner));
		JSDatePickerVersionMgr ptDatePickerVersionMgr = new JSDatePickerVersionMgr();

		// Deal with the date/time-picker
		m_ptJSDateTimePickerMgr = new JSDateTimePickerMgr(ptJSRegistry, ptDatePickerVersionMgr, m_ptPageMgr);
		m_ptPageMgr.RegisterComponentMgr(m_ptJSDateTimePickerMgr);
	}

	/**
	 * Returns the datetime-picker control manager.
	 *
	 * @return The datetime-picker control manager.
	 */
	public JSDateTimePickerMgr GetDateTimePickerControlManager()
	{
		return m_ptJSDateTimePickerMgr;
	}

	/**
	 * Helper method to return XPDateTime from datetime picker, using the unique instance ID.
	 *
	 * @param _strUniqueInstanceID The unique instance ID is the special prefix plus the row number.
	 * @return
	 */
	protected XPDateTime GetDateTimeFromDatePicker(String _strUniqueInstanceID)
	{
		ADatePickerComponent ptJSDateTimePicker = (ADatePickerComponent) m_ptJSDateTimePickerMgr.GetComponent(_strUniqueInstanceID);

		if (null != ptJSDateTimePicker)
		{
			return ptJSDateTimePicker.GetDateTimeAsDateTime();
		}
		else
		{
			return null;
		}
	}

	/**
	 * Converts a property value from its string representation to the
	 * appropriate type.
	 *
	 * @param strValue String value
	 * @param nPropertyType Property type
	 *
	 * @return Property value as an Object
	 *
	 * @throws Exception
	 */
	public static Object GetPropValueFromString(String strValue, int nPropertyType)
	{
		// Convert string value to object type
		Object oValue = null;

		if ((null != strValue) && (strValue.length() > 0))
		{
			switch (nPropertyType)
			{
			case PT_PROPERTY_TYPES.PT_PROPTYPE_DOUBLE:
				oValue = new Double(XPConvert.ToDouble(strValue));

				break;

			case PT_PROPERTY_TYPES.PT_PROPTYPE_LONG:
			case PT_PROPERTY_TYPES.PT_PROPTYPE_REF:
				oValue = new Integer(XPConvert.ToInteger(strValue));

				break;

			case PT_PROPERTY_TYPES.PT_PROPTYPE_STRING:
			case PT_PROPERTY_TYPES.PT_PROPTYPE_ENCRYPTED:
				oValue = strValue;

				break;

			default:
				break;
			}
		}

		return oValue;
	}

	/**
	 * Returns the name of the property in the indicated row
	 *
	 * @param _nRowIndex Row index
	 *
	 * @return Name of property
	 */
	public String GetRowName(int _nRowIndex)
	{
		return (String) m_saData.GetItem(_nRowIndex, EXPLIST_COL_NAME);
	}

	/**
	 * Returns the property ID of the property in the indicated row
	 *
	 * @param _nRowIndex Row index
	 *
	 * @return Property ID
	 */
	public int GetRowPropertyID(int _nRowIndex)
	{
		return XPConvert.ToInteger(m_saData.GetItem(_nRowIndex, EXPLIST_COL_PROPID));
	}

	/**
	 * Returns the status of the row (added, deleted, edited, modified)
	 *
	 * @param _nRowIndex Row index
	 *
	 * @return Row status
	 */
	public int GetRowStatus(int _nRowIndex)
	{
		return XPConvert.ToInteger(m_saData.GetItem(_nRowIndex, EXPLIST_COL_STATUS));
	}

	/**
	 * Returns the value type of the property
	 *
	 * @param _nRowIndex Row index
	 *
	 * @return Value type of property
	 */
	public int GetRowType(int _nRowIndex)
	{
		return XPConvert.ToInteger(m_saData.GetItem(_nRowIndex, EXPLIST_COL_TYPE));
	}

	/**
	 * Returns the value of the property
	 *
	 * @param _nRowIndex Row index
	 *
	 * @return Value of property
	 */
	public Object GetRowValue(int _nRowIndex)
	{
		return m_saData.GetItem(_nRowIndex, EXPLIST_COL_VALUE);
	}

	/**
	 * Returns the unique date/time picker string ID for the row.
	 *
	 * @param _nRowIndex
	 * @return
	 */
	public String GetUniqueDateTimePickerKey(int _nRowIndex)
	{
		return STR_ROW_PROPERTY_DATE_PICKER_UNIQUE_ID_PREFIX + _nRowIndex;
	}

	/**
	 * Set the row status
	 *
	 * @param _nRowIndex Row index
	 * @param _nStatus Status of row
	 */
	protected void SetRowStatus(int _nRowIndex, int _nStatus)
	{
		m_saData.SetItem(_nRowIndex, EXPLIST_COL_STATUS, new Integer(_nStatus));
	}

	/**
	 * Set the row value
	 *
	 * @param _nRowIndex Row index
	 * @param _oValue Value of property in row
	 */
	protected void SetRowValue(int _nRowIndex, Object _oValue)
	{
		m_saData.SetItem(_nRowIndex, EXPLIST_COL_VALUE, _oValue);
	}

	/**
	 * Updates a row in the list with new values
	 *
	 * @param _nRowIndex Row index
	 * @param _oValue Value of property in row
	 */
	protected void UpdateRowInList(int _nRowIndex, Object _oValue)
	{
		boolean bUpdateRow = false;

		Object oPreviousValue = GetRowValue(_nRowIndex);

		if (null == _oValue)
		{
			// If the value is null and the previous value is not null, then update.
			if (null != oPreviousValue)
			{
				bUpdateRow = true;
			}
		}
		else
		{
			// If the value is not null and not equal to the previous value, then update.
			if (!_oValue.equals(oPreviousValue))
			{
				bUpdateRow = true;
			}
		}

		if (bUpdateRow)
		{
			SetRowValue(_nRowIndex, _oValue);

			if (ROW_STATUS_NO_CHANGE == GetRowStatus(_nRowIndex))
			{
				SetRowStatus(_nRowIndex, ROW_STATUS_MODIFIED);
			}
		}
	}

	/**
	 * Saves the date/time value
	 *
	 * @param _nRowIndex Row index
	 * @param _htFormData Form data
	 *
	 * @return Valid if row is saved successfully. Invalid if mandatory and value is null.
	 */
	protected int SaveDateTimeRow(int _nRowIndex, XPHashtable _htFormData)
	{
		UIError myError;
	   int nStatus = RepostControl.PAGE_STATUS_VALID;
	   nStatus = m_ptPageMgr.StoreAllUserInput(_htFormData);

	   // if the status is invalid, print out error msgs stored in the page manager
	   // page mgr holds errors msgs generated from storing all the inputs
	   if (nStatus == RepostControl.PAGE_STATUS_INVALID)
	   {
		   myError = new UIError(UIErrorLevel.FRIENDLY,
								 UIErrorScope.CURRENT_USER,
								 m_ptPageMgr.GetErrorMessages(m_asOwner),
								 m_ptPageMgr.GetErrorMessages(m_asOwner));
		   UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(),
									 m_asOwner.GetApplication());
	   }

		String strUniqueKey = GetUniqueDateTimePickerKey(_nRowIndex);
		XPDateTime myDateTime = GetDateTimeFromDatePicker(strUniqueKey);

		if (null != myDateTime)
		{
        	UpdateRowInList(_nRowIndex, myDateTime);
		}
		else if (CheckRowIsMandatory(_nRowIndex))
		{
			// raise error to display to user
			String strErrorMsg = GetRowErrorMessage(_nRowIndex, m_asOwner.GetString(99, "ptmsgs_portalinfrastructure"));
			myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER, strErrorMsg, strErrorMsg);
			UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), m_asOwner.GetApplication());

			return RepostControl.PAGE_STATUS_INVALID;
		}


		return nStatus;
	}


    /**
     * Returns data key for a row.
     *
     * @param _nRowIndex Row index
     * @param _htFormData Form data
     *
     * @return Data key for row
     */
    protected String GetDataKey(int _nRowIndex, XPHashtable _htFormData)
    {
        return GetUniqueString(ExpListPropEditView.HTMLINPUT_ITEM) +
               _nRowIndex;
    }

	/**
	 * Returns data key for a row.
	 *
	 * @param _nRowIndex Row index
	 * @param _htFormData Form data
	 *
	 * @return Data key for row
	 */
	protected String GetPasswordKey(int _nRowIndex, XPHashtable _htFormData)
	{
		return GetUniqueString(ExpListPropEditView.HTMLINPUT_PASSWORD) +
			   _nRowIndex;
	}


	/**
	 * Check that a mandatory row does not have a null string value or empty string.
	 * Raise error if string value is null or empty string. 
	 *
	 * @param _nRowIndex Row index
	 * @param strValue String value
	 *
	 * @return Valid if row is not mandatory or if it is mandatory and has non-empty value.
	 */
	protected int ValidateMandatoryDataValue(int _nRowIndex, String strValue)
	{
		// Validate mandatory row
		if (CheckRowIsMandatory(_nRowIndex))
		{
			if (((null == strValue) || (0 == strValue.length())))
			{
				// raise error to display to user
				String strErrorMsg = GetRowErrorMessage(_nRowIndex, m_asOwner.GetString(99, "ptmsgs_portalinfrastructure"));
				UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER, strErrorMsg, strErrorMsg);
				UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), m_asOwner.GetApplication());

				return RepostControl.PAGE_STATUS_INVALID;
			}
		}

		return RepostControl.PAGE_STATUS_VALID;
	}

	/**
	 * Saves the data value for the row
	 *
	 * @param _nRowIndex Row index
	 * @param _htFormData Form data
	 *
	 * @return Valid if row was saved successfully.
	 */
	protected int SaveDataValue(int _nRowIndex, XPHashtable _htFormData)
	{
		String sDataKey = GetDataKey(_nRowIndex, _htFormData);

		if (_htFormData.ContainsKey(sDataKey))
		{
			String[] aDataValues = (String[]) _htFormData.GetElement(sDataKey);

			if (aDataValues.length > 0)
			{
				Object oValue = null;
				String strValue = aDataValues[0];
				int nPropertyType = GetRowType(_nRowIndex);

				int nStatus = ValidateMandatoryDataValue(_nRowIndex, strValue);

				if (RepostControl.PAGE_STATUS_INVALID == nStatus)
				{
					return nStatus;
				}

				try
				{
					// Convert string to object
					oValue = GetPropValueFromString(strValue, nPropertyType);
				}
				catch (Exception e)
				{
					// There was a problem converting the string value to the object value.
					String strErrorMsg = GetRowErrorMessage(_nRowIndex, m_asOwner.GetString(98, "ptmsgs_portalinfrastructure"));
					UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER, e, strErrorMsg, strErrorMsg);
					UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), m_asOwner.GetApplication());

					return RepostControl.PAGE_STATUS_INVALID;
				}

				// Update the value
				UpdateRowInList(_nRowIndex, oValue);
			}
		}

		return RepostControl.PAGE_STATUS_VALID;
	}

	/**
	 * Saves the data value for the row
	 *
	 * @param _nRowIndex Row index
	 * @param _htFormData Form data
	 *
	 * @return Valid if row was saved successfully.
	 */
	protected int SavePasswordValue(int _nRowIndex, XPHashtable _htFormData)
	{
		String sDataKey = GetPasswordKey(_nRowIndex, _htFormData);

		if (_htFormData.ContainsKey(sDataKey))
		{
			String[] aDataValues = (String[]) _htFormData.GetElement(sDataKey);

			if (aDataValues.length > 0)
			{
				Object oValue = null;
				String strValue = aDataValues[0];
				int nPropertyType = GetRowType(_nRowIndex);

				int nStatus = ValidateMandatoryDataValue(_nRowIndex, strValue);

				if (RepostControl.PAGE_STATUS_INVALID == nStatus)
				{
					return nStatus;
				}

				try
				{
					// Convert string to object
					oValue = GetPropValueFromString(strValue, nPropertyType);
				}
				catch (Exception e)
				{
					// There was a problem converting the string value to the object value.
					String strErrorMsg = GetRowErrorMessage(_nRowIndex, m_asOwner.GetString(98, "ptmsgs_portalinfrastructure"));
					UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER, e, strErrorMsg, strErrorMsg);
					UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), m_asOwner.GetApplication());

					return RepostControl.PAGE_STATUS_INVALID;
				}

				// Update the value
				if (oValue != null)
				{
					if (!(oValue.equals(PlumtreeExtensibility.STR_NULL_PASSWORD)))
					{
						UpdateRowInList(_nRowIndex, oValue);
					}
				}
				else
				{
					UpdateRowInList(_nRowIndex, oValue);
				}
			}
		}

		return RepostControl.PAGE_STATUS_VALID;
	}

	/**
	 * Return an error message for row
	 *
	 * @param _nRowIndex Row index
	 * @param strMessage Message
	 *
	 * @return Error message for row
	 */
	protected String GetRowErrorMessage(int _nRowIndex, String strMessage)
	{
		XPStringBuilder sb = new XPStringBuilder(100);
		String strPropertyName = GetRowName(_nRowIndex);

		sb.Append(strPropertyName).Append(": ");
		sb.Append(strMessage);

		return sb.ToString();
	}

	/**
	 * Saves the data on the row _nRowIndex.
	 *
	 * @param _saData List to update the data on row _nRowIndex
	 * @param _nRowIndex Index of the row to save.
	 * @param _htFormData Data posted by the form.
	 *
	 * @return int Status for server side validation.
	 */
	protected int SaveRow(PTGrowableSortedArrayWrapper _saData, int _nRowIndex, XPHashtable _htFormData)
	{
		int nPropertyType = GetRowType(_nRowIndex);

		try
		{
			switch (nPropertyType)
			{
			case PT_PROPERTY_TYPES.PT_PROPTYPE_DATE:
				return SaveDateTimeRow(_nRowIndex, _htFormData);

			case PT_PROPERTY_TYPES.PT_PROPTYPE_STRING:
			case PT_PROPERTY_TYPES.PT_PROPTYPE_LONG:
			case PT_PROPERTY_TYPES.PT_PROPTYPE_DOUBLE:
				return SaveDataValue(_nRowIndex, _htFormData);

			case PT_PROPERTY_TYPES.PT_PROPTYPE_ENCRYPTED:
				return SavePasswordValue(_nRowIndex, _htFormData);

			case PT_PROPERTY_TYPES.PT_PROPTYPE_REF:
				return RepostControl.PAGE_STATUS_VALID;

			default:
				return RepostControl.PAGE_STATUS_INVALID;
			}
		}
		catch (Exception e)
		{
			return RepostControl.PAGE_STATUS_INVALID;
		}
	}

	/**
	 * Remove all existing datetime components from the datetime picker manager, and
	 * re-register all datetime pickers.  This method must be called whenever the expandable
	 * list is modified (whenever rows are added or deleted) because the row numbers, which
	 * are used to construct the unique key, are changed.
	 */
	protected void UpdateDateTimeUniqueKeys()
	{
		// Remove first component in the list until no more components
		while (0 < m_ptJSDateTimePickerMgr.GetNumComponents())
		{
			AComponent ptJSComponent = m_ptJSDateTimePickerMgr.GetComponent(0);
			m_ptJSDateTimePickerMgr.UnRegisterComponent(ptJSComponent.GetUniqueInstanceID());
		}

		// Iterate through all rows and re-register all date-time pickers
		for (int nRowIndex = 0; nRowIndex < m_saData.GetCount(); nRowIndex++)
		{
			Object oValue = GetRowValue(nRowIndex);

			if (PT_PROPERTY_TYPES.PT_PROPTYPE_DATE == GetRowType(nRowIndex))
			{
				String strUniqueKey = GetUniqueDateTimePickerKey(nRowIndex);

				ADatePickerComponent ptJSDateTimePicker;

				if ((null == oValue) || oValue.equals(""))
				{
					ptJSDateTimePicker = (ADatePickerComponent) m_ptPageMgr.AddPageControl("DateTimePicker");
					ptJSDateTimePicker.Init(AFormDP.HTMLFORM_NAME, m_ptPageMgr, strUniqueKey);
				}
				else
				{
					ptJSDateTimePicker = (ADatePickerComponent) m_ptPageMgr.AddPageControl("DateTimePicker");
					ptJSDateTimePicker.Init(AFormDP.HTMLFORM_NAME, (XPDateTime) oValue, m_ptPageMgr, strUniqueKey);
				}

				m_ptJSDateTimePickerMgr.RegisterComponent(ptJSDateTimePicker);
			}
		}
	}

	/**
	 * Implemented by the child class in order to load the properties.
	 */
	protected abstract void LoadProperties();

	/**
	 * Implemented by the child class in order to save the properties.
	 */
	public abstract void Finish();

	/**
	 * Check whether row is mandatory
	 *
	 * @param _nRowIndex Row index
	 *
	 * @return True if row is mandatory
	 */
	public abstract boolean CheckRowIsMandatory(int _nRowIndex);

	/**
	 * Must update date/time picker rows when sorting by column.
	 *
	 * @param _htFormData Form data
	 */
	public void SortByColumn(XPHashtable _htFormData)
	{
		super.SortByColumn(_htFormData);

		UpdateDateTimeUniqueKeys();
	}

	/**
	 * Must update date/time picker rows when sorting by column.
	 */
	public void ReverseCurrentSort()
	{
		super.ReverseCurrentSort();

		UpdateDateTimeUniqueKeys();
	}
}

