package com.plumtree.portaluiinfrastructure.search;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.application.varpacks.*;
import com.plumtree.portaluiinfrastructure.classtypedescriptors.*;
import com.plumtree.portaluiinfrastructure.filter.*;
import com.plumtree.portaluiinfrastructure.login.*;
import com.plumtree.portaluiinfrastructure.opener.*;
import com.plumtree.portaluiinfrastructure.search.exceptions.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.server.portalpagesperf.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.htmlconstructs.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.web.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * @author CraigS
 */
public class SearchResultModel implements ISearchResultModelRO,
  ISearchResultModel
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.search.SearchResultModel");

  /** Name of this class */
  public static final String STR_MVC_CLASS_NAME = "SearchResult";

  public static final String MODEL_MODE_INPUT = "in_hi_model_mode";
  
  /** String that getResultApp() returns for portal items */
  public static final String PORTAL_APP  = "PTPORTAL";

  /** String that getResultApp() returns for community templating items
   *  (whose subtypes are the same as those for portal items, but do not
   *   correspond to "real" portal objects) */
  public static final String TEMPLATING_APP  = "PTCONTENTTEMPLATE";

  /** String that getResultApp() returns for Collab items */
  public static final String COLLAB_APP  = "PTCOLLAB";

  /** String that getResultApp() returns for Content items */
  public static final String CONTENT_APP = "PTCONTENT";

  /** String that getResultAbsType() returns for documents */
  public static final String DOCUMENT_ABSTYPE  = "DOCUMENT";

  /** String that getResultAbsType() returns for containers */
  public static final String CONTAINER_ABSTYPE  = "CONTAINER";

  /** String that getResultAbsType() returns for any other item */
  public static final String OTHER_ABSTYPE  = "OTHER";

  /**
   *  Supported sort orders.  Note that any numeric or date property
   *  should work, but it's convenient to provide these simple definitions.
   */
  public static final String SORTORDER_RELEVANCE     = "0";

  public static final String SORTORDER_CREATED_ASC   = "3"; // == PT_INTRINSICS.PT_PROPERTY_OBJECTCREATED

  public static final String SORTORDER_CREATED_DESC  = "-3";

  public static final String SORTORDER_MODIFIED_ASC  = "4"; // == PT_INTRINSICS.PT_PROPERTY_OBJECTLASTMODIFIED

  public static final String SORTORDER_MODIFIED_DESC = "-4";
  
  public static final String SORTORDER_NAME_ASC = "1";      // == PT_INTRINSICS.PT_PROPERTY_OBJECTNAME
  
  public static final String SORTORDER_NAME_DESC = "-1";
  /**
   *  Built-in group-by string identifiers.
   */
  public static final String GROUPBY_FOLDER = "ancestors";
  public static final String GROUPBY_OBJECTTYPE = "subtype";

  private static final String CACHE_KEY_SEPARATOR_STR = "/";
  private static final char CACHE_KEY_SEPARATOR_CHAR  = '/';

  public static final String GROUP_BY_PREFS_KEY = "GroupBySearchPrefs";

    /** 
     * Name of folder returned when db lookup of name fails- must
     * be localized before displaying. NOTE: this must be kept in 
     * sync with what is returned by PTSearch.dll
     **/
    public static final String UNKNOWN_FOLDER_NAME="Unknown Folder";
  
  /**
   *  User's original search filter
   */
  protected IPTFilter _advancedFilter = null;

  /**
   *  User's original search string
   */
  protected String _initialSearchString = null;

  /**
   *  Whether best bets were initially requested
   */
  protected boolean _initialBestBets = false;

  /**
   *  Field that was used for group-by on the original search.  This field
   *  will also be the default for group-by on any later drilldown, or
   *  after the user uses a breadcrumb control to undo the drilldown.
   */
  protected String _initialGroupBy = null;

  /**
   *  Name this search has been saved under
   */
  protected String _savedSearchName = null;

  /**
   *  Fields that search-within-results queries will run against
   */
  protected String _swrFields = null;

  /**
   *  Whether the original search used a filter (i.e. advanced search)
   */
  protected boolean _isAdvancedSearch = false;

  /**
   *  The current "mode" that the client page is in.  This class doesn't care about
   *  the contents; this is just a convenient holder for client code.  See get/set
   *  methods below.
   */
  protected String _pageMode = null;
  
  /**
   *  User's desired number of results per page
   */
  protected int _pageSize = 20;

  /**
   *  Integer that client views can use to get/set error conditions for
   *  saved search (or they can use the getError() mechanism, but
   *  this affects *all* views, not just the saved search view).
   */
  protected int _savedSearchErrorCode = 0;

  /*
   *  Number of search-within-results done so far
   */
   protected int _swrCount = 0;

  
  //
  //  Private data
  //

  /**
   *  Owning ActivitySpace
   */
  private AActivitySpace m_asOwner;

  /**
   *  COM-side search request object we are using
   */
  private IPTSearchRequest _request = null;

  /**
   *  COM-side response we are currently displaying
   */
  private IPTSearchResponse _currentResponse = null;

  /**
   *  Owning session
   */
  private IPTSession _session;

  /**
   *  Error on last search attempt
   */
  private SearchExecutionError _error = null;

  /**
   *  Warning on last search attempt
   */
  private SearchExecutionWarning _warning = null;

  /**
   *  Search settings that could be removed on a followup search
   */
  private SearchSettingCollection _removableSettings = null;

  /**
   *  Key of current response being displayed
   */
  private String _currentKey = null;
  private XPHashtable _crumbCache  = null;

  /**
   *  Previous queries we have run, corresponding breadcrumbs,
   *  and the resulting responses.
   *  _queryCache is a map from String key to IPTSearchQuery
   *  _crumbCache is a map from String key to String
   *  _responseCache is a map from String key to IPTSearchResponse
   */
  private XPHashtable _queryCache = null;
  private XPHashtable _responseCache = null;

  /**
   *  Whether this model instance has been initialized by calling
   *  Search(); if false, search results aren't available yet
   */
  private boolean _initialized = false;

  /**
   *  @see com.plumtree.portalpages.search.ISearchQueryRO.IsASTooComplexToSummarize()
   */
  public boolean IsASTooComplexToSummarize()
  {
    checkInitialized();

    if (!_isAdvancedSearch)
    {
      throw new NotAdvancedSearchException();
    }

    IPTPropertyFilterItem fi =  _advancedFilter.GetPropertyFilter();
    if (null == fi) return false;

    if (PT_FILTER_ITEM_TYPES.PT_FILTER_ITEM_STATEMENT == fi.GetType()) {
      return false;
    }

    IPTPropertyFilterClauses clause = (IPTPropertyFilterClauses) fi;

    for (int j = 0; j < clause.GetCount(); j++) {
      fi = clause.GetItem(j);
      if (PT_FILTER_ITEM_TYPES.PT_FILTER_ITEM_CLAUSES == fi.GetType()) {
        return true;
      }
    }

    return false;
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchQueryRO.getASConstraintOperator()
   */
  public String GetASConstraintOperator(int condition)

  {
    checkInitialized();

    if (!_isAdvancedSearch)
    {
      throw new NotAdvancedSearchException();
    }

    IPTPropertyFilterClauses clause = (IPTPropertyFilterClauses) _advancedFilter.GetPropertyFilter();
    IPTPropertyFilterStatement stmt = (IPTPropertyFilterStatement) clause.GetItem(condition);

    boolean isDateTime = IsDateTimeProperty(stmt.GetOperand());
    int opID = stmt.GetOperator();

    switch (opID)
    {
    case PT_FILTEROPS.PT_FILTEROP_CONTAINS:
     return m_asOwner.GetString(324, "ptmsgs_portalinfrastructure");

    case PT_FILTEROPS.PT_FILTEROP_DOESNOTCONTAIN:
      return m_asOwner.GetString(325, "ptmsgs_portalinfrastructure");

    case PT_FILTEROPS.PT_FILTEROP_EQ:
    case PT_FILTEROPS.PT_FILTEROP_EQ_NOCASE:
    case PT_FILTEROPS.PT_FILTEROP_NUMERIC_EQ:
      return m_asOwner.GetString(326, "ptmsgs_portalinfrastructure");

    case PT_FILTEROPS.PT_FILTEROP_NE:
    case PT_FILTEROPS.PT_FILTEROP_NUMERIC_NOTEQ:
     return m_asOwner.GetString(327, "ptmsgs_portalinfrastructure");

    case PT_FILTEROPS.PT_FILTEROP_LT:
      if (isDateTime) {
        // Comes Before
        return m_asOwner.GetString(36, "ptmsgs_portalinfrastructure");
      } else {
        return m_asOwner.GetString(328, "ptmsgs_portalinfrastructure");
      }

    case PT_FILTEROPS.PT_FILTEROP_GT:
      if (isDateTime) {
        // Comes After
        return m_asOwner.GetString(35, "ptmsgs_portalinfrastructure");
      } else {
        return m_asOwner.GetString(330, "ptmsgs_portalinfrastructure");
      }

    case PT_FILTEROPS.PT_FILTEROP_LTE:
      if (isDateTime) {
        // Comes Before
        return m_asOwner.GetString(36, "ptmsgs_portalinfrastructure");
      } else {
       return m_asOwner.GetString(329, "ptmsgs_portalinfrastructure");
      }

    case PT_FILTEROPS.PT_FILTEROP_GTE:
      if (isDateTime) {
        // Comes After
        return m_asOwner.GetString(35, "ptmsgs_portalinfrastructure");
      } else {
        return m_asOwner.GetString(331, "ptmsgs_portalinfrastructure");
      }

    case PT_FILTEROPS.PT_FILTEROP_RELATIVE_GTE:
      // "in the last.."
      {
        long relativeTime = XPConvert.ToLong(stmt.GetValueAsString());

        final int MS_PER_DAY = 86400000;
        final int MS_PER_WEEK = MS_PER_DAY * 7;

        if (relativeTime < 0) relativeTime = -1 * relativeTime;

        if (0 == (relativeTime % MS_PER_WEEK)) {
          return XPResourceManager.FormatMessage(m_asOwner.GetString(531, "ptmsgs_portalinfrastructure"), XPConvert.ToString(relativeTime / MS_PER_WEEK));
        }
        else if (0 == (relativeTime % MS_PER_DAY)) {
          return XPResourceManager.FormatMessage(m_asOwner.GetString(530, "ptmsgs_portalinfrastructure"), XPConvert.ToString(relativeTime / MS_PER_DAY));
        }
        else {
          return XPResourceManager.FormatMessage(m_asOwner.GetString(532, "ptmsgs_portalinfrastructure"), XPConvert.ToString(relativeTime));
        }
      }

    case PT_FILTEROPS.PT_FILTEROP_ISEMPTY:
      return m_asOwner.GetString(332, "ptmsgs_portalinfrastructure");

    case PT_FILTEROPS.PT_FILTEROP_ISNOTEMPTY:
      return m_asOwner.GetString(333, "ptmsgs_portalinfrastructure");

    case PT_FILTEROPS.PT_FILTEROP_REGEXPMATCH:
      return m_asOwner.GetString(335, "ptmsgs_portalinfrastructure");

    case PT_FILTEROPS.PT_FILTEROP_REGEXPNOTMATCH:
      return m_asOwner.GetString(336, "ptmsgs_portalinfrastructure");

    case PT_FILTEROPS.PT_FILTEROP_IN:
      return m_asOwner.GetString(334, "ptmsgs_portalinfrastructure");
    }

    return m_asOwner.GetString(341, "ptmsgs_portalinfrastructure");
  }

  private boolean IsDateTimeProperty(int propID) {
    // Is this cached somewhere?
    IPTQueryResult ptQR = _session.GetProperties().QuerySingleObject(propID);
    if (null != ptQR && ptQR.RowCount() > 0) {
      		return (PT_PROPERTY_TYPES.PT_PROPTYPE_DATE == ptQR.ItemAsInt(0, PT_PROPIDS.PT_PROPID_PROP_VALUETYPE));
    }
    return false;
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchQueryRO.getASConstraintPropertyID()
   */
  public String GetASConstraintProperty(int condition)

  {
    checkInitialized();

    if (!_isAdvancedSearch)
    {
      throw new NotAdvancedSearchException();
    }

    IPTPropertyFilterClauses clause = (IPTPropertyFilterClauses) _advancedFilter.GetPropertyFilter();
    IPTPropertyFilterStatement stmt = (IPTPropertyFilterStatement) clause.GetItem(condition);

    int propID = stmt.GetOperand();

    try
    {
      IPTQueryResult ptQR = _session.GetProperties().QuerySingleObject(propID);

      if (null != ptQR && ptQR.RowCount() > 0)
      {
        String result = ptQR.ItemAsString(0, PT_PROPIDS.PT_PROPID_NAME);

        if (null != result)
        {
          return result;
        }
      }
    }
    catch (Exception e)
    {
      
        log.Error(e, "Error in GetASConstraintProperty()");
      
    }

    return m_asOwner.GetString(340, "ptmsgs_portalinfrastructure");
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchQueryRO.getASConstraintValue()
   */
  public String GetASConstraintValue(int condition)

  {
    checkInitialized();

    if (!_isAdvancedSearch)
    {
      throw new NotAdvancedSearchException();
    }

    IPTPropertyFilterClauses clause = (IPTPropertyFilterClauses) _advancedFilter.GetPropertyFilter();
    IPTPropertyFilterStatement stmt = (IPTPropertyFilterStatement) clause.GetItem(condition);

    if (PT_FILTEROPS.PT_FILTEROP_ISEMPTY == stmt.GetOperator())
    {
      // "property Foo contains no value"
      return "";
    }

    if (PT_FILTEROPS.PT_FILTEROP_RELATIVE_GTE == stmt.GetOperator())
    {
      // "in the last X", handled above in GetASConstraintOperator
      return "";
    }

    // If it's a date/time
    if (IsDateTimeProperty(stmt.GetOperand())) {
      // Have to initialize this to something non-null for jumpability.
      XPDateTime dtConstraint = new XPDateTime();
      boolean gotDate = false;
      try {
        dtConstraint = stmt.GetValueAsXPDateTime();
        gotDate = true;
      } catch (Exception e) {
        gotDate = false;
      }

      if (gotDate) {
        // Format the date nicely
        XPDateFormat fmt = GetDateFormat(m_asOwner);
        return fmt.Format(dtConstraint);
      }
    }

    // Otherwise, treat as String
    return stmt.GetValueAsString();
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchQueryRO.getASNumPropertyConstraints()
   */
  public int GetASNumPropertyConstraints()

  {
    checkInitialized();

    if (!_isAdvancedSearch)
    {
      throw new NotAdvancedSearchException();
    }

    IPTPropertyFilterItem pfi = _advancedFilter.GetPropertyFilter();

    if (null == pfi)
    {
      return 0;
    }

    return ((IPTPropertyFilterClauses) pfi).GetCount();
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchQueryRO.getASOverallPropertyOperator()
   */
  public boolean GetASPropertyClauseOperator()

  {
    checkInitialized();

    if (!_isAdvancedSearch)
    {
      throw new NotAdvancedSearchException();
    }

    IPTPropertyFilterClauses clause = (IPTPropertyFilterClauses) _advancedFilter.GetPropertyFilter();

    return (clause.GetOperator() == PT_BOOLOPS.PT_BOOLOP_AND);
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchQueryRO.getASTopLevelOperator()
   */
  public boolean GetASTopLevelOperator()

  {
    checkInitialized();

    if (!_isAdvancedSearch)
    {
      throw new NotAdvancedSearchException();
    }

    return (_advancedFilter.GetOperator() == PT_BOOLOPS.PT_BOOLOP_AND);
  }

  /**
   *  @see com.plumtree.portalpages.search.IBaseSearchQueryRO.getAbstractTypeRestrictions()
   */
  public int GetAbstractTypeRestrictions()

  {
    checkInitialized();

    return (_request.GetSettingsAsInt(PT_SEARCH_SETTING.PT_SEARCHSETTING_ABSTYPES));
  }

  /**
   *  @see com.plumtree.portalpages.search.IBaseSearchQueryRO.getAdminFolderRestrictions()
   */
  public int[] GetAdminFolderRestrictions()

  {
    checkInitialized();

    return objectArrayToIntArray(_request.GetSettingsAsObject(
        PT_SEARCH_SETTING.PT_SEARCHSETTING_ADMINFOLDERS));
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchQueryRO.isAdvancedSearch()
   */
  public boolean isAdvancedSearch()

  {
    checkInitialized();

    return _isAdvancedSearch;
  }

  /**
   *  @see com.plumtree.portalpages.search.IBaseSearchQueryRO.getApplicationRestrictions()
   */
  public int GetApplicationRestrictions()

  {
    checkInitialized();

    return (_request.GetSettingsAsInt(PT_SEARCH_SETTING.PT_SEARCHSETTING_APPS));
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getBreadcrumbKey()
   */
  public String GetBreadcrumbKey(int step)

  {
    checkInitialized();

    String key = ancestorKey(_currentKey, step - 1);

    // If the default grouping is, say, "ancestors", then the breadcrumb should
    // lead the user back to a search that's grouped that way
    if (null != _initialGroupBy) {
      key += _initialGroupBy;
    }

    return key;
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getBreadcrumbLabel()
   */
  public String GetBreadcrumbLabel(int step)

  {
    checkInitialized();

    return (String) _crumbCache.GetElement(ancestorKey(_currentKey, step -
        1));
  }

  /**
   *  @see com.plumtree.portalpages.search.IBaseSearchQueryRO.getCommunityNameForID()
   */
  public String GetCommunityNameForID(int id)
  {
    try
    {
      IPTQueryResult ptQR = _session.GetCommunities().QuerySingleObject(id);

      if (null != ptQR && ptQR.RowCount() > 0)
      {
        String result = ptQR.ItemAsString(0, PT_PROPIDS.PT_PROPID_NAME);

        if (null != result)
        {
          return result;
        }
      }
    }
    catch (Exception e)
    {
      
        log.Error(e, "Error in GetCommunityNameForID()");
      
    }

    return m_asOwner.GetString(338, "ptmsgs_portalinfrastructure");
  }

  /**
   *  @see com.plumtree.portalpages.search.IBaseSearchQueryRO.GetPortletNameForID()
   */
  public String GetPortletNameForID(int id)
  {
    try
    {
      IPTQueryResult ptQR = _session.GetGadgets().QuerySingleObject(id);

      if (null != ptQR && ptQR.RowCount() > 0)
      {
        String result = ptQR.ItemAsString(0, PT_PROPIDS.PT_PROPID_NAME);

        if (null != result)
        {
          return result;
        }
      }
    }
    catch (Exception e)
    {
      
        log.Error(e, "Error in GetPortletNameForID()");
      
    }

    return m_asOwner.GetString(451, "ptmsgs_portalinfrastructure");
  }

  /**
   *  @see com.plumtree.portalpages.search.IBaseSearchQueryRO.getCommunityRestrictions()
   */
  public int[] GetCommunityRestrictions()

  {
    checkInitialized();

    return objectArrayToIntArray(_request.GetSettingsAsObject(
        PT_SEARCH_SETTING.PT_SEARCHSETTING_COMMUNITY_ID));
  }

  /**
   *  @see com.plumtree.portalpages.search.IBaseSearchQueryRO.getPortletRestrictions()
   */
  public int[] GetPortletRestrictions()
  {
    checkInitialized();

    return objectArrayToIntArray(_request.GetSettingsAsObject(PT_SEARCH_SETTING.PT_SEARCHSETTING_ASSOC_PORTLET_ID));
  }

  /**
   *  @see com.plumtree.portalpages.search.IBaseSearchQueryRO.getCollabProjectRestrictions()
   */
  public int[] GetCollabProjectRestrictions()
  {
    checkInitialized();

    return objectArrayToIntArray(_request.GetSettingsAsObject(PT_SEARCH_SETTING.PT_SEARCHSETTING_ASSOC_COLLAB_PROJECT_ID));
  }

	/**
	 * Return the stylesheet for the community; used when processing the style
	 * sheet.
	 */
	public String GetStyleSheet(int nCommunityID)
	{
		try
		{
			IPTSession ptSession = (IPTSession) m_asOwner.GetUserSession();
			IPTCommunityManager ptOM     = (IPTCommunityManager) ptSession.GetCommunities();
			IPTCommunityInfo objCommunityInfo  = ptOM.CachedOpenCommunityInfo(nCommunityID, false);
			
		return (objCommunityInfo).GetStyleSheet();
		}
		catch (Exception e)
		{
			
				log.Error(e, "Error in GetStyleSheet()");
			

			UIError myError = new UIError(UIErrorLevel.FRIENDLY,
				UIErrorScope.CURRENT_USER, e,
				m_asOwner.GetString(1522, "ptmsgs_portalbrowsingmsgs"),
				m_asOwner.GetString(1523, "ptmsgs_portalbrowsingmsgs"));
	
			UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(),
				m_asOwner.GetApplication());
		}
	
		return "";
	}
	
	/**
	 * Return the headerID for the community; used when processing the style
	 * sheet.
	 */
	public int GetHeaderID(int nCommunityID)
	{
		try
		{
			IPTSession ptSession = (IPTSession) m_asOwner.GetUserSession();
			IPTCommunityManager ptOM     = (IPTCommunityManager) ptSession.GetCommunities();
			IPTCommunityInfo objCommunityInfo  = ptOM.CachedOpenCommunityInfo(nCommunityID, false);
			
			return (objCommunityInfo).GetHeaderID();
		}
		catch (Exception e)
		{
			
				log.Error(e, "Error in GetHeaderID()");
			

			UIError myError = new UIError(UIErrorLevel.FRIENDLY,
				UIErrorScope.CURRENT_USER, e,
				m_asOwner.GetString(1520, "ptmsgs_portalbrowsingmsgs"),
				m_asOwner.GetString(1521, "ptmsgs_portalbrowsingmsgs"));

			UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(),
				m_asOwner.GetApplication());
		}

		return 0;
	}

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getCorrectionFrequency()
   */
  public int GetCorrectionFrequency(int misspellingIndex, int correctionIndex)
  {
    checkInitialized();

    return _currentResponse.GetCorrectionFrequency(misspellingIndex,
      correctionIndex);
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getCurrentKey()
   */
  public String GetCurrentKey()
  {
    return _currentKey;
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.hasDrilledDownIntoSingleCategory()
   */
  public boolean hasDrilledDownIntoSingleCategory(String orgType)
  {
    checkInitialized();

    int orgIdx = _currentKey.indexOf(CACHE_KEY_SEPARATOR_STR + orgType);

    // No reorg by this type
    if (-1 == orgIdx) return false;

    int lastSlash = _currentKey.lastIndexOf(CACHE_KEY_SEPARATOR_STR);

    // We've drilled down if there's anything after "/subtype", for example
    return (lastSlash != orgIdx);
  }

  /**
   *  Helper method that returns true if the first drilldown step was into 
   *  the "collab server" object-type category
   */
  private boolean firstDrilldownIsIntoCollabCategory(String theKey)
  {
    if (null == theKey) return false;

    // theKey is something like "/subtype.5/..." or "/ancestors.2/..."
    // we want to return true if it starts with "/subtype.", and the number afterwards
    // (the 5 in this case) corresponds to the "PTCOLLAB" category..

    // this String is "/subtype."
    final String neededPrefix = CACHE_KEY_SEPARATOR_STR + GROUPBY_OBJECTTYPE + ".";

    // if it starts with something else, return false
    if (! theKey.startsWith(neededPrefix)) return false;

    // extract the group number that was selected
    String groupNum = theKey.substring(neededPrefix.length());

    // groupNum is now something like "5/ancestors.2/...", or just "5/"

    // look for the "/" and cut it off, along with anything after it
    int endIdx = groupNum.indexOf(CACHE_KEY_SEPARATOR_CHAR);
    if (-1 != endIdx) {
      groupNum = groupNum.substring(0, endIdx);
    }

    // now try to extract a group number
    int groupId = 0;
    try {
      groupId = XPConvert.ToInteger(groupNum);
    } catch (Exception e) {
      // if anything goes wrong..
      return false;
    }

    // groupId now contains the group number.. so extract the original search
    // response from cache
    IPTSearchResponse firstResponse =
      (IPTSearchResponse) _responseCache.GetElement(CACHE_KEY_SEPARATOR_STR + GROUPBY_OBJECTTYPE);

    // make sure it wasn't the "all others" group
    if ((groupId >= firstResponse.GetNumGroups()) ||
        (groupId < 0)) {
      return false;
    }

    // get the label for the desired group
    String groupLabel = firstResponse.GetGroupLabel(groupId);

    // finally!  if the label was "PTCOLLAB", it's what we want
    return ((null != groupLabel) && (groupLabel.equals("PTCOLLAB")));
  }

  /**
   * 
   *
   * @param m_asOwner 
   * @param classId 
   *
   * @return 
   */
  public static String getDisplayableNameForType(AActivitySpace m_asOwner,
    int classId)
  {
    switch (classId)
    {
    case PT_CLASSIDS.PT_USER_ID:
      return m_asOwner.GetString(298, "ptmsgs_portalinfrastructure");

    case PT_CLASSIDS.PT_USERGROUP_ID:
      return m_asOwner.GetString(299, "ptmsgs_portalinfrastructure");

    case PT_CLASSIDS.PT_AUTHSOURCE_ID:
      return m_asOwner.GetString(300, "ptmsgs_portalinfrastructure");

    case PT_CLASSIDS.PT_PROFILESOURCE_ID:
      return m_asOwner.GetString(301, "ptmsgs_portalinfrastructure");

    case PT_CLASSIDS.PT_CATALOGFOLDER_ID:
      return m_asOwner.GetString(297, "ptmsgs_portalinfrastructure");

    case PT_CLASSIDS.PT_CATALOGCARD_ID:
      return m_asOwner.GetString(296, "ptmsgs_portalinfrastructure");

    case PT_CLASSIDS.PT_ADMIN_FOLDER_ID:
      return m_asOwner.GetString(302, "ptmsgs_portalinfrastructure");

    case PT_CLASSIDS.PT_FILTER_ID:
      return m_asOwner.GetString(303, "ptmsgs_portalinfrastructure");

    case PT_CLASSIDS.PT_SAVEDSEARCH_ID:
      return m_asOwner.GetString(304, "ptmsgs_portalinfrastructure");

    case PT_CLASSIDS.PT_DATASOURCE_ID:
      return m_asOwner.GetString(305, "ptmsgs_portalinfrastructure");

    case PT_CLASSIDS.PT_PROPERTY_ID:
      return m_asOwner.GetString(306, "ptmsgs_portalinfrastructure");

    case PT_CLASSIDS.PT_DOCUMENTTYPE_ID:
     return m_asOwner.GetString(307, "ptmsgs_portalinfrastructure");

    case PT_CLASSIDS.PT_CRAWLER_ID:
     return m_asOwner.GetString(308, "ptmsgs_portalinfrastructure");

    case PT_CLASSIDS.PT_GADGET_ID:
      return m_asOwner.GetString(309, "ptmsgs_portalinfrastructure");

    case PT_CLASSIDS.PT_INVITATION_ID:
      return m_asOwner.GetString(310, "ptmsgs_portalinfrastructure");

    case PT_CLASSIDS.PT_FEDERATEDPORTAL_ID:
      return m_asOwner.GetString(470, "ptmsgs_portalinfrastructure");

    case PT_CLASSIDS.PT_GADGETCONTENTSERVER_ID:
      return m_asOwner.GetString(312, "ptmsgs_portalinfrastructure");

    case PT_CLASSIDS.PT_COMMUNITYTEMPLATE_ID:
      return m_asOwner.GetString(313, "ptmsgs_portalinfrastructure");

    case PT_CLASSIDS.PT_GADGETBUNDLE_ID:
      return m_asOwner.GetString(314, "ptmsgs_portalinfrastructure");

    case PT_CLASSIDS.PT_PAGETEMPLATE_ID:
      return m_asOwner.GetString(315, "ptmsgs_portalinfrastructure");

    case PT_CLASSIDS.PT_WEBSERVICE_ID:
      return m_asOwner.GetString(316, "ptmsgs_portalinfrastructure");

    case PT_CLASSIDS.PT_EXTOPERATION_ID:
      return m_asOwner.GetString(317, "ptmsgs_portalinfrastructure");

    case PT_CLASSIDS.PT_GADGETTEMPLATE_ID:
     return m_asOwner.GetString(318, "ptmsgs_portalinfrastructure");

    case PT_CLASSIDS.PT_JOB_ID:
     return m_asOwner.GetString(319, "ptmsgs_portalinfrastructure");

    case PT_CLASSIDS.PT_COMMUNITY_ID:
      return m_asOwner.GetString(320, "ptmsgs_portalinfrastructure");

    case PT_CLASSIDS.PT_SITEMAPFOLDER_ID:
      return m_asOwner.GetString(321, "ptmsgs_portalinfrastructure");

    case PT_CLASSIDS.PT_PAGE_ID:
      return m_asOwner.GetString(505, "ptmsgs_portalinfrastructure");

    case PT_CLASSIDS.PT_SUBPORTAL_ID:
      return m_asOwner.GetString(519, "ptmsgs_portalinfrastructure");
    }

    return m_asOwner.GetString(339, "ptmsgs_portalinfrastructure");
  }

  /**
   *  @see com.plumtree.portalpages.search.IBaseSearchQueryRO.getDocFolderRestrictions()
   */
  public int[] GetDocFolderRestrictions()
  {
    checkInitialized();

    return objectArrayToIntArray(_request.GetSettingsAsObject(
        PT_SEARCH_SETTING.PT_SEARCHSETTING_DDFOLDERS));
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getDrilldownDepth()
   */
  public int GetDrilldownDepth()
  {
    checkInitialized();

    int depth = (new XPStringTokenizer(_currentKey,
        CACHE_KEY_SEPARATOR_STR, false)).GetTokenCount();

    if (_currentKey.endsWith(CACHE_KEY_SEPARATOR_STR))
    {
      depth++;
    }

    return depth;
  }

  /**
   *  Helper method to check whether groupIdx is actually the
   *  "all others" group
   */
  private boolean isAllOthersGroup(int groupIdx)
  {
    return (groupIdx == _currentResponse.GetNumGroups());
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.isEntireGroupReturned()
   */
  public boolean isEntireGroupReturned(int groupIdx)
  {
    checkInitialized();

    // If it's the "all others" group
    if (isAllOthersGroup(groupIdx)) {
      // always say there is more stuff to see
      return false;
    }

    int returned = GetReturnedResultsInGroup(groupIdx);
    int total    = 0;

    try
    {
      total = XPConvert.ToInteger(GetTotalResultsInGroup(groupIdx));
    }
    catch (Exception e)
    {
      // If the total results string contained something non-numeric,
      // like "~200", then we know not all results were returned
      return false;
    }

    return (returned == total);
  }

  /**
   *  @see com.plumtree.portalpages.search.IBaseSearchQueryRO.getFolderNameForID()
   */
  public String GetFolderNameForID(int id, boolean isAdmin)
  {
    try
    {
      IPTObject ptObj = null;

      if (isAdmin)
      {
        ptObj = _session.GetAdminCatalog().OpenAdminFolder(id, /*lock=*/
            false);
      }
      else
      {
        ptObj = _session.GetCatalog().OpenFolder(id, /*lock=*/
            false);
      }

      if (null != ptObj)
      {
        String name = ptObj.GetLocalizedName();

        if (null != name)
        {
          return name;
        }
      }
    }
    catch (Exception e)
    {
      
        log.Warn(e, "Exception thrown in GetFolderNameForID().  This is often caused by a clickthrough to the search results page from the \"More\" link in a Snapshot Query portlet, and in this case it means that the user does not have rights to search some of the folders specified in the Snapshot Query.  In this case, the warning is harmless.");
      
    }

    return m_asOwner.GetString(337, "ptmsgs_portalinfrastructure");
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getGroupName()
   */
  public String GetGroupName(int groupIdx)
  {
    checkInitialized();
    
    if (isAllOthersGroup(groupIdx)) {
      return m_asOwner.GetString(520, "ptmsgs_portalinfrastructure");
    }
    
    String preLocalizedName= _currentResponse.GetGroupLabel(groupIdx);

    // Folder names are looked up from the database, and if this lookup
    // fails (e.g. because the db and search server are out of sync) then
    // PTSearch will return "Unknown Folder". We need to look for this 
    // and localize it here    
    if (preLocalizedName != null &&
	preLocalizedName.trim().equals(UNKNOWN_FOLDER_NAME) ) {
 
	return m_asOwner.GetString(1605, "ptmsgs_portalbrowsingmsgs");
    }

    String lastGroupBy = GetLastGroupByField();

    // if the last group-by was anything other than OBJECTTYPE, just return the
    // group label we already have
    if ((null == lastGroupBy) ||
        (! (lastGroupBy.equals(GROUPBY_OBJECTTYPE)))) {
      return preLocalizedName;
    }

    return localizeObjectTypeName(preLocalizedName);
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.GetGroupIcon()
   */
  public Object GetGroupIcon(int groupIdx)
  {
    checkInitialized();

    String lastGroupBy = GetLastGroupByField();

    if (null == lastGroupBy) return null;

    if (lastGroupBy.equals(GROUPBY_FOLDER)) {
      // Folder categorization.. return the "open folder" icon
      PTConstruct c = UIConstructFactory.GetConstruct(PTImageType.ICON_FOLDER,
                                                      true, m_asOwner);
      if (null != c) {
        // See bug 35590.  PTConstruct doesn't put an alt-tag on this icon, so we
        // have to do it here.  This means downcasting the HTMLElement to an HTMLImg.
        // Apparently this is done elsewhere in the portal code, too.
        HTMLImg img = (HTMLImg) c.GetHTMLElement();
        img.SetAlt(m_asOwner.GetString(821, "ptmsgs_portalbrowsingmsgs"));
        return img;
      } else {
        return null;
      }
    }

    // For any other type of categorization except object type, return no icon
    if (! lastGroupBy.equals(GROUPBY_OBJECTTYPE)) {
      return null;
    }

    // Under object type categorization, return no icon for "all others"
    if (isAllOthersGroup(groupIdx)) {
      return null;
    }

    // Otherwise look up the icon using the group name

    String label = _currentResponse.GetGroupLabel(groupIdx);

    if (! label.startsWith("PT")) {
      // Non-portal or unknown type
      return null;
    }

    ClassTypeDescManager cdm = PTConfigHelper.GetPTClassTypeDescrManager();
	String strLangID = (String) PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.LANGUAGE, m_asOwner);

    if (null != cdm) {
      if (label.equals("PTCARD")) return cdm.GetDefaultIconForClassType(PT_CLASSIDS.PT_CATALOGCARD_ID, strLangID);
      if (label.equals("PTFOLDER")) return cdm.GetDefaultIconForClassType(PT_CLASSIDS.PT_CATALOGFOLDER_ID, strLangID);
      if (label.equals("PTGADGET")) return cdm.GetDefaultIconForClassType(PT_CLASSIDS.PT_GADGET_ID, strLangID);
      if (label.equals("PTCOMMUNITY")) return cdm.GetDefaultIconForClassType(PT_CLASSIDS.PT_COMMUNITY_ID, strLangID);
      if (label.equals("PTUSER")) return cdm.GetDefaultIconForClassType(PT_CLASSIDS.PT_USER_ID, strLangID);
      if (label.equals("PTPAGE")) return cdm.GetDefaultIconForClassType(PT_CLASSIDS.PT_PAGE_ID, strLangID);

      // What icons should we use for Collab and Content?
      if (label.equals("PTCOLLAB")) return cdm.GetDefaultIconForClassType(PT_CLASSIDS.PT_CATALOGCARD_ID, strLangID);
      if (label.equals("PTCONTENT")) return cdm.GetDefaultIconForClassType(PT_CLASSIDS.PT_CATALOGCARD_ID, strLangID);

      if (label.equals("PTGROUP")) return cdm.GetDefaultIconForClassType(PT_CLASSIDS.PT_USERGROUP_ID, strLangID);
      if (label.equals("PTAUTHSOURCE")) return cdm.GetDefaultIconForClassType(PT_CLASSIDS.PT_AUTHSOURCE_ID, strLangID);
      if (label.equals("PTPROFILESOURCE")) return cdm.GetDefaultIconForClassType(PT_CLASSIDS.PT_PROFILESOURCE_ID, strLangID);
      if (label.equals("PTADMINFOLDER")) return cdm.GetDefaultIconForClassType(PT_CLASSIDS.PT_ADMIN_FOLDER_ID, strLangID);
      if (label.equals("PTFILTER")) return cdm.GetDefaultIconForClassType(PT_CLASSIDS.PT_FILTER_ID, strLangID);
      if (label.equals("PTSAVEDSEARCH")) return cdm.GetDefaultIconForClassType(PT_CLASSIDS.PT_SAVEDSEARCH_ID, strLangID);
      if (label.equals("PTDATASOURCE")) return cdm.GetDefaultIconForClassType(PT_CLASSIDS.PT_DATASOURCE_ID, strLangID);
      if (label.equals("PTCARDPROPERTY")) return cdm.GetDefaultIconForClassType(PT_CLASSIDS.PT_PROPERTY_ID, strLangID);
      if (label.equals("PTDOCUMENTTYPE")) return cdm.GetDefaultIconForClassType(PT_CLASSIDS.PT_DOCUMENTTYPE_ID, strLangID);
      if (label.equals("PTCRAWLER")) return cdm.GetDefaultIconForClassType(PT_CLASSIDS.PT_CRAWLER_ID, strLangID);
      if (label.equals("PTINVITATION")) return cdm.GetDefaultIconForClassType(PT_CLASSIDS.PT_INVITATION_ID, strLangID);
      if (label.equals("PTFEDPORTAL")) return cdm.GetDefaultIconForClassType(PT_CLASSIDS.PT_FEDERATEDPORTAL_ID, strLangID);
      if (label.equals("PTGADGETSERVER")) return cdm.GetDefaultIconForClassType(PT_CLASSIDS.PT_GADGETCONTENTSERVER_ID, strLangID);
      if (label.equals("PTCOMMTEMPLATE")) return cdm.GetDefaultIconForClassType(PT_CLASSIDS.PT_COMMUNITYTEMPLATE_ID, strLangID);
      if (label.equals("PTGADGETBUNDLE")) return cdm.GetDefaultIconForClassType(PT_CLASSIDS.PT_GADGETBUNDLE_ID, strLangID);
      if (label.equals("PTPAGETEMPLATE")) return cdm.GetDefaultIconForClassType(PT_CLASSIDS.PT_PAGETEMPLATE_ID, strLangID);
      if (label.equals("PTWEBSERVICE")) return cdm.GetDefaultIconForClassType(PT_CLASSIDS.PT_WEBSERVICE_ID, strLangID);
      if (label.equals("PTEXTOPERATION")) return cdm.GetDefaultIconForClassType(PT_CLASSIDS.PT_EXTOPERATION_ID, strLangID);
      if (label.equals("PTGADGETTEMPLATE")) return cdm.GetDefaultIconForClassType(PT_CLASSIDS.PT_GADGETTEMPLATE_ID, strLangID);
      if (label.equals("PTJOB")) return cdm.GetDefaultIconForClassType(PT_CLASSIDS.PT_JOB_ID, strLangID);
      if (label.equals("PTSITEMAP")) return cdm.GetDefaultIconForClassType(PT_CLASSIDS.PT_SITEMAPFOLDER_ID, strLangID);
      if (label.equals("PTSUBPORTAL")) return cdm.GetDefaultIconForClassType(PT_CLASSIDS.PT_SUBPORTAL_ID, strLangID);
    }

    // If still unknown, use no icon
    return null;
  }

  /**
   *  Special-case method for use only by DrilldownView.  Not exposed through ISearchResultModelRO
   *  interface.
   *
   *  Returns the set of groups that was generated for some ancestor of the drilled-down
   *  response currently being shown.  This is used to generate a view that lets the user
   *  jump to a sibling of the current response.  For example, if the user has drilled down
   *  into the "Documents" category, such a view might let the user jump over to the (sibling)
   *  "Portlets" category.
   *
   *  For now, this method only works if the requested ancestor was grouped by object type.
   */
  public XPHashtable GetGroupsForAncestorKey(String ancestorKey)
  {
    checkInitialized();

    // Check whether oldKey is an ancestor of the current key.
    if (! XPStringUtility.StartsWith(_currentKey, ancestorKey)) {
      throw new SearchModelException("In SearchResultModel.GetGroupsForAncestorKey, provided key is not an ancestor of the current key");
    }

    // Check that the requested ancestor used object-type categorization.
    if (! XPStringUtility.EndsWith(ancestorKey, GROUPBY_OBJECTTYPE)) {
      throw new SearchModelException("SearchResultModel.GetGroupsForAncestorKey currently only supports ancestors that were grouped by object type.");
    }

    // Fetch the old response.
    IPTSearchResponse ancestorResponse = (IPTSearchResponse) _responseCache.GetElement(ancestorKey);
    if (null == ancestorResponse) {
      throw new CacheFlushedException(m_asOwner);
    }

    XPHashtable groups = new XPHashtable();

    int numGroups = ancestorResponse.GetNumGroups();
    for (int j = 0; j < numGroups; j++) {
      String rawName = ancestorResponse.GetGroupLabel(j);
      groups.PutElement(localizeObjectTypeName(rawName), new Integer(j));
    }

    return groups;
  }

  /**
   *  Special-case method for use only by DrilldownView.  Not exposed through ISearchResultModelRO
   *  interface.
   *
   *  Returns the ID of the last group we drilled down into.  For example, if the initial
   *  search was grouped by Object Type, and the user selected Portlets (group 3) and drilled down,
   *  this would return "3".
   */
  public String GetLastDrilldownGroup()
  {
    int lastSlash = _currentKey.lastIndexOf(CACHE_KEY_SEPARATOR_STR);
    String keyFragment = XPStringUtility.Substring(_currentKey, 0, lastSlash);
    int lastDot = keyFragment.lastIndexOf('.');
    return XPStringUtility.Substring(keyFragment, lastDot+1);
  }

  /**
   *  @see com.plumtree.portalpages.search.ISaveSearchModelRO.GetHasBeenSaved()
   */
  public boolean GetHasBeenSaved()
  {
    return (_savedSearchName != null);
  }

  /**
   *  @see com.plumtree.portalpages.search.IBaseSearchQueryRO.getIncludeSubfolders()
   */
  public boolean GetIncludeSubfolders()
  {
    checkInitialized();

    return (XPConvert.ToBoolean((_request.GetSettingsAsObject(PT_SEARCH_SETTING.PT_SEARCHSETTING_INCLUDE_SUBFOLDERS))));
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getKWIC()
   */
  public String GetKWIC(int rank)
  {
    checkInitialized();

                // Note PTSearch actually checks for a KWIC for the content field
                // if there is no KWIC for PT_PROPERTY_OBJECTSUMMARY.
                // Also, the result of GetKWICFields is always html-encoded.

                String kwic = _currentResponse.GetKWICFieldsAsString(rank,
                                           PT_INTRINSICS.PT_PROPERTY_OBJECTSUMMARY);

                if ((null != kwic) && (kwic.length() > 0)) {
                  return kwic;
                }

                // If no KWIC, simply return the entire summary field,
                // but HTML-encode it to match behavior of GetKWICFields.

                return XPHttpUtility.HtmlEncode(_currentResponse.GetFieldsAsString(rank,
                                           PT_INTRINSICS.PT_PROPERTY_OBJECTSUMMARY));
  }

  /**
   *  @see com.plumtree.portalpages.search.IBaseSearchQueryRO.getLanguageRestriction()
   */
  public String GetLanguageRestriction()
  {
    checkInitialized();

    boolean wasRestricted = (XPConvert.ToBoolean((_request.GetSettingsAsObject(PT_SEARCH_SETTING.PT_SEARCHSETTING_LIMITBYLOCALE))));
    //boolean wasRestricted = (_request.GetSettingsAsInt(PT_SEARCH_SETTING.PT_SEARCHSETTING_LIMITBYLOCALE) != 0);

    if (wasRestricted)
    {
      return ((_request.GetSettingsAsString(PT_SEARCH_SETTING.PT_SEARCHSETTING_LANGUAGE)));
    }

    return null;
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getLastGroupByField()
   **/
  public String GetLastGroupByField()
  {
   	String key = GetLastGroupByFieldForKey(_currentKey);
	return key;
  }

  /**
   *  @see com.plumtree.portaluiinfrastructure.search.ISearchResultModel.SetPageMode()
   */
  public void SetPageMode(String pageMode)
  {
    _pageMode = pageMode;
  }

  /**
   *  @see com.plumtree.portaluiinfrastructure.search.ISearchResultModelRO.GetPageMode()
   */
  public String GetPageMode()
  {
    return _pageMode;
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getMisspelledToken()
   */
  public String GetMisspelledToken(int misspellingIndex)
  {
    checkInitialized();

    return _currentResponse.GetMisspelledToken(misspellingIndex);
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getNumCorrections()
   */
  public int GetNumCorrections(int misspellingIndex)
  {
    checkInitialized();

    return _currentResponse.GetNumCorrections(misspellingIndex);
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getNumMisspelledTokens()
   */
  public int GetNumMisspelledTokens()
  {
    checkInitialized();

    return _currentResponse.GetNumMisspelledTokens();
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getNumResultGroups()
   */
  public int GetNumResultGroups()
  {
    checkInitialized();

    // Get number of actual groups
    int numGroups = _currentResponse.GetNumGroups();

    // If there was already at least one group
    if (numGroups > 0) {

      // Put everything else into its own group
      if (allOthersGroupExists()) numGroups++;
    }

    return numGroups;
  }

  /**
   *  Helper method to figure out whether there is an "all others" group.
   */
  private boolean allOthersGroupExists() 
   {
    if (((String) _currentResponse.GetTotalResultsNotInAnyGroup()).equals("0") ||
        ((String) _currentResponse.GetTotalResultsNotInAnyGroup()).equals("~ 0")) {
      return false;
    }
    return true;
  }

  /**
   *  @see com.plumtree.portalpages.search.IBaseSearchQueryRO.getObjectTypeRestrictions()
   */
  public int[] GetObjectTypeRestrictions()
  {
    checkInitialized();

    return objectArrayToIntArray(_request.GetSettingsAsObject(
        PT_SEARCH_SETTING.PT_SEARCHSETTING_OBJTYPES));
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getReconstructionArgs()
   */
  public XPHashtable GetReconstructionArgs()
  {
    return GetReconstructionArgs(true);
  }
  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getReconstructionArgs()
   */
  public XPHashtable GetReconstructionArgs(boolean wantPageContext)
  {
	return GetReconstructionArgs(SearchResultControl.RECONSTRUCTION_ARGS_PREFIX, wantPageContext);
  }
  
  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getReconstructionArgs()
   */
  public XPHashtable GetReconstructionArgs(String prefix, boolean wantPageContext)
  {
	  // TODO: This is called once per link on the advanced search page.  It would be
	  // nice if we could cache this so we only do the DB query once per page instead
	  // of doing an identical query 10x on every page.  Bug#67468
    IXPCollection exclusions = null;

    if (!wantPageContext)
    {
      exclusions = SearchSettingCollection.GetPageContextSettings();
    }

    XPHashtable args = new XPHashtable();

    internalGetArgs(args, prefix,
      
    /* settingsArgs= */ true, 
    /* includedSettings= */ null, 
    /* excludedSettings= */ exclusions, 
    /* queryArgs= */ true);

    return args;
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getResultAbsType()
   */
  public String GetResultAbsType(int rank)
  {
    checkInitialized();

    return _currentResponse.GetResultAbsType(rank);
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getResultApp()
   */
  public String GetResultApp(int rank)
  {
    checkInitialized();

    return _currentResponse.GetResultApp(rank);
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getResultName()
   */
  public boolean isResultBestBet(int rank)
  {
    checkInitialized();

    return _currentResponse.GetIsBestBet(rank);
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getResultBestBetKey()
   */
  public String GetResultBestBetKey(int rank)
  {
    checkInitialized();

    String app = GetResultApp(rank);

    // if a portal item
    if (app.equals(PORTAL_APP))
    {
      return (GetResultClassID(rank) + "." + GetResultObjectID(rank));
    }

    return _currentResponse.GetResultInternalField(rank,
      PT_SEARCH_INTERNAL_FIELD.PT_SEARCH_IF_BESTBETKEY);
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getResultClassID()
   */
  public int GetResultClassID(int rank)
    
  {
    checkInitialized();

    int classId = -1;

    try {
      classId = _currentResponse.GetFieldsAsInt(rank, PT_INTRINSICS.PT_PROPERTY_CLASSID);
    } catch (Exception e) {
      // ClassID is not defined for non-portal objects.
      classId = -1;
    }

    return classId;
  }
  
  /**
   *  Returns an edit link that may be used for clickthrough.
   */
  public HTMLAnchor GetResultEditLink(int rank, boolean allowPopup) throws SearchModelException
  {
    return GetResultClickthroughURL(rank, allowPopup, /* editMode= */ true);
  }

  /**
   *  Returns a view link that may be used for clickthrough.
   */
  public HTMLAnchor GetResultClickthroughURL(int rank, boolean allowPopup)
  {
    return GetResultClickthroughURL(rank, allowPopup, /* editMode= */ false);
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getResultClickthroughURL()
   */
  public HTMLAnchor GetResultClickthroughURL(int rank, boolean allowPopup, boolean editMode)
  {
    return GetResultClickthroughURL(rank, allowPopup, editMode, /* extraParams= */ null);
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getResultClickthroughURL()
   */
  public HTMLAnchor GetResultClickthroughURL(int rank, boolean allowPopup, boolean editMode, String extraParams)
  {
    checkInitialized();

    String app = GetResultApp(rank);

	if (app.equals(PORTAL_APP))
    {
        int nClassID = GetResultClassID(rank);
        int nObjectID = GetResultObjectID(rank);
        String strObjectName = GetResultName(rank);

        if (allowPopup && (PT_CLASSIDS.PT_GADGET_ID == nClassID)) 
        {
            // Open gadget preview, in a popup whenever it's possible
            HTMLAnchor anchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);

            String strOptQSArgs = "&" + ASConstants.SUBSPACE + "=" + "mySubspaceKey"; 

            // Add extra parameters if requested
            if(null != extraParams)
            {
                strOptQSArgs = strOptQSArgs + "&" + extraParams;
            }
            
            anchor.SetOnClick(
				PTOpenerLinks.GetJSOpenInNewWindow(
					PTOpenerLinks.GetOpenerJSURLOpenObjID(nClassID, nObjectID, strOptQSArgs, 
                        (editMode ? IPTOpenerConstants.OPENER_MODE_EDIT : IPTOpenerConstants.OPENER_MODE_VIEW)),
					PTOpenerLinks.GetWindowName("" + nClassID + "_" + nObjectID), 
                    "450", "450", false));
                
            return anchor;
        }
        else
        {
            // Otherwise, make a plain opener URL

            // Add extra parameters if requested
            String strOptQSArgs = null;
            if(null != extraParams)
            {
                strOptQSArgs = "&" + extraParams;
            }

            return new HTMLAnchor(PTOpenerLinks.GetOpenerURLOpenObjID(nClassID, nObjectID, strObjectName,
                     (editMode ? IPTOpenerConstants.OPENER_MODE_EDIT : IPTOpenerConstants.OPENER_MODE_VIEW),
                     m_asOwner, m_asOwner.GetName(), m_asOwner.GetSpaceID(), strOptQSArgs));
        }
    }
    else
    {
      // for Collab and Content, the URL is stored in field PT5
      String url = _currentResponse.GetFieldsAsString(rank,
          PT_INTRINSICS.PT_PROPERTY_DOCUMENTURL);

      int gatewayObjectId = 0;
      int gatewayClassId = PT_CLASSIDS.PT_GADGET_ID;
      String appPrefix = "";

      if (app.equals(COLLAB_APP))
      {
        gatewayObjectId = GetCollabPortletOrWebServiceID();
        appPrefix = GetCollabURLPrefix();
        if (gatewayObjectId < 0) {
          gatewayClassId = PT_CLASSIDS.PT_WEBSERVICE_ID;
          gatewayObjectId = -1 * gatewayObjectId;
        }
      }
      else
      {
      	//SE: VV- Bug 54592 (Jatco): Checking if multiple portlets are associated to a single
      	//Content Item. If there are multiple portlets associated, then we use the web service id
      	//as gatewayClassId. If there's just one portlet, use the gadget id as gatewayClassId.

      	if (HasMultiplePortlets(rank)) {
		  	// There are multiple portlets so use web service id.	
	        String contentWSID = (String) (_currentResponse.GetResultInternalField(rank,
	               PT_SEARCH_INTERNAL_FIELD.PT_SEARCH_IF_CONTENT_WS_ID));
	
	        gatewayClassId = PT_CLASSIDS.PT_WEBSERVICE_ID;
	
	        if ((null != contentWSID) && (0 < contentWSID.length())) {
	          try {
	            gatewayObjectId = XPConvert.ToInteger(contentWSID);
	          } catch (Exception e) {
	            gatewayObjectId = 0;
	          }
	        }
      	} else {
      		// There's only one portlet so use gadget id
			int [] gatewayObjectIds = GetResultContentPortletIDs(rank);
		    try {
		    	gatewayObjectId = gatewayObjectIds[0];
		    } catch (Exception e) {
		    	gatewayObjectId = 0;
		    }
      	}
      }

      // Collab and Content URLs need to be gatewayed
      String gatewayPfx = GatewayHelpers.ConstructPrefPageLink(m_asOwner,
          
        /* objectID= */ gatewayObjectId, 
        /* communityID= */ 0,
        /* pageID= */ 0,
        /* prefType= */ 0,
        /* classID= */ gatewayClassId);

      if (null != extraParams) {
        // Stick the extra parameters on the end of the gatewayed URL.  It's not clear
        // that this is the best way to accomplish this, but at least we shouldn't make
        // an invalid URL this way.

        if (-1 == XPStringUtility.IndexOf(url, "?")) {
          url = url + "?" + extraParams;
        } else {
          url = url + "&" + extraParams;
        }
      }
      
      String strHref;

      // SE: Bug 62927 - Fix for Sumitomo. Avoid double-encoding publishedcontent URLs that 
      // we get from the search index. We are assuming here that these URLs are always encoded.
      // Note: Prior to this fix, the double-encoded URL does work if there's a web server front-end
      // because it un-encodes the '%'s before passing the request through. However if the user did 
      // a click-through directly against the appserver (Tomcat, Weblogic), they will encounter a 404.
      if (app.equals(CONTENT_APP))
      {
    	  // A published content URL with multi-byte characters encoded looks like the following:
    	  // http://se611dnet.devnet.plumtree.com/portal/server.pt/gateway/PTARGS_0_0_334_0_0_43/http://se611dnet-cs640.devnet.plumtree.com:7087/publishedcontent/publish/kanji_test_%E7%94%B8/%E8%A8%98%E4%BA%8B/kanji_test_%E7%94%B8.html
    	  //
    	  // We need to make this URL "gateway-friendly" but avoid re-encoding the '%' in the URL. So we should
    	  // only pass the base URL into GatewayHelpers.CreateGatewayFriendlyURL and leave the rest of the
    	  // URL alone as that's already been encoded.
    	  
    	  // Find the end index of the base URL
    	  int iGatewayBaseURLEndIndex = XPStringUtility.ToLowerCase(url).indexOf("://");
    	  iGatewayBaseURLEndIndex =  XPStringUtility.ToLowerCase(url).indexOf("/", iGatewayBaseURLEndIndex + 3 /* move past "://" */);
    	  String strGatewayBaseURL = "";
    	  String strGatewaySfx = "";
    	      	  
    	  if (-1 != iGatewayBaseURLEndIndex)
    	  {
	    	  // Get the base URL, i.e. http://<server>:<port>/
	    	  strGatewayBaseURL = url.substring(0, iGatewayBaseURLEndIndex);
	    	  log.Debug("SearchResultModel::getResultClickthroughURL - strGatewayBaseURL = " + strGatewayBaseURL);
	    	  // The rest of the URL
	    	  strGatewaySfx = url.substring(iGatewayBaseURLEndIndex);
	    	  log.Debug("SearchResultModel::getResultClickthroughURL - strGatewaySfx = " + strGatewaySfx);
		      
	    	  // Make the base gateway URL gateway friendly
	    	  strGatewayBaseURL = GatewayHelpers.CreateGatewayFriendlyURL(appPrefix + strGatewayBaseURL);
    	  }
		      
    	  // Now reconstruct the fully encoded gateway URL
    	  strHref = gatewayPfx + strGatewayBaseURL + strGatewaySfx;
    	  log.Debug("SearchResultModel::getResultClickthroughURL - strHref = " + strHref);
	    	  
      }
      else
      {
    	  strHref = gatewayPfx + GatewayHelpers.CreateGatewayFriendlyURL(appPrefix + url);
      }
      
      return new HTMLAnchor(strHref);
    }
  }
  
  //SE: VV- Bug 54592 (Jatco):This is the actual method that checks if there are multiple
  //portlets. If there is more than 1 token in String strMultiplePortletIDs, then there are
  //multiple portlets, if not, there's just one portlet.
  private boolean HasMultiplePortlets(int rank) {
  	String strMultiplePortletIDs = (String) (_currentResponse.GetResultInternalField(rank,
  			PT_SEARCH_INTERNAL_FIELD.PT_SEARCH_IF_CONTENT_PORTLET_ID));
  	XPStringTokenizer st = new XPStringTokenizer(strMultiplePortletIDs);
	if(st.GetTokenCount() > 1){
		return true;
	} else{
		return false;
	}
  }
  
  /**
   *  @see com.plumtree.portaluiinfrastructure.search.ISearchResultModelRO.GetResultCollabProjectName()
   */
  public String GetResultCollabProjectName(int rank)
  {
    checkInitialized();

    String projName = _currentResponse.GetResultInternalField(rank,
        PT_SEARCH_INTERNAL_FIELD.PT_SEARCH_IF_COLLAB_PROJ_NAME);

    if ((null == projName) || (0 == projName.length())) {
      return null;
    }

    return projName;
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getResultCollabProjectURL()
   */
  public String GetResultCollabProjectURL(int rank)
  {
    checkInitialized();

    String url =  _currentResponse.GetResultInternalField(rank,
        PT_SEARCH_INTERNAL_FIELD.PT_SEARCH_IF_COLLAB_PROJ_URL);

    if ((null != url) && (url.length() > 0))
    {
      int gatewayObjectId = GetCollabPortletOrWebServiceID();
      int gatewayClassId = PT_CLASSIDS.PT_GADGET_ID;
      if (gatewayObjectId < 0) {
        gatewayClassId = PT_CLASSIDS.PT_WEBSERVICE_ID;
        gatewayObjectId = -1 * gatewayObjectId;
      }

      String gatewayPfx = GatewayHelpers.ConstructPrefPageLink(m_asOwner,
          
        /* objectID= */ gatewayObjectId,
        /* communityID= */ 0,
        /* pageID= */ 0,
        /* prefType= */ 0,
        /* classID= */ gatewayClassId);

      return gatewayPfx + GatewayHelpers.CreateGatewayFriendlyURL(GetCollabURLPrefix() + url);
    }
    else
    {
      return null;
    }
  }

  /**
   *  @see com.plumtree.portalpages.search.IBaseSearchResultModelRO.GetResultPortletField()
   */
  public String GetResultPortletField(int rank, int fieldId)
    
  {
    checkInitialized();

    boolean ok =
      ((PT_SEARCH_INTERNAL_FIELD.PT_SEARCH_IF_GADGET_TYPE == fieldId) ||
       (PT_SEARCH_INTERNAL_FIELD.PT_SEARCH_IF_GADGET_ALIGNMENT == fieldId) ||
       (PT_SEARCH_INTERNAL_FIELD.PT_SEARCH_IF_GADGET_USER_INTERFACES == fieldId));

    if (! ok) {
      throw new SearchModelException(
        "SearchResultModel.GetResultPortletField does not support field " +
        fieldId);
    }

    return _currentResponse.GetResultInternalField(rank, fieldId);
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getResultContentLastPubUser()
   */
  public String GetResultContentLastPubUser(int rank)
    
  {
    checkInitialized();

    return _currentResponse.GetResultInternalField(rank,
      PT_SEARCH_INTERNAL_FIELD.PT_SEARCH_IF_CONTENT_PUBLISHED_USER);
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getResultContentPortletIDs()
   */
  public int[] GetResultContentPortletIDs(int rank)
  {
    checkInitialized();

    String s = _currentResponse.GetResultInternalField(rank,
        PT_SEARCH_INTERNAL_FIELD.PT_SEARCH_IF_CONTENT_PORTLET_ID);

    if ((null == s) || (0 == s.length())) {
      return null;
    }

    XPArrayList al = new XPArrayList();

    XPStringTokenizer tok = new XPStringTokenizer(s);
    while (tok.HasMoreTokens()) {
      String oneID = tok.GetNextToken();

      int id = -1;
      try {
        id = XPConvert.ToInteger(oneID);
      } catch (Exception e) {
        log.Warn("Unable to parse Content Server portlet ID: " + oneID + ".  Please report this error to Plumtree.");
        id = -1;
      }

      if (-1 != id) {
        al.Add(new Integer(id));
      }
    }

    if (0 == al.GetSize()) {
      return null;
    }

    int[] ids = new int[al.GetSize()];
    for (int j = 0; j < al.GetSize(); j++) {
      ids[j] = XPConvert.ToInteger(al.GetElement(j));
    }
    return ids;
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getResultCreated()
   */
  public XPDateTime GetResultCreated(int rank)
    
  {
    checkInitialized();

    return ConvertSearchDateTimeToXPDateTime(_currentResponse.GetFieldsAsDouble(rank,
                                             PT_INTRINSICS.PT_PROPERTY_OBJECTCREATED));
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getResultField()
   */
  public String GetResultEODField(int rank, int propertyId)
    
  {
    checkInitialized();

    return _currentResponse.GetFieldsAsString(rank, propertyId);
  }

  public Object GetResultEODFieldAsObject(int rank, int propertyId)  
  {
	checkInitialized();

	return _currentResponse.GetFieldsAsObject(rank, propertyId);
  }
  
  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getResultFolderID()
   */
  public int GetResultFolderID(int rank)
    
  {
    // Eventually, we should make a full-fledged GetResultFolderID method
    // on PTSearchResponse..
    String parentIDField = _currentResponse.GetResultInternalField(rank,
        PT_SEARCH_INTERNAL_FIELD.PT_SEARCH_IF_PARENTIDS);

    if ((null != parentIDField) && (parentIDField.length() > 2))
    {
      // Some cards have more than one parent, space-separated.  If so,
      // do not return any of them - we can't be sure which one the user
      // has rights to see without doing some expensive database queries.
      // We may want to revisit this for a future release.
      int firstSpace = parentIDField.trim().indexOf(' ');

      // If a space in the parentids field, meaning more than one parent..
      if (-1 != firstSpace)
      {
        // return 0 for "unknown parent"
        return 0;
      }

      try
      {
        // The parentid field is of the form od123 or dd123, so look at
        // the third character and after
        return XPConvert.ToInteger(parentIDField.substring(2));
      }
      catch (Exception e)
      {
        throw XPException.GetInstance(e);
      }
    }

    return 0;
  }

  public void ExternalLightInitialization(IPTSearchResponse sr, AActivitySpace asSpace)
  {
  	_currentResponse = sr;
  	_initialized = true;
  	m_asOwner = asSpace;
  }
  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getResultIcon()
   */
  public Object GetResultIcon(int rank)
  {
    checkInitialized();

    HTMLImg icon = null;

    int classId = GetResultClassID(rank);

    if (-1 != classId)
    {
      // Portal objects.

      String altTag = null;
      String strLangID = (String) PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.LANGUAGE, m_asOwner);

      if (PT_CLASSIDS.PT_CATALOGCARD_ID == classId)
      {
        // CARDS: Use the icon associated with the card's actual file type.
        String uuid = GetResultTypeUUID(rank);

        // If the image UUID is defined
        if ((null != uuid) && (0 < uuid.length()))
        {
          // get the image for this UUID
          icon = PlumtreeHelpers.GetHTMLImage(uuid, /* smallImage= */
              true);

          // See if we can get a tooltip based on the UUID.  Use the PTDocTypesVarPack.
          IApplication application = ApplicationManager.GetInstance()
            .GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString());
          if (null != application) {
            PTDocTypesVarPack varPack = (PTDocTypesVarPack) application.GetVarPackManager()
              .GetVariablePackage(PTDocTypesVarPack.VARPACK_ID);
            if (null != varPack) {
              altTag = varPack.GetLocalizedName(uuid, strLangID);
            }
          }
        }
      }

      if (null == icon)
      {
        // If we couldn't get an icon corresponding to the UUID, use the default
        // icon for the item's type.
        ClassTypeDescManager cdm = PTConfigHelper.GetPTClassTypeDescrManager();

        if (null != cdm)
        {
          icon = cdm.GetDefaultIconForClassType(classId, strLangID);
        }
      }

      if (null != icon)
      {
        if ((null == altTag) || (0 == altTag.length())) {
          String subType = GetResultSubType(rank);
          altTag = localizeObjectTypeName(subType);
        }

        icon.SetAlt(altTag);
      }
    }
    else
    {
      // The URL for collab/content/templating icons are stored in field PT53
      String indexedURL = (_currentResponse.GetFieldsAsString(rank,
          PT_INTRINSICS.PT_PROPERTY_OBJECTIMAGEURL));

    //relative, so we need to append the image server base url. We truly need the 
    //base folder itself. Simply calling ConfigHelper.GetImageBaseURL() will
    //append /plumtree on the end of the base folder, which we don't want, since the 
    //network search result images don't necessarily live under the plumtree
    //folder. That's why we have to do this
      boolean isSecure = m_asOwner.GetIsSecuredSpace();
      String imgSvrBaseURL = ((ImgSvrURLHelper) ApplicationManager.GetInstance().GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString()).GetCachingManager().GetEntry("ImageServerHelper")).GetRootURL(isSecure);

      String altText = "";
      String app     = GetResultApp(rank);

      if (app.equals(COLLAB_APP))
      {
        altText =  _currentResponse.GetResultInternalField(rank,
            PT_SEARCH_INTERNAL_FIELD.PT_SEARCH_IF_COLLAB_ICON_ALT_TEXT);

        if (null == altText) {
          altText = m_asOwner.GetString(342, "ptmsgs_portalinfrastructure");
        } else {
          int altTextId = -1;
          try {
            altTextId = XPConvert.ToInteger(altText);
          } catch (Exception e) {
            // on any failure, just use the generic text
            altText = m_asOwner.GetString(342, "ptmsgs_portalinfrastructure");
            altTextId = -1;
          }

          if (altTextId != -1) {
            altText = m_asOwner.GetString(altTextId, "ptmsgs_collabserverobjecttypes");
          }
        }
      }
      else if (app.equals(CONTENT_APP))
      {
        altText = m_asOwner.GetString(343, "ptmsgs_portalinfrastructure");
      }
      else if (app.equals(TEMPLATING_APP))
      {
        altText = m_asOwner.GetString(320, "ptmsgs_portalinfrastructure");
      }

      if (null != indexedURL)
      {
        icon = new HTMLImg(imgSvrBaseURL + indexedURL, altText);
        icon.SetUseDisplayOptionsBaseUrl(false);
      }
      else
      {
        // If PT53 isn't indexed for some reason, use the "sheet of paper" PTCARD
        // icon as a default
        ClassTypeDescManager cdm = PTConfigHelper.GetPTClassTypeDescrManager();

        if (null != cdm)
        {
		  String strLangID = (String) PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.LANGUAGE, m_asOwner);
          icon = cdm.GetDefaultIconForClassType(PT_CLASSIDS.PT_CATALOGCARD_ID, strLangID);
          icon.SetAlt(altText);
        }
      }
    }

    return icon;
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getResultLastModified()
   */
  public XPDateTime GetResultLastModified(int rank)
    
  {
    checkInitialized();

    return ConvertSearchDateTimeToXPDateTime(_currentResponse.GetFieldsAsDouble(rank,
                                             PT_INTRINSICS.PT_PROPERTY_OBJECTLASTMODIFIED));
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getResultName()
   */
  public String GetResultName(int rank)
    
  {
    checkInitialized();

    return _currentResponse.GetFieldsAsString(rank,
      PT_INTRINSICS.PT_PROPERTY_OBJECTNAME);
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getResultObjectID()
   */
  public int GetResultObjectID(int rank)
    
  {
    checkInitialized();

    int objectId = -1;

    try {
      objectId = _currentResponse.GetFieldsAsInt(rank, PT_INTRINSICS.PT_PROPERTY_OBJECTID);
    } catch (Exception e) {
      // ObjectID is not defined for non-portal objects.
      objectId = -1;
    }

    return objectId;
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getResultPath()
   */
  public String GetResultPath(int rank)
    
  {
    checkInitialized();

    // For documents in multiple folders, this returns one path to the document
    // chosen more or less at random.  The user may not be able to see this
    // particular path, so be careful displaying it if GetResultFolderID()
    // returns zero.

    return  _currentResponse.GetResultPath(rank);
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getResultPropertiesURL()
   */
  public HTMLAnchor GetResultPropertiesURL(int rank, boolean allowPopup)
  {
    checkInitialized();

    String app = GetResultApp(rank);

    if (app.equals(CONTENT_APP) || app.equals(TEMPLATING_APP))
    {
      // No properties page for Content or templating items
      return null;
    }
    else if (app.equals(PORTAL_APP))
    {
      HTMLAnchor a = null;
      
      int nClassID = GetResultClassID(rank);
      int nObjectID = GetResultObjectID(rank);
		
      if (allowPopup) {
        a = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
        
        String strOptQSArgs = "&" + ASConstants.SUBSPACE + "=" + "mySubspaceKey";
        String strOnclick = PTOpenerLinks.GetJSOpenInNewWindow(
				PTOpenerLinks.GetOpenerJSURLOpenObjID(nClassID, nObjectID, strOptQSArgs,
	                    IPTOpenerConstants.OPENER_MODE_VIEW_META_DATA),
					PTOpenerLinks.GetWindowName("" + nClassID + "_" + nObjectID), 
	                "600", "450", true) + "; return false;";
        a.SetOnClick(strOnclick);
      } 
      else 
      {
          a = new HTMLAnchor(PTOpenerLinks.GetOpenerURLOpenObjID(GetResultClassID(rank), GetResultObjectID(rank),
              IPTOpenerConstants.OPENER_MODE_VIEW_META_DATA, m_asOwner));
      }
      
      return a;
    }
    else
    {
      // Collab properties
      String url =  _currentResponse.GetResultInternalField(rank,
          PT_SEARCH_INTERNAL_FIELD.PT_SEARCH_IF_COLLAB_PROPS_URL);

      if ((null == url) || (url.length() == 0)) {
        return null;
      }

      int gatewayObjectId = GetCollabPortletOrWebServiceID();
      int gatewayClassId = PT_CLASSIDS.PT_GADGET_ID;
      if (gatewayObjectId < 0) {
        gatewayClassId = PT_CLASSIDS.PT_WEBSERVICE_ID;
        gatewayObjectId = -1 * gatewayObjectId;
      }

      String gatewayPfx = GatewayHelpers.ConstructPrefPageLink(m_asOwner,
        /* objectID= */ gatewayObjectId,
        /* communityID= */ 0,
        /* pageID= */ 0,
        /* prefType= */ 0,
        /* classId= */ gatewayClassId);

      HTMLAnchor collabProps = null;
      if (allowPopup) {
        collabProps = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
        collabProps.SetOnClick("collabPopUp('" + gatewayPfx + 
                               GatewayHelpers.CreateGatewayFriendlyURL(GetCollabURLPrefix() + url) +
                               "'); return false;");
      } else {
        collabProps = new HTMLAnchor(gatewayPfx + GatewayHelpers.CreateGatewayFriendlyURL(GetCollabURLPrefix() + url));
      }
      return collabProps;
    }
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getResultRank()
   */
  public int GetResultRank(int groupNum, int resultNum)
    
  {
    checkInitialized();

    if (isAllOthersGroup(groupNum)) {
      return _currentResponse.GetRankForUncategorizedDoc(resultNum);
    }
    return _currentResponse.GetRank(groupNum, resultNum);
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getResultSize()
   */
  public int GetResultSize(int rank)
    
  {
    checkInitialized();

    return _currentResponse.GetResultSize(rank);
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getResultSubType()
   */
  public String GetResultSubType(int rank)
    
  {
    checkInitialized();

    return _currentResponse.GetResultSubType(rank);
  }
  
  /**
   * @see com.plumtree.portaluiinfrastructure.search.ISearchResultModelRO#GetResultStyleClass(java.lang.String)
   */
  public String GetResultStyleClass(String subType) {
	  String styleClass = "ali-kd-doc-text";
	  
	  // check for bad input first
	  if ((null == subType) || (0 == subType.length())) {
		  return styleClass;
      }

      // first, check for Collab types.. these are always 1 or 2 characters
      if (subType.length() <= 2) {

    	  // see if it's an integer
    	  int collabTypeId = 0;
    	  try {
    		  collabTypeId = XPConvert.ToInteger(subType);
    	  } catch (Exception e) {
    		  collabTypeId = 0;
    	  }
		
    	  // Note that "ptmsgs_collabserverobjecttypes.xml" contains the user-facing
    	  // names of the Collab types, and the IDs of strings in that file correspond
    	  // to the same values as the collab_type field.  We just need to make sure
    	  // the values are reasonable..
			
    	  if (0 != collabTypeId) {
    		  //return m_asOwner.GetString(collabTypeId, "ptmsgs_collabserverobjecttypes");
    		  return styleClass;
    	  }
      }

      // now, if it doesn't start with "PT", it's not something we know about
      if (!subType.startsWith("PT"))
      {
    	  return styleClass;
      }

      // This list needs to remain consistent with the 
      // list in PTIndexer.cpp, TypeStringFromClassID()

      // NOTE: These are mostly commented out because we don't have custom
      // styles and images for them yet.  We will add them in as we get
      // the new styles and images.
      /*
      if (subType.equals("PTUSER"))
      {
    	  //return m_asOwner.GetString(298, "ptmsgs_portalinfrastructure");
      }

      if (subType.equals("PTGROUP"))
      {
    	  //return m_asOwner.GetString(299, "ptmsgs_portalinfrastructure");
      }

      if (subType.equals("PTAUTHSOURCE"))
      {
    	  //return m_asOwner.GetString(300, "ptmsgs_portalinfrastructure");
      }

      if (subType.equals("PTPROFILESOURCE"))
      {
    	  //return m_asOwner.GetString(301, "ptmsgs_portalinfrastructure");
      }
*/
      if (subType.equals("PTFOLDER"))
      {
    	  styleClass = "ali-search-folder";
      }
/*
      if (subType.equals("PTCARD"))
      {
    	  //return m_asOwner.GetString(296, "ptmsgs_portalinfrastructure");
      }
*/
      if (subType.equals("PTADMINFOLDER"))
      {
    	  styleClass = "ali-search-folder";
      }
/*
      if (subType.equals("PTFILTER"))
      {
    	  //return m_asOwner.GetString(303, "ptmsgs_portalinfrastructure");
      }

      if (subType.equals("PTSAVEDSEARCH"))
      {
    	  //return m_asOwner.GetString(304, "ptmsgs_portalinfrastructure");
      }

      if (subType.equals("PTDATASOURCE"))
      {
    	  //return m_asOwner.GetString(305, "ptmsgs_portalinfrastructure");
      }

      if (subType.equals("PTCARDPROPERTY"))
      {
    	  //return m_asOwner.GetString(306, "ptmsgs_portalinfrastructure");
      }

      if (subType.equals("PTDOCUMENTTYPE"))
      {
    	 // return m_asOwner.GetString(307, "ptmsgs_portalinfrastructure");
      }

      if (subType.equals("PTCRAWLER"))
      {
    	 // return m_asOwner.GetString(308, "ptmsgs_portalinfrastructure");
      }

      if (subType.equals("PTGADGET"))
      {
    	  //return m_asOwner.GetString(309, "ptmsgs_portalinfrastructure");
      }

      if (subType.equals("PTINVITATION"))
      {
    	  //return m_asOwner.GetString(310, "ptmsgs_portalinfrastructure");
      }

      if (subType.equals("PTFEDPORTAL"))
      {
    	  //return m_asOwner.GetString(470, "ptmsgs_portalinfrastructure");
      }

      if (subType.equals("PTGADGETSERVER"))
      {
    	  //return m_asOwner.GetString(312, "ptmsgs_portalinfrastructure");
      }

      if (subType.equals("PTCOMMTEMPLATE"))
      {
    	  //return m_asOwner.GetString(313, "ptmsgs_portalinfrastructure");
      }

      if (subType.equals("PTGADGETBUNDLE"))
      {
    	  //return m_asOwner.GetString(314, "ptmsgs_portalinfrastructure");
      }

      if (subType.equals("PTPAGETEMPLATE"))
      {
    	  //return m_asOwner.GetString(315, "ptmsgs_portalinfrastructure");
      }

      if (subType.equals("PTWEBSERVICE"))
      {
    	  //return m_asOwner.GetString(316, "ptmsgs_portalinfrastructure");
      }

      if (subType.equals("PTEXTOPERATION"))
      {
    	  //return m_asOwner.GetString(317, "ptmsgs_portalinfrastructure");
      }

      if (subType.equals("PTGADGETTEMPLATE"))
      {
    	  //return m_asOwner.GetString(318, "ptmsgs_portalinfrastructure");
      }

      if (subType.equals("PTJOB"))
      {
    	  //return m_asOwner.GetString(319, "ptmsgs_portalinfrastructure");
      }

      if (subType.equals("PTCOMMUNITY"))
      {
    	  //return m_asOwner.GetString(320, "ptmsgs_portalinfrastructure");
      }

      if (subType.equals("PTPAGE"))
      {
    	  //return m_asOwner.GetString(505, "ptmsgs_portalinfrastructure");
      }

      if (subType.equals("PTSITEMAP"))
      {
    	  //return m_asOwner.GetString(321, "ptmsgs_portalinfrastructure");
      }

      if (subType.equals("PTSUBPORTAL"))
      {
    	  //return m_asOwner.GetString(519, "ptmsgs_portalinfrastructure");
      }

      if (subType.equals("PTCOLLAB"))
      {
    	  //return m_asOwner.GetString(323, "ptmsgs_portalinfrastructure");
      }

      if (subType.equals("PTCONTENT"))
      {
    	  //return m_asOwner.GetString(322, "ptmsgs_portalinfrastructure");
      }
*/
      return styleClass;
  }

  /**
   * 
   *
   * @param rank 
   *
   * @return 
   *
   * @ 
   */
  public String GetResultTypeUUID(int rank)
    
  {
    checkInitialized();

    return _currentResponse.GetFieldsAsString(rank,
      PT_INTRINSICS.PT_PROPERTY_OBJECTIMAGEUUID);
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getReturnedResultsInGroup()
   */
  public int GetReturnedResultsInGroup(int groupIdx)
    
  {
    checkInitialized();

    if (isAllOthersGroup(groupIdx)) {
      return _currentResponse.GetReturnedResultsNotInAnyGroup();
    }

    return _currentResponse.GetReturnedResultsInGroup(groupIdx);
  }

  /**
   *  @see com.plumtree.portalpages.search.ISaveSearchModel.SetSavedSearchErrorCode()
   */
  public void SetSavedSearchErrorCode(int code)
  {
    _savedSearchErrorCode = code;
  }

  /**
   *  @see com.plumtree.portalpages.search.ISaveSearchModelRO.GetSavedSearchErrorCode()
   */
  public int GetSavedSearchErrorCode()
  {
    return _savedSearchErrorCode;
  }

  /**
   *  @see com.plumtree.portalpages.search.ISaveSearchModel.SetSavedSearchName()
   */
  public void SetSavedSearchName(String name)
  {
    _savedSearchName = name;
  }

  /**
   *  @see com.plumtree.portalpages.search.ISaveSearchModelRO.GetSavedSearchName()
   */
  public String GetSavedSearchName()
  {
    return _savedSearchName;
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getSearchAgainArgs()
   */
  public XPHashtable GetSearchAgainArgs(boolean includeOptional)
    
  {
    // Never want to include current page context when we "search again"
    IXPList exclusions = SearchSettingCollection.GetPageContextSettings();

    // Add all the optional (aka removable) settings from the exclusions
    // list.  If the user wanted them, we'll add them separately below
    IXPList optional = new XPArrayList();

    // If requested and there are optional settings, add them to the
    // exclusions list
    if (null != _removableSettings)
    {
      IXPEnumerator optionalIDs = _removableSettings.GetSettingIDs();

      while (optionalIDs.MoveNext())
      {
        Object id = optionalIDs.GetCurrent();
        optional.Add(id);
        exclusions.Add(id);
      }
    }

    // Now, we can get all required settings 
    XPHashtable args = new XPHashtable();
    internalGetArgs(args, StartSearchControl.REQUIRED_CONSTRAINT_PREFIX,
      
    /* settingsArgs= */ true, 
    /* includedSettings= */ null, 
    /* excludedSettings= */ exclusions, 
    /* queryArgs= */ false);

    // If the caller wanted optional settings, add these as well
    if (includeOptional && (0 < optional.GetSize()))
    {
      final String FAKE_OPTION = "a";
      internalGetArgs(args,
        StartSearchControl.OPTIONAL_CONSTRAINT_PREFIX + FAKE_OPTION +
        "_", 
      /* settingsArgs= */ true, 
      /* includedSettings= */ optional, 
      /* excludedSettings= */ null, 
      /* queryArgs= */ false);

      // Finally, add an argument to tell StartSearchControl that the
      // fake "optional selection" was actually selected
      args.PutElement(StartSearchControl.SELECTED_PREFIX,
        new String[] { FAKE_OPTION });
    }

    return args;
  }

  /**
   *  ISearchResultModelRO implementation: read-only methods
   */
  /**
   *  @see com.plumtree.portalpages.search.IBaseSearchQueryRO.getSearchString()
   */
  public String GetSearchString()
    
  {
    checkInitialized();

    return _initialSearchString;
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getSpellingCorrection()
   */
  public String GetSpellingCorrection(int misspellingIndex,
    int correctionIndex)
    
  {
    checkInitialized();

    return  _currentResponse.GetCorrectedToken(misspellingIndex,
      correctionIndex);
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.getTotalResultsInGroup()
   */
  public String GetTotalResultsInGroup(int groupIdx)
    
  {
    checkInitialized();

    if (isAllOthersGroup(groupIdx)) {
      return (String) _currentResponse.GetTotalResultsNotInAnyGroup();
    }

    return  _currentResponse.GetTotalResultsInGroup(groupIdx);
  }
    
  /**
   * @see com.plumtree.uiinfrastructure.activityspace.IModel#CleanupTempData()
   */
  public void CleanupTempData()
  {
  }

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#CleanupAllData()
	 */
	public void CleanupAllData()
	{
    // Release references to COM objects to allow them to be garbage 
    // collected as soon as possible

    reinit(/*createRequest=*/ false);
	}

  /**
   * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
   */
  public Object Create()
  {
    return new SearchResultModel();
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModel.Drilldown()
   */
  public void Drilldown(String keyOnPage, String groupByField, int category)
    throws SearchExecutionException
  {
    restoreState(keyOnPage);

    // Sanity check: The key on the page should end with the group-by field
    // used to generate the page.  If not, something bad happened.
    if ((null == groupByField) ||
        (0 == groupByField.length()) ||
        (! keyOnPage.endsWith(groupByField)))
    {
      throw new InvalidStateException(keyOnPage + " groupByField=" +
        groupByField);
    }

    // Go back to page one
    SetFirstPageAndPageSize();

    // Make sure the request still has the group-by setting used on the
    // page where the link was clicked - someone might have clicked "back"
    // after changing it
    _request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_GROUPBYPROP, groupByField);

    // .. and we need relevance ranking
    _request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_ORDERBY, new Integer(0));

    // Get the query to be ANDed with all previous queries
    IPTSearchQuery query;

    boolean isAllOthers = isAllOthersGroup(category);

    if (isAllOthers) {
      query = _currentResponse.GetQueryForAllOthers();
    }
    else {
      query = _currentResponse.GetQueryForGroup(category);
    }

    String breadcrumb;
    if (isAllOthers) {
      breadcrumb = m_asOwner.GetString(520, "ptmsgs_portalinfrastructure");
    }
    else {
      breadcrumb= localizeObjectTypeName( _currentResponse.GetGroupLabel(category));
    }
 
    // We need to cache the result of this search.
    // The new key equals the current key, plus a dot, plus the index of the
    // category we're drilling down into, plus a slash, plus the group-by setting.
    // indicates that the model is now uncategorized:
    //    /objtype  =>  /objtype.3/    or
    //    /objtype  =>  /objtype.3/ancestors
    String newKey = _currentKey + "." + category + CACHE_KEY_SEPARATOR_STR + groupByField;

    doSearch(query, breadcrumb, newKey);
  }

  /**
   *  @see com.plumtree.portalpages.search.IBaseSearchResultModelRO.GetError()
   */
  public SearchExecutionError GetError()
  {
    return _error;
  }

  /**
   *  @see com.plumtree.uiinfrastructure.pagination.IPaginationRO#GetFirstDisplayableItem
   */
  public int GetFirstDisplayableItem()
  {
    checkInitialized();

    return (_currentResponse.GetSkipped() + 1);
  }

  /**
   *  @see com.plumtree.uiinfrastructure.pagination.IPaginationRO#GetItemsPerPage
   */
  public int GetItemsPerPage()
  {
    checkInitialized();

    return _pageSize;
  }

  /**
   * @see com.plumtree.portaluiinfrastructure.activityspace.IModelRO#GetName()
   */
  public String GetName()
  {
    return STR_MVC_CLASS_NAME;
  }

  /**
   *  @see com.plumtree.uiinfrastructure.pagination.IPaginationRO#GetNumDisplayableItems
   */
  public int GetNumDisplayableItems()
  {
    checkInitialized();

    return _currentResponse.GetResultsReturned();
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.GetRemovableSettings()
   */
  public SearchSettingCollection GetRemovableSettings()
  {
    return _removableSettings;
  }

  /**
   *  @see com.plumtree.uiinfrastructure.pagination.IPaginationRO#GetItemsPerPage
   */
  public String GetSortOrder()
  {
  	
    checkInitialized();

    int orderByProperty = 0;

    try
    {
      orderByProperty = _request.GetSettingsAsInt(PT_SEARCH_SETTING.PT_SEARCHSETTING_ORDERBY);
    }
    catch (Exception e)
    {
      rethrowCOMModelException(e);
    }

    return XPConvert.ToString(orderByProperty);
  }

  /**
   *  @see com.plumtree.uiinfrastructure.pagination.IPaginationRO#GetTotalItems
   */
  public int GetTotalItems()
  {
    checkInitialized();

    return _currentResponse.GetTotalMatches();
  }

  /**
   *  @see com.plumtree.portalpages.search.IBaseSearchResultModelRO.GetWarning()
   */
  public SearchExecutionWarning GetWarning()
  {
    return _warning;
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModel.RefreshResults()
   */
  public void RefreshResults(String keyOnPage)
  {
    // If the results were uncategorized (ungrouped)
    if (keyOnPage.endsWith(CACHE_KEY_SEPARATOR_STR)) {
      // Just call GoToPage() as if a pagination link were chosen

      GoToPage(XPConvert.ToString(_request.GetSettingsAsInt(PT_SEARCH_SETTING.PT_SEARCHSETTING_ORDERBY)),
               _request.GetSettingsAsInt(PT_SEARCH_SETTING.PT_SEARCHSETTING_SKIPRESULTS) + 1,
               _request.GetSettingsAsInt(PT_SEARCH_SETTING.PT_SEARCHSETTING_MAXRESULTS),
               keyOnPage);
    } else {
      // Otherwise, if grouped, we need to call Reorganize() and recache the
      // result, since the generated categories may change.

      // Figure out what the last group-by field was.
      String lastGroupBy = GetLastGroupByFieldForKey(keyOnPage);

      // Go back to the way things were before the categorization occurred.
      restoreState(lastUncategorizedKey(keyOnPage));

      // _currentKey now contains the uncategorized key.
      Reorganize(_currentKey, lastGroupBy);
    }
  }
	
  public boolean GetHasDisplayedObjects(int classID)
  {
  	if(! _initialized)
  	{
  		return false;
	}
	int num = GetNumDisplayableItems();
	
	for(int rank = 0; rank < num; rank++)
	{
		try
		{
			if( GetResultClassID(rank) == classID)
			{
				return true;				
			}
		} 
		catch (Exception e) 
		{
			log.Error(e, "In GetHasDisplayedObjects: Invalid Rank? = " + rank + " PageSize=" + num);
  		}
	}
	return false;
  }
  
  /**
   * Same as GoToPage but only changes sort order.
   * Called from SortOrderControl
   * 
   * @param sortOrder
   * @param additionalInfo
   */
  public void ChangeSortOrder(String sortOrder, Object additionalInfo)
  {
	String keyOnPage = (String) additionalInfo;

    try
    {
      restoreState(lastUncategorizedKey(keyOnPage));

      if (null == sortOrder)
      {
        throw new InvalidArgumentException("sortOrder", "null");
      }

      int orderByProp = 0;

      try
      {
        orderByProp = XPConvert.ToInteger(sortOrder);
      }
      catch (Exception e)
      {
        throw new InvalidArgumentException("sortOrder", sortOrder);
      }

      try
      {
        
        // Set the sort order
        _request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_ORDERBY,
          new Integer(orderByProp));
      }
      catch (Exception e)
      {
		SearchUIUtility.RethrowCOMExecutionException(e, m_asOwner);
      }
      doSearch(null, null, null);
    }
    catch (SearchExecutionWarning e)
    {
      this.SetWarning(e);
    }
    catch (SearchExecutionError e)
    {
      this.SetError(e);
    }
    catch (SearchExecutionException e)
    {
      // shouldn't happen; all instances should be an Error or Warning
    }
  }
  
  /**
   *  @see com.plumtree.uiinfrastructure.pagination.IPagination#GoToPage
   */
  public void GoToPage(String sortOrder, int first, int pageSize,
    Object additionalInfo)
  {
    String keyOnPage = (String) additionalInfo;

    try
    {
      restoreState(lastUncategorizedKey(keyOnPage));

      if (first <= 0)
      {
        throw new InvalidArgumentException("first",
          XPConvert.ToString(first));
      }

      if (pageSize <= 0)
      {
        throw new InvalidArgumentException("pageSize",
          XPConvert.ToString(pageSize));
      }

      if (null == sortOrder)
      {
        throw new InvalidArgumentException("sortOrder", "null");
      }

      int orderByProp = 0;

      try
      {
        orderByProp = XPConvert.ToInteger(sortOrder);
      }
      catch (Exception e)
      {
        throw new InvalidArgumentException("sortOrder", sortOrder);
      }

      // Save the user's new desired page size
      _pageSize = pageSize;

      try
      {
        // Set the new first result and page size on the request object
        _request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_SKIPRESULTS,
          new Integer(first - 1));
        _request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_MAXRESULTS,
          new Integer(pageSize));

        // Set the sort order
        _request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_ORDERBY,
          new Integer(orderByProp));

        // Make sure we aren't grouped by any property; we don't support
        // pagination on grouped results
        _request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_GROUPBYPROP,
          "");
      }
      catch (Exception e)
      {
		SearchUIUtility.RethrowCOMExecutionException(e, m_asOwner);
      }
      
      // Call doSearch to do the real work.
      // No new query or breadcrumb, and no need to save this response in the cache.
      doSearch(null, null, null);
    }
    catch (SearchExecutionWarning e)
    {
      this.SetWarning(e);
    }
    catch (SearchExecutionError e)
    {
      this.SetError(e);
    }
    catch (SearchExecutionException e)
    {
      // shouldn't happen; all instances should be an Error or Warning
    }
  }

  /**
   * @see com.plumtree.portaluiinfrastructure.activityspace.IModel#Init(AActivitySpace)
   */
  public void Init(AActivitySpace parent)
  {
    m_asOwner     = parent;
    _session    = (IPTSession) m_asOwner.GetUserSession();
  }

  /**
   *  Write methods inherited from ISearchResultModel
   */
  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModel.isInitialized()
   */
  public boolean IsInitialized()
  {
    return _initialized;
  }

  /**
   * @see com.plumtree.portalpages.search.ISearchResultModel.Reorganize
   */
  public void Reorganize(String keyOnPage, String groupByField)
    throws SearchExecutionException
  {
    restoreState(lastUncategorizedKey(keyOnPage));

    try
    {
      // Set the group-by field
      _request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_GROUPBYPROP,
        groupByField);

      // Make sure we're relevance ranked 
      _request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_ORDERBY,
        new Integer(0));
    }
    catch (Exception e)
    {
		SearchUIUtility.RethrowCOMExecutionException(e, m_asOwner);
    }

    // Make sure we're on page one..
    SetFirstPageAndPageSize();

    // We need to cache the result of this search.
    // Note that the current key now ends with a slash, thanks to the
    // lastUncategorizedKey() call above.
    // The new key equals the current key up to and including the slash,
    // followed by the group-by field.
    // For example, if grouping by "ancestor":
    //   if currently not grouped,           /         =>  /ancestor
    //   if currently grouped by objtype,    /objtype  =>  /ancestor
    String newKey = _currentKey + groupByField;

    // No new query or breadcrumb, but new key
    doSearch(null, null, newKey);
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModel.RestoreFromBreadcrumb()
   */
  public void RestoreFromBreadcrumb(String keyOfPageWhereLinkAppears,
    String keyToRestore)
    throws SearchExecutionException
  {
    restoreState(keyToRestore);

    // Clear any previous errors
    _error     = null;
    _warning     = null;

    try
    {
      // Restore the group-by and order-by settings to reasonable values.
      _request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_ORDERBY,
        new Integer(0));

      String groupby = GetLastGroupByFieldForKey(_currentKey);
      _request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_GROUPBYPROP,
        groupby);
    }
    catch (Exception e)
    {
		SearchUIUtility.RethrowCOMExecutionException(e, m_asOwner);
    }

    // Rerun the search.
    // note: Earlier versions of this class just redisplayed the page using
    // the result set in _currentResponse (which the call to restoreState()
    // brought back).  But we can't be sure what the ORDERBY setting was when
    // that result set was generated, so it's safest to rerun the search
    // using relevance ranking.

    doSearch(/* no additional query */ null,
             /* no additional breadcrumb */ null,
             /* no new key */ null);
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModel.Search()
   */
  public void Search(String searchString, String basicFields, // only if basic search
    IPTFilter advancedFilter, // only if advanced search
    SearchSettingCollection settings)
    throws SearchExecutionException
  {
    Search(searchString, basicFields, advancedFilter, settings, null);
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModel.Search()
   */
  public void Search(String searchString, String basicFields, // only if basic search
    IPTFilter advancedFilter, // only if advanced search
    SearchSettingCollection settings,
    SearchSettingCollection removableSettings)
    throws SearchExecutionException
  {
    // Release references to any previous searches.
    // This is a brand-new search..
    reinit(/* createRequest= */ true);

    if ((null == searchString) && (null == advancedFilter))
    {
      throw new EmptyQueryException(m_asOwner);
    }

    // If this was a basic search
    if (null != searchString)
    {
      // Save the initial search string, for display on the result page
      _initialSearchString = searchString;
    }
    else
    {
      // For advanced search, if there is a top-level text search, save
      // it as the initial search string instead
      _initialSearchString = advancedFilter.GetSearchString();

      // Don't keep around empty strings
      if ((null != _initialSearchString) &&
          (0 == _initialSearchString.length()))
      {
        _initialSearchString = null;
      }

      // If the user didn't provide a search string
      if (null == _initialSearchString)
      {
        // Check that the filter has at least one property condition
        IPTPropertyFilterItem propFilter = advancedFilter.GetPropertyFilter();

        if (null == propFilter)
        {
          throw new EmptyQueryException(m_asOwner);
        }
        else
        {
          if (0 == ((IPTPropertyFilterClauses) propFilter).GetCount())
          {
            throw new EmptyQueryException(m_asOwner);
          }
        }
      }

      // Keep the filter around as well
      _advancedFilter = advancedFilter;
    }

    // Keep removable settings around for a Control to use later
    _removableSettings = removableSettings;

    IPTSearchQuery query = null;

    try
    {
      // Bump up the counter of total searches
      // !!!TODO: David Phipps to rename this counter to ..._TOTAL_SEARCHES
      PerfCounters.INSTANCE.IncrementCounter(PORTALPAGESPERF_COUNTERS.PORTALPAGESPERF_SELECTION_SEARCHES);

      // Set desired request settings
      settings.updateRequest(_request);
      
	  // Manufacture the initial query
	  if (null != advancedFilter)
	  {
		// Manufacture an advanced query given the supplied filter
		query     = _request.CreateAdvancedQuery(advancedFilter);

		// Remember that this is advanced search
		_isAdvancedSearch     = true;

		// Search-within-results queries always go against
		// the "cardsearch" alias
		_swrFields = "cardsearch";
	  }
	  else
	  {
		// Manufacture a basic query given the supplied search string
		query     = _request.CreateBasicQuery(searchString, basicFields);
		// Save fields for use by search-within-results
		_swrFields = basicFields;
	  }

      // Get the page size that is being used and save it
      // for convenient access later
      _pageSize = _request.GetSettingsAsInt(PT_SEARCH_SETTING.PT_SEARCHSETTING_MAXRESULTS);

      // Keep the initial page's grouping setting around - if the user
      // clicks a breadcrumb link to undo a drilldown step, then we'll
      // need to know how to group the resulting page.

      _initialGroupBy = _request.GetSettingsAsString(PT_SEARCH_SETTING.PT_SEARCHSETTING_GROUPBYPROP);

      // .. but don't keep around any empty strings.
      if ((null != _initialGroupBy) && 
          (0 == _initialGroupBy.length())) {
        _initialGroupBy = null;
      }

      // Keep track of whether Best Bets will be used at all
      _initialBestBets = XPConvert.ToBoolean(_request.GetSettingsAsObject(PT_SEARCH_SETTING.PT_SEARCHSETTING_BESTBETS));
    }
    catch (Exception e)
    {
		SearchUIUtility.RethrowCOMExecutionException(e, m_asOwner);
    }

    // Make sure we're on the first page
    SetFirstPageAndPageSize();

    // Construct the initial key
    String initialKey = CACHE_KEY_SEPARATOR_STR;
    if (null != _initialGroupBy) {
      initialKey += _initialGroupBy;
    }

    doSearch(query, 
             /* breadcrumb= */ null, 
             /* key= */ initialKey);
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModel.SearchWithinResults()
   */
  public void SearchWithinResults(String keyOnPage, String additionalQuery)
    throws SearchExecutionException
  {
    restoreState(lastUncategorizedKey(keyOnPage));

    IPTSearchQuery query = null;

    try
    {
      // Make a new basic query given the string and the
      // set of fields that were used for the original search
      query = _request.CreateBasicQuery(additionalQuery, _swrFields);

      // Make sure we go to page one..
      SetFirstPageAndPageSize();

      // No categorization after doing search-within-results
      _request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_GROUPBYPROP,
        "");

      // .. and we want to order by relevance
      _request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_ORDERBY,
        new Integer(0));
    }
    catch (Exception e)
    {
		SearchUIUtility.RethrowCOMExecutionException(e, m_asOwner);
    }

    // Keep track of how many times we've done search-within-results
    _swrCount++;

    // We need to cache the result of this search.
    // The new key is the current key up to the last slash, plus the
    // string "swr" and a unique index, plus a trailing slash that indicates
    // that the results are now uncategorized.
    int lastSlash = _currentKey.lastIndexOf(CACHE_KEY_SEPARATOR_CHAR);
    String newKey = _currentKey.substring(0, lastSlash + 1) + "swr" +
      _swrCount + CACHE_KEY_SEPARATOR_CHAR;

    // Call doSearch to do the real work
    doSearch(query, /* breadcrumb= */
      additionalQuery, newKey);
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModel.SetError()
   */
  public void SetError(SearchExecutionError e)
  {
    _error = e;
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModel.SetError()
   */
  public void SetWarning(SearchExecutionWarning e)
  {
    _warning = e;
  }

  /**
   *  @see com.plumtree.portalpages.search.ISearchResultModelRO.wasSpellCorrected()
   */
  public boolean wasSpellCorrected()
    
  {
    checkInitialized();

    return (0 < GetNumMisspelledTokens());
  }

  /**
   *  doSearch() actually calls the COM side to do the search;
   *  this is called from any method that has made a change to the
   *  _request object and needs to update the search results.
   *
   *  doSearch() also handles the various caches.
   */
  protected void doSearch(IPTSearchQuery additionalQuery,
    String additionalBreadcrumb, String newKey)
    throws SearchExecutionException
  {
    // Clear any previous errors
    _error     = null;
    _warning     = null;

    IPTSearchResponse resp = null;

    // If we're doing an initial query
    if (null == _currentKey)
    {
      // Use Best Bets if and only if we are relevance ranked and have no group-by.  (See bug 42054.)
      if (_initialBestBets &&
          (0 == _request.GetSettingsAsInt(PT_SEARCH_SETTING.PT_SEARCHSETTING_ORDERBY)) &&
          ((null == _request.GetSettingsAsString(PT_SEARCH_SETTING.PT_SEARCHSETTING_GROUPBYPROP)) ||
           (0 == _request.GetSettingsAsString(PT_SEARCH_SETTING.PT_SEARCHSETTING_GROUPBYPROP).length()))) {
        _request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_BESTBETS, Boolean.TRUE);
      } else {
        _request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_BESTBETS, Boolean.FALSE);
      }

      // call Search()
      try
      {
        resp = _request.Search(additionalQuery);
      }
      catch (Exception e)
      {
		SearchUIUtility.RethrowCOMExecutionException(e, m_asOwner);
      }

      if (null == resp)
      {
        // Something went wrong on the COM side
        throw new SearchExecutionError(
          "Received null pointer from IPTSearchRequest.Search");
      }
    }
    else
    {
      String[] ancestorKeys = allAncestors(_currentKey);

      int numQueries = ancestorKeys.length;

      if (null != additionalQuery)
      {
        numQueries++;
      }

      // Use Best Bets if and only if we are relevance ranked and have no group-by or drilldown.  (See bug 42054.)
      if (_initialBestBets &&
          (1 == numQueries) &&
          (0 == _request.GetSettingsAsInt(PT_SEARCH_SETTING.PT_SEARCHSETTING_ORDERBY)) &&
          ((null == _request.GetSettingsAsString(PT_SEARCH_SETTING.PT_SEARCHSETTING_GROUPBYPROP)) ||
           (0 == _request.GetSettingsAsString(PT_SEARCH_SETTING.PT_SEARCHSETTING_GROUPBYPROP).length()))) {
        _request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_BESTBETS, Boolean.TRUE);
      } else {
        _request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_BESTBETS, Boolean.FALSE);
      }

      // construct a simple array of queries that can be unpacked on the COM side
      Object[] queryArray = new Object[numQueries];

      for (int j = 0; j < ancestorKeys.length; j++)
      {
        queryArray[j] = _queryCache.GetElement(ancestorKeys[j]);
      }

      if (null != additionalQuery)
      {
        queryArray[queryArray.length - 1] = additionalQuery;
      }

      // (bit of a hack..)
      // If we're in "object type" categorization and have already drilled down
      // into a single object type category, categorize on Collab types this time..
      // this will do no good if the drilldown wasn't into the "Collab Server Objects"
      // category, but will do no harm, either..
      boolean needToRestoreGroupByObjectType = false;
      {
        String currGroupBy = _request.GetSettingsAsString(PT_SEARCH_SETTING.PT_SEARCHSETTING_GROUPBYPROP);
        if (((null != currGroupBy) && (currGroupBy.equals(GROUPBY_OBJECTTYPE))) &&
            (firstDrilldownIsIntoCollabCategory(newKey))) {
          _request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_GROUPBYPROP, "collab_type");
          needToRestoreGroupByObjectType = true;
        }
      }

      // call FollowupSearch() to AND the queries together on the COM side
      try
      {
        resp = _request.FollowupSearch(queryArray);
      }
      catch (Exception e)
      {
		SearchUIUtility.RethrowCOMExecutionException(e, m_asOwner);
      }

      // restore group-by setting if necessary
      if (needToRestoreGroupByObjectType) {
        _request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_GROUPBYPROP, GROUPBY_OBJECTTYPE);
      }

      if (null == resp)
      {
        // Something went wrong on the COM side
        throw new SearchExecutionError(
          "Received null pointer from IPTSearchRequest.FollowupSearch ( submitted " +
          queryArray.length + " queries)");
      }
    }

    // Now that the search has run, unset the INITIALSEARCHSTRING setting,
    // to be sure it is only logged once per sequence of searches
    _request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_INITIALSEARCHSTRING, "");

    // If the caller supplied a key
    if (null != newKey)
    {
      // We need to cache the query, breadcrumb, and response.

      int lastSlash = newKey.lastIndexOf(CACHE_KEY_SEPARATOR_CHAR);
      String baseOfNewKey = newKey.substring(0, lastSlash+1);

      if (null != additionalQuery)
      {
        _queryCache.PutElement(newKey, additionalQuery);

        // If this key is something like "/ancestors", and we haven't already
        // cached a query for "/", then cache this query under this name as well
        if ((! newKey.equals(baseOfNewKey)) &&
            (! _queryCache.ContainsKey(baseOfNewKey))) {
          _queryCache.PutElement(baseOfNewKey, additionalQuery);
        }
      }

      if (null != additionalBreadcrumb)
      {
        _crumbCache.PutElement(newKey, additionalBreadcrumb);

        // If this key is something like "/ancestors", and we haven't already
        // cached a breadcrumb for "/", then cache this query under this name as well
        if ((! newKey.equals(baseOfNewKey)) &&
            (! _crumbCache.ContainsKey(baseOfNewKey))) {
          _crumbCache.PutElement(baseOfNewKey, additionalBreadcrumb);
        }
      }

      _responseCache.PutElement(newKey, resp);
      _currentKey = newKey;

    }

    // Keep the last key and response around for easy access
    _currentResponse = resp;

    // Check for warnings
    int warning = _currentResponse.GetWarning();

    if (PT_SEARCH_WARNING.PT_SEARCHWARNING_NONE != warning)
    {
      switch (warning)
      {
      case PT_SEARCH_WARNING.PT_SEARCHWARNING_QUERY_TIMED_OUT:
        _warning = new QueryTimeoutWarning(m_asOwner);

        break;

      case PT_SEARCH_WARNING.PT_SEARCHWARNING_WILDCARD_TOO_GENERAL:
        _warning = new WildcardExpansionWarning(m_asOwner);

        break;
      case PT_SEARCH_WARNING.PT_SEARCHWARNING_QUERY_IS_PARTIAL:
          _warning = new PartialQueryWarning(m_asOwner);

          break;
      }
    }

    // We have displayable results now
    _initialized = true;
  }

  /**
   *  Helper method to convert an indexed object type or abstract
   *  type to its localized equivalent
   */
  protected String localizeObjectTypeName(String objType)
  {
    // check for bad input first
    if ((null == objType) || (0 == objType.length())) {
      return objType;
    }

    // first, check for Collab types.. these are always 1 or 2 characters
    if (objType.length() <= 2) {

      // see if it's an integer
      int collabTypeId = 0;
      try {
        collabTypeId = XPConvert.ToInteger(objType);
      } catch (Exception e) {
        collabTypeId = 0;
      }

      // Note that "ptmsgs_collabserverobjecttypes.xml" contains the user-facing
      // names of the Collab types, and the IDs of strings in that file correspond
      // to the same values as the collab_type field.  We just need to make sure
      // the values are reasonable..

      if (0 != collabTypeId) {
        return m_asOwner.GetString(collabTypeId, "ptmsgs_collabserverobjecttypes");
      }
    }

    // now, if it doesn't start with "PT", it's not something we know about
    if (!objType.startsWith("PT"))
    {
      return objType;
    }

    // This list needs to remain consistent with the 
    // list in PTIndexer.cpp, TypeStringFromClassID()
    if (objType.equals("PTUSER"))
    {
      return m_asOwner.GetString(298, "ptmsgs_portalinfrastructure");
    }

    if (objType.equals("PTGROUP"))
    {
     return m_asOwner.GetString(299, "ptmsgs_portalinfrastructure");
    }

    if (objType.equals("PTAUTHSOURCE"))
    {
      return m_asOwner.GetString(300, "ptmsgs_portalinfrastructure");
    }

    if (objType.equals("PTPROFILESOURCE"))
    {
      return m_asOwner.GetString(301, "ptmsgs_portalinfrastructure");
    }

    if (objType.equals("PTFOLDER"))
    {
      return m_asOwner.GetString(297, "ptmsgs_portalinfrastructure");
    }

    if (objType.equals("PTCARD"))
    {
      return m_asOwner.GetString(296, "ptmsgs_portalinfrastructure");
    }

    if (objType.equals("PTADMINFOLDER"))
    {
      return m_asOwner.GetString(302, "ptmsgs_portalinfrastructure");
    }

    if (objType.equals("PTFILTER"))
    {
      return m_asOwner.GetString(303, "ptmsgs_portalinfrastructure");
    }

    if (objType.equals("PTSAVEDSEARCH"))
    {
      return m_asOwner.GetString(304, "ptmsgs_portalinfrastructure");
    }

    if (objType.equals("PTDATASOURCE"))
    {
      return m_asOwner.GetString(305, "ptmsgs_portalinfrastructure");
    }

    if (objType.equals("PTCARDPROPERTY"))
    {
      return m_asOwner.GetString(306, "ptmsgs_portalinfrastructure");
    }

    if (objType.equals("PTDOCUMENTTYPE"))
    {
      return m_asOwner.GetString(307, "ptmsgs_portalinfrastructure");
    }

    if (objType.equals("PTCRAWLER"))
    {
      return m_asOwner.GetString(308, "ptmsgs_portalinfrastructure");
    }

    if (objType.equals("PTGADGET"))
    {
      return m_asOwner.GetString(309, "ptmsgs_portalinfrastructure");
    }

    if (objType.equals("PTINVITATION"))
    {
      return m_asOwner.GetString(310, "ptmsgs_portalinfrastructure");
    }

    if (objType.equals("PTFEDPORTAL"))
    {
      return m_asOwner.GetString(470, "ptmsgs_portalinfrastructure");
    }

    if (objType.equals("PTGADGETSERVER"))
    {
      return m_asOwner.GetString(312, "ptmsgs_portalinfrastructure");
    }

    if (objType.equals("PTCOMMTEMPLATE"))
    {
      return m_asOwner.GetString(313, "ptmsgs_portalinfrastructure");
    }

    if (objType.equals("PTGADGETBUNDLE"))
    {
      return m_asOwner.GetString(314, "ptmsgs_portalinfrastructure");
    }

    if (objType.equals("PTPAGETEMPLATE"))
    {
      return m_asOwner.GetString(315, "ptmsgs_portalinfrastructure");
    }

    if (objType.equals("PTWEBSERVICE"))
    {
      return m_asOwner.GetString(316, "ptmsgs_portalinfrastructure");
    }

    if (objType.equals("PTEXTOPERATION"))
    {
      return m_asOwner.GetString(317, "ptmsgs_portalinfrastructure");
    }

    if (objType.equals("PTGADGETTEMPLATE"))
    {
      return m_asOwner.GetString(318, "ptmsgs_portalinfrastructure");
    }

    if (objType.equals("PTJOB"))
    {
      return m_asOwner.GetString(319, "ptmsgs_portalinfrastructure");
    }

    if (objType.equals("PTCOMMUNITY"))
    {
      return m_asOwner.GetString(320, "ptmsgs_portalinfrastructure");
    }

    if (objType.equals("PTPAGE"))
    {
      return m_asOwner.GetString(505, "ptmsgs_portalinfrastructure");
    }

    if (objType.equals("PTSITEMAP"))
    {
      return m_asOwner.GetString(321, "ptmsgs_portalinfrastructure");
    }

    if (objType.equals("PTSUBPORTAL"))
    {
      return m_asOwner.GetString(519, "ptmsgs_portalinfrastructure");
    }

    //if (objType.equals("PTSYNDICATED")) return _ui_PTSYNDICATED;
    //if (objType.equals("PTSYNDGADGET")) return _ui_PTSYNDGADGET;

    if (objType.equals("PTCOLLAB"))
    {
      return m_asOwner.GetString(323, "ptmsgs_portalinfrastructure");
    }

    if (objType.equals("PTCONTENT"))
    {
      return m_asOwner.GetString(322, "ptmsgs_portalinfrastructure");
    }

    return objType;
  }

  /**
   *  If the user's hit the BACK button before clicking a link, this
   *  method restores our state to the way things were on the page
   *  where the user clicked the button.
   */
  protected void restoreState(String keyOnPage)
    throws SearchExecutionException
  {
    // If we're not even initialized yet
    if ((!_initialized) || (null == _currentKey) ||
        (null == _responseCache))
    {
      throw new InvalidStateException(keyOnPage);
    }

    // If BACK button wasn't hit
    if (keyOnPage.equals(_currentKey))
    {
      return;
    }

    // Fetch the response on the page where the link was selected
    IPTSearchResponse r = (IPTSearchResponse) _responseCache.GetElement(keyOnPage);

    // NOTE: r may be null if this search was never run; the caller should
    // be able to handle this case!

    // Restore state
    _currentKey       = keyOnPage;
    _currentResponse     = r;

    // Restore group-by settings to their former state
    int lastSlash      = _currentKey.lastIndexOf(CACHE_KEY_SEPARATOR_CHAR);
    String groupByField = _currentKey.substring(lastSlash + 1);

    try
    {
      _request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_GROUPBYPROP,
        groupByField);
    }
    catch (Exception e)
    {
		SearchUIUtility.RethrowCOMExecutionException(e, m_asOwner);
    }
  }

  /**
   * 
   *
   * @param args 
   * @param prefix 
   * @param wantSettingsArgs 
   * @param includedSettings 
   * @param excludedSettings 
   * @param wantQueryArgs 
   *
   * @throws SearchModelException 
   */
  private void internalGetArgs(XPHashtable args, String prefix,
    boolean wantSettingsArgs, IXPCollection includedSettings,
    IXPCollection excludedSettings, boolean wantQueryArgs)
    
  {
    // Encode user's query.
    if (wantQueryArgs)
    {
      if (null != _advancedFilter)
      {
        FilterForm.EncodeFilterAsControlArgs(_session, m_asOwner,
          _advancedFilter, args, prefix);
      }
      else
      {
        if (null != _initialSearchString)
        {
          args.PutElement(prefix + SearchResultControl.USER_QUERY,
            new String[] { _initialSearchString });
        }

        if (null != _swrFields)
        {
          args.PutElement(prefix + SearchResultControl.BASIC_FIELDS,
            new String[] { _swrFields });
        }
      }
    }

    if (wantSettingsArgs)
    {
      // Encode search settings.
      try
      {
        SearchSettingCollection.encodeRequestSettings(_request, args,
          prefix, includedSettings, excludedSettings);
      }
      catch (Exception e)
      {
        rethrowCOMModelException(e);
      }
    }
  }

  /**
   * 
   *
   * @param o 
   *
   * @return 
   */
  private int[] objectArrayToIntArray(Object o)
  {
    if (null == o)
    {
      return null;
    }

    Object[] oa = (Object[]) o;

    if (0 == oa.length)
    {
      return null;
    }

    int[] ia = new int[oa.length];

    for (int j = 0; j < oa.length; j++)
    {
      ia[j] = XPConvert.ToInteger(oa[j]);
    }

    return ia;
  }

  /**
   *  Returns a portlet or web service ID to be used in gatewaying any Collab Server
   *  search results.
   *  This ID is the same for all Collab results in the system.
   *
   *  Under the covers, the portal server is doing a query against the Collab database
   *  to get this ID (and caching it; the query is done only once).  Collab 3.0 returns
   *  a positive number that corresponds to a portlet ID; Collab 4.0 returns a negative
   *  number corresponding to (-1 *) a web service ID.
   *  Callers of this function must be prepared to handle both cases!
   */
  private int GetCollabPortletOrWebServiceID()
  {
          int id = -1;

          // The rank doesn't matter here; this is just a constant that's the same
          // for all results.
          String s = _currentResponse.GetResultInternalField(0,
                                PT_SEARCH_INTERNAL_FIELD.PT_SEARCH_IF_COLLAB_PORTLET_ID);
          try {
            id = XPConvert.ToInteger(s);
          } catch (Exception e) {
            final String BAD_COLLAB_MESSAGE =
              "Got invalid Collab portlet or webservice ID.  For Collab 3.0, complete Collab installation by using the Project Selector portlet to open a project.  For Collab 4.0, visit the Collaboration Server Administration utility, which should automatically fix this problem.  If these steps do not solve the problem, check for Collab installer errors or an incorrect Collab configuration.";

            
              log.Error(e, BAD_COLLAB_MESSAGE);
            
            throw new XPException(BAD_COLLAB_MESSAGE);
          }
          return id;
  }

  // This is the same for all Collab results.
  private String GetCollabURLPrefix()
  {
          // The rank doesn't matter here; this is just a constant that's the same
          // for all results.
          return _currentResponse.GetResultInternalField(0,
                                  PT_SEARCH_INTERNAL_FIELD.PT_SEARCH_IF_COLLAB_URL_PREFIX);
  }

  /**
   *  Helper function to tell the request object to go to
   *  the first page; should call this when any conditions are
   *  added to the search
   */
  private void SetFirstPageAndPageSize()
    throws SearchExecutionException
  {
    try
    {
      _request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_SKIPRESULTS,
        new Integer(0));
      _request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_MAXRESULTS,
        new Integer(_pageSize));
    }
    catch (Exception e)
    {
		SearchUIUtility.RethrowCOMExecutionException(e, m_asOwner);
    }
  }

  /**
   * 
   *
   * @param key 
   *
   * @return 
   */
  private String GetLastGroupByFieldForKey(String key)
  {
    int lastSlash = key.lastIndexOf(CACHE_KEY_SEPARATOR_CHAR);

    // If the key ends with a slash
    if (lastSlash == key.length())
    {

      return "";
    }
    else
    {
      // The group-by field is everything following the slash
      return key.substring(lastSlash + 1);
    }
  }

  /**
   *  Helper method to return all ancestor keys for a key.
   */
  private String[] allAncestors(String key)
  {
    XPStringTokenizer tok = new XPStringTokenizer(key,
        CACHE_KEY_SEPARATOR_STR, false);

    int numAncestors = tok.GetTokenCount();

    if (key.endsWith(CACHE_KEY_SEPARATOR_STR))
    {
      numAncestors++;
    }

    String[] result = new String[numAncestors];
    result[0] = CACHE_KEY_SEPARATOR_STR;

    for (int j = 1; j < numAncestors; j++)
    {
      result[j] = result[j - 1] + tok.GetNextToken() +
        CACHE_KEY_SEPARATOR_STR;
    }

    return result;
  }

  /**
   *  Helper method to break a key into its ancestor keys,
   *  and return a single ancestor at the indicated depth.
   *   For example, if the key is "/subtype.3/ancestor.4/swr1/"
   *    then ancestorKey(key, 0) == "/"
   *         ancestorKey(key, 1) == "/subtype.3/"
   *         ancestorKey(key, 2) == "/subtype.3/ancestor.4/"
   */
  private String ancestorKey(String key, int depth)
    throws ArgumentBoundsException
  {
    XPStringTokenizer tok = new XPStringTokenizer(key,
        CACHE_KEY_SEPARATOR_STR, false);

    String result = CACHE_KEY_SEPARATOR_STR;

    for (int j = 0; j < depth; j++)
    {
      if (!tok.HasMoreTokens())
      {
        throw new ArgumentBoundsException("key", depth);
      }

      result += (tok.GetNextToken() + CACHE_KEY_SEPARATOR_STR);
    }

    return result;
  }

  /**
   *  Convenience method to check whether the search has been run
   *  yet
   */
  private void checkInitialized()
    throws NotInitializedException
  {
    if (!_initialized)
    {
      throw new NotInitializedException();
    }
  }

  /**
   *  Helper method to return the last uncategorized key before the
   *  given key, i.e.
   *   /subtype => /
   *   /subtype.0/ancestors => /subtype.0/
   */
  private String lastUncategorizedKey(String key)
  {
    if (key.endsWith(CACHE_KEY_SEPARATOR_STR))
    {
      return key;
    }

    int lastSlash = key.lastIndexOf(CACHE_KEY_SEPARATOR_CHAR);

    return key.substring(0, lastSlash + 1);
  }

  /**
   *  Clear out any results from previous searches
   */
  private void reinit(boolean createRequest)
    throws SearchExecutionException
  {
    _initialized     = false;

    // No search string or filter yet.
    _initialSearchString     = null;
    _initialBestBets = false;
    _advancedFilter       = null;

    // Default is basic search.
    _isAdvancedSearch     = false;

    // Default number of results per page.
    _pageSize     = 20;

    // No search within results
    _swrCount     = 0;

    // No error state.
    _error     = null;
    _warning     = null;

    // If there are any old cached responses, call their Clear() methods
    // to release memory ASAP, in case garbage collection doesn't run for
    // a long time.
    if (null != _responseCache) {
      IXPEnumerator e = _responseCache.GetElements();
      while (e.MoveNext()) {
        IPTSearchResponse r = (IPTSearchResponse) e.GetCurrent();
        r.Clear();
      }
    }

    // Initialize caches.
    _queryCache       = new XPHashtable();
    _crumbCache       = new XPHashtable();
    _responseCache     = new XPHashtable();

    // No search yet, so current key and response are null
    _currentKey       = null;
    _currentResponse     = null;

    // No removable settings
    _removableSettings = null;

    if (createRequest) {
      // Get the IPTSearchRequest we'll use for searches.
      try {
        _request = _session.GetSearchRequest();

        // Since this will always be an end-user search from some page in the
        // UI, use the short (30 second) timeout instead of the default 10-minute.
        _request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_USE_SHORT_TIMEOUT,
                             Boolean.TRUE);

	// Initialize the global "max number of drilldown groups" setting, using the
	// helper function in ReorganizationViewHelper.
	int maxNumGroups = ReorganizationViewHelper.GetMaxNumGroups();
	_request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_MAXNUMGROUPS,
			     new Integer(maxNumGroups));
      }
      catch (Exception e) {
		SearchUIUtility.RethrowCOMExecutionException(e, m_asOwner);
      }
    }
  }


  /**
   *  Helper method to translate and rethrow a COM exception as
   *  a SearchModelException
   */
  private void rethrowCOMModelException(Exception e)
    
  {
    
      log.Error(e, e.getMessage());
    

    XPStringBuilder sb = new XPStringBuilder(200);
    sb.Append(m_asOwner.GetString(345, "ptmsgs_portalinfrastructure"));
    sb.Append(e.getMessage());

    throw new SearchModelException(sb.ToString());
  }

  public static XPDateTime ConvertSearchDateTimeToXPDateTime(double searchTime) {
    // PTSearch indexes date/times as minutes since 2000/01/01 00:00:00 UTC.
    // XPDateTime wants milliseconds since 1970/01/01 00:00:00 UTC.

    long msec2000 = (long) (searchTime * 60000.);
    long msec1970 = msec2000 + 946684800000L;
    return new XPDateTime(msec1970);
  }

  public XPDateFormat GetDateFormat(AActivitySpace owner) {
    XPLocale xpLocale = PTPersonalSettingsHelper.GetPersonalSettingValue_Locale(owner);
    XPTimeZone xpTimeZone = PTPersonalSettingsHelper.GetPersonalSettingValue_TimeZone(owner);

    return XPDateFormat.GetDateTimeInstance(XPDateFormat.SHORT, XPDateFormat.SHORT,
                                            xpLocale, xpTimeZone);
  }

  	public void Filter(String key, int filterId) {
  		restoreState(lastUncategorizedKey(key));
  		// Go back to page one
  		SetFirstPageAndPageSize();
  		// add filter to search request
  		if (filterId < 0 /*no filter*/) {
  			ClearFilterOnSearch();
  		}
  		else if (filterId < SearchObjectTypes.MIN_SPECIAL_CLASSID_VALUE) {
  			_request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_OBJTYPES, new int[] {filterId});
  			_request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_APPS, new Integer(PT_SEARCH_APPS.PT_SEARCH_APPS_PORTAL));
  			_request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_ABSTYPES, new Integer(0));
  		} 
  		else if (filterId == SearchObjectTypes.CONTENT_APPLICATION_ON_DETAIL_PAGE) {
  			_request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_OBJTYPES, new int[] {});
  			_request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_APPS, new Integer(PT_SEARCH_APPS.PT_SEARCH_APPS_CONTENT));
  			_request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_ABSTYPES, new Integer(0));
  		}
  		else if (filterId == SearchObjectTypes.COLLAB_APPLICATION_ON_DETAIL_PAGE) {
  			_request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_OBJTYPES, new int[] {});
  			_request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_APPS, new Integer(PT_SEARCH_APPS.PT_SEARCH_APPS_COLLAB));
  			_request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_ABSTYPES, new Integer(0));
  		}
  		else if (filterId > SearchObjectTypes.MIN_SPECIAL_CLASSID_VALUE) {
  			_request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_OBJTYPES, new int[] {});
  			_request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_APPS, new Integer(0));
  			_request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_ABSTYPES, new Integer(filterId));
  		}
  		doSearch(null, null, null);
  	}
  	
  	private void ClearFilterOnSearch() {
  		int maxToDisplay = SearchObjectTypes.ENDUSERCOUNT;
		boolean bShouldDisplayAdminObjects = ((IPTSession) m_asOwner.GetUserSession()).GetSessionInfo().
        	CheckForActivityRight(PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_ACCESS_ADMINISTRATION);
	    if (bShouldDisplayAdminObjects && isAdvancedSearch()) {
	    	maxToDisplay = SearchObjectTypes.DETAILEDCOUNT;
	    }

		int[] searchObjectTypes = SearchObjectTypes.GetOrderedSearchObjectTypes();
		XPArrayList xpObjectTypes = new XPArrayList();
		boolean searchPortal = false;
		boolean searchContent = false;
		boolean searchCollab = false;
		for (int i = 0; i < maxToDisplay; i += 1)
	    {
			if (searchObjectTypes[i] < SearchObjectTypes.MIN_SPECIAL_CLASSID_VALUE) {
				xpObjectTypes.Add(XPConvert.ToString(searchObjectTypes[i]));
				searchPortal = true;
			} 
			else if (searchObjectTypes[i] == SearchObjectTypes.CONTENT_APPLICATION_ON_DETAIL_PAGE)
			{
				searchContent = ContentInstalled();
			}
			else if (searchObjectTypes[i] == SearchObjectTypes.COLLAB_APPLICATION_ON_DETAIL_PAGE)
			{
				searchCollab = CollabInstalled();
			}
	    }
		int[] arrObjectTypes = new int[xpObjectTypes.GetSize()];
		for (int i = 0; i < xpObjectTypes.GetSize(); i += 1)
	    {
			arrObjectTypes[i] = XPConvert.ToInteger(xpObjectTypes.GetElement(i));
	    }
		// Finally, add the application constraint
		int apps = 0;
		if (searchPortal) {
			apps |= PT_SEARCH_APPS.PT_SEARCH_APPS_PORTAL;
		}
		if (searchCollab) {
			apps |= PT_SEARCH_APPS.PT_SEARCH_APPS_COLLAB;
		}
		if (searchContent) {
			apps |= PT_SEARCH_APPS.PT_SEARCH_APPS_CONTENT;
		}
		
		_request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_OBJTYPES, arrObjectTypes);
		_request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_APPS, new Integer(apps));
		_request.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_ABSTYPES, new Integer(0));
  	}
  
  	public boolean ContentInstalled()
	{
		IPTSession ptSession = (IPTSession)m_asOwner.GetUserSession();
		IPTServerConfigSettings ptSC = (IPTServerConfigSettings) ptSession.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_SERVERCONFIGSETTINGS,
			false);

		int setting2 = ptSC.GetSettingAsInt(PT_SERVER_CONFIG_SETTINGNAME.PT_SETTINGNAME_PLUMTREE_CONTENT_SERVER_INSTALLED);
		return (setting2 ==1);
	}

	public boolean CollabInstalled()
	{
		IPTSession ptSession = (IPTSession)m_asOwner.GetUserSession();
		IPTServerConfigSettings ptSC = (IPTServerConfigSettings) ptSession.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_SERVERCONFIGSETTINGS,
		false);

		int set = ptSC.GetSettingAsInt(PT_SERVER_CONFIG_SETTINGNAME.PT_SETTINGNAME_PLUMTREE_COLLAB_SERVER_INSTALLED);
	  	return (set == 1);
	}
	
	public int GetSelectedTypeFilter() {
		int selectedFilter = -1;
	    int[] objTypefilter = GetObjectTypeRestrictions();
	    int abstractFilter = GetAbstractTypeRestrictions();
	    int appFilter = GetApplicationRestrictions();
	    if (null != objTypefilter && objTypefilter.length == 1) {
	    	selectedFilter = objTypefilter[0];
	    } else if (abstractFilter > 0) {
	    	selectedFilter = abstractFilter;
	    } else if (appFilter == PT_SEARCH_APPS.PT_SEARCH_APPS_CONTENT) {
	    	selectedFilter = SearchObjectTypes.CONTENT_APPLICATION_ON_DETAIL_PAGE;
	    } else if (appFilter == PT_SEARCH_APPS.PT_SEARCH_APPS_COLLAB) {
	    	selectedFilter = SearchObjectTypes.COLLAB_APPLICATION_ON_DETAIL_PAGE;
	    }
	    
	    return selectedFilter;
	}
}

