package com.plumtree.portaluiinfrastructure.statichelpers;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.classtypedescriptors.*;
import com.plumtree.portaluiinfrastructure.friendlyurl.*;
import com.plumtree.portaluiinfrastructure.opener.*;
import com.plumtree.server.*;
import com.plumtree.server.impl.core.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.application.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.helpers.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.web.*;


/**
 *    <p>This class provides a set of static helpers for constucting valid gateway URLs.
 *    Gateway URLs follow a well-defined format to communicate information about the request
 *    to the gateway control. This section describes the format of the gateway URLs. For simplicity
 *    portal url is assumed to be http://machinename/portal/server.pt</p>
 * 
 *    <p>The gateway URLs are identified by a special substring "gateway/PTARGS_" that should follow
 *    the portal URL. So, a general gateway URL looks like:
 *    <BR><BR>
 *    		<B>&nbsp;&nbsp;&nbsp;http://machinename/portal/server.pt/gateway/PTARGS_1_2_3_4_5_6/http://remotemachine/url_to_gateway/somefile.html</B>
 *    <BR><BR>
 *    where 1,2,3,4,5,6 are numeric arguments described below.</p>
 * 
 * 	<OL>
 *    	<LI>The first argument in an integer that contains a bitmask of gateway flags and type of associated gadget content
 *     that is being requested.
 * 		<UL>
 * 		<LI>
 *    		<B>POPUPFLAG</B>	-	notifies the gateway that requested page will be displayed in the popup window and therefore
 *    		gateway should set the RETURN-URI to "close window" javascript.</LI>
 * 		<LI>
 *    		<B>RETURNFLAG</B>  - 	notifies the gateway that return URI is requested. The gateway will check that POPUPFLAG and
 *    		either redirect to the last visited community or mypage or close the window.</LI>
 * 		<LI>
 *    		<B>REDIRECTGUEST</B> -	notifies the gateway that it should redirect guest users to the login page before displaying
 *    		the request resource. (This is mostly used by Collaboration server)</LI>
 * 		</UL>
 * 	</LI>
 * 	<LI>The second argument is a numeric id of the user requesting the page. This argument is not used by the gateway
 *     and is intended for PPC PTTracker tool that analyzes the logs from the webserver. If user id is not known you can
 *     set it to any value, but 0 is prefered.
 * 	</LI>
 * 	<LI>The third argument is a numeric id of the object that is being requested. For example, if you are requesting
 *     a page through gadget 201, then this argument should be set to 201.
 * 	</LI>
 * 	<LI>The fourth argument is a community id.</LI>
 * 	<LI>The fifth arguemnt is a page id.</LI>
 * 	<LI>The sixth argument is a resource type id. It defines what kind of object is identified by the object id provided
 *     in the third argument. Refer to Plumtree.idl to see what are numeric values for various resource types. Gateway supports
 *    	the following resource types: Gadget, WebService, Card, DataSource.</LI>
 * 	</OL>
 *  		
 * @author daniilk
 */
public class GatewayHelpers
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_GATEWAY),
		"com.plumtree.portaluiinfrastructure.statichelpers.GatewayHelpers");

	
	public static final String SPECIAL_HEADER_NAME		  = "PTParams";
	public static final String GATEWAY_URL_PREFIX		  = "/gateway/PTARGS_";
	/** this variable is set in the static intializer for this class **/
	public static String GATEWAY_FRIENDLYURL_KEYWORD = null;
	public static final String GATEWAY_ARG_DELIMITER		  = "_";
	public static final String PORTLET_SAVED_ON_SESSION   = "PortletBeingEdited";
	public static final String LAST_VISITED_HOSTED_PAGE	  = "LastVisitedHostGatewayPage";
	public static final String GATEWAY_HOSTED_MODE_HEADER = "csp-display-mode";
	public static final String GATEWAY_SPACE_NAME = "Gateway";

	
	public static final int NOFLAGS				 = 0x00;
	public static final int POPUPFLAG			 = 0x08;
	public static final int RETURNFLAG			 = 0x10;
	public static final int REDIRECTGUEST		 = 0x20;
	public static final int REFRESHFLAG	 		 = 0x40;
	public static final int POPUPREFRESHFLAG	 = REFRESHFLAG | POPUPFLAG;
	public static final int ADMINPREFS_POPUP     = (PT_ASSOCIATEDCONTENT.PT_ASSOCIATEDCONTENT_ADMINPREFSPAGE + 1) | POPUPFLAG;
	public static final int ADMINPREFS			 = PT_ASSOCIATEDCONTENT.PT_ASSOCIATEDCONTENT_ADMINPREFSPAGE + 1;
	public static final int GADGETPREFS			 = (PT_ASSOCIATEDCONTENT.PT_ASSOCIATEDCONTENT_PREFSPAGE + 1);
	public static final int GADGETPREFS_POPUP    = (PT_ASSOCIATEDCONTENT.PT_ASSOCIATEDCONTENT_PREFSPAGE + 		1) | POPUPREFRESHFLAG;
	public static final int COMMUNITYPREFS		 = (PT_ASSOCIATEDCONTENT.PT_ASSOCIATEDCONTENT_COMMUNITYPREFSPAGE + 1);
	public static final int COMMUNITYPREFS_POPUP = (PT_ASSOCIATEDCONTENT.PT_ASSOCIATEDCONTENT_COMMUNITYPREFSPAGE + 1) | POPUPFLAG;
	public static final int HELP		  = (PT_ASSOCIATEDCONTENT.PT_ASSOCIATEDCONTENT_HELPPAGE + 1);
	public static final int HELP_POPUP    = (PT_ASSOCIATEDCONTENT.PT_ASSOCIATEDCONTENT_HELPPAGE + 1) | POPUPFLAG;
	public static final int USERPLUGPAGE  = PT_ASSOCIATEDCONTENT.PT_ASSOCIATEDCONTENT_PERSONAL_PLUG + 1;
	public static final int ADMINPLUGPAGE = PT_ASSOCIATEDCONTENT.PT_ASSOCIATEDCONTENT_ADMIN_PLUG + 1;
	public static final int PORTLETCONTENT = PT_ASSOCIATEDCONTENT.PT_ASSOCIATEDCONTENT_PORTLETCONTENT + 1;

	public static final String PT_REQUEST_TYPE_HEADER = "PT-HTTPRequest-Type";
	public static final String PT_CLIENT_SIDE_HEADER_VALUE = "CLIENT_SIDE";
	public static final String PT_PORTLET_REFRESH_HEADER_VALUE = "PORTLET_REFRESH";
	public static final String PT_REFERER_HEADER = "Referer";
	public static final String CLOSEPOPUPJAVASCRIPT = 
		"<HTML><BODY onload=\"window.close();\"></BODY></HTML>";
	public static final String CLOSEPOPUPANDREFRESHJAVASCRIPT =
		"<HTML><BODY onload=\"parent.opener.location.reload();window.close();\"></BODY></HTML>";

	public static final String UI_STD		= "";
	public static final String UI_508 = "http://www.plumtree.com/uis/section508";
	public static final String UI_LOW = UI_508;
	public static final String UI_WML = "http://www.plumtree.com/uis/wml";
	public static final String UI_PDA = "http://www.plumtree.com/uis/pda";
	public static final String DEV_SUFFIX_PDA = "_HTML32";
	public static final String DEV_SUFFIX_WML = "_WML";

	public static final String DEFAULT_FRIENDLYGATEWAYURL_KEYWORD = "gw";
	public static final String URL_SEPARATOR = "/";
	
	public static final String GWARG_PORTLET_PREFIX 	= Global.GWARG_PORTLET_PREFIX;
	public static final String GWARG_WEBSRV_PREFIX 		=  Global.GWARG_WEBSRV_PREFIX;
	public static final String GWARG_COMMUNITY_PREFIX 	=  Global.GWARG_COMMUNITY_PREFIX;
	public static final String GWARG_PAGE_PREFIX 		=  Global.GWARG_PAGE_PREFIX;
	public static final String GWARG_PREFTYPE_PREFIX 	=  Global.GWARG_PREFTYPE_PREFIX;
	public static final String GWARG_USER_PREFIX 		=  Global.GWARG_USER_PREFIX;
	
	public static final String OPENCONFIG_GATEWAY_COMPONENT	   = "Gateway";
	public static final String OPENCONFIG_ALLOW_EXTGWREDIRECTS = "AllowExternalGatewayRedirect";
	
	static {
		// This code block allows us to tell the Interpreter not to parse the POST body of gateway
		// requests.  Otherwise we won't be able to access the Input Stream.
		
		// lookup gateway friendly url keyword
		IApplication application = ApplicationManager.GetInstance().GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString());
		IFriendlyURLManager ptFriendlyURLMgr = (IFriendlyURLManager) application.GetCachingManager().GetEntry(AppWarmUp.KEY_CACHEMAN_CLASS_TYPE_MANAGER);
		GATEWAY_FRIENDLYURL_KEYWORD = ptFriendlyURLMgr.LookupFriendlyURLKeyFromClassId(FriendlyURLConstants.FRIENDLY_GATEWAY_URL);
		// if gateway friendly url is enabled
		if ( GATEWAY_FRIENDLYURL_KEYWORD != null ) {
			com.plumtree.uiinfrastructure.interpreter.Interpreter.AddNonFormDataRequestPattern( URL_SEPARATOR+GATEWAY_FRIENDLYURL_KEYWORD+URL_SEPARATOR );
		}
		// always add the regular gateway pattern, it is always supported even if friendly gateway urls are enabled
		com.plumtree.uiinfrastructure.interpreter.Interpreter.AddNonFormDataRequestPattern( GATEWAY_URL_PREFIX );
	}

	/**
	 * This method is used to determine the current UI mode of the portal. UI mode depends both
	 * on the user setting (Standard, 508, Low Bandwidth) and type of browser (PDA, Phone or full-featured
	 * browser like IE or Mozilla). 
	 * 
	 * @param xpRequest - current request object
	 * @param asOwner	- current activity space
	 * @return This method returns one of the following string constants: UI_STD, UI_508, UI_LOW, UI_WML, UI_PDA
	 */ 
	public static String GetUIMode(IXPRequest xpRequest, AActivitySpace asOwner)
	{
		try
		{
			int iStyleID = asOwner.GetAccessStyle().GetAccessID();
			
			if (iStyleID == AccessStyles.SECTION_508.GetAccessID())
			{
				return UI_508;
			}
			else if (iStyleID == AccessStyles.LOW_BANDWIDTH.GetAccessID())
			{
				return UI_LOW;
			}
			else
			{
				String strAccepts	 = xpRequest.GetHeader("Accept");
				String strUserHeader = xpRequest.GetHeader("User-Agent");
				DevicesVarPack vpPTD = (DevicesVarPack) asOwner.GetApplication().
							GetVarPackManager().GetVariablePackage(DevicesVarPack.VARPACK_ID);
				if (vpPTD != null)
				{
					String sDevSuffix    = vpPTD.GetDevice(strAccepts, strUserHeader);
					
					if (sDevSuffix != null && DEV_SUFFIX_PDA.compareTo(sDevSuffix) == 0)
					{
						return UI_PDA;
					}
					else if (sDevSuffix != null && DEV_SUFFIX_WML.compareTo(sDevSuffix) == 0)
					{
						return UI_WML;
					}
				}
			}
		}
		catch (Exception e)
		{
			log.Warn(e, "Exception trying to get AccessStyle.");
		}
		return UI_STD;
	}


	/**
	 *  This function returns absolute URL to the admin prefs page.
	 *
	 *     @param bPopup    -     boolean flag to indicate if the admin page will be displayed
	 *                         in a separate popup window or not. All this flag does is tells
	 *                         gateway to set Return-URI action to "close window".
	 */
	public static String ConstructAdminPrefsLink(AActivitySpace asOwner,
		boolean bPopup)
	{
		return ConstructPrefPageLink(asOwner, 0, 0, 0,
			bPopup ? ADMINPREFS_POPUP : ADMINPREFS);
	}

	/**
	 *  This function returns absolute URL to the admin prefs page.
	 *
	 *     @param bPopup    -     boolean flag to indicate if the admin page will be displayed
	 *                         in a separate popup window or not. All this flag does is tells
	 *                         gateway to set Return-URI action to "close window".
	 */
	public static String ConstructTemplateAdminPrefsLink(AActivitySpace asOwner,
		boolean bPopup)
	{
		return ConstructPrefPageLink(asOwner, 0, 0, 0,
			bPopup ? ADMINPREFS_POPUP : ADMINPREFS, PT_CLASSIDS.PT_GADGETTEMPLATE_ID);
	}

	/**
	 *  This function returns absolute URL to the admin prefs page and treats it as popup.
	 */
	public static String ConstructAdminPrefsLink(AActivitySpace asOwner)
	{
		return ConstructPrefPageLink(asOwner, 0, 0, 0, ADMINPREFS_POPUP);
	}

	/**
	 *  This function returns absolute URL to the community prefs page.
	 *
	 *  @param iPortletID - ID of the portlet for which the gadget community preferences are set
	 *
	 *  @param iCommunityID - ID of the community for which the gadget community preferences are set
	 *
	 *     @param bPopup    -     boolean flag to indicate if the pref page will be displayed
	 *                         in a separate popup window or not. All this flag does is tells
	 *                         gateway to set Return-URI action to "close window".
	 */
	public static String ConstructCommunityPrefsLink(AActivitySpace asOwner,
		int iPortletID, int iCommunityID, boolean bPopup)
	{
		return ConstructPrefPageLink(asOwner, iPortletID, iCommunityID, 0,
			bPopup ? COMMUNITYPREFS_POPUP : COMMUNITYPREFS);
	}

	/**
	 *  This function returns absolute URL to the community prefs page.
	 *
	 *  @param iPortletID - ID of the portlet for which the gadget community preferences are set
	 *
	 *  @param iCommunityID - ID of the community for which the gadget community preferences are set
	 *
	 *  @param iPageID        - ID of the community page which the gadget community preferences are set
	 *
	 *     @param bPopup    -     boolean flag to indicate if the pref page will be displayed
	 *                         in a separate popup window or not. All this flag does is tells
	 *                         gateway to set Return-URI action to "close window".
	 */
	public static String ConstructCommunityPrefsLink(AActivitySpace asOwner,
		int iPortletID, int iCommunityID, int iPageID, boolean bPopup)
	{
		return ConstructPrefPageLink(asOwner, iPortletID, iCommunityID, iPageID,
			bPopup ? COMMUNITYPREFS_POPUP : COMMUNITYPREFS);
	}

	/**
	 *  This function returns absolute URL to the gadget prefs page.
	 *
	 *  @param iPortletID - ID of the portlet for which the gadget preferences are set
	 *
	 *     @param bPopup    -     boolean flag to indicate if the pref page will be displayed
	 *                         in a separate popup window or not. All this flag does is tells
	 *                         gateway to set Return-URI action to "close window".
	 */
	public static String ConstructPortletPrefsLink(AActivitySpace asOwner,
		int iPortletID, boolean bPopup)
	{
		return ConstructPrefPageLink(asOwner, iPortletID, 0, 0,
			bPopup ? GADGETPREFS_POPUP : GADGETPREFS);
	}

	/**
	 *  This function returns absolute URL to the gadget prefs page.
	 *
	 *  @param iPortletID - ID of the portlet for which the gadget preferences are set
	 *
	 *  @param iPageID        - ID of the community page which the gadget preferences are set
	 *
	 *     @param bPopup    -     boolean flag to indicate if the pref page will be displayed
	 *                         in a separate popup window or not. All this flag does is tells
	 *                         gateway to set Return-URI action to "close window".
	 */
	public static String ConstructPortletPrefsLink(AActivitySpace asOwner,
		int iPortletID, int iPageID, boolean bPopup)
	{
		return ConstructPrefPageLink(asOwner, iPortletID, 0, iPageID,
			bPopup ? GADGETPREFS_POPUP : GADGETPREFS);
	}

	/**
	 *  This function returns absolute URL to the community user prefs page.
	 *
	 *  @param iPortletID - ID of the portlet for which the gadget community user preferences are set
	 *
	 *  @param iCommunityID - ID of the community for which the gadget community user preferences are set
	 *
	 *  @param iPageID        - ID of the community page which the gadget community user preferences are set
	 *
	 *     @param bPopup    -     boolean flag to indicate if the prefs page will be displayed
	 *                         in a separate popup window or not. All this flag does is tells
	 *                         gateway to set Return-URI action to "close window".
	 */
	public static String ConstructPortletUserCommunityPrefsLink(
		AActivitySpace asOwner, int iPortletID, int iCommunityID, boolean bPopup)
	{
		return ConstructPrefPageLink(asOwner, iPortletID, iCommunityID, 0,
			bPopup ? GADGETPREFS_POPUP : GADGETPREFS);
	}

	/**
	 *  This function returns absolute URL to the help page.
	 *
	 *  @param iPortletID - ID of the portlet for which the help should be displayed
	 *
	 *  @param iCommunityID - ID of the community for which the help should be displayed
	 *
	 *  @param iPageID        - ID of the community page which the help should be displayed
	 *
	 *     @param bPopup    -     boolean flag to indicate if the help page will be displayed
	 *                         in a separate popup window or not. All this flag does is tells
	 *                         gateway to set Return-URI action to "close window".
	 */
	public static String ConstructHelpLink(AActivitySpace asOwner,
		int iPortletID, int iCommunityID, int iPageID, boolean bPopup)
	{
		return ConstructPrefPageLink(asOwner, iPortletID, iCommunityID, iPageID,
			bPopup ? HELP_POPUP : HELP);
	}

	/**
	 * This method constructs gateway URL that points to a specific associated resource (preference page, help page).
	 * If this method is called with iPrefType parameter set to 0, the result is a gateway prefix that should be 
	 * followed by the actual absolute URL to the gatewayed resource.
	 *
	 * @param asOwner 			- current activity space object
	 * @param iPortletID 		- portlet id
	 * @param iCommunityID 		- community ID (0 if not applicable)
	 * @param iPageID 			- page ID (0 if not applicable)
	 * @param iPrefType 		- preference type
	 *
	 * @return 		string that is a gateway URL.
	 */
	public static String ConstructPrefPageLink(AActivitySpace asOwner,
		int iPortletID, int iCommunityID, int iPageID, int iPrefType)
	{
		return ConstructPrefPageLink(asOwner, iPortletID, iCommunityID, iPageID, iPrefType, PT_CLASSIDS.PT_GADGET_ID);
	}

	/**
	 * This method constructs gateway URL that points to a specific associated resource (preference page, help page).
	 * If this method is called with iPrefType parameter set to 0, the result is a gateway prefix that should be 
	 * followed by the actual absolute URL to the gatewayed resource.
	 *
	 * @param asOwner 			- current activity space object
	 * @param iPortletID 		- portlet or web service id
	 * @param iCommunityID 		- community ID (0 if not applicable)
	 * @param iPageID 			- page ID (0 if not applicable)
	 * @param iPrefType 		- preference type
	 * @param iClassID 			- class id that describles iPortletID parameter.
	 *
	 * @return 		string that is a gateway URL.
	 */
	public static String ConstructPrefPageLink(AActivitySpace asOwner,
		int iPortletID, int iCommunityID, int iPageID, int iPrefType,
		int iClassID)
	{	   
		return ConstructGatewayURL(asOwner.GetCurrentRequestURL(), 
								   iPortletID, iCommunityID, iPageID, iPrefType, iClassID, 
								   "");
	}

	/**
	 * This method generates a gateway URL pointing to a resource specified by strURLToGateway parameter.
	 * 
	 * @param strCurrentRequestURL	- the URL of the current request 
	 * 								  (normally, the result of AActivitySpace.GetCurrentRequestURL())
	 * 
	 * @param iPortletID			- numeric ID of the portlet object or webservice object
	 * @param iCommunityID			- community ID (0 if not applicable)
	 * @param iPageID				- page ID (0 if not applicable)
	 * @param iPrefType				- preference type. For normal portlet click-throughs it should be 0.
	 * @param iClassID				- class ID. This arguments tells whether second argument of this method is 
	 * 								  portlet ID or webservice ID.
	 * @param strURLToGateway		- absolute URL to the resource that should be gatewayed (i.e. http://www.google.com)
	 * 
	 * @return						- a gateway URL string that points to the resource specified.
	 */
	public static String ConstructGatewayURL(String strCurrentRequestURL, int iPortletID, int iCommunityID, 
											 int iPageID, int iPrefType, int iClassID, String strURLToGateway)
	{
		return ConstructGatewayURL(strCurrentRequestURL, iPortletID, iCommunityID, iPageID, iPrefType, iClassID, 0, strURLToGateway);
	}
	
	/**
	 * This method generates a gateway URL pointing to a resource specified by strURLToGateway parameter.
	 * 
	 * @param curReqUrl - the URL of the current request (normally, the result of AActivitySpace.GetCurrentRequestURL())
	 * @param objId		- numeric ID of the portlet object or webservice object
	 * @param commId	- community ID (0 if not applicable)
	 * @param pageId	- page ID (0 if not applicable)
	 * @param prefType	- preference type. For normal portlet click-throughs it should be 0.
	 * @param classId	- This arguments tells whether second argument of this method is portlet ID or webservice ID.
	 * @param userId	- user id, or 0 not to use one
	 * @param gatewayPath - absolute URL to the resource that should be gatewayed (i.e. http://www.google.com)
	 * @return			- a gateway URL string that points to the resource specified.
	 */
	public static String ConstructGatewayURL(String curReqUrl, int objId, int commId, int pageId, int prefType, int classId, int userId, String gatewayPath) 
	{
		int iSecurityMode = PTConfigHelper.GetSecurityMode();

		boolean bSecureOnlyMode = iSecurityMode > 1;
		String strAppBaseURL = "";

		/* if we got a valid request URL */
		if (null != curReqUrl && curReqUrl.length() > 0)
		{
			
			strAppBaseURL = ConfigService.getInstance().GetApplicationBaseURL(
								ConfigHelper.GetNormalizedBaseURL(curReqUrl),
								bSecureOnlyMode 
								|| curReqUrl.startsWith(ConfigHelper.HTTPS_SCHEME));
		}
		else /* it was messed up */
		{
			strAppBaseURL = ConfigHelper.GetMainServerApplicationURL(false);	
		}
		
		/* GetMainServerApplicationURL returns URL with "?" appended. Since it is too late 
		 * to change it and bugging Julie is not part of my plans for the nearest future i 
		 * will just fix the URL here. In case someone will ever decide to fix it, we will 
		 * take an intelligent approach and search for "?" first, instead of blindly removing
		 * the last character.
		 * 		-- Moose
		 */
		if (null != strAppBaseURL)
		{
			int iQuestionMarkLocation = strAppBaseURL.indexOf("?");
			if (-1 != iQuestionMarkLocation)
			{
				strAppBaseURL = strAppBaseURL.substring(0, iQuestionMarkLocation);
			}
		}
		
		
		String frGWkeyword = GetFriendlyGatewayURLKeyword();
		if( null == frGWkeyword ) {
			return ConstructRegularGatewayURL(strAppBaseURL, objId, commId, pageId, prefType, classId, userId, gatewayPath);
		} else {
			return ConstructFriendlyGatewayURL(strAppBaseURL, objId, commId, pageId, prefType, classId, userId, gatewayPath);
		}
	}
	
	/**
	 * This method generates a gateway URL pointing to a resource specified by strURLToGateway parameter.
	 * 
	 * @param curReqUrl - the URL of the current request (normally, the result of AActivitySpace.GetCurrentRequestURL())
	 * @param objId		- numeric ID of the portlet object or webservice object
	 * @param commId	- community ID (0 if not applicable)
	 * @param pageId	- page ID (0 if not applicable)
	 * @param prefType	- preference type. For normal portlet click-throughs it should be 0.
	 * @param classId	- This arguments tells whether second argument of this method is portlet ID or webservice ID.
	 * @param userId	- user id, or 0 not to use one
	 * @param gatewayPath - absolute URL to the resource that should be gatewayed (i.e. http://www.google.com)
	 * @return			- a gateway URL string that points to the resource specified.
	 */
	public static String ConstructRegularGatewayURL(String strAppBaseURL, int objId, int commId, int pageId, int prefType, int classId, int userId, String gatewayPath)
	{
		return strAppBaseURL + GATEWAY_URL_PREFIX + prefType + "_" + userId + "_" + objId + "_" + commId + "_"  
			   + pageId + "_" + classId + "/" + CreateGatewayFriendlyURL(gatewayPath);
	}
	
	/**
	 * 
	 * @return null if friendly gateway urls is disabled
	 */
	public static String GetFriendlyGatewayURLKeyword()
	{
		return GATEWAY_FRIENDLYURL_KEYWORD;
	}
	
	/**
	 * This method generates a gateway URL pointing to a resource specified by strURLToGateway parameter.
	 * 
	 * @param appBaseUrl- 
	 * @param objId		- numeric ID of the portlet object or webservice object
	 * @param commId	- community ID (0 if not applicable)
	 * @param pageId	- page ID (0 if not applicable)
	 * @param prefType	- preference type. For normal portlet click-throughs it should be 0.
	 * @param classId	- This arguments tells whether second argument of this method is portlet ID or webservice ID.
	 * @param userId	- user id, or 0 not to use one
	 * @param gatewayPath - absolute URL to the resource that should be gatewayed (i.e. http://www.google.com)
	 * @return			- a gateway URL string that points to the resource specified.
	 */
	public static String ConstructFriendlyGatewayURL(String appBaseUrl, int objId, int commId, int pageId, int prefType, int classId, int userId, String gatewayPath) 
	{
		return Global.ConstructFriendlyGatewayURI(GetFriendlyGatewayURLKeyword(), appBaseUrl, objId, commId, pageId, prefType, classId, userId, gatewayPath);
		
	}
	/**
	 * Used by AppDataObject to creates a partial gateway url with just the preference type and user id to send
	 * to the server, which will complete out the second half of the gateway url. 
	 * 
	 * 
	 * @return
	 */
	public static String ConstructPartialGatewayURL(String appBaseUrl, int prefType, int userId)
	{
		return appBaseUrl + GATEWAY_URL_PREFIX + prefType + "_" + userId + "_";
	}
	
	/**
	 * Used by AppDataObject to creates a partial gateway url with just the preference type and user id to send
	 * to the server, which will complete out the second half of the gateway url.
	 * 
	 * This method creates a friendly gateway url
	 * 
	 * @param appBaseUrl
	 * @param prefType
	 * @param userId
	 * @return null if friendly gateway url is disabled
	 */
	public static String ConstructPartialFriendlyGatewayURLKeyword(String appBaseUrl, int prefType, int userId)
	{
		String frGWkw = GetFriendlyGatewayURLKeyword();
		if ( null != frGWkw ) {
			return appBaseUrl + URL_SEPARATOR + frGWkw + URL_SEPARATOR +
				// only add in arguments if they have values
				(( prefType == 0 ) ? "" : GWARG_PREFTYPE_PREFIX+prefType )+
				(( prefType > 0 && userId > 0) ? "_" : "" )+ // add a separator if both arguments have values
				(( userId == 0 )   ? "" : GWARG_USER_PREFIX+userId);
		}
		return null;
		
	}
	
	/**
	 * procedure:	ConstructReturnURI
	 * purpose: 
	 * @param asOwner
	 * @param iPageID
	 * @param iCommunityID
	 * @param bPopup
	 * @param bRefreshPopup
	 * @return
	 */
	public static String ConstructReturnURI(AActivitySpace asOwner,
											  int iPageID, int iCommunityID, 
											  boolean bPopup, boolean bRefreshPopup)
	{
		// SE: Bug 55717 - Fix for Unicredit. Backport of DavidP's fix (CL 238479) to get rid of extra 302 redirect
		// in cases where a GET or POST is done on a portlet in normal view (Non-popups and non-gatewayed pages).
		// DAP: This redirect refers to a portal page *only*.
		String retURL = "";

		// If we are not redirecting to a MyPage or Community Page, then
		// we need to redirect the "old way" using ConstructPrefPageLink.
		// DLH Updated fix for bug #58040: use the "old way" 302 redirect for 
		// utilities that return to admin UI.
		boolean bIsPTPageRedirect = ((iCommunityID != 0) || (iPageID != 0));

		// DAP: For popups, we do require a (second) 302 redirect processed 
		// by the gateway (this is "old way") as a page request
		if (bPopup || bRefreshPopup || !bIsPTPageRedirect)
		{
			int iFlags = RETURNFLAG;
			iFlags |= bPopup ? POPUPFLAG : 0;
			iFlags |= bRefreshPopup ? POPUPREFRESHFLAG : 0;

			retURL = ConstructPrefPageLink(asOwner, 0, iCommunityID, iPageID, iFlags);
			
			log.Debug("GatewayHelpers.ConstructReturnURI - Constructing return URL for popup: [" + retURL + "]");
		}
		else // DAP: Normal flat view mode refresh can use opener directly to avoid excess 302 redirect
		{
			if (iCommunityID > 0)
			{
				if (iPageID == 0) {
					// If we have a community, but not page ID, then try to open the community itself.
					retURL = PTOpenerLinks.GetOpenerURLOpenObjID(PT_CLASSIDS.PT_COMMUNITY_ID, iCommunityID, IPTOpenerConstants.OPENER_MODE_VIEW, asOwner);
				} else {
					retURL = PTOpenerLinks.GetOpenerURLOpenObjID(PT_CLASSIDS.PT_PAGE_ID, iPageID, IPTOpenerConstants.OPENER_MODE_VIEW, asOwner);
				}
				log.Debug("GatewayHelpers.ConstructReturnURI - Constructing return URL for Community Page: [" + retURL + "]");
			}
			else
			{
				retURL = PTOpenerLinks.GetOpenerURLOpenObjID(PT_CLASSIDS.PT_MYPAGESCONTAINER_ID, iPageID, IPTOpenerConstants.OPENER_MODE_VIEW, asOwner);
				log.Debug("GatewayHelpers.ConstructReturnURI - Constructing return URL for normal view page: [" + retURL + "]");
			}
		}
		
		return retURL;
	}
	
	/**
	 * Helper function to encode
	 * 
	 * @param strURL
	 * @return
	 */
	public static String CreateGatewayFriendlyURL(String strURL)
	{
		int iPos = strURL.indexOf('?');
		String strResult = "";

		// NOTE: I'm not sure why this doesn't use the Global.PTGatewayEncode method.
		// Converting pre-existing ;'s to their % encoded form (%3b) seems problematic,
		// because the decode method in globals converts all %3Bs into ;'s, which means they
		// could be converted back to colons incorrectly.  However, this method also encodes
		// the % as %25 in the following line, leaving you with %253b, which may allow
		// this to work.  This code really needs a comment explaining why it's doing this.

		if (-1 == iPos)
		{
			strResult = GeneralHelpers.Replace(strURL, ";", "%3B");
			strResult = GeneralHelpers.Replace(strResult, "%", "%25");
			strResult = GeneralHelpers.Replace(strResult, "://", ";/");
			strResult = GeneralHelpers.Replace(strResult, ":", ";");
			return strResult;
		}
		else
		{			
			String strURLPath = strURL.substring(0, iPos);
			strResult = GeneralHelpers.Replace(strURLPath, ";", "%3B");
			strResult = GeneralHelpers.Replace(strResult, "%", "%25");
			strResult = GeneralHelpers.Replace(strResult, "://", ";/");
			strResult = GeneralHelpers.Replace(strResult, ":", ";");
			return strResult + strURL.substring(iPos);
		}
	}	
	
	/**
	 * Converts a friendly gateway url to regular.
	 * 
	 * 
	 * Legend: {} = required, [] = optional, | = or, <> = value
	 * Friendly format: /gw/{pt|ws<obj id>}[_p<page id>][_c<community id>][_u<user id>][_r<preference type>]/[<gateway path>]
	 * Regular format:  /gateway/PTARGS_<preference type>_<user id>_<object id>_<community id>_<page id>_<object type>/[<gateway path>]
	 * 
	 * @param friendlyUrl
	 * @param keyword
	 * @return
	 */
	public static String ConvertToRegularGatewayURL(String friendlyUrl, String keyword)
	{
		// First we need to separate the server base url, gateway keyword, gateway arguments and gateway path
		// Then, parse the gateway arguments
		// and finally construct the gateway url 
		
		// Tokenize the string based 
		XPStringTokenizer urlSeg = new XPStringTokenizer(friendlyUrl, URL_SEPARATOR, true);
		// This string buffer for the server name of the url, e.g. http://myserver:8080
		XPStringBuilder baseUrl = new XPStringBuilder();
		// To store the token with the gateway arguments, e.g. /ws3420_c2340_p22342_u2342/
		String argsSeg = null;
		// This string buffer for the portlet/webservice path e.g. /path1/path2/file1
		XPStringBuilder gwPathSeg = new XPStringBuilder();
		// flag to signal if we got to the friendly gateway keyword, -1 for before, 0 for at, and 1 for after
		int postKeyUrlSegment = -1;
		// Iterate through the tokens. Add tokens to the base url buffer if we haven't encountered the keyword,
		// and add tokens to the path segment buffer if we already encountered the keyword
		while ( urlSeg.HasMoreTokens() ) {
			String cur = urlSeg.GetNextToken();
			if ( postKeyUrlSegment == -1 ) {;
				if ( keyword.equals(cur) ) {
					postKeyUrlSegment = 0; // found the keyword, skip it and handle the arguments the next iteration
					// remove the last slash before the gateway keyword
					baseUrl.RemoveCharAt(baseUrl.GetLength()-1);
					// don't do anything with the key token
				} else { // any token before the keyword is part of the baseurl
					baseUrl.Append(cur);
				}
			} else if ( URL_SEPARATOR.equals(cur) ) {
				// after the base url, ignore the slashes
				continue;
			} else if ( postKeyUrlSegment == 0 ) {
				argsSeg = cur; // keep the argument token to parse later
				postKeyUrlSegment++;
			} else if ( postKeyUrlSegment >= 1 ) {
				// there is only the path after the keyword and arguments 
				if ( postKeyUrlSegment != 1 ) {
					gwPathSeg.Append(URL_SEPARATOR);
				}
				gwPathSeg.Append(cur);
				postKeyUrlSegment++;
			}
		}
		// parse the args, in this format: {pt|ws<obj id>}[_p<page id>][_c<community id>][_u<user id>][_r<preference type>]/[<gateway path>]
		XPStringTokenizer args = new XPStringTokenizer(argsSeg, "_");
		int objectType = 0;
		int objectId = 0;
		int commId = 0;
		int pageId = 0;
		int prefType = 0;
		int userId = 0;
		
		// go through the gw arguments, make sure a portlet or webservice id is included
		// if there are errors processing ids for arguments, just set that value to 0
		boolean kwFound = false;
		while ( args.HasMoreTokens() ) {
			String arg = args.GetNextToken();
			if ( arg.startsWith(GWARG_WEBSRV_PREFIX) || arg.startsWith(GWARG_PORTLET_PREFIX) ) {
				objectType = ( arg.startsWith(GWARG_WEBSRV_PREFIX) ? PT_CLASSIDS.PT_WEBSERVICE_ID : PT_CLASSIDS.PT_GADGET_ID );
				objectId = GetId(arg.substring(2));
				kwFound = true;
			
			} else if ( arg.startsWith(GWARG_COMMUNITY_PREFIX) ) {
				commId = GetId(arg.substring(1));
			} else if ( arg.startsWith(GWARG_PAGE_PREFIX) ) {
				pageId = GetId(arg.substring(1));
			} else if ( arg.startsWith(GWARG_PREFTYPE_PREFIX) ) {
				prefType = GetId(arg.substring(1));
			} else if ( arg.startsWith(GWARG_USER_PREFIX) ) {
				userId = GetId(arg.substring(1));
			} else {
				
			}
		}
		if ( !kwFound ) {
			objectId = 0;
		}
		
		// construct regular url
		String regGWUrl = GatewayHelpers.ConstructRegularGatewayURL(baseUrl.ToString(), objectId, commId, pageId, prefType, objectType, userId, "");
		// don't want to encode the gateway path again so don't pass it in to the convert function 
		return regGWUrl + gwPathSeg.ToString();
		
	}
	
	/**
	 * 
	 * @param url
	 * @param keyword
	 * @return
	 */
	public static boolean IsFriendlyGatewayURL(String url, String keyword)
	{
		// sanity checks
		if ( keyword == null || "".equals(keyword) || url == null ) {
			return false;
		}
		return (-1 != url.indexOf(URL_SEPARATOR+keyword+URL_SEPARATOR));
	}
	
	/**
	 * Return 0 if the excpected id is not numeric
	 * 
	 * @param stringId
	 * @return
	 */
	public static int GetId(String stringId)
	{
		try { 
			return XPConvert.ToInteger(stringId);
		} catch (XPException e) {
			// no a valid number, just return 0
			return 0;
		}
	}
}
