package com.plumtree.portaluiinfrastructure.tags.common;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.search.*;
import com.plumtree.portaluiinfrastructure.tags.data.*;
import com.plumtree.uiinfrastructure.activityspace.*;

/**
 * This class generates pagination data from a search result
 * for use in various paginatin tags.
 */
public class SearchPaginationHelper
{
	public AActivitySpace m_asOwner;
	public ISearchResultModelRO m_model;
	public ISearchURLHelper m_URLHelper;

	public int m_nFirstPageToDisplay;
	public int m_nLastPageToDisplay;
	public int m_nCurrentPage;
	public int m_nLastPage;

	/**
	 * 
	 * @param _asOwner
	 * @param _model
	 * @param _URLHelper
	 * @param _nPagesToDisplay
	 */
	public SearchPaginationHelper(AActivitySpace _asOwner, ISearchResultModelRO _model,
		ISearchURLHelper _URLHelper, int _nPagesToDisplay)
	{
		m_asOwner = _asOwner;
		m_model = _model;
		m_URLHelper = _URLHelper;

		// Calculate various shared pieces of pagination data that are
		// used in both the main data object and the pages list.
		int totResults = _model.GetTotalItems();
		int pageSize   = _model.GetItemsPerPage();

		m_nLastPage = totResults/pageSize;
		// If we spill over into the next page, add 1
		if ((totResults % pageSize > 0) && (totResults > pageSize)) {
			m_nLastPage += 1;
		}

		int currFirst  = _model.GetFirstDisplayableItem();
		m_nCurrentPage = (currFirst - 1) / pageSize;

		m_nFirstPageToDisplay = m_nCurrentPage - _nPagesToDisplay;
		if (m_nFirstPageToDisplay < 0) {
			m_nFirstPageToDisplay = 0;
		}

		m_nLastPageToDisplay = m_nCurrentPage + _nPagesToDisplay;
		if (m_nLastPageToDisplay > m_nLastPage) {
			m_nLastPageToDisplay = m_nLastPage;
		}
	}

	/**
	 * This method creates the main data object and adds the pagination data to it.
	 * 
	 * @return
	 */
	public IDataObject createMainData()
	{
		HashtableDataObject htObject = new HashtableDataObject();

		String sortOrder = m_model.GetSortOrder();

		int currFirst  = m_model.GetFirstDisplayableItem();
		int totResults = m_model.GetTotalItems();
		int pageSize   = m_model.GetItemsPerPage();
		
		htObject.SetData(PaginationDataConstants.TOTAL_RESULTS, XPConvert.ToString(totResults));
		htObject.SetData(PaginationDataConstants.NUM_PER_PAGE, XPConvert.ToString(pageSize));
		// If there are no search results then we display start count as 0
		if (totResults == 0 ) {
			htObject.SetData(PaginationDataConstants.START_COUNT, XPConvert.ToString(0));
		}
		else
		{
			htObject.SetData(PaginationDataConstants.START_COUNT, XPConvert.ToString(currFirst));
		}
		

		// If there is only one page
		if (totResults <= pageSize)
		{
			htObject.SetData(PaginationDataConstants.END_COUNT, XPConvert.ToString(totResults));
		}
		else
		{
			// This is to display # of results on the last page
			int endCount = 0;
			// If total results are > calculated results then it means 
			// there are still more pages left and we end this page with
			// the calculated results.
			if ( totResults > currFirst + pageSize - 1) {
				endCount = currFirst + pageSize - 1;
			// There are no more pages left to display and this is the 
			// last page.
			} else {
				endCount = totResults;
			}
			
			htObject.SetData(PaginationDataConstants.END_COUNT, XPConvert.ToString(endCount));
		}

		// If we're showing multiple groups, then we're done.
		if(m_model.GetNumResultGroups() > 1)
		{
			return htObject;
		}
		
		// First, check whether we can display a "previous" link.
		int prevFirst = 0;

		if (currFirst > pageSize)
		{
			// we're at least one page beyond the beginning;
			// we can back up one page
			prevFirst = currFirst - pageSize;
		}
		else if (currFirst > 1)
		{
			// odd case: we're partway thru the first page.
			// how did we get here?
			// display a link to go back to the first result
			prevFirst = 1;
		}
		if (0 < prevFirst)
		{
			String url = m_URLHelper.makeURL(m_asOwner, m_model, sortOrder, prevFirst, pageSize);
			htObject.SetData(PaginationDataConstants.PREVIOUS_URL, url);
		}

		htObject.SetData(PaginationDataConstants.LAST_PAGE, XPConvert.ToString(m_nLastPage));
		
		// DCA - display first page link 
		if (m_nFirstPageToDisplay > 0) {
			htObject.SetData(PaginationDataConstants.FIRST_PAGE_URL, m_URLHelper.makeURL(m_asOwner, m_model, sortOrder, 1, pageSize));
		}
		
		if (m_nLastPageToDisplay < (m_nLastPage-1)) {
			htObject.SetData(PaginationDataConstants.LAST_PAGE_URL, m_URLHelper.makeURL(m_asOwner, m_model, sortOrder, ((m_nLastPage-1) * pageSize) + 1, pageSize));
		}

		// Finally, handle the "next" link
		// Calculate start of next page
		int nextFirst = currFirst + pageSize;

		// Check that "next" won't go beyond the end of
		// the result set
		if (nextFirst > totResults)
		{
			nextFirst = 0;
		}

		// if displaying "next" link
		if (0 != nextFirst)
		{
			int numOnNextPage = totResults - nextFirst + 1;
			if (numOnNextPage > pageSize)
			{
				numOnNextPage = pageSize;
			}
			htObject.SetData(PaginationDataConstants.NEXT_URL, m_URLHelper.makeURL(m_asOwner, m_model, sortOrder, nextFirst, pageSize));
		}

		return htObject;
	}

	/**
	 * Create the list of pages.
	 * 
	 * @return
	 */
	public IXPList createPageList()
	{
		String sortOrder = m_model.GetSortOrder();
		int totalResults = m_model.GetTotalItems();
		int pageSize   = m_model.GetItemsPerPage();

		IXPList listPages = new XPArrayList();

	    // Output the data for the display page links
		HashtableDataObject htPageData;
		for (int page = m_nFirstPageToDisplay; page <= m_nLastPageToDisplay; page++)
		{
			if (page < 0)
			{
				continue;
			}

			int firstResultInPage = (page * pageSize) + 1;
			int lastResultInPage  = (firstResultInPage + pageSize) - 1;

			// If we're over the total number of results
			if (firstResultInPage > totalResults)
			{
				if (page == 0) {
					htPageData = new HashtableDataObject();
					htPageData.SetData(PaginationDataConstants.PAGE_NUMBER, XPConvert.ToString(page+1));					
					htPageData.SetData(PaginationDataConstants.START_COUNT, "1");															
					htPageData.SetData(PaginationDataConstants.END_COUNT, "1");
					listPages.Add(htPageData);
				}
				// we're all done
				break;
			}
			
			// If this is the last page
			if (lastResultInPage > totalResults)
			{
				// show the exact number of results we'll be displaying
				lastResultInPage = totalResults;
			}
						

			htPageData = new HashtableDataObject();
			htPageData.SetData(PaginationDataConstants.PAGE_NUMBER, XPConvert.ToString(page+1));
			htPageData.SetData(PaginationDataConstants.START_COUNT, XPConvert.ToString(firstResultInPage));
			htPageData.SetData(PaginationDataConstants.END_COUNT, XPConvert.ToString(lastResultInPage));

			// If this isn't the current page
			if (page != m_nCurrentPage)
			{
				htPageData.SetData(PaginationDataConstants.PAGINATION_URL, m_URLHelper.makeURL(m_asOwner, m_model, sortOrder, firstResultInPage, pageSize));
			}
			listPages.Add(htPageData);
		}

		return listPages;
	}
}