package com.plumtree.portaluiinfrastructure.tags.helper;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.data.*;
import com.plumtree.portaluiinfrastructure.tags.displaytree.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.uitasks.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This helper class contains the framework code that tags use
 * and helps to keep the ATag class simple.
 */
public class TagHelper
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADAPTIVETAGS),
		"com.plumtree.portaluiinfrastructure.tags.helper.TagHelper");

	/**
	 * The marker for the beginning of a $var or $data.var attribute value replacement.
	 */
	public static final char ATTRIBUTE_VARIABLE_MARKER = '$';

	/**
	 * The separator between data collection name and value key in $data.var attribute
	 * value replacement.
	 */
	public static final char ATTRIBUTE_VARIABLE_SEPARATOR = '.';

	/**
	 * The marker used for attribute i18n string replacement ($#n.file).
	 */
	public static final char ATTRIBUTE_I18N_MARKER = '#';

	/**
	 * This key is used to store the unique ID counter in the
	 * HTTP request data.
	 */
	public static final String UNIQUE_ID_COUNTER_KEY = "uniqueidcounter";

	/**
	 * This key is used to store the unique ID for each tag
	 * in Tag Scope.
	 */
	public static final String UNIQUE_ID_KEY = "uniqueid";

	protected TagVisitor m_tagVisitor = null;

	protected Object[][] m_attributeObjXML;
	protected boolean m_bPTAttributesProcessed;
	protected String m_strXMLAttributeDisplaystring;
	protected boolean m_bXMLAttributesProcessed;

	// NOTE: On an empty mypage using layout mode, creation of XPHashtable
	// objects associated with TagHelper dropped from 1500 PT and 1500 XML
	// to 1000 PT and 200 XML.

	// NOTE: These hashtables should never be accessed
	// directly, as they are lazy initialized and may
	// not exist.  Use the GetProcessedPTAttributes()
	// and GetProcessedXMLAttributes() methods instead.
	protected XPHashtable m_processedPTAttributes;
	protected XPHashtable m_processedXMLAttributes;

	protected IDisplayElement m_tagBody;
	protected ATag m_tag;

	protected String m_strUniqueID;

	/**
	 * Constructor for the TagHelper.  This object
	 * should only be created by the tag framework.
	 * 
	 * @param _attributeObjXML The XML data that
	 * describes this tag and its attributes.
	 * @param _tagBody The child display element of
	 * this tag in the display tree.  May be null.
	 * @param _Tag The current tag.
	 */
	public TagHelper(Object[][] _attributeObjXML,
		IDisplayElement _tagBody, ATag _Tag)
	{
		m_attributeObjXML = _attributeObjXML;
		m_bXMLAttributesProcessed = false;
		m_strXMLAttributeDisplaystring = null;
		m_bXMLAttributesProcessed = false;

		m_tagBody = _tagBody;

		m_tag = _Tag;

		m_strUniqueID = null;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#GetEnvironment()
	 */
	public IEnvironment GetEnvironment()
	{
		return m_tagVisitor.GetEnvironment();
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#GetState()
	 */
	public TagState GetState()
	{
		return m_tagVisitor.GetState();
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#ProcessTagBody()
	 */
	public HTMLElement ProcessTagBody()
	{
		if (null != m_tagBody)
		{
			return m_tagBody.Display(m_tagVisitor);
		}

		return null;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#AddJavascript(HTMLScriptCollection)
	 */
	public void AddJavascript(HTMLScriptCollection _script)
	{
		if (null != _script)
		{
			m_tagVisitor.GetJavaScriptCollection().AddInnerHTMLElement(_script);
//			m_JavaScript.AddInnerHTMLElement(_script);
		}
	}

	/**
	 * Check whether or not the current Tag is nested
	 * inside a given parent Tag.  The parent tag must
	 * be in the same library as the current tag.
	 * 
	 * E.G.
	 * <pt:parenttag><pt:thistag/></pt:parenttag>
	 * would return 1 for "parenttag".
	 * 
	 * Tag Utility Method
	 * 
	 * @param ITagMetaData _parentTag The Tag Meta Data
	 * of the requested parent tag
	 * @return int 0 implies the parent tag was not found and a
	 * positive integer implies that the parent tag was found n
	 * levels above the current tag (only counting PT Transformer
	 * Tags, not normal HTML tags).
	 */
	public int HasParentTag(ITagMetaData _parentTag)
	{
		return m_tagVisitor.HasParentTag(_parentTag,
			m_tag.GetTagMetaDataHelper().GetTagLibraryMetaData());
	}

	/**
	 * Check whether or not a particular Transformer
	 * Tag is nested inside the current Tag.  The
	 * child tag must be in the same library as the
	 * current tag.
	 * 
	 * E.G.
	 * <pt:thistag><span><pt:firstchildtag><pt:subsubchildtag/></pt:firstchildtag>
	 * <pt:secondchildtag/><span></pt:thistag>
	 * would return a positive integer for "firstchildtag", "secondchildtag",
	 * and "subsubchildtag", but not "span", since it is
	 * an HTML Tag, not a Transformer Tag.
	 * 
	 * Tag Utility Method
	 * 
	 * @param ITagMetaData _childTag A Tag Meta
	 * Data objects of the requested child tag
	 * @return int 0 implies the child tag was not found and a
	 * positive integer implies that the child tag was found n
	 * levels below the current tag (only counting PT Transformer
	 * Tags, not normal HTML tags).
	 */
	public int HasChildTag(ITagMetaData _childTag)
	{
		Object[] nResults =  new Object[1];

		if (!m_tagVisitor.HasChildTags(new ITagMetaData[] {_childTag}, nResults,
			m_tag.GetTagMetaDataHelper().GetTagLibraryMetaData()))
		{
			// The tag was not found.
			return 0;
		}

		// The depth that the tag was found at.
		return XPConvert.ToInteger(nResults[0]);
	}

	/*********************
	 * Attribute Methods *
	 *********************/

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#GetAttributeAsString(ATagAttribute)
	 */
	public String GetAttributeAsString(ATagAttribute _attribute)
	{
		CheckAttribute(_attribute, AttributeType.STRING);

		return InternalGetAttribute(_attribute);
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#GetAttributeAsInt(ATagAttribute)
	 */
	public int GetAttributeAsInt(ATagAttribute _attribute)
	{
		CheckAttribute(_attribute, AttributeType.INT);

		String strValue = InternalGetAttribute(_attribute);

		int nResult;

		try
		{
			nResult = XPConvert.ToInteger(strValue);
		} catch (Exception e)
		{
			// Try to use the default value, if we didn't already
			// try to use it.
			String strDefault = _attribute.GetDefaultValue();
			if ((null != strDefault) &&
				(XPStringUtility.Compare(strValue, strDefault) != 0))
			{
				// We don't need to put this conversion in a try/catch
				// because we have already check for type validity in
				// the default attribute constructor.
				nResult = XPConvert.ToInteger(strDefault);

				log.Warn(e, "In Tag " + m_tag.GetName() +
					", Attribute " + _attribute.GetName() +
					" with value " + strValue + 
					" was unable to be converted to type " +
					AttributeType.INT + ".  Using default value " +
					strDefault + ".");

				return nResult;
			}

			// We should never get here because all required attributes,
			// which don't have default values and could therefore get
			// here, should have had their values checked before the tag
			// is executed.  Optional attributes, which could easily
			// enter this try/catch block due to invalid user input,
			// should have pre-tested default values.
			throw new XPFormatException("In Tag " + m_tag.GetName() +
				", Attribute " + _attribute.GetName() +
				" with value " + strValue +
				" and default value " + strDefault + 
				" was unable to be converted to type " +
				AttributeType.INT + ".", e);
		}

		return nResult;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#GetAttributeAsBoolean(ATagAttribute)
	 */
	public boolean GetAttributeAsBoolean(ATagAttribute _attribute)
	{
		CheckAttribute(_attribute, AttributeType.BOOLEAN);

		String strValue = InternalGetAttribute(_attribute);

		boolean bResult;

		try
		{
			bResult = XPConvert.ToBoolean(strValue);
		} catch (Exception e)
		{
			// Try to use the default value, if we didn't already
			// try to use it.
			String strDefault = _attribute.GetDefaultValue();
			if ((null != strDefault) &&
				(XPStringUtility.Compare(strValue, strDefault) != 0))
			{
				// We don't need to put this conversion in a try/catch
				// because we have already check for type validity in
				// the default attribute constructor.
				bResult = XPConvert.ToBoolean(strDefault);

				log.Warn(e, "In Tag " + m_tag.GetName() +
					", Attribute " + _attribute.GetName() +
					" with value " + strValue + 
					" was unable to be converted to type " +
					AttributeType.BOOLEAN + ".  Using default value " +
					strDefault + ".");

				return bResult;
			}

			// We should never get here because all required attributes,
			// which don't have default values and could therefore get
			// here, should have had their values checked before the tag
			// is executed.  Optional attributes, which could easily
			// enter this try/catch block due to invalid user input,
			// should have pre-tested default values.
			throw new XPFormatException("In Tag " + m_tag.GetName() +
				", Attribute " + _attribute.GetName() +
				" with value " + strValue +
				" and default value " + strDefault + 
				" was unable to be converted to type " +
				AttributeType.BOOLEAN + ".", e);
		}

		return bResult;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#GetAttributeAsChar(ATagAttribute)
	 */
	public char GetAttributeAsChar(ATagAttribute _attribute)
	{
		CheckAttribute(_attribute, AttributeType.CHAR);

		String strValue = InternalGetAttribute(_attribute);

		char cResult;

		try
		{
			if (1 == strValue.length())
			{
				cResult = strValue.charAt(0);
			} else
			{
				throw new XPFormatException(
					"String not of length 1, so it could not be converted to a char: " +
					strValue);
			}
		} catch (Exception e)
		{
			// Try to use the default value, if we didn't already
			// try to use it.
			String strDefault = _attribute.GetDefaultValue();
			if ((null != strDefault) &&
				(XPStringUtility.Compare(strValue, strDefault) != 0))
			{
				// We don't need to put this conversion in a try/catch
				// because we have already check for type validity in
				// the default attribute constructor.
				cResult = strDefault.charAt(0);

				log.Warn(e, "In Tag " + m_tag.GetName() +
					", Attribute " + _attribute.GetName() +
					" with value " + strValue + 
					" was unable to be converted to type " +
					AttributeType.CHAR + ".  Using default value " +
					strDefault + ".");

				return cResult;
			}

			// We should never get here because all required attributes,
			// which don't have default values and could therefore get
			// here, should have had their values checked before the tag
			// is executed.  Optional attributes, which could easily
			// enter this try/catch block due to invalid user input,
			// should have pre-tested default values.
			throw new XPFormatException("In Tag " + m_tag.GetName() +
				", Attribute " + _attribute.GetName() +
				" with value " + strValue +
				" and default value " + strDefault + 
				" was unable to be converted to type " +
				AttributeType.CHAR + ".", e);
		}

		return cResult;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#GetAttributeAsDouble(ATagAttribute)
	 */
	public double GetAttributeAsDouble(ATagAttribute _attribute)
	{
		CheckAttribute(_attribute, AttributeType.DOUBLE);

		String strValue = InternalGetAttribute(_attribute);

		double dResult;

		try
		{
			dResult = XPConvert.ToDouble(strValue);
		} catch (Exception e)
		{
			// Try to use the default value, if we didn't already
			// try to use it.
			String strDefault = _attribute.GetDefaultValue();
			if ((null != strDefault) &&
				(XPStringUtility.Compare(strValue, strDefault) != 0))
			{
				// We don't need to put this conversion in a try/catch
				// because we have already check for type validity in
				// the default attribute constructor.
				dResult = XPConvert.ToDouble(strDefault);

				log.Warn(e, "In Tag " + m_tag.GetName() +
					", Attribute " + _attribute.GetName() +
					" with value " + strValue + 
					" was unable to be converted to type " +
					AttributeType.DOUBLE + ".  Using default value " +
					strDefault + ".");

				return dResult;
			}

			// We should never get here because all required attributes,
			// which don't have default values and could therefore get
			// here, should have had their values checked before the tag
			// is executed.  Optional attributes, which could easily
			// enter this try/catch block due to invalid user input,
			// should have pre-tested default values.
			throw new XPFormatException("In Tag " + m_tag.GetName() +
				", Attribute " + _attribute.GetName() +
				" with value " + strValue +
				" and default value " + strDefault + 
				" was unable to be converted to type " +
				AttributeType.DOUBLE + ".", e);
		}

		return dResult;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#GetAttributeAsLong(ATagAttribute)
	 */
	public long GetAttributeAsLong(ATagAttribute _attribute)
	{
		CheckAttribute(_attribute, AttributeType.LONG);

		String strValue = InternalGetAttribute(_attribute);

		long lResult;

		try
		{
			lResult = XPConvert.ToLong(strValue);
		} catch (Exception e)
		{
			// Try to use the default value, if we didn't already
			// try to use it.
			String strDefault = _attribute.GetDefaultValue();
			if ((null != strDefault) &&
				(XPStringUtility.Compare(strValue, strDefault) != 0))
			{
				// We don't need to put this conversion in a try/catch
				// because we have already check for type validity in
				// the default attribute constructor.
				lResult = XPConvert.ToLong(strDefault);

				log.Warn(e, "In Tag " + m_tag.GetName() +
					", Attribute " + _attribute.GetName() +
					" with value " + strValue + 
					" was unable to be converted to type " +
					AttributeType.LONG + ".  Using default value " +
					strDefault + ".");

				return lResult;
			}

			// We should never get here because all required attributes,
			// which don't have default values and could therefore get
			// here, should have had their values checked before the tag
			// is executed.  Optional attributes, which could easily
			// enter this try/catch block due to invalid user input,
			// should have pre-tested default values.
			throw new XPFormatException("In Tag " + m_tag.GetName() +
				", Attribute " + _attribute.GetName() +
				" with value " + strValue +
				" and default value " + strDefault + 
				" was unable to be converted to type " +
				AttributeType.LONG + ".", e);
		}

		return lResult;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#HasAttribute(ATagAttribute)
	 */
	public boolean HasAttribute(ATagAttribute _attribute)
	{
		CheckAttribute(_attribute, null);

		if (_attribute.GetIsRequired())
		{
			// The Tag Container guarantees that required tags
			// are always present.
			return true;
		}

		String strAttribute = InternalGetAttributeFromName(_attribute.GetName(), 
			PT_XML_TYPES.PT_XML_TYPES_PTUI_ATT);

		return (null != strAttribute);
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#GetTagAttribute(String)
	 */
	public String GetAttribute(String _strAttribute)
	{
		return InternalGetAttributeFromName(_strAttribute,
			PT_XML_TYPES.PT_XML_TYPES_PTUI_ATT);
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#GetAttributeNames()
	 */
	public IXPEnumerator GetAttributeNames()
	{
		ProcessAllAttributes();

		return GetProcessedPTAttributes().GetKeys();
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#GetXMLAttribute(String)
	 */
	public String GetXMLAttribute(String _strAttribute)
	{
		return InternalGetAttributeFromName(_strAttribute,
			PT_XML_TYPES.PT_XML_TYPES_XHTML_ATT);
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#HasXMLAttribute(String)
	 */
	public boolean HasXMLAttribute(String _strAttribute)
	{
		String strAttribute = InternalGetAttributeFromName(_strAttribute,
			PT_XML_TYPES.PT_XML_TYPES_XHTML_ATT);

		return (null != strAttribute);
	}

	/**
	 * This helper method gets an attribute as a string.
	 * 
	 * @param _attribute The requested Attribute.
	 * @return The attribute as a string.
	 */
	protected String InternalGetAttribute(ATagAttribute _attribute)
	{
		String strAttribute = InternalGetAttributeFromName(_attribute.GetName(), 
			PT_XML_TYPES.PT_XML_TYPES_PTUI_ATT);

		if ((null == strAttribute) &&
			(null != _attribute.GetDefaultValue()))
		{
			return _attribute.GetDefaultValue();
		}

		return strAttribute;
	}

	/**
	 * This helper method gets an attribute from a String object rather than an attribute
	 * object.
	 * 
	 * @param _strAttribute  The name of the attribute to retrieve.
	 * @param _nXMLAttributeType The attribute type.
	 * @return The value of the attribute.
	 */
	protected String InternalGetAttributeFromName(String _strAttribute, int _nXMLAttributeType)
	{
		// True means ignore processing errors so that the attribute is cached and
		// we don't attempt to process it every time we get it and get an error
		// each time.
		return InternalGetAttributeFromName(_strAttribute, _nXMLAttributeType, true);
	}

	/**
	 * This helper method gets an attribute from a String object rather than an attribute
	 * object.  It also allows you to set the error handling behavior of this method.
	 * 
	 * @param _strAttribute  The name of the attribute to retrieve.
	 * @param _nXMLAttributeType The attribute type.
	 * @param _bIgnoreProcessingErrors False implies that exceptions should be generated
	 * when processing attributes for value replacement.  This is not the default behavior.
	 * @return The value of the attribute.
	 * @throws XPIllegalArgumentException if the attribute specifies an invalid variable
	 * for variable replacement and _bIgnoreProcessingErrors is false.
	 */
	protected String InternalGetAttributeFromName(String _strAttribute, int _nXMLAttributeType,
		boolean _bIgnoreProcessingErrors)
	{
		Object oAttribute;

		// Check if we have already processed the attribute.
		if (PT_XML_TYPES.PT_XML_TYPES_PTUI_ATT == _nXMLAttributeType)
		{
			oAttribute = GetProcessedPTAttributes().GetElement(_strAttribute);
		} else if (PT_XML_TYPES.PT_XML_TYPES_XHTML_ATT == _nXMLAttributeType)
		{
			oAttribute = GetProcessedXMLAttributes().GetElement(_strAttribute);
		} else
		{
			log.Error("Attempting to get attribute " + _strAttribute +
				" of illegal type " + _nXMLAttributeType);
			return null;
		}

		if (null != oAttribute)
		{
			return (String) oAttribute;
		}

		if (m_attributeObjXML.length == 0)
		{
			return null;
		}

		// Look up the args -jas
		for (int iAtt = 0; iAtt < m_attributeObjXML[0].length; iAtt++)
		{
			// check the type -jas
			int nXMLType = XPConvert.ToInteger(m_attributeObjXML[PT_XML_ARRAY_COLS.PT_XML_ARRAY_COLS_TYPE][iAtt]);

			if (_nXMLAttributeType == nXMLType)
			{
				// check if the name matches -jas
				String strAttributeName = (String) m_attributeObjXML[PT_XML_ARRAY_COLS.PT_XML_ARRAY_COLS_NAME][iAtt];

				if (0 == strAttributeName.compareTo(_strAttribute))
				{
					String strAttribute;
					try
					{
						strAttribute = (String) m_attributeObjXML[PT_XML_ARRAY_COLS.PT_XML_ARRAY_COLS_VALUE][iAtt];
					} catch (Exception e)
					{
						log.Error(e, "In Tag " + m_tag.GetName() +
							", Attribute " + strAttributeName +
							" was unable to be converted to a String.");

						strAttribute = null;
					}

					String strAttributeToCache;

					try
					{
						// Process the attribute for variable replacement.
						strAttribute = ProcessAttribute(strAttribute);

						strAttributeToCache = strAttribute;
					} catch (XPIllegalArgumentException xpiae)
					{
						if (_bIgnoreProcessingErrors)
						{
							log.Error(xpiae, "Ignoring failed attribute processing for Tag " +
								m_tag.GetName() + " and Attribute " + strAttributeName + ".");

							// Cache the attribute as the empty string so it won't get processed
							// again, but return null so we can use default values appropriately.
							strAttributeToCache = "";
							strAttribute = null;
						} else
						{
							throw xpiae;
						}
					}

					// Cache the processed result
					if (PT_XML_TYPES.PT_XML_TYPES_PTUI_ATT == _nXMLAttributeType)
					{
						GetProcessedPTAttributes().PutElement(_strAttribute, strAttributeToCache);
					} else if (PT_XML_TYPES.PT_XML_TYPES_XHTML_ATT == _nXMLAttributeType)
					{
						GetProcessedXMLAttributes().PutElement(_strAttribute, strAttributeToCache);
					}

					return strAttribute;
				}
			}
		}

		return null;
	}

	/**
	 * Check the type safety of the requested attribute.
	 * 
	 * @param ATagAttribute _attribute The requested attribute. 
	 * @param AttributeType _type The requested data type.
	 * @throws XPIllegalArgumentException if the requested data
	 * type does not match the attribute.
	 */
	protected void CheckAttribute(ATagAttribute _attribute, AttributeType _type)
	{
		if (null == _attribute)
		{
			throw new XPIllegalArgumentException ("In Tag " + m_tag.GetName() +
				", tried to access attribute with null ATagAttribute argument.");
		}

		if ((null != _type) &&
			(_type !=_attribute.GetType()))
		{
			throw new XPFormatException ("In Tag " + m_tag.GetName() +
				", Attribute " + _attribute.GetName() +
				" is of type " + _attribute.GetType() +
				" but was requested as type " + _type + ".");
		}
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#GetXMLAttributeNames()
	 */
	public IXPEnumerator GetXMLAttributeNames()
	{
		ProcessAllXMLAttributes();

		return GetProcessedXMLAttributes().GetKeys();
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#GetXMLTagAttributesAsString()
	 */
	public String GetXMLTagAttributesAsString()
	{
		ProcessAllXMLAttributes();

		// If there are no attributes, then we can return here without
		// having created the underlying hashtable, increasing our
		// performance/memory usage.
		if (IsProcessedXMLAttributesEmpty())
		{
			return "";
		}

		// This will create the hashtable if it hasn't already been created.
		XPHashtable processedXMLAttributes = GetProcessedXMLAttributes();

		// TODO: Evaluate this guess in terms of the expected size of XML attributes;
		XPStringBuilder result = new XPStringBuilder(processedXMLAttributes.GetSize() * 10);

		IXPEnumerator keys = processedXMLAttributes.GetKeys();

		while (keys.MoveNext())
		{
			Object oKey = keys.GetCurrent();
			result.Append(oKey);
			result.Append("=\"");
			result.Append(processedXMLAttributes.GetElement(oKey));
			result.Append("\" ");
		}

		if (result.GetLength() > 0)
		{
			// Remove the trailing space
			result.RemoveCharAt(result.GetLength() - 1);
		}

		return result.ToString();
	}

	/**
	 * This helper method process all the PT attributes
	 * and stores them in the hashtable.  This is only
	 * used if we are looping through the PT attributes
	 * instead of accessing them using their associated
	 * ATagAttribute objects.
	 */
	protected void ProcessAllAttributes()
	{
		if (m_bPTAttributesProcessed)
		{
			return;
		}

		m_bPTAttributesProcessed = true;

		// Look up the args -jas
		for (int iAtt = 0; iAtt < m_attributeObjXML[0].length; iAtt++)
		{
			// check the type -jas
			int nXMLType = XPConvert.ToInteger(m_attributeObjXML[PT_XML_ARRAY_COLS.PT_XML_ARRAY_COLS_TYPE][iAtt]);

			if (PT_XML_TYPES.PT_XML_TYPES_PTUI_ATT == nXMLType)
			{
				Object oAttributeName = m_attributeObjXML[PT_XML_ARRAY_COLS.PT_XML_ARRAY_COLS_NAME][iAtt];

				// If we have not already processed this attribute
				if (!GetProcessedPTAttributes().ContainsKey(oAttributeName))
				{
					String strAttribute;
					try
					{
						strAttribute = (String) m_attributeObjXML[PT_XML_ARRAY_COLS.PT_XML_ARRAY_COLS_VALUE][iAtt];

						try
						{
							// Process the attribute for variable replacement.
							strAttribute = ProcessAttribute(strAttribute);
						} catch (XPIllegalArgumentException xpiae)
						{
							log.Error(xpiae, "Ignoring failed attribute processing for Tag " +
								m_tag.GetName() + " and Attribute " + oAttributeName + ".");

							// Cache the attribute as the empty string so it won't get processed
							// again.
							strAttribute = "";
						}

						// Cache the processed result
						GetProcessedPTAttributes().PutElement(oAttributeName, strAttribute);
					} catch (Exception e)
					{
						log.Error(e, "In Tag " + m_tag.GetName() +
							", PT Attribute " + oAttributeName +
							" was unable to be converted to a String and processed for attribute value replacement.");
					}
				}
			}
		}
	}

	/**
	 * This helper method process all the XML attributes
	 * and stores them in the hashtable.
	 */
	protected void ProcessAllXMLAttributes()
	{
		if (m_bXMLAttributesProcessed)
		{
			return;
		}

		m_bXMLAttributesProcessed = true;

		// Look up the args -jas
		for (int iAtt = 0; iAtt < m_attributeObjXML[0].length; iAtt++)
		{
			// check the type -jas
			int nXMLType = XPConvert.ToInteger(m_attributeObjXML[PT_XML_ARRAY_COLS.PT_XML_ARRAY_COLS_TYPE][iAtt]);

			if (PT_XML_TYPES.PT_XML_TYPES_XHTML_ATT == nXMLType)
			{
				Object oAttributeName = m_attributeObjXML[PT_XML_ARRAY_COLS.PT_XML_ARRAY_COLS_NAME][iAtt];

				// If we have not already processed this attribute
				if (!GetProcessedXMLAttributes().ContainsKey(oAttributeName))
				{
					String strAttribute;
					try
					{
						strAttribute = (String) m_attributeObjXML[PT_XML_ARRAY_COLS.PT_XML_ARRAY_COLS_VALUE][iAtt];

						try
						{
							// Process the attribute for variable replacement.
							strAttribute = ProcessAttribute(strAttribute);
						} catch (XPIllegalArgumentException xpiae)
						{
							log.Error(xpiae, "Ignoring failed attribute processing for Tag " +
								m_tag.GetName() + " and Attribute " + oAttributeName + ".");

							// Cache the attribute as the empty string so it won't get processed
							// again.
							strAttribute = "";
						}
	
						// Cache the processed result
						GetProcessedXMLAttributes().PutElement(oAttributeName, strAttribute);
					} catch (Exception e)
					{
						log.Error(e, "In Tag " + m_tag.GetName() +
							", XML Attribute " + oAttributeName +
							" was unable to be converted to a String and processed for attribute value replacement.");
					}
				}
			}
		}
	}

	/**
	 * Performs attribute value replacement on the attribute.
	 * 
	 * I.E.  Replaces $var with the value of the tag variable
	 * var and $data.var with the value of the key var in the
	 * tag variable data collection data.  Replaces $#n.file
	 * with the internationalized string with ID n in file file.
	 * 
	 * @param _strAttribute The pre-processed attribute.
	 * @return String The processed attribute.  Empty string if
	 * there was an error.
	 */
	protected String ProcessAttribute(String _strAttribute)
	{
		if ((null == _strAttribute) || (_strAttribute.length() < 2))
		{
			// This is an empty string or is not long enough to need
			// processing (must be at least 2 characters long - i.e. $a).
			return _strAttribute;
		}

		if (ATTRIBUTE_VARIABLE_MARKER != _strAttribute.charAt(0))
		{
			// This is not an attribute value replacement string
			// because it does not start with $.
			return _strAttribute;
		}

		if (ATTRIBUTE_VARIABLE_MARKER == _strAttribute.charAt(1))
		{
			// This is not an attribute value replacement string
			// because the $ is escaped as $$.
			return _strAttribute.substring(1);
		}

		if (ATTRIBUTE_I18N_MARKER == _strAttribute.charAt(1))
		{
			// This is an i18n string of the form ($#n.file).
			return ProcessI18NStringAttribute(_strAttribute);
		}

		String strProcessedAttribute;

		int nSeparatorIndex = XPStringUtility.IndexOf(_strAttribute, ATTRIBUTE_VARIABLE_SEPARATOR);

		if (nSeparatorIndex > 0)
		{
			// This is of the form $data.var, so get from after the first character to the separator.
			String strDataCollection = XPStringUtility.Substring(_strAttribute, 1, nSeparatorIndex);

			// This is of the form $data.var, so get after the separator.
			String strKey = XPStringUtility.Substring(_strAttribute,
				nSeparatorIndex + 1, _strAttribute.length());

			strProcessedAttribute = GetCollectionVariable(strDataCollection, strKey);
		} else
		{
			// This is of the form $var, so skip the first character.
			String strKey = XPStringUtility.Substring(_strAttribute, 1, _strAttribute.length());

			strProcessedAttribute = GetVariable(strKey);
		}

		if (null == strProcessedAttribute)
		{
			throw new XPIllegalArgumentException(
				"Unable to perform attribute value replacement on attribute " +
				_strAttribute + ".");
		}

		return strProcessedAttribute;
	}

	/**
	 * Performs attribute value replacement on the attribute.
	 * 
	 * I.E.  Replaces $var with the value of the tag variable
	 * var and $data.var with the value of the key var in the
	 * tag variable data collection data.  Replaces $#n.file
	 * with the internationalized string with ID n in file file.
	 * 
	 * @param _strAttribute The pre-processed attribute.
	 * @return String The processed attribute.  Empty string if
	 * there was an error.
	 */
	protected String ProcessI18NStringAttribute(String _strAttribute)
	{
		// This is an i18n string of the form $#n.file

		int nSeparatorIndex = XPStringUtility.IndexOf(_strAttribute, ATTRIBUTE_VARIABLE_SEPARATOR);
	
		if (nSeparatorIndex > 0)
		{
			// This is of the form $#n.file, so get from after the second character to the separator.
			String strID = XPStringUtility.Substring(_strAttribute, 2, nSeparatorIndex);

			int nID;

			try
			{
				nID = XPConvert.ToInteger(strID);
			} catch (Exception e)
			{
				throw new XPIllegalArgumentException(
					"Unable to perform attribute i18n string replacement on attribute " +
					_strAttribute + ".", e);
			}

			// This is of the form $#n.file, so get after the separator.
			String strFileName = XPStringUtility.Substring(_strAttribute,
				nSeparatorIndex + 1, _strAttribute.length());
	
			return this.m_tagVisitor.GetEnvironment().GetString(nID, strFileName);
		} else
		{
			throw new XPIllegalArgumentException(
				"Unable to perform attribute i18n string replacement on attribute " +
				_strAttribute + ".");
		}
	}

	/**
	 * Looks up the variable in memory and converts the value to a string.
	 * 
	 * @param String _strKey The name of the variable.
	 * @return String The value stored in the variable.  Null if there was
	 * an error.
	 */
	protected String GetVariable(String _strKey)
	{
		Object oResult = GetVariableObject(_strKey);

		if (null == oResult)
		{
			// Variable could not be found.
			log.Error("Unable to find variable " + _strKey +
				" for use in attribute value replacement in tag " +
				m_tag.GetName());

			return null;
		}

		try
		{
			return XPConvert.ToString(oResult);
		} catch (Exception e)
		{
			log.Error(e, "Unable to perform attribute value replacement on variable " +
				_strKey + " with value " + oResult +
				" to a String using XPConvert in tag " + m_tag.GetName() + ".");

			return null;
		}
	}

	/**
	 * Looks up the data collection in memory, accesses the specified key, and
	 * converts the value to a string.
	 * 
	 * @param String _strDataCollection The name of the data collection.
	 * @param String _strKey The key into the data collection.
	 * @return String The value stored in the data collection under the specified
	 * key.  Null if there was an error.
	 */
	protected String GetCollectionVariable(String _strDataCollection, String _strKey)
	{
		Object oCollection = GetVariableObject(_strDataCollection);

		if (null == oCollection)
		{
			log.Error("Unable to find data collection " +
				_strDataCollection + " for use in attribute value replacement in tag " +
				m_tag.GetName());

			return null;
		}

		IDataObject dataCollection;

		try
		{
			 dataCollection = (IDataObject) oCollection;
		} catch (Exception e)
		{
			log.Error(e, "Unable to convert data collection " +
				_strDataCollection +
				" to an IDataObject for use in attribute value replacement in tag " +
				m_tag.GetName());

			return null;
		}

		return dataCollection.GetData(_strKey);
	}

	/**
	 * Looks up the variable in memory and converts the value to a string.
	 * 
	 * @param String _strKey The name of the variable.
	 * @return Object The value stored in the variable.  Null if it could
	 * not be found.
	 */
	protected Object GetVariableObject(String _strKey)
	{
		ITagState state = GetState();

		Object oResult = null;

		Scope[] scopeSearchOrder = Scope.GetScopeSearchOrder();

		// Check variables
		for (int i = 0; i < scopeSearchOrder.length; i++)
		{
			oResult = state.GetVariable(_strKey, scopeSearchOrder[i]);

			if (null != oResult)
			{
				return oResult;
			}
		}

		// Check shared variables
		for (int i = 0; i < scopeSearchOrder.length; i++)
		{
			oResult = state.GetSharedVariable(_strKey, scopeSearchOrder[i]);

			if (null != oResult)
			{
				return oResult;
			}
		}

		return null;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#GetUniqueID()
	 */
	public String GetUniqueID()
	{
		// See if the ID has already been generated in this tag.
		if (null != m_strUniqueID)
		{
			return m_strUniqueID;
		}

		Object oUniqueIDCounter = GetState().GetVariable(UNIQUE_ID_COUNTER_KEY, Scope.HTTP_REQUEST);

		if (null == oUniqueIDCounter)
		{
			GetState().SetVariable(UNIQUE_ID_COUNTER_KEY, new Integer(1), Scope.HTTP_REQUEST);

			return GetUniqueID();
		}

		Integer nUniqueIDCounter = (Integer) oUniqueIDCounter;

		m_strUniqueID = XPConvert.ToString(nUniqueIDCounter);

		// Increment the ID counter.  This is not threadsafe, but this method
		// should never be called by multiple threads.
		GetState().SetVariable(UNIQUE_ID_COUNTER_KEY, new Integer(nUniqueIDCounter.intValue() + 1), Scope.HTTP_REQUEST);

		return m_strUniqueID;
	}

	/********************************************************
	 * Framework Helper methods                            	*
	 * These methods are used by the framework and should	*
	 * not be used by the Tag developer.					*
	 ********************************************************/

	/**
	 * The Tag Container uses this method to display
	 * this tag.  This method calls the DisplayTag() method.
	 *
	 * The TagVisitor must be set before this method can be
	 * called.
	 *  
	 * Framework Helper Method
	 * 
	 * @return HTMLElement The result of displaying this tag
	 */
	public HTMLElement DisplayTag()
	{
		// Check if this tag supports the current access style.
		if (!m_tag.SupportsAccessStyle(GetEnvironment().GetAccessStyle()))
		{
			return new HTMLComment("Tag " + m_tag.GetName() + " in library " +
				m_tag.GetTagMetaDataHelper().GetTagLibraryMetaData().GetName() +
				" does not support the " + GetEnvironment().GetAccessStyle().ToString() +
				" access style.");
		}

		HTMLElementCollection result = new HTMLElementCollection();

		// Check and set if SharedJS has been called before
		Object oDisplayedTags = GetState().GetSharedVariable(TagState.SHARED_JS_VAR_LIST, Scope.HTTP_REQUEST);
		XPArrayList displayedTags;
		if (null == oDisplayedTags)
		{
			displayedTags = new XPArrayList();
			GetState().SetSharedVariable(TagState.SHARED_JS_VAR_LIST, displayedTags, Scope.HTTP_REQUEST, false);
		} else
		{
			displayedTags = (XPArrayList) oDisplayedTags;
		}

		// TODO: Should this also include the library version?
		String strKey = m_tag.GetTagMetaDataHelper().GetTagLibraryMetaData().GetName() + "." + m_tag.GetName();

		// Only add javascript if we are not in 508 mode.
		if (GetEnvironment().GetAccessStyle() != AccessStyles.SECTION_508)
		{
			if (!displayedTags.Contains(strKey))
			{
				// Set that we have displayed the shared javascript
				// TODO: Do we have to unset this below if the tag threw an exception?
				displayedTags.Add(strKey);
	
				// TODO: What should we do about different version of the
				// same js file being used in the same tag, but from different
				// tag library versions?
				HTMLScriptCollection sharedJavaScript = m_tag.DisplaySharedJavascript();

				if (null != sharedJavaScript)
				{
					m_tagVisitor.GetJavaScriptCollection().AddInnerHTMLElement
						(sharedJavaScript);
				}
			}

			// The main javascript for the tag is added to the shared JavaScript collection automatically.
		}

		// Display the tag
		HTMLElement tag = m_tag.DisplayTag();

		if (null != tag)
		{
			// Add the HTML that the tag displayed
			result.AddInnerHTMLElement(tag);
		}

		return result;
	}

	/**
	 * This method sets the tag visitor so we can process the tag.
	 * This needs to be called before we can validate or process the tag.
	 * The tag visitor needs to be released after the tag has been
	 * validated and processed.
	 * 
	 * @param _tagVisitor The tag visitor.
	 */
	public void SetTagVisitor(TagVisitor _tagVisitor)
	{
		m_tagVisitor = _tagVisitor;
	}

	/**
	 * This method releases the tag visitor to make sure it is not
	 * leaked (retained).  This should be called after the tag has
	 * been validated and processed.
	 */
	public void ReleaseTagVisitor()
	{
		m_tagVisitor = null;
	}

	/**
	 * The Tag Container uses this method to check whether
	 * or not this tag is being displayed in the correct
	 * context.  This method checks for required parent
	 * and child tags, as well as for proper formatting and
	 * presence of required attributes.  If the tag is not
	 * being used in a valid way, the Tag Container will
	 * log an error and not display this tag.
	 * 
	 * Framework Helper Method
	 * 
	 * @return String The error message if this tag is not
	 * being used correctly and should not be displayed.
	 * Null implies the tag is being used correctly and should
	 * be displayed.
	 */
	public String ValidateContext()
	{
		String strError = ValidateRequiredAttributes();

		if (null != strError)
		{
			// Return failure message.
			return strError;
		}

		strError = ValidateRequiredChildTags();

		if (null != strError)
		{
			// Return failure message.
			return strError;
		}

		return ValidateRequiredParentTags();	
	}

	/**
	 * This helper method processes the tag display tree for required
	 * parent tags (at least one must be present).
	 * 
	 * This method is public for testing purposes and should not be called
	 * otherwise.
	 * 
	 * @return String The error message if a required parent tag is
	 * missing.  Null implies that at least one required parent tag was present.
	 */
	public String ValidateRequiredParentTags()
	{
		ITagMetaData[] requiredParentTags = m_tag.GetTagMetaDataHelper().GetRequiredParentTags();

		if (requiredParentTags.length > 0)
		{
			boolean bFoundParent = false;

			for (int i = 0; i < requiredParentTags.length; i++)
			{
				if (m_tagVisitor.HasParentTag(requiredParentTags[i], m_tag.GetTagMetaDataHelper().GetTagLibraryMetaData()) > 0)
				{
					bFoundParent = true;
					break;
				}
			}
			if (!bFoundParent)
			{
				XPStringBuilder sb = new XPStringBuilder(30);

				sb.Append("Tag ").Append(m_tag.GetName());
				sb.Append(" missing required parent tag (");

				// Build up the list of possible parent tags.
				for (int i = 0; i < requiredParentTags.length; i++)
				{
					if (i == (requiredParentTags.length - 1))
					{
						// Last tag
						if (i > 0)
						{
							sb.Append("or ");
							sb.Append(requiredParentTags[i].GetName());
						} else
						{
							// only one tag
							sb.Append(requiredParentTags[i].GetName());
						}
					} else
					{
						// Middle or beginning tag
						sb.Append(requiredParentTags[i].GetName());
						sb.Append(", ");
					}
				}

				sb.Append(").");

				String strError =  sb.ToString();

				log.Error(strError);

				// Fail
				return strError;
			}
		}

		// All parent tags validated correctly.
		return null;
	}

	/**
	 * This helper method processes the tag display tree for required
	 * child tags.
	 * 
	 * This method is public for testing purposes and should not be called
	 * otherwise.
	 * 
	 * @return String The error message if a required child tag is
	 * missing.  Null implies that all required child tags were present.
	 */
	public String ValidateRequiredChildTags()
	{
		ITagMetaData[] requiredChildTags = m_tag.GetTagMetaDataHelper().GetRequiredChildTags();

		if ((null != requiredChildTags) && (requiredChildTags.length > 0))
		{
			Object results[] = new Object[requiredChildTags.length];

			// Initialize results array
			for (int i = 0; i < results.length; i++)
			{
				results[i] = null;
			}

			// Check for child tags
			if (!m_tagVisitor.HasChildTags(requiredChildTags, results, m_tag.GetTagMetaDataHelper().GetTagLibraryMetaData()))
			{
				// Find the missing tag
				for (int i = 0; i < results.length; i++)
				{
					if (null == results[i])
					{
						// Required child tag could not be found.  Fail.
						String strError = "Tag " + m_tag.GetName() +
							" missing required child tag " + requiredChildTags[i].GetName() +
							".";

						log.Error(strError);

						return strError;
					}
				}
			}
		}

		// All child tags validated correctly.
		return null;
	}

	/**
	 * This helper method processes all the required PT attributes,
	 * validates them, and stores them in the hashtable.
	 * 
	 * This method is public for testing purposes and should not be called
	 * otherwise.
	 * 
	 * @param _attributes The array of required attributes for this tag.
	 * @return String The error message if an attribute was invalid.  Null
	 * implies that all attributes were valid.
	 */
	public String ValidateRequiredAttributes()
	{
		RequiredTagAttribute[] attributes = m_tag.GetTagMetaDataHelper().GetRequiredTagAttributes();

		for (int i = 0; i < attributes.length; i++)
		{
			String strValue;

			try
			{			
				// This does attribute value replacement and caches the results.
				// false means we do not want to ignore $var processing errors/
				strValue = InternalGetAttributeFromName(attributes[i].GetName(), 
					PT_XML_TYPES.PT_XML_TYPES_PTUI_ATT, false);
			} catch (XPFormatException xpfe)
			{
				String strError = "Unable to perform attribute value replacement on required attribute  " +
					attributes[i].GetName() + " in tag " +
					m_tag.GetName() + ".";

				log.Error(xpfe, strError);

				return strError + " " + xpfe.GetMessage() + ": " +
					xpfe.GetStackTrace();
			}

			if (null == strValue)
			{
				String strError = "Missing required attribute " +
					attributes[i].GetName() + " in tag " +
					m_tag.GetName() + ".";

				log.Error(strError);

				return strError;
			}

			try
			{
				attributes[i].CheckValue(strValue);
			} catch (Exception e)
			{
				String strError = "Invalid input " + strValue +
					" for required attribute " + attributes[i].GetName() +
					" in tag " + m_tag.GetName() + ".";

				log.Error(e, strError);

				XPException xpe = XPException.GetInstance(e);

				return strError + " " + xpe.GetMessage() + ": " +
					xpe.GetStackTrace();
			}
		}

		// All attributes validated correctly.
		return null;
	}

	// NOTE: On an empty mypage using layout mode, creation of XPHashtable
	// objects associated with TagHelper dropped from 1500 PT and 1500 XML
	// to 1000 PT and 200 XML.

	/**
	 * Helper access method to allow lazy initialization of the processed
	 * PT attribute hashtable.  This method will create the hashtable when
	 * the method is called.
	 * 
	 * @return The processed PT attributes.
	 */
	protected XPHashtable GetProcessedPTAttributes()
	{
		if (m_processedPTAttributes == null)
		{
			m_processedPTAttributes = new XPHashtable();
		}

		return m_processedPTAttributes;
	}

	/**
	 * Helper access method to allow lazy initialization of the processed
	 * XML attribute hashtable.  This method will create the hashtable when
	 * the method is called.
	 * 
	 * @return The processed XML attributes.
	 */
	protected XPHashtable GetProcessedXMLAttributes()
	{
		if (m_processedXMLAttributes == null)
		{
			m_processedXMLAttributes = new XPHashtable();
		}

		return m_processedXMLAttributes;
	}

	/**
	 * Helper method to check if the hashtable is empty.  This method
	 * will not create the hashtable if it does not exist.
	 * 
	 * @return Whether there are any processed XML attributes or not.
	 */
	protected boolean IsProcessedXMLAttributesEmpty()
	{
		if (m_processedXMLAttributes == null)
		{
			return true;
		}

		return m_processedXMLAttributes.IsEmpty();
	}
}