package com.plumtree.portaluiinfrastructure.tags.helper;

import com.plumtree.openfoundation.reflect.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;

/**
 * This helper class uses XPReflect to read in the tag meta data.
 * This class should be called once per tag class and stored as a
 * static on the ATag class.
 * 
 * @author DonH
 */
public class TagMetaDataHelper
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.tags.helper.TagMetaDataHelper");

	protected ITagMetaData m_TagMetaData = null;
	protected TagLibraryMetaData m_TagLibraryMetaData = null;
	protected RequiredTagAttribute[] m_RequiredTagAttributes;
	protected OptionalTagAttribute[] m_OptionalTagAttributes;
	protected ITagMetaData[] m_RequiredParentTags;
	protected ITagMetaData[] m_RequiredChildTags;

	/**
	 * This method uses reflection to discover all of the
	 * meta data about this tag and caches it for quick lookup
	 * in the future.  This method is only called once per
	 * tag class when the class is loaded.
	 * 
	 * Framework Helper Method
	 * 
	 * @return boolean false means that the meta data was
	 * unable to be initialized properly, and this tag
	 * cannot be used.
	 */
	public boolean Init(ATag _Tag, TagLibraryMetaData _TagLibMetaData)
	{
		if (null != m_TagMetaData)
		{
			throw new XPException("Tag meta data can only be initialized once.  Meta data already initialized for tag " +
				_Tag + ".");
		}

		XPType thisTagType = new XPType(_Tag.getClass());

		// The docs specify to use ITagMetaData, but we check for both
		// just to be nice.
		XPType tagMetaDataType = new XPType(TagMetaData.class);
		XPType tagMetaDataInterfaceType = new XPType(ITagMetaData.class);

		XPType requiredTagAttributeType = new XPType(RequiredTagAttribute.class);
		XPArrayList requiredTagAttributes = new XPArrayList();

		XPType optionalTagAttributeType = new XPType(OptionalTagAttribute.class);
		XPArrayList optionalTagAttributes = new XPArrayList();

		XPType requiredParentTagType = new XPType(RequiredParentTag.class);
		XPArrayList requiredParentTags = new XPArrayList();

		XPType requiredChildTagType = new XPType(RequiredChildTag.class);
		XPArrayList requiredChildTags = new XPArrayList();

		// Get the meta data fields for this tag
		XPField[] fieldlist = thisTagType.GetPublicFields();
		for (int i  = 0; i < fieldlist.length; i++)
		{
			XPField field = fieldlist[i];
			if (field.IsStatic())
			{
				if (field.GetFieldType().Equals(requiredTagAttributeType))
				{
					// public static RequiredTagAttribute
					requiredTagAttributes.Add(field.GetValue(null));
				} else if (field.GetFieldType().Equals(optionalTagAttributeType))
				{
					// public static OptionalTagAttribute
					optionalTagAttributes.Add(field.GetValue(null));
				} else if ((field.GetFieldType().Equals(tagMetaDataInterfaceType)) ||
					(field.GetFieldType().Equals(tagMetaDataType)))
				{
					if (null != m_TagMetaData)
					{
						log.Error("Tags can only have one public static final ITagMetaData member variable.  " +
							"This can be caused by extending a Tag and trying to override its ITagMetaData.  Tag " +
							_Tag + " in library " + _TagLibMetaData.GetName() + ".");

						return false;
					}

					// public static ITagMetaData or TagMetaData
					m_TagMetaData = (ITagMetaData) field.GetValue(null);
				} else if (field.GetFieldType().Equals(requiredParentTagType))
				{
					// public static RequiredParentTag
					requiredParentTags.Add(field.GetValue(null));
				} else if (field.GetFieldType().Equals(requiredChildTagType))
				{
					// public static RequiredChildTag
					requiredChildTags.Add(field.GetValue(null));
				}
				// There is also a related child tag field, but that is for
				// documentation purposes only, so we don't check for it here.
			}
		}

		// Transfer the RequiredTagAttributes to the new array
		m_RequiredTagAttributes = new RequiredTagAttribute[requiredTagAttributes.GetSize()];
		for (int i = 0; i < requiredTagAttributes.GetSize(); i++)
		{
			m_RequiredTagAttributes[i] = (RequiredTagAttribute) requiredTagAttributes.GetElement(i);
		}

		// Transfer the OptionalTagAttributes to the new array
		m_OptionalTagAttributes = new OptionalTagAttribute[optionalTagAttributes.GetSize()];
		for (int i = 0; i < optionalTagAttributes.GetSize(); i++)
		{
			m_OptionalTagAttributes[i] = (OptionalTagAttribute) optionalTagAttributes.GetElement(i);
		}

		// Transfer the RequiredParentTags to the new array
		m_RequiredParentTags = new ITagMetaData[requiredParentTags.GetSize()];
		for (int i = 0; i < requiredParentTags.GetSize(); i++)
		{
			m_RequiredParentTags[i] = ((RequiredParentTag) requiredParentTags.GetElement(i)).GetTagData();
		}

		// Transfer the RequiredChildTags to the new array
		m_RequiredChildTags = new ITagMetaData[requiredChildTags.GetSize()];
		for (int i = 0; i < requiredChildTags.GetSize(); i++)
		{
			m_RequiredChildTags[i] = ((RequiredChildTag) requiredChildTags.GetElement(i)).GetTagData();
		}

		m_TagLibraryMetaData = _TagLibMetaData;

		if (null == m_TagLibraryMetaData)
		{
			log.Error("Unable to intialize tag meta data for tag " +
				_Tag + " with null TagLibraryMetaData argument.  Tag Libraries must have exactly one ITagLibraryMetaData class with exactly one public static final TagLibraryMetaData member.");

			return false;
		}

		if (null == m_TagMetaData)
		{
			log.Error("Unable to load tag meta data for tag " +
				_Tag + ".  Tags must have a public static final ITagMetaData member variable.");

			return false;
		}

		return true;
	}

	/**
	 * Store the Tag Library meta data object for the Tag Library
	 * that contains this Tag.
	 * 
	 * @return TagLibraryMetaData The meta data object for the
	 * Tag Library that contains this Tag.
	 */
	public TagLibraryMetaData GetTagLibraryMetaData()
	{
		if (null == m_TagLibraryMetaData)
		{
			throw new XPException("Attempt to use tag library meta data in tag " +
				GetTagMetaData().GetName() +
				" before it has been set.  This should be initialized when the Tag meta data is initialized.");
		}

		return m_TagLibraryMetaData;
	}

	/**
	 * Gets the meta data for this tag.
	 * 
	 * @return TagMetaData the meta data for this tag.
	 */
	public ITagMetaData GetTagMetaData()
	{
		return m_TagMetaData;
	}

	/**
	 * Gets the meta data for the required tag attributes for this tag.
	 * 
	 * @return RequiredTagAttribute[] The meta data for the required tag
	 * attributes for this tag.  This array may be empty.
	 */
	public RequiredTagAttribute[] GetRequiredTagAttributes()
	{
		return m_RequiredTagAttributes;
	}

	/**
	 * Gets the meta data for the optional tag attributes for this tag.
	 * 
	 * @return OptionalTagAttribute[] The meta data for the optional tag
	 * attributes for this tag.  This array may be empty.
	 */
	public OptionalTagAttribute[] GetOptionalTagAttributes()
	{
		return m_OptionalTagAttributes;
	}

	/**
	 * Gets the required parent tags meta data for this tag.
	 *  
	 * @return ITagMetaData[] The required parent tags
	 * meta data for this tag.  This array may be empty.
	 */
	public ITagMetaData[] GetRequiredParentTags()
	{
		return m_RequiredParentTags;
	}

	/**
	 * Gets the required child tags meta data for this tag.
	 * 
	 * @return ITagMetaData[] The meta data for the
	 * required child tags for this tag.  This array may be empty.
	 */
	public ITagMetaData[] GetRequiredChildTags()
	{
		return m_RequiredChildTags;
	}
}
