package com.plumtree.portaluiinfrastructure.tree;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.resultwrapper.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.tree.*;

/**
 * This model handles generating a tree for admin objects.
 *
 * @author Don Hayler
 */
public class AdminTreeModel extends AObjectTreeModel
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.tree.AdminTreeModel");

	/** MVC object name */
	public static final String STR_MVC_CLASS_NAME = "AdminTree";

	/** the maximum number of objects to list */
	protected static final int m_nOBJECTS_TO_LIST = 50;
	
	/** server objects */
	protected IPTAdminCatalog m_ptCatalog	    = null;
	protected IPTSession m_ptSession		    = null;
	protected ObjectManagerHolder m_GroupObjMgr = null;
	protected ObjectManagerHolder m_UserObjMgr  = null;
	protected XPArrayList m_ObjMgrs;

	/** Class IDs to display */
	protected int[] m_nDisplayedClassIDs;

	/** data for fake query filters in search */
	protected int[] m_nPortletAlignment;
	protected int[] m_nPortletWebServiceID;
	protected int[] m_nPortletType;
	// TODO: this should just be a String, and we should check that people
	// aren't trying to filter on more than one CLSID.  Also, make sure we
	// set this String correctly in ResetTree().
	protected String[] m_sProviderCLSID;
	protected int m_nProviderPropIDType;
	protected int m_nFilterClassID;
	
	protected int[] m_nUserTypeFilter;
	
	/** Whether or not we are displaying portlets */
	protected boolean m_bDisplayingPortlets;

	/** Whether or not we are only displaying pre-selected objects */
	protected boolean m_bDisplayPreSelectedList = false;

	/** Whether or not we are showing global objects */
	protected boolean m_bShowGlobalObjects = false;

	/** internal data for community modes */
	protected XPArrayList m_nxpCommunityFolderIDs;
	protected int m_nCommunityMode;

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new AdminTreeModel();
	}

	/**
	 * This method expands a node in the tree (Admin folder)
	 *
	 * @param bExpand whether to expand or collapse (true implies expand)
	 * @param nNodeID the ID of the node to expand
	 */
	public void ExpandNode(boolean bExpand, int nNodeID)
	{
		super.ExpandNode(bExpand, nNodeID);

		TreeCompositeNode node = (TreeCompositeNode) m_Root.GetNode(nNodeID);

		if (node == null)
		{
			
				log.Error("Could not find node ID " + nNodeID);
			

			return;
		}

		node.m_bExpanded = bExpand;

		if ((node.m_bChildrenLoaded == false) && bExpand)
		{
			// We need to do a special query for selectable folders if we
			// are selecting folders and we have a special folder select query filter
			boolean bUseFolderSelectQuery = ((m_ptFolderSelectqf != null) &&
				((m_nSelectMode == TreeConstants.N_COMPOSITESELECT) ||
				(m_nSelectMode == TreeConstants.N_LEAFCOMPOSITESELECT)));

			if (node.m_nClassID == PT_CLASSIDS.PT_USERGROUP_ID)
			{
				ExpandGroup(node);
			}
			else
			{
				IPTAdminFolder nodeFolder = null;
				IASQueryResult query	  = null;

				// 0 means just this level for Admin Folders (not for Catalogs - 1)
				int nLevel = 0;

				TreeCompositeNode myFolder;
				TreeLeafNode myLeaf;
				int nCount;
				int nObjectID;

				// Get all folders

				try
				{
					// 0 = false = open object unlocked
					nodeFolder     = m_ptCatalog.OpenAdminFolder(node.m_nObjectID, false);
					// Use a null query filter because we always want to display all subfolders,
					// even if they're not selectable, except for in the NOCOMMUNITIES case
					Object[][] qf = null;
					if (AObjectTreeModel.NOCOMMUNITIES == m_nCommunityMode)
					{
						qf = new Object[3][1];
						qf[0][0] = new Integer(PT_PROPIDS.PT_PROPID_FOLDER_FOLDERTYPE);
						qf[1][0] = new Integer(PT_FILTEROPS.PT_FILTEROP_NE);
						qf[2][0] = new Integer(PT_ADMIN_FOLDER_TYPES.PT_ADMIN_FOLDER_TYPE_COMMUNITYFOLDER);
					}
					query		   = new ASQueryResultWrapper(nodeFolder.QuerySubfolders(
						PT_PROPIDS.PT_PROPID_OBJECTID | PT_PROPIDS.PT_PROPID_NAME |
						PT_PROPIDS.PT_PROPID_FOLDER_FOLDERTYPE | PT_PROPIDS.PT_PROPID_DESCRIPTION,
						nLevel, new Integer(PT_PROPIDS.PT_PROPID_NAME), 0, -1, qf));
				}
				catch (Exception e)
				{
					
						log.Error(e, "Failed getting subfolders of folder # " + nNodeID);
					

					return;
				}

				nCount = query.GetCount();

				int nFolderType;

				// Add composite node children
				for (int i = 0; i < nCount; i++)
				{
					nObjectID     = XPConvert.ToInteger(query.GetFields(i, PT_PROPIDS.PT_PROPID_OBJECTID));
					
					myFolder = GetCompositeIfAlreadySelected(PT_CLASSIDS.PT_ADMIN_FOLDER_ID +
							"," + nObjectID);

					if (myFolder == null)
					{
						myFolder				 = new TreeCompositeNode(GetNextUniqueID());
						myFolder.SetName(XPConvert.ToString(query.GetFields(
									i, PT_PROPIDS.PT_PROPID_NAME)));
						myFolder.SetDescription(XPConvert.ToString(query.GetFields(
									i, PT_PROPIDS.PT_PROPID_DESCRIPTION)));
						myFolder.m_nObjectID     = nObjectID;
						myFolder.m_nClassID		 = PT_CLASSIDS.PT_ADMIN_FOLDER_ID;
					}

					nFolderType = XPConvert.ToInteger(query.GetFields(i,
						PT_PROPIDS.PT_PROPID_FOLDER_FOLDERTYPE));

					if (nFolderType == PT_ADMIN_FOLDER_TYPES.PT_ADMIN_FOLDER_TYPE_COMMUNITYFOLDER)
					{
						// This determines whether or not to disable a folder based upon the community mode and ID
						CheckDisabledCommunity(myFolder);
					}

					if (bUseFolderSelectQuery)
					{
						// We will set selectable folders below
						myFolder.m_bBrowsableOnly = true;
					} else
					{
						// All folders are browsable and selectable
						// (if folders are selectable)
						myFolder.m_bBrowsableOnly = false;
					}

					SetCompositeImage(myFolder, nFolderType, m_asOwner);

					node.AddCompositeChild(myFolder);

					
						log.Debug("In node " + nNodeID + ", loaded composite child " +
							myFolder.GetEncodedName());
					
				}

				if (bUseFolderSelectQuery) 
				{
					// Get selectable folders

					// NOTE: The previous query should really return the security levels so
					// I can selectively set browsing only folders but
					// PTAdminFolder.QuerySubfolders() doesn't allow you to add
					// the access level to the query for some reason.

					try
					{
						// 0 = false = open object unlocked
						Object[][] qf;
						if (AObjectTreeModel.NOCOMMUNITIES == m_nCommunityMode)
						{
							// get the length of the operator column
							qf = new Object[3][m_ptFolderSelectqf[1].length+1];
							for (int i=0; i<m_ptFolderSelectqf[1].length; i++)
							{
								qf[0][i] = m_ptFolderSelectqf[0][i];
								qf[1][i] = m_ptFolderSelectqf[1][i];
								qf[2][i] = m_ptFolderSelectqf[2][i];
							}
							qf[0][m_ptFolderSelectqf[1].length] = new Integer(PT_PROPIDS.PT_PROPID_FOLDER_FOLDERTYPE);
							qf[1][m_ptFolderSelectqf[1].length] = new Integer(PT_FILTEROPS.PT_FILTEROP_NE);
							qf[2][m_ptFolderSelectqf[1].length] = new Integer(PT_ADMIN_FOLDER_TYPES.PT_ADMIN_FOLDER_TYPE_COMMUNITYFOLDER);
						}
						else
						{
							qf = m_ptFolderSelectqf;
						}
						query = new ASQueryResultWrapper(nodeFolder.QuerySubfolders(
							PT_PROPIDS.PT_PROPID_OBJECTID | PT_PROPIDS.PT_PROPID_NAME |
							PT_PROPIDS.PT_PROPID_FOLDER_FOLDERTYPE | PT_PROPIDS.PT_PROPID_DESCRIPTION,
							nLevel, new Integer(PT_PROPIDS.PT_PROPID_NAME), 0, -1, qf));
					}
					catch (Exception e)
					{
						
							log.Error(e, "Failed getting selectable subfolders of folder # " + nNodeID);
						
	
						return;
					}
	
					nCount = query.GetCount();
	
					// set selectable composite node children
					for (int i = 0; i < nCount; i++)
					{
						nObjectID     = XPConvert.ToInteger(query.GetFields(i, PT_PROPIDS.PT_PROPID_OBJECTID));

						myFolder = (TreeCompositeNode) node.GetNode(PT_CLASSIDS.PT_ADMIN_FOLDER_ID, nObjectID);

						if (myFolder != null)
						{
							myFolder.m_bBrowsableOnly = false;
						}

						
							log.Debug("In node " + nNodeID + ", set selectable composite child " +
								myFolder.GetEncodedName());
						
					}
				}

				Object[][] ptqf;

				// Add leaf node children
				for (int j = 0; j < m_ObjMgrs.GetSize(); j++)
				{
					ObjectManagerHolder omHolder = (ObjectManagerHolder)m_ObjMgrs.GetElement(j);
					
					// Get the correct query filter
					// TODO: fix this so we don't set ptqf in the result
					ptqf = (omHolder.m_ptqf == null) ? (ptqf = m_ptqf)
														 : omHolder.m_ptqf;

					PTObjectManagerQueryWrapper omqw = new PTObjectManagerQueryWrapper(omHolder.m_ptObjMgr);
					omqw.Query(PT_PROPIDS.PT_PROPID_OBJECTID |
						PT_PROPIDS.PT_PROPID_DESCRIPTION |
						PT_PROPIDS.PT_PROPID_NAME |
						PT_PROPIDS.PT_PROPID_USER_LOGINNAME, node.m_nObjectID,
						null, ptqf);
					query	   = omqw;
					nCount     = query.GetCount();

					for (int i = 0; i < nCount; i++)
					{
						nObjectID = XPConvert.ToInteger(query.GetFields(i,
								PT_PROPIDS.PT_PROPID_OBJECTID));

						// If the object is not hidden
						if (!omHolder.HiddenObject(nObjectID))
						{
							if ((omHolder.m_nManagedClassID == PT_CLASSIDS.PT_USERGROUP_ID) &&
									(m_UserObjMgr != null))
							{
								// If we are creating a group object for display, and we also display users,
								// make the group browseable
								myLeaf = (TreeLeafNode) m_xpmSelectedItems.GetElement(omHolder.m_nManagedClassID +
										"," + nObjectID);

								if (myLeaf != null)
								{
									// If the group is already selected but not in the tree, remove it
									// and add a new composite node for it.
									myFolder				 = new TreeCompositeNode(GetNextUniqueID());
									myFolder.SetNameAndDescription(myLeaf);
									myFolder.m_nObjectID     = myLeaf.m_nObjectID;
									myFolder.m_nClassID		 = myLeaf.m_nClassID;

									if (myLeaf.m_nTreeNodeID == -1)
									{
										m_xpmSelectedItems.PutElement(omHolder.m_nManagedClassID +
											"," + nObjectID, myFolder);
									}
								}
								else
								{
									// Add groups as folders so they can be browsed.
									myFolder				 = new TreeCompositeNode(GetNextUniqueID());
									myFolder.SetName((String) query.GetFields(i,
											PT_PROPIDS.PT_PROPID_NAME));
									myFolder.SetDescription((String) query.GetFields(i,
											PT_PROPIDS.PT_PROPID_DESCRIPTION));
									myFolder.m_nObjectID     = nObjectID;
									myFolder.m_nClassID		 = PT_CLASSIDS.PT_USERGROUP_ID;
								}

								SetCompositeImage(myFolder, m_asOwner);
								myFolder.m_bMultiplyInstanced = true;

								if ((m_nSelectMode == TreeConstants.N_LEAFSELECT) &&
									(m_nSelectOptions != TreeConstants.N_STRICTLEAF) &&
									(m_GroupObjMgr != null))
								{
									// This ensures that this group will be selectable
									// even though normal composite nodes are not.
									myFolder.m_bForceSelectable = true;
								}

								node.AddCompositeChild(myFolder);

								
									log.Debug("In node " + nNodeID +
										", loaded composite group child " +
										myFolder.GetEncodedName());
								
							}
							else
							{
								myLeaf = GetLeafIfAlreadySelected(omHolder.m_nManagedClassID +
										"," + nObjectID);

								if (myLeaf == null)
								{
									myLeaf = new TreeLeafNode(GetNextUniqueID());

									if (omHolder.m_nManagedClassID == PT_CLASSIDS.PT_USER_ID)
									{
										// Display both the human readable name and the loginname
										myLeaf.SetName((String) query.GetFields(i,
												PT_PROPIDS.PT_PROPID_NAME) +
											" (" +
											query.GetFields(i,
												PT_PROPIDS.PT_PROPID_USER_LOGINNAME) +
											")");
									}
									else
									{
										myLeaf.SetName((String) query.GetFields(i,
												PT_PROPIDS.PT_PROPID_NAME));
									}

									//TODO: Should all of these String casts use XPConvert.ToString?
									myLeaf.SetDescription((String) query.GetFields(i, PT_PROPIDS.PT_PROPID_DESCRIPTION));
									myLeaf.m_nObjectID     = nObjectID;
									myLeaf.m_nClassID	   = omHolder.m_ptObjMgr.GetManagedClassID();
								}

								SetLeafImage(myLeaf, m_asOwner);

								if (omHolder.m_nManagedClassID == PT_CLASSIDS.PT_USER_ID)
								{
									myLeaf.m_bMultiplyInstanced = true;
								}

								node.AddLeafChild(myLeaf);

								
									log.Debug("In node " + nNodeID +
										", loaded leaf child " +
										myLeaf.GetEncodedName());
								
							}
						}
					}
				}

				// Global objects
				if (m_bShowGlobalObjects && (node == m_Root))
				{
					// Add global schedulable objects (class ID 0 - PT_CLASSIDS.PT_GLOBALOBJECT_ID)
					// 0 means not locked
					// Note, this can return either a query result, or an object
					query     = new ASQueryResultWrapper((IPTQueryResult) m_ptSession.OpenGlobalObject(
								PT_GLOBALOBJECTS.PT_GLOBAL_SCHEDULED_AGENTS, false));

					nCount = query.GetCount();

					for (int i = 0; i < nCount; i++)
					{
						nObjectID     = XPConvert.ToInteger(query.GetFields(i,
								PT_PROPIDS.PT_PROPID_OBJECTID));
								
						myLeaf		  = GetLeafIfAlreadySelected(PT_CLASSIDS.PT_GLOBALOBJECT_ID +
								"," + nObjectID);

						if (myLeaf == null)
						{
							myLeaf				   = new TreeLeafNode(GetNextUniqueID());
							String strGlobalObjectName = GetLocalizedNameForGlobalObject(nObjectID);
							if(null == strGlobalObjectName)
							{
								strGlobalObjectName = (String) query.GetFields(i, PT_PROPIDS.PT_PROPID_NAME);
							}
							myLeaf.SetName(strGlobalObjectName);

							String strGlobalObjectDesc = GetLocalizedDescriptionForGlobalObject(nObjectID);
							if(null == strGlobalObjectDesc)
							{
								strGlobalObjectDesc = (String) query.GetFields(i, PT_PROPIDS.PT_PROPID_DESCRIPTION);
							}
							myLeaf.SetDescription(strGlobalObjectDesc);
							myLeaf.m_nObjectID     = nObjectID;
							myLeaf.m_nClassID	   = PT_CLASSIDS.PT_GLOBALOBJECT_ID;
						}

						SetLeafImage(myLeaf, m_asOwner);
						node.AddLeafChild(myLeaf);

						
							log.Debug("In node " + nNodeID +
								", loaded global schedulable leaf child " +
								myLeaf.GetEncodedName());
						
					}
				}
			}

			node.m_bChildrenLoaded = true;

			
				log.Info("Loaded children of folder # " + nNodeID);
			
		}
	}

	private String GetLocalizedNameForGlobalObject(int nGlobalObjectID)
	{
		String strAgentName;
		switch(nGlobalObjectID)
		{
			case PT_GLOBALOBJECTS.PT_GLOBAL_CARD_REFRESH_AGENT:
				strAgentName = m_asOwner.GetString(473, "ptmsgs_portalinfrastructure");
				break;
			case PT_GLOBALOBJECTS.PT_GLOBAL_CATALOG_SEARCH_UPDATE_AGENT:
				strAgentName = m_asOwner.GetString(475, "ptmsgs_portalinfrastructure");
				break;
			case PT_GLOBALOBJECTS.PT_GLOBAL_WEEKLY_HOUSEKEEPING_AGENT:
				strAgentName = m_asOwner.GetString(477, "ptmsgs_portalinfrastructure");
				break;
			case PT_GLOBALOBJECTS.PT_GLOBAL_AUDITMANAGER:
				strAgentName = m_asOwner.GetString(479, "ptmsgs_portalinfrastructure");
				break;
			default:
				strAgentName = null;
				break;
		}
		
		return strAgentName;
	}

	private String GetLocalizedDescriptionForGlobalObject(int nGlobalObjectID)
	{
		String strAgentDesc;
		switch(nGlobalObjectID)
		{
			case PT_GLOBALOBJECTS.PT_GLOBAL_CARD_REFRESH_AGENT:
				strAgentDesc = m_asOwner.GetString(474, "ptmsgs_portalinfrastructure");
				break;
			case PT_GLOBALOBJECTS.PT_GLOBAL_CATALOG_SEARCH_UPDATE_AGENT:
				strAgentDesc = m_asOwner.GetString(476, "ptmsgs_portalinfrastructure");
				break;
			case PT_GLOBALOBJECTS.PT_GLOBAL_WEEKLY_HOUSEKEEPING_AGENT:
				strAgentDesc = m_asOwner.GetString(478, "ptmsgs_portalinfrastructure");
				break;
			case PT_GLOBALOBJECTS.PT_GLOBAL_AUDITMANAGER:
				strAgentDesc = m_asOwner.GetString(480, "ptmsgs_portalinfrastructure");
				break;
			default:
				strAgentDesc = null;
				break;
		}
		
		return strAgentDesc;
	}


	/**
	 * This method expands all items in the obj mgr into a flat list and hides
	 * the root folder.
	 */
	public void ExpandToFlatList()
	{
		m_bShowRoot     = false;
		m_bFlatList     = true;

		m_Root.m_bExpanded			 = true;
		m_Root.m_bChildrenLoaded     = true;

		int nTotalFlatListCount = 0;

		IASQueryResult query;
		int nCount;
		int nObjectID;
		Object[][] ptqf;
		TreeLeafNode myLeaf;
		TreeCompositeNode myFolder;
		TreeLeafNode previousLeaf = null;
		String strPreviousName = null;
		String strCurrentName = null;

		for (int j = 0; j < m_ObjMgrs.GetSize(); j++)
		{
			ObjectManagerHolder omHolder = (ObjectManagerHolder)m_ObjMgrs.GetElement(j);

			// Get the correct query filter
			ptqf = (omHolder.m_ptqf == null) ? (ptqf = m_ptqf)
												 : omHolder.m_ptqf;

			PTObjectManagerQueryWrapper omqw = new PTObjectManagerQueryWrapper(omHolder.m_ptObjMgr);

			// -1 means get all objects
			omqw.Query(PT_PROPIDS.PT_PROPID_OBJECTID |
				PT_PROPIDS.PT_PROPID_DESCRIPTION |
				PT_PROPIDS.PT_PROPID_NAME |
				PT_PROPIDS.PT_PROPID_USER_LOGINNAME, -1, null, ptqf);
			query	   = omqw;
			nCount     = query.GetCount();

			for (int i = 0; i < nCount; i++)
			{
				nObjectID = XPConvert.ToInteger(query.GetFields(i,
						PT_PROPIDS.PT_PROPID_OBJECTID));

				// If the object is not hidden
				if (!omHolder.HiddenObject(nObjectID))
				{
					nTotalFlatListCount++;

					strPreviousName = strCurrentName;
					strCurrentName = (String) query.GetFields(i, PT_PROPIDS.PT_PROPID_NAME);

					if ((omHolder.m_nManagedClassID == PT_CLASSIDS.PT_USERGROUP_ID) &&
							(m_UserObjMgr != null))
					{
						// If we are in a group and we are displaying users, make the group browseable
						myLeaf = (TreeLeafNode) m_xpmSelectedItems.GetElement(omHolder.m_nManagedClassID +
								"," + nObjectID);

						if (myLeaf != null)
						{
							// If the group is already selected but not in the tree, remove it
							// and add a new composite node for it.
							myFolder				 = new TreeCompositeNode(GetNextUniqueID());
							myFolder.SetNameAndDescription(myLeaf);
							myFolder.m_nObjectID     = myLeaf.m_nObjectID;
							myFolder.m_nClassID		 = myLeaf.m_nClassID;

							if (myLeaf.m_nTreeNodeID == -1)
							{
								m_xpmSelectedItems.PutElement(omHolder.m_nManagedClassID +
									"," + nObjectID, myFolder);
							}
						}
						else
						{
							// Add groups as folders so they can be browsed.
							myFolder				 = new TreeCompositeNode(GetNextUniqueID());
							myFolder.SetName(strCurrentName);
							myFolder.SetDescription((String) query.GetFields(i,
									PT_PROPIDS.PT_PROPID_DESCRIPTION));
							myFolder.m_nObjectID     = nObjectID;
							myFolder.m_nClassID		 = PT_CLASSIDS.PT_USERGROUP_ID;
						}

						myFolder.m_bMultiplyInstanced = true;

						if ((m_nSelectMode == TreeConstants.N_LEAFSELECT) &&
							(m_nSelectOptions != TreeConstants.N_STRICTLEAF) &&
							(m_GroupObjMgr != null))
						{
							// This ensures that this group will be selectable
							// even though normal composite nodes are not.
							myFolder.m_bForceSelectable = true;
						}

						SetCompositeImage(myFolder, m_asOwner);
						m_Root.AddCompositeChild(myFolder);

						CheckForIdenticalNames(previousLeaf, strPreviousName, myFolder, strCurrentName);

						previousLeaf = myFolder;

						
							log.Debug("Loaded composite group child into flat list" +
								myFolder.GetEncodedName());
						
					}
					else
					{
						myLeaf = GetLeafIfAlreadySelected(omHolder.m_ptObjMgr.GetManagedClassID() +
								"," + nObjectID);

						if (myLeaf == null)
						{
							myLeaf = new TreeLeafNode(GetNextUniqueID());

							if (omHolder.m_nManagedClassID == PT_CLASSIDS.PT_USER_ID)
							{
								// Display both the human readable name and the loginname
								strCurrentName = (String) query.GetFields(i,
										PT_PROPIDS.PT_PROPID_NAME) + " (" +
									query.GetFields(i,
										PT_PROPIDS.PT_PROPID_USER_LOGINNAME) + ")";

								myLeaf.SetName(strCurrentName);
							}
							else
							{
								myLeaf.SetName(strCurrentName);
							}

							myLeaf.SetDescription((String) query.GetFields(i,
									PT_PROPIDS.PT_PROPID_DESCRIPTION));
							myLeaf.m_nObjectID     = nObjectID;
							myLeaf.m_nClassID	   = omHolder.m_ptObjMgr.GetManagedClassID();
						}

						SetLeafImage(myLeaf, m_asOwner);
						m_Root.AddLeafChild(myLeaf);

						CheckForIdenticalNames(previousLeaf, strPreviousName, myLeaf, strCurrentName);

						previousLeaf = myLeaf;

						
							log.Debug("loaded leaf child into flat list" +
								myLeaf.GetEncodedName());
						
					}
				}
			}
		}

		// Global objects
		if (m_bShowGlobalObjects)
		{
			// Add global schedulable objects (class ID 0 - PT_CLASSIDS.PT_GLOBALOBJECT_ID)
			// Note, this can return either a query result, or an object
			query     = new ASQueryResultWrapper((IPTQueryResult) m_ptSession.OpenGlobalObject(
						PT_GLOBALOBJECTS.PT_GLOBAL_SCHEDULED_AGENTS, false));

			nCount = query.GetCount();

			nTotalFlatListCount += nCount;

			for (int i = 0; i < nCount; i++)
			{
				nObjectID     = XPConvert.ToInteger(query.GetFields(i,
						PT_PROPIDS.PT_PROPID_OBJECTID));
						
				myLeaf		  = GetLeafIfAlreadySelected(PT_CLASSIDS.PT_GLOBALOBJECT_ID +
						"," + nObjectID);

				if (myLeaf == null)
				{
					myLeaf				   = new TreeLeafNode(GetNextUniqueID());
					
					String strGlobalObjectName = GetLocalizedNameForGlobalObject(nObjectID);
					if(null == strGlobalObjectName)
					{
						strGlobalObjectName = (String) query.GetFields(i, PT_PROPIDS.PT_PROPID_NAME);
					}
					
					myLeaf.SetName(strGlobalObjectName);

					String strGlobalObjectDesc = GetLocalizedDescriptionForGlobalObject(nObjectID);
					if(null == strGlobalObjectDesc)
					{
						strGlobalObjectDesc = (String) query.GetFields(i, PT_PROPIDS.PT_PROPID_DESCRIPTION);
					}
					myLeaf.SetDescription(strGlobalObjectDesc);
					
					myLeaf.m_nObjectID     = nObjectID;
					myLeaf.m_nClassID	   = PT_CLASSIDS.PT_GLOBALOBJECT_ID;
				}

				SetLeafImage(myLeaf, m_asOwner);
				m_Root.AddLeafChild(myLeaf);

				
					log.Debug("In root node, loaded global schedulable leaf child " +
						myLeaf.GetEncodedName());
				
			}
		}

		if ((nTotalFlatListCount < 1) && (m_nSelectMode == TreeConstants.N_LEAFSELECT))
		{
			m_bEmptyList = true;
		}

		
			log.Info("Loaded flat list into root.");
		
	}

	//////////////////////////////////////
	// Read Methods for IObjectTreeModelRO
	//////////////////////////////////////

	/**
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModelRO#GetDisplayedClassIDs()
	 */
	public int[] GetDisplayedClassIDs()
	{
		return m_nDisplayedClassIDs;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModelRO#GetFolderClassID()
	 */
	public int GetFolderClassID()
	{
		return PT_CLASSIDS.PT_ADMIN_FOLDER_ID;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModelRO#GetPortletAlignment()
	 */
	public int[] GetPortletAlignment()
	{
		return m_nPortletAlignment;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModelRO#GetPortletWebServiceID()
	 */
	public int[] GetPortletWebServiceID()
	{
		return m_nPortletWebServiceID;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModelRO#GetPortletType()
	 */
	public int[] GetPortletType()
	{
		return m_nPortletType;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModelRO#GetUserTypeFilter()
	 */
	public int[] GetUserTypeFilter()
	{
		return m_nUserTypeFilter;
	}
	
	/**
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModelRO#GetFilterCLSID()
	 */
	public String[] GetFilterCLSID()
	{
		return m_sProviderCLSID;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModelRO#GetFilterCLSIDPropID()
	 */
	public int GetFilterCLSIDPropID()
	{
		return m_nProviderPropIDType;
	}

	/**
	 * This method returns the class ID to be filtered.
	 *
	 * @return the class ID to apply the above filter to
	 */
	public int GetFilterClassID()
	{
		return m_nFilterClassID;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModelRO#GetItemDisplayed(int,
	 *      int)
	 */
	public boolean GetItemDisplayed(int nClassID, int nObjectID)
	{
		if (nClassID == PT_CLASSIDS.PT_ADMIN_FOLDER_ID)
		{
			if ((nObjectID == PT_INTRINSICS.PT_ADMIN_FOLDER_ROOTFOLDER) &&
				(!m_bShowRoot))
			{
				// Do not return display the root folder if we are not supposed to.
				// This won't show up in normal browsing, but the root folder could
				// be returned in a search result.
				return false;
			} else if ((nObjectID == PT_INTRINSICS.PT_ADMIN_FOLDER_DEFAULTPROFILES) ||
				(nObjectID == PT_INTRINSICS.PT_ADMIN_FOLDER_PROFILELAYOUT))
			{
				// The default profiles and profile layout folders are outside of
				// the admin heirarchy and therefore should never be displayed as
				// an admin folder.
				return false;
			}
		}

		for (int j = 0; j < m_ObjMgrs.GetSize(); j++)
		{
			ObjectManagerHolder omHolder = (ObjectManagerHolder)m_ObjMgrs.GetElement(j);

			if (omHolder.m_nManagedClassID == nClassID)
			{
				return !omHolder.HiddenObject(nObjectID);
			}
		}

		return true;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModelRO#GetSearchEnabled()
	 */
	public boolean GetSearchEnabled()
	{
		// Always keep search on, even if it means we can return objects that we shouldn't.
		// This is important for adding ACLS with only certain items displayed.  We can
		// easily have a large number of users in a folder and need search to make it
		// manageable.
		return true;

		// turn search off if we are displaying a pre-selected list
		// because search will return other objects.
//		return !m_bDisplayPreSelectedList;
	}

	///////////////////////////
	// RO Methods from IModelRO
	///////////////////////////

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	////////////////////////////
	// Write Methods from IModel
	////////////////////////////

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#Init(AActivitySpace)
	 */
	public void Init(AActivitySpace parent)
	{
		super.Init(parent);
		m_ptSession = (IPTSession) m_asOwner.GetUserSession();

		if (m_ptSession == null)
		{
			
				log.Error("Failed getting IPTSession.");
			
		}

		m_ptCatalog = m_ptSession.GetAdminCatalog();

		if (m_ptCatalog == null)
		{
			
				log.Error("Failed getting admin catalog.");
			
		}
	}

	/**
	 * 
	 */
	public void ResetTree()
	{
		super.ResetTree();
		m_ObjMgrs				 = new XPArrayList();
		m_bShowGlobalObjects     = false;
		m_nDisplayedClassIDs     = new int[0];
		m_nPortletType			 = new int[0];
		m_nPortletAlignment		 = new int[0];
		m_nPortletWebServiceID	 = new int[0];
		m_nProviderPropIDType	 = 0;
		m_nFilterClassID 		 = 0;
		m_sProviderCLSID		 = new String[0];

		// community modes
		m_nCommunityMode			   = AdminTreeModel.NOCOMMUNITIES;
		m_nxpCommunityFolderIDs		   = new XPArrayList();

		// Helper variables for groups in groups
		m_UserObjMgr			  = null;
		m_GroupObjMgr			  = null;
		m_bDisplayingPortlets     = false;
		m_bDisplayPreSelectedList = false;
	}

	/**
	 * This method is used to display multiple groups at the root level
	 * instead of a folder heirarchy.
	 *
	 * @param nIDs 
	 */
	public void ExpandDisplayedItemsOnly()
	{
		m_bShowRoot     = false;
		m_bFlatList     = true;

		m_Root				   = new TreeCompositeNode(GetNextUniqueID());
		m_Root.SetName("");
		m_Root.m_nClassID	   = -1;
		m_Root.m_nObjectID     = -1;

		IPTObject myObject;
		TreeLeafNode myLeafNode;
		TreeCompositeNode myCompositeNode;
		int[] nIDs;


		for (int j = 0; j < m_ObjMgrs.GetSize(); j++)
		{
			ObjectManagerHolder omHolder = (ObjectManagerHolder)m_ObjMgrs.GetElement(j);
			nIDs = omHolder.m_nDisplayedObjectIDs;

			if (nIDs != null)
			{
				if (omHolder.m_nManagedClassID == PT_CLASSIDS.PT_USERGROUP_ID)
				{
					for (int i = 0; i < nIDs.length; i++)
					{
						myCompositeNode = GetCompositeIfAlreadySelected(omHolder.m_nManagedClassID +
							"," + nIDs[i]);

						if (myCompositeNode == null)
						{
							try
							{
								myObject = (IPTObject) omHolder.m_ptObjMgr.Open(nIDs[i], false);
	
								myCompositeNode				    = new TreeCompositeNode(GetNextUniqueID());
								myCompositeNode.SetName(myObject.GetLocalizedName());
								myCompositeNode.SetDescription(myObject.GetLocalizedDescription());
								myCompositeNode.m_nClassID	    = omHolder.m_nManagedClassID;
								myCompositeNode.m_nObjectID     = nIDs[i];
							} catch (Exception e)
							{
								
									log.Error(e, "Could not load default composite in class " + omHolder.m_nManagedClassID +
										" with ID " + nIDs[i]);
								
							}
						}
						SetCompositeImage(myCompositeNode, m_asOwner);
						m_Root.AddCompositeChild(myCompositeNode);

						if ((m_nSelectMode == TreeConstants.N_LEAFSELECT) &&
							(m_nSelectOptions != TreeConstants.N_STRICTLEAF) &&
							(m_GroupObjMgr != null))
						{
							// This ensures that this group will be selectable
							// even though normal composite nodes are not.
							myCompositeNode.m_bForceSelectable = true;
						}

						
							log.Debug("In root, loaded default composite child " +
								myCompositeNode.GetEncodedName());
						
					}
				} else {
					for (int i = 0; i < nIDs.length; i++)
					{
						myLeafNode = GetLeafIfAlreadySelected(omHolder.m_nManagedClassID +
							"," + nIDs[i]);

						if (myLeafNode == null)
						{
							try
							{
								myObject = (IPTObject) omHolder.m_ptObjMgr.Open(nIDs[i], false);
	
								myLeafNode				    = new TreeCompositeNode(GetNextUniqueID());
								myLeafNode.SetName(myObject.GetLocalizedName());
								myLeafNode.SetDescription(myObject.GetLocalizedDescription());
								myLeafNode.m_nClassID	    = omHolder.m_nManagedClassID;
								myLeafNode.m_nObjectID     = nIDs[i];
							} catch (Exception e)
							{
								
									log.Error(e, "Could not load default leaf in class " + omHolder.m_nManagedClassID +
										" with ID " + nIDs[i]);
								
							}
						}

						if (null != myLeafNode)
						{
							SetLeafImage(myLeafNode, m_asOwner);
							m_Root.AddLeafChild(myLeafNode);

							
								log.Debug("In root, loaded default leaf child " +
									myLeafNode.GetEncodedName());
							
						}
					}
				}
			}
		}

		m_Root.m_bChildrenLoaded     = true;
		m_Root.m_bExpanded			 = true;
	}

	/**
	 * 
	 *
	 * @param nIDs 
	 */
	public void SetClassIDsToDisplay(int[] nIDs)
	{
		for (int i = 0; i < nIDs.length; i++)
		{
			if (PT_CLASSIDS.PT_GLOBALOBJECT_ID == nIDs[i])
			{
				// Handle classID 0 -PT_CLASSIDS.PT_GLOBALOBJECT_ID
				// as a special case -- it has no object manager
				m_bShowGlobalObjects = true;
			}
			else if (PT_CLASSIDS.PT_ADMIN_FOLDER_ID == nIDs[i])
			{
				// Handle admin folder ID as a special case -- it also has no object manager.
				// Don't do anything in this case.
			}
			else if (PT_CLASSIDS.PT_USERINTERFACE_ID == nIDs[i])
			{
				// Don't ever show user interface objects.
			}
			else
			{
				IPTObjectManager ptOM = null;
				
				try
				{
					ptOM = m_ptCatalog.GetObjectManager(nIDs[i]);
				}
				catch(Exception e)
				{
					
						log.Error(e, "Could not load object manager for class ID " +
							nIDs);
					
				}
				
				if (null != ptOM)
				{
					ObjectManagerHolder omHolder = new ObjectManagerHolder(ptOM);
					m_ObjMgrs.Add(omHolder);
								
					if (nIDs[i] == PT_CLASSIDS.PT_USER_ID)
					{
						m_UserObjMgr = omHolder;
					}
					else if (nIDs[i] == PT_CLASSIDS.PT_USERGROUP_ID)
					{
						m_GroupObjMgr = omHolder;
					}
					else if (nIDs[i] == PT_CLASSIDS.PT_GADGET_ID)
					{
						m_bDisplayingPortlets = true;
					}
				}
			}
		}

		// Create the list of displayed object IDs for search
		m_nDisplayedClassIDs = new int[m_ObjMgrs.GetSize()];

		for (int i = 0; i < m_ObjMgrs.GetSize(); i++)
		{
			m_nDisplayedClassIDs[i] = ((ObjectManagerHolder)m_ObjMgrs.GetElement(i)).m_nManagedClassID;
		}
	}

	/**
	 * @see com.plumtree.uiinfrastructure.tree.ITreeModel#SetObjectIDsToDisplay(int[])
	 */
	public void SetObjectIDsToDisplay(int[] nIDs)
	{
		// TODO: implement for folders as well.
		boolean bFoundObjects = false;

		int[] nTemp     = new int[nIDs.length / 2];
		int nArrayIndex = 0;

		for (int j = 0; j < m_ObjMgrs.GetSize(); j++)
		{
			ObjectManagerHolder omHolder = (ObjectManagerHolder)m_ObjMgrs.GetElement(j);
			
			// Check for displayed objects with this managed class ID
			for (int i = 0; (i + 1) < nIDs.length; i += 2)
			{
				
				if (nIDs[i] == omHolder.m_nManagedClassID)
				{
					nTemp[nArrayIndex] = nIDs[i + 1];
					nArrayIndex++;
				}
			}

			// Copy them into a new array if we found any
			if (nArrayIndex > 0)
			{
				bFoundObjects = true;

				omHolder.m_nDisplayedObjectIDs = new int[nArrayIndex];

				for (int i = 0; i < nArrayIndex; i++)
				{
					omHolder.m_nDisplayedObjectIDs[i] = nTemp[i];
				}

				nArrayIndex = 0;
			}
		}

		if (bFoundObjects)
		{
			// Set notification to only display these objects.
			m_bDisplayPreSelectedList = true;
		}
	}

	/**
	 * @see com.plumtree.uiinfrastructure.tree.ITreeModel#SetObjectIDsToHide(int[])
	 */
	public void SetObjectIDsToHide(int[] nIDs)
	{
		// TODO: should be able to hide certain folders
		int[] nTemp     = new int[nIDs.length / 2];
		int nArrayIndex = 0;

		for (int j = 0; j < m_ObjMgrs.GetSize(); j++)
		{
			ObjectManagerHolder omHolder = (ObjectManagerHolder)m_ObjMgrs.GetElement(j);
			
			// Check for hidden objects with this managed class ID
			for (int i = 0; (i + 1) < nIDs.length; i += 2)
			{
				
				if (nIDs[i] == omHolder.m_nManagedClassID)
				{
					nTemp[nArrayIndex] = nIDs[i + 1];
					nArrayIndex++;
				}
			}

			// Copy them into a new array if we found any
			if (nArrayIndex > 0)
			{
				omHolder.m_nHiddenObjectIDs = new int[nArrayIndex];

				for (int i = 0; i < nArrayIndex; i++)
				{
					omHolder.m_nHiddenObjectIDs[i] = nTemp[i];
				}

				nArrayIndex = 0;
			}
		}

		if (m_Root != null)
		{
			// we are refreshing hidden items, so we need to check if any items that
			// were previously unselected need to be removed.
			RefreshHiddenObjects();
		}
	}

	/**
	 * 
	 *
	 * @param nIDs 
	 */
	public void SetObjectIDsToSelect(int[] nIDs)
	{
		if (m_Root != null)
		{
			// we are refreshing selected items, so we clear all old items.
			// clear all selected items
			m_Root.ClearSelected(false, true);
			m_xpmSelectedItems.Clear();
		}

		PTObjectManagerQueryWrapper omqw;
		TreeLeafNode node;
		IPTAdminFolder folder;

		for (int i = 0; (i + 1) < nIDs.length; i += 2)
		{
			if (nIDs[i] == PT_CLASSIDS.PT_ADMIN_FOLDER_ID)
			{
				node = null;

				if (m_Root != null)
				{
					// since we are refreshing, the folder may already exist
					// Check if the folder is already in the tree
					node = m_Root.GetNode(nIDs[i], nIDs[i + 1]);
				}

				if (node == null)
				{
					// 0 = false = open object unlocked
					folder = m_ptCatalog.OpenAdminFolder(nIDs[i + 1], false);

					if (folder != null)
					{
						node				 = new TreeCompositeNode(-1);
						node.m_nClassID		 = PT_CLASSIDS.PT_ADMIN_FOLDER_ID;
						node.m_nObjectID     = nIDs[i + 1];
						node.SetName(folder.GetLocalizedName());
						node.SetDescription(folder.GetLocalizedDescription());
						SetCompositeImage((TreeCompositeNode) node, m_asOwner);
					}
					else 
						log.Error("Unable to open admin folder: " + nIDs[i + 1]);
					
				}

				if (node != null)
				{
					node.m_bSelected = true;
					m_xpmSelectedItems.PutElement(GetKeyFromNode(node), node);
				}
			} else if (nIDs[i] == PT_CLASSIDS.PT_GLOBALOBJECT_ID)
			{
				node = null;

				if (m_Root != null)
				{
					// Since we are refreshing, the object may already exist
					// Check if the object is already in the tree
					node = m_Root.GetNode(nIDs[i], nIDs[i + 1]);
				}

				if (node == null)
				{
					// Add global schedulable objects (class ID 0 - PT_CLASSIDS.PT_GLOBALOBJECT_ID)
					// 0 means not locked
					// Note, this can return either a query result, or an object
					IASQueryResult query = new ASQueryResultWrapper((IPTQueryResult)
						m_ptSession.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_SCHEDULED_AGENTS, false));

					int nCount = query.GetCount();
					int nObjectID;

					for (int j = 0; j < nCount; j++)
					{
						nObjectID     = XPConvert.ToInteger(query.GetFields(j,
								PT_PROPIDS.PT_PROPID_OBJECTID));
						
						if (nObjectID == nIDs[i + 1])
						{
							node = new TreeLeafNode(-1);
							node.m_nClassID  = PT_CLASSIDS.PT_GLOBALOBJECT_ID;
							node.m_nObjectID = nIDs[i + 1];

							String strGlobalObjectName = GetLocalizedNameForGlobalObject(nObjectID);
							if(null == strGlobalObjectName)
							{
								strGlobalObjectName = (String) query.GetFields(i, PT_PROPIDS.PT_PROPID_NAME);
							}
							node.SetName(strGlobalObjectName);
							
							String strGlobalObjectDesc = GetLocalizedDescriptionForGlobalObject(nObjectID);
							if(null == strGlobalObjectDesc)
							{
								strGlobalObjectDesc = (String) query.GetFields(i, PT_PROPIDS.PT_PROPID_DESCRIPTION);
							}
							node.SetDescription(strGlobalObjectDesc);
							SetLeafImage(node, m_asOwner);
						}
					}
				}

				if (node != null)
				{
					node.m_bSelected = true;
					m_xpmSelectedItems.PutElement(GetKeyFromNode(node), node);
				} else
				{
					
						log.Error("Unable to find global object (ObjectID " + nIDs[i + 1] + ").");
					
				}
			}
		}

		for (int j = 0; j < m_ObjMgrs.GetSize(); j++)
		{
			ObjectManagerHolder omHolder = (ObjectManagerHolder)m_ObjMgrs.GetElement(j);
			
			omqw = null;

			for (int i = 0; (i + 1) < nIDs.length; i += 2)
			{
				if (nIDs[i] == omHolder.m_nManagedClassID)
				{
					node = null;

					if (m_Root != null)
					{
						// Since we are refreshing, the object may already exist
						// Check if the object is already in the tree
						node = m_Root.GetNode(nIDs[i], nIDs[i + 1]);
					}

					if (node == null)
					{
						if (omqw == null)
						{
							omqw = new PTObjectManagerQueryWrapper(omHolder.m_ptObjMgr);
						}

						// NOTE: We trust the objects added in the QS rather than
						// checking security and filters ourselves.
						omqw.QuerySingleObject(nIDs[i + 1]);

						if (omqw.GetCount() > 0)
						{
							if (omHolder.m_nManagedClassID == PT_CLASSIDS.PT_USERGROUP_ID)
							{
								node = new TreeCompositeNode(-1);
							}
							else
							{
								node = new TreeLeafNode(-1);
							}

							node.m_nClassID     = omHolder.m_nManagedClassID;
							node.m_nObjectID = nIDs[i + 1];

							if (omHolder.m_nManagedClassID == PT_CLASSIDS.PT_USER_ID)
							{
								// loginname should be in the query (pt_propids_all by default for single objects).
								// Display both the human readable name and the loginname
								node.SetName((String) omqw.GetFields(0,
										PT_PROPIDS.PT_PROPID_NAME) + " (" +
									omqw.GetFields(0,
										PT_PROPIDS.PT_PROPID_USER_LOGINNAME) +
									")");
							}
							else
							{
								node.SetName((String) omqw.GetFields(0,
										PT_PROPIDS.PT_PROPID_NAME));
							}

							node.SetDescription((String) omqw.GetFields(0,
									PT_PROPIDS.PT_PROPID_DESCRIPTION));
							SetLeafImage(node, m_asOwner);
						}
						else 
							log.Error("Unable to find object (ClassID " + nIDs[i] +
								", ObjectID " + nIDs[i + 1] + ").");
						
					}

					if (node != null)
					{
						node.m_bSelected = true;
						m_xpmSelectedItems.PutElement(GetKeyFromNode(node), node);
					}
				}
			}
		}
	}

	/**
	 * This must be called after SetAccessLevel() and SetClassIDsToDisplay().
	 *
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModel#SetQueryFilter(int,
	 *      int, int[])
	 */
	public void SetQueryFilter(int nClassID, int nPropID, int[] nValues)
	{
		int nObjMgrIndex = -1;

		for (int j = 0; j < m_ObjMgrs.GetSize(); j++)
		{
			if (nClassID == ((ObjectManagerHolder)m_ObjMgrs.GetElement(j)).m_nManagedClassID)
			{
				nObjMgrIndex = j;
			}
		}

		if (nObjMgrIndex == -1)
		{
			
				log.Error("Attempting to set a query filter on a class id that is not displayed: " +
					nClassID);
			

			return;
		}

		switch (nClassID)
		{
		case PT_CLASSIDS.PT_GADGET_ID:
		{
			// Check that it's only used for certain portlet fields (supported by search)
			int intPropID = nPropID;

			if (intPropID == PT_PROPIDS.PT_PROPID_GADGET_GADGETTYPE)
			{
				m_nPortletType = nValues;
			}
			else if (intPropID == PT_PROPIDS.PT_PROPID_GADGET_ALIGNMENT)
			{
				m_nPortletAlignment = nValues;
			}
			else if (intPropID == PT_PROPIDS.PT_PROPID_GADGET_WEBSERVICEID)
			{
				m_nPortletAlignment = nValues;
			}
			else
			{
				
					log.Error("Attempting to set a  query filter on a property other than type, web service id or alignment: " +
						nPropID);
				

				return;
			}

			break;
		}
		
		case PT_CLASSIDS.PT_USER_ID:
		{
			// Check that it's only used for certain fields (supported by search)
			if (nPropID == PT_PROPIDS.PT_PROPID_USER_USERTYPE)
			{
				// note: m_nUserTypeFilter will be used for search 
				m_nUserTypeFilter = nValues;				 
			} 
			else
			{
				log.Error("Attempting to set a user query filter on a property other than usertype: " +
					nPropID);
				return;
			}
			
			break;
		}
		
		default:

			
				log.Error("Attempting to set a query filter on a class other than portlets, data sources, or web services: " +
					nClassID);
			

			return;
		}

		// Normal security filter
		Object[][] newFilter = new Object[3][2];

		// Copy the security filter over
		newFilter[0][0]     = m_ptqf[0][0];
		newFilter[1][0]     = m_ptqf[1][0];
		newFilter[2][0]     = m_ptqf[2][0];

		// Add new query filter
		newFilter[0][1]     = new Integer(nPropID);
		newFilter[1][1]     = new Integer(PT_FILTEROPS.PT_FILTEROP_IN);
		newFilter[2][1]     = nValues;

		((ObjectManagerHolder)m_ObjMgrs.GetElement(nObjMgrIndex)).m_ptqf = newFilter;
	}

	/**
	 * This must be called after SetAccessLevel() and SetClassIDsToDisplay().
	 *
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModel#SetQueryFilter(int,
	 *      int, String[])
	 */
	public void SetQueryFilter(int nClassID, int nPropID, String[] sValues)
	{
		int nObjMgrIndex = -1;

		for (int j = 0; j < m_ObjMgrs.GetSize(); j++)
		{
			if (nClassID == ((ObjectManagerHolder)m_ObjMgrs.GetElement(j)).m_nManagedClassID)
			{
				nObjMgrIndex = j;
			}
		}

		if (nObjMgrIndex == -1)
		{
			
				log.Error("Attempting to set a query filter on a class id that is not displayed: " +
					nClassID);
			

			return;
		}

		if ((nClassID == PT_CLASSIDS.PT_DATASOURCE_ID) ||
				(nClassID == PT_CLASSIDS.PT_WEBSERVICE_ID))
		{
			// Check that it's only used for certain fields (supported by search)

			if ((nPropID == PT_PROPIDS.PT_PROPID_DS_PROVIDERCLSID) ||
					(nPropID == PT_PROPIDS.PT_PROPID_WEBSERVICE_PROVIDERCLSID))
			{
				m_nProviderPropIDType     = nPropID;
				m_nFilterClassID 		  = nClassID;
				m_sProviderCLSID		  = sValues;
			}
			else
			{
				
					log.Error("Attempting to set a data source/web service query filter on a property other than ds provider clsid or ws provider clsid: " +
						nPropID);
				

				return;
			}
		}
		else
		{
			
				log.Error("Attempting to set a query filter on a class other than portlets, data sources, or web services: " +
					nClassID);
			

			return;
		}

		Object[][] newFilter = new Object[3][2];

		// Copy the security filter over
		newFilter[0][0]     = m_ptqf[0][0];
		newFilter[1][0]     = m_ptqf[1][0];
		newFilter[2][0]     = m_ptqf[2][0];

		// Add new query filter
		newFilter[0][1]     = new Integer(nPropID);
		newFilter[1][1]     = new Integer(PT_FILTEROPS.PT_FILTEROP_IN);
		newFilter[2][1]     = sValues;

		((ObjectManagerHolder)m_ObjMgrs.GetElement(nObjMgrIndex)).m_ptqf = newFilter;
	}

	/**
	 * 
	 *
	 * @param nMode 
	 * @param nFolderID 
	 */
	public void SetCommunityMode(int nMode, int nFolderID)
	{
		m_nCommunityMode = nMode;

		m_nxpCommunityFolderIDs = new XPArrayList();

		if (m_nCommunityMode == AdminTreeModel.THISCOMMUNITY)
		{
			try
			{
				// false = open object unlocked
				IPTAdminFolder folder = m_ptCatalog.OpenAdminFolder(nFolderID, false);

				while ((nFolderID > PT_INTRINSICS.PT_ADMIN_FOLDER_ROOTFOLDER) &&
					(folder.GetFolderType() == PT_ADMIN_FOLDER_TYPES.PT_ADMIN_FOLDER_TYPE_COMMUNITYFOLDER))
				{
					m_nxpCommunityFolderIDs.Add(new Integer(nFolderID));
					nFolderID = folder.GetParentFolderID();
					// false = open object unlocked
					folder = m_ptCatalog.OpenAdminFolder(nFolderID, false);
				}
			}
			catch (Exception e)
			{
				
					log.Error(e, "Failed checking parent folders of community folder: " + nFolderID);
				

				return;
			}
		}
	}

	/**
	 * This must be called last.  It completes the setup
	 * of the tree.
	 *
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModel#SetRootFolderID(int)
	 */
	public void SetRootFolderID(int nID)
	{
		if (m_bDisplayPreSelectedList)
		{
			// Display list of items from displayed items list only.
			ExpandDisplayedItemsOnly();
			return;
		}

		m_Root = GetCompositeIfAlreadySelected
			(PT_CLASSIDS.PT_ADMIN_FOLDER_ID + "," + nID);

		if (m_Root == null)
		{
			// 0 = false = open object unlocked
			IPTAdminFolder rootFolder = m_ptCatalog.OpenAdminFolder(nID, false);
	
			m_Root				   = new TreeCompositeNode(GetNextUniqueID());
			m_Root.SetName(rootFolder.GetLocalizedName());
			m_Root.SetDescription(rootFolder.GetLocalizedDescription());
			m_Root.m_nClassID	   = PT_CLASSIDS.PT_ADMIN_FOLDER_ID;
			m_Root.m_nObjectID     = nID;
			SetCompositeImage(m_Root, m_asOwner);
		}

		// Calculate total number of objects to display
		// If this is less than 50, display as a list.
		int nTotalObjects = 0;

		ObjectManagerHolder curObjMgrHolder;
		Object[][] ptqf;

		for (int i = 0; i < m_ObjMgrs.GetSize(); i++)
		{
			curObjMgrHolder = (ObjectManagerHolder) m_ObjMgrs.GetElement(i);

			// Get the correct query filter
			ptqf = (curObjMgrHolder.m_ptqf == null)
				? (ptqf = m_ptqf) : curObjMgrHolder.m_ptqf;

			// This counts all objects from the root node.  This will return an inflated
			// number if you don't start at the root node.
			nTotalObjects += ((ObjectManagerHolder)m_ObjMgrs.GetElement(i)).m_ptObjMgr.QueryCount(-1, ptqf);
		}

		boolean bExpandToFlatList = false;
		
		if((m_nSelectMode == TreeConstants.N_LEAFSELECT) && (!m_bDisplayingPortlets)) 
		{
			bExpandToFlatList = true;	
		}
		
		if(bExpandToFlatList) 
		{
			for(int i = 0; i < m_nDisplayedClassIDs.length; i++)
			{
				if(m_nDisplayedClassIDs[i] == PT_CLASSIDS.PT_SITEMAPFOLDER_ID)
				{
					bExpandToFlatList = false;
					break;
				}		
			}

			if (nTotalObjects > m_nOBJECTS_TO_LIST)
			{
				bExpandToFlatList = false;	
			}
		}

		if ((nTotalObjects < 1) && (m_nSelectMode == TreeConstants.N_LEAFSELECT))
		{
			m_bEmptyList = true;
			return;
		}

		if (nID != PT_INTRINSICS.PT_ADMIN_FOLDER_ROOTFOLDER)
		{
			// We can't expand to flat list this way because we need to check
			// what folder the objects are in.
			bExpandToFlatList = false;
		}

		if (bExpandToFlatList)
		{
			ExpandToFlatList();
		}
		else
		{
			// pre-expand the root node for everyone
			ExpandNode(true, m_Root.m_nTreeNodeID);

			if (!m_bMultiSelect)
			{
				// Open to the selected item for single selects
				// Could be too expensive for multi-select
				ExpandToSelectedItem();
			}
		}

		// If we are not showing the root, and have therefore expanded the root ...
		if (!m_bShowRoot)
		{
			// And there are no children ...
			if ((!m_Root.GetCompositeChildIterator().MoveNext()) &&
				(!m_Root.GetLeafChildIterator().MoveNext()))
			{
				// then the list is empty.
				m_bEmptyList = true;
			}
		}
	}

	/**
	 * 
	 *
	 * @param node 
	 */
	protected void ExpandGroup(TreeCompositeNode node)
	{
		if (m_GroupObjMgr == null)
		{
			
				log.Error("Attempting to expand group but could not find group object manager.");
			

			return;
		}

		// 0 = false = open object unlocked
		IPTUserGroup myGroup = (IPTUserGroup) m_GroupObjMgr.m_ptObjMgr.Open(node.m_nObjectID, false);

		// Get all groups within this group -- NOTE: this does not handle user security
		ASQueryResultWrapper asqrChildGroups = new ASQueryResultWrapper(myGroup.QueryDirectChildGroups());
		int nChildGroupCount = asqrChildGroups.GetCount();

		// Only create a new query filter if there are any child groups.
		// You cannot have a zero length array in a query filter.
		if (nChildGroupCount > 0)
		{
			if (nChildGroupCount > 255)
			{
				// We can only have 255 items in an IN filter
				// Should there be some kind of warning here?
				nChildGroupCount = 255;
			}

			Object[] nGroupIDs = new Object[nChildGroupCount];
	
			for (int i = 0; i < nChildGroupCount; i++)
			{
				// Test
				nGroupIDs[i] = new Integer(XPConvert.ToInteger(
					asqrChildGroups.GetFields(i, PT_PROPIDS.PT_PROPID_OBJECTID)));
			}
	
			// New filter for security + groups returned by previous query (with no security)
			Object[][] groupQF = new Object[3][2];
	
			// Copy the security filter over
			groupQF[0][0]     = m_ptqf[0][0];
			groupQF[1][0]     = m_ptqf[1][0];
			groupQF[2][0]     = m_ptqf[2][0];
	
			// Add new query filter for groups
			groupQF[0][1]     = new Integer(PT_PROPIDS.PT_PROPID_OBJECTID);
			groupQF[1][1]     = new Integer(PT_FILTEROPS.PT_FILTEROP_IN);
			groupQF[2][1]     = nGroupIDs;

			// Query for all groups with a query filter for the groups in the current
			// group.  This will handle checking for security.
			PTObjectManagerQueryWrapper omqwChildGroups = new PTObjectManagerQueryWrapper(m_GroupObjMgr.m_ptObjMgr);
			// -1 means return all groups
			omqwChildGroups.Query(PT_PROPIDS.PT_PROPID_OBJECTID |
				PT_PROPIDS.PT_PROPID_DESCRIPTION |
				PT_PROPIDS.PT_PROPID_NAME |
				PT_PROPIDS.PT_PROPID_USER_LOGINNAME, -1,
				null, groupQF);

			// This count (with security) should be less than or equal to the count
			// without security.
			int nChildGroupCountWithSecurity = omqwChildGroups.GetCount();
	
			TreeCompositeNode myGroupNode;
			int nGroupObjectID;
	
			for (int i = 0; i < nChildGroupCountWithSecurity; i++)
			{
				nGroupObjectID = XPConvert.ToInteger(omqwChildGroups.GetFields(i,
						PT_PROPIDS.PT_PROPID_OBJECTID));
	
				// If the object is not hidden
				if ((nGroupObjectID != node.m_nObjectID) &&
						(!m_GroupObjMgr.HiddenObject(nGroupObjectID)))
				{
					myGroupNode = GetCompositeIfAlreadySelected(PT_CLASSIDS.PT_USERGROUP_ID +
							"," + nGroupObjectID);
	
					if (myGroupNode == null)
					{
						// Add groups as folders so they can be browsed.
						myGroupNode				    = new TreeCompositeNode(GetNextUniqueID());
						myGroupNode.SetName((String) omqwChildGroups.GetFields(i,
							PT_PROPIDS.PT_PROPID_NAME));
						myGroupNode.SetDescription((String) omqwChildGroups.GetFields(i,
							PT_PROPIDS.PT_PROPID_DESCRIPTION));
						myGroupNode.m_nObjectID     = nGroupObjectID;
						myGroupNode.m_nClassID	    = PT_CLASSIDS.PT_USERGROUP_ID;
					}
	
					SetCompositeImage(myGroupNode, m_asOwner);
					myGroupNode.m_bMultiplyInstanced = true;
	
					if ((m_nSelectMode == TreeConstants.N_LEAFSELECT) &&
						(m_nSelectOptions != TreeConstants.N_STRICTLEAF) &&
						(m_GroupObjMgr != null))
					{
						// This ensures that this group will be selectable
						// even though normal composite nodes are not.
						myGroupNode.m_bForceSelectable = true;
					}
	
					node.AddCompositeChild(myGroupNode);
	
					
						log.Debug("In node " + node.m_nTreeNodeID +
							", loaded composite group child " +
							myGroupNode.GetEncodedName());
					
				}
			}
		}

		// Get users in this group
		if (m_UserObjMgr != null)
		{
			// Get all groups within this group -- NOTE: this does not handle user security
			ASQueryResultWrapper asqrChildUsers = new ASQueryResultWrapper(myGroup.QueryUsers(true /* include dynamic members */, true /* include static members */));

			// TODO: what if nCount is 10,000?
			int nChildUserCount = asqrChildUsers.GetCount();

			// Only create a new query filter if there are any child groups.
			// You cannot have a zero length array in a query filter.
			if (nChildUserCount > 0)
			{
				if (nChildUserCount > 255)
				{
					// We can only have 255 items in an IN filter
					// Should there be some kind of warning here?
					nChildUserCount = 255;
				}

				Object[] nUserIDs = new Object[nChildUserCount];
	
				for (int i = 0; i < nChildUserCount; i++)
				{
					nUserIDs[i] = new Integer(XPConvert.ToInteger(
						asqrChildUsers.GetFields(i, PT_PROPIDS.PT_PROPID_OBJECTID)));
				}
	
				// New filter for security + users returned by previous query (with no security)
				Object[][] userQF = new Object[3][2];
		
				// Copy the security filter over
				userQF[0][0]     = m_ptqf[0][0];
				userQF[1][0]     = m_ptqf[1][0];
				userQF[2][0]     = m_ptqf[2][0];
		
				// Add new query filter for users
				userQF[0][1]     = new Integer(PT_PROPIDS.PT_PROPID_OBJECTID);
				userQF[1][1]     = new Integer(PT_FILTEROPS.PT_FILTEROP_IN);
				userQF[2][1]     = nUserIDs;
	
				// Query for all groups with a query filter for the groups in the current
				// group.  This will handle checking for security.
				PTObjectManagerQueryWrapper omqwChildUsers = new PTObjectManagerQueryWrapper(m_UserObjMgr.m_ptObjMgr);
				// -1 means return all users
				omqwChildUsers.Query(PT_PROPIDS.PT_PROPID_OBJECTID |
					PT_PROPIDS.PT_PROPID_DESCRIPTION |
					PT_PROPIDS.PT_PROPID_NAME |
					PT_PROPIDS.PT_PROPID_USER_LOGINNAME, -1,
					null, userQF);

				// This count (with security) should be less than or equal to the count
				// without security.
				int nChildUserCountWithSecurity = omqwChildUsers.GetCount();
	
				TreeLeafNode myLeaf;
				int nUserObjectID;
	
				for (int i = 0; i < nChildUserCountWithSecurity; i++)
				{
					nUserObjectID = XPConvert.ToInteger(omqwChildUsers.GetFields(i, PT_PROPIDS.PT_PROPID_OBJECTID));
	
					// If the object is not hidden
					if (!m_GroupObjMgr.HiddenObject(nUserObjectID))
					{
						myLeaf = GetLeafIfAlreadySelected(PT_CLASSIDS.PT_USER_ID +
								"," + nUserObjectID);
	
						if (myLeaf == null)
						{
							myLeaf     = new TreeLeafNode(GetNextUniqueID());
	
							// Display both the human readable name and the loginname
							myLeaf.SetName((String) omqwChildUsers.GetFields(i,
									PT_PROPIDS.PT_PROPID_NAME) + " (" +
								omqwChildUsers.GetFields(i,
									PT_PROPIDS.PT_PROPID_USER_LOGINNAME) + ")");
							myLeaf.SetDescription((String) omqwChildUsers.GetFields(i,
									PT_PROPIDS.PT_PROPID_DESCRIPTION));
							myLeaf.m_nObjectID     = nUserObjectID;
							myLeaf.m_nClassID	   = PT_CLASSIDS.PT_USER_ID;
						}
	
						SetLeafImage(myLeaf, m_asOwner);
						myLeaf.m_bMultiplyInstanced = true;
						node.AddLeafChild(myLeaf);
	
						
							log.Debug("In node " + node.m_nTreeNodeID +
								", loaded leaf user child " + myLeaf.GetEncodedName());
						
					}
				}
			}
		}
	}

	/**
	 * This method expands the tree to display the
	 * single selected item.
	 */
	protected void ExpandToSelectedItem()
	{
		if (m_bMultiSelect || (m_xpmSelectedItems.GetSize() > 1))
		{
			
				log.Error("Attempt to pre-expand multiple selected items.");
			

			return;
		}

		IXPEnumerator tempEnum = m_xpmSelectedItems.GetElements();
		if (tempEnum.MoveNext())
		{
			TreeLeafNode selectedItem = (TreeLeafNode) tempEnum.GetCurrent();
			int nClassID = selectedItem.m_nClassID;
			int nObjectID = selectedItem.m_nObjectID;
			int nParentFolderID;
			
			if (nClassID == PT_CLASSIDS.PT_GLOBALOBJECT_ID)
			{
				// Global objects don't live in folders, already at root level.
				return;
			}

			// Get the ID of the parent folder
			if (nClassID == PT_CLASSIDS.PT_ADMIN_FOLDER_ID)
			{
				// 0 = false = open object unlocked
				IPTAdminFolder selectedFolder = m_ptCatalog.OpenAdminFolder(nObjectID, false);
				nParentFolderID = selectedFolder.GetParentFolderID();
			} else
			{
				ObjectManagerHolder omHolder = null;
	
				for (int j = 0; j < m_ObjMgrs.GetSize(); j++)
				{
					omHolder = (ObjectManagerHolder)m_ObjMgrs.GetElement(j);
					
					if (omHolder.m_nManagedClassID == nClassID)
					{
						break;
					}
	
					omHolder = null;
				}
	
				if (omHolder == null)
				{
					
						log.Error("ExpandToSelectedItem unable to find object manager for class ID "
							+ nClassID);
					
		
					return;
				}

				// 0 = false = open object unlocked
				IPTObject temp = (IPTObject) omHolder.m_ptObjMgr.Open(nObjectID, false);
				nParentFolderID = temp.GetAdminFolderID();
			}

			// Some objects, such as the global document type map, have non-existent parent
			// folder IDs of 0.  We need to ignore these, as they will cause a server error.
			if (nParentFolderID > 0)
			{
				// 0 = false = open object unlocked
				IPTAdminFolder parentFolder = m_ptCatalog.OpenAdminFolder(nParentFolderID, false);
	
				Object[] oIDs = parentFolder.GetParentIDs();
				TreeLeafNode currentNode;
	
				// Open the folders up to the parent folder
				for (int i = 0; i < oIDs.length; i++)
				{
					nParentFolderID = XPConvert.ToInteger(oIDs[i]);
	
					// GetParentIDs returns 0 as the first ID, which we should ignore.
					if (nParentFolderID >= PT_INTRINSICS.PT_ADMIN_FOLDER_ROOTFOLDER)
					{
						currentNode = m_Root.GetNode(PT_CLASSIDS.PT_ADMIN_FOLDER_ID, nParentFolderID);
	
						if (currentNode != null)
						{
							// Note: if user doesn't have the correct security on a parent folder
							// then it will not show up when you expand.
							ExpandNode(true, currentNode.m_nTreeNodeID);
						} else
						{
							
								log.Error("Unable to pre-expand folder " + nParentFolderID);
							
						}
					}
				}
			}
		}
	}

	/**
	 * This helper method makes sure that the tree and the selected items list
	 * conform to the hidden objects list.  This method should only be called
	 * when the hidden objects lists have changed.
	 */
	public void RefreshHiddenObjects()
	{
		if (CheckIsHidden(m_Root))
		{
			m_Root = null;
			m_xpmSelectedItems.RemoveElement(GetKeyFromNode(m_Root));

			return;
		}

		CheckForHiddenChildren(m_Root);
	}

	/**
	 * This helper method makes sure that the children of this node and the
	 * selected items list conform to the hidden objects list.
	 */
	private void CheckForHiddenChildren(TreeCompositeNode root)
	{
		IXPEnumerator compositeChildren = root.GetCompositeChildIterator();

		TreeCompositeNode tempComposite;

		while (compositeChildren.MoveNext())
		{
			tempComposite = (TreeCompositeNode) compositeChildren.GetCurrent();

			if (CheckIsHidden(tempComposite))
			{
				root.RemoveCompositeChild(tempComposite);
				m_xpmSelectedItems.RemoveElement(GetKeyFromNode(tempComposite));

				// Since we have removed a node, we cannot continue using
				// this iterator, so we redo this method call
				CheckForHiddenChildren(root);
				return;
			} else
			{
				CheckForHiddenChildren(tempComposite);
			}
		}

		IXPEnumerator leafChildren = root.GetLeafChildIterator();

		TreeLeafNode tempLeaf;

		while (leafChildren.MoveNext())
		{
			tempLeaf = (TreeLeafNode) leafChildren.GetCurrent();

			if (CheckIsHidden(tempLeaf))
			{
				root.RemoveLeafChild(tempLeaf);
				m_xpmSelectedItems.RemoveElement(GetKeyFromNode(tempLeaf));

				// Since we have removed a node, we cannot continue using
				// this iterator, so we redo this method call
				CheckForHiddenChildren(root);
				return;
			}
		}
	}

	/** This helper method determines if a node is hidden.  If it
	 * is hidden, it needs to be removed from the tree. */
	public boolean CheckIsHidden(TreeLeafNode node)
	{
		ObjectManagerHolder omHolder = null;

		for (int j = 0; j < m_ObjMgrs.GetSize(); j++)
		{
			omHolder = (ObjectManagerHolder)m_ObjMgrs.GetElement(j);
			
			if (omHolder.m_nManagedClassID == node.m_nClassID)
			{
				break;
			}

			omHolder = null;
		}

		if (omHolder != null)
		{
			return omHolder.HiddenObject(node.m_nObjectID);
		}

		return false;
	}

	/**
	 * This helper method determines if two nodes in the flat list
	 * have the same names, and then gets appends their folder name
	 * if they are identical.
	 * 
	 * @param TreeLeafNode the first node (may be null)
	 * @param String the original name of the first node (in case
	 * it has already had the folder name appended to it. (may be null)
	 * @param TreeLeafNode the second node
	 * @param String the original name of the second node (in case
	 * it would be truncated by the node)
	 */
	public void CheckForIdenticalNames(TreeLeafNode firstNode,
		String strFirstNodeName, TreeLeafNode secondNode, String strSecondNodeName)
	{
		if ((firstNode == null) || (strFirstNodeName == null))
		{
			// This happens for the first node in the list, since there is no
			// previous node.
			return;
		}

		if (secondNode == null)
		{
			
				log.Error("Second node null while checking for identical node names.  First node name is "
					+ strFirstNodeName);
			

			return;
		}

		if (strSecondNodeName == null)
		{
			
				log.Error("Second node name null while checking for identical node names.  First node name is "
					+ strFirstNodeName);
			

			return;
		}

		if (strFirstNodeName.equalsIgnoreCase(strSecondNodeName))
		{
			if (firstNode.GetName().equalsIgnoreCase(strFirstNodeName))
			{
				// Only append the folder name if it has not already been appended.
				AppendFolderToNodeName(firstNode);
			}
			AppendFolderToNodeName(secondNode);
		}
	}

	/**
	 * This helper method appends the folder name to the name of the
	 * tree node and the full path to the description.
	 * 
	 * @param TreeLeafNode the node to append to.
	 */
	public void AppendFolderToNodeName(TreeLeafNode node)
	{
		ObjectManagerHolder omHolder = null;

		int nClassID = node.m_nClassID;

		for (int j = 0; j < m_ObjMgrs.GetSize(); j++)
		{
			omHolder = (ObjectManagerHolder)m_ObjMgrs.GetElement(j);
			
			if (omHolder.m_nManagedClassID == nClassID)
			{
				break;
			}

			omHolder = null;
		}

		if (omHolder == null)
		{
			
				log.Error("AppendFolderToNodeName unable to find object manager for class ID "
					+ nClassID);
			

			return;
		}

		String strFolderName;
		String strFolderPath;

		try
		{
			// 0 = false = open object unlocked
			IPTObject temp = (IPTObject) omHolder.m_ptObjMgr.Open(node.m_nObjectID, false);
			int nParentFolderID = temp.GetAdminFolderID();
	
			IPTAdminFolder parentFolder = m_ptCatalog.OpenAdminFolder(nParentFolderID, false);
			strFolderName = parentFolder.GetName();
			strFolderPath = parentFolder.GetPath();
		} catch (Exception e)
		{
			
				log.Error(e, "AppendFolderToNodeName unable to get parent folder name for node "
					+ node.GetName());
			

			strFolderName = null;
			strFolderPath = null;
		}

		if ((strFolderName != null) && (!"".equals(strFolderName)))
		{
			// We must be careful not to double HTML encode things.

			// node.GetName() is unencoded
			node.SetName(node.GetName() + " (" + strFolderName + ")");
			// but node.GetDescription is encoded
			node.SetDescription(XPHttpUtility.HtmlDecode(node.GetDescription()) + " (" + strFolderPath + ")");
		}
	}

	/**
	 * This method determines whether or not to disable a
	 * folder based upon the community mode and ID.
	 * 
	 * @param TreeCompositeNode Folder to check.
	 */
	public void CheckDisabledCommunity(TreeCompositeNode myFolder)
	{
		switch (m_nCommunityMode)
		{
		case AdminTreeModel.NOCOMMUNITIES:
			myFolder.m_bDisabled = true;

			break;

		case AdminTreeModel.ALLCOMMUNITIES:

			// default is not disabled
			break;

		case AdminTreeModel.THISCOMMUNITY:

			myFolder.m_bDisabled = true;

			for (int i = 0; i < m_nxpCommunityFolderIDs.GetSize(); i++)
			{
				if (myFolder.m_nObjectID ==
					XPConvert.ToInteger(m_nxpCommunityFolderIDs.GetElement(i)))
				{
					myFolder.m_bDisabled = false;
				}
				
			}

			break;
		}
	}

	/**
	 * Helper class to wrap object managers
	 *
	 * @author Don Hayler
	 */
	public static class ObjectManagerHolder
	{
		
		public IPTObjectManager m_ptObjMgr;

		
		public int[] m_nHiddenObjectIDs;

		
		public int[] m_nDisplayedObjectIDs;

		
		public Object[][] m_ptqf;

		
		public int m_nManagedClassID = -1;

		/**
		 * Creates a new ObjectManagerHolder object.
		 *
		 * @param ptObjMgr 
		 */
		public ObjectManagerHolder(IPTObjectManager ptObjMgr)
		{
			m_ptObjMgr = ptObjMgr;

			if (m_ptObjMgr != null)
			{
				m_nManagedClassID = m_ptObjMgr.GetManagedClassID();
			}

			m_nHiddenObjectIDs     = null;
			m_nDisplayedObjectIDs  = null;
			m_ptqf				   = null;
		}

		/**
		 * This returns whether or not the object should be hidden
		 *
		 * @param nID nID the object ID of the object to check
		 *
		 * @return boolean true implies hidden, false implies visible
		 */
		public boolean HiddenObject(int nID)
		{
			if (m_nHiddenObjectIDs != null)
			{
				for (int i = 0; i < m_nHiddenObjectIDs.length; i++)
				{
					if (nID == m_nHiddenObjectIDs[i])
					{
						return true;
					}
				}
			}

			return false;
		}
	}

	/**
	 * This method calculates the read only query filter.
	 * 
	 * @param int the access level required by the tree
	 * @return the qf.
	 */
	protected Object[][] CalculateFolderSelectQueryFilter(int nAccessLevel)
	{
		// This implementation will break if a new access level is
		// added because of the way the IN filter operator works.

		Object[][] result = null;

		if (nAccessLevel >= PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT)
		{
			Object[] nLevels = null;
	
			switch (nAccessLevel) {
				case PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT:
	
					nLevels = new Object[3];
					nLevels[0] = new Integer(PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT);
					nLevels[1] = new Integer(PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_EDIT);
					nLevels[2] = new Integer(PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_ADMIN);

	
					break;
			
				case PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_EDIT:
	
					nLevels = new Object[2];
					nLevels[0] = new Integer(PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_EDIT);
					nLevels[1] = new Integer(PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_ADMIN);

					break;
			
				case PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_ADMIN:
	
					nLevels = new Object[1];
					nLevels[0] = new Integer(PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_ADMIN);
	
					break;
			
				default:
					// Unknown access level.  Ignore.
					break;
			}

			if (nLevels != null)
			{
				result = PlumtreeHelpers.GetQueryFilter(PT_PROPIDS.PT_PROPID_ACCESSLEVEL,
					PT_FILTEROPS.PT_FILTEROP_IN, nLevels);
			}
		}

		return result;
	}
}

