package com.plumtree.portaluiinfrastructure.tree;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.classtypedescriptors.*;
import com.plumtree.portaluiinfrastructure.login.*;
import com.plumtree.portaluiinfrastructure.search.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.htmlconstructs.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.tree.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * Generate a search box for the tree page and manage some hidden inputs
 * and javascript regardless of which page we are on.
 * 
 * @author Don Hayler
 */
public class TreeSearchBoxView implements IView
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.tree.TreeSearchBoxView");

	/** name of this object */
	public static final String STR_MVC_CLASS_NAME = "TreeSearchBoxView";

	/** constants */
	public static final String JAVASCRIPT_START_SEARCH = "submitSearch";

	// The following constant is defined by Search API:
	// TODO: Should com.plumtree.server.impl.search.legacy.LegacySearchRequest 
	// (or some other Search interface) expose this constant, so that we don't 
	// have to re-define it here?
	private final String USER_TYPE_GUEST = "guest";
	private final String USER_TYPE_REGULAR = "regular";

	/** internal data */
	protected AActivitySpace m_asOwner;
	protected IObjectTreeModelRO m_asModel;
	protected String _singleClassFilterJavascript = "";

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new TreeSearchBoxView();
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.activityspace.IView#Display()
	 */
	public HTMLElement Display()
	{
		HTMLElement result = null;

		boolean bSearchEnabled = m_asModel.GetSearchEnabled();
		boolean bSearchResults = m_asModel.GetDisplayingSearchResults();

		if ((bSearchEnabled) && (!bSearchResults))
		{
			try
			{
				// Get the search box if we're in tree mode
				result = makeObjectSearchForm();
			}
			catch (HTMLException he)
			{
				
					log.Error(he, "Unable to create search box form elements.");
				

				result = null;
			}
		}

		if (result == null)
		{
			// still need to display the hidden input if we're in search mode
			// or the search box failed
			result = new HTMLElementCollection();
		}

		try
		{
			// The hidden input for the view type of the tree (tree vs. search)
			HTMLInput myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
					TreeWithSearchDP.HTMLINPUT_DISPLAYING_SEARCH, "");
			result.AddInnerHTMLElement(myInput);
			myInput.SetValue(XPConvert.ToString(bSearchResults));
		}
		catch (HTMLException he)
		{
			
				log.Error(he, "Unable to create hidden view type input.");
			
		}

		return result;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.activityspace.IView#DisplayJavascript()
	 *
	 * Warning: this can return null Javascript, which must be checked for before adding
	 * to an HTMLScriptCollection.
	 */
	public HTMLScript DisplayJavascript()
	{
		// Can't guarantee order of Display() vs. DisplayJavascript() so have to do this.
		if(MustPickSingleSearchClass() && "".equals(_singleClassFilterJavascript))
			makeObjectSearchForm();
		if (m_asModel.GetDisplayingSearchResults())
		{
			// No javascript to display if we are in search mode already
			return null;
		}

		XPStringBuilder sb = new XPStringBuilder(150);
		sb.Append("\n");

		//  start search / switch view type function
		sb.Append("function ").Append(JAVASCRIPT_START_SEARCH).Append("()\n");
		sb.Append("{\n");

		// Switch view type to search
		sb.Append("  document.").Append(AFormDP.HTMLFORM_NAME).Append(".")
		.Append(TreeWithSearchDP.HTMLINPUT_DISPLAYING_SEARCH).Append(".value = 'true';\n");

		sb.Append(_singleClassFilterJavascript);
		_singleClassFilterJavascript = "";
		sb.Append("  return true;\n");
		sb.Append("}\n");
		sb.Append("\n");

		HTMLScript result = new HTMLScript("text/javaScript");

		try
		{
			result.AddInnerHTMLString(sb.ToString());
		}
		catch (HTMLException he)
		{
			
				log.Error(he, "DisplayJavascript couldn't add javascript to the HTMLScript element.");
			

			return null;
		}

		return result;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.activityspace.IView#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.activityspace.IView#Init(IModelRO, AActivitySpace)
	 */
	public void Init(IModelRO model, AActivitySpace parent)
	{
		m_asModel     = (IObjectTreeModelRO) model;
		m_asOwner     = parent;
	}

	/**
	 * 
	 *
	 * @return 
	 *
	 * @throws HTMLException 
	 */
	private HTMLElement makeObjectSearchForm()

	{
		// Some search forms have optional constraints that a user can choose via
		// a pulldown or set of radio button.  This example shows how to create such
		// a form (in this case, a form that searches for gadgets, crawlers, or all
		// objects, in a single folder, a folder plus its subfolders, or all folders).
		// Make the factory with a unique name
		// false means to add this to a larger form
		SearchFormFactory factory = new SearchFormFactory(AFormDP.HTMLFORM_NAME,
				AFormDP.HTMLFORM_NAME, m_asOwner, false);

		// Don't set the search form's target, as we are using our own form.
		// Set the javascript validation function
		factory.SetValidationFunction(JAVASCRIPT_START_SEARCH);

		// Tell the factory to pop up an alert if the user doesn't type anything in the box.
		// This would also pop up if the validation function (JAVASCRIPT_START_SEARCH) returned
		// false, but in this case it doesn't.
		factory.SetValidateAlert(true);

		// Add required constraints that should be used for ALL searches.
		// In this case, we only want portal objects:
		factory.AddRequiredSetting(PT_SEARCH_SETTING.PT_SEARCHSETTING_APPS,
			PT_SEARCH_APPS.PT_SEARCH_APPS_PORTAL);

		// Portlet stuff (templates, wide, etc...)
		int[] nPortletValues = m_asModel.GetPortletType();

		if (nPortletValues.length > 0)
		{
			factory.AddRequiredSetting(PT_SEARCH_SETTING.PT_SEARCHSETTING_GADGET_TYPE,
				nPortletValues);
		}

		nPortletValues = m_asModel.GetPortletAlignment();

		if (nPortletValues.length > 0)
		{
			factory.AddRequiredSetting(PT_SEARCH_SETTING.PT_SEARCHSETTING_GADGET_ALIGNMENT,
				nPortletValues);
		}

		int[] nUserTypes = m_asModel.GetUserTypeFilter();

		if ((null != nUserTypes) && (nUserTypes.length > 0))
		{
			// Note: only the first entry in the array will be used as filter, as usertypes
			// can only have 2 types (if the array contains more than 1 value, then it is
			// not really a filter).
			String strUserType = null;
			if (PT_USER_TYPES.PT_USER_TYPE_GUEST == nUserTypes[0])
			{
				strUserType = USER_TYPE_GUEST;
			}
			else
			{
				strUserType = USER_TYPE_REGULAR;
			}
			
			factory.AddRequiredSetting(PT_SEARCH_SETTING.PT_SEARCHSETTING_USER_IS_GUEST,
				strUserType);
		}
		
		// NOTE If we don't need the filter prop ID, we should remove this and the method
		// and member variables associated with it.
//		int nFilterPropID = m_asModel.GetFilterCLSIDPropID();
		String[] sFilterCLSIDs = m_asModel.GetFilterCLSID();

		if (sFilterCLSIDs.length > 0)
		{
				
			for (int i = 0; i < sFilterCLSIDs.length; i++)
			{
				factory.AddFilterStatement(PT_INTRINSICS.PT_PROPERTY_PROVIDERCLSID,
					PT_FILTEROPS.PT_FILTEROP_CONTAINS, sFilterCLSIDs[i]);
			}

			if (sFilterCLSIDs.length > 1)
			{
				factory.SetFilterClauseOperator(PT_BOOLOPS.PT_BOOLOP_OR);
			}
		}

		// Add constraint for which folder to search in
		int nRootID = m_asModel.GetRootTreeIterator().GetBaseObjectID();

		if (nRootID == -1)
		{
			// NOTE: There is no folder to search in when we are browsing
			// groups.  Currently we just browse everything
			nRootID = PT_INTRINSICS.PT_ADMIN_FOLDER_ROOTFOLDER;
		}

		factory.AddRequiredSetting(PT_SEARCH_SETTING.PT_SEARCHSETTING_INCLUDE_SUBFOLDERS, true);

		// Add pulldown to select object type to search on
		HTMLSelect typeSelect = null;
		int[] nIDs			  = GetSearchableClassIDs(m_asModel.GetDisplayedClassIDs());
		int nFolderClassID	  = -1;
		int nMode			  = m_asModel.GetSelectMode();

		// Determine what kind of folder ID we are using.
		if ((nMode == TreeConstants.N_COMPOSITESELECT) ||
				(nMode == TreeConstants.N_LEAFCOMPOSITESELECT))
		{
			nFolderClassID = m_asModel.GetFolderClassID();

			// TODOBUG: if person is just browsing users and groups, but not folders, they 
			// don't want this to work.  D'oh!
		}

		// Search handles documents slightly differently from admin objects, so we have
		// to start the search differently
		boolean bDocumentDirectory = false;

		// Document folders now have full security, but documents do not.
		boolean bDocumentFoldersOnly = true;

		// Determine if we are searching for documents or document folders.
		for (int i = 0; i < nIDs.length; i++)
		{
			if ((nIDs[i] == PT_CLASSIDS.PT_CATALOGFOLDER_ID))
			{
				bDocumentDirectory = true;
			}
			if ((nIDs[i] == PT_CLASSIDS.PT_CATALOGCARD_ID))
			{
				bDocumentDirectory = true;
				bDocumentFoldersOnly = false;
			}
		}

		if (nFolderClassID == PT_CLASSIDS.PT_CATALOGFOLDER_ID)
		{
			bDocumentDirectory = true;
		}

		if (bDocumentDirectory)
		{
			// Do not set the security level for document directory.  Document
			// security is handled differently in search.  They only index read
			// access, so there is no need to specify the setting. This is true
			// for both documents and document folders (cards and catalogs).
		    // note: this has been partially fixed for 6.0
			// When bug#46727 is fixed, this if statement should be removed and
			// the security setting should be set for all objects (admin & doc).
		    if (bDocumentFoldersOnly)
		    {
		    	// This has only been fixed for document folders, not documents.
				factory.AddRequiredSetting(PT_SEARCH_SETTING.PT_SEARCHSETTING_ACCESS_LEVEL,
					m_asModel.GetAccessLevel());
		    }

			// Set dd folders for directory browsing (cards & catalogs)
			factory.AddRequiredSetting(PT_SEARCH_SETTING.PT_SEARCHSETTING_DDFOLDERS,
				new int[] { nRootID });
			
		} else
		{
			// Security level -- only for admin directory searches
			factory.AddRequiredSetting(PT_SEARCH_SETTING.PT_SEARCHSETTING_ACCESS_LEVEL,
				m_asModel.GetAccessLevel());

			// Set admin folders for admin browsing
			// For bug 42506: include default profile folder
			factory.AddRequiredSetting(PT_SEARCH_SETTING.PT_SEARCHSETTING_ADMINFOLDERS,
				new int[] { nRootID, PT_INTRINSICS.PT_ADMIN_FOLDER_DEFAULTPROFILES });
		}

		if ((nIDs.length == 1) && (nMode == TreeConstants.N_LEAFSELECT))
		{
			// Just search on the one item, no display
			factory.AddRequiredSetting(PT_SEARCH_SETTING.PT_SEARCHSETTING_OBJTYPES, nIDs);
		}
		else if ((nMode == TreeConstants.N_COMPOSITESELECT) ||
			((nMode == TreeConstants.N_LEAFCOMPOSITESELECT) &&
			(nIDs.length == 0)))
		{
			// If we are only selecting composite nodes (what about browseable
			// groups in groups?), or we are selecting both, but have no leaf
			// nodes, just search on the folder (either admin or document)
			// type, no display.
			factory.AddRequiredSetting(PT_SEARCH_SETTING.PT_SEARCHSETTING_OBJTYPES,
				new int[] { nFolderClassID });
		}
		else
		{
			// Allow the user to choose what classes to search for
			// Create the SELECT element.  You MUST use the return value of
			// factory.GetNextSelectName() as its name.
			typeSelect = new HTMLSelect(factory.GetNextSelectName(), "");

			// First, the default "all" option.  This option constrains to the class
			// IDs shown in the tree.
			if(! MustPickSingleSearchClass())
			{
				HTMLOption optionAll = new HTMLOption("all");

				optionAll.AddInnerHTMLString(m_asOwner.GetString(295, "ptmsgs_portalinfrastructure"));
				// Add the "all" option directly to the select so it will be listed at the top
				typeSelect.AddInnerHTMLElement(optionAll);
				optionAll.SetSelected(true);
			}
			// Skip the 'All' option if we don't need it.
			if(! MustPickSingleSearchClass())
				if (nFolderClassID == -1)
				{
					factory.AddOptionalSetting("all", // same as HTMLOption value!!
						PT_SEARCH_SETTING.PT_SEARCHSETTING_OBJTYPES, nIDs);
				}
				else
				{
					// Need to create a new int[] with the folder ID in it.
					int[] nTemp = new int[nIDs.length + 1];
					nTemp[0] = nFolderClassID;
	
					for (int i = 1; i < nTemp.length; i++)
					{
						nTemp[i] = nIDs[i - 1];
					}
	
					factory.AddOptionalSetting("all", // same as HTMLOption value!!
						PT_SEARCH_SETTING.PT_SEARCHSETTING_OBJTYPES, nTemp);
				}

			// Next add options to the SELECT.  As you add options, inform the
			// factory of the settings that correspond to each option:
			HTMLOption myOption;

			// Add the options to an HTMLElementCollection so they will appear sorted
			HTMLElementCollection sortedOptions = new HTMLElementCollection();
			typeSelect.AddInnerHTMLElement(sortedOptions);

			if (nFolderClassID != -1)
			{
				myOption = new HTMLOption(m_asOwner.GetString(262,
							"ptmsgs_portalinfrastructure"));

				myOption.AddInnerHTMLString(m_asOwner.GetString(262,
						"ptmsgs_portalinfrastructure"));
				sortedOptions.AddInnerHTMLElement(myOption);
				factory.AddOptionalSetting(m_asOwner.GetString(262,
						"ptmsgs_portalinfrastructure"), // same as HTMLOption value!!
					PT_SEARCH_SETTING.PT_SEARCHSETTING_OBJTYPES,
					new int[] { nFolderClassID });
			}

			String strClassName;
			String strOptionName;

			String strLangID = (String) PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingsHelper.LANGUAGE, m_asOwner);
			ClassTypeDescManager ptClassTypeDescManager = PTConfigHelper.GetPTClassTypeDescrManager();

			for (int i = 0; i < nIDs.length; i++)
			{
				strClassName     = ptClassTypeDescManager.GetClassTypeLocalizedName(nIDs[i], strLangID);

				// We use the class ID rather than the localized name because the
				// localized name can contain illegal characters for an HTMLElement ID.
				strOptionName    = "ClassID" + nIDs[i];

				myOption = new HTMLOption(strOptionName);
				myOption.AddInnerHTMLString(strClassName);
				sortedOptions.AddInnerHTMLElement(myOption);
				factory.AddOptionalSetting(strOptionName, // same as HTMLOption value!!
					PT_SEARCH_SETTING.PT_SEARCHSETTING_OBJTYPES,
					new int[] { nIDs[i] });
			}
		}
		// Load the javascript for the selective filter usage here.
		if(MustPickSingleSearchClass())
		{
			_singleClassFilterJavascript = factory.GetSingleClassFilterJavascript("ClassID" + m_asModel.GetFilterClassID());
		}	
		// Finally, get the form from the factory and add the pulldown to it,
		// along with whatever other HTML elements you want.  This part is
		// basically the same as the first example.
		HTMLElementCollection result = (HTMLElementCollection) factory.GetResult();

		HTMLTable table;
		HTMLTableRow row;
		HTMLTableCell cell;

		// Search table
		table = new HTMLTable();
		result.AddInnerHTMLElement(table);
		table.SetCellSpacing(CommonHTMLStrings.ZERO);
		table.SetCellPadding(CommonHTMLStrings.ZERO);
		table.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
		table.SetBorder(CommonHTMLStrings.ZERO);

		row = new HTMLTableRow();
		table.AddInnerHTMLElement(row);
		row.SetStyleClass(PTStyleClass.DIALOGUE_SEARCH_BG);

		// text
		cell = new HTMLTableCell();
		row.AddInnerHTMLElement(cell);
		cell.SetHeight("25");
		cell.SetVAlign(CommonHTMLStrings.MIDDLE);
		cell.SetStyleClass(PTStyleClass.OBJECTSEARCH_TEXT);
		cell.SetNoWrap(true);

		HTMLSpan mySpan = new HTMLSpan();
		cell.AddInnerHTMLElement(mySpan);
		mySpan.SetStyleClass(PTStyleClass.OBJECTSEARCH_HEADER);

		mySpan.AddInnerHTMLString(m_asOwner.GetString(261,
				"ptmsgs_portalinfrastructure"));
		mySpan.AddInnerHTMLString("&nbsp;&nbsp;&nbsp;");

		// Input box
		cell = new HTMLTableCell();
		row.AddInnerHTMLElement(cell);
		cell.SetHeight("25");
		cell.SetVAlign(CommonHTMLStrings.MIDDLE);
		cell.SetStyleClass(PTStyleClass.OBJECTSEARCH_TEXT);
		cell.SetNoWrap(true);
		
		
//		HTMLInput input = new HTMLInput(HTMLInputTypes.TEXT,
//				factory.GetInputBoxName(), "myID");
//		
		HTMLInput input = new HTMLInput(HTMLInputTypes.TEXT_TYPE, "");
		input.SetValidateName(false);
		input.SetName(factory.GetInputBoxName());
		input.SetID("myID");
		input.SetOnKeyDown(factory.GetInputBoxOnKeyDown());
		input.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);
		cell.AddInnerHTMLElement(input);

		// Object type select
		if (typeSelect != null)
		{
			cell = new HTMLTableCell();
			row.AddInnerHTMLElement(cell);
			cell.SetHeight("25");
			cell.SetVAlign(CommonHTMLStrings.MIDDLE);
			cell.SetStyleClass(PTStyleClass.OBJECTSEARCH_TEXT);
			cell.SetNoWrap(true);
			typeSelect.SetStyleClass(PTStyleClass.FORM_SELECT_TEXT);
			cell.AddInnerHTMLElement(typeSelect);
		}

		// Add submit button
		cell = new HTMLTableCell();
		row.AddInnerHTMLElement(cell);
		cell.SetHeight("25");
		cell.SetVAlign(CommonHTMLStrings.MIDDLE);
		cell.SetStyleClass(PTStyleClass.OBJECTSEARCH_TEXT);
		cell.SetNoWrap(true);

		HTMLAnchor submitLink = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
		submitLink.SetOnClick(factory.GetAnchorOnClick());
		submitLink.SetPreComment("Search");

		PTConstruct searchButton = null;

		try
		{
			searchButton = UIConstructFactory.GetConstruct(PTImageType.ACTION_SEARCH_BUTTON,
					true, this.m_asOwner);
		}
		catch (Exception e)
		{
			
				log.Error(e, "Unable to get search button.");
			
		}

		if (null != searchButton)
		{
			HTMLImg searchButtonImg = (HTMLImg) searchButton.GetHTMLElement();

			searchButtonImg.SetAlt(m_asOwner.GetString(261,
					"ptmsgs_portalinfrastructure"));
      //			searchButtonImg.SetWidth("16");
      //			searchButtonImg.SetHeight("16");
			submitLink.AddInnerHTMLElement(searchButtonImg);
		}
		else
		{
			submitLink.AddInnerHTMLString(m_asOwner.GetString(261,
					"ptmsgs_portalinfrastructure"));
		}

		cell.AddInnerHTMLElement(submitLink);

		// Spacer cell at end
		cell = new HTMLTableCell();
		row.AddInnerHTMLElement(cell);
		cell.SetHeight("25");
		cell.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
		cell.SetNoWrap(true);

		// spacer table
		table = new HTMLTable();
		result.AddInnerHTMLElement(table);
		table.SetCellSpacing(CommonHTMLStrings.ZERO);
		table.SetCellPadding(CommonHTMLStrings.ZERO);
		table.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
		table.SetBorder(CommonHTMLStrings.ZERO);

		row = new HTMLTableRow();
		table.AddInnerHTMLElement(row);

		cell = new HTMLTableCell();
		row.AddInnerHTMLElement(cell);

		try
		{
			PTConstruct myConstruct = UIConstructFactory.GetConstruct(PTImageType.SPACER);
			HTMLImg myImg		    = (HTMLImg) myConstruct.GetHTMLElement();
			myImg.SetHeight("10");
			cell.AddInnerHTMLElement(myImg);
		}
		catch (Exception e)
		{
			
				log.Error(e, "Unable to create spacer image.");
			
		}

		return result;
	}

	/**
	 * This helper method removes non-searchable class IDs from the
	 * list of IDs to search.
	 * 
	 * @param int[] the list of IDs to search
	 * @result int[] the list of IDs to search without unsafe class IDs
	 */
	private static int[] GetSearchableClassIDs(int[] nAllIDs)
	{
		int nUnSearchableIDs = 0;
		for (int i = 0; i < nAllIDs.length; i++)
		{
			if (nAllIDs[i] == PT_CLASSIDS.PT_TAXONOMIST_ID)
			{
				nUnSearchableIDs++;
			}
		}

		if (nUnSearchableIDs == 0)
		{
			return nAllIDs;
		}

		int[] nResultIDs = new int[nAllIDs.length - nUnSearchableIDs];
		int nResultIndex = 0;

		for (int i = 0; i < nAllIDs.length; i++)
		{
			if (nResultIndex < nResultIDs.length)
			{
				if (nAllIDs[i] != PT_CLASSIDS.PT_TAXONOMIST_ID)
				{
					nResultIDs[nResultIndex] = nAllIDs[i];
					nResultIndex++;
				}
			}
		}

		return nResultIDs;
	}
	
	protected boolean MustPickSingleSearchClass()
	{
		String[] sFilterCLSIDs = m_asModel.GetFilterCLSID();
		return (sFilterCLSIDs != null && sFilterCLSIDs.length > 0 &&
				m_asModel.GetDisplayedClassIDs().length > 1);
	}
}

