package com.plumtree.portaluiinfrastructure.mock.tags;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.server.*;

/**
 * Helper class that provides an object oriented way to generate markup fragment.
 * This helper only supports adaptive tag and pure content (strings or html) markup elements. 
 * 
 * @author rozhang
 */
public class XMLFragmentTestHelper 
{
	protected static final String ATTRIB_HTML = "h";
	protected static final String ATTRIB_PT = "pt";
	
	/** negative values so they aren't confused with the markup types **/
	public static final int TAGTYPE_HTML = -100;
	public static final int TAGTYPE_ADAPTIVETAG = -200;
	
	public static final Integer[] MARKUP_TYPE_MAP = 
		new Integer[] {	
					new Integer(PT_MARKUP_FRAGMENT_TYPES.PT_MARKUP_FRAGMENT_TYPE_FINISHED),
					new Integer(PT_MARKUP_FRAGMENT_TYPES.PT_MARKUP_FRAGMENT_TYPE_PT_XML)
				  };
	
	/** shorter, simpler and clearerconstant definitions **/
	protected static final int TYPECOL = PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_COLS_TYPE;
	protected static final int NAMECOL = PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_COLS_FRAGMENT;
	protected static final int VALUECOL = PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_NUM_COLS;
	
	protected int m_tagType = 0;
	protected String m_libName = null;
	protected String m_tagName = null;
	
	/** 
	 * Array list to store content items (nested tag inside a tag body)
	 * It is also used to store a list of processed fragments when creating sequential elements
	 * with a list of fragmenttesthelpers.  
	 * This could get a little confusing since it used to store both processed fragments and
	 * raw fragments and they will have different types (either the internal tag type or the
	 * xml fragment type).
	 **/
	protected XPArrayList m_content = new XPArrayList();
	protected XPArrayList m_attribs = new XPArrayList();
	
	/**
	 * Constructor for Adaptive Tags
	 * 
	 * @param libName
	 * @param tagName
	 */
	public XMLFragmentTestHelper(String libName, String tagName)
	{
		m_tagType = TAGTYPE_ADAPTIVETAG;
		m_libName = libName;
		m_tagName = tagName;
	}
	
	/**
	 * Constructor for HTML markup entry
	 * 
	 * @param fragment
	 */
	public XMLFragmentTestHelper(XMLFragmentTestHelper fragment)
	{
		// for html fragement, the html is stored in the content array list
		m_tagType = TAGTYPE_HTML;
		// store the processed fragment into the content array, more might get added later
		m_content.Add(fragment.ToArrayEntireFragment());
	}
	
	/**
	 * Use this helper as a container for a list of tag fragments created with the
	 * helper
	 * 
	 * @param fragments
	 */
	public XMLFragmentTestHelper(XMLFragmentTestHelper[] fragments)
	{
		// html type since this is just a container for other tags
		m_tagType = TAGTYPE_HTML;
		for ( int i = 0; i < fragments.length; i++ ) {
			// store the internal tag type (not the xml tag type) in the first element
			// and the processed fragment in the second element
			m_content.Add(new Object[] {new Integer(fragments[i].GetTagType()), fragments[i].ToArrayEntireFragment()});
		}
		
	}
	
	/**
	 * Constructor for HTML markup entry
	 * @TODO what kind of object[][] do we want to support? full markup fragment or
	 * just adaptive tag attribute array? 
	 * @param fragment
	 */
//	public XMLFragmentTestHelper(Object[][] fragment)
//	{
//		m_tagType = TAGTYPE_ADAPTIVETAG;
//		m_content.Add(fragment);
//	}
	
	/**
	 * Add tag body content
	 * @param fragment
	 */
	public void AddContent(XMLFragmentTestHelper fragment)
	{
		m_content.Add(new Object[] {new Integer(fragment.GetTagType()), fragment.ToArrayEntireFragment()});
	}
	
	/**
	 * Add tag body content
	 * 
	 * @param fragmentType
	 * @param bodyFragment	Markup fragment (with markup type in the first column)
	 */
	public void AddContent(int fragmentType, Object[][] bodyFragment)
	{
		m_content.Add(new Object[] {new Integer(fragmentType), bodyFragment});
	}
	
	/**
	 * Add HTML tag body content 
	 * @param content
	 */
	public void AddContent(String content)
	{
		m_content.Add(new Object[] {new Integer(PT_MARKUP_FRAGMENT_TYPES.PT_MARKUP_FRAGMENT_TYPE_FINISHED), content});
	}
	
	/**
	 * Add opener HTML tag attribute
	 * @param attribName
	 * @param attribValue
	 */
	public void AddAttribute(String attribName, String attribValue)
	{
		m_attribs.Add(new String[] {ATTRIB_HTML, attribName, attribValue});
	}
	
	/**
	 * Add opener PT tag attribute (for adaptive tags)
	 * @param attribName
	 * @param attribValue
	 */
	public void AddPTAttribute(String attribName, String attribValue)
	{
		m_attribs.Add(new String[] {ATTRIB_PT, attribName, attribValue});
	}
	
	/**
	 * Generate the markup fragment value opener or begin tag.
	 * 
	 * Only the markup fragment value is returned, not the full markup row with the type. 
	 * 
	 * Note that this is only applicable for adaptive tag fragments, not fragment with
	 * just html content.  
	 * @return
	 */
	public Object[][] ToArrayOpenTag()
	{
		// for the adaptive tag attribute array we need elements for all the attributes + 1 for the tag name
		// and possible one to close the tag if the tag doesnt have a body
		int size = 1 + m_attribs.GetSize() + ( (ClosedTag())? 1:0 ) ;
		// tag data columns for type, name and value 
		Object[][] markup = new Object[3][size];
		
		switch(m_tagType) {
			case TAGTYPE_ADAPTIVETAG:
				// add tag name to markup array
				markup[TYPECOL][0] = new Integer(PT_XML_TYPES.PT_XML_TYPES_PTUI);
				markup[NAMECOL][0] = m_libName+"."+m_tagName;
				markup[VALUECOL][0] = null;
				
				// add arguments, iterate over the argument array and add array data for each argument
				for ( int args = 0; args < m_attribs.GetSize(); args++ ) {
					String[] attribArray = ((String[])m_attribs.GetElement(args)); 
					// add one to the markup index since the name entry occupies the first element
					int markupIndex = args+1;
					// figure out which type to set here, different for tag attribute with pt: and regular attributes
					if ( ATTRIB_PT.equals(attribArray[0]) ) {
						markup[TYPECOL][markupIndex] = new Integer(PT_XML_TYPES.PT_XML_TYPES_PTUI_ATT);
					} else if ( ATTRIB_HTML.equals(attribArray[0]) ) {
						markup[TYPECOL][markupIndex] = new Integer(PT_XML_TYPES.PT_XML_TYPES_XHTML_ATT);
					} else {
						throw new XPIllegalArgumentException("Invalid attribute type passed in: "+attribArray[0]+
								". Refer to the ATTRIB_x constants for valid attribute types.");
					}
					// set the name and value for the attribute 
					markup[NAMECOL][markupIndex] = attribArray[1];
					markup[VALUECOL][markupIndex] = attribArray[2];
				}
				// add a closed attribute if there is no content in the tag
				if ( ClosedTag() ) {
					markup[TYPECOL][size-1] = new Integer(PT_XML_TYPES.PT_XML_TYPES_PTUI_CLOSE);
					markup[NAMECOL][size-1] = null;
					markup[VALUECOL][size-1] = null;
				}
				
			break;
			case TAGTYPE_HTML:
				throw new XPIllegalStateException("Cannot generate an opener tag for HTML Fragment");
		}
		return markup;
	}
	
	/**
	 * Convert the body to an markup array format (where the first column contains the value for the fragment type). 
	 * For HTML fragments, the body is the entire fragment while adaptive tags contain an 2d object array with
	 * info about the tag.
	 * 
	 * @return
	 */
	public Object[][] ToArrayTagBody()
	{
		return ToArrayTagBody(false);
	}
	
	/**
	 * 
	 * @param withOpenCloseEntries If this is used to generate the markup fragment for the entire
	 * tag we want to leave space in the array for the open and close tags  
	 * @return
	 */
	protected Object[][] ToArrayTagBody(boolean withOpenCloseEntries)
	{
		// the markup array needs to have 2 additional elements, one for open and one for close tags
		int additionalElements = (withOpenCloseEntries) ? 2 : 0;
		// don't need a close markup entry if we already closed the open tag (no content exists)
		if ( ClosedTag() ) 
			additionalElements--;
		
		// to leave the first element open for the open tag, add one to the array index 
		int indexOffset = (withOpenCloseEntries) ? 1 : 0;
		// keep two array list to store all the markup entries from the content
		// assembly them into an array later. we need to go through all the content
		// entries first since we don't know how many markup entries each
		// content entry contains
		XPArrayList markupTypeList = new XPArrayList();
		XPArrayList markupFragList = new XPArrayList();
		Object[][] markup = null;
		
			
			
		// don't bother go through the content if there is no body set
		if ( !ClosedTag() ) 
		{
			for ( int i = 0; i < m_content.GetSize(); i++ ) {
				Object[] contentEntry = (Object[])m_content.GetElement(i);
				

				// check what type of content, could be a tag attribute array or just a string
				if ( PT_MARKUP_FRAGMENT_TYPES.PT_MARKUP_FRAGMENT_TYPE_FINISHED == ((Integer)contentEntry[0]).intValue() ) 
				{
					markupTypeList.Add(new Integer(PT_MARKUP_FRAGMENT_TYPES.PT_MARKUP_FRAGMENT_TYPE_FINISHED));
					markupFragList.Add(contentEntry[1]);
				} 
				else
				{
					Object[][] contentMarkup = (Object[][])contentEntry[1];
					// iterate over all the markup entries and add them into the list
					for ( int j = 0; j < contentMarkup[0].length; j++ )
					{
						// a list for the markup type and the markup fragment
						markupTypeList.Add(contentMarkup[PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_COLS_TYPE][j]);
						markupFragList.Add(contentMarkup[PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_COLS_FRAGMENT][j]);
					}
				}
			}
			// ok done with creating the temporary array, now put it all in a nice big 2d array
			markup = new Object[2][markupTypeList.GetSize()+additionalElements];
			for ( int k = 0; k < markupTypeList.GetSize(); k++) {
				markup[PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_COLS_TYPE][k+indexOffset] = markupTypeList.GetElement(k);
				markup[PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_COLS_FRAGMENT][k+indexOffset] = markupFragList.GetElement(k);
			}
			
			
		} else {
			// still need to create a markup array for the caller
			markup = new Object[2][additionalElements];
		}
		return markup;
	}
	
	/**
	 * Generate the markup fragment value opener or begin tag.
	 * 
	 * Only the markup fragment value is returned, not the full markup row with the type. 
	 * 
	 * Note that this is only applicable for adaptive tag fragments, not fragment with
	 * just html content.
	 *  
	 * @return
	 */
	public Object[][] ToArrayTagClose()
	{
		// return a symbolic empty 2d array, this should get used though
		if ( ClosedTag() )
			return null;
		// close markup can only have one entry
		Object[][] markup = new Object[3][1];
		
		switch(m_tagType) 
		{
			case TAGTYPE_ADAPTIVETAG:
				markup[TYPECOL][0] = new Integer(PT_XML_TYPES.PT_XML_TYPES_PTUI_CLOSE);
				markup[NAMECOL][0] = m_libName+"."+m_tagName;
				markup[VALUECOL][0] = null;
				break;
			case TAGTYPE_HTML:
				throw new XPIllegalStateException("Cannot generate a close tag for HTML markup");
		}
		return markup;
	}

	/**
	 * Helper to see if this should be a singleton tag
	 * (tags that are closed in the opener, eg <tag ... />)
	 * @return
	 */
	protected boolean ClosedTag() {
		boolean closedTagEntry = ( m_content.GetSize() == 0 );
		return closedTagEntry;
	}
	
	/**
	 * Generate full markup fragment for this tag with markup fragement type and value columns
	 * Internally calls ToArrayTagOpen, ToArrayTagBody and ToArrayTagClose
	 * 
	 * @return
	 */
	public Object[][] ToArrayEntireFragment()
	{
		Object[][] markup = null;
		switch(m_tagType) 
		{
			case TAGTYPE_ADAPTIVETAG:
				// this creates a markup array with slots for open and close tags at the
				// beginning and end of the array
				markup = ToArrayTagBody(true);
				// now set the open 
				markup[PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_COLS_TYPE][0] = new Integer(PT_MARKUP_FRAGMENT_TYPES.PT_MARKUP_FRAGMENT_TYPE_PT_XML);
				markup[PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_COLS_FRAGMENT][0] = ToArrayOpenTag();
				// don't set a close tag if the begin tag is already closed
				if ( !ClosedTag() ) {
					markup[PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_COLS_TYPE][markup[0].length-1] = new Integer(PT_MARKUP_FRAGMENT_TYPES.PT_MARKUP_FRAGMENT_TYPE_PT_XML);
					markup[PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_COLS_FRAGMENT][markup[0].length-1] = ToArrayTagClose();
				}
			break;
			case TAGTYPE_HTML:
				// no begin or close tag for html content, just the body
				markup = ToArrayTagBody();
			break;
		}
		return markup;
	}

	/**
	 * Tag type
	 * @see TAGTYPE_HTML, TAGTYPE_ADAPTIVETAG
	 * @return
	 */
	public int GetTagType() {
		return m_tagType;
	}

}
