
package com.plumtree.taglib.logic;

import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This tags concatenates two provided values and stores the result
 * in a memory variable with a specified name.
 * 
 * @author robertz
 */
public class ConcatTag extends ATag
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADAPTIVETAGS),
		"com.plumtree.taglib.logic.BooleanExprTag");
	
	public static String CONCAT_EXAMPLE = 
		"<![CDATA[&lt;pt:logic.variable pt:key=\"somepageid\" pt:value=\"2000\"/&gt;]]><![CDATA[<br>]]>" +
		"<![CDATA[&lt;pt:logic.concat pt:key=\"target\" pt:value1=\"pageid\" pt:value2=\"$somepageid\" /&gt;]]><![CDATA[<br>]]>" +
		"<![CDATA[&lt;pt:logic.value pt:value=\"$target\"/&gt;]]><![CDATA[<br>]]>";
		
	
	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;
	public static final RequiredTagAttribute VALUE1;
	public static final RequiredTagAttribute VALUE2;
	public static final OptionalTagAttribute SCOPE;
	public static final RequiredTagAttribute KEY;

	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData("concat",
			"This tag concatenates two values into one and sets the new value  " +
			"in a variable with a specified name.",
			CONCAT_EXAMPLE
			);

		VALUE1 = new RequiredTagAttribute("value1",
			"Value added to the front in the final concatenated value.");
		
		VALUE2 = new RequiredTagAttribute("value2",
			"Value added to the back in the final concatenated value.");

		KEY = new RequiredTagAttribute("key",
			"Name of the resulting concatenated value from value1 and value2.");

		SCOPE = new OptionalTagAttribute("scope",
			"The scope used to store the result in memory.  " + LogicTagLibraryData.SCOPE_REFERENCE,
			AttributeType.STRING, Scope.PORTLET_REQUEST.toString());
	}
	
	public HTMLElement DisplayTag()
	{
		
		// Get the values to concatenate
		String value1 = GetTagAttributeAsString(VALUE1);
		String value2 = GetTagAttributeAsString(VALUE2);
		
		// Perform the concat
		String result = value1 + value2;
		
		// Get the variable name to store the result string
		String varname = GetTagAttributeAsString(KEY);
		
		// Determine the appropriate scope
		Scope scope = Scope.GetScope(GetTagAttributeAsString(SCOPE));
		if (null == scope)
		{
			log.Warn("Unable to convert scope attribute " +
				GetTagAttributeAsString(SCOPE) + " to a valid scope in Tag " +
				TAG.GetName() + ".");

			scope = Scope.PORTLET_REQUEST;
		}

		// Set the resulting variable 
		SetStateSharedVariable(varname, result, scope, false);
		
		return ProcessTagBody();
	}
	
	public boolean SupportsAccessStyle(AccessStyles style)
	{
		// This tag does not display any JavaScript, and therefore
		// supports all access styles.
		return true;
	}

	public ATag Create()
	{
		
		return new ConcatTag();
	}

}
