package com.plumtree.taglib.ptdata;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.helpers.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This tag will display a link back to the current page or to
 * the base URL of the current page (without the ?).  The HTTP
 * entry point portion of the link can be modified.
 */
public class PageLinkTag extends ATag
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADAPTIVETAGS),
		"com.plumtree.taglib.ptdata.PageLinkTag");

	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;
	public static final OptionalTagAttribute KEY;
	public static final OptionalTagAttribute SCOPE;
	public static final OptionalTagAttribute ENTRY_POINT;
	public static final OptionalTagAttribute USE_BASE_URL;

	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData("pagelink",
			"This tag will display a link back to the current page or to the base URL of the " +
			"current page (without the ?).  The HTTP entry point portion of the link can be modified.",
			ATag.EncodeForTagDoc("<pt:ptdata.pagelink pt:usebaseurl=\"true\"/><br>" +
			"<pt:logic.value pt:value=\"$pagelink\"/><br>" +
			"or<br>" +
			"<pt:ptdata.pagelink pt:entrypoint=\"/portal/test.pt\"/><br>" +
			"<pt:logic.value pt:value=\"$pagelink\"/>"));

		KEY = new OptionalTagAttribute("key", "The key used to store the variable in memory.",
			AttributeType.STRING, "pagelink");

		SCOPE = new OptionalTagAttribute("scope",
			"The scope used to store the variable.  See the Logic Tag Library " +
			"description for more details about valid scope arguments.",
			AttributeType.STRING, Scope.PORTLET_REQUEST.toString());

		ENTRY_POINT = new OptionalTagAttribute("entrypoint", "The new HTTP entry point of the URL that will " +
			"replace the main HTTP entry point (i.e. /portal/server.pt).", AttributeType.STRING, "");

		USE_BASE_URL = new OptionalTagAttribute("usebaseurl", "Use the normalized base URL of the current " +
			"page, rather than the full URL.  This will cause the query string, and any portion of the " +
			"URL after the entry point (i.e. friendly URL sections) to be removed.",
			AttributeType.BOOLEAN, "false");
	}

	public HTMLElement DisplayTag()
	{
		String strURL = GetEnvironment().GetCurrentRequestURL();

		if (GetTagAttributeAsBoolean(USE_BASE_URL))
		{
			// Use the base URL instead of the full URL.
			strURL = GetEnvironment().GetCurrentNormalizedBaseURL();
			if (strURL.endsWith("?"))
			{
				strURL = strURL.substring(0, strURL.length() - 1);
			}
		}

		String strNewEntryPoint = GetTagAttributeAsString(ENTRY_POINT);

		// If we have a new entry point, modify the URL.
		if ((strNewEntryPoint != null) && (XPStringUtility.Compare("", strNewEntryPoint) != 0))
		{
			String strCurrentEntryPoint = ConfigService.getInstance().GetHTTPEntryPoint();

			int nMethodIndex = XPStringUtility.IndexOf(strURL, "//");
			int nHostIndex = XPStringUtility.IndexOf(strURL, "/", nMethodIndex + 2);
			int nEntryPointIndex = XPStringUtility.IndexOf(strURL, strCurrentEntryPoint, nHostIndex + 1);

			// Replace the old entry point with the new one
			strURL = XPStringUtility.Substring(strURL, 0, nHostIndex) + strNewEntryPoint +
				XPStringUtility.Substring(strURL, nEntryPointIndex + strCurrentEntryPoint.length(), strURL.length());
		}
		
		Scope scope = Scope.GetScope(GetTagAttributeAsString(SCOPE));

		if (null == scope)
		{
			log.Warn("Tag variable unable to convert scope attribute {0} to a valid scope with key '{1}'",
				GetTagAttributeAsString(SCOPE), GetTagAttributeAsString(KEY));

			scope = Scope.PORTLET_REQUEST;
		}

		SetStateSharedVariable(GetTagAttributeAsString(KEY), strURL, scope, false);

		return null;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#GetTagType()
	 */
	public TagType GetTagType()
	{
		return TagType.NO_BODY;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#SupportsAccessStyle(AccessStyles)
	 */
	public boolean SupportsAccessStyle(AccessStyles _style)
	{
		// This tag does not display any content by itself, and therefore
		// supports all access styles.
		return true;
	}

	public ATag Create()
	{
		return new PageLinkTag();
	}
}