using System;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.uitasks;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.login;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.web;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.activityspace
{
  /// <summary>
  /// AActivitySpaces contain the MVC setup for a set of Display Pages, as well
  /// as the Display Pages themselves.  All objects should come to their parent
  /// AActivitySpace for access to other objects, such as IModels, IViews, and
  /// user objects like IPTSession and UserPreferences.
  /// Activity spaces setup and manage the interaction between the different MVC
  /// objects and the IDisplayPage.  They initialize all of the objects, and can
  /// get and set data on the Models if necessary.  Activity Spaces are not subject
  /// to the strict data/logic/presentation separation that MVC components are.
  /// This class provides helper implementations of the methods to get MVC Objects and
  /// manages them automatically (IModel, IView, IControl, IDisplayPage).
  /// Just register things and then call GetModel, etc... to access them.
  /// It also provides base implementations of some of the common AActivitySpace
  /// functionality.
  /// </summary>
  /// <author>Don Hayler</author>
  public abstract class AActivitySpace : IMVCObject,IManagedObject,IEnvironment
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.activityspace.AActivitySpace");
    
    /// <summary>
    /// NOTE: This number needs to be tuned.  Should probably be linked to cache
    /// size in Interpreter somehow.
    /// </summary>
    private const int m_nSUBSPACE_WARNING_MAX = 5;
    
    /// <summary>
    /// Security mode on this activity space 
    /// </summary>
    private bool m_bIsSecuredSpace = false;
    
    /// <summary>
    /// Application 
    /// </summary>
    private IApplication m_apApplication = null;
    
    /// <summary>
    /// The current Display Page. 
    /// </summary>
    private IDisplayPage m_asdpCurrentPage = null;
    
    /// <summary>
    /// Sub-session (cleared on logout) 
    /// </summary>
    private ISessionManager m_smSession = null;
    
    /// <summary>
    /// Persistent Sub-session (NOT cleared on logout) 
    /// </summary>
    private ISessionManager m_smPersistentSession = null;
    
    /// <summary>
    /// generic user session. 
    /// </summary>
    private Object m_UserSession = null;
    
    /// <summary>
    /// sub-space key.  If this is null, not a sub-space (default) 
    /// </summary>
    private String m_sSubSpaceKey = null;
    private String m_strParentSpaceID = null;
    
    /// <summary>
    /// The name and ID of the parent space (i.e. the one that launched this one) 
    /// </summary>
    private String m_strParentSpaceName = null;
    
    /// <summary>
    /// the full url of the current request 
    /// </summary>
    private String m_strCurrentRequestUrl = null;
    
    /// <summary>
    /// the normalized base url of the current request 
    /// </summary>
    private String m_strCurrentNormalizedBaseUrl = null;
    
    /// <summary>
    /// The HTTP Request object for the current request - should be released at end of request. 
    /// </summary>
    private IXPRequest m_xpRequest = null;
    
    /// <summary>
    /// The HTTP Response object for the current request - should be released at end of request. 
    /// </summary>
    private IXPResponse m_xpResponse = null;
    
    /// <summary>
    /// Hastable with the name/value pairs from XPRequest (This for PEI use only.) 
    /// </summary>
    private XPHashtable m_htFormData = null;
    
    /// <summary>
    /// The current access style (cached). 
    /// </summary>
    private AccessStyles m_Style = null;
    
    /// <summary>
    /// The Space ID is the unique key used to locate spaces in the AS cache
    /// (in the Interpreter)
    /// </summary>
    private String m_strSpaceID = null;
    private XPHashtable m_mControlMap;
    
    /// <summary>
    /// Registered objects 
    /// </summary>
    private XPHashtable m_mModelMap;
    private XPHashtable m_mPageMap;
    private XPHashtable m_mSubSpaceMap;
    private XPHashtable m_mViewMap;
    private String m_sHelpTopic = null;
    
    /// <summary>
    /// overwrite sub-spaces?
    /// This needs to be true by default because someone can pop-up a sub-space
    /// and close it before it gets a chance to run it's javascript, which can
    /// lead to the opening page getting confused and opening a new pop-up,
    /// which then re-registers and fails (if this is set to false).  Therefore,
    /// it needs to be set to true.  This may hide bugs in someone's use of the
    /// sub-space code, but I can't see any way around this.
    /// </summary>
    private bool m_bOverWriteSubSpaces = true;
    
    /// <summary>
    /// Current Display Page 
    /// </summary>
    private bool m_bTrackCurrentPage = false;
    
    /// <summary>
    /// This hashtable is used to store references to extremely long query string parameters.
    /// e.g. hide lists to tree control
    /// Only certain parts of the tree control know how to use this.
    /// </summary>
    /// <author>olegs</author>
    protected XPHashtable m_stringReferences;
    
    /// <summary>
    /// True to enable this AS for in place refresh. 
    /// </summary>
    private bool m_bInPlaceRefreshEnabled = false;
    
    /// <summary>
    /// True to have Display Pages show in place content. 
    /// </summary>
    private bool m_bInPlaceRefreshDisplayOn = false;
    ///////////////////
    // Abstract methods
    ///////////////////
    
    /// <summary>
    /// This method is called by the Interpreter at the beginning of every HTTP
    /// request to determine whether or not the current user has the security
    /// privileges necessary to view the requested page.
    /// This method is designed for checking whether or not a particular user has
    /// any access to the space, page, and control.  More detailed checking (of
    /// query string parameters, for instance), should be done in the
    /// IControl.CheckActionSecurityAndExecute() method.
    /// For Plumtree portal applications (IPTActivitySpaces), since the IPTSession
    /// is guaranteed to be set before Init() & CheckBasicAccess() are called, you
    /// can check access as follows:
    /// return (GetIPTSession().GetSessionInfo().CheckForActivityRight
    /// (new Integer(PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_ACCESS_ADMINISTRATION))
    /// == 1);
    /// </summary>
    /// <param name="String">the requested IDisplayPage name (may be null for default)</param>
    /// <param name="String">the requested IControl name (may be null if not used)</param>
    /// <param name="boolean">
    /// true implies this is from an internal redirect from the same space
    /// false implies this is an URL from the user, or a redirect from
    /// another space.
    /// </param>
    /// <returns>boolean true implies access is allowed, false implies access is denied</returns>
    public abstract bool CheckBasicAccess(String strPage, String strControl,
     bool bSameSpace);
    
    /// <summary>
    /// This method is used by the ASManager to return new instances of managed
    /// objects.
    /// </summary>
    /// <returns>A new instance of the managed class (i.e. return new Foo();)</returns>
    public abstract Object Create();
    
    /// <summary>
    /// This method is used to identify the type of this space.
    /// </summary>
    /// <returns>String the name of this AActivitySpace</returns>
    public abstract String GetName();
    ///////////////////////
    // Non-abstract methods
    ///////////////////////
    public virtual void SetHelpTopic(String s)
    {
      // Toward the end of the release, help filenames all became
      // lowercase.
      m_sHelpTopic = XPStringUtility.ToLowerCase(s);
    }
    public virtual String GetHelpTopic()
    {
      return m_sHelpTopic;
    }
    
    /// <summary>
    /// Changes the key of a sub space.
    /// </summary>
    /// <param name="String">the key of the sub-space to change the key</param>
    /// <param name="String">the new key for that sub space</param>
    public virtual void ChangeSubSpaceKey(String subSpaceKey, String subSpaceNewKey)
    {
      if (m_mSubSpaceMap.ContainsKey(subSpaceKey))
      {
        if (!m_mSubSpaceMap.ContainsKey(subSpaceNewKey))
        {
          AActivitySpace asSubSpace = (AActivitySpace) m_mSubSpaceMap.RemoveElement(subSpaceKey);
          m_mSubSpaceMap.PutElement(subSpaceNewKey, asSubSpace);
          asSubSpace.SetSubSpaceKey(subSpaceNewKey);
          log.Info(GetName() + " ChangeSubSpaceKey key for sub space: " + asSubSpace.GetName() + " has been changed from: " + subSpaceKey + " to: " + subSpaceNewKey);
          } else
        {
          log.Error(GetName() + " ChangeSubSpaceKey failed: New key " + subSpaceNewKey + " is already used.");
          }
      } else
      {
        log.Error(GetName() + " ChangeSubSpaceKey failed: Could not find sub-space " + subSpaceKey);
        }
    }
    
    /// <summary>
    /// Delete the specified Sub-space and remove it from the Activity
    /// Space cache.  This is permanent!
    /// The sub-space key is different from the space ID.
    /// </summary>
    /// <param name="String">the sub-space key of the Sub-space to delete</param>
    public virtual void DeleteSubSpace(String subSpaceKey)
    {
      if ((null == subSpaceKey) || ("".Equals(subSpaceKey)))
      {
        log.Error(GetName() + " DeleteSubSpace failed: Called with empty subspacekey.");
        return;
      }
      AActivitySpace asDelete = (AActivitySpace) m_mSubSpaceMap.RemoveElement(subSpaceKey);
      if (null == asDelete)
      {
        log.Error(GetName() + " DeleteSubSpace failed: Could not find sub-space " + subSpaceKey);
        return;
      }
      ASCache myCache = (ASCache) m_smSession.GetAttribute(ASConstants.SANDBOXASCACHE);
      myCache.RemoveSubSpace(asDelete.GetName(), asDelete.GetSpaceID());
    }
    
    /// <summary>
    /// This method returns the current access style (i.e. Section
    /// 508 compliance, low bandwidth, or standard).
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.constants.AccessStyles"></seealso>
    /// <returns>AccessStyles The access style</returns>
    public virtual AccessStyles GetAccessStyle()
    {
      // NOTE: This method gets called approximately 30 times per
      // page hit.  XPConvert is relatively cheap, so we probably
      // don't need to optimize this (by storing a style object
      // rather than a String or an int), but we need to be careful
      // not to do expensive things here, like throw exceptions.
      // NEWER NOTE: With the new page layouts and tag engine, this 
      // gets called up to 1000 times per page, so we should cache the
      // result.
      // NOTE: the style is nulled out at the end of every request so
      // we can pick up new styles if it changes.
      if (m_Style != null)
      {
        return m_Style;
      }
      AccessStyles style = AccessStyles.STANDARD;
      Object accessStyle = PersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingsHelper.ACCESS_STYLE,
       this);
      if (null != accessStyle)
      {
        try
        {
          String strAccessStyle = XPConvert.ToString(accessStyle);
          if (!"".Equals(strAccessStyle))
          {
            int val = XPConvert.ToInteger(strAccessStyle);
            style = AccessStyles.GetAccessStyle(val);
          }
        }
        catch (XPFormatException e)
        {
          // goes to default
          log.Error(e, GetName() + " could not convert access style " + accessStyle + " correctly.");
          style = AccessStyles.STANDARD;
        }
      }
      m_Style = style;
      return m_Style;
    }
    
    /// <summary>
    /// Return true if the current access style is STANDARD otherwise
    /// return false.
    /// </summary>
    /// <returns>true if the access style is STANDARD</returns>
    public virtual bool GetIsAccessStyleStandard()
    {
      AccessStyles style = GetAccessStyle();
      if (style == AccessStyles.STANDARD)
      {
        return true;
      }
      return false;
    }
    
    /// <summary>
    /// Return true if the current access style is not STANDARD (it is
    /// SECTION_508 or LOW_BANDWIDTH) otherwise return false.
    /// </summary>
    /// <returns>true if the access style is not STANDARD</returns>
    public virtual bool GetIsAccessStyleNonStandard()
    {
      AccessStyles style = GetAccessStyle();
      if (style != AccessStyles.STANDARD)
      {
        return true;
      }
      return false;
    }
    
    /// <summary>
    /// This method is used to return whether or not
    /// this space is secured (SSL) or not (regular HTTP).
    /// </summary>
    /// <returns>boolean true implies secured</returns>
    public virtual bool GetIsSecuredSpace()
    {
      return m_bIsSecuredSpace;
    }
    
    /// <summary>
    /// Return the Application.
    /// </summary>
    /// <returns>the application</returns>
    public virtual IApplication GetApplication()
    {
      return m_apApplication;
    }
    
    /// <summary>
    /// Return the requested Control.  This method should probably
    /// wrap control objects in aggregating control objects to do page specific tasks.
    /// In other words, the default package control should have no page specific
    /// knowledge.
    /// </summary>
    /// <param name="String">the name of the control to get</param>
    /// <returns>IControl the requested control</returns>
    public virtual IControl GetControl(String controlName)
    {
      IControl result;
      if (controlName == null)
      {
        result = (IControl) m_mControlMap.GetElement("");
      } else
      {
        result = (IControl) m_mControlMap.GetElement(controlName);
      }
      if ((result == null) && (controlName != null))
      {
        log.Error(GetName() + " GetControl failed: Could not find control " + controlName);
        }
      // SE: Bug 62787 - Add diagnostics for tracking MVC registration and displaying
      log.Debug(GetName() + " GetControl succeeded: Found " + controlName);
      return result;
    }
    
    /// <summary>
    /// This method checks whether or not a specific control exists in this AActivitySpace.
    /// </summary>
    /// <param name="String">the name of the control to check</param>
    /// <returns>boolean whether or not the control exists</returns>
    public virtual bool GetControlExists(String controlName)
    {
      return m_mControlMap.ContainsKey(controlName);
    }
    
    /// <summary>
    /// Return an Enumerator for the registered controls.
    /// </summary>
    /// <returns>IXPEnumerator the registered controls</returns>
    public virtual IXPEnumerator GetControlsEnumerator()
    {
      return m_mControlMap.GetElements();
    }
    
    /// <summary>
    /// Return the most recent DisplayPage.
    /// </summary>
    /// <returns>the most recent page</returns>
    public virtual IDisplayPage GetCurrentPage()
    {
      return m_asdpCurrentPage;
    }
    
    /// <summary>
    /// Return the full url of the current request.
    /// </summary>
    /// <returns>the url of the current request</returns>
    public virtual String GetCurrentRequestURL()
    {
      if (m_strCurrentRequestUrl == null)
      {
        throw new XPException(GetName() + " GetCurrentRequestURL called when URL has not been set.");
        }
      return m_strCurrentRequestUrl;
    }
    
    /// <summary>
    /// Return the normalized base url of the current request.
    /// The normalized base url is essentially the request url
    /// up to and including the '?'.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.statichelpers.ConfigHelpers#GetNormalizedBaseURL(String)"></seealso>
    /// <returns>The normalized base url of the current request</returns>
    public virtual String GetCurrentNormalizedBaseURL()
    {
      if (m_strCurrentNormalizedBaseUrl == null)
      {
        log.Error(GetName() + " GetCurrentNormalizedBaseURL called when URL has not been set.");
        }
      return m_strCurrentNormalizedBaseUrl;
    }
    
    /// <summary>
    /// Return the HTTP Request for the current request.
    /// </summary>
    /// <returns>the current request</returns>
    public virtual IXPRequest GetCurrentHTTPRequest()
    {
      return m_xpRequest;
    }
    
    /// <summary>
    /// Return the HTTP Response for the current request.
    /// </summary>
    /// <returns>the current response</returns>
    public virtual IXPResponse GetCurrentHTTPResponse()
    {
      return m_xpResponse;
    }
    
    /// <summary>
    /// JF- Returns the current name/value pairs from the Request.
    /// (This for PEI use only.)
    /// </summary>
    /// <returns>XPHashtable</returns>
    public virtual XPHashtable GetCurrentFormData()
    {
      if (m_htFormData == null)
      {
        log.Error(GetName() + " GetCurrentFormData called when form data has not been set.");
        }
      return m_htFormData;
    }
    
    /// <summary>
    /// Gets the current language of the user.
    /// I.E. en, fr, jp
    /// </summary>
    /// <returns>The current language of the user</returns>
    public virtual String GetLanguage()
    {
      return (String) PersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingsHelper.LANGUAGE,
       this);
    }
    
    /// <summary>
    /// Gets the current locale of the user.
    /// I.E. en-us, en-gb, fr-fr, jp-jp
    /// </summary>
    /// <returns>The current locale of the user.</returns>
    public virtual String GetLocale()
    {
      return (String) PersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingsHelper.LOCALE,
       this);
    }
    
    /// <summary>
    /// Return the requested Model.
    /// </summary>
    /// <param name="String">the name of the model to get</param>
    /// <returns>IModel the requested model</returns>
    public virtual IModel GetModel(String modelName)
    {
      IModel result = (IModel) m_mModelMap.GetElement(modelName);
      if (result == null)
      {
        log.Error(GetName() + " GetModel failed: Could not find model " + modelName);
        }
      // SE: Bug 62787 - Add diagnostics for tracking MVC registration and displaying
      log.Debug(GetName() + " GetModel succeeded: Found " + modelName);
      return result;
    }
    
    /// <summary>
    /// This method checks whether or not a specific model exists in this AActivitySpace.
    /// </summary>
    /// <param name="String">the name of the model to check</param>
    /// <returns>boolean whether or not the model exists</returns>
    public virtual bool GetModelExists(String modelName)
    {
      return m_mModelMap.ContainsKey(modelName);
    }
    
    /// <summary>
    /// Return an Enumerator for the registered models.
    /// </summary>
    /// <returns>IXPEnumerator the registered models</returns>
    public virtual IXPEnumerator GetModelsEnumerator()
    {
      return m_mModelMap.GetElements();
    }
    
    /// <summary>
    /// Return the requested DisplayPage.
    /// </summary>
    /// <param name="String">the name of the IDisplayPage to get</param>
    /// <returns>IDisplayPage the requested page</returns>
    public virtual IDisplayPage GetPage(String pageName)
    {
      IDisplayPage result = (IDisplayPage) m_mPageMap.GetElement(pageName);
      if (result == null)
      {
        log.Error(GetName() + " GetPage failed: Could not find page " + pageName);
        }
      // SE: Bug 62787 - Add diagnostics for tracking MVC registration and displaying
      log.Debug(GetName() + " GetPage succeeded: Found " + pageName);
      return result;
    }
    
    /// <summary>
    /// This method checks whether or not a specific page exists in this AActivitySpace.
    /// </summary>
    /// <param name="String">the name of the page to check</param>
    /// <returns>boolean whether or not the page exists</returns>
    public virtual bool GetPageExists(String pageName)
    {
      return m_mPageMap.ContainsKey(pageName);
    }
    
    /// <summary>
    /// Return an Enumerator for the registered pages.
    /// </summary>
    /// <returns>IXPEnumerator the registered pages</returns>
    public virtual IXPEnumerator GetPagesEnumerator()
    {
      return m_mPageMap.GetElements();
    }
    
    /// <summary>
    /// Return the parent AActivitySpace.  If there is no parent space,
    /// this will return null.
    /// </summary>
    /// <returns>
    /// AActivitySpace the parent AActivitySpace.  null if there
    /// is no parent space.
    /// </returns>
    public virtual AActivitySpace GetParentSpace()
    {
      if ((m_strParentSpaceName == null) || (m_strParentSpaceID == null))
      {
        // No parent space
        return null;
      }
      ASCache myCache = (ASCache) m_smSession.GetAttribute(ASConstants.SANDBOXASCACHE);
      return myCache.GetSpace(m_strParentSpaceName, m_strParentSpaceID);
    }
    
    /// <summary>
    /// Return the space ID of the parent AActivitySpace.  If there is no parent
    /// space, this will return null.
    /// </summary>
    /// <returns>
    /// String the space ID of the parent AActivitySpace.  null if there
    /// is no parent space.
    /// </returns>
    public virtual String GetParentSpaceID()
    {
      return m_strParentSpaceID;
    }
    
    /// <summary>
    /// Return the name of the parent AActivitySpace.  If there is no parent space,
    /// this will return null.
    /// </summary>
    /// <returns>
    /// String the name of the parent AActivitySpace.  null if there
    /// is no parent space.
    /// </returns>
    public virtual String GetParentSpaceName()
    {
      return m_strParentSpaceName;
    }
    
    /// <summary>
    /// Return the XPResourceManager
    /// AActivitySpace.GetString() and the static method
    /// XPResourceManager.FormatString() should be used
    /// instead.
    /// </summary>
    /// <returns>the Resource manager</returns>
    public virtual XPResourceManager GetResourceManager()
    {
      XPResourceManager rm = (XPResourceManager) m_apApplication.GetCachingManager().GetEntry("ResourceManager");
      return rm;
    }
    
    /// <summary>
    /// Return the unique space ID for this AActivitySpace.
    /// </summary>
    /// <returns>String the unique space ID for this AActivitySpace</returns>
    public virtual String GetSpaceID()
    {
      return m_strSpaceID;
    }
    
    /// <summary>
    /// This method gets localized strings for an
    /// internationalized string file based on the user's
    /// locale.
    /// Available string files:
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.constants.ResourceConstants">Custom string files can also be added to the portal.</seealso>
    /// <param name="_nID">The ID of the string to retrieve</param>
    /// <param name="_strStringFile">The name of the string file to use</param>
    /// <returns>String The localized string</returns>
    public virtual String GetString(int _nStringID, String _strResourceFileName)
    {
      String strLanguage;
      try
      {
        strLanguage = XPConvert.ToString(PersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingsHelper.LANGUAGE,
         this));
      }
      catch (Exception e)
      {
        log.Error(e, GetName() + " GetString() unable to get language for user " + LoginHelper.INSTANCE.GetUserSessionName(m_UserSession) + " with ID " + LoginHelper.INSTANCE.GetUserSessionID(m_UserSession) + ".");
        throw XPException.GetInstance(e);
      }
      String strResult;
      try
      {
        XPResourceManager rm = GetResourceManager();
        strResult = rm.GetString(_nStringID, _strResourceFileName, strLanguage);
        }
      catch (Exception e)
      {
        log.Error(e, GetName() + " GetString() unable to get string with ID " + _nStringID + " from resource file " + _strResourceFileName + " in language " + strLanguage + ".");
        throw XPException.GetInstance(e);
      }
      return strResult;
    }
    
    /// <summary>
    /// Figure out the style sheet; this method can be overriden by classes that
    /// require a special style sheet.
    /// </summary>
    /// <returns>
    /// An HTMLCollection that contains one or more PTStyleSheetLink objects
    /// (almost always just one)
    /// </returns>
    public virtual HTMLElementCollection GetStyleSheetLinks()
    {
      return LoginHelper.INSTANCE.GetStyleSheetLinks(GetUserSession(), this);
      }
    
    /// <summary>
    /// Gets the time zone of the current user.
    /// I.E. TODO
    /// </summary>
    /// <returns>The time zone of the user.</returns>
    public virtual String GetTimeZone()
    {
      return (String) PersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingsHelper.TIMEZONE,
       this);
    }
    //////////////////////////
    // Internal helper methods
    //////////////////////////
    
    /// <summary>
    /// This method is called at the end of every HTTP request by the Interpreter.
    /// This method then calls IModel.CleanupTempData() on all IModels registered
    /// using RegisterModel().
    /// This method is intended for cleaning up temporary data that is only used
    /// for the lifetime of a single request, like DB query results.
    /// </summary>
    public virtual void CleanupTempData()
    {
      IXPEnumerator myIter = m_mModelMap.GetElements();
      while (myIter.MoveNext() != false)
      {
        ((IModel) myIter.GetCurrent()).CleanupTempData();
      }
      m_strCurrentRequestUrl = null;
      m_strCurrentNormalizedBaseUrl = null;
      m_xpRequest = null;
      m_xpResponse = null;
      m_htFormData = null;
      // Since we now cache the access style for performance reasons,
      // we need to clear this every request so that any updates to
      // the access style will be visible.
      m_Style = null;
    }
    
    /// <summary>
    /// This method is called when an ActivitySpace is removed from the cache.
    /// This method then calls IModel.CleanupAllData() on all IModels registered
    /// using RegisterModel().
    /// This method is intended for releasing data stored in the server when
    /// the activity space is no longer needed, such as search objects that
    /// need to be maintained as long as the user is clicking through the
    /// search, but should be freed when the space can no longer be used.
    /// This can be used to prevent Java references to COM memory from hanging
    /// around in the long lived Java heap without cleaning up the underlying
    /// COM memory.  The Java reference will still stay in the long lived heap
    /// until it gets garbage collected (which could be a while), but the COM
    /// memory should get released here.
    /// This method will not be called if an HTTPSession times out.
    /// CleanupTempData will be called before CleanupAllData.
    /// </summary>
    public virtual void CleanupAllData()
    {
      IXPEnumerator myIter = m_mModelMap.GetElements();
      while (myIter.MoveNext() != false)
      {
        ((IModel) myIter.GetCurrent()).CleanupAllData();
      }
    }
    
    /// <summary>
    /// Returns the subsession that the AActivitySpace can use to store cross-space data.
    /// This also provides access to the ASCache object (ASConstants.SANDBOXASCACHE).
    /// This SubSession is cleared of all data on logout.
    /// </summary>
    /// <returns>
    /// ISessionManager the subsession that the AActivitySpace can use to store
    /// cross-space data.
    /// </returns>
    public virtual ISessionManager GetSubSession()
    {
      return m_smSession;
    }
    
    /// <summary>
    /// Returns the persistent subsession that the AActivitySpace can use to store cross-space data
    /// for use across multiple login/logout cycles.
    /// This SubSession is not cleared of its data on logout, so be sure not to cache
    /// anything on this session that could be considered a security risk if it was
    /// leaked to another user.
    /// Most developers should use the normal subsession (GetSubSession()) for cross-space
    /// data storage.  This is currently only used when editing the default profiles.
    /// </summary>
    /// <returns>
    /// ISessionManager the persistent subsession that the AActivitySpace can use to store
    /// cross-space data across multiple login/logout cycles.
    /// </returns>
    public virtual ISessionManager GetPersistentSubSession()
    {
      return m_smPersistentSession;
    }
    
    /// <summary>
    /// Return the requested Sub-space.
    /// The sub-space key is different from the space ID.
    /// </summary>
    /// <param name="String">the sub-space key of the Sub-space to get</param>
    /// <returns>AActivitySpace the requested Sub-space</returns>
    public virtual AActivitySpace GetSubSpace(String subSpaceKey)
    {
      AActivitySpace result = (AActivitySpace) m_mSubSpaceMap.GetElement(subSpaceKey);
      if (result == null)
      {
        log.Error(GetName() + " GetSubSpace failed: Could not find sub-space " + subSpaceKey);
        }
      return result;
    }
    
    /// <summary>
    /// This method checks whether or not a specific sub-space exists in this AActivitySpace.
    /// </summary>
    /// <param name="String">the name of the sub-space to check</param>
    /// <returns>boolean whether or not the sub-space exists</returns>
    public virtual bool GetSubSpaceExists(String subSpaceKey)
    {
      return m_mSubSpaceMap.ContainsKey(subSpaceKey);
    }
    
    /// <summary>
    /// Returns the sub-space key.  If this is null, this is not a sub-space.
    /// This key is used by the parent AActivitySpace to identify sub-spaces
    /// (in case there are multiple instances of the same type of sub-space).
    /// The sub-space key is different from the space ID.
    /// </summary>
    /// <returns>String the sub-space key</returns>
    public virtual String GetSubSpaceKey()
    {
      return m_sSubSpaceKey;
    }
    
    /// <summary>
    /// Return an Enumerator for the registered sub-spaces.
    /// </summary>
    /// <returns>IXPEnumerator the registered sub-spaces</returns>
    public virtual IXPEnumerator GetSubSpacesEnumerator()
    {
      return m_mSubSpaceMap.GetElements();
    }
    
    /// <summary>
    /// Return the user's Session.  This is a generic Object that
    /// can be used for different kinds of sessions.  This will
    /// have to be cast to the correct type of session (IPTSession
    /// for the portal).
    /// </summary>
    /// <returns>Object the user's session</returns>
    public virtual Object GetUserSession()
    {
      return m_UserSession;
    }
    
    /// <summary>
    /// Helper method to access Variable Packages.
    /// </summary>
    /// <returns>IVarPack the requested VarPack</returns>
    public virtual IVarPack GetVarPack(String strVarPack)
    {
      return m_apApplication.GetVarPackManager().GetVariablePackage(strVarPack);
      }
    
    /// <summary>
    /// Return the requested View.
    /// </summary>
    /// <param name="String">the name of the view to get</param>
    /// <returns>IView the requested view</returns>
    public virtual IView GetView(String viewName)
    {
      IView result = null;
      if (GetAccessStyle() != AccessStyles.STANDARD)
      {
        // Try to get the 508 version of this view, if it exists.
        result = (IView) m_mViewMap.GetElement(viewName + ASConstants.STR_508_VIEW_SUFFIX);
        }
      if (result == null)
      {
        result = (IView) m_mViewMap.GetElement(viewName);
      }
      if (result == null)
      {
        log.Error(GetName() + " GetView failed: Could not find view " + viewName);
        }
      // SE: Bug 62787 - Add diagnostics for tracking MVC registration and displaying
      log.Debug(GetName() + " GetView succeeded: Found " + viewName);
      return result;
    }
    
    /// <summary>
    /// This method checks whether or not a specific view exists in this AActivitySpace.
    /// </summary>
    /// <param name="String">the name of the view to check</param>
    /// <returns>boolean whether or not the view exists</returns>
    public virtual bool GetViewExists(String viewName)
    {
      return m_mViewMap.ContainsKey(viewName);
    }
    
    /// <summary>
    /// Return an Enumerator for the registered views.
    /// </summary>
    /// <returns>IXPEnumerator the registered views</returns>
    public virtual IXPEnumerator GetViewsEnumerator()
    {
      return m_mViewMap.GetElements();
    }
    /////////////////////////////////////////////////////////////////
    // Base methods that should be overridden and called as super.*()
    /////////////////////////////////////////////////////////////////
    
    /// <summary>
    /// Initialize the Activity Space.
    /// This method is called after common attributes like ID, parent, session,
    /// application, etc... are set.
    /// Get methods are not valid until this method is called.
    /// This method should be overridden and called as super.Init();
    /// </summary>
    public virtual void Init()
    {
      // Create all the maps
      m_mModelMap = new XPHashtable();
      m_mViewMap = new XPHashtable();
      m_mControlMap = new XPHashtable();
      m_mPageMap = new XPHashtable();
      m_mSubSpaceMap = new XPHashtable();
      // Check initialization prerequisites
      if (m_smSession == null)
      {
        log.Error(GetName() + " Init called before sub-session set. Sub-spaces will not be registered.");
        return;
      }
      if (m_apApplication == null)
      {
        log.Error(GetName() + " Init called before application set. Sub-spaces will not be registered.");
        return;
      }
      if ((m_strSpaceID == null) || ("".Equals(m_strSpaceID)))
      {
        log.Error(GetName() + " Init called and Space ID is invalid: " + m_strSpaceID + ". Sub-spaces will not be registered.");
        return;
      }
      if ((m_sSubSpaceKey != null) && (!"".Equals(m_sSubSpaceKey)))
      {
        ASCache myCache = (ASCache) m_smSession.GetAttribute(ASConstants.SANDBOXASCACHE);
        AActivitySpace parent = myCache.GetSpace(m_strParentSpaceName, m_strParentSpaceID);
        if (parent == null)
        {
          log.Error(GetName() + " No parent space found when attempting to initialize sub-space " + GetName() + ". Sub-spaces will not be registered.");
          } else
        {
          parent.RegisterSubSpace(this, m_sSubSpaceKey);
        }
      }
    }
    
    /// <summary>
    /// Re-Initialize the Activity Space in case models are holding on to an old session.
    /// </summary>
    public virtual void ReInit()
    {
    }
    ///////////////////////////////////////
    // Common implementations of basic code
    ///////////////////////////////////////
    ///////////////////////////////
    // General Get Accessor Methods
    ///////////////////////////////
    
    /// <summary>
    /// Returns whether or not this space is a sub-space.
    /// </summary>
    /// <returns>boolean true implies this is a sub-space</returns>
    public virtual bool IsSubSpace()
    {
      return m_sSubSpaceKey != null;
    }
    
    /// <summary>
    /// This method registers the control so the AActivitySpace can manage it
    /// automatically.
    /// It also initializes it with the IModel argument.
    /// This operation will fail and return false if the control is null or if
    /// there is already a control with the same name registered with the
    /// AActivitySpace.
    /// </summary>
    /// <param name="String">controlName the name of the IControl to register</param>
    /// <param name="IModel">the model to initialize this IControl with</param>
    /// <returns>
    /// boolean true implies success
    /// false implies failure
    /// </returns>
    public virtual bool RegisterControl(String controlName, IModel model)
    {
      return RegisterControl(controlName, controlName, model);
    }
    
    /// <summary>
    /// This method registers the control so the AActivitySpace can manage it
    /// automatically.  This method allows the client to store the control under
    /// a name other than the one returned by IControl.GetName().  This allows
    /// the client to use more than one control of the same type in a single
    /// AActivitySpace.
    /// It also initializes it with the IModel argument.
    /// This operation will fail and return false if the control is null or if
    /// there is already a control with the same name registered with the
    /// AActivitySpace.
    /// If you register a control with a storage name of "" (the empty string),
    /// this will be used as the default control in your space.  This means that
    /// anytime an URL does not have a control specified in it, this default
    /// control will be used.
    /// </summary>
    /// <param name="String">controlName the name of the IControl to register</param>
    /// <param name="String">storageName the name to store this IControl under</param>
    /// <param name="IModel">the model to initialize this IControl with</param>
    /// <returns>
    /// boolean true implies success
    /// false implies failure
    /// </returns>
    public virtual bool RegisterControl(String controlName, String storageName,
     IModel model)
    {
      IControl myControl = ASManager.INSTANCE.GetControl(controlName);
      if (myControl == null)
      {
        log.Error(GetName() + " RegisterControl failed: Could not get control " + controlName);
        return false;
      }
      // Put the control into the map
      Object mOldControl = m_mControlMap.PutElement(storageName, myControl);
      // If there was already a control there, put it back and fail
      if (mOldControl != null)
      {
        log.Error(GetName() + " RegisterControl failed: Control already registered " + storageName);
        m_mControlMap.PutElement(storageName, mOldControl);
        return false;
      }
      // Initialize the Control
      myControl.Init(model, this);
      // SE: Bug 62787 - Add diagnostics for tracking MVC registration and displaying
      log.Debug(GetName() + " RegisterControl succeeded: " + controlName + " registered");
      return true;
    }
    
    /// <summary>
    /// This method registers and initializes the model so the AActivitySpace can
    /// manage it automatically.
    /// This operation will fail and return false if the model is null or if
    /// there is already a model with the same name registered with the
    /// AActivitySpace.
    /// </summary>
    /// <param name="String">modelName the name of the IModel to register</param>
    /// <returns>
    /// boolean true implies success
    /// false implies failure
    /// </returns>
    public virtual bool RegisterModel(String modelName)
    {
      return RegisterModel(modelName, modelName);
    }
    
    /// <summary>
    /// This method registers and initializes the model so the AActivitySpace can
    /// manage it automatically.  This method allows the user to use a name other
    /// than the model name returned by IModel.GetName(), which allows the user to
    /// have multiple models of the same type in one AActivitySpace.
    /// This operation will fail and return false if the model is null or if
    /// there is already a model with the same name registered with the
    /// AActivitySpace.
    /// </summary>
    /// <param name="String">modelName the name of the IModel to register</param>
    /// <param name="String">storageName the name to store the IModel under</param>
    /// <returns>
    /// boolean true implies success
    /// false implies failure
    /// </returns>
    public virtual bool RegisterModel(String modelName, String storageName)
    {
      IModel myModel = ASManager.INSTANCE.GetModel(modelName);
      if (myModel == null)
      {
        log.Error(GetName() + " RegisterModel failed: Could not get model  " + modelName);
        return false;
      }
      // Put the model into the map
      Object mOldModel = m_mModelMap.PutElement(storageName, myModel);
      // If there was already a model there, put it back and fail
      if (mOldModel != null)
      {
        log.Error(GetName() + " RegisterModel failed: model already registered " + storageName);
        m_mModelMap.PutElement(storageName, mOldModel);
        return false;
      }
      // Initialize the Model
      myModel.Init(this);
      // SE: Bug 62787 - Add diagnostics for tracking MVC registration and displaying
      log.Debug(GetName() + " RegisterModel succeeded: " + modelName + " registered");
      return true;
    }
    
    /// <summary>
    /// Registers the Sub-space.
    /// The sub-space key is different from the space ID.
    /// </summary>
    /// <param name="String">the key of the Sub-space to register</param>
    /// <param name="AActivitySpace">the sub-space to register</param>
    /// <returns>true if registration succesful</returns>
    public virtual bool RegisterSubSpace(AActivitySpace subSpace, String storageName)
    {
      if (subSpace == null)
      {
        log.Error(GetName() + " RegisterSubSpace failed: Null subSpace with key " + storageName);
        return false;
      }
      // Put the subspace into the map
      AActivitySpace mOldSubSpace = (AActivitySpace) m_mSubSpaceMap.PutElement(storageName,
       subSpace);
      // If there was already a subspace there,
      if (mOldSubSpace != null)
      {
        if (!m_bOverWriteSubSpaces)
        {
          // put it back and fail
          log.Error(GetName() + " RegisterSubSpace failed: Subspace already registered " + storageName);
          m_mSubSpaceMap.PutElement(storageName, mOldSubSpace);
          return false;
        } else
        {
          // or delete it
          ASCache myCache = (ASCache) m_smSession.GetAttribute(ASConstants.SANDBOXASCACHE);
          myCache.RemoveSubSpace(mOldSubSpace.GetName(), mOldSubSpace.GetSpaceID());
          log.Info(GetName() + "RegisterSubSpace: Identical storage name - deleting old subspace with name " + mOldSubSpace.GetName() + " and ID " + mOldSubSpace.GetSpaceID());
          }
      }
      if (m_mSubSpaceMap.GetSize() >= m_nSUBSPACE_WARNING_MAX)
      {
        log.Warn(GetName() + " has an excessively large number of sub-spaces: " + m_mSubSpaceMap.GetSize());
        }
      return true;
    }
    
    /// <summary>
    /// This method registers the view so the AActivitySpace can manage it
    /// automatically.
    /// It also initializes it with the IModel and AActivitySpace arguments.
    /// This operation will fail and return false if the view is null or if
    /// there is already a view with the same name registered with the
    /// AActivitySpace.
    /// </summary>
    /// <param name="String">viewName the name of the IView to register</param>
    /// <param name="IModel">the model to initialize this IView with</param>
    /// <returns>
    /// boolean true implies success
    /// false implies failure
    /// </returns>
    public virtual bool RegisterView(String viewName, IModelRO model)
    {
      return RegisterView(viewName, viewName, model);
    }
    
    /// <summary>
    /// This method registers the view so the AActivitySpace can manage it
    /// automatically.  This method allows the user to store the view with a name
    /// other than the one returned by IView.GetName().  This allows the user to
    /// use multiple views of the same type in one AActivitySpace.
    /// It also initializes it with the IModel and AActivitySpace arguments.
    /// This operation will fail and return false if the view is null or if
    /// there is already a view with the same name registered with the
    /// AActivitySpace.
    /// </summary>
    /// <param name="String">viewName the name of the IView to register</param>
    /// <param name="String">storageName the name to store this IView under</param>
    /// <param name="IModel">the model to initialize this IView with</param>
    /// <returns>
    /// boolean true implies success
    /// false implies failure
    /// </returns>
    public virtual bool RegisterView(String viewName, String storageName,
     IModelRO model)
    {
      IView myView = ASManager.INSTANCE.GetView(viewName);
      if (myView == null)
      {
        log.Error(GetName() + " RegisterView failed: Could not get view " + viewName);
        return false;
      }
      // Put the view into the map
      Object mOldView = m_mViewMap.PutElement(storageName, myView);
      // If there was already a view there, put it back and fail
      if (mOldView != null)
      {
        log.Error(GetName() + " RegisterView failed: View already registered " + storageName);
        m_mViewMap.PutElement(storageName, mOldView);
        return false;
      }
      // Initialize the View
      myView.Init(model, this);
      // SE: Bug 62787 - Add diagnostics for tracking MVC registration and displaying
      log.Debug(GetName() + " RegisterView succeeded: " + viewName + " registered");
      return true;
    }
    
    /// <summary>
    /// Sets the Application.
    /// This method must be called before Init();
    /// </summary>
    /// <param name="the">application</param>
    public virtual void SetApplication(IApplication application)
    {
      m_apApplication = application;
    }
    
    /// <summary>
    /// Sets the current Request URL and computes the normalized
    /// base url.
    /// This method must be called at the beginning of every request.
    /// </summary>
    /// <param name="the">current request url</param>
    public virtual void SetCurrentRequestURL(String strCurrentRequestURL)
    {
      m_strCurrentRequestUrl = strCurrentRequestURL;
      m_strCurrentNormalizedBaseUrl = ConfigHelper.GetNormalizedBaseURL(strCurrentRequestURL);
      }
    
    /// <summary>
    /// Sets the HTTP Request and Response Objects for the current request.
    /// This method must be called at the beginning of every request.
    /// </summary>
    /// <param name="The">current XPRequest</param>
    /// <param name="The">current XPResponse</param>
    public virtual void SetCurrentHTTPRequestResponseObjects(IXPRequest xpRequest,
     IXPResponse xpResponse)
    {
      m_xpRequest = xpRequest;
      m_xpResponse = xpResponse;
    }
    
    /// <summary>
    /// JF- Sets the hastable with the name/value pairs from XPRequest.
    /// (This for PEI use only.)
    /// This method must be called at the beginning of every request.
    /// </summary>
    /// <param name="_htFormData"></param>
    public virtual void SetCurrentFormData(XPHashtable _htFormData)
    {
      m_htFormData = _htFormData;
    }
    
    /// <summary>
    /// Set the current page for this AActivitySpace.  If this AActivitySpace only
    /// has one IDisplayPage, the current page will automatically be set to that
    /// IDisplayPage.
    /// If current page tracking is on (off by default), the Interpeter will set the
    /// current page to be the most recently accessed page.
    /// </summary>
    /// <param name="IDisplayPage">the current Display Page</param>
    public virtual void SetCurrentPage(IDisplayPage page)
    {
      if (m_bTrackCurrentPage && (page != null))
      {
        m_asdpCurrentPage = page;
      }
    }
    
    /// <summary>
    /// Should be set to the request secure flag (IsSecureConnection).
    /// This method must be called before Init();
    /// </summary>
    /// <param name="_bIsSecured"></param>
    public virtual void SetIsASSecured(bool _bIsSecured)
    {
      m_bIsSecuredSpace = _bIsSecured;
    }
    
    /// <summary>
    /// This method turns on and off overwriting of sub-spaces when they register
    /// with this AActivitySpace.
    /// If bOverWrite is false, the space will give an error and ignore requests
    /// to overwrite a sub-space.  This is the default state.
    /// </summary>
    /// <param name="boolean">
    /// true implies allow overwriting of sub-space
    /// false implies ignore attempts to overwrite a sub-space (default)
    /// </param>
    public virtual void SetOverWriteSubSpaces(bool bOverWrite)
    {
      m_bOverWriteSubSpaces = bOverWrite;
    }
    
    /// <summary>
    /// Set the name and ID of the parent AActivitySpace (i.e. the one that launched
    /// this space).  If there is no parent space, then this method does not need to
    /// be called.
    /// </summary>
    /// <param name="String">spaceName the name of the parent space</param>
    /// <param name="String">spaceID the ID of the parent space</param>
    public virtual void SetParentSpace(String spaceName, String spaceID)
    {
      // Bug 66243 - Check against ASManager and set the space name if it exists. 
      // The ASManager holds all the ActivitySpaces of the portal. This is basically
      // any class that extends AActivitySpace and this is initialized when the portal starts up.
      if (null != spaceName && ASManager.INSTANCE.GetSpaceExists(spaceName))
      {
        m_strParentSpaceName = spaceName;
      }
      // Case if space ID is "None" which is considered a valid value.
      if (ASCache.SPACEID_INVALID.Equals(spaceID))
      {
        m_strParentSpaceID = spaceID;
      } else
      {
        // DCA - check to make sure parent space exist before setting it
        // Get the Activity Space cache from the user's session and check for the parent space.
        ASCache myCache = (ASCache) m_smSession.GetAttribute(ASConstants.SANDBOXASCACHE);
        if (null != myCache && null != spaceID)
        {
          // Set the space ID if we can get the parent space from the cache. This should be
          // the usual case when a valid space ID is passed in.
          AActivitySpace parentSpace = myCache.GetSpace(spaceName, spaceID);
          if (null != parentSpace)
          {
            m_strParentSpaceID = spaceID;
          } else
          {
            // If parent space is not valid or not in the cache, we shouldn't set the space ID.
            // Instead just set it to "None". This is to guard against junk getting passed in.
            m_strParentSpaceID = ASCache.SPACEID_INVALID;
          }
        }
      }
    }
    
    /// <summary>
    /// Sets the unique space ID for this AActivitySpace.
    /// This method must be called before Init();
    /// </summary>
    /// <param name="the">unique space ID for this AActivitySpace</param>
    public virtual void SetSpaceID(String spaceID)
    {
      m_strSpaceID = spaceID;
    }
    
    /// <summary>
    /// Sets the subsession that the AActivitySpace can use to store cross-space data.
    /// This SubSession will be cleared on logout.
    /// This method must be called before Init();
    /// </summary>
    /// <param name="the">
    /// subsession that the AActivitySpace can use to store
    /// cross-space data.
    /// </param>
    public virtual void SetSubSession(ISessionManager session)
    {
      m_smSession = session;
    }
    
    /// <summary>
    /// Sets the persistent subsession that the AActivitySpace can use to store
    /// cross-space data across multiple login/logout cycles.
    /// This SubSession will not be cleared on logout.
    /// This method must be called before Init();
    /// </summary>
    /// <param name="the">
    /// subsession that the AActivitySpace can use to store
    /// cross-space data.
    /// </param>
    public virtual void SetPersistentSubSession(ISessionManager session)
    {
      m_smPersistentSession = session;
    }
    /////////////////////////////////////////////////////////////////////////////////
    // General Set Accessor Methods
    //
    // These methods will be used by the Interpreter to initialize the AActivitySpace
    // and will be called before Init()
    /////////////////////////////////////////////////////////////////////////////////
    
    /// <summary>
    /// Sets the sub-space key.  If this is null, this is
    /// not a sub-space.  By default, spaces are not sub-spaces.
    /// This key is used by the parent AActivitySpace to identify sub-spaces
    /// (in case there are multiple instances of the same type of sub-space).
    /// The sub-space key is different from the space ID.
    /// </summary>
    /// <param name="String">sub-space key</param>
    public virtual void SetSubSpaceKey(String sSubSpaceKey)
    {
      m_sSubSpaceKey = sSubSpaceKey;
      if ("".Equals(m_sSubSpaceKey))
      {
        m_sSubSpaceKey = null;
      }
    }
    
    /// <summary>
    /// This method turns on and off automatic tracking of the current IDisplayPage.
    /// The Interpreter calls SetCurrentPage at the end of every request with the
    /// most recently viewed IDisplayPage.
    /// If bTrack is false, MainAS will ignore this information.  This is the default
    /// state.
    /// </summary>
    /// <param name="boolean">
    /// true implies track current page information
    /// false implies ignore page tracking information (default)
    /// </param>
    public virtual void SetTrackCurrentPage(bool bTrack)
    {
      m_bTrackCurrentPage = bTrack;
    }
    
    /// <summary>
    /// Set the user's Session.  This is a generic object that can be used
    /// for different kinds of user sessions.  For the portal it is an
    /// IPTSession.
    /// This will be called before Init() and CheckBasicAccess() are called.
    /// </summary>
    /// <param name="Object">userSession to be set</param>
    public virtual void SetUserSession(Object userSession)
    {
      m_UserSession = userSession;
    }
    /////////////////////////////////////////////
    // Register / Get methods for managed objects
    /////////////////////////////////////////////
    
    /// <summary>
    /// Un-registers the requested view (removes it from AActivitySpace)
    /// </summary>
    /// <param name="String">the name of the view to un register</param>
    public virtual void UnRegisterPage(String pageName)
    {
      m_mPageMap.RemoveElement(pageName);
      // Set the current page if it's not set so AActivitySpaces with only one
      // IDisplayPage don't need to worry about this.
      if (m_asdpCurrentPage.GetName().Equals(pageName))
      {
        IXPEnumerator temp = m_mPageMap.GetElements();
        temp.MoveNext();
        m_asdpCurrentPage = (IDisplayPage) temp.GetCurrent();
      }
    }
    
    /// <summary>
    /// Un-registers the requested view (removes it from AActivitySpace)
    /// </summary>
    /// <param name="String">the name of the view to un register</param>
    public virtual void UnRegisterView(String viewName)
    {
      m_mViewMap.RemoveElement(viewName);
    }
    
    /// <summary>
    /// Un-registers the requested control (removes it from AActivitySpace)
    /// </summary>
    /// <param name="String">the name of the control to unregister</param>
    public virtual void UnRegisterControl(String controlName)
    {
      m_mControlMap.RemoveElement(controlName);
    }
    
    /// <summary>
    /// Un-registers the requested model (removes it from AActivitySpace)
    /// </summary>
    /// <param name="String">the name of the model to unregister</param>
    public virtual void UnRegisterModel(String modelName)
    {
      m_mModelMap.RemoveElement(modelName);
    }
    
    /// <summary>
    /// This method registers the IDisplayPage so the AActivitySpace can manage it
    /// automatically.
    /// It also initializes it with the AActivitySpace argument.
    /// This operation will fail and return false if the IDisplayPage is null or if
    /// there is already a IDisplayPage with the same name registered with the
    /// AActivitySpace.
    /// The first DisplayPage will be set as the current IDisplayPage.
    /// </summary>
    /// <param name="IDisplayPage">page to register</param>
    /// <returns>
    /// boolean true implies success
    /// false implies failure
    /// </returns>
    public virtual bool RegisterPage(IDisplayPage page)
    {
      if (page == null)
      {
        return false;
      }
      return RegisterPage(page, page.GetName());
    }
    
    /// <summary>
    /// This method registers the IDisplayPage so the AActivitySpace can manage it
    /// automatically.
    /// It also initializes it with the AActivitySpace argument.
    /// This operation will fail and return false if the IDisplayPage is null or if
    /// there is already a IDisplayPage with the same name registered with the
    /// AActivitySpace.
    /// The first DisplayPage will be set as the current IDisplayPage.
    /// </summary>
    /// <param name="IDisplayPage">page to register</param>
    /// <param name="String">storageName the name to store the IDisplayPage under</param>
    /// <returns>
    /// boolean true implies success
    /// false implies failure
    /// </returns>
    public virtual bool RegisterPage(IDisplayPage page, String strStorageName)
    {
      if (page == null)
      {
        return false;
      }
      // Put the page into the map
      Object mOldPage = m_mPageMap.PutElement(strStorageName, page);
      // If there was already a page there, put it back and fail
      if (mOldPage != null)
      {
        log.Error(GetName() + " RegisterPage failed: Page already registered " + strStorageName);
        m_mPageMap.PutElement(strStorageName, mOldPage);
        return false;
      }
      // Initialize the page
      page.Init(this);
      // Set the current page if it's not set so AActivitySpaces with only one
      // IDisplayPage don't need to worry about this.
      if (m_asdpCurrentPage == null)
      {
        m_asdpCurrentPage = page;
      }
      // SE: Bug 62787 - Add diagnostics for tracking MVC registration and displaying
      log.Debug(GetName() + " RegisterPage succeeded: " + page.GetName() + " registered");
      return true;
    }
    /////////////////////
    // Referenced Strings
    /////////////////////
    
    /// <summary>
    /// This creates a reference to a long query string parameter.  Currently
    /// only the tree knows how to unreference this for certain parameters.
    /// </summary>
    /// <param name="value"></param>
    /// <returns>
    /// returns the key that is used to unref the value - This needs
    /// to be preceded with "@" for this to be used.
    /// </returns>
    public virtual String refString(String value)
    {
      if (value == null)
      {
        return null;
      }
      if (m_stringReferences == null)
      {
        m_stringReferences = new XPHashtable();
      }
      String key = XPConvert.ToString(value.GetHashCode());
      m_stringReferences.PutElement(key, value);
      return key;
    }
    
    /// <summary>
    /// This unreferences a key to a long query string parameter
    /// </summary>
    /// <param name="key">-- This must not have the "@" in front of it</param>
    /// <returns>returns value or null if there was no value</returns>
    public virtual String unrefString(String key)
    {
      if ((key == null) || (m_stringReferences == null))
      {
        return null;
      }
      String result = (String) m_stringReferences.GetElement(key);
      // Currently, we don't remove this.  This is a minor temporary
      // memory leak, or at least overuse of memory.  This could be
      // improved.
      // (let it die with the AS) m_stringReferences.RemoveElement(key);
      return result;
    }
    
    /// <summary>
    /// This method is used to return whether this Activity Space
    /// has in place refresh enabled.  If it is enabled, then we 
    /// can, for example, include the JavaScript for JSXML.
    /// This method should be used carefully in shared components because 
    /// if shared components use this method, then any time you add any 
    /// in-place to an AS, you will have to implement it for all the 
    /// shared components. Shared components that require you to implement 
    /// a partial content display method should use another way to determine 
    /// if they should be enabled or not.
    /// </summary>
    /// <returns>boolean True if this AS has enabled in place refresh.</returns>
    public virtual bool GetIsInPlaceRefreshEnabled()
    {
      return m_bInPlaceRefreshEnabled;
    }
    
    /// <summary>
    /// This method is used to determine whether this Activity Space
    /// has in place refresh enabled.  If it is enabled, then we 
    /// can, for example, include the JavaScript for JSXML.
    /// This method should be used carefully in shared components because 
    /// if shared components use this method, then any time you add any 
    /// in-place to an AS, you will have to implement it for all the 
    /// shared components. Shared components that require you to implement 
    /// a partial content display method should use another way to determine 
    /// if they should be enabled or not.
    /// </summary>
    /// <param name="boolean">True to enabled this AS for in place refresh.</param>
    public virtual void SetIsInPlaceRefreshEnabled(bool IsInPlaceRefreshEnabled)
    {
      m_bInPlaceRefreshEnabled = IsInPlaceRefreshEnabled;
    }
    
    /// <summary>
    /// This method is used to return whether Display Pages
    /// should currently be displaying in place content instead 
    /// of their normal content.  
    /// </summary>
    /// <returns>boolean True to show in place content.</returns>
    public virtual bool GetIsInPlaceRefreshDisplayOn()
    {
      return m_bInPlaceRefreshDisplayOn;
    }
    
    /// <summary>
    /// This method is used to determin whether Display Pages
    /// should currently be displaying in place content instead 
    /// of their normal content.  
    /// </summary>
    /// <param name="boolean">True to show in place content.</param>
    public virtual void SetIsInPlaceRefreshDisplayOn(bool IsInPlaceRefreshDisplayOn)
    {
      m_bInPlaceRefreshDisplayOn = IsInPlaceRefreshDisplayOn;
    }
  }
}