using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace.utils;
using com.plumtree.uiinfrastructure.interpreter;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.uiinfrastructure.activityspace
{
  /// <summary>
  /// This handles the cache for developers so that it can't
  /// get into an inconsistent state.
  /// Note: ClearCache() should not be used.  This is only for
  /// debug purposes.
  /// </summary>
  /// <author>Don Hayler</author>
  public class ASCache
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.activityspace.ASCache");
    private readonly ISessionManager m_smCache;
    private readonly XPHashtable m_xpmCacheList;
    
    /// <summary>
    /// internal cache constants 
    /// </summary>
    private const int m_nMAX_CACHE_SIZE = 5;
    private const int m_nMIN_CACHE_SIZE = 3;
    private const int m_nTIME_TO_LIVE_MINUTES = 7;
    
    /// <summary>
    /// Used by the gateway in the interpreter, since it doesn't use cache IDs.
    /// This should not be used by Activity Space developers.  This will trigger
    /// different Activity Space caching (see Interpreter.HandleCacheLimit for
    /// details)
    /// </summary>
    public const String SPACEID_INVALID = "None";
    
    /// <summary>
    /// Constructor.
    /// </summary>
    /// <param name="ISessionManager">the cache to wrap</param>
    public ASCache(ISessionManager cache, XPHashtable xpmCacheList)
    {
      m_smCache = cache;
      m_xpmCacheList = xpmCacheList;
    }
    
    /// <summary>
    /// Returns an enumaration of the keys for the ASCache.
    /// </summary>
    /// <returns>IXPEnumerator Keys</returns>
    public virtual IXPEnumerator GetKeys()
    {
      return m_smCache.GetKeys();
    }
    
    /// <summary>
    /// Returns the value corresponding to the key in argument.
    /// This method will reset the timestamp on the requested space.
    /// </summary>
    /// <param name="sKey">Key of the requested value</param>
    /// <returns>Object Value</returns>
    public virtual Object GetAttribute(String sKey)
    {
      return m_smCache.GetAttribute(sKey);
    }
    
    /// <summary>
    /// Returns whether or not the space corresponding to the key in argument
    /// is a subspace.
    /// This method will not reset the timestamp on the requested space.
    /// </summary>
    /// <param name="sKey">Key of the requested value</param>
    /// <returns>boolean true implies the requested space is a sub-space.</returns>
    public virtual bool GetSpaceIsSubSpace(String sKey)
    {
      CacheListItem listItem = (CacheListItem) m_xpmCacheList.GetElement(sKey);
      if (null != listItem)
      {
        return listItem.m_bIsSubSpace;
      }
      return false;
    }
    
    /// <summary>
    /// This method returns the requested AActivitySpace
    /// from the cache.
    /// This method will reset the timestamp on the requested space.
    /// </summary>
    /// <param name="String">name the name of the AActivitySpace</param>
    /// <param name="String">ID the cache ID of the AActivitySpace</param>
    /// <returns>AActivitySpace the requested space</returns>
    public virtual AActivitySpace GetSpace(String strName, String nID)
    {
      if (SPACEID_INVALID.Equals(nID))
      {
        log.Info("GetSpace called with non-cached space ID for space " + strName);
        return null;
      }
      if ((null == nID) || ("".Equals(nID)))
      {
        log.Warn("GetSpace called with empty or null ID for space " + strName);
        return null;
      }
      AActivitySpace result = (AActivitySpace) m_smCache.GetAttribute(nID);
      if (null != strName && (result == null || !result.GetName().Equals(strName)))
      {
        if (log.IsDebugEnabled())
        {
          log.Debug("Unable to find cached space with name " + strName + " and ID " + nID + ".");
          }
        return null;
      }
      return result;
    }
    
    /// <summary>
    /// This method returns the requested AActivitySpace
    /// from the cache, if it exists.
    /// This method will reset the timestamp on the requested space.
    /// </summary>
    /// <param name="String">name the name of the AActivitySpace</param>
    /// <returns>
    /// AActivitySpace the requested space, if it is cached.
    /// may return null.
    /// </returns>
    public virtual AActivitySpace GetSpaceIfCached(String strName)
    {
      if ((strName == null) || ("".Equals(strName)))
      {
        log.Info("GetSpaceIfCached called with null or empty name.");
        return null;
      }
      AActivitySpace result = null;
      // Try to find previously cached AActivitySpace without using ID
      IXPEnumerator spaceKeys = m_xpmCacheList.GetKeys();
      while (spaceKeys.MoveNext())
      {
        if (strName.Equals(((CacheListItem) m_xpmCacheList.GetElement(spaceKeys.GetCurrent())).m_strName))
        {
          result = (AActivitySpace) m_smCache.GetAttribute((String) spaceKeys.GetCurrent());
          break;
        }
      }
      return result;
    }
    
    /// <summary>
    /// This method removes the specified sub-space.
    /// This will fail if the specified space is not a sub-space.
    /// </summary>
    /// <param name="String">name the name of the AActivitySpace</param>
    /// <param name="String">ID the cache ID of the AActivitySpace</param>
    public virtual void RemoveSubSpace(String name, String ID)
    {
      if (SPACEID_INVALID.Equals(ID))
      {
        log.Info("RemoveSubSpace called with non-cached space ID for space " + name);
        return;
      }
      AActivitySpace result = (AActivitySpace) m_smCache.GetAttribute(ID);
      if ((result != null) && result.GetName().Equals(name) && (result.GetSubSpaceKey() != null))
      {
        // Remove this sub-space from the cache
        m_smCache.RemoveAttribute(ID);
        // Remove all sub-spaces of this sub-space from the cache.
        RemoveSubSpacesFromCache(ID);
      } else
      {
        log.Warn("Unable to remove cached space with name " + name + " and ID " + ID + ".");
        }
    }
    
    /// <summary>
    /// This method clears the AActivitySpace cache.  This should not be used
    /// and is only here for debugging purposes.  Doing so will remove all
    /// cached spaces and cause them to be regenerated.  All editors will fail
    /// when this happens.
    /// </summary>
    public virtual void ClearCache()
    {
      m_smCache.Clear();
      m_xpmCacheList.Clear();
    }
    
    /// <summary>
    /// This method handles cache management after the request has been sent to the user.
    /// This should only be called by the Interpreter.
    /// </summary>
    public virtual void HandleCacheLimit()
    {
      // When spaces are opened from a gateway space, they do not know their
      // parent space name or id.  This means that they cannot properly set
      // themselves as sub-spaces of their parent space.  Therefore, they
      // can fill up the cache and force out the most recent non-gatewayed
      // space.  To solve this problem, spaces opened from a gatewayed
      // space must be treated differently in the cache.
      // We must also be careful not to let those same gateway generated
      // spaces hang around forever in the cache.  Therefore, when they
      // get older than the oldest normal space, we remove the oldest
      // one.
      bool bRemovedSpace = false;
      String strCurrentKey;
      CacheListItem curItem;
      int nMainSpacesCount = 0;
      String strOldestMainASKey = null;
      long nOldestMainASTimeStamp = -1;
      long nYoungestMainASTimeStamp = -1;
      int nFromGatewayASCount = 0;
      String strOldestFromGatewayASKey = null;
      long nOldestFromGatewayASTimeStamp = -1;
      IXPEnumerator cacheKeys = m_xpmCacheList.GetKeys();
      // Calculate AS cache size and find oldest main space
      // and oldest space from a gatewayed space.
      while (cacheKeys.MoveNext())
      {
        strCurrentKey = (String) cacheKeys.GetCurrent();
        curItem = (CacheListItem) m_xpmCacheList.GetElement(strCurrentKey);
        log.Debug("Cached space with key " + strCurrentKey + " and timestamp " + m_smCache.GetTimeStamp(strCurrentKey));
        if (!curItem.m_bIsSubSpace && !curItem.m_bIsFromGatewaySpace)
        {
          // Not a sub-space or a space generated from a gatewayed space
          nMainSpacesCount++;
          if ((nOldestMainASTimeStamp == -1) || (m_smCache.GetTimeStamp(strCurrentKey) < nOldestMainASTimeStamp))
          {
            // Record oldest main space
            nOldestMainASTimeStamp = m_smCache.GetTimeStamp(strCurrentKey);
            strOldestMainASKey = strCurrentKey;
          }
          if ((nYoungestMainASTimeStamp == -1) || (m_smCache.GetTimeStamp(strCurrentKey) > nYoungestMainASTimeStamp))
          {
            // Record youngest main space
            nYoungestMainASTimeStamp = m_smCache.GetTimeStamp(strCurrentKey);
            }
        } else
        if (curItem.m_bIsFromGatewaySpace)
        {
          nFromGatewayASCount++;
          if ((nOldestFromGatewayASTimeStamp == -1) || (m_smCache.GetTimeStamp(strCurrentKey) < nOldestFromGatewayASTimeStamp))
          {
            nOldestFromGatewayASTimeStamp = m_smCache.GetTimeStamp(strCurrentKey);
            strOldestFromGatewayASKey = strCurrentKey;
          }
        }
      }
      // This assumes that the youngest space will be the current space, which will
      // have the current time.
      // (Multiply by 60000 to convert minutes to milliseconds)
      long nTimeToLive = nYoungestMainASTimeStamp - (m_nTIME_TO_LIVE_MINUTES * 60000);
      log.Debug("ASCache: TimeToLive = " + nTimeToLive + " & youngest = " + nYoungestMainASTimeStamp + " & oldest = " + nOldestMainASTimeStamp + " & cache size = " + nMainSpacesCount);
      // If we are over the cache size for normal activity spaces
      if ((nMainSpacesCount > m_nMAX_CACHE_SIZE) || 
      // or the oldest space is past the time to live limit and
      // the cache is over the minimum size
      ((nOldestMainASTimeStamp < nTimeToLive) && (nMainSpacesCount > m_nMIN_CACHE_SIZE)))
      {
        // Remove the oldest space
        AActivitySpace deadSpace = (AActivitySpace) m_smCache.GetAttribute(strOldestMainASKey);
        if (log.IsDebugEnabled())
        {
          log.Debug("Removing main space from AS cache: " + strOldestMainASKey);
          }
        m_smCache.RemoveAttribute(strOldestMainASKey);
        m_xpmCacheList.RemoveElement(strOldestMainASKey);
        if (deadSpace != null)
        {
          deadSpace.CleanupAllData();
          // Remove sub-spaces of the oldest space
          RemoveSubSpacesFromCache(deadSpace.GetSpaceID());
        }
        bRemovedSpace = true;
      }
      // If we are over the cache size for spaces generated from a gatewayed space
      // or the oldest gateway generated space is older than the oldest normal space
      // This is done to ensure that gateway generated spaces don't hang around
      // forever if you generate a space from the gateway once and never do it again.
      if ((nFromGatewayASCount > m_nMAX_CACHE_SIZE) || ((nOldestFromGatewayASTimeStamp != -1) && (nOldestFromGatewayASTimeStamp < nOldestMainASTimeStamp)))
      {
        // Remove the oldest space
        AActivitySpace deadSpace = (AActivitySpace) m_smCache.GetAttribute(strOldestFromGatewayASKey);
        log.Info("Removing gateway generated space from AS cache: " + strOldestFromGatewayASKey);
        m_smCache.RemoveAttribute(strOldestFromGatewayASKey);
        m_xpmCacheList.RemoveElement(strOldestFromGatewayASKey);
        if (deadSpace != null)
        {
          deadSpace.CleanupAllData();
          // Remove sub-spaces of the oldest space
          RemoveSubSpacesFromCache(deadSpace.GetSpaceID());
        }
        bRemovedSpace = true;
      }
      if (bRemovedSpace)
      {
        // TODOCR: DONH - We should not use recursion here.  A while loop would be
        // safer.
        // If we removed any spaces, we need to check again and make sure we are not
        // under the limit.  If we don't do this, we can overflow the cache.  If a user
        // request creates a space, and then creates an error space, we will have added
        // two spaces to the cache in a single request,while only removing one (not counting
        // sub-spaces).  This can cause the cache to overflow.
        // We have to make sure this doesn't get into an infinite loop, however.
        HandleCacheLimit();
      }
    }
    
    /// <summary>
    /// This helper method removes sub-spaces of a dead space, and removes
    /// sub-spaces of each sub-space recursively.
    /// </summary>
    private void RemoveSubSpacesFromCache(String strDeadSpaceID)
    {
      if (strDeadSpaceID == null)
      {
        // This shouldn't happen either, but ...
        return;
      }
      CacheListItem curItem;
      XPArrayList itemsToRemove = new XPArrayList();
      AActivitySpace deadSpace;
      // remove sub-spaces of the dead space
      IXPEnumerator cacheKeys = m_xpmCacheList.GetKeys();
      while (cacheKeys.MoveNext())
      {
        curItem = (CacheListItem) m_xpmCacheList.GetElement((String) cacheKeys.GetCurrent());
        // curItem shouldn't ever be null, but apparently it can be sometimes.
        if ((curItem != null) && curItem.m_bIsSubSpace && strDeadSpaceID.Equals(curItem.m_strParentSpaceID))
        {
          itemsToRemove.Add(cacheKeys.GetCurrent());
        }
      }
      int nItemsToRemove = itemsToRemove.GetSize();
      if (nItemsToRemove > 0)
      {
        log.Info("Removing " + itemsToRemove.GetSize() + " sub-spaces from AS cache.");
        for (int i = 0;i < nItemsToRemove;i++)
        {
          deadSpace = (AActivitySpace) m_smCache.GetAttribute((String) itemsToRemove.GetElement(i));
          m_smCache.RemoveAttribute((String) itemsToRemove.GetElement(i));
          m_xpmCacheList.RemoveElement((String) itemsToRemove.GetElement(i));
          if (deadSpace != null)
          {
            deadSpace.CleanupAllData();
            // remove any sub-spaces of this sub-space
            RemoveSubSpacesFromCache(deadSpace.GetSpaceID());
          }
        }
      }
    }
    
    /// <summary>
    /// Gets the ASCache session manager off of the top-level session manager,
    /// creating it if necessary; made into a separate method so the SSO login
    /// code can call it.
    /// Creates the cache (if necessary) and puts a safe wrapper around it
    /// into the user's sandbox session.
    /// This should only be used by the Interpreter and SSO login code.
    /// </summary>
    /// <param name="session"></param>
    /// <param name="smSandbox"></param>
    /// <returns>ISessionManager</returns>
    public static ISessionManager GetASCache(ISessionManager session, ISessionManager smSandbox)
    {
      ISessionManager smCache = session.GetSubSession(InterpreterAttributes.SPACEATTRIBUTE);
      // Set up the cache if it's brand new.
      if (null == smSandbox.GetAttribute(ASConstants.SANDBOXASCACHE))
      {
        // Put the cache key list in place.
        XPHashtable cacheList = new XPHashtable();
        session.SetAttribute(InterpreterAttributes.CACHELISTATTRIBUTE, cacheList);
        // Put the User's AS Cache object in place.
        ASCache cacheObject = new ASCache(smCache, cacheList);
        smSandbox.SetAttribute(ASConstants.SANDBOXASCACHE, cacheObject);
      }
      return smCache;
    }
    
    /// <summary>
    /// Gets the CacheList off of the top-level session manager.
    /// This should only be used by the Interpreter and SSO login code.
    /// </summary>
    /// <param name="session"></param>
    /// <returns>XPHashtable</returns>
    public static XPHashtable GetCacheList(ISessionManager session)
    {
      return (XPHashtable) session.GetAttribute(InterpreterAttributes.CACHELISTATTRIBUTE);
      }
  }
}