using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.xml.dom;
using com.plumtree.openlog;
namespace com.plumtree.uiinfrastructure.activityspace
{
  /// <summary>
  /// The ASManager is a singleton class that returns Activity Spaces,
  /// Models, Views, & Controls. This uses dynamic discovery to import
  /// all of the IActivitySpace, IModel, IView, and IControl objects
  /// in certain locations, and then maintain a hashmap of clones,
  /// with possible resource pooling.
  /// The maps are initialized at startup, and are then assumed to be
  /// read only from that point on.  Therefore, they should not need
  /// to be locked.
  /// </summary>
  /// <author>Don Hayler</author>
  public class ASManager
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.activityspace.ASManager");
    // constants for interfaces to look for
    
    /// <summary>
    /// model interface 
    /// </summary>
    private const String IMODEL = "com.plumtree.uiinfrastructure.activityspace.IModel";
    
    /// <summary>
    /// view interface 
    /// </summary>
    private const String IVIEW = "com.plumtree.uiinfrastructure.activityspace.IView";
    
    /// <summary>
    /// control interface 
    /// </summary>
    private const String ICONTROL = "com.plumtree.uiinfrastructure.activityspace.IControl";
    
    /// <summary>
    /// activity space interface 
    /// </summary>
    private const String AACTIVITYSPACE = "com.plumtree.uiinfrastructure.activityspace.AActivitySpace";
    
    /// <summary>
    /// pagecontrol interface 
    /// </summary>
    private const String IPAGECONTROL = "com.plumtree.uiinfrastructure.activityspace.IPageControl";
    
    /// <summary>
    /// name of the jar file that the above interfaces are in. 
    /// </summary>
    private const String INFRASTRUCTURE = "uiinfrastructure";
    
    /// <summary>
    /// The exception that caused the system to fail startup. 
    /// </summary>
    public Exception m_StartupFailure = null;
    
    /// <summary>
    /// Use this class static to access the one and only instance of
    /// ActivitySpaceManager.
    /// </summary>
    public static readonly ASManager INSTANCE = new ASManager();
    
    /// <summary>
    /// whether or not the system was able to start up correctly. 
    /// </summary>
    public bool m_bSystemStartupSuccesful = false;
    // Internal arrays to store MVC objects in
    
    /// <summary>
    /// map of the controls 
    /// </summary>
    private XPHashtable m_xphControls = new XPHashtable();
    
    /// <summary>
    /// map of the models 
    /// </summary>
    private XPHashtable m_xphModels = new XPHashtable();
    
    /// <summary>
    /// map of the spaces 
    /// </summary>
    private XPHashtable m_xphSpaces = new XPHashtable();
    
    /// <summary>
    /// map of the views 
    /// </summary>
    private XPHashtable m_xphViews = new XPHashtable();
    
    /// <summary>
    /// map of the page controls 
    /// </summary>
    private XPHashtable m_xphPageControls = new XPHashtable();
    
    /// <summary>
    /// Whether or not this object has been initialized 
    /// </summary>
    private bool m_bInitialized = false;
    
    /// <summary>
    /// Counters used in the diagnostic page 
    /// </summary>
    private int m_nNumCustomControls = 0;
    private int m_nNumCustomModels = 0;
    private int m_nNumCustomSpaces = 0;
    private int m_nNumCustomViews = 0;
    private int m_nNumCustomPageControls = 0;
    
    /// <summary>
    /// Private constructor to ensure there is only one instance.
    /// </summary>
    private ASManager()
    {
    }
    
    /// <summary>
    /// Create the requested Control.
    /// This method should not be called by developers.  It is only for
    /// AActivitySpace.
    /// </summary>
    /// <param name="controlName">The name of the control to create</param>
    /// <returns>The control, null on failure</returns>
    public virtual IControl GetControl(String controlName)
    {
      IControl myControl = null;
      try
      {
        try
        {
          Object temp = m_xphControls.GetElement(controlName);
          myControl = (IControl) temp;
        }
        catch (Exception ex)
        {
          throw XPException.GetInstance(ex);
        }
      }
      catch (XPClassCastException e)
      {
        log.Error(e, "GetControl: XPClassCastException: The control " + controlName + " is being miscast.");
        }
      catch (XPException e)
      {
        log.Error(e, "Couldn\'t retrieve control " + controlName + ".");
      }
      try
      {
        myControl = (IControl) myControl.Create();
      }
      catch (Exception e)
      {
        log.Error(e, "Couldn\'t create control " + controlName + ".");
        myControl = null;
      }
      return myControl;
    }
    
    /// <summary>
    /// Checks to determine if specified control has been loaded by dynamic
    /// discovery into the hashtable of controls.
    /// </summary>
    /// <param name="controlName">Control name</param>
    /// <returns>True if exists. Otherwise false.</returns>
    public virtual bool GetControlExists(String controlName)
    {
      try
      {
        return m_xphControls.ContainsKey(controlName);
      }
      catch (Exception ex)
      {
        return false;
      }
    }
    
    /// <summary>
    /// Get the table containing the page controls
    /// </summary>
    /// <returns>XPHashtable</returns>
    public virtual XPHashtable GetPageControlTable()
    {
      return m_xphPageControls;
    }
    
    /// <summary>
    /// Create the Model specified by the model name.
    /// This method should not be called by developers.  It is only for
    /// AActivitySpace.
    /// </summary>
    /// <param name="modelName">The name of the model to be created</param>
    /// <returns>the requested model, null on failure</returns>
    public virtual IModel GetModel(String modelName)
    {
      IModel myModel = null;
      try
      {
        try
        {
          Object temp = m_xphModels.GetElement(modelName);
          myModel = (IModel) temp;
        }
        catch (Exception ex)
        {
          throw XPException.GetInstance(ex);
        }
      }
      catch (XPClassCastException e)
      {
        log.Error(e, "GetModel: XPClassCastException: The model " + modelName + " is being miscast.");
        }
      catch (XPException e)
      {
        log.Error(e, "Couldn\'t retrieve model " + modelName + ".");
      }
      try
      {
        myModel = (IModel) myModel.Create();
      }
      catch (Exception e)
      {
        log.Error(e, "Couldn\'t create model " + modelName + ".");
        myModel = null;
      }
      return myModel;
    }
    
    /// <summary>
    /// Checks to determine if specified model has been loaded by dynamic
    /// discovery into the hashtable of models.
    /// </summary>
    /// <param name="modelName">Model name</param>
    /// <returns>True if exists. Otherwise false.</returns>
    public virtual bool GetModelExists(String modelName)
    {
      try
      {
        return m_xphModels.ContainsKey(modelName);
      }
      catch (Exception ex)
      {
        return false;
      }
    }
    
    /// <summary>
    /// Create the requested Activity Space.
    /// This method should not be called by developers.  It is only for
    /// the Interpreter.
    /// </summary>
    /// <param name="spaceName">The name of the space to create</param>
    /// <returns>The requested space, null on failure</returns>
    public virtual AActivitySpace GetSpace(String spaceName)
    {
      AActivitySpace mySpace = null;
      try
      {
        try
        {
          Object temp = m_xphSpaces.GetElement(spaceName);
          mySpace = (AActivitySpace) temp;
        }
        catch (Exception ex)
        {
          throw XPException.GetInstance(ex);
        }
      }
      catch (XPClassCastException ex)
      {
        log.Error(ex, "GetSpace: XPClassCastException: The space " + spaceName + " is being miscast.");
        }
      catch (XPException e)
      {
        log.Error(e, "Couldn\'t retrieve space " + spaceName + ".");
      }
      try
      {
        mySpace = (AActivitySpace) mySpace.Create();
      }
      catch (Exception e)
      {
        log.Error(e, "Couldn\'t create space " + spaceName + ".");
        mySpace = null;
      }
      return mySpace;
    }
    
    /// <summary>
    /// Checks to determine if specified activity space has been loaded by dynamic
    /// discovery into the hashtable of activity spaces.
    /// </summary>
    /// <param name="spaceName">Activity space name</param>
    /// <returns>True if exists. Otherwise false.</returns>
    public virtual bool GetSpaceExists(String spaceName)
    {
      try
      {
        return m_xphSpaces.ContainsKey(spaceName);
      }
      catch (Exception ex)
      {
        return false;
      }
    }
    
    /// <summary>
    /// Create the view specified by the view name.
    /// This method should not be called by developers.  It is only for
    /// AActivitySpace.
    /// </summary>
    /// <param name="viewName">The name of the view to create</param>
    /// <returns>the requested view, null on failure</returns>
    public virtual IView GetView(String viewName)
    {
      IView myView = null;
      try
      {
        try
        {
          Object temp = m_xphViews.GetElement(viewName);
          myView = (IView) temp;
        }
        catch (Exception ex)
        {
          throw XPException.GetInstance(ex);
        }
      }
      catch (XPClassCastException e)
      {
        log.Error(e, "GetView: XPClassCastException: The view " + viewName + " is being miscast.");
        }
      catch (XPException e)
      {
        log.Error(e, "Couldn\'t retrieve view " + viewName + ".");
      }
      try
      {
        myView = (IView) myView.Create();
      }
      catch (Exception e)
      {
        log.Error(e, "Couldn\'t create view " + viewName + ".");
        myView = null;
      }
      return myView;
    }
    
    /// <summary>
    /// Checks to determine if specified view has been loaded by dynamic
    /// discovery into the hashtable of views.
    /// </summary>
    /// <param name="viewName">View name</param>
    /// <returns>True if exists. Otherwise false.</returns>
    public virtual bool GetViewExists(String viewName)
    {
      try
      {
        return m_xphViews.ContainsKey(viewName);
      }
      catch (Exception ex)
      {
        return false;
      }
    }
    
    /// <summary>
    /// Create the page control specified by the pagecontrol name.
    /// This method should not be called by developers.  It is only for
    /// AActivitySpace.
    /// </summary>
    /// <param name="pagecontrolName">The name of the pagecontrol to create</param>
    /// <returns>the requested pagecontrol, null on failure</returns>
    public virtual IPageControl GetPageControl(String pageControlName)
    {
      IPageControl myPageControl = null;
      try
      {
        try
        {
          Object temp = m_xphPageControls.GetElement(pageControlName);
          myPageControl = (IPageControl) temp;
        }
        catch (Exception ex)
        {
          throw XPException.GetInstance(ex);
        }
      }
      catch (XPClassCastException e)
      {
        log.Error(e, "GetPageControl: XPClassCastException: The pagecontrol " + pageControlName + " is being miscast.");
        }
      catch (XPException e)
      {
        log.Error(e, "Couldn\'t retrieve view " + pageControlName + ".");
        }
      try
      {
        if (myPageControl != null)
        {
          myPageControl = (IPageControl) myPageControl.Create();
        }
      }
      catch (Exception e)
      {
        log.Error(e, "Couldn\'t create pagecontrol " + pageControlName + ".");
        myPageControl = null;
      }
      return myPageControl;
    }
    
    /// <summary>
    /// Checks to determine if specified page control has been loaded by dynamic
    /// discovery into the hashtable of pagecontrols.
    /// </summary>
    /// <param name="pageControlName">pagecontrol name</param>
    /// <returns>True if exists. Otherwise false.</returns>
    public virtual bool GetPageControlExists(String pageControlName)
    {
      try
      {
        return m_xphPageControls.ContainsKey(pageControlName);
      }
      catch (Exception ex)
      {
        return false;
      }
    }
    
    /// <summary>
    /// This method initializes the ASManager and should only be called by the
    /// warmup servlet.  This method can only be called once.
    /// </summary>
    /// <param name="strLibDir">The file directory where jars and dlls can be found</param>
    /// <param name="strConfigFile">the name of the config file</param>
    public virtual void LoadClasses(String strLibDir, String strConfigFile)
    {
      if (m_bInitialized)
      {
        log.Error("ASManager LoadClasses called twice.");
        return;
      }
      m_bInitialized = true;
      log.Info("Loading from library: " + strLibDir);
      log.Info("Loading config file: " + strConfigFile);
      XPDocument document = XPBuilder.LoadDocument(strConfigFile);
      InternalLoadClasses(strLibDir, document, false);
    }
    
    /// <summary>
    /// This method initializes the ASManager with custom lib files
    /// to overwrite the original ones and should only be called by the
    /// warmup servlet.
    /// </summary>
    /// <param name="strLibDir">The file directory where jars and dlls can be found</param>
    /// <param name="strConfigFile">the name of the custom config file</param>
    public virtual void LoadCustomClasses(String strLibDir, String strConfigFile)
    {
      log.Info("Loading from library: " + strLibDir);
      log.Info("Loading config file: " + strConfigFile);
      XPDocument document = XPBuilder.LoadDocument(strConfigFile);
      if (document != null)
      {
        // Reset Custom Object Counters;
        m_nNumCustomControls = 0;
        m_nNumCustomModels = 0;
        m_nNumCustomPageControls = 0;
        m_nNumCustomSpaces = 0;
        m_nNumCustomViews = 0;
        InternalLoadClasses(strLibDir, document, true);
      } else
      {
        log.Info("No custom lib file xml file found at " + strLibDir);
      }
    }
    
    /// <summary>
    /// This method loads the lib files from an XML document into the
    /// ASManager.
    /// </summary>
    /// <param name="strLibDir">The file directory where jars and dlls can be found</param>
    /// <param name="strConfigFile">the name of the config file to load from</param>
    /// <param name="boolean">
    /// bOverWrite false implies error if an MVC component overwrite is attempted
    /// true implies that the Custom Object Counters will be incremented for these classes.
    /// </param>
    private void InternalLoadClasses(String strLibDir, XPDocument document,
     bool bOverwrite)
    {
      // Currently we store names to full package names and re-reflect.
      // We should store objects and clone off them.
      int i;
      int length;
      Object oldObject;
      String strObjectName;
      XPNodeList libFiles = document.GetElementsByTagName("libfile");
      if (libFiles == null)
      {
        // No library files were found
        return;
      }
      for (int j = 0;j < libFiles.GetLength();j++)
      {
        String strLibFileName = ((XPElement) libFiles.Item(j)).GetAttributeNode("name").GetNodeValue();
        log.Info("Loading lib file: " + strLibFileName);
        Object[] models = null;
        Object[] views = null;
        Object[] controls = null;
        Object[] activityspaces = null;
        Object[] pagecontrols = null;
        try
        {
          models = XPDynamicDiscovery.GetInstancesFromLib(strLibDir, INFRASTRUCTURE,
           strLibFileName, IMODEL);
          views = XPDynamicDiscovery.GetInstancesFromLib(strLibDir, INFRASTRUCTURE,
           strLibFileName, IVIEW);
          controls = XPDynamicDiscovery.GetInstancesFromLib(strLibDir, INFRASTRUCTURE,
           strLibFileName, ICONTROL);
          activityspaces = XPDynamicDiscovery.GetInstancesFromLib(strLibDir,
           INFRASTRUCTURE, strLibFileName, AACTIVITYSPACE);
          pagecontrols = XPDynamicDiscovery.GetInstancesFromLib(strLibDir,
           INFRASTRUCTURE, strLibFileName, IPAGECONTROL);
        }
        catch (Exception ex)
        {
          log.Fatal(ex, "Fatal Exception loading lib file: " + strLibFileName);
          }
        log.Debug("Putting infrastructure objects from lib files into hash tables");
        if (models != null)
        {
          // Load Models
          IModel myModel;
          length = models.Length;
          for (i = 0;i < length;i++)
          {
            myModel = (IModel) models[i];
            oldObject = null;
            if (myModel != null)
            {
              strObjectName = myModel.GetName();
              if ((strObjectName != null) && (!"".Equals(strObjectName)))
              {
                oldObject = m_xphModels.PutElement(myModel.GetName(), myModel);
                if (!bOverwrite && (oldObject != null))
                {
                  log.Warn("Loaded model " + strObjectName + " twice.  This is usually caused two Models sharing the same MVC object name.");
                  }
              } else
              {
                // Model has an invalid MVC object name.
                log.Error("Model " + myModel + " does not have a valid MVC object name.  Ignoring.");
                }
            }
          }
        }
        if (views != null)
        {
          // Load Views
          IView myView;
          length = views.Length;
          for (i = 0;i < length;i++)
          {
            myView = (IView) views[i];
            oldObject = null;
            if (myView != null)
            {
              strObjectName = myView.GetName();
              if ((strObjectName != null) && (!"".Equals(strObjectName)))
              {
                oldObject = m_xphViews.PutElement(myView.GetName(), myView);
                if (!bOverwrite && (oldObject != null))
                {
                  log.Warn("Loaded view " + strObjectName + " twice.  This is usually caused two Views sharing the same MVC object name.");
                  }
              } else
              {
                // View has an invalid MVC object name.
                log.Error("View " + myView + " does not have a valid MVC object name.  Ignoring.");
                }
            }
          }
        }
        if (controls != null)
        {
          // Load Controls
          IControl myControl;
          length = controls.Length;
          for (i = 0;i < length;i++)
          {
            myControl = (IControl) controls[i];
            oldObject = null;
            if (myControl != null)
            {
              strObjectName = myControl.GetName();
              if ((strObjectName != null) && (!"".Equals(strObjectName)))
              {
                oldObject = m_xphControls.PutElement(myControl.GetName(),
                 myControl);
                if (!bOverwrite && (oldObject != null))
                {
                  log.Warn("Loaded control " + strObjectName + " twice.  This is usually caused two Controls sharing the same MVC object name.");
                  }
              } else
              {
                // Control has an invalid MVC object name.
                log.Error("Control " + myControl + " does not have a valid MVC object name.  Ignoring.");
                }
            }
          }
        }
        if (pagecontrols != null)
        {
          // Load PageControls
          IPageControl myPageControl;
          length = pagecontrols.Length;
          for (i = 0;i < length;i++)
          {
            myPageControl = (IPageControl) pagecontrols[i];
            oldObject = null;
            if (myPageControl != null)
            {
              strObjectName = myPageControl.GetName();
              if ((strObjectName != null) && (!"".Equals(strObjectName)))
              {
                oldObject = m_xphPageControls.PutElement(myPageControl.GetName(),
                 myPageControl);
                if (!bOverwrite && (oldObject != null))
                {
                  log.Warn("Loaded pagecontrol " + strObjectName + " twice.  This is usually caused two pagecontrols sharing the same MVC object name.");
                  }
              } else
              {
                // PageControl has an invalid MVC object name.
                log.Error("PageControl " + myPageControl + " does not have a valid MVC object name.  Ignoring.");
                }
            }
          }
        }
        if (activityspaces != null)
        {
          // Load Spaces
          AActivitySpace mySpace;
          length = activityspaces.Length;
          for (i = 0;i < length;i++)
          {
            mySpace = (AActivitySpace) activityspaces[i];
            oldObject = null;
            if (mySpace != null)
            {
              strObjectName = mySpace.GetName();
              if ((strObjectName != null) && (!"".Equals(strObjectName)))
              {
                oldObject = m_xphSpaces.PutElement(mySpace.GetName(), mySpace);
                if (!bOverwrite && (oldObject != null))
                {
                  log.Warn("Loaded space " + strObjectName + " twice.  This is usually caused two Spaces sharing the same MVC object name.");
                  }
              } else
              {
                // Space has an invalid MVC object name.
                log.Error("Space " + mySpace + " does not have a valid MVC object name.  Ignoring.");
                }
            }
          }
        }
        // Update Custom Object Counters;
        m_nNumCustomControls = (null != controls) ? controls.Length : 0;
        m_nNumCustomModels = (null != models) ? models.Length : 0;
        m_nNumCustomPageControls = (null != pagecontrols) ? pagecontrols.Length : 0;
        m_nNumCustomSpaces = (null != activityspaces) ? activityspaces.Length : 0;
        m_nNumCustomViews = (null != views) ? views.Length : 0;
      }
    }
    
    /// <summary>
    /// This method returns the number of custom Controls
    /// currently loaded.  This number is reset every time
    /// custom objects are loaded.
    /// </summary>
    /// <returns>The number of custom Controls currently loaded.</returns>
    public virtual int GetNumCustomControls()
    {
      return m_nNumCustomControls;
    }
    
    /// <summary>
    /// This method returns the number of custom Models
    /// currently loaded.  This number is reset every time
    /// custom objects are loaded.
    /// </summary>
    /// <returns>The number of custom Models currently loaded.</returns>
    public virtual int GetNumCustomModels()
    {
      return m_nNumCustomModels;
    }
    
    /// <summary>
    /// This method returns the number of custom Page Controls
    /// currently loaded.  This number is reset every time
    /// custom objects are loaded.
    /// </summary>
    /// <returns>The number of custom Page Controls currently loaded.</returns>
    public virtual int GetNumCustomPageControls()
    {
      return m_nNumCustomPageControls;
    }
    
    /// <summary>
    /// This method returns the number of custom Activity Spaces
    /// currently loaded.  This number is reset every time
    /// custom objects are loaded.
    /// </summary>
    /// <returns>The number of custom Activity Spaces currently loaded.</returns>
    public virtual int GetNumCustomSpaces()
    {
      return m_nNumCustomSpaces;
    }
    
    /// <summary>
    /// This method returns the number of custom Views
    /// currently loaded.  This number is reset every time
    /// custom objects are loaded.
    /// </summary>
    /// <returns>The number of custom Views currently loaded.</returns>
    public virtual int GetNumCustomViews()
    {
      return m_nNumCustomViews;
    }
  }
}