using System;
namespace com.plumtree.uiinfrastructure.activityspace
{
  /// <summary>
  /// This interface provides a common way to create an object that links to
  /// an AActivitySpace.  It also provides methods to specify optional Page,
  /// Controls, and Query String arguments.
  /// To use this interface, first create an object that implements the
  /// interface, then call one of the SetSpace* methods to set what kind of
  /// link you want.  Than add optional arguments (page, etc...)
  /// </summary>
  /// <author>Don Hayler</author>
  public interface ILink
  {
    
    /// <summary>
    /// This method allows you to add a control argument to the HTML URL.
    /// Do not use this method to set arguments that already have their own
    /// methods, such as sub-space key, display page, and control.
    /// For HTTP URLs, strValue should be automatically URL encoded.
    /// The behavior when arguments are added with the same name is
    /// undefined.  Sometimes it will overwrite the first name/value
    /// pair, and sometimes it will append the second name/value pair
    /// after the first.  Arguments should not be added twice.
    /// </summary>
    /// <param name="strName">the name of the argument</param>
    /// <param name="strValue">the value of the argument</param>
    void AddControlArgument(String strName, String strValue);
    
    /// <summary>
    /// This method allows you to add a control argument to the HTML URL.
    /// Do not use this method to set arguments that already have their own
    /// methods, such as sub-space key, display page, and control.
    /// For HTTP URLs, strValues should be automatically URL encoded.
    /// The behavior when arguments are added with the same name is
    /// undefined.  Sometimes it will overwrite the first name/value
    /// pair, and sometimes it will append the second name/value pair
    /// after the first.  Arguments should not be added twice.
    /// </summary>
    /// <param name="strName">the name of the argument</param>
    /// <param name="strValues">the value of the argument</param>
    void AddControlArgument(String strName, String[] strValues);
    
    /// <summary>
    /// This method allows you to add a control argument to the HTML URL.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.ILink#AddControlArgument(String,String)"></seealso>
    /// <param name="strName"></param>
    /// <param name="nValue"></param>
    void AddControlArgument(String strName, int nValue);
    
    /// <summary>
    /// This method allows you to add a control argument to the HTML URL.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.ILink#AddControlArgument(String,String)"></seealso>
    /// <param name="strName"></param>
    /// <param name="booleanValue"></param>
    void AddControlArgument(String strName, bool bValue);
    
    /// <summary>
    /// This method allows you to add a control argument to the HTML URL with no value.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.ILink#AddControlArgument(String,String)"></seealso>
    /// <param name="strName"></param>
    /// <param name="booleanValue"></param>
    void AddControlArgument(String strName);
    
    /// <param name="strControl">the name of the control to link to</param>
    void SetControl(String strControl);
    
    /// <param name="strPage">the name of the page to link to</param>
    void SetDisplayPage(String strPage);
    //////////////////////////////////////////////////////////
    // Set space methods.  One of these (and only one) must be
    // called before getting the result of this URL
    //////////////////////////////////////////////////////////
    
    /// <summary>
    /// This method creates a brand new AActivitySpace.
    /// </summary>
    /// <param name="strSpace">The name of the space to create</param>
    /// <param name="parent">the parent space</param>
    /// <returns>boolean true implies success, false implies failure</returns>
    bool SetLinkCreateNewSpace(String strSpace, AActivitySpace parent);
    
    /// <summary>
    /// This method takes a specific space and returns a link to that (including space ID).
    /// </summary>
    /// <param name="space">the space to create a link to</param>
    /// <returns>boolean true implies success, false implies failure</returns>
    bool SetLinkGetCachedSpace(AActivitySpace space);
    
    /// <summary>
    /// This method gets a named AActivitySpace from the cache if it is there, or
    /// creates a new one if it is not in the cache.
    /// This method should be used to get common AActivitySpaces such as MyPage and
    /// Community, not minor spaces like Tree and Editors.
    /// The SetLinkGetCachedSpace(AActivitySpace) should be used instead of this one
    /// when you can.
    /// </summary>
    /// <param name="strSpace">the name of the space to get or create (if it can't be found)</param>
    /// <param name="parent">the parent space</param>
    /// <returns>boolean true implies success, false implies failure</returns>
    bool SetLinkGetSpaceIfCached(String strSpace, AActivitySpace parent);
    ///////////////////////////
    // Optional methods on URL.
    ///////////////////////////
    
    /// <summary>
    /// This method sets the sub space key.  If this is not called, or is empty,
    /// the space will not be a sub-space.  Otherwise it will be.
    /// The sub-space key is different from the space ID.
    /// </summary>
    /// <param name="sSubSpaceKey">the key for the sub-space</param>
    void SetSubSpaceKey(String sSubSpaceKey);
    bool SetLinkToExternalURL(String url);
  }
}