using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.activityspace
{
  /// <summary>
  /// This object wraps HTMLPages and binary data for output to the browser.
  /// There will be methods for setting HTTP Headers for the browser.
  /// This interface is intended for use in IDisplayPages.
  /// </summary>
  /// <author>Don Hayler</author>
  public interface IWebData
  {
    
    /// <summary>
    /// Call this method to add a cookie to the response.
    /// </summary>
    /// <param name="XPCookie">Pass an XPCookie</param>
    void AddCookie(XPCookie c);
    
    /// <summary>
    /// This optional method flushes all current data in the HTMLPage to the
    /// browser.  This should only be called when the banner has been fully added
    /// to the HTMLPage.
    /// This method requires SetData to have been called first.
    /// This means that the HTMLHead element must be completed and that all
    /// HTMLObjects (like tables) that are currently in the HTMLBody element
    /// need to be complete.  Data will not be able to be added to them after
    /// flushing.  Data will be able to be added to the HTMLBody only.
    /// This is not currently implemented.
    /// </summary>
    void FlushData();
    
    /// <summary>
    /// Sends a true 302 redirect immediatly.
    /// </summary>
    /// <param name="_strLocation">Where to go</param>
    void SendRedirect(String _strLocation);
    ///////////////////////////////////
    // SetData methods
    // Only one of these may be called.
    ///////////////////////////////////
    
    /// <summary>
    /// This method sets the HTMLElement as the data to be sent to the browser.
    /// The element should be an HTMLPage element in most cases - the only time
    /// that the element will not be an HTMLPage is when you want to display 
    /// partial page content.
    /// Only one SetData method should be called.
    /// </summary>
    /// <param name="HTMLElement">to output to the browser</param>
    void SetData(HTMLElement element);
    
    /// <summary>
    /// This method sets array of bytes as the data to be sent to the browser.
    /// Only one SetData method should be called.
    /// </summary>
    /// <param name="byte[]">binary data to output to the browser</param>
    void SetData(byte[] arrBodyBinary);
    
    /// <summary>
    /// This method sets string as the data to be sent to the browser.
    /// Only one SetData method should be called.
    /// </summary>
    /// <param name="String">string to output to the browser</param>
    void SetData(String strData);
    //////////////////
    // General methods
    //////////////////    
    
    /// <summary>
    /// This method specifies whether the text data for this WebData object
    /// is HTML or not.  This method is not relevant for binary data.
    /// Text data is assumed to be HTML by default.
    /// </summary>
    /// <param name="boolean">whether or not the text data is HTML</param>
    void SetIsHTMLText(bool _bIsHTMLText);
    
    /// <summary>
    /// JF- Sets the value of the content language.
    /// The current value of the content language will be set on every response.
    /// </summary>
    /// <param name="_sLocale">(e.g.: en-US)</param>
    void SetContentLanguage(String _sLocale);
    
    /// <summary>
    /// This method sets a specified header on the response.
    /// </summary>
    /// <param name="String">that specifies the name of the HTTP header to set</param>
    /// <param name="String">that specifies the value of the HTTP header</param>
    void SetHeader(String strHeaderName, String strHeaderValue);
    
    /// <summary>
    /// Sets a response date header with the given name and date-value.
    /// </summary>
    /// <param name="String">strHeaderName - a String containing the name of the Date header</param>
    /// <param name="XPDateTime">xpDate - an XPDateTime containing the date.</param>
    void SetDateHeader(String strHeaderName, XPDateTime xpDate);
    
    /// <summary>
    /// Sets a response date header with the given name and date-value.
    /// </summary>
    /// <param name="String">strHeaderName - a String containing the name of the Date header</param>
    /// <param name="long">
    /// iDate - a long containing the date in milliseconds since EPOCH.
    /// Use XPDateTime.GetTimeInMilliseconds() to produce a commensurate value.
    /// </param>
    void SetDateHeader(String strHeaderName, long iDate);
    
    /// <summary>
    /// This method sets a status of the response.,
    /// </summary>
    /// <param name="iStatus">an integer corresponding to the HTTP status code</param>
    void SetStatus(int iStatus);
    
    /// <summary>
    /// This method determines if WebData object will throw an exception in case of 
    /// sending an empty body of not.
    /// </summary>
    /// <param name="bRequireNonEmptyBody">
    /// - TRUE if you want WebData object to throw an exception
    /// in case of sending an emptry body, FALSE otherwise.
    /// </param>
    /// <returns>previous value of the flag</returns>
    bool RequireNonEmptyBody(bool bRequireNonEmptyBody);
  }
}