using System;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.openfoundation.io;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openkernel.http;
using com.plumtree.openlog;
using com.plumtree.server.impl.core;
using com.plumtree.uiinfrastructure.activityspace.utils;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.activityspace
{
  /// <summary>
  /// This object wraps HTMLPages and binary data for output to the browser.
  /// There will be methods for setting HTTP Headers for the browser.
  /// This implementation of the IWebData interface is intended for use by the Interpreter.
  /// </summary>
  /// <author>Don Hayler</author>
  /// <author>DaniilK</author>
  public class WebData : IWebData
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.activityspace.WebData");
    
    /// <summary>
    /// Data to be output (only one can be non-null at a time) 
    /// </summary>
    private byte[] m_arrBinaryBody = null;
    private HTMLElement m_DisplayElement = null;
    private String m_strDisplayString = null;
    // True by default.
    private bool m_bIsHTMLData = true;
    
    /// <summary>
    /// Private variables 
    /// </summary>
    private IXPResponse m_xpResponse = null;
    private String m_strImageBaseURL = null;
    private String m_strLang = null;
    
    /// <summary>
    /// All header keys in this table must be lower case to ensure there are no duplicates. 
    /// </summary>
    private XPHashtable m_htHeaders = null;
    private bool m_bSuppressAutomaticHeaders = false;
    // Default OK status
    private int m_iStatus = 200;
    private bool m_bRequireNonEmptyBody = true;
    private int m_nSecurityMode;
    private bool m_bIsSecure;
    private bool m_bSendRedirect = false;
    private BrowserType m_ptBrowserType;
    private bool m_bDataSent = false;
    private int m_iGatewayDownloadStreamID = -1;
    
    /// <summary>
    /// Creates a new WebData object.
    /// This should only be created by the Interpreter.
    /// </summary>
    /// <param name="_response">IXPResponse assigned to the member variable m_xpResponse</param>
    /// <param name="_strImageBaseURL">String assigned to the member variable m_strImageBaseURL</param>
    /// <param name="_nSecurityMode">int assigned to the member variable _nSecurityMode</param>
    /// <param name="_isSecure">boolean assigned to the member variable m_bIsSecure</param>
    /// <param name="_ptBrowserType">BrowserType assigned to the member variable m_ptBrowserType</param>
    public WebData(IXPResponse _response, String _strImageBaseURL, int _nSecurityMode,
     bool _isSecure, BrowserType _ptBrowserType)
    {
      m_htHeaders = new XPHashtable();
      m_xpResponse = _response;
      m_strImageBaseURL = _strImageBaseURL;
      m_nSecurityMode = _nSecurityMode;
      m_bIsSecure = _isSecure;
      m_ptBrowserType = _ptBrowserType;
      if (m_xpResponse == null)
      {
        log.Error("Response not set in constructor.");
      }
    }
    public virtual void setResponse(IXPResponse _response)
    {
      m_xpResponse = _response;
    }
    
    /// <summary>
    /// This method determines if WebData object will throw an exception in case of 
    /// sending an empty body of not.
    /// </summary>
    /// <param name="bRequireNonEmptyBody">
    /// - TRUE if you want WebData object to throw an exception
    /// in case of sending an emptry body, FALSE otherwise.
    /// </param>
    /// <returns>previous value of the flag</returns>
    public virtual bool RequireNonEmptyBody(bool bRequireNonEmptyBody)
    {
      bool bPrevValue = m_bRequireNonEmptyBody;
      m_bRequireNonEmptyBody = bRequireNonEmptyBody;
      return bPrevValue;
    }
    
    /// <summary>
    /// JF- Helper method to set the content language header from the locale.
    /// </summary>
    /// <param name="_sLocale">(e.g.: en-US)</param>
    public virtual void SetContentLanguage(String _sLocale)
    {
      try
      {
        XPLocale xpLocale = new XPLocale(_sLocale);
        m_strLang = xpLocale.GetLanguage();
        m_htHeaders.PutElement(XPStringUtility.ToLowerCase(HTTPHeaderConstants.CONTENT_LANGUAGE),
         new HTTPHeader(HTTPHeaderConstants.CONTENT_LANGUAGE, m_strLang));
        }
      catch (Exception e)
      {
        log.Error(e, "Unable to get the current language from the current locale: " + _sLocale + ". The content language (Content-Language) will not be set in the HTTP header");
        }
    }
    
    /// <summary>
    /// Call this method to add a cookie to the response.
    /// </summary>
    /// <param name="XPCookie">Pass an XPCookie</param>
    public virtual void AddCookie(XPCookie c)
    {
      if (XPStringUtility.EqualsIgnoreCase(c.GetName(), "PT-Stream-ID"))
      {
        log.Debug("AddCookie() - Trying to set special cookie indicating streaming " + "gateway download. The value of the cookie is [{0}].",
         c.GetValue());
        m_iGatewayDownloadStreamID = XPConvert.ToInteger(c.GetValue());
      } else
      {
        this.m_xpResponse.AddCookie(c);
      }
    }
    
    /// <summary>
    /// This method is used to clear all of the underlying data from this
    /// object to ensure that no memory is "leaked" if a Display Page
    /// mistakenly maintains a reference to this object.
    /// </summary>
    public virtual void ClearAllData()
    {
      // Clear data from this object.
      m_xpResponse = null;
      m_arrBinaryBody = null;
      m_htHeaders = new XPHashtable();
      m_DisplayElement = null;
      m_strDisplayString = null;
    }
    
    /// <summary>
    /// This clears the data to be output (text or binary) so that
    /// this object can be used again.  Some of this data can still
    /// be transmitted if it has already been written to the response
    /// by the time this method is called.
    /// </summary>
    public virtual void ClearOutputData()
    {
      // Clear data from this object.
      m_arrBinaryBody = null;
      m_htHeaders = new XPHashtable();
      m_DisplayElement = null;
      m_strDisplayString = null;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IWebData#FlushData()">
    /// This method flushes all current data in the HTMLPage to the browser.
    /// This should only be called when the banner has been fully added to the
    /// HTMLPage.
    /// This method requires SetData to have been called first.
    /// This means that the HTMLHead element must be completed and that all
    /// HTMLObjects (like tables) that are currently in the HTMLBody element
    /// need to be complete.  Data will not be able to be added to them after
    /// flushing.  Data will be able to be added to the HTMLBody only.
    /// </seealso>
    public virtual void FlushData()
    {
      if ((m_DisplayElement == null) && (m_arrBinaryBody == null) && (m_strDisplayString == null) && ResponseRequiresBody(m_iStatus))
      {
        log.Error("Attempted to flush data before setting it.");
        return;
      }
    }
    // Flush headers
    // flush HTMLPage
    
    /// <summary>
    /// This method tells you whether or not the data on this object is binary.
    /// If it is binary, you can't add text data after the binary data.
    /// </summary>
    /// <returns>boolean true implies binary data</returns>
    public virtual bool IsBinaryData()
    {
      return m_arrBinaryBody != null;
    }
    
    /// <summary>
    /// This method tells you whether or not the data on this object is HTML text.
    /// </summary>
    /// <returns>boolean true implies HTML text data</returns>
    public virtual bool IsHTMLData()
    {
      if (IsBinaryData())
      {
        // Binary data cannot be HTML data.
        return false;
      }
      return m_bIsHTMLData;
    }
    
    /// <summary>
    /// This method tells you whether or not the data on this object is already 
    /// set using either of the SetData() methods.
    /// </summary>
    /// <returns>boolean true implies data set</returns>
    public virtual bool IsDataSet()
    {
      return ((m_DisplayElement != null) || (m_arrBinaryBody != null) || (m_strDisplayString != null) || m_bSendRedirect);
      }
    
    /// <summary>
    /// This method tells you whether or not the data on this object has already 
    /// been sent using the OutputData() method.
    /// </summary>
    /// <returns>boolean true implies data sent</returns>
    public virtual bool HasSentData()
    {
      return m_bDataSent;
    }
    
    /// <summary>
    /// This method marks the data as being sent so that this page cannot
    /// be redisplayed.
    /// </summary>
    public virtual void SetDataSent()
    {
      m_bDataSent = true;
    }
    
    /// <summary>
    /// This method outputs the data to the browser.  This should only be called
    /// by the Interpreter after data has been set on this object by the IDisplayPage.
    /// </summary>
    public virtual void OutputData()
    {
      // if we do not have a body and we really need it - well, lets complain
      if ((m_DisplayElement == null) && (m_arrBinaryBody == null) && (m_strDisplayString == null) && ResponseRequiresBody(m_iStatus) && m_bRequireNonEmptyBody)
      {
        throw new XPException("Attempted to output data before setting it.");
        }
      // if we do not have response object, that is a problem - lets complain
      if (m_xpResponse == null)
      {
        throw new XPException("Response not set.");
      }
      if (m_bDataSent)
      {
        throw new XPException("Attempted to re-send data.");
      }
      // set status on the response
      log.Debug("Setting HTTP status code on response to [" + m_iStatus + "]");
      m_xpResponse.SetStatus(m_iStatus);
      SetDataSent();
      // set body (which sets the headers)
      if (null != m_arrBinaryBody)
      {
        SendBinaryData();
      } else
      {
        SendTextData();
      }
    }
    
    /// <summary>
    /// Sends a true 302 redirect immediatly.
    /// </summary>
    /// <param name="_strLocation">Where to go</param>
    public virtual void SendRedirect(String _strLocation)
    {
      SetHeader("LOCATION", _strLocation);
      SetStatus(302);
      m_bSendRedirect = true;
      FlushData();
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.activityspace.IWebData#SetData(com.plumtree.xpshared.htmlelements.HTMLElement)
     */
     
    public virtual void SetData(HTMLElement element)
    {
      m_DisplayElement = element;
      if ((m_strDisplayString != null) || (m_arrBinaryBody != null))
      {
        log.Error("Only one SetData method can be called on WebData object.");
        }
      m_DisplayElement.SetImgBaseUrl(m_strImageBaseURL);
      m_DisplayElement.SetIsNetscape(m_ptBrowserType.IsNetscape());
      if (null != m_strLang)
      {
        m_DisplayElement.SetLangGlobal(m_strLang);
      }
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IWebData#SetData()">
    /// This method sets the string for output to the browser.
    /// Only one SetData method should be called.
    /// </seealso>
    /// <param name="String">text to output to the browser</param>
    public virtual void SetData(String strData)
    {
      if ((m_DisplayElement != null) || (m_arrBinaryBody != null))
      {
        log.Error("Only one SetData method can be called on WebData object.");
        }
      m_strDisplayString = strData;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IWebData#SetData()">
    /// This method sets the HTMLPage for output to the browser.
    /// Only one SetData method should be called.
    /// </seealso>
    /// <param name="HTMLPage">page to output to the browser</param>
    public virtual void SetData(byte[] arrBodyBinary)
    {
      if ((m_strDisplayString != null) || (m_DisplayElement != null))
      {
        log.Error("Only one SetData method can be called on WebData object.");
        }
      m_arrBinaryBody = arrBodyBinary;
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.activityspace.IWebData#SetIsHTMLText(boolean)
     */
     
    public virtual void SetIsHTMLText(bool _bIsHTMLText)
    {
      m_bIsHTMLData = _bIsHTMLText;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IWebData#SetHeader(String,String)">This method sets a HTTP header to a specified value</seealso>
    /// <param name="String">header name</param>
    /// <param name="String">header value</param>
    public virtual void SetHeader(String strHeaderName, String strHeaderValue)
    {
      m_htHeaders.PutElement(XPStringUtility.ToLowerCase(strHeaderName), new HTTPHeader(strHeaderName,
       strHeaderValue));
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IWebData#SetDateHeader(String,XPDateTime)"></seealso>
    public virtual void SetDateHeader(String strHeaderName, XPDateTime xpDate)
    {
      SetDateHeader(strHeaderName, xpDate.GetTimeInMilliSeconds());
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IWebData#SetDateHeader(String,long)"></seealso>
    public virtual void SetDateHeader(String strHeaderName, long iDate)
    {
      m_htHeaders.PutElement(XPStringUtility.ToLowerCase(strHeaderName), new HTTPHeader(strHeaderName,
       iDate));
    }
    
    /// <param name="iStatus"></param>
    public virtual void SetStatus(int iStatus)
    {
      m_iStatus = iStatus;
    }
    
    /// <summary>
    /// This method turns on or off automatic HTTP headers
    /// (such as the automatic expires header set to now).
    /// Default is to add automatic HTTP headers.
    /// </summary>
    /// <param name="boolean">
    /// true implies suppress automatic headers
    /// false implies show automatic headers
    /// </param>
    public virtual void SetSuppressAutomaticHeaders(bool bSuppress)
    {
      m_bSuppressAutomaticHeaders = bSuppress;
    }
    
    /// <param name="iStatus"></param>
    /// <returns></returns>
    private bool ResponseRequiresBody(int iStatus)
    {
      // iStatus is 2xx
      return ((iStatus > 199) && (iStatus < 300));
    }
    private void SendBinaryData()
    {
      // WE WANT TO SEND HEADERS EVEN IF THERE IS NO BODY!
      SetHTTPHeadersOnResponse(true);
      // On .NET we need to call Flush() to ensure that OnPreSendRequestHeaders event
      // is triggered. Since this call does not really do anything on Java, i put it 
      // here regardless of what platform we are running on.
      m_xpResponse.Flush();
      log.Debug("SendBinaryData() - Set headers on response.");
      if (m_iGatewayDownloadStreamID > -1)
      {
        SendStreamData();
      } else
      {
        // In order for the binary gateway to work, we need to call Write()
        // method on the response object, even if m_arrBinaryBody array is 
        // empty (length = 0). So, here we only check for m_arrBinaryBody being
        // NULL, which actually should never happen.
        if (null == m_arrBinaryBody)
        {
          return;
        }
        /* Get output stream */
        XPOutputStream xpStream = m_xpResponse.GetOutputStream();
        log.Debug("SendBinaryData() - Got output stream.");
        /* write out binary body into the output stream */
        try
        {
          if (null != m_arrBinaryBody && m_arrBinaryBody.Length != 0)
          {
            xpStream.Write(m_arrBinaryBody);
            xpStream.Flush();
          }
          m_xpResponse.Flush();
        }
        catch (XPIOException e)
        {
          log.Warn("There was an error writing data to the socket. ");
          log.Warn(e, "It usually happens when the browser window was closed before " + "all of the data was sent to the client.");
          }
      }
      return;
    }
    private void SendStreamData()
    {
      long lBytesSent = 0;
      long lStartTime = XPSystem.GetCurrentTimeMilliSec();
      long lPerfTime = log.PerformanceBegin();
      IOKHttpInputStream streamRemoteServerToPortal = (IOKHttpInputStream) Global.GATEWAY_STREAM_MANAGER.GetAndClear(m_iGatewayDownloadStreamID);
      XPOutputStream streamPortalToBrowser = m_xpResponse.GetOutputStream();
      try
      {
        while (streamRemoteServerToPortal.IsOpen())
        {
          byte[] barrData = streamRemoteServerToPortal.Read(5000);
          try
          {
            if (null != barrData)
            {
              log.Debug("SendStreamData(): Sending [{0}] bytes to the browser.",
               barrData.Length);
              lBytesSent+=barrData.Length;
              streamPortalToBrowser.Write(barrData);
              streamPortalToBrowser.Flush();
              m_xpResponse.Flush();
            } else
            {
              if (streamRemoteServerToPortal.IsOpen())
              {
                log.Warn("SendStreamData(): Could not read data from the remote server for 5 seconds.");
                } else
              {
                log.Info("SendStreamData(): Stream was closed by the sending side.");
                }
              return;
            }
          }
          catch (Exception e)
          {
            log.Warn(e, "SendStreamData(): There was an exception writing to the output stream. " + "Reading rest of data from input stream to ensure it closes correctly.");
            // NOTE: HACK: Due to bug#67622 in OpenHTTP, we need to continue reading
            // all of the data out of the stream before closing it, otherwise
            // it will leak the socket.  This happens if the browser window is
            // closed while we are in the middle of streaming.  This code should be
            // removed when the OpenHTTP bug is fixed.
            while (barrData != null)
            {
              barrData = streamRemoteServerToPortal.Read(5000);
            }
            return;
          }
        }
      }
      catch (Exception e)
      {
        log.Warn(e, "SendStreamData(): There was an exception trying to read data from the remote server.");
        }
      finally
      {
        log.PerformanceEnd("SendStreamData(): Sent " + lBytesSent + " bytes in " + (XPSystem.GetCurrentTimeMilliSec() - lStartTime) + "ms.",
         lPerfTime);
        streamRemoteServerToPortal.Close();
      }
    }
    private void SendTextData()
    {
      /* WE WANT TO SEND HEADERS EVEN IF THERE IS NOT BODY! */
      SetHTTPHeadersOnResponse(false);
      if (null != m_DisplayElement)
      {
        try
        {
          m_DisplayElement.Display(m_xpResponse);
        }
        catch (HTMLException he)
        {
          log.Error(he, "Error displaying page.");
          throw new XPException(he);
        }
      } else
      if (null != m_strDisplayString)
      {
        // Output as bytes as the rest of the code does (cannot mix bytes and String;
        // XPResponse will throw an exception.
        // m_xpResponse.Write(m_strDisplayString);
        m_xpResponse.GetOutputStream().Write(HTMLElement.ToByteArray(m_strDisplayString));
        }
      /* do nothing if none of the cases above succeeded */
      return;
    }
    
    /// <param name="strContentTypeArg"></param>
    private void SetContentTypeIfNotAlreadySpecified(String strContentTypeArg)
    {
      /* Get content type header, since it's a binary data it might not be text/html */
      HTTPHeader header = (HTTPHeader) m_htHeaders.GetElement(XPStringUtility.ToLowerCase(HTTPHeaderConstants.CONTENT_TYPE));
      /* if the header was not set specifically, we use the default one "text/html" */
      if (null == header)
      {
        log.Debug("Setting HTTP Content-Type header to [" + strContentTypeArg + "]");
        m_xpResponse.SetContentType(strContentTypeArg);
      } else
      {
        String strContentType = header.m_strValue;
        log.Debug("Setting HTTP Content-Type header to [" + strContentType + "]");
        m_xpResponse.SetContentType(strContentType);
      }
    }
    
    /// <summary>
    /// Sets various headers on the response for both binary and text data. This method
    /// always sets the Content-Type header. If m_bSuppressAutomaticHeaders is false it
    /// sets the following headers: Expires, Cache-control, Last-Modified and Pragma (
    /// however Cache-control, Last-Modified and Pragma are not set if the browser is IE 
    /// and either the security mode is 3 or the session is secure.) It also sets whatever
    /// headers are present in m_htHeaders.
    /// </summary>
    /// <param name="bSendingBinaryData"></param>
    /// <returns></returns>
    private int SetHTTPHeadersOnResponse(bool bSendingBinaryData)
    {
      int iHeadersAdded = 0;
      /* Set content type first, since it requires a special call on the interface */
      if (bSendingBinaryData)
      {
        SetContentTypeIfNotAlreadySpecified("text/plain");
      } else
      {
        SetContentTypeIfNotAlreadySpecified("text/html");
      }
      // These headers should only be sent for portal pages and never for gateway pages.
      //		-- MOOSE
      if (!m_bSuppressAutomaticHeaders)
      {
        // JF- Set "Expires" to the day before (this is to avoid any problems in case the client 
        // and the server don't have exactly the same time)
        log.Debug("Setting HTTP Expires header to [" + (XPSystem.GetCurrentTimeMilliSec() - 24 * 60 * 60 * 1000) + "]");
        m_xpResponse.SetDateHeader(HTTPHeaderConstants.EXPIRES, XPSystem.GetCurrentTimeMilliSec() - 24 * 60 * 60 * 1000);
        if (!m_ptBrowserType.IsNetscape4())
        {
          // Set UTF-8 internationalization header
          SetContentTypeIfNotAlreadySpecified("text/html; charset=utf-8");
          } else
        {
          // If it's not netscape 4.x, use utf8
          // If it is netscape 4.x, use 8859_1, because NS4
          // can't handle utf-8 chars
          SetContentTypeIfNotAlreadySpecified("text/html; charset=8859_1");
          }
        // these headers s/b set unless the the browser type is ie & the session is secure or
        // the security mode is 3. this is to resolve ie's problem with downloading binaries
        // with these headers in https (bug 39333)
        if (bSendingBinaryData && m_ptBrowserType.IsIE() && ((m_bIsSecure) || (m_nSecurityMode == 3)))
        {
          log.Debug("Not Setting default HTTP headers");
        } else
        {
          log.Debug("Setting HTTP headers \r\n" + "[" + HTTPHeaderConstants.CACHE_CONTROL + ": no-store, no-cache, must-revalidate, max-age=0, post-check=0, pre-check=0]\r\n" + "[" + HTTPHeaderConstants.PRAGMA + ": no-cache]\r\n");
          m_xpResponse.AddHeader(HTTPHeaderConstants.CACHE_CONTROL, "no-store, no-cache, must-revalidate, max-age=0, post-check=0, pre-check=0");
          m_xpResponse.AddHeader(HTTPHeaderConstants.PRAGMA, "no-cache");
          m_xpResponse.SetDateHeader(HTTPHeaderConstants.LAST_MODIFIED, XPSystem.GetCurrentTimeMilliSec());
          }
      }
      /* Set the rest of the headers */
      for (IXPEnumerator xpEnumHeaders = m_htHeaders.GetElements();xpEnumHeaders.MoveNext();
      )
      {
        HTTPHeader header = (HTTPHeader) xpEnumHeaders.GetCurrent();
        if (null != header)
        {
          log.Debug("Setting HTTP header [" + header.m_strName + ": " + header.m_strValue + "]");
          header.SetHeader(m_xpResponse);
          iHeadersAdded+=1;
        }
      }
      return iHeadersAdded;
    }
    
    /// <summary>
    /// Internal helper class that represents HTTP Headers 
    /// </summary>
    class HTTPHeader
    {
      public String m_strName;
      public String m_strValue;
      public long m_nValue;
      public bool m_bDateHeader = false;
      
      /// <summary>
      /// Creates a new HTTPHeader object.
      /// </summary>
      /// <param name="name"></param>
      /// <param name="value"></param>
      public HTTPHeader(String name, String value)
      {
        m_strName = name;
        m_strValue = value;
        m_bDateHeader = false;
      }
      
      /// <summary>
      /// Creates a new HTTPHeader object for a date.
      /// </summary>
      /// <param name="name"></param>
      /// <param name="value"></param>
      public HTTPHeader(String name, long value)
      {
        m_strName = name;
        m_nValue = value;
        m_bDateHeader = true;
      }
      
      /// <summary>
      /// Adds the header to the response.
      /// </summary>
      /// <param name="_response"></param>
      public virtual void SetHeader(IXPResponse _response)
      {
        if (m_bDateHeader)
        {
          _response.SetDateHeader(m_strName, m_nValue);
        } else
        {
          _response.AddHeader(m_strName, m_strValue);
        }
      }
    }
  }
}