using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.web;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.application.varpacks;
using com.plumtree.uiinfrastructure.errordisplay;
using com.plumtree.uiinfrastructure.interpreter;
using com.plumtree.uiinfrastructure.login;
namespace com.plumtree.uiinfrastructure.activityspace.utils
{
  /// <summary>
  /// This class contains helper methods for the Interpreter class that handle
  /// Login.
  /// </summary>
  /// <author>donh</author>
  public class LoginHandlers
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.activityspace.utils.LoginHandlers");
    
    /// <summary>
    /// mj: whether static init has been called. 
    /// </summary>
    private static bool s_bInitialized = false;
    
    /// <summary>
    /// mj: whether guest access is enabled. 
    /// </summary>
    public static bool s_bGuestAccessAllowed = true;
    
    /// <summary>
    /// Default login space information 
    /// </summary>
    public const String LOGINSPACE = "Login";
    public const String LOGINCONTROL = "Login";
    public const String DEFAULTLOGINCONTROL = "DefaultLoginControl";
    //DEFAULTSPACE.equals(LOGINSPACE);
    
    /// <summary>
    /// mj: boolean value used by sso 
    /// </summary>
    public const bool DEFAULTSPACEISLOGIN = true;
    
    /// <summary>
    /// login occurred cookie name 
    /// </summary>
    public const String LOGIN_OCCURED_COOKIE_NAME = "plloginoccured";
    
    /// <summary>
    /// Initializes static member variables from ConfigVarPack.
    /// </summary>
    /// <param name="varpack">a <c>ConfigVarPack</c></param>
    public static void StaticInit(ConfigVarPack varpack)
    {
      if (s_bInitialized)
      {
        log.Warn("LoginHandlers.StaticInit reinitializing.");
      }
      try
      {
        // mj: check whether guest access is allowed. 0 means disallowed, otherwise allow it.
        s_bGuestAccessAllowed = (0 != varpack.GetConfigValueAsInt(ConfigConstants.AUTHENTICATION_SECTION,
         ConfigConstants.GUEST_ACCESS_SUBSECTION, ConfigConstants.VALUE_ATTRIBUTE));
        }
      catch (Exception e)
      {
        log.Error(e, "Unable to access Config variable AllowGuestAccess. Guest access is enabled by default.");
        s_bGuestAccessAllowed = true;
      }
      log.Info("Guest access is " + (s_bGuestAccessAllowed ? "enabled" : "disabled"));
      s_bInitialized = true;
    }
    ///////////////////////
    // Login helper methods
    ///////////////////////
    
    /// <summary>
    /// Handles guest login.
    /// This is not technically thread safe, but the odds of anything
    /// catastrophic happening are fairly low, since multiple threads
    /// calling this simultaneously will only be writing the same data
    /// to the same places, with the sole exception of the IPTSession.
    /// </summary>
    /// <param name="RequestData"></param>
    /// <returns>Redirect object from login PEIs</returns>
    public static Redirect DoGuestLogin(RequestData tempData)
    {
      Redirect rReturn = null;
      // WilliamA - Multiple-guest-user support
      // -1 will be treated as default guest as well
      int nGuestUserID = 2;
      String str = (String) tempData.m_SessionManager.GetAttribute(Authenticator.GUESTLOGIN_GUESTUSERID);
      if (null != str)
      {
        nGuestUserID = XPConvert.ToInteger(str);
      }
      // Guest username info here is only used for Login PEIs.
      String strGuest = (String) tempData.m_SessionManager.GetAttribute(Authenticator.GUESTLOGIN_GUESTUSERNAME);
      if (-1 == nGuestUserID)
      {
        log.Info("Logging in as guest user: default guest");
      } else
      {
        log.Info("Logging in as guest user: {0}", nGuestUserID);
      }
      // This sets the session validity for us.
      LoginResult loginResult = LoginHelper.INSTANCE.AttemptGuestLogin(nGuestUserID,
       strGuest, null, tempData.m_smSandbox, tempData.m_smPersistentSandbox,
       tempData.m_Application, tempData.GetRequestURL(), tempData.m_xpRequest,
       tempData.m_wdPageData);
      tempData.m_bGuest = true;
      if (loginResult.m_bSuccess)
      {
        if (!loginResult.m_bRedirectedBeforeLogin)
        {
          tempData.SetUserSession(loginResult.m_UserSession);
        }
        rReturn = loginResult.m_Redirect;
        // Note: This may not be necessary, and produces some unexpected results.
        // When this is called, it nulls out the Activity Space on the tempData,
        // which was causing some NPEs during login pei customizations.
        // This method is really meant for SSO to call.
        if (loginResult.m_Redirect != null)
        {
          tempData.SetRedirect(loginResult.m_Redirect);
        }
      } else
      {
        // Do not output error message from loginResult to end user,
        // as this error indicates serious problems that were not
        // caused by the end user.
        log.Error("Couldn\'t log in as guest user. Failing: " + loginResult.m_strError);
        throw new XPException("Couldn\'t log in as guest user. Failing.");
        }
      return rReturn;
    }
    
    /// <summary>
    /// Helper method for login which can be called from SSO login
    /// code outside this class.
    /// </summary>
    /// <param name="RequestData">the data for this HTTPRequest</param>
    /// <returns>Redirect a redirect object from Guest Login PEIs, if it occured</returns>
    public static Redirect DoLogin(RequestData tempData)
    {
      // Cache the user session, because it can get cleared on logout.
      Object userSession = tempData.GetUserSession();
      // This clears the AS cache and other data.
      DoLogout(tempData);
      // A null value for the previously cached user session implies that
      // we should do a guest login.
      if (null == userSession)
      {
        // this will do the login tasks, since they were not handled by the login
        // control
        return DoGuestLogin(tempData);
      } else
      {
        // login actions are handled by the login control.  LoginHelper also handles
        // setting session validity for us.
        // We don't deny guest login here, because this is how Edit Default Profiles
        // logs in as guest.  Guest login needs to be denied at a higher level.
        // This method sets it on the tempData and on the HTTP Session
        tempData.SetUserSession(userSession);
        tempData.m_bGuest = false;
        log.Info("Logging on as user " + LoginHelper.INSTANCE.GetUserSessionID(tempData.GetUserSession()) + ", " + LoginHelper.INSTANCE.GetUserSessionName(tempData.GetUserSession()));
        }
      return null;
    }
    
    /// <summary>
    /// Helper method for login which can be called from SSO login
    /// code outside this class.
    /// </summary>
    /// <param name="RequestData">the data for this HTTPRequest</param>
    public static void DoLogin(String strLoginToken, RequestData tempData)
    {
      // This clears the AS cache and other data.
      DoLogout(tempData);
      log.Info("Logging in using login token from cookie.");
      LoginResult loginResult = LoginHelper.INSTANCE.AttemptLogin(strLoginToken,
       tempData.m_smSandbox, tempData.m_smPersistentSandbox, tempData.m_Application,
       tempData.GetRequestURL(), tempData.m_xpRequest, tempData.m_wdPageData);
      if (loginResult.m_Redirect != null)
      {
        tempData.SetRedirect(loginResult.m_Redirect);
      }
      tempData.m_bGuest = loginResult.m_bIsGuest;
      if (loginResult.m_bSuccess)
      {
        // You should never login as guest from a login token.  This is
        // different from the above case with Edit Default Profiles.
        // However, we can't check for it here because we don't know the
        // guest user name or ID.
        tempData.SetUserSession(loginResult.m_UserSession);
      } else
      {
        // If there was a redirect, then a before login PEI redirected us
        // away before we could login, so it's not an error.
        if (loginResult.m_Redirect == null)
        {
          log.Error("Unable to log in using login token from cookie: " + loginResult.m_strError);
          log.Info("Clearing invalid login token in cookie.");
          // We need to clear the invalid cookie here so that we don't get
          // into an infinite redirect in Security Mode 1
          // (https token login fails, login as guest, 302 redirect to http mypage, repeat).
          LoginHelper.INSTANCE.ClearAutologinToken(tempData.m_xpRequest.GetRequestURL(),
           tempData.m_wdPageData);
          // We don't set the activity space name, because we don't know it.
          UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,
           null, loginResult.m_strError, loginResult.m_strError);
          myError.SetActivitySpace(tempData.GetSpaceName());
          myError.SetClearWithActivitySpaceChange(true);
          UIErrorHandler.RaiseError(myError, tempData.m_smSandbox, tempData.m_Application);
          }
      }
    }
    
    /// <summary>
    /// helper method to log users out.
    /// </summary>
    /// <param name="RequestData">the data for this HTTPRequest</param>
    public static void DoLogout(RequestData tempData)
    {
      log.Info("Logging off user, clearing cache.");
      // The personal settings helper, ui Errors, and sandbox are cleared in
      // the login helper so that they can be used by the OnAfterLogin PEI.
      if (tempData.m_smCache != null)
      {
        tempData.m_smCache.Clear();
      }
      if (tempData.m_xpmCacheList != null)
      {
        tempData.m_xpmCacheList.Clear();
      }
      // This method removes it from the session manager as well as the
      // Request Data object. 
      tempData.ClearUserSession();
      // Set to guest user, LoginHelper handles setting session validity for us.
      tempData.m_bGuest = true;
    }
    
    /// <summary>
    /// This helper method determines whether or not the login occurred
    /// cookie is present.
    /// </summary>
    /// <param name="request"></param>
    /// <returns>boolean true implies the cookie is present.</returns>
    public static bool IsLoginOccurredCookiePresent(IXPRequest request)
    {
      XPCookie[] cookies = request.GetCookies();
      XPCookie cookie;
      bool bFoundLoginOccuredCookie = false;
      int iNumCookies = cookies.Length;
      for (int iCount = 0;iCount < iNumCookies;iCount++)
      {
        cookie = cookies[iCount];
        if (LOGIN_OCCURED_COOKIE_NAME.Equals(cookie.GetName()))
        {
          Object cookieValue = cookie.GetValue();
          if (null != cookieValue)
          {
            // A value of false implies that the cookie is no longer
            // valid (perhaps should be deleted).
            bFoundLoginOccuredCookie = XPConvert.ToBoolean(cookieValue);
          }
          break;
        }
      }
      return bFoundLoginOccuredCookie;
    }
    
    /// <summary>
    /// This method clears the login occurred cookie in the response.
    /// </summary>
    /// <param name="response">IXPResponse The current response.</param>
    public static void ClearLoginOccurredCookiePresent(IXPResponse response)
    {
      // Not setting an expires value on a cookie uses the default behaviour in both
      // Java and .NET (deleted when browser is closed).
      // Perhaps this should delete the cookie (0 in Java, time in the past on .NET)
      XPCookie cookie = new XPCookie(LoginHandlers.LOGIN_OCCURED_COOKIE_NAME,
       XPConvert.ToString(false));
      cookie.SetPath("/");
      response.AddCookie(cookie);
    }
    
    /// <summary>
    /// This helper method handles logging users in and out from the Login Control.
    /// </summary>
    /// <param name="loginControl"></param>
    /// <param name="RequestData">the data for this HTTPRequest</param>
    public static Redirect DoLogin(IControl loginControl, RequestData tempData)
    {
      ILoginControl myLoginControl = (ILoginControl) loginControl;
      Redirect rReturn = null;
      if (myLoginControl.DoGetSession())
      {
        Object myUserSession = ((ILoginControl) loginControl).GetSession();
        if (myUserSession != tempData.GetUserSession())
        {
          tempData.SetUserSession(myUserSession);
          rReturn = DoLogin(tempData);
          // In certain guest login cases, the space can be null at this point.
          if (null != tempData.m_asSpace)
          {
            // need to set session on space before calling dotasks after login, otherwise
            // space may have the wrong session.
            tempData.m_asSpace.SetUserSession(tempData.GetUserSession());
            // DCA - re-initialize activityspace because models might also have wrong
            // session
            tempData.m_asSpace.ReInit();
          }
        }
      }
      return rReturn;
    }
    
    /// <summary>
    /// This method is used to determine if auto connect is enabled
    /// Turning off auto-connect only disables displaying the
    /// 'remember my password' checkbox on login and checking for
    /// the login token cookie.
    /// Checking for the admin QS login token is always enabled.
    /// </summary>
    /// <param name="RequestData"></param>
    /// <returns>boolean true implies auto connect is enabled</returns>
    public static bool GetIsAutoConnectEnabled(RequestData tempData)
    {
      ConfigVarPack vpConfig = (ConfigVarPack) tempData.m_Application.GetVarPackManager().GetVariablePackage(ConfigVarPack.VARPACK_ID);
      int i = vpConfig.GetConfigValueAsInt(ConfigConstants.AUTHENTICATION_SECTION,
       "AllowAutoConnect", ConfigConstants.VALUE_ATTRIBUTE);
      if (i == 0)
      {
        return false;
      }
      return true;
    }
    
    /// <summary>
    /// This method is used to determine if the guest user should
    /// be redirected to their default page or to the login page
    /// if a space is not specified.
    /// </summary>
    /// <param name="RequestData"></param>
    /// <returns>
    /// boolean true implies the guest should redirect to
    /// login if no space is specified.  Login is default.
    /// </returns>
    public static bool GetIsGuestRedirectToLoginEnabled(RequestData tempData)
    {
      return GetIsGuestRedirectToLoginEnabled(tempData.m_Application, tempData.m_SessionManager);
      }
    
    /// <summary>
    /// This method is used to determine if the guest user should
    /// be redirected to their default page or to the login page
    /// if a space is not specified.
    /// </summary>
    /// <param name="application"></param>
    /// <param name="sessionManager"></param>
    /// <returns>
    /// boolean true implies the guest should redirect to
    /// login if no space is specified.  Login is default.
    /// </returns>
    public static bool GetIsGuestRedirectToLoginEnabled(IApplication application,
     ISessionManager sessionManager)
    {
      // setting from the config file
      ConfigVarPack vpConfig = (ConfigVarPack) application.GetVarPackManager().GetVariablePackage(ConfigVarPack.VARPACK_ID);
      String strGuestRedirect = vpConfig.GetConfigValueAsString(ConfigConstants.AUTHENTICATION_SECTION,
       "GuestRedirectToLogin", ConfigConstants.VALUE_ATTRIBUTE);
      // default value
      bool bResult = true;
      if ((strGuestRedirect != null) && (!"".Equals(strGuestRedirect)))
      {
        if (XPConvert.ToInteger(strGuestRedirect) == 0)
        {
          bResult = false;
        }
      }
      return bResult;
    }
  }
}