/*
 * Purpose: Checks that the database is the right version
 * Author: 	D. Phipps May 20, 2005
 */
 
using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openkernel.db;
using com.plumtree.openlog;
using com.plumtree.server;
using com.plumtree.server.impl.core;
using com.plumtree.uiinfrastructure.application.notification;
using com.plumtree.uiinfrastructure.application.startup.exceptions;
using com.plumtree.uiinfrastructure.application.startup.messages;
using com.plumtree.uiinfrastructure.diagnostic.reporter;
using com.plumtree.uiinfrastructure.util;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.uiinfrastructure.application.startup
{
  /// <summary>
  /// Verifies database version and some of the important tables.
  /// Also fully excercises the object managers.
  /// </summary>
  /// <author>DavidP</author>
  public class CheckDBVersion : IStartupTask
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.application.startup.CheckDBVersion");
    private String m_applicationName = "";
    private IApplication m_application = null;
    private IPTSession m_session = null;
    private bool m_bVerbose = false;
    // array of messages
    private static StartupTaskMessage[] m_diag = {new StartupTaskMessage(
    // steps are one-based; step zero is the init call
    // DAP TODO this really needs to be in an external file
    0, Platform.ALL, PT_STARTUP_RESULTCODES.FATAL, "Check Database Version",
     "RECOMMENDATION: You must upgrade your portal database schema to the latest version before " + "the portal will operate correctly (I.E. A 6.0 portal will not work with a 5.0.4 database).  " + "Please review the Aqualogic Interaction Upgrade Guide."),
     new StartupTaskMessage(1, Platform.ALL, PT_STARTUP_RESULTCODES.FATAL,
     "Exercise Object Managers", "RECOMMENDATION: This failure indicates you are missing tables, or the data in " + "your tables is corrupt.  You should rescript your database completely using " + "the SQL scripts installed to your PORTAL_HOME/sql directory.")};
    public CheckDBVersion() : base()
    {
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskName()
     */
     
    public virtual String getTaskName()
    {
      return "CheckDBVersion";
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskID()
     */
     
    public virtual int getTaskID()
    {
      return PTStartupTasks.CHECKDBVER.getOrdinal();
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getNumSteps()
     */
     
    public virtual int getNumSteps()
    {
      return 2;
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#init(java.lang.String)
     */
     
    public virtual ReturnValue init(String _strApplicationName)
    {
      m_applicationName = _strApplicationName;
      return new ReturnValue(PT_STARTUP_RESULTCODES.OK, getTaskName() + " init successful",
       null);
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#executeStep(int)
     */
     
    public virtual ReturnValue executeStep(int sequentialStepID)
    {
      ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, "Incorrect step request on init",
       StackTracer.MakeXPException("(internal stacktrace)"));
      switch (sequentialStepID){
      case 0:
      // check the database version
      retVal = executeStepCheckDBVersion();
      break;
      case 1:
      // excercise all the object managers
      retVal = exerciseObjectManagers();
      break;
      }
      return retVal;
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#executeStep(int)
     */
     
    public virtual ReturnValue executeStepCheckDBVersion()
    {
      ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK, "CheckDBVersion successful.",
       null);
      log.Info("Initializing CheckDBVersion.");
      try
      {
        IPTSession ptSession = PortalObjectsFactory.GetGuestReadOnlySession();
        // bad portalserver version
        String portalVersion = ptSession.GetVersion();
        if (XPStringUtility.CompareIgnoreCase(portalVersion, GlobalSysInfo.PT_RELEASE_VERSION) != 0)
        {
          throw new XPException("Incorrect server version: bad installation or build.  " + "Should be \'" + GlobalSysInfo.PT_RELEASE_VERSION + "\' but was \'" + portalVersion + "\'");
          }
        // check the database schema version
        String strDatabaseVersion = "";
        IOKDatabaseService testDBService = InternalSession.GetDatabaseService();
        String strSql = "SELECT VALUE FROM PTSERVERCONFIG WHERE SETTINGID=?";
        IOKDBCursor cursor = testDBService.CreatePTCursor(strSql);
        int nSetting = 45;
        cursor.SetInt(0, nSetting);
        IOKDBResultSet resultset = cursor.Open(1);
        IOKDBRow row = resultset.GetRow(0);
        strDatabaseVersion = row.GetString("VALUE");
        // Compare the db version
        if (XPStringUtility.CompareIgnoreCase(strDatabaseVersion, GlobalSysInfo.PT_RDBMS_SCHEMA_VERSION_EXPECTED) != 0)
        {
          throw new XPException("Incorrect Database Version: " + "Should be \'" + GlobalSysInfo.PT_RDBMS_SCHEMA_VERSION_EXPECTED + "\' but was \'" + strDatabaseVersion + "\'");
          }
      }
      catch (Exception e)
      {
        String errorMessage = "Database Version Problem: ";
        retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, errorMessage,
         XPException.GetInstance(e));
      }
      return retVal;
    }
    
    /// <summary>
    /// Check each and every object manager and session variable.
    /// </summary>
    /// <param name="sequentialStepID"></param>
    /// <returns></returns>
    public virtual ReturnValue exerciseObjectManagers()
    {
      ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK, "excerciseObjectManagers successful.",
       null);
      log.Info("Initializing excerciseObjectManagers.");
      String str = XPEnvironment.GetEnvironmentVariable("PT_VERIFY");
      if ((str != null) && (XPStringUtility.IndexOf(XPStringUtility.ToUpperCase(str),
       "TRUE") != -1))
      {
        m_bVerbose = true;
      }
      try
      {
        //IPTSession ptSession = PortalObjectsFactory.CreateSession();
        IPTSession ptSession = PortalObjectsFactory.GetGuestReadOnlySession();
        // Get all object types (not terribly useful)
        IPTAdminCatalog ptAdminCatalog = ptSession.GetAdminCatalog();
        if (ptAdminCatalog == null)
        {
          throw new XPException("Cannot retrieve admin catalog");
        }
        IPTObjectManager ptAuthSources = ptSession.GetAuthSources();
        if (ptAuthSources == null)
        {
          throw new XPException("Cannot retrieve AuthSources");
        }
        IPTObjectManager ptCommunityTemplates = ptSession.GetCommunityTemplates();
        if (ptCommunityTemplates == null)
        {
          throw new XPException("Cannot retrieve CommunityTemplates");
        }
        IPTObjectManager ptCommunities = ptSession.GetCommunities();
        if (ptCommunities == null)
        {
          throw new XPException("Cannot retrieve Communities");
        }
        IPTObjectManager ptDataSources = ptSession.GetDataSources();
        if (ptDataSources == null)
        {
          throw new XPException("Cannot retrieve DataSources");
        }
        IPTObjectManager ptExpressions = ptSession.GetExpressions();
        if (ptExpressions == null)
        {
          throw new XPException("Cannot retrieve Expressions");
        }
        IPTObjectManager ptDocTypes = ptSession.GetDocumentTypes();
        if (ptDocTypes == null)
        {
          throw new XPException("Cannot retrieve DocTypes");
        }
        IPTObjectManager ptExtOps = ptSession.GetExternalOperations();
        if (ptExtOps == null)
        {
          throw new XPException("Cannot retrieve ExtOps");
        }
        IPTObjectManager ptFedPorts = ptSession.GetFederatedPortals();
        if (ptFedPorts == null)
        {
          throw new XPException("Cannot retrieve FedPorts");
        }
        IPTObjectManager ptCrawlers = ptSession.GetCrawlers();
        if (ptCrawlers == null)
        {
          throw new XPException("Cannot retrieve Crawlers");
        }
        IPTObjectManager ptFilters = ptSession.GetFilters();
        if (ptFilters == null)
        {
          throw new XPException("Cannot retrieve Filters");
        }
        IPTObjectManager ptGadgetBundles = ptSession.GetGadgetBundles();
        if (ptGadgetBundles == null)
        {
          throw new XPException("Cannot retrieve GadgetBundles");
        }
        IPTObjectManager ptGadgetContentServers = ptSession.GetGadgetContentServers();
        if (ptGadgetContentServers == null)
        {
          throw new XPException("Cannot retrieve GadgetContentServers");
        }
        IPTObjectManager ptGuestLogins = ptSession.GetGuestLogins();
        if (ptGuestLogins == null)
        {
          throw new XPException("Cannot retrieve GuestLogins");
        }
        IPTObjectManager ptInvitations = ptSession.GetInvitations();
        if (ptInvitations == null)
        {
          throw new XPException("Cannot retrieve Invitations");
        }
        IPTObjectManager ptPages = ptSession.GetPages();
        if (ptPages == null)
        {
          throw new XPException("Cannot retrieve Pages");
        }
        IPTObjectManager ptPageTemplates = ptSession.GetPageTemplates();
        if (ptPageTemplates == null)
        {
          throw new XPException("Cannot retrieve PageTemplates");
        }
        IPTObjectManager ptProfilePages = ptSession.GetProfilePages();
        if (ptProfilePages == null)
        {
          throw new XPException("Cannot retrieve ProfilePages");
        }
        IPTObjectManager ptProfileSections = ptSession.GetProfileSections();
        if (ptProfileSections == null)
        {
          throw new XPException("Cannot retrieve ProfileSections");
        }
        IPTObjectManager ptProfileSources = ptSession.GetProfileSources();
        if (ptProfileSources == null)
        {
          throw new XPException("Cannot retrieve ProfileSources");
        }
        IPTObjectManager ptProperties = ptSession.GetProperties();
        if (ptProperties == null)
        {
          throw new XPException("Cannot retrieve Properties");
        }
        IPTObjectManager ptSavedSearches = ptSession.GetSavedSearches();
        if (ptSavedSearches == null)
        {
          throw new XPException("Cannot retrieve SavedSearches");
        }
        IPTObjectManager ptSiteMapFolders = ptSession.GetSiteMapFolders();
        if (ptSiteMapFolders == null)
        {
          throw new XPException("Cannot retrieve SiteMapFolders");
        }
        IPTObjectManager ptSubPortals = ptSession.GetSubPortals();
        if (ptSubPortals == null)
        {
          throw new XPException("Cannot retrieve SubPortals");
        }
        IPTObjectManager ptUserGroups = ptSession.GetUserGroups();
        if (ptUserGroups == null)
        {
          throw new XPException("Cannot retrieve UserGroups");
        }
        IPTObjectManager ptWebServices = ptSession.GetWebServices();
        if (ptWebServices == null)
        {
          throw new XPException("Cannot retrieve WebServices");
        }
        // use the managers to open at least one intrinsic object
        // database sizing in debug mode
        if (m_bVerbose == true)
        {
          IOKDatabaseService testDBService = InternalSession.GetDatabaseService();
          // pages
          String strSql = "SELECT COUNT(*) AS NUMPAGES FROM PTPAGES WHERE OBJECTID > ?";
          IOKDBCursor cursor = testDBService.CreatePTCursor(strSql);
          int nSetting = 199;
          cursor.SetInt(0, nSetting);
          IOKDBResultSet resultset = cursor.Open(1);
          IOKDBRow row = resultset.GetRow(0);
          int numpages = row.GetInt("NUMPAGES");
          // communities
          strSql = "SELECT COUNT(*) AS NUMCOMM FROM PTCOMMUNITIES WHERE OBJECTID > ?";
          cursor = testDBService.CreatePTCursor(strSql);
          nSetting = 2;
          cursor.SetInt(0, nSetting);
          resultset = cursor.Open(1);
          row = resultset.GetRow(0);
          int numcommunities = row.GetInt("NUMCOMM");
          // portlets
          strSql = "SELECT COUNT(*) AS NUMGADGETS FROM PTGADGETS WHERE OBJECTID > ?";
          cursor = testDBService.CreatePTCursor(strSql);
          nSetting = 199;
          cursor.SetInt(0, nSetting);
          resultset = cursor.Open(1);
          row = resultset.GetRow(0);
          int numgadgets = row.GetInt("NUMGADGETS");
          // users
          strSql = "SELECT COUNT(*) AS NUMUSERS FROM PTUSERS WHERE OBJECTID > ?";
          cursor = testDBService.CreatePTCursor(strSql);
          nSetting = 199;
          cursor.SetInt(0, nSetting);
          resultset = cursor.Open(1);
          row = resultset.GetRow(0);
          int numusers = row.GetInt("NUMUSERS");
          // cards
          strSql = "SELECT COUNT(*) AS NUMCARDS FROM PTCARDS WHERE OBJECTID > ?";
          cursor = testDBService.CreatePTCursor(strSql);
          nSetting = 199;
          cursor.SetInt(0, nSetting);
          resultset = cursor.Open(1);
          row = resultset.GetRow(0);
          int numcards = row.GetInt("NUMCARDS");
          // group membership
          strSql = "SELECT COUNT(*) AS NUMGROUPMEM FROM PTVGROUPMEMBERSHIP WHERE USERID > ?";
          cursor = testDBService.CreatePTCursor(strSql);
          nSetting = 199;
          cursor.SetInt(0, nSetting);
          resultset = cursor.Open(1);
          row = resultset.GetRow(0);
          int numgroupmem = row.GetInt("NUMGROUPMEM");
          // joblogs
          strSql = "SELECT COUNT(*) AS NUMJOBLOGS FROM PTJOBLOGS WHERE INSTANCEID <> ?";
          cursor = testDBService.CreatePTCursor(strSql);
          nSetting = 0;
          cursor.SetInt(0, nSetting);
          resultset = cursor.Open(1);
          row = resultset.GetRow(0);
          int numjoblogs = row.GetInt("NUMJOBLOGS");
          // security
          strSql = "SELECT COUNT(*) AS NUMPTObjectSecurity FROM PTObjectSecurity WHERE OBJECTID > ?";
          cursor = testDBService.CreatePTCursor(strSql);
          nSetting = 199;
          cursor.SetInt(0, nSetting);
          resultset = cursor.Open(1);
          row = resultset.GetRow(0);
          int numobsec = row.GetInt("NUMPTObjectSecurity");
          // properties
          strSql = "SELECT COUNT(*) AS NUMPROPS FROM PTOBJECTPROPERTIES WHERE OBJECTID > ?";
          cursor = testDBService.CreatePTCursor(strSql);
          nSetting = 199;
          cursor.SetInt(0, nSetting);
          resultset = cursor.Open(1);
          row = resultset.GetRow(0);
          int numobprops = row.GetInt("NUMPROPS");
          // prefs
          strSql = "SELECT COUNT(*) AS NUMPREFS FROM PTPREFS WHERE USERID > ?";
          cursor = testDBService.CreatePTCursor(strSql);
          nSetting = 2;
          cursor.SetInt(0, nSetting);
          resultset = cursor.Open(1);
          row = resultset.GetRow(0);
          int numprefs = row.GetInt("NUMPREFS");
          // web services
          strSql = "SELECT COUNT(*) AS NUMSVCS FROM PTWEBSERVICES WHERE OBJECTID > ?";
          cursor = testDBService.CreatePTCursor(strSql);
          nSetting = 199;
          cursor.SetInt(0, nSetting);
          resultset = cursor.Open(1);
          row = resultset.GetRow(0);
          int numwebsvcs = row.GetInt("NUMSVCS");
          // remote servers
          strSql = "SELECT COUNT(*) AS NUMSERVERS FROM PTGCSERVERS WHERE OBJECTID > ?";
          cursor = testDBService.CreatePTCursor(strSql);
          nSetting = 199;
          cursor.SetInt(0, nSetting);
          resultset = cursor.Open(1);
          row = resultset.GetRow(0);
          int numremservers = row.GetInt("NUMSERVERS");
          // experience definitions
          strSql = "SELECT COUNT(*) AS NUMEXP FROM PTSUBPORTALS WHERE OBJECTID > ?";
          cursor = testDBService.CreatePTCursor(strSql);
          nSetting = 199;
          cursor.SetInt(0, nSetting);
          resultset = cursor.Open(1);
          row = resultset.GetRow(0);
          int numexpdef = row.GetInt("NUMEXP");
          // experience rules
          strSql = "SELECT COUNT(*) AS NUMRULES FROM PTEXPRESSIONS WHERE OBJECTID > ?";
          cursor = testDBService.CreatePTCursor(strSql);
          nSetting = 199;
          cursor.SetInt(0, nSetting);
          resultset = cursor.Open(1);
          row = resultset.GetRow(0);
          int numrules = row.GetInt("NUMRULES");
          strSql = "SELECT COUNT(*) AS NUMRULES2 FROM PTEXPRESSIONSINEXPRESSIONS WHERE PARENTEXPRESSIONID > ?";
          cursor = testDBService.CreatePTCursor(strSql);
          nSetting = 0;
          cursor.SetInt(0, nSetting);
          resultset = cursor.Open(1);
          row = resultset.GetRow(0);
          int numrulesinrules = row.GetInt("NUMRULES2");
          // jobs
          strSql = "SELECT COUNT(*) AS NUMJOBS FROM PTJOBS WHERE OBJECTID > ?";
          cursor = testDBService.CreatePTCursor(strSql);
          nSetting = 199;
          cursor.SetInt(0, nSetting);
          resultset = cursor.Open(1);
          row = resultset.GetRow(0);
          int numjobs = row.GetInt("NUMJOBS");
          /* generate report */
          Reporter.log(true, "----------- DATABASE SIZING STATISTICS ------------");
          Reporter.log(true, "pages: " + (numpages).ToString());
          Reporter.log(true, "communities: " + (numcommunities).ToString());
          Reporter.log(true, "portlets: " + (numgadgets).ToString());
          Reporter.log(true, "users: " + (numusers).ToString());
          Reporter.log(true, "cards: " + (numcards).ToString());
          Reporter.log(true, "group memberships: " + (numgroupmem).ToString());
          Reporter.log(true, "jobs: " + (numjobs).ToString());
          Reporter.log(true, "joblogs: " + (numjoblogs).ToString());
          Reporter.log(true, "object security: " + (numobsec).ToString());
          Reporter.log(true, "object properties: " + (numobprops).ToString());
          Reporter.log(true, "preferences: " + (numprefs).ToString());
          Reporter.log(true, "web services: " + (numwebsvcs).ToString());
          Reporter.log(true, "remote servers: " + (numremservers).ToString());
          Reporter.log(true, "experience definitions: " + (numexpdef).ToString());
          Reporter.log(true, "experience rules: " + (numrules + numrulesinrules).ToString());
          Reporter.log(true, "----------------------------------------------------");
          }
      }
      catch (Exception e)
      {
        String errorMessage = "Database Version Problem: ";
        retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, errorMessage,
         XPException.GetInstance(e));
      }
      return retVal;
    }
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsg(int)
     */
     
    public virtual XPArrayList getMsg(int stepid, Platform platform, ReturnValue ret)
    {
      // Use the diagnostic helper to find the right message        
      return RecommendationHelper.getList(stepid, platform, ret.getResultCode(),
       m_diag);
    }
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsgArray()
     */
     
    public virtual StartupTaskMessage[] getMsgArray()
    {
      return m_diag;
    }
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getNumMsgs()
     */
     
    public virtual int getNumMsgs()
    {
      return m_diag.Length;
    }
    
    /// <summary>
    /// Returns a simple queryfilter based on three parameters
    /// </summary>
    /// <author>JohnH</author>
    /// <param name="int">nOperand - the Plumtree PropID operand (of type PT_PROPIDS)</param>
    /// <param name="int">nOperator - the Plumtree Operator (of type PT_FILTEROPS)</param>
    /// <param name="Object">objValue - the value to be filtered on (Integer, String, [], etc...)</param>
    public static Object[][] GetQueryFilter(int nOperand, int nOperator, Object objValue)
    {
      Object[][] ptqf = new Object[3][];
      for (int j2c_index = 0;j2c_index < 3;j2c_index++)
      {
        ptqf[j2c_index] = new Object[1];
      }
      ptqf[0][0] = nOperand;
      ptqf[1][0] = nOperator;
      ptqf[2][0] = objValue;
      return ptqf;
    }
    
    /// <summary>
    /// Generic object counter by manager
    /// </summary>
    /// <param name="mgr"></param>
    /// <returns>int num items</returns>
    public static int GetObjectCount(IPTObjectManager mgr)
    {
      Object[] nLevels = new Object[1];
      nLevels[0] = PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_READ;
      Object[][] ptqf = GetQueryFilter(PT_PROPIDS.PT_PROPID_ACCESSLEVEL, PT_FILTEROPS.PT_FILTEROP_IN,
       nLevels);
      int numObjects = mgr.QueryCount(-1, ptqf);
      return numObjects;
    }
  }
}