using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.xml.dom;
using com.plumtree.openkernel.config;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.application;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.uiinfrastructure.application.varpacks
{
  /// <summary>
  /// This varpack contains the base application data.
  /// This is read from the base x_config.xml file.
  /// </summary>
  public class ConfigVarPack : BaseVarPack
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.application.varpacks.ConfigVarPack");
    
    /// <summary>
    /// The string ID of this variable package. 
    /// </summary>
    public const String VARPACK_ID = "ConfigVarPack";
    // OpenConfig component types
    public const String PORTAL_MAINURLS_COMPONENT_TYPE = "http://www.plumtree.com/config/component/types/portal/mainurls";
    public const String PORTAL_SYSTEMPROPERTIES_COMPONENT_TYPE = "http://www.plumtree.com/config/component/types/portal/systemproperties";
    public const String PORTAL_URLMAPPING_COMPONENT_TYPE = "http://www.plumtree.com/config/component/types/portal/urlmapping";
    public const String PORTAL_PERSONALSETTINGS_COMPONENT_TYPE = "http://www.plumtree.com/config/component/types/portal/personalsettings";
    public const String PORTAL_CACHEDSETTINGS_COMPONENT_TYPE = "http://www.plumtree.com/config/component/types/portal/cachedsettings";
    public const String PORTAL_AUTHENTICATION_COMPONENT_TYPE = "http://www.plumtree.com/config/component/types/portal/authentication";
    public const String PORTAL_SECURITY_COMPONENT_TYPE = "http://www.plumtree.com/config/component/types/portal/security";
    public const String PORTAL_INTERNATIONAL_COMPONENT_TYPE = "http://www.plumtree.com/config/component/types/portal/international";
    protected bool m_bIsJava = false;
    protected bool m_bIsDotNet = false;
    
    /// <summary>
    /// JF- This XPHashtable contains all the settings from x_config.xml (keys and values)
    /// We need it for the Memory Debug page where we need to be able to iterate through the keys to display all the values.
    /// </summary>
    protected XPHashtable m_htConfigXMLSettings;
    
    /// <summary>
    /// JF- Security Mode: No SSL check. 
    /// </summary>
    public const int SECURITY_MODE_NO_SSL = 0;
    
    /// <summary>
    /// JF- Security Mode: SSL on some AS only. List of AS defined in SecureActivitySpaces.xml. 
    /// </summary>
    public const int SECURITY_MODE_SSL_CRITICAL_AS_ONLY = 1;
    
    /// <summary>
    /// JF- Security Mode: SSL on every AS. 
    /// </summary>
    public const int SECURITY_MODE_SSL_ALL = 2;
    
    /// <summary>
    /// JF- Security Mode: SSL on every AS using an SSL Accelerator hardware. 
    /// </summary>
    public const int SECURITY_MODE_SSL_ACCELERATOR = 3;
    public const String APP_PLATFORM_DOT_NET = "DotNet";
    public const String APP_PLATFORM_JAVA = "Java";
    
    /// <seealso cref="com.plumtree.uiinfrastructure.web.IVarPack#GetVarPackID()"></seealso>
    public override String GetVarPackID()
    {
      return VARPACK_ID;
    }
    
    /// <summary>
    /// Constructor for ConfigVarPack.
    /// </summary>
    /// <param name="__strPlatform">The current platform "Java" or "DotNet")</param>
    public ConfigVarPack(String _strPlatform) : base()
    {
      if (0 == XPStringUtility.CompareIgnoreCase(APP_PLATFORM_DOT_NET, _strPlatform))
      {
        m_bIsDotNet = true;
        m_bIsJava = false;
      } else
      if (0 == XPStringUtility.CompareIgnoreCase(APP_PLATFORM_JAVA, _strPlatform))
      {
        m_bIsDotNet = false;
        m_bIsJava = true;
      } else
      {
        log.Error("ConfigVarPack initialized with unknown platform (expecting " + APP_PLATFORM_DOT_NET + " or " + APP_PLATFORM_JAVA + "): " + _strPlatform);
        }
    }
    
    /// <summary>
    /// Returns the XPHashtable with all the setings from config.xml
    /// </summary>
    /// <returns>XPHashtable</returns>
    public virtual XPHashtable GetConfigXMLSettings()
    {
      return m_htConfigXMLSettings;
    }
    
    /// <summary>
    /// Returns whether or not the current platform is Java.
    /// </summary>
    /// <returns>boolean true implies Java.</returns>
    public virtual bool GetIsJava()
    {
      return m_bIsJava;
    }
    
    /// <summary>
    /// Returns whether or not the current platform is .NET.
    /// </summary>
    /// <returns>boolean true implies .NET.</returns>
    public virtual bool GetIsDotNet()
    {
      return m_bIsDotNet;
    }
    
    /// <param name="_strSection"></param>
    /// <param name="_strSubSection"></param>
    /// <param name="_strAttribute"></param>
    /// <returns></returns>
    public virtual int GetConfigValueAsInt(String _strSection, String _strSubSection,
     String _strAttribute)
    {
      String strValue = GetConfigValueAsString(_strSection, _strSubSection,
       _strAttribute);
      if ((null == strValue) || ("".Equals(strValue)))
      {
        // Unable to find config value.  Cannot convert to integer.
        throw new XPException("Unable to retrieve config value " + _strSection + " : " + _strSubSection + " : " + _strAttribute + " as integer.  Value was null or empty.");
        }
      return XPConvert.ToInteger(strValue);
    }
    
    /// <param name="_strSection"></param>
    /// <param name="_strSubSection"></param>
    /// <param name="_strAttribute"></param>
    /// <returns></returns>
    public virtual String GetConfigValueAsString(String _strSection, String _strSubSection,
     String _strAttribute)
    {
      String strRetVal = InternalGetConfigValueAsString(_strSection, _strSubSection,
       _strAttribute, true);
      // can be null, so check and replace with empty string.
      if (strRetVal == null)
      {
        strRetVal = "";
      }
      return strRetVal;
    }
    
    /// <summary>
    /// This is an internal helper method.
    /// </summary>
    /// <param name="xmlDoc">the base xml doc to read the values from</param>
    /// <param name="_strSection"></param>
    /// <param name="_strSubSection"></param>
    /// <param name="_strAttribute"></param>
    /// <param name="bReportErrors">true implies report errors</param>
    /// <returns>String null if not found</returns>
    protected virtual String InternalGetConfigValueAsString(String _strSection,
     String _strSubSection, String _strAttribute, bool bReportErrors)
    {
      log.Debug("Retrieving Config Value from xml doc: " + _strSection + ", " + _strSubSection + ", " + _strAttribute + ".");
      String strRetVal = null;
      try
      {
        String strKey = _strSection + _strSubSection;
        strRetVal = XPConvert.ToString(m_htConfigXMLSettings.GetElement(strKey));
        }
      catch (Exception e)
      {
        // Default error
        if (bReportErrors && log.IsErrorEnabled())
        {
          log.Error(e, "GetConfigValueAsString() failed.  The section is " + _strSection + " and the subsection is " + _strSubSection + ".");
          }
        // Just return null string if we fail to get the value
        strRetVal = null;
      }
      return strRetVal;
    }
    
    /// <param name="_strSection"></param>
    /// <param name="_strSubSection"></param>
    /// <param name="_strAttribute"></param>
    /// <returns></returns>
    public virtual Object[] GetConfigValues(String _strSection, String _strSubSection,
     String _strAttribute)
    {
      return InternalGetConfigValues(_strSection, _strSubSection, _strAttribute,
       true);
    }
    
    /// <summary>
    /// This is an internal helper method
    /// </summary>
    /// <param name="xmlDoc">the base xml doc to read the values from</param>
    /// <param name="_strSection"></param>
    /// <param name="_strSubSection"></param>
    /// <param name="_strAttribute"></param>
    /// <param name="bReportErrors">true implies report errors</param>
    /// <returns></returns>
    protected virtual Object[] InternalGetConfigValues(String _strSection,
     String _strSubSection, String _strAttribute, bool bReportErrors)
    {
      if (bReportErrors && (log.IsInfoEnabled()))
      {
        log.Debug("Retrieving Config Values from xml doc: " + _strSection + ", " + _strSubSection + ", " + _strAttribute + ".");
        }
      XPArrayList alElements = new XPArrayList();
      try
      {
        IXPEnumerator eKeys = m_htConfigXMLSettings.GetKeys();
        while (eKeys.MoveNext())
        {
          String strKey = XPConvert.ToString(eKeys.GetCurrent());
          if (XPStringUtility.StartsWith(strKey, _strSection))
          {
            alElements.Add(m_htConfigXMLSettings.GetElement(strKey));
          }
        }
      }
      catch (Exception e)
      {
        // Default error
        if (bReportErrors && (log.IsErrorEnabled()))
        {
          log.Error(e, "GetConfigValues() failed.  The Section was " + _strSection + ".");
          }
        // Just return empty array if we fail to get the value
        return new Object[0];
      }
      return alElements.ToArray();
    }
    
    /// <summary>
    /// Helper method that returns an xml element consisting of all the XML
    /// in the  specified section and subsection. Returns null if no such
    /// section or subsection exists
    /// </summary>
    /// <param name="_xmlDoc">the XPDocument</param>
    /// <param name="_strSection"></param>
    /// <param name="_strSubSection"></param>
    /// <returns></returns>
    protected static XPElement InternalGetXMLElement(XPDocument _xmlDoc, String _strSection,
     String _strSubSection)
    {
      XPElement xmlRoot = _xmlDoc.GetDocumentElement();
      XPNodeList xmlList = xmlRoot.GetElementsByTagName(_strSection);
      if ((xmlList != null) && (xmlList.GetLength() > 0))
      {
        XPElement xmlSection = (XPElement) xmlList.Item(0);
        XPNodeList xmlSubList = xmlSection.GetElementsByTagName(_strSubSection);
        if ((xmlSubList != null) && (xmlSubList.GetLength() > 0))
        {
          return (XPElement) xmlSubList.Item(0);
        } else
        {
          return null;
        }
      } else
      {
        return null;
      }
    }
    
    /// <summary>
    /// This method will return an enumeration of the top level nodes in the XML
    /// tree that is x_config.xml.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.web.IVarPack#GetVarPackKeys()"></seealso>
    public override IXPEnumerator GetVarPackKeys()
    {
      return m_htConfigXMLSettings.GetKeys();
    }
    
    /// <summary>
    /// You cannot expire the x_config.xml content. You can only reload it. This
    /// method does nothing.
    /// </summary>
    /// <seealso cref="com.plumtree.xpsupport.web.BaseVarPack#ExpireContent()"></seealso>
    protected override void ExpireContent()
    {
    }
    
    /// <summary>
    /// This method will return a chunk of from one level under the Config
    /// node.
    /// </summary>
    /// <seealso cref="com.plumtree.xpsupport.web.BaseVarPack#GetValue(String)"></seealso>
    protected override Object GetValue(String arg0)
    {
      return m_htConfigXMLSettings.GetElement(arg0);
    }
    
    /// <summary>
    /// This method loads the x_config.xml file into memory.
    /// </summary>
    /// <seealso cref="com.plumtree.xpsupport.web.BaseVarPack#Load()"></seealso>
    protected override void Load()
    {
      /*
       String strFileSeparator = XPFileSystemObject.GetPathSeparator();
       
       String strConfigFileName;
       if (m_bIsDotNet)
       {
       strConfigFileName = "n_config.xml";
       }
       else if (m_bIsJava)
       {
       strConfigFileName = "j_config.xml";
       } else
       {
       throw new XPException("Could not load Config XML document.  Unknown platform.");
       }
       
       String strConfigPath = GetConfigPath() + strFileSeparator + strConfigFileName;
       strConfigPath = GeneralHelpers.FormatPath(strConfigPath, strFileSeparator, false);
       
       
       log.Info("Loading Config XML document from " + strConfigPath);
       
       
       XPDocument xmlDoc = XPBuilder.LoadDocument(strConfigPath);
       
       if (xmlDoc == null)
       {
       
       log.Error("Could not load Config XML document from " + strConfigPath);
       
       
       throw new XPException("Could not load Config XML document from " + strConfigPath);
       }
       */
       
      // NOTE DonH: Reloading this will not change the base URLS, because
      // they are stored in ConfigHelper.  Reloading isn't be
      // threadsafe anyways.
      LoadSettingsIntoXPHashtable();
      log.Info("Finished loading uiinfrastructure config XML document.");
      }
    
    /// <summary>
    /// Helper method to load settings in the specified OpenConfig component
    /// into the XPHashtable.
    /// </summary>
    /// <param name="component"></param>
    protected virtual void LoadComponentSettings(IOKComponent component, String sectionName)
    {
      if (null == component)
      {
        log.Warn("Passing in null OpenConfig component for section name: {0}",
         sectionName);
        return;
      }
      IOKSetting[] settings = component.querySettings();
      for (int j = 0;j < settings.Length;j++)
      {
        String strName = settings[j].getName();
        String strValue = null;
        //log.Info("Setting Loaded: " + strName);		        
        //log.Info("Setting type: " + settings[j].getDataType());
        int nSettingType = settings[j].getDataType();
        switch (nSettingType){
        case OKConfigDataTypes.TYPE_BOOLEAN:
        strValue = XPConvert.ToString(settings[j].getBooleanValue());
        break;
        case OKConfigDataTypes.TYPE_STRING:
        strValue = settings[j].getStringValue();
        break;
        case OKConfigDataTypes.TYPE_INTEGER:
        strValue = XPConvert.ToString(settings[j].getIntValue());
        break;
        case OKConfigDataTypes.TYPE_FLOAT:
        strValue = XPConvert.ToString(settings[j].getFloatValue());
        break;
        case OKConfigDataTypes.TYPE_DATE:
        strValue = XPConvert.ToString(settings[j].getDateValue());
        break;
        default:
        throw new XPException("Unknown data type:" + nSettingType);
        break;
        }
        //log.Info("Setting value: " + strValue);
        if ((null != strName) && (null != strValue))
        {
          // JF- x_config.xml is two levels deep.
          // WilliamA- we preserve this prefix by "section" still when we use 
          // OpenConfig here
          String strKey = sectionName + strName;
          m_htConfigXMLSettings.PutElement(strKey, strValue);
        }
      }
    }
    
    /// <summary>
    /// Helper method to load a number of component settings from portalconfig.xml 
    /// and put them in a XPHashtable.
    /// This Hashtable will be used for the Memory Debug page where we need to be able
    /// to iterate through the keys and display all the name/value pairs.
    /// </summary>
    private void LoadSettingsIntoXPHashtable()
    {
      m_htConfigXMLSettings = new XPHashtable();
      // Use OpenConfig to read the settings
      IOKContext configContext = AppWarmUp.GetOKConfigContext();
      IOKComponent component = null;
      component = configContext.getComponentByType(PORTAL_MAINURLS_COMPONENT_TYPE);
      LoadComponentSettings(component, ConfigConstants.MAIN_URLS_SECTION);
      component = configContext.getComponentByType(PORTAL_SYSTEMPROPERTIES_COMPONENT_TYPE);
      LoadComponentSettings(component, ConfigConstants.SYSTEM_PROPERTIES_SECTION);
      component = configContext.getComponentByType(PORTAL_URLMAPPING_COMPONENT_TYPE);
      LoadComponentSettings(component, ConfigConstants.URL_MAPPING_SECTION);
      component = configContext.getComponentByType(PORTAL_PERSONALSETTINGS_COMPONENT_TYPE);
      LoadComponentSettings(component, ConfigConstants.PERSONAL_SETTINGS_SECTION);
      component = configContext.getComponentByType(PORTAL_CACHEDSETTINGS_COMPONENT_TYPE);
      LoadComponentSettings(component, ConfigConstants.CACHED_SETTINGS_SECTION);
      component = configContext.getComponentByType(PORTAL_AUTHENTICATION_COMPONENT_TYPE);
      LoadComponentSettings(component, ConfigConstants.AUTHENTICATION_SECTION);
      component = configContext.getComponentByType(PORTAL_SECURITY_COMPONENT_TYPE);
      LoadComponentSettings(component, ConfigConstants.SECURITY_SECTION);
      component = configContext.getComponentByType(PORTAL_INTERNATIONAL_COMPONENT_TYPE);
      LoadComponentSettings(component, ConfigConstants.INTERNATIONAL_SECTION);
      }
    
    /// <summary>
    /// Helper method to load the settings from x_config.xml and put them in a XPHashtable.
    /// This Hashtable will be used for the Memory Debug page where we need to be able
    /// to iterate through the keys and display all the name/value pairs.
    /// </summary>
    /// <deprecated>Use LoadSettingsIntoXPHashtable() instead that uses OpenConfig API</deprecated>
    private void LoadSettingsIntoXPHashtable(XPDocument xmlDoc)
    {
      m_htConfigXMLSettings = new XPHashtable();
      // JF- x_config.xml is two levels deep.
      // We need to iterate through both levels before we can actually get the name/value pairs.
      if (xmlDoc == null)
      {
        throw new XPException("Config XML document not found.");
      }
      XPElement nTopNode = xmlDoc.GetDocumentElement();
      log.Info("The top root node of x_config.xml is " + nTopNode.GetNodeName());
      if (nTopNode == null)
      {
        throw new XPException("Config XML document top node not found.");
        }
      XPNodeList nlFirstLevel = nTopNode.GetChildNodes();
      if (nlFirstLevel == null)
      {
        throw new XPException("Config XML document first level nodes not found.");
        }
      XPNodeList nlSecondLevel;
      String sName;
      String sValue;
      String sFirstLevelName;
      for (int i = 0;i < nlFirstLevel.GetLength();i++)
      {
        // JF- Make sure this is a valid node and not a comment
        if (!XPStringUtility.StartsWith(nlFirstLevel.Item(i).GetNodeName(),
         "#"))
        {
          nlSecondLevel = nlFirstLevel.Item(i).GetChildNodes();
          sFirstLevelName = nlFirstLevel.Item(i).GetNodeName();
          for (int k = 0;k < nlSecondLevel.GetLength();k++)
          {
            sName = nlSecondLevel.Item(k).GetNodeName();
            // JF- Make sure this is a valid node and not a comment
            if (!XPStringUtility.StartsWith(sName, "#"))
            {
              String sKey = sFirstLevelName + sName;
              sValue = ((XPElement) nlSecondLevel.Item(k)).GetAttributeNode(ConfigConstants.VALUE_ATTRIBUTE).GetNodeValue();
              m_htConfigXMLSettings.PutElement(sKey, sValue);
            }
          }
        }
      }
    }
    
    /// <summary>
    /// This method reloads the x_config.xml file into memory. It clears the
    /// current data rather than delegating to Load().
    /// </summary>
    /// <seealso cref="com.plumtree.xpsupport.web.BaseVarPack#Reload()"></seealso>
    protected override void Reload()
    {
      //m_xmlDoc					    = null;
      m_htConfigXMLSettings = null;
      Load();
    }
  }
}