using System;
using com.plumtree.openlog;
using com.plumtree.openfoundation.io;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.xml.dom;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.web;
using com.plumtree.uiinfrastructure.application;
using com.plumtree.openkernel.config;
namespace com.plumtree.uiinfrastructure.application.varpacks
{
  /// <summary>
  /// This VarPack is used to store the list of secure ASs. The list is loaded by
  /// the WarmUp Servlet from SecureActivitySpaces.xml. In Security Mode 1,
  /// Activity Spaces in this list will be automatically redirected to https.
  /// </summary>
  /// <author>John Osborne</author>
  public class SecureASVarPack : BaseVarPack
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.application.varpacks.SecureASVarPack");
    
    /// <summary>
    /// The string ID of this variable package. 
    /// </summary>
    public const String VARPACK_ID = "SecureASVarPack";
    // OpenConfig component type
    public const String PORTAL_SECURESPACES_COMPONENT_TYPE = "http://www.plumtree.com/config/component/types/portal/secureactivityspaces";
    
    /// <summary>
    /// Constant for the entry name in the xml file. 
    /// </summary>
    private const String XML_ENTRY_SPACE = "space";
    
    /// <summary>
    /// Constant for the entry name in the xml file. 
    /// </summary>
    private const String XML_ENTRY_NAME = "name";
    
    /// <summary>
    /// Holds a hashtable of the secure Activity Spaces. 
    /// </summary>
    private XPHashtable m_htSecureASList;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.web.IVarPack#GetVarPackID()"></seealso>
    public override String GetVarPackID()
    {
      return VARPACK_ID;
    }
    
    /// <summary>
    /// This method will return an enumeration of the secure Activity Spaces.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.web.IVarPack#GetVarPackKeys()"></seealso>
    public override IXPEnumerator GetVarPackKeys()
    {
      if (m_htSecureASList == null)
      {
        return null;
      } else
      {
        return m_htSecureASList.GetKeys();
      }
    }
    
    /// <summary>
    /// Returns true if the AS identified by the name in argument is in the list
    /// and should be secured.
    /// </summary>
    /// <param name="_sASName"></param>
    /// <returns>boolean</returns>
    public virtual bool IsSpaceSecured(String _sASName)
    {
      if (m_htSecureASList == null)
      {
        throw new XPException("The Secure AS has not been initialized.");
        }
      if (m_htSecureASList.ContainsKey(_sASName))
      {
        return true;
      }
      return false;
    }
    
    /// <summary>
    /// You cannot the secure space list.
    /// </summary>
    /// <seealso cref="com.plumtree.xpsupport.web.BaseVarPack#ExpireContent()"></seealso>
    protected override void ExpireContent()
    {
    }
    
    /// <summary>
    /// This method will return the name of the secure space if it is in this
    /// variable package.
    /// </summary>
    /// <seealso cref="com.plumtree.xpsupport.web.BaseVarPack#GetValue(String)"></seealso>
    protected override Object GetValue(String arg0)
    {
      try
      {
        if (m_htSecureASList == null)
        {
          return null;
        }
        return m_htSecureASList.GetElement(arg0);
      }
      catch (XPException e)
      {
        log.Error("SecureASVarPack::GetValue Unable to get the value for key: " + arg0 + " The following error ocurred: " + e);
        }
      return null;
    }
    
    /// <summary>
    /// This method loads the secure space list into memory.
    /// </summary>
    /// <seealso cref="com.plumtree.xpsupport.web.BaseVarPack#Load()"></seealso>
    protected override void Load()
    {
      /*
       String strFileSeparator = XPFileSystemObject.GetPathSeparator();
       String strSecurityFilePath = GetConfigPath() + strFileSeparator + "SecureActivitySpaces.xml";
       
       strSecurityFilePath = GeneralHelpers.FormatPath(strSecurityFilePath, strFileSeparator, false);
       XPDocument xmlDoc = XPBuilder.LoadDocument(strSecurityFilePath);
       */
       
      LoadASIntoXPHashtable();
    }
    
    /// <summary>
    /// This method reloads the x_config.xml file into memory.  It clears the
    /// current data rather than delegating to Load().
    /// </summary>
    /// <seealso cref="com.plumtree.xpsupport.web.BaseVarPack#Reload()"></seealso>
    protected override void Reload()
    {
      // JF- Reset member variables
      m_htSecureASList = null;
      // JF- Reload the values
      Load();
    }
    
    /// <summary>
    /// Helper method to load the Activity Space list from the XML DOM into an
    /// XPHashtable.
    /// </summary>
    /// <throws>XPException</throws>
    private void LoadASIntoXPHashtable()
    {
      m_htSecureASList = new XPHashtable();
      // Use OpenConfig to read the settings
      IOKContext configContext = AppWarmUp.GetOKConfigContext();
      IOKComponent component = null;
      component = configContext.getComponentByType(PORTAL_SECURESPACES_COMPONENT_TYPE);
      if (null == component)
      {
        log.Warn("Cannot retrieve component type: {0}", PORTAL_SECURESPACES_COMPONENT_TYPE);
        return;
      }
      IOKSetting[] settings = component.querySettings();
      for (int j = 0;j < settings.Length;j++)
      {
        String strName = settings[j].getName();
        log.Debug("Secure activity space: " + strName);
        if (null != strName)
        {
          m_htSecureASList.PutElement(strName, strName);
        }
      }
    }
    
    /// <summary>
    /// Helper method to load the Activity Space list from the XML DOM into an
    /// XPHashtable.
    /// </summary>
    /// <param name="xmlDoc">The document that contains the secure space list.</param>
    /// <throws>XPException</throws>
    /// <deprecated>Use LoadASIntoXPHashtable() instead, which uses OpenConfig API.</deprecated>
    private void LoadASIntoXPHashtable(XPDocument xmlDoc)
    {
      if (xmlDoc == null)
      {
        throw new XPException("LoadASIntoXPHashtable called with null parameter.");
        }
      XPElement nTopNode = xmlDoc.GetDocumentElement();
      log.Info("The top root node is " + nTopNode.GetNodeName());
      if (nTopNode == null)
      {
        throw new XPException("The Secure AS list XML document top node not found.");
        }
      XPElement xpElem = xmlDoc.GetDocumentElement();
      XPNodeList nlFirstLevel = xpElem.GetElementsByTagName(XML_ENTRY_SPACE);
      if (nlFirstLevel == null)
      {
        throw new XPException("The Secure AS list XML document first level nodes not found.");
        }
      m_htSecureASList = new XPHashtable();
      for (int i = 0;i < nlFirstLevel.GetLength();i++)
      {
        if (!XPStringUtility.StartsWith(nlFirstLevel.Item(i).GetNodeName(),
         "#"))
        {
          String strKey = ((XPElement) nlFirstLevel.Item(i)).GetAttributeNode(XML_ENTRY_NAME).GetNodeValue();
          String strValue = strKey;
          m_htSecureASList.PutElement(strKey, strValue);
        }
      }
    }
  }
}