using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.application;
using com.plumtree.uiinfrastructure.web;
using com.plumtree.openkernel.config;
namespace com.plumtree.uiinfrastructure.application.varpacks
{
  /// <summary>
  /// This VarPack is used to determine which Activity Space to go
  /// to if the space is not on the URL, based on what parameters
  /// are available on the URL. 
  /// </summary>
  /// <author>Ricky Wong</author>
  public class URLASMappingsVarPack : BaseVarPack
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.application.varpacks.URLASMappingsVarPack");
    
    /// <summary>
    /// The String ID of this VarPack. 
    /// </summary>
    public const String VARPACK_ID = "URLASMappingsVarPack";
    
    /// <summary>
    /// The XML file this VarPack loads. 
    /// </summary>
    public const String VARPACK_FILE = "URLASMappings.xml";
    // OpenConfig component type
    public const String PORTAL_URLASMAPPINGS_COMPONENT_TYPE = "http://www.plumtree.com/config/component/types/portal/urlasmappings";
    
    /// <summary>
    /// Parameter specific mappings. 
    /// </summary>
    private XPHashtable m_htParameterMap;
    
    /// <summary>
    /// Node names from our XML file. 
    /// </summary>
    private const String NODE_PARAMETER = "ParameterMaps";
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.web.IVarPack#GetVarPackID()
     */
     
    public override String GetVarPackID()
    {
      return VARPACK_ID;
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.varpacks.XMLBaseVarPack#GetVarPackXMLFileName()
     */
     
    public virtual String GetVarPackXMLFileName()
    {
      return VARPACK_FILE;
    }
    
    /// <summary>
    /// Loads in the same manner as XMLBaseVarPack, but then we
    /// organize the data for URLASMapping specific purposes.
    /// </summary>
    protected override void Load()
    {
      // initialize storage members.
      m_htParameterMap = new XPHashtable();
      // Use OpenConfig to read the settings
      IOKContext configContext = AppWarmUp.GetOKConfigContext();
      IOKComponent component = null;
      component = configContext.getComponentByType(PORTAL_URLASMAPPINGS_COMPONENT_TYPE);
      if (null == component)
      {
        log.Warn("Cannot retrieve component type: {0}", PORTAL_URLASMAPPINGS_COMPONENT_TYPE);
        return;
      }
      IOKSetting[] settings = component.querySettings();
      for (int j = 0;j < settings.Length;j++)
      {
        String strName = settings[j].getName();
        String strValue = null;
        // log.Info("Setting Loaded: " + strName);		        
        int nSettingType = settings[j].getDataType();
        switch (nSettingType){
        // the value is a space name, so only TYPE_STRING should be supported
        case OKConfigDataTypes.TYPE_STRING:
        strValue = settings[j].getStringValue();
        break;
        default:
        throw new XPException("Unknown data type:" + nSettingType);
        break;
        }
        // log.Info("Setting value: " + strValue);
        if ((null != strName) && (null != strValue))
        {
          m_htParameterMap.PutElement(strName, strValue);
        }
      }
    }
    
    /// <summary>
    /// Returns an enumeration of the Parameter map keys.
    /// </summary>
    /// <returns>IXPEnumerator of the Parameter map keys.</returns>
    public virtual IXPEnumerator GetParameterMapKeys()
    {
      if (null != m_htParameterMap)
      {
        return m_htParameterMap.GetKeys();
      } else
      {
        return null;
      }
    }
    
    /// <summary>
    /// Returns the Parameter map value for the given key.
    /// </summary>
    /// <param name="strKey">The Parameter key.</param>
    /// <returns>The Parameter value.</returns>
    public virtual String GetParameterMapValue(String strKey)
    {
      if (null != m_htParameterMap)
      {
        return XPConvert.ToString(m_htParameterMap.GetElement(strKey));
      } else
      {
        return null;
      }
    }
    
    /// <summary>
    /// This method will return an enumeration of the top level nodes in the XML
    /// tree that is x_config.xml.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.web.IVarPack#GetVarPackKeys()"></seealso>
    public override IXPEnumerator GetVarPackKeys()
    {
      if (null != m_htParameterMap)
      {
        return m_htParameterMap.GetKeys();
      } else
      {
        return null;
      }
    }
    
    /// <summary>
    /// This method will return a chunk of from one level under the PTConfig
    /// node.
    /// </summary>
    /// <seealso cref="com.plumtree.xpsupport.web.BaseVarPack#GetValue(String)"></seealso>
    protected override Object GetValue(String arg0)
    {
      if (null != m_htParameterMap)
      {
        return XPConvert.ToString(m_htParameterMap.GetElement(arg0));
      } else
      {
        return null;
      }
    }
    
    /// <summary>
    /// You cannot expire the x_config.xml content. You can only reload it. This
    /// method does nothing.
    /// </summary>
    /// <seealso cref="com.plumtree.xpsupport.web.BaseVarPack#ExpireContent()"></seealso>
    protected override void ExpireContent()
    {
    }
    
    /// <summary>
    /// This method reloads the XML file into memory. It delegates to
    /// Load().  This may not be entirely threadsafe.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.web.BaseVarPack#Reload()"></seealso>
    protected override void Reload()
    {
      Load();
    }
  }
}