using System;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
namespace com.plumtree.uiinfrastructure.approval
{
  /// <author>
  /// Beng
  /// This abstract model for the approval framework provides much functionality
  /// for the ui.
  /// <br /><br />
  /// Concrete models for specific implementations of approval should extend this
  /// abstract model.
  /// <br /><br />
  /// Currently there are two known concrete subclasses of this class:
  /// DocumentApprovalModel and ObjectApprovalModel.
  /// <br /><br />
  /// This class implements general functionality common to approval display pages.
  /// Specifically, page management and caching.
  /// </author>
  public abstract class AApprovalModel : EditorModel,IModel,IApprovalModelRO
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.approval.AApprovalModel");
    internal String sItemID = null;
    internal bool bAddComment = false;
    private bool bDisplayItemComment = true;
    private bool bDisplayViewItems = false;
    private bool bSortName = false;
    private bool bSortSubmittedBy = false;
    private int iPageSize = 15;
    private int m_iLow = -1;
    private int m_iHigh = -1;
    private String sNoItemsDescription = null;
    private ApprovalItemType currentType = null;
    // JF- Associated Activity Space
    private XPHashtable countMap = null;
    private XPHashtable globalItemMap = null;
    private XPHashtable itemMap = null;
    private XPHashtable pageIndexMap = null;
    private bool bSortDate = false;
    public AApprovalModel()
    {
      currentType = this.GetDefaultItemType();
      // JF- Associated Activity Space
      countMap = new XPHashtable();
      globalItemMap = new XPHashtable();
      itemMap = new XPHashtable();
      pageIndexMap = new XPHashtable();
    }
    public virtual void SetNoItemsDescription(String s)
    {
      sNoItemsDescription = s;
    }
    public virtual String GetNoItemsDescription()
    {
      return sNoItemsDescription;
    }
    
    /// <returns></returns>
    public abstract bool AddColumnsHeaderForEachItem();
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IModel#CleanupTempData()"></seealso>
    public override void CleanupTempData()
    {
      log.Debug("CleanupTempData()_____________________");
      this.bAddComment = false;
    }
    
    /// <returns></returns>
    public abstract bool DisplayDescriptionImmediatelyUnderTitle();
    
    /// <returns></returns>
    public virtual bool DoSortDate()
    {
      return bSortDate;
    }
    
    /// <returns></returns>
    public virtual bool DoSortName()
    {
      return bSortName;
    }
    
    /// <returns></returns>
    public virtual bool DoSortSubmittedBy()
    {
      return bSortSubmittedBy;
    }
    
    /// <returns></returns>
    public virtual bool GetAddComment()
    {
      return bAddComment;
    }
    
    /// <returns></returns>
    public virtual bool DisplayItemComment()
    {
      return bDisplayItemComment;
    }
    
    /// <returns></returns>
    public virtual bool DisplayViewItems()
    {
      return bDisplayViewItems;
    }
    
    /// <returns></returns>
    public abstract bool GetAllowClearingOfApprovalRequests();
    
    /// <param name="item"></param>
    /// <returns></returns>
    public abstract bool GetAllowClearingOfItem(AItemDescriptor item);
    
    /// <summary>
    /// Call this method to obtain the index of the page being viewed for
    /// the given ApprovalItemType
    /// <br /><br />
    /// </summary>
    /// <param name="type">ApprovalItemType</param>
    /// <returns>int</returns>
    public virtual int GetCurrentPageIndex(ApprovalItemType type)
    {
      if (type == null)
      {
        return -1;
      }
      Object oi = pageIndexMap.GetElement(type.GetName());
      if (oi == null)
      {
        oi = 0;
        pageIndexMap.PutElement(type.GetName(), oi);
      }
      return XPConvert.ToInteger(oi);
    }
    
    /// <summary>
    /// Call this method to obtain the ApprovalItemType currently being viewed.
    /// <br /><br />
    /// </summary>
    /// <returns>ApprovalItemType</returns>
    public virtual ApprovalItemType GetCurrentType()
    {
      return currentType;
    }
    
    /// <summary>
    /// Call this method to determine the default ApprovalItemType displayed by the
    /// view.
    /// <br /><br />
    /// </summary>
    /// <returns>ApprovalItemType</returns>
    public abstract ApprovalItemType GetDefaultItemType();
    
    /// <returns></returns>
    public abstract bool GetDisableRadioSelection();
    public abstract bool DisplayItemTypeHeaderAboveActionbar();
    
    /// <returns></returns>
    public abstract bool GetDisplayHeadersForItemTypesWithNoItems();
    
    /// <summary>
    /// Concrete subclasses must implement this method to return a boolean value
    /// indicating whether instance information is displayed for the specified
    /// type.
    /// <br /><br />
    /// The DocumentApprovalModel class for example, returns true.  This is
    /// because each document, being an item, consists of one or more copies
    /// existing in various portal folders, or INSTANCES.  The
    /// ObjectApprovalModel, however, return false for this method.
    /// <br /><br />
    /// </summary>
    /// <param name="type">ApprovalItemType</param>
    /// <returns>boolean value</returns>
    public abstract bool GetDisplayInstances(ApprovalItemType type);
    
    /// <summary>
    /// Concrete subclasses must implement this method to return a boolean
    /// indicating whether or not a header banner should be displayed for the
    /// specified type.
    /// <br /><br />
    /// </summary>
    /// <param name="type">ApprovalItemType</param>
    /// <returns>boolean value</returns>
    public abstract bool GetDisplayTypeHeader(ApprovalItemType type);
    
    /// <summary>
    /// Concrete subclasses must implement this method to return a string that
    /// is displayed in sections containg no items awaiting approval.
    /// <br /><br />
    /// </summary>
    /// <param name="type">ApprovalItemType</param>
    /// <returns>String</returns>
    public abstract String GetEmptyTypeSectionSubTitle(ApprovalItemType type);
    
    /// <summary>
    /// Call this method to obtain the index of the last item on the
    /// current page for a specific ApprovalItemType.
    /// <br /><br />
    /// </summary>
    /// <param name="type">ApprovalItemType</param>
    /// <returns>int value</returns>
    public virtual int GetHighItemIndexForCurrentPage(ApprovalItemType type)
    {
      /* m_iHigh is set when a user wants to override the normal paging by
       * using the View Items indices.
       */
       
      if (m_iHigh > -1)
      {
        int iMax = this.GetTypeCount(type) - 1;
        return (m_iHigh > iMax) ? iMax : m_iHigh;
      } else
      /* use the normal paging indices */
      {
        int iHigh = 0;
        int iPageIndex;
        iPageIndex = this.GetCurrentPageIndex(type);
        iHigh = GetHighItemIndexForGivenPage(iPageIndex, type);
        return iHigh;
      }
    }
    
    /// <param name="iPageIndex"></param>
    /// <param name="type"></param>
    /// <returns></returns>
    public virtual int GetHighItemIndexForGivenPage(int iPageIndex, ApprovalItemType type)
    {
      int iHigh = 0;
      int iMax = this.GetTypeCount(type) - 1;
      iHigh = ((iPageIndex + 1) * this.iPageSize) - 1;
      if (iHigh > iMax)
      {
        iHigh = iMax;
      }
      return iHigh;
    }
    
    /// <returns></returns>
    public virtual String GetItemID()
    {
      return sItemID;
    }
    
    /// <summary>
    /// Call this method to obtain the index of the first item on the
    /// current page for a specific ApprovalItemType.
    /// <br /><br />
    /// </summary>
    /// <param name="type">ApprovalItemType</param>
    /// <returns>int value</returns>
    public virtual int GetLowItemIndexForCurrentPage(ApprovalItemType type)
    {
      /* JHQ: m_iLow is set by the control when a user wants to define the
       * items shown on a page using the view items indices
       */
       
      if (m_iLow > -1)
      {
        int iMax = this.GetTypeCount(type) - 1;
        return (m_iLow > iMax) ? iMax : m_iLow;
      } else
      /* figure out the low index based on page */
      {
        int iPageIndex = this.GetCurrentPageIndex(type);
        int iLow = GetLowItemIndexForGivenPage(iPageIndex, type);
        return iLow;
      }
    }
    
    /// <param name="iPageIndex"></param>
    /// <param name="type"></param>
    /// <returns></returns>
    public virtual int GetLowItemIndexForGivenPage(int iPageIndex, ApprovalItemType type)
    {
      int iLow = 0;
      int iMax = this.GetTypeCount(type) - 1;
      iLow = iPageIndex * this.iPageSize;
      if (iLow < 0)
      {
        iLow = 0;
      }
      if (iLow > iMax)
      {
        iLow = iMax;
      }
      return iLow;
    }
    
    /// <summary>
    /// Concrete subclasses must implement this method to return the name
    /// of the model.
    /// <br /><br />
    /// </summary>
    /// <returns>String</returns>
    public abstract override String GetName();
    
    /// <param name="type"></param>
    /// <returns></returns>
    public virtual int GetNumPagesAvailable(ApprovalItemType type)
    {
      int i = 0;
      if (type == null)
      {
        return 0;
      }
      int iNumItems = this.GetTypeCount(type);
      log.Debug("GetNumPagesAvailable() - type: " + type);
      log.Debug("GetNumPagesAvailable() - iNumItems: " + iNumItems);
      i = iNumItems / iPageSize;
      int r = iNumItems % iPageSize;
      if (r > 0)
      {
        i = i + 1;
      }
      log.Debug("GetNumPagesAvailable() - i: " + i);
      log.Debug("GetNumPagesAvailable() - r: " + r);
      return i;
    }
    
    /// <summary>
    /// Get the size of a page.  This is the maximum number of items awaiting
    /// approval displayed.
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetPageSize()
    {
      return iPageSize;
    }
    
    /// <summary>
    /// set the size of a page.  This is the maximum number of items awaiting
    /// approval displayed.
    /// </summary>
    /// <returns></returns>
    public virtual void SetPageHighIndex(int _index)
    {
      m_iHigh = _index;
    }
    
    /// <summary>
    /// set the low item index of a page.  This is the index of the first item 
    /// to be displayed on a page.
    /// </summary>
    /// <returns></returns>
    public virtual void SetPageLowIndex(int _index)
    {
      m_iLow = _index;
    }
    
    /// <summary>
    /// Concrete subclasses must implement this method to return the subtitle.
    /// This string is displayed in a banner at the top of the view.
    /// </summary>
    /// <returns>String</returns>
    public abstract String GetSubTitle();
    
    /// <summary>
    /// Concrete subclasses must implement this method to return an XPArrayList
    /// of those ItemTypes that are supporeted by the model.
    /// <br /><br />
    /// </summary>
    /// <returns>XPArrayList of AItemDescriptors</returns>
    public abstract XPArrayList GetSupportedItemTypes();
    
    /// <summary>
    /// Concrete subclasses must implement this method to return the title.  This
    /// string is displayed in a banner at the top of the view.
    /// </summary>
    /// <returns>String</returns>
    public abstract String GetTitle();
    
    /// <param name="type"></param>
    /// <param name="iPageNumber"></param>
    public virtual void GotoPage(ApprovalItemType type, int iPageNumber)
    {
      log.Debug("GotoPage() - type: " + type.GetName());
      log.Debug("GotoPage() - i: " + iPageNumber);
      if (!IsPageAvailable(type, iPageNumber))
      {
        return;
      }
      SetCurrentPageIndex(type, iPageNumber - 1);
      /* clear view item indices */
      m_iLow = -1;
      m_iHigh = -1;
    }
    
    /// <summary>
    /// Concrete subclasses must implement this method to return an XPArrayList
    /// of AItemDescriptor objects.
    /// <br /><br />
    /// </summary>
    /// <param name="type">ApprovalItemType</param>
    /// <returns>XPArrayList of AItemDescriptor objects.</returns>
    public abstract XPArrayList InternalGetItemDescriptors(ApprovalItemType type,
     int iLow, int iHigh);
    
    /// <summary>
    /// Concrete subclasses must implement this method to return the count
    /// of specified ApprovalItemType.  This is the total number of object of the
    /// specified type, visible to the current user, awaiting approval.
    /// <br /><br />
    /// </summary>
    /// <param name="type">ApprovalItemType</param>
    /// <returns>int</returns>
    public abstract int InternalGetTypeCount(ApprovalItemType type);
    
    /// <returns></returns>
    public abstract bool IsDateSortable();
    
    /// <returns></returns>
    public abstract bool IsDescriptionEditable();
    
    /// <returns></returns>
    public abstract bool IsNameSortable();
    
    /// <summary>
    /// Call this method to determine if there is another page of items of a
    /// given type.
    /// </summary>
    /// <param name="type">Pass the ApprovalItemType</param>
    /// <returns>boolean value</returns>
    public virtual bool IsNextPageAvailable(ApprovalItemType type)
    {
      int iHigh = this.GetHighItemIndexForCurrentPage(type);
      int iMax = this.GetTypeCount(type) - 1;
      bool b = true;
      if (iHigh >= iMax)
      {
        b = false;
      }
      return b;
    }
    
    /// <param name="type"></param>
    /// <param name="iPageNumber"></param>
    /// <returns></returns>
    public virtual bool IsPageAvailable(ApprovalItemType type, int iPageNumber)
    {
      log.Debug("IsPageAvailable() - type: " + type);
      log.Debug("IsPageAvailable() - i: " + iPageNumber);
      bool b = false;
      if (iPageNumber >= 0)
      {
        int iNumPages = this.GetNumPagesAvailable(type);
        if (iPageNumber <= iNumPages)
        {
          b = true;
        }
      }
      return b;
    }
    
    /// <summary>
    /// Call this method to determine if there is a previous page of items of a
    /// given type.
    /// </summary>
    /// <param name="type">Pass the ApprovalItemType</param>
    /// <returns>boolean value</returns>
    public virtual bool IsPreviousPageAvailable(ApprovalItemType type)
    {
      int iLow = this.GetLowItemIndexForCurrentPage(type);
      bool b = false;
      if (iLow > 0)
      {
        b = true;
      }
      return b;
    }
    
    /// <returns></returns>
    public abstract bool IsSubmittedBySortable();
    
    /// <returns></returns>
    public abstract bool IsTitleEditable();
    public virtual void ClearCache()
    {
      log.Debug("Entering ClearCache()");
      itemMap.Clear();
      countMap.Clear();
      globalItemMap.Clear();
    }
    
    /// <param name="type"></param>
    public virtual void ClearCache(ApprovalItemType type)
    {
      log.Debug("Entering ClearCache()");
      log.Debug("ClearCache() - type: " + type);
      try
      {
        itemMap.RemoveElement(type.GetName());
      }
      catch (Exception ex)
      {
      }
      //   do nothing
      try
      {
        countMap.RemoveElement(type.GetName());
      }
      catch (Exception ex)
      {
      }
    }
    //   do nothing
    
    /// <returns></returns>
    public virtual IXPEnumerator GetAllCachedItems()
    {
      XPArrayList alNew = new XPArrayList();
      IXPEnumerator eKeys = globalItemMap.GetKeys();
      AItemDescriptor item;
      Object oKey;
      while (eKeys.MoveNext())
      {
        oKey = eKeys.GetCurrent();
        item = (AItemDescriptor) globalItemMap.GetElement(oKey);
        alNew.Add(item);
      }
      return alNew.GetEnumerator();
    }
    
    /// <returns></returns>
    public abstract bool GetDisplayStatusColumn();
    
    /// <returns></returns>
    public abstract bool GetDisplaySelectAllButton();
    
    /// <summary>
    /// Call this method to obtain a specific AItemDescriptor object.
    /// <br /><br />
    /// If this ItemDescriptor is not in the internal cache, the server
    /// will not be contacted.
    /// </summary>
    /// <param name="type">ApprovalItemType</param>
    /// <param name="sID">String id of the specific AItemDescriptor wanted.</param>
    /// <returns>AItemDescriptor</returns>
    public virtual AItemDescriptor GetItemDescriptor(String sID)
    {
      log.Debug("GetItemDescriptor() - sID: " + sID);
      AItemDescriptor id = (AItemDescriptor) globalItemMap.GetElement(sID);
      return id;
    }
    
    /// <summary>
    /// Call this method to obtain an enumeration of AItemDescriptor objects.
    /// <br /><br />
    /// Included AItemDescriptor objects will be those object existing on the
    /// currently viewed page of item descriptors.
    /// <br /><br />
    /// </summary>
    /// <param name="type">ApprovalItemType</param>
    /// <returns>IXPEnumerator of AItemDescriptor objects</returns>
    public virtual IXPEnumerator GetItemDescriptors(ApprovalItemType type)
    {
      int iPage = this.GetCurrentPageIndex(type);
      return this.GetItemDescriptors(type, iPage);
    }
    
    /// <summary>
    /// Call this method to obtain the count of items awaiting approval of the
    /// specified type.
    /// <br /><br />
    /// </summary>
    /// <param name="type">ApprovalItemType</param>
    /// <returns>int</returns>
    public virtual int GetTypeCount(ApprovalItemType type)
    {
      if (type == null)
      {
        return 0;
      }
      String sTypeName = type.GetName();
      log.Debug("GetTypeCount() - sTypeName: " + sTypeName);
      Object oi = countMap.GetElement(sTypeName);
      int i;
      if (oi != null)
      {
        i = XPConvert.ToInteger(oi);
      } else
      {
        i = this.InternalGetTypeCount(type);
        oi = i;
        countMap.PutElement(type.GetName(), oi);
      }
      return i;
    }
    
    /// <summary>
    /// Call this method to move the view to the next page of items of
    /// the specified type.  If there is no next page available, calling this
    /// method will do nothing.
    /// </summary>
    /// <param name="type">ApprovalItemType</param>
    public virtual void NextPage(ApprovalItemType type)
    {
      if (this.IsNextPageAvailable(type))
      {
        int iPageIndex = this.GetCurrentPageIndex(type);
        SetCurrentPageIndex(type, iPageIndex + 1);
        /* clear view item indices */
        m_iLow = -1;
        m_iHigh = -1;
      }
    }
    
    /// <summary>
    /// Call this method to move the view to the previous page of items of
    /// the specified type.  If there is no previous page available, calling this
    /// method will do nothing.
    /// </summary>
    /// <param name="type">ApprovalItemType</param>
    public virtual void PreviousPage(ApprovalItemType type)
    {
      if (this.IsPreviousPageAvailable(type))
      {
        int iPageIndex = this.GetCurrentPageIndex(type);
        SetCurrentPageIndex(type, iPageIndex - 1);
        /* clear view item indices */
        m_iLow = -1;
        m_iHigh = -1;
      }
    }
    
    /// <param name="b"></param>
    public virtual void SetAddComment(bool b)
    {
      bAddComment = b;
    }
    
    /// <summary>
    /// Call this method to set the type currently being viewed.  Only one type
    /// of item may be viewed at a given time.  This is intended to reduce stress
    /// on the portal server.
    /// <br /><br />
    /// </summary>
    /// <param name="type">ApprovalItemType</param>
    public virtual void SetCurrentType(ApprovalItemType type)
    {
      currentType = type;
    }
    
    /// <param name="b"></param>
    public virtual void SetDoSortDate(bool b)
    {
      bSortDate = b;
    }
    
    /// <param name="b"></param>
    public virtual void SetDoSortName(bool b)
    {
      bSortName = b;
    }
    
    /// <param name="b"></param>
    public virtual void SetDoSortSubmittedBy(bool b)
    {
      bSortSubmittedBy = b;
    }
    
    /// <param name="s"></param>
    public virtual void SetItemID(String s)
    {
      sItemID = s;
    }
    //	/**
    //	 *
    //	 *
    //	 * @param alTarget
    //	 * @param al
    //	 */
    //	private void AddArrayLists(XPArrayList alTarget, XPArrayList al)
    //	{
    //		Object o = null;
    //
    //		if ((al != null) && (alTarget != null))
    //		{
    //			IXPEnumerator enumerator = al.GetEnumerator();
    //
    //			while (enum.MoveNext())
    //			{
    //				o = enumerator.GetCurrent();
    //
    //				alTarget.Add(o);
    //			}
    //		}
    //	}
    
    /// <summary>
    /// Call this method to obtain an XPArrayList of ItemDescriptor objects
    /// for the specified type.
    /// <br /><br />
    /// An arrayList of cached item descriptors will be returned if possible.
    /// <br /><br />
    /// If the passed high index is greater than the max index of the cached
    /// item descriptors, the server will be contacted and the cache will be
    /// increased.
    /// </summary>
    /// <param name="type">ApprovalItemType</param>
    /// <returns>XPArrayList of AItemDescriptor objects</returns>
    private XPArrayList GetIDArrayList(ApprovalItemType type, int iPageIndex)
    {
      log.Debug("GetIDArrayList - type: " + type.GetName());
      log.Debug("GetIDArrayList - iPageIndex: " + iPageIndex);
      XPHashtable mTemp = (XPHashtable) itemMap.GetElement(type.GetName());
      XPArrayList alTemp = null;
      if (mTemp == null)
      {
        mTemp = new XPHashtable();
        itemMap.PutElement(type.GetName(), mTemp);
      } else
      {
        alTemp = (XPArrayList) mTemp.GetElement(iPageIndex);
      }
      if (alTemp == null)
      {
        /* indices depend on whether View Items is being used
         * (m_iLow & m_iHigh) or the normal paging (pageIndex & PageSize)
         */
         
        int iLow;
        if (m_iLow > -1)
        {
          iLow = m_iLow;
        } else
        {
          iLow = iPageIndex * iPageSize;
        }
        int iHigh;
        if (m_iHigh > -1)
        {
          iHigh = m_iHigh;
        } else
        {
          iHigh = (iLow + iPageSize) - 1;
        }
        log.Debug("GetIDArrayList - About to call InternalGetItemDescriptors.");
        log.Debug("GetIDArrayList - iNewLow: " + iLow);
        log.Debug("GetIDArrayList - iNewHigh: " + iHigh);
        alTemp = this.InternalGetItemDescriptors(type, iLow, iHigh);
        IXPEnumerator eTemp = alTemp.GetEnumerator();
        AItemDescriptor iTemp;
        while (eTemp.MoveNext())
        {
          iTemp = (AItemDescriptor) eTemp.GetCurrent();
          this.globalItemMap.PutElement(iTemp.GetID(), iTemp);
        }
        mTemp.PutElement(iPageIndex, alTemp);
      }
      return alTemp;
    }
    /*
     XPArrayList alTemp = (XPArrayList) itemMap.GetElement(type.GetName());
     
     
     if (alTemp == null)
     {
     if ( log.IsDebugEnabled() )
     {
     log.Debug("GetIDArrayList - Cached ArrayList is null.");
     }
     
     int iNewLow  = 0;
     int iNewHigh = iGrabSize;
     
     if (iHigh > (iGrabSize - 1))
     {
     if ( log.IsDebugEnabled() )
     {
     log.Debug("GetIDArrayList - Requested iHigh is greater than iGrabSize.");
     }
     
     iNewHigh = iHigh + iGrabSize;
     }
     
     if ( log.IsDebugEnabled() )
     {
     log.Debug("GetIDArrayList - About to call InternalGetItemDescriptors.");
     log.Debug("GetIDArrayList - iNewLow: " + iNewLow);
     log.Debug("GetIDArrayList - iNewHigh: " + iNewHigh);
     }
     
     alTemp = this.InternalGetItemDescriptors(type, iNewLow, iNewHigh);
     
     itemMap.PutElement(type.GetName(), alTemp);
     
     }
     else
     {
     int iCurrentSize = alTemp.GetSize();
     
     if ( log.IsDebugEnabled() )
     {
     log.Debug("GetIDArrayList - Got a cached ArrayList.");
     log.Debug("GetIDArrayList - iCurrentSize: " + iCurrentSize);
     }
     
     if ((iCurrentSize - 1) < iHigh)
     {
     int iMax = this.GetTypeCount(type);
     
     if (iCurrentSize < (iMax - 1))
     {
     //   alTemp isn't large enough, and there
     //   are more items available than are in our cache.
     int iNewLow  = iCurrentSize;
     int iNewHigh = iNewLow + iGrabSize;
     
     if (iNewHigh < iHigh)
     {
     iNewHigh = iHigh + iGrabSize;
     }
     
     if ( log.IsDebugEnabled() )
     {
     log.Debug("GetIDArrayList - Cached ArrayList isnt large enough, and there's more available.");
     log.Debug("GetIDArrayList - iNewLow: " + iNewLow);
     log.Debug("GetIDArrayList - iNewHigh: " + iNewHigh);
     }
     
     XPArrayList alAdditional = this.InternalGetItemDescriptors(type, iNewLow, iNewHigh);
     
     AddArrayLists(alTemp, alAdditional);
     
     }
     
     }
     
     }
     
     return alTemp;
     
     */
     
    
    /// <summary>
    /// Call this method to obtain an XPEnumerator of AItemDescriptor objects.
    /// <br /><br />
    /// </summary>
    /// <param name="type">ApprovalItemType</param>
    /// <param name="iLow">int low index</param>
    /// <param name="iHigh">int high index</param>
    /// <returns>IXPEnumerator of AItemDescriptor objects</returns>
    private IXPEnumerator GetItemDescriptors(ApprovalItemType type, int iPageIndex)
    {
      XPArrayList alTemp;
      IXPEnumerator enumTemp = null;
      log.Debug("Entering GetItemDescriptors()");
      log.Debug("GetItemDescriptors() - type: " + type.GetName());
      alTemp = GetIDArrayList(type, iPageIndex);
      if (alTemp != null)
      {
        enumTemp = alTemp.GetEnumerator();
      }
      return enumTemp;
    }
    
    /// <summary>
    /// Call this method to set the index of the page being viewed of the
    /// given ApprovalItemType.
    /// <br /><br />
    /// If the index passed is not within the valid range, then
    /// GetLowItemIndexForCurrentPage() and GetHighItemIndexForCurrentPage()
    /// will return correct values for either the first or last valid page.
    /// <br /><br />
    /// </summary>
    /// <param name="type">ApprovalItemType</param>
    /// <param name="i">int</param>
    public virtual void SetCurrentPageIndex(ApprovalItemType type, int i)
    /* JHQ: changed this to public so that we use View
     * Items to specify which items to show */
     
    {
      if (i < 0)
      {
        throw new XPException("The page index must be greater than or equal to 0.");
        }
      pageIndexMap.PutElement(type.GetName(), i);
    }
  }
}