using System;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
namespace com.plumtree.uiinfrastructure.approval
{
  /// <author>
  /// Ben Geislinger
  /// This abstarct class represents an item awaiting approval.
  /// <br /><br />
  /// Knows concrete subclasses are DocumentItemDescriptor and
  /// ObjectItemDescriptor.
  /// </author>
  public abstract class AItemDescriptor
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.approval.AItemDescriptor");
    protected static int iPrimaryID = 0;
    protected AActivitySpace mySpace = null;
    private ApprovalItemType myType = null;
    private Redirect editRedirect = null;
    private String sComment = "";
    private String sDescription;
    private String sEditHRef = null;
    private String sImgUUID = null;
    private String sID;
    private String sName;
    private String sPreviewHRef = null;
    private String sSubmittedBy;
    private XPArrayList alInstances;
    private XPDateTime dSubmitted;
    private XPHashtable mapInstances;
    private bool bApprove = false;
    private bool bClear = false;
    private bool bHold = true;
    private bool bReject = false;
    
    /// <summary>
    /// Creates a new AItemDescriptor object.
    /// </summary>
    /// <param name="name">String name of the object awaiting approval</param>
    /// <param name="description">String description of the object awaiting approval</param>
    /// <param name="submittedBy">String</param>
    /// <param name="submittedDate">XPDateTime</param>
    protected AItemDescriptor(String name, String description, String submittedBy,
     XPDateTime submittedDate, AActivitySpace space, ApprovalItemType type)
    {
      sDescription = description;
      sName = name;
      sSubmittedBy = submittedBy;
      dSubmitted = submittedDate;
      mySpace = space;
      myType = type;
      sID = type.GetName() + "_IT_" + XPConvert.ToString(iPrimaryID);
      iPrimaryID++;
    }
    
    /// <summary>
    /// Call this method to add an AInstanceDescriptor to this AItemDescriptor
    /// </summary>
    /// <param name="instance">AInstanceDescriptor</param>
    public virtual void AddInstance(AInstanceDescriptor instance)
    {
      if (alInstances == null)
      {
        alInstances = new XPArrayList();
        mapInstances = new XPHashtable();
      }
      log.Debug("AddInstance() - My ID: " + sID);
      log.Debug("AddInstance() - Instance ID: " + instance.GetID());
      alInstances.Add(instance);
      mapInstances.PutElement(instance.GetID(), instance);
    }
    public abstract void Execute();
    public virtual void Approve()
    {
      log.Debug("ENTERING Approve()");
      bApprove = true;
      bReject = false;
      bHold = false;
      IXPEnumerator eInstances = this.GetInstances();
      AInstanceDescriptor inst;
      if (eInstances != null)
      {
        while (eInstances.MoveNext())
        {
          inst = (AInstanceDescriptor) eInstances.GetCurrent();
          inst.Approve();
        }
      }
    }
    public virtual void ClearMigrationRequest()
    {
      log.Debug("ClearMigrationRequest()");
      bApprove = false;
      bReject = false;
      bHold = false;
      bClear = true;
    }
    
    /// <returns></returns>
    public virtual bool GetClear()
    {
      return bClear;
    }
    public virtual void SetClear(bool b)
    {
      bClear = b;
    }
    
    /// <returns></returns>
    public virtual String GetComment()
    {
      return sComment;
    }
    
    /// <returns></returns>
    public virtual XPDateTime GetDateSubmitted()
    {
      return dSubmitted;
    }
    
    /// <summary>
    /// Call this method to obtain the description of this item
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetDescription()
    {
      return sDescription;
    }
    
    /// <summary>
    /// Call this method to get the HREF that will allow users to edit
    /// the object represented by this AItemDescriptor.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetEditHRef()
    {
      return sEditHRef;
    }
    
    /// <returns></returns>
    public virtual Redirect GetEditRedirect()
    {
      return editRedirect;
    }
    public virtual bool GetHold()
    {
      return bHold;
    }
    
    /// <summary>
    /// Call this method to get the ID of this item.
    /// </summary>
    /// <returns>String id</returns>
    public virtual String GetID()
    {
      return sID;
    }
    public virtual String GetImgUUID()
    {
      return sImgUUID;
    }
    public virtual void SetImgUUID(String s)
    {
      sImgUUID = s;
    }
    
    /// <summary>
    /// Call this method to obtain a reference to the specified AInstance
    /// descriptor.
    /// </summary>
    /// <param name="sID">String ID</param>
    /// <returns>AInstanceDescriptor</returns>
    public virtual AInstanceDescriptor GetInstance(String sID)
    {
      AInstanceDescriptor iTemp = null;
      if (mapInstances != null)
      {
        iTemp = (AInstanceDescriptor) mapInstances.GetElement(sID);
      }
      return iTemp;
    }
    
    /// <summary>
    /// Call this method to obtain an IXPEnumerator of all AInstanceDescriptors
    /// added to this item descriptor.
    /// </summary>
    /// <returns>IXPEnumerator of AInstanceDescriptors</returns>
    public virtual IXPEnumerator GetInstances()
    {
      IXPEnumerator enumerator = null;
      if (alInstances != null)
      {
        enumerator = alInstances.GetEnumerator();
      }
      return enumerator;
    }
    
    /// <summary>
    /// Call this method to obtain the name of this item.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetName()
    {
      return sName;
    }
    
    /// <summary>
    /// Call this method to get the HREF that will allow users to preview
    /// the object represented by this AItemDescriptor.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetPreviewHRef()
    {
      return sPreviewHRef;
    }
    
    /// <returns></returns>
    public abstract String GetStatus();
    public virtual bool GetApprove()
    {
      return bApprove;
    }
    public virtual bool GetReject()
    {
      return bReject;
    }
    
    /// <returns></returns>
    public virtual String GetSubmittedBy()
    {
      return sSubmittedBy;
    }
    
    /// <returns></returns>
    public virtual ApprovalItemType GetType()
    {
      return myType;
    }
    public virtual void Hold()
    {
      log.Debug("ENTERING Hold()");
      bApprove = false;
      bReject = false;
      bHold = true;
      IXPEnumerator eInstances = this.GetInstances();
      AInstanceDescriptor inst;
      if (eInstances != null)
      {
        while (eInstances.MoveNext())
        {
          inst = (AInstanceDescriptor) eInstances.GetCurrent();
          inst.Hold();
        }
      }
    }
    public virtual void Reject()
    {
      log.Debug("ENTERING Reject()");
      bApprove = false;
      bReject = true;
      bHold = false;
      IXPEnumerator eInstances = this.GetInstances();
      AInstanceDescriptor inst;
      if (eInstances != null)
      {
        while (eInstances.MoveNext())
        {
          inst = (AInstanceDescriptor) eInstances.GetCurrent();
          inst.Reject();
        }
      }
    }
    
    /// <param name="s"></param>
    public virtual void SetComment(String s)
    {
      sComment = s;
    }
    public virtual void SetName(String s)
    {
      sName = s;
    }
    
    /// <param name="s"></param>
    public virtual void SetDescription(String s)
    {
      sDescription = s;
    }
    
    /// <summary>
    /// Call this method to set the HREF that will allow users to edit
    /// the object represented by this AItemDescriptor.
    /// </summary>
    /// <param name="s">String</param>
    public virtual void SetEditHRef(String s)
    {
      sEditHRef = s;
    }
    
    /// <param name="r"></param>
    public virtual void SetEditRedirect(Redirect r)
    {
      editRedirect = r;
    }
    
    /// <summary>
    /// Call this method to set the HREF that will allow users to preview
    /// the object represented by this AItemDescriptor.
    /// </summary>
    /// <param name="s">String</param>
    public virtual void SetPreviewHRef(String s)
    {
      sPreviewHRef = s;
    }
  }
}