using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
namespace com.plumtree.uiinfrastructure.arraywrapper
{
  /// <summary>
  /// MES- The <c>PTGrowableSortedArrayWrapper</c> class is a subclass of
  /// PSortedArrayWrapper.  It offers some additional functionality- the ability
  /// to add and remove rows "dynamically."
  /// </summary>
  /// <version>1.0</version>
  /// <author>MichaelS</author>
  public class PTGrowableSortedArrayWrapper : PTSortedArrayWrapper,IPTGrowableSortedArrayWrapperRO
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.arraywrapper.PTGrowableSortedArrayWrapper");
    
    /// <summary>
    /// MES- PTGSAW_DEFAULT_INITIAL_SIZE is the default initial size for the
    /// array, if it is not specified by the client
    /// </summary>
    internal const int PTGSAW_DEFAULT_INITIAL_SIZE = 10;
    
    /// <summary>
    /// MES- PTGSAW_DEFAULT_PCT_INCREASE is the default percent increase for the
    /// array, if it is not specified by the client
    /// </summary>
    internal const int PTGSAW_DEFAULT_PCT_INCREASE = 200;
    
    /// <summary>
    /// MES- PTGSAW_MINIMUM_PCT_INCREASE is the minimum percent increase for the
    /// array
    /// </summary>
    protected const int PTGSAW_MINIMUM_PCT_INCREASE = 110;
    
    /// <summary>
    /// MES- m_lstUnusedRowMapping is a list of the empty rows.  We use
    /// this when we want to add a new row to the array- we may be able
    /// to sneak the data into an existing row.  Every row should either be
    /// in m_lstRowMapping or m_lstUnusedRowMapping. 
    /// </summary>
    protected XPArrayList m_lstUnusedRowMapping;
    
    /// <summary>
    /// MES- We store the mapping of column IDs, because we sometimes have
    /// to rebuild m_hashPropIDtoColumn 
    /// </summary>
    protected int[] m_arrColumnIDs;
    
    /// <summary>
    /// MES- m_iPercentIncrease tells us how much we should grow by when we want to grow 
    /// </summary>
    protected int m_iPercentIncrease;
    
    /// <summary>
    /// ROB- Clients use the AddRow method to add a row to this array.
    /// Use <c>SortArray</c> to re-sort after adding multiple entries
    /// Added as to fix bug #51045:
    /// </summary>
    /// <param name="row">
    /// is the data that the row should hold.  The number of
    /// entries in row MUST equal the number of columns in m_arrData.
    /// </param>
    public virtual void AddRowWithoutSort(Object[] row)
    {
      //MES- Call the helper function
      AddRowInternal(row);
    }
    
    /// <summary>
    /// ROB- Resort the internal array. Use after calling 
    /// <c>AddRowWithoutSort</c> multiple times. This is
    /// an alternative to calling AddRows with a 2d array.
    /// Added as to fix bug #51045: 
    /// </summary>
    public virtual void SortArray()
    {
      SortInternal();
    }
    
    /// <summary>
    /// MES- Clients use the AddRow method to add a row to this array.
    /// NOTE: This will resort the entire array when adding a row.  This
    /// can get very expensive.  Therefore, this method should not be
    /// used for adding large numbers of rows. Use AddRows() instead.
    /// </summary>
    /// <param name="row">
    /// is the data that the row should hold.  The number of
    /// entries in row MUST equal the number of columns in m_arrData.
    /// </param>
    public virtual void AddRow(Object[] row)
    {
      //MES- Call the helper function
      AddRowInternal(row);
      //MES- And resort ourself, since we probably screwed up our sort
      SortInternal();
    }
    
    /// <summary>
    /// MES- AddRows is the bulk equivalent of AddRow- it lets clients add
    /// multiple rows in a bulk operation.
    /// NOTE: This method only resorts the array once when adding bulk row.
    /// Therefore, this method should be used for adding large numbers of
    /// rows instead of calling AddRow() multiple times.  AddRow will resort
    /// each time, which can get expensive.
    /// </summary>
    /// <param name="rows">
    /// is an array of row arrays.  The first index is the rownum,
    /// the second index is the row data.  This is a little confusing, but it
    /// makes sense- AddRows takes an array of the items passed to AddRow.
    /// </param>
    public virtual void AddRows(Object[][] rows)
    {
      //MES- Make sure we have room
      int iNumNewRows = rows.Length;
      GrowInternalArrayIfNeeded(iNumNewRows);
      //MES- Call the helper function a bunch of times
      for (int iRowIndex = 0;iRowIndex < iNumNewRows;++iRowIndex)
      {
        AddRowInternal(rows[iRowIndex]);
      }
      //MES- And resort ourself, since we probably screwed up our sort
      SortInternal();
    }
    
    /// <summary>
    /// JF- Clears the entire array of Data.
    /// </summary>
    public virtual void ClearArray()
    {
      // JF- We are moving all the values from the rowMapping list 
      // to the unusedRowMapping list.
      m_lstUnusedRowMapping.AddRange(m_lstRowMapping);
      m_lstRowMapping.Clear();
    }
    
    /// <summary>
    /// MES- Initialize is equivalent to the PTSortedArrayWrapper call of the same name
    /// </summary>
    /// <param name="arrColumnIDs">
    /// contains the Property IDs for each column in
    /// arrData. The size of arrColumnIDs should be equal to the number
    /// of columns in arrData.
    /// </param>
    /// <param name="arrData">
    /// contains the actual data to be stored and manipulated.
    /// The index convention matches the Plumtree server- the first
    /// index is column, and the second is the row, like
    /// arrData[iColIndex][iRowIndex]
    /// </param>
    /// <param name="arrSort">
    /// is a 2D int array of PropertyIDs to sort by.  It should
    /// contain a column for each sort.  The first row holds the
    /// PropertyID, and the second row holds the sort direction,
    /// PTSAW_SORT_ASCENDING or PTSAW_SORT_DESCENDING
    /// </param>
    /// <param name="bRequiresSort">
    /// indicates if arrData is already sorted as indicated
    /// by arrSort.  If the client knows that the data is already
    /// sorted, we can skip the sort, and we can also optimize access.
    /// </param>
    public override void Initialize(int[] arrColumnIDs, Object[][] arrData,
     int[][] arrSort, bool bRequiresSort)
    {
      int iInitialSize;
      if (null != arrData)
      {
        iInitialSize = CalculateNextSize(arrData[0].Length);
      } else
      {
        iInitialSize = CalculateNextSize(0);
      }
      Initialize(arrColumnIDs, arrData, arrSort, bRequiresSort, iInitialSize,
       PTGSAW_DEFAULT_PCT_INCREASE);
    }
    
    /// <summary>
    /// MES- Initialize is equivalent to the PTSortedArrayWrapper call of the same name
    /// </summary>
    /// <param name="arrColumnIDs">
    /// contains the Property IDs for each column in
    /// arrData. The size of arrColumnIDs should be equal to the number
    /// of columns in arrData.
    /// </param>
    /// <param name="arrData">
    /// contains the actual data to be stored and manipulated.
    /// The index convention matches the Plumtree server- the first
    /// index is column, and the second is the row, like
    /// arrData[iColIndex][iRowIndex]
    /// </param>
    /// <param name="arrSort">
    /// is a 2D int array of PropertyIDs to sort by.  It should
    /// contain a column for each sort.  The first row holds the
    /// PropertyID, and the second row holds the sort direction,
    /// PTSAW_SORT_ASCENDING or PTSAW_SORT_DESCENDING
    /// </param>
    /// <param name="bRequiresSort">
    /// indicates if arrData is already sorted as indicated
    /// by arrSort.  If the client knows that the data is already
    /// sorted, we can skip the sort, and we can also optimize access.
    /// </param>
    /// <param name="iInitialSize">
    /// indicates the initial size of the array, if the client
    /// wishes to preallocate some rows.
    /// </param>
    public virtual void Initialize(int[] arrColumnIDs, Object[][] arrData,
     int[][] arrSort, bool bRequiresSort, int iInitialSize)
    {
      Initialize(arrColumnIDs, arrData, arrSort, bRequiresSort, iInitialSize,
       PTGSAW_DEFAULT_PCT_INCREASE);
    }
    
    /// <summary>
    /// MES- Initialize is equivalent to the PTSortedArrayWrapper call of the same name
    /// </summary>
    /// <param name="arrColumnIDs">
    /// contains the Property IDs for each column in
    /// arrData. The size of arrColumnIDs should be equal to the number
    /// of columns in arrData.
    /// </param>
    /// <param name="arrData">
    /// contains the actual data to be stored and manipulated.
    /// The index convention matches the Plumtree server- the first
    /// index is column, and the second is the row, like
    /// arrData[iColIndex][iRowIndex]
    /// </param>
    /// <param name="arrSort">
    /// is a 2D int array of PropertyIDs to sort by.  It should
    /// contain a column for each sort.  The first row holds the
    /// PropertyID, and the second row holds the sort direction,
    /// PTSAW_SORT_ASCENDING or PTSAW_SORT_DESCENDING
    /// </param>
    /// <param name="bRequiresSort">
    /// indicates if arrData is already sorted as indicated
    /// by arrSort.  If the client knows that the data is already
    /// sorted, we can skip the sort, and we can also optimize access.
    /// </param>
    /// <param name="iInitialSize">
    /// indicates the initial size of the array, if the client
    /// wishes to preallocate some rows.
    /// </param>
    /// <param name="iPercentIncrease">
    /// allows the client to indicate the amount by which
    /// we should grow the array when we add rows.  It's expressed as a percent,
    /// e.g. a client would pass in 300 to have the array triple in size when
    /// expanding.  Values less than 110 are replaced with 110.
    /// </param>
    public virtual void Initialize(int[] arrColumnIDs, Object[][] arrData,
     int[][] arrSort, bool bRequiresSort, int iInitialSize, int iPercentIncrease)
    {
      if (iPercentIncrease >= PTGSAW_MINIMUM_PCT_INCREASE)
      {
        m_iPercentIncrease = iPercentIncrease;
      } else
      {
        m_iPercentIncrease = PTGSAW_MINIMUM_PCT_INCREASE;
      }
      //MES- We support null arrays by converting them to empty arrays
      if (null == arrData)
      {
        arrData = new Object[arrColumnIDs.Length][];
        for (int j2c_index = 0;j2c_index < arrColumnIDs.Length;j2c_index++)
        {
          arrData[j2c_index] = new Object[0];
        }
      }
      //MES- Do we have to make some extra space?
      int iOriginalNumRows = arrData[0].Length;
      if (iOriginalNumRows < iInitialSize)
      {
        //MES- We need to grow the array
        arrData = ResizeArray(arrData, iInitialSize, arrColumnIDs.Length);
        }
      //MES- Initialize ourselves in the "normal" manner
      base.Initialize(arrColumnIDs, arrData, arrSort, bRequiresSort);
      //MES- We have to store the column IDs, so that we can remap them
      //	when we grow the array
      m_arrColumnIDs = arrColumnIDs;
      //MES- Make sure we don't record any "unused" rows
      m_lstUnusedRowMapping = new XPArrayList();
      //        m_lstUnusedRowMapping.ensureCapacity(iInitialSize - iOriginalNumRows);
      //MES- Fix up m_lstRowMapping.  It shouldn't include any of the rows we
      //	added with GrowArray, but m_lstUnusedRowMapping should.
      for (int iBadMapping = iOriginalNumRows;iBadMapping < iInitialSize;
      ++iBadMapping)
      {
        m_lstRowMapping.Remove(iBadMapping);
        m_lstUnusedRowMapping.Add(iBadMapping);
      }
    }
    
    /// <summary>
    /// MES- RemoveRow lets the client indicate the row number of a row to remove.
    /// </summary>
    /// <param name="iRowID">
    /// is the row ID of the row to remove.  Note that this is the
    /// row ID in the SORTED array, we must dereference it to get the "real"
    /// row number.
    /// </param>
    public virtual void RemoveRow(int iRowID)
    {
      //MES- Removing is simple, we just move the row from m_lstRowMapping
      //	to m_lstUnusedRowMapping.  That way, we won't return it in calls.
      //	We don't have to resort, since removing items doesn't screw up
      //	the sort!
      //MES- Dereference the row number
      Object intInternalRowNum = m_lstRowMapping.GetElement(iRowID);
      m_lstRowMapping.RemoveAt(iRowID);
      m_lstUnusedRowMapping.Add(intInternalRowNum);
      //MES- After removing rows, we're no longer "optimized"
      m_bSortOptimized = false;
    }
    
    /// <summary>
    /// MES- RemoveRows is the bulk anologue of RemoveRow
    /// </summary>
    /// <param name="iRowIDs">is an int array of the row IDs to remove</param>
    public virtual void RemoveRows(int[] iRowIDs)
    {
      //MES- This is kinda tricky, since every time we remove an
      //	item, we make the collection smaller.  Removing 0, and then 1
      //	would actually remove the first and third items, since the second
      //	would be promoted to first when the first was removed.
      //MES- To do it, we make an ArrayList that contains all the items
      //	we want to move from one list to the other, then we perform
      //	"bulk" operations on the lists
      XPArrayList lstRemovedItems = new XPArrayList();
      for (int iRowIndex = 0;iRowIndex < iRowIDs.Length;++iRowIndex)
      {
        lstRemovedItems.Add(m_lstRowMapping.GetElement(iRowIDs[iRowIndex]));
        }
      m_lstRowMapping.RemoveAll(lstRemovedItems);
      m_lstUnusedRowMapping.AddRange(lstRemovedItems);
      //MES- After removing rows, we're no longer "optimized"
      m_bSortOptimized = false;
    }
    
    /// <summary>
    /// MES- AddRowInternal is an internal helper function that does the
    /// heavy lifting when adding a row.
    /// </summary>
    /// <param name="row">
    /// is the row to add, in the same format as is passed to
    /// AddRow.
    /// </param>
    protected virtual void AddRowInternal(Object[] row)
    {
      if (row == null)
      {
        // Ignore empty rows.
        return;
      }
      //MES- Does it have the proper number of columns?
      if (row.Length != m_arrData.Length)
      {
        throw new XPException("Error in PTGrowableSortedArrayWrapper::AddRowInternal, new row does not have the same number of columns as existing data (new row has " + XPConvert.ToString(row.Length) + " columns, should be " + XPConvert.ToString(m_arrData.Length) + ".");
        }
      //MES- Make sure we have room for more data
      GrowInternalArrayIfNeeded(1);
      if (m_lstUnusedRowMapping.IsEmpty())
      {
        throw new XPException("Error in PTGrowableSortedArrayWrapper::AddRowInternal, unable to resize array to make room.");
        }
      //MES- Find a blank row
      int iRowIndex = XPConvert.ToInteger(m_lstUnusedRowMapping.GetElement(0));
      //MES- Put the data into the row
      for (int iColIndex = 0;iColIndex < row.Length;++iColIndex)
      {
        m_arrData[iColIndex][iRowIndex] = row[iColIndex];
      }
      //MES- Mark the row as used
      m_lstUnusedRowMapping.RemoveAt(0);
      m_lstRowMapping.Add(iRowIndex);
    }
    
    /// <summary>
    /// MES- CalculateNexSize is an internal helper function that
    /// figures out, based on the current size, what the
    /// next size would be (taking into account
    /// m_iPercentIncrease, etc.)
    /// </summary>
    /// <param name="iNominalSize">
    /// is the current nominal size- the function
    /// calculates the "next" size bigger than this size.
    /// </param>
    /// <returns>the next size larger than iNominalSize</returns>
    protected virtual int CalculateNextSize(int iNominalSize)
    {
      //MES- Multiply by m_iPercentIncrease
      int iNextSize = iNominalSize * (m_iPercentIncrease / 100);
      //MES- Is it still too small?
      if (iNextSize < PTGSAW_DEFAULT_INITIAL_SIZE)
      {
        iNextSize = PTGSAW_DEFAULT_INITIAL_SIZE;
      }
      //MES- Is it the same size as our input?  If so,
      //	we need to grow!
      if (iNextSize <= iNominalSize)
      {
        iNextSize = iNominalSize + 1;
      }
      return iNextSize;
    }
    
    /// <summary>
    /// MES- GrowInternalArrayIfNeeded is an internal helper function
    /// that checks if m_arrData is large enough to hold more rows,
    /// and makes it larger if needed.
    /// </summary>
    /// <param name="iRowsToAdd">is the number of free rows that the client requires.</param>
    /// <returns>
    /// a boolean indicating if we added rows or not.  If m_arrData
    /// already has room for the requested rows, the return value is false.
    /// </returns>
    protected virtual bool GrowInternalArrayIfNeeded(int iRowsToAdd)
    {
      //MES- Do we already have enough room?
      if (m_lstUnusedRowMapping.GetSize() >= iRowsToAdd)
      {
        //MES- We already have enough room, nothing to do
        return false;
      }
      //MES- Need to make the array bigger.  How big should it be?
      int iNewRowcount = -1;
      int iPasses = 0;
      int iOriginalNumRows = m_arrData[0].Length;
      while (iNewRowcount < (iOriginalNumRows + iRowsToAdd))
      {
        iNewRowcount = CalculateNextSize(iNewRowcount);
        ++iPasses;
        if (iPasses > 100000)
        {
          //MES- Something's wrong, we're in an infinite loop!
          throw new XPException("Error in PTGrowableSortedArrayWrapper::GrowInternalArrayIfNeeded, infinite loop detected.");
          }
      }
      //MES- Grow the array	
      m_arrData = ResizeArray(m_arrData, iNewRowcount, m_arrColumnIDs.Length);
      //MES- And add the rows to m_lstUnusedRowMapping
      for (int iNewRowIndex = iOriginalNumRows;iNewRowIndex < iNewRowcount;
      ++iNewRowIndex)
      {
        m_lstUnusedRowMapping.Add(iNewRowIndex);
      }
      //MES- Remap the column IDs, since we've changed the data
      MapColumnIDs(m_arrColumnIDs);
      return true;
    }
    
    /// <summary>
    /// MES- ResizeArray is an internal helper function that changes the number of
    /// rows in an array (by creating a new array and copying the data over.)
    /// </summary>
    /// <param name="arrData">is the array we're growing or shrinking</param>
    /// <param name="iNewRowcount">is the number of rows desired</param>
    /// <returns>is the resized array- the array is NOT resized in place.</returns>
    protected virtual Object[][] ResizeArray(Object[][] arrData, int iNewRowcount,
     int iColcount)
    {
      //MES_ NOTE: It'd be nice to use System.arrayCopy, but that's not 
      //	"jumpable" to C#.
      //MES- The size should match iColcount
      if (arrData.Length != iColcount)
      {
        throw new XPException("Error in PTGrowableSortedArrayWrapper::ResizeArray, iColCount doesn\'t match arrData.length (" + XPConvert.ToString(iColcount) + " vs. " + XPConvert.ToString(arrData.Length) + ".)");
        }
      //MES- Make the new array
      Object[][] arrReturn = new Object[iColcount][];
      for (int j2c_index = 0;j2c_index < iColcount;j2c_index++)
      {
        arrReturn[j2c_index] = new Object[iNewRowcount];
      }
      int iSourceRows = arrData[0].Length;
      //MES- Run through the source array, copying data over.
      for (int iColIndex = 0;iColIndex < iColcount;++iColIndex)
      {
        for (int iRowIndex = 0;(iRowIndex < iSourceRows) && (iRowIndex < iNewRowcount);
        ++iRowIndex)
        {
          arrReturn[iColIndex][iRowIndex] = arrData[iColIndex][iRowIndex];
          }
      }
      return arrReturn;
    }
  }
}