using System;
using com.plumtree.openfoundation.util;
namespace com.plumtree.uiinfrastructure.arraywrapper
{
  /// <summary>
  /// MES- The <c>PTSortedArrayWrapper</c> class wraps a 2D Object array. It
  /// allows a client to re-sort the rows in the array by multiple columns in a
  /// performant way.  Additionally, it allows addressing of columns by ID,
  /// rather than index (for example, if col 0 is NAME and col 1 is DESC, a
  /// client can ask for the NAME for row 2, rather than asking for col 0.)
  /// </summary>
  /// <version>1.0</version>
  /// <author>MichaelS</author>
  public class PTSortedArrayWrapper
  {
    /* MES- This class is somewhat confusing, so some notes about
     * the general implementation are in order.
     * PTSortedArrayWrapper is a wrapper arounds an Object[][] array.
     * The difference between a simple Object[][] and an PTSortedArrayWrapper
     * is that PTSortedArrayWrapper creates a level of indirection between
     * the client and the cells in the array.  This indirection is both
     * for rows and for columns.
     *
     * The level of indirection for rows allows us to sort the results
     * quickly and conveniently without moving the data in the array around.
     * When a client asks for a cell, it specifies row and column.  The client
     * indicates the row number in the "virtual" set of sorted rows, even
     * though our internal data structure is not necessarily sorted the way
     * that the client expects.  We "dereference" the row by translating the
     * row number given by the client into the row number we use internally.
     * The mapping of sorted rows to internal rows is stored in m_lstRowMapping,
     * and when we do a sort, all we're really doing is reorganizing
     * m_lstRowMapping.
     *
     * Similarly, the indirection for columns allows us to present columns
     * as "properties", in much the same way that IPTQueryResults in the
     * server does.  A client does not request the data for column 3.
     * Instead, it requests the data for, say, PT_PROPIDS.PT_PROPID_NAME,
     * and this class figures out which column is holding the Name.
     * When the client initializes this class, it indicates what the
     * IDs for each column are, and this class then respects those IDs in
     * calls to GetItem.
     */
     
    
    /// <summary>
    /// MES- PTSAW_SORT_ASCENDING indicates that a column is sorted ascending 
    /// </summary>
    public const int PTSAW_SORT_ASCENDING = 1;
    
    /// <summary>
    /// MES- PTSAW_SORT_DESCENDING indicates that a column is sorted descending
    /// </summary>
    public const int PTSAW_SORT_DESCENDING = 0;
    
    /// <summary>
    /// MES- Sort direction is indicated via a 2D array.  PTSAW_SORT_ROW_PROPID
    /// identifies the row in the array that stores the Property ID to sort by.
    /// </summary>
    public const int PTSAW_SORT_ROW_PROPID = 0;
    
    /// <summary>
    /// MES- Sort direction is indicated via a 2D array.
    /// PTSAW_SORT_ROW_DIRECTION identifies the row in the array that stores
    /// the sort direction, which should store PTSAW_SORT_ASCENDING or
    /// PTSAW_SORT_DESCENDING.
    /// </summary>
    public const int PTSAW_SORT_ROW_DIRECTION = 1;
    
    /// <summary>
    /// JF- This is used by the SetItems method. The new items info 
    /// (row index, prop ID and new cell value) is a 2D array.
    /// PTSAW_NEWITEMSINFO_INDEX identifies the column in the array 
    /// that stores the row index of the cell to modify.
    /// </summary>
    public const int PTSAW_NEWITEMSINFO_INDEX = 0;
    
    /// <summary>
    /// JF- This is used by the SetItems method. The new items info 
    /// (row index, prop ID and new cell value) is a 2D array.
    /// PTSAW_NEWITEMSINFO_COLUMNID identifies the column in the array 
    /// that stores the column of the cell to modify.
    /// </summary>
    public const int PTSAW_NEWITEMSINFO_PROPERTYID = 1;
    
    /// <summary>
    /// JF- This is used by the SetItems method. The new items info 
    /// (row index, prop ID and new cell value) is a 2D array.
    /// PTSAW_NEWITEMSINFO_NEWVALUE identifies the column in the array 
    /// that stores the new cell value.
    /// </summary>
    public const int PTSAW_NEWITEMSINFO_NEWVALUE = 2;
    
    /// <summary>
    /// MES- m_lstRowMapping holds the row numbers.  When we do a sort, we
    /// rearrange the items in m_lstRowMapping, but we do NOT rearrange the
    /// data itself. This layer of indirection lets us resort easily and
    /// relatively cheaply.
    /// </summary>
    protected XPArrayList m_lstRowMapping;
    
    /// <summary>
    /// MES- m_hashPropIDtoColumn stores a map of Property ID to the column that
    /// contains data for that property.  Note that the map points to the
    /// actual column object.
    /// </summary>
    protected XPHashtable m_hashPropIDtoColumn;
    
    /// <summary>
    /// MES- m_arrData stores the actual data 
    /// </summary>
    protected Object[][] m_arrData;
    
    /// <summary>
    /// MES- m_arrSort stores the current sort we're displaying 
    /// </summary>
    protected int[][] m_arrSort;
    
    /// <summary>
    /// MES- When the data is correctly sorted according to m_arrSort, we can
    /// skip the step of dereferencing rows through m_lstRowMapping.
    /// m_bSortOptimized holds a boolean to indicate if we can skip the
    /// dereferencing.
    /// </summary>
    protected bool m_bSortOptimized;
    
    /// <summary>
    /// MES- ContainsPropID is used by clients to see if a given Property is
    /// represented by a column in the data.
    /// </summary>
    /// <param name="iPropID">is the ID of the Property</param>
    /// <returns>a boolean indicating if the Property is found</returns>
    public virtual bool ContainsPropID(int iPropID)
    {
      //MES- TODO: Optimize this
      return m_hashPropIDtoColumn.ContainsKey(iPropID);
    }
    
    /// <summary>
    /// MES- GetCount returns the number of rows in our data set
    /// </summary>
    /// <returns>the number of rows</returns>
    public virtual int GetCount()
    {
      return m_lstRowMapping.GetSize();
    }
    
    /// <summary>
    /// MES- GetItem is used by clients to retrieve data from the array
    /// </summary>
    /// <param name="iRow">
    /// indicates the row that's desired.  That's the SORTED row,
    /// rather than the RAW row.  GetItem will dereference the row
    /// number to  figure out the real, raw row.
    /// </param>
    /// <param name="iPropID">indicates the Property that the client wants</param>
    /// <returns>the item found with row iRow and Property iPropID, as an Object</returns>
    public virtual Object GetItem(int iRow, int iPropID)
    {
      //MES- If we don't contain any data, we can't return anything
      if (null == m_arrData)
      {
        return null;
      }
      //MES- Note, we don't check arguments rigorously.  The underlying
      //	functions that we call will do so, and if we duplicated that
      //	effort, we'd be adding overhead.  This function should be very
      //	fast.
      //MES- First, map the row to the "real" row number, if needed
      if (!m_bSortOptimized)
      {
        //MES- TODO: Optimize this!
        m_lstRowMapping.GetElement(iRow);
        iRow = XPConvert.ToInteger(m_lstRowMapping.GetElement(iRow));
      }
      //MES- Map the PropID to column index
      //MES- TODO: Optimize this!
      Object[] arrCol = (Object[]) m_hashPropIDtoColumn.GetElement(iPropID);
      try
      {
        return arrCol[iRow];
      }
      catch (Exception exception)
      {
        throw new XPException("Invalid row and or propID", exception);
      }
    }
    
    /// <summary>
    /// MES- GetSort returns the current sort array
    /// </summary>
    /// <returns>the sort array, as passed into Initialize or Sort</returns>
    public virtual int[][] GetSort()
    {
      //MES- We don't want to return a reference to our internal 
      //array, because we don't want clients to be able to change 
      //our sort.  Therefore, we copy m_arrSort, and return the 
      //copy.
      int[][] arrResult = new int[m_arrSort.Length][];
      for (int j2c_index = 0;j2c_index < m_arrSort.Length;j2c_index++)
      {
        arrResult[j2c_index] = new int[2];
      }
      int iNumSorts = m_arrSort.Length;
      for (int iSortIndex = 0;iSortIndex < iNumSorts;++iSortIndex)
      {
        arrResult[iSortIndex][PTSAW_SORT_ROW_PROPID] = m_arrSort[iSortIndex][PTSAW_SORT_ROW_PROPID];
        arrResult[iSortIndex][PTSAW_SORT_ROW_DIRECTION] = m_arrSort[iSortIndex][PTSAW_SORT_ROW_DIRECTION];
        }
      return arrResult;
    }
    
    /// <summary>
    /// MES- Initialize sets up this object with the data it needs to return
    /// items, as well as to perform sorts, etc.
    /// </summary>
    /// <param name="arrColumnIDs">
    /// contains the Property IDs for each column in
    /// arrData. The size of arrColumnIDs should be equal to the number
    /// of columns in arrData.
    /// </param>
    /// <param name="arrData">
    /// contains the actual data to be stored and manipulated.
    /// The index convention matches the Plumtree server- the first
    /// index is column, and the second is the row, like
    /// arrData[iColIndex][iRowIndex]
    /// </param>
    /// <param name="arrSort">
    /// is a 2D int array of PropertyIDs to sort by.  It should
    /// contain a column for each sort.  The first row holds the
    /// PropertyID, and the second row holds the sort direction,
    /// PTSAW_SORT_ASCENDING or PTSAW_SORT_DESCENDING
    /// </param>
    /// <param name="bRequiresSort">
    /// indicates if arrData is already sorted as indicated
    /// by arrSort.  If the client knows that the data is already
    /// sorted, we can skip the sort, and we can also optimize access.
    /// </param>
    /// <throws>XPException when arguments are not valid.</throws>
    public virtual void Initialize(int[] arrColumnIDs, Object[][] arrData,
     int[][] arrSort, bool bRequiresSort)
    {
      //MES- Format is arrData[ColID][RowID] to match Plumtree server
      //MES- Validate arguments
      //MES- Number of columns must match data
      int iNumCols = arrColumnIDs.Length;
      //MES- We support null arrays by converting them to empty arrays
      if (null == arrData)
      {
        arrData = new Object[iNumCols][];
        for (int j2c_index = 0;j2c_index < iNumCols;j2c_index++)
        {
          arrData[j2c_index] = new Object[0];
        }
      }
      if (iNumCols != arrData.Length)
      {
        throw new XPException("Error in PTSortedArrayWrapper::Initialize, arrColumnIDs size does not match arrData size.");
        }
      //MES- There must be at least one column
      if (1 > iNumCols)
      {
        throw new XPException("Error in PTSortedArrayWrapper::Initialize, arrColumnIDs is zero length.");
        }
      //MES- The sort array must have two rows (ID to sort on, and asc/desc)
      if (2 != arrSort[0].Length)
      {
        throw new XPException("Error in PTSortedArrayWrapper::Initialize, arrSort doesn\'t have 2 rows.");
        }
      //MES- The sort must be on at least one column (i.e. the sort array must have at least one row)
      if (1 > arrSort.Length)
      {
        throw new XPException("Error in PTSortedArrayWrapper::Initialize, arrSort is zero length.");
        }
      //MES- Note, from the COM world, the sort direction (ascending or descending)
      //	is a boolean, where non-zero means ascending and zero means descending.
      //	We'll have to be careful to treat any non-zero value as descending, which
      //	also implies that we shouldn't validate the values.
      m_arrData = arrData;
      //MES- We COPY the input sort array, so that the client can't change their 
      //	reference to the sort array and screw up our member variable.
      //	NOTE:  This could be a problem for the data as well, but it's potentially expensive
      //	to copy the data, so we won't...
      m_arrSort = new int[arrSort.Length][];
      for (int j2c_index = 0;j2c_index < arrSort.Length;j2c_index++)
      {
        m_arrSort[j2c_index] = new int[2];
      }
      int iNumSorts = arrSort.Length;
      for (int iSortIndex = 0;iSortIndex < iNumSorts;++iSortIndex)
      {
        m_arrSort[iSortIndex][PTSAW_SORT_ROW_PROPID] = arrSort[iSortIndex][PTSAW_SORT_ROW_PROPID];
        m_arrSort[iSortIndex][PTSAW_SORT_ROW_DIRECTION] = arrSort[iSortIndex][PTSAW_SORT_ROW_DIRECTION];
        }
      int iNumRows = m_arrData[0].Length;
      //MES- Initialize the map of Property ID to column index
      MapColumnIDs(arrColumnIDs);
      //MES- Set up the default row mapping    
      m_lstRowMapping = new XPArrayList();
      //        m_lstRowMapping.ensureCapacity(iNumRows);
      for (int iRowMappingIndex = 0;iRowMappingIndex < iNumRows;++iRowMappingIndex)
      {
        m_lstRowMapping.Add(iRowMappingIndex);
      }
      //        m_lstRowMapping.trimToSize();
      if (bRequiresSort)
      {
        SortInternal();
        m_bSortOptimized = false;
      } else
      {
        //MES- The data is sorted correctly, we can optimize this case
        m_bSortOptimized = true;
      }
    }
    
    /// <summary>
    /// JF- SetItem is used by clients modify one cell in the array.
    /// </summary>
    /// <param name="iRow">
    /// indicates the row of the cell to modify.  That's
    /// the SORTED row, rather than the RAW row. SetItem will
    /// dereference the row number to  figure out the real, raw row.
    /// </param>
    /// <param name="iPropID">indicates the Property that the client wants to modify</param>
    /// <param name="oItem">
    /// is the new Object to set in the cell defined with
    /// iRow and iPropID
    /// </param>
    public virtual void SetItem(int iRow, int iPropID, Object oItem)
    {
      // JF- Create the array of new items info and just use the other method.
      Object[][] arrNewItem = new Object[1][];
      for (int j2c_index = 0;j2c_index < 1;j2c_index++)
      {
        arrNewItem[j2c_index] = new Object[3];
      }
      arrNewItem[0][PTSAW_NEWITEMSINFO_INDEX] = iRow;
      arrNewItem[0][PTSAW_NEWITEMSINFO_PROPERTYID] = iPropID;
      arrNewItem[0][PTSAW_NEWITEMSINFO_NEWVALUE] = oItem;
      SetItems(arrNewItem);
    }
    
    /// <summary>
    /// JF- SetItems is used by clients modify a number of cells in the array.
    /// </summary>
    /// <param name="arrNewItemsInfo">
    /// This is a 2D array containing 
    /// the row index, prop ID and new cell value of each cell to modify.
    /// Use the PTSAW_NEWITEMSINFO_ constant to build the array.
    /// </param>
    public virtual void SetItems(Object[][] arrNewItemsInfo)
    {
      int nPropID;
      int nRowIndex;
      bool bSortNeeded = false;
      // JF- Make sure all the column IDs are valid
      for (int i = 0;i < arrNewItemsInfo.Length;i++)
      {
        nPropID = XPConvert.ToInteger(arrNewItemsInfo[i][PTSAW_NEWITEMSINFO_PROPERTYID]);
        if (!ContainsPropID(nPropID))
        {
          throw new XPException("Error in PTSortedArrayWrapper::SetItem, column ID at index " + i + " is invalid.");
          }
      }
      // JF- First go through the row indexes and map the row index to the real one.
      if (!m_bSortOptimized)
      {
        for (int i = 0;i < arrNewItemsInfo.Length;i++)
        {
          nRowIndex = XPConvert.ToInteger(arrNewItemsInfo[i][PTSAW_NEWITEMSINFO_INDEX]);
          //MES- TODO: Optimize this!
          Object obj = m_lstRowMapping.GetElement(nRowIndex);
          // JF- Check if the row we got from the mapping is null
          if (null == obj)
          {
            throw new XPException("Error in PTSortedArrayWrapper::SetItem, iRow is invalid");
            }
          arrNewItemsInfo[i][PTSAW_NEWITEMSINFO_INDEX] = obj;
        }
      }
      for (int i = 0;i < arrNewItemsInfo.Length;i++)
      {
        Object oPropID = arrNewItemsInfo[i][PTSAW_NEWITEMSINFO_PROPERTYID];
        Object[] arrCol = (Object[]) m_hashPropIDtoColumn.GetElement(oPropID);
        nRowIndex = XPConvert.ToInteger(arrNewItemsInfo[i][PTSAW_NEWITEMSINFO_INDEX]);
        // JF- Check if the row is invalid
        if (nRowIndex >= arrCol.Length)
        {
          throw new XPException("Error in PTSortedArrayWrapper::SetItem, iRow is invalid");
          }
        // JF- Set the new Object in the cell
        arrCol[nRowIndex] = arrNewItemsInfo[i][PTSAW_NEWITEMSINFO_NEWVALUE];
        // JF- Check if the propID of the cell we just changed is in the sort array
        // If it is we will have to resort the array
        nPropID = XPConvert.ToInteger(oPropID);
        if (!bSortNeeded)
        {
          // JF- Loop through the sort and compare the prop IDs
          for (int k = 0;k < m_arrSort.Length;k++)
          {
            if (nPropID == m_arrSort[k][PTSAW_SORT_ROW_PROPID])
            {
              // JF- We found the PropID in the list of sorted columns, 
              // we need to resort
              bSortNeeded = true;
              break;
            }
          }
        }
      }
      if (bSortNeeded)
      {
        SortInternal();
      }
    }
    
    /// <summary>
    /// MES- Sort is used by clients to resort the internal data according to
    /// new criteria.
    /// </summary>
    /// <param name="arrSort">
    /// is a 2D int array of PropertyIDs to sort by.  It should
    /// contain a column for each sort.  The first row holds the
    /// PropertyID, and the second row holds the sort direction,
    /// PTSAW_SORT_ASCENDING or PTSAW_SORT_DESCENDING.
    /// </param>
    /// <throws>XPException when arguments are not valid.</throws>
    public virtual void Sort(int[][] arrSort)
    {
      //MES- Validate the argument
      //MES- The sort array must have two rows (ID to sort on, and asc/desc)
      if (2 != arrSort[0].Length)
      {
        throw new XPException("Error in PTSortedArrayWrapper::Initialize, arrSort doesn\'t have 2 columns.");
        }
      //MES- The sort must be on at least one column
      if (1 > arrSort.Length)
      {
        throw new XPException("Error in PTSortedArrayWrapper::Initialize, arrSort is zero length.");
        }
      //MES- If we're already sorted properly, skip the sort (optimization)
      //MES- Check the size
      if (arrSort.Length == m_arrSort.Length)
      {
        //MES- Iterate through the items
        bool bAllMatch = true;
        for (int iSortIndex = 0;iSortIndex < arrSort.Length;++iSortIndex)
        {
          //MES- Does this item match?
          if ((arrSort[iSortIndex][PTSAW_SORT_ROW_PROPID] != m_arrSort[iSortIndex][PTSAW_SORT_ROW_PROPID]) || (arrSort[iSortIndex][PTSAW_SORT_ROW_DIRECTION] != m_arrSort[iSortIndex][PTSAW_SORT_ROW_DIRECTION]))
          {
            //MES- No match!
            bAllMatch = false;
            break;
          }
        }
        //MES- Are we already sorted?
        if (bAllMatch)
        {
          return;
        }
      }
      //MES- Record the sort order
      int[][] arrTempSort = new int[arrSort.Length][];
      for (int j2c_index = 0;j2c_index < arrSort.Length;j2c_index++)
      {
        arrTempSort[j2c_index] = new int[2];
      }
      int iNumSorts = arrSort.Length;
      for (int iSortIndex = 0;iSortIndex < iNumSorts;++iSortIndex)
      {
        arrTempSort[iSortIndex][PTSAW_SORT_ROW_PROPID] = arrSort[iSortIndex][PTSAW_SORT_ROW_PROPID];
        arrTempSort[iSortIndex][PTSAW_SORT_ROW_DIRECTION] = arrSort[iSortIndex][PTSAW_SORT_ROW_DIRECTION];
        }
      m_arrSort = arrTempSort;
      //MES- And re-sort ourself
      SortInternal();
    }
    
    /// <summary>
    /// MES- MapColumnIDs is an internal function that sets up the
    /// m_hashPropIDtoColumn mapping based on an array of ColumnIDs
    /// </summary>
    /// <param name="arrColumnIDs">
    /// is an array of column IDs.  It should     contain
    /// one int for each column in m_arrData, and may     identify the
    /// column in any way the client desires.
    /// </param>
    protected virtual void MapColumnIDs(int[] arrColumnIDs)
    {
      //MES- TODO: Optimize this
      int iNumCols = arrColumnIDs.Length;
      //MES- Run through the columns, putting them into the hashmap	
      m_hashPropIDtoColumn = new XPHashtable(iNumCols);
      for (int iIndex = 0;iIndex < iNumCols;++iIndex)
      {
        m_hashPropIDtoColumn.PutElement(arrColumnIDs[iIndex], m_arrData[iIndex]);
        }
    }
    
    /// <summary>
    /// MES- SortInternal is an internal helper function that actually performs
    /// the sort based on m_arrSort.
    /// </summary>
    protected virtual void SortInternal()
    {
      //MES- If we don't have any data, there's nothing to do
      // m_1stRowMapping contains a map of column IDs to properties,
      // which, if it's empty, also means there's no data.  This is
      // to address bug#64089 in the XDK/OF (bug#64086 in the portal).
      if ((null == m_arrData) || (m_lstRowMapping.IsEmpty()))
      {
        return;
      }
      //MES- m_arrSort contains PropertyIDs, but we want to convert them
      //	to columns for sorting.  Do that now.
      Object[][] arrColSort = new Object[m_arrSort.Length][];
      for (int j2c_index = 0;j2c_index < m_arrSort.Length;j2c_index++)
      {
        arrColSort[j2c_index] = new Object[2];
      }
      int iNumSorts = m_arrSort.Length;
      for (int iSortIndex = 0;iSortIndex < iNumSorts;++iSortIndex)
      {
        arrColSort[iSortIndex][SortedArrayWrapperComparator.PTSAWC_SORT_ROW_DATAROW] = m_hashPropIDtoColumn.GetElement(m_arrSort[iSortIndex][PTSAW_SORT_ROW_PROPID]);
        arrColSort[iSortIndex][SortedArrayWrapperComparator.PTSAWC_SORT_ROW_DIRECTION] = m_arrSort[iSortIndex][PTSAW_SORT_ROW_DIRECTION];
        }
      //MES- Create and initialize a comparator
      SortedArrayWrapperComparator comp = new SortedArrayWrapperComparator();
      comp.Initialize(arrColSort);
      //MES- Do the sort
      XPCollections.Sort(m_lstRowMapping, comp);
      m_bSortOptimized = false;
    }
    
    /// <summary>
    /// MES- <c>SortedArrayWrapperComparator</c> is a comparator class
    /// used internally to perform multi-column sorts on the data.
    /// </summary>
    /// <version>1.0</version>
    /// <author>MichaelS</author>
    class SortedArrayWrapperComparator : IXPComparer
    {
      
      /// <summary>
      /// MES- SortedArrayWrapperComparator will compare two elements based on
      /// data in the outer class.  It will be passed a 2D array of columns
      /// to sort on.  The PTSAWC_SORT_ROW_DATAROW row in the sort array
      /// indicates the column to sort on.
      /// </summary>
      public const int PTSAWC_SORT_ROW_DATAROW = 0;
      
      /// <summary>
      /// MES- SortedArrayWrapperComparator will compare two elements based on
      /// data in the outer class.  It will be passed a 2D array of columns
      /// to sort on.  The PTSAWC_SORT_ROW_DIRECTION row in the sort array
      /// indicates the sort direction.  It should be PTSAW_SORT_ASCENDING or
      /// PTSAW_SORT_DESCENDING.
      /// </summary>
      public const int PTSAWC_SORT_ROW_DIRECTION = 1;
      
      /// <summary>
      /// MES- m_arrobjSortCols is the actual data we're going to use for
      /// sorting. It's a 2D array, where the first row contains Objects that
      /// point to columns in our parent class, and the second row indicates
      /// the sort direction.
      /// </summary>
      protected Object[][] m_arrobjSortCols;
      
      /// <summary>
      /// MES- compare implements the Comparator.  See the doc on Comparators
      /// </summary>
      /// <param name="o1">
      /// is the row index of the first item we're comparing in
      /// m_arrobjSortCols
      /// </param>
      /// <param name="o2">
      /// is the row index of the first item we're comparing in
      /// m_arrobjSortCols
      /// </param>
      /// <returns>
      /// is greater than 0 if o1 sorts after o2, is less than 0 if o1
      /// sorts before o2, and is 0 if o1 and o2 sort the same.
      /// </returns>
      public virtual int Compare(Object o1, Object o2)
      {
        //MES- o1 and o2 are integers that represent
        //	rows in m_arrobjSortCols (o1 and o2 are items in m_lstRowMapping.)
        //  We look up the proper row and use m_arrSortColumnsIndices to know
        //	which column(s) to sort on.
        int iRawRow1 = XPConvert.ToInteger(o1);
        int iRawRow2 = XPConvert.ToInteger(o2);
        //MES- Loop through the sort columns
        int iSortColumns = m_arrobjSortCols.Length;
        for (int iSortColIndex = 0;iSortColIndex < iSortColumns;++iSortColIndex)
        {
          //MES- Get the items for this sort column
          Object[] arrCol = (Object[]) m_arrobjSortCols[iSortColIndex][PTSAWC_SORT_ROW_DATAROW];
          int iSortDirection = XPConvert.ToInteger(m_arrobjSortCols[iSortColIndex][PTSAWC_SORT_ROW_DIRECTION]);
          IComparable obj1 = (IComparable) arrCol[iRawRow1];
          IComparable obj2 = (IComparable) arrCol[iRawRow2];
          //MES- Did we find items?
          if ((null != obj1) && (null != obj2))
          {
            int iCompareTo;
            //MES- Are the objects strings?	
            if (obj1 is String && obj2 is String)
            {
              //MES- The objects are strings, compare no case
              iCompareTo = XPStringUtility.CompareIgnoreCase((String) obj1,
               (String) obj2);
            } else
            {
              //MES- Use the Comparable.compareTo method to compare the items correctly.
              iCompareTo = obj1.CompareTo(obj2);
            }
            //MES- If the items aren't equal, we're done.  If the items are equal, we
            //	have to step to the next column.
            if (0 != iCompareTo)
            {
              //MES- The items aren't equal, are we sorting ascending or descending?
              return AccountForSortOrder(iCompareTo, iSortDirection);
            }
          } else
          if (null != obj1)
          {
            //MES- obj1 is not null, obj2 is null, so obj1 is greater than obj2
            return AccountForSortOrder(1, iSortDirection);
          } else
          if (null != obj2)
          {
            //MES- obj2 is not null, obj1 is null, so return -1 (obj2 is greater than obj1)
            return AccountForSortOrder(-1, iSortDirection);
          }
        }
        //MES- NOTE:  If they're both null, we should go to the next column.
        //MES- If we got here, the items are equal
        return 0;
      }
      
      /// <summary>
      /// MES- compare is the same as Compare, but is required for the Comparator
      /// Java interface that IXPComparer is derived from
      /// </summary>
      /// <param name="o1">see Compare</param>
      /// <param name="o2">see Compare</param>
      /// <returns>see Compare</returns>
      public virtual int compare(Object o1, Object o2)
      {
        return Compare(o1, o2);
      }
      
      /// <summary>
      /// MES- AccountForSortOrder is a tiny helper function that "fixes" a
      /// comparison based on the sort order.  If the sort order is
      /// ascending, the comparison is unmodified, but if the sort order is
      /// descending, the sign of the comparison is reversed.
      /// </summary>
      /// <param name="iRawCompare">is the comparison to be fixed up</param>
      /// <param name="iSortDirection">is the sort order, ascending or descending</param>
      /// <returns>
      /// iRawCompare when sort is ascending, -iRawCompare when sort
      /// is descending.
      /// </returns>
      public virtual int AccountForSortOrder(int iRawCompare, int iSortDirection)
      {
        if (PTSAW_SORT_ASCENDING == iSortDirection)
        {
          return iRawCompare;
        } else
        {
          return -iRawCompare;
        }
      }
      
      /// <summary>
      /// MES- initialize is called by clients to set the comparator up for
      /// comparing
      /// </summary>
      /// <param name="arrobjSortCols">
      /// is a 2D Object array that contains the "actual
      /// data" to be sorted.  It contains a column for each column to
      /// be sorted, and the columns are in the order we should do
      /// sort (e.g. if we're doing a sort on name, then objectid
      /// arrobjSortCols will contain two columns, one for name and
      /// one for objectid.)  It  also contains a row for the data
      /// we're sorting on and a row for the sort direction.
      /// </param>
      public virtual void Initialize(Object[][] arrobjSortCols)
      {
        m_arrobjSortCols = arrobjSortCols;
      }
    }
  }
}