using System;
namespace com.plumtree.uiinfrastructure.constants
{
  /// <summary>
  /// AccessStyles is an emumeration of the various access types allowed.
  /// </summary>
  /// <author>John Osborne</author>
  public class AccessStyles
  {
    
    /// <summary>
    /// internal constants for our conversion function.  These correspond to
    /// the values used in the portal.
    /// </summary>
    private const int nSection508 = 1;
    private const int nLowBandwidth = 2;
    private const int nStandard = 3;
    
    /// <summary>
    /// The standard portal style 
    /// </summary>
    public static readonly AccessStyles STANDARD = new AccessStyles(nStandard);
    
    /// <summary>
    /// The style that is 508 complant 
    /// </summary>
    public static readonly AccessStyles SECTION_508 = new AccessStyles(nSection508);
    
    /// <summary>
    /// A 508-like style UI that allows for regular gadgets to be displayed 
    /// </summary>
    public static readonly AccessStyles LOW_BANDWIDTH = new AccessStyles(nLowBandwidth);
    
    /// <summary>
    /// The integer ID that specifies the access style 
    /// </summary>
    private int m_nAccessID;
    
    /// <summary>
    /// Creates a new AccessStyles object.
    /// </summary>
    /// <param name="_nAccessID"></param>
    private AccessStyles(int _nAccessID)
    {
      m_nAccessID = _nAccessID;
    }
    
    /// <summary>
    /// This converts an int into an Access Style.  This is necessary because
    /// the access style comes from the portal, where it is stored as an int,
    /// rather than an enumerator.
    /// </summary>
    /// <returns>AccessStyles the enumerator value of the int.  null if there was an error</returns>
    public static AccessStyles GetAccessStyle(int nAccessStyle)
    {
      switch (nAccessStyle){
      case nStandard:
      return STANDARD;
      goto case nSection508;
      case nSection508:
      return SECTION_508;
      goto case nLowBandwidth;
      case nLowBandwidth:
      return LOW_BANDWIDTH;
      goto default;
      default:
      // Couldn't determine the correct style, return standard as default.
      return STANDARD;
      break;
      }
    }
    
    /// <summary>
    /// Gives back the integer representation of the access style.
    /// NOTE: This is the right way to work; the above method is for debug
    /// purposes.
    /// </summary>
    /// <returns>An integer version of the access style.</returns>
    public virtual int GetAccessID()
    {
      return m_nAccessID;
    }
    
    /// <summary>
    /// Gives back the string representation of the access style.
    /// </summary>
    /// <returns>
    /// A string version of the access style. For example, if the mode
    /// is SECTION_508, you get back the string: "SECTION_508". In the
    /// event of an error, null is returned.
    /// </returns>
    public virtual String ToString()
    {
      switch (m_nAccessID){
      case nStandard:
      return "STANDARD";
      goto case nSection508;
      case nSection508:
      return "SECTION_508";
      goto case nLowBandwidth;
      case nLowBandwidth:
      return "LOW_BANDWIDTH";
      goto default;
      default:
      return null;
      break;
      }
    }
    
    /// <summary>
    /// JF- Helper method to validate an Access ID.
    /// </summary>
    /// <param name="_nAccessID">ID to validate</param>
    /// <returns>boolean</returns>
    public static bool IsValidAccessID(int _nAccessID)
    {
      bool bIsValid = true;
      if ((_nAccessID < 1) || (_nAccessID > 3))
      {
        bIsValid = false;
      }
      return bIsValid;
    }
    
    /// <summary>
    /// Returns the default access ID.
    /// </summary>
    /// <returns>int</returns>
    public static int GetDefaultAccessID()
    {
      return nStandard;
    }
  }
}