/*
 * Purpose: Runs diagnostic tests
 * Author: 	D. Phipps Feb 24, 2005
 */
 
using System;
using com.plumtree.uiinfrastructure.diagnostic.testinterface;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.application;
using IStartupNotification=com.plumtree.uiinfrastructure.application.notification.IStartupNotification;
using PT_STARTUP_RESULTCODES=com.plumtree.uiinfrastructure.application.notification.PT_STARTUP_RESULTCODES;
using ReturnValue=com.plumtree.uiinfrastructure.application.notification.ReturnValue;
using com.plumtree.uiinfrastructure.application.startup.messages;
using com.plumtree.uiinfrastructure.diagnostic.reporter;
using com.plumtree.uiinfrastructure.util;
namespace com.plumtree.uiinfrastructure.diagnostic
{
  /// <summary>
  /// This class connects to uiinfrastructure's appwarmup via a notification interface and 
  /// factory that allows us to "listen in" on the warmup cycle, receiving detailed notifications 
  /// on not only the individual tasks status, but the steps within each task.  This allows
  /// for extremely detailed diagnostics of potential startup problems.
  /// Execute the following tests:
  /// 1. Check for bad database hostname+port, or database type+username+password combo.  Tricky to even get that level of detail but I do have it.
  /// 2. Check for bad PT_HOME environment variable, or bad pthome.xml file, or bad directory structure under the application home
  /// 3. Check for bad or missing settings files, with feedback on the specific file.  This happens sometimes when customers edit their XML.
  /// 4. Check for bad or missing dynamic discovery files (badly formed XML).
  /// 5. Check for a bad class in one of the dynamically discovered JAR or DLL files, or a situation where the JAR/DLL is in the lib directory but not in the war/webhome directory (dynamic discovery, as implemented, requires two copies of the same file in different places).
  /// 6. Check for bad language files.
  /// 7. Check for bad imageserver settings for HTTP connectivity, or missing resources on that imageserver (this test is inherently somewhat limited, see "diagnostic page").
  /// 8. Checking the values in the main portal settings file for validity.  These tests include:
  /// a. do the http settings have valid http url's?
  /// b. do the URL rewrite rules make sense / form valid HTTP urls under normal practice?
  /// c. do the security mode settings and URL mapping settings make sense? (Note: very difficult)
  /// d. based on security settings and URL rewrites, is the IsInvitationURLSecure setting right?
  /// e. does the webhome point to a valid directory and allow referencing of a valid file?
  /// f. is adminsitebaseurl an absolute or relative url? (must be absolute)
  /// g. are the http port and secureport valid and functional?
  /// 9. Check version information.  Is the database the right version?  If not, how should it be corrected?  Print out version information on the UI as well (it's not possible to "verify" these values, but printing them helps support).
  /// 10. Check the JDK / .NET SDK version compatibility
  /// 11. Check all the config varpacks for validity, including custom varpacks
  /// 12. Verify the automation server varpack: connect top the host and port
  /// 13. Verify the gateway and search directory settings and check for file validity
  /// 14. Verify the search locales loading sequence
  /// 15. Verify the document types loading sequence
  /// </summary>
  /// <author>DavidP</author>
  public class DiagnosticRunner : IStartupNotification
  {
    // Singleton instance
    private static DiagnosticRunner m_instance = null;
    private int m_numTests = 0;
    // Current progress tracker
    private ProgressManager m_progressMgr = null;
    // Verbosity flag, overridabl via env var
    private bool m_bVerbose = false;
    internal DiagnosticRunner()
    {
    }
    public static void create()
    {
      // delay static instance init until construction 
      // for speed and debuggability
      m_instance = new DiagnosticRunner();
    }
    
    /// <summary>
    /// delay static instance init until construction for speed and debuggability
    /// </summary>
    public virtual bool init()
    {
      bool bRetVal = false;
      try
      {
        // Set verbosity
        String str = XPEnvironment.GetEnvironmentVariable("PT_VERIFY");
        if ((str != null) && (XPStringUtility.IndexOf(XPStringUtility.ToUpperCase(str),
         "TRUE") != -1))
        {
          m_bVerbose = true;
          XPSystem.Out.WriteLine("verbose output enabled.");
        }
        // Set the notification class on appwarmup's factory
        AppWarmUp.setNotification(this);
        // success!
        Reporter.log(true, "Diagnostic system initialization OK.");
        bRetVal = true;
      }
      catch (XPException e)
      {
        Reporter.log(true, "Cannot initialize diagnostic tool. Primary classpath or referencepath error. Exiting.");
        bRetVal = false;
      }
      return bRetVal;
    }
    
    /// <summary>
    /// Singleton instance accessor
    /// </summary>
    /// <returns></returns>
    public static DiagnosticRunner instance()
    {
      return m_instance;
    }
    
    /// <summary>
    /// Executes the appwarmup directly in uiinfrastructure.
    /// </summary>
    public virtual bool executeAllTestsWithoutPause()
    {
      AppWarmUp.Init("portal");
      Reporter.log(true, "Diagnostic Test execution complete.");
      return ASManager.INSTANCE.m_bSystemStartupSuccesful;
    }
    
    /// <summary>
    /// Executes the appwarmup directly in uiinfrastructure, then pauses at the end.
    /// </summary>
    public virtual bool executeAllTests()
    {
      // Execute all tests
      bool bResult = executeAllTestsWithoutPause();
      // Pause for user input; this is to support console mode triggered 
      // from windows launcher
      XPSystem.Out.WriteLine("Press <enter> key to exit.");
      XPSystem.In.ReadLine();
      return bResult;
    }
    
    /// <summary>
    /// Called at the beginning of the startup process.
    /// Here we instantiate reporting classes and all other objects used
    /// to track startup.
    /// </summary>
    /// <param name="number">of tasks to execute overall</param>
    /// <returns>true to continue; false to halt immediately</returns>
    public virtual bool runStart(int numTasks)
    {
      m_numTests = numTasks;
      // Initialize trackers
      m_progressMgr = new ProgressManager(m_numTests, m_bVerbose);
      Reporter.log(true, "Portal Startup begins: " + (numTasks).ToString() + " total tasks.");
      return true;
    }
    public virtual bool runStart()
    {
      throw new XPException("Never call runStart without parameters");
    }
    
    /// <summary>
    /// Called when the startup process completes.  Always called, via finally.
    /// </summary>
    public virtual void runEnd(ReturnValue initRet)
    {
      Reporter.log(true, "======= Diagnostic Run Complete =======");
      // feedback on return value
      if ((initRet.getResultCode() == PT_STARTUP_RESULTCODES.FATAL) || (initRet.getResultCode() == PT_STARTUP_RESULTCODES.ERROR))
      {
        if (initRet.getResultCode() == PT_STARTUP_RESULTCODES.FATAL)
        {
          Reporter.log(m_bVerbose, "ERROR: " + initRet.getMsg());
        }
        if (initRet.getResultCode() == PT_STARTUP_RESULTCODES.ERROR)
        {
          Reporter.log(m_bVerbose, "FATAL: " + initRet.getMsg());
        }
        if (m_bVerbose && (initRet.getException() != null))
        {
          Reporter.log(m_bVerbose, "    Exception: " + initRet.getException().GetStackTrace());
          }
        // report
        Reporter.log(m_bVerbose, "Recommendation:");
        Reporter.log(m_bVerbose, RecommendationHelper.ConvertExceptionToRecommendation(initRet.getException()));
        }
      // generate reports here
      // loop through the reporter to summarize what happened
      int test;
      int num_completed = 0;
      int total_steps = 0;
      int num_steps_completed = 0;
      for (test = 0;test < m_numTests;++test)
      {
        // calc total percent
        IProgressTracker tracker = m_progressMgr.getTracker(test);
        if (tracker.getCompleted())
        {
          ++num_completed;
          num_steps_completed+=tracker.getCompletedSteps();
        }
        total_steps+=tracker.getNumSteps() + 1;
      }
      int pct = (int) ((double) num_steps_completed * (double) 100 / (double) total_steps);
      String completionPct = (pct).ToString();
      String stepsCompletedStr = (num_steps_completed).ToString();
      String totalStepsStr = (total_steps).ToString();
      Reporter.log(true, "Startup completion percentage: " + completionPct + "% (" + stepsCompletedStr + " steps out of " + totalStepsStr + " complete)");
      if (num_steps_completed >= total_steps)
      {
        Reporter.log(true, "===== Portal should now start successfully =====");
        }
      return;
    }
    
    /// <summary>
    /// Called at the beginning of the task.  Simple start notification.
    /// </summary>
    /// <param name="taskLoop"></param>
    /// <param name="taskName"></param>
    /// <param name="taskID"></param>
    /// <returns>true to continue; false to halt immediately</returns>
    public virtual bool taskBegin(int taskLoop, String taskName, int taskID)
    {
      IProgressTracker track = m_progressMgr.getTracker(taskLoop);
      track.setName(taskName);
      track.notifyevent("Task #" + (taskID + 1).ToString() + " (" + taskName + ") BEGIN (" + (track.getNumSteps()).ToString() + " steps)",
       true);
      // second loop through; increment here to signify we have initialized in prev loop
      track.increment();
      return true;
    }
    
    /// <summary>
    /// Called once task initialization is complete.  Returns the number of 
    /// steps in this task.  The ReturnValue contains any errors or exceptions
    /// that may have occurred during initialization.
    /// </summary>
    /// <param name="taskLoop"></param>
    /// <param name="taskName"></param>
    /// <param name="taskID"></param>
    /// <param name="totalNumSteps"></param>
    /// <param name="initRet"></param>
    /// <returns>true to continue; false to halt immediately</returns>
    public virtual bool taskInitComplete(int taskLoop, String taskName, int taskID,
     int totalNumSteps, ReturnValue initRet, IMessage msgArray)
    {
      IProgressTracker track = m_progressMgr.getTracker(taskLoop);
      track.setNumSteps(totalNumSteps);
      // init is step 0 to the tracker
      track.setRetval(0, initRet);
      bool bRetval = true;
      String taskIDstr = (taskLoop + 1).ToString();
      if (initRet.getResultCode() == PT_STARTUP_RESULTCODES.FATAL)
      {
        track.notifyevent("    Task #" + taskIDstr + " (" + taskName + ") failed to initialize.",
         false);
        track.notifyevent("    " + initRet.getMsg(), false);
        if (initRet.getException() != null)
        {
          track.notifyevent("    Exception: " + initRet.getException().GetStackTrace(),
           true);
        }
        // report
        Reporter.report(m_bVerbose, taskLoop, 0, taskName, Platform.getCurPlatform(),
         initRet, msgArray);
        bRetval = false;
      } else
      if (initRet.getResultCode() == PT_STARTUP_RESULTCODES.ERROR)
      {
        track.notifyevent("    Task #" + taskIDstr + " (" + taskName + ") failed to initialize. Error.",
         false);
        track.notifyevent("    " + initRet.getMsg(), false);
        if (initRet.getException() != null)
        {
          track.notifyevent("    Exception: " + initRet.getException().GetStackTrace(),
           true);
        }
        // report
        Reporter.report(m_bVerbose, taskLoop, 0, taskName, Platform.getCurPlatform(),
         initRet, msgArray);
        bRetval = false;
      } else
      if (initRet.getResultCode() == PT_STARTUP_RESULTCODES.WARN)
      {
        track.notifyevent("    Task #" + taskIDstr + " (" + taskName + ") initialized with warning.",
         false);
        track.notifyevent("    " + initRet.getMsg(), false);
        bRetval = true;
      } else
      if (initRet.getResultCode() == PT_STARTUP_RESULTCODES.OK)
      {
        track.notifyevent("    Task #" + taskIDstr + " (" + taskName + ") initialized.",
         true);
        bRetval = true;
      } else
      {
        track.notifyevent("    Task #" + taskIDstr + " (" + taskName + ") failed to initialize. Unrecognized return value.  Possible corruption.  Fatal",
         false);
        bRetval = false;
      }
      track.increment();
      return bRetval;
    }
    
    /// <summary>
    /// Called before a specific task step begins.  Simply a notifier.
    /// </summary>
    /// <param name="taskLoop"></param>
    /// <param name="taskName"></param>
    /// <param name="taskID"></param>
    /// <param name="stepLoop"></param>
    /// <returns>true to continue; false to halt immediately</returns>
    public virtual bool taskStepBegin(int taskLoop, String taskName, int taskID,
     int stepLoop)
    {
      IProgressTracker track = m_progressMgr.getTracker(taskLoop);
      String taskIDstr = (taskLoop + 1).ToString();
      return true;
    }
    
    /// <summary>
    /// Called at the end of an individual tasks' step.  Always called, with a finally.
    /// Exceptions and error return values in the ReturnValue object.
    /// </summary>
    /// <param name="taskLoop"></param>
    /// <param name="taskName"></param>
    /// <param name="taskID"></param>
    /// <param name="stepLoop"></param>
    /// <param name="initRet"></param>
    /// <returns>true to continue; false to halt immediately</returns>
    public virtual bool taskStepEnd(int taskLoop, String taskName, int taskID,
     int stepLoop, ReturnValue initRet, IMessage msgArray)
    {
      IProgressTracker track = m_progressMgr.getTracker(taskLoop);
      String taskIDstr = (taskLoop + 1).ToString();
      // init is step 0 to the tracker
      track.setRetval(taskLoop + 1, initRet);
      String taskStep = (stepLoop + 1).ToString();
      bool bRetval = true;
      if (initRet.getResultCode() == PT_STARTUP_RESULTCODES.FATAL)
      {
        track.notifyevent("    FATAL: Task #" + taskIDstr + " (" + taskName + ") step " + taskStep + " failed to complete.",
         false);
        track.notifyevent("    " + initRet.getMsg(), false);
        if (initRet.getException() != null)
        {
          track.notifyevent("    Exception: " + initRet.getException().GetStackTrace(),
           true);
        }
        // report
        Reporter.report(m_bVerbose, taskLoop, stepLoop, taskName, Platform.getCurPlatform(),
         initRet, msgArray);
        bRetval = false;
      } else
      if (initRet.getResultCode() == PT_STARTUP_RESULTCODES.ERROR)
      {
        track.notifyevent("    ERROR: Task #" + taskIDstr + " (" + taskName + ") step " + taskStep + " failed to complete.",
         false);
        track.notifyevent("    " + initRet.getMsg(), false);
        if (initRet.getException() != null)
        {
          track.notifyevent("    Exception: " + initRet.getException().GetStackTrace(),
           true);
        }
        // report
        Reporter.report(m_bVerbose, taskLoop, stepLoop, taskName, Platform.getCurPlatform(),
         initRet, msgArray);
        bRetval = false;
      } else
      if (initRet.getResultCode() == PT_STARTUP_RESULTCODES.WARN)
      {
        track.notifyevent("    WARN: Task #" + taskIDstr + " (" + taskName + ") step " + taskStep + " completed with warning.",
         false);
        track.notifyevent("    " + initRet.getMsg(), false);
        // report
        Reporter.report(m_bVerbose, taskLoop, stepLoop, taskName, Platform.getCurPlatform(),
         initRet, msgArray);
        bRetval = true;
      } else
      if (initRet.getResultCode() == PT_STARTUP_RESULTCODES.OK)
      {
        track.notifyevent("    OK: Task #" + taskIDstr + " (" + taskName + ") step " + taskStep + " completed.",
         true);
        bRetval = true;
      } else
      {
        track.notifyevent("    FATAL: Task failed to initialize. Unrecognized return value.  Possible corruption.",
         false);
        bRetval = false;
      }
      track.increment();
      return bRetval;
    }
    
    /// <summary>
    /// Called at the end of a specific task.  Always called, with a finally.
    /// The ReturnValue here contains the final result of that task: the integer
    /// result code holds what the systemn feels the final result is.  Notice that 
    /// an individual step could fail with error, but the overall task simply 
    /// show a warning. 
    /// </summary>
    /// <param name="taskLoop"></param>
    /// <param name="taskName"></param>
    /// <param name="taskID"></param>
    /// <param name="initRet"></param>
    /// <returns>true to continue; false to halt immediately</returns>
    public virtual bool taskEnd(int taskLoop, String taskName, int taskID,
     ReturnValue initRet, IMessage msgArray)
    {
      IProgressTracker track = m_progressMgr.getTracker(taskLoop);
      String taskIDstr = (taskLoop + 1).ToString();
      bool bRetval = true;
      if (initRet.getResultCode() == PT_STARTUP_RESULTCODES.FATAL)
      {
        track.notifyevent("FATAL: Task #" + taskIDstr + " (" + taskName + ") failed to complete.  Fatal.",
         false);
        track.notifyevent("    " + initRet.getMsg(), false);
        if (initRet.getException() != null)
        {
          track.notifyevent("    Exception: " + initRet.getException().GetStackTrace(),
           true);
        }
        // report
        Reporter.report(m_bVerbose, taskLoop, 0, taskName, Platform.getCurPlatform(),
         initRet, msgArray);
        bRetval = false;
      } else
      if (initRet.getResultCode() == PT_STARTUP_RESULTCODES.ERROR)
      {
        track.notifyevent("ERROR: Task #" + taskIDstr + " (" + taskName + ") failed to complete.  Error.",
         false);
        track.notifyevent("    " + initRet.getMsg(), false);
        if (initRet.getException() != null)
        {
          track.notifyevent("    Exception: " + initRet.getException().GetStackTrace(),
           true);
        }
        // report
        Reporter.report(m_bVerbose, taskLoop, 0, taskName, Platform.getCurPlatform(),
         initRet, msgArray);
        bRetval = false;
      } else
      if (initRet.getResultCode() == PT_STARTUP_RESULTCODES.WARN)
      {
        track.notifyevent("WARN: Task #" + taskIDstr + " (" + taskName + ") completed with warning.",
         false);
        track.notifyevent("    " + initRet.getMsg(), false);
        // report
        Reporter.report(m_bVerbose, taskLoop, 0, taskName, Platform.getCurPlatform(),
         initRet, msgArray);
        bRetval = true;
      } else
      if (initRet.getResultCode() == PT_STARTUP_RESULTCODES.OK)
      {
        track.notifyevent("Task #" + taskIDstr + " (" + taskName + ") END (OK)",
         false);
        bRetval = true;
      } else
      {
        track.notifyevent("Task failed to complete. Unrecognized return value.  Possible corruption.  Fatal",
         false);
        bRetval = false;
      }
      track.increment();
      return bRetval;
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupNotification#getReport()
     */
     
    public virtual String[] getReport()
    {
      return Reporter.getReport();
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupNotification#getReport()
     */
     
    public virtual EventLog[] getReportEventLogs()
    {
      return Reporter.getEventLogReport();
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupNotification#getReport()
     */
     
    public virtual void setReportEventLogs(EventLog[] eventVector, XPArrayList reportLog)
    {
      Reporter.setEventLogReport(eventVector, reportLog);
    }
  }
}