/*
 * Created on Mar 11, 2005
 */
 
using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.application.notification;
using com.plumtree.uiinfrastructure.application.startup.messages;
using com.plumtree.uiinfrastructure.util;
namespace com.plumtree.uiinfrastructure.diagnostic.reporter
{
  /// <summary>
  /// Manages error reporting. Handles line wrapping for command consoles, and 
  /// manages event logs for UI reporting.   
  /// </summary>
  /// <author>DavidP</author>
  public class Reporter
  {
    private static OpenLogger logger = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.diagnostic.reporter.Reporter");
    public const int LINEWIDTH = 60;
    public const int INDENT1 = 4;
    // straight string log of issues
    private static XPArrayList m_reportLog = new XPArrayList();
    // more intelligent structure: event reports
    private static XPArrayList m_eventLog = new XPArrayList();
    
    /// <summary>
    /// Outputs recommendations with descriptions to the console.
    /// </summary>
    /// <param name="bSystemOut:">if true, logs more verbosely</param>
    public static void report(bool bSystemOut, int taskID, int stepID, String taskName,
     Platform platform, ReturnValue retval, IMessage msgClass)
    {
      String errorMessage = "";
      String errorDescription = "";
      String exceptionDetail = "";
      if (msgClass != null)
      {
        XPArrayList list = msgClass.getMsg(stepID, platform, retval);
        if (list != null)
        {
          IXPEnumerator walker = list.GetEnumerator();
          String msg = "Recommendations follow:";
          // at this time we just grab the first one
          if (walker.MoveNext())
          {
            logNoIndent(bSystemOut, msg);
            // Simple writeln for now
            // Possibly log to file?
            StartupTaskMessage elt = (StartupTaskMessage) walker.GetCurrent();
            errorMessage = elt.getErrMessage();
            linewrap(true, errorMessage, LINEWIDTH, INDENT1);
            errorDescription = elt.getErrDescription();
            linewrap(true, errorDescription, LINEWIDTH, INDENT1);
          }
          // is there a more accurate answer?
          if (retval.getException() != null)
          {
            exceptionDetail = RecommendationHelper.ConvertExceptionToRecommendation(retval.getException());
            msg = "More Detail:";
            logNoIndent(bSystemOut, msg);
            linewrap(bSystemOut, exceptionDetail, LINEWIDTH, INDENT1);
          }
        } else
        {
          int i = stepID;
          errorMessage = "No further details for step #" + i.ToString();
          logNoIndent(bSystemOut, errorMessage);
        }
      }
      // Only log events if they are errors or worse
      if (retval.getResultCode().getID() >= PT_STARTUP_RESULTCODES.ERROR.getID())
      {
        Platform thisPlatform = Platform.getCurPlatform();
        logEvent(taskID, stepID, taskName, thisPlatform.getName(), errorMessage,
         errorDescription, exceptionDetail, retval);
      }
    }
    
    /// <summary>
    /// Simply logs the string to the output.
    /// </summary>
    /// <param name="bSystemOut:">if true, logs to stdout as well</param>
    /// <param name="line"></param>
    public static void logNoIndent(bool bSystemOut, String line)
    {
      // Simple writeln for now
      // Also adds to a global array
      linewrap(bSystemOut, line, LINEWIDTH, 0);
    }
    
    /// <summary>
    /// Simply logs the string to the output.
    /// </summary>
    /// <param name="bSystemOut:">if true, logs to stdout as well</param>
    /// <param name="line"></param>
    public static void log(bool bSystemOut, String line)
    {
      // Simple writeln for now
      // Also adds to a global array
      linewrap(bSystemOut, line, LINEWIDTH, INDENT1);
    }
    
    /// <summary>
    /// Event logger.  Tracks errors for later display.
    /// </summary>
    public static void logEvent(int taskID, int stepID, String taskName, String platformName,
     String errorMessage, String errorDescription, String exceptionRecommendation,
     ReturnValue retVal)
    {
      EventLog eventItem = new EventLog();
      eventItem.taskID = taskID;
      eventItem.taskStep = stepID;
      eventItem.taskName = taskName;
      eventItem.platformName = platformName;
      eventItem.errorMessage = errorMessage;
      eventItem.errorDescription = errorDescription;
      eventItem.exceptionRecommendation = exceptionRecommendation;
      eventItem.retVal = retVal;
      m_eventLog.Add(eventItem);
    }
    
    /// <summary>
    /// Formats strings into N-char chunks with indent; breaks on word boundary
    /// </summary>
    /// <param name="line"></param>
    public static void linewrap(bool bSystemOut, String line, int width, int indent)
    {
      int linePos = 0;
      String currentLine = "";
      String indentStr = "";
      if (line == null || line.Length < 2)
      {
        return;
      }
      // to spy
      if (bSystemOut)
      {
        logger.Info(line);
      }
      // break up into 80 char chunks for systemout
      for (int loop = 0;loop < indent;++loop)
      {
        indentStr+=" ";
      }
      if (bSystemOut)
      {
        XPSystem.Out.Write(indentStr);
      }
      currentLine+=indentStr;
      XPStringTokenizer strTok = new XPStringTokenizer(line, " ", false);
      if (strTok == null)
      {
        if (bSystemOut)
        {
          XPSystem.Out.WriteLine(line);
        }
        m_reportLog.Add(line);
        return;
      }
      // Loop through word tokens	    
      while (strTok.HasMoreTokens())
      {
        String word = strTok.GetNextToken();
        if (word != null && word.Length > 1)
        {
          linePos+=word.Length;
          if (linePos > width)
          {
            if (bSystemOut)
            {
              XPSystem.Out.WriteLine(" ");
            }
            m_reportLog.Add(currentLine);
            currentLine = "";
            linePos = word.Length + 1;
            if (bSystemOut)
            {
              XPSystem.Out.Write(indentStr);
              linePos+=indentStr.Length;
            }
            currentLine+=indentStr;
          }
          if (bSystemOut)
          {
            XPSystem.Out.Write(word + " ");
          }
          currentLine+=word;
          currentLine+=" ";
        }
      }
      if (bSystemOut)
      {
        XPSystem.Out.WriteLine(" ");
      }
      m_reportLog.Add(currentLine);
    }
    
    /// <summary>
    /// Formats strings into N-char chunks with indent; breaks on word boundary
    /// </summary>
    /// <param name="line"></param>
    public static void SystemReport(String line, int width, int indent, String frontMsg)
    {
      bool bSystemOut = true;
      String currentLine = "";
      String indentStr = "";
      int linePos = 0;
      if (line == null || line.Length < 2)
      {
        return;
      }
      // to spy
      logger.Info(line);
      // break into 80 char chunks for systemout
      for (int loop = 0;loop < indent;++loop)
      {
        indentStr+=" ";
      }
      if (bSystemOut)
      {
        if (frontMsg != null)
        {
          XPSystem.Out.Write(frontMsg);
          linePos+=frontMsg.Length;
          currentLine+=frontMsg;
        } else
        {
          XPSystem.Out.Write(indentStr);
          linePos+=indentStr.Length;
          currentLine+=indentStr;
        }
      }
      XPStringTokenizer strTok = new XPStringTokenizer(line, " ", false);
      if (strTok == null)
      {
        if (bSystemOut)
        {
          XPSystem.Out.WriteLine(line);
        }
        currentLine+=line;
        return;
      }
      // Loop through word tokens
      while (strTok.HasMoreTokens())
      {
        String word = strTok.GetNextToken();
        if (word != null && word.Length > 1)
        {
          linePos+=word.Length + 1;
          if (linePos > width)
          {
            if (bSystemOut)
            {
              XPSystem.Out.WriteLine(" ");
            }
            m_reportLog.Add(currentLine);
            currentLine = "";
            linePos = word.Length + 1;
            if (bSystemOut)
            {
              XPSystem.Out.Write(indentStr);
              linePos+=indentStr.Length;
            }
            currentLine+=indentStr;
          }
          if (bSystemOut)
          {
            XPSystem.Out.Write(word + " ");
          }
          currentLine+=word;
          currentLine+=" ";
        }
      }
      if (bSystemOut)
      {
        XPSystem.Out.WriteLine(" ");
      }
    }
    
    /// <summary>
    /// Returns current report as a string array
    /// </summary>
    /// <returns>String[] lines of report</returns>
    public static String[] getReport()
    {
      try
      {
        Object[] arr = m_reportLog.ToArray();
        int loop = 0;
        String[] resultList = new String[arr.Length + 1];
        for (loop = 0;loop < arr.Length;++loop)
        {
          resultList[loop] = (String) arr[loop];
        }
        return resultList;
      }
      catch (Exception e)
      {
        LogAndWriteError("Could not convert report log to array.");
      }
      return null;
    }
    
    /// <summary>
    /// Returns current report as an array list of event objects
    /// </summary>
    /// <returns>EventLog[] lines of report</returns>
    public static EventLog[] getEventLogReport()
    {
      try
      {
        Object[] arr = m_eventLog.ToArray();
        int loop = 0;
        EventLog[] resultList = new EventLog[arr.Length + 1];
        for (loop = 0;loop < arr.Length;++loop)
        {
          resultList[loop] = (EventLog) arr[loop];
        }
        return resultList;
      }
      catch (Exception e)
      {
        LogAndWriteError("Could not convert event log to array.");
      }
      return null;
    }
    
    /// <summary>
    /// Both write to system out and log to spy.
    /// </summary>
    /// <param name="msg"></param>
    public static void LogAndWrite(String msg)
    {
      LogNoCarriageReturns(msg);
      XPSystem.Out.WriteLine(msg);
    }
    
    /// <summary>
    /// Both write to system out and log to spy (error version).
    /// </summary>
    /// <param name="msg"></param>
    public static void LogAndWriteError(String msg)
    {
      LogNoCarriageReturnsError(msg);
      XPSystem.Out.WriteLine(msg);
    }
    
    /// <summary>
    /// Both write to system out and log to spy (error version).
    /// </summary>
    /// <param name="msg"></param>
    public static void LogAndWriteFatal(String msg)
    {
      LogNoCarriageReturnsFatal(msg);
      XPSystem.Out.WriteLine(msg);
    }
    
    /// <summary>
    /// Both write to system out and log to spy (error version).
    /// </summary>
    /// <param name="msg"></param>
    private static void LogNoCarriageReturns(String msg)
    {
      logger.Action(StripCarriageReturns(msg));
    }
    
    /// <summary>
    /// Both write to system out and log to spy (error version).
    /// </summary>
    /// <param name="msg"></param>
    private static void LogNoCarriageReturnsError(String msg)
    {
      logger.Error(StripCarriageReturns(msg));
    }
    
    /// <summary>
    /// Both write to system out and log to spy (error version).
    /// </summary>
    /// <param name="msg"></param>
    private static void LogNoCarriageReturnsFatal(String msg)
    {
      logger.Fatal(StripCarriageReturns(msg));
    }
    
    /// <summary>
    /// Both write to system out and log to spy (error version).
    /// </summary>
    /// <param name="msg"></param>
    private static String StripCarriageReturns(String msg)
    {
      String stripped = "";
      for (int loop = 0;loop < msg.Length;++loop)
      {
        char c = (char) XPStringUtility.CharAt(msg, loop);
        if (!((c == 13) || (c == 10)))
        {
          stripped+=c;
        }
      }
      return stripped;
    }
    
    /// <summary>
    /// Overrides existing event and report logs and replaces them with new ones.
    /// </summary>
    /// <param name="eventVector"></param>
    public static void setEventLogReport(EventLog[] eventVector, XPArrayList reportLog)
    {
      m_eventLog = new XPArrayList();
      try
      {
        // store the new event log
        int loop = 0;
        for (loop = 0;loop < eventVector.Length;++loop)
        {
          m_eventLog.Add((EventLog) eventVector[loop]);
        }
        // save the report log
        m_reportLog = reportLog;
      }
      catch (Exception e)
      {
        LogAndWriteError("Could not convert event log vector to array list.");
        }
    }
  }
}