using System;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.errordisplay;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.uiinfrastructure.tips;
// Framework Imports
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
using com.plumtree.xpshared.htmlconstructs;
namespace com.plumtree.uiinfrastructure.editor
{
  /// <summary>
  /// This is the Activity Space class for the Editor Framework.
  /// Editor Activity Space classes should extend this class.
  /// </summary>
  /// <author>Julie Fournier</author>
  public abstract class EditorAS : AFormAS,IFormAS
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.editor.EditorAS");
    
    /// <summary>
    /// JF- Associated model name. 
    /// </summary>
    public String m_strModelName = null;
    
    /// <summary>
    /// DH- The actual MVC name of the model to use if we're going to be storing it with a different name. 
    /// </summary>
    public String m_strModelMVCName = null;
    
    /// <summary>
    /// JF - This is where we store the Editor left navigation. 
    /// </summary>
    protected LeftNavMenu m_mEditorMenu;
    
    /// <summary>
    /// JF- Keep track of the previous page.
    /// </summary>
    private IDisplayPage m_dpPrevPage = null;
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.activityspace.AActivitySpace#CheckBasicAccess(String,String,boolean)"></seealso>
    public override bool CheckBasicAccess(String strPage, String strControl,
     bool bSameSpace)
    {
      // JF- If this is a redirect inside the same AS, just return true, we know the access has already been checked.
      if (bSameSpace)
      {
        return true;
      }
      // JF- If the user is returning to a cached Editor.
      IModel myModel = GetModel(m_strModelName);
      if (null != myModel)
      {
        try
        {
          if (((EditorModel) myModel).GetUserHasAccessToEditor())
          {
            return true;
          }
        }
        catch (Exception e)
        {
          log.Debug("Unable to cast the Model: " + myModel + " to EditorModel." + "The access to the Editor for the current user has to be checked.");
          }
      }
      // JF- Otherwise make sure this is either the start or the repost control. The access will be checked by those controls.
      if ((null != strControl) && (strControl.Equals(GetRepostControlName()) || strControl.Equals(EditorStartControl.STR_MVC_CLASS_NAME)))
      {
        return true;
      }
      // JF- Either the current user does not have access or the access will be checked by the Editor itself.
      return false;
    }
    
    /// <summary>
    /// Returns the Editor main title.
    /// It will get printed out in EditorHeaderView.
    /// </summary>
    /// <returns>String</returns>
    public abstract String GetEditorTitle();
    
    /// <summary>
    /// Optional method to implement a sub banner.
    /// </summary>
    /// <returns>PTConstruct</returns>
    public virtual PTConstruct GetSubBanner()
    {
      return null;
    }
    
    /// <summary>
    /// Returns the Editor left nav menu.
    /// </summary>
    /// <returns>LeftNavMenu</returns>
    public virtual LeftNavMenu GetLeftMenu()
    {
      return (m_mEditorMenu);
    }
    
    /// <summary>
    /// Returns the previous Display Page.
    /// Return null if there was no previous display page.
    /// </summary>
    /// <returns>IDisplayPage</returns>
    public virtual IDisplayPage GetPreviousPage()
    {
      return (m_dpPrevPage);
    }
    
    /// <summary>
    /// Returns the name of the repost control class.
    /// E.g.: PropRepostControl.GetName()
    /// </summary>
    /// <returns>String</returns>
    public abstract override String GetRepostControlName();
    
    /// <summary>
    /// Deletes a previously created sub Editor from the sub Editor
    /// list contained in the main Editor AS class.
    /// </summary>
    /// <param name="_sSubEditorKey"></param>
    public virtual void DeleteSubEditorFromList(String _sSubEditorKey)
    {
      DeleteSubSpace(_sSubEditorKey);
    }
    // **************** Helper methods to start, edit and delete sub Editors ***************
    
    /// <summary>
    /// Creates and returns a ASURL object going to the
    /// Editor whose name and key were passed in argument.
    /// </summary>
    /// <param name="_sSubEditorName"></param>
    /// <param name="_sSubEditorKey">
    /// - The 1st time you access that Editor,
    /// you can decide the value of the key. Next time you want to
    /// access that Editor you need to pass in the same key.
    /// </param>
    /// <param name="_nSubEditorType">- Sub Editor Type from EditorStartControl.</param>
    /// <param name="_sControlKey">
    /// Optional - only set it if you want
    /// to set an additional setting for the Control. If not,
    /// should be null. (E.g.: Can be used to set an Object class ID.)
    /// </param>
    /// <param name="_sControlValue">
    /// Optional - Should be the value associated
    /// to the ControlKey.
    /// </param>
    /// <returns>ASURL</returns>
    public virtual ASURL GoToSubEditor(String _sSubEditorName, String _sSubEditorKey,
     int _nSubEditorType, String _sControlKey, String _sControlValue)
    {
      ASURL subEditor = new ASURL(this);
      if (!GetSubSpaceExists(_sSubEditorKey))
      {
        subEditor = GoToNonCachedSubEditor(_sSubEditorName, _sSubEditorKey,
         _nSubEditorType, _sControlKey, _sControlValue);
      } else
      {
        subEditor = GoToCachedSubEditor(_sSubEditorKey);
      }
      return (subEditor);
    }
    
    /// <summary>
    /// Registers Views, Controls and Models used by the Editor Framework.
    /// Warning: m_myModel must be set before calling Init().
    /// </summary>
    public override void Init()
    {
      base.Init();
      if (m_strModelName == null)
      {
        log.Error("Attempting to Init() before m_strModelName has been set.");
        }
      // Turn on automatic tracking of current page (off by default).
      SetTrackCurrentPage(true);
      // JF- Register Model and then get it 
      // because we need it to initialize Views and Controls.
      if (m_strModelMVCName != null)
      {
        // We're registering a model and storing it under a different name.
        RegisterModel(m_strModelMVCName, m_strModelName);
      } else
      {
        RegisterModel(m_strModelName);
      }
      IModel myModel = GetModel(m_strModelName);
      // JF- Register Views
      RegisterView(EditorHeaderView.STR_MVC_CLASS_NAME, myModel);
      RegisterView(EditorLeftNavView.STR_MVC_CLASS_NAME, myModel);
      // JF- Register Controls
      RegisterControl(EditorStartControl.STR_MVC_CLASS_NAME, myModel);
      RegisterControl(EditorRepostControl.STR_MVC_CLASS_NAME, myModel);
      // JF- Register Display Pages
      RegisterPage(new ClosePopUpDP());
      // Error Model
      RegisterModel(ErrorModel.STR_MVC_CLASS_NAME);
      IModel errorModel = GetModel(ErrorModel.STR_MVC_CLASS_NAME);
      RegisterView(ErrorView.STR_MVC_CLASS_NAME, errorModel);
      RegisterControl(ErrorControl.STR_MVC_CLASS_NAME, errorModel);
      RegisterModel(TipsModel.STR_MVC_CLASS_NAME);
      IModel tipsModel = GetModel(TipsModel.STR_MVC_CLASS_NAME);
      RegisterView(TipsView.STR_MVC_CLASS_NAME, tipsModel);
      RegisterControl(TipsControl.STR_MVC_CLASS_NAME, tipsModel);
      // JF- Init Editor left nav menu.
      m_mEditorMenu = new LeftNavMenu(GetName());
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.AActivitySpace#SetCurrentPage()"></seealso>
    /// <param name="IDisplayPage">the current Display Page</param>
    public override void SetCurrentPage(IDisplayPage page)
    {
      // JF- Keep track of the previous page
      if ((null != page) && (null != GetCurrentPage()))
      {
        m_dpPrevPage = GetCurrentPage();
      }
      // JF- ClosePopUpDP should never become the current display page. 
      // This is not a real display page.
      if (page.GetName() != ClosePopUpDP.STR_MVC_CLASS_NAME)
      {
        base.SetCurrentPage(page);
      }
    }
    
    /// <summary>
    /// Sets a new current page from any Display Page name.
    /// </summary>
    /// <param name="_sNewCurrentPage"></param>
    public virtual void SetCurrentPage(String _sNewCurrentPage)
    {
      EditorDP dpNewPage = null;
      // JF- This is a helper method to be able to set the current display page 
      // with only the name of the DP whereas the AS requires to pass in the DP 
      // itself.
      // JF- Classes that extend this one use that method in their init method 
      // to set the page selected by default the 1st time the user accesses the Editor.
      try
      {
        try
        {
          dpNewPage = (EditorDP) GetPage(_sNewCurrentPage);
        }
        catch (Exception e)
        {
          throw XPException.GetInstance(e);
        }
      }
      catch (XPClassCastException cce)
      {
        log.Error("EditorAS::SetCurrentPage Unable to cast the Object " + dpNewPage + " to EditorDP. The following exception occured: " + cce);
        // JF- No need to extract, this is an unexpected error.
        throw new XPException("Unable to cast the Object " + dpNewPage + " to EditorDP.",
         cce);
      }
      if (null != dpNewPage)
      {
        SetCurrentPage(dpNewPage);
      } else
      {
        log.Error("EditorAS::SetCurrentPage Unable to set current page to: " + _sNewCurrentPage + ". The current has not been changed.");
        }
    }
    
    /// <summary>
    /// Helper method to build the ASURL Object for a cached
    /// Sub Editor.
    /// </summary>
    /// <param name="_sSubEditorKey">
    /// Key of the sub Editor you
    /// want to redirect to.
    /// </param>
    /// <returns>ASURL</returns>
    private ASURL GoToCachedSubEditor(String _sSubEditorKey)
    {
      ASURL subEditor = new ASURL(this);
      if (null != _sSubEditorKey)
      {
        // JF- First, change "DeleteOnCancel" flag to false.
        EditorAS mySubEditor = (EditorAS) GetSubSpace(_sSubEditorKey);
        subEditor.SetLinkGetCachedSpace(mySubEditor);
      }
      return (subEditor);
    }
    
    /// <summary>
    /// Helper method to build the ASURL Object for a new
    /// Sub Editor.
    /// </summary>
    /// <param name="_sSubEditorName"></param>
    /// <param name="_sSubEditorKey">
    /// - The 1st time you access that Editor,
    /// you can decide the value of the key. Next time you want to
    /// access that Editor you need to pass in the same key.
    /// </param>
    /// <param name="_nSubEditorType">- Sub Editor Type from EditorStartControl.</param>
    /// <param name="_sControlKey">
    /// Optional - only set it if you want
    /// to set an additional setting for the Control. If not,
    /// should be null. (E.g.: Can be used to set an Object class ID.)
    /// </param>
    /// <param name="_sControlValue">
    /// Optional - Should be the value associated
    /// to the ControlKey.
    /// </param>
    /// <returns>ASURL</returns>
    private ASURL GoToNonCachedSubEditor(String _sSubEditorName, String _sSubEditorKey,
     int _nEditorType, String _sControlKey, String _sControlValue)
    {
      ASURL subEditor = new ASURL(this);
      subEditor.SetLinkCreateNewSpace(_sSubEditorName, this);
      subEditor.SetControl(EditorStartControl.STR_MVC_CLASS_NAME);
      subEditor.AddControlArgument(EditorStartControl.QS_EDITOR_TYPE, XPConvert.ToString(_nEditorType));
      subEditor.AddControlArgument(EditorStartControl.QS_DELETE_ON_CANCEL,
       XPConvert.ToString(true));
      subEditor.SetSubSpaceKey(_sSubEditorKey);
      if ((null != _sControlKey) && (null != _sControlValue))
      {
        subEditor.AddControlArgument(_sControlKey, _sControlValue);
      }
      return (subEditor);
    }
  }
}