using System;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.openlog;
// Framework Imports
using com.plumtree.openfoundation.util;
namespace com.plumtree.uiinfrastructure.editor
{
  /// <summary>
  /// This is the Model class for the Editor Framework.
  /// Model classes should extend this class.
  /// </summary>
  /// <author>Julie Fournier</author>
  public abstract class EditorModel : IRepostModel,IEditorModelRO
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.editor.EditorModel");
    
    /// <summary>
    /// Associated Activity Space 
    /// </summary>
    public AActivitySpace m_asOwner;
    
    /// <summary>
    /// JF- This is for sub editors. 
    /// The first time we access the Editor, if the user clicks on cancel, we want to delete
    /// it from the sub Editor list.
    /// </summary>
    protected bool m_bDeleteOnCancel = false;
    
    /// <summary>
    /// JF- If this is true, the framework will close the window on finish and cancel.
    /// </summary>
    protected bool m_bCloseOnDone = false;
    
    /// <summary>
    /// JF- This is to hide the left navigation 
    /// </summary>
    protected bool m_bHideLeftNav = false;
    
    /// <summary>
    /// JF- If the user only has read access to the Editor. 
    /// </summary>
    protected bool m_bIsEditorReadOnly = false;
    
    /// <summary>
    /// JF- Is true if the current user has access to this Editor. 
    /// </summary>
    protected bool m_bUserHasAccessToEditor = false;
    
    /// <summary>
    /// JF- The Apply button next to the finish button is optional 
    /// </summary>
    protected bool m_bEnableApplyButton = false;
    
    /// <summary>
    /// JF- This is the Editor type. Constants for that variable are in EditorStartControl.
    /// This is mainly to know if it's create or edit mode.
    /// </summary>
    protected int m_nEditorType = EditorStartControl.EDITOR_START_FLAG_INVALID;
    
    /// <summary>
    /// This method is used by the ASManager to return new instances of managed
    /// objects.
    /// </summary>
    /// <returns>A new instance of the managed class (i.e. return new Foo();)</returns>
    public abstract Object Create();
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public abstract String GetName();
    
    /// <summary>
    /// Implement some code to execute when the user clicks on "Cancel".
    /// </summary>
    public virtual void Cancel()
    {
    }
    
    /// <summary>
    /// Implements some actions to perform on start.
    /// Use the Editor Type if needed.
    /// </summary>
    /// <param name="_htFormData"></param>
    /// <returns>boolean Is true if the current user has the rights to access this Editor.</returns>
    /// <returns>boolean Security flag: if false user will be redirected</returns>
    public abstract bool CheckEditorAccessAndStart(XPHashtable _htFormData);
    
    /// <summary>
    /// Implement some code to execute when the user clicks on "Finish".
    /// </summary>
    public abstract void Finish();
    
    /// <summary>
    /// That method contains the code that should be executed just
    /// after the object has been stored in the DB.
    /// </summary>
    public virtual void DoTaskAfterStore()
    {
    }
    
    /// <summary>
    /// That method contains the code that should be executed just
    /// before the object is stored in the DB.
    /// </summary>
    public virtual void DoTaskBeforeStore()
    {
    }
    
    /// <summary>
    /// Returns the current value of the repost action.
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetCurrentRepostAction()
    {
      RepostControl myRepostControl;
      // JF- Retrieve the repost control because this is where we store the repost action
      myRepostControl = (RepostControl) m_asOwner.GetControl(((EditorAS) m_asOwner).GetRepostControlName());
      return myRepostControl.GetRepostAction();
    }
    
    /// <summary>
    /// Returns the DeleteOnCancel flag.
    /// For sub Editors only:
    /// this flag will determine if the Editor
    /// should be deleted from the list of sub Editors
    /// on cancel.
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool GetDeleteOnCancel()
    {
      return (m_bDeleteOnCancel);
    }
    
    /// <summary>
    /// Returns the CloseOnDone flag.
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool GetCloseOnDone()
    {
      return (m_bCloseOnDone);
    }
    
    /// <summary>
    /// Returns the Editor Type
    /// (see EditorStartControl for constant names).
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetEditorType()
    {
      return (m_nEditorType);
    }
    
    /// <summary>
    /// Returns true if the apply button should be displayed.
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool GetEnableApplyButton()
    {
      return m_bEnableApplyButton;
    }
    
    /// <summary>
    /// Enables or disables the apply button next to the finish button.
    /// </summary>
    /// <param name="_bEnableApplyButton"></param>
    public virtual void SetEnableApplyButton(bool _bEnableApplyButton)
    {
      m_bEnableApplyButton = _bEnableApplyButton;
    }
    
    /// <summary>
    /// Returns the DeleteOnCancel flag.
    /// For sub Editors only:
    /// this flag will determine if the Editor
    /// should be deleted from the list of sub Editors
    /// on cancel.
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool GetHideLeftNav()
    {
      return (m_bHideLeftNav);
    }
    
    /// <summary>
    /// Returns true if the current user only has read access to this Editor.
    /// (can be true in Edit mode only)
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool GetIsEditorReadOnly()
    {
      return (m_bIsEditorReadOnly);
    }
    
    /// <summary>
    /// Returns the Redirect object
    /// (this is were we need to go at the end of the Editor).
    /// By default this is going to the previous AS
    /// (the one that opened this one). But Editors can override
    /// that method if they need to redirect somewhere else.
    /// </summary>
    /// <returns>Redirect</returns>
    public virtual Redirect GetRedirectAfterFinish()
    {
      Redirect result = new Redirect();
      // JF- If the parent space is already in the cache, we redirect to it.
      if (null != m_asOwner.GetParentSpace())
      {
        result.SetLinkGetCachedSpace(m_asOwner.GetParentSpace());
      } else
      // JF- If not, we redirect to a new one.
      {
        result.SetLinkCreateNewSpace(m_asOwner.GetParentSpaceName(), m_asOwner);
        }
      return (result);
    }
    
    /// <summary>
    /// Returns true if the current user has access to this Editor.
    /// (If false the user will be redirected.)
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool GetUserHasAccessToEditor()
    {
      return (m_bUserHasAccessToEditor);
    }
    
    /// <summary>
    /// Inits the Model with the Activity Space.
    /// </summary>
    /// <param name="parent"></param>
    public virtual void Init(AActivitySpace parent)
    {
      m_asOwner = parent;
    }
    
    /// <summary>
    /// Calls the right save function according to the page name passed in argument.
    /// </summary>
    /// <param name="_sPageName"></param>
    /// <param name="_htFormData"></param>
    /// <returns>int Page status for server side validation</returns>
    public abstract int SavePage(String _sPageName, XPHashtable _htFormData);
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IModel#CleanupTempData()"></seealso>
    public virtual void CleanupTempData()
    {
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IModel#CleanupAllData()"></seealso>
    public virtual void CleanupAllData()
    {
    }
    
    /// <summary>
    /// This method can be implemented by classes extending this one.
    /// It will be executed each time the user changes page on the left
    /// navigation menu.
    /// </summary>
    /// <param name="_sNewSpaceName"></param>
    /// <param name="_sNewPageName"></param>
    public virtual void DoTaskOnChangePage(String _sNewSpaceName, String _sNewPageName)
    {
    }
    
    /// <summary>
    /// This method will be called when you come back to the main Editor from a sub Editor.
    /// </summary>
    public virtual void DoTaskOnComeBackFromSubEditor()
    {
    }
    
    /// <summary>
    /// That method contains the code that should be executed just
    /// after the object has been initialized when the new editor
    /// is started.
    /// </summary>
    /// <param name="_htFormData"></param>
    public virtual void DoTaskOnStartEditor(XPHashtable _htFormData)
    {
    }
    
    /// <summary>
    /// Sets the DeleteOnCancel flag.
    /// For sub Editors only:
    /// this flag will determine if the Editor
    /// should be deleted from the list of sub Editors
    /// on cancel.
    /// </summary>
    /// <param name="_bDeleteOnCancel"></param>
    public virtual void SetDeleteOnCancel(bool _bDeleteOnCancel)
    {
      m_bDeleteOnCancel = _bDeleteOnCancel;
    }
    
    /// <summary>
    /// To tell the framework to close the window on finish or cancel.
    /// </summary>
    /// <param name="_bCloseOnDone"></param>
    public virtual void SetCloseOnDone(bool _bCloseOnDone)
    {
      m_bCloseOnDone = _bCloseOnDone;
    }
    
    /// <summary>
    /// Sets the Editor Type.
    /// (see EditorStartControl for constant names)
    /// </summary>
    /// <param name="_nEditorType"></param>
    public virtual void SetEditorType(int _nEditorType)
    {
      m_nEditorType = _nEditorType;
    }
    
    /// <summary>
    /// Sets the m_bHideLeftNav flag.
    /// If this flag is set to true, the left navigation will not be displayed.
    /// </summary>
    /// <param name="_bHideLeftNav"></param>
    public virtual void SetHideLeftNav(bool _bHideLeftNav)
    {
      m_bHideLeftNav = _bHideLeftNav;
    }
    
    /// <summary>
    /// Sets a new Page status.
    /// </summary>
    /// <param name="_nNewPageStatus"></param>
    public virtual void SetPageStatus(int _nNewPageStatus)
    {
      RepostControl myRepostControl;
      // JF- Retrieve the repost control because this is where we store the page status 
      myRepostControl = (RepostControl) m_asOwner.GetControl(((EditorAS) m_asOwner).GetRepostControlName());
      // JF- Set the new value
      myRepostControl.SetPageStatus(_nNewPageStatus);
      log.Debug("EditorModel::SetPageStatus Page status has been changed to :" + _nNewPageStatus);
      }
    
    /// <summary>
    /// Sets a new value for the m_bUserHasAccessToEditor flag.
    /// </summary>
    /// <param name="_bUserHasAccessToEditor"></param>
    public virtual void SetUserHasAccessToEditor(bool _bUserHasAccessToEditor)
    {
      m_bUserHasAccessToEditor = _bUserHasAccessToEditor;
    }
  }
}