using System;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.errordisplay;
using com.plumtree.uiinfrastructure.statichelpers;
// Framework Imports
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
namespace com.plumtree.uiinfrastructure.editor
{
  /// <summary>
  /// This start control is called anytime
  /// an Editor is started.
  /// Start Control classes should extend this class.
  /// </summary>
  /// <author>Julie Fournier</author>
  public class EditorStartControl : IControl
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.editor.EditorStartControl");
    
    /// <summary>
    /// Object Name for AS 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "EditorStart";
    
    /// <summary>
    /// JF- Query String constants: Editor type 
    /// </summary>
    public const String QS_EDITOR_TYPE = "editorType";
    
    /// <summary>
    /// JF- Query String constants: Param to tell the Activity space to delete it if the user clicks on cancel. 
    /// </summary>
    public const String QS_DELETE_ON_CANCEL = "deleteOnCancel";
    
    /// <summary>
    /// JF- Query String constants: Param to tell the framework to close the window on finish and cancel. 
    /// </summary>
    public const String QS_CLOSE_WINDOW_ON_DONE = "closeOnDone";
    
    /// <summary>
    /// JF- Query String constants: Param to tell the Framework to hide the left navigation 
    /// </summary>
    public const String QS_EDITOR_HIDE_LEFTNAV = "hideLeftNav";
    
    /// <summary>
    /// JF- Editor type: Invalid 
    /// </summary>
    public const int EDITOR_START_FLAG_INVALID = -1;
    
    /// <summary>
    /// JF- Editor type: Create mode 
    /// </summary>
    public const int EDITOR_START_FLAG_CREATE = 1;
    
    /// <summary>
    /// JF- Editor type: Edit mode 
    /// </summary>
    public const int EDITOR_START_FLAG_EDIT = 2;
    
    /// <summary>
    /// JF- Editor type: Other 
    /// </summary>
    public const int EDITOR_START_FLAG_MISC = 10;
    
    /// <summary>
    /// JF- Associated Activity Space 
    /// </summary>
    public AActivitySpace m_asOwner;
    
    /// <summary>
    /// JF- Associated Model 
    /// </summary>
    protected EditorModel m_model;
    
    /// <summary>
    /// JF- Contains all the form params from the request. 
    /// </summary>
    protected XPHashtable m_htFormData;
    
    /// <summary>
    /// This code is called by the AS Servlet each time there is an Editor is started.
    /// It calls the startEditor method in the Model if a valid Editor Type was in the QS.
    /// If the Redirect object returned is not null,
    /// the Servlet will the page specified by the Redirect.
    /// If the Redirect is null, the Servlet will get the current page
    /// from the AS and display it.
    /// </summary>
    /// <returns>Redirect</returns>
    public virtual Redirect CheckActionSecurityAndExecute(XPHashtable arguments)
    {
      m_htFormData = arguments;
      // JF- Get data from the query String and set them in the Model.
      SetModelVariables();
      if ((AccessStyles.LOW_BANDWIDTH == m_asOwner.GetAccessStyle()) || (AccessStyles.SECTION_508 == m_asOwner.GetAccessStyle()))
      {
        // JF- Disable the left nav for 508 pages
        m_model.SetHideLeftNav(true);
      }
      // JF- Call the Start Editor method on the Model.
      if (EDITOR_START_FLAG_INVALID != m_model.GetEditorType())
      {
        try
        {
          m_model.SetUserHasAccessToEditor(m_model.CheckEditorAccessAndStart(m_htFormData));
          }
        catch (Exception e)
        {
          IXPEnumerator arrErrors = UIErrorHandler.GetErrors(m_asOwner.GetSubSession(),
           m_asOwner.GetApplication(), m_asOwner, true, true);
          if (!arrErrors.MoveNext())
          {
            GeneralHelpers.SetUIError(m_asOwner, OpenLogService.GetComponent("UI_Infrastructure"),
             "EditorStartControl::CheckActionSecurityAndExecute", e, true,
             UIErrorLevel.SERIOUS, m_asOwner.GetString(131, ResourceConstants.INFRASTRUCTURE));
            } else
          {
            throw XPException.GetInstance(e);
          }
        }
        if (!m_model.GetUserHasAccessToEditor())
        {
          IXPEnumerator arrErrors = UIErrorHandler.GetErrors(m_asOwner.GetSubSession(),
           m_asOwner.GetApplication(), m_asOwner, true, true);
          if (!arrErrors.MoveNext())
          {
            GeneralHelpers.SetUIError(m_asOwner, OpenLogService.GetComponent("UI_Infrastructure"),
             "EditorStartControl::CheckActionSecurityAndExecute", new XPException(m_asOwner.GetString(132,
             ResourceConstants.INFRASTRUCTURE)), true, UIErrorLevel.SERIOUS,
             m_asOwner.GetString(132, ResourceConstants.INFRASTRUCTURE));
            } else
          {
            throw new XPException(m_asOwner.GetString(132, ResourceConstants.INFRASTRUCTURE));
            }
        }
      } else
      {
        log.Error("EditorStartControl::Execute The Editor was not started because the Editor Type is invalid.");
        IXPEnumerator arrErrors = UIErrorHandler.GetErrors(m_asOwner.GetSubSession(),
         m_asOwner.GetApplication(), m_asOwner, true, true);
        if (!arrErrors.MoveNext())
        {
          GeneralHelpers.SetUIError(m_asOwner, OpenLogService.GetComponent("UI_Infrastructure"),
           "EditorStartControl::CheckActionSecurityAndExecute", new XPException(m_asOwner.GetString(44,
           ResourceConstants.INFRASTRUCTURE)), true, UIErrorLevel.SERIOUS,
           m_asOwner.GetString(44, ResourceConstants.INFRASTRUCTURE));
        } else
        {
          throw new XPException(m_asOwner.GetString(44, ResourceConstants.INFRASTRUCTURE));
          }
      }
      // JF- Remove form params from hashtable. 
      // So we start with a clean one, next time this Control is called.
      m_htFormData = null;
      // JF- return null because we are not redirecting anywhere, 
      // We just want to see the current DP in the current AS.
      return null;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public virtual Object Create()
    {
      return new EditorStartControl();
    }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Inits the class with the associated model and activity space.
    /// </summary>
    /// <param name="model"></param>
    /// <param name="space"></param>
    public virtual void Init(IModel model, AActivitySpace space)
    {
      try
      {
        try
        {
          m_model = (EditorModel) model;
        }
        catch (Exception e)
        {
          throw XPException.GetInstance(e);
        }
      }
      catch (XPClassCastException e)
      {
        log.Error(e, "EditorStartControl::Init Unable to cast the Object " + model + " to EditorModel.");
        // JF- No need to extract, this is an unexpected error.
        throw new XPException("Unable to cast the Object " + model + " to EditorModel.",
         e);
      }
      m_asOwner = space;
    }
    
    /// <summary>
    /// Helper method to set the Model member
    /// variables from the form params.
    /// </summary>
    private void SetModelVariables()
    {
      // JF- Get and set the Editor type.
      String[] data = (String[]) m_htFormData.GetElement(QS_EDITOR_TYPE);
      if ((null != data) && (0 != data.Length) && (!"".Equals(data[0])))
      {
        m_model.SetEditorType(XPConvert.ToInteger(data[0]));
        log.Debug("EditorStartControl::SetModelVariables Editor Type: " + data[0]);
        } else
      {
        log.Error("EditorStartControl::SetModelVariables Editor Type ID was not in Query String.");
        throw new XPException("Unable to start the Editor.");
      }
      // JF- Get and set the DeleteOnCancel flag.
      data = (String[]) m_htFormData.GetElement(QS_DELETE_ON_CANCEL);
      if ((null != data) && (0 != data.Length) && (!"".Equals(data[0])))
      {
        m_model.SetDeleteOnCancel(XPConvert.ToBoolean(data[0]));
        log.Debug("EditorStartControl::SetModelVariables DeleteOnCancel: " + data[0]);
        } else
      {
        // JF- This should happen each time this is NOT a sub Editor (= most of the time).
        log.Debug("EditorStartControl::SetModelVariables DeleteOnCancel was not in Query String.");
        }
      // JF- Get and set the CloseOnDone flag.
      data = (String[]) m_htFormData.GetElement(QS_CLOSE_WINDOW_ON_DONE);
      if ((null != data) && (0 != data.Length) && (!"".Equals(data[0])))
      {
        m_model.SetCloseOnDone(XPConvert.ToBoolean(data[0]));
        log.Debug("EditorStartControl::SetModelVariables CloseOnDone: " + data[0]);
        } else
      {
        // JF- This should happen each time this is NOT a sub Editor (= most of the time).
        log.Debug("EditorStartControl::SetModelVariables DeleteOnCancel was not in Query String.");
        }
      // JF- Get and set the hideLeftNav flag.
      data = (String[]) m_htFormData.GetElement(QS_EDITOR_HIDE_LEFTNAV);
      if ((null != data) && (0 != data.Length) && (!"".Equals(data[0])))
      {
        m_model.SetHideLeftNav(XPConvert.ToBoolean(data[0]));
        log.Debug("EditorStartControl::SetModelVariables hideLeftNav: " + data[0]);
        } else
      {
        // JF- This should happen each time we don't want to hide the left navigation
        log.Debug("EditorStartControl::SetModelVariables hideLeftNav was not in Query String.");
        }
    }
  }
}