using System;
// Framework Imports
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
namespace com.plumtree.uiinfrastructure.editor
{
  /// <summary>
  /// This is helper class that contains the implementation
  /// of the left navigation menu as well as  two inner classes.
  /// One for the sub menus and one the links.
  /// This class is mainly used by EditorAS and classes extending it.
  /// The left navigation menu is printed out in EditorHeaderView.
  /// </summary>
  /// <author>Julie Fournier</author>
  public class LeftNavMenu
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.editor.LeftNavMenu");
    
    /// <summary>
    /// JF- Current AS name (name of the AS you create that menu from). 
    /// </summary>
    private String m_sCurrentASName;
    
    /// <summary>
    /// JF- Array of Sub Menus. 
    /// </summary>
    private XPArrayList m_alMenus;
    
    /// <summary>
    /// JF- Array of Mandatory Pages. 
    /// </summary>
    private XPArrayList m_alMandatoryPages;
    
    /// <summary>
    /// Creates a new Left Nav Menu object.
    /// Should have only one menu per Editor.
    /// </summary>
    /// <param name="_sCurrentASName">name of the AS you create that menu from</param>
    public LeftNavMenu(String _sCurrentASName)
    {
      m_alMenus = new XPArrayList();
      m_alMandatoryPages = new XPArrayList();
      m_sCurrentASName = _sCurrentASName;
    }
    
    /// <summary>
    /// Removes the page in argument from the array of mandatory pages.
    /// This is to keep track of the mandatory pages. When the page is 
    /// being visited by the user it should be removed from the array.
    /// </summary>
    /// <param name="_dpName">Name of the Display Page to remove</param>
    public virtual void RemovePageFromMandatoryArray(String _dpName)
    {
      LeftNavLink link = GetLinkGivenDPName(_dpName);
      if (null != link)
      {
        if (m_alMandatoryPages.Contains(link))
        {
          m_alMandatoryPages.Remove(link);
        }
      } else
      {
        log.Debug("LeftNavMenu::RemovePageFromMandatoryArray " + "The link corresponding to " + _dpName + " was not found in the left navigation menu.");
        }
    }
    
    /// <summary>
    /// Changes the status of the page in argument to mandatory or to non-mandatory.
    /// </summary>
    /// <param name="_bIsMandatory">New status</param>
    /// <param name="_dpName">Name of the Display Page</param>
    public virtual void SetMandatoryPage(bool _bIsMandatory, String _dpName)
    {
      LeftNavLink link = GetLinkGivenDPName(_dpName);
      if (null != link)
      {
        if (_bIsMandatory)
        {
          link.SetMandatory(true);
          if (!m_alMandatoryPages.Contains(link))
          {
            m_alMandatoryPages.Add(link);
          }
        } else
        {
          link.SetMandatory(false);
          if (m_alMandatoryPages.Contains(link))
          {
            m_alMandatoryPages.Remove(link);
          }
        }
      } else
      {
        log.Debug("LeftNavMenu::AddPageToMandatoryArray " + "The link corresponding to " + _dpName + " was not found in the left navigation menu.");
        }
    }
    
    /// <summary>
    /// To set a JavaScript to be called when the user tries to go to a particular Display Page. 
    /// </summary>
    /// <param name="_sDPName">Name of the Display Page to modify</param>
    /// <param name="_sJSName">Name of the JavaScript function</param>
    public virtual void AddJavaScriptValidationToLink(String _sDPName, String _sJSName)
    {
      LeftNavLink link = GetLinkGivenDPName(_sDPName);
      if (null != link)
      {
        link.SetJSValidationName(_sJSName);
      } else
      {
        log.Error("LeftNavMenu::AddJavaScriptValidationToLink " + "The link corresponding to " + _sDPName + " was not found in the left navigation menu.");
        }
    }
    
    /// <summary>
    /// Returns the array of non visited mandatory pages.
    /// </summary>
    /// <returns>XPArrayList</returns>
    public virtual XPArrayList GetArrayOfNonVisitedMandatoryPages()
    {
      return m_alMandatoryPages;
    }
    
    /// <summary>
    /// Returns true if some of the mandatory pages have not been visited
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool HasUnvisitedMandatoryPage()
    {
      return !m_alMandatoryPages.IsEmpty();
    }
    
    /// <summary>
    /// Creates a new link and adds it to the sub menu identified
    /// by the ID passed in argument.
    /// That link is going to the DP passed in argument in the current AS.
    /// </summary>
    /// <param name="_nSubMenuID">ID of the sub menu you want to add that link to</param>
    /// <param name="_dpName">name of the display page this link should go to</param>
    /// <returns>
    /// int ID of the newly created link. You need that ID only if
    /// you want to set optional settings on the link.
    /// </returns>
    public virtual int AddLinkToSubMenu(int _nSubMenuID, String _dpName)
    {
      int nNewLinkID = AddLinkToSubMenu(_nSubMenuID, m_sCurrentASName, _dpName,
       false);
      return (nNewLinkID);
    }
    
    /// <summary>
    /// Creates a new link and adds it to the sub menu identified
    /// by the ID passed in argument.
    /// That link is going to the DP passed in argument in the current AS.
    /// </summary>
    /// <param name="_nSubMenuID">ID of the sub menu you want to add that link to</param>
    /// <param name="_dpName">name of the display page this link should go to</param>
    /// <param name="_bIsMandatory">true if the link is mandatory</param>
    /// <returns>
    /// int ID of the newly created link. You need that ID only if
    /// you want to set optional settings on the link.
    /// </returns>
    public virtual int AddLinkToSubMenu(int _nSubMenuID, String _dpName, bool _bIsMandatory)
    {
      int nNewLinkID = AddLinkToSubMenu(_nSubMenuID, m_sCurrentASName, _dpName,
       _bIsMandatory);
      return (nNewLinkID);
    }
    
    /// <summary>
    /// Returns the array of sub menus.
    /// </summary>
    /// <returns>XPArrayList</returns>
    public virtual XPArrayList GetArraySubMenus()
    {
      return m_alMenus;
    }
    
    /// <summary>
    /// Will display or hide the link identified by the ID in argument.
    /// </summary>
    /// <param name="_bHide">true to hide it, false to not hide it</param>
    /// <param name="_nSubMenuID">ID of the sub menu containing the link</param>
    /// <param name="_nSubMenuID">ID of the link in that sub menu</param>
    public virtual void HideLink(bool _bHide, int _nSubMenuID, int _nLinkID)
    {
      LeftNavLink link = GetSubMenu(_nSubMenuID).GetLink(_nLinkID);
      link.SetHide(_bHide);
      RemovePageFromMandatoryArray(link.GetDPName());
    }
    
    /// <summary>
    /// Will display or hide the link identified by the Display Page name in argument.
    /// If multiple instances of that DP are found, they will all be changed.
    /// </summary>
    /// <param name="_bHide">true to hide it, false to not hide it</param>
    /// <param name="_sDPName">Name of the Display Page to hide</param>
    public virtual void HideLink(bool _bHide, String _sDPName)
    {
      LeftNavLink link = GetLinkGivenDPName(_sDPName);
      if (null != link)
      {
        link.SetHide(_bHide);
      } else
      {
        log.Warn("LeftNavMenu::HideLink " + "The link corresponding to " + _sDPName + " was not found in the left navigation menu.");
        }
      RemovePageFromMandatoryArray(_sDPName);
    }
    
    /// <summary>
    /// Will display or hide the sub menu identified by the ID in argument.
    /// </summary>
    /// <param name="_bHide">true to hide it, false to not hide it</param>
    /// <param name="_nSubMenuID">ID of the sub menu</param>
    public virtual void HideSubMenu(bool _bHide, int _nSubMenuID)
    {
      GetSubMenu(_nSubMenuID).SetHide(_bHide);
    }
    
    /// <summary>
    /// Creates a new sub menu, adds it to the array of
    /// menus and returns the ID of that new sub menu.
    /// The ID is the position of the sub menu in the array.
    /// </summary>
    /// <param name="_sSubMenuTitle"></param>
    /// <returns>int</returns>
    public virtual int StartNewSubMenu(String _sSubMenuTitle)
    {
      int nNewSubMenuID;
      LeftNavSubMenu smNewSubMenu = new LeftNavSubMenu(_sSubMenuTitle);
      nNewSubMenuID = m_alMenus.Add(smNewSubMenu);
      smNewSubMenu.SetID(nNewSubMenuID);
      log.Debug("LeftNavMenu::StartNewSubMenu Started following menu: " + _sSubMenuTitle + ", and returned following menu ID: " + nNewSubMenuID + ".");
      return (nNewSubMenuID);
    }
    
    /// <summary>
    /// Creates a new link and adds it to the sub menu identified
    /// by the ID passed in argument.
    /// That link is going to the AS and the DP passed in argument.
    /// </summary>
    /// <param name="_nSubMenuID">ID of the sub menu you want to add that link to</param>
    /// <param name="_sASName">Name of the AS where the link should go.</param>
    /// <param name="_dpName">Name of the display page this link should go to.</param>
    /// <param name="_bIsMandatory">true if the link is mandatory</param>
    /// <returns>
    /// int ID of the newly created link. You need that ID only if
    /// you want to set optional settings on the link.
    /// </returns>
    private int AddLinkToSubMenu(int _nSubMenuID, String _sASName, String _dpName,
     bool _bIsMandatory)
    {
      int nNewLinkID;
      LeftNavLink lNewLink = new LeftNavLink(_dpName, _sASName);
      lNewLink.SetMandatory(_bIsMandatory);
      LeftNavSubMenu smNewSubMenu = (LeftNavSubMenu) m_alMenus.GetElement(_nSubMenuID);
      nNewLinkID = smNewSubMenu.GetArrayLinks().Add(lNewLink);
      if (_bIsMandatory)
      {
        m_alMandatoryPages.Add(lNewLink);
      }
      log.Debug("LeftNavMenu::AddLinkToSubMenu Added following link: Activity Space: " + _sASName + " - Display Page: " + _dpName + ", to following menu ID: " + _nSubMenuID + ", with link ID: " + nNewLinkID + ".");
      return (nNewLinkID);
    }
    
    /// <summary>
    /// Helper method that returns the Link Object given the Display Page name.
    /// Returns null if the Link was not found
    /// </summary>
    /// <param name="_sDPName">Name of the Display Page</param>
    private LeftNavLink GetLinkGivenDPName(String _sDPName)
    {
      XPArrayList alLinks;
      LeftNavLink link;
      // JF- We loop through the sub menus
      for (int i = 0;i < m_alMenus.GetSize();i++)
      {
        alLinks = ((LeftNavSubMenu) m_alMenus.GetElement(i)).GetArrayLinks();
        // JF- Then for each sub menu, we loop through the links
        for (int j = 0;j < alLinks.GetSize();j++)
        {
          link = (LeftNavLink) alLinks.GetElement(j);
          if (_sDPName.Equals(link.GetDPName()))
          {
            return link;
          }
        }
      }
      return null;
    }
    
    /// <summary>
    /// Helper method that returns the sub menu
    /// corresponding to the ID passed in argument.
    /// </summary>
    /// <param name="_nSubMenuID"></param>
    /// <returns>LeftNavSubMenu</returns>
    private LeftNavSubMenu GetSubMenu(int _nSubMenuID)
    {
      return (LeftNavSubMenu) m_alMenus.GetElement(_nSubMenuID);
    }
  }
  /// <summary>
  /// Represents a Left Navigation sub menu.
  /// </summary>
  /// <author>Julie Fournier</author>
  internal class LeftNavSubMenu
  {
    
    /// <summary>
    /// JF- Title of the sub menu. 
    /// </summary>
    private String m_sTitle;
    
    /// <summary>
    /// JF- Array of links to display in that sub menu. 
    /// </summary>
    private XPArrayList m_alLinks;
    
    /// <summary>
    /// JF- To hide this sub menu 
    /// </summary>
    private bool m_bHide;
    
    /// <summary>
    /// JF- ID of the sub menu. 
    /// </summary>
    private int m_nID;
    
    /// <summary>
    /// Creates a new sub menu object.
    /// </summary>
    /// <param name="_sTitle"></param>
    public LeftNavSubMenu(String _sTitle)
    {
      m_alLinks = new XPArrayList();
      m_sTitle = _sTitle;
      m_nID = -1;
      m_bHide = false;
    }
    
    /// <summary>
    /// Returns the array of links contained in that sub menu.
    /// </summary>
    /// <returns>XPArrayList</returns>
    public virtual XPArrayList GetArrayLinks()
    {
      return m_alLinks;
    }
    
    /// <summary>
    /// Returns true if this sub menu should not be displayed
    /// in the left navigation.
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool GetHide()
    {
      return m_bHide;
    }
    
    /// <summary>
    /// Returns the ID of the sub menu.
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetID()
    {
      return m_nID;
    }
    
    /// <summary>
    /// Returns the link corresponding to the ID
    /// passed in argument.
    /// </summary>
    /// <param name="_nLinkID"></param>
    /// <returns>LeftNavLink</returns>
    public virtual LeftNavLink GetLink(int _nLinkID)
    {
      return (LeftNavLink) m_alLinks.GetElement(_nLinkID);
    }
    
    /// <summary>
    /// Returns the title of the sub menu.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetTitle()
    {
      return m_sTitle;
    }
    
    /// <summary>
    /// Set it to true if this sub menu should not be displayed
    /// in the left navigation.
    /// </summary>
    /// <param name="_bHide"></param>
    public virtual void SetHide(bool _bHide)
    {
      m_bHide = _bHide;
    }
    
    /// <summary>
    /// Sets the ID of the sub menu.
    /// Has to be the position of that sub menu in the array
    /// of the main menu.
    /// </summary>
    /// <param name="_nID"></param>
    public virtual void SetID(int _nID)
    {
      m_nID = _nID;
    }
    
    /// <summary>
    /// Sets a new title for that sub menu.
    /// </summary>
    /// <param name="_sTitle"></param>
    public virtual void SetTitle(String _sTitle)
    {
      m_sTitle = _sTitle;
    }
  }
  /// <summary>
  /// Represents a left navigation link.
  /// </summary>
  /// <author>Julie Fournier</author>
  internal class LeftNavLink
  {
    
    /// <summary>
    /// JF- Name of the AS where this link should go to. 
    /// </summary>
    private String m_sASName;
    
    /// <summary>
    /// JF- Name of the DP where this link should go to. 
    /// </summary>
    private String m_sDPName;
    
    /// <summary>
    /// JF- Name of a JavaScript to call before changing page when the user clicks on that link. 
    /// </summary>
    private String m_sValidationJS = "";
    
    /// <summary>
    /// JF- To hide this link 
    /// </summary>
    private bool m_bHide;
    
    /// <summary>
    /// JF- If this is a mandatory page, we display an red star next to the link. 
    /// </summary>
    private bool m_bIsMandatory;
    
    /// <summary>
    /// Create a new left nav link with the
    /// name of the DP the link is going to and the
    /// name of the AS that DP belongs to.
    /// </summary>
    /// <param name="_sDPName"></param>
    /// <param name="_sASName"></param>
    public LeftNavLink(String _sDPName, String _sASName)
    {
      m_sDPName = _sDPName;
      m_sASName = _sASName;
      m_bHide = false;
      m_bIsMandatory = false;
    }
    
    /// <summary>
    /// Returns the AS name.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetASName()
    {
      return m_sASName;
    }
    
    /// <summary>
    /// Returns the DP name.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetDPName()
    {
      return m_sDPName;
    }
    
    /// <summary>
    /// Returns true if this link should not be displayed
    /// in the left navigation.
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool GetHide()
    {
      return m_bHide;
    }
    
    /// <summary>
    /// Returns true if this link is mandatory.
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool GetMandatory()
    {
      return m_bIsMandatory;
    }
    
    /// <summary>
    /// Returns the name of the validation JavaScript for this link.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetJSValidationName()
    {
      return m_sValidationJS;
    }
    
    /// <summary>
    /// Sets a new AS name.
    /// </summary>
    /// <param name="_sASName"></param>
    public virtual void SetASName(String _sASName)
    {
      m_sASName = _sASName;
    }
    
    /// <summary>
    /// Sets a new DP name.
    /// </summary>
    /// <param name="_sDPName"></param>
    public virtual void SetDPName(String _sDPName)
    {
      m_sDPName = _sDPName;
    }
    
    /// <summary>
    /// Set it to true if this link should not be displayed
    /// in the left navigation.
    /// </summary>
    /// <param name="_bHide"></param>
    public virtual void SetHide(bool _bHide)
    {
      m_bHide = _bHide;
    }
    
    /// <summary>
    /// Set it to true if this link is mandatory.
    /// </summary>
    /// <param name="_bMandatory"></param>
    public virtual void SetMandatory(bool _bMandatory)
    {
      m_bIsMandatory = _bMandatory;
    }
    
    /// <summary>
    /// Sets the name of the JavaScript to call before changing
    /// page when the user clicks on this link.
    /// </summary>
    /// <param name="_sValidationJS"></param>
    public virtual void SetJSValidationName(String _sValidationJS)
    {
      m_sValidationJS = _sValidationJS;
    }
  }
}