using System;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.web;
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
namespace com.plumtree.uiinfrastructure.errordisplay
{
  /// <summary>
  /// This is an abstract model extended by various concrete error model.
  /// </summary>
  /// <author>Beng</author>
  public abstract class AErrorModel : IModel,IErrorModelRO
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.errordisplay.AErrorModel");
    // Parent Activity Space
    protected AActivitySpace m_asOwner;
    // TODOCR JF 12/10/02 - What is this?
    private String sCurrentASKey = "currentAS";
    
    /// <summary>
    /// Implement to remove the specified error.
    /// Concrete implementations should differ
    /// to the UIErrorHandler class, or one of its subclasses.
    /// <br /><br />
    /// </summary>
    /// <param name="sKey">String key used to store the error to be removed.</param>
    /// <param name="mySessionManager">ISessionManager</param>
    /// <param name="myApplication">IApplication</param>
    public abstract void ClearError(String sKey, ISessionManager mySessionManager,
     IApplication myApplication);
    
    /// <summary>
    /// Implement to remove all errors available to the current user.
    /// Concrete implementations should differ
    /// to the UIErrorHandler class, or one of its subclasses.
    /// <br /><br />
    /// </summary>
    /// <param name="mySessionManager">ISessionManager</param>
    /// <param name="myApplication">IApplication</param>
    public abstract void ClearErrors(ISessionManager mySessionManager, IApplication myApplication);
    
    /// <summary>
    /// Implement to clear those errors that need to be cleared when
    /// the user changes activity space.  Concrete implementations should differ
    /// to the UIErrorHandler class, or one of its subclasses.
    /// <br /><br />
    /// </summary>
    /// <param name="sASName">String name of the new activity space.</param>
    /// <param name="mySessionManager">ISessionManager</param>
    /// <param name="myApplication">IApplication</param>
    public abstract void ClearErrorsForASChange(String sASName, ISessionManager mySessionManager,
     IApplication myApplication);
    
    /// <summary>
    /// This method is used by the ASManager to return new instances of managed
    /// objects.
    /// </summary>
    /// <returns>A new instance of the managed class (i.e. return new Foo();)</returns>
    public abstract Object Create();
    
    /// <summary>
    /// Implement to return a boolean indicating whether this MVC
    /// should display errors requiring a separate display page.
    /// </summary>
    /// <returns>boolean value</returns>
    public abstract bool GetDisplaySeparatePageErrors();
    
    /// <summary>
    /// Implement to obtain an enumeration of errors appropriate for the current
    /// user.  Concrete implementations should differ
    /// to the UIErrorHandler class, or one of its subclasses.
    /// <br /><br />
    /// </summary>
    /// <param name="session">ISessionManager</param>
    /// <param name="myApplication">IApplication</param>
    /// <param name="as">AActivitySpace</param>
    /// <param name="bGetErrorsAssociatedWithAnAS">boolean value</param>
    /// <returns>IXPEnumerator of UIError objects</returns>
    public abstract IXPEnumerator GetErrors(ISessionManager session, IApplication myApplication,
     AActivitySpace activitySpace, bool bGetErrorsAssociatedWithAnAS);
    
    /// <summary>
    /// Implement to return a boolean indicating whether this MVC
    /// should display only those errors associated with the
    /// current MVC.
    /// <br /><br />
    /// </summary>
    /// <returns>boolean value</returns>
    public abstract bool GetErrorsAssociatedWithAnAS();
    
    /// <summary>
    /// Return the name of the MVC object.
    /// </summary>
    /// <returns>the name of the object.</returns>
    public abstract String GetName();
    
    /// <summary>
    /// This method initializes the model.  This is called automatically when
    /// AActivitySpace.RegisterModel() is called.
    /// </summary>
    /// <param name="parent">the parent Activity Space</param>
    public abstract void Init(AActivitySpace parent);
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IModel#CleanupTempData()"></seealso>
    public virtual void CleanupTempData()
    {
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IModel#CleanupAllData()"></seealso>
    public virtual void CleanupAllData()
    {
    }
    
    /// <summary>
    /// Call this method to clear the specified error.
    /// </summary>
    /// <param name="sKey">
    /// String key used to store the error.  Available from
    /// UIError.GetSessionKey().
    /// </param>
    public virtual void ClearError(String sKey)
    {
      ISessionManager mySessionManager = m_asOwner.GetSubSession();
      IApplication myApplication = m_asOwner.GetApplication();
      try
      {
        this.ClearError(sKey, mySessionManager, myApplication);
      }
      catch (XPException ex)
      {
        log.Error(ex, "Unable to clear error");
      }
    }
    
    /// <summary>
    /// Call this method to clear all errors exposed to the current user.
    /// </summary>
    public virtual void ClearErrors()
    {
      ISessionManager mySessionManager = m_asOwner.GetSubSession();
      IApplication myApplication = m_asOwner.GetApplication();
      this.ClearErrors(mySessionManager, myApplication);
    }
    
    /// <summary>
    /// Call this method to obtain an enumeration of all errors that
    /// are appropriate for the current user.
    /// <br /><br />
    /// </summary>
    /// <returns>IXPEnumerator containing UIError objects.</returns>
    public virtual IXPEnumerator GetErrors()
    {
      ISessionManager mySessionManager = m_asOwner.GetSubSession();
      IApplication myApplication = m_asOwner.GetApplication();
      String sCurrentAS = null;
      //   what is the current AS
      String sASName = this.m_asOwner.GetName();
      //   what we think to be the current AS
      sCurrentAS = (String) mySessionManager.GetAttribute(sCurrentASKey);
      //   correct the session
      mySessionManager.SetAttribute(sCurrentASKey, sASName);
      if (!sASName.Equals(sCurrentAS))
      {
        log.Debug("Activity Space Change.");
        this.ClearErrorsForASChange(sASName, mySessionManager, myApplication);
        }
      IXPEnumerator eTemp;
      eTemp = this.GetErrors(mySessionManager, myApplication, this.m_asOwner,
       this.GetErrorsAssociatedWithAnAS());
      return eTemp;
    }
  }
}