using System;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.errordisplay
{
  /// <author>
  /// Ben Geislinger
  /// UIError objects describe an error condition that needs to be communicated
  /// to one or more users of the portal.
  /// <br /><br />
  /// UI code encountering an error condition should create a new UIError object,
  /// configure that UIError as appropriate, and raise it using the
  /// PTErrorHandler class.  Error messages will then be generated and displayed
  /// as appropriate.
  /// </author>
  public class UIError
  {
    private Exception m_myException = null;
    private HTMLElement m_myHTMLMessage = null;
    private String m_myExtendedMessage = null;
    // TODOCR JF 12/10/02 - Can you comment each of those member variables?
    private String m_myMessage = null;
    private String m_myTitle = null;
    private String m_strASName = null;
    private String m_strKey = null;
    private UIErrorLevel m_myLevel = null;
    private UIErrorScope m_myScope = null;
    private bool m_bClearWithASChange = false;
    private bool m_bDisplayOnSeparatePage = false;
    private bool m_bImmediateClear = true;
    private bool m_bManualClear = false;
    
    /// <summary>
    /// Creates a new UIError object.
    /// </summary>
    /// <param name="level">UIErrorLevel enumeration indicates the severity of this error.</param>
    /// <param name="scope">UIErrorScope enumeration indicates the intended audience of this error.</param>
    /// <param name="exception">Previously caught exception might provide valuable clues to debuggers.</param>
    /// <param name="message">This message will be displayed to the user.</param>
    /// <param name="extendedMessage">This message will be embedded in HTML comments.</param>
    public UIError(UIErrorLevel level, UIErrorScope scope, Exception exception,
     String message, String extendedMessage)
    {
      m_myLevel = level;
      m_myScope = scope;
      m_myException = exception;
      m_myMessage = message;
      m_myExtendedMessage = extendedMessage;
    }
    
    /// <summary>
    /// Creates a new UIError object.
    /// </summary>
    /// <param name="level">UIErrorLevel enumeration indicates the severity of this error.</param>
    /// <param name="scope">UIErrorScope enumeration indicates the intended audience of this error.</param>
    /// <param name="message">This message will be displayed to the user.</param>
    /// <param name="extendedMessage">This message will be embedded in HTML comments.</param>
    public UIError(UIErrorLevel level, UIErrorScope scope, String message,
     String extendedMessage)
    {
      m_myLevel = level;
      m_myScope = scope;
      m_myMessage = message;
      m_myExtendedMessage = extendedMessage;
    }
    
    /// <summary>
    /// Call this method to determine the activity space with which this error
    /// is associated.  This method returns null if this error is not associated
    /// with an activity space.
    /// </summary>
    /// <returns>String name of the activity space.</returns>
    public virtual String GetActivitySpace()
    {
      return m_strASName;
    }
    
    /// <summary>
    /// Call this method to determine whether or not this error should be cleared
    /// when the user changes activity spaces.
    /// </summary>
    /// <returns>boolean value</returns>
    public virtual bool GetClearWithActivitySpaceChange()
    {
      return m_bClearWithASChange;
    }
    
    /// <summary>
    /// Call this method to determine whether or not the error message for this
    /// UIError object should be displayed on a separate ui page.
    /// <br /><br />
    /// </summary>
    /// <returns>boolean value</returns>
    public virtual bool GetDisplayOnSeparatePage()
    {
      return m_bDisplayOnSeparatePage;
    }
    
    /// <summary>
    /// Call this method to obtain the Exception associated with this error.
    /// <br /><br />
    /// </summary>
    /// <returns>Exception</returns>
    public virtual Exception GetException()
    {
      return m_myException;
    }
    
    /// <summary>
    /// Call this method to obtain the extended message of this error.
    /// <br /><br />
    /// </summary>
    /// <returns>String extended message</returns>
    public virtual String GetExtendedMessage()
    {
      return m_myExtendedMessage;
    }
    
    /// <summary>
    /// Call this method to obtain the HTMLElement message displayed with this
    /// error.
    /// <br /><br />
    /// </summary>
    /// <returns>HTMLElement message</returns>
    public virtual HTMLElement GetHTMLMessage()
    {
      return m_myHTMLMessage;
    }
    
    /// <returns></returns>
    public virtual bool GetImmediateClear()
    {
      return m_bImmediateClear;
    }
    
    /// <returns></returns>
    public virtual UIErrorLevel GetLevel()
    {
      return m_myLevel;
    }
    
    /// <summary>
    /// TODOCR JF 12/10/02 - Javadoc?
    /// </summary>
    /// <returns></returns>
    public virtual bool GetManualClear()
    {
      return m_bManualClear;
    }
    
    /// <summary>
    /// Call this method to obtain the String message displayed along with this
    /// error.
    /// <br /><br />
    /// </summary>
    /// <returns>String message.</returns>
    public virtual String GetMessage()
    {
      return m_myMessage;
    }
    
    /// <returns></returns>
    public virtual UIErrorScope GetScope()
    {
      return m_myScope;
    }
    
    /// <summary>
    /// Call this method to obtain the key with which this error was stored on
    /// the user's session or application.
    /// <br /><br />
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetSessionKey()
    {
      return m_strKey;
    }
    
    /// <summary>
    /// Call this method to obtain the title of this error.
    /// <br /><br />
    /// </summary>
    /// <returns>String title</returns>
    public virtual String GetTitle()
    {
      return m_myTitle;
    }
    
    /// <summary>
    /// Call this method to associate this error with an activity space.  This
    /// error will only be seen by the user when the user is on the specified
    /// activity space.
    /// <br /><br />
    /// Note that it does not make sense to associate this error with an activity
    /// space if this error gets its own display page, because the error display
    /// page is its own activity space.
    /// <br /><br />
    /// </summary>
    /// <param name="sName">Pass the String name of the activity space.</param>
    public virtual void SetActivitySpace(String sName)
    {
      m_strASName = sName;
    }
    
    /// <summary>
    /// Call this method to toggle whether or not this error should be cleared
    /// from memory when the user changes activity spaces.  This clearing will
    /// only take affect if you also call the SetActivitySpace( String ) method
    /// to tie this error to an activity space.
    /// </summary>
    /// <param name="b">boolean value</param>
    public virtual void SetClearWithActivitySpaceChange(bool b)
    {
      m_bClearWithASChange = b;
    }
    
    /// <summary>
    /// Call this method to toggle whether or not the error message for this
    /// UIError object should be displayed on a separate ui page.
    /// <br /><br />
    /// Pass false to have this error displayed on the main portal or editor
    /// page.  Pass true to have this error displayed on a separate page.
    /// </summary>
    /// <param name="b">boolean value.</param>
    public virtual void SetDisplayOnSeparatePage(bool b)
    {
      m_bDisplayOnSeparatePage = b;
    }
    
    /// <summary>
    /// Call this method to set the Exception included within this error.
    /// <br /><br />
    /// Information from this exception will be logged, but it will not
    /// be displayed to the user.
    /// <br /><br />
    /// </summary>
    /// <param name="ex">Exception</param>
    public virtual void SetException(Exception ex)
    {
      m_myException = ex;
    }
    
    /// <summary>
    /// Call this method to set the message displayed along with this error.
    /// <br /><br />
    /// An error can contain both a String message and an HTMLElement message,
    /// and both will be displayed along with the error.
    /// <br /><br />
    /// </summary>
    /// <param name="e">HTMLElement message.</param>
    public virtual void SetHTMLMessage(HTMLElement e)
    {
      m_myHTMLMessage = e;
    }
    
    /// <param name="b"></param>
    public virtual void SetImmediateClear(bool b)
    {
      m_bImmediateClear = b;
    }
    
    /// <summary>
    /// TODOCR JF 12/10/02 - What is this doing? No Javadoc?
    /// </summary>
    /// <param name="b"></param>
    public virtual void SetManualClear(bool b)
    {
      m_bManualClear = b;
    }
    
    /// <summary>
    /// Call this method to set the title of this error.
    /// <br /><br />
    /// This title will be displayed above the error description.
    /// <br /><br />
    /// </summary>
    /// <param name="title">String title</param>
    public virtual void SetTitle(String title)
    {
      m_myTitle = title;
    }
    
    /// <summary>
    /// Call this method to set the session key that was used to store this
    /// error on the user's session or application.
    /// <br /><br />
    /// </summary>
    /// <param name="s">String key.</param>
    public virtual void SetSessionKey(String s)
    {
      m_strKey = s;
    }
  }
}