using System;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.web;
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
namespace com.plumtree.uiinfrastructure.errordisplay
{
  /// <author>
  /// Ben Geislinger
  /// This class provides static methods for managing the current user's
  /// collection of UIError objects.  UIError objects describe those errors which
  /// need to be communicated to users.
  /// <br /><br />
  /// UIError objects are created and raised by ui code encountering an error
  /// condition.  Raised errors are then added to the error collections of the
  /// appropriate users.
  /// <br /><br />
  /// UIError objects are retrieved by ui code responsible for presenting error
  /// messages to the users of the portal.
  /// </author>
  public class UIErrorHandler
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.errordisplay.UIErrorHandler");
    // TODOCR JF 12/10/02 - Can you comment each of those member variables?
    private const String sErrorsSubSessionKey = "sesserrors";
    private const String sMetaDataSubSessionKey = "metadata";
    private const String sErrorsSubApplicationKey = "apperrors";
    private const String sAdminErrorsSubApplicationKey = "appAdminErrors";
    private const String sMetaDataSubApplicationKey = "appmetadata";
    private const String sMetaDataAdminSubApplicationKey = "appadminErrorsMetadata";
    private const String sErrorPageErrorsCountKey = "epecount";
    // TODOCR JF 12/10/02 - May be you could also have a member variable to set the max number of errors you can have.
    // I know in Editors for instance, I think the requirement is one error at the time
    // TODOCR JF 12/10/02 - Also could we have a flag to display errors only once.
    // Once again in Editors it doesn't make sense to keep the errors from one page to the other.
    // We want to display it once and them remove it.
    // TODOCR JF 12/10/02 - Those 2 settings above would need to be set once by the Editor Framework and then they
    // would be applied to the entire Editor.
    private static int iMaxNumCurrentUserErrors = 1;
    private static int iMaxNumAllUserErrors = 1;
    private static int iMaxNumAdminUserErrors = 1;
    
    /// <summary>
    /// private helper object 
    /// </summary>
    private static IAdminUIErrorHelper s_AdminUIErrorHelper = null;
    
    /// <summary>
    /// has this been statically initialized 
    /// </summary>
    private static bool s_bInitialized = false;
    
    /// <summary>
    /// Initializes static admin error helper.
    /// Can only be called once globally.
    /// </summary>
    /// <param name="IAdminUIErrorHelper">a helper class</param>
    public static void StaticInit(IAdminUIErrorHelper _adminUIErrorHelper)
    {
      if (s_bInitialized)
      {
        log.Error("UIErrorHandler being initialized twice.  Ignoring.");
        return;
      }
      s_bInitialized = true;
      s_AdminUIErrorHelper = _adminUIErrorHelper;
    }
    
    /// <summary>
    /// Call this method to clear the specified error from the current user's
    /// collection of UIError objects.
    /// <br /><br />
    /// This method should be called when it is no longer appropriate for users
    /// to see error messages generated for the passed error.
    /// <br /><br />
    /// </summary>
    /// <param name="error">
    /// Pass the UIError object you wish cleared from the current
    /// user's collection of errors.
    /// </param>
    /// <param name="sessionManager">Pass the current user's ISessionManager</param>
    /// <param name="application">Pass the IApplication</param>
    public static void ClearError(UIError error, ISessionManager sessionManager,
     IApplication application)
    {
      UIErrorScope scope = error.GetScope();
      log.Debug("Entering ClearError()");
      log.Debug("ClearError() - scope: " + scope);
      if (scope == UIErrorScope.CURRENT_USER)
      {
        ISessionManager subSession = sessionManager.GetSubSession(sErrorsSubSessionKey);
        String sKey = error.GetSessionKey();
        subSession.RemoveAttribute(sKey);
        //   If the error being raised gets a display page, update the count
        // TODOCR JF 12/10/02 - Same here why don't you have a helper method?
        if (error.GetDisplayOnSeparatePage())
        {
          int iOld = UIErrorHandler.GetNumErrorsRequiringErrorPage(sessionManager,
           application, UIErrorScope.CURRENT_USER);
          int iNew = iOld - 1;
          UIErrorHandler.PutNumErrorsRequiringErrorPage(iNew, sessionManager,
           application, UIErrorScope.CURRENT_USER);
        }
      } else
      if (scope == UIErrorScope.ALL_USERS)
      {
        IApplication subApp = application.GetSubApplication(UIErrorHandler.sErrorsSubApplicationKey);
        String sKey = error.GetSessionKey();
        subApp.GetCachingManager().RemoveEntry(sKey);
        //   If the error being raised gets a display page, update the count
        if (error.GetDisplayOnSeparatePage())
        {
          int iOld = UIErrorHandler.GetNumErrorsRequiringErrorPage(sessionManager,
           application, UIErrorScope.ALL_USERS);
          int iNew = iOld - 1;
          UIErrorHandler.PutNumErrorsRequiringErrorPage(iNew, sessionManager,
           application, UIErrorScope.ALL_USERS);
        }
      } else
      if (scope == UIErrorScope.ADMIN_USERS)
      {
        IApplication subApp = application.GetSubApplication(UIErrorHandler.sAdminErrorsSubApplicationKey);
        String sKey = error.GetSessionKey();
        subApp.GetCachingManager().RemoveEntry(sKey);
        //   If the error being raised gets a display page, update the count
        if (error.GetDisplayOnSeparatePage())
        {
          int iOld = UIErrorHandler.GetNumErrorsRequiringErrorPage(sessionManager,
           application, UIErrorScope.ADMIN_USERS);
          int iNew = iOld - 1;
          UIErrorHandler.PutNumErrorsRequiringErrorPage(iNew, sessionManager,
           application, UIErrorScope.ADMIN_USERS);
        }
      }
    }
    
    /// <summary>
    /// Call this method to clear the specified error from the current user's
    /// collection of UIError objects.
    /// <br /><br />
    /// This method should be called when it is no longer appropriate for users
    /// to see error messages generated for the passed error.
    /// </summary>
    /// <param name="sKey">Pass the key used to store the error you wish to clear.</param>
    /// <param name="sessionManager">Pass the current user's ISessionManager</param>
    /// <param name="application">Pass the IApplication</param>
    public static void ClearError(String sKey, ISessionManager sessionManager,
     IApplication application)
    {
      log.Debug("Entering ClearError()");
      UIErrorScope scope = null;
      UIError error = null;
      IApplication subApp;
      ISessionManager subSession = sessionManager.GetSubSession(sErrorsSubSessionKey);
      try
      {
        error = (UIError) subSession.GetAttribute(sKey);
      }
      catch (Exception ex)
      {
        log.Error(ex, "A non error was stored in an error only subsession.");
        }
      if (error != null)
      {
        scope = UIErrorScope.CURRENT_USER;
        subSession.RemoveAttribute(sKey);
        log.Debug("Found the error on the session.");
      } else
      {
        // we couldn't get the error from the user's session
        subApp = application.GetSubApplication(UIErrorHandler.sErrorsSubApplicationKey);
        try
        {
          error = (UIError) subApp.GetCachingManager().GetEntry(sKey);
        }
        catch (Exception ex)
        {
          log.Error(ex, "A non error was stored in an error only subsession.");
          }
        if (error != null)
        {
          scope = UIErrorScope.ALL_USERS;
          subApp.GetCachingManager().RemoveEntry(sKey);
          log.Debug("Found the error on the app for all users.");
        } else
        {
          //    we couldnt get the error from either current user's session, the
          //   applicaiton for errors for all users, so it must be an admin
          //   error
          subApp = application.GetSubApplication(UIErrorHandler.sAdminErrorsSubApplicationKey);
          try
          {
            error = (UIError) subApp.GetCachingManager().GetEntry(sKey);
          }
          catch (Exception ex)
          {
            log.Error(ex, "A non error was stored in an error only subsession.");
            }
          if (error != null)
          {
            scope = UIErrorScope.ADMIN_USERS;
            subApp.GetCachingManager().RemoveEntry(sKey);
            log.Debug("Found the error on the app for admin errors.");
          } else
          {
            log.Error("Could not find a UIError stored with the key: " + sKey);
            }
        }
      }
      if (error != null)
      {
        //   If the error being raised gets a display page, update the count
        if (error.GetDisplayOnSeparatePage())
        {
          int iOld = UIErrorHandler.GetNumErrorsRequiringErrorPage(sessionManager,
           application, scope);
          int iNew = iOld - 1;
          UIErrorHandler.PutNumErrorsRequiringErrorPage(iNew, sessionManager,
           application, scope);
        }
      }
    }
    
    /// <summary>
    /// Call this method to clear all errors from the current user's collection
    /// of errors.
    /// <br /><br />
    /// Note that this will clear all errors having scope ALL_USERS, and all
    /// errors having scope ADMIN_USERS, and all errors having scope
    /// CURRENT_USER raised under usage of the current users.  Errors raised
    /// during usage of other users will not be cleared.
    /// <br /><br />
    /// </summary>
    /// <param name="sessionManager">Pass the current user's ISessionManager</param>
    /// <param name="application">Pass the IApplication</param>
    public static void ClearErrors(ISessionManager sessionManager, IApplication application)
    {
      log.Debug("Entering ClearErrors()");
      sessionManager.RemoveSubSession(sErrorsSubSessionKey);
      application.RemoveSubApplication(UIErrorHandler.sAdminErrorsSubApplicationKey);
      application.RemoveSubApplication(UIErrorHandler.sErrorsSubApplicationKey);
      UIErrorHandler.PutNumErrorsRequiringErrorPage(0, sessionManager, application,
       UIErrorScope.CURRENT_USER);
      UIErrorHandler.PutNumErrorsRequiringErrorPage(0, sessionManager, application,
       UIErrorScope.ADMIN_USERS);
      UIErrorHandler.PutNumErrorsRequiringErrorPage(0, sessionManager, application,
       UIErrorScope.ALL_USERS);
    }
    
    /// <summary>
    /// Call this method to clear those errors that are associated with a
    /// specific activity space.
    /// <br /><br />
    /// UIError objects can be associated with a particular activity space. When
    /// the user leaves that activity space, such errors should be cleared from
    /// the user's collection of errors.
    /// <br /><br />
    /// </summary>
    /// <param name="sNewASName">
    /// Pass the name of the activity space which the user
    /// is entering.
    /// </param>
    /// <param name="sessionManager">Pass the current user's ISessionManager</param>
    /// <param name="application">Pass the IApplication</param>
    public static void ClearErrorsForASChange(String sNewASName, ISessionManager sessionManager,
     IApplication application)
    {
      IXPEnumerator enumErrors = InternalGetErrors(sessionManager, application,
       null, false, false);
      UIError error;
      String sErrorAS;
      log.Debug("ClearErrorsForASChange - sNewASName: " + sNewASName);
      while (enumErrors.MoveNext())
      {
        error = (UIError) enumErrors.GetCurrent();
        if (error.GetClearWithActivitySpaceChange())
        {
          log.Debug("Found an error that clears with an AS change.");
          sErrorAS = error.GetActivitySpace();
          if (sErrorAS != null)
          {
            if (!sErrorAS.Equals(sNewASName))
            {
              log.Debug("There\'s been a change! Clearing this error.");
              UIErrorHandler.ClearError(error, sessionManager, application);
              }
          }
        }
      }
    }
    
    /// <param name="sessionManager"></param>
    /// <param name="application"></param>
    /// <param name="space"></param>
    /// <param name="bForThisAs"></param>
    /// <param name="bIncludeAdminErrors"></param>
    public static void ClearErrorsRequiringASeparateDisplayPage(ISessionManager sessionManager,
     IApplication application, AActivitySpace space, bool bForThisAs, bool bIncludeAdminErrors)
    {
      log.Debug("ClearErrorsRequiringASeparateDisplayPage()");
      IXPEnumerator enumerator = UIErrorHandler.InternalGetErrors(sessionManager,
       application, space, bForThisAs, bIncludeAdminErrors);
      UIError eTemp = null;
      while (enumerator.MoveNext())
      {
        eTemp = (UIError) enumerator.GetCurrent();
        log.Debug("Found an error: " + eTemp.GetTitle());
        UIErrorHandler.ClearError(eTemp, sessionManager, application);
      }
    }
    
    /// <summary>
    /// Call this method to determine whether or not the current user should
    /// see admin-only errors.
    /// <br /><br />
    /// This implementation of this method will always return false.  More
    /// specific subclasses of UIErrorHandler will return more meaningful values.
    /// For example, PTErrorHandler will determine if the current user is an
    /// admin users, and return a value based on this fact.
    /// </summary>
    /// <param name="space">AActivitySpace</param>
    /// <returns>boolean value</returns>
    public static bool GetDisplayAdminErrors(AActivitySpace space)
    {
      if (s_AdminUIErrorHelper != null)
      {
        return s_AdminUIErrorHelper.GetDisplayAdminErrors(space);
      }
      return false;
    }
    
    /// <summary>
    /// Call this method to obtain a boolean value indicating whether or not
    /// the current user should see errors intended for administrators.
    /// <br /><br />
    /// </summary>
    /// <param name="Object">session</param>
    /// <returns>boolean value</returns>
    public static bool GetDisplayAdminErrors(Object session)
    {
      if (s_AdminUIErrorHelper != null)
      {
        return s_AdminUIErrorHelper.GetDisplayAdminErrors(session);
      }
      return false;
    }
    
    /// <summary>
    /// Call this method to obtain an enumeration of the current user's collection
    /// of errors.
    /// <br /><br />
    /// </summary>
    /// <param name="sessionManager">Pass the current user's ISessionManager</param>
    /// <param name="application">Pass the IApplication</param>
    /// <returns>
    /// An IXPEnumerator containing UIError objects appropriate for the
    /// current user.
    /// </returns>
    /// <deprecated></deprecated>
    public static IXPEnumerator GetErrors(ISessionManager sessionManager,
     IApplication application)
    {
      return GetErrors(sessionManager, application, null, false, false);
    }
    
    /// <summary>
    /// Call this method to obtain an enumeration of the current user's collection
    /// of errors that are associated with a given activity space.
    /// <br /><br />
    /// </summary>
    /// <param name="sessionManager">Pass the current user's ISessionManager</param>
    /// <param name="application">Pass the IApplication</param>
    /// <param name="activitySpace">
    /// Pass the activity space for which you wish
    /// to obtain appropriate errors.
    /// </param>
    /// <param name="bForThisSpace">
    /// Pass a boolean value indicating if you wish to obtain
    /// only errors associated with the passed activity space.  If you pass true,
    /// you will also obtain those errors not associated with any activity space,
    /// you will only not obtain those errors associated with another activity
    /// space.  If you pass false, you will obtain all errors for this user.
    /// </param>
    /// <param name="bIncludeAdminErrors">
    /// Pass a boolean indicating whether or not you
    /// wish to include errors for admin users.  Use
    /// ErrorHanlder.GetDisplayAdminErrors() to obtain the correct value for this
    /// parameter.
    /// </param>
    /// <returns>
    /// An IXPEnumerator containing UIError objects appropriate for the
    /// current user.
    /// </returns>
    public static IXPEnumerator GetErrors(ISessionManager sessionManager,
     IApplication application, AActivitySpace space, bool bForThisAs, bool bIncludeAdminErrors)
    {
      return InternalGetErrors(sessionManager, application, space, bForThisAs,
       bIncludeAdminErrors);
    }
    
    /// <summary>
    /// Call this method to determine whether or not there are any errors for
    /// this user requiring their own error display page.  Some errors are
    /// displayed on the current portal page, other errors require their own
    /// separate display page.
    /// <br /><br />
    /// </summary>
    /// <param name="sessionManager">Pass the current user's ISessionManager</param>
    /// <param name="application">Pass the IApplication</param>
    /// <returns>
    /// boolean value indicating whether or not to include errors of
    /// scope ADMIN_USERS.  Use UIErrorHandler.GetDisplayAdminErrors() to obtain
    /// the appropriate value for this parameter.
    /// </returns>
    public static bool IsErrorPageRequired(ISessionManager sessionManager,
     IApplication application, bool bIncludeAdminErrors)
    {
      log.Debug("Entering IsErrorPageRequired()");
      bool b = false;
      int iCurrent = UIErrorHandler.GetNumErrorsRequiringErrorPage(sessionManager,
       application, UIErrorScope.CURRENT_USER);
      int iAll = UIErrorHandler.GetNumErrorsRequiringErrorPage(sessionManager,
       application, UIErrorScope.ALL_USERS);
      int iAdmin = 0;
      if (bIncludeAdminErrors)
      {
        iAdmin = UIErrorHandler.GetNumErrorsRequiringErrorPage(sessionManager,
         application, UIErrorScope.ADMIN_USERS);
      }
      int i = iCurrent + iAdmin + iAll;
      if (i > 0)
      {
        b = true;
      }
      log.Debug("IsErrorPageRequired() i: " + i);
      return b;
    }
    
    /// <summary>
    /// Call this method to raise an error for display.  The appropriate users
    /// will see this error displayed in the appropriate way.
    /// <br /><br />
    /// Calling this method will log error information.
    /// <br /><br />
    /// Calling this method will cause the passed UIError object to be saved
    /// on the session or application as appropriate.
    /// <br /><br />
    /// </summary>
    /// <param name="error">Pass a UIError object.</param>
    /// <param name="sessionManager">Pass an ISessionManager</param>
    /// <param name="application">Pass an IApplication</param>
    public static void RaiseError(UIError error, ISessionManager sessionManager,
     IApplication application)
    {
      UIErrorScope errorScope = error.GetScope();
      log.Debug("RaiseError()");
      log.Debug("RaiseError() - error: " + error.GetMessage());
      log.Debug("RaiseError() - error.GetActivitySpace(): " + error.GetActivitySpace());
      log.Debug("RaiseError() - error.GetClearWithActivitySpaceChange(): " + error.GetClearWithActivitySpaceChange());
      log.Debug("RaiseError() - errorScope: " + errorScope);
      log.Info("A UIError has been raised.");
      log.Info("-- Message: " + error.GetMessage());
      log.Info("-- Extended Message: " + error.GetExtendedMessage());
      if (error.GetException() != null)
      {
        log.Info("-- Exception: " + error.GetException().Message);
      }
      if (errorScope == UIErrorScope.CURRENT_USER)
      {
        // Save the error on the session.
        ISessionManager subSession = sessionManager.GetSubSession(sErrorsSubSessionKey);
        subSession.SetMaxCount(iMaxNumCurrentUserErrors);
        String sKey = subSession.SetAttribute(error);
        error.SetSessionKey(sKey);
        //   If the error being raised gets a display page, update the count
        // TODOCR JF 12/10/02 - Looks like you could have a helper method for the few lines below.
        if (error.GetDisplayOnSeparatePage())
        {
          int iOld = UIErrorHandler.GetNumErrorsRequiringErrorPage(sessionManager,
           application, UIErrorScope.CURRENT_USER);
          int iNew = iOld + 1;
          UIErrorHandler.PutNumErrorsRequiringErrorPage(iNew, sessionManager,
           application, UIErrorScope.CURRENT_USER);
        }
      } else
      if (errorScope == UIErrorScope.ALL_USERS)
      {
        IApplication subApp = application.GetSubApplication(UIErrorHandler.sErrorsSubApplicationKey);
        CachingManager cm = subApp.GetCachingManager();
        cm.SetMaxCount(iMaxNumAllUserErrors);
        String sKey = cm.SetEntry(error);
        error.SetSessionKey(sKey);
        //   If the error being raised gets a display page, update the count
        if (error.GetDisplayOnSeparatePage())
        {
          int iOld = UIErrorHandler.GetNumErrorsRequiringErrorPage(sessionManager,
           application, UIErrorScope.ALL_USERS);
          int iNew = iOld + 1;
          UIErrorHandler.PutNumErrorsRequiringErrorPage(iNew, sessionManager,
           application, UIErrorScope.ALL_USERS);
        }
      } else
      if (errorScope == UIErrorScope.ADMIN_USERS)
      {
        IApplication subApp = application.GetSubApplication(sAdminErrorsSubApplicationKey);
        CachingManager cm = subApp.GetCachingManager();
        cm.SetMaxCount(iMaxNumAdminUserErrors);
        String sKey = cm.SetEntry(error);
        error.SetSessionKey(sKey);
        //   If the error being raised gets a display page, update the count
        if (error.GetDisplayOnSeparatePage())
        {
          int iOld = UIErrorHandler.GetNumErrorsRequiringErrorPage(sessionManager,
           application, UIErrorScope.ADMIN_USERS);
          int iNew = iOld + 1;
          UIErrorHandler.PutNumErrorsRequiringErrorPage(iNew, sessionManager,
           application, UIErrorScope.ADMIN_USERS);
        }
      }
    }
    //	/**
    //	 * Call this method to obtain the count of the number of errors in the
    //	 *     current user's collection of errors requiring their own display page.
    //	 *     <br><br>
    //	 * UIError objects can require that the error message describing the error
    //	 *     be displayed on its own page.  Error messages for PTErrors not requiring
    //	 *     this can be displayed on the main portal or editor page.
    //	 * <br><br>
    //	 * @deprecated
    //	 * @param sessionManager Pass the current user's ISessionManager
    //	 * @param application Pass the IApplication
    //	 * @return int count of errors requiring their own page
    //	 */
    //	private static int GetNumErrorsRequiringErrorPage(
    //		ISessionManager sessionManager, IApplication application)
    //	{
    //		
    //			log.Debug(//				"Entering GetNumErrorsRequiringErrorPage()");
    //		
    //
    //		int iCurrentUser = 0;
    //		int iAllUsers    = 0;
    //		int iAdmin		 = 0;
    //		int iTotal		 = 0;
    //
    //		iCurrentUser     = UIErrorHandler.GetNumErrorsRequiringErrorPage(sessionManager,
    //				application, UIErrorScope.CURRENT_USER);
    //		iAllUsers		 = UIErrorHandler.GetNumErrorsRequiringErrorPage(sessionManager,
    //				application, UIErrorScope.ALL_USERS);
    //		iAdmin			 = UIErrorHandler.GetNumErrorsRequiringErrorPage(sessionManager,
    //				application, UIErrorScope.ADMIN_USERS);
    //
    //		iTotal = iCurrentUser + iAllUsers + iAdmin;
    //
    //		
    //			log.Debug(//				"GetNumErrorsRequiringErrorPage() - iTotal: " + iTotal);
    //		
    //
    //		return iTotal;
    //	}
    
    /// <summary>
    /// Call this method to obtain the number of errors requiring a separate
    /// display page.
    /// <br /><br />
    /// Pass the UIErrorScope for which you wish to query.  For example, pass
    /// UIErrorScope.CURRENT_USER, and you will obtain the number of errors for
    /// the current user requiring their own display page.
    /// <br /><br />
    /// </summary>
    /// <param name="sessionManager">ISessionManager</param>
    /// <param name="application">IApplication</param>
    /// <param name="scope">UIErrorScope</param>
    /// <returns>boolean value</returns>
    private static int GetNumErrorsRequiringErrorPage(ISessionManager sessionManager,
     IApplication application, UIErrorScope scope)
    {
      log.Debug("Entering GetNumErrorsRequiringErrorPage()");
      log.Debug("GetNumErrorsRequiringErrorPage() - scope: " + scope);
      int iTotal = 0;
      Object temp;
      if (scope == UIErrorScope.CURRENT_USER)
      {
        //   errors for this user
        log.Debug(" sessionManager.ContainsSubSession( sMetaDataSubSessionKey ): " + sessionManager.ContainsSubSession(sMetaDataSubSessionKey));
        ISessionManager subSession = sessionManager.GetSubSession(sMetaDataSubSessionKey);
        temp = subSession.GetAttribute(UIErrorHandler.sErrorPageErrorsCountKey);
        if (temp != null)
        {
          iTotal = XPConvert.ToInteger(temp);
        }
      } else
      if (scope == UIErrorScope.ALL_USERS)
      {
        log.Debug("application.ContainsSubApplication( sMetaDataSubApplicationKey ) : " + application.ContainsSubApplication(sMetaDataSubApplicationKey));
        //   errors for all users
        IApplication subApp = application.GetSubApplication(UIErrorHandler.sMetaDataSubApplicationKey);
        temp = subApp.GetCachingManager().GetEntry(UIErrorHandler.sErrorPageErrorsCountKey);
        if (temp != null)
        {
          iTotal = XPConvert.ToInteger(temp);
        }
      } else
      if (scope == UIErrorScope.ADMIN_USERS)
      {
        log.Debug("application.ContainsSubApplication( sMetaDataSubApplicationKey ) : " + application.ContainsSubApplication(sMetaDataSubApplicationKey));
        //   errors for admin users
        IApplication subApp = application.GetSubApplication(UIErrorHandler.sMetaDataAdminSubApplicationKey);
        temp = subApp.GetCachingManager().GetEntry(UIErrorHandler.sErrorPageErrorsCountKey);
        if (temp != null)
        {
          iTotal = XPConvert.ToInteger(temp);
        }
      }
      log.Debug("GetNumErrorsRequiringErrorPage() - iTotal: " + iTotal);
      return iTotal;
    }
    
    /// <param name="sessionManager"></param>
    /// <param name="application"></param>
    /// <param name="space"></param>
    /// <param name="bForThisAs"></param>
    /// <param name="bIncludeAdminErrors"></param>
    /// <returns></returns>
    private static IXPEnumerator InternalGetErrors(ISessionManager sessionManager,
     IApplication application, AActivitySpace space, bool bForThisAs, bool bIncludeAdminErrors)
    {
      XPArrayList alTemp = new XPArrayList();
      IXPEnumerator enumTemp;
      String sKey;
      UIError error;
      String sErrorAS;
      String sASName = null;
      if (space != null)
      {
        sASName = space.GetName();
      } else
      {
        bForThisAs = false;
      }
      log.Debug("InternalGetErrors() - sASName: " + sASName);
      log.Debug("InternalGetErrors() - bForThisAs: " + XPConvert.ToString(bForThisAs));
      log.Debug("InternalGetErrors() - bIncludeAdminErrors: " + XPConvert.ToString(bIncludeAdminErrors));
      //   Current user errors
      ISessionManager subSession = sessionManager.GetSubSession(sErrorsSubSessionKey);
      enumTemp = subSession.GetKeys();
      while (enumTemp.MoveNext())
      {
        sErrorAS = null;
        error = null;
        sKey = (String) enumTemp.GetCurrent();
        try
        {
          error = (UIError) subSession.GetAttribute(sKey);
        }
        catch (Exception ex)
        {
          log.Error(ex, "A non UIError was stored in the error subsession.");
          }
        if (error != null)
        {
          if (bForThisAs)
          {
            if (sASName != null)
            {
              sErrorAS = error.GetActivitySpace();
              if (sErrorAS != null)
              {
                if (sErrorAS.Equals(sASName))
                {
                  alTemp.Add(error);
                }
              } else
              {
                //   this error is not associated with an activity space.
                //   we'll just add it.
                alTemp.Add(error);
              }
            }
          } else
          {
            alTemp.Add(error);
          }
        }
      }
      // error != null
      //   for each session key
      //   All user errors
      IApplication subApp = application.GetSubApplication(sErrorsSubApplicationKey);
      enumTemp = subApp.GetCachingManager().GetKeys();
      while (enumTemp.MoveNext())
      {
        sErrorAS = null;
        error = null;
        sKey = (String) enumTemp.GetCurrent();
        try
        {
          error = (UIError) subApp.GetCachingManager().GetEntry(sKey);
        }
        catch (Exception ex)
        {
          log.Error(ex, "InternalGetErrors() - A non UIError was stored in the error subapplication.");
          }
        if (error != null)
        {
          if (bForThisAs)
          {
            if (sASName != null)
            {
              sErrorAS = error.GetActivitySpace();
              if (sErrorAS != null)
              {
                if (sErrorAS.Equals(sASName))
                {
                  alTemp.Add(error);
                  if (error.GetImmediateClear())
                  {
                    ClearError(error, sessionManager, application);
                  }
                }
              } else
              {
                //   this error is not associated with an activity space.
                //   we'll just add it.
                alTemp.Add(error);
                if (error.GetImmediateClear())
                {
                  ClearError(error, sessionManager, application);
                }
              }
            }
          } else
          {
            alTemp.Add(error);
          }
        }
      }
      // error != null
      //   for each application key
      if (bIncludeAdminErrors)
      {
        //   Admin errors
        subApp = application.GetSubApplication(sAdminErrorsSubApplicationKey);
        enumTemp = subApp.GetCachingManager().GetKeys();
        log.Debug("InternalGetErrors() - getting admin errors.");
        while (enumTemp.MoveNext())
        {
          sErrorAS = null;
          error = null;
          sKey = (String) enumTemp.GetCurrent();
          log.Debug("InternalGetErrors() - Got something.");
          try
          {
            error = (UIError) subApp.GetCachingManager().GetEntry(sKey);
          }
          catch (Exception ex)
          {
            log.Error(ex, "InternalGetErrors() - A non UIError was stored in the error subapplication.");
            }
          if (error != null)
          {
            if (bForThisAs)
            {
              if (sASName != null)
              {
                sErrorAS = error.GetActivitySpace();
                if (sErrorAS != null)
                {
                  if (sErrorAS.Equals(sASName))
                  {
                    alTemp.Add(error);
                    if (error.GetImmediateClear())
                    {
                      ClearError(error, sessionManager, application);
                    }
                  }
                } else
                {
                  //   this error is not associated with an activity space.
                  //   we'll just add it.
                  alTemp.Add(error);
                  if (error.GetImmediateClear())
                  {
                    ClearError(error, sessionManager, application);
                  }
                }
              }
            } else
            {
              alTemp.Add(error);
            }
          }
        }
      }
      // error != null
      //   for each application key
      //   get admin errors
      log.Debug("InternalGetErrors() - alTemp.GetSize(): " + alTemp.GetSize());
      return alTemp.GetEnumerator();
    }
    
    /// <summary>
    /// Call this method to store the number of errors requiring their own display
    /// page.
    /// <br /><br />
    /// </summary>
    /// <param name="i">int count</param>
    /// <param name="sessionManager">Pass the current user's ISessionManager</param>
    /// <param name="application">Pass the IApplication</param>
    /// <param name="scope">UIErrorScope</param>
    private static void PutNumErrorsRequiringErrorPage(int i, ISessionManager sessionManager,
     IApplication application, UIErrorScope scope)
    {
      log.Debug("Entering PutNumErrorsRequiringErrorPage() - i: " + i);
      log.Debug("PutNumErrorsRequiringErrorPage() - scope: " + scope);
      if (scope == UIErrorScope.CURRENT_USER)
      {
        if (sessionManager != null)
        {
          ISessionManager subSession = sessionManager.GetSubSession(sMetaDataSubSessionKey);
          subSession.SetAttribute(UIErrorHandler.sErrorPageErrorsCountKey,
           i);
        }
      } else
      if (scope == UIErrorScope.ADMIN_USERS)
      {
        if (application != null)
        {
          IApplication subApp;
          subApp = application.GetSubApplication(UIErrorHandler.sMetaDataAdminSubApplicationKey);
          subApp.GetCachingManager().SetEntry(UIErrorHandler.sErrorPageErrorsCountKey,
           i);
        }
      } else
      if (scope == UIErrorScope.ALL_USERS)
      {
        if (application != null)
        {
          IApplication subApp;
          subApp = application.GetSubApplication(UIErrorHandler.sMetaDataSubApplicationKey);
          subApp.GetCachingManager().SetEntry(UIErrorHandler.sErrorPageErrorsCountKey,
           i);
        }
      }
    }
  }
}