using System;
// Framework Imports
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.arraywrapper;
using com.plumtree.uiinfrastructure.common;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.uiinfrastructure.form.pagination;
using com.plumtree.xpshared.htmlconstructs;
namespace com.plumtree.uiinfrastructure.expandablelist
{
  /// <summary>
  /// This class contains the state of the Expandable List. The data contained in
  /// that Model are printed out by  the associated View: ExpListView. The repost
  /// actions will be  handled by the ExpListControl class.  That class extends
  /// the Editor Framework plug-in section  classes so it can be added to any
  /// Editor page pretty easily. (see template 7)  The Expandable List Framework
  /// is abstract and has to be  extended in order to be used.  Note that the
  /// Expandable List uses the Array Wrapper  Framework to actually store the
  /// data and facilitate the  implementation of the sorting requirements.
  /// </summary>
  /// <author>Julie Fournier</author>
  public abstract class ExpListModel : EditorHelperModel,IExpListModelRO,
  IPaginationModel
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.expandablelist.ExpListModel");
    
    /// <summary>
    /// Key (as an Integer) to get back the column ID  from m_arrExpListColInfo
    /// </summary>
    public const int EXPLIST_COLUMN_ID = 0;
    
    /// <summary>
    /// Key (as a String) to get back the column title  from m_arrExpListColInfo
    /// </summary>
    public const int EXPLIST_COLUMN_TITLE = 1;
    
    /// <summary>
    /// Key (as a String) to get back the column width  from m_arrExpListColInfo
    /// </summary>
    public const int EXPLIST_COLUMN_WIDTH = 2;
    
    /// <summary>
    /// Colspan for the HTML cell. 
    /// </summary>
    public const int EXPLIST_COLUMN_COLSPAN = 3;
    
    /// <summary>
    /// Flag to know if the link to sort by that column should be enabled. 
    /// </summary>
    public const int EXPLIST_COLUMN_ENABLESORT = 4;
    
    /// <summary>
    /// MES- Flag to know how many flags there are- how many columns 
    /// </summary>
    public const int EXPLIST_COLUMN_INVALID = 5;
    
    /// <summary>
    /// We are defining here a column ID for the Growable Sorted Array objects.
    /// This column has to reflect the position of the row in the list. Methods
    /// to implement the move up, down, first and last actions are already
    /// implemented in this class and are ready to be used. However they will
    /// work only if the column ID below exists in the object array.
    /// </summary>
    public const int EXPLIST_SORTEDARRAY_PROPID_INDEX = 0;
    
    /// <summary>
    /// Will move the corresponding item to the top of the list. 
    /// </summary>
    public const int MOVEACTION_FIRST = 1;
    
    /// <summary>
    /// Will move the corresponding item one row up. 
    /// </summary>
    public const int MOVEACTION_UP = 2;
    
    /// <summary>
    /// Will move the corresponding item one row down. 
    /// </summary>
    public const int MOVEACTION_DOWN = 3;
    
    /// <summary>
    /// Will move the corres ponding item to the bottom of the list. 
    /// </summary>
    public const int MOVEACTION_LAST = 4;
    
    /// <summary>
    /// To keep track of the next available index.  This index is incremented
    /// each time a row is added  to the list.
    /// </summary>
    public int m_nNextIndex = 0;
    
    /// <summary>
    /// Associated AS class 
    /// </summary>
    protected AActivitySpace parent;
    
    /// <summary>
    /// This is where we store the data displayed on the list. We are taking
    /// advantage of the sorting features offered  by the Array Wrapper
    /// framework.
    /// </summary>
    protected PTGrowableSortedArrayWrapper m_saData;
    
    /// <summary>
    /// JF- Wrap attribute for the column headers
    /// </summary>
    protected bool m_bWrapColumnHeaders = true;
    
    /// <summary>
    /// The add icon. 
    /// </summary>
    protected PTImageType m_imgAddIcon;
    
    /// <summary>
    /// The delete icon. 
    /// </summary>
    protected PTImageType m_imgDeleteIcon;
    
    /// <summary>
    /// Alt text for the add icon. 
    /// </summary>
    protected String m_sAddIconAlt;
    
    /// <summary>
    /// Text to display next to the add icon. 
    /// </summary>
    protected String m_sAddLabel;
    
    /// <summary>
    /// Alt text for the delete icon. 
    /// </summary>
    protected String m_sDeleteIconAlt;
    
    /// <summary>
    /// Message to display as a JavaScript alert if the list is empty. 
    /// </summary>
    protected String m_sEmptyListAlert;
    
    /// <summary>
    /// Instructions to display when the list is empty. 
    /// </summary>
    protected String m_sInstructions = "";
    
    /// <summary>
    /// Main message to display when the list is empty (in red). 
    /// </summary>
    protected String m_sMainMessage = "";
    
    /// <summary>
    /// The section main title. 
    /// </summary>
    protected String m_sSectionTitle = "";
    
    /// <summary>
    /// Array of column titles, associated column IDs and column width 
    /// </summary>
    protected Object[][] m_arrExpListColInfo;
    
    /// <summary>
    /// DCA - Variable to show add button or not.
    /// </summary>
    protected bool m_bAddButtonDisabled = false;
    
    /// <summary>
    /// DCA - Variable to show check boxes or not.
    /// </summary>
    protected bool m_bCheckBoxesDisabled = false;
    
    /// <summary>
    /// Set this flag to true if you want to display the move up and  down
    /// buttons at the end of each row.  Warning: if this is set to true, then
    /// the sort by column will  NOT be available anymore. You can't have both
    /// it doesn't make sense.
    /// </summary>
    protected bool m_bEnableMoveRowActions = false;
    
    /// <summary>
    /// Special mode to have a read-only expandable list: all actions are hidden.
    /// </summary>
    protected bool m_bIsReadOnly;
    
    /// <summary>
    /// JF- List instructions.
    /// </summary>
    protected String m_sTableInstructions;
    
    /// <summary>
    /// JF- COnfirmation message to display when the user deletes a row.
    /// </summary>
    protected String m_sDelRowConfirmMsg;
    
    /// <summary>
    /// This key is used when there is more than one expandable  list on the
    /// same Display Page.
    /// </summary>
    protected int m_nListKey;
    
    /// <summary>
    /// Will be called each time the add action is performed. Should take care
    /// of adding the row(s) to the list. Use the GetItems() method to get the
    /// list.
    /// </summary>
    /// <param name="_htFormData">Form data</param>
    public abstract void AddRowsOnRepost(XPHashtable _htFormData);
    
    /// <summary>
    /// Gets the array of checked rows and delete  them from the list.
    /// </summary>
    /// <param name="_htFormData"></param>
    public virtual void DeleteRowsOnRepost(XPHashtable _htFormData)
    {
      if (!m_bIsReadOnly)
      {
        String sDataKey = GetUniqueString(ExpListView.HTMLCHECKBOX_DELROW);
        // JF- We get back the list of index of rows to delete.
        if (_htFormData.ContainsKey(sDataKey))
        {
          int[] aRowsToDelete = GetArrayOfObjToDelete(_htFormData);
          // JF- We can only delete rows if the array is not empty
          if (m_saData.GetCount() > 0)
          {
            m_saData.RemoveRows(aRowsToDelete);
          }
        }
      }
    }
    
    /// <summary>
    /// DCA - Choose not to display add button
    /// </summary>
    public virtual void DisableAddButton()
    {
      m_bAddButtonDisabled = true;
    }
    
    /// <summary>
    /// DCA - Disable check boxes.
    /// </summary>
    public virtual void DisableCheckBoxes()
    {
      m_bCheckBoxesDisabled = true;
    }
    
    /// <summary>
    /// JF- Sets the wrap cell attribute for the column headers.
    /// Is true by default.
    /// </summary>
    /// <param name="_bWrapColumnHeaders"></param>
    public virtual void SetWrapColumnHeaders(bool _bWrapColumnHeaders)
    {
      m_bWrapColumnHeaders = _bWrapColumnHeaders;
    }
    
    /// <summary>
    /// JF- Returns the wrap attribute for the column headers.
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool GetWrapColumnHeaders()
    {
      return m_bWrapColumnHeaders;
    }
    
    /// <summary>
    /// JF- Returns table intructions
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetTableInstructions()
    {
      return m_sTableInstructions;
    }
    
    /// <summary>
    /// JF- Sets table intructions.
    /// </summary>
    /// <param name="_sTableInstructions"></param>
    public virtual void SetTableInstructions(String _sTableInstructions)
    {
      m_sTableInstructions = _sTableInstructions;
    }
    
    /// <summary>
    /// JF- Returns the confirmation message to display when the user tries to delete a row.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetDelRowConfirmMsg()
    {
      return m_sDelRowConfirmMsg;
    }
    
    /// <summary>
    /// JF- Sets the confirmation message to display when the user tried to delete a row.
    /// </summary>
    /// <param name="m_sDelRowConfirmMsg"></param>
    public virtual void SetDelRowConfirmMsg(String _sDelRowConfirmMsg)
    {
      m_sDelRowConfirmMsg = _sDelRowConfirmMsg;
    }
    
    /// <summary>
    /// Returns icon for the add action.
    /// </summary>
    /// <returns>PTImageType</returns>
    public virtual PTImageType GetAddIcon()
    {
      return m_imgAddIcon;
    }
    
    /// <summary>
    /// Returns the alt text for the add icon.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetAddIconAlt()
    {
      return m_sAddIconAlt;
    }
    
    /// <summary>
    /// Returns the String to display next to the add icon.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetAddLabel()
    {
      return m_sAddLabel;
    }
    
    /// <summary>
    /// Returns the message to display by the JavaScript when the list is empty.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetAlertMessageForEmptyList()
    {
      return m_sEmptyListAlert;
    }
    
    /// <summary>
    /// Returns the array of column titles,  associated column IDs and column
    /// width.
    /// </summary>
    /// <returns>Object[][]</returns>
    public virtual Object[][] GetArrExpListColInfo()
    {
      return m_arrExpListColInfo;
    }
    
    /// <summary>
    /// ROB: Make the available for other models.
    /// </summary>
    /// <param name="_htFormData"></param>
    /// <returns>int[]</returns>
    public virtual int[] GetArrayOfObjToDelete(XPHashtable _htFormData)
    {
      String sDataKey = GetUniqueString(ExpListView.HTMLCHECKBOX_DELROW);
      String[] aDataValue = (String[]) _htFormData.GetElement(sDataKey);
      int[] aRowsToDelete = new int[aDataValue.Length];
      int nRowIndex;
      for (int i = 0;i < aDataValue.Length;i++)
      {
        try
        {
          nRowIndex = XPConvert.ToInteger(aDataValue[i]);
          if (nRowIndex < m_saData.GetCount())
          {
            aRowsToDelete[i] = nRowIndex;
          }
        }
        catch (Exception e)
        {
          log.Error("ExpListModel::GetArrayOfObjToDelete Unbale to retrieve row index.");
          }
      }
      return aRowsToDelete;
    }
    
    /// <summary>
    /// Returns icon for the delete action.
    /// </summary>
    /// <returns>PTImageType</returns>
    public virtual PTImageType GetDeleteIcon()
    {
      return m_imgDeleteIcon;
    }
    
    /// <summary>
    /// Returns the alt text for the delete icon.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetDeleteIconAlt()
    {
      return m_sDeleteIconAlt;
    }
    
    /// <summary>
    /// Returns the secondary String to display when the String is empty.
    /// (Should give instructions to the user on what to do).
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetEmptyListInstructions()
    {
      return m_sInstructions;
    }
    
    /// <summary>
    /// Returns the main String to diplay when the list is empty.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetEmptyListMainMessage()
    {
      return m_sMainMessage;
    }
    
    /// <summary>
    /// Returns true if the move up and down buttons should be displayed. (use
    /// GetHTMLForMoveRowActions in ExpListView to dsiplay the HTMl  on each
    /// row)
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool GetEnableMoveRowActions()
    {
      return m_bEnableMoveRowActions;
    }
    
    /// <summary>
    /// Returns the value of the m_bIsReadOnly member variable.
    /// If this is true, the user will not be able to perform any action on the list.
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool GetIsListReadOnly()
    {
      return m_bIsReadOnly;
    }
    
    /// <summary>
    /// Returns the array of data.
    /// </summary>
    /// <returns>PTGrowableSortedArrayWrapper</returns>
    public virtual PTGrowableSortedArrayWrapper GetItems()
    {
      return m_saData;
    }
    
    /// <summary>
    /// Returns the key of the expandable list. (This key is used to avoid
    /// conflicts when  there are multiple lists on the same page.)
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetListKey()
    {
      return m_nListKey;
    }
    
    /// <summary>
    /// Sets a new value for this list key.
    /// </summary>
    /// <param name="_nListKey"></param>
    public virtual void SetListKey(int _nListKey)
    {
      m_nListKey = _nListKey;
    }
    
    /// <summary>
    /// Returns the column ID of the primary sorted column. (the array can be
    /// sorted by multiple columns)
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetMainColumnSort()
    {
      int[][] arrSort = m_saData.GetSort();
      return arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_PROPID];
    }
    
    /// <summary>
    /// Returns the sort direction of the primary sorted column.
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetMainColumnSortDirection()
    {
      int[][] arrSort = m_saData.GetSort();
      return arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION];
    }
    
    /// <summary>
    /// Returns a read-only interface of the data.
    /// </summary>
    /// <returns>IPTGrowableSortedArrayWrapperRO</returns>
    public virtual IPTGrowableSortedArrayWrapperRO GetReadOnlyItems()
    {
      return (IPTGrowableSortedArrayWrapperRO) m_saData;
    }
    
    /// <summary>
    /// Returns the section title.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetSectionTitle()
    {
      return m_sSectionTitle;
    }
    
    /// <summary>
    /// Creates dynamically a unique int and returns it. This method can be used
    /// for repost actions for example. This is to make sure that if there are
    /// more than one expandable  list on the same dislpay page, there won't be
    /// any conflict  between the two lists.
    /// </summary>
    /// <param name="_nValue"></param>
    /// <returns>int</returns>
    public virtual int GetUniqueInt(int _nValue)
    {
      return XPConvert.ToInteger(XPConvert.ToString(_nValue) + XPConvert.ToString(m_nListKey));
      }
    
    /// <summary>
    /// Creates dynamically a unique name with the list key and returns it. This
    /// is to make sure that if there are more than one expandable  list on the
    /// same dislpay page, there won't be any conflict  between the two lists.
    /// This method should be used to create unique input names.  It can also
    /// be used for JavaScript function names (other than  postToSelf - for
    /// this one you just need to use the GetUniqueInt  method to make sure
    /// your repost actions and other int values  are unique).
    /// </summary>
    /// <param name="_sName"></param>
    /// <returns>String</returns>
    public virtual String GetUniqueString(String _sName)
    {
      return (_sName + m_nListKey);
    }
    
    /// <summary>
    /// Initializes the expandable list.
    /// </summary>
    /// <param name="_sSectionTitle">Main title.</param>
    /// <param name="_arrExpListColInfo">
    /// Information about columns: title, ID and
    /// width.
    /// </param>
    /// <param name="_imgAddIcon">Icon for the add action.</param>
    /// <param name="_sAddLabel">Text to display next to the add icon.</param>
    /// <param name="_nListKey">
    /// In case you have multiple Expandable Lists on the same
    /// DP, they should have a different key (otherwise, just set it to
    /// 1).
    /// </param>
    public virtual void InitializeExpandableList(String _sSectionTitle, Object[][] _arrExpListColInfo,
     PTImageType _imgAddIcon, String _sAddLabel, int _nListKey)
    {
      if (5 > _arrExpListColInfo[0].Length)
      {
        throw new XPException("ExpListModel::InitializeExpandableList _arrExpListColInfo doesn\'t have 5 columns.");
        }
      m_sSectionTitle = _sSectionTitle;
      m_arrExpListColInfo = _arrExpListColInfo;
      m_nListKey = _nListKey;
      m_imgAddIcon = _imgAddIcon;
      m_sAddLabel = _sAddLabel;
      m_saData = new PTGrowableSortedArrayWrapper();
      m_imgDeleteIcon = PTImageType.ACTION_REMOVE;
      m_sAddIconAlt = "";
      m_sDeleteIconAlt = "";
      InitializeSortedArray(m_saData);
      // Register Pagination Controls
      if (!m_asOwner.GetControlExists(PaginationControl.STR_MVC_CLASS_NAME))
      {
        m_asOwner.RegisterControl(PaginationControl.STR_MVC_CLASS_NAME, this);
        }
      if (!m_asOwner.GetControlExists(PassthruControl.STR_MVC_CLASS_NAME))
      {
        m_asOwner.RegisterControl(PassthruControl.STR_MVC_CLASS_NAME, this);
        }
    }
    
    /// <summary>
    /// Initializes the read-only expandable list.
    /// </summary>
    /// <param name="_sSectionTitle">Main title.</param>
    /// <param name="_arrExpListColInfo">
    /// Information about columns: title, ID and
    /// width.
    /// </param>
    /// <param name="_nListKey">
    /// In case you have multiple Expandable Lists on the same
    /// DP, they should have a different key (otherwise, just set it to
    /// 1).
    /// </param>
    public virtual void InitializeReadOnlyExpandableList(String _sSectionTitle,
     Object[][] _arrExpListColInfo, int _nListKey)
    {
      if (5 > _arrExpListColInfo[0].Length)
      {
        throw new XPException("ExpListModel::InitializeExpandableList _arrExpListColInfo doesn\'t have 5 columns.");
        }
      m_sSectionTitle = _sSectionTitle;
      m_arrExpListColInfo = _arrExpListColInfo;
      m_nListKey = _nListKey;
      m_bIsReadOnly = true;
      m_saData = new PTGrowableSortedArrayWrapper();
      m_imgDeleteIcon = PTImageType.ACTION_REMOVE;
      m_sAddIconAlt = "";
      m_sDeleteIconAlt = "";
      InitializeSortedArray(m_saData);
    }
    
    /// <summary>
    /// Will be called when this class is initalized.  Should take care of
    /// initializing _saData (calling Initialize).
    /// </summary>
    /// <param name="_saData"></param>
    public abstract void InitializeSortedArray(PTGrowableSortedArrayWrapper _saData);
    
    /// <summary>
    /// DCA - Check if add button is to be displayed
    /// </summary>
    public virtual bool IsAddButtonDisabled()
    {
      return m_bAddButtonDisabled;
    }
    
    /// <summary>
    /// DCA - Check to see if check boxes has been disabled.
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool IsCheckBoxesDisabled()
    {
      return m_bCheckBoxesDisabled;
    }
    
    /// <summary>
    /// Action for modifying a row
    /// </summary>
    /// <param name="_htFormData"></param>
    public virtual void ModifyRow(XPHashtable _htFormData)
    {
    }
    // This should be implemented by child classes if it is needed.
    
    /// <summary>
    /// Gets the row index back and the move action back. Then perform the
    /// associated action.
    /// </summary>
    /// <param name="_htFormData"></param>
    public virtual void MoveRow(XPHashtable _htFormData)
    {
      // JF- The move rows action is supported only if the EXPLIST_SORTEDARRAY_INDEX 
      // column is in the object array.
      if (m_saData.ContainsPropID(EXPLIST_SORTEDARRAY_PROPID_INDEX))
      {
        int nRowIndex = GetRowIndex(_htFormData);
        int nMoveAction = GetMoveAction(_htFormData);
        log.Debug("ExpListModel::MoveRow The move action value is: " + nMoveAction);
        if ((-1 != nRowIndex) && (-1 != nMoveAction))
        {
          switch (nMoveAction){
          case MOVEACTION_FIRST:
          MoveFirst(nRowIndex);
          break;
          case MOVEACTION_UP:
          MoveUp(nRowIndex);
          break;
          case MOVEACTION_DOWN:
          MoveDown(nRowIndex);
          break;
          case MOVEACTION_LAST:
          MoveLast(nRowIndex);
          break;
          default:
          log.Warn("ExpListModel::MoveRow Move action: " + nMoveAction + " is invalid.");
          break;
          }
        }
      }
    }
    
    /// <summary>
    /// Reverses the sort of the primary sorted column.
    /// </summary>
    public virtual void ReverseCurrentSort()
    {
      int[][] arrSort = m_saData.GetSort();
      // JF- Get the current sort.
      int nCurrentDirection = arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION];
      // JF- 
      if (PTSortedArrayWrapper.PTSAW_SORT_ASCENDING == nCurrentDirection)
      {
        arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION] = PTSortedArrayWrapper.PTSAW_SORT_DESCENDING;
        } else
      {
        arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION] = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;
        }
      m_saData.Sort(arrSort);
    }
    
    /// <summary>
    /// Saves the data when a repost occurs. Editors using this framework need
    /// to call that method from  the savePage method of the main Model.
    /// </summary>
    /// <param name="_htFormData"></param>
    /// <returns>int Status for server side validation</returns>
    public override int SaveSection(XPHashtable _htFormData)
    {
      int nSaveStatus = RepostControl.PAGE_STATUS_VALID;
      // JF- For each row in the list we call the abstract saveRow method.
      for (int nRowIndex = 0;nRowIndex < m_saData.GetCount();nRowIndex++)
      {
        if (RepostControl.PAGE_STATUS_INVALID == SaveRow(m_saData, nRowIndex,
         _htFormData))
        {
          nSaveStatus = RepostControl.PAGE_STATUS_INVALID;
        }
      }
      return nSaveStatus;
    }
    
    /// <summary>
    /// The delete icon will be set by default to add.gif.  Use that method
    /// if you want to change it.
    /// </summary>
    /// <param name="__imgAddIcon"></param>
    public virtual void SetAddIcon(PTImageType _imgAddIcon)
    {
      m_imgAddIcon = _imgAddIcon;
    }
    
    /// <summary>
    /// Sets the value of the alt text for the add icon. This is optional, by
    /// default this is set to empty String.
    /// </summary>
    /// <param name="_sAddIconAlt"></param>
    public virtual void SetAddIconAlt(String _sAddIconAlt)
    {
      m_sAddIconAlt = _sAddIconAlt;
    }
    
    /// <summary>
    /// The delete icon will be set by default to delete.gif.  Use that method
    /// if you want to change it.
    /// </summary>
    /// <param name="_imgDeleteIcon"></param>
    public virtual void SetDeleteIcon(PTImageType _imgDeleteIcon)
    {
      m_imgDeleteIcon = _imgDeleteIcon;
    }
    
    /// <summary>
    /// Sets the value of the alt text for the delete icon. This is optional, by
    /// default this is set to empty String.
    /// </summary>
    /// <param name="_sDeleteIconAlt"></param>
    public virtual void SetDeleteIconAlt(String _sDeleteIconAlt)
    {
      m_sDeleteIconAlt = _sDeleteIconAlt;
    }
    
    /// <summary>
    /// Sets messages to display when the list is empty.
    /// </summary>
    /// <param name="_sMainMessage">Main message to display in red.</param>
    /// <param name="_sInstructions">Instructions on what to do.</param>
    public virtual void SetEmptyListMessages(String _sMainMessage, String _sInstructions)
    {
      m_sMainMessage = _sMainMessage;
      m_sInstructions = _sInstructions;
    }
    
    /// <summary>
    /// Enable the JavaScript validation to ensure that the list is not empty.
    /// The String in argument will be displayed by the JavaScript alert. And
    /// the String returned is the JavaScript name that should be  registered
    /// for validation in the Editor. The name returned will be  prefixed with
    /// the Expandable List key to avoid conflicts.
    /// </summary>
    /// <param name="_sEmptyListAlert"></param>
    /// <returns>String JavaScript function name</returns>
    public virtual String SetEnableEmptyListClientValidation(String _sEmptyListAlert)
    {
      m_sEmptyListAlert = _sEmptyListAlert;
      return GetUniqueString("CheckEmptyList");
    }
    
    /// <summary>
    /// Set this flag to true if you want to display the move up  and down
    /// buttons at the end of each row.  Warning: if this is set to true, then
    /// the sort  by column will NOT be available anymore.
    /// </summary>
    /// <param name="_bEnableMoveRowActions"></param>
    public virtual void SetEnableMoveRowActions(bool _bEnableMoveRowActions)
    {
      m_bEnableMoveRowActions = _bEnableMoveRowActions;
      // JF- Disable the sort on each column
      if (m_bEnableMoveRowActions)
      {
        for (int i = 0;i < m_arrExpListColInfo.Length;i++)
        {
          m_arrExpListColInfo[i][EXPLIST_COLUMN_ENABLESORT] = false;
        }
      }
      // JF- Resort the array by index
      int[][] arrSort = new int[1][];
      for (int j2c_index = 0;j2c_index < 1;j2c_index++)
      {
        arrSort[j2c_index] = new int[2];
      }
      arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_PROPID] = EXPLIST_SORTEDARRAY_PROPID_INDEX;
      arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION] = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;
      m_saData.Sort(arrSort);
    }
    
    /// <summary>
    /// Changes the value of the m_bIsReadOnly member variable.
    /// If the list is read only, no action will be displayed in the action bar.
    /// </summary>
    /// <param name="_bIsReadOnly"></param>
    public virtual void SetIsListReadOnly(bool _bIsReadOnly)
    {
      m_bIsReadOnly = _bIsReadOnly;
    }
    
    /// <summary>
    /// Sets the section title.
    /// </summary>
    /// <param name="_sSectionTitle"></param>
    public virtual void SetSectionTitle(String _sSectionTitle)
    {
      m_sSectionTitle = _sSectionTitle;
    }
    
    /// <summary>
    /// Gets the column ID back from the form data and resort  the array by that
    /// column ID, ascending.
    /// </summary>
    /// <param name="_htFormData"></param>
    public virtual void SortByColumn(XPHashtable _htFormData)
    {
      String sDataKey = GetUniqueString(ExpListView.HTMLHIDDEN_COLUMNID);
      // JF- First we need to get back the column ID from the form data.
      if (_htFormData.ContainsKey(sDataKey))
      {
        String[] aDataValue = (String[]) _htFormData.GetElement(sDataKey);
        if ((null != aDataValue) && (0 != aDataValue.Length))
        {
          try
          {
            int nColumnID = XPConvert.ToInteger(aDataValue[0]);
            // JF- Then we get back the sort and update and modify the 1st row.
            // If the array was sorted by more than 1 column we want to keep the other columns.
            int[][] arrSort = m_saData.GetSort();
            arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_PROPID] = nColumnID;
            arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION] = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;
            // JF- Finally we resort the array
            m_saData.Sort(arrSort);
          }
          catch (Exception e)
          {
            log.Error("ExpListModel::SortByColumn An exception occured" + " when trying to resort the array by column with ID: " + aDataValue[0]);
            }
        }
      }
    }
    
    /// <summary>
    /// Helper method to retrieve the move action type from the form data.
    /// </summary>
    /// <param name="_htFormData"></param>
    /// <returns>int</returns>
    protected virtual int GetMoveAction(XPHashtable _htFormData)
    {
      int nMoveAction = -1;
      String sDataKey = GetUniqueString(ExpListView.HTMLHIDDEN_MOVEACTION);
      if (_htFormData.ContainsKey(sDataKey))
      {
        String[] aDataValue = (String[]) _htFormData.GetElement(sDataKey);
        if ((null != aDataValue) && (0 != aDataValue.Length))
        {
          try
          {
            nMoveAction = XPConvert.ToInteger(aDataValue[0]);
          }
          catch (Exception e)
          {
            log.Error("ExpListModel::GetMoveAction An exception occured" + " when trying to get the move action back from the form data. -1 will be returned instead.");
            }
        }
      }
      return nMoveAction;
    }
    
    /// <summary>
    /// Helper method to retrieve the row index from the form data.
    /// </summary>
    /// <param name="_htFormData"></param>
    /// <returns>int</returns>
    protected virtual int GetRowIndex(XPHashtable _htFormData)
    {
      int nRowIndex = -1;
      String sDataKey = GetUniqueString(ExpListView.HTMLHIDDEN_ROWINDEX);
      if (_htFormData.ContainsKey(sDataKey))
      {
        String[] aDataValue = (String[]) _htFormData.GetElement(sDataKey);
        if ((null != aDataValue) && (0 != aDataValue.Length))
        {
          try
          {
            nRowIndex = XPConvert.ToInteger(aDataValue[0]);
          }
          catch (Exception e)
          {
            log.Error("ExpListModel::GetRowIndex An exception occured" + " when trying to get the row index back from the form data. -1 will be returned instead.");
            }
        }
      }
      return nRowIndex;
    }
    
    /// <summary>
    /// Moves the row identified by the index in argument, one row down. The
    /// idea is that we are going to change the value of the index column.  We
    /// just need to swap that value with the one of the row just after.
    /// </summary>
    /// <param name="_nRowIndex">Index of the row to move.</param>
    protected virtual void MoveDown(int _nRowIndex)
    {
      // Make sure the row is not already in last position.
      if (m_saData.GetCount() > (_nRowIndex + 1))
      {
        try
        {
          // JF- We get back the index values of the row to move and the one just before.
          Object oCurrentRow = m_saData.GetItem(_nRowIndex, EXPLIST_SORTEDARRAY_PROPID_INDEX);
          Object oNextRow = m_saData.GetItem(_nRowIndex + 1, EXPLIST_SORTEDARRAY_PROPID_INDEX);
          Object[][] arrNewItem = new Object[2][];
          for (int j2c_index = 0;j2c_index < 2;j2c_index++)
          {
            arrNewItem[j2c_index] = new Object[3];
          }
          arrNewItem[0][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_INDEX] = _nRowIndex;
          arrNewItem[0][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_PROPERTYID] = EXPLIST_SORTEDARRAY_PROPID_INDEX;
          arrNewItem[0][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_NEWVALUE] = oNextRow;
          arrNewItem[1][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_INDEX] = _nRowIndex + 1;
          arrNewItem[1][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_PROPERTYID] = EXPLIST_SORTEDARRAY_PROPID_INDEX;
          arrNewItem[1][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_NEWVALUE] = oCurrentRow;
          // JF- Then we swap the values.
          m_saData.SetItems(arrNewItem);
        }
        // JF- The object array is sorted automatically when a value is set.
        catch (Exception e)
        {
          log.Error("ExpListModel::MoveDown An exception occured" + " when trying to move down the row: " + _nRowIndex + ". " + e);
          }
      }
    }
    
    /// <summary>
    /// Moves the row identified by the index in argument,  to the top of the
    /// list.
    /// </summary>
    /// <param name="_nRowIndex">Index of the row to move.</param>
    protected virtual void MoveFirst(int _nRowIndex)
    {
      // Make sure the row is not already in first position.
      if (0 != _nRowIndex)
      {
        try
        {
          Object[][] arrNewItem = new Object[_nRowIndex + 1][];
          for (int j2c_index = 0;j2c_index < _nRowIndex + 1;j2c_index++)
          {
            arrNewItem[j2c_index] = new Object[3];
          }
          arrNewItem[0][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_INDEX] = _nRowIndex;
          arrNewItem[0][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_PROPERTYID] = EXPLIST_SORTEDARRAY_PROPID_INDEX;
          arrNewItem[0][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_NEWVALUE] = 0;
          int i = 1;
          // JF- Then, one by one, we move the row to the top.
          while (i <= _nRowIndex)
          {
            arrNewItem[i][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_INDEX] = i - 1;
            arrNewItem[i][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_PROPERTYID] = EXPLIST_SORTEDARRAY_PROPID_INDEX;
            arrNewItem[i][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_NEWVALUE] = i;
            i++;
          }
          m_saData.SetItems(arrNewItem);
        }
        // JF- The object array is sorted automatically when a value is set.
        catch (Exception e)
        {
          log.Error("ExpListModel::MoveFirst An exception occured" + " when trying to move first the row: " + _nRowIndex + ". " + e);
          }
      }
    }
    
    /// <summary>
    /// Moves the row identified by the index in argument,  to the last position
    /// of the list.
    /// </summary>
    /// <param name="_nRowIndex">Index of the row to move.</param>
    protected virtual void MoveLast(int _nRowIndex)
    {
      int nCount = m_saData.GetCount();
      // Make sure the row is not already in last position.
      if (nCount > (_nRowIndex + 1))
      {
        try
        {
          Object[][] arrNewItem = new Object[nCount - _nRowIndex][];
          for (int j2c_index = 0;j2c_index < nCount - _nRowIndex;j2c_index++)
          {
            arrNewItem[j2c_index] = new Object[3];
          }
          arrNewItem[0][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_INDEX] = _nRowIndex;
          arrNewItem[0][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_PROPERTYID] = EXPLIST_SORTEDARRAY_PROPID_INDEX;
          arrNewItem[0][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_NEWVALUE] = nCount - 1;
          int i = _nRowIndex + 1;
          // JF- Then, one by one, we move the row to the bottom.
          while (i < nCount)
          {
            arrNewItem[i - _nRowIndex][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_INDEX] = i;
            arrNewItem[i - _nRowIndex][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_PROPERTYID] = EXPLIST_SORTEDARRAY_PROPID_INDEX;
            arrNewItem[i - _nRowIndex][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_NEWVALUE] = i - 1;
            i++;
          }
          m_saData.SetItems(arrNewItem);
        }
        // JF- The object array is sorted automatically when a value is set.
        catch (Exception e)
        {
          log.Error("ExpListModel::MoveLast An exception occured" + " when trying to move last the row: " + _nRowIndex + ". " + e);
          }
      }
    }
    
    /// <summary>
    /// Moves the row identified by the index in argument, one row up. The idea
    /// is that we are going to change the value of the index column.  We just
    /// need to swap that value with the one of the previous row.
    /// </summary>
    /// <param name="_nRowIndex">Index of the row to move.</param>
    protected virtual void MoveUp(int _nRowIndex)
    {
      // Make sure the row is not already in first position.
      if (0 != _nRowIndex)
      {
        try
        {
          // JF- We get back the index values of the row to move and the one just before.
          Object oCurrentRow = m_saData.GetItem(_nRowIndex, EXPLIST_SORTEDARRAY_PROPID_INDEX);
          Object oPreviousRow = m_saData.GetItem(_nRowIndex - 1, EXPLIST_SORTEDARRAY_PROPID_INDEX);
          Object[][] arrNewItem = new Object[2][];
          for (int j2c_index = 0;j2c_index < 2;j2c_index++)
          {
            arrNewItem[j2c_index] = new Object[3];
          }
          arrNewItem[0][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_INDEX] = _nRowIndex;
          arrNewItem[0][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_PROPERTYID] = EXPLIST_SORTEDARRAY_PROPID_INDEX;
          arrNewItem[0][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_NEWVALUE] = oPreviousRow;
          arrNewItem[1][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_INDEX] = _nRowIndex - 1;
          arrNewItem[1][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_PROPERTYID] = EXPLIST_SORTEDARRAY_PROPID_INDEX;
          arrNewItem[1][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_NEWVALUE] = oCurrentRow;
          // JF- Then we swap the values.
          m_saData.SetItems(arrNewItem);
        }
        // JF- The object array is sorted automatically when a value is set.
        catch (Exception e)
        {
          log.Error("ExpListModel::MoveUp An exception occured" + " when trying to move up the row: " + _nRowIndex + ". " + e);
          }
      }
    }
    
    /// <summary>
    /// Will be called each ime there is a repost to save  the data on the list.
    /// Should contain the code to update the data on the  row _nRowIndex.Use
    /// the SetItem method to update _saData and  use _htFormData to retrieve
    /// the posted data.
    /// </summary>
    /// <param name="_saData"></param>
    /// <param name="_nRowIndex"></param>
    /// <param name="_htFormData"></param>
    /// <returns>int</returns>
    protected abstract int SaveRow(PTGrowableSortedArrayWrapper _saData, int _nRowIndex,
     XPHashtable _htFormData);
    // ************** Pagination methods *************
    
    /// <summary>
    /// Max number of items per page.
    /// </summary>
    public const int MAX_ROWS = 1000;
    
    /// <summary>
    /// Storage key of the current class (used for pagination). 
    /// </summary>
    private String m_sModelStorageName = null;
    
    /// <summary>
    /// Pagination Current Lower Bound 
    /// </summary>
    private int m_nPaginationLowerBound = -1;
    
    /// <summary>
    /// Pagination Current Upper Bound 
    /// </summary>
    private int m_nPaginationUpperBound = -1;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.form.pagination.IPaginationModel#SetPagination()"></seealso>
    public virtual void SetPagination(int nPaginationID, int nPaginationLowerBound,
     int nPaginationUpperBound)
    {
      if (m_nListKey == nPaginationID)
      {
        m_nPaginationLowerBound = nPaginationLowerBound;
        m_nPaginationUpperBound = nPaginationUpperBound;
      } else
      {
        log.Warn("Trying to set Pagination settings with an invalid Pagination ID: " + nPaginationID + ". The expected pagination ID was: " + m_nListKey);
        }
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.form.pagination.IPaginationModelRO#GetCurrentLowerObjectCount()"></seealso>
    public virtual int GetCurrentLowerObjectCount()
    {
      int nCount = m_nPaginationLowerBound;
      if (m_nPaginationLowerBound > m_saData.GetCount())
      {
        nCount = m_saData.GetCount() - GetPaginationSize() + 1;
      }
      if (m_nPaginationLowerBound < 1)
      {
        nCount = 1;
      }
      return nCount;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.form.pagination.IPaginationModelRO#GetCurrentUpperObjectCount()"></seealso>
    public virtual int GetCurrentUpperObjectCount()
    {
      int nCount = m_saData.GetCount();
      if (IsPaginationEnabled())
      {
        if (nCount > GetPaginationSize())
        {
          nCount = GetPaginationSize();
        }
        if (-1 != m_nPaginationUpperBound)
        {
          if (m_nPaginationUpperBound <= m_saData.GetCount())
          {
            nCount = m_nPaginationUpperBound;
          } else
          {
            nCount = m_saData.GetCount();
          }
        }
      }
      return nCount;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.form.pagination.IPaginationModelRO#GetNumObjects()"></seealso>
    public virtual int GetNumObjects()
    {
      return m_saData.GetCount();
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.form.pagination.IPaginationModelRO#GetPaginationSize()"></seealso>
    public virtual int GetPaginationSize()
    {
      return MAX_ROWS;
    }
    
    /// <summary>
    /// Returns the storage key of this Model.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetModelStorageName()
    {
      return m_sModelStorageName;
    }
    
    /// <summary>
    /// Enables the pagination.
    /// </summary>
    /// <param name="_sModelStorageName">Storage name of the Expandable List Model.</param>
    public virtual void EnablePagination(String _sModelStorageName)
    {
      m_sModelStorageName = _sModelStorageName;
    }
    
    /// <summary>
    /// Returns true if the pagination is enabled for this expandable list.
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool IsPaginationEnabled()
    {
      if (null == m_sModelStorageName)
      {
        return false;
      }
      return true;
    }
    
    /// <returns></returns>
    public virtual XPSortedDictionary GetUnusedProperties()
    {
      throw new XPUnsupportedOperationException();
    }
  }
}