using System;
using com.plumtree.uiinfrastructure.expandablelist.multicolumnlist.sorted;
using com.plumtree.xpshared.htmlconstructs;
namespace com.plumtree.uiinfrastructure.expandablelist.multicolumnlist.sortedmetadata
{
  /// <summary>
  /// This list model allows you to sort by meta data, rather
  /// than the actual HTML displayed, allowing you to add images
  /// and descriptions to row entries without affecting sort order.
  /// This list is currently read only.
  /// </summary>
  /// <author>DonH</author>
  public class SortedMetaDataListModel : SortedMultiColumnListModel
  {
    
    /// <summary>
    /// Class name for the activity space 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "SortedMetaDataList";
    protected int m_nNumVisibleColumns;
    
    /// <summary>
    /// Initializes the expandable meta data list.
    /// This list keeps track of 2 kinds of data, visible data and non-visible data.
    /// Most of the inputs to this method are arrays of the same length as the number
    /// of visible columns (I.E. Columns that display and need a column header).
    /// The exception is the column visibility array, which should be the same length
    /// as the number of total columns (data + visible).
    /// The data in the visible columns will be used to sort the rows, and will be
    /// displayed by default.  This data should be Strings.  Use a custom List View
    /// to display different values.
    /// All the visible columns should come first, and then the data only columns should
    /// come afterwards.  This helps us to sort correctly.
    /// Non-visible column data can be of any object type.
    /// </summary>
    /// <param name="_sListTitle">List title.</param>
    /// <param name="_sColumnHeaders">The titles for each visible column</param>
    /// <param name="_sColumnWidth">
    /// The numeric width for each visible column (not percentage width).
    /// If left null for a column, that column will recieve an equal percentage of the remainder (100%).
    /// </param>
    /// <param name="_bEnableSort">Enable sort for each visible column</param>
    /// <param name="_nDefaultSortColumn">The zero based column id for the default visible column to sort by.</param>
    /// <param name="_nTotalColumsn">The total number of visible and data only columns.</param>
    /// <param name="_nListKey">
    /// In case you have multiple Expandable Lists on the same
    /// DP, they should have a different key (otherwise, just set it to
    /// 1).
    /// </param>
    public virtual void InitializeMetaDataList(String _sListTitle, String[] _sColumnHeaders,
     String[] _sColumnWidth, bool[] _bEnableSort, int _nDefaultSortColumn,
     int _nTotalColumns, int _nListKey)
    {
      // Don't set the list to read only since we want the HTML Action Bar.
      // Disable checkboxes and the add button.  Since we can't disable
      // the delete button directly, views using this model should
      // override the GetHTMLForDeleteAction method.
      // TODO: will the data columns work with form reposts?
      DisableCheckBoxes();
      DisableAddButton();
      // The number of visible columns can be very different from
      // the actual data columns.  This controls how much data
      // the parent class will store.
      SetNumColumns(_nTotalColumns);
      // Visible columns and column headers should match.
      // This controls how many columns the List View will display.
      m_nNumVisibleColumns = _sColumnHeaders.Length;
      // Set the default visible sort column.
      // This needs to happen after storing the data column visibility array.
      // The sort column is zero based.
      m_nDefaultSortColumn = _nDefaultSortColumn - 1;
      int nNumPresetColumnWidths = 0;
      for (int i = 0;i < _sColumnWidth.Length;i++)
      {
        if (null != _sColumnWidth[i])
        {
          nNumPresetColumnWidths++;
        }
      }
      // Set the width based on the number of columns, minus
      // the ones with preset column widths.
      int nColWidth = 100 / (m_nNumVisibleColumns - nNumPresetColumnWidths);
      String strDefaultWidth = nColWidth + "%";
      Object[][] arrExpListColInfo = new Object[m_nNumVisibleColumns][];
      for (int j2c_index = 0;j2c_index < m_nNumVisibleColumns;j2c_index++)
      {
        arrExpListColInfo[j2c_index] = new Object[EXPLIST_COLUMN_INVALID];
        }
      for (int i = 0;i < m_nNumVisibleColumns;i++)
      {
        arrExpListColInfo[i][EXPLIST_COLUMN_ID] = i + 1;
        arrExpListColInfo[i][EXPLIST_COLUMN_TITLE] = _sColumnHeaders[i];
        if (null != _sColumnWidth[i])
        {
          arrExpListColInfo[i][EXPLIST_COLUMN_WIDTH] = _sColumnWidth[i];
        } else
        {
          arrExpListColInfo[i][EXPLIST_COLUMN_WIDTH] = strDefaultWidth;
        }
        arrExpListColInfo[i][EXPLIST_COLUMN_COLSPAN] = "1";
        arrExpListColInfo[i][EXPLIST_COLUMN_ENABLESORT] = _bEnableSort[i];
        }
      base.InitializeExpandableList(_sListTitle, arrExpListColInfo, 
      // Add Icon is not used
      PTImageType.ACTION_ADD, "", _nListKey);
    }
    
    /// <summary>
    /// Helper method to get the width of a column.
    /// </summary>
    /// <param name="_nVisibleColumnIndex">The non-zero based column index.</param>
    /// <returns>String The column width (can be an integer or %).</returns>
    public virtual String GetColumnWidth(int _nVisibleColumnIndex)
    {
      if (_nVisibleColumnIndex >= m_nNumVisibleColumns)
      {
        // TODO: log an error
        return null;
      }
      // We are passed in a non-zero based column index, but the data is stored zero based.
      return (String) m_arrExpListColInfo[_nVisibleColumnIndex - 1][EXPLIST_COLUMN_WIDTH];
      }
    
    /// <summary>
    /// This returns the number of visible columns, as opposed to the
    /// number of data columns.
    /// </summary>
    /// <returns></returns>
    public virtual int GetNumVisibleColumns()
    {
      return m_nNumVisibleColumns;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new SortedMetaDataListModel();
    }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
  }
}