using System;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.form
{
  /// <summary>
  /// This Display Page adds client specified Javascript to the page
  /// and then closes the window.  This is useful for closing pop-ups
  /// after they are finished.
  /// </summary>
  /// <author>Don Hayler</author>
  public class ClosePopUpDP : IDisplayPage
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.form.ClosePopUpDP");
    
    /// <summary>
    /// Object Name 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "ClosePopUp";
    
    /// <summary>
    /// static javascript to close the window 
    /// </summary>
    private const String JAVASCRIPT_CLOSE = "window.close();";
    
    /// <summary>
    /// Javascript to call before closing the window 
    /// </summary>
    private HTMLScript m_Script = null;
    
    /// <summary>
    /// Whether or not this popup should just close 
    /// </summary>
    private bool m_bShouldClose = true;
    
    /// <summary>
    /// The parent activity space 
    /// </summary>
    private AActivitySpace m_asOwner;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IDisplayPage#Display(IWebData)"></seealso>
    public virtual void Display(IWebData webData)
    {
      HTMLPage myPage = new HTMLPage();
      // Set the HTMLPage as the data to be sent to the browser.
      webData.SetData(myPage);
      HTMLHead myHead = new HTMLHead();
      myPage.SetHead(myHead);
      HTMLBody myBody = new HTMLBody();
      myPage.SetBody(myBody);
      HTMLScript closeScript = new HTMLScript("text/javaScript");
      try
      {
        // DCA - add JSPortlet scripts
        HTMLScriptCollection jsxmlScript = new HTMLScriptCollection();
        jsxmlScript.AddInnerHTMLElement(ConfigHelper.GetJSIncluderJSComponentInclude(ConfigHelper.JSCOMPONENT_JSPORTLET,
         GetOwner()));
        myHead.AddInnerHTMLElement(jsxmlScript);
        // Add client defined script
        if (m_Script != null)
        {
          myHead.AddInnerHTMLElement(m_Script);
        }
        // Close the window
        if (m_bShouldClose)
        {
          closeScript.AddInnerHTMLString(JAVASCRIPT_CLOSE);
        }
        myHead.AddInnerHTMLElement(closeScript);
      }
      catch (HTMLException he)
      {
        log.Error(he, "Display: HTML Error while attempting to display the HTML.");
        }
      catch (Exception e)
      {
        log.Error(e, "Display: Error while attempting to display the HTML.");
        }
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public virtual String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IDisplayPage#GetParent()"></seealso>
    public virtual AActivitySpace GetOwner()
    {
      return m_asOwner;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IDisplayPage#Init(AActivitySpace)"></seealso>
    public virtual void Init(AActivitySpace parent)
    {
      m_asOwner = parent;
    }
    
    /// <summary>
    /// Set the script that will be added before the window is closed. This is
    /// where you should call javascript functions on the opening window if
    /// desired.
    /// </summary>
    /// <param name="script">the script to add before closing</param>
    public virtual void SetScript(HTMLScript script)
    {
      m_Script = script;
    }
    
    /// <summary>
    /// This function sets a boolean to check whether the ClosePopUpDP should
    /// actually close.  This is useful for pages that redirect and would like
    /// to set some JavaScript but not close.  A good example of this is a page
    /// that uses frames and needs the parent.location to change - a redirect
    /// object doesn't work, and ClosePopupDP gives us the script. so we call
    /// this function to keep the window open.
    /// </summary>
    /// <param name="bShouldClose"></param>
    public virtual void SetToClose(bool bShouldClose)
    {
      m_bShouldClose = bShouldClose;
    }
  }
}