using System;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.constants;
// Framework Imports
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.uiinfrastructure.form
{
  /// <summary>
  /// This class is used along with IRepostModel and IRepostAS
  /// to partially implement the logic of the repost action.
  /// Those two classes are used by the Editor Framework to help
  /// implementing the repost mechanism on any Editor page.
  /// They can also be used by any other Activity Space.
  /// To do so here is what you need to do:
  /// - Extend this class in your activity space and add any repost
  /// action (e.g.: com.plumtree.portalpagessample.editor.template1.PropRepostControl).
  /// - Your model must implement IRepostModel.
  /// - Your AS must implement IRepostAS.
  /// - In your view, you can implement any repost action using the postToSelf
  /// JavaScript method.
  /// - If you extend PlumtreeDP and call SetAddMainForm(true) in the init
  /// method of your DP, the form and the JS postToSelf will be printed out
  /// automatically.
  /// </summary>
  /// <author>Julie Fournier</author>
  public class RepostControl : IControl
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.form.RepostControl");
    
    /// <summary>
    /// Object Name for AS 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "RepostControl";
    
    /// <summary>
    /// Form element names 
    /// </summary>
    public const String HTMLINPUT_POSTTOSELF = "in_hi_PostToSelf";
    
    /// <summary>
    /// HTML Element: 508 submit button 
    /// </summary>
    public const String HTMLSUBMIT_508_POSTTOSELF = "in_su_508PostToSelf_";
    
    /// <summary>
    /// The pass thru inputs need to be manually added in 508 mode if they are needed. 
    /// </summary>
    public const String HTMLINPUT_PASSTHRU_CONTROL = "in_hi_Passthru";
    
    /// <summary>
    /// HTML Element: Pass thru arguments 
    /// </summary>
    public const String HTMLINPUT_PASSTHRU_ARGS = "in_hi_PassthruArgs";
    
    /// <summary>
    /// JF- Repost Action: Undefined 
    /// </summary>
    public const int POSTTOSELF_ACTION_UNDEFINED = -1;
    
    /// <summary>
    /// RW- Repost Action: In place refresh 
    /// </summary>
    public const int POSTTOSELF_ACTION_IN_PLACE_REFRESH = 0;
    
    /// <summary>
    /// JF- Repost Action: Simple repost 
    /// </summary>
    public const int POSTTOSELF_ACTION_SIMPLEREPOST = 1;
    
    /// <summary>
    /// JF- Repost Action: Finish 
    /// </summary>
    public const int POSTTOSELF_ACTION_FINISH = 2;
    
    /// <summary>
    /// JF- Repost Action: Cancel 
    /// </summary>
    public const int POSTTOSELF_ACTION_CANCEL = 3;
    
    /// <summary>
    /// JF- Repost Action: Change page 
    /// </summary>
    public const int POSTTOSELF_ACTION_GOTOPAGE = 4;
    
    /// <summary>
    /// JF- Repost Action: Apply changes 
    /// </summary>
    public const int POSTTOSELF_ACTION_APPLY_CHANGES = 5;
    
    /// <summary>
    /// JF- Repost Action: Pass thru 
    /// </summary>
    public const int POSTTOSELF_ACTION_PASSTHRU = 9;
    
    /// <summary>
    /// JF- Page status: Valid 
    /// </summary>
    public const int PAGE_STATUS_VALID = 1;
    
    /// <summary>
    /// JF- Page status: Invalid 
    /// </summary>
    public const int PAGE_STATUS_INVALID = 2;
    
    /// <summary>
    /// JF- Associated Activity Space. 
    /// </summary>
    protected AActivitySpace m_asOwner;
    
    /// <summary>
    /// JF- Associated Model 
    /// </summary>
    protected IRepostModel m_model;
    
    /// <summary>
    /// JF- Redirect object. Used to redirect at the end of the Form. 
    /// </summary>
    protected Redirect m_rRedirect = null;
    
    /// <summary>
    /// Variables for the lifetime of the execute().  Just for convenience. 
    /// (Contains all the form params from the request.)
    /// </summary>
    protected XPHashtable m_htFormData;
    
    /// <summary>
    /// has Init() been called properly?
    /// </summary>
    protected bool m_bInitialized = false;
    
    /// <summary>
    /// JF- Value of the repost action
    /// </summary>
    protected int m_nAction = POSTTOSELF_ACTION_UNDEFINED;
    
    /// <summary>
    /// JF- Server side validation. Current state.
    /// Note that if the state is set to invalid, all it does is 
    /// it will not perform the changePage and finish actions.
    /// Also note that this framework does not provide any way to 
    /// set the error/status message to display to the end user. 
    /// Another framework is responsible for it.
    /// Finaly note that multiple save methods might return invalid on the same DP 
    /// but we don't want to keep an array of status returned, 
    /// we just need to know if invalid was returned at least once. 
    /// </summary>
    protected int m_nPageStatus = PAGE_STATUS_VALID;
    
    /// <summary>
    /// This code is called by the AS Servlet each time there is a repost.
    /// First it saves the page and then performs any repost action.
    /// If the Redirect object returned is not null,
    /// the Servlet will go to the page specified by the Redirect.
    /// If the Redirect is null, the Servlet will get the current page
    /// from the AS and display it.
    /// </summary>
    /// <returns>Redirect</returns>
    public virtual Redirect CheckActionSecurityAndExecute(XPHashtable arguments)
    {
      if (!m_bInitialized)
      {
        log.Error("Repost control has not been initialized: likely cause - Init() overridden but super.Init() not called.");
        }
      // Initialize control
      m_htFormData = arguments;
      m_rRedirect = null;
      m_nAction = POSTTOSELF_ACTION_UNDEFINED;
      // Set the repost action mode in the member variable.
      SetRepostAction();
      // Save the page
      SaveCurrentPage();
      // perform action
      PerformAction(m_nAction);
      // clear hashtable of form params
      m_htFormData = null;
      return m_rRedirect;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public virtual Object Create()
    {
      return new RepostControl();
    }
    
    /// <summary>
    /// Return the class name.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Returns the current page status.
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetPageStatus()
    {
      return m_nPageStatus;
    }
    
    /// <summary>
    /// Returns the current value of the repost action.
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetRepostAction()
    {
      return m_nAction;
    }
    
    /// <summary>
    /// Inits the class with the associated model and activity space.
    /// If this method is overridden in a sub-class, this version of
    /// Init must be called as super.Init();
    /// </summary>
    /// <param name="model"></param>
    /// <param name="space"></param>
    public virtual void Init(IModel model, AActivitySpace space)
    {
      try
      {
        try
        {
          m_model = (IRepostModel) model;
        }
        catch (Exception e)
        {
          throw XPException.GetInstance(e);
        }
      }
      catch (XPClassCastException cce)
      {
        log.Error(cce, "RepostControl::Init Unable to cast the Object " + model + " to IRepostModel. The following exception occured.");
        // JF- No need to extract, this is an unexpected error.
        throw new XPException("Unable to cast the Object " + model + " to IRepostModel.",
         cce);
      }
      m_asOwner = space;
      m_bInitialized = true;
    }
    
    /// <summary>
    /// Sets a new page status.
    /// </summary>
    /// <param name="_nPageStatus"></param>
    public virtual void SetPageStatus(int _nPageStatus)
    {
      m_nPageStatus = _nPageStatus;
    }
    
    /// <summary>
    /// Given the repost action mode, calls the associated method in the model.
    /// </summary>
    /// <param name="_nAction"></param>
    protected virtual void PerformAction(int _nAction)
    {
      switch (_nAction){
      case POSTTOSELF_ACTION_FINISH:
      if (PAGE_STATUS_VALID == m_nPageStatus)
      {
        m_model.Finish();
        SetRedirect(m_model.GetRedirectAfterFinish());
      }
      break;
      case POSTTOSELF_ACTION_CANCEL:
      SetRedirect(m_model.GetRedirectAfterFinish());
      break;
      case POSTTOSELF_ACTION_PASSTHRU:
      PerformPassthruAction();
      break;
      case POSTTOSELF_ACTION_IN_PLACE_REFRESH:
      // set a flag so that display pages will show in place 
      // content instead of normal content on the next pass.
      m_asOwner.SetIsInPlaceRefreshDisplayOn(true);
      break;
      }
    }
    // JF: Do not add a default case because sub classes will 
    // extend this one and add their own action modes.
    
    /// <summary>
    /// Sets the Pass thru arguments from the pass thru control into m_htFormData.
    /// And executes the Pass thru control.
    /// </summary>
    protected virtual void PerformPassthruAction()
    {
      if (m_htFormData.ContainsKey(HTMLINPUT_PASSTHRU_CONTROL) && m_htFormData.ContainsKey(HTMLINPUT_PASSTHRU_ARGS))
      {
        // Get the passthru control
        String[] ctlName = (String[]) m_htFormData.GetElement(HTMLINPUT_PASSTHRU_CONTROL);
        IControl passthru = m_asOwner.GetControl(ctlName[0]);
        // Get the passthru arguments
        String[] ctlArgs = (String[]) m_htFormData.GetElement(HTMLINPUT_PASSTHRU_ARGS);
        if (null != ctlArgs[0])
        {
          // URL-decode the arguments (since they were submitted through an onclick they
          // were not decoded automatically)
          ctlArgs[0] = XPHttpUtility.UrlDecode(ctlArgs[0], LinkConstants.UTF_8_ENCODING);
          // Add arguments to the form data
          XPStringTokenizer tokenizer = new XPStringTokenizer(ctlArgs[0],
           "&");
          while (tokenizer.HasMoreTokens())
          {
            String token = tokenizer.GetNextToken();
            int equals = XPStringUtility.IndexOf(token, '=');
            if (-1 != equals)
            {
              m_htFormData.PutElement(XPStringUtility.Substring(token, 0,
               equals), new String[]{XPStringUtility.Substring(token, equals + 1)});
              }
          }
          passthru.CheckActionSecurityAndExecute(m_htFormData);
        }
      }
    }
    
    /// <summary>
    /// Get the current page name and call the save method on the model.
    /// </summary>
    protected virtual void SaveCurrentPage()
    {
      // We always save the data on the page except on cancel.
      if (POSTTOSELF_ACTION_CANCEL != m_nAction)
      {
        // JF- Get back the page name from the form.
        String[] data = (String[]) m_htFormData.GetElement(ASConstants.PAGE_INPUT);
        if (data == null)
        {
          data = (String[]) m_htFormData.GetElement(ASConstants.PAGE);
        }
        if (data == null)
        {
          log.Error("RepostControl::SaveCurrentPage The page name has not been posted by the form.");
          throw new XPException("Unexpected error when trying to save the current page.");
          }
        if ((0 != data.Length) && (null != data[0]))
        {
          // JF- Save the page
          m_nPageStatus = m_model.SavePage(data[0], m_htFormData);
          log.Debug("RepostControl::SaveCurrentPage Saving page: " + data[0]);
          } else
        {
          log.Warn("RepostControl::SaveCurrentPage The page name value is invalid. The page will not be saved.");
          }
      }
    }
    
    /// <summary>
    /// Sets the redirect member variable.
    /// (Used to redirect at the end of the Editor.)
    /// </summary>
    /// <param name="_rRedirect"></param>
    protected virtual void SetRedirect(Redirect _rRedirect)
    {
      m_rRedirect = _rRedirect;
    }
    
    /// <summary>
    /// Get the repost action mode from the form data
    /// hashtable and set it in the member variable.
    /// </summary>
    protected virtual void SetRepostAction()
    {
      if ((AccessStyles.LOW_BANDWIDTH == m_asOwner.GetAccessStyle()) || (AccessStyles.SECTION_508 == m_asOwner.GetAccessStyle()))
      {
        // JF- If we are in 508 mode, we get the repost action from the submit button name.
        // E.g.: if the button that submitted the form was:
        //       <input type="submit" name="in_su_508PostToSelf_2" value="Done">
        //       The repost action will be 2.
        IXPEnumerator enumFormKeys = m_htFormData.GetKeys();
        String sKey;
        while (enumFormKeys.MoveNext())
        {
          sKey = (String) enumFormKeys.GetCurrent();
          if (XPStringUtility.StartsWith(sKey, HTMLSUBMIT_508_POSTTOSELF))
          {
            try
            {
              String sRepostAction = XPStringUtility.Substring(sKey, HTMLSUBMIT_508_POSTTOSELF.Length);
              m_nAction = XPConvert.ToInteger(sRepostAction);
              log.Debug("RepostControl::SetRepostAction Repost action ID value: " + m_nAction);
              break;
            }
            catch (Exception e)
            {
              log.Error("RepostControl::SetRepostAction Unable to retrieve the 508 repost " + "action from the submit button: " + sKey);
              }
          }
        }
      } else
      if (m_htFormData.ContainsKey(HTMLINPUT_POSTTOSELF))
      {
        String[] data = (String[]) m_htFormData.GetElement(HTMLINPUT_POSTTOSELF);
        if ((0 != data.Length) && (null != data[0]))
        {
          m_nAction = XPConvert.ToInteger(data[0]);
          log.Debug("RepostControl::SetRepostAction Repost action ID value: " + m_nAction);
          } else
        {
          log.Warn("RepostControl::SetRepostAction Repost action ID was not in Query String.");
          }
      }
    }
  }
}