using System;
using com.plumtree.openlog;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.common;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.uiinfrastructure.htmlconstructs;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.form.pagination
{
  /// <summary>
  /// This control sets the current pagination that the user has requested.
  /// </summary>
  /// <author>Don Hayler</author>
  public class PaginationControl : IControl
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.form.pagination.PaginationControl");
    
    /// <summary>
    /// Object Name for AS 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "PaginationControl";
    
    /// <summary>
    /// The id of the objects to paginate (hidden input) 
    /// </summary>
    public const String HTMLINPUT_PAGINATION_ID = "in_hi_PaginationID";
    
    /// <summary>
    /// The object count to paginate from (hidden input) 
    /// </summary>
    public const String HTMLINPUT_PAGINATE_FROM = "in_hi_PaginateFrom";
    
    /// <summary>
    /// The object count to paginate to (hidden input) 
    /// </summary>
    public const String HTMLINPUT_PAGINATE_TO = "in_hi_PaginateTo";
    
    /// <summary>
    /// The storage name of the associated Model (hidden input) 
    /// </summary>
    public const String HTMLINPUT_MODEL_STORAGE_NAME = "in_hi_ModelStorageName";
    
    /// <summary>
    /// The object count to paginate from (text input) 
    /// </summary>
    public const String HTMLINPUT_PAGINATE_FROM_TEXT = "in_tx_PaginateFrom";
    
    /// <summary>
    /// The object count to paginate to (text input) 
    /// </summary>
    public const String HTMLINPUT_PAGINATE_TO_TEXT = "in_tx_PaginateTo";
    
    /// <summary>
    /// the name of the javascript method that pages that use the pagination bar need
    /// to implement.  This javascript method should set the html inputs and post the page to
    /// this control (potentially as a pass-thru control).
    /// </summary>
    public const String JAVASCRIPT_PAGINATE = "Paginate";
    // NOTE: some of these should be moved to .js files
    
    /// <summary>
    /// internal javascript function names. 
    /// </summary>
    private const String JAVASCRIPT_CHECK_NUMERIC_TO_FROM_VALUES = "CheckNumericToFrom";
    
    /// <summary>
    /// the bar that separates different sections in the pagination links. 
    /// </summary>
    public const String PAGINATION_LINK_SEPARATOR = "&nbsp;|&nbsp;";
    
    /// <summary>
    /// the number of links to pagination sections that we will show at the bottom. 
    /// </summary>
    private const int NUM_PAGINATION_LINKS = 10;
    
    /// <summary>
    /// model 
    /// </summary>
    private IPaginationModel m_asModel;
    
    /// <summary>
    /// Activity Space 
    /// </summary>
    private AActivitySpace m_asOwner;
    
    /// <summary>
    /// Static method to create a pagination bar.
    /// DisplayPages that include this pagination bar (through their views) must ensure that the
    /// JAVASCRIPT_PAGINATE method is implemented on the page.
    /// warning: in 508 mode, this will add the RepostControl.HTMLINPUT_PASSTHRU_CONTROL
    /// elements (since they won't be in the form), so if another view tries to use them
    /// there will be problems.  The pagination bar should also not be added to a page
    /// more than once in 508 mode.
    /// This bar does pagination filtering (view from X to Y).
    /// </summary>
    /// <param name="AActivitySpace">the owner space for this HTML</param>
    /// <param name="int">the pagination ID (usually the class ID)</param>
    /// <param name="String">the name of the HTML form</param>
    /// <returns>PTConstruct a construct for the pagination bar of type PTBannerType.OBJECT_ACTION_BAR</returns>
    public static PTConstruct CreatePaginationFilterBar(AActivitySpace asOwner,
     int nPaginationID, int nObjectCount)
    {
      bool bNoJavascript = (asOwner.GetAccessStyle() != AccessStyles.STANDARD);
      HTMLInput myInput;
      PTConstruct bannerConstruct = UIConstructFactory.GetConstruct(PTBannerType.OBJECT_ACTION_BAR,
       true, asOwner);
      try
      {
        // View Items
        HTMLSpan mySpan = new HTMLSpan();
        mySpan.SetStyleClass(PTStyleClass.ACTIONBAR_TEXT);
        //_ui_ViewItems
        mySpan.AddInnerHTMLString(asOwner.GetString(88, ResourceConstants.INFRASTRUCTURE));
        bannerConstruct.AddElementToNestedElement(PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL_ELEMENT_NAME,
         mySpan);
        bannerConstruct.AddHTMLStringToNestedElement(PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL_ELEMENT_NAME,
         CommonHTMLStrings.SPACE);
        if (bNoJavascript)
        {
          // Just use normal paginate input name in 508 mode
          myInput = new HTMLInput(HTMLInputTypes.TEXT, HTMLINPUT_PAGINATE_FROM_TEXT,
           "");
        } else
        {
          myInput = new HTMLInput(HTMLInputTypes.TEXT, HTMLINPUT_PAGINATE_FROM_TEXT + nPaginationID,
           "");
        }
        myInput.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);
        myInput.SetMaxLength(8);
        myInput.SetSize(5);
        bannerConstruct.AddElementToNestedElement(PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL_ELEMENT_NAME,
         myInput);
        bannerConstruct.AddHTMLStringToNestedElement(PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL_ELEMENT_NAME,
         CommonHTMLStrings.SPACE);
        mySpan = new HTMLSpan();
        mySpan.SetStyleClass(PTStyleClass.ACTIONBAR_TEXT);
        //_ui_To
        mySpan.AddInnerHTMLString(asOwner.GetString(89, ResourceConstants.INFRASTRUCTURE));
        bannerConstruct.AddElementToNestedElement(PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL_ELEMENT_NAME,
         mySpan);
        bannerConstruct.AddHTMLStringToNestedElement(PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL_ELEMENT_NAME,
         CommonHTMLStrings.SPACE);
        if (bNoJavascript)
        {
          // Just use normal paginate input name in 508 mode
          myInput = new HTMLInput(HTMLInputTypes.TEXT, HTMLINPUT_PAGINATE_TO_TEXT,
           "");
        } else
        {
          myInput = new HTMLInput(HTMLInputTypes.TEXT, HTMLINPUT_PAGINATE_TO_TEXT + nPaginationID,
           "");
        }
        myInput.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);
        myInput.SetMaxLength(8);
        myInput.SetSize(5);
        bannerConstruct.AddElementToNestedElement(PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL_ELEMENT_NAME,
         myInput);
        bannerConstruct.AddHTMLStringToNestedElement(PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL_ELEMENT_NAME,
         CommonHTMLStrings.SPACE);
        if (bNoJavascript)
        {
          // Name of passthru control to run when PASSTHRU action is selected
          myInput = new HTMLInput(HTMLInputTypes.HIDDEN, RepostControl.HTMLINPUT_PASSTHRU_CONTROL,
           "");
          myInput.SetValue(STR_MVC_CLASS_NAME);
          bannerConstruct.AddElementToNestedElement(PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL_ELEMENT_NAME,
           myInput);
          myInput = new HTMLInput(HTMLInputTypes.SUBMIT, RepostControl.HTMLSUBMIT_508_POSTTOSELF + RepostControl.POSTTOSELF_ACTION_PASSTHRU,
           "");
        } else
        {
          myInput = new HTMLInput(HTMLInputTypes.BUTTON, "in_bu_GOTO", "");
          myInput.SetOnClick(JAVASCRIPT_PAGINATE + "(" + nPaginationID + ", " + HTMLINPUT_PAGINATE_FROM_TEXT + nPaginationID + ".value, " + HTMLINPUT_PAGINATE_TO_TEXT + nPaginationID + ".value, " + nObjectCount + "); return false;");
          }
        myInput.SetValue(">>");
        myInput.SetStyleClass(PTStyleClass.FORM_BTN_TEXT);
        bannerConstruct.AddElementToNestedElement(PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL_ELEMENT_NAME,
         myInput);
        bannerConstruct.AddHTMLStringToNestedElement(PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL_ELEMENT_NAME,
         CommonHTMLStrings.SPACE);
      }
      catch (Exception e)
      {
        log.Error(e, "Unable to build pagination bar.");
        bannerConstruct = null;
      }
      return bannerConstruct;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.form.pagination.PaginationControl#CreatePaginationLinks()"></seealso>
    public static HTMLElement CreatePaginationLinks(AActivitySpace asOwner,
     int nPaginationID, int nObjectCount, int nCurrentStart, int nCurrentFinish,
     int nPaginationSize)
    {
      return PaginationControl.CreatePaginationLinks(asOwner, nPaginationID,
       nObjectCount, nCurrentStart, nCurrentFinish, nPaginationSize, "");
      }
    
    /// <summary>
    /// Static method to create the pagination links that go at the bottom of the display.
    /// DisplayPages that include this pagination links (through their views) must ensure that the
    /// JAVASCRIPT_PAGINATE method is implemented on the page.
    /// This bar does pagination links (prev, 1-50, 50-100, etc... next).
    /// </summary>
    /// <param name="AActivitySpace">the owner space for this HTML</param>
    /// <param name="int">the pagination ID (usually the class ID)</param>
    /// <param name="String">the name of the HTML form</param>
    /// <returns>HTMLElement the links for the pagination at the bottom of the display</returns>
    public static HTMLElement CreatePaginationLinks(AActivitySpace asOwner,
     int nPaginationID, int nObjectCount, int nCurrentStart, int nCurrentFinish,
     int nPaginationSize, String sModelStorageKey)
    {
      HTMLElement result = new HTMLElementCollection();
      HTMLSpan mySpan = new HTMLSpan();
      mySpan.SetStyleClass(PTStyleClass.OBJECT_TEXT);
      result.AddInnerHTMLElement(mySpan);
      bool bNoJavascript = (asOwner.GetAccessStyle() != AccessStyles.STANDARD);
      try
      {
        // DCA - check also if number of objects is equal to number of objects being shown
        if (nObjectCount == nCurrentFinish - (nCurrentStart - 1))
        {
          // No pagination necessary.
          return result;
        } else
        {
          // if we have more than 2 sets of pagination and we're not on the first one ...
          if ((nObjectCount > (2 * nPaginationSize * NUM_PAGINATION_LINKS)) && (nCurrentFinish > (nPaginationSize * NUM_PAGINATION_LINKS)))
          {
            // first link
            // Display link
            mySpan.AddInnerHTMLElement(PaginationControl.GetPaginationAnchor(1,
             nPaginationSize, nPaginationID, asOwner.GetString(139, ResourceConstants.INFRASTRUCTURE),
             bNoJavascript, sModelStorageKey, asOwner));
            // Display spacer					
            mySpan.AddInnerHTMLString(PAGINATION_LINK_SEPARATOR);
          }
          // prev link
          if (nCurrentStart > 1)
          {
            // link
            int nLinkLowerLimit = 1;
            int nLinkUpperLimit = nPaginationSize;
            for (int i = 1;i <= nCurrentStart;i+=nPaginationSize)
            {
              // Calculate the upper and lower bounds of the previous section
              // for previous link
              nLinkLowerLimit = i - nPaginationSize;
              if (nLinkLowerLimit < 1)
              {
                nLinkLowerLimit = 1;
              }
              nLinkUpperLimit = nLinkLowerLimit + nPaginationSize - 1;
            }
            // Display link
            mySpan.AddInnerHTMLElement(PaginationControl.GetPaginationAnchor(nLinkLowerLimit,
             nLinkUpperLimit, nPaginationID, asOwner.GetString(207, ResourceConstants.PORTALINFRASTRUCTURE),
             bNoJavascript, sModelStorageKey, asOwner));
            // Display spacer					
            mySpan.AddInnerHTMLString(PAGINATION_LINK_SEPARATOR);
          }
          // Calculate beginning and end of current pagination sections to display
          // integer division will truncate remainder to give us current section.
          int nPaginationSectionStart = nCurrentStart / (nPaginationSize * NUM_PAGINATION_LINKS);
          // get start of current section
          nPaginationSectionStart = (nPaginationSectionStart * nPaginationSize * NUM_PAGINATION_LINKS) + 1;
          int nPaginationSectionEnd = nPaginationSectionStart + (nPaginationSize * NUM_PAGINATION_LINKS) - 1;
          if (nPaginationSectionStart > 1)
          {
            // link to previous set of pagination links
            int nPreviousPaginationSectionStart = nPaginationSectionStart - (nPaginationSize * NUM_PAGINATION_LINKS);
            int nPreviousPaginationSectionEnd = nPreviousPaginationSectionStart + nPaginationSize - 1;
            // Display link
            mySpan.AddInnerHTMLElement(PaginationControl.GetPaginationAnchor(nPreviousPaginationSectionStart,
             nPreviousPaginationSectionEnd, nPaginationID, asOwner.GetString(137,
             ResourceConstants.INFRASTRUCTURE), bNoJavascript, sModelStorageKey,
             asOwner));
            mySpan.AddInnerHTMLString(PAGINATION_LINK_SEPARATOR);
          }
          int nCurrentUpperLimit;
          // object group links
          for (int i = 1;i < nObjectCount;i+=nPaginationSize)
          {
            nCurrentUpperLimit = i + nPaginationSize - 1;
            if (nCurrentUpperLimit > nObjectCount)
            {
              nCurrentUpperLimit = nObjectCount;
            }
            // Only display pagination sections within the pagination section limits (10 sections)
            if ((i >= nPaginationSectionStart) && (nCurrentUpperLimit <= nPaginationSectionEnd))
            {
              // m - n string
              String strCurrentRange = XPResourceManager.FormatMessage(asOwner.GetString(138,
               ResourceConstants.INFRASTRUCTURE), i, nCurrentUpperLimit);
              if ((nCurrentStart >= i) && (nCurrentFinish <= nCurrentUpperLimit))
              {
                // Display as text without a link
                mySpan.AddInnerHTMLString(strCurrentRange);
              } else
              {
                // Display link
                mySpan.AddInnerHTMLElement(PaginationControl.GetPaginationAnchor(i,
                 nCurrentUpperLimit, nPaginationID, strCurrentRange, bNoJavascript,
                 sModelStorageKey, asOwner));
              }
              mySpan.AddInnerHTMLString(PAGINATION_LINK_SEPARATOR);
            }
          }
          if (nPaginationSectionEnd < nObjectCount)
          {
            // link to next set of pagination links
            int nNextPaginationSectionStart = nPaginationSectionStart + (nPaginationSize * NUM_PAGINATION_LINKS);
            int nNextPaginationSectionEnd = nNextPaginationSectionStart + nPaginationSize - 1;
            if (nNextPaginationSectionEnd > nObjectCount)
            {
              nNextPaginationSectionEnd = nObjectCount;
            }
            // Display link
            mySpan.AddInnerHTMLElement(PaginationControl.GetPaginationAnchor(nNextPaginationSectionStart,
             nNextPaginationSectionEnd, nPaginationID, asOwner.GetString(137,
             ResourceConstants.INFRASTRUCTURE), bNoJavascript, sModelStorageKey,
             asOwner));
            mySpan.AddInnerHTMLString(PAGINATION_LINK_SEPARATOR);
          }
          // next link
          if (nCurrentFinish < nObjectCount)
          {
            // link
            // DCA - set new upper limit to 1 more than the current last item being shown
            int nNewLowerLimit = nCurrentFinish + 1;
            int nNewUpperLimit = nObjectCount;
            // Calculate the upper and lower bounds of the next section
            // for next link
            for (int i = 1;i < nObjectCount;i+=nPaginationSize)
            {
              if (i >= nCurrentFinish)
              {
                nNewLowerLimit = i;
                nNewUpperLimit = i + nPaginationSize - 1;
                break;
              }
            }
            if (nNewUpperLimit > nObjectCount)
            {
              nNewUpperLimit = nObjectCount;
            }
            if (nNewLowerLimit < 1)
            {
              nNewLowerLimit = 1;
            }
            // Display link
            mySpan.AddInnerHTMLElement(PaginationControl.GetPaginationAnchor(nNewLowerLimit,
             nNewUpperLimit, nPaginationID, asOwner.GetString(208, ResourceConstants.PORTALINFRASTRUCTURE),
             bNoJavascript, sModelStorageKey, asOwner));
          }
          // if we have more than 2 sets of pagination and we're not on the last one ...
          if ((nObjectCount > (2 * nPaginationSize * NUM_PAGINATION_LINKS)) && (nPaginationSectionEnd < nObjectCount))
          {
            // last link
            // integer division truncates, leaving us with number of pagination groups
            int nLastLowerLimit = nObjectCount / nPaginationSize;
            // Get start of last pagination group
            nLastLowerLimit = (nLastLowerLimit * nPaginationSize) + 1;
            if (nLastLowerLimit > nObjectCount)
            {
              nLastLowerLimit = nLastLowerLimit - nPaginationSize;
            }
            // Display separator
            mySpan.AddInnerHTMLString(PAGINATION_LINK_SEPARATOR);
            // Display link
            mySpan.AddInnerHTMLElement(PaginationControl.GetPaginationAnchor(nLastLowerLimit,
             nObjectCount, nPaginationID, asOwner.GetString(140, ResourceConstants.INFRASTRUCTURE),
             bNoJavascript, sModelStorageKey, asOwner));
          }
        }
      }
      catch (Exception e)
      {
        log.Error(e, "Unable to create pagination links.");
        result = new HTMLElementCollection();
      }
      return result;
    }
    
    /// <summary>
    /// Helper method to generate links to new paginations. 
    /// </summary>
    private static HTMLAnchor GetPaginationAnchor(int nLowerLimit, int nUpperLimit,
     int nPaginationID, String strText, bool bNoJavascript, String sModelStorageKey,
     AActivitySpace asOwner)
    {
      HTMLAnchor result;
      // Display link
      if (bNoJavascript)
      {
        ASURL myURL = new ASURL(asOwner);
        myURL.SetLinkGetCachedSpace(asOwner);
        myURL.SetControl(PassthruControl.STR_MVC_CLASS_NAME);
        myURL.AddControlArgument(PassthruControl.QS_PASSTHRU_CONTROL, STR_MVC_CLASS_NAME);
        myURL.AddControlArgument(PassthruControl.QS_MAIN_CONTROL, ((IFormAS) asOwner).GetRepostControlName());
        myURL.AddControlArgument(ASConstants.PAGE, asOwner.GetCurrentPage().GetName());
        myURL.AddControlArgument(HTMLINPUT_PAGINATION_ID, nPaginationID);
        myURL.AddControlArgument(HTMLINPUT_PAGINATE_FROM, nLowerLimit);
        myURL.AddControlArgument(HTMLINPUT_PAGINATE_TO, nUpperLimit);
        result = (HTMLAnchor) myURL.GetURLAsHTMLElement();
        result.AddInnerHTMLString(strText);
      } else
      {
        result = ConstructStaticHelpers.GetJavascriptAnchor(strText, JAVASCRIPT_PAGINATE + "(" + nPaginationID + ", " + nLowerLimit + ", " + nUpperLimit + ",\'\',\'" + sModelStorageKey + "\')");
        }
      return result;
    }
    
    /// <summary>
    /// This method creates the default javascript for the pagination control.
    /// This assumes that there is an AFormDP that should be reposted via a
    /// pass-thru control.
    /// </summary>
    /// <returns>String the javascript to display.</returns>
    public static String CreatePaginationJavascript(AActivitySpace asOwner)
    {
      XPStringBuilder sb = new XPStringBuilder(1250);
      sb.Append("function ").Append(JAVASCRIPT_CHECK_NUMERIC_TO_FROM_VALUES).Append("(strBegin, strEnd)\n");
      sb.Append("{\n");
      // Check if the strings are numeric or empty
      sb.Append("  if ((false == PTNumberUtil.isInteger(strBegin)) ||\n");
      sb.Append("      (false == PTNumberUtil.isInteger(strEnd)) ||\n");
      // Check that the strings are not empty.
      sb.Append("      (true == PTStringUtil.isAllWhitespace(strBegin)) ||\n");
      sb.Append("      (true == PTStringUtil.isAllWhitespace(strEnd)))\n");
      sb.Append("  {\n");
      //_ui_JavaScriptAlert_OutOfRangeValue)
      sb.Append("      alert(\"").Append(asOwner.GetString(160, ResourceConstants.INFRASTRUCTURE)).Append("\");\n");
      sb.Append("      return false;\n");
      sb.Append("  }\n");
      sb.Append("  return true;\n");
      sb.Append("}\n");
      sb.Append("function ").Append(JAVASCRIPT_PAGINATE).Append("(nID,strBegin,strEnd,strMax,strModelStorageKey)\n").Append("{\n").Append("  if (true == ").Append(JAVASCRIPT_CHECK_NUMERIC_TO_FROM_VALUES).Append("(strBegin, strEnd))\n").Append("  {\n").Append("     var nBegin = parseInt(strBegin);\n").Append("     var nEnd = parseInt(strEnd);\n").Append("     var nMax = parseInt(strMax);\n").Append("     if (nBegin < 1)\n").Append("     {\n").Append("        nBegin = 1;\n").Append("     }\n").Append("     if (nEnd < 1)\n").Append("     {\n").Append("        nEnd = 1;\n").Append("     }\n").Append("     if (nBegin > nMax)\n").Append("     {\n").Append("        nBegin = nMax;\n").Append("     }\n").Append("     if (nEnd > nMax)\n").Append("     {\n").Append("        nEnd = nMax;\n").Append("     }\n").Append("     if (nEnd < nBegin)\n").Append("     {\n").Append("        nEnd = nBegin;\n").Append("     }\n").Append("     frm = document.").Append(AFormDP.HTMLFORM_NAME).Append(";\n").Append("     frm.").Append(HTMLINPUT_PAGINATION_ID).Append(".value = nID;\n").Append("     frm.").Append(HTMLINPUT_PAGINATE_FROM).Append(".value = nBegin;\n").Append("     frm.").Append(HTMLINPUT_PAGINATE_TO).Append(".value = nEnd;\n").Append("     frm.").Append(HTMLINPUT_MODEL_STORAGE_NAME).Append(".value = strModelStorageKey;\n").Append("     ").Append(AFormDP.JAVASCRIPT_SUBMIT_TO_CONTROL).Append("(\'").Append(STR_MVC_CLASS_NAME).Append("\');\n").Append("  }\n").Append("}\n");
      return sb.ToString();
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IControl#CheckActionSecurityAndExecute(XPHashtable)"></seealso>
    public virtual Redirect CheckActionSecurityAndExecute(XPHashtable arguments)
    {
      int nPaginationID;
      int nPaginationUpperBound;
      int nPaginationLowerBound;
      // Find pagination ID
      String[] sValues = (String[]) arguments.GetElement(HTMLINPUT_PAGINATION_ID);
      if (sValues != null)
      {
        nPaginationID = XPConvert.ToInteger(sValues[0]);
      } else
      {
        log.Warn("Unable to find pagination ID.  Using default.");
        nPaginationID = 1;
      }
      // Find Pagination lower bound
      sValues = (String[]) arguments.GetElement(HTMLINPUT_PAGINATE_FROM);
      if (sValues != null)
      {
        nPaginationLowerBound = XPConvert.ToInteger(sValues[0]);
      } else
      {
        sValues = (String[]) arguments.GetElement(HTMLINPUT_PAGINATE_FROM_TEXT);
        if (sValues != null)
        {
          nPaginationLowerBound = XPConvert.ToInteger(sValues[0]);
        } else
        {
          log.Error("Unable to find pagination lower bound.  Using default.");
          nPaginationLowerBound = 1;
        }
      }
      // Find Pagination upper bound
      sValues = (String[]) arguments.GetElement(HTMLINPUT_PAGINATE_TO);
      if (sValues != null)
      {
        nPaginationUpperBound = XPConvert.ToInteger(sValues[0]);
      } else
      {
        sValues = (String[]) arguments.GetElement(HTMLINPUT_PAGINATE_TO_TEXT);
        if (sValues != null)
        {
          nPaginationUpperBound = XPConvert.ToInteger(sValues[0]);
        } else
        {
          log.Error("Unable to find pagination upper bound.  Using default.");
          nPaginationUpperBound = 1;
        }
      }
      // Find Model storage key
      String sModelStorageKey = "";
      sValues = (String[]) arguments.GetElement(HTMLINPUT_MODEL_STORAGE_NAME);
      if ((sValues != null) && (null != sValues[0]))
      {
        sModelStorageKey = XPConvert.ToString(sValues[0]);
      }
      if ((("").Equals(sModelStorageKey)) || (!m_asOwner.GetModelExists(sModelStorageKey)))
      {
        log.Debug("Unable to find Model storage name.  Using default Model.");
        m_asModel.SetPagination(nPaginationID, nPaginationLowerBound, nPaginationUpperBound);
        } else
      {
        try
        {
          IPaginationModel myModel = (IPaginationModel) m_asOwner.GetModel(sModelStorageKey);
          myModel.SetPagination(nPaginationID, nPaginationLowerBound, nPaginationUpperBound);
          }
        catch (Exception e)
        {
          log.Error("Retrieved wrong Model type with storage key: " + sModelStorageKey + ". Expecting IPaginationModel");
          }
      }
      return null;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public virtual Object Create()
    {
      return new PaginationControl();
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public virtual String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IControl#Init(IModel,AActivitySpace)"></seealso>
    public virtual void Init(IModel model, AActivitySpace space)
    {
      m_asOwner = space;
      try
      {
        m_asModel = (IPaginationModel) model;
      }
      catch (Exception e)
      {
        log.Error(e, "Initialized with wrong type of model.  Expecting IPaginationModel.");
        }
    }
  }
}