using System;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.activityspace.utils;
using com.plumtree.uiinfrastructure.application;
using com.plumtree.uiinfrastructure.application.startup.messages;
using com.plumtree.uiinfrastructure.application.varpacks;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.diagnostic.reporter;
using com.plumtree.uiinfrastructure.errordisplay;
using com.plumtree.uiinfrastructure.helpers;
using com.plumtree.uiinfrastructure.interpreter.filter;
using com.plumtree.uiinfrastructure.interpreter.filter.utils;
using com.plumtree.uiinfrastructure.login;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.util;
using com.plumtree.uiinfrastructure.web;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.interpreter
{
  /*
   * This class is written based on
   * com.plumtree.uiinfrastructure.activityspace.Interpreter class.
   */
   
  
  /// <summary>
  /// The Interpreter class takes HTTP requests, checks the authentication 
  /// on the request sessions, processes the requests for any activity-space 
  /// related operations, including IControl executions and PEI invocations, 
  /// and finally writes the response to the client browser.
  /// </summary>
  /// <author>Don Hayler</author>
  /// <author>robertz  (StoreBrowserSettings)</author>
  /// <author>Daniilk  (Gateway)</author>
  /// <author>Mike Jahr (SSO)</author>
  /// <author>Julie Fournier (Security Modes)</author>
  /// <author>WilliamA  (Writing this new class)</author>
  public class Interpreter : IXPPage
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.interpreter.Interpreter");
    
    /// <summary>
    /// mj: default error space. 
    /// </summary>
    public const String DEFAULTERRORSPACE = "ErrorAS";
    
    /// <summary>
    /// This is used so that PTTracker can ignore login redirects and
    /// not count them as valid myPage hits.
    /// NOTE: Is this a good error code to use?  It seems to work ok
    /// in NS and IE.
    /// </summary>
    private const int PT_TRACKER_REDIRECT_400_STATUS = 499;
    
    /// <summary>
    /// stored as a string 
    /// </summary>
    public const String HOSTNAME = "strHostName";
    
    /// <summary>
    /// stored as an Integer 
    /// </summary>
    public const String HOSTPORT = "nHostPort";
    
    /// <summary>
    /// false implies guest -- this should only be read in the Interpreter
    /// (and elsewhere). It is set in the LoginHelper.SetSessionValidity() method. 
    /// </summary>
    public const String USERSESSIONVALID = "bSessionValid";
    
    /// <summary>
    /// stored as a string 
    /// </summary>
    public const String USERLOCALE = "strUserLocale";
    
    /// <summary>
    /// stored as a BrowserType object 
    /// </summary>
    public const String BROWSERTYPE = "objBrowserType";
    
    /// <summary>
    /// internal timer checkpoint constants 
    /// </summary>
    public const String CHECKPOINT_CONTROL_START = "StartControl";
    public const String CHECKPOINT_CONTROL_END = "EndControl";
    public const String CHECKPOINT_PAGE_START = "StartPage";
    public const String CHECKPOINT_PAGE_END = "EndPage";
    public const String CHECKPOINT_DISPLAY_START = "StartDisplay";
    public const String CHECKPOINT_DISPLAY_END = "EndDisplay";
    
    /// <summary>
    /// Mime type for JSON data.  JSON requests will not have the POST data parsed automatically.
    /// </summary>
    public const String JSON_DATA_TYPE = "application/json";
    protected static readonly IAuthenticator m_Authenticator = Authenticator.GetInstance();
    protected static readonly MVCController m_MVCController = MVCController.GetInstance();
    
    /// <summary>
    /// Stores all possible non-form request (I.E. gateway) uri patterns *
    /// </summary>
    private static XPArrayList NON_FORM_PATTERNS = new XPArrayList();
    
    /// <summary>
    /// Adds non-form data url patterns (I.E. gateway requests).  These signal the Interpreter
    /// not to parse the request body as form-encoded.  This allows subsequent code to
    /// get the input stream.
    /// NOTe: This method is not threadsafe and should only be called by static initializers.
    /// Called on the first request after startup by GatewayHelpers.
    /// </summary>
    /// <param name="pattern">
    /// A pattern is just a part of a non-form data url, for example: '/gateway/PTARGS_' or '/gw/'
    /// The latter example is the default keyword for gateway friendly urls, it isn't completely unique
    /// since you could name a community or page 'gw' but it is up to the customer to choose a unique
    /// enough friendly url keyword.
    /// </param>
    public static void AddNonFormDataRequestPattern(String _strPattern)
    {
      if (null == _strPattern || "".Equals(_strPattern))
      {
        return;
      }
      // don't add the same pattern twice
      if (NON_FORM_PATTERNS.Contains(_strPattern))
      {
        log.Warn("Attempting to add Non-form Data Request pattern twice. [" + _strPattern + "]");
        return;
      }
      NON_FORM_PATTERNS.Add(_strPattern);
    }
    
    /// <summary>
    /// This is the only way to safely clear the sandbox so that it gets
    /// correctly re-initialized. This does not reset the SESSIONVALID variable.
    /// </summary>
    /// <param name="smSandbox"></param>
    /// <returns>ISessionManager</returns>
    public static void ClearSandbox(ISessionManager smSandbox)
    {
      // Get the old cache object.
      ASCache cacheObject = (ASCache) smSandbox.GetAttribute(ASConstants.SANDBOXASCACHE);
      // HACK - there should be a better way to do this.  A sub-session perhaps.
      // These names should also be constants.
      // Get the old browser settings
      Object bIsMac = smSandbox.GetAttribute("ISMAC");
      Object bIsIE = smSandbox.GetAttribute("ISIE");
      Object bIsNS = smSandbox.GetAttribute("ISNS");
      Object bIsNS4 = smSandbox.GetAttribute("ISNS4");
      Object bIsNS67 = smSandbox.GetAttribute("ISNS67");
      Object strGWFlag = smSandbox.GetAttribute(GatewayHandlers.GATEWAY_FAILURE_FLAG);
      Object browserType = smSandbox.GetAttribute(BROWSERTYPE);
      // Clear the sandbox
      smSandbox.Clear();
      // put the ASCache back where it belongs
      smSandbox.SetAttribute(ASConstants.SANDBOXASCACHE, cacheObject);
      // Put the old browser settings back in place
      // DAP NOTE this are for backward compatibility only - see BrowserControl - 
      // please remove after 8/1/05 
      smSandbox.SetAttribute("ISMAC", bIsMac);
      smSandbox.SetAttribute("ISIE", bIsIE);
      smSandbox.SetAttribute("ISNS", bIsNS);
      smSandbox.SetAttribute("ISNS4", bIsNS4);
      smSandbox.SetAttribute("ISNS67", bIsNS67);
      smSandbox.SetAttribute(BROWSERTYPE, browserType);
      // only store it on a session if it is not null.
      if (strGWFlag != null)
      {
        smSandbox.SetAttribute(GatewayHandlers.GATEWAY_FAILURE_FLAG, strGWFlag);
        }
      // This re-initializes the personal settings subsession
      InitializePersonalSettingsSession(smSandbox);
    }
    
    /// <summary>
    /// Initializes the personal settings session manager off the sandbox session.
    /// This is done automatically when a new RequestData object is created.
    /// If the sandbox has been cleared, this re-initializes the personal
    /// settings session manager.
    /// </summary>
    public static void InitializePersonalSettingsSession(ISessionManager session)
    {
      // Get Personal Settings session off of the sandbox
      session.GetSubSession(PersonalSettingsHelper.PREFS_SUBSESSION_NAME);
      }
    
    /// <summary>
    /// Pass the HTTP request to the static control code.
    /// </summary>
    /// <param name="request"></param>
    /// <param name="response"></param>
    /// <param name="session"></param>
    /// <param name="application"></param>
    public virtual void DoService(IXPRequest request, IXPResponse response,
     ISessionManager session, IApplication application)
    {
      HandleRequest(request, response, session, application);
    }
    
    /// <summary>
    /// Handle the HTTP request by getting the appropriate control, filling out
    /// its data, and returning the Display page from the control.
    /// </summary>
    /// <param name="request"></param>
    /// <param name="response"></param>
    /// <param name="session"></param>
    /// <param name="application"></param>
    public static void HandleRequest(IXPRequest request, IXPResponse response,
     ISessionManager session, IApplication application)
    {
      if ((session == null) || (application == null) || (!ASManager.INSTANCE.m_bSystemStartupSuccesful))
      {
        // System startup must have failed.  Do not proceed.
        if (session == null)
        {
          log.Error("In Handle Request: session is null.  Calling OutputStartupFailure");
          }
        if (application == null)
        {
          log.Error("In Handle Request: application is null.  Calling OutputStartupFailure");
          }
        if (!ASManager.INSTANCE.m_bSystemStartupSuccesful)
        {
          log.Error("In Handle Request: ASManager.INSTANCE.m_bSystemStartupSuccesful is false.  Calling OutputStartupFailure");
          }
        OutputStartupFailure(request, response);
        return;
      }
      String strRequestFullURL = HttpUtils.GetRequestFullURL(request);
      log.Action("Entering handleRequest: " + request.GetMethod() + " " + strRequestFullURL);
      // Still need to evaluate rules before displaying page
      bool bEvaluateRules = true;
      // boolean flag to check if a redirect has been sent
      bool bRedirectSent = false;
      // ** Extract Request Info **		
      // TODO Bug#51044: RequestData creation can fail, which will
      // cause the catch block to fail, throwing an error all the
      // way up to XPPage, which will report (mistakenly) that
      // startup has failed.  We should probably put a try/catch
      // around everything in HandleRequest.
      RequestData tempData = null;
      try
      {
        // We don't want to parse the POST data for requests that have
        // non-form post data.  This includes gateway requests (checked
        // by URL), and REST APIs (checked by content type).
        // Check for gateway requests.
        bool bNonFormRequestURI = IsNotFormRequest(strRequestFullURL);
        // Check for non-form content type (i.e. REST APIs)
        String sContentType = request.GetContentType();
        bool bNonFormContentType = false;
        if ((sContentType != null) && (XPStringUtility.StartsWith(XPStringUtility.ToLowerCase(sContentType),
         JSON_DATA_TYPE)))
        {
          bNonFormContentType = true;
        }
        // By default we want to parse the request body, however, we don't parse the
        // request body for binary data (i.e. gateway URIs) and other non-form POST
        // data (i.e. REST APIs).
        if (!bNonFormRequestURI && !bNonFormContentType)
        {
          request.ParseRequest();
        }
        // Temporary data structure for this HTTPRequest, & webdata object to wrap XPResponse
        // Also gets User's sandbox session -- this needs to occur before we attempt to get the lock
        // so that we can correctly localize error messages if we can't get the lock.
        tempData = new RequestData(request, response, session, application);
        tempData.LoadQSSettings();
        tempData.SetRequestFullURL(strRequestFullURL);
      }
      catch (Exception e)
      {
        log.Error(e, "Unable to load request data.");
        // Unable to process QS.  Try to display error page.
        tempData = new RequestData(request, response, session, application);
        // false means do not display the stack trace of this error in an
        // HTMLComment on the page.  We do this because LoadQSSettings()
        // checks for possible cross-site scripting attacks, and we don't
        // want to give any information to possible hackers.
        HandleError(e, false, tempData);
        return;
      }
      // ** Detect Browser **
      DetectBrowser(tempData);
      // ** Execute Pre-Filters **
      int nTerminatingFilterIndex = DoPreFilter(tempData);
      if (nTerminatingFilterIndex != -1)
      {
        // We are done processing this request, but we still need
        // to execute the postfilters for every filter that was
        // executed so they can clean up properly. (roll back
        // through them in reverse order) 
        DoPostFilter(nTerminatingFilterIndex, tempData);
        log.Action("Leaving handleRequest - pre filter terminated execution.");
        return;
      }
      // Increment the performance counters for page hit
      try
      {
        PerfCounters.INSTANCE.IncrementCounter(PerfCounterConstants.PORTALPAGESPERF_TOTAL_HITS);
        }
      catch (Exception e)
      {
        // Simply log this exception and ignore it, as we don't want things like
        // this to effect the end user experience.
        log.Error(e, "Unable to increment performance counter.");
      }
      String strUserID = "";
      // Get host name for IPageActions PEI (host name header is incorrect.  we want server name, not host name)
      String strHostName = ConfigHelper.GetConfigVariableAsString(ConfigConstants.SYSTEM_PROPERTIES_SECTION,
       ConfigConstants.MACHINE_NAME_SUBSECTION, ConfigConstants.VALUE_ATTRIBUTE,
       application);
      HTMLElement pageActionComments = null;
      // try catch block for the locked portion of the code.
      try
      {
        // Get User's AActivitySpace cache and cachelist
        tempData.m_smCache = ASCache.GetASCache(session, tempData.m_smSandbox);
        tempData.m_xpmCacheList = ASCache.GetCacheList(session);
        // JF- Set the content language on the PageData
        SetContentLanguageOnPageData(tempData);
        Redirect curRedirect = null;
        // ** Check session authentication **
        curRedirect = CheckSessionAuth(tempData);
        strUserID = LoginHelper.INSTANCE.GetUserSessionID(tempData.GetUserSession());
        log.Info("Current user: " + strUserID + ", " + LoginHelper.INSTANCE.GetUserSessionName(tempData.GetUserSession()));
        // Check for external redirect from Login Actions PEI
        if ((null != curRedirect) && (curRedirect.GetIsExternalURL()))
        {
          SendHTTPRedirect(curRedirect, tempData);
          // The lock will be released and the cleanup will be handled by the finally clause
          return;
        }
        // We want the cached space here.
        IsErrorPageRequired(tempData);
        // Check for appropriate login/logout actions when there's no space name,
        // or there's explicit login or logout.
        if ((tempData.m_xpRequest.GetParameterValue(LoginHelper.PARAM_LOGIN) != null) && (tempData.m_bGuest == true))
        {
          // mj: PARAM_LOGIN indicates that they're guest and have clicked the login
          // link at the top of the page.  If they are actually a guest user, then try
          // to log them in.  The guest user check is to prevent duplicate login attempts
          // if the user has already logged in and are simply hitting the same URL again.
          // This can happen with basic auth requests.
          Redirect loginChallenge = m_Authenticator.GetOrSendLoginChallenge(tempData);
          if (null == loginChallenge)
          {
            // if null, the login challenge has been sent directly
            // by the Authenticator (e.g., in the HTTP Basic Auth case)
            return;
          } else
          {
            curRedirect = loginChallenge;
          }
        } else
        if (tempData.m_xpRequest.GetParameterValue(LoginHelper.PARAM_DOLOGOUT) != null)
        {
          // mj: PARAM_DOLOGOUT means they clicked the logout link.
          bool loggedOut = m_Authenticator.Logout(tempData);
          if (loggedOut)
          {
            return;
          }
        } else
        if ((null == tempData.GetRootSpaceName()) && (null == curRedirect))
        {
          // Handle the case where there is no space name, even though we're logged in.
          // Guest Home will be either login or the default page of subportal
          // For guest, set current space to either the login space (if GuestRedirectToLogin=1)
          // or otherwise to the guest-user MyPage (with Login portlet on it)
          // (if GuestRedirectToLogin=0).		
          bool isGuestRedirectToLogin = LoginHandlers.GetIsGuestRedirectToLoginEnabled(tempData);
          if (tempData.m_bGuest && isGuestRedirectToLogin)
          {
            Redirect loginChallenge = m_Authenticator.GetOrSendLoginChallenge(tempData);
            if (loginChallenge == null)
            {
              // if null, the login challenge has been sent directly 
              // by the Authenticator (e.g., in the HTTP Basic Auth case)
              return;
            } else
            {
              curRedirect = loginChallenge;
            }
          } else
          {
            // run the expression engine to determine the subportal id to use
            IExpressionEvaluator expressionEvaluator = (IExpressionEvaluator) tempData.m_Application.GetCachingManager().GetEntry("ExpressionEvaluator");
            if (null != expressionEvaluator)
            {
              expressionEvaluator.EvaluateExpressionForSubportalAction(tempData.m_asSpace,
               tempData.m_Application, tempData.m_xpRequest, tempData.GetUserSession());
              } else
            {
              log.Warn("The Expression Evaluator was not found.");
            }
            Redirect tempRedirect = (Redirect) LoginHelper.INSTANCE.GetDefaultPageRedirect(new Redirect(),
             tempData.GetUserSession(), tempData.m_xpRequest);
            if (tempRedirect != null)
            {
              curRedirect = tempRedirect;
            } else
            {
              // NOTE: This shouldn't happen -- It would be better to redirect to
              // an error page here.
              Redirect loginChallenge = m_Authenticator.GetOrSendLoginChallenge(tempData);
              if (null == loginChallenge)
              {
                // if null, the login challenge has been sent directly 
                // by the Authenticator (e.g., in the HTTP Basic Auth case)
                return;
              } else
              {
                curRedirect = loginChallenge;
              }
            }
          }
        }
        // Check for HTTP redirect that has been set in the RequestData, if any
        // (set by the GatewayHandler in most cases).
        if (tempData.GetHasHTTPRedirect())
        {
          curRedirect = tempData.GetRedirect();
        }
        // Check for any HTTP redirect (from login challenge or GatewayHandler)
        if ((null != curRedirect) && (curRedirect.GetIsHTTPRedirect()))
        {
          SendHTTPRedirect(curRedirect, tempData);
          // The lock will be released and the cleanup will be handled by the finally clause
          return;
        }
        // ** Invoke Application Logic **
        // IPageActions OnPageStart PEI
        try
        {
          m_MVCController.DoOnPageStartActions(curRedirect, tempData, strHostName);
          // We already set curRedirect into tempData in the DoOnPageStartActions
          curRedirect = null;
        }
        catch (UnauthorizedAccessException uae)
        {
          if (uae.GetRedirect() != null)
          {
            // This exception is due to secured activity space check
            // not successful.
            // Switch security modes through a 302 redirect
            curRedirect = uae.GetRedirect();
          }
        }
        // The 'retry' flag will be set to true when UnauthorizedAccessException is
        // thrown below and we want to display either the login page or error page 
        bool retry = false;
        do
        {
          if ((null != curRedirect) && (curRedirect.GetIsHTTPRedirect()))
          {
            break;
          }
          try
          {
            curRedirect = m_MVCController.HandleRedirect(curRedirect, tempData);
            // if everything is fine (no unauthorized exception) retry is false
            retry = false;
          }
          catch (UnauthorizedAccessException uae)
          {
            Redirect errorRedirect = uae.GetRedirect();
            if (errorRedirect == null)
            {
              UIError myError = null;
              // first bool is bForThisSpace, second is bShowAdminErrors
              IXPEnumerator errors = UIErrorHandler.GetErrors(tempData.m_smSandbox,
               tempData.m_Application, tempData.m_asSpace, true, false);
              if (errors.MoveNext())
              {
                myError = (UIError) errors.GetCurrent();
                // Clear this error to handle the (currently not supported) case where there can be multiple errors
                // This should not cause an XPIllegalStateException because
                // the enumerator is from a temporary collection, which is not
                // affected by the following method.
                UIErrorHandler.ClearError(myError, tempData.m_smSandbox, tempData.m_Application);
                }
              if (!tempData.m_bGuest)
              {
                // If a user is logged in and gets denied access, raise an error
                if (myError == null)
                {
                  // If there was no previous error, raise a No permission error
                  String strErrorMessage = GetString(124, "ptmsgs_infrastructure",
                   tempData);
                  myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,
                   strErrorMessage, strErrorMessage);
                }
                myError.SetDisplayOnSeparatePage(true);
                myError.SetClearWithActivitySpaceChange(false);
                myError.SetActivitySpace(null);
                myError.SetImmediateClear(false);
                UIErrorHandler.RaiseError(myError, tempData.m_smSandbox, tempData.m_Application);
                // Redirect to the error page					
                // Error space can't be cached.  
                // set parent to current so we can redirect back.
                errorRedirect = new Redirect();
                errorRedirect.SetLinkCreateNewSpace(DEFAULTERRORSPACE, tempData.m_asSpace);
                } else
              {
                // But don't raise an error if the guest user gets bounced.  Just redirect them
                // to the login page.
                errorRedirect = m_Authenticator.GetOrSendLoginChallenge(tempData);
                // set parent to current so we can redirect back (plus use cache if possible)
                // this logs an error saying the redirect has been set previously 
                // -- errorRedirect.SetLinkGetSpaceIfCached(errorRedirect.GetSpaceName(), tempData.m_asSpace);
                if (errorRedirect == null)
                {
                  // if null, the login challenge has been sent directly 
                  // by the Authenticator (e.g., in the HTTP Basic Auth case)
                  return;
                } else
                {
                  if (EqualRedirects(errorRedirect, curRedirect))
                  {
                    // Should not happen...
                    // The guest user is not authorized to see the "login page" (depending
                    // on the GuestRedirectToLogin setting may be Login AS or the subportal's
                    // default page)
                    errorRedirect = null;
                    log.Error("Error displaying a login page - UserID: " + strUserID);
                    } else
                  {
                    // DCA - This is for ajax requests
                    tempData.m_xpResponse.AddHeader("PT-HTTPResponse-Type",
                     "SESSION_TIMEOUT");
                    String strLogiURL = ConfigService.getInstance().GetApplicationBaseURLToAS(ConfigService.getInstance().GetNormalizedBaseURL(tempData.GetRequestFullURL()),
                     errorRedirect.GetSpaceName(), ConfigHelper.IsGoingToSubSpace(errorRedirect));
                    // DCA- Append the QS arguments 
                    strLogiURL+=errorRedirect.GetQueryString();
                    tempData.m_xpResponse.AddHeader("PT-Login-URL", strLogiURL);
                    log.Info("redirecting to login page - space name " + errorRedirect.GetSpaceName());
                    }
                }
              }
            }
            if (null != errorRedirect)
            {
              // Display the error page or the login page
              curRedirect = errorRedirect;
              retry = true;
            } else
            {
              // Should not happen...
              // Can not even display the error or login page!
              log.Error("Error displaying an error or login page!");
              HandleError(uae, false, tempData);
              return;
            }
          }
          catch (InterpreterException ie)
          {
            log.Error(ie, "Exception while invoking MVCController.HandleRedirect()");
            if (ie.GetMessageStringID() != InterpreterException.INVALID_ID)
            {
              DisplayBasicError(tempData, ie.GetMessageStringID(), ie.GetResourceFileName(),
               ie.GetNonLocalizedMessage());
            } else
            {
              DisplayBasicError(tempData, 97, ResourceConstants.INFRASTRUCTURE,
               "The server has experienced an error.  Please try again, or contact your system administrator if you continue experiencing problems.");
              }
            // Set the HTTP status code.
            if (ie.GetHTTPStatusCode() != InterpreterException.INVALID_HTTP_STATUS_CODE)
            {
              tempData.m_xpResponse.SetStatus(ie.GetHTTPStatusCode());
            }
            return;
          }
        } while (retry);
        bool bAddPageActionComments = true;
        // ** Handle final DisplayPage or send HTTP redirect ** 
        if ((null != curRedirect) && (curRedirect.GetIsHTTPRedirect()))
        {
          SendHTTPRedirect(curRedirect, tempData);
          bRedirectSent = true;
          bAddPageActionComments = false;
        } else
        {
          // run the expression engine to determine the subportal id to use
          IExpressionEvaluator expressionEvaluator = (IExpressionEvaluator) tempData.m_Application.GetCachingManager().GetEntry("ExpressionEvaluator");
          if (null != expressionEvaluator)
          {
            expressionEvaluator.EvaluateExpressionForSubportalAction(tempData.m_asSpace,
             tempData.m_Application, tempData.m_xpRequest, tempData.GetUserSession());
            bEvaluateRules = false;
          } else
          {
            log.Warn("The Expression Evaluator was not found.");
          }
          // actually display the page
          HandleDisplayPage(curRedirect, tempData);
          bAddPageActionComments = true;
        }
        // IPageActions OnPageFinish PEI
        try
        {
          HTMLElement comments = m_MVCController.DoOnPageFinishActions(tempData,
           strHostName);
          // if not a 302 redirect, add the comment to the page
          if (bAddPageActionComments)
          {
            pageActionComments = comments;
          }
        }
        catch (Exception uae)
        {
          log.Error(uae, "Exception occurs while invoking OnPageFinish PEI.");
          }
      }
      catch (Exception e)
      {
        log.Error(e, "Unable to process request.");
        if (null != tempData.m_asSpace)
        {
          // Clean up the temporary data on the old space before we display
          // the error page.
          tempData.m_asSpace.CleanupTempData();
        }
        // run the expression engine to determine the subportal id to use
        try
        {
          if (bEvaluateRules)
          {
            IExpressionEvaluator expressionEvaluator = (IExpressionEvaluator) tempData.m_Application.GetCachingManager().GetEntry("ExpressionEvaluator");
            expressionEvaluator.EvaluateExpressionForSubportalAction(tempData.m_asSpace,
             tempData.m_Application, tempData.m_xpRequest, tempData.GetUserSession());
            bEvaluateRules = false;
          }
        }
        catch (Exception ex)
        {
          log.Warn(ex, "The Expression Evaluation did not occur.");
        }
        HandleError(e, tempData);
      }
      finally
      {
        // Methods in this finally block should handle all of their own exceptions.
        // Store the current request url on the session so it can be used in the
        // next request as the previous request URL
        if (!bRedirectSent)
        {
          StorePreviousRequestURL(tempData);
        }
        // ** Execute Post Filters **
        DoPostFilter(tempData);
        try
        {
          // Output the response data unless it has already been output
          // during error handling.
          if ((tempData.m_wdPageData.IsDataSet()) && (!tempData.m_wdPageData.HasSentData()))
          {
            tempData.m_ttTimer.SetCheckpoint(CHECKPOINT_DISPLAY_START);
            tempData.m_wdPageData.OutputData();
            tempData.m_ttTimer.SetCheckpoint(CHECKPOINT_DISPLAY_END);
          }
          DisplayComments(tempData, pageActionComments, strHostName);
          tempData.m_xpResponse.GetOutputStream().Close();
        }
        catch (Exception t)
        {
          log.Error(t, "Exception thrown while sending out WebData output.");
          }
        // WilliamA - Moved the clean up code into RequestDataFilter
        // since the clean up needs to be done while the session is locked
        // (bug 39578).
        // -- Cleanup(tempData);
        if (null != tempData)
        {
          // null out data in case we lost a reference to this object.
          if (null != tempData.m_wdPageData)
          {
            tempData.m_wdPageData.ClearAllData();
          }
          if (null != tempData.m_xpRequest)
          {
            tempData.m_xpRequest.ClearUnderlyingObject();
          }
          if (null != tempData.m_xpResponse)
          {
            tempData.m_xpResponse.ClearUnderlyingObject();
          }
        }
        // Bug 37323 - Make sure leaving handlerequest ACTION trace does not use cached user ID.
        strUserID = LoginHelper.INSTANCE.GetUserSessionID(tempData.GetUserSession());
        log.Action("Leaving handleRequest - User ID: " + strUserID);
      }
    }
    
    /// <summary>
    /// Checks if the provided url is not a form POST URL (i.e. gateway,
    /// or REST API). 
    /// NOTE: This is based off of a static list of patterns for non-form
    /// POST URLs.  This list is initialized by the GatewayHelpers static
    /// initializer, so the list will not be complete until that class
    /// is used (i.e. the second request).
    /// </summary>
    /// <param name="uri">url of the request to check</param>
    /// <returns>
    /// true means that this request is not a form post request.
    /// false means we don't know.
    /// </returns>
    protected static bool IsNotFormRequest(String uri)
    {
      if (null == uri || "".Equals(uri))
      {
        return false;
      }
      // iterate through the patterns in the list of non-form POST uri patterns
      for (int i = 0;i < NON_FORM_PATTERNS.GetSize();i++)
      {
        String pattern = (String) NON_FORM_PATTERNS.GetElement(i);
        if (XPStringUtility.IndexOf(uri, pattern) != -1)
        {
          return true;
        }
      }
      return false;
    }
    
    /// <summary>
    /// Method: \u0009OutputStartupFailure
    /// Purpose:\u0009Helper method to output startup failure message.  Uses diagnostic notification
    /// arrays to form recommendation. 
    /// </summary>
    public static void OutputStartupFailure(IXPRequest request, IXPResponse response)
    {
      // Output the diagnostic tool startup report to the browser
      if (response != null)
      {
        // Security check: Is this coming from the local machine?
        bool bIsSecure = false;
        String clientIP = request.GetClientAddress();
        clientIP = XPStringUtility.Trim(clientIP);
        PlatformDNS localhost = new PlatformDNS();
        if (localhost.init())
        {
          String localhostIP = localhost.getLocalIPAddrAsString();
          if (localhostIP.CompareTo(clientIP) == 0)
          {
            // the request is local
            bIsSecure = true;
          }
        }
        if (XPStringUtility.CompareIgnoreCase(clientIP, "127.0.0.1") == 0)
        {
          // the request is local
          bIsSecure = true;
        }
        // Verify the override env var is not set
        String strVal = XPEnvironment.GetEnvironmentVariable("PT_REMOTE_ERR");
        if ((strVal != null) && (XPStringUtility.CompareIgnoreCase(strVal,
         "true") == 0))
        {
          bIsSecure = true;
        }
        // Display the error
        if (bIsSecure)
        {
          String[] report = AppWarmUp.getStartupNotificationErrorReport();
          if (report != null)
          {
            String str = GenerateFailureHTML();
            response.Write(str);
            log.Error("System did not startup successfully 6.  Full HTML failure: " + str);
            } else
          // not sure what this is for
          if (null != ASManager.INSTANCE.m_StartupFailure)
          {
            response.Write("<HTML><head><title>Startup Failure Error Message</title>" + "<meta http-equiv=\'Content-Type\' content=\'text/html; charset=iso-8859-1\'></head>" + "<BODY><font face=\'verdana,arial,helvetica\'>");
            response.Write("<H1>The server has experienced an error on startup.</H1><br><p>Error details:<br>");
            response.Write("<!-- Exception message: " + ASManager.INSTANCE.m_StartupFailure.Message + " -->");
            response.Write("</font></BODY></HTML>");
            log.Error(ASManager.INSTANCE.m_StartupFailure, "System did not startup successfully 1; Exception: " + ASManager.INSTANCE.m_StartupFailure.Message);
            } else
          {
            response.Write("<HTML><head><title>Startup Failure Error Message</title>" + "<meta http-equiv=\'Content-Type\' content=\'text/html; charset=iso-8859-1\'></head>" + "<BODY><font face=\'verdana,arial,helvetica\'>");
            response.Write("<H1>The server has experienced an error on startup.</H1><br><p>Error details: No further detail.<br>");
            response.Write("</font></BODY></HTML>");
            log.Error(ASManager.INSTANCE.m_StartupFailure, "System did not startup successfully 2.");
            }
        } else
        {
          String[] report = AppWarmUp.getStartupNotificationErrorReport();
          if (report != null)
          {
            String str = GenerateFailureHTMLInsecure();
            response.Write(str);
            log.Error("System did not startup successfully 5.  Full HTML failure: " + str);
            } else
          // not sure what this is for
          if (null != ASManager.INSTANCE.m_StartupFailure)
          {
            response.Write("<HTML><head><title>Startup Failure Error Message</title>" + "<meta http-equiv=\'Content-Type\' content=\'text/html; charset=iso-8859-1\'></head>" + "<BODY><font face=\'verdana,arial,helvetica\'>");
            response.Write("<H1>The server has experienced an error on startup.</H1><br><p>Error details:<br>");
            response.Write("To enable the details of this specific error message to be viewable on remote machines, please create a system environment variable named PT_REMOTE_ERR and set it to \'true\'.");
            response.Write("</font></BODY></HTML>");
            log.Error(ASManager.INSTANCE.m_StartupFailure, "System did not startup successfully 3; Exception: " + ASManager.INSTANCE.m_StartupFailure.Message);
            } else
          {
            response.Write("<HTML><head><title>Startup Failure Error Message</title>" + "<meta http-equiv=\'Content-Type\' content=\'text/html; charset=iso-8859-1\'></head>" + "<BODY><font face=\'verdana,arial,helvetica\'>");
            response.Write("<H1>The server has experienced an error on startup.</H1><br><p>Error details: To enable the details of this specific error message to be viewable on remote machines, please create a system environment variable named PT_REMOTE_ERR and set it to \'true\'.<br>");
            response.Write("</font></BODY></HTML>");
            log.Error(ASManager.INSTANCE.m_StartupFailure, "System did not startup successfully 4.");
            }
        }
      } else
      // no handler exists - nothing to print to
      {
        throw new XPException("STARTUP FAILED.  You cannot proceed.  Application, session, or response classes are null.");
        }
    }
    
    /// <summary>
    /// Method: \u0009GenerateDiagnosticErrorPage
    /// Purpose:\u0009Helper method to output an error as a diagnostic style startup error page.
    /// Uses diagnostic notification arrays to form recommendation.  Useful if you
    /// have a fundamental error that is similar to a startup check, such as licensing.
    /// Notes:   Be sure to supply at least one element on the eventlog vector; the reportlog can
    /// be null or a series of strings.  Note also that the eventlog should ideally have valid 
    /// values that tie back to a real startup task in order to load an appropriate 
    /// recommendation.  Here, for example, are some licensing settings:
    /// EventLog[] eventVector = new EventLog[1];
    /// eventVector[0].taskID = PTStartupTasks.VALIDATELICENSE.getOrdinal();
    /// eventVector[0].taskStep = 1;
    /// eventVector[0].taskName = "ValidateLicense";
    /// eventVector[0].platformName = Platform.getCurPlatform().getName();
    /// eventVector[0].errorMessage = "Session License Failure";
    /// eventVector[0].errorDescription = "You have exceeded your allowed session count on the evaluation of your license.";
    /// eventVector[0].exceptionRecommendation = "Contact BEA sales and purchase a production license.";
    /// eventVector[0].retVal = PT_STARTUP_RESULTCODES.FATAL;
    /// GenerateDiagnosticErrorPage(request, eventVector, null);
    /// </summary>
    public static HTMLElementCollection GenerateDiagnosticErrorPage(IXPRequest request,
     EventLog[] eventVector, XPArrayList reportLog)
    {
      HTMLElementCollection resultGroup = new HTMLElementCollection();
      bool bIsSecure = false;
      // Store the new override logs
      AppWarmUp.setStartupNotificationErrorLogs(eventVector, reportLog);
      // Security check: Is this coming from the local machine?
      String clientIP = request.GetClientAddress();
      clientIP = XPStringUtility.Trim(clientIP);
      PlatformDNS localhost = new PlatformDNS();
      if (localhost.init())
      {
        String localhostIP = localhost.getLocalIPAddrAsString();
        if (localhostIP.CompareTo(clientIP) == 0)
        {
          // the request is local
          bIsSecure = true;
        }
      }
      if (XPStringUtility.CompareIgnoreCase(clientIP, "127.0.0.1") == 0)
      {
        // the request is local
        bIsSecure = true;
      }
      // Verify the override env var is not set
      String strVal = XPEnvironment.GetEnvironmentVariable("PT_REMOTE_ERR");
      if ((strVal != null) && (XPStringUtility.CompareIgnoreCase(strVal, "true") == 0))
      {
        bIsSecure = true;
      }
      // Generate the error HTML
      String[] report = AppWarmUp.getStartupNotificationErrorReport();
      String errMsg = "";
      if (bIsSecure)
      {
        if (report != null)
        {
          errMsg = GenerateFailureHTML();
        } else
        {
          errMsg = "<HTML><head><title>Startup Failure Error Message</title>" + "<meta http-equiv=\'Content-Type\' content=\'text/html; charset=iso-8859-1\'></head>" + "<BODY><font face=\'verdana,arial,helvetica\'>" + "<H1>The server has experienced an error on startup.</H1>" + "<br><p>Error details: No further detail.<br>" + "</font></BODY></HTML>";
          }
      } else
      {
        if (report != null)
        {
          errMsg = GenerateFailureHTMLInsecure();
        } else
        {
          errMsg = "<HTML><head><title>Startup Failure Error Message</title>" + "<meta http-equiv=\'Content-Type\' content=\'text/html; charset=iso-8859-1\'></head>" + "<BODY><font face=\'verdana,arial,helvetica\'>" + "<H1>The server has experienced an error on startup.</H1>" + "<br><p>Error details: No further detail.<br>" + "</font></BODY></HTML>";
          }
      }
      // format html output
      try
      {
        HTMLElement errorMsgElt = new HTMLGenericElement(errMsg);
        resultGroup.AddInnerHTMLElement(errorMsgElt);
      }
      catch (Exception e)
      {
        resultGroup.AddInnerHTMLElement(new HTMLGenericElement("Error generating error message HTMLElement: " + e.Message));
        }
      log.Info("Error message output as diagnostic.");
      return resultGroup;
    }
    
    /// <summary>
    /// Uses the startup diagnostics strings to form an HTML message.
    /// </summary>
    /// <returns></returns>
    public static String GenerateFailureHTML()
    {
      String errorHtml = "";
      EventLog[] errLogs = AppWarmUp.getStartupNotificationErrorLogs();
      String[] logReport = AppWarmUp.getStartupNotificationErrorReport();
      bool bErrDisplayed = false;
      if (errLogs != null && errLogs.Length > 0)
      {
        EventLog primaryErr = errLogs[0];
        if ((primaryErr != null) && (primaryErr.taskName != null))
        {
          errorHtml = "<HTML><HEAD><TITLE>Aqualogic Interaction Startup Error</TITLE>";
          errorHtml+="<meta http-equiv=\'Content-Type\' content=\'text/html; charset=iso-8859-1\'></HEAD>";
          errorHtml+="<BODY style=\'background-color: #FFFFFF; font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif;\'>";
          errorHtml+="<TABLE BORDER=\'0\' WIDTH=\'95%\' CELLPADDING=\'1\' CELLSPACING=\'0\'>";
          errorHtml+="<HR style=\'width: 95%; text-align: left; color: #999; height: 1px;\' align=\'left\'>";
          errorHtml+="<H2 style=\'font-size: 18px; font-weight: bold;\'>Task \'" + primaryErr.taskName + "\' Failed on Startup</H2>";
          errorHtml+="<PRE>Task #" + (primaryErr.taskID + 1) + " (" + primaryErr.taskName + ")<br>";
          errorHtml+="   |<br>";
          errorHtml+="   +--<b>Step #" + (primaryErr.taskStep + 1) + "</b><br>";
          errorHtml+="</PRE>";
          errorHtml+="<HR style=\'width: 95%; text-align: left; color: #999; height: 1px;\' align=\'left\'><DL>";
          errorHtml+="<DT>error: ";
          errorHtml+="   <B>" + primaryErr.errorMessage + "</B></DL><P>";
          errorHtml+="<BR/>Platform: <I>" + primaryErr.platformName + "</I><BR/><BR/>";
          errorHtml+="</P><HR style=\'width: 95%; text-align: left; color: #999; height: 1px;\' align=\'left\'>";
          // if there is a recommendation
          String exceptionStr = "";
          String messageStr = "";
          String secondLine = "";
          XPException ex = primaryErr.retVal.getException();
          if (ex != null)
          {
            exceptionStr = RecommendationHelper.ConvertExceptionToShortString(ex,
             false, false);
            messageStr = RecommendationHelper.ConvertExceptionToShortString(ex,
             true, false);
            secondLine = RecommendationHelper.ConvertExceptionToShortString(ex,
             false, true);
          }
          errorHtml+="<TABLE style=\'solid #C9CAB7;\' BORDER=\'1\' CELLPADDING=\'3\' CELLSPACING=\'0\' WIDTH=\'95%\'>";
          errorHtml+="\t<TR BGCOLOR=\'#CCCCFF\' style=\'background: #DDE4D9;\' width=\'95%\'>";
          errorHtml+="\t\t<TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\' COLSPAN=2>";
          errorHtml+="\t\t\t<H1 style=\'font-size: 15px; font-weight: bold; display: inline-block; padding: 3px;\'>Recommendation Summary</H1>";
          errorHtml+="\t\t</TD>";
          errorHtml+="\t</TR>";
          errorHtml+="\t<TR BGCOLOR=\'white\' style=\'background: #FFFFFF;\' width=\'95%\'>";
          errorHtml+="\t\t<TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\' ALIGN=\'right\' VALIGN=\'top\' WIDTH=\'22%\' >";
          errorHtml+="\t\t<FONT SIZE=\'-1\'>";
          errorHtml+="\t\t\t<CODE style=\'font-size: 12px; font-family: \"courier new\", courier, monospace;\'>&nbsp;";
          errorHtml+="\t\t    \t<SPAN style=\'color: #007711;\'>failure</SPAN>";
          errorHtml+="\t\t    </CODE>";
          errorHtml+="\t\t </FONT>";
          errorHtml+="\t      </TD>";
          errorHtml+="\t      <TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\'>";
          errorHtml+="\t\t " + primaryErr.retVal.getResultCode().getName() + ": " + primaryErr.errorMessage;
          errorHtml+="\t      </TD>";
          errorHtml+="\t   </TR>";
          errorHtml+="\t<TR BGCOLOR=\'white\' style=\'background: #FFFFFF;\' width=\'95%\'>";
          errorHtml+="\t\t<TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\' ALIGN=\'right\' VALIGN=\'top\' WIDTH=\'22%\' >";
          errorHtml+="\t\t<FONT SIZE=\'-1\'>";
          errorHtml+="\t\t\t<CODE style=\'font-size: 12px; font-family: \"courier new\", courier, monospace;\'>&nbsp;";
          errorHtml+="\t\t    \t<SPAN style=\'color: #007711;\'>recommendation</SPAN>";
          errorHtml+="\t\t    </CODE>";
          errorHtml+="\t\t </FONT>";
          errorHtml+="\t      </TD>";
          errorHtml+="\t      <TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\'>";
          errorHtml+="\t\t " + primaryErr.errorDescription + "";
          errorHtml+="\t      </TD>";
          errorHtml+="\t   </TR>";
          errorHtml+="\t   <TR BGCOLOR=\'white\' style=\'background: #FFFFFF;\' width=\'95%\'>";
          errorHtml+="\t      <TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\' ALIGN=\'right\' VALIGN=\'top\' WIDTH=\'22%\' >";
          errorHtml+="\t\t\t<FONT SIZE=\'-1\'>";
          errorHtml+="\t\t\t\t<CODE style=\'font-size: 12px; font-family: \"courier new\", courier, monospace;\'>&nbsp;";
          errorHtml+="\t\t\t\t\t<SPAN style=\'color: #007711;\'>exception</SPAN>";
          errorHtml+="\t    \t\t\t<B>" + exceptionStr + "</B>";
          errorHtml+="\t\t\t\t</CODE>";
          errorHtml+="\t\t\t</FONT>";
          errorHtml+="\t\t</TD>";
          errorHtml+="\t\t<TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\' WIDTH=\'100%\' >";
          errorHtml+="\t\t\t" + primaryErr.exceptionRecommendation + "";
          errorHtml+="\t\t</TD>";
          errorHtml+="\t  </TR>";
          if (messageStr.Length > 1)
          {
            errorHtml+="\t   <TR BGCOLOR=\'white\' style=\'background: #FFFFFF;\' width=\'95%\'>";
            errorHtml+="\t      <TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\' ALIGN=\'right\' VALIGN=\'top\' WIDTH=\'22%\' >";
            errorHtml+="\t\t\t<FONT SIZE=\'-1\'>";
            errorHtml+="\t\t\t\t<CODE style=\'font-size: 12px; font-family: \"courier new\", courier, monospace;\'>&nbsp;";
            errorHtml+="\t\t\t\t\t<SPAN style=\'color: #007711;\'>message</SPAN>";
            errorHtml+="\t\t\t\t</CODE>";
            errorHtml+="\t\t\t</FONT>";
            errorHtml+="\t\t</TD>";
            errorHtml+="\t\t<TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\' WIDTH=\'100%\' >";
            errorHtml+="\t\t\t" + messageStr + "";
            errorHtml+="\t\t</TD>";
            errorHtml+="\t  </TR>";
          }
          if (secondLine.Length > 1)
          {
            errorHtml+="\t   <TR BGCOLOR=\'white\' style=\'background: #FFFFFF;\' width=\'95%\'>";
            errorHtml+="\t      <TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\' ALIGN=\'right\' VALIGN=\'top\' WIDTH=\'22%\' >";
            errorHtml+="\t\t\t<FONT SIZE=\'-1\'>";
            errorHtml+="\t\t\t\t<CODE style=\'font-size: 12px; font-family: \"courier new\", courier, monospace;\'>&nbsp;";
            errorHtml+="\t\t\t\t\t<SPAN style=\'color: #007711;\'>secondary exception</SPAN>";
            errorHtml+="\t\t\t\t</CODE>";
            errorHtml+="\t\t\t</FONT>";
            errorHtml+="\t\t</TD>";
            errorHtml+="\t\t<TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\' WIDTH=\'100%\' >";
            errorHtml+="\t\t\t" + secondLine + "";
            errorHtml+="\t\t</TD>";
            errorHtml+="\t  </TR>";
          }
          errorHtml+="</TABLE>";
          // straight logs
          if (logReport.Length > 1)
          {
            errorHtml+="<TABLE style=\'solid #C9CAB7;\' BORDER=\'1\' CELLPADDING=\'3\' CELLSPACING=\'0\' WIDTH=\'95%\'>";
            errorHtml+="\t<TR style=\'background: #CCCCEE;\'>";
            errorHtml+="\t\t<TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\' COLSPAN=2 WIDTH=\'100%\' >";
            errorHtml+="\t\t<H1 style=\'font-size: 15px; font-weight: bold; display: inline-block; padding: 3px;\'>Startup Log</H1>";
            errorHtml+="\t\t</TD>";
            errorHtml+="\t</TR>";
            errorHtml+="\t<TR BGCOLOR=\'white\' style=\'background: #FFFFFF;\'>";
            errorHtml+="\t      <TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\' ALIGN=\'right\' VALIGN=\'top\' WIDTH=\'22%\' >";
            errorHtml+="\t\t\t<FONT SIZE=\'-1\'>";
            errorHtml+="\t\t\t\t<CODE style=\'font-size: 12px; font-family: \"courier new\", courier, monospace;\'>&nbsp;";
            errorHtml+="\t\t\t\t\t<SPAN style=\'color: #007711;\'>raw startup logs</SPAN>";
            errorHtml+="\t\t\t\t</CODE>";
            errorHtml+="\t\t\t</FONT>";
            errorHtml+="\t\t</TD>";
            errorHtml+="\t\t<TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\' ALIGN=\'left\' VALIGN=\'top\' WIDTH=\'100%\' >";
            errorHtml+="\t\t\t<FONT SIZE=\'-1\'>";
            errorHtml+="\t\t\t<CODE style=\'font-size: 12px; font-family: \"courier new\", courier, monospace;\'>&nbsp;";
            errorHtml+="\t\t\t\t<SPAN style=\'color: #666666;\'>";
            // output the simple logs
            int loop;
            for (loop = 0;loop < logReport.Length;++loop)
            {
              if (logReport[loop] != null)
              {
                String str = logReport[loop];
                errorHtml+=XPStringUtility.Trim(str);
                errorHtml+="<br>";
              }
            }
            // finish the straight logs HTML
            errorHtml+="</SPAN>";
            errorHtml+="\t\t\t</CODE>";
            errorHtml+="\t\t\t</FONT>";
            errorHtml+="\t\t</TD>";
            errorHtml+="\t</TR>";
            errorHtml+="</TABLE>";
            errorHtml+="</TABLE>";
            errorHtml+="</BODY>";
            errorHtml+="</HTML>";
          }
          bErrDisplayed = true;
        }
      }
      // Handle pointer errors in the error display itself
      if (!bErrDisplayed)
      {
        errorHtml = "<HTML><HEAD><TITLE>Aqualogic Interaction Startup Error</TITLE>";
        errorHtml+="<meta http-equiv=\'Content-Type\' content=\'text/html; charset=iso-8859-1\'></HEAD>";
        errorHtml+="<BODY style=\'background-color: #FFFFFF; font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif;\'>";
        errorHtml+="<TABLE BORDER=\'0\' WIDTH=\'95%\' CELLPADDING=\'1\' CELLSPACING=\'0\'>";
        errorHtml+="<HR style=\'width: 95%; text-align: left; color: #999; height: 1px;\' align=\'left\'>";
        errorHtml+="<H2 style=\'font-size: 18px; font-weight: bold;\'>Task \'Unknown\' Failed on Startup</H2>";
        errorHtml+="<PRE>Task #0 (unknown or undefined)<br>";
        errorHtml+="</PRE>";
        errorHtml+="<HR style=\'width: 95%; text-align: left; color: #999; height: 1px;\' align=\'left\'><DL>";
        errorHtml+="<DT>error: ";
        errorHtml+="   <B>General Failure</B></DL><P>";
        Platform platform = Platform.getCurPlatform();
        errorHtml+="<BR/>Platform: <I>" + platform.getName() + "</I><BR/><BR/>";
        errorHtml+="</P><HR style=\'width: 95%; text-align: left; color: #999; height: 1px;\' align=\'left\'>";
        // if there is a recommendxation
        String exceptionStr = "Run the diagnostic tool for more detail.";
        String messageStr = "";
        String secondLine = "";
        errorHtml+="<TABLE style=\'solid #C9CAB7;\' BORDER=\'1\' CELLPADDING=\'3\' CELLSPACING=\'0\' WIDTH=\'95%\'>";
        errorHtml+="\t<TR BGCOLOR=\'#CCCCFF\' style=\'background: #DDE4D9;\' width=\'95%\'>";
        errorHtml+="\t\t<TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\' COLSPAN=2>";
        errorHtml+="\t\t\t<H1 style=\'font-size: 15px; font-weight: bold; display: inline-block; padding: 3px;\'>Recommendation Summary</H1>";
        errorHtml+="\t\t</TD>";
        errorHtml+="\t</TR>";
        errorHtml+="\t<TR BGCOLOR=\'white\' style=\'background: #FFFFFF;\' width=\'95%\'>";
        errorHtml+="\t\t<TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\' ALIGN=\'right\' VALIGN=\'top\' WIDTH=\'22%\' >";
        errorHtml+="\t\t<FONT SIZE=\'-1\'>";
        errorHtml+="\t\t\t<CODE style=\'font-size: 12px; font-family: \"courier new\", courier, monospace;\'>&nbsp;";
        errorHtml+="\t\t    \t<SPAN style=\'color: #007711;\'>failure</SPAN>";
        errorHtml+="\t\t    </CODE>";
        errorHtml+="\t\t </FONT>";
        errorHtml+="\t      </TD>";
        errorHtml+="\t      <TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\'>";
        errorHtml+="\t\t FATAL: Startup halted.";
        errorHtml+="\t      </TD>";
        errorHtml+="\t   </TR>";
        errorHtml+="\t<TR BGCOLOR=\'white\' style=\'background: #FFFFFF;\' width=\'95%\'>";
        errorHtml+="\t\t<TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\' ALIGN=\'right\' VALIGN=\'top\' WIDTH=\'22%\' >";
        errorHtml+="\t\t<FONT SIZE=\'-1\'>";
        errorHtml+="\t\t\t<CODE style=\'font-size: 12px; font-family: \"courier new\", courier, monospace;\'>&nbsp;";
        errorHtml+="\t\t    \t<SPAN style=\'color: #007711;\'>recommendation</SPAN>";
        errorHtml+="\t\t    </CODE>";
        errorHtml+="\t\t </FONT>";
        errorHtml+="\t      </TD>";
        errorHtml+="\t      <TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\'>";
        errorHtml+="\t\t Unknown or undefined error.";
        errorHtml+="\t      </TD>";
        errorHtml+="\t   </TR>";
        errorHtml+="\t   <TR BGCOLOR=\'white\' style=\'background: #FFFFFF;\' width=\'95%\'>";
        errorHtml+="\t      <TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\' ALIGN=\'right\' VALIGN=\'top\' WIDTH=\'22%\' >";
        errorHtml+="\t\t\t<FONT SIZE=\'-1\'>";
        errorHtml+="\t\t\t\t<CODE style=\'font-size: 12px; font-family: \"courier new\", courier, monospace;\'>&nbsp;";
        errorHtml+="\t\t\t\t\t<SPAN style=\'color: #007711;\'>exception</SPAN>";
        errorHtml+="\t    \t\t\t<B>Unknown</B>";
        errorHtml+="\t\t\t\t</CODE>";
        errorHtml+="\t\t\t</FONT>";
        errorHtml+="\t\t</TD>";
        errorHtml+="\t\t<TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\' WIDTH=\'100%\' >";
        errorHtml+="\t\t\t" + exceptionStr + "";
        errorHtml+="\t\t</TD>";
        errorHtml+="\t  </TR>";
        if (messageStr.Length > 1)
        {
          errorHtml+="\t   <TR BGCOLOR=\'white\' style=\'background: #FFFFFF;\' width=\'95%\'>";
          errorHtml+="\t      <TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\' ALIGN=\'right\' VALIGN=\'top\' WIDTH=\'22%\' >";
          errorHtml+="\t\t\t<FONT SIZE=\'-1\'>";
          errorHtml+="\t\t\t\t<CODE style=\'font-size: 12px; font-family: \"courier new\", courier, monospace;\'>&nbsp;";
          errorHtml+="\t\t\t\t\t<SPAN style=\'color: #007711;\'>message</SPAN>";
          errorHtml+="\t\t\t\t</CODE>";
          errorHtml+="\t\t\t</FONT>";
          errorHtml+="\t\t</TD>";
          errorHtml+="\t\t<TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\' WIDTH=\'100%\' >";
          errorHtml+="\t\t\t" + messageStr + "";
          errorHtml+="\t\t</TD>";
          errorHtml+="\t  </TR>";
        }
        if (secondLine.Length > 1)
        {
          errorHtml+="\t   <TR BGCOLOR=\'white\' style=\'background: #FFFFFF;\' width=\'95%\'>";
          errorHtml+="\t      <TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\' ALIGN=\'right\' VALIGN=\'top\' WIDTH=\'22%\' >";
          errorHtml+="\t\t\t<FONT SIZE=\'-1\'>";
          errorHtml+="\t\t\t\t<CODE style=\'font-size: 12px; font-family: \"courier new\", courier, monospace;\'>&nbsp;";
          errorHtml+="\t\t\t\t\t<SPAN style=\'color: #007711;\'>secondary exception</SPAN>";
          errorHtml+="\t\t\t\t</CODE>";
          errorHtml+="\t\t\t</FONT>";
          errorHtml+="\t\t</TD>";
          errorHtml+="\t\t<TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\' WIDTH=\'100%\' >";
          errorHtml+="\t\t\t" + secondLine + "";
          errorHtml+="\t\t</TD>";
          errorHtml+="\t  </TR>";
        }
        errorHtml+="</TABLE>";
        // straight logs
        errorHtml+="<TABLE style=\'solid #C9CAB7;\' BORDER=\'1\' CELLPADDING=\'3\' CELLSPACING=\'0\' WIDTH=\'95%\'>";
        errorHtml+="\t<TR style=\'background: #CCCCEE;\'>";
        errorHtml+="\t\t<TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\' COLSPAN=2 WIDTH=\'100%\' >";
        errorHtml+="\t\t<H1 style=\'font-size: 15px; font-weight: bold; display: inline-block; padding: 3px;\'>Startup Log</H1>";
        errorHtml+="\t\t</TD>";
        errorHtml+="\t</TR>";
        errorHtml+="\t<TR BGCOLOR=\'white\' style=\'background: #FFFFFF;\'>";
        errorHtml+="\t      <TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\' ALIGN=\'right\' VALIGN=\'top\' WIDTH=\'22%\' >";
        errorHtml+="\t\t\t<FONT SIZE=\'-1\'>";
        errorHtml+="\t\t\t\t<CODE style=\'font-size: 12px; font-family: \"courier new\", courier, monospace;\'>&nbsp;";
        errorHtml+="\t\t\t\t\t<SPAN style=\'color: #007711;\'>raw startup logs</SPAN>";
        errorHtml+="\t\t\t\t</CODE>";
        errorHtml+="\t\t\t</FONT>";
        errorHtml+="\t\t</TD>";
        errorHtml+="\t\t<TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\' ALIGN=\'left\' VALIGN=\'top\' WIDTH=\'100%\' >";
        errorHtml+="\t\t\t<FONT SIZE=\'-1\'>";
        errorHtml+="\t\t\t<CODE style=\'font-size: 12px; font-family: \"courier new\", courier, monospace;\'>&nbsp;";
        errorHtml+="\t\t\t\t<SPAN style=\'color: #666666;\'>";
        // finish the straight logs HTML
        errorHtml+="</SPAN>";
        errorHtml+="\t\t\t</CODE>";
        errorHtml+="\t\t\t</FONT>";
        errorHtml+="\t\t</TD>";
        errorHtml+="\t</TR>";
        errorHtml+="</TABLE>";
        errorHtml+="</TABLE>";
        errorHtml+="</BODY>";
        errorHtml+="</HTML>";
      }
      return errorHtml;
    }
    
    /// <summary>
    /// Insecure, outward-facing, simplified HTML for startup failure message
    /// </summary>
    /// <returns></returns>
    public static String GenerateFailureHTMLInsecure()
    {
      String errorHtml = "";
      EventLog[] errLogs = AppWarmUp.getStartupNotificationErrorLogs();
      bool bErrDisplayed = false;
      if (errLogs != null && errLogs.Length > 0)
      {
        EventLog primaryErr = errLogs[0];
        if ((primaryErr != null) && (primaryErr.taskName != null))
        {
          errorHtml = "<HTML><HEAD><TITLE>Aqualogic Interaction Startup Error</TITLE>";
          errorHtml+="<meta http-equiv=\'Content-Type\' content=\'text/html; charset=iso-8859-1\'></HEAD>";
          errorHtml+="<BODY style=\'background-color: #FFFFFF; font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif;\'>";
          errorHtml+="<TABLE BORDER=\'0\' WIDTH=\'95%\' CELLPADDING=\'1\' CELLSPACING=\'0\'>";
          errorHtml+="<HR style=\'width: 95%; text-align: left; color: #999; height: 1px;\' align=\'left\'>";
          // errorHtml += "<H2 style='font-size: 18px; font-weight: bold;'>Task '"+primaryErr.taskName+"' Failed on Startup</H2>";
          errorHtml+="<H2 style=\'font-size: 18px; font-weight: bold;\'>Task Failed on Startup</H2>";
          // ("+primaryErr.taskName+")<br>";
          errorHtml+="<PRE>Task #" + (primaryErr.taskID + 1) + " <br>";
          errorHtml+="   |<br>";
          errorHtml+="   +--<b>Step #" + (primaryErr.taskStep + 1) + "</b><br>";
          errorHtml+="</PRE>";
          errorHtml+="<HR style=\'width: 95%; text-align: left; color: #999; height: 1px;\' align=\'left\'><DL>";
          errorHtml+="<DT>error: ";
          errorHtml+="   <B>please refer to system administrator</B></DL><P>";
          errorHtml+="<BR/>";
          errorHtml+="</P><HR style=\'width: 95%; text-align: left; color: #999; height: 1px;\' align=\'left\'>";
          errorHtml+="<TABLE style=\'solid #C9CAB7;\' BORDER=\'1\' CELLPADDING=\'3\' CELLSPACING=\'0\' WIDTH=\'95%\'>";
          errorHtml+="\t<TR BGCOLOR=\'#CCCCFF\' style=\'background: #DDE4D9;\' width=\'95%\'>";
          errorHtml+="\t\t<TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\' COLSPAN=2>";
          errorHtml+="\t\t\t<H1 style=\'font-size: 15px; font-weight: bold; display: inline-block; padding: 3px;\'>Recommendation Summary</H1>";
          errorHtml+="\t\t</TD>";
          errorHtml+="\t</TR>";
          errorHtml+="\t<TR BGCOLOR=\'white\' style=\'background: #FFFFFF;\' width=\'95%\'>";
          errorHtml+="\t\t<TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\' ALIGN=\'right\' VALIGN=\'top\' WIDTH=\'22%\' >";
          errorHtml+="\t\t<FONT SIZE=\'-1\'>";
          errorHtml+="\t\t\t<CODE style=\'font-size: 12px; font-family: \"courier new\", courier, monospace;\'>&nbsp;";
          errorHtml+="\t\t    \t<SPAN style=\'color: #007711;\'>failure</SPAN>";
          errorHtml+="\t\t    </CODE>";
          errorHtml+="\t\t </FONT>";
          errorHtml+="\t      </TD>";
          errorHtml+="\t      <TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\'>";
          errorHtml+="\t\t An application error occurred on the portal.<P>The portal by default prevents the details of the application error from being viewed remotely (for security reasons). It could, however, be viewed by browsers running on the local server machine. ";
          errorHtml+="\t      </TD>";
          errorHtml+="\t   </TR>";
          errorHtml+="\t<TR BGCOLOR=\'white\' style=\'background: #FFFFFF;\' width=\'95%\'>";
          errorHtml+="\t\t<TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\' ALIGN=\'right\' VALIGN=\'top\' WIDTH=\'22%\' >";
          errorHtml+="\t\t<FONT SIZE=\'-1\'>";
          errorHtml+="\t\t\t<CODE style=\'font-size: 12px; font-family: \"courier new\", courier, monospace;\'>&nbsp;";
          errorHtml+="\t\t    \t<SPAN style=\'color: #007711;\'>recommendation</SPAN>";
          errorHtml+="\t\t    </CODE>";
          errorHtml+="\t\t </FONT>";
          errorHtml+="\t      </TD>";
          errorHtml+="\t      <TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\'>";
          errorHtml+="\t\t The detailed error message can be viewed by accessing the portal using a browser on the <i>local</i> machine.<P>To enable the details of this specific error message to be viewable on <i>remote</i> machines, please create a system environment variable named PT_REMOTE_ERR and set it to \'true\'.";
          errorHtml+="\t      </TD>";
          errorHtml+="\t   </TR>";
          errorHtml+="</TABLE>";
          errorHtml+="</TABLE>";
          errorHtml+="</BODY>";
          errorHtml+="</HTML>";
          bErrDisplayed = true;
        }
      }
      // Handle pointer errors in the error display itself
      if (!bErrDisplayed)
      {
        errorHtml = "<HTML><HEAD><TITLE>Aqualogic Interaction Startup Error</TITLE>";
        errorHtml+="<meta http-equiv=\'Content-Type\' content=\'text/html; charset=iso-8859-1\'></HEAD>";
        errorHtml+="<BODY style=\'background-color: #FFFFFF; font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif;\'>";
        errorHtml+="<TABLE BORDER=\'0\' WIDTH=\'95%\' CELLPADDING=\'1\' CELLSPACING=\'0\'>";
        errorHtml+="<HR style=\'width: 95%; text-align: left; color: #999; height: 1px;\' align=\'left\'>";
        errorHtml+="<H2 style=\'font-size: 18px; font-weight: bold;\'>Task \'Unknown\' Failed on Startup</H2>";
        errorHtml+="<PRE>Task #0 (unknown or undefined)<br>";
        errorHtml+="</PRE>";
        errorHtml+="<HR style=\'width: 95%; text-align: left; color: #999; height: 1px;\' align=\'left\'><DL>";
        errorHtml+="<DT>error: ";
        errorHtml+="   <B>General Failure</B></DL><P>";
        errorHtml+="</P><HR style=\'width: 95%; text-align: left; color: #999; height: 1px;\' align=\'left\'>";
        // if there is a recommendxation
        errorHtml+="<TABLE style=\'solid #C9CAB7;\' BORDER=\'1\' CELLPADDING=\'3\' CELLSPACING=\'0\' WIDTH=\'95%\'>";
        errorHtml+="\t<TR BGCOLOR=\'#CCCCFF\' style=\'background: #DDE4D9;\' width=\'95%\'>";
        errorHtml+="\t\t<TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\' COLSPAN=2>";
        errorHtml+="\t\t\t<H1 style=\'font-size: 15px; font-weight: bold; display: inline-block; padding: 3px;\'>Recommendation Summary</H1>";
        errorHtml+="\t\t</TD>";
        errorHtml+="\t</TR>";
        errorHtml+="\t<TR BGCOLOR=\'white\' style=\'background: #FFFFFF;\' width=\'95%\'>";
        errorHtml+="\t\t<TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\' ALIGN=\'right\' VALIGN=\'top\' WIDTH=\'22%\' >";
        errorHtml+="\t\t<FONT SIZE=\'-1\'>";
        errorHtml+="\t\t\t<CODE style=\'font-size: 12px; font-family: \"courier new\", courier, monospace;\'>&nbsp;";
        errorHtml+="\t\t    \t<SPAN style=\'color: #007711;\'>failure</SPAN>";
        errorHtml+="\t\t    </CODE>";
        errorHtml+="\t\t </FONT>";
        errorHtml+="\t      </TD>";
        errorHtml+="\t      <TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\'>";
        errorHtml+="\t\t FATAL: Startup halted.";
        errorHtml+="\t      </TD>";
        errorHtml+="\t   </TR>";
        errorHtml+="\t<TR BGCOLOR=\'white\' style=\'background: #FFFFFF;\' width=\'95%\'>";
        errorHtml+="\t\t<TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\' ALIGN=\'right\' VALIGN=\'top\' WIDTH=\'22%\' >";
        errorHtml+="\t\t<FONT SIZE=\'-1\'>";
        errorHtml+="\t\t\t<CODE style=\'font-size: 12px; font-family: \"courier new\", courier, monospace;\'>&nbsp;";
        errorHtml+="\t\t    \t<SPAN style=\'color: #007711;\'>recommendation</SPAN>";
        errorHtml+="\t\t    </CODE>";
        errorHtml+="\t\t </FONT>";
        errorHtml+="\t      </TD>";
        errorHtml+="\t      <TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\'>";
        errorHtml+="\t\t Unknown or undefined error.";
        errorHtml+="\t      </TD>";
        errorHtml+="\t   </TR>";
        errorHtml+="\t   <TR BGCOLOR=\'white\' style=\'background: #FFFFFF;\' width=\'95%\'>";
        errorHtml+="\t      <TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\' ALIGN=\'right\' VALIGN=\'top\' WIDTH=\'22%\' >";
        errorHtml+="\t\t\t<FONT SIZE=\'-1\'>";
        errorHtml+="\t\t\t\t<CODE style=\'font-size: 12px; font-family: \"courier new\", courier, monospace;\'>&nbsp;";
        errorHtml+="\t\t\t\t\t<SPAN style=\'color: #007711;\'>exception</SPAN>";
        errorHtml+="\t    \t\t\t<B>Unknown</B>";
        errorHtml+="\t\t\t\t</CODE>";
        errorHtml+="\t\t\t</FONT>";
        errorHtml+="\t\t</TD>";
        errorHtml+="\t\t<TD style=\'font-size: 9pt; font-family: Verdana, Helvetica, Arial, sans-serif; padding-left: 3px;\' WIDTH=\'100%\' >";
        errorHtml+="\t\t\tTo enable the details of this specific error message to be viewable on remote machines, please create a system environment variable named PT_REMOTE_ERR and set it to \'true\'.";
        errorHtml+="\t\t</TD>";
        errorHtml+="\t  </TR>";
        errorHtml+="</TABLE>";
        errorHtml+="</TABLE>";
        errorHtml+="</BODY>";
        errorHtml+="</HTML>";
      }
      return errorHtml;
    }
    
    /// <summary>
    /// This method handles displaying an error page, or displaying an error
    /// if that is not possible.
    /// </summary>
    /// <param name="Exception">the exception that caused us to go here.</param>
    public static void HandleError(Exception e, RequestData tempData)
    {
      HandleError(e, true, tempData);
    }
    
    /// <summary>
    /// This method handles displaying an error page, or displaying an error
    /// if that is not possible.  The exception will always be logged to
    /// PTSpy.
    /// </summary>
    /// <param name="Exception">the exception that caused us to go here.</param>
    /// <param name="boolean">
    /// false implies do not display the error message in an
    /// HTML comment.  This should be used for possible hack attempts.
    /// </param>
    /// <param name="RequestData">the current request data</param>
    private static void HandleError(Exception e, bool bDisplayErrorInComments,
     RequestData tempData)
    {
      try
      {
        log.Error(e, "Server error.");
        // Clear the WebData object in case we have already put something in there
        // This causes a problem if it is binary data, because then we send the binary
        // data twice.
        tempData.m_wdPageData.ClearOutputData();
        // Try to display the error page
        DisplayServerErrorPage(tempData);
        if (bDisplayErrorInComments && ConfigHelper.GetPerformanceStacktracesEnabled() && tempData.m_wdPageData.IsHTMLData())
        {
          String exceptionCommentString = XPHttpUtility.HtmlEncode(e.Message + ": " + XPException.GetInstance(e).GetStackTrace());
          HTMLComment exceptionComment = new HTMLComment(exceptionCommentString);
          tempData.m_xpResponse.GetOutputStream().Write(HTMLElement.ToByteArray(exceptionComment.GetDisplayString()));
          }
      }
      catch (Exception errorPageException)
      {
        log.Error(errorPageException, "Error displaying error page.");
      }
    }
    
    /// <summary>
    /// This method stores the current request URL on the HTTP Session
    /// for use in the next request as the previous request URL.
    /// </summary>
    /// <param name="tempData"></param>
    private static void StorePreviousRequestURL(RequestData tempData)
    {
      tempData.m_smSandbox.SetAttribute(ASConstants.PREVIOUS_REQUEST_URL,
       tempData.GetRequestFullURL());
      bool bIsPost = 0 == XPStringUtility.CompareIgnoreCase("post", tempData.m_xpRequest.GetMethod());
      tempData.m_smSandbox.SetAttribute(ASConstants.PREVIOUS_REQUEST_IS_POST,
       bIsPost);
    }
    
    /// <summary>
    /// This method attempts to display the server error page for when there
    /// was a catastrophic error.
    /// </summary>
    private static void DisplayServerErrorPage(RequestData tempData)
    {
      // We assume that tempData at least has valid request, response, etc... objects.
      // This includes the sandboxes, personal settings, and user session object.
      // Try to display the error page
      // Get User's AActivitySpace cache and cachelist
      tempData.m_smCache = ASCache.GetASCache(tempData.m_SessionManager, tempData.m_smSandbox);
      tempData.m_xpmCacheList = ASCache.GetCacheList(tempData.m_SessionManager);
      log.Info("Server error, redirecting to error page.");
      // Move old space info to parent space info, so error page knows where to
      // return to.
      tempData.m_sParentName = tempData.GetSpaceName();
      tempData.m_sParentID = tempData.m_sSpaceID;
      // Redirect to error space
      tempData.SetSpaceName(DEFAULTERRORSPACE);
      // Create new error space
      m_MVCController.CreateSpace(tempData);
      // Error for display
      UIError myError = null;
      // first bool is bForThisSpace, second is bShowAdminErrors
      IXPEnumerator errors = UIErrorHandler.GetErrors(tempData.m_smSandbox,
       tempData.m_Application, tempData.m_asSpace, true, false);
      if (errors.MoveNext())
      {
        myError = (UIError) errors.GetCurrent();
        // Clear this error to handle the (currently not supported) case where there can be multiple errors
        // This should not cause an XPIllegalStateException because
        // the enumerator is from a temporary collection, which is not
        // affected by the following method.
        UIErrorHandler.ClearError(myError, tempData.m_smSandbox, tempData.m_Application);
        }
      if (myError == null)
      {
        //_uiServerError
        String strError = GetString(97, "ptmsgs_infrastructure", tempData);
        myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,
         strError, strError);
      }
      myError.SetDisplayOnSeparatePage(true);
      myError.SetImmediateClear(true);
      UIErrorHandler.RaiseError(myError, tempData.m_smSandbox, tempData.m_Application);
      // otherwise get default Display page
      tempData.m_dpNextDisplay = tempData.m_asSpace.GetCurrentPage();
      // Pass in a web data object to be filled in by the display page
      // with either an HTMLPage, or some other form of data (binary, XML, etc...)
      tempData.m_ttTimer.SetCheckpoint(CHECKPOINT_PAGE_START);
      tempData.m_dpNextDisplay.Display(tempData.m_wdPageData);
      tempData.m_ttTimer.SetCheckpoint(CHECKPOINT_PAGE_END);
      // This will mark the page as sent so the finally clause of HandleRequest
      // will not call WebData.OutputData(..).
      tempData.m_ttTimer.SetCheckpoint(CHECKPOINT_DISPLAY_START);
      tempData.m_wdPageData.OutputData();
      tempData.m_ttTimer.SetCheckpoint(CHECKPOINT_DISPLAY_END);
    }
    
    /// <summary>
    /// This method attempts to get a localized string.
    /// </summary>
    /// <returns>String</returns>
    private static String GetString(int _nStringID, String _strResourceFileName,
     RequestData tempData)
    {
      String strLanguage = XPConvert.ToString(PersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingsHelper.LANGUAGE,
       tempData.m_smSandbox, tempData.m_Application));
      XPResourceManager rm = (XPResourceManager) tempData.m_Application.GetCachingManager().GetEntry("ResourceManager");
      return rm.GetString(_nStringID, _strResourceFileName, strLanguage);
      }
    
    /// <summary>
    /// This method attempts to display the server error page for when there
    /// was a catastrophic error.
    /// </summary>
    /// <param name="RequestData">tempData - the request data</param>
    /// <param name="int">the string ID</param>
    /// <param name="String">the string resource file name</param>
    /// <param name="String">the non-localized message, in case the localized string can't be retrieved.</param>
    private static void DisplayBasicError(RequestData tempData, int nStringID,
     String strResource, String strNonLocalizedMessage)
    {
      String strErrorMessage;
      try
      {
        if (null != strResource)
        {
          strErrorMessage = GetString(nStringID, strResource, tempData);
        } else
        {
          strErrorMessage = strNonLocalizedMessage;
        }
      }
      catch (Exception localizeException)
      {
        // Can't even get strings, just output something.
        strErrorMessage = strNonLocalizedMessage;
        log.Error(localizeException, "Localization error.");
      }
      try
      {
        bool bIsNetscape4 = false;
        // This should be non-null, but there may be situations where the first
        // request has failed before the browser settings could be stored
        // properly.
        if (null != tempData.m_smSandbox.GetAttribute("ISNS4"))
        {
          bIsNetscape4 = XPConvert.ToBoolean(tempData.m_smSandbox.GetAttribute("ISNS4"));
          }
        if (!bIsNetscape4)
        {
          // Set UTF-8 internationalization header
          tempData.m_xpResponse.SetContentType("text/html; charset=utf-8");
          } else
        {
          // If it's not netscape 4.x, use utf8
          // If it is netscape 4.x, use 8859_1, because NS4
          // can't handle utf-8 chars
          tempData.m_xpResponse.SetContentType("text/html; charset=8859_1");
          }
      }
      catch (Exception e)
      {
        // Problems setting content type
        log.Error(e, "Could not set i18n content-type");
      }
      // DCA - use outputstream to utilize httpcompression
      tempData.m_xpResponse.GetOutputStream().Write(XPConvert.ToByteArray(strErrorMessage,
       XPStringUtility.DEFAULT_ENCODING));
      // This will mark the page as sent so the finally clause of HandleRequest
      // will not call WebData.OutputData(..).
      tempData.m_wdPageData.SetDataSent();
    }
    ///////////////////////////////
    // Miscellaneous helper methods
    ///////////////////////////////
    
    /// <summary>
    /// This method checks whether or not an error page is required and
    /// changes the space information if necessary, but does not get
    /// the actual space.
    /// </summary>
    /// <param name="RequestData">the temporary request data object.</param>
    /// <returns>
    /// true means error page is required, and therefore we
    /// shouldn't get it from the cache.
    /// </returns>
    public static bool IsErrorPageRequired(RequestData tempData)
    {
      // Check if there are any errors that require their own Activity Space
      if (UIErrorHandler.IsErrorPageRequired(tempData.m_smSandbox, tempData.m_Application,
       UIErrorHandler.GetDisplayAdminErrors(tempData.GetUserSession())))
      {
        // set parent space to current so we can redirect back.
        tempData.m_sParentName = tempData.GetSpaceName();
        tempData.m_sParentID = tempData.m_sParentID;
        // Redirect to error space if we are not already in it
        if (!DEFAULTERRORSPACE.Equals(tempData.GetRootSpaceName()))
        {
          // redirect to error space
          tempData.SetSpaceName(DEFAULTERRORSPACE);
          // Do not set space to null here, in case we have the
          // same space already
          tempData.m_bGetCacheNoID = true;
          tempData.m_sPage = null;
          tempData.m_sControl = null;
          log.Info("Redirecting to error page.");
        }
        if ((tempData.m_asSpace != null) && (tempData.m_asSpace.GetName().Equals(DEFAULTERRORSPACE)))
        {
          // we don't need to get a new error space if we're already
          // in it.
          return false;
        }
        return true;
      }
      return false;
    }
    
    /// <summary>
    /// Stores the host name and port number from the current request on the subsession.
    /// </summary>
    /// <param name="subSession"></param>
    /// <param name="request"></param>
    public static void StoreHostPortSettings(ISessionManager subSession, IXPRequest request)
    {
      subSession.SetAttribute(HOSTNAME, request.GetServerName());
      subSession.SetAttribute(HOSTPORT, request.GetServerPort());
    }
    
    /// <summary>
    /// Stores the browser settings from the current request on the subsession.
    /// </summary>
    /// <param name="subSession"></param>
    /// <param name="request"></param>
    public static void StoreBrowserSettings(ISessionManager subSession, IXPRequest request)
    {
      // HACK - there should be a better way to do this.  A sub-session perhaps.
      // These names should also be constants.
      // For now, ClearSandbox needs to be updated with these if they change names
      // or if a new setting is added.
      // Only need to do it once every session.
      if (null == subSession.GetAttribute("ISIE"))
      {
        BrowserType browserType = new BrowserType(request);
        subSession.SetAttribute(BROWSERTYPE, browserType);
        // Leave the code below in so that we can view it on the memory debug page, even though
        // it is deprecated.   
        // Everyone should start using the BrowserType object instead of the booleans below.
        // So perhaps when these are removed from other code, we should rename the variables below.
        subSession.SetAttribute("ISMAC", browserType.IsMac());
        subSession.SetAttribute("ISIE", browserType.IsIE());
        subSession.SetAttribute("ISNS", browserType.IsNetscape());
        subSession.SetAttribute("ISNS4", browserType.IsNetscape4());
        subSession.SetAttribute("ISNS67", browserType.IsNetscape67());
      }
    }
    
    /// <summary>
    /// JF- Helper method to retrieve the current locale from the Session and set it on the PageData.
    /// (It will later be set in the HTTP header by WebData.)
    /// </summary>
    /// <param name="_tempData"></param>
    public static void SetContentLanguageOnPageData(RequestData _tempData)
    {
      try
      {
        if (null != _tempData.m_smSandbox.GetAttribute(USERLOCALE))
        {
          String sLocale = (String) _tempData.m_smSandbox.GetAttribute(USERLOCALE);
          _tempData.m_wdPageData.SetContentLanguage(sLocale);
        }
      }
      catch (Exception e)
      {
        log.Error(e, "Unable to find cached user locale.");
      }
    }
    private static void DetectBrowser(RequestData tempData)
    {
      StoreBrowserSettings(tempData.m_smSandbox, tempData.m_xpRequest);
      StoreHostPortSettings(tempData.m_smSandbox, tempData.m_xpRequest);
      // This is set by StoreHostPortSettings()
      bool bIsNetscape = false;
      // This should be non-null, but there may be situations where the 
      // request has no headers and browser settings could be stored
      // properly.
      if (null != tempData.m_smSandbox.GetAttribute("ISNS"))
      {
        bIsNetscape = XPConvert.ToBoolean(tempData.m_smSandbox.GetAttribute("ISNS"));
        }
    }
    
    /// <summary>
    /// Invoke the PreFilters of all registered InterpreterFilters.
    /// </summary>
    /// <param name="tempData">the request</param>
    /// <returns>
    /// int An int indicating whether the request should be filtered out.
    /// A result of -1 indicates that further processing is required, whereas
    /// a result of 0 or greater indicates that the filter at that position in
    /// the list of filters has performed the necessary processing for the request
    /// and the Interpreter shall not do any further processing of it.
    /// </returns>
    private static int DoPreFilter(RequestData tempData)
    {
      Object[] oa;
      Object o = null;
      oa = (Object[]) tempData.m_Application.GetCachingManager().GetEntry("InterpreterFilters");
      int nFilterIndex = 0;
      try
      {
        if (oa != null)
        {
          for (nFilterIndex = 0;nFilterIndex < oa.Length;nFilterIndex++)
          {
            o = oa[nFilterIndex];
            log.Debug("Interpreter - About to execute PreFilter on " + o);
            if (o != null)
            {
              IInterpreterFilter filter = (IInterpreterFilter) o;
              if (filter.PreFilter(tempData))
              {
                return nFilterIndex;
              }
            }
          }
        }
      }
      catch (InterpreterException ie)
      {
        log.Error(ie, "Exception while executing PreFilter on " + o + " , error message ID " + ie.GetMessageStringID());
        if (ie.GetMessageStringID() != InterpreterException.INVALID_ID)
        {
          DisplayBasicError(tempData, ie.GetMessageStringID(), ie.GetResourceFileName(),
           ie.GetNonLocalizedMessage());
        }
        // Set the HTTP status code.
        if (ie.GetHTTPStatusCode() != InterpreterException.INVALID_HTTP_STATUS_CODE)
        {
          tempData.m_xpResponse.SetStatus(ie.GetHTTPStatusCode());
        }
        log.Action("Leaving handleRequest due to filter throwing InterpreterException.");
        return nFilterIndex;
      }
      return -1;
    }
    
    /// <summary>
    /// Invoke the PostFilters of all registered InterpreterFilters.
    /// </summary>
    /// <param name="tempData">the request</param>
    private static void DoPostFilter(RequestData tempData)
    {
      DoPostFilter(-1, tempData);
    }
    
    /// <summary>
    /// Invoke the PostFilters of all registered InterpreterFilters that
    /// were executed in the pre-filter phase.
    /// </summary>
    /// <param name="the">last executed prefilter.  -1 means all filters.</param>
    /// <param name="tempData">the request</param>
    private static void DoPostFilter(int nLastExecutedPreFilter, RequestData tempData)
    {
      Object[] oa;
      Object o = null;
      oa = (Object[]) tempData.m_Application.GetCachingManager().GetEntry("InterpreterFilters");
      if (-1 == nLastExecutedPreFilter)
      {
        nLastExecutedPreFilter = oa.Length - 1;
      }
      try
      {
        if (oa != null)
        {
          for (int x = nLastExecutedPreFilter;x >= 0;x--)
          {
            o = oa[x];
            log.Debug("Interpreter - About to execute PostFilter on " + o);
            if (o != null)
            {
              IInterpreterFilter filter = (IInterpreterFilter) o;
              filter.PostFilter(tempData);
            }
          }
        }
      }
      catch (InterpreterException ie)
      {
        log.Error(ie, "Exception while executing PostFilter on " + o + " , error message ID " + ie.GetMessageStringID());
        }
      catch (Exception t)
      {
        log.Error(t, "Exception while executing PostFilter on " + o);
      }
    }
    
    /// <summary>
    /// This helper method performs basic session authentication.  It
    /// does not issue login challenges.
    /// </summary>
    /// <param name="tempData">The current request data.</param>
    /// <returns>Redirect Any redirect generated by the login process.</returns>
    public static Redirect CheckSessionAuth(RequestData tempData)
    {
      Redirect curRedirect = null;
      if ((null == tempData.GetUserSession()) || (LoginHelper.INSTANCE.GetIsGuestUser(tempData.GetUserSession())) || (null != tempData.m_smSandbox.GetAttribute("ModeOneRedirectToLogin")))
      {
        bool validUserSession = false;
        if (null != tempData.GetUserSession())
        {
          validUserSession = XPConvert.ToBoolean(tempData.m_smSandbox.GetAttribute(Interpreter.USERSESSIONVALID));
          }
        if (validUserSession && LoginHelper.INSTANCE.GetIsGuestUser(tempData.GetUserSession()))
        {
          // This happens when impersonating "guest" user, e.g., when administrator
          // edits the layout of guest-user MyPage.
          tempData.m_bGuest = false;
          log.Info("Detected guest user impersonation.");
        } else
        {
          // Try login, we will be logged in as guest if all fails
          LoginResult loginResult = m_Authenticator.Login(tempData);
          // loginResult will be null if no changes of
          if (loginResult != null)
          // user auth in the session (e.g., "guest" stays as "guest")
          // but it won't be null if "guest" becomes "guest2" 
          // (when multiple guest user is supported).
          {
            tempData.m_bGuest = loginResult.m_bIsGuest;
            // Note from original code in LoginHandlers.DoGuestLogin(): 
            // For guest login: 
            // This may not be necessary, and produces some unexpected results.
            // When this is called, it nulls out the Activity Space on the tempData,
            // which was causing some NPEs during login pei customizations.
            // This method is really meant for SSO to call.
            if (loginResult.m_Redirect != null)
            {
              //tempData.SetRedirect(loginResult.m_Redirect);
              curRedirect = loginResult.m_Redirect;
            }
            if (loginResult.m_bSuccess)
            {
              if (loginResult.m_bIsGuest)
              {
                if (!loginResult.m_bRedirectedBeforeLogin)
                {
                  tempData.SetUserSession(loginResult.m_UserSession);
                }
              } else
              {
                tempData.SetUserSession(loginResult.m_UserSession);
              }
            } else
            {
              if (loginResult.m_bIsGuest)
              {
                // Do not output error message from loginResult to end user,
                // as this error indicates serious problems that were not
                // caused by the end user.
                log.Error("Couldn\'t log in as guest user. Failing: " + loginResult.m_strError);
                throw new XPException("Couldn\'t log in as guest user. Failing.");
                } else
              {
                // If there was a redirect, then a before login PEI redirected us
                // away before we could login, so it's not an error.
                if (loginResult.m_Redirect == null)
                {
                  // Note: Invalid cookie if any has been cleared inside the Authenticator
                  // We don't set the activity space name, because we don't know it.
                  UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,
                   null, loginResult.m_strError, loginResult.m_strError);
                  myError.SetActivitySpace(tempData.GetSpaceName());
                  myError.SetClearWithActivitySpaceChange(true);
                  UIErrorHandler.RaiseError(myError, tempData.m_smSandbox,
                   tempData.m_Application);
                }
              }
            }
          }
        }
      } else
      {
        tempData.m_bGuest = !(XPConvert.ToBoolean(tempData.m_smSandbox.GetAttribute(Interpreter.USERSESSIONVALID)));
        }
      return curRedirect;
    }
    
    /// <summary>
    /// This helper method sends a 302 redirect.
    /// </summary>
    /// <param name="myRedirect">The 302 redirect to send.</param>
    /// <param name="tempData">The current request data.</param>
    public static void SendHTTPRedirect(Redirect myRedirect, RequestData tempData)
    {
      String sRedirectURL;
      // Do a true 302 HTTP redirect
      if (myRedirect.GetIsExternalURL())
      {
        sRedirectURL = myRedirect.GetExternalURL();
      } else
      {
        // JF- Build the URL to redirect to
        sRedirectURL = ConfigHelper.GetApplicationBaseURLToAS(ConfigHelper.GetNormalizedBaseURL(tempData.GetRequestFullURL()),
         myRedirect.GetSpaceName(), ConfigHelper.IsGoingToSubSpace(myRedirect));
        // JF- Append the QS arguments 
        sRedirectURL+=myRedirect.GetQueryString();
        sRedirectURL = tempData.AppendRedirectURLTo302URL(myRedirect.GetSpaceName(),
         sRedirectURL);
      }
      if (tempData.RedirectWithMetaRefresh(sRedirectURL))
      {
        // MOOSE- we have to URL-encode at least the semicolons, since they act as delimiters
        // between header attributes. Below is a partial fix for the problem that should be 
        // enough for the overdrive, but should be fixed for the hyperdrive release.
        // TODO: Redo in a decent manner for hyperdrive
        //
        // JF- Do meta data refresh
        log.Info("Redirecting (meta refresh) to " + sRedirectURL);
        // Bug 66494 - Redirection fails
        // The proper attribute value for meta HTTP-EQUIV is
        // CONTENT="0; URL=..." (note that "0; URL=..." is one value, URL is supposed to be 
        // part of the value for attribute "CONTENT")
        tempData.m_wdPageData.SetData("<html><head><meta HTTP-EQUIV=\"REFRESH\" CONTENT=\"0; URL=" + GeneralHelpers.Replace(sRedirectURL,
         ";", "%3B") + "\"></head></html>");
      } else
      {
        log.Info("Redirecting to " + sRedirectURL);
        // JF- Otherwise just do a regular 302 redirect
        tempData.m_xpResponse.Redirect(sRedirectURL);
      }
    }
    private static void HandleDisplayPage(Redirect myRedirect, RequestData tempData)
    {
      try
      {
        if (IsErrorPageRequired(tempData))
        {
          // Create the error space if we need it
          m_MVCController.CreateSpace(tempData);
        }
        tempData.m_dpNextDisplay = m_MVCController.GetFinalDisplayPage(tempData);
        if (tempData.m_dpNextDisplay != null)
        {
          log.Info("Displaying page " + tempData.m_dpNextDisplay.GetName());
          // Set the current page
          tempData.m_asSpace.SetCurrentPage(tempData.m_dpNextDisplay);
          tempData.m_ttTimer.SetCheckpoint(CHECKPOINT_PAGE_START);
          // Pass in a web data object to be filled in by the display page
          // with either an HTMLPage, or some other form of data (binary, XML, etc...)
          tempData.m_dpNextDisplay.Display(tempData.m_wdPageData);
          tempData.m_ttTimer.SetCheckpoint(CHECKPOINT_PAGE_END);
        } else
        {
          log.Error("Unable to find display page.");
          DisplayServerErrorPage(tempData);
        }
      }
      catch (UnauthorizedAccessException uae)
      {
        // At this point this exception can only occur due to Secure AS check
        // so the UnauthorizedAccessException should contain a Redirect
        Redirect secureRedirect = uae.GetRedirect();
        if (secureRedirect != null)
        {
          SendHTTPRedirect(secureRedirect, tempData);
        } else
        {
          log.Error("Unable to access display page (UnauthorizedAccessException).");
          DisplayServerErrorPage(tempData);
        }
      }
    }
    private static void DisplayComments(RequestData tempData, HTMLElement pageActionComments,
     String strHostName)
    {
      if (ConfigHelper.GetPerformanceCommentsEnabled() && tempData.m_wdPageData.IsHTMLData())
      {
        HTMLComment hostNameComment = new HTMLComment("Hostname: " + strHostName);
        tempData.m_xpResponse.GetOutputStream().Write(HTMLElement.ToByteArray(hostNameComment.GetDisplayString()));
        if (null != pageActionComments)
        {
          tempData.m_xpResponse.GetOutputStream().Write(HTMLElement.ToByteArray(pageActionComments.GetDisplayString()));
          }
        // Only display performance comments if they are enabled, and we are
        // displaying on a normal device (turn them off for WML, etc... mobile
        // devices).
        if (null == tempData.m_sDevicesSpaceNameSuffix)
        {
          HTMLComment pageDisplayTimeComment = new HTMLComment("Total Request Time: " + tempData.m_ttTimer.GetCheckpointTimeFromStart(CHECKPOINT_DISPLAY_END) + "\n" + "Control Time: " + tempData.m_ttTimer.GetCheckpointTimeFromCheckpoint(CHECKPOINT_CONTROL_START,
           CHECKPOINT_CONTROL_END) + "\n" + "Page Construction Time: " + tempData.m_ttTimer.GetCheckpointTimeFromCheckpoint(CHECKPOINT_PAGE_START,
           CHECKPOINT_PAGE_END) + "\n" + "Page Display Time: " + tempData.m_ttTimer.GetCheckpointTimeFromCheckpoint(CHECKPOINT_DISPLAY_START,
           CHECKPOINT_DISPLAY_END) + "\n");
          tempData.m_xpResponse.GetOutputStream().Write(HTMLElement.ToByteArray(pageDisplayTimeComment.GetDisplayString()));
          }
        HTMLComment versionComment = new HTMLComment(VersionHelper.GetVersionMessage());
        tempData.m_xpResponse.GetOutputStream().Write(HTMLElement.ToByteArray(versionComment.GetDisplayString()));
        }
    }
    private static bool EqualStringsOrBothNull(String str1, String str2)
    {
      if ((null == str1) && (null == str2))
      {
        return true;
      } else
      {
        if (null != str1)
        {
          if (null != str2)
          {
            if (0 == XPStringUtility.Compare(str1, str2))
            {
              return true;
            } else
            {
              return false;
            }
          } else
          {
            return false;
          }
        } else
        {
          return false;
        }
      }
    }
    protected static bool EqualRedirects(Redirect r1, Redirect r2)
    {
      if ((null == r1) || (null == r2))
      {
        return false;
      }
      if (!EqualStringsOrBothNull(r1.GetSpaceName(), r2.GetSpaceName()))
      {
        return false;
      }
      if (!EqualStringsOrBothNull(r1.GetSpaceID(), r2.GetSpaceID()))
      {
        return false;
      }
      if (r1.GetCachedNoID() != r2.GetCachedNoID())
      {
        return false;
      }
      if (!EqualStringsOrBothNull(r1.GetParentSpaceName(), r2.GetParentSpaceName()))
      {
        return false;
      }
      if (!EqualStringsOrBothNull(r1.GetParentSpaceID(), r2.GetParentSpaceID()))
      {
        return false;
      }
      if (!EqualStringsOrBothNull(r1.GetSubSpaceKey(), r2.GetSubSpaceKey()))
      {
        return false;
      }
      if (!EqualStringsOrBothNull(r1.GetDisplayPage(), r2.GetDisplayPage()))
      {
        return false;
      }
      if (!EqualStringsOrBothNull(r1.GetControl(), r2.GetControl()))
      {
        return false;
      }
      return true;
    }
  }
}