using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.activityspace.utils;
using com.plumtree.uiinfrastructure.application.varpacks;
using com.plumtree.uiinfrastructure.common;
using com.plumtree.uiinfrastructure.opener;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.uiinfrastructure.interpreter
{
  /// <summary>
  /// Helper class for managing the request data for the Interpreter.
  /// </summary>
  /// <author>Don Hayler</author>
  public class RequestData
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.interpreter.RequestData");
    
    /// <summary>
    /// Activity Space 
    /// </summary>
    public AActivitySpace m_asSpace = null;
    
    /// <summary>
    /// application 
    /// </summary>
    public IApplication m_Application;
    
    /// <summary>
    /// display page 
    /// </summary>
    public IDisplayPage m_dpNextDisplay = null;
    
    /// <summary>
    /// base session manager 
    /// </summary>
    public ISessionManager m_SessionManager;
    
    /// <summary>
    /// Activity Space Cache 
    /// </summary>
    public ISessionManager m_smCache = null;
    
    /// <summary>
    /// personal settings session manager 
    /// </summary>
    public ISessionManager m_smPersonalSettings = null;
    
    /// <summary>
    /// sandbox session manager 
    /// </summary>
    public ISessionManager m_smSandbox = null;
    
    /// <summary>
    /// persistent sandbox session manager 
    /// </summary>
    public ISessionManager m_smPersistentSandbox = null;
    
    /// <summary>
    /// request 
    /// </summary>
    public IXPRequest m_xpRequest;
    
    /// <summary>
    /// response 
    /// </summary>
    public IXPResponse m_xpResponse;
    
    /// <summary>
    /// Control name 
    /// </summary>
    public String m_sControl = null;
    
    /// <summary>
    /// optional _WML, etc... devices suffix 
    /// </summary>
    public String m_sDevicesSpaceNameSuffix = null;
    
    /// <summary>
    /// display page name 
    /// </summary>
    public String m_sPage = null;
    
    /// <summary>
    /// parent space ID 
    /// </summary>
    public String m_sParentID = null;
    
    /// <summary>
    /// parent space name 
    /// </summary>
    public String m_sParentName = null;
    
    /// <summary>
    /// space id 
    /// </summary>
    public String m_sSpaceID = null;
    
    /// <summary>
    /// sub-space key 
    /// </summary>
    public String m_sSubSpaceKey = null;
    
    /// <summary>
    /// web data 
    /// </summary>
    public WebData m_wdPageData;
    
    /// <summary>
    /// list of items in the cache 
    /// </summary>
    public XPHashtable m_xpmCacheList = null;
    
    /// <summary>
    /// whether or not to get the item from the cache without knowing its ID 
    /// </summary>
    public bool m_bGetCacheNoID = false;
    
    /// <summary>
    /// whether or not to clear the session after this request has finished. 
    /// </summary>
    public bool m_bClearSession = false;
    
    /// <summary>
    /// whether or not the current user is a guest user 
    /// </summary>
    public bool m_bGuest = true;
    
    /// <summary>
    /// SSO state 
    /// </summary>
    public int m_nSSOState = 0;
    
    /// <summary>
    /// A timer used to track various events in the requests lifetime 
    /// </summary>
    public TimeTracker m_ttTimer = null;
    
    /// <summary>
    /// User session object (IPTSession) 
    /// </summary>
    private Object m_UserSession = null;
    // base space name
    
    /// <summary>
    /// private space name data 
    /// </summary>
    private String m_sRootSpaceName = null;
    // base space name plus optional _WML, etc... devices suffix
    private String m_sSpaceName = null;
    
    /// <summary>
    /// Request URL 
    /// </summary>
    private String m_sRequestURL;
    
    /// <summary>
    /// request full URL 
    /// </summary>
    private String m_sRequestFullURL = null;
    private Redirect m_rRedirect = null;
    
    /// <summary>
    /// Hastable with the name/value pairs from XPRequest 
    /// </summary>
    private XPHashtable m_htQueryString = null;
    // sub-session
    
    /// <summary>
    /// Internal string constants for ISessionManager 
    /// </summary>
    private const String SANDBOXATTRIBUTE = "Sandbox";
    // sub-session
    private const String PERSISTENTSANDBOXATTRIBUTE = "PersistentSandbox";
    private const String SESSIONATTRIBUTE = "IPTSession";
    
    /// <summary>
    /// Creates a new RequestData object.
    /// </summary>
    /// <param name="request"></param>
    /// <param name="response"></param>
    /// <param name="session"></param>
    /// <param name="application"></param>
    public RequestData(IXPRequest request, IXPResponse response, ISessionManager session,
     IApplication application)
    {
      m_xpRequest = request;
      m_xpResponse = response;
      m_SessionManager = session;
      m_Application = application;
      m_ttTimer = new TimeTracker();
      // Get sandbox session
      m_smSandbox = m_SessionManager.GetSubSession(SANDBOXATTRIBUTE);
      // Create WebData object to wrap XPResponse
      BrowserType browserType = (BrowserType) m_smSandbox.GetAttribute(Interpreter.BROWSERTYPE);
      if (null == browserType)
      {
        try
        {
          browserType = new BrowserType(m_xpRequest);
        }
        catch (Exception e)
        {
          log.Error(e, "Unable to determine browser type.");
        }
      }
      m_wdPageData = new WebData(response, ConfigHelper.GetImageServerBaseURL(ConfigHelper.GetIsSpaceSecure(request.IsSecureConnection())),
       ConfigHelper.GetSecurityMode(), ConfigHelper.GetIsSpaceSecure(request.IsSecureConnection()),
       browserType);
      // Get Persistent sandbox session
      m_smPersistentSandbox = m_SessionManager.GetSubSession(PERSISTENTSANDBOXATTRIBUTE);
      // Get Personal Settings session off of the sandbox
      m_smPersonalSettings = m_smSandbox.GetSubSession(PersonalSettingsHelper.PREFS_SUBSESSION_NAME);
      // Get user session object off of the top-level HTTP Session
      m_UserSession = session.GetAttribute(SESSIONATTRIBUTE);
    }
    
    /// <summary>
    /// Accessor method for the root space name (without
    /// devices suffix).
    /// </summary>
    /// <returns>String the root space name.</returns>
    public virtual String GetRootSpaceName()
    {
      return m_sRootSpaceName;
    }
    
    /// <summary>
    /// Accessor method for the full space name (including
    /// devices suffix, if applicable).
    /// </summary>
    /// <returns>String the full space name.</returns>
    public virtual String GetSpaceName()
    {
      return m_sSpaceName;
    }
    
    /// <summary>
    /// Helper method that constructs and returns a redirect .
    /// </summary>
    /// <returns>String the full space name.</returns>
    public virtual XPHashtable GetQueryString()
    {
      if (m_rRedirect != null)
      {
        return m_rRedirect.GetControlArguments();
      }
      // JF- If the method has already been called, we already have the hashtable.
      if (null != m_htQueryString)
      {
        return m_htQueryString;
      }
      // JF- Otherwise we build it.
      XPHashtable result = new XPHashtable();
      // Fill in the control with all data from the request.
      // irrelevant parameters (like package and control) will be ignored.
      String sName;
      String[] sValues;
      IXPEnumerator parameters = m_xpRequest.GetParameterNames();
      while (parameters.MoveNext())
      {
        sName = (String) parameters.GetCurrent();
        // We check for null because in .NET, if you add an extra &
        // to the end of the QS, this will return null for the last one.
        if (sName != null)
        {
          sValues = m_xpRequest.GetParameterValues(sName);
          result.PutElement(sName, sValues);
        }
      }
      m_htQueryString = result;
      return result;
    }
    
    /// <summary>
    /// Helper method that checks whether or not an external redirect
    /// has been set on this object (i.e. "http://www.mysite.com").
    /// </summary>
    /// <returns>boolean true implies there is an external redirect.</returns>
    public virtual bool GetHasExternalRedirect()
    {
      if (m_rRedirect != null)
      {
        return m_rRedirect.GetIsExternalURL();
      }
      return false;
    }
    
    /// <summary>
    /// Helper method that checks whether or not an HTTP redirect
    /// has been set on this object (i.e. Redirect.SetIsHTTPRedirect(true)).
    /// </summary>
    /// <returns>boolean true implies there is an external redirect.</returns>
    public virtual bool GetHasHTTPRedirect()
    {
      if (m_rRedirect != null)
      {
        return m_rRedirect.GetIsHTTPRedirect();
      }
      return false;
    }
    
    /// <summary>
    /// Helper method that constructs and returns a redirect.
    /// </summary>
    /// <returns>String the full space name.</returns>
    public virtual Redirect GetRedirect()
    {
      if (m_rRedirect != null)
      {
        return m_rRedirect;
      }
      // Create redirect
      Redirect r = new Redirect();
      if (m_bGetCacheNoID)
      {
        r.SetLinkGetSpaceIfCached(GetSpaceName(), m_asSpace);
      } else
      if (m_sSpaceID == null)
      {
        r.SetLinkCreateNewSpace(GetSpaceName(), m_asSpace);
      } else
      if (null == m_xpRequest.GetQueryString() || m_xpRequest.GetQueryString().Length < 1)
      {
        // DCA - URL has no query string parameters to indicate space or page destination
        r.SetLinkToExternalURL(m_sRequestFullURL);
      } else
      {
        if (m_asSpace == null)
        {
          r.SetLinkCreateNewSpace(GetSpaceName(), null);
          r.AddControlArgument(ASConstants.SPACEID, m_sSpaceID);
        } else
        {
          r.SetLinkGetCachedSpace(m_asSpace);
        }
      }
      r.SetDisplayPage(m_sPage);
      r.SetControl(m_sControl);
      r.SetSubSpaceKey(m_sSubSpaceKey);
      return r;
    }
    
    /// <summary>
    /// JF- In security mode 1, returns the redirect URL (ASConstants.REDIRECT_AFTER_LOGIN) from the QS. 
    /// (Returns null if it was not there.)
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetRedirectURLFromQS()
    {
      String sRedirectURL = null;
      // WilliamA - Enhancement request bug 40145 (enabling redirect-after-login URL 
      // to be specified as a QS param regardless of security mode)
      //
      // if (ConfigVarPack.SECURITY_MODE_SSL_CRITICAL_AS_ONLY != ConfigHelper.GetSecurityMode()) 
      // {
      //	return sRedirectURL;
      // }
      try
      {
        XPHashtable htArgs = GetQueryString();
        if (htArgs.ContainsKey(ASConstants.REDIRECT_AFTER_LOGIN))
        {
          // JF- Get the redirect URL from the QS and make sure it looks like a valid URL
          String[] aData = (String[]) htArgs.GetElement(ASConstants.REDIRECT_AFTER_LOGIN);
          sRedirectURL = aData[0];
          if (!XPStringUtility.StartsWith(sRedirectURL, "http"))
          {
            sRedirectURL = null;
          }
        }
      }
      catch (Exception e)
      {
        log.Error(e, "An unexpected error occured when trying to retrieve the redirect URL from the Query String.");
        }
      return sRedirectURL;
    }
    
    /// <summary>
    /// Helper method that sets request data based on a redirect
    /// </summary>
    /// <param name="Redirect"></param>
    public virtual void SetRedirect(Redirect newRedirect)
    {
      SetSpaceName(newRedirect.GetSpaceName());
      m_sSpaceID = newRedirect.GetSpaceID();
      m_bGetCacheNoID = newRedirect.GetCachedNoID();
      m_sParentName = newRedirect.GetParentSpaceName();
      m_sParentID = newRedirect.GetParentSpaceID();
      m_sSubSpaceKey = newRedirect.GetSubSpaceKey();
      m_sPage = newRedirect.GetDisplayPage();
      m_sControl = newRedirect.GetControl();
      m_asSpace = null;
      m_dpNextDisplay = null;
      m_rRedirect = newRedirect;
    }
    
    /// <summary>
    /// JF- Adds the redirect URL to the URL where we are about to 302 redirect to.
    /// (only appends the redirect URL if we are in security mode 1 and 
    /// if we are redirecting to the login space)
    /// </summary>
    /// <param name="_sTargetSpace">- Space where we are 302 redirecting to.</param>
    /// <param name="_sURL">
    /// - URL to append the redirect to
    /// (if it's not needed, the url will be returned unchanged)
    /// </param>
    /// <returns>String</returns>
    public virtual String AppendRedirectURLTo302URL(String _sTargetSpace,
     String _sURL)
    {
      String sNewURL = _sURL;
      try
      {
        // JF- This is only needed if it's not a Gateway URL, in Security Mode 1 and if we are going to the login space
        // (mode 1 is the only mode when we might do 302 redirects to the login page) 
        if ((-1 != XPStringUtility.IndexOf(m_xpRequest.GetRequestURL(), ConfigHelper.GATEWAY_URL)) || (ConfigVarPack.SECURITY_MODE_SSL_CRITICAL_AS_ONLY != ConfigHelper.GetSecurityMode()) || (null == _sTargetSpace) || !XPStringUtility.EqualsIgnoreCase(_sTargetSpace,
         LoginHandlers.LOGINSPACE))
        {
          return sNewURL;
        }
        // JF- Construct the current URL with the QS
        String sBaseURL = ConfigHelper.GetNormalizedBaseURL(m_xpRequest.GetRequestURL());
        String sQSArgs = m_xpRequest.GetQueryString();
        if (null == sQSArgs)
        {
          sQSArgs = "";
        }
        // JF- Add the current URL to the QS
        String sRedirect = ASConstants.REDIRECT_AFTER_LOGIN + "=";
        sRedirect+=XPHttpUtility.UrlEncode(sBaseURL + sQSArgs, LinkConstants.UTF_8_ENCODING);
        // JF- Append the new QS arg to the URL
        if (!XPStringUtility.EndsWith(sNewURL, "&"))
        {
          sNewURL+="&";
        }
        sNewURL+=sRedirect;
      }
      catch (Exception e)
      {
        log.Error(e, "An unexpected error occured when trying to append the current URL to the redirect URL.");
        }
      return sNewURL;
    }
    
    /// <summary>
    /// JF- We need to do a meta refresh instead of a 302 redirect 
    /// if we are going from https to http in order to not see the 
    /// browser security alert "You are about to be redirected to a 
    /// connection that is not secure".
    /// </summary>
    /// <param name="_sRedirectURL"></param>
    /// <returns>boolean</returns>
    public virtual bool RedirectWithMetaRefresh(String _sRedirectURL)
    {
      bool bDoMetaRefresh = false;
      // JF- Get the type of the current browser
      BrowserType browserType = (BrowserType) m_smSandbox.GetAttribute(Interpreter.BROWSERTYPE);
      // JF- Meta refresh doesn't work with Netscape
      if (!browserType.IsNetscape())
      {
        //JF- Figure out if this URL is secure or not
        bool bIsSecureRedirect = true;
        if (-1 == XPStringUtility.IndexOf(XPStringUtility.ToLowerCase(_sRedirectURL),
         ConfigHelper.HTTPS_SCHEME))
        {
          bIsSecureRedirect = false;
        }
        // JF- Are we going from https to http? 
        if (m_xpRequest.IsSecureConnection() && !bIsSecureRedirect)
        {
          bDoMetaRefresh = true;
        } else
        // WilliamA - else are we going from http to https (bug 46380) 
        if (!m_xpRequest.IsSecureConnection() && bIsSecureRedirect)
        {
          bDoMetaRefresh = true;
        }
      }
      return bDoMetaRefresh;
    }
    
    /// <summary>
    /// This method loads various settings from the query string.
    /// If the query string parameters contain potential cross-site
    /// scripting attacks, this will throw an XPException.
    /// </summary>
    public virtual void LoadQSSettings()
    {
      IXPRequest request = m_xpRequest;
      // Get the event parameters that describe the control
      // Required Query String information
      // Space name
      String strRootSpaceName = request.GetParameterValue(ASConstants.SPACENAME);
      if (strRootSpaceName == null)
      {
        strRootSpaceName = request.GetParameterValue(ASConstants.SPACENAME_INPUT);
        }
      if ((strRootSpaceName != null) && (strRootSpaceName.Equals("")))
      {
        strRootSpaceName = null;
      }
      // if no space name is provided, use the URLASMappings
      // VarPack to check if there's a space we want to use.
      if (strRootSpaceName == null)
      {
        // retrieve the VarPack with our mappings. 
        URLASMappingsVarPack vpMappings = (URLASMappingsVarPack) m_Application.GetVarPackManager().GetVariablePackage(URLASMappingsVarPack.VARPACK_ID);
        // find an AS to go to based on parameters in the QS.
        IXPEnumerator myEnum = vpMappings.GetParameterMapKeys();
        if (null != myEnum)
        {
          while (myEnum.MoveNext())
          {
            String strCurrentKey = XPConvert.ToString(myEnum.GetCurrent());
            if (null != request.GetParameterValue(strCurrentKey))
            {
              // we go to the first one that we find.
              strRootSpaceName = vpMappings.GetParameterMapValue(strCurrentKey);
              break;
            }
          }
        }
        // check for slashes.  note that if the application server is 
        // redirecting the URL such that the URI is different than the 
        // HTTPEntryPoint, we may have a problem.
        if (null == strRootSpaceName)
        {
          String strRequestURI = request.GetRequestURI();
          String strHTTPEntryPoint = ConfigHelper.GetHTTPEntryPoint();
          // find the end of our request URL.
          int nHTTPEntryPointIndex = XPStringUtility.IndexOf(strRequestURI,
           strHTTPEntryPoint);
          if (-1 != nHTTPEntryPointIndex)
          {
            // everything after request URL is the rest of the URL.
            String strRestOfURL = XPStringUtility.Substring(strRequestURI,
             nHTTPEntryPointIndex + strHTTPEntryPoint.Length);
            if (-1 != XPStringUtility.IndexOf(strRestOfURL, "/") && strRestOfURL.Length > 1)
            {
              // if there's a slash something at the rest of 
              // the URL then go to the Opener for processing.
              strRootSpaceName = BaseOpenerAS.STR_MVC_CLASS_NAME;
            }
          }
        }
      }
      // Get accepts and user-agent headers and add devices suffix and modify space name.
      String strAccepts = m_xpRequest.GetHeader("Accept");
      String strUserHeader = m_xpRequest.GetHeader("User-Agent");
      DevicesVarPack vpPTDevices = (DevicesVarPack) m_Application.GetVarPackManager().GetVariablePackage(DevicesVarPack.VARPACK_ID);
      m_sDevicesSpaceNameSuffix = vpPTDevices.GetDevice(strAccepts, strUserHeader);
      // This fills out the real space name for us and handles the device suffix, if it exists
      SetSpaceName(strRootSpaceName);
      // Optional arguments on QS/forms.
      // Space ID
      m_sSpaceID = request.GetParameterValue(ASConstants.SPACEID);
      if (m_sSpaceID == null)
      {
        m_sSpaceID = request.GetParameterValue(ASConstants.SPACEID_INPUT);
        }
      if ((m_sSpaceID != null) && (m_sSpaceID.Equals("")))
      {
        m_sSpaceID = null;
      }
      // Control name
      m_sControl = request.GetParameterValue(ASConstants.CONTROL);
      if (m_sControl == null)
      {
        m_sControl = request.GetParameterValue(ASConstants.CONTROL_INPUT);
        }
      if ((m_sControl != null) && (m_sControl.Equals("")))
      {
        m_sControl = null;
      }
      if ((null != strRootSpaceName) && (strRootSpaceName.Equals(BaseOpenerAS.STR_MVC_CLASS_NAME)))
      {
        // Use the default control for the opener space
        m_sControl = null;
      }
      // Display page name
      m_sPage = request.GetParameterValue(ASConstants.PAGE);
      if (m_sPage == null)
      {
        m_sPage = request.GetParameterValue(ASConstants.PAGE_INPUT);
      }
      if ((m_sPage != null) && (m_sPage.Equals("")))
      {
        m_sPage = null;
      }
      // Parent space name
      m_sParentName = request.GetParameterValue(ASConstants.PARENTSPACENAME);
      if (m_sParentName == null)
      {
        m_sParentName = request.GetParameterValue(ASConstants.PARENTSPACENAME_INPUT);
        }
      if ((m_sParentName != null) && (m_sParentName.Equals("")))
      {
        m_sParentName = null;
      }
      // Parent space ID
      m_sParentID = request.GetParameterValue(ASConstants.PARENTSPACEID);
      if (m_sParentID == null)
      {
        m_sParentID = request.GetParameterValue(ASConstants.PARENTSPACEID_INPUT);
        }
      if ((m_sParentID != null) && (m_sParentID.Equals("")))
      {
        m_sParentID = null;
      }
      // Sub-space key
      m_sSubSpaceKey = request.GetParameterValue(ASConstants.SUBSPACE);
      if (m_sSubSpaceKey == null)
      {
        m_sSubSpaceKey = request.GetParameterValue(ASConstants.SUBSPACE_INPUT);
        }
      if ((m_sSubSpaceKey != null) && (m_sSubSpaceKey.Equals("")))
      {
        m_sSubSpaceKey = null;
      }
      // Get if cached boolean
      String sCache = request.GetParameterValue(ASConstants.CACHED);
      if (sCache == null)
      {
        sCache = request.GetParameterValue(ASConstants.CACHED_INPUT);
      }
      if ((sCache != null) && (!sCache.Equals("")))
      {
        m_bGetCacheNoID = XPConvert.ToBoolean(sCache);
      } else
      {
        m_bGetCacheNoID = false;
      }
      // Get clean session boolean
      String sCleanSession = request.GetParameterValue(ASConstants.CLEARSESSION);
      if ((sCleanSession != null) && (!sCleanSession.Equals("")))
      {
        m_bClearSession = XPConvert.ToBoolean(sCleanSession);
      } else
      {
        m_bClearSession = false;
      }
      CheckForCrossSiteHacks();
    }
    
    /// <summary>
    /// This helper method sets the root space name and the real space name
    /// based upon the optional devices space name suffix (_WML, etc...).
    /// This should be used to avoid breaking the devices scheme.
    /// </summary>
    /// <param name="String">strName the root name of the new space</param>
    public virtual void SetSpaceName(String strName)
    {
      m_sRootSpaceName = strName;
      if (m_sDevicesSpaceNameSuffix != null)
      {
        m_sSpaceName = m_sRootSpaceName + m_sDevicesSpaceNameSuffix;
      } else
      {
        m_sSpaceName = m_sRootSpaceName;
      }
    }
    
    /// <summary>
    /// Helper method that checks all of the QS parameters for
    /// cross site scripting attacks.
    /// Logs an error and throws an exception if any unsafe
    /// strings are detected.
    /// Throws an XPException if any unsafe strings are detected.
    /// </summary>
    private void CheckForCrossSiteHacks()
    {
      // NOTE: We currently only check Activity Space specific
      // query string arguments.  Since we know that these args
      // should only be alpha-numeric, we can be very strict
      // about disallowing unsafe characters.
      // We cannot be this strict if we were checking the entire
      // query string.  In the future, when we check the entire
      // query string, perhaps we could re-encode dangerous
      // characters.  That way they wouldn't be unsafe, and
      // developers could write code to use them if they needed
      // to.
      // Another possibility is that we could remove unsafe characters
      // rather than stopping all execution of the page in an attempt
      // to provide a degree of robustness in case someone accidentally
      // corrupts a URL while cut'n pasting from an e-mail (for example).
      // We don't do this here because there are places in the code where
      // developers have access to the unsafe request URL.  Unless we can
      // guarantee that all access to this URL is safe, we should halt
      // execution of this request.
      // Check for cross-site attacks (helper method throws exception):
      CheckForUnsafeCharacters(m_sControl);
      CheckForUnsafeCharacters(m_sDevicesSpaceNameSuffix);
      CheckForUnsafeCharacters(m_sPage);
      CheckForUnsafeCharacters(m_sParentID);
      CheckForUnsafeCharacters(m_sParentName);
      CheckForUnsafeCharacters(m_sRootSpaceName);
      CheckForUnsafeCharacters(m_sSpaceID);
      CheckForUnsafeCharacters(m_sSpaceName);
      CheckForUnsafeCharacters(m_sSubSpaceKey);
    }
    // 		NOTE: The following implementation will catch all params with '<' or '>' in it.  This
    // 		means that objects with those characters cannot be created.  This is a pretty serious
    // 		change.
    //		String strRequestQS = m_xpRequest.GetQueryString();
    //		if (strRequestQS != null)
    //		{
    //			bScriptDetected = StringContainsMarkUpBraces(m_xpRequest.GetQueryString());
    //		} else
    //		{
    //			// Form post, need to check values for all form data
    //			IXPEnumerator paramNames = m_xpRequest.GetParameterNames();
    //			String[] sValues;
    //
    //			while (paramNames.MoveNext() && !bScriptDetected)
    //			{
    //				sValues = m_xpRequest.GetParameterValues((String) paramNames.GetCurrent());
    //				if ((sValues != null) && (sValues.length > 0))
    //				{
    //					for (int i = 0; i < sValues.length; i++)
    //					{
    //						bScriptDetected = (((sValues[i] != null) && StringContainsMarkUpBraces(sValues[i]))
    //							|| bScriptDetected);
    //					}
    //				}
    //			}
    //		}
    
    /// <summary>
    /// Helper method that checks whether or not a string contains unsafe
    /// characters, such as angle braces for HTML or curly braces for
    /// javascript.
    /// This is useful for determining if someone is trying to do a javascript hack.
    /// </summary>
    /// <param name="String">the string to process</param>
    /// <returns>
    /// boolean true implies contains unsafe characters.
    /// Throws an XPException if any unsafe characters are detected.
    /// </returns>
    private static void CheckForUnsafeCharacters(String _strStringIn)
    {
      if (GeneralHelpers.ContainsUnsafeCharacters(_strStringIn, true))
      {
        throw new XPException("Query string contained unsafe characters, such as angle braces or curly braces.  Possible hack attempt: " + _strStringIn);
        }
    }
    //	private static final String FilterUnsafeCharacters(String _strStringIn)
    //	{
    //		if (StringContainsUnsafeCharacters(_strStringIn))
    //		{
    //			
    //				log.Error(//					"Query string contained unsafe characters, such as angle braces or curly braces.  Possible hack attempt: "
    //					+ _strStringIn);
    //			
    //
    //			XPStringBuilder sb = new XPStringBuilder(_strStringIn.length());
    //			char currentChar;
    //	
    //			for (int i = 0; i < _strStringIn.length(); i++)
    //			{
    //				currentChar = _strStringIn.charAt(i);
    //	
    //				if ((currentChar != '<') &&
    //					(currentChar != '>') &&
    //					(currentChar != '{') &&
    //					(currentChar != '}') &&
    //					(currentChar != ';') &&
    //					(currentChar != '\'') &&
    //					(currentChar != '"') &&
    //					(currentChar != '\\'))
    //				{
    //					sb.Append(currentChar);
    //				}
    //			}
    //	
    //			return sb.ToString();
    //		}
    //
    //		return _strStringIn;
    //	}
    
    /// <summary>
    /// Returns the request URL. This method is used in login code instead of 
    /// <c>m_xpRequest.GetRequestURL()</c> so the SSO login code can make
    /// it look like the request came to server.pt instead of ssologin.pt.
    /// </summary>
    /// <returns>the request URL</returns>
    public virtual String GetRequestURL()
    {
      if (m_sRequestURL != null)
      {
        return m_sRequestURL;
      }
      return m_xpRequest.GetRequestURL();
    }
    
    /// <summary>
    /// Sets the request URL. This method is used by SSO login code to make
    /// it look like the request came to server.pt instead of ssologin.pt.
    /// </summary>
    /// <param name="sRequestURL">the request url</param>
    public virtual void SetRequestURL(String sRequestURL)
    {
      m_sRequestURL = sRequestURL;
    }
    
    /// <summary>
    /// Returns the request full URL. This method is used to avoid having to
    /// recalculate this URL, since the call to URLDecode is fairly expensive.
    /// This method calculates this value if it has not already been set on
    /// the RequestData object.
    /// </summary>
    /// <returns>the request full URL</returns>
    public virtual String GetRequestFullURL()
    {
      if (null == m_sRequestFullURL)
      {
        m_sRequestFullURL = HttpUtils.GetRequestFullURL(m_xpRequest);
      }
      return m_sRequestFullURL;
    }
    
    /// <summary>
    /// Sets the request full URL. This method is used to avoid having to
    /// recalculate this URL, since the call to URLDecode is fairly expensive.
    /// </summary>
    /// <param name="sRequestFullURL">the request full url</param>
    public virtual void SetRequestFullURL(String sRequestFullURL)
    {
      m_sRequestFullURL = sRequestFullURL;
    }
    
    /// <summary>
    /// Gets the IPTSession from the Request Data object
    /// (originally off the top-level session).
    /// </summary>
    /// <returns>IPTSession</returns>
    public virtual Object GetUserSession()
    {
      return m_UserSession;
    }
    
    /// <summary>
    /// sets the IPTSession on the top-level session and
    /// on the Request Data object.
    /// </summary>
    /// <param name="User">Session Object</param>
    public virtual void SetUserSession(Object oUserSession)
    {
      m_UserSession = oUserSession;
      m_SessionManager.SetAttribute(SESSIONATTRIBUTE, oUserSession);
    }
    
    /// <summary>
    /// Removes the User Session from the top-level session and
    /// on the Request Data object.
    /// </summary>
    public virtual void ClearUserSession()
    {
      // We should probably clear the stored variable as well
      //		m_UserSession = null;
      m_SessionManager.RemoveAttribute(SESSIONATTRIBUTE);
    }
  }
}