using System;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.interpreter;
namespace com.plumtree.uiinfrastructure.interpreter.filter
{
  /// <summary>
  /// A filter to init and clean-up temporary data in the activity space
  /// and cache. Currently, we don't do any init, only clean up, so only
  /// the PostFilter method is implemented here.
  /// Note that this filter should listed <i>after</i> the RequestQueueFilter
  /// in the InterpreterFilters.xml (since the clean up must be done while 
  /// the session is still locked).
  /// </summary>
  /// <author>Don Hayler</author>
  /// <author>WilliamA  (Writing this new class)</author>
  public class RequestDataFilter : BaseInterpreterFilter
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.interpreter.filter.RequestDataFilter");
    
    /// <summary>
    /// This method performs clean up of temporary data in the activity
    /// space and the cache.
    /// </summary>
    public override void PostFilter(RequestData requestData)
    {
      Cleanup(requestData);
    }
    
    /// <summary>
    /// Cleans up temp data on AS and cache.
    /// </summary>
    /// <param name="tempData"></param>
    public static void Cleanup(RequestData tempData)
    {
      if (null == tempData)
      {
        return;
      }
      if (tempData.m_bClearSession)
      {
        try
        {
          // Clear everything stored on the session so that
          // network monitoring requests don't impact our
          // memory footprint.
          log.Info("Clearing and abandoning session.");
          tempData.m_SessionManager.Clear();
          tempData.m_SessionManager.Abandon();
        }
        catch (Exception e)
        {
          log.Error(e, "Error Clearing and abandoning session (for network monitoring) after request was processed.");
          }
      } else
      {
        // Cleanup temporary data on the AS
        if (tempData.m_asSpace != null)
        {
          try
          {
            tempData.m_asSpace.CleanupTempData();
            // Most subspaces will touch their parent space during the course of
            // building their HTML (ASURL.SetLinkGetCachedSpace() does this),
            // but we manually touch them just to be safe.
            TouchParentSpaces(tempData.m_asSpace, tempData);
            // Touch the space so it shows as being the last accessed
            tempData.m_smCache.GetAttribute(tempData.m_asSpace.GetSpaceID());
            }
          catch (Exception e)
          {
            log.Error(e, "Error cleaning up temp data on space " + tempData.m_asSpace.GetName() + " after request was processed.");
            }
        }
        try
        {
          ASCache cacheObject = (ASCache) tempData.m_smSandbox.GetAttribute(ASConstants.SANDBOXASCACHE);
          if (null != cacheObject)
          {
            // Clean up AS cache
            cacheObject.HandleCacheLimit();
          }
        }
        catch (Exception e)
        {
          log.Error(e, "Error handling cache limit after request was processed.");
          }
      }
    }
    
    /// <summary>
    /// This method accesses the parent spaces of subspaces so that they show as being
    /// "live" in the ASCache.  This works recursively to touch all the parent spaces
    /// of a subspace (if the parentspace is also a subspace).
    /// </summary>
    private static void TouchParentSpaces(AActivitySpace currentSpace, RequestData tempData)
    {
      if (null == currentSpace)
      {
        return;
      }
      // If this space is a sub-space
      if (null != currentSpace.GetSubSpaceKey())
      {
        // This touches the parent space so it shows as being the last accessed
        AActivitySpace parentSpace = currentSpace.GetParentSpace();
        TouchParentSpaces(parentSpace, tempData);
      }
    }
  }
}