using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.application.varpacks;
using com.plumtree.uiinfrastructure.interpreter;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.uiinfrastructure.interpreter.filter
{
  /// <summary>
  /// A filter to check the security of the incoming request whether it conforms
  /// to the security mode setting, in particular the security mode 2. 
  /// Security mode 1 check will be performed by the MVC controller.  
  /// </summary>
  /// <author>Julie Fournier</author>
  /// <author>William Adjie-Winoto (Writing this new class)</author>
  public class SecurityModeFilter : BaseInterpreterFilter
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.interpreter.filter.SecurityModeFilter");
    
    /// <summary>
    /// Check the whether the incoming  request satisfy the security mode 
    /// setting. The filter is effective when security mode is 2, and the 
    /// incoming request does not come from a secure port, in which case the 
    /// PreFilter method will send an HTTP redirect to the corresponding secure URL
    /// (and return true to indicate that the Interpreter shall not process this 
    /// request further).
    /// </summary>
    public override bool PreFilter(RequestData requestData)
    {
      IXPRequest request = requestData.m_xpRequest;
      IXPResponse response = requestData.m_xpResponse;
      IApplication application = requestData.m_Application;
      // JF- First we make sure that the request coming in correspond to the security mode from x_config.xml.
      // We redirect the request if needed.
      if (CheckHTTPSecurityAndRedirect(request, response, application))
      {
        return true;
      } else
      {
        return false;
      }
    }
    
    /// <summary>
    /// JF- Checks if the URL in the request corresponds to the security mode from x_config.xml.
    /// If it doesn't match, we redirect.
    /// </summary>
    /// <param name="request"></param>
    /// <param name="response"></param>
    /// <param name="application"></param>
    /// <returns>boolean true if the response has been redirected.</returns>
    private bool CheckHTTPSecurityAndRedirect(IXPRequest request, IXPResponse response,
     IApplication application)
    {
      int nSecurityMode = ConfigHelper.GetSecurityMode();
      // JF- Store the Security Mode in the Session so we can get it back from XPPage
      if (null == ((XPRequest) request).GetSession().GetAttribute(XPPage.SESSION_KEY_SECURITYMODE))
      {
        ((XPRequest) request).GetSession().SetAttribute(XPPage.SESSION_KEY_SECURITYMODE,
         XPConvert.ToString(nSecurityMode));
      }
      String sQueryString = request.GetQueryString();
      if (null == sQueryString)
      {
        sQueryString = "";
      }
      // WilliamA - In this filter we only check Security Mode 2
      // WilliamA - Secure AS check for security mode 1 will be performed within 
      //            the MVCController
      if (ConfigVarPack.SECURITY_MODE_SSL_ALL == nSecurityMode)
      {
        if (!request.IsSecureConnection())
        {
          response.Redirect(ConfigHelper.GetApplicationBaseURL(ConfigHelper.GetNormalizedBaseURL(request.GetRequestURL()),
           true) + sQueryString);
          return true;
        }
      }
      return false;
    }
  }
}