using System;
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.application.varpacks;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.uiinfrastructure.interpreter.filter.utils
{
  /// <summary>
  /// This class implements part of a multi-threaded queue.
  /// Currently it only maintains a counter of objects in the
  /// queue, but it will eventually be able to optimize
  /// identical requests out of the queue.
  /// </summary>
  /// <author>DonH</author>
  public class ASQueue
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.interpreter.filter.utils.ASQueue");
    private int m_nLength;
    private int m_nMaxQueueLength;
    private XPLock m_xpLock;
    public ASQueue(IApplication application)
    {
      // Attempt to get the max length from the config file
      String strMaxLength = ConfigHelper.GetConfigVariableAsString(ConfigConstants.SYSTEM_PROPERTIES_SECTION,
       ConfigConstants.MAX_REQUEST_QUEUE_LENGTH_SUBSECTION, ConfigConstants.VALUE_ATTRIBUTE,
       application);
      if ((strMaxLength != null) && (!"".Equals(strMaxLength)))
      {
        try
        {
          m_nMaxQueueLength = XPConvert.ToInteger(strMaxLength);
        }
        catch (XPFormatException e)
        {
          log.Warn(e, "Unable to convert max queue length from config file (" + strMaxLength + ") to int.");
          m_nMaxQueueLength = 10;
        }
      } else
      {
        // Use default if not available in config.xml
        m_nMaxQueueLength = 10;
      }
      m_nLength = 0;
      m_xpLock = new XPLock();
    }
    
    /// <summary>
    /// This method checks whether or not there is
    /// space available on the queue.  If there is,
    /// the registration succeeds and the queue length
    /// is incremented.  If not, the registration fails
    /// and the length remains unchanged.
    /// There is no need to call RemoveFromQueue() if
    /// registration fails.
    /// </summary>
    /// <returns>boolean true implies success, false implies failure.</returns>
    public virtual bool Register()
    {
      bool bAttemptFailed = false;
      bool bQueueAvailable = true;
      try
      {
        // Short lock attempt -- just fail if we can't get it.
        m_xpLock.Attempt(1000);
        if (m_nLength > m_nMaxQueueLength)
        {
          bQueueAvailable = false;
        } else
        {
          m_nLength++;
        }
      }
      catch (XPThreadInterruptedException e)
      {
        // Unable to acquire lock
        // no need to release, therefore set attemptfailed
        // so we don't release it in the finally phase.
        bAttemptFailed = true;
        bQueueAvailable = false;
        log.Error(e, "Unable to acquire lock for user.");
      }
      catch (Exception e)
      {
        log.Error(e, "Unable to acquire lock for user.");
        bQueueAvailable = false;
      }
      finally
      {
        // If the attempt failed, then an exception occured before we were
        // able to get the lock, so we don't need to release it.
        if (!bAttemptFailed)
        {
          m_xpLock.Release();
        }
      }
      return bQueueAvailable;
    }
    
    /// <summary>
    /// This method removes an item from the queue.
    /// There is no need to call RemoveFromQueue() if
    /// Register() failed.
    /// </summary>
    /// <returns>boolean true implies success, false implies failure.</returns>
    public virtual bool RemoveFromQueue()
    {
      bool bAttemptSuccessful = true;
      try
      {
        m_xpLock.Attempt(1000);
        if (m_nLength > 0)
        {
          m_nLength--;
        }
      }
      catch (XPThreadInterruptedException e)
      {
        // Unable to acquire lock
        // no need to release, therefore set attemptfailed
        // so we don't release it in the finally phase.
        bAttemptSuccessful = false;
        log.Error(e, "Unable to acquire lock for user.");
      }
      catch (Exception e)
      {
        log.Error(e, "Unable to acquire lock for user.");
      }
      finally
      {
        // If the attempt failed, then an exception occured before we were
        // able to get the lock, so we don't need to release it.
        if (bAttemptSuccessful)
        {
          m_xpLock.Release();
        }
      }
      return bAttemptSuccessful;
    }
  }
}
// NOTE: To better handle a single user refreshing their page constantly
// and using up all of the available threads on the app server we need to
// implement a queue of threads waiting on the lock.  When a new request
// adds itself to the queue, the queue should check for identical previous
// requests and cancel them.  When those requests get the lock, they will
// check their status and return immediately if they have been cancelled.
// Only 100% identical requests will be cancelled.  This means that if a
// user refreshes their mypage 5 times, the middle 3 requests will be
// cancelled, and only the last (relevant) request will be processed.
// See NOTE below.
// NOTE: DonH -- We want this lock to be long because we are trying to optimize this
// for the end user experience, not for server performance or to stop denial of service
// attacks.
//
// If this lock times out on a user, they will see an annoying 'server busy processing
// your previous request' message that won't endear them to Plumtree.  Then will then
// have to refresh their page, which may time out again, compounding the problem.
// We want the user to eventually get a valid page without seeing the server busy
// message, if at all possible.
//
// A sidebar to this is that we can optimize the usage of this lock for the end user
// experience by cancelling identical page refreshes.  If the user hits their mypage,
// which takes 30 seconds to load, and then hits refresh several times before the
// page has had a chance to finish, we know that all of the refreshes between the
// currently executing page and the last request are not going to be displayed.  This
// is only true for identical URLs from the same user waiting on the same lock.  This
// does mean that if a user has two browser windows open on their mypage and they are
// refreshing both many times, one of the windows could display a server busy message.
//
// This can be implemented by having threads register their URL with a per session
// multi-threaded queue before attempting to get the lock.  Then, when a thread registers,
// it can check if any of the threads before it are the same as it.  If they are, it can
// mark those as cancelled.  Then, when one of those threads gets the lock, it can check
// its status and return a server busy message rather than trying to display the page.
// When the last request (which has not been cancelled) finally gets the lock, it will
// remove itself from the queue and display the requested page.
// Make sure we release this lock later.
// Attempt to get a lock.  This will throw an exception if it fails or cannot obtain
// the lock.
