using System;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.activityspace.utils;
using com.plumtree.uiinfrastructure.interpreter;
using com.plumtree.uiinfrastructure.interpreter.filter;
using com.plumtree.uiinfrastructure.login;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.web;
//Put explicit import for Interpreter (to avoid ambiguity of using the deprecated class)
using Interpreter=com.plumtree.uiinfrastructure.interpreter.Interpreter;
using com.plumtree.openfoundation.util;
namespace com.plumtree.uiinfrastructure.interpreter.filter.utils
{
  /// <summary>
  /// This class contains helper methods for the Interpreter class that handle
  /// Gateway requests.
  /// </summary>
  /// <author>daniilk</author>
  public class GatewayHandlers
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("Gateway"),
     "com.plumtree.uiinfrastructure.activityspace.utils.GatewayHandlers");
    
    /// <summary>
    /// Internal string constants for gateway 
    /// </summary>
    public const String GATEWAY_UNIQUE = "/gateway/PTARGS_";
    public const String GATEWAY_FAILURE_FLAG = "GWAccessDenied";
    private const String STR_GATEWAY = "Gateway";
    public const String RESPONSE_TYPE_HEADER_NAME = "PT-HTTPResponse-Type";
    public const String SESSION_TIMEOUT_HEADER_VALUE = "SESSION_TIMEOUT";
    public const String REQUEST_TYPE_HEADER_NAME = "PT-HTTPRequest-Type";
    public const String PORTLET_REFRESH_HEADER_VALUE = "PORTLET_REFRESH";
    public const String CLIENT_SIDE_HEADER_VALUE = "CLIENT_SIDE";
    public const String PORTAL_LOGIN_URL_HEADER_NAME = "PT-Login-URL";
    
    /// <summary>
    /// Name of the session flag indicating being in the default profile
    /// editor.  Defined here so that the AppDataObject can access it too.
    /// This is stored on the persistent session.
    /// </summary>
    public const String DPROFILE_EDITOR_SESSION_FLAG = "DefProfEditorActiveFlag";
    
    /// <summary>
    /// This method handles gatewayed content.
    /// This method does not need to use the per-session XPLock, since it
    /// does not use the ASCache and destroys its AActivitySpace after
    /// every request.
    /// </summary>
    /// <param name="RequestData"></param>
    /// <returns>
    /// true implies the request has been fully processed.  False implies the
    /// interpreter should continue processing this request.
    /// </returns>
    public static bool HandleGatewayRequest(RequestData tempData)
    {
      if (log.IsActionEnabled())
      {
        log.Action("Entering HandleGatewayRequest: " + tempData.m_xpRequest.GetMethod() + " " + tempData.GetRequestFullURL());
        }
      String strUserID = "";
      // Get the event parameters that describe the control
      // Required Query String information
      tempData.SetSpaceName(STR_GATEWAY);
      tempData.m_sControl = STR_GATEWAY;
      // This flag tells us whether or not we should clean up before returning
      // control to the Interpreter.
      bool bProcessingCompleted = true;
      // try catch block for the locked portion of the code.
      try
      {
        // Lock the session so we can't have multiple threads
        // in here.  IIS does this for us, but some app servers don't.
        //
        // The reason for this lock was that WDP session object was not
        // thread-safe. Now, with OpenHTTP 2.0 it is and we can remove this lock.
        // NOTICE: There might have been other changes to the gateway stack 
        // (UI, Server, Portlet Provider, Transformer and support for PT Tags, etc.)
        // that relied on this lock being here. I am not aware of any such changes 
        // and since i'm still a gateway owner i am removing this lock with the belief
        // that everything should work as before, except better. :) 
        //			-- Moose
        // P.S. I will make sure that this change will be stress-tested by the 
        // performance and stability team.
        //
        // RequestLockFilter.LockSession(tempData.m_SessionManager);
        // Run the Expression Engine interpreter filter to load up
        // the environment data for the engine and run the expression
        // engine. It will figure out the right subportal to use for
        // this request.
        ExpressionEngineFilter.RunExpressionEngine(tempData);
        // Increment the performance counters for gateway requests
        PerfCounters.INSTANCE.IncrementCounter(PerfCounterConstants.PORTALPAGESPERF_GATEWAY_PAGES);
        // Do not set the smCache on the tempData because this is a gateway request.
        // CreateSpace relies on this being null to tell it that it is from a gateway
        // request.
        Redirect loginPEIRedirect = null;
        // ** Check session authentication **
        // This will at least log us in as the guest user.
        loginPEIRedirect = Interpreter.CheckSessionAuth(tempData);
        // Check for redirect from Login Actions PEI
        if (null != loginPEIRedirect)
        {
          tempData.SetRedirect(loginPEIRedirect);
          // Let the normal Interpreter deal with the login PEI redirect.
          // SE: Bug 63116 - Fix for Waikato. Update bProcessingCompleted flag
          // to prevent wiping out the WebData object later in the finally clause
          // since we're letting Interpreter handle the redirect.
          bProcessingCompleted = false;
          return bProcessingCompleted;
        }
        // Auto connect may have logged us in so we need to check again.
        if ((null == tempData.GetUserSession()) || (LoginHelper.INSTANCE.GetIsGuestUser(tempData.GetUserSession())))
        {
          // We ignore redirects coming from the login challenge.
          // This is where the redirect to the appropriate login page
          // comes from, but we just want to display gatewayed pages.
          // Bug 50969 - Check for BasicAuth only (no SSO) at this point
          if (HTTPBasicAuthenticator.GetInstance().UseThisAuth(tempData))
          {
            if (null == HTTPBasicAuthenticator.GetInstance().GetOrSendLoginChallenge(tempData))
            {
              // if null, the login challenge has been sent directly
              // by the Authenticator (e.g., in the HTTP Basic Auth case)
              // The lock will be released and the cleanup will be handled by the finally clause
              bProcessingCompleted = true;
              return bProcessingCompleted;
            }
          }
        }
        bool bIsGuest = ((null == tempData.GetUserSession()) || LoginHelper.INSTANCE.GetIsGuestUser(tempData.GetUserSession()));
        bool bLoginOccurredCookiePresent = LoginHandlers.IsLoginOccurredCookiePresent(tempData.m_xpRequest);
        strUserID = LoginHelper.INSTANCE.GetUserSessionID(tempData.GetUserSession());
        log.Info("Current user: " + LoginHelper.INSTANCE.GetUserSessionID(tempData.GetUserSession()) + ", " + LoginHelper.INSTANCE.GetUserSessionName(tempData.GetUserSession()));
        log.Debug("LoginHelper.INSTANCE.GetUserSessionID(tempData.GetUserSession())" + LoginHelper.INSTANCE.GetUserSessionID(tempData.GetUserSession()) + "]");
        log.Debug("bLoginOccuredCookiePresent is [" + (bLoginOccurredCookiePresent ? "true" : "false") + "]");
        bool bIsEditingDefaultProfile = !((null == tempData.m_smPersistentSandbox) || (null == tempData.m_smPersistentSandbox.GetAttribute(DPROFILE_EDITOR_SESSION_FLAG)));
        log.Debug("DPROFILE_EDITOR_SESSION_FLAG is [" + (bIsEditingDefaultProfile ? (String) tempData.m_smPersistentSandbox.GetAttribute(DPROFILE_EDITOR_SESSION_FLAG) : "not present") + "]");
        //			boolean bHandleLogin = false;
        // if the user is guest
        if (bIsGuest && 
        // and he/she logged in previously 
        bLoginOccurredCookiePresent && 
        // and we are not currently editing guest profile				
        !bIsEditingDefaultProfile)
        {
          log.Info("Redirecting to login page due to session timeout.");
          bProcessingCompleted = HandleGatewayLogin(tempData);
          return bProcessingCompleted;
        }
        //				bHandleLogin = true;
        /* Note: Bug#45898 We need to block guest access to most gatewayed
         * materials when guest access is disallowed in the portal.  However,
         * collab currently depends on non-authenticated gateway requests
         * for WebDav (collab handles authentication themselves in this case).
         * In the future, this should probably be converted so that all
         * authentication is handled by the portal (or runner).
         * Guest users can also have header portlets (or portlets on their
         * mypage which can be used as a login page) which can contain
         * gatewayed images or links.
         * Therefore, we can't simply block all gateway access and this will
         * need to be fixed later.
         */
         
        /*
         // if the user is guest
         if (bIsGuest
         // and guest access is disallowed
         && !LoginHandlers.s_bGuestAccessAllowed)
         {
         log.Info("Guest cannot see non-login pages.  Redirecting to login page.");
         bHandleLogin = true;
         }
         
         if (bHandleLogin)
         {
         bProcessingCompleted = HandleGatewayLogin(tempData);
         return bProcessingCompleted;
         }
         */
         
        // Process the Gateway Space
        MVCController.GetInstance().CreateSpace(tempData);
        IHTTPControl myControl = (IHTTPControl) tempData.m_asSpace.GetControl(tempData.m_sControl);
        myControl.SetHTTPItems(tempData.m_xpRequest, tempData.m_wdPageData);
        /* Gateway never redirects. However, in order to indicate that the user
         * does not have sufficient privileges and should be redirected to login
         * it does return a non-null empty redirect object. Basically, we just use
         * it as a flag.
         */
         
        Redirect redirectFlag = myControl.CheckActionSecurityAndExecute(null);
        if (redirectFlag != null)
        {
          log.Info("Redirecting to login page due to insufficient privileges.");
          bProcessingCompleted = HandleGatewayLogin(tempData);
          return bProcessingCompleted;
        }
        // run the expression engine to determine the subportal id to use
        IExpressionEvaluator expressionEvaluator = (IExpressionEvaluator) tempData.m_Application.GetCachingManager().GetEntry("ExpressionEvaluator");
        if (null != expressionEvaluator)
        {
          expressionEvaluator.EvaluateExpressionForSubportalAction(tempData.m_asSpace,
           tempData.m_Application, tempData.m_xpRequest, tempData.GetUserSession());
          } else
        {
          log.Warn("The Expression Evaluator was not found.");
        }
        IDisplayPage nextDisplay = tempData.m_asSpace.GetCurrentPage();
        if (nextDisplay != null)
        {
          log.Info("Displaying page " + nextDisplay.GetName());
          // Set the current page
          tempData.m_asSpace.SetCurrentPage(nextDisplay);
          // JF- Set the content language on the PageData
          Interpreter.SetContentLanguageOnPageData(tempData);
          nextDisplay.Display(tempData.m_wdPageData);
          tempData.m_wdPageData.OutputData();
          tempData.m_xpResponse.GetOutputStream().Close();
        }
        // Cleanup temporary data on the AS
        tempData.m_asSpace.CleanupTempData();
      }
      catch (Exception e)
      {
        log.Error(e, "Error in HandleGatewayRequest.");
        Interpreter.HandleError(e, tempData);
      }
      finally
      {
        // RequestLockFilter.UnlockSession(tempData.m_SessionManager);
        if (bProcessingCompleted)
        {
          // null out the objects on the web data object to prevent memory leakage if a
          // display page mistakenly maintains a reference to this object.
          if (null != tempData.m_wdPageData)
          {
            tempData.m_wdPageData.ClearAllData();
          }
          log.Action("Leaving HandleGatewayRequest (request completed) - User ID: " + strUserID);
          } else
        {
          log.Action("Leaving HandleGatewayRequest (returning control flow to the interpreter) - User ID: " + strUserID);
          }
      }
      return true;
    }
    
    /// <summary>
    /// Helper method to handle gateway login.
    /// </summary>
    /// <param name="RequestData"></param>
    /// <returns>
    /// true means to exit this request immediately.  false means to continue
    /// processing in the normal interpreter.
    /// </returns>
    private static bool HandleGatewayLogin(RequestData tempData)
    {
      LoginHandlers.ClearLoginOccurredCookiePresent(tempData.m_xpResponse);
      // Figure out if this is a JSXML Request
      bool bJSXMLRequest = IsJSXMLRequest(tempData);
      // Create a redirect that will take us to the login page for this
      // guest user.
      Redirect loginRedirect = Authenticator.GetGuestLoginRedirect(tempData);
      loginRedirect.SetIsHTTPRedirect(true);
      // mj: sso code. if we don't have a session, we don't have a space, and the default
      //     space is the login space, then we want to try an sso login.
      Redirect ssoRedirect = SSOHelper.INSTANCE.DoLoginRedirect(tempData,
       tempData.m_xpRequest.GetQueryString());
      if (ssoRedirect != null)
      {
        // get the sso redirect url
        String ssoRedirectURL = ssoRedirect.GetExternalURL();
        // fix for 62714, handle jsxml (pthttppostrequest in specific) time out requests
        // response handler for pthttppostrequest excepts a response with redirect url in a header
        if (bJSXMLRequest)
        {
          //Set everything up on the response
          HandleJSXMLSessionTimeout(tempData, ssoRedirectURL);
          // done, send back the response with redirect url in the header, no redirect is necessary
          return true;
        }
        log.Info("Issuing redirect to the SSO login page due to a non-existent session.");
        log.Info("Redirecting to: " + ssoRedirectURL);
        tempData.SetRedirect(ssoRedirect);
        // mj: nothing to clean up this early in the method
        // The lock will be released by the finally clause.
        return true;
      }
      // Redirect us back to the current URL after login.
      loginRedirect.AddControlArgument(ASConstants.REDIRECT_AFTER_LOGIN, GetRedirectToURL(tempData,
       bJSXMLRequest));
      // TODO: This may not be the correct place to go if there is a custom
      // login pei that redirects to a custom login page.
      tempData.SetRedirect(loginRedirect);
      if (bJSXMLRequest)
      {
        //Now, we need to put together the login URL.
        String sRedirectURL = ConfigHelper.GetApplicationBaseURLToAS(ConfigHelper.GetNormalizedBaseURL(tempData.GetRequestFullURL()),
         loginRedirect.GetSpaceName(), ConfigHelper.IsGoingToSubSpace(loginRedirect));
        // Append the QS arguments 
        sRedirectURL+=loginRedirect.GetQueryString();
        sRedirectURL = tempData.AppendRedirectURLTo302URL(loginRedirect.GetSpaceName(),
         sRedirectURL);
        //Set everything up on the response
        HandleJSXMLSessionTimeout(tempData, sRedirectURL);
        //return true, so that interpreter won't process it any more
        return true;
      }
      // Allow the normal interpreter to process this login redirect.
      log.Info("HandleGatewayRequest returning control to Interpreter to process login page.");
      return false;
    }
    public static String GetRedirectToURL(RequestData tempData, bool bJSXMLRequest)
    {
      if ((XPStringUtility.CompareIgnoreCase("POST", tempData.m_xpRequest.GetMethod()) == 0) || bJSXMLRequest)
      {
        String strReferer = tempData.m_xpRequest.GetHeader("Referer");
        if ((null != strReferer) && (XPStringUtility.StartsWith(strReferer,
         HttpUtils.HTTP_SCHEME)))
        {
          log.Debug("POST or JSXML request caused a redirect to login page. " + "Using referer header to redirect back.");
          log.Debug("Original URL was [" + tempData.GetRequestFullURL() + "]");
          log.Debug("Value of the referer header is [" + tempData.m_xpRequest.GetHeader("Referer") + "]");
          return strReferer;
        }
      }
      return HttpUtils.GetNormalizedURL(HttpUtils.GetSafeURL(tempData.GetRequestFullURL()));
      }
    /*
     * Returns true if the request is made from JSXML, false otherwise.
     */
     
    public static bool IsJSXMLRequest(RequestData tempData)
    {
      //Figure out if this is a JSXML request
      String strRequestType = tempData.m_xpRequest.GetHeader(REQUEST_TYPE_HEADER_NAME);
      bool bJSXMLRequest = false;
      if ((null != strRequestType) && (XPStringUtility.EqualsIgnoreCase(strRequestType,
       PORTLET_REFRESH_HEADER_VALUE) || XPStringUtility.EqualsIgnoreCase(strRequestType,
       CLIENT_SIDE_HEADER_VALUE)))
      {
        bJSXMLRequest = true;
      }
      return bJSXMLRequest;
    }
    
    /// <summary>
    /// Adds appropriate headers and sets up the response so that the JSXML knows the session has
    /// expired, and can redirect to the right URL afterwards.
    /// </summary>
    public static void HandleJSXMLSessionTimeout(RequestData tempData, Redirect loginRedirect)
    {
      // Tell JSXML that session expired
      tempData.m_xpResponse.AddHeader(RESPONSE_TYPE_HEADER_NAME, SESSION_TIMEOUT_HEADER_VALUE);
      //Set the status to 200, as JSXML expects
      tempData.m_xpResponse.SetStatus(200);
      //Now, we need to put together the login URL.
      String sRedirectURL = ConfigHelper.GetApplicationBaseURLToAS(ConfigHelper.GetNormalizedBaseURL(tempData.GetRequestFullURL()),
       loginRedirect.GetSpaceName(), ConfigHelper.IsGoingToSubSpace(loginRedirect));
      // Append the QS arguments 
      sRedirectURL+=loginRedirect.GetQueryString();
      sRedirectURL = tempData.AppendRedirectURLTo302URL(loginRedirect.GetSpaceName(),
       sRedirectURL);
      // Add the header
      tempData.m_xpResponse.AddHeader(PORTAL_LOGIN_URL_HEADER_NAME, sRedirectURL);
      }
    
    /// <summary>
    /// Adds appropriate headers and sets up the response so that the JSXML knows the session has
    /// expired, and can redirect to the right URL afterwards.
    /// </summary>
    private static void HandleJSXMLSessionTimeout(RequestData tempData, String redirectURL)
    {
      // Tell JSXML that session expired
      tempData.m_xpResponse.AddHeader(RESPONSE_TYPE_HEADER_NAME, SESSION_TIMEOUT_HEADER_VALUE);
      //Set the status to 200, as JSXML expects
      tempData.m_xpResponse.SetStatus(200);
      // Add the header
      tempData.m_xpResponse.AddHeader(PORTAL_LOGIN_URL_HEADER_NAME, redirectURL);
      }
  }
}