using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.web;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.activityspace.utils;
using com.plumtree.uiinfrastructure.interpreter;
namespace com.plumtree.uiinfrastructure.login
{
  /// <summary>
  /// This is the main authenticator module, the single entry point for performing
  /// authentication. Internally, it communicates with other authentication modules
  /// (TokenAuthenticator, HTTPBasicAuthenticator and SSOAuthenticator) to decide
  /// which authentication method is applicable to current request, and it may fail 
  /// over to a different authentication method if one method fails (e.g., if login
  /// token auth fails, it will try HTTP basic auth, and if that fails, it will 
  /// log in as guest).  
  /// </summary>
  /// <author>Don Hayler</author>
  /// <author>Mike Jahr (SSO)</author>
  /// <author>Julie Fournier (Security Modes)</author>
  /// <author>William Adjie-Winoto (Writing this new class)</author>
  public class Authenticator : IAuthenticator
  {
    // CONSTANTS
    // Keys for login-related states in the HTTP Session
    // (for multiple-guest-user and multiple-login-page supports)
    public const String GUESTLOGIN_GUESTUSERNAME = "PTGLGuestName";
    public const String GUESTLOGIN_GUESTUSERID = "PTGLGuestID";
    public const String GUESTLOGIN_SSOVENDOR = "PTGLSSOVendor";
    public const String GUESTLOGIN_REDIRECTTOLOGIN = "PTGLLoginRdr";
    public const String GUESTLOGIN_REDIRECTONLOGOUT = "PTGLLogoutRdr";
    // This key is for optimization (so that we know which space corresponds
    // to the "login page" for this session, typically either the Login AS
    // or the MyPage AS, and MVCController would then allow the display of
    // this AS even though the session times out, since this is the login page).
    public const String LOGINPAGESPACE_ATTR_KEY = "PTLoginPageSpace";
    // VARIABLES	
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.login.Authenticator");
    private static Authenticator m_Instance = new Authenticator();
    protected TokenAuthenticator m_TokenAuth = TokenAuthenticator.GetInstance();
    protected HTTPBasicAuthenticator m_HTTPBasicAuth = HTTPBasicAuthenticator.GetInstance();
    protected SSOAuthenticator m_SSOAuth = SSOAuthenticator.GetInstance();
    // CONSTRUCTORS
    private Authenticator()
    {
    }
    // METHODS (static)
    public static Authenticator GetInstance()
    {
      return m_Instance;
    }
    // METHODS
    
    /// <summary>
    /// Performs the login operation. It will first try with login token auth. If
    /// the token does not exist or it fails, it will check if HTTP basic auth is
    /// appropriate for this request. If all fail, the user will be logged in as 
    /// guest user.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.login.IAuthenticator#Login(com.plumtree.uiinfrastructure.application.RequestData)"></seealso>
    public virtual LoginResult Login(RequestData requestData)
    {
      LoginResult loginResult = null;
      Object userSession = requestData.GetUserSession();
      // We have to check for the admin login token even if we are already
      // logged in as guest, because otherwise we can redirect the user
      // to the login page, revealing the login token in the query string.
      // This is a security risk.  This is only possible when redirecting
      // to the admin server from the browsing server after already having
      // accessed the admin server as guest (i.e. viewed a page).
      // We have to make sure that we don't call GetIsGuestUser() with
      // a null user session because it assumes that a null user session
      // object is the guest user.
      // WilliamA - "ModeOneRedirectToLogin" may no longer be needed if we can
      // use the GetIsGuestUser() to indicate guest or not. 
      if ((null == userSession) || (LoginHelper.INSTANCE.GetIsGuestUser(userSession)) || (null != requestData.m_smSandbox.GetAttribute("ModeOneRedirectToLogin")))
      {
        // JF- We just checked the auto login cookie. We can remove the flag from the session.
        if (null != requestData.m_smSandbox.GetAttribute("ModeOneRedirectToLogin"))
        {
          requestData.m_smSandbox.RemoveAttribute("ModeOneRedirectToLogin");
          }
        if (m_TokenAuth.UseThisAuth(requestData))
        {
          // Do admin Query String token login, or remember my password login token login
          loginResult = m_TokenAuth.Login(requestData);
          if (null != loginResult)
          {
            userSession = loginResult.m_UserSession;
          }
        }
        // Do guest login if user session is null (either because there were no tokens,
        // or token login failed)
        // -1 will be treated as default guest as well
        int nGuestUserID = 2;
        String str = (String) requestData.m_SessionManager.GetAttribute(Authenticator.GUESTLOGIN_GUESTUSERID);
        if (null != str)
        {
          nGuestUserID = XPConvert.ToInteger(str);
        }
        // Guest username info here is only used for Login PEIs.
        String strGuest = (String) requestData.m_SessionManager.GetAttribute(Authenticator.GUESTLOGIN_GUESTUSERNAME);
        if (null == userSession)
        {
          // If there is still no PTSession, log in as guest
          if (-1 == nGuestUserID)
          {
            log.Info("Logging in as guest user: default guest");
          } else
          {
            log.Info("Logging in as guest user: {0}", nGuestUserID);
          }
          // This sets the session validity for us.
          loginResult = LoginHelper.INSTANCE.AttemptGuestLogin(nGuestUserID,
           strGuest, null, requestData.m_smSandbox, requestData.m_smPersistentSandbox,
           requestData.m_Application, requestData.GetRequestURL(), requestData.m_xpRequest,
           requestData.m_wdPageData);
          if (null != loginResult)
          {
            userSession = loginResult.m_UserSession;
          }
        } else
        if (LoginHelper.INSTANCE.GetIsGuestUser(userSession))
        {
          // WilliamA - Multiple-guest-user support:
          // If current-session guest user is not the same as the
          // GuestLogin's guest user, then re-login using the new
          // guest user in the GuestLogin
          int nCurrentSessionGuestUser = XPConvert.ToInteger(LoginHelper.INSTANCE.GetUserSessionID(userSession));
          if (nCurrentSessionGuestUser != nGuestUserID)
          {
            // If there is still no PTSession, log in as guest
            if (-1 == nGuestUserID)
            {
              log.Info("Re-Logging in as guest user: default guest");
            } else
            {
              log.Info("Re-Logging in as guest user: {0}", nGuestUserID);
              }
            // This sets the session validity for us.
            loginResult = LoginHelper.INSTANCE.AttemptGuestLogin(nGuestUserID,
             strGuest, null, requestData.m_smSandbox, requestData.m_smPersistentSandbox,
             requestData.m_Application, requestData.GetRequestURL(), requestData.m_xpRequest,
             requestData.m_wdPageData);
            if (null != loginResult)
            {
              userSession = loginResult.m_UserSession;
            }
          }
        }
      }
      if ((null == userSession) || LoginHelper.INSTANCE.GetIsGuestUser(userSession))
      {
        // Try HTTP Basic auth, it will return null if it's not for HTTP Basic auth 
        LoginResult lr = m_HTTPBasicAuth.Login(requestData);
        if (null != lr)
        {
          loginResult = lr;
          userSession = loginResult.m_UserSession;
        }
      }
      return loginResult;
    }
    
    /// <summary>
    /// Returns a Redirect object to the login page, or issues an HTTP redirect to the
    /// login page. In the latter case, the method returns null.
    /// The way this method works: first it will check for the SSO setting whether 
    /// SSO is enabled. It will also check whether HTTP Basic should override SSO (for 
    /// WebDAV cases). Depending on the setting of the two, either SSO login challenge 
    /// or HTTP Basic login challenge will be sent (in which case this method returns
    /// null). If neither SSO nor HTTP basic is appropriate for current request, then 
    /// a default-login-page Redirect object is returned.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.login.IAuthenticator#GetOrSendLoginChallenge(com.plumtree.uiinfrastructure.application.RequestData)"></seealso>
    public virtual Redirect GetOrSendLoginChallenge(RequestData requestData)
    {
      // Bug 37744 - fix session-timeout handling in regard to plloginoccured cookie.
      // (clear the cookie once we send a login challenge, so that guest-accessible
      // pages are accessible after a login challenge is sent.)
      LoginHandlers.ClearLoginOccurredCookiePresent(requestData.m_xpResponse);
      if (m_SSOAuth.UseThisAuth(requestData))
      {
        // SSO does not currently work for Web Dav logins, so use
        // HTTP Basic Auth if this is a Web Dav URL.
        if (m_HTTPBasicAuth.GetBasicAuthOverrideSSO(requestData))
        {
          return m_HTTPBasicAuth.GetOrSendLoginChallenge(requestData);
        } else
        {
          try
          {
            // if SSO is enabled, but redirect to the SSO login cannot
            // be generated for some reason, it will throw an exception
            return m_SSOAuth.GetOrSendLoginChallenge(requestData);
          }
          catch (InterpreterException ie)
          {
            log.Info("Redirecting to login space defined by the current Experience Definition.");
            // Calculate the appropriate login page for the guest user.
            return GetGuestLoginRedirect(requestData);
          }
        }
      } else
      {
        if (m_HTTPBasicAuth.UseThisAuth(requestData))
        {
          return m_HTTPBasicAuth.GetOrSendLoginChallenge(requestData);
        } else
        {
          // Calculate the appropriate login page for the guest user.
          return GetGuestLoginRedirect(requestData);
        }
      }
    }
    
    /// <summary>
    /// This method calculates the appropriate login page for the guest user.
    /// </summary>
    /// <param name="_requestData">The current request data.</param>
    /// <returns>Redirect to the appropriate guest user login page.</returns>
    public static Redirect GetGuestLoginRedirect(RequestData _requestData)
    {
      return GetGuestLoginRedirect(_requestData.m_Application, _requestData.m_SessionManager,
       _requestData.m_xpRequest, _requestData.GetUserSession());
    }
    
    /// <summary>
    /// This method calculates the appropriate login page for the guest user.
    /// </summary>
    /// <param name="application"></param>
    /// <param name="sessionManager"></param>
    /// <param name="request"></param>
    /// <param name="userSession"></param>
    /// <returns>Redirect to the appropriate guest user login page.</returns>
    public static Redirect GetGuestLoginRedirect(IApplication application,
     ISessionManager sessionManager, IXPRequest request, Object userSession)
    {
      // WilliamA - The "GuestRedirectToLogin" setting now effectively specifies
      // whether the "login page" is, the Login AS or the Guest MyPage
      // (with Login portlet on it).
      // This behavior takes the best parts of both worlds: allowing a different login
      // page (by having a different guest-user MyPage with Login portlet on it), and
      // customers who already have their login page customized will continue to work.
      bool isGuestRedirectToLogin = true;
      String str = (String) sessionManager.GetAttribute(Authenticator.GUESTLOGIN_REDIRECTTOLOGIN);
      if (null != str)
      {
        try
        {
          isGuestRedirectToLogin = XPConvert.ToBoolean(str);
        }
        catch (Exception e)
        {
          log.Warn("Found invalid GuestRedirectToLogin from current subportal settings.");
          }
      }
      Redirect r = null;
      if (isGuestRedirectToLogin)
      {
        // Redirect to standard (basic) login page
        r = new Redirect();
        log.Info("Redirecting to login space " + LoginHandlers.LOGINSPACE);
        r.SetLinkCreateNewSpace(LoginHandlers.LOGINSPACE, null);
      } else
      {
        r = new Redirect();
        log.Info("Redirecting to Guest MyPage as login space");
        /*MyPageAS.STR_MVC_CLASS_NAME*/
        r.SetLinkCreateNewSpace("MyPage", null);
        /*SetPageControl.STR_MVC_CLASS_NAME*/
        r.SetControl("SetPage");
        /*SetPageControl.STR_PAGE_ID*/
        r.AddControlArgument("PageID", /*SetPageControl.N_HOME_PAGE_ID*/
        0);
      }
      // WilliamA - support for multiple-login-page idea
      // Set the space name of the login page  (so that the MVCController 
      // knows to allow display of this page even when HTTP session 
      // times out, since this is the login page)
      sessionManager.SetAttribute(LOGINPAGESPACE_ATTR_KEY, r.GetSpaceName());
      // Check if there is "redirect=.." param in the QS (enhancement request bug 40145)
      String redirectAfterLoginURL = request.GetParameterValue(ASConstants.REDIRECT_AFTER_LOGIN);
      if (null != redirectAfterLoginURL)
      {
        r.AddControlArgument(ASConstants.REDIRECT_AFTER_LOGIN, redirectAfterLoginURL);
        }
      return r;
    }
    
    /// <summary>
    /// Performs the logout operation. Currently only the SSO uses this, as
    /// the other authentication methods perfom the logout operations
    /// from within the Login activity space. For ths SSO case, this method sends
    /// an HTTP redirect to the SSO logout URL. 
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.login.IAuthenticator#Logout(com.plumtree.uiinfrastructure.application.RequestData)"></seealso>
    public virtual bool Logout(RequestData requestData)
    {
      // Currently only SSO logout is handled here,
      // regular logout is handled by the Login control in the Login AS
      if (m_SSOAuth.UseThisAuth(requestData))
      {
        return m_SSOAuth.Logout(requestData);
      } else
      {
        return false;
      }
    }
    
    /// <summary>
    /// Since this is the main authenticator module, it always returns 
    /// a boolean true.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.login.IAuthenticator#UseThisAuth(com.plumtree.uiinfrastructure.application.RequestData)"></seealso>
    public virtual bool UseThisAuth(RequestData requestData)
    {
      return true;
    }
  }
}