using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.activityspace.utils;
using com.plumtree.uiinfrastructure.application.varpacks;
using com.plumtree.uiinfrastructure.interpreter;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.uiinfrastructure.login
{
  /// <summary>
  /// The authentication module for HTTP Basic authentication. 
  /// </summary>
  /// <author>DonH</author>
  /// <author>WilliamA (Writing this new class)</author>
  public class HTTPBasicAuthenticator : IAuthenticator
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.login.HTTPBasicAuthenticator");
    
    /// <summary>
    /// Internal string constants for basic auth challenge.
    /// Note that the AUTH_CHALLENGE string cannot come between
    /// /gateway and /PTARGS_ or the gateway code will not work.
    /// The AUTH_CHALLENGE string must also come immediately
    /// after the HTTP entry point (server.pt).
    /// </summary>
    public const String AUTH_CHALLENGE = "/auth";
    public const String AUTH_DOMAIN = "default";
    
    /// <summary>
    /// Internal string constants for decoding basic auth credentials.
    /// </summary>
    public const String AUTHORIZATION_HEADER = "Authorization";
    public const String BASIC_AUTH = "basic ";
    
    /// <summary>
    /// Ordered with most common first (win-1252 latin characters). 
    /// </summary>
    public static readonly String[] BASIC_AUTH_ENCODINGS_ORDERED = {"Windows-1252",
     "shift-jis", "big5", "gb2312", "KSC5601"};
    
    /// <summary>
    /// Key to data stored on the HTTP Session 
    /// </summary>
    public const String LOGIN_ATTEMPTS = "BasicAuthAttempts";
    public const int MAX_LOGIN_ATTEMPTS = 10;
    private static HTTPBasicAuthenticator m_Instance = new HTTPBasicAuthenticator();
    private HTTPBasicAuthenticator()
    {
    }
    public static HTTPBasicAuthenticator GetInstance()
    {
      return m_Instance;
    }
    
    /// <summary>
    /// Performs login using HTTP basic authentication credentials in the request
    /// header.
    /// </summary>
    /// <returns>
    /// The LoginResult, or null if login is not attempted (since
    /// the request appears to be not for HTTP Basic auth).
    /// </returns>
    /// <seealso cref="com.plumtree.uiinfrastructure.login.IAuthenticator#Login(com.plumtree.uiinfrastructure.application.RequestData)"></seealso>
    public virtual LoginResult Login(RequestData requestData)
    {
      // Warning: This code is very brittle.  The various Microsoft
      // WebDav clients (different OS versions, different OS languages)
      // have many foibles, and this solution seems to work around all
      // of them.  If anything is changed, it could easily fail in some
      // way that is difficult to notice unless you test all clients,
      // especially Asian Character OS'es.
      if (UseThisAuth(requestData))
      {
        // Check how many times we have tried to log in before.  We don't want
        // to get caught in an infinite loop if the client gets our challenge
        // and sends back the same invalid username / password pair continually.
        // Most web dav clients will fail after serveral attempts, but we should
        // make sure we don't get caught in an accidental infinite loop if there
        // is a bug in a webdav client.
        Object oAttempts = requestData.m_smSandbox.GetAttribute(LOGIN_ATTEMPTS);
        int nAttempts = 0;
        // Can't use Integer, because it maps to int in c#, and you can't compare it to null
        if (null != oAttempts)
        {
          nAttempts = XPConvert.ToInteger(oAttempts);
        }
        LoginResult loginResult = null;
        // Handle basic auth login for WebDav clients.
        if (GetBasicAuthCredentialsExist(requestData))
        {
          nAttempts++;
          loginResult = DoBasicAuthLogin(requestData);
          if (loginResult.m_bSuccess)
          {
            // reset counter if login successful
            nAttempts = 0;
          } else
          {
            // Ignore the error until we reach max login attempts, then throw the error.
            // This will give the user the chance to re-enter their credentials and login.
            if (nAttempts < MAX_LOGIN_ATTEMPTS)
            {
              // Ignore this error so that user will be challenged again if they
              // typed in the incorrect credentials.
              log.Info("Basic Auth attempt failed. Ignoring so that user will be challenged again. Error: " + loginResult.m_strError);
              // Returning null will allow the control flow to continue as if basic auth
              // was not attempted.  This should cause another challenge to be issued
              // to the user.
              loginResult = null;
            } else
            {
              log.Info("Issued basic authentication challenge more than " + MAX_LOGIN_ATTEMPTS + " times, aborting basic authentication.");
              // Update the cache of how many times we have sent the challenge.
              // Reset this because they will now see a standard login page in HTML
              // which will break the challenge/response cycle.
              nAttempts = 0;
            }
          }
        }
        requestData.m_smSandbox.SetAttribute(LOGIN_ATTEMPTS, nAttempts);
        return loginResult;
      }
      // Returning null will allow the control flow to continue as if basic auth
      // was not attempted.
      return null;
    }
    
    /// <summary>
    /// Sends an HTTP Basic authentication challenge.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.login.IAuthenticator#GetOrSendLoginChallenge()"></seealso>
    public virtual Redirect GetOrSendLoginChallenge(RequestData requestData)
    {
      // Check how many times we have tried to log in before.  We don't want
      // to get caught in an infinite loop if the client gets our challenge
      // and sends back the same invalid username / password pair continually.
      // Most web dav clients will fail after serveral attempts, but we should
      // make sure we don't get caught in an accidental infinite loop if there
      // is a bug in a webdav client.
      Object oAttempts = requestData.m_smSandbox.GetAttribute(LOGIN_ATTEMPTS);
      int nAttempts = 0;
      // Can't use Integer, because it maps to int in c#, and you can't compare it to null
      if (null != oAttempts)
      {
        nAttempts = XPConvert.ToInteger(oAttempts);
      }
      if (nAttempts < MAX_LOGIN_ATTEMPTS)
      {
        log.Info("Issuing basic authentication challenge.");
        // Send basic authentication challenge for clients that cannot
        // handle the login page (i.e. WebDav).
        SendBasicAuthChallenge(requestData);
        // The calling method should exit
        return null;
      } else
      {
        log.Info("Issued basic authentication challenge more than " + MAX_LOGIN_ATTEMPTS + " times, aborting basic authentication.");
        // Update the cache of how many times we have sent the challenge.
        // Reset this because they will now see a standard login page in HTML
        // which will break the challenge/response cycle.
        requestData.m_smSandbox.SetAttribute(LOGIN_ATTEMPTS, 0);
        // WilliamA - In order to display the standard login page, send a
        // redirect to the same page, but without the /auth keyword.  This
        // will not prompt for basic auth, and should redirect to the login
        // page instead.
        Redirect redirect = new Redirect();
        String strURL = requestData.m_xpRequest.GetRequestURL();
        String redirectURL = null;
        // This handles removing the /auth keyword.  These are the URL formats:
        // http://www.host.com/portal/server.pt/auth?a=b
        // http://www.host.com/portal/auth/server.pt?a=b
        int ndx = XPStringUtility.IndexOf(strURL, AUTH_CHALLENGE);
        if (ndx != -1)
        {
          // remove the "/auth" from the URL
          redirectURL = XPStringUtility.Substring(strURL, 0, ndx) + XPStringUtility.Substring(strURL,
           ndx + AUTH_CHALLENGE.Length);
        } else
        {
          // should not happen, since otherwise if there's no AUTH_CHALLENGE
          // we won't get here.
          redirectURL = strURL;
        }
        redirect.SetIsHTTPRedirect(true);
        redirect.SetLinkToExternalURL(redirectURL);
        return redirect;
      }
    }
    
    /// <summary>
    /// Logout is handled by the ILoginControl in the Login AS. This method
    /// returns false to indicate no logout action has been performed.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.login.IAuthenticator#Logout(com.plumtree.uiinfrastructure.application.RequestData)"></seealso>
    public virtual bool Logout(RequestData requestData)
    {
      // Logout is handled by the ILoginControl in the Login AS
      return false;
    }
    
    /// <summary>
    /// Helper method to determine whether or not we should use an HTTP
    /// basic auth challenge to authenticate the user.
    /// </summary>
    /// <param name="requestData"></param>
    public virtual bool UseThisAuth(RequestData requestData)
    {
      String strURL = requestData.m_xpRequest.GetRequestURL();
      return IsBasicAuthURL(strURL);
    }
    public virtual bool IsBasicAuthURL(String strURL)
    {
      // The AUTH_CHALLENGE string must have another "directory" string
      // following it for Basic Auth to work.  E.g. portal/server.pt/auth/gateway
      // This is because many of the WebDav clients first attempt to authenticate
      // against the root URL (i.e. remove the last directory).  If /auth is the
      // last directory, it will be removed, and the web dav client will be
      // incorrectly logged in as guest and will be unable to authenticate
      // afterwards.
      // We require that the AUTH_CHALLENGE string occurs immediately after
      // the HTTP Entry Point (server.pt).  This is so it won't get confused
      // with a server named auth, or a gateway string containing auth.
      // E.g. http://portal.company.com/portal/server.pt/auth?
      // Alternately, the auth prefix can be right before the entry point.  This
      // allows us to use basic auth with friendly URLs.
      String strEntryPoint = ConfigHelper.GetHTTPEntryPoint();
      int iEntryPointIndex = XPStringUtility.IndexOf(strURL, strEntryPoint);
      String strPostfix = strURL;
      if (-1 != iEntryPointIndex)
      {
        strPostfix = XPStringUtility.Substring(strPostfix, iEntryPointIndex + strEntryPoint.Length);
        }
      if (XPStringUtility.StartsWith(strPostfix, AUTH_CHALLENGE))
      {
        // An URL of the form http://portal.company.com/portal/server.pt/auth?
        return true;
      }
      // Alternately, the auth prefix can be right before the entry point.  This
      // allows us to use basic auth with friendly URLs.
      String strPrefix = strURL;
      if (-1 != iEntryPointIndex)
      {
        strPrefix = XPStringUtility.Substring(strPrefix, 0, iEntryPointIndex + strEntryPoint.Length);
        int nLastSlash = XPStringUtility.LastIndexOf(strPrefix, '/');
        if (nLastSlash != -1)
        {
          // Remove the last slash (i.e. /server.pt)
          strPrefix = XPStringUtility.Substring(strPrefix, 0, nLastSlash);
          }
        nLastSlash = XPStringUtility.LastIndexOf(strPrefix, '/');
        if (nLastSlash != -1)
        {
          // Get everything after the last remaining slash (i.e. /auth)
          strPrefix = XPStringUtility.Substring(strPrefix, nLastSlash);
        }
        if (XPStringUtility.StartsWith(strPrefix, AUTH_CHALLENGE))
        {
          // An URL of the form http://portal.company.com/portal/auth/server.pt?
          return true;
        }
      }
      return false;
    }
    
    /// <summary>
    /// Helper method for basic auth login.
    /// </summary>
    /// <param name="RequestData">the data for this HTTPRequest</param>
    /// <returns>boolean true implies login was successful</returns>
    public static LoginResult DoBasicAuthLogin(RequestData tempData)
    {
      log.Info("Attempting to verify login credentials from basic authentication challenge.");
      String strCredentials = tempData.m_xpRequest.GetHeader(AUTHORIZATION_HEADER);
      // Validate the credentials before actually attempting login.  If we attempt
      // login, then the session manager will be cleared and we will lose track
      // of how many basic auth attempts have been made.  This could cause us to
      // get in an infinite loop with a poorly written or failed basic auth client.
      if (!LoginHelper.INSTANCE.HasValidBasicAuthLogin(strCredentials))
      {
        // Create a "fake" failed login result to terminate the basic auth login process.
        LoginResult result = new LoginResult(null);
        result.m_bSuccess = false;
        return result;
      }
      // This clears the AS cache and other data.
      LoginHandlers.DoLogout(tempData);
      log.Info("Logging in using login credentials from basic authentication challenge.");
      LoginResult loginResult = LoginHelper.INSTANCE.AttemptBasicAuthLogin(strCredentials,
       tempData.m_smSandbox, tempData.m_smPersistentSandbox, tempData.m_Application,
       tempData.GetRequestURL(), tempData.m_xpRequest, tempData.m_wdPageData);
      if (loginResult.m_bSuccess)
      {
        // You should never login as guest from login credentials from
        // basic authentication challenge.  This is
        // different from the above case with Edit Default Profiles.
        // However, we can't check for it here because we don't know the
        // guest user name or ID.
        // -- tempData.SetUserSession(loginResult.m_UserSession);
        // This is to handle a bug in the Windows WebDav client where it loses temporary
        // cookies when you go from browsing a directory to opening a document.  Therefore,
        // we set a permanent 30 minute cookie.  This code should be removed when we no
        // longer support the broken webdav client.
        // Get cookie expiration value from config file in seconds.
        int nExpiration = GetBasicAuthCookieExpiration(tempData);
        // 0 means no cookie.  negative is an invalid value
        if (0 != nExpiration)
        {
          log.Info("Setting " + nExpiration + " second permanent cookie for WebDav client.");
          LoginHelper.INSTANCE.SetAutologinToken(tempData.m_wdPageData, tempData.m_xpRequest,
           loginResult.m_UserSession, nExpiration, true);
        }
      } else
      {
        log.Error("Unable to log in using login credentials from basic authentication challenge: " + loginResult.m_strError);
        }
      return loginResult;
    }
    
    /// <summary>
    /// Helper method to determine whether or not we should use an HTTP
    /// basic auth challenge to authenticate the user.
    /// </summary>
    /// <param name="_tempData"></param>
    public virtual bool GetBasicAuthOverrideSSO(RequestData _tempData)
    {
      String strURL = _tempData.m_xpRequest.GetRequestURL();
      // We require that the AUTH_CHALLENGE string occurs immediately after
      // the HTTP Entry Point (server.pt).  This is so it won't get confused
      // with a server named auth, or a gateway string containing auth.
      // E.g. http://portal.company.com/portal/server.pt/auth?
      String strEntryPoint = ConfigHelper.GetHTTPEntryPoint();
      int iEntryPointIndex = XPStringUtility.IndexOf(strURL, strEntryPoint);
      if (-1 != iEntryPointIndex)
      {
        strURL = XPStringUtility.Substring(strURL, iEntryPointIndex + strEntryPoint.Length);
        }
      if (XPStringUtility.StartsWith(strURL, AUTH_CHALLENGE))
      {
        // At this point, we know we are in an auth challenge string,
        // but we don't know if this is set to override SSO or not.
        IApplication application = ApplicationManager.GetInstance().GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString());
        ConfigVarPack vpConfig = (ConfigVarPack) application.GetVarPackManager().GetVariablePackage(ConfigVarPack.VARPACK_ID);
        // Should this value be cached?
        String strOverride = vpConfig.GetConfigValueAsString(ConfigConstants.AUTHENTICATION_SECTION,
         ConfigConstants.AUTHOVERRIDESSO, ConfigConstants.VALUE_ATTRIBUTE);
        if ((null != strOverride) && (!"".Equals(strOverride)))
        {
          try
          {
            int nOverride = XPConvert.ToInteger(strOverride);
            if (0 == nOverride)
            {
              // Don't override SSO
              return false;
            }
          }
          catch (Exception e)
          {
            log.Warn(e, "Invalid non-integer value for " + ConfigConstants.AUTHOVERRIDESSO + " in " + ConfigConstants.AUTHENTICATION_SECTION + " section: " + strOverride + ".  Using default behavior - overriding SSO.");
            }
        }
        // By default, override SSO
        return true;
      }
      return false;
    }
    
    /// <summary>
    /// Helper method to get the basic auth cookie expiration value from
    /// the config files.
    /// Logs an error and returns a reasonable default value if there
    /// are problems.
    /// </summary>
    /// <param name="_tempData"></param>
    /// <returns>int nExpiration in seconds</returns>
    public static int GetBasicAuthCookieExpiration(RequestData _tempData)
    {
      int nExpiration;
      try
      {
        nExpiration = ConfigHelper.GetConfigVariableAsInt(ConfigConstants.AUTHENTICATION_SECTION,
         ConfigConstants.AUTHTOKENEXPIRATION, ConfigConstants.VALUE_ATTRIBUTE,
         (Application) _tempData.m_Application);
      }
      catch (Exception e)
      {
        log.Warn(e, "Unable to get valid " + ConfigConstants.AUTHTOKENEXPIRATION + " value.  Using default.");
        // Default value
        nExpiration = 30;
      }
      if (nExpiration < 0)
      {
        log.Warn(ConfigConstants.AUTHTOKENEXPIRATION + " invalid negative value.  Using default.");
        // Default value
        nExpiration = 30;
      }
      return nExpiration * 60;
    }
    
    /// <summary>
    /// This helper method checks whether or not there are basic auth
    /// credentials in the header.
    /// </summary>
    /// <param name="_tempData"></param>
    /// <returns>true implies there are credentials.</returns>
    private static bool GetBasicAuthCredentialsExist(RequestData _tempData)
    {
      // NOTE: All of this code was taken from XPRequest in Java.
      // The .NET version used server variables, and so didn't get
      // the auth headers.
      String strBase64Credentials = _tempData.m_xpRequest.GetHeader(AUTHORIZATION_HEADER);
      if ((null != strBase64Credentials) && (strBase64Credentials.Length > 0))
      {
        if (XPStringUtility.StartsWith(XPStringUtility.ToLowerCase(strBase64Credentials),
         BASIC_AUTH))
        {
          return true;
        }
      }
      return false;
    }
    
    /// <summary>
    /// Helper method to send an HTTP basic auth challenge to authenticate
    /// the user.  The calling method should exit without setting a response
    /// after calling this method.
    /// </summary>
    /// <param name="_tempData"></param>
    public static void SendBasicAuthChallenge(RequestData _tempData)
    {
      // Set Unauthorized status code
      _tempData.m_xpResponse.SetStatus(401);
      // Set basic auth challenge header
      _tempData.m_xpResponse.AddHeader("WWW-Authenticate", "Basic realm=\"" + AUTH_DOMAIN + "\"");
      }
  }
}