using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.errordisplay;
using com.plumtree.uiinfrastructure.pei;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.web;
using com.plumtree.xpshared.htmlelements;
// Put explicit import for Interpreter (to avoid ambiguity of using the deprecated class)
using Interpreter=com.plumtree.uiinfrastructure.interpreter.Interpreter;
namespace com.plumtree.uiinfrastructure.login
{
  /// <summary>
  /// This Singleton class controls session login.  To implement your own session
  /// login code, simply implement an instance of ILoginHelper and initialize this
  /// class with your helper using the StaticInit method.  All logins should be
  /// done through this class, rather than directly calling the custom login code
  /// in an ILoginHelper.
  /// </summary>
  /// <author>DonH</author>
  public sealed class LoginHelper
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.login.LoginHelper");
    
    /// <summary>
    /// helper class for custom login code. 
    /// </summary>
    private static ILoginHelper m_LoginHelper = null;
    
    /// <summary>
    /// whether or not this class has already been initialized. 
    /// </summary>
    private static bool m_bInitialized = false;
    
    /// <summary>
    /// constants for login token cookies. 
    /// </summary>
    public const String LOGIN_TOKEN_COOKIE_NAME = "ptLTCN";
    public const String LOGIN_TOKEN_COOKIE_DEFUNCT_VALUE = "ptLTCDV";
    
    /// <summary>
    /// constant for replication cookie used in high availability 
    /// </summary>
    public const String REPLICATION_COOKIE_NAME = "ptREPCN";
    
    /// <summary>
    /// This string is used by SSO login to determine if the user has clicked the login link. 
    /// </summary>
    public const String PARAM_LOGIN = "login";
    
    /// <summary>
    /// This string is used by SSO login to determine if the user has clicked the logout link. 
    /// </summary>
    public const String PARAM_DOLOGOUT = "doLogout";
    
    /// <summary>
    /// Parameter that indicates that the login control should show the SSO logout
    /// page rather than the standard login page.
    /// </summary>
    public const String PARAM_SSOLOGOUT = "in_hi_ssologout";
    
    /// <summary>
    /// Parameter indicates to loginControl that it should not
    /// attempt to login, it should just redirect as though it has
    /// logged in.  Used by SSO to have the login control handle
    /// redirecting to the correct place.
    /// </summary>
    public const String PARAM_DONT_LOGIN_JUST_REDIRECT = "dljr";
    
    /// <summary>
    /// The name of the guest user that will be passed in to the
    /// OnBeforeLogin method of the LoginActions PEI.  This could
    /// theoretically be different from the name of the guest user
    /// in the portal.
    /// </summary>
    public const String GUEST_USER_NAME = "Guest";
    
    /// <summary>
    /// Use this class static to access the one and only instance of
    /// LoginHelper.
    /// </summary>
    public static readonly LoginHelper INSTANCE = new LoginHelper();
    
    /// <summary>
    /// Private constructor to ensure there is only one instance.
    /// </summary>
    private LoginHelper()
    {
    }
    
    /// <summary>
    /// Static initialization method.  Use this method to add custom user
    /// session object handling code.
    /// This method does not need to be called.
    /// </summary>
    /// <param name="ILoginHelper">custom login code</param>
    public static void StaticInit(ILoginHelper loginHelper)
    {
      if (m_bInitialized)
      {
        log.Error("LoginHelper StaticInit attempted twice.  Ignoring second attempt.");
        return;
      }
      m_LoginHelper = loginHelper;
      m_bInitialized = true;
    }
    
    /// <summary>
    /// Attempts to create and connect a new user session.
    /// Calls Login PEIs.
    /// </summary>
    /// <returns>LoginResult the result of this login attempt.</returns>
    public LoginResult AttemptLogin(String strUserName, String strPassword,
     Object token, bool bUseNumericConnect, AActivitySpace asOwner, IXPRequest request,
     IWebData webData)
    {
      return AttemptLogin(strUserName, strPassword, token, bUseNumericConnect,
       asOwner.GetSubSession(), asOwner.GetPersistentSubSession(), asOwner.GetApplication(),
       request.GetRequestURL(), request, webData);
    }
    
    /// <summary>
    /// Attempts to create and connect a new user session.
    /// Calls Login PEIs. Note that this signature of AttemptGuestLogin
    /// will log in as the default guest user.
    /// </summary>
    /// <returns>LoginResult the result of this login attempt.</returns>
    public LoginResult AttemptGuestLogin(Object token, ISessionManager subSession,
     ISessionManager persistentSubSession, IApplication application, String strRequestURL,
     IXPRequest request, IWebData webData)
    {
      return InternalAttemptLogin(null, null, token, false, true, -1, subSession,
       persistentSubSession, application, strRequestURL, request, webData);
      }
    
    /// <summary>
    /// Attempts to create and connect a new guest user session.
    /// The session connects as the given guest user ID. The given username
    /// is used only for Login PEIs.
    /// Calls Login PEIs.
    /// </summary>
    /// <returns>LoginResult the result of this login attempt.</returns>
    public LoginResult AttemptGuestLogin(int nUserID, String strUserName,
     Object token, ISessionManager subSession, ISessionManager persistentSubSession,
     IApplication application, String strRequestURL, IXPRequest request, IWebData webData)
    {
      // Multiple-guest-user support, use the passed-in guest username
      return InternalAttemptLogin(strUserName, null, token, false, true, nUserID,
       subSession, persistentSubSession, application, strRequestURL, request,
       webData);
    }
    
    /// <summary>
    /// Attempts to create and connect a new user session.
    /// Calls Login PEIs.
    /// </summary>
    /// <returns>LoginResult the result of this login attempt.</returns>
    public LoginResult AttemptLogin(String strUserName, String strPassword,
     Object token, bool bUseNumericConnect, ISessionManager subSession, ISessionManager persistentSubSession,
     IApplication application, String strRequestURL, IXPRequest request, IWebData webData)
    {
      return InternalAttemptLogin(strUserName, strPassword, token, bUseNumericConnect,
       false, -1, subSession, persistentSubSession, application, strRequestURL,
       request, webData);
    }
    
    /// <summary>
    /// Attempts to create and connect a new user session.
    /// Calls Login PEIs.
    /// </summary>
    /// <returns>LoginResult the result of this login attempt.</returns>
    private LoginResult InternalAttemptLogin(String strUserName, String strPassword,
     Object token, bool bUseNumericConnect, bool bGuestLogin, int nGuestUserID,
     ISessionManager subSession, ISessionManager persistentSubSession, IApplication application,
     String strRequestURL, IXPRequest request, IWebData webData)
    {
      if (m_LoginHelper == null)
      {
        // The guest user is not a valid logged in user.
        SetSessionValidity(!bGuestLogin, subSession);
        return new LoginResult(null, null, bGuestLogin);
      }
      bool bSuccess = true;
      Redirect redirect = null;
      Object userSession = null;
      String strError = null;
      Exception loginException = null;
      if ((null == strUserName) && !bGuestLogin)
      {
        // This means the user is an auto-login user.
        strUserName = "";
      }
      try
      {
        String strPEIUserName = strUserName;
        if (null == strPEIUserName)
        {
          // We do not know the guest user name at this point, so we simply
          // hardcode it.
          strPEIUserName = GUEST_USER_NAME;
        }
        redirect = DoTasksBeforeLogin(strPEIUserName, request, webData, application,
         subSession, persistentSubSession);
        if (bGuestLogin && (null != redirect) && (!redirect.GetIsExternalURL()))
        {
          // We ignore internal guest redirects because you
          // cannot access the portal without logging in as
          // guest.
          redirect = null;
          log.Warn("Ignoring internal redirect for guest user from OnBeforeLogin PEI.");
          }
        if (redirect != null)
        {
          // Since the guest user is not a valid logged in user,
          // and we haven't actually attempted login yet, this
          // user is invalid.
          SetSessionValidity(false, subSession);
          return new LoginResult(redirect);
        }
        if (bGuestLogin)
        {
          // WilliamA - for multiple-guest-user support, pass in
          // the guest username
          userSession = m_LoginHelper.AttemptGuestLogin(nGuestUserID, token,
           subSession, application, strRequestURL);
        } else
        {
          userSession = m_LoginHelper.AttemptLogin(strUserName, strPassword,
           token, bUseNumericConnect, subSession, application, strRequestURL);
          }
        if (userSession == null)
        {
          bSuccess = false;
        } else
        {
          redirect = DoTasksAfterLogin(userSession, request, webData, application,
           subSession, persistentSubSession);
        }
      }
      catch (Exception e)
      {
        // Don't log this exception here - it will get logged down below.
        loginException = e;
        userSession = null;
        redirect = null;
        bSuccess = false;
      }
      if (!bSuccess)
      {
        // If login failed
        FailedLoginResult failedLoginResult = LoginHelper.INSTANCE.DoTaskOnFailedLogin(strUserName,
         XPException.GetInstance(loginException), request, webData, application,
         subSession, persistentSubSession);
        strError = failedLoginResult.GetErrorMessage();
        strError+=m_LoginHelper.GetDetailedFailureMessage(loginException,
         subSession, application);
        if (null != failedLoginResult.GetFailureRedirect())
        {
          redirect = failedLoginResult.GetFailureRedirect();
        }
        if ((bGuestLogin) && (null == strUserName || XPStringUtility.EqualsIgnoreCase("",
         strUserName)))
        {
          // This may not actually be the guest user name, but since it is
          // only used in PTSpy, it should be okay.
          strUserName = "guest user";
        }
        if (null != loginException)
        {
          log.Warn(loginException, "Unable to log in user " + strUserName + ": " + strError);
          } else
        {
          log.Warn("Unable to log in user " + strUserName + ": " + strError);
          }
        // The guest user is not a valid logged in user.
        SetSessionValidity(!bGuestLogin, subSession);
        return new LoginResult(userSession, redirect, strError, bGuestLogin);
        }
      // The guest user is not a valid logged in user.
      SetSessionValidity(!bGuestLogin, subSession);
      return new LoginResult(userSession, redirect, bGuestLogin);
    }
    
    /// <summary>
    /// Attempts to create and connect a new user session.
    /// Calls Login PEIs.
    /// </summary>
    /// <returns>LoginResult the result of this login attempt.</returns>
    public LoginResult AttemptLogin(String sLoginToken, AActivitySpace asOwner,
     IXPRequest request, IWebData webData)
    {
      return AttemptLogin(sLoginToken, asOwner.GetSubSession(), asOwner.GetPersistentSubSession(),
       asOwner.GetApplication(), request.GetRequestURL(), request, webData);
      }
    
    /// <summary>
    /// Attempts to create and connect a new user session.
    /// Calls Login PEIs.
    /// </summary>
    /// <returns>LoginResult the result of this login attempt.</returns>
    public LoginResult AttemptLogin(String sLoginToken, ISessionManager subSession,
     ISessionManager persistentSubSession, IApplication application, String sRequestURL,
     IXPRequest request, IWebData webData)
    {
      if (m_LoginHelper == null)
      {
        // Until we actually log in this new user, assume it is an invalid /
        // not logged in session.
        SetSessionValidity(false, subSession);
        return new LoginResult(null, null, true);
      }
      bool bSuccess = true;
      bool bIsGuestUser = true;
      Redirect redirect = null;
      Object userSession = null;
      String strError = null;
      String strUserName = "";
      Exception loginException = null;
      try
      {
        // We do not know the user name at this point.
        redirect = DoTasksBeforeLogin(strUserName, request, webData, application,
         subSession, persistentSubSession);
        if (redirect != null)
        {
          // Until we actually log in this new user, assume it is an invalid /
          // not logged in session.
          SetSessionValidity(false, subSession);
          return new LoginResult(null, redirect, null, true);
        }
        //ES - pass in the XPrequest
        userSession = m_LoginHelper.AttemptLogin(sLoginToken, subSession,
         application, sRequestURL, request);
        if (userSession == null)
        {
          bSuccess = false;
        } else
        {
          strUserName = m_LoginHelper.GetUserSessionName(userSession);
          bIsGuestUser = m_LoginHelper.GetIsGuestUser(userSession);
          redirect = DoTasksAfterLogin(userSession, request, webData, application,
           subSession, persistentSubSession);
        }
      }
      catch (Exception e)
      {
        log.Error(e, "Unable to log in user " + strUserName + " using login token.");
        loginException = XPException.GetInstance(e);
        userSession = null;
        redirect = null;
        bSuccess = false;
      }
      if (!bSuccess)
      {
        // If login failed
        FailedLoginResult failedLoginResult = LoginHelper.INSTANCE.DoTaskOnFailedLogin(strUserName,
         XPException.GetInstance(loginException), request, webData, application,
         subSession, persistentSubSession);
        strError = failedLoginResult.GetErrorMessage();
        if ((strError == null) || ("".Equals(strError)))
        {
          strError = m_LoginHelper.GetDetailedFailureMessage(loginException,
           subSession, application);
        }
        if (null != failedLoginResult.GetFailureRedirect())
        {
          redirect = failedLoginResult.GetFailureRedirect();
        }
        log.Error("Unable to log in user " + strUserName + ": " + strError);
        // The guest user is not a valid logged in user.
        SetSessionValidity(!bIsGuestUser, subSession);
        return new LoginResult(userSession, redirect, strError, bIsGuestUser);
        }
      // The guest user is not a valid logged in user.
      SetSessionValidity(!bIsGuestUser, subSession);
      return new LoginResult(userSession, redirect, bIsGuestUser);
    }
    
    /// <summary>
    /// Handles login tasks for the new session.
    /// Calls Login PEIs, but does not call AttemptLogin.
    /// This is used while editing default profiles and impersonating
    /// users.  This should not be used otherwise.
    /// </summary>
    /// <param name="Object">userSession - a pre-connected user session.</param>
    /// <returns>LoginResult the result of this login attempt.</returns>
    public LoginResult AttemptLogin(Object userSession, AActivitySpace asOwner,
     IXPRequest request, IWebData webData)
    {
      return AttemptLogin(userSession, asOwner.GetSubSession(), asOwner.GetPersistentSubSession(),
       asOwner.GetApplication(), request, webData);
    }
    
    /// <summary>
    /// Handles login tasks for the new session.
    /// Calls Login PEIs, but does not call AttemptLogin.
    /// This is used while editing default profiles and impersonating
    /// users.  This should not be used otherwise.
    /// </summary>
    /// <param name="Object">userSession - a pre-connected user session.</param>
    /// <returns>LoginResult the result of this login attempt.</returns>
    public LoginResult AttemptLogin(Object userSession, ISessionManager subSession,
     ISessionManager persistentSubSession, IApplication application, IXPRequest request,
     IWebData webData)
    {
      if ((m_LoginHelper == null) || (userSession == null))
      {
        // This method is asked to login a specific userSession object, therefore
        // it is assumed that this is a valid logged in session, not the guest user.
        SetSessionValidity(true, subSession);
        return new LoginResult(null, null, false);
      }
      bool bSuccess = true;
      Redirect redirect = null;
      String strError = null;
      String strUserName = null;
      Exception loginException = null;
      try
      {
        strUserName = m_LoginHelper.GetUserSessionName(userSession);
        redirect = DoTasksBeforeLogin(strUserName, request, webData, application,
         subSession, persistentSubSession);
        if (redirect != null)
        {
          // This method is asked to login a specific userSession object, therefore
          // it is assumed that this is a valid logged in session, not the guest user.
          SetSessionValidity(true, subSession);
          return new LoginResult(null, redirect, null, false);
        }
        redirect = DoTasksAfterLogin(userSession, request, webData, application,
         subSession, persistentSubSession);
      }
      catch (Exception e)
      {
        log.Error(e, "Unable to log in user " + strUserName + " using pre-supplied user session.");
        loginException = XPException.GetInstance(e);
        userSession = null;
        redirect = null;
        bSuccess = false;
      }
      if (!bSuccess)
      {
        // If login failed
        FailedLoginResult failedLoginResult = LoginHelper.INSTANCE.DoTaskOnFailedLogin(strUserName,
         XPException.GetInstance(loginException), request, webData, application,
         subSession, persistentSubSession);
        strError = failedLoginResult.GetErrorMessage();
        if ((strError == null) || ("".Equals(strError)))
        {
          strError = m_LoginHelper.GetDetailedFailureMessage(loginException,
           subSession, application);
        }
        if (null != failedLoginResult.GetFailureRedirect())
        {
          redirect = failedLoginResult.GetFailureRedirect();
        }
        log.Error("Unable to log in user " + strUserName + ": " + strError);
        // This method is asked to login a specific userSession object, therefore
        // it is assumed that this is a valid logged in session, not the guest user.
        SetSessionValidity(true, subSession);
        return new LoginResult(userSession, redirect, strError, false);
      }
      // This method is asked to login a specific userSession object, therefore
      // it is assumed that this is a valid logged in session, not the guest user.
      SetSessionValidity(true, subSession);
      return new LoginResult(userSession, redirect, false);
    }
    
    /// <summary>
    /// This method checks whether or not the supplied credentials are valid basic
    /// auth credentials, without actually logging the user in completely (a new PTSession
    /// is created).
    /// </summary>
    /// <param name="sEncodedCredentials">the encoded basic auth credentials to validate</param>
    /// <returns>whether or not the supplied credentials are valid basic auth credentials.</returns>
    public bool HasValidBasicAuthLogin(String sEncodedCredentials)
    {
      try
      {
        // Attempt to login using the basic auth credentials
        if (null != m_LoginHelper.AttemptBasicAuthLogin(sEncodedCredentials))
        {
          // If we got back a valid user session, then basic auth will work.
          return true;
        }
      }
      catch (Exception e)
      {
        log.Debug(e, "Exception attempting to verify if the basic auth login is valid or not.");
        return false;
      }
      return false;
    }
    
    /// <summary>
    /// Attempts to create and connect a new user session.
    /// This method is used for Basic Auth logins.  Unfortunately, in Basic Auth
    /// the user name and password are encoded in the character encoding of the
    /// client machine, and the server does not know what that is.  Therefore
    /// we have to guess the appropriate encoding.  The user credentials
    /// passed to this method need to be unencoded as win-1252, shift-jis, Big5,
    /// etc... before they can be used.  The credentials are basic auth credentials;
    /// user name and password are separated by a colon.
    /// </summary>
    /// <returns>LoginResult the result of this login attempt.</returns>
    public LoginResult AttemptBasicAuthLogin(String sEncodedCredentials, ISessionManager subSession,
     ISessionManager persistentSubSession, IApplication application, String strRequestURL,
     IXPRequest request, IWebData webData)
    {
      if (m_LoginHelper == null)
      {
        // Assume this is a valid user.
        SetSessionValidity(true, subSession);
        return new LoginResult(null, null, false);
      }
      if (null == sEncodedCredentials)
      {
        // We can't do basic auth login with empty credentials
        return new LoginResult(null, null, "Invalid basic auth user credentials.",
         true);
      }
      bool bSuccess = true;
      Redirect redirect = null;
      Object userSession = null;
      String strError = null;
      Exception loginException = null;
      String strUserName = null;
      try
      {
        // Attempt to figure out the appropriate encoding of the user name and password.
        userSession = m_LoginHelper.AttemptBasicAuthLogin(sEncodedCredentials);
        if (null == userSession)
        {
          throw new XPException("Unable to log in user using basic auth.");
          }
        strUserName = GetUserSessionName(userSession);
        redirect = DoTasksBeforeLogin(strUserName, request, webData, application,
         subSession, persistentSubSession);
        if (redirect != null)
        {
          // This method is asked to login a specific userSession object, therefore
          // it is assumed that this is a valid logged in session, not the guest user.
          SetSessionValidity(true, subSession);
          return new LoginResult(null, redirect, null, false);
        }
        redirect = DoTasksAfterLogin(userSession, request, webData, application,
         subSession, persistentSubSession);
      }
      catch (Exception e)
      {
        // Don't log this exception here - it will get logged down below.
        loginException = e;
        userSession = null;
        redirect = null;
        bSuccess = false;
      }
      if (!bSuccess)
      {
        // If login failed
        XPException loginXPException = null;
        if (null != loginException)
        {
          loginXPException = XPException.GetInstance(loginException);
        }
        FailedLoginResult failedLoginResult = LoginHelper.INSTANCE.DoTaskOnFailedLogin(strUserName,
         loginXPException, request, webData, application, subSession, persistentSubSession);
        strError = failedLoginResult.GetErrorMessage();
        strError+=m_LoginHelper.GetDetailedFailureMessage(loginException,
         subSession, application);
        if (null != failedLoginResult.GetFailureRedirect())
        {
          redirect = failedLoginResult.GetFailureRedirect();
        }
        if (null != loginException)
        {
          log.Warn(loginException, "Unable to do basic auth log in for user " + strUserName + ": " + strError);
          } else
        {
          log.Warn("Unable to do basic auth log in for user " + strUserName + ": " + strError);
          }
        SetSessionValidity(false, subSession);
        return new LoginResult(userSession, redirect, strError, false);
      }
      // The guest user is not a valid logged in user.
      SetSessionValidity(true, subSession);
      return new LoginResult(userSession, redirect, false);
    }
    
    /// <summary>
    /// This method gets the user ID from the supplied user
    /// session object.
    /// </summary>
    /// <param name="Object">the user session</param>
    /// <returns>
    /// String the user ID (return "" instead of null
    /// if necessary)
    /// </returns>
    /// <seealso cref="com.plumtree.uiinfrastructure.login.ILoginHelper#GetUserSessionID(Object)"></seealso>
    public String GetUserSessionID(Object userSession)
    {
      if (m_LoginHelper == null)
      {
        return "";
      }
      return m_LoginHelper.GetUserSessionID(userSession);
    }
    
    /// <summary>
    /// This method gets the user Name from the supplied user
    /// session object.
    /// </summary>
    /// <param name="Object">the user session</param>
    /// <returns>
    /// String the user name (return "" instead of null
    /// if necessary)
    /// </returns>
    /// <seealso cref="com.plumtree.uiinfrastructure.login.ILoginHelper#GetUserSessionName(Object)"></seealso>
    public String GetUserSessionName(Object userSession)
    {
      if (m_LoginHelper == null)
      {
        return "";
      }
      return m_LoginHelper.GetUserSessionName(userSession);
    }
    
    /// <summary>
    /// This method returns whether or not the user is a guest user.
    /// </summary>
    /// <param name="Object">the user session</param>
    /// <returns>boolean true implies the user is a guest user.</returns>
    /// <seealso cref="com.plumtree.uiinfrastructure.login.ILoginHelper#GetIsGuestUser(Object)"></seealso>
    public bool GetIsGuestUser(Object userSession)
    {
      if (m_LoginHelper == null)
      {
        // Return true because it's safer
        // to assume everyone is the guest than the other way around.
        return true;
      }
      return m_LoginHelper.GetIsGuestUser(userSession);
    }
    
    /// <summary>
    /// Call this method to set an autologin token for the user.
    /// </summary>
    /// <param name="oWebData"></param>
    /// <param name="xpRequest"></param>
    /// <param name="oUserSession"></param>
    /// <param name="nExpires">cookie and token expiration time in seconds.  -1 means use default value.</param>
    /// <param name="bIsPersistentCookie">
    /// false means don't set an expires time
    /// (expire at end of session).
    /// </param>
    public void SetAutologinToken(IWebData oWebData, IXPRequest xpRequest,
     Object oUserSession, int nExpires, bool bIsPersistentCookie)
    {
      if (null != m_LoginHelper)
      {
        m_LoginHelper.SetAutologinToken(oWebData, oUserSession, nExpires,
         bIsPersistentCookie, ConfigHelper.IsLoginTokenSecure(xpRequest.GetRequestURL()));
        }
    }
    
    /// <summary>
    /// Call this method to obtain an autologin token from the request.
    /// </summary>
    /// <param name="request">IXPRequest</param>
    /// <returns>
    /// String token that may be used to log in.  Null if no
    /// token is available.
    /// </returns>
    public String GetAutologinToken(IXPRequest request)
    {
      XPCookie[] ca = null;
      String sCookieVal = null;
      if (request != null)
      {
        ca = request.GetCookies();
      }
      if (ca != null)
      {
        int iNum = ca.Length;
        XPCookie c;
        log.Debug("CheckActionSecurityAndExecute() - got cookies!");
        log.Debug("CheckActionSecurityAndExecute() - iNum: " + iNum);
        for (int x = 0;x < iNum;x++)
        {
          c = ca[x];
          log.Debug("CheckActionSecurityAndExecute() - Got a cookie: " + c.GetName());
          if (LoginHelper.LOGIN_TOKEN_COOKIE_NAME.Equals(c.GetName()))
          {
            log.Debug("CheckActionSecurityAndExecute() - It\'s a token");
            sCookieVal = c.GetValue();
            if (LoginHelper.LOGIN_TOKEN_COOKIE_DEFUNCT_VALUE.Equals(sCookieVal))
            {
              sCookieVal = null;
            }
          }
        }
      }
      //	it's defunct
      // it's the login token cookie
      //	for each cookie
      //	got cookies
      return sCookieVal;
    }
    
    /// <summary>
    /// Call this method to clear an autologin token cookie from the
    /// users session.
    /// </summary>
    /// <param name="String">Current Request URL.</param>
    /// <param name="IWebData">the page data object for the current request.</param>
    public void ClearAutologinToken(String strCurrentRequestURL, IWebData webData)
    {
      // Not setting an expires value on a cookie uses the default behaviour in both
      // Java and .NET (deleted when browser is closed).
      // Perhaps this should delete the cookie (0 in Java, time in the past on .NET)
      XPCookie cookie = new XPCookie(LoginHelper.LOGIN_TOKEN_COOKIE_NAME,
       LoginHelper.LOGIN_TOKEN_COOKIE_DEFUNCT_VALUE);
      cookie.SetPath("");
      cookie.SetSecure(ConfigHelper.IsLoginTokenSecure(strCurrentRequestURL));
      webData.AddCookie(cookie);
    }
    
    /// <summary>
    /// Call this method to obtain a redirect to the default space.
    /// </summary>
    /// <param name="ILink">
    /// user supplied link that will get filled in with the
    /// proper parameters. (Redirect or ASURL object)
    /// </param>
    /// <param name="Object">userSession</param>
    /// <param name="AActivitySpace"></param>
    /// <returns>ILink to the default page for this user. (null if error)</returns>
    public ILink GetDefaultPageRedirect(ILink suppliedLink, Object userSession,
     AActivitySpace asOwner)
    {
      if (suppliedLink == null)
      {
        return null;
      }
      if (m_LoginHelper == null)
      {
        // Note This is bad - Should use Interpreter.LOGINSPACE once Interpreter
        // is moved down to UIInfrastructure level.
        suppliedLink.SetLinkCreateNewSpace("Login", asOwner);
        return suppliedLink;
      }
      return m_LoginHelper.GetDefaultPageRedirect(suppliedLink, userSession,
       asOwner);
    }
    
    /// <summary>
    /// Call this method to obtain a redirect to the default space.
    /// </summary>
    /// <param name="ILink">
    /// user supplied link that will get filled in with the
    /// proper parameters. (Redirect or ASURL object)
    /// </param>
    /// <param name="Object">userSession</param>
    /// <param name="IXPRequest">request</param>
    /// <returns>ILink to the default page for this user. (null if error)</returns>
    public ILink GetDefaultPageRedirect(ILink suppliedLink, Object userSession,
     IXPRequest request)
    {
      if (suppliedLink == null)
      {
        return null;
      }
      if (m_LoginHelper == null)
      {
        // Note This is bad - Should use Interpreter.LOGINSPACE once Interpreter
        // is moved down to UIInfrastructure level.
        suppliedLink.SetLinkCreateNewSpace("Login", null);
        return suppliedLink;
      }
      return m_LoginHelper.GetDefaultPageRedirect(suppliedLink, userSession,
       request);
    }
    
    /// <summary>
    /// This function will process all of the OnLogout ILoginActions functions
    /// and send the necessary Redirect back to the caller.
    /// </summary>
    /// <param name="AActivitySpace"></param>
    /// <returns>
    /// Redirect - Redirect to where the user should be sent after the
    /// logout.
    /// </returns>
    public Redirect DoTasksBeforeLogout(AActivitySpace asSpace, IXPRequest request,
     IWebData webData)
    {
      return DoTasksBeforeLogout(asSpace.GetUserSession(), request, webData,
       asSpace.GetApplication(), asSpace.GetSubSession(), asSpace.GetPersistentSubSession());
      }
    
    /// <summary>
    /// This function will process all of the OnLogout ILoginActions functions
    /// and send the necessary Redirect back to the caller.
    /// </summary>
    /// <param name="userSession"></param>
    /// <param name="application"></param>
    /// <param name="sessionManager"></param>
    /// <returns>
    /// Redirect - Redirect to where the user should be sent after the
    /// logout.
    /// </returns>
    public Redirect DoTasksBeforeLogout(Object userSession, IXPRequest request,
     IWebData webData, IApplication application, ISessionManager sessionManager,
     ISessionManager persistentSessionManager)
    {
      ApplicationData myData = new ApplicationData(request, webData, application,
       sessionManager, persistentSessionManager);
      //	ON LOGOUT
      log.Info("Entering DoTasksBeforeLogout()");
      Object[] oa;
      Object o;
      ILoginActions iActions;
      oa = (Object[]) application.GetCachingManager().GetEntry("LoginActions");
      Redirect rReturn = null;
      if (oa != null)
      {
        for (int x = 0;x < oa.Length;x++)
        {
          o = oa[x];
          log.Debug("DoTasksBeforeLogout() - About to call OnBeforeLogout on an: " + o);
          if (o != null)
          {
            iActions = (ILoginActions) o;
            //	OnBeforeLogout returns a Redirect.
            //	If it returns a valid object then this Redirect is stored,
            //	and possibly returned.  If there are multiple implementations
            //	of this method, and multiple valid redirects are returned,
            //	then the last one will stick, and the others will be forgotten.
            rReturn = iActions.OnBeforeLogout(userSession, myData);
            if (rReturn != null)
            {
              return rReturn;
            }
          }
        }
      }
      return rReturn;
    }
    
    /// <summary>
    /// This method sets the validity of the user session (guest or not)
    /// on the subSession.
    /// </summary>
    /// <param name="boolean">true implies the session is valid</param>
    /// <param name="subSession"></param>
    private void SetSessionValidity(bool bValidSession, ISessionManager subSession)
    {
      // A session is valid if a user has been signed in at some point.  Therefore,
      // a normal Plumtree user's session is valid, and so is a guest session when
      // it comes from editing the guest profile from an administrator account
      // (that was properly signed in).
      //
      // A session is invalid if no one has signed in.  When a user first browses
      // to the portal, in general, they are automatically connected as the guest
      // user.  Since they have not explicitly entered a password anywhere, that
      // session is treated as invalid, meaning it does not have full access to
      // everything that a normal session would (editing that users settings,
      // etc...).
      if (bValidSession)
      {
        // logged in user
        subSession.SetAttribute(Interpreter.USERSESSIONVALID, true);
      } else
      {
        // automatically connected guest user
        subSession.SetAttribute(Interpreter.USERSESSIONVALID, false);
      }
    }
    
    /// <summary>
    /// This method does tasks before login.
    /// </summary>
    /// <param name="String">the name of the user</param>
    /// <param name="application"></param>
    /// <param name="subSession"></param>
    /// <returns>Redirect (internal redirects will be ignored for guest)</returns>
    /// <seealso cref="com.plumtree.uiinfrastructure.login.ILoginHelper#DoTasksBeforeLogin()"></seealso>
    private Redirect DoTasksBeforeLogin(String strUserName, IXPRequest request,
     IWebData webData, IApplication application, ISessionManager sessionManager,
     ISessionManager persistentSessionManager)
    {
      ApplicationData myData = new ApplicationData(request, webData, application,
       sessionManager, persistentSessionManager);
      Object[] oa;
      Object o;
      ILoginActions iActions;
      Redirect actionReturn = new Redirect();
      oa = (Object[]) application.GetCachingManager().GetEntry("LoginActions");
      //	ON BEFORE LOGIN
      if (oa != null)
      {
        for (int x = 0;x < oa.Length;x++)
        {
          o = oa[x];
          log.Debug("internalLogin() - About to call OnLogin on an: " + o);
          if (o != null)
          {
            iActions = (ILoginActions) o;
            //	OnBeforeLogin returns a Redirect.
            //	If it returns a valid object then this Redirect is stored,
            //	and possibly returned.  If there are multiple implementations
            //	of this method, and multiple valid redirects are returned,
            //	then the last one will stick, and the others will be forgotten.
            actionReturn = iActions.OnBeforeLogin(myData, strUserName);
          }
          if (actionReturn != null)
          {
            return actionReturn;
          }
        }
      }
      return actionReturn;
    }
    
    /// <summary>
    /// This method does tasks after login.
    /// </summary>
    /// <param name="userSession"></param>
    /// <param name="application"></param>
    /// <param name="sessionManager"></param>
    /// <returns>Redirect</returns>
    /// <seealso cref="com.plumtree.uiinfrastructure.login.ILoginHelper#DoTasksAfterLogin()"></seealso>
    private Redirect DoTasksAfterLogin(Object userSession, IXPRequest request,
     IWebData webData, IApplication application, ISessionManager sessionManager,
     ISessionManager persistentSessionManager)
    {
      ApplicationData myData = new ApplicationData(request, webData, application,
       sessionManager, persistentSessionManager);
      //	ON AFTER LOGIN
      log.Info("Entering DoTasksAfterLogin()");
      if (userSession == null)
      {
        // This happens when logging in the guest, because the login
        // control doesn't have the new session.  The interpreter handles
        // calling this again with the correct session.
        log.Info("DoTasksAfterLogin called with null session.  Exiting.");
        return null;
      }
      // Clear the user preferences for the user who we just logged out.  This needs
      // to be done before the first login action since otherwise we would cache the
      // settings for the user who just logged in, and then clear them out.
      if (PersonalSettingsHelper.DoesPrefsSubSessionExist(sessionManager))
      {
        // clear the prefs subsession
        sessionManager.GetSubSession(PersonalSettingsHelper.PREFS_SUBSESSION_NAME).Clear();
        } else
      {
        log.Warn("SubSession for prefs cache does not exist. Make sure AActivitySpace is valid.");
        }
      // Clear the UI Errors before doing onAfterLogin actions so that we can raise
      // ui errors in the PEIs and not have them get cleared later.
      UIErrorHandler.ClearErrors(sessionManager, application);
      // Clear the entire session manager and re-initialize it.
      Interpreter.ClearSandbox(sessionManager);
      Object[] oa;
      Object o;
      ILoginActions iActions;
      oa = (Object[]) application.GetCachingManager().GetEntry("LoginActions");
      // mj: if we get this far, myIPTSession must be non-null and connected
      // now we call all the login actions
      Redirect rReturn = null;
      Redirect rTemp = null;
      if (oa != null)
      {
        for (int x = 0;x < oa.Length;x++)
        {
          o = oa[x];
          log.Debug("DoTasksAfterLogin() - About to call OnAfterLogin on an: " + o);
          if (o != null)
          {
            iActions = (ILoginActions) o;
            //	OnAfterLogin returns a Redirect.
            //	If it returns a valid object then this Redirect is stored,
            //	and possibly returned.  If there are multiple implementations
            //	of this method, and multiple valid redirects are returned,
            //	then the last one will stick, and the others will be forgotten.
            rTemp = iActions.OnAfterLogin(userSession, myData);
            if (rTemp != null)
            {
              rReturn = rTemp;
              return rReturn;
            }
          }
        }
      }
      return rReturn;
    }
    
    /// <summary>
    /// This function will load all of the implemented LoginActions PEIs and then
    /// process through them and call the OnFailedLogin function for each one.
    /// In general, this method should only be called by the LoginHelper itself.
    /// The only reason to call this  method directly is if you are failing
    /// the login before calling the AttemptLogin method (i.e. because of a
    /// hack attempt).
    /// </summary>
    /// <param name="_strUserName">- user that has failed the login process.</param>
    /// <param name="error">- the exception that caused login to fail, may be null</param>
    /// <param name="application"></param>
    /// <param name="sessionManager"></param>
    /// <returns>FailedLoginResult - Error message and/or Redirect from the PEI.</returns>
    public FailedLoginResult DoTaskOnFailedLogin(String _strUserName, XPException error,
     IXPRequest request, IWebData webData, IApplication application, ISessionManager sessionManager,
     ISessionManager persistentSessionManager)
    {
      ApplicationData myData = new ApplicationData(request, webData, application,
       sessionManager, persistentSessionManager);
      //	ON FAILED LOGIN
      log.Info("Entering DoTasksOnFailedLogin()");
      Object[] oa;
      Object o;
      ILoginActions2 iActions;
      String sErrorMsgs = "";
      Redirect rReturn = null;
      oa = (Object[]) application.GetCachingManager().GetEntry("LoginActions");
      if (oa != null)
      {
        for (int x = 0;x < oa.Length;x++)
        {
          o = oa[x];
          log.Debug("DoTasksOnFailedLogin() - About to call OnFailedLogin and onFailedLoginDoRedirect on an: " + o);
          if (o != null)
          {
            String sError;
            iActions = (ILoginActions2) o;
            sError = iActions.OnFailedLogin(myData, _strUserName, error);
            if ((null != sError) && (sError.Length > 0))
            {
              sErrorMsgs+=sError + " ";
            }
            rReturn = iActions.OnFailedLoginDoRedirect(myData, _strUserName,
             error);
            if (null != rReturn)
            {
              // If we get a valid redirect, that will shortcircuit the PEI process.
              return new FailedLoginResult(sErrorMsgs, rReturn);
            }
          }
        }
      }
      if (null == rReturn)
      {
        // Bug 41762 - Failed login should redirect based on the subportal's login page setting.
        // Bug 46109 - SSO always clears the sandbox regardless of whether the pending
        // login will be sucessful or not - store in the persistent sandbox instead. 
        rReturn = Authenticator.GetGuestLoginRedirect(application, persistentSessionManager,
         request, null);
      }
      if (null == sErrorMsgs)
      {
        sErrorMsgs = "";
      }
      return new FailedLoginResult(sErrorMsgs, rReturn);
    }
    
    /// <summary>
    /// Returns an HTMLElementCollection of style sheet links.  A style sheet is
    /// named "stylesheetname-language", for instance "mainstyle12-en".
    /// </summary>
    /// <param name="userSession"></param>
    /// <param name="AActivitySpace"></param>
    public HTMLElementCollection GetStyleSheetLinks(Object userSession, AActivitySpace asOwner)
    {
      if (null != m_LoginHelper)
      {
        return m_LoginHelper.GetStyleSheetLinks(userSession, asOwner);
      } else
      {
        // TODO: There should probably be some appropriate default behavior
        // for this. 
        return new HTMLElementCollection();
      }
    }
    
    /// <summary>
    /// This method is used to handle COM Process Recycling in .NET.  This should
    /// not be used by anyone outside of the Interpreter.
    /// </summary>
    /// <param name="userSession"></param>
    /// <returns></returns>
    public Object HandleCOMProcessRecycled(Object userSession)
    {
      if (null != m_LoginHelper)
      {
        return m_LoginHelper.HandleCOMPlusProcessRecycled(userSession);
      } else
      {
        return null;
      }
    }
  }
}