using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.interpreter;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.uiinfrastructure.login
{
  /// <summary>
  /// The authentication module for SSO. It allows the caller to send a 
  /// redirect to the SSO Login / Logout URL.
  /// </summary>
  /// <author>Mike Jahr (Original SSO code)</author>
  /// <author>WilliamA  (Writing this new class)</author>
  public class SSOAuthenticator : IAuthenticator
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.login.SSOAuthenticator");
    // Cookie name for asp.net session id
    public const String ASPNET_SESSIONID_COOKIENAME = "ASP.NET_SessionId";
    private static SSOAuthenticator m_Instance = new SSOAuthenticator();
    private SSOAuthenticator()
    {
    }
    public static SSOAuthenticator GetInstance()
    {
      return m_Instance;
    }
    
    /// <summary>
    /// The actual login is performed by the SSO server, not here. Thus, this
    /// method does nothing for SSO case.
    /// </summary>
    /// <returns>null since login is not performed here.</returns>
    /// <seealso cref="com.plumtree.uiinfrastructure.login.IAuthenticator#Login(com.plumtree.uiinfrastructure.application.RequestData)"></seealso>
    public virtual LoginResult Login(RequestData requestData)
    {
      // The actual login is performed by the SSO server, not here
      return null;
    }
    
    /// <summary>
    /// This method sends an HTTP redirect to the SSO login URL. If for some 
    /// reason the redirect can not be generated/sent, this method will throw an 
    /// InterpreterException.
    /// </summary>
    /// <returns>
    /// null, indicating the login challenge redirect has been sent 
    /// internally.
    /// </returns>
    /// <seealso cref="com.plumtree.uiinfrastructure.login.IAuthenticator#GetOrSendLoginChallenge(com.plumtree.uiinfrastructure.application.RequestData)"></seealso>
    public virtual Redirect GetOrSendLoginChallenge(RequestData requestData)
    {
      String sRedirectFromQS = requestData.GetRedirectURLFromQS();
      Redirect ssoRedirect = null;
      if (null != sRedirectFromQS)
      {
        ssoRedirect = SSOHelper.INSTANCE.DoLoginRedirect(requestData, requestData.m_xpRequest.GetQueryString());
        } else
      {
        ssoRedirect = SSOHelper.INSTANCE.DoLoginRedirect(requestData, null);
        }
      if (ssoRedirect != null)
      {
        log.Info("Issuing redirect to the SSO login page due to a non-existent session and activity space.");
        log.Info("Redirecting to: " + ssoRedirect.GetExternalURL());
        // Session fixation vunerability protection for SSO in dotnet
        // This is a MS recommended "hack" way to get IIS to generate new session id 
        // IIS does not create a new session directly after abandoning session as in java, so we clear
        // the session id cookie before redirecting to ssologinpage. IIS will assign a new session id
        // on the next request. 
        // Fix in Java abandons the session on the request after succesful sso login in XPPage 
        if (XPSystem.IsRunningInDotNet())
        {
          // clear cookie by setting its value to empty string
          XPCookie cookie = new XPCookie(ASPNET_SESSIONID_COOKIENAME, "");
          cookie.SetPath("/");
          requestData.m_xpResponse.AddCookie(cookie);
          log.Debug("Cleared ASP.NET SessionId cookie for session fixation vunerability protection.");
          }
        // WilliamA - looks like SSOHelper.DoLoginRedirect already sets 
        // an HTTP redirect to the XPResponse directly, so return null here
        // to indicate that login challenge has been sent 
        // -- return ssoRedirect;
        return null;
      } else
      {
        log.Error("SSO is enabled, but redirect to SSO login cannot be generated!");
        throw new InterpreterException();
      }
    }
    
    /// <summary>
    /// The method sends an HTTP redirect to the SSO logout URL.
    /// </summary>
    /// <returns>a boolean indicating whether an HTTP redirect has been sent.</returns>
    /// <seealso cref="com.plumtree.uiinfrastructure.login.IAuthenticator#Logout(com.plumtree.uiinfrastructure.application.RequestData)"></seealso>
    public virtual bool Logout(RequestData requestData)
    {
      // check to see if we have to do an SSO logout redirect.
      if (SSOHelper.INSTANCE.DoLogoutRedirect(requestData))
      {
        // NOTE: Don't we want to call DoLogout() here to clean up
        // the session?
        // The lock will be released and cleanup will be handled by the finally clause.
        return true;
      } else
      {
        // mj: add the SSO logout parameter so the login page knows to display 
        // the SSO logout page rather than the just regular login page.
        XPHashtable htArgs = requestData.GetQueryString();
        if (null != htArgs)
        {
          htArgs.PutElement(LoginHelper.PARAM_SSOLOGOUT, new String[0]);
        }
        return false;
      }
    }
    
    /// <summary>
    /// This method indicates whether SSO authentication is applicable to current
    /// request. Currently, SSO setting is portal-wide, and is not on per-request
    /// basis, so this method will return true as long as SSO is enabled in the 
    /// configuration.
    /// </summary>
    /// <returns>a boolean indicating whether SSO is enabled.</returns>
    /// <seealso cref="com.plumtree.uiinfrastructure.login.IAuthenticator#UseThisAuth(com.plumtree.uiinfrastructure.application.RequestData)"></seealso>
    public virtual bool UseThisAuth(RequestData requestData)
    {
      ISessionManager sessionManager = requestData.m_SessionManager;
      String strSubportalSSOVendor = (String) (sessionManager.GetAttribute(Authenticator.GUESTLOGIN_SSOVENDOR));
      // Check for global config setting
      if (SSOHelper.INSTANCE.IsSSOEnabled())
      {
        // Check for a subportal setting that overrides the portal setting
        if (null != strSubportalSSOVendor)
        {
          // 0 = no SSO vendor
          if (XPConvert.ToInteger(strSubportalSSOVendor) != 0)
          {
            // Anything besides 0 means SSO is enabled
            log.Info("SSO is enabled for this subportal.");
            return true;
          } else
          {
            // If there is a subportal setting of 0,
            // SSO should be disabled for the subportal.
            log.Info("SSO is disabled for this subportal.");
            return false;
          }
        } else
        {
          // No subportal setting. SSO is enabled
          log.Info("SSO is enabled for this subportal.");
          return true;
        }
      } else
      {
        // SSO is disabled globally.
        log.Info("SSO is disabled globally.");
        return false;
      }
    }
  }
}