using System;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.activityspace;
namespace com.plumtree.uiinfrastructure.opener
{
  /// <summary>
  /// This object is used to encapsulate the destination
  /// information for the Common Opener.  The destination
  /// can be a redirect (allowing us to go to Activity 
  /// Spaces), or there can be information in the QS 
  /// arguments.  Plugins can use the QS information to 
  /// perform different actions, such as converting a UUID
  /// into a ClassID/ObjectID pair.  
  /// This object also has a method that allows the Plugin
  /// process to be ended prematurely (if multiple Plugins
  /// are installed).
  /// </summary>
  /// <author>RickyW</author>
  public class RedirectData
  {
    
    /// <summary>
    /// Contains an optional Redirect object. 
    /// </summary>
    private Redirect m_rRedirect;
    
    /// <summary>
    /// Contains our Querystring arguments. 
    /// </summary>
    private XPHashtable m_htArguments;
    
    /// <summary>
    /// Member to prematurely stop Plugin processing. 
    /// </summary>
    private bool m_bHaltPlugins;
    
    /// <summary>
    /// Constructor simply initializes members. 
    /// </summary>
    public RedirectData()
    {
      m_rRedirect = null;
      m_htArguments = new XPHashtable();
      m_bHaltPlugins = false;
    }
    
    /// <returns>True if there is a Redirect.</returns>
    public virtual bool HasRedirect()
    {
      return (null != m_rRedirect);
    }
    
    /// <summary>
    /// Get the Redirect destination.
    /// </summary>
    /// <returns>Redirect destination.</returns>
    public virtual Redirect GetRedirect()
    {
      return m_rRedirect;
    }
    
    /// <summary>
    /// Set the Redirect destination.
    /// </summary>
    /// <param name="_rRedirect">Redirect destination.</param>
    public virtual void SetRedirect(Redirect _rRedirect)
    {
      m_rRedirect = _rRedirect;
    }
    
    /// <summary>
    /// Get the Querystring arguments.
    /// </summary>
    /// <returns>Querystring arguments.</returns>
    public virtual XPHashtable GetQSArguments()
    {
      return m_htArguments;
    }
    
    /// <summary>
    /// Set the Querystring arguments.
    /// </summary>
    /// <param name="_htArguments">Querystring arguments.</param>
    public virtual void SetQSArguments(XPHashtable _htArguments)
    {
      m_htArguments = _htArguments;
    }
    
    /// <summary>
    /// Adds a querystring argument to the Querystring arguments
    /// hashtable.  If the argument is already there, then it is
    /// replaced.  If the key or hashtable are null, do nothing.
    /// </summary>
    /// <param name="_strKey">The key to add into the Querystring hashtable.</param>
    /// <param name="_strValue">The value to add into the Querystring hashtable.</param>
    public virtual void AddQSArgument(String _strKey, String _strValue)
    {
      if (null != m_htArguments)
      {
        if (null != _strKey)
        {
          String[] aValues = null;
          if (null != _strValue)
          {
            aValues = new String[]{_strValue};
          }
          m_htArguments.PutElement(_strKey, aValues);
        }
      }
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.opener.RedirectData#AddQSArgument(String, String)
     */
     
    public virtual void AddQSArgument(String _strKey, int _nValue)
    {
      AddQSArgument(_strKey, XPConvert.ToString(_nValue));
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.opener.RedirectData#AddQSArgument(String, String)
     */
     
    public virtual void AddQSArgument(String _strKey, bool _bValue)
    {
      AddQSArgument(_strKey, XPConvert.ToString(_bValue));
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.opener.RedirectData#AddQSArgument(String, String)
     */
     
    public virtual void AddQSArgument(String _strKey, Object _oValue)
    {
      AddQSArgument(_strKey, XPConvert.ToString(_oValue));
    }
    
    /// <summary>
    /// Get whether to halt plugin processing.
    /// </summary>
    /// <returns>True to halt plugin processing.</returns>
    public virtual bool GetHaltPlugins()
    {
      return m_bHaltPlugins;
    }
    
    /// <summary>
    /// Set whether to halt plugin processing.
    /// </summary>
    /// <param name="_bHaltPlugins">True to halt plugin processing.</param>
    public virtual void SetHaltPlugins(bool _bHaltPlugins)
    {
      m_bHaltPlugins = _bHaltPlugins;
    }
  }
}