using System;
using com.plumtree.openfoundation.util;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.pagecontrols.framework
{
  /// <summary>
  /// This is the abstract class for all page controls. A page control is a single
  /// input/output component on an web page. One example of a control is the
  /// datepicker. For each instance of the datepicker on a single page, you need
  /// an instance of an AComponent. Components of like controls are managed
  /// through a single component manger. So, on a single page you might have 3
  /// datepickers (3 instances of AComponent) all managed by a single datepicker
  /// manager. If you were to have other types of controls on that same page, you
  /// would need an instance of the appropriate componetent manager for those
  /// controls. The Page manager aggregates all instances of component mangers
  /// and provides access to page specific information that might be relevant to
  /// controls (i.e. locale and timezone).  In summary: Component - Might have
  /// many on a page (i.e. datepicker). Note: There is really no difference
  /// between a "component" and a "control." Component Manager - One per type of
  /// component on a page. Page Manager - Only one per page. Aggregates all the
  /// component managers.  How is the component manager differnt from the page
  /// manager? Why have both? The component manager serves two funtions. First,
  /// it aggregates like components. Second, it handles the printing of
  /// page-level content (contrast with component-level content.) Consider the
  /// datepicker example again. If you have 2 datepickers on one page, there is
  /// data that needs to be outputted once (page-level) and data that needs to be
  /// outputted once for each datepicker (component-level). One special note
  /// about the page-controls framework... It assumes that the client will be
  /// using the HTTP Session for state management. Given this, the client needs
  /// to have implemented a good HTTP Session mangment strategy that will release
  /// old or unused objects. See the documentation on Activity Spaces for such a
  /// framework.
  /// </summary>
  /// <author>John Osborne</author>
  public abstract class AComponent
  {
    
    /// <summary>
    /// The page manager associated with this component. 
    /// </summary>
    protected PageMgr m_ptPageMgr;
    
    /// <summary>
    /// Each component needs an special string tag to allow it to be uniquely
    /// identified on a page. This tage is appended to the end of the HTML name
    /// of the component's input elements.
    /// </summary>
    private String m_strUniqueInstanceID;
    
    /// <summary>
    /// Creates a new AComponent
    /// </summary>
    public AComponent()
    {
    }
    
    /// <summary>
    /// Initializes a new AComponent()
    /// </summary>
    /// <param name="_ptComponentMgr">The component manager for this component.</param>
    /// <param name="_strUniqueInstanceID">The unique ID for this component.</param>
    public virtual void Init(PageMgr _ptPageMgr, String _strUniqueInstanceID)
    {
      m_ptPageMgr = _ptPageMgr;
      m_strUniqueInstanceID = _strUniqueInstanceID;
    }
    
    /// <summary>
    /// This method serves as a getter for component HTML. Sub-classes must
    /// implement this method.
    /// </summary>
    /// <returns>The HTML for the component itself.</returns>
    public abstract HTMLElementCollection GetComponentHTML();
    
    /// <summary>
    /// This method returns the page manager that houses this component manager.
    /// </summary>
    /// <returns>The page manager.</returns>
    public virtual PageMgr GetPageMgr()
    {
      return m_ptPageMgr;
    }
    
    /// <summary>
    /// This method serves as a getter for "sub-component" HTML. The breakdown
    /// of sub-components is decided by the component author. One example is
    /// the date/time-picker. It is made up of two sub-components: the
    /// date-picker and the time-picker. The constants for component sub-index
    /// should be specified in the component file and prefixed with
    /// "N_SUB_COMPONENT_INDEX." If the specified index is not valid, null
    /// should be returned. This method returns null in the parent instead of
    /// being abstract becuase many components will not need to use this
    /// method.
    /// </summary>
    /// <param name="_nSubComponentIndex">
    /// The sub-component index specified in the the
    /// componet file itself.
    /// </param>
    /// <returns>The HTML for the sub-component requested.</returns>
    public virtual HTMLElementCollection GetSubComponentHTMLByIndex(int _nSubComponentIndex)
    {
      return null;
    }
    
    /// <summary>
    /// This abstract method stores the input submitted by the user. It should
    /// be called after a POST or GET. In the case of the date picker, it would
    /// store the date.
    /// </summary>
    /// <param name="_htRequestParameters">
    /// A hashtable containing the request's name-
    /// value pairs.
    /// </param>
    public abstract int StoreUserInput(XPHashtable _htRequestParameters);
    
    /// <summary>
    /// This method returns script that needs to be printed within the HTML head
    /// tag. This method could have been abstract except that most components
    /// do not have a use for it. It therefore returns null and should be
    /// overridden as needed. (Note: there is an anologous method for printing
    /// page- level content in ComponentMgr.)
    /// </summary>
    /// <returns>HTML that goes within the head tag.</returns>
    public virtual HTMLElementCollection GetComponentHeadContent()
    {
      return null;
    }
    
    /// <summary>
    /// This method returns script that needs to be printed after the component
    /// itself has been printed. This method could have been abstract except
    /// that most components do not have a use for it. It therefore returns
    /// null and should be overridden as needed. (Note: there is an anologous
    /// method for printing page- level content in ComponentMgr.)
    /// </summary>
    /// <returns>HTML that should be printed after the component itself.</returns>
    public virtual HTMLElementCollection GetComponentPostContent()
    {
      return null;
    }
    
    /// <summary>
    /// This method returns script that needs to be printed before the component
    /// itself has been printed. This method could have been abstract except
    /// that most components do not have a use for it. It therefore returns
    /// null and should be overridden as needed. (Note: there is an anologous
    /// method for printing page- level content in ComponentMgr.)
    /// </summary>
    /// <returns>HTML that should be printed before the component itself.</returns>
    public virtual HTMLElementCollection GetComponentPreContent()
    {
      return null;
    }
    
    /// <summary>
    /// This method returns script that needs to be printed before the form in
    /// which this component is embedded. It therefore returns null and should
    /// be overridden as needed. (Note: there is an anologous method for
    /// printing page- level content in ComponentMgr.)
    /// </summary>
    /// <returns></returns>
    public virtual HTMLElementCollection GetComponentPreFormContent()
    {
      return null;
    }
    
    /// <summary>
    /// This method returns the string used to uniquely idenify this component
    /// on a page.
    /// </summary>
    /// <returns></returns>
    public virtual String GetUniqueInstanceID()
    {
      return m_strUniqueInstanceID;
    }
    
    /// <summary>
    /// This method should be used to add the unique identifier to HTML inputs
    /// related to this component. If my element name is "in_cb_MyCheckBox" and
    /// my unique ID is "one", then this method will return
    /// "in_cb_MyCheckBox_one".
    /// </summary>
    /// <param name="_strElementName">
    /// The name of the element that needs to have the
    /// unique ID appended to it.
    /// </param>
    /// <returns>The element name plus the unique ID.</returns>
    protected String AppendUniqueIDToElementName(String _strElementName)
    {
      return _strElementName + "_" + m_strUniqueInstanceID;
    }
    
    /// <summary>
    /// Return the name or this component.
    /// </summary>
    /// <returns>the element name</returns>
    public abstract String GetName();
    
    /// <summary>
    /// Create method for the object returns an instance of the object.
    /// </summary>
    /// <returns>and object that extends AComponent</returns>
    public abstract Object Create();
    
    /// <summary>
    /// For every Validator associated with this object execute it's Validate method.
    /// Return an Arraylist of UIError objects generated from executing the Validators.
    /// Return and empty Arraylist if no problems were found.
    /// </summary>
    /// <returns>Arraylist of UIErrors.</returns>
    public abstract XPArrayList Validate();
    
    /// <summary>
    /// Return an Arraylist of validators associated with this object.
    /// </summary>
    /// <returns></returns>
    public abstract XPArrayList GetValidators();
  }
}