using System;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.errordisplay;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.pagecontrols.framework
{
  /// <summary>
  /// JF- This is the abstract class for all HTML Element page controls.
  /// </summary>
  /// <author>Julie Fournier</author>
  public abstract class AHTMLComponent : AComponent
  {
    
    /// <summary>
    /// JF- HTML Component Name 
    /// </summary>
    private String m_sHTMLComponentName;
    
    /// <summary>
    /// JF- HTML Component Value 
    /// </summary>
    private String m_sHTMLComponentValue;
    
    /// <summary>
    /// HTML Component Size 
    /// </summary>
    private String m_sHTMLComponentSize;
    
    /// <summary>
    /// HTML Component Style Class 
    /// </summary>
    private PTStyleClass m_HTMLComponentStyleClass;
    
    /// <summary>
    /// HTML Component Label 
    /// </summary>
    private String m_sLabel;
    
    /// <summary>
    /// HTML Component Validators 
    /// </summary>
    private XPArrayList m_Validators;
    
    /// <summary>
    /// Creates an AHTML Component
    /// </summary>
    public AHTMLComponent()
    {
    }
    
    /// <summary>
    /// JF- Initializes a new AHTMLComponent object.
    /// </summary>
    /// <param name="_ptComponentMgr">The component manager for this component.</param>
    /// <param name="_strUniqueInstanceID">The unique ID for this component.</param>
    public override void Init(PageMgr _ptPageMgr, String _strUniqueInstanceID)
    {
      base.Init(_ptPageMgr, _strUniqueInstanceID);
      m_sHTMLComponentName = AppendUniqueIDToElementName(GetHTMLComponentBaseName());
      m_sHTMLComponentSize = "";
      m_sHTMLComponentValue = "";
      m_Validators = new XPArrayList(0);
    }
    
    /// <summary>
    /// JF- Store the new value of this HTML component.
    /// </summary>
    /// <param name="_htRequestParameters"></param>
    public override int StoreUserInput(XPHashtable _htRequestParameters)
    {
      Object oElement = _htRequestParameters.GetElement(m_sHTMLComponentName);
      if (oElement != null)
      {
        String[] aStrValues = (String[]) oElement;
        if (aStrValues.Length > 0)
        {
          m_sHTMLComponentValue = aStrValues[0];
        }
      }
      return RepostControl.PAGE_STATUS_VALID;
    }
    
    /// <summary>
    /// Add a validator to this component's collection of validators.
    /// </summary>
    /// <param name="_ptValidator">
    /// a validator to add to the list of validators
    /// owned by this component
    /// </param>
    public virtual void AddValidator(IValidator _ptValidator)
    {
      m_Validators.Add(_ptValidator);
    }
    
    /// <summary>
    /// Return an XPArrayList of UIError messages gathered by calling the 
    /// validate() method on all Validators in this Component's collection
    /// of Validators.
    /// </summary>
    public override XPArrayList Validate()
    {
      XPArrayList myErrors = new XPArrayList();
      IXPEnumerator enumerator = m_Validators.GetEnumerator();
      while (enumerator.MoveNext())
      {
        IValidator myValidator = (IValidator) enumerator.GetCurrent();
        UIError myError = myValidator.Validate(GetHTMLComponentValue());
        if (null != myError)
        {
          myErrors.Add(myError);
        }
      }
      return myErrors;
    }
    
    /// <summary>
    /// Return the list of Validators associated with this object.
    /// </summary>
    /// <returns>XPArrayList of validators</returns>
    public override XPArrayList GetValidators()
    {
      return m_Validators;
    }
    
    /// <summary>
    /// Return a string made up of all the JSValidationCode for each of the
    /// Validator's associated with this component.
    /// </summary>
    /// <returns>JavaScript validation code</returns>
    public virtual String GetJSValidationCode()
    {
      XPStringBuilder sb = new XPStringBuilder(1250);
      for (int i = 0;i < m_Validators.GetSize();i++)
      {
        IValidator myValidator = (IValidator) m_Validators.GetElement(i);
        sb.Append(myValidator.GetJSValidationCode());
      }
      return sb.ToString();
    }
    
    /// <summary>
    /// JF- Returns the base name of this HTML Component.
    /// E.g.: in_tx_pc
    /// </summary>
    /// <returns>String</returns>
    protected abstract String GetHTMLComponentBaseName();
    
    /// <summary>
    /// JF- Returns the name of this HTML Component.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetHTMLComponentName()
    {
      return m_sHTMLComponentName;
    }
    
    /// <summary>
    /// JF- Returns the size of this HTML Component.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetHTMLComponentSize()
    {
      return m_sHTMLComponentSize;
    }
    
    /// <summary>
    /// JF- Returns the style class of this HTML Component.
    /// </summary>
    /// <returns>PTStyleClass</returns>
    public virtual PTStyleClass GetHTMLComponentStyleClass()
    {
      return m_HTMLComponentStyleClass;
    }
    
    /// <summary>
    /// JF- Returns the value of this HTML Component.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetHTMLComponentValue()
    {
      return m_sHTMLComponentValue;
    }
    
    /// <summary>
    /// JF- Sets the name of this HTML Component.
    /// </summary>
    /// <param name="_sHTMLComponentName"></param>
    public virtual void SetHTMLComponentName(String _sHTMLComponentName)
    {
      m_sHTMLComponentName = _sHTMLComponentName;
    }
    
    /// <summary>
    /// JF- Sets the size of this HTML Component.
    /// </summary>
    /// <param name="_sHTMLComponentSize"></param>
    public virtual void SetHTMLComponentSize(String _sHTMLComponentSize)
    {
      m_sHTMLComponentSize = _sHTMLComponentSize;
    }
    
    /// <summary>
    /// JF- Sets the style class of this HTML Component.
    /// </summary>
    /// <param name="_HTMLComponentStyleClass"></param>
    public virtual void SetHTMLComponentStyleClass(PTStyleClass _HTMLComponentStyleClass)
    {
      m_HTMLComponentStyleClass = _HTMLComponentStyleClass;
    }
    
    /// <summary>
    /// Transfer the properties set to this object to the associated properties
    /// on the HTMLInput object. For example, Set the HTMLInput's styleClass 
    /// with this object's GetHTMLComponentStyleClass.
    /// </summary>
    /// <param name="_ptHTMLInput"></param>
    /// <returns></returns>
    protected virtual HTMLInput SetParameters(HTMLInput _ptHTMLInput)
    {
      if (null != GetHTMLComponentStyleClass())
      {
        _ptHTMLInput.SetStyleClass(GetHTMLComponentStyleClass());
      }
      if (null != GetHTMLComponentSize())
      {
        _ptHTMLInput.SetSize(GetHTMLComponentSize());
      }
      if (null != GetHTMLComponentValue())
      {
        _ptHTMLInput.SetValue(GetHTMLComponentValue());
      }
      return _ptHTMLInput;
    }
    
    /// <summary>
    /// JF- Sets the value of this HTML Component.
    /// </summary>
    /// <param name="_sHTMLComponentValue"></param>
    public virtual void SetHTMLComponentValue(String _sHTMLComponentValue)
    {
      m_sHTMLComponentValue = _sHTMLComponentValue;
    }
    
    /// <summary>
    /// Returns the Label of this component
    /// </summary>
    /// <returns>Returns the m_sLabel member variable</returns>
    public virtual String GetLabel()
    {
      return m_sLabel;
    }
    
    /// <summary>
    /// Sets the Label of this component
    /// </summary>
    /// <param name="label">The m_sLabel to set</param>
    public virtual void SetLabel(String label)
    {
      m_sLabel = label;
    }
    
    /// <summary>
    /// Returns a ComponentMgr for this component. By default classes that extend
    /// AHTMLComponent may use this method to return the defualt ComponentMgr
    /// HTMLComponentMgr. If a component needs another ComponentMgr it should override
    /// this method.
    /// </summary>
    /// <param name="_ptPageMgr">the PageMgr that the ComponentMgr is registered.</param>
    /// <returns>the default ComponentMgr, HTMLComponentMgr.</returns>
    public virtual ComponentMgr GetComponentMgr(PageMgr _ptPageMgr)
    {
      return new HTMLComponentMgr(_ptPageMgr);
    }
    
    /// <summary>
    /// Returns the name of the  ComponentMgr for this component. By default classes 
    /// that extend AHTMLComponent may use this method to return the defualt ComponentMgr
    /// name, HTMLComponentMgr.STR_CLASS_NAME. If a component needs another ComponentMgr 
    /// it should override this method.
    /// </summary>
    /// <returns>the name of the default ComponentMgr, HTMLComponentMgr.STR_CLASS_NAME.</returns>
    public virtual String GetComponentMgrName()
    {
      return HTMLComponentMgr.STR_CLASS_NAME;
    }
  }
}