using System;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.pagecontrols.framework
{
  /// <summary>
  /// This class manages instances of like components that all appear on a single
  /// page. For more information on the page controls framework in general,
  /// please consult the documentation for AComponent.  There is one important
  /// concept to understand when it comes to the ComponentMgr class: for page
  /// controls, there is script to be outputted that has "component scope" and
  /// script to be outputted that has "page scope." Component scope means that
  /// the data needs to be printed once per component. Page scope means that the
  /// data needs to be printed once per page, independent of the number of times
  /// a component of that type is written to the page. The component manager
  /// allows you to print data for either or both scopes through a series of
  /// methods.  Components may be registered with the component manager and then
  /// obtained either using their unique instance ID or by way of an int index.
  /// The index is determined by the order in which you registered the
  /// components.
  /// </summary>
  /// <author>John Osborne</author>
  public class ComponentMgr
  {
    
    /// <summary>
    /// The page manager that aggregates all component managers on this page. 
    /// </summary>
    private PageMgr m_ptPageMgr;
    
    /// <summary>
    /// A list of all the registered components on this page. 
    /// </summary>
    private XPArrayList m_alRegisteredComponents;
    
    /// <summary>
    /// class name constant 
    /// </summary>
    public const String STR_CLASS_NAME = "ComponentMgr";
    
    /// <summary>
    /// a boolean to track whether jsfunctions have been added for this componentmgr 
    /// </summary>
    private bool m_bAddedFunctions = false;
    
    /// <summary>
    /// Creates a new ComponentMgr object. Clients need one of these per
    /// component type (i.e. datepicker).
    /// </summary>
    /// <param name="_ptPageMgr">The page manager for this component manager.</param>
    public ComponentMgr(PageMgr _ptPageMgr)
    {
      m_ptPageMgr = _ptPageMgr;
      m_alRegisteredComponents = new XPArrayList();
    }
    
    /// <summary>
    /// This method returns ALL HTML--page scope and component scope for all
    /// components under this manger--that belongs between the HTML head tags.
    /// </summary>
    /// <returns>
    /// All HTML content that should go between the head tags for
    /// components of this type.
    /// </returns>
    public virtual HTMLElementCollection GetAllHeadContent()
    {
      HTMLElementCollection retVal = new HTMLElementCollection();
      HTMLElementCollection htmlContent1 = GetPageHeadContent();
      if (htmlContent1 != null)
      {
        retVal.AddInnerHTMLElement(htmlContent1);
      }
      for (int i = 0;i < m_alRegisteredComponents.GetSize();i++)
      {
        HTMLElementCollection htmlContent2 = ((AComponent) m_alRegisteredComponents.GetElement(i)).GetComponentHeadContent();
        if (htmlContent2 != null)
        {
          retVal.AddInnerHTMLElement(htmlContent2);
        }
      }
      return retVal;
    }
    
    /// <summary>
    /// This method returns ALL HTML--page scope and component scope for all
    /// components under this manger--that should be printed after the
    /// component itself.
    /// </summary>
    /// <returns>
    /// All HTML content that should be printed after the component
    /// itself.
    /// </returns>
    public virtual HTMLElementCollection GetAllPostContent()
    {
      HTMLElementCollection retVal = new HTMLElementCollection();
      HTMLElementCollection htmlContent1 = GetPagePostContent();
      if (htmlContent1 != null)
      {
        retVal.AddInnerHTMLElement(htmlContent1);
      }
      for (int i = 0;i < m_alRegisteredComponents.GetSize();i++)
      {
        HTMLElementCollection htmlContent2 = ((AComponent) m_alRegisteredComponents.GetElement(i)).GetComponentPostContent();
        if (htmlContent2 != null)
        {
          retVal.AddInnerHTMLElement(htmlContent2);
        }
      }
      return retVal;
    }
    
    /// <summary>
    /// This method returns ALL HTML--page scope and component scope for all
    /// components under this manger--that should be printed before the
    /// component itself.
    /// </summary>
    /// <returns>
    /// All HTML content that should be printed before the component
    /// itself.
    /// </returns>
    public virtual HTMLElementCollection GetAllPreContent()
    {
      HTMLElementCollection retVal = new HTMLElementCollection();
      HTMLElementCollection htmlContent1 = GetPagePreContent();
      if (htmlContent1 != null)
      {
        retVal.AddInnerHTMLElement(htmlContent1);
      }
      for (int i = 0;i < m_alRegisteredComponents.GetSize();i++)
      {
        HTMLElementCollection htmlContent2 = ((AComponent) m_alRegisteredComponents.GetElement(i)).GetComponentPreContent();
        if (htmlContent2 != null)
        {
          retVal.AddInnerHTMLElement(htmlContent2);
        }
      }
      return retVal;
    }
    
    /// <summary>
    /// This method returns ALL HTML--page scope and component scope for all
    /// components under this manger--that should be printed before the form in
    /// which this group of components is embedded.
    /// </summary>
    /// <returns>All HTML content that should be printed before the form.</returns>
    public virtual HTMLElementCollection GetAllPreFormContent()
    {
      HTMLElementCollection retVal = new HTMLElementCollection();
      HTMLElementCollection htmlContent1 = GetPagePreFormContent();
      if (htmlContent1 != null)
      {
        retVal.AddInnerHTMLElement(htmlContent1);
      }
      for (int i = 0;i < m_alRegisteredComponents.GetSize();i++)
      {
        HTMLElementCollection htmlContent2 = ((AComponent) m_alRegisteredComponents.GetElement(i)).GetComponentPreFormContent();
        if (htmlContent2 != null)
        {
          retVal.AddInnerHTMLElement(htmlContent2);
        }
      }
      return retVal;
    }
    
    /// <summary>
    /// This method returns an AComponent object given its unique instance ID.
    /// Null is returned if the component is not found.
    /// </summary>
    /// <param name="_strUniqueInstanceID">
    /// The ID that uniquely represents the
    /// component that you want.
    /// </param>
    /// <returns>The AComponent instance requested (or null).</returns>
    public virtual AComponent GetComponent(String _strUniqueInstanceID)
    {
      // This is not very efficient, but the assumption is that there are not many instances per page.
      for (int i = 0;i < m_alRegisteredComponents.GetSize();i++)
      {
        AComponent oInstance = (AComponent) m_alRegisteredComponents.GetElement(i);
        if (null != oInstance && null != oInstance.GetUniqueInstanceID() && oInstance.GetUniqueInstanceID().Equals(_strUniqueInstanceID))
        {
          return oInstance;
        }
      }
      return null;
    }
    
    /// <summary>
    /// Returns an AComponent given an index. The index is determined by the
    /// registration order. This method can help you loop through all
    /// registered AComponents.
    /// </summary>
    /// <param name="_nIndex">The index of the component you want.</param>
    /// <returns>The AComponent instance requested (or null).</returns>
    public virtual AComponent GetComponent(int _nIndex)
    {
      Object oElement = null;
      try
      {
        oElement = m_alRegisteredComponents.GetElement(_nIndex);
      }
      catch (Exception e)
      {
        oElement = null;
      }
      // Do nothing... just return null
      if (oElement != null)
      {
        AComponent oComponent = (AComponent) oElement;
        return oComponent;
      }
      return null;
    }
    
    /// <summary>
    /// Returns the HTML for the designated component.
    /// </summary>
    /// <param name="_strUniqueInstanceID">
    /// The unique instance ID by which to identify
    /// the component.
    /// </param>
    /// <returns>The HTML for the component itself.</returns>
    public virtual HTMLElementCollection GetComponentHTML(String _strUniqueInstanceID)
    {
      AComponent oComponent = GetComponent(_strUniqueInstanceID);
      if (oComponent != null)
      {
        return oComponent.GetComponentHTML();
      }
      return null;
    }
    
    /// <summary>
    /// Returns the HTML for the designated component.
    /// </summary>
    /// <param name="_nIndex">The index by which to identify the component.</param>
    /// <returns>The HTML for the component itself.</returns>
    public virtual HTMLElementCollection GetComponentHTML(int _nIndex)
    {
      Object oElement = m_alRegisteredComponents.GetElement(_nIndex);
      if (oElement != null)
      {
        AComponent oComponent = (AComponent) oElement;
        return oComponent.GetComponentHTML();
      }
      return null;
    }
    
    /// <summary>
    /// Returns the HTML for the designated component that is should be written
    /// out within the HTML head tags.
    /// </summary>
    /// <param name="_strUniqueInstanceID">
    /// The unique instance ID by which to identify
    /// the component.
    /// </param>
    /// <returns>The script to write into the HTML head.</returns>
    public virtual HTMLElementCollection GetComponentHeadContent(String _strUniqueInstanceID)
    {
      AComponent oComponent = GetComponent(_strUniqueInstanceID);
      if (oComponent != null)
      {
        oComponent.GetComponentHeadContent();
      }
      return null;
    }
    
    /// <summary>
    /// Returns the HTML for the designated component that is should be written
    /// out within the HTML head tags.
    /// </summary>
    /// <param name="_nIndex">The index by which to identify the component.</param>
    /// <returns>The script to write into the HTML head.</returns>
    public virtual HTMLElementCollection GetComponentHeadContent(int _nIndex)
    {
      Object oElement = m_alRegisteredComponents.GetElement(_nIndex);
      if (oElement != null)
      {
        AComponent oComponent = (AComponent) oElement;
        return oComponent.GetComponentHeadContent();
      }
      return null;
    }
    
    /// <summary>
    /// Returns the HTML for the designated component that should be written out
    /// after the component itself.
    /// </summary>
    /// <param name="_strUniqueInstanceID">
    /// The unique instance ID by which to identify
    /// the component.
    /// </param>
    /// <returns>The script to write after the component.</returns>
    public virtual HTMLElementCollection GetComponentPostContent(String _strUniqueInstanceID)
    {
      AComponent oComponent = GetComponent(_strUniqueInstanceID);
      if (oComponent != null)
      {
        oComponent.GetComponentPostContent();
      }
      return null;
    }
    
    /// <summary>
    /// Returns the HTML for the designated component that should be written out
    /// after the component itself.
    /// </summary>
    /// <param name="_nIndex">The index by which to identify the component.</param>
    /// <returns>The script to write after the component.</returns>
    public virtual HTMLElementCollection GetComponentPostContent(int _nIndex)
    {
      Object oElement = m_alRegisteredComponents.GetElement(_nIndex);
      if (oElement != null)
      {
        AComponent oComponent = (AComponent) oElement;
        return oComponent.GetComponentPostContent();
      }
      return null;
    }
    
    /// <summary>
    /// Returns the HTML for the designated component that should be written out
    /// before the component itself.
    /// </summary>
    /// <param name="_strUniqueInstanceID">
    /// The unique instance ID by which to identify
    /// the component.
    /// </param>
    /// <returns>The script to write before the component.</returns>
    public virtual HTMLElementCollection GetComponentPreContent(String _strUniqueInstanceID)
    {
      AComponent oComponent = GetComponent(_strUniqueInstanceID);
      if (oComponent != null)
      {
        return oComponent.GetComponentPreContent();
      }
      return null;
    }
    
    /// <summary>
    /// Returns the HTML for the designated component that should be written out
    /// before the component itself.
    /// </summary>
    /// <param name="_nIndex">The index by which to identify the component.</param>
    /// <returns>The script to write before the component.</returns>
    public virtual HTMLElementCollection GetComponentPreContent(int _nIndex)
    {
      Object oElement = m_alRegisteredComponents.GetElement(_nIndex);
      if (oElement != null)
      {
        AComponent oComponent = (AComponent) oElement;
        return oComponent.GetComponentPreContent();
      }
      return null;
    }
    
    /// <summary>
    /// Returns the HTML for the designated component that should be written out
    /// before the form in which the component is embedded.
    /// </summary>
    /// <param name="_nIndex">The index by which to identify the component.</param>
    /// <returns>
    /// The script to write before the form in which the component is
    /// embedded.
    /// </returns>
    public virtual HTMLElementCollection GetComponentPreFormContent(int _nIndex)
    {
      Object oElement = m_alRegisteredComponents.GetElement(_nIndex);
      if (oElement != null)
      {
        AComponent oComponent = (AComponent) oElement;
        return oComponent.GetComponentPreFormContent();
      }
      return null;
    }
    
    /// <summary>
    /// Returns the HTML for the designated component that should be written out
    /// before the form in which the component is embedded.
    /// </summary>
    /// <param name="_strUniqueInstanceID">
    /// The unique instance ID by which to identify
    /// the component.
    /// </param>
    /// <returns>
    /// The script to write before the form in which the component is
    /// embedded.
    /// </returns>
    public virtual HTMLElementCollection GetComponentPreFormContent(String _strUniqueInstanceID)
    {
      AComponent oComponent = GetComponent(_strUniqueInstanceID);
      if (oComponent != null)
      {
        return oComponent.GetComponentPreFormContent();
      }
      return null;
    }
    
    /// <summary>
    /// Returns the number of components registered with this component manger.
    /// </summary>
    /// <returns>An int representing the number of registered components.</returns>
    public virtual int GetNumComponents()
    {
      return m_alRegisteredComponents.GetSize();
    }
    
    /// <summary>
    /// This method is meant to return the page-level head content for
    /// components of this type. This method just returns null. It could have
    /// been abstract, but many components do need to use it. If you need this,
    /// you should override it.
    /// </summary>
    /// <returns>The HTML and script that should go into the head tag.</returns>
    public virtual HTMLElementCollection GetPageHeadContent()
    {
      return null;
    }
    
    /// <summary>
    /// Returns the page manger that is associated with this component manager.
    /// </summary>
    /// <returns>The page manger.</returns>
    public virtual PageMgr GetPageMgr()
    {
      return m_ptPageMgr;
    }
    
    /// <summary>
    /// This method is meant to return the page-level "post" content for
    /// components of this type. This method just returns null. It could have
    /// been abstract, but many components do need to use it. If you need this,
    /// you should override it.
    /// </summary>
    /// <returns>The HTML and script that should be written after the component.</returns>
    public virtual HTMLElementCollection GetPagePostContent()
    {
      return null;
    }
    
    /// <summary>
    /// This method is meant to return the page-level head "pre" for components
    /// of this type. This method just returns null. It could have been
    /// abstract, but many components do need to use it. If you need this, you
    /// should override it.
    /// </summary>
    /// <returns>The HTML and script that should be written before the component.</returns>
    public virtual HTMLElementCollection GetPagePreContent()
    {
      return null;
    }
    
    /// <summary>
    /// This method is meant to return the page-level head content for
    /// components of this type. This method just returns null. It could have
    /// been abstract, but many components do need to use it. If you need this,
    /// you should override it.
    /// </summary>
    /// <returns>
    /// The HTML and script that should be written before the form in
    /// which this component is embedded.
    /// </returns>
    public virtual HTMLElementCollection GetPagePreFormContent()
    {
      return null;
    }
    
    /// <summary>
    /// This method allows you to register a component with the component
    /// manager. The order in which you register components is the order in
    /// which calls will be made when doing any looping operation--storing or
    /// printing. If you attempt to re-register the same component, it will
    /// overwrite the one that existed there before.
    /// </summary>
    /// <param name="_ptComponent">The component that you wish to register.</param>
    public virtual void RegisterComponent(IPageControl _ptComponent)
    {
      if (_ptComponent != null)
      {
        if (m_alRegisteredComponents.Contains(_ptComponent))
        {
          int nIndex = m_alRegisteredComponents.IndexOf(_ptComponent);
          m_alRegisteredComponents.Insert(nIndex, _ptComponent);
        } else
        {
          m_alRegisteredComponents.Add(_ptComponent);
        }
      }
    }
    
    /// <summary>
    /// Stores the user input for all componets registered with this component
    /// manager.
    /// </summary>
    /// <param name="_htRequestParameters">
    /// A hashtable of name-value pairs passed in
    /// with the request.
    /// </param>
    public virtual int StoreAllUserInput(XPHashtable _htRequestParameters)
    {
      int nError = RepostControl.PAGE_STATUS_VALID;
      int nStatus;
      for (int i = 0;i < m_alRegisteredComponents.GetSize();i++)
      {
        nStatus = StoreUserInput(i, _htRequestParameters);
        if (nStatus == RepostControl.PAGE_STATUS_INVALID)
        {
          nError = nStatus;
        }
      }
      return nError;
    }
    
    /// <summary>
    /// Call the validation method on all registered componenets. Returns an XPArrayList of all
    /// UIErrors generated by the componenets. 
    /// </summary>
    /// <returns>an XPArrayList of UIError objects</returns>
    public virtual XPArrayList Validate()
    {
      XPArrayList myErrors = new XPArrayList(0);
      for (int i = 0;i < m_alRegisteredComponents.GetSize();i++)
      {
        AComponent myComponent = GetComponent(i);
        if (null != myComponent.Validate())
        {
          myErrors.AddRange(myComponent.Validate());
        }
      }
      return myErrors;
    }
    
    /// <summary>
    /// Return JavaScript validation code for each of the validators associated with
    /// all components. This function returns the validation code once for each validator.
    /// Although the validator may be used several times it is assumed that the JavaScript
    /// code is only required one time.
    /// </summary>
    /// <returns>String JavaScript code for validators</returns>
    public virtual String GetJSValidationCode()
    {
      XPStringBuilder sb = new XPStringBuilder(1250);
      XPArrayList allValidators = new XPArrayList();
      for (int i = 0;i < m_alRegisteredComponents.GetSize();i++)
      {
        AComponent myComponent = GetComponent(i);
        XPArrayList myValidators = myComponent.GetValidators();
        if (null != myValidators)
        {
          for (int j = 0;j < myValidators.GetSize();j++)
          {
            IValidator myValidator = (IValidator) myValidators.GetElement(j);
            if (!allValidators.Contains(myValidator.GetName()))
            {
              sb.Append(myValidator.GetJSValidationCode());
              allValidators.Add(myValidator.GetName());
            }
          }
        }
      }
      return sb.ToString();
    }
    
    /// <summary>
    /// For all registered components, for every Validator associated with that
    /// component call AFormDP.AddToValidationFunction for the validator. This inserts
    /// the JavaScript Validation function name from the Validator to the page's 
    /// JavaScript Validate method.
    /// </summary>
    /// <param name="_ptAFormDP">
    /// AFormDP from the activity space where this ComponentMgr
    /// is used.
    /// </param>
    public virtual void AddToValidationFunction(AFormDP _ptAFormDP)
    {
      //if they've been added already no need to do it again.
      if (m_bAddedFunctions)
      {
        return;
      }
      for (int i = 0;i < m_alRegisteredComponents.GetSize();i++)
      {
        AComponent myComponent = GetComponent(i);
        XPArrayList myValidators = myComponent.GetValidators();
        if (null != myValidators)
        {
          for (int j = 0;j < myValidators.GetSize();j++)
          {
            IValidator myValidator = (IValidator) myValidators.GetElement(j);
            _ptAFormDP.AddToValidationFunction(myValidator.GetJSFunction(null));
            }
        }
      }
      // track that we've added the functions
      m_bAddedFunctions = true;
    }
    
    /// <summary>
    /// This method stores the user input for a particular component given its
    /// unique ID.
    /// </summary>
    /// <param name="_strUniqueInstanceID">
    /// The unique instance ID by which to identify
    /// the component.
    /// </param>
    /// <param name="_htRequestParameters">
    /// A hashtable of name-value pairs passed in
    /// with the request.
    /// </param>
    public virtual void StoreUserInput(String _strUniqueInstanceID, XPHashtable _htRequestParameters)
    {
      AComponent oComponent = GetComponent(_strUniqueInstanceID);
      oComponent.StoreUserInput(_htRequestParameters);
    }
    
    /// <summary>
    /// This method stores the user input for a particular component given its
    /// index.
    /// </summary>
    /// <param name="_nIndex">The index by which to identify the component.</param>
    /// <param name="_htRequestParameters">
    /// A hashtable of name-value pairs passed in
    /// with the request.
    /// </param>
    public virtual int StoreUserInput(int _nIndex, XPHashtable _htRequestParameters)
    {
      AComponent oComponent = GetComponent(_nIndex);
      return oComponent.StoreUserInput(_htRequestParameters);
    }
    
    /// <summary>
    /// This method unregisters a component given its unique instance ID.
    /// </summary>
    /// <param name="_strUniqueInstanceID">The unique instance ID.</param>
    public virtual void UnRegisterComponent(String _strUniqueInstanceID)
    {
      // This is not very efficient, but the assumption is that there are not many instances per page.
      int nIndexToRemove = -1;
      for (int i = 0;i < m_alRegisteredComponents.GetSize();i++)
      {
        AComponent oInstance = (AComponent) m_alRegisteredComponents.GetElement(i);
        if (oInstance.GetUniqueInstanceID().Equals(_strUniqueInstanceID))
        {
          nIndexToRemove = i;
          break;
        }
      }
      if (nIndexToRemove != -1)
      {
        m_alRegisteredComponents.RemoveAt(nIndexToRemove);
      }
    }
  }
}