using System;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.openfoundation.util;
using com.plumtree.xpshared.htmlelements;
using com.plumtree.xpshared.jsutil;
namespace com.plumtree.uiinfrastructure.pagecontrols.framework
{
  /// <summary>
  /// This class is meant to manage JS components. Most of the work is done by the
  /// parent class. However, there are some things that are specific to the JS
  /// classes that are handled by the JS registry. For many JS components (like
  /// the date-picker), you will want to override this class.
  /// </summary>
  /// <author>John Osborne</author>
  public class JSComponentMgr : ComponentMgr
  {
    
    /// <summary>
    /// The JS Registry that handles versioning for JS components. 
    /// </summary>
    private JSRegistry m_ptJSRegistry;
    
    /// <summary>
    /// class name constant 
    /// </summary>
    public const String STR_CLASS_NAME = "JSComponentMgr";
    
    /// <summary>
    /// The name for JS controls of this type (ie.e jsdatepicker"). It is this
    /// name that will be used to retreive the version number from the
    /// IJSComponentVersionMgr.
    /// </summary>
    private String m_strJSControlName;
    
    /// <summary>
    /// The version (as an String) to use for JS controls of this type. 
    /// </summary>
    private String m_strVersionID;
    
    /// <summary>
    /// The version (as an int) to use for JS controls of this type. 
    /// </summary>
    private int m_nVersionID;
    
    /// <summary>
    /// Creates a new JSComponentMgr object.
    /// </summary>
    /// <param name="_strControlName">
    /// The name for JS controls of this type (ie.e
    /// jsdatepicker").
    /// </param>
    /// <param name="_ptJSRegistry">
    /// The JS Registry that handles versioning for JS
    /// components.
    /// </param>
    /// <param name="_ptJSCompVerMgr">
    /// An object that can tell you the version number to
    /// use for a particular JS component (given a string name).
    /// </param>
    /// <param name="_ptPageMgr">The page manager for this component manager.</param>
    public JSComponentMgr(String _strControlName, JSRegistry _ptJSRegistry,
     IJSComponentVersionMgr _ptJSCompVerMgr, PageMgr _ptPageMgr) : base(_ptPageMgr)
    {
      m_ptJSRegistry = _ptJSRegistry;
      m_strJSControlName = _strControlName;
      m_nVersionID = _ptJSCompVerMgr.GetJSComponentVersionNumber(_strControlName);
      m_strVersionID = "LATEST";
    }
    
    /// <summary>
    /// Returns the name for JS controls of this type (ie.e jsdatepicker").
    /// </summary>
    /// <returns>The name for JS controls of this type.</returns>
    public virtual String GetJSControlName()
    {
      return m_strJSControlName;
    }
    
    /// <summary>
    /// This method returns the HTML script include tags required by JS
    /// components of this type. If none are found, it will return null.
    /// </summary>
    /// <returns>The HTML script includes.</returns>
    public virtual HTMLElementCollection GetJSIncludes()
    {
      HTMLElementCollection retVal = null;
      try
      {
        XPLocale locale = GetPageMgr().GetLocale();
        String strHTMLLinks = m_ptJSRegistry.GetJSComponent(m_strJSControlName,
         m_nVersionID).GenIncludes(locale);
        retVal = new HTMLElementCollection();
        retVal.AddInnerHTMLString(strHTMLLinks);
      }
      catch (Exception e)
      {
        // TODOCR: DONH - This should have better error handling.  Trace error.
        return null;
      }
      return retVal;
    }
    
    /// <summary>
    /// Returns the version number of the JS component that is being used.
    /// </summary>
    /// <returns>
    /// The version number of the JS component that is being used (as an
    /// int).
    /// </returns>
    public virtual int GetVersionIDAsInt()
    {
      return m_nVersionID;
    }
    
    /// <summary>
    /// Returns the version number of the JS component that is being used.
    /// </summary>
    /// <returns>
    /// The version number of the JS component that is being used (as a
    /// String).
    /// </returns>
    public virtual String GetVersionIDAsString()
    {
      return m_strVersionID;
    }
  }
}