using System;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.form;
namespace com.plumtree.uiinfrastructure.pagecontrols.framework
{
  /// <summary>
  /// The page manager (PageMgr) class aggregates all of the comoponent managers for 
  /// a given page. It also page-scope information such as locale and time zone. 
  /// It has some convenience methods that act on all ComponenteMgrs registered with 
  /// the PageMgr and all their Page Controls -- StoreAllUserInput and Validate. 
  /// </summary>
  /// <author>John Osborne</author>
  public class PageMgr
  {
    
    /// <summary>
    /// logger of errors 
    /// </summary>
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.pagecontrols.framework.PageMgr");
    
    /// <summary>
    /// The base URL to the image server. 
    /// </summary>
    private String m_strImageServerBaseURL;
    
    /// <summary>
    /// The connection URL to the image server 
    /// </summary>
    private String m_strImageServerConnectionURL;
    
    /// <summary>
    /// The language ID for this page (i.e. "en") 
    /// </summary>
    private String m_strLangID;
    
    /// <summary>
    /// A list of all the registered component managers on this page. 
    /// </summary>
    private XPArrayList m_alRegisteredComponentMgrs;
    
    /// <summary>
    /// hashtable associating page controls with their component mgrs 
    /// </summary>
    private XPHashtable m_hashControlsManagers;
    
    /// <summary>
    /// The locale for this page. 
    /// </summary>
    private XPLocale m_xpLocale;
    
    /// <summary>
    /// The timezone to be used by date-pickers under the control of this
    /// manager
    /// </summary>
    private XPTimeZone m_xpTimeZone;
    
    /// <summary>
    /// array list of error messages 
    /// </summary>
    private XPArrayList m_xpErrorMsgs = new XPArrayList();
    
    /// <summary>
    /// The access mode for the page 
    /// </summary>
    private AccessStyles m_asAccessStyle;
    
    /// <summary>
    /// The table of component managers 
    /// </summary>
    private XPHashtable m_xpComponentManagers = new XPHashtable();
    
    /// <summary>
    /// Creates a new PageMgr object. At the time of creation, the page manager
    /// has no component managers; they must be registered. If the locale or
    /// time zone is null, the default is used. The image server base URL
    /// cannot be null or emppty string.
    /// </summary>
    /// <param name="_asAccessStyle">The access mode for this page</param>
    /// <param name="_xpLocale">The locale for this page.</param>
    /// <param name="_xpTimeZone">The time-zone for this page.</param>
    /// <param name="_strImageServerBaseURL">The base URL to the image server.</param>
    /// <param name="_strImageConnectionBaseURL">The connection URL to the image server</param>
    public PageMgr(AccessStyles _asAccessStyle, XPLocale _xpLocale, XPTimeZone _xpTimeZone,
     String _strImageServerBaseURL, String _strImageServerConnectionURL)
    {
      m_asAccessStyle = _asAccessStyle;
      m_strImageServerConnectionURL = _strImageServerConnectionURL;
      m_alRegisteredComponentMgrs = new XPArrayList();
      if (null == _xpLocale)
      {
        m_xpLocale = XPLocale.GetDefault();
      } else
      {
        m_xpLocale = _xpLocale;
      }
      if (null != _xpTimeZone)
      {
        m_xpTimeZone = _xpTimeZone;
      } else
      {
        m_xpTimeZone = XPTimeZone.GetCurrentTimeZone();
      }
      m_strLangID = m_xpLocale.GetLanguage();
      if ((_strImageServerBaseURL == null) || _strImageServerBaseURL.Equals(""))
      {
        throw new XPException("You must pass in a valid Image Server URL to the Page Manger.");
        }
      m_strImageServerBaseURL = _strImageServerBaseURL;
      m_hashControlsManagers = new XPHashtable();
    }
    
    /// <summary>
    /// Returns the image server base URL.
    /// </summary>
    /// <returns>The image server base url.</returns>
    public virtual String GetImgSvrBaseURL()
    {
      return m_strImageServerBaseURL;
    }
    
    /// <summary>
    /// Returns the language ID for this page. (It is derived from the locale
    /// the client used when the
    /// </summary>
    /// <returns>The language ID (i.e. "en").</returns>
    public virtual String GetLangID()
    {
      return m_strLangID;
    }
    
    /// <summary>
    /// Returns the locale that is being used by this page manager.
    /// </summary>
    /// <returns>The XPLocale for this page.</returns>
    public virtual XPLocale GetLocale()
    {
      return m_xpLocale;
    }
    
    /// <summary>
    /// Returns the number of component managers currently registered with this
    /// page manager.
    /// </summary>
    /// <returns>
    /// An int that tells the clinent how many component managers are
    /// registered.
    /// </returns>
    public virtual int GetNumComponentMgrs()
    {
      return m_alRegisteredComponentMgrs.GetSize();
    }
    
    /// <summary>
    /// Returns the timezone being used by this page manager.
    /// </summary>
    /// <returns>The timezone.</returns>
    public virtual XPTimeZone GetTimeZone()
    {
      return m_xpTimeZone;
    }
    
    /// <summary>
    /// This method allows you to register a component manager with the page
    /// manager. The order in which you register components is the order in
    /// which calls will be made when doing any looping operation--storing or
    /// printing. If you attempt to re-register the same component, it will
    /// overwrite the one that existed there before.
    /// </summary>
    /// <param name="_ptComponentMgr">The component manger that you wish to register.</param>
    public virtual void RegisterComponentMgr(ComponentMgr _ptComponentMgr)
    {
      if (_ptComponentMgr != null)
      {
        if (m_alRegisteredComponentMgrs.Contains(_ptComponentMgr))
        {
          int nIndex = m_alRegisteredComponentMgrs.IndexOf(_ptComponentMgr);
          //m_alRegisteredComponentMgrs.Insert(nIndex, _ptComponentMgr);
          m_alRegisteredComponentMgrs.SetElement(nIndex, _ptComponentMgr);
          } else
        {
          m_alRegisteredComponentMgrs.Add(_ptComponentMgr);
        }
      }
    }
    
    /// <summary>
    /// This method stores all data entered by users to controls on this page.
    /// It does so by looping through the registered component mangers and
    /// asking them to save themselves. (The component managers, in turn, loop
    /// through the components and ask them to save themselves. The order in
    /// which the save is called is the same as the order in which the
    /// component managers were registered.
    /// </summary>
    /// <param name="_htRequestParameters">A hashtable of the request parameters.</param>
    public virtual int StoreAllUserInput(XPHashtable _htRequestParameters)
    {
      int nError = RepostControl.PAGE_STATUS_VALID;
      int nStatus;
      for (int i = 0;i < m_alRegisteredComponentMgrs.GetSize();i++)
      {
        ComponentMgr ptComponentMgr = GetComponentMgr(i);
        nStatus = ptComponentMgr.StoreAllUserInput(_htRequestParameters);
        if (nStatus == RepostControl.PAGE_STATUS_INVALID)
        {
          nError = nStatus;
        }
      }
      return nError;
    }
    
    /// <summary>
    /// This method validates all Page Controls owned by this PageMgr. It does so by
    /// looping through all registered ComponentMgrs which in turn loop through all their
    /// registered components (Page Controls). It returns an XPArrayList of all UIError
    /// messages generated by the various Page Controls.
    /// </summary>
    /// <returns>XPArrayList of UIError objects.</returns>
    public virtual XPArrayList Validate()
    {
      XPArrayList myErrors = new XPArrayList(0);
      for (int i = 0;i < m_alRegisteredComponentMgrs.GetSize();i++)
      {
        ComponentMgr ptComponentMgr = GetComponentMgr(i);
        myErrors.AddRange(ptComponentMgr.Validate());
      }
      return myErrors;
    }
    
    /// <summary>
    /// This method provides js validation code for all Page Controls owned by this PageMgr.
    /// It does so by looping through all registered ComponentMgrs which in turn loop through
    /// all their registered components (Page Controls) to return their validation code. 
    /// </summary>
    /// <returns>XPArrayList of UIError objects.</returns>
    public virtual String GetJSValidationCode()
    {
      XPStringBuilder sb = new XPStringBuilder(1250);
      for (int i = 0;i < m_alRegisteredComponentMgrs.GetSize();i++)
      {
        ComponentMgr ptComponentMgr = GetComponentMgr(i);
        sb.Append(ptComponentMgr.GetJSValidationCode());
      }
      return sb.ToString();
    }
    public virtual void AddToValidationFunction(AFormDP _ptAFormDP)
    {
      for (int i = 0;i < m_alRegisteredComponentMgrs.GetSize();i++)
      {
        ComponentMgr ptComponentMgr = GetComponentMgr(i);
        ptComponentMgr.AddToValidationFunction(_ptAFormDP);
      }
    }
    
    /// <summary>
    /// Returns a component manager given a valid index. This is useful if the
    /// client wants to loop through all the component managers on the page and
    /// perform some operation. To do this, please refer to the
    /// GetNumComponentMgrs method. If you pass in an invalid index, null is
    /// returned.
    /// </summary>
    /// <param name="_nIndex">
    /// The index for a component manager registered with this
    /// page manager.
    /// </param>
    /// <returns>A component manager (or null).</returns>
    private ComponentMgr GetComponentMgr(int _nIndex)
    {
      Object oElement = null;
      try
      {
        oElement = m_alRegisteredComponentMgrs.GetElement(_nIndex);
      }
      catch (Exception e)
      {
        oElement = null;
      }
      // Do nothing, just return null
      if (oElement != null)
      {
        ComponentMgr oComponentMgr = (ComponentMgr) oElement;
        return oComponentMgr;
      }
      return null;
    }
    
    /// <summary>
    /// Adds the error message to the array containing the rest of the 
    /// error messages.
    /// </summary>
    /// <param name="_strErrorMsg"></param>
    public virtual void AddErrorMessage(String _strErrorNum, String _strFileName)
    {
      if (!m_xpErrorMsgs.Contains(_strErrorNum + " " + _strFileName))
      {
        m_xpErrorMsgs.Add(_strErrorNum + " " + _strFileName);
      }
    }
    
    /// <summary>
    /// Returns all the error messages for the Activity Apace
    /// </summary>
    /// <returns>String - collection of error messages</returns>
    public virtual String GetErrorMessages(AActivitySpace _asOwner)
    {
      XPStringBuilder strErrorMsgs = new XPStringBuilder();
      XPStringTokenizer strMsg;
      IXPEnumerator myXPEnumerator = m_xpErrorMsgs.GetEnumerator();
      // extract all the error number and file name pairs from the m_xpErrorMsgs hashtable
      // and append them together to return as a String
      while (myXPEnumerator.MoveNext())
      {
        try
        {
          strMsg = new XPStringTokenizer((String) myXPEnumerator.GetCurrent());
          strErrorMsgs.Append(_asOwner.GetString(XPConvert.ToInteger(strMsg.GetNextToken()),
           strMsg.GetNextToken()) + "\n");
        }
        catch (Exception e)
        {
          log.Error(e, e.Message);
        }
      }
      // clear the collection of error msgs
      m_xpErrorMsgs.Clear();
      return strErrorMsgs.ToString();
    }
    
    /// <summary>
    /// Retreives an instance of a Page Control specified by _strMVCPageControlName 
    /// and returns it. Registers that Page Control with the ComponentMgr appropriate
    /// for it which is registered with this PageMgr instance. If no ComponentMgr of
    /// that type exists the Page Control creates a ComponentMgr of the correct type
    /// and that ComponentMgr is registered with the PageMgr.
    /// </summary>
    /// <param name="_strMVCPageControlName">the name of the Page Control (i.e. HTMLTextBox)</param>
    /// <returns>IPageControl an instance of the Page Control Object</returns>
    public virtual IPageControl AddPageControl(String _strMVCPageControlName)
    {
      IPageControl myPageControl;
      ComponentMgr myComponentMgr;
      // if in standard mode, retrive component from PageControl table
      if (m_asAccessStyle.Equals(AccessStyles.STANDARD))
      {
        myPageControl = (IPageControl) ASManager.INSTANCE.GetPageControl(_strMVCPageControlName);
        } else
      // in 508 mode, retrivie the component with the MVC name appended with "_D"
      {
        if ((myPageControl = (IPageControl) ASManager.INSTANCE.GetPageControl(_strMVCPageControlName + "_D")) == null)
        {
          myPageControl = (IPageControl) ASManager.INSTANCE.GetPageControl(_strMVCPageControlName);
          }
      }
      // associate this type of page control with this type of ComponentMgr
      m_hashControlsManagers.PutElement(_strMVCPageControlName, myPageControl.GetComponentMgrName());
      // retrieve component manager and register page control
      //if (m_xpComponentManagers.ContainsKey(_strMVCPageControlName))
      if (m_xpComponentManagers.ContainsKey(myPageControl.GetComponentMgrName()))
      {
        //myComponentMgr = (ComponentMgr) m_xpComponentManagers.GetElement(_strMVCPageControlName);
        myComponentMgr = (ComponentMgr) m_xpComponentManagers.GetElement(myPageControl.GetComponentMgrName());
        myComponentMgr.RegisterComponent(myPageControl);
      } else
      // create component manager, store in hashtable, and register page control
      {
        myComponentMgr = myPageControl.GetComponentMgr(this);
        myComponentMgr.RegisterComponent(myPageControl);
        //m_xpComponentManagers.PutElement(_strMVCPageControlName, myComponentMgr);
        m_xpComponentManagers.PutElement(myPageControl.GetComponentMgrName(),
         myComponentMgr);
      }
      // register the component manager with the page
      RegisterComponentMgr(myComponentMgr);
      return myPageControl;
    }
    
    /// <summary>
    /// Return the Page Control of the type specified by _strMVCPageControlName (i.e. 
    /// HTMLTextBox) with the unique instance ID specified by _strUniqueInstanceID.
    /// </summary>
    /// <param name="_strMVCPageControlName">the name of the Page Control (i.e. HTMLTextBox)</param>
    /// <param name="_strUniqueInstanceID"></param>
    /// <returns>IPageControl</returns>
    public virtual IPageControl GetPageControl(String _strMVCPageControlName,
     String _strUniqueInstanceID)
    {
      ComponentMgr myComponentMgr;
      IPageControl myPageControl = null;
      try
      {
        // retrieve the component manager
        String myComponentMgrName = (String) m_hashControlsManagers.GetElement(_strMVCPageControlName);
        myComponentMgr = (ComponentMgr) m_xpComponentManagers.GetElement(myComponentMgrName);
        // retrieve the component using the unique instance id
        myPageControl = (IPageControl) myComponentMgr.GetComponent(_strUniqueInstanceID);
        }
      catch (Exception e)
      {
        log.Error(e, e.Message);
      }
      return myPageControl;
    }
    
    /// <summary>
    /// Return a ComponentMgr of the correct type for the given Page Control. 
    /// </summary>
    /// <param name="_strMVCPageControlName">the name of the Page Control (i.e. HTMLTextBox)</param>
    /// <returns>The ComponentMgr of the correct type for the Page Control </returns>
    public virtual ComponentMgr GetComponentMgr(String _strMVCPageControlName)
    {
      // retrieve the component manager with the MVC name
      String myComponentMgrName = (String) m_hashControlsManagers.GetElement(_strMVCPageControlName);
      return (ComponentMgr) m_xpComponentManagers.GetElement(myComponentMgrName);
      }
    
    /// <summary>
    /// Return the ImageServerConnectionURL member variable.
    /// </summary>
    /// <returns>The image server connectin URL as a string </returns>
    public virtual String GetImageServerConnectionURL()
    {
      return m_strImageServerConnectionURL;
    }
  }
}