using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.pagecontrols.framework;
using com.plumtree.uiinfrastructure.validators;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols
{
  /// <summary>
  /// HTMLDuration Page Control handles display and values of a page control
  /// made up of a text box and a select dropdown. A bitwise operators (with
  /// values defined in PTDurationCombo) dictates that time increments to display
  /// in the select.
  /// </summary>
  /// <author>CaiusV</author>
  public class HTMLDuration : AHTMLComponent,IPageControl
  {
    
    /// <summary>
    /// for logging 
    /// </summary>
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols.HTMLDuration");
    
    /// <summary>
    /// base name constant 
    /// </summary>
    private const String STR_HTML_INPUT_BASE_NAME = "in_tx_pc";
    
    /// <summary>
    /// class name constant 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "HTMLDuration";
    
    /// <summary>
    /// the textbox for this object 
    /// </summary>
    public HTMLTextBox m_TextBox;
    
    /// <summary>
    /// the selectdropdown for this object 
    /// </summary>
    public HTMLSelectDropDown m_SelectDropDown;
    
    /// <summary>
    /// the defautl value of the textbox 
    /// </summary>
    private String m_sDefaultValue;
    private String m_sTextBoxErrorMessage;
    
    /// <summary>
    /// Default constructor. Creates a new HTMLTextAndDropDown object.
    /// </summary>
    public HTMLDuration()
    {
    }
    
    /// <summary>
    /// Initializes a new HTMLTextAndDropDown object. Creates and initializes
    /// the HTMLTextBox and HTMLSelectDropdown objects contained by this object.
    /// A bitwise operators (with values defined in PTDurationCombo) dictates 
    /// that time increments to display in the select.
    /// </summary>
    /// <param name="_ptPageMgr">PageManager</param>
    /// <param name="_strUniqueInstanceID">Unique Instance ID</param>
    /// <param name="_intDisplayFlags">bitwise operator</param>
    /// <param name="_asOwner">AActivitySpace where this appears</param>
    /// <param name="_strFormName">name of the form on which this appears</param>
    /// <param name="_strDefaultValue">default value for textbox (must be an int)</param>
    public virtual void Init(PageMgr _ptPageMgr, String _strUniqueInstanceID,
     int _intDisplayFlags, AActivitySpace _asOwner, String _strFormName, String _strDefaultValue)
    {
      try
      {
        XPConvert.ToInteger(_strDefaultValue);
      }
      catch (XPException exception)
      {
        log.Error(exception, "default value " + _strDefaultValue + " is not an int");
        }
      base.Init(_ptPageMgr, _strUniqueInstanceID);
      m_TextBox = (HTMLTextBox) _ptPageMgr.AddPageControl(HTMLTextBox.STR_MVC_CLASS_NAME);
      m_TextBox.Init(_ptPageMgr, _strUniqueInstanceID);
      m_TextBox.SetHTMLComponentSize("3");
      IValidator myIntValidator = new PositiveIntValidator(_strFormName, m_TextBox.GetHTMLComponentName(),
       _strDefaultValue);
      myIntValidator.SetErrorMessage("HTMLDuration must be a positive int");
      m_TextBox.AddValidator(myIntValidator);
      if (_asOwner.GetIsAccessStyleStandard())
      {
        m_TextBox.m_ptHTMLInput.SetOnChange(myIntValidator.GetJSFunction("this"));
        }
      m_TextBox.SetHTMLComponentValue(_strDefaultValue);
      XPArrayList myValues = new XPArrayList();
      XPArrayList myLabels = new XPArrayList();
      //Determine which options to display through bitwise comparison
      if ((_intDisplayFlags & PTDurationCombo.DISPLAYMILLISECONDS) != 0)
      {
        myValues.Add(XPConvert.ToString(PTDurationCombo.DISPLAYMILLISECONDS));
        myLabels.Add(_asOwner.GetString(113, ResourceConstants.INFRASTRUCTURE));
        }
      if ((_intDisplayFlags & PTDurationCombo.DISPLAYSECONDS) != 0)
      {
        myValues.Add(XPConvert.ToString(PTDurationCombo.DISPLAYSECONDS));
        myLabels.Add(_asOwner.GetString(114, ResourceConstants.INFRASTRUCTURE));
        }
      if ((_intDisplayFlags & PTDurationCombo.DISPLAYMINUTES) != 0)
      {
        myValues.Add(XPConvert.ToString(PTDurationCombo.DISPLAYMINUTES));
        myLabels.Add(_asOwner.GetString(115, ResourceConstants.INFRASTRUCTURE));
        }
      if ((_intDisplayFlags & PTDurationCombo.DISPLAYHOURS) != 0)
      {
        myValues.Add(XPConvert.ToString(PTDurationCombo.DISPLAYHOURS));
        myLabels.Add(_asOwner.GetString(116, ResourceConstants.INFRASTRUCTURE));
        }
      if ((_intDisplayFlags & PTDurationCombo.DISPLAYDAYS) != 0)
      {
        myValues.Add(XPConvert.ToString(PTDurationCombo.DISPLAYDAYS));
        myLabels.Add(_asOwner.GetString(117, ResourceConstants.INFRASTRUCTURE));
        }
      if ((_intDisplayFlags & PTDurationCombo.DISPLAYWEEKS) != 0)
      {
        myValues.Add(XPConvert.ToString(PTDurationCombo.DISPLAYWEEKS));
        myLabels.Add(_asOwner.GetString(118, ResourceConstants.INFRASTRUCTURE));
        }
      if ((_intDisplayFlags & PTDurationCombo.DISPLAYMONTHS) != 0)
      {
        myValues.Add(XPConvert.ToString(PTDurationCombo.DISPLAYMONTHS));
        myLabels.Add(_asOwner.GetString(119, ResourceConstants.INFRASTRUCTURE));
        }
      if ((_intDisplayFlags & PTDurationCombo.DISPLAYYEARS) != 0)
      {
        myValues.Add(XPConvert.ToString(PTDurationCombo.DISPLAYYEARS));
        myLabels.Add(_asOwner.GetString(120, ResourceConstants.INFRASTRUCTURE));
        }
      String[][] nameValuePairs = new String[myValues.GetSize()][];
      for (int j2c_index = 0;j2c_index < myValues.GetSize();j2c_index++)
      {
        nameValuePairs[j2c_index] = new String[2];
      }
      for (int i = 0;i < myValues.GetSize();i++)
      {
        nameValuePairs[i][0] = (String) myValues.GetElement(i);
        nameValuePairs[i][1] = (String) myLabels.GetElement(i);
      }
      m_SelectDropDown = (HTMLSelectDropDown) _ptPageMgr.AddPageControl(HTMLSelectDropDown.STR_MVC_CLASS_NAME);
      m_SelectDropDown.Init(_ptPageMgr, _strUniqueInstanceID, nameValuePairs,
       "foo");
    }
    
    /// <summary>
    /// Returns the base name of this HTML Component.
    /// </summary>
    /// <returns>String</returns>
    protected override String GetHTMLComponentBaseName()
    {
      return STR_HTML_INPUT_BASE_NAME;
    }
    
    /// <summary>
    /// Returns the HTML for this component.
    /// </summary>
    /// <returns>HTML for this object</returns>
    public override HTMLElementCollection GetComponentHTML()
    {
      HTMLElementCollection retValue = new HTMLElementCollection();
      retValue.AddInnerHTMLElement(m_TextBox.GetComponentHTML());
      retValue.AddInnerHTMLString(CommonHTMLStrings.SPACE);
      if (null != GetHTMLComponentStyleClass())
      {
        m_TextBox.SetHTMLComponentStyleClass(GetHTMLComponentStyleClass());
        m_SelectDropDown.SetHTMLComponentStyleClass(GetHTMLComponentStyleClass());
        }
      retValue.AddInnerHTMLElement(m_SelectDropDown.GetComponentHTML());
      return retValue;
    }
    
    /// <summary>
    /// Return the component values of the TextBox and SelectDropdown
    /// separated by a space.
    /// </summary>
    public override String GetHTMLComponentValue()
    {
      return m_TextBox.GetHTMLComponentValue() + " " + m_SelectDropDown.GetHTMLComponentValue();
      }
    
    /// <summary>
    /// Return the value of this page contrl in seconds. so if the units is set to 
    /// minutes and the value of the textbox is 5 return 5 * 60 or 300.
    /// </summary>
    /// <returns>the value in seconds</returns>
    public virtual int GetValueInSeconds()
    {
      try
      {
        int returnValue = XPConvert.ToInteger(m_TextBox.GetHTMLComponentValue());
        int nDurationType = XPConvert.ToInteger(m_SelectDropDown.GetHTMLComponentValue());
        switch (nDurationType){
        case PTDurationCombo.DISPLAYYEARS:
        returnValue = 12 * returnValue;
        goto case PTDurationCombo.DISPLAYMONTHS;
        case PTDurationCombo.DISPLAYMONTHS:
        returnValue = 4 * returnValue;
        goto case PTDurationCombo.DISPLAYWEEKS;
        case PTDurationCombo.DISPLAYWEEKS:
        returnValue = 7 * returnValue;
        goto case PTDurationCombo.DISPLAYDAYS;
        case PTDurationCombo.DISPLAYDAYS:
        returnValue = 24 * returnValue;
        goto case PTDurationCombo.DISPLAYHOURS;
        case PTDurationCombo.DISPLAYHOURS:
        returnValue = 60 * returnValue;
        goto case PTDurationCombo.DISPLAYMINUTES;
        case PTDurationCombo.DISPLAYMINUTES:
        returnValue = 60 * returnValue;
        break;
        }
        return returnValue;
      }
      catch (XPException e)
      {
        log.Warn(e, e.Message);
        return 0;
      }
    }
    
    /// <summary>
    /// This method is used by the ASManager to return new instances of managed
    /// objects.
    /// </summary>
    /// <returns>A new instance of the managed class (i.e. return new Foo();)</returns>
    public override Object Create()
    {
      return new HTMLDuration();
    }
    
    /// <summary>
    /// Return the name of the MVC object.
    /// </summary>
    /// <returns>the name of the object.</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// For every Validator associated with this object execute it's Validate method.
    /// Return an Arraylist of UIError objects generated from executing the Validators.
    /// Return and empty Arraylist if no problems were found.
    /// </summary>
    /// <returns>Arraylist of UIErrors.</returns>
    public override XPArrayList Validate()
    {
      XPArrayList myErrorList = base.Validate();
      if (m_TextBox.Validate().GetSize() > 0)
      {
        myErrorList.Add(m_TextBox.Validate().GetElement(0));
      }
      return myErrorList;
    }
    
    /// <summary>
    /// Return a string made up of all the JSValidationCode for each of the
    /// Validator's associated with this component.
    /// </summary>
    /// <returns>JavaScript validation code</returns>
    public override String GetJSValidationCode()
    {
      XPStringBuilder sb = new XPStringBuilder(1250);
      sb.Append(base.GetJSValidationCode());
      sb.Append(m_TextBox.GetJSValidationCode());
      return sb.ToString();
    }
    
    /// <returns>Returns the m_sTextBoxErrorMessage.</returns>
    public virtual String GetTextBoxErrorMessage()
    {
      return m_sTextBoxErrorMessage;
    }
    
    /// <param name="textBoxErrorMessage">The m_sTextBoxErrorMessage to set.</param>
    public virtual void SetTextBoxErrorMessage(String textBoxErrorMessage)
    {
      m_sTextBoxErrorMessage = textBoxErrorMessage;
    }
  }
}