using System;
using com.plumtree.xpshared.htmlelements;
using com.plumtree.openfoundation.util;
namespace com.plumtree.uiinfrastructure.statichelpers
{
  /// <summary>
  /// This class is used to insert anchors in localized strings using placeholders.
  /// Strings should have placeholders {0} and {1} around the section that should be
  /// turned into a link.  This class does not handle escaped placeholders (\\{0}).
  /// </summary>
  public class AnchorInLocalizedString
  {
    private const String PLACEHOLDER0 = "{0}";
    private const String PLACEHOLDER1 = "{1}";
    
    /// <summary>
    /// This method returns the section of the string before the place holder.
    /// </summary>
    /// <param name="String">the entire localized string</param>
    /// <returns>
    /// String the section before the first placeholder.  "" If there
    /// is no place holder.
    /// </returns>
    public static String GetStringBeforeAnchor(String strBaseText)
    {
      int nPlaceHolderPosition = XPStringUtility.IndexOf(strBaseText, PLACEHOLDER0);
      if (nPlaceHolderPosition == -1)
      {
        return "";
      }
      return XPStringUtility.Substring(strBaseText, 0, nPlaceHolderPosition);
      }
    
    /// <summary>
    /// This method returns the section of the string after the place holder.
    /// </summary>
    /// <param name="String">the entire localized string</param>
    /// <returns>String the section before the first placeholder.</returns>
    public static String GetStringAfterAnchor(String strBaseText)
    {
      int nPlaceHolderPosition = XPStringUtility.IndexOf(strBaseText, PLACEHOLDER1);
      if (nPlaceHolderPosition == -1)
      {
        return "";
      }
      return XPStringUtility.Substring(strBaseText, nPlaceHolderPosition + PLACEHOLDER1.Length);
      }
    
    /// <summary>
    /// This method returns the HTMLAnchor generated from the section of the
    /// string between the {0} and {1} placeholders.
    /// </summary>
    /// <param name="String">the entire localized string</param>
    /// <param name="String">the URL of the anchor</param>
    /// <returns>HTMLAnchor the anchor generated from the text inbetween {0} and {1}</returns>
    public static HTMLAnchor GetAnchor(String strBaseText, String strURL)
    {
      int nFirstPlaceHolderPosition = XPStringUtility.IndexOf(strBaseText,
       PLACEHOLDER0);
      int nSecondPlaceHolderPosition = XPStringUtility.IndexOf(strBaseText,
       PLACEHOLDER1);
      if ((nFirstPlaceHolderPosition == -1) || (nFirstPlaceHolderPosition == -1))
      {
        return null;
      }
      String strAnchorText = XPStringUtility.Substring(strBaseText, nFirstPlaceHolderPosition + PLACEHOLDER0.Length,
       nSecondPlaceHolderPosition);
      HTMLAnchor result = new HTMLAnchor(strURL);
      result.AddInnerHTMLEncodedString(strAnchorText);
      result.SetTitle(strAnchorText);
      return result;
    }
    
    /// <summary>
    /// This method returns an HTMLElementCollection containing the string with the
    /// anchor element inserted between the {0} and {1} placeholders.
    /// </summary>
    /// <param name="String">the entire localized string</param>
    /// <param name="String">the URL of the anchor</param>
    /// <returns>HTMLElementCollection the String with the anchor generated from the text inbetween {0} and {1}</returns>
    public static HTMLElementCollection GetAnchorInString(String strBaseText,
     String strURL)
    {
      HTMLElementCollection result = new HTMLElementCollection();
      result.AddInnerHTMLEncodedString(GetStringBeforeAnchor(strBaseText));
      result.AddInnerHTMLElement(GetAnchor(strBaseText, strURL));
      result.AddInnerHTMLEncodedString(GetStringAfterAnchor(strBaseText));
      return result;
    }
    
    /// <summary>
    /// This method takes the string with the anchor element inserted between
    /// the {0} and {1} placeholders and adds it to the given element.
    /// </summary>
    /// <param name="String">the entire localized string</param>
    /// <param name="String">the URL of the anchor</param>
    /// <param name="HTMLElement">the element to add the String with the anchor generated from the text inbetween {0} and {1}</param>
    public static void AddAnchorInStringToElement(String strBaseText, String strURL,
     HTMLElement element)
    {
      element.AddInnerHTMLEncodedString(GetStringBeforeAnchor(strBaseText));
      element.AddInnerHTMLElement(GetAnchor(strBaseText, strURL));
      element.AddInnerHTMLEncodedString(GetStringAfterAnchor(strBaseText));
      }
  }
}