using System;
using com.plumtree.openlog;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.application.varpacks;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.uiinfrastructure.statichelpers
{
  /// <summary>
  /// Helper to get and set key values on the HTTPSession sandbox.  Please refer
  /// to AActivitySpace to see usage.  Includes things originally in LoginHelper;
  /// intent is to support core personal settings applicable to any product:
  /// access style, locale, language.
  /// </summary>
  /// <author>DavidP</author>
  public class PersonalSettingsHelper
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.statichelpers.PersonalSettingsHelper");
    
    /// <summary>
    /// key session setting names used by defaults 
    /// </summary>
    public const String ACCESS_STYLE = "strAccessStyle";
    public const String LOCALE = "strLocale";
    public const String LANGUAGE = "strLanguage";
    public const String TIMEZONE = "strTimeZone";
    public const String BASE_URL = "strBaseURL";
    
    /// <summary>
    /// name of the personal settings sub-session.  This should only be used
    /// by the interpreter.
    /// </summary>
    public const String PREFS_SUBSESSION_NAME = "userprefsss";
    
    /// <summary>
    /// Call this method to determine if the subsession for caching personal
    /// settings exists in the specified ISessionManager.
    /// <br /><br />
    /// </summary>
    /// <param name="AActivitySpace"></param>
    public static bool DoesPrefsSubSessionExist(AActivitySpace activitySpace)
    {
      // This method simply insures you're using the right HTTPSession "sandbox" off of the AActivitySpace
      ISessionManager sessionManager = activitySpace.GetSubSession();
      return DoesPrefsSubSessionExist(sessionManager);
    }
    
    /// <summary>
    /// Use this version of DoesPrefsSubSessionExist if all you have is
    /// the  session manager.
    /// <br /><br />
    /// </summary>
    /// <param name="ISessionManager"></param>
    public static bool DoesPrefsSubSessionExist(ISessionManager sessionManager)
    {
      bool b = false;
      if (sessionManager.ContainsSubSession(PersonalSettingsHelper.PREFS_SUBSESSION_NAME))
      {
        b = true;
      }
      return b;
    }
    
    /// <summary>
    /// Call this method to find the value associated with given setting name.
    /// <br /><br />
    /// </summary>
    /// <param name="oaSettings">Pass an Object[][] obtained from LookupPersonalSettings</param>
    /// <param name="String">name of setting</param>
    public static Object FindPersonalSetting(Object[][] oaSettings, String sName)
    {
      Object oVal = null;
      for (int iSettingIndex = 0;iSettingIndex < oaSettings[0].Length;++iSettingIndex)
      {
        String sSettingName = (String) oaSettings[0][iSettingIndex];
        if (0 == sSettingName.CompareTo(sName))
        {
          //oVal = (String) oaSettings[1][iSettingIndex]; // DAP commented out because cast throws exception
          oVal = oaSettings[1][iSettingIndex];
          break;
        }
      }
      return oVal;
    }
    
    /// <summary>
    /// Only use this version if all you have is the subsession
    /// </summary>
    /// <param name="sSettingName">String name of personal setting</param>
    /// <param name="sessionManager">ISessionManager</param>
    /// <throws>
    /// XPException This method will throw an XPException if you do not
    /// pass an ISessionManager already containing the correct subSession.  The
    /// ISessionManager obtained from your Activity Space will contain the correct
    /// subsession.
    /// </throws>
    public static Object GetPersonalSettingValue(String sSettingName, ISessionManager sessionManager,
     IApplication application)
    {
      log.Debug("Entering GetPersonalSettingValue() - sSettingsName: " + sSettingName);
      // Try to retrieve the requested setting directly from the low-level HTTPSession
      Object oVal = null;
      ISessionManager subSession = sessionManager.GetSubSession(PersonalSettingsHelper.PREFS_SUBSESSION_NAME);
      oVal = subSession.GetAttribute(sSettingName);
      if (oVal == null)
      {
        log.Debug("GetPersonalSettingValue() - Couldn\'t get from the session cache, checking defaults.");
        // On fail, fall through to hardcoded default values in certain cases
        if (sSettingName.Equals(ACCESS_STYLE))
        {
          oVal = ConfigHelper.GetConfigVariableAsString(ConfigConstants.PERSONAL_SETTINGS_SECTION,
           ConfigConstants.ACCESSIBILITY_SUBSECTION, ConfigConstants.VALUE_ATTRIBUTE,
           application);
        } else
        if (sSettingName.Equals(LOCALE))
        {
          oVal = ConfigHelper.GetConfigVariableAsString(ConfigConstants.INTERNATIONAL_SECTION,
           ConfigConstants.LOCALE_SUBSECTION, ConfigConstants.VALUE_ATTRIBUTE,
           application);
        } else
        if (sSettingName.Equals(LANGUAGE))
        {
          oVal = ConfigHelper.GetConfigVariableAsString(ConfigConstants.INTERNATIONAL_SECTION,
           ConfigConstants.LOCALE_SUBSECTION, ConfigConstants.VALUE_ATTRIBUTE,
           application);
          XPResourceManager rm = (XPResourceManager) application.GetCachingManager().GetEntry("ResourceManager");
          // Convert the default locale to a language
          oVal = rm.MapLocaleToSupportedUILangID(XPConvert.ToString(oVal));
          }
      }
      log.Debug("GetPersonalSettingValue() - oVal: " + oVal);
      return oVal;
    }
    
    /// <summary>
    /// Call this method to obtain the value of a personal setting.
    /// <br /><br />
    /// This value will be retrieved from the user's web session if possible,
    /// otherwise the portal server will be queried.
    /// <br /><br />
    /// </summary>
    /// <param name="sSettingName">String name of personal setting</param>
    /// <param name="AActivitySpace">as</param>
    /// <throws>
    /// XPException This method will throw an XPException if you do not
    /// pass an ISessionManager already containing the correct subSession.  The
    /// ISessionManager obtained from your Activity Space will contain the correct
    /// subsession.
    /// </throws>
    public static Object GetPersonalSettingValue(String sSettingName, AActivitySpace activitySpace)
    {
      if (!PersonalSettingsHelper.DoesPrefsSubSessionExist(activitySpace))
      {
        throw new XPException("SubSession for prefs cache does not exist.  Make sure AActivitySpace is valid.");
        }
      // This method simply insures you're using the right HTTPSession "sandbox" off of the AActivitySpace
      ISessionManager sessionManager = activitySpace.GetSubSession();
      IApplication application = activitySpace.GetApplication();
      return GetPersonalSettingValue(sSettingName, sessionManager, application);
      }
    
    /// <summary>
    /// Call this method to set a personal setting name value pair on the
    /// session (infrastructure HttpSession sandbox)
    /// <br /><br />
    /// </summary>
    /// <param name="sSettingName">String name of personal setting</param>
    /// <param name="oSettingVal">Object value of personal setting</param>
    /// <param name="as">
    /// AActivitySpace Pass an AActivitySpace implementing
    /// IXPActivitySpace.
    /// </param>
    /// <throws>XPException This method may throw an XPException.</throws>
    public static void SetPersonalSettingValue(String sSettingName, Object oSettingVal,
     AActivitySpace activitySpace)
    {
      // This method simply insures you're using the right HTTPSession "sandbox" off of the AActivitySpace
      ISessionManager sessionManager = activitySpace.GetSubSession();
      SetPersonalSettingValue(sSettingName, oSettingVal, sessionManager);
      }
    
    /// <summary>
    /// Call this method to set a personal setting name value pair on the
    /// session (infrastructure HttpSession sandbox)
    /// <br /><br />
    /// </summary>
    /// <param name="sSettingName">String name of personal setting</param>
    /// <param name="oSettingVal">Object value of personal setting</param>
    /// <param name="sessionManager">
    /// ISessionManager Pass the correct subsession (AS.
    /// GetSubSession)
    /// </param>
    /// <throws>XPException This method may throw an XPException.</throws>
    public static void SetPersonalSettingValue(String sSettingName, Object oSettingVal,
     ISessionManager sessionManager)
    {
      log.Debug("Entering SetAndCachePersonalSetting() - sSettingsName: " + sSettingName);
      ISessionManager subSession = sessionManager.GetSubSession(PersonalSettingsHelper.PREFS_SUBSESSION_NAME);
      subSession.SetAttribute(sSettingName, oSettingVal);
    }
    
    /// <summary>
    /// Call this method to remove a personal setting name on the
    /// session (infrastructure HttpSession sandbox)
    /// <br /><br />
    /// </summary>
    /// <param name="sSettingName">String name of personal setting</param>
    /// <param name="sessionManager">
    /// ISessionManager Pass the correct subsession (AS.
    /// GetSubSession)
    /// </param>
    /// <throws>XPException This method may throw an XPException.</throws>
    public static void RemovePersonalSetting(String sSettingName, ISessionManager sessionManager)
    {
      log.Debug("Entering RemovePersonalSetting() - sSettingsName: " + sSettingName);
      ISessionManager subSession = sessionManager.GetSubSession(PersonalSettingsHelper.PREFS_SUBSESSION_NAME);
      subSession.RemoveAttribute(sSettingName);
    }
  }
}