using System;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
namespace com.plumtree.uiinfrastructure.tree
{
  /// <summary>
  /// This abstract class handles setting most of the portal independent variables
  /// on the ITreeModel when a new tree is started.
  /// </summary>
  /// <author>Don Hayler</author>
  public abstract class AStartTreeControl : IControl
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.tree.AStartTreeControl");
    
    /// <summary>
    /// true = check boxes, false = radio buttons
    /// (multi select vs. single select of objects)
    /// </summary>
    public const String MULTI_SELECT = "Multi";
    
    /// <summary>
    /// See TreeConstants for select modes (leaves vs. folders vs. both) 
    /// </summary>
    public const String SELECT_MODE = "SelectMd";
    
    /// <summary>
    /// See TreeConstants for select options (strict leaves). Optional. 
    /// </summary>
    public const String SELECT_OPTIONS = "SelectOptions";
    
    /// <summary>
    /// The class IDs of the types of objects you want to display.
    /// If you don't want to display any objects (folders only), don't use this.
    /// Make sure to include catalogfolder if you want catalogfolders, otherwise
    /// only folders will appear.
    /// </summary>
    public const String CLASS_IDS_TO_DISPLAY = "Class";
    
    /// <summary>
    /// classID1,objectID1,classID2,objectID2,etc... 
    /// </summary>
    public const String SELECTED_ITEMS = "Select";
    
    /// <summary>
    /// classID1,objectID1,classID2,objectID2,etc...
    /// This will display all objects except for the following
    /// </summary>
    public const String HIDDEN_ITEMS = "Hide";
    
    /// <summary>
    /// classID1,objectID1,classID2,objectID2,etc...
    /// This will only display the selected objects.
    /// This does not work for IPTFolders.  The class IDs
    /// still need to be specified and the root folder ID
    /// needs to be specified, even though it will be
    /// ignored.
    /// Normal folder browsing will be ignored
    /// </summary>
    public const String DISPLAYED_ITEMS = "Display";
    
    /// <summary>
    /// This determines whether or not the root folder will be displayed.
    /// By default, it will be displayed.  Pass in true or false.
    /// </summary>
    public const String SHOW_ROOT = "ShowRoot";
    
    /// <summary>
    /// See TreeConstants for select modes (server vs. Javascript) 
    /// </summary>
    public const String SUBMIT_MODE = "SubmitMd";
    
    /// <summary>
    /// These two should be used together or not at all.
    /// The name of the parent form that you would like the data passed back to.
    /// </summary>
    public const String PARENT_FORM_NAME = "Form";
    
    /// <summary>
    /// The name of the input in the parent form that you would like the
    /// AActivitySpace ID of the tree space put into.  This is used to
    /// reopen the tree after it has been created.
    /// </summary>
    public const String PARENT_FORM_SPACE_ID = "Input";
    
    /// <summary>
    /// The name of the javascript function in the parent page to call when the tree
    /// is submitted.  This should not include the '();'.
    /// In Javascript submit mode this method should take an array argument which will
    /// be made up of objects with name, ObjectID, and ClassID member variables.
    /// see ATreeAS for details.  Warning: this array will dissapear after the function
    /// call is over.  If you want to save the items, you must copy them somewhere in
    /// your page.
    /// The names of the objects will be HTMLEncoded and truncated to 50 characters.
    /// </summary>
    public const String PARENT_SUBMIT_CALL = "Submit";
    
    /// <summary>
    /// The name of the javascript function in the parent page to call when tree
    /// elements are clicked.  This should include the whole path to the function
    /// (i.e. parent.myframe.myfunction) without the ending parentheses or semicolon
    /// '();'.  This should not include the '();'.  This function will be called
    /// with the clicked input as the only argument in the onclick method of the
    /// tree items.  It should return true or false depending on whether or not the
    /// button should be clicked (true implies it will be clicked, false implies it
    /// will not).  The function name should also not contain {,},',",\
    /// Using this method will turn off dynamic updating of the 'you have selected'
    /// frame, which uses the same mechanism.
    /// This should be used extremely sparingly (only in the save page currently).
    /// </summary>
    public const String ONCLICK_CALL = "OnClick";
    
    /// <summary>
    /// The title of the tree pop-up 
    /// </summary>
    public const String TITLE = "Title";
    
    /// <summary>
    /// The action sub-title of the tree 
    /// </summary>
    public const String ACTIONSUBTITLE = "SubTitle";
    
    /// <summary>
    /// Allow users to click finish on a tree with nothing selected. default is false 
    /// </summary>
    public const String ALLOW_EMPTY_SUBMISSION = "AllowEmpty";
    
    /// <summary>
    /// The help topic. 
    /// </summary>
    public const String HELP_TOPIC = "Help";
    protected ATreeAS m_asOwner;
    
    /// <summary>
    /// Internal data *
    /// </summary>
    protected ITreeModel m_asModel;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IControl#Execute()"></seealso>
    public virtual Redirect CheckActionSecurityAndExecute(XPHashtable arguments)
    {
      String[] sValues;
      // Reset tree
      m_asModel.ResetTree();
      // Multiple select
      sValues = (String[]) arguments.GetElement(MULTI_SELECT);
      if (sValues != null)
      {
        m_asModel.SetMultipleSelect(XPConvert.ToBoolean(sValues[0]));
      }
      // Selection mode
      sValues = (String[]) arguments.GetElement(SELECT_MODE);
      if (sValues != null)
      {
        try
        {
          int nSelectMode = XPConvert.ToInteger(sValues[0]);
          if ((nSelectMode == TreeConstants.N_COMPOSITESELECT) || (nSelectMode == TreeConstants.N_LEAFCOMPOSITESELECT) || (nSelectMode == TreeConstants.N_LEAFSELECT) || (nSelectMode == TreeConstants.N_NOSELECT))
          {
            m_asModel.SetSelectMode(nSelectMode);
          } else
          {
            log.Warn("Attempted to set Select Mode to an invalid value: " + nSelectMode);
            }
        }
        catch (XPFormatException e)
        {
          log.Error("could not parse Select Mode data in query string: " + sValues[0]);
          }
      }
      // Selection options
      sValues = (String[]) arguments.GetElement(SELECT_OPTIONS);
      if (sValues != null)
      {
        try
        {
          int nSelectOptions = XPConvert.ToInteger(sValues[0]);
          if ((nSelectOptions == TreeConstants.N_LOOSELEAF) || (nSelectOptions == TreeConstants.N_STRICTLEAF))
          {
            m_asModel.SetSelectOptions(nSelectOptions);
          } else
          {
            log.Warn("Attempted to set Select Options to an invalid value: " + nSelectOptions);
            }
        }
        catch (XPFormatException e)
        {
          log.Error("could not parse Select Options data in query string: " + sValues[0]);
          }
      }
      // Submit mode
      sValues = (String[]) arguments.GetElement(SUBMIT_MODE);
      if (sValues != null)
      {
        try
        {
          int nSubmitMode = XPConvert.ToInteger(sValues[0]);
          if ((nSubmitMode == TreeConstants.N_SERVERSUBMIT) || (nSubmitMode == TreeConstants.N_JAVASCRIPTSUBMIT))
          {
            m_asModel.SetSubmitMode(nSubmitMode);
          } else
          {
            log.Warn("Attempted to set Submit Mode to an invalid value: " + nSubmitMode);
            }
        }
        catch (XPFormatException e)
        {
          log.Error("could not parse Submit Mode data in query string: " + sValues[0]);
          }
      }
      // parent form name
      sValues = (String[]) arguments.GetElement(PARENT_FORM_NAME);
      if (sValues != null)
      {
        m_asModel.SetParentFormName(sValues[0]);
      }
      // parent form space ID
      sValues = (String[]) arguments.GetElement(PARENT_FORM_SPACE_ID);
      if (sValues != null)
      {
        m_asModel.SetParentFormSpaceID(sValues[0]);
      }
      // parent form submit call
      sValues = (String[]) arguments.GetElement(PARENT_SUBMIT_CALL);
      if (sValues != null)
      {
        if ((XPStringUtility.IndexOf(sValues[0], '(') == -1) && (XPStringUtility.IndexOf(sValues[0],
         ')') == -1) && (XPStringUtility.IndexOf(sValues[0], ';') == -1))
        {
          m_asModel.SetParentFormSubmitFunction(sValues[0]);
        } else
        {
          log.Error("Submit method cannot contain ();");
        }
      }
      // onclick call
      sValues = (String[]) arguments.GetElement(ONCLICK_CALL);
      if (sValues != null)
      {
        if ((XPStringUtility.IndexOf(sValues[0], '(') == -1) && (XPStringUtility.IndexOf(sValues[0],
         ')') == -1) && (XPStringUtility.IndexOf(sValues[0], ';') == -1))
        {
          if (GeneralHelpers.ContainsUnsafeCharacters(sValues[0], true))
          {
            log.Error("Onclick method cannot characters such as <,>,{,},;, etc... " + sValues[0]);
            } else
          {
            m_asModel.SetOnClickFunction(sValues[0]);
          }
        } else
        {
          log.Error("Onclick method cannot contain ();");
        }
      }
      // Allow empty tree submission
      sValues = (String[]) arguments.GetElement(ALLOW_EMPTY_SUBMISSION);
      if (sValues != null)
      {
        m_asModel.SetAllowEmptySubmission(XPConvert.ToBoolean(sValues[0]));
        }
      // Title for the tree pop-up
      sValues = (String[]) arguments.GetElement(TITLE);
      if (sValues != null)
      {
        m_asModel.SetTitle(sValues[0]);
      }
      // Action sub-title for the tree pop-up
      sValues = (String[]) arguments.GetElement(ACTIONSUBTITLE);
      if (sValues != null)
      {
        m_asModel.SetActionSubTitle(sValues[0]);
      }
      // help topic for the tree pop-up
      sValues = (String[]) arguments.GetElement(HELP_TOPIC);
      if (sValues != null)
      {
        m_asModel.SetHelpTopic(sValues[0]);
      }
      // Class ids to display
      sValues = (String[]) arguments.GetElement(CLASS_IDS_TO_DISPLAY);
      if (sValues != null)
      {
        m_asModel.SetClassIDsToDisplay(GeneralHelpers.ConvertToInts(sValues[0]));
        }
      // Object ids to select
      sValues = (String[]) arguments.GetElement(SELECTED_ITEMS);
      if (sValues != null)
      {
        // Check for referenced long strings
        String strSelectedObjects = TreeDataHelpers.UnReferenceQueryStringData(sValues,
         m_asOwner);
        if (strSelectedObjects == null)
        {
          // Use the normal data if there were no long strings referenced
          strSelectedObjects = sValues[0];
        }
        m_asModel.SetObjectIDsToSelect(GeneralHelpers.ConvertToInts(strSelectedObjects));
        }
      // Can only use one of these two arguments (HIDDEN_ITEMS & DISPLAYED_ITEMS)
      // Object ids to hide
      sValues = (String[]) arguments.GetElement(HIDDEN_ITEMS);
      if (sValues != null)
      {
        // Check for referenced long strings
        String strHiddenObjects = TreeDataHelpers.UnReferenceQueryStringData(sValues,
         m_asOwner);
        if (strHiddenObjects == null)
        {
          // Use the normal data if there were no long strings referenced
          strHiddenObjects = sValues[0];
        }
        m_asModel.SetObjectIDsToHide(GeneralHelpers.ConvertToInts(strHiddenObjects));
        } else
      {
        // Object ids to display
        sValues = (String[]) arguments.GetElement(DISPLAYED_ITEMS);
        if (sValues != null)
        {
          m_asModel.SetObjectIDsToDisplay(GeneralHelpers.ConvertToInts(sValues[0]));
          }
      }
      // Whether or not to display the root folder.  Default true.
      sValues = (String[]) arguments.GetElement(SHOW_ROOT);
      if (sValues != null)
      {
        m_asModel.SetShowRoot(XPConvert.ToBoolean(sValues[0]));
      }
      // default DP is TreeFrameDP
      return null;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IControl#Init(IModel,AActivitySpace)"></seealso>
    public virtual void Init(IModel model, AActivitySpace space)
    {
      m_asModel = (ITreeModel) model;
      m_asOwner = (ATreeAS) space;
    }
    // Abstract Methods
    
    /// <summary>
    /// This method is used by the ASManager to return new instances of managed
    /// objects.
    /// </summary>
    /// <returns>A new instance of the managed class (i.e. return new Foo();)</returns>
    public abstract Object Create();
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.IMVCObject#GetName()"></seealso>
    public abstract String GetName();
  }
}