using System;
namespace com.plumtree.uiinfrastructure.tree
{
  /// <summary>
  /// This interface contains all of the writable (set) methods that a tree model
  /// must implement.
  /// </summary>
  /// <author>Don Hayler</author>
  public interface ITreeModel : ITreeModelRO
  {
    
    /// <summary>
    /// Expand or collapse a node in the tree
    /// </summary>
    void ExpandNode(bool bExpand, int nNodeID);
    
    /// <summary>
    /// Accessor for managed series of unique tree ids.  All objects
    /// in a folder should have sequential tree ids.
    /// </summary>
    int GetNextUniqueID();
    
    /// <summary>
    /// This method initializes tree variables.  sub-classes can extend this
    /// method.
    /// </summary>
    void ResetTree();
    
    /// <summary>
    /// This resets the unique tree ids.  This should only be called
    /// when resetting the entire tree.
    /// </summary>
    void ResetUniqueIDs();
    
    /// <summary>
    /// Set the action sub-title for the tree popup
    /// I.E. select a user
    /// </summary>
    /// <param name="String">the action sub-title</param>
    void SetActionSubTitle(String strSubTitle);
    
    /// <summary>
    /// Set the help topic for the tree popup
    /// </summary>
    /// <param name="String">the help topic</param>
    void SetHelpTopic(String strHelpTopic);
    
    /// <summary>
    /// Allow users to click finish on trees without selecting
    /// anything.
    /// </summary>
    /// <param name="boolean">true implies allow</param>
    void SetAllowEmptySubmission(bool bAllow);
    
    /// <summary>
    /// This method will set the class IDs to be displayed.
    /// This must be run before SetObjectIDsToHide() and
    /// SetObjectIDsToSelect().
    /// </summary>
    /// <param name="int[]">nIDs the class IDs to show</param>
    void SetClassIDsToDisplay(int[] nIDs);
    
    /// <param name="bMultiSelect"></param>
    void SetMultipleSelect(bool bMultiSelect);
    
    /// <summary>
    /// This method will display only the given objects.
    /// Only this method or SetObjectIDsToHide() can be called,
    /// not both.
    /// </summary>
    /// <param name="int[]">nIDs classID1,objectID1,classID2,objectID2,etc...</param>
    void SetObjectIDsToDisplay(int[] nIDs);
    
    /// <summary>
    /// This method will hide the given objects.
    /// Only this method or SetObjectIDsToDisplay() can be called,
    /// not both.
    /// </summary>
    /// <param name="int[]">nIDs classID1,objectID1,classID2,objectID2,etc...</param>
    void SetObjectIDsToHide(int[] nIDs);
    
    /// <summary>
    /// This method will select the given objects.
    /// </summary>
    /// <param name="int[]">nIDs classID1,objectID1,classID2,objectID2,etc...</param>
    void SetObjectIDsToSelect(int[] nIDs);
    
    /// <summary>
    /// Removes an object from the selected items if it exists and is selected.
    /// The pair of classID,objectID uniquely identify each object.
    /// This method has no effect if the object does not exist or is not selected.
    /// This method does not throw exceptions but does log errors.
    /// </summary>
    /// <param name="classID">The classID from PT_CLASSIDS of the object to remove</param>
    /// <param name="objectID">The objectID of the object to remove</param>
    void RemoveFromSelection(int classID, int objectID);
    
    /// <param name="name"></param>
    void SetOnClickFunction(String name);
    
    /// <param name="name"></param>
    void SetParentFormName(String name);
    
    /// <param name="name"></param>
    void SetParentFormSpaceID(String name);
    
    /// <param name="name"></param>
    void SetParentFormSubmitFunction(String name);
    
    /// <param name="nSelectMode"></param>
    void SetSelectMode(int nSelectMode);
    
    /// <param name="nSelectOptions"></param>
    void SetSelectOptions(int nSelectOptions);
    
    /// <summary>
    /// This method sets whether or not to show the root folder.
    /// </summary>
    /// <param name="boolean">true implies show root (default)</param>
    void SetShowRoot(bool bShowRoot);
    
    /// <param name="nSubmitMode"></param>
    void SetSubmitMode(int nSubmitMode);
    
    /// <summary>
    /// Set the title for the tree popup
    /// </summary>
    /// <param name="String">the title</param>
    void SetTitle(String strTitle);
  }
}