using System;
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.htmlconstructs;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.tree
{
  /// <summary>
  /// This iterator is used for walking through the tree.  The
  /// GetBase* methods should always be valid, but the GetCurrent*
  /// methods are only valid after MoveToNextCompositeChild() has
  /// been called and has returned true.
  /// </summary>
  /// <author>Don Hayler</author>
  public class TreeCompositeIterator : ITreeIterator
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.tree.TreeCompositeIterator");
    private AActivitySpace m_asOwner;
    private HTMLImg m_DefaultClosedImage = null;
    private HTMLImg m_DefaultOpenImage = null;
    private IXPEnumerator m_CompositeEnumerator;
    private IXPEnumerator m_LeafEnumerator;
    private TreeCompositeNode m_CurrentComposite;
    private TreeCompositeNode m_Root;
    
    /// <summary>
    /// Constructor for iterator
    /// </summary>
    /// <param name="TreeCompositeNode">the root node of this iterator</param>
    /// <param name="AActivitySpace">
    /// asOwner the space that this tree is in
    /// (used to get default localized strings)
    /// </param>
    public TreeCompositeIterator(TreeCompositeNode root, AActivitySpace asOwner)
    {
      m_asOwner = asOwner;
      m_Root = root;
      m_LeafEnumerator = root.GetLeafChildIterator();
      m_CompositeEnumerator = root.GetCompositeChildIterator();
      m_CurrentComposite = null;
    }
    
    /// <summary>
    /// Private constructor so no one can use it.
    /// </summary>
    private TreeCompositeIterator()
    {
    }
    
    /// <returns></returns>
    public virtual int GetBaseClassID()
    {
      if (m_Root != null)
      {
        return m_Root.m_nClassID;
      }
      return -1;
    }
    
    /// <returns></returns>
    public virtual bool GetBaseDisabled()
    {
      if (m_Root != null)
      {
        return m_Root.m_bDisabled;
      }
      return false;
    }
    
    /// <returns></returns>
    public virtual bool GetBaseBrowsableOnly()
    {
      if (m_Root != null)
      {
        return m_Root.m_bBrowsableOnly;
      }
      return false;
    }
    
    /// <returns></returns>
    public virtual bool GetBaseExpanded()
    {
      if (m_Root != null)
      {
        return m_Root.m_bExpanded;
      }
      return false;
    }
    
    /// <returns></returns>
    public virtual bool GetBaseForceSelectable()
    {
      if (m_Root != null)
      {
        return m_Root.m_bForceSelectable;
      }
      return false;
    }
    
    /// <summary>
    /// This method returns the current image.  This will
    /// automatically handle open and closed images.
    /// </summary>
    /// <returns>HTMLImg the current image</returns>
    public virtual HTMLImg GetBaseImage()
    {
      HTMLImg result = null;
      if (m_Root != null)
      {
        if (m_Root.m_bExpanded)
        {
          result = m_Root.m_OpenImage;
        } else
        {
          result = m_Root.m_Image;
        }
      }
      if (result == null)
      {
        result = GetDefaultImage(m_Root.m_bExpanded);
      }
      return result;
    }
    
    /// <summary>
    /// This method returns an iterator to the leaf child nodes
    /// of the root node.
    /// </summary>
    /// <returns>iterator to the leaf children of the root node.</returns>
    public virtual TreeLeafIterator GetBaseLeafChildIterator()
    {
      return new TreeLeafIterator(this.m_LeafEnumerator, m_asOwner);
    }
    
    /// <summary>
    /// This method returns the number of leaf child nodes
    /// of the root node.
    /// </summary>
    /// <returns>the number of leaf children of the root node.</returns>
    public virtual int GetBaseLeafChildCount()
    {
      if (m_Root != null)
      {
        return m_Root.GetLeafChildCount();
      }
      return 0;
    }
    
    /// <returns></returns>
    public virtual String GetBaseLocation()
    {
      if (m_Root != null)
      {
        return m_Root.m_strLocation;
      }
      return null;
    }
    
    /// <returns></returns>
    public virtual String GetBaseEncodedName()
    {
      if (m_Root != null)
      {
        return m_Root.GetEncodedName();
      }
      return null;
    }
    
    /// <returns></returns>
    public virtual String GetBaseDescription()
    {
      if (m_Root != null)
      {
        return m_Root.GetDescription();
      }
      return null;
    }
    
    /// <returns></returns>
    public virtual int GetBaseObjectID()
    {
      if (m_Root != null)
      {
        return m_Root.m_nObjectID;
      }
      return -1;
    }
    
    /// <returns></returns>
    public virtual bool GetBaseSelected()
    {
      if (m_Root != null)
      {
        return m_Root.m_bSelected;
      }
      return false;
    }
    
    /// <returns></returns>
    public virtual int GetBaseTreeNodeID()
    {
      if (m_Root != null)
      {
        return m_Root.m_nTreeNodeID;
      }
      return -1;
    }
    
    /// <returns></returns>
    public virtual int GetCurrentClassID()
    {
      if (m_CurrentComposite != null)
      {
        return m_CurrentComposite.m_nClassID;
      }
      return -1;
    }
    
    /// <returns></returns>
    public virtual TreeCompositeIterator GetCurrentCompositeChildIterator()
    {
      if (m_CurrentComposite != null)
      {
        return new TreeCompositeIterator(m_CurrentComposite, m_asOwner);
      }
      return null;
    }
    
    /// <returns></returns>
    public virtual bool GetCurrentDisabled()
    {
      if (m_CurrentComposite != null)
      {
        return m_CurrentComposite.m_bDisabled;
      }
      return false;
    }
    
    /// <returns></returns>
    public virtual bool GetCurrentExpanded()
    {
      if (m_CurrentComposite != null)
      {
        return m_CurrentComposite.m_bExpanded;
      }
      return false;
    }
    
    /// <returns></returns>
    public virtual bool GetCurrentForceSelectable()
    {
      if (m_CurrentComposite != null)
      {
        return m_CurrentComposite.m_bForceSelectable;
      }
      return false;
    }
    
    /// <summary>
    /// This method returns the current image.  This will
    /// automatically handle open and closed images.
    /// </summary>
    /// <returns>HTMLImg the current image</returns>
    public virtual HTMLImg GetCurrentImage()
    {
      HTMLImg result = null;
      if (m_CurrentComposite != null)
      {
        if (m_CurrentComposite.m_bExpanded)
        {
          result = m_CurrentComposite.m_OpenImage;
        } else
        {
          result = m_CurrentComposite.m_Image;
        }
      }
      if (result == null)
      {
        result = GetDefaultImage(m_CurrentComposite.m_bExpanded);
      }
      return result;
    }
    
    /// <returns></returns>
    public virtual String GetCurrentLocation()
    {
      if (m_CurrentComposite != null)
      {
        return m_CurrentComposite.m_strLocation;
      }
      return null;
    }
    
    /// <returns></returns>
    public virtual String GetCurrentEncodedName()
    {
      if (m_CurrentComposite != null)
      {
        return m_CurrentComposite.GetEncodedName();
      }
      return null;
    }
    
    /// <returns></returns>
    public virtual int GetCurrentObjectID()
    {
      if (m_CurrentComposite != null)
      {
        return m_CurrentComposite.m_nObjectID;
      }
      return -1;
    }
    
    /// <returns></returns>
    public virtual bool GetCurrentSelected()
    {
      if (m_CurrentComposite != null)
      {
        return m_CurrentComposite.m_bSelected;
      }
      return false;
    }
    
    /// <returns></returns>
    public virtual int GetCurrentTreeNodeID()
    {
      if (m_CurrentComposite != null)
      {
        return m_CurrentComposite.m_nTreeNodeID;
      }
      return -1;
    }
    
    /// <summary>
    /// Moves to the next composite child.
    /// </summary>
    /// <returns>boolean false if there are no more composite children.</returns>
    public virtual bool MoveToNextCompositeChild()
    {
      if (m_CompositeEnumerator.MoveNext())
      {
        m_CurrentComposite = (TreeCompositeNode) m_CompositeEnumerator.GetCurrent();
        return true;
      }
      m_CurrentComposite = null;
      return false;
    }
    /////////////////
    // Helper methods
    /////////////////
    private HTMLImg GetDefaultImage(bool bExpanded)
    {
      HTMLImg result = null;
      if (bExpanded)
      {
        if (m_DefaultOpenImage == null)
        {
          try
          {
            m_DefaultOpenImage = (HTMLImg) UIConstructFactory.GetConstruct(PTImageType.ICON_FOLDER_OPEN,
             true, m_asOwner).GetHTMLElement();
            m_DefaultOpenImage.SetBorder("0");
            m_DefaultOpenImage.SetAlt(m_asOwner.GetString(78, ResourceConstants.INFRASTRUCTURE));
            }
          catch (Exception e)
          {
            log.Error(e, "Unable to construct generic open folder image.");
            m_DefaultOpenImage = null;
          }
        }
        result = m_DefaultOpenImage;
      } else
      {
        if (m_DefaultClosedImage == null)
        {
          try
          {
            m_DefaultClosedImage = (HTMLImg) UIConstructFactory.GetConstruct(PTImageType.ICON_FOLDER,
             true, m_asOwner).GetHTMLElement();
            m_DefaultClosedImage.SetBorder("0");
            m_DefaultClosedImage.SetAlt(m_asOwner.GetString(78, ResourceConstants.INFRASTRUCTURE));
            }
          catch (Exception e)
          {
            log.Error(e, "Unable to construct generic closed folder image.");
            m_DefaultClosedImage = null;
          }
        }
        result = m_DefaultClosedImage;
      }
      return result;
    }
  }
}