using System;
using com.plumtree.openfoundation.util;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.tree
{
  /// <summary>
  /// This class represents composite nodes in the tree such
  /// as folders, or browseable groups.
  /// </summary>
  /// <author>Don Hayler</author>
  public class TreeCompositeNode : TreeLeafNode
  {
    
    /// <summary>
    /// The image when the container is open 
    /// </summary>
    public HTMLImg m_OpenImage = null;
    
    /// <summary>
    /// Have the children of this node been loaded? 
    /// </summary>
    public bool m_bChildrenLoaded = false;
    
    /// <summary>
    /// Neither browsable nor selectable 
    /// </summary>
    public bool m_bDisabled = false;
    
    /// <summary>
    /// true implies browsable but not selectable 
    /// </summary>
    public bool m_bBrowsableOnly = false;
    
    /// <summary>
    /// Is this folder expanded 
    /// </summary>
    public bool m_bExpanded = false;
    
    /// <summary>
    /// children arrays 
    /// </summary>
    private XPArrayList m_CompositeChildren = new XPArrayList();
    private XPArrayList m_LeafChildren = new XPArrayList();
    
    /// <summary>
    /// Creates a new TreeCompositeNode object.
    /// All objects in a folder should have sequential tree ids.
    /// </summary>
    /// <param name="int">internal ID of this node</param>
    public TreeCompositeNode(int nTreeNodeID) : base(nTreeNodeID)
    {
    }
    // This should never be called.
    private TreeCompositeNode() : base(-1)
    {
    }
    
    /// <param name="child"></param>
    public virtual void AddCompositeChild(TreeCompositeNode child)
    {
      if (child != null)
      {
        m_CompositeChildren.Add(child);
      }
    }
    
    /// <param name="child"></param>
    public virtual void RemoveCompositeChild(TreeCompositeNode child)
    {
      if (child != null)
      {
        m_CompositeChildren.Remove(child);
      }
    }
    
    /// <param name="child"></param>
    public virtual void AddLeafChild(TreeLeafNode child)
    {
      if (child != null)
      {
        m_LeafChildren.Add(child);
      }
    }
    
    /// <param name="child"></param>
    public virtual void RemoveLeafChild(TreeLeafNode child)
    {
      if (child != null)
      {
        m_LeafChildren.Remove(child);
      }
    }
    // This method recursively clears the selected bits on all nodes and leaves
    // underneath this node
    public virtual void ClearSelected(bool bVisibleOnly, bool bRecursive)
    {
      m_bSelected = false;
      // Only clear the children if this node is expanded or we are clearing all children
      if (m_bExpanded || !bVisibleOnly)
      {
        IXPEnumerator enumerator = m_CompositeChildren.GetEnumerator();
        while (enumerator.MoveNext())
        {
          if (bRecursive)
          {
            ((TreeCompositeNode) enumerator.GetCurrent()).ClearSelected(bVisibleOnly,
             bRecursive);
          } else
          {
            ((TreeCompositeNode) enumerator.GetCurrent()).m_bSelected = false;
            }
        }
        enumerator = m_LeafChildren.GetEnumerator();
        while (enumerator.MoveNext())
        {
          ((TreeLeafNode) enumerator.GetCurrent()).m_bSelected = false;
        }
      }
    }
    
    /// <returns></returns>
    public virtual IXPEnumerator GetCompositeChildIterator()
    {
      return m_CompositeChildren.GetEnumerator();
    }
    
    /// <returns></returns>
    public virtual IXPEnumerator GetLeafChildIterator()
    {
      return m_LeafChildren.GetEnumerator();
    }
    
    /// <summary>
    /// This method calculates how many leaf children this node contains.
    /// </summary>
    /// <returns>the number of leaf children in this node.</returns>
    public virtual int GetLeafChildCount()
    {
      return m_LeafChildren.GetSize();
    }
    
    /// <param name="nNodeID"></param>
    /// <returns></returns>
    public virtual TreeLeafNode GetNode(int nNodeID)
    {
      // Search this node
      if (nNodeID == m_nTreeNodeID)
      {
        return this;
      }
      // Search child leaf nodes
      IXPEnumerator enumerator = m_LeafChildren.GetEnumerator();
      while (enumerator.MoveNext())
      {
        if (((TreeLeafNode) enumerator.GetCurrent()).m_nTreeNodeID == nNodeID)
        {
          return (TreeLeafNode) enumerator.GetCurrent();
        }
      }
      // Search child composite nodes
      TreeLeafNode result = null;
      enumerator = m_CompositeChildren.GetEnumerator();
      while ((enumerator.MoveNext()) && (result == null))
      {
        result = ((TreeCompositeNode) enumerator.GetCurrent()).GetNode(nNodeID);
        }
      return result;
    }
    
    /// <param name="nClassID"></param>
    /// <param name="nObjectID"></param>
    /// <returns></returns>
    public virtual TreeLeafNode GetNode(int nClassID, int nObjectID)
    {
      // Search this node
      if ((nClassID == m_nClassID) && (nObjectID == m_nObjectID))
      {
        return this;
      }
      TreeLeafNode current;
      // Search child leaf nodes
      IXPEnumerator enumerator = m_LeafChildren.GetEnumerator();
      while (enumerator.MoveNext())
      {
        current = (TreeLeafNode) enumerator.GetCurrent();
        if ((current.m_nClassID == nClassID) && (current.m_nObjectID == nObjectID))
        {
          return current;
        }
      }
      // Search child composite nodes
      enumerator = m_CompositeChildren.GetEnumerator();
      while (enumerator.MoveNext())
      {
        current = ((TreeCompositeNode) enumerator.GetCurrent()).GetNode(nClassID,
         nObjectID);
        if (current != null)
        {
          return current;
        }
      }
      return null;
    }
    
    /// <summary>
    /// Note: This function compares the location to
    /// get the node
    /// </summary>
    /// <param name="nClassID"></param>
    /// <param name="nObjectID"></param>
    /// <returns></returns>
    public virtual TreeLeafNode GetNode(String strLocation)
    {
      // Search this node
      if (m_strLocation.Equals(strLocation))
      {
        return this;
      }
      TreeLeafNode current;
      // Search child leaf nodes
      IXPEnumerator enumerator = m_LeafChildren.GetEnumerator();
      while (enumerator.MoveNext())
      {
        current = (TreeLeafNode) enumerator.GetCurrent();
        if (current.m_strLocation.Equals(strLocation))
        {
          return current;
        }
      }
      // Search child composite nodes
      enumerator = m_CompositeChildren.GetEnumerator();
      while (enumerator.MoveNext())
      {
        current = ((TreeCompositeNode) enumerator.GetCurrent()).GetNode(strLocation);
        if (current != null)
        {
          return current;
        }
      }
      return null;
    }
    
    /// <summary>
    /// This method searches the entire tree for nodes with the classID
    /// and objectID and sets them to selected.  This should only be used
    /// for multiply instanced nodes.
    /// </summary>
    public virtual void SetSelected(int nClassID, int nObjectID)
    {
      // Search this node
      if ((m_nClassID == nClassID) && (m_nObjectID == nObjectID))
      {
        m_bSelected = true;
      }
      // Set child leaf nodes
      IXPEnumerator enumerator = m_LeafChildren.GetEnumerator();
      TreeLeafNode curLeafNode;
      while (enumerator.MoveNext())
      {
        curLeafNode = (TreeLeafNode) enumerator.GetCurrent();
        if ((curLeafNode.m_nClassID == nClassID) && (curLeafNode.m_nObjectID == nObjectID))
        {
          curLeafNode.m_bSelected = true;
        }
      }
      // Set child composite nodes
      enumerator = m_CompositeChildren.GetEnumerator();
      while (enumerator.MoveNext())
      {
        ((TreeCompositeNode) enumerator.GetCurrent()).SetSelected(nClassID,
         nObjectID);
      }
    }
  }
}