using System;
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.activityspace;
namespace com.plumtree.uiinfrastructure.tree
{
  /// <summary>
  /// This class provides static methods to parse the data returned by the tree
  /// for selected items.
  /// </summary>
  /// <author>Don Hayler</author>
  public class TreeDataHelpers
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.tree.TreeDataHelpers");
    // Internal constants (the positions of the data in the string sections
    private const int NAME = 0;
    private const int OBJECTID = 0;
    private const int CLASSID = 0;
    
    /// <summary>
    /// Parses out the Class IDs of the selected objects from the data
    /// string returned by a Tree Control.
    /// </summary>
    /// <param name="strTreeData">data string from a Tree Control</param>
    /// <returns>int[] an array of Object IDs of the selected Objects.</returns>
    public static int[] ParseTreeDataGetClassID(String strTreeData)
    {
      // Check basic validity of data
      XPStringTokenizer stTreeData = CheckTreeData(strTreeData);
      if (stTreeData == null)
      {
        return null;
      }
      int[] intItems = new int[stTreeData.GetTokenCount() / 3];
      try
      {
        for (int nTokenCount = 0;stTreeData.HasMoreTokens();nTokenCount++)
        {
          if ((nTokenCount % 3) == CLASSID)
          {
            intItems[nTokenCount / 3] = XPConvert.ToInteger(stTreeData.GetNextToken());
            } else
          {
            stTreeData.GetNextToken();
          }
        }
      }
      catch (Exception e)
      {
        log.Error("ParseTreeDataGetClassID failed to parse data: " + strTreeData);
        return null;
      }
      return intItems;
    }
    
    /// <summary>
    /// Parses out the Name of an object from the data string returned by a Tree Control
    /// </summary>
    /// <param name="strTreeData">data string from a Tree Control</param>
    /// <returns>String[] an array of Object Names of the selected Objects.</returns>
    public static String[] ParseTreeDataGetName(String strTreeData)
    {
      // Check basic validity of data
      XPStringTokenizer stTreeData = CheckTreeData(strTreeData);
      if (stTreeData == null)
      {
        return null;
      }
      String[] strItems = new String[stTreeData.GetTokenCount() / 3];
      try
      {
        for (int nTokenCount = 0;stTreeData.HasMoreTokens();nTokenCount++)
        {
          if ((nTokenCount % 3) == NAME)
          {
            strItems[nTokenCount / 3] = stTreeData.GetNextToken();
          } else
          {
            stTreeData.GetNextToken();
          }
        }
      }
      catch (Exception e)
      {
        log.Error("ParseTreeDataGetName failed to parse data: " + strTreeData);
        return null;
      }
      return strItems;
    }
    
    /// <summary>
    /// Parses out the Object IDs of the selected objects from the data
    /// string returned by a Tree Control.
    /// </summary>
    /// <param name="strTreeData">data string from a Tree Control</param>
    /// <returns>int[] an array of Object IDs of the selected Objects.</returns>
    public static int[] ParseTreeDataGetObjectID(String strTreeData)
    {
      // Check basic validity of data
      XPStringTokenizer stTreeData = CheckTreeData(strTreeData);
      if (stTreeData == null)
      {
        return null;
      }
      int[] intItems = new int[stTreeData.GetTokenCount() / 3];
      try
      {
        for (int nTokenCount = 0;stTreeData.HasMoreTokens();nTokenCount++)
        {
          if ((nTokenCount % 3) == OBJECTID)
          {
            intItems[nTokenCount / 3] = XPConvert.ToInteger(stTreeData.GetNextToken());
            } else
          {
            stTreeData.GetNextToken();
          }
        }
      }
      catch (Exception e)
      {
        log.Error("ParseTreeDataGetObjectID failed to parse data: " + strTreeData);
        return null;
      }
      return intItems;
    }
    
    /// <summary>
    /// This method checks that the data is in a valid format
    /// </summary>
    /// <param name="strTreeData">the data to check.</param>
    /// <returns>
    /// XPStringTokenizer the tokenizer to use to parse the data.
    /// null implies invalid data.
    /// </returns>
    private static XPStringTokenizer CheckTreeData(String strTreeData)
    {
      if (strTreeData == null)
      {
        log.Error("ParseTreeDataGetName passed null data.");
        return null;
      }
      XPStringTokenizer result = new XPStringTokenizer(strTreeData, ",");
      int nMaxTokens = result.GetTokenCount();
      if ((nMaxTokens < 3) || ((nMaxTokens % 3) != 0))
      {
        log.Error("ParseTreeDataGetName passed data with incorrect number of sections: " + nMaxTokens);
        return null;
      }
      return result;
    }
    
    /// <summary>
    /// This helper method checks if a query string parameter needs
    /// to be unreferenced.
    /// This is currently only used for selected and hidden items.
    /// </summary>
    /// <param name="String[]">the query string parameter</param>
    /// <param name="AActivitySpace">asOwner the current space</param>
    /// <returns>
    /// the referenced string data or null, if no data was
    /// referenced.
    /// </returns>
    /// <author>olegs</author>
    public static String UnReferenceQueryStringData(String[] sValues, AActivitySpace asOwner)
    {
      if ((sValues[0] != null) && (sValues[0].Length > 0) && (XPStringUtility.CharAt(sValues[0],
       0) == '@'))
      {
        // Might be a reference to a long string stored on the parent space.
        // Attempt to unref it
        AActivitySpace asParent = asOwner.GetParentSpace();
        if (asParent != null)
        {
          String strUnReferencedValue = asParent.unrefString(XPStringUtility.Substring(sValues[0],
           1));
          if (strUnReferencedValue != null)
          {
            // If unref succeeded, replace the query string argument
            // with the unreferenced version.
            return sValues[0] = strUnReferencedValue;
          }
        }
      }
      return null;
    }
  }
}
//	/**
//	 * This method takes a String with numbers separated by ','s and splits it into
//	 * an int array.
//	 *
//	 * @param data
//	 *
//	 * @return
//	 */
//	private static int[] GetInts(String data)
//	{
//		if (data == null)
//		{
//			return null;
//		}
//
//		XPStringTokenizer st = new XPStringTokenizer(data, ",");
//		int[] result		 = new int[st.GetTokenCount()];
//
//		try
//		{
//			for (int i = 0; st.HasMoreTokens(); i++)
//			{
//				result[i] = XPConvert.ToInteger(st.GetNextToken());
//			}
//		}
//		catch (Exception e)
//		{
//			
//				log.Warn(//					"Bad data - expected numeric data separated by ','s, got : " +
//					data);
//			
//		}
//
//		return result;
//	}
