using System;
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.htmlconstructs;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.tree
{
  /// <summary>
  /// This iterator is used for walking through the tree.  The
  /// GetGetCurrent* methods are only valid after MoveToNextChild()
  /// has been called and has returned true.
  /// </summary>
  /// <author>Don Hayler</author>
  public class TreeLeafIterator : ITreeIterator
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.tree.TreeLeafIterator");
    private AActivitySpace m_asOwner;
    private IXPEnumerator m_ChildEnumerator;
    private TreeLeafNode m_CurrentChild;
    
    /// <summary>
    /// Constructor for iterator
    /// </summary>
    /// <param name="IXPEnumerator">the child nodes for this iterator</param>
    /// <param name="AActivitySpace">
    /// asOwner the space that this tree is in
    /// (used to get default localized strings)
    /// </param>
    public TreeLeafIterator(IXPEnumerator children, AActivitySpace asOwner)
    {
      m_asOwner = asOwner;
      m_ChildEnumerator = children;
      m_CurrentChild = null;
    }
    
    /// <summary>
    /// Private constructor so no one can use it.
    /// </summary>
    private TreeLeafIterator()
    {
    }
    
    /// <returns></returns>
    public virtual int GetCurrentClassID()
    {
      if (m_CurrentChild != null)
      {
        return m_CurrentChild.m_nClassID;
      }
      return -1;
    }
    
    /// <returns></returns>
    public virtual bool GetCurrentForceSelectable()
    {
      if (m_CurrentChild != null)
      {
        return m_CurrentChild.m_bForceSelectable;
      }
      return false;
    }
    
    /// <summary>
    /// This method returns the current image.  This will
    /// automatically handle open and closed images.
    /// </summary>
    /// <returns>HTMLImg the current image</returns>
    public virtual HTMLImg GetCurrentImage()
    {
      HTMLImg result = null;
      if (m_CurrentChild != null)
      {
        result = m_CurrentChild.m_Image;
      }
      if (result == null)
      {
        try
        {
          result = (HTMLImg) UIConstructFactory.GetConstruct(PTImageType.ICON_DOCUMENT,
           true, m_asOwner).GetHTMLElement();
          result.SetBorder("0");
          result.SetAlt(m_asOwner.GetString(79, ResourceConstants.INFRASTRUCTURE));
          }
        catch (Exception e)
        {
          log.Error(e, "Unable to construct generic image.");
          return null;
        }
      }
      return result;
    }
    
    /// <returns></returns>
    public virtual String GetCurrentLocation()
    {
      if (m_CurrentChild != null)
      {
        return m_CurrentChild.m_strLocation;
      }
      return null;
    }
    
    /// <returns></returns>
    public virtual String GetCurrentEncodedName()
    {
      if (m_CurrentChild != null)
      {
        return m_CurrentChild.GetEncodedName();
      }
      return null;
    }
    
    /// <returns></returns>
    public virtual String GetCurrentDescription()
    {
      if (m_CurrentChild != null)
      {
        return m_CurrentChild.GetDescription();
      }
      return null;
    }
    
    /// <returns></returns>
    public virtual int GetCurrentObjectID()
    {
      if (m_CurrentChild != null)
      {
        return m_CurrentChild.m_nObjectID;
      }
      return -1;
    }
    
    /// <returns></returns>
    public virtual bool GetCurrentSelected()
    {
      if (m_CurrentChild != null)
      {
        return m_CurrentChild.m_bSelected;
      }
      return false;
    }
    
    /// <returns></returns>
    public virtual int GetCurrentTreeNodeID()
    {
      if (m_CurrentChild != null)
      {
        return m_CurrentChild.m_nTreeNodeID;
      }
      return -1;
    }
    
    /// <summary>
    /// Moves to the next child.
    /// </summary>
    /// <returns>boolean false if there are no more children.</returns>
    public virtual bool MoveToNextChild()
    {
      if (m_ChildEnumerator.MoveNext())
      {
        m_CurrentChild = (TreeLeafNode) m_ChildEnumerator.GetCurrent();
        return true;
      }
      m_CurrentChild = null;
      return false;
    }
  }
}