using System;
using com.plumtree.openfoundation.web;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.tree
{
  /// <summary>
  /// This class represents a node in the tree.
  /// </summary>
  /// <author>Don Hayler</author>
  public class TreeLeafNode
  {
    
    /// <summary>
    /// unique internal ID for this node 
    /// </summary>
    public int m_nTreeNodeID;
    
    /// <summary>
    /// The default image for this object.  For a composite node, this will be
    /// the closed image (the composite node will store the open image).
    /// </summary>
    public HTMLImg m_Image = null;
    public String m_strLocation = null;
    
    /// <summary>
    /// The name is private so we can truncate it properly. 
    /// </summary>
    private String m_strName = null;
    
    /// <summary>
    /// The description is private so we can truncate it properly. 
    /// </summary>
    private String m_strDescription = null;
    
    /// <summary>
    /// Whether or not to override the normal selectable logic and
    /// force this node to be selectable.
    /// true implies this node should be selectable no matter what.
    /// </summary>
    public bool m_bForceSelectable = false;
    
    /// <summary>
    /// Whether or not this node can be simultaneously located
    /// in multiple places.  true implies there can be more
    /// than one instance of the object this node points to in
    /// the same tree.
    /// N.B.  Multiply instanced items must use unique class ID /
    /// object ID pairs (per item, not per instance).
    /// </summary>
    public bool m_bMultiplyInstanced = false;
    
    /// <summary>
    /// Whether or not this node is selected. 
    /// </summary>
    public bool m_bSelected = false;
    public int m_nClassID = -1;
    public int m_nObjectID = -1;
    
    /// <summary>
    /// Creates a new TreeLeafNode object.
    /// All objects in a folder should have sequential tree ids.
    /// </summary>
    /// <param name="int">internal ID of this node</param>
    public TreeLeafNode(int nTreeNodeID)
    {
      m_nTreeNodeID = nTreeNodeID;
    }
    
    /// <summary>
    /// Creates a new TreeLeafNode object.
    /// </summary>
    /// <param name="node"></param>
    public TreeLeafNode(TreeLeafNode node)
    {
      m_nTreeNodeID = node.m_nTreeNodeID;
      m_strName = node.m_strName;
      m_strDescription = node.m_strDescription;
      m_nObjectID = node.m_nObjectID;
      m_nClassID = node.m_nClassID;
      m_strLocation = node.m_strLocation;
      m_Image = node.m_Image;
      m_bSelected = node.m_bSelected;
      m_bMultiplyInstanced = node.m_bMultiplyInstanced;
      m_bForceSelectable = node.m_bForceSelectable;
    }
    // No default constructor - this should never be called
    private TreeLeafNode()
    {
      m_nTreeNodeID = -1;
    }
    
    /// <summary>
    /// This method returns the name of the node (truncated to
    /// 50 characters).
    /// This method unencodes the stored
    /// encoded name, so it should only be used when retrieving
    /// the final tree selections.  Tracing should use the
    /// GetEncodedName method.
    /// </summary>
    /// <returns>String the name of the node.</returns>
    public virtual String GetName()
    {
      return XPHttpUtility.HtmlDecode(m_strName);
    }
    
    /// <summary>
    /// This method returns the name of the node (truncated to
    /// 50 characters and HTMLEncoded).
    /// The name is stored truncated and encoded, so this method
    /// is fast.  Tracing should use this method, even though it
    /// does not need to encode the name.
    /// </summary>
    /// <returns>String the name of the node.</returns>
    public virtual String GetEncodedName()
    {
      return m_strName;
    }
    
    /// <summary>
    /// This method sets the name of the node, truncates it to
    /// 50 characters, and HTMLEncodes it.  This also sets the
    /// description to the full name, if there is no description
    /// set.
    /// </summary>
    /// <param name="String">the name of the node.</param>
    public virtual void SetName(String strName)
    {
      // true means only truncate if there is a word longer
      // than the max length.
      m_strName = GeneralHelpers.TruncateLongString(strName, ATreeAS.MAX_DISPLAYED_NAME_LENGTH,
       true);
      m_strName = XPHttpUtility.HtmlEncode(m_strName);
      if ((m_strDescription == null) || ("".Equals(m_strDescription)))
      {
        SetDescription(strName);
      }
    }
    
    /// <summary>
    /// This method sets the name and description
    /// of the node by copying the truncated and
    /// encoded versions from the node.
    /// </summary>
    /// <param name="TreeLeafNode">the node to take the info from.</param>
    public virtual void SetNameAndDescription(TreeLeafNode node)
    {
      m_strName = node.m_strName;
      m_strDescription = node.m_strDescription;
    }
    
    /// <summary>
    /// This method returns the description of the node (HTMLEncoded).
    /// If there is no description, it returns the name.
    /// </summary>
    /// <returns>String the description of the node.</returns>
    public virtual String GetDescription()
    {
      if (m_strDescription == null)
      {
        // Return the name if there is no description
        return GetEncodedName();
      }
      return m_strDescription;
    }
    
    /// <summary>
    /// This method sets the description of the node and HTMLEncodes it.
    /// This will ignore the description if it is null or the empty string.
    /// </summary>
    /// <param name="String">the description of the node.</param>
    public virtual void SetDescription(String strDescription)
    {
      if ((strDescription != null) && (!"".Equals(strDescription)))
      {
        m_strDescription = XPHttpUtility.HtmlEncode(strDescription);
      }
    }
  }
}