using System;
using com.plumtree.openlog;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.uiinfrastructure.htmlconstructs;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.tree
{
  /// <summary>
  /// This view displays a tree (TreeCompositeNode) generated by an ITreeModel.
  /// </summary>
  /// <author>Don Hayler</author>
  public class TreeView : IView
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.tree.TreeView");
    
    /// <summary>
    /// name of the tree 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "Tree";
    
    /// <summary>
    /// HTML Object names. 
    /// </summary>
    public const String HTMLBUTTON_TREE_FINISH = "in_bu_Done";
    public const String HTMLBUTTON_TREE_CANCEL = "in_bu_Cancel";
    public const String HTMLBUTTON_RADIO = "in_ra_SelectedItem";
    public const String HTMLBUTTON_CHECKBOX = "in_cb_SelectedItems";
    public const String HTMLHIDDEN_SELECTEDIDS = "in_hi_SelectedIDs";
    public const String HTMLINPUT_CHECK_ALL = "in_cb_checkAll";
    public const String JAVASCRIPT_SUBMIT_TO_PARENT = "submitToParent";
    // Set this JS boolean to true if you want to shortcut the form submission check
    public const String JAVASCRIPT_SELECTED_ITEMS = "bSelectedItems";
    // Variable to mark when the page has been loaded.
    public const String JAVASCRIPT_PAGE_LOADED = "bPageLoaded";
    public const String JAVASCRIPT_LIST_CHECK_ALL = "listCheckAll";
    public const String JAVASCRIPT_TREE_CHECK_ALL = "treeCheckAll";
    private const String JAVASCRIPT_EXPAND = "expandNode";
    private const String JAVASCRIPT_TREE_ONLOAD = "treeOnLoad";
    private const String NODE_ID_PREFIX = "node";
    private const int INDENTWIDTH = 26;
    private const int LEAFINDENTWIDTH = 26;
    private const int MIN_CHECK_ALL_SIZE = 15;
    private const int MAX_FOLDER_OBJECTS = 1000;
    private AActivitySpace m_asOwner;
    
    /// <summary>
    /// internal data 
    /// </summary>
    private ITreeModelRO m_asModel;
    
    /// <summary>
    /// helper variable for Display
    /// comma separated list of treenodeids to be put in a hidden input
    /// (although search will return classid,objid,...)
    /// </summary>
    private XPStringBuilder m_sbSelectedItems;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public virtual Object Create()
    {
      return new TreeView();
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#Display()"></seealso>
    public virtual HTMLElement Display()
    {
      HTMLElement result = new HTMLElementCollection();
      TreeCompositeIterator myRoot = m_asModel.GetRootTreeIterator();
      // Store the previously selected items so we can check if the user
      // unchecked an item.
      m_sbSelectedItems = new XPStringBuilder(40);
      HTMLInput myInput;
      try
      {
        // Hidden inputs
        // Expand input
        myInput = new HTMLInput(HTMLInputTypes.HIDDEN, TreeRepostControl.m_strEXPAND,
         "");
        result.AddInnerHTMLElement(myInput);
        // FolderID input
        myInput = new HTMLInput(HTMLInputTypes.HIDDEN, TreeRepostControl.m_strFOLDERID,
         "");
        result.AddInnerHTMLElement(myInput);
        if (m_asModel.GetDisplayingFlatList() && m_asModel.GetMultipleSelect() && !m_asModel.GetTreeIsEmpty())
        {
          HTMLTable innerTable = new HTMLTable();
          result.AddInnerHTMLElement(innerTable);
          innerTable.SetBorder(CommonHTMLStrings.ZERO);
          HTMLTableRow myRow = new HTMLTableRow();
          innerTable.AddInnerHTMLElement(myRow);
          HTMLTableCell myCell = new HTMLTableCell();
          myRow.AddInnerHTMLElement(myCell);
          //myCell.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
          myCell.SetNoWrap(true);
          myCell.SetVAlign(CommonHTMLStrings.TOP);
          // Add select all check box (check all)
          myInput = new HTMLInput(HTMLInputTypes.CHECKBOX, /* _ui_CheckAll */
          HTMLINPUT_CHECK_ALL, m_asOwner.GetString(85, ResourceConstants.INFRASTRUCTURE));
          myCell.AddInnerHTMLElement(myInput);
          myInput.SetOnClick(JAVASCRIPT_LIST_CHECK_ALL + "(this, \'" + HTMLBUTTON_CHECKBOX + "\'); return true;");
          myInput.SetChecked(m_asModel.GetAllChecked());
          // Add indentation spacer image (1 for the icon that this doesn't have)
          myCell = new HTMLTableCell();
          myRow.AddInnerHTMLElement(myCell);
          HTMLSpan mySpan = new HTMLSpan();
          mySpan.SetStyleClass(PTStyleClass.DIALOGUE_TEXT);
          mySpan.AddInnerHTMLString(m_asOwner.GetString(85, ResourceConstants.INFRASTRUCTURE));
          myCell.AddInnerHTMLElement(mySpan);
        }
        if (m_asModel.GetTreeIsEmpty())
        {
          // Display 'no items to select' message
          HTMLSpan mySpan = new HTMLSpan();
          mySpan.SetStyleClass(PTStyleClass.DIALOGUE_TEXT);
          mySpan.AddInnerHTMLString(m_asOwner.GetString(100, 
          // no items to select
          ResourceConstants.INFRASTRUCTURE));
          result.AddInnerHTMLElement(mySpan);
        } else
        {
          if (m_asModel.GetShowRoot())
          {
            // Display the root node
            DisplayCompositeNodes(result, myRoot, 0);
          } else
          {
            // If we are not supposed to show the root folder, show it's children
            if (myRoot.GetBaseExpanded())
            {
              log.Debug("In node " + myRoot.GetBaseEncodedName() + " displaying composite nodes.");
              // Loop through composite children
              while (myRoot.MoveToNextCompositeChild())
              {
                DisplayCompositeNodes(result, myRoot.GetCurrentCompositeChildIterator(),
                 0);
              }
              log.Debug("In node " + myRoot.GetBaseEncodedName() + " displaying leaf nodes.");
              // Display the leaf nodes of this composite node
              DisplayLeafNodes(result, myRoot.GetBaseLeafChildIterator(),
               myRoot.GetBaseLeafChildCount(), 0);
            }
          }
        }
        // Hidden input for already selected items.
        myInput = new HTMLInput(HTMLInputTypes.HIDDEN, HTMLHIDDEN_SELECTEDIDS,
         "");
        myInput.SetValue(m_sbSelectedItems.ToString());
        result.AddInnerHTMLElement(myInput);
      }
      catch (Exception e)
      {
        log.Error(e, "Display: The following error occured when trying to display the HTML.");
        }
      // Clear out the sb
      m_sbSelectedItems = null;
      return result;
    }
    /////////////////
    // Helper methods
    /////////////////
    public virtual void DisplayFormValidationFunction(XPStringBuilder sb)
    {
      // Select submit function (multi and single)
      // Different cases for a single radio/checkbox button vs. multiple
      // radio/checkbox buttons:
      // - single is just an object
      // - multiple is an array of objects
      sb.Append(JAVASCRIPT_SELECTED_ITEMS).Append(" = false;\n");
      sb.Append("\n");
      sb.Append("function ").Append(JAVASCRIPT_SUBMIT_TO_PARENT).Append("()\n");
      sb.Append("{\n");
      if (m_asModel.GetAllowEmptySubmission())
      {
        // don't check for selections
        sb.Append("  return true;\n");
      } else
      {
        // Check for currently selected items
        sb.Append("  if ((").Append(JAVASCRIPT_SELECTED_ITEMS).Append(") && (").Append(JAVASCRIPT_SELECTED_ITEMS).Append(" == true))\n");
        sb.Append("  {\n");
        sb.Append("    return true;\n");
        sb.Append("  }\n");
        // Check for selected items in the tree
        sb.Append("  varMultiButton = document.").Append(AFormDP.HTMLFORM_NAME).Append(".").Append(m_asModel.GetMultipleSelect() ? HTMLBUTTON_CHECKBOX : HTMLBUTTON_RADIO).Append(";\n");
        sb.Append("  if ((varMultiButton) && (varMultiButton.length)) {\n");
        sb.Append("    for ( var i = 0; i < varMultiButton.length; i++ ) {\n");
        sb.Append("      if ( varMultiButton[i].checked == true ) {\n");
        sb.Append("        return true;\n");
        sb.Append("      }\n");
        sb.Append("    }\n");
        sb.Append("  } else if ((varMultiButton) && (varMultiButton.checked == true)) {\n");
        sb.Append("      return true;\n");
        sb.Append("  }\n");
        sb.Append("  alert(\'").Append(m_asOwner.GetString(82, ResourceConstants.INFRASTRUCTURE)).Append("\');\n");
        sb.Append("  return false;\n");
      }
      sb.Append("}\n");
      sb.Append("\n");
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()"></seealso>
    public virtual HTMLScript DisplayJavascript()
    {
      // Set up parent form space ID
      String strSpaceIDInput = m_asModel.GetParentFormSpaceID();
      int nLastExpandedID = m_asModel.GetLastExpandedNodeID();
      bool bListCheckAll = m_asModel.GetDisplayingFlatList() && m_asModel.GetMultipleSelect();
      // Calculate approximate string builder size
      int nSize = 600;
      if ((strSpaceIDInput != null) && (!"".Equals(strSpaceIDInput)))
      {
        // Add space for setting the space ID on the parent page
        nSize+=80;
      }
      if (nLastExpandedID != -1)
      {
        // Add size to scroll the correct div into view.
        nSize+=80;
      }
      if (bListCheckAll)
      {
        // Add space for the check all function
        nSize+=200;
      }
      XPStringBuilder sb = new XPStringBuilder(nSize);
      // Expand function
      sb.Append("function ").Append(JAVASCRIPT_EXPAND).Append("(bExpand, nNodeID)\n");
      sb.Append("{\n");
      sb.Append("  document.").Append(AFormDP.HTMLFORM_NAME).Append(".").Append(TreeRepostControl.m_strEXPAND).Append(".value= bExpand;\n");
      sb.Append("  document.").Append(AFormDP.HTMLFORM_NAME).Append(".").Append(TreeRepostControl.m_strFOLDERID).Append(".value= nNodeID;\n");
      sb.Append("  postToSelf(").Append(TreeRepostControl.POSTTOSELF_EXPAND).Append(");\n");
      sb.Append("}\n");
      sb.Append("\n");
      DisplayFormValidationFunction(sb);
      // Helper variable to mark whether or not the page has been loaded.
      sb.Append("var ").Append(JAVASCRIPT_PAGE_LOADED).Append(" = false;\n");
      sb.Append("function ").Append(JAVASCRIPT_TREE_ONLOAD).Append("() {\n");
      if (((strSpaceIDInput != null) && (!"".Equals(strSpaceIDInput))) || (nLastExpandedID > 1))
      {
        // set the Space ID when the page loads
        if ((strSpaceIDInput != null) && (!"".Equals(strSpaceIDInput)))
        {
          sb.Append("parent.opener.document.forms[\'").Append(JSHelper.escapeJS(m_asModel.GetParentFormName())).Append("\'].elements[\'").Append(JSHelper.escapeJS(strSpaceIDInput)).Append("\'].value= ").Append(m_asOwner.GetSpaceID()).Append(";\n");
          sb.Append("window.focus();\n");
        }
        // We don't want to scroll if nothing has been opened, or if the first node
        // is the only thing opened.
        if (nLastExpandedID > 1)
        {
          sb.Append("PTCommonUtil.scrollDivIntoView(document.getElementById(\'").Append(NODE_ID_PREFIX).Append(nLastExpandedID).Append("\'), document.body);\n");
          }
      }
      // Mark page as loaded.
      sb.Append("  ").Append(JAVASCRIPT_PAGE_LOADED).Append(" = true;\n");
      sb.Append("}\n");
      // This should probably use PCC, but the PCC is broken on this page.
      // Setting this onload function breaks the PCC, although it is already broken.
      //		sb.Append("document.PCC.RegisterForWindowEvent(\"onload\", \"")
      //		.Append(JAVASCRIPT_TREE_ONLOAD).Append("();\");\n");
      sb.Append("window.onload = ").Append(JAVASCRIPT_TREE_ONLOAD).Append(";\n");
      if (bListCheckAll)
      {
        // JavaScript to check all the rows in a flat list
        sb.Append("function ").Append(JAVASCRIPT_LIST_CHECK_ALL).Append("(inputCBAll, cbName)\n").Append("{\n").Append("  var frm = document.").Append(AFormDP.HTMLFORM_NAME).Append(";\n").Append("  var len = frm.elements.length;\n").Append("  var nValue = inputCBAll.checked;\n").Append("  var i = 0;\n").Append("  for(i=0 ; i<len ; i++)\n").Append("  {\n").Append("    if (frm.elements[i].name == cbName)\n").Append("    {\n").Append("      frm.elements[i].checked = nValue;\n").Append("    }\n").Append("  }\n").Append("  postToSelf(").Append(TreeRepostControl.POSTTOSELF_ACTION_SIMPLEREPOST).Append(");\n").Append("}\n");
        }
      // JavaScript to check all the rows in a large folder in the tree
      // This javascript assumes that all objects in a folder have
      // sequential ids
      sb.Append("function ").Append(JAVASCRIPT_TREE_CHECK_ALL).Append("(inputCBAll, nFirstID, nLastID)\n").Append("{\n").Append("  var elements = document.").Append(AFormDP.HTMLFORM_NAME).Append(".elements;\n").Append("  var len = elements.length;\n").Append("  var nValue = inputCBAll.checked;\n").Append("  var i = 0;\n").Append("  for(i=0 ; i<len ; i++)\n").Append("  {\n").Append("    if (elements[i].name == \'").Append(HTMLBUTTON_CHECKBOX).Append("\')\n").Append("    {\n").Append("      if ((elements[i].value >= nFirstID) && (elements[i].value <= nLastID))\n").Append("      {\n").Append("        elements[i].checked = nValue;\n").Append("      }\n").Append("    }\n").Append("  }\n").Append("  postToSelf(").Append(TreeRepostControl.POSTTOSELF_ACTION_SIMPLEREPOST).Append(");\n").Append("}\n");
      HTMLScript result = new HTMLScript("text/javaScript");
      try
      {
        result.AddInnerHTMLString(sb.ToString());
      }
      catch (HTMLException he)
      {
        log.Error(he, "DisplayJavascript couldn\'t add javascript to the HTMLScript element.");
        return null;
      }
      return result;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public virtual String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#Init(IModelRO,AActivitySpace)"></seealso>
    public virtual void Init(IModelRO model, AActivitySpace parent)
    {
      m_asModel = (ITreeModelRO) model;
      m_asOwner = parent;
    }
    
    /// <param name="myTable"></param>
    /// <param name="myRoot"></param>
    /// <param name="nDepth"></param>
    /// <throws>Exception</throws>
    private void DisplayCompositeNodes(HTMLElement result, TreeCompositeIterator myRoot,
     int nDepth)
    {
      log.Debug("Displaying composite node " + myRoot.GetBaseEncodedName());
      int nIndent = nDepth * INDENTWIDTH;
      HTMLDiv myDiv;
      HTMLTableRow myRow;
      HTMLTableCell myCell;
      HTMLInput myInput;
      HTMLImg myImg;
      HTMLAnchor myAnchor;
      // Are composites selectable, and how?
      int nSelectMode = m_asModel.GetSelectMode();
      bool bSelectable = ((nSelectMode == TreeConstants.N_COMPOSITESELECT) || (nSelectMode == TreeConstants.N_LEAFCOMPOSITESELECT));
      bool bMultiSelect = m_asModel.GetMultipleSelect();
      bool bDisabled = myRoot.GetBaseDisabled();
      bool bBrowsableOnly = myRoot.GetBaseBrowsableOnly();
      if (bDisabled)
      {
        // extra spacing for the items we are going to skip
        // skip expand image
        nIndent+=INDENTWIDTH;
        if (bSelectable)
        {
          // skip select button
          nIndent+=INDENTWIDTH;
        }
      } else
      if (bBrowsableOnly)
      {
        // Extra spacing for the items we are going to skip
        // skip select button
        nIndent+=INDENTWIDTH;
      }
      // Display Base node
      // Set Div so we can scroll to this element
      myDiv = new HTMLDiv();
      myDiv.SetID(NODE_ID_PREFIX + XPConvert.ToString(myRoot.GetBaseTreeNodeID()));
      result.AddInnerHTMLElement(myDiv);
      //JH-Add a table so things will line up properly
      HTMLTable innerTable = new HTMLTable();
      myDiv.AddInnerHTMLElement(innerTable);
      innerTable.SetBorder(CommonHTMLStrings.ZERO);
      myRow = new HTMLTableRow();
      innerTable.AddInnerHTMLElement(myRow);
      //Add the first cell of the inner table
      myCell = new HTMLTableCell();
      myRow.AddInnerHTMLElement(myCell);
      // Add indentation spacer
      myCell.AddInnerHTMLElement(GetSpacerImage(nIndent));
      String strCollapseAlt = null;
      if (!bDisabled)
      {
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        PTImageType collapseImageType;
        // Set folder open/closed images
        if (myRoot.GetBaseExpanded())
        {
          collapseImageType = PTImageType.ICON_TREE_COLLAPSE;
          strCollapseAlt = m_asOwner.GetString(83, ResourceConstants.INFRASTRUCTURE);
          } else
        {
          collapseImageType = PTImageType.ICON_TREE_EXPAND;
          strCollapseAlt = m_asOwner.GetString(84, ResourceConstants.INFRASTRUCTURE);
          }
        // Add collapse/expand image
        myAnchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
        myImg = (HTMLImg) UIConstructFactory.GetConstruct(collapseImageType,
         true, m_asOwner).GetHTMLElement();
        myImg.SetBorder(CommonHTMLStrings.ZERO);
        myImg.SetAlt(strCollapseAlt);
        myAnchor.AddInnerHTMLElement(myImg);
        myAnchor.SetOnClick(JAVASCRIPT_EXPAND + "(" + XPConvert.ToString(!myRoot.GetBaseExpanded()) + "," + myRoot.GetBaseTreeNodeID() + "); return false;");
        myCell.AddInnerHTMLElement(myAnchor);
      }
      // Add input
      if (!bDisabled && (bSelectable || myRoot.GetBaseForceSelectable()) && !bBrowsableOnly)
      {
        if (bMultiSelect)
        {
          // Check box
          myInput = new HTMLInput(HTMLInputTypes.CHECKBOX, HTMLBUTTON_CHECKBOX,
           HTMLBUTTON_CHECKBOX);
        } else
        {
          // Radio button
          myInput = new HTMLInput(HTMLInputTypes.RADIO, HTMLBUTTON_RADIO,
           HTMLBUTTON_RADIO);
        }
        // This has already been parsed by the control for dangerous characters,
        // so we don't need to encode it.
        String strOnClick = m_asModel.GetOnClickFunction();
        if (strOnClick != null)
        {
          myInput.SetOnClick("return " + strOnClick + "(this);");
        }
        myInput.SetValue(XPConvert.ToString(myRoot.GetBaseTreeNodeID()));
        if (myRoot.GetBaseSelected())
        {
          myInput.SetChecked(true);
          // Add this to the list of selected nodes
          if (m_sbSelectedItems.GetLength() != 0)
          {
            m_sbSelectedItems.Append(",");
          }
          m_sbSelectedItems.Append(myRoot.GetBaseTreeNodeID());
        }
        //Add the second cell of the inner table
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myCell.AddInnerHTMLElement(myInput);
      }
      //Add the third cell of the inner table
      myCell = new HTMLTableCell();
      myCell.SetWidth("20");
      myCell.SetAlign(CommonHTMLStrings.CENTER);
      myRow.AddInnerHTMLElement(myCell);
      // Add Folder icon and name
      HTMLSpan mySpan = new HTMLSpan();
      mySpan.SetStyleClass(PTStyleClass.DIALOGUE_TEXT);
      myImg = myRoot.GetBaseImage();
      myImg.SetAlt(myRoot.GetBaseDescription());
      if (!bDisabled)
      {
        myAnchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
        myAnchor.AddInnerHTMLElement(myImg);
        myAnchor.SetOnClick(JAVASCRIPT_EXPAND + "(" + XPConvert.ToString(!myRoot.GetBaseExpanded()) + "," + myRoot.GetBaseTreeNodeID() + "); return false;");
        myAnchor.SetTitle(myRoot.GetBaseDescription());
        mySpan.AddInnerHTMLElement(myAnchor);
        myCell.AddInnerHTMLElement(mySpan);
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        mySpan = new HTMLSpan();
        mySpan.SetStyleClass(PTStyleClass.DIALOGUE_TEXT);
        myAnchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
        myAnchor.AddInnerHTMLString(myRoot.GetBaseEncodedName());
        myAnchor.SetOnClick(JAVASCRIPT_EXPAND + "(" + XPConvert.ToString(!myRoot.GetBaseExpanded()) + "," + myRoot.GetBaseTreeNodeID() + "); return false;");
        myAnchor.SetTitle(myRoot.GetBaseDescription());
        mySpan.AddInnerHTMLElement(myAnchor);
      } else
      {
        mySpan.AddInnerHTMLElement(myImg);
        myCell.AddInnerHTMLElement(mySpan);
        myCell = new HTMLTableCell();
        myCell.SetStyleClass(PTStyleClass.DIALOGUE_TEXT);
        myRow.AddInnerHTMLElement(myCell);
        mySpan = new HTMLSpan();
        mySpan.AddInnerHTMLString(myRoot.GetBaseEncodedName());
        mySpan.SetTitle(myRoot.GetBaseDescription());
      }
      myCell.AddInnerHTMLElement(mySpan);
      // If this node is opened, show it's children
      if (myRoot.GetBaseExpanded())
      {
        log.Debug("In node " + myRoot.GetBaseEncodedName() + " displaying composite nodes.");
        // Loop through composite children
        while (myRoot.MoveToNextCompositeChild())
        {
          DisplayCompositeNodes(result, myRoot.GetCurrentCompositeChildIterator(),
           nDepth + 1);
        }
        log.Debug("In node " + myRoot.GetBaseEncodedName() + " displaying leaf nodes.");
        // Display the leaf nodes of this composite node
        DisplayLeafNodes(result, myRoot.GetBaseLeafChildIterator(), myRoot.GetBaseLeafChildCount(),
         nDepth + 1);
      }
    }
    
    /// <param name="myTable"></param>
    /// <param name="myLeaves"></param>
    /// <param name="nDepth"></param>
    /// <throws>Exception</throws>
    private void DisplayLeafNodes(HTMLElement result, TreeLeafIterator myLeaves,
     int nNumLeaves, int nDepth)
    {
      int nSelectMode = m_asModel.GetSelectMode();
      int nIndent = (nDepth + (((nSelectMode == TreeConstants.N_COMPOSITESELECT) || (nSelectMode == TreeConstants.N_LEAFCOMPOSITESELECT)) ? 1 : 0) + (((nSelectMode != TreeConstants.N_LEAFSELECT) && (nSelectMode != TreeConstants.N_LEAFCOMPOSITESELECT)) ? 1 : 0)) * LEAFINDENTWIDTH;
      if (0 == nIndent)
      {
        nIndent = 25;
      }
      // Are leaves selectable, and how?
      bool bSelectable = (nSelectMode == TreeConstants.N_LEAFSELECT) || (nSelectMode == TreeConstants.N_LEAFCOMPOSITESELECT);
      bool bMultiSelect = m_asModel.GetMultipleSelect();
      bool bIsFlatList = m_asModel.GetDisplayingFlatList();
      HTMLTableRow myRow;
      HTMLTableCell myCell;
      HTMLInput myInput;
      HTMLImg myImg;
      HTMLInput checkAllInput = null;
      int nLowestNodeID = -1;
      int nHighestNodeID = -1;
      bool bAllChecked = true;
      //JH-Add a table so things will line up properly
      // We can share the table across all children because all of
      // the columns are the same.
      HTMLTable innerTable = new HTMLTable();
      result.AddInnerHTMLElement(innerTable);
      innerTable.SetBorder(CommonHTMLStrings.ZERO);
      // If leaves are multi-selectable, and there are a lot of them, 
      // and it's not a flat list (if it is, there is already a Check All checkbox 
      // at the top of the tree) ...
      if (bSelectable && bMultiSelect && (nNumLeaves > MIN_CHECK_ALL_SIZE) && !bIsFlatList)
      {
        // Add a check all box
        String strMessage;
        if (nNumLeaves >= MAX_FOLDER_OBJECTS)
        {
          // Select all displayed items message
          strMessage = m_asOwner.GetString(136, ResourceConstants.INFRASTRUCTURE);
          } else
        {
          // normal Select all message
          strMessage = m_asOwner.GetString(135, ResourceConstants.INFRASTRUCTURE);
          }
        myRow = new HTMLTableRow();
        innerTable.AddInnerHTMLElement(myRow);
        myCell = new HTMLTableCell();
        // Add indentation spacer image
        myCell.AddInnerHTMLElement(GetSpacerImage(nIndent));
        myRow.AddInnerHTMLElement(myCell);
        //Add second cell in inner table
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        // Add input check box
        checkAllInput = new HTMLInput(HTMLInputTypes.CHECKBOX, HTMLINPUT_CHECK_ALL,
         HTMLINPUT_CHECK_ALL);
        // The onclick method for this input box will be added at
        // the end of this method.
        myCell.AddInnerHTMLElement(checkAllInput);
        //Add third-fourth cell in inner table
        myCell = new HTMLTableCell();
        myCell.SetColSpan(CommonHTMLStrings.TWO);
        myRow.AddInnerHTMLElement(myCell);
        //Add the message
        HTMLSpan mySpan = new HTMLSpan();
        mySpan.SetStyleClass(PTStyleClass.DIALOGUE_TEXT);
        mySpan.AddInnerHTMLString(strMessage);
        mySpan.SetTitle(strMessage);
        myCell.AddInnerHTMLElement(mySpan);
      }
      int i = 0;
      // Loop through leaf children up to MAX_FOLDER_OBJECTS (1000) children
      while (myLeaves.MoveToNextChild() && (i < MAX_FOLDER_OBJECTS))
      {
        i++;
        if (checkAllInput != null)
        {
          // Calculate lowest and highest node ids in leaves for check
          // all box
          if (nLowestNodeID == -1)
          {
            nLowestNodeID = myLeaves.GetCurrentTreeNodeID();
            nHighestNodeID = nLowestNodeID;
          } else
          {
            int nCurrentNodeID = myLeaves.GetCurrentTreeNodeID();
            if (nCurrentNodeID < nLowestNodeID)
            {
              nLowestNodeID = nCurrentNodeID;
            } else
            if (nCurrentNodeID > nHighestNodeID)
            {
              nHighestNodeID = nCurrentNodeID;
            }
          }
          if (!myLeaves.GetCurrentSelected())
          {
            bAllChecked = false;
          }
        }
        myRow = new HTMLTableRow();
        innerTable.AddInnerHTMLElement(myRow);
        myCell = new HTMLTableCell();
        // Add indentation spacer image
        myCell.AddInnerHTMLElement(GetSpacerImage(nIndent));
        myRow.AddInnerHTMLElement(myCell);
        //Add second cell in inner table
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        // Add input
        if (bSelectable || myLeaves.GetCurrentForceSelectable())
        {
          if (bMultiSelect)
          {
            // Check box
            myInput = new HTMLInput(HTMLInputTypes.CHECKBOX, HTMLBUTTON_CHECKBOX,
             HTMLBUTTON_CHECKBOX);
          } else
          {
            // Radio button
            myInput = new HTMLInput(HTMLInputTypes.RADIO, HTMLBUTTON_RADIO,
             HTMLBUTTON_RADIO);
          }
          // This has already been parsed by the control for dangerous characters,
          // so we don't need to encode it.
          String strOnClick = m_asModel.GetOnClickFunction();
          if (strOnClick != null)
          {
            myInput.SetOnClick("return " + strOnClick + "(this);");
          }
          myInput.SetValue(XPConvert.ToString(myLeaves.GetCurrentTreeNodeID()));
          if (myLeaves.GetCurrentSelected())
          {
            myInput.SetChecked(true);
            // Add this to the list of selected nodes
            if (m_sbSelectedItems.GetLength() != 0)
            {
              m_sbSelectedItems.Append(",");
            }
            m_sbSelectedItems.Append(myLeaves.GetCurrentTreeNodeID());
          }
          myCell.AddInnerHTMLElement(myInput);
        }
        //Add third cell in inner table
        myCell = new HTMLTableCell();
        myCell.SetWidth("20");
        myCell.SetAlign(CommonHTMLStrings.CENTER);
        myRow.AddInnerHTMLElement(myCell);
        // Add Object icon
        myImg = myLeaves.GetCurrentImage();
        myImg.SetAlt(myLeaves.GetCurrentDescription());
        myCell.AddInnerHTMLElement(myImg);
        //Add fourth cell in inner table
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        //Add the name of the leaf node
        HTMLSpan mySpan = new HTMLSpan();
        mySpan.SetStyleClass(PTStyleClass.DIALOGUE_TEXT);
        mySpan.AddInnerHTMLString(myLeaves.GetCurrentEncodedName());
        mySpan.SetTitle(myLeaves.GetCurrentDescription());
        myCell.AddInnerHTMLElement(mySpan);
        log.Debug("Displayed leaf node " + myLeaves.GetCurrentEncodedName());
        }
      if (checkAllInput != null)
      {
        // Set the on click on the check all box generated previously.
        // This javascript assumes that all objects in a folder have
        // sequential ids
        checkAllInput.SetOnClick("return " + JAVASCRIPT_TREE_CHECK_ALL + "(this, " + nLowestNodeID + ", " + nHighestNodeID + ");");
        if (bAllChecked)
        {
          checkAllInput.SetChecked(true);
        }
      }
      // We need to check equals below, because the queries only return 1000 objects,
      // and so the folders should only ever have 1000 objects, even if there are
      // more in the query, so we need to cheat a little to figure out if there
      // could be extra objects in the query.
      if (i >= MAX_FOLDER_OBJECTS)
      {
        // Display 'use search' message
        myRow = new HTMLTableRow();
        innerTable.AddInnerHTMLElement(myRow);
        myCell = new HTMLTableCell();
        // Add indentation spacer image
        myCell.AddInnerHTMLElement(GetSpacerImage(nIndent));
        myRow.AddInnerHTMLElement(myCell);
        //Add second cell in inner table
        myCell = new HTMLTableCell();
        // Set colspan since we are just displaying a message, not an object.
        myCell.SetColSpan(CommonHTMLStrings.THREE);
        myRow.AddInnerHTMLElement(myCell);
        //Add maximum objects warning message
        HTMLSpan mySpan = new HTMLSpan();
        mySpan.SetStyleClass(PTStyleClass.ALERT_ERROR_TEXT);
        myCell.AddInnerHTMLElement(mySpan);
        mySpan.AddInnerHTMLEncodedString(XPResourceManager.FormatMessage(m_asOwner.GetString(133,
         ResourceConstants.INFRASTRUCTURE), XPConvert.ToString(MAX_FOLDER_OBJECTS)));
        myCell.AddInnerHTMLElement(new HTMLBr());
        //Add use search message
        String strMessage = m_asOwner.GetString(134, ResourceConstants.INFRASTRUCTURE);
        mySpan = new HTMLSpan();
        mySpan.SetStyleClass(PTStyleClass.ALERT_ERROR_TEXT);
        myCell.AddInnerHTMLElement(mySpan);
        mySpan.AddInnerHTMLEncodedString(AnchorInLocalizedString.GetStringBeforeAnchor(strMessage));
        myCell.AddInnerHTMLElement(AnchorInLocalizedString.GetAnchor(strMessage,
         "#" + ATreeAS.TREE_BAR_ANCHOR));
        mySpan = new HTMLSpan();
        mySpan.SetStyleClass(PTStyleClass.ALERT_ERROR_TEXT);
        myCell.AddInnerHTMLElement(mySpan);
        mySpan.AddInnerHTMLEncodedString(AnchorInLocalizedString.GetStringAfterAnchor(strMessage));
        }
    }
    
    /// <param name="nIndent"></param>
    /// <returns></returns>
    private HTMLImg GetSpacerImage(int nIndent)
    {
      HTMLImg myImg;
      try
      {
        myImg = (HTMLImg) UIConstructFactory.GetConstruct(PTImageType.SPACER,
         true, m_asOwner).GetHTMLElement();
        myImg.SetWidth(XPConvert.ToString(nIndent));
        myImg.SetHeight(CommonHTMLStrings.ONE);
        myImg.SetBorder(CommonHTMLStrings.ZERO);
        myImg.SetAlt("spacer image");
      }
      catch (Exception e)
      {
        log.Error(e, "GetSpacerImage: The following error occured when trying to create the spacer image.");
        return null;
      }
      return myImg;
    }
  }
}