/*
 * Created on Jan 6, 2005
 */
 
using System;
using com.plumtree.openfoundation.util;
namespace com.plumtree.uiinfrastructure.utils
{
  /// <summary>
  /// A data structure that is equivalent to a list of hashtables or a 2D staggered array but simpler interface and with
  /// convenience methods to retrieve String and integers.
  /// Logical image of the data structure:
  /// [0][key][key2][key3]...[keyn]
  /// [1][key][key2][key3]...[keyn]
  /// [2][key][key2][key3]...[keym]
  /// Uses one hashtable internally for efficiency. 
  /// Example:
  /// Setting data
  /// <c>
  /// PTListMap foo = new PTListMap();
  /// int nIndex = foo.AddNewElement();
  /// foo.AddElementToIndex(nIndex, "fooKey1", "fooValue");
  /// foo.AddElementToIndex(nIndex, "fooKey2", new Integer(100) );
  /// foo.AddElementToIndex(nIndex, "fooKey3", new FooClass() );
  /// nIndex = foo.AddNewElement();
  /// foo.AddElementToIndex(nIndex, "fooKey1", "fooValue");
  /// foo.AddElementToIndex(nIndex, "fooKey2", new Integer(100) );
  /// foo.AddElementToIndex(nIndex, "fooKey3", new FooClass() );
  /// </c>
  /// Getting data
  /// <c>
  /// for ( int i = 0; i < foo.GetSize(); i++ )
  /// {
  /// String strVal = foo.GetElementAsString(i, "fooKey1");
  /// int nVal = foo.GetElementAsInt(i, "fooKey2");
  /// Object oVal = int nVal = foo.GetElementAs(i, "fooKey3");
  /// }
  /// </c>
  /// </summary>
  /// <author>robertz</author>
  public class PTListMap
  {
    //
    private XPHashtable m_hashtable;
    //
    private int m_nListEntries;
    
    /// <summary>
    /// Creates an empty PTListMap
    /// </summary>
    public PTListMap()
    {
      m_hashtable = new XPHashtable();
      m_nListEntries = 0;
    }
    
    /// <summary>
    /// Get the number of rows.
    /// </summary>
    /// <returns></returns>
    public virtual int GetSize()
    {
      return m_nListEntries;
    }
    
    /// <summary>
    /// Gets the index of the next row
    /// </summary>
    /// <returns>The index of the new row</returns>
    public virtual int AddNewElement()
    {
      m_nListEntries++;
      return m_nListEntries - 1;
    }
    
    /// <summary>
    /// Adds a key/value pair to an existing row.
    /// </summary>
    /// <param name="nIndex">Row index</param>
    /// <param name="strKey"></param>
    /// <param name="Value"></param>
    public virtual void AddElementToIndex(int nIndex, String strKey, Object Value)
    {
      m_hashtable.PutElement(CreateInternalKeyName(nIndex, strKey), Value);
      }
    
    /// <summary>
    /// Get an entry as Object
    /// </summary>
    /// <param name="nIndex"></param>
    /// <param name="strKey"></param>
    /// <returns></returns>
    public virtual Object GetElement(int nIndex, String strKey)
    {
      return m_hashtable.GetElement(CreateInternalKeyName(nIndex, strKey));
      }
    
    /// <summary>
    /// Get an entry as Int
    /// </summary>
    /// <param name="nIndex"></param>
    /// <param name="strKey"></param>
    /// <returns></returns>
    public virtual int GetElementAsInt(int nIndex, String strKey)
    {
      Object o = m_hashtable.GetElement(CreateInternalKeyName(nIndex, strKey));
      int nVal = XPConvert.ToInteger(o);
      return nVal;
    }
    
    /// <summary>
    /// Get an entry as String
    /// </summary>
    /// <param name="nIndex"></param>
    /// <param name="strKey"></param>
    /// <returns></returns>
    public virtual String GetElementAsString(int nIndex, String strKey)
    {
      Object o = m_hashtable.GetElement(CreateInternalKeyName(nIndex, strKey));
      String strVal = XPConvert.ToString(o);
      return strVal;
    }
    
    /// <param name="nIndex"></param>
    /// <param name="strKey"></param>
    /// <returns></returns>
    private InternalListTableEntry CreateInternalKeyName(int nIndex, String strKey)
    {
      return new InternalListTableEntry(nIndex, strKey);
    }
    
    /// <param name="oInternalKey"></param>
    /// <returns></returns>
    private int GetIndexFromInternalKeyName(Object oInternalKey)
    {
      return ((InternalListTableEntry) oInternalKey).GetIndex();
    }
    
    /// <param name="oInternalKey"></param>
    /// <returns></returns>
    private String GetKeyNameFromInternalKeyName(Object oInternalKey)
    {
      return ((InternalListTableEntry) oInternalKey).GetKey();
    }
    
    /// <summary>
    /// Internal class for entries
    /// </summary>
    /// <author>robertz</author>
    private class InternalListTableEntry
    {
      private int m_nIndex;
      private String m_strKey;
      public InternalListTableEntry(int nIndex, String strKey)
      {
        m_nIndex = nIndex;
        m_strKey = strKey;
      }
      public virtual int GetIndex()
      {
        return m_nIndex;
      }
      public virtual String GetKey()
      {
        return m_strKey;
      }
      
      /// <seealso cref="java.lang.Object#hashCode()"></seealso>
      public override int GetHashCode()
      {
        return (m_nIndex + m_strKey).GetHashCode();
      }
      public override bool Equals(Object o)
      {
        InternalListTableEntry entry = null;
        try
        {
          entry = (InternalListTableEntry) o;
        }
        catch (XPException e)
        {
          return false;
        }
        return (entry.m_nIndex == m_nIndex && entry.m_strKey.Equals(m_strKey));
        }
    }
  }
}