using System;
using com.plumtree.openfoundation.util;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.xpshared.htmlconstructs
{
  /// <author>
  /// beng
  /// Use this class to build tables for which each cell spans exactly 1 column.
  /// <br /><br />
  /// This class is intended to simplify the creation of such tables.
  /// </author>
  public class PTTableBuilder
  {
    internal int iCols = -1;
    internal HTMLTable myTable = null;
    internal XPArrayList alRows = null;
    internal PTStyleClass scTable = null;
    internal PTStyleClass scRow = null;
    internal PTStyleClass scCell = null;
    
    /// <summary>
    /// Creates a new PTTableBuilder object.
    /// <br /><br />
    /// The table produced by this builder constructed in this way will
    /// contain no rows, will be "100%" wide, with cellspacing and cellpadding
    /// each "0", no style classes will be set.
    /// </summary>
    /// <param name="cols">Pass the number of columns which this table spans.</param>
    public PTTableBuilder(int cols)
    {
      InternalConstructor(0, cols, "100%", "0", "0", null, null, null);
    }
    
    /// <summary>
    /// Creates a new PTTableBuilder object.
    /// <br /><br />
    /// The table produced by this builder constructed in this way will
    /// contain the specified number of rows, will be "100%" wide, with 
    /// cellspacing and cellpadding
    /// each "0", no style classes will be set.
    /// </summary>
    /// <param name="rows">Pass the number of rows this table spans.</param>
    /// <param name="cols">Pass the number of columns this table spans.</param>
    public PTTableBuilder(int rows, int cols)
    {
      InternalConstructor(rows, cols, "100%", "0", "0", null, null, null);
      }
    
    /// <summary>
    /// Creates a new PTTableBuilder object.
    /// <br /><br />
    /// No style classes will be set for the table produced by this 
    /// builder constructed in this way,
    /// </summary>
    /// <param name="rows">Pass the number of rows this table spans.</param>
    /// <param name="cols">Pass the number of columns this table spans.</param>
    /// <param name="sWidth">Pass the width of this table.</param>
    /// <param name="sCellPadding">Pass the cell padding.</param>
    /// <param name="sCellSpacing">Pass the cell spacing.</param>
    public PTTableBuilder(int rows, int cols, String sWidth, String sCellPadding,
     String sCellSpacing)
    {
      InternalConstructor(rows, cols, sWidth, sCellPadding, sCellSpacing,
       null, null, null);
    }
    
    /// <summary>
    /// Creates a new PTTableBuilder object.
    /// <br /><br />
    /// The table produced by this builder constructed in this way will
    /// contain no rows, will be "100%" wide, with cellspacing and cellpadding
    /// each "0", style classes will be set as specified.
    /// </summary>
    /// <param name="cols">Pass the number of columns this table spans.</param>
    /// <param name="tableClass">Pass the style class of this table.</param>
    /// <param name="rowClass">Pass the style class for the rows of this table.</param>
    /// <param name="cellClass">Pass the style class for the cell sof this table.</param>
    public PTTableBuilder(int cols, PTStyleClass tableClass, PTStyleClass rowClass,
     PTStyleClass cellClass)
    {
      InternalConstructor(0, cols, "100%", "0", "0", tableClass, rowClass,
       cellClass);
    }
    
    /// <summary>
    /// Creates a new PTTableBuilder object.
    /// <br /><br />
    /// The table produced by this builder constructed in this way will
    /// contain the specified number of rows, will be "100%" wide, with 
    /// cellspacing and cellpadding each "0", style classes will be set 
    /// as specified.
    /// </summary>
    /// <param name="rows">Pass the number of rows this table spans.</param>
    /// <param name="cols">Pass the number of columns this table spans.</param>
    /// <param name="tableClass">Pass the style class of this table.</param>
    /// <param name="rowClass">Pass the style class for the rows of this table.</param>
    /// <param name="cellClass">Pass the style class for the cell sof this table.</param>
    public PTTableBuilder(int rows, int cols, PTStyleClass tableClass, PTStyleClass rowClass,
     PTStyleClass cellClass)
    {
      InternalConstructor(rows, cols, "100%", "0", "0", tableClass, rowClass,
       cellClass);
    }
    
    /// <summary>
    /// Creates a new PTTableBuilder object.
    /// <br /><br />
    /// </summary>
    /// <param name="rows">Pass the number of rows this table spans.</param>
    /// <param name="cols">Pass the number of columns this table spans.</param>
    /// <param name="sWidth">Pass the width of this table.</param>
    /// <param name="sCellPadding">Pass the cell padding of this table.</param>
    /// <param name="sCellSpacing">Pass the cell spacing of this table.</param>
    /// <param name="tableClass">Pass the style class of this table.</param>
    /// <param name="rowClass">Pass the style class of this table's rows.</param>
    /// <param name="cellClass">Pass the style class of this table's cells.</param>
    public PTTableBuilder(int rows, int cols, String sWidth, String sCellPadding,
     String sCellSpacing, PTStyleClass tableClass, PTStyleClass rowClass,
     PTStyleClass cellClass)
    {
      InternalConstructor(rows, cols, sWidth, sCellPadding, sCellSpacing,
       tableClass, rowClass, cellClass);
    }
    
    /// <summary>
    /// Internal constructor used by all public constructors.
    /// </summary>
    /// <param name="rows">Pass the number of rows this table spans.</param>
    /// <param name="cols">Pass the number of columns this table spans.</param>
    /// <param name="sWidth">Pass the width of this table.</param>
    /// <param name="sCellPadding">Pass the cell padding of this table.</param>
    /// <param name="sCellSpacing">Pass the cell spacing of this table.</param>
    /// <param name="tableClass">Pass the style class of this table.</param>
    /// <param name="rowClass">Pass the style class of this table's rows.</param>
    /// <param name="cellClass">Pass the style class of this table's cells.</param>
    private void InternalConstructor(int rows, int cols, String sWidth, String sCellPadding,
     String sCellSpacing, PTStyleClass cTable, PTStyleClass cRow, PTStyleClass cCell)
    {
      iCols = cols;
      scTable = cTable;
      scRow = cRow;
      scCell = cCell;
      alRows = new XPArrayList();
      myTable = new HTMLTable();
      myTable.SetWidth(sWidth);
      myTable.SetCellPadding(sCellPadding);
      myTable.SetCellSpacing(sCellSpacing);
      if (cTable != null)
      {
        try
        {
          myTable.SetStyleClass(cTable);
        }
        catch (Exception ex)
        {
        }
      }
      // do nothing            	
      for (int x = 0;x < rows;x++)
      {
        InternalAddRow();
      }
    }
    
    /// <summary>
    /// Call this method to obtain a reference to the table created by this
    /// table builder.
    /// </summary>
    /// <returns>HTMLTable table</returns>
    public virtual HTMLTable GetTable()
    {
      return myTable;
    }
    
    /// <summary>
    /// Call this method to obtain the number of rows in the table built by this
    /// builder.
    /// </summary>
    /// <returns>int rows</returns>
    public virtual int GetNumRows()
    {
      XPArrayList alTemp = myTable.GetInnerHTMLElementsAsArrayList(false);
      int i = 0;
      if (alTemp != null)
      {
        i = alTemp.GetSize();
      }
      return i;
    }
    
    /// <summary>
    /// Call this method to determine the number of columns in the table
    /// built by this builder.
    /// </summary>
    /// <returns>int columns</returns>
    public virtual int GetNumCols()
    {
      return iCols;
    }
    
    /// <summary>
    /// Call this method to add a row to the table built by this builder.
    /// <br /><br />
    /// The appropriate number of cells will be added to this row.
    /// </summary>
    /// <returns>HTMLTablRow row added</returns>
    public virtual HTMLTableRow AddRow()
    {
      return InternalAddRow();
    }
    
    /// <summary>
    /// Internal row adding method.
    /// </summary>
    /// <returns>HTMLTableRow row added</returns>
    private HTMLTableRow InternalAddRow()
    {
      HTMLTableRow myRow;
      HTMLTableCell myCell;
      myRow = new HTMLTableRow();
      try
      {
        myTable.AddInnerHTMLElement(myRow);
        if (scRow != null)
        {
          myRow.SetStyleClass(scRow);
        }
        alRows.Add(myRow);
      }
      catch (Exception ex)
      {
      }
      // do nothing        	
      for (int y = 0;y < iCols;y++)
      {
        myCell = new HTMLTableCell();
        try
        {
          if (scCell != null)
          {
            myCell.SetStyleClass(scCell);
          }
          myRow.AddInnerHTMLElement(myCell);
        }
        catch (Exception ex)
        {
        }
      }
      // do nothing            	
      return myRow;
    }
    
    /// <summary>
    /// Call this method to obtain an enumeration of the rows in the table
    /// built by this builder.
    /// </summary>
    /// <returns>IXPEnumerator rows</returns>
    public virtual IXPEnumerator GetRows()
    {
      return myTable.GetInnerHTMLElements(false);
    }
    
    /// <summary>
    /// Call this method to obtain an enumeration of the cells within a given
    /// row within the table built by this builder.
    /// </summary>
    /// <param name="row">
    /// Pass the index of the row you wish to obtain.  If this row 
    /// doesnt exist in the table, you will receive null;
    /// </param>
    /// <returns>IXPEnumerator cells</returns>
    public virtual IXPEnumerator GetCells(int row)
    {
      XPArrayList alTemp = myTable.GetInnerHTMLElementsAsArrayList(false);
      HTMLTableRow r = (HTMLTableRow) alTemp.GetElement(row);
      IXPEnumerator eTemp = null;
      if (r != null)
      {
        eTemp = r.GetInnerHTMLElements(false);
      }
      return eTemp;
    }
  }
}