using System;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// This helper class contains the display options for displaying
  /// HTMLElements.
  /// These are used throughout the entire HTML tree (usually from HTMLPage
  /// down, although sometimes the root of the tree will be another type of
  /// HTMLElement).
  /// Since the Display Options are set on every HTMLElement, if someone
  /// caches an HTMLElement on the session or application, this object
  /// will be leaked (retained) until the session or application is cleared.
  /// There may also be some multi-threading issues when using a cached
  /// HTMLElement for multiple users (the login page Auth Source dropdown
  /// is stored on the application), since each user will attempt to set
  /// their own Display Options on the cached HTMLElement.
  /// </summary>
  /// <author>DonH</author>
  public class DisplayOptions
  {
    private String sImgBaseUrl;
    private String sStyleClassPrefix;
    private String sLang = null;
    private bool bIsNetscape = false;
    private bool bForceLineBreaksAfterSafeElements = true;
    private bool bDo508Checking = false;
    
    /// <summary>
    /// Call this method to obtain the base url used for all HTMLImg objects
    /// being displayed
    /// </summary>
    /// <returns>String base url.</returns>
    public virtual String GetImgBaseUrl()
    {
      return sImgBaseUrl;
    }
    
    /// <summary>
    /// Call this method to set the base url used for all HTMLImg objects being
    /// displayed.
    /// <br /><br />
    /// Call this method only on the root HTMLElement object.  If you are building
    /// an HTMLPage, for example, call myPage.SetImgBaseUrl( "http://blah/" );.
    /// All nested HTMLImg objects may use the base url specified on the HTMLPage
    /// object.
    /// <br /><br />
    /// HTMLImg objects may or may not use this base url specified.
    /// </summary>
    /// <param name="s">
    /// Pass a base url string.  It should look something like: 
    /// "http://blah/".
    /// </param>
    public virtual void SetImgBaseUrl(String s)
    {
      sImgBaseUrl = s;
    }
    
    /// <summary>
    /// Set the value of the lang attribute.
    /// </summary>
    /// <param name="s">String value</param>
    public virtual void SetLang(String s)
    {
      sLang = s;
    }
    
    /// <summary>
    /// Get the value of the lang attribute.
    /// </summary>
    /// <returns>String value (default null)</returns>
    public virtual String GetLang()
    {
      return sLang;
    }
    
    /// <summary>
    /// This method sets whether or not the browser is Netscape.
    /// Certain HTMLElements will display differently on Netscape
    /// versus other browsers.
    /// </summary>
    /// <param name="b">boolean true implies the browser is Netscape</param>
    public virtual void SetIsNetscape(bool b)
    {
      bIsNetscape = b;
    }
    
    /// <summary>
    /// This method returns whether or not the browser is Netscape.
    /// </summary>
    /// <returns>boolean true implies the browser is Netscape.</returns>
    public virtual bool GetIsNetscape()
    {
      return bIsNetscape;
    }
    
    /// <summary>
    /// This method controls whether or not to use line breaks
    /// after elements where it does not affect the HTML output.
    /// </summary>
    /// <param name="b">boolean true implies use line breaks.</param>
    public virtual void SetForceLineBreaksAfterSafeElements(bool b)
    {
      this.bForceLineBreaksAfterSafeElements = b;
    }
    
    /// <summary>
    /// This method returns whether or not to use line breaks
    /// after elements where it does not affect the HTML output.
    /// </summary>
    /// <returns>boolean true implies use line breaks.</returns>
    public virtual bool GetForceLineBreaksAfterSafeElements()
    {
      return this.bForceLineBreaksAfterSafeElements;
    }
    
    /// <summary>
    /// This method returns the style class prefix used for all
    /// HTMLElement objects being displayed.
    /// </summary>
    /// <returns>The base URL used for style classes.</returns>
    public virtual String GetStyleClassPrefix()
    {
      return sStyleClassPrefix;
    }
    
    /// <summary>
    /// Call this method to set the style class prefix used for all
    /// HTMLElement objects being displayed.
    /// <br /><br />
    /// Call this method only on the root HTMLElement object.  If you are building
    /// an HTMLPage, for example, call myPage.SetStyleClassPrefix( "PREFIX" );.
    /// All nested objects may use the prefix specified on the HTMLPage
    /// object.
    /// <br /><br />
    /// </summary>
    /// <param name="s">
    /// Pass a base url string.  It should look something like: 
    /// "http://blah/".
    /// </param>
    public virtual void SetStyleClassPrefix(String s)
    {
      sStyleClassPrefix = s;
    }
    
    /// <summary>
    /// This method sets whether or not to do 508 validity
    /// checking for HTMLElements.  If this is true, HTMLElements
    /// will try to enforce that the developer does not make HTML
    /// that is not 508 compliant.
    /// </summary>
    /// <param name="b">boolean true implies enforce 508 validity.</param>
    public virtual void SetDo508Checking(bool b)
    {
      bDo508Checking = b;
    }
    
    /// <summary>
    /// This method returns whether or not to do 508 validity
    /// checking for HTMLElements.  If this is true, HTMLElements
    /// will try to enforce that the developer does not make HTML
    /// that is not 508 compliant.
    /// </summary>
    /// <returns>boolean true implies enforce 508 validity.</returns>
    public virtual bool GetDo508Checking()
    {
      return bDo508Checking;
    }
  }
}