using System;
//import com.plumtree.openfoundation.logging.*;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// This class can be used to display an HTML anchor element. <br />
  /// <br />
  /// <a target="new"
  /// href="http://www.w3.org/TR/html4/struct/links.html#h-12.2">Spec</a><br /><br />
  /// </summary>
  public class HTMLAnchor : HTMLElement,TabableHTMLElement
  {
    private static readonly byte[] BEGINTAG = HTMLElement.ToByteArray("<a href=\"");
    private static readonly byte[] TAB_ATTR = HTMLElement.ToByteArray(" tabindex=\"");
    private static readonly byte[] TARGET_ATTR = HTMLElement.ToByteArray(" target=\"");
    private static readonly byte[] TITLE_ATTR = HTMLElement.ToByteArray(" title=\"");
    private static readonly byte[] ACCESSKEY_ATTR = HTMLElement.ToByteArray(" accesskey=\"");
    private static readonly byte[] ONCONTEXTMENU_ATTR = HTMLElement.ToByteArray(" oncontextmenu=\"");
    private static readonly byte[] ENDTAG = HTMLElement.ToByteArray("</a>");
    internal static int iNextID = 0;
    public static bool bEnforceTitle = false;
    private String sAccessKey;
    private String sHRef;
    private String sOnContextMenu;
    private String sTabIndex;
    private String sTarget;
    private String sTitle;
    private bool bAllowNullHref = false;
    
    /// <summary>
    /// Creates a new HTMLAnchor object.
    /// </summary>
    /// <param name="sAnchorHRef">Pass the value of the href parameter.</param>
    public HTMLAnchor(String sAnchorHRef)
    {
      sHRef = sAnchorHRef;
      sTarget = null;
      sTitle = null;
      sTabIndex = null;
      //   by default, anchor elements do not format.
      SetDoFormat(false);
    }
    
    /// <summary>
    /// Call this method to get the value of the accesskey attribute.
    /// </summary>
    /// <returns>The String value of the accesskey attribute.</returns>
    public virtual String GetAccessKey()
    {
      return sAccessKey;
    }
    
    /// <summary>
    /// Get the value of the href attribute
    /// </summary>
    /// <returns>The String value of the href attribute.</returns>
    public virtual String GetHRef()
    {
      return sHRef;
    }
    
    /// <summary>
    /// Call this method to get the value of the tabindex attribute.
    /// </summary>
    /// <returns>The String value of the tabindex attribute.</returns>
    public virtual String GetTabIndex()
    {
      return sTabIndex;
    }
    
    /// <summary>
    /// Get the value of the target attribute.
    /// </summary>
    /// <returns>The String value of the target attribute.</returns>
    public virtual String GetTarget()
    {
      return sTarget;
    }
    
    /// <summary>
    /// Get the value of the title attribute.
    /// </summary>
    /// <returns>The String value of the title attribute.</returns>
    public override String GetTitle()
    {
      return sTitle;
    }
    
    /// <summary>
    /// Call this method to set the value of the accesskey attribute.
    /// </summary>
    /// <param name="s">Pass the String value.</param>
    public virtual void SetAccessKey(String s)
    {
      sAccessKey = s;
    }
    
    /// <summary>
    /// Call this method with true to allow for the absence of an href
    /// attribute.  By default, a null href value causes the anchor to not
    /// display at all, and only display nested HTMLElement objects.
    /// </summary>
    /// <param name="b"></param>
    public virtual void SetAllowNullHRef(bool b)
    {
      bAllowNullHref = b;
    }
    
    /// <summary>
    /// Use this method to set the value of the href attribute.
    /// </summary>
    /// <param name="sAnchorHRef">Pass the value of the href attribute.</param>
    public virtual void SetHRef(String sAnchorHRef)
    {
      sHRef = sAnchorHRef;
    }
    
    /// <param name="s"></param>
    public virtual String GetOnContextMenu(String s)
    {
      return sOnContextMenu;
    }
    
    /// <param name="s"></param>
    public virtual void SetOnContextMenu(String s)
    {
      if (!HTMLElement.bAllowEvents)
      {
      }
      //            if (bError)
      //            {
      //                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
      //
      //                Exception e = new Exception(sDesc);
      //
      //                myLogger.Error(sDesc, e);
      //            }
      sOnContextMenu = s;
    }
    
    /// <summary>
    /// Call this method to set the value of the tabindex attribute.
    /// </summary>
    /// <param name="s">Pass the String value.</param>
    public virtual void SetTabIndex(String s)
    {
      sTabIndex = s;
    }
    
    /// <summary>
    /// Set the value of the target attribute.
    /// </summary>
    /// <param name="s">The String value of the target attribute.</param>
    public virtual void SetTarget(String s)
    {
      sTarget = s;
    }
    
    /// <summary>
    /// Set the value of the title attribute.
    /// </summary>
    /// <param name="s">The String value of the title attribute.</param>
    public override void SetTitle(String s)
    {
      sTitle = s;
    }
    /*
     The following methods are necessary to implement TabableHTMLElement
     */
     
    
    /// <summary>
    /// Call this method to remove the tab index from this HTMLElement.
    /// </summary>
    public virtual void ClearTabIndex()
    {
      sTabIndex = null;
    }
    
    /// <summary>
    /// This method contains the bulk of the logic for writting an HTMLElement.
    /// For subclasses needing to modify their display behavior, this is the
    /// primary method to override.
    /// </summary>
    /// <param name="bFormat">
    /// Pass a boolean indicating whether or not to format the
    /// output.
    /// </param>
    /// <param name="sbLocal">
    /// Pass an XPStringBuilder to which you wish the String
    /// appended.  Pass null if you do not wish the string appended to
    /// an XPStringBuilder.
    /// </param>
    /// <param name="response">
    /// Pass an IXPResponse to which you wish the String written.
    /// Pass null if you do not wish the string written to an
    /// IXPResponse.
    /// </param>
    /// <throws>HTMLException</throws>
    protected override void InternalWrite(bool bFormat, XPStringBuilder sbLocal,
     IXPResponse response)
    {
      if ((sHRef != null) || bAllowNullHref)
      {
        String sComment = GetPreComment();
        if ((sComment != null) && bPrintComments)
        {
          WriteNugget(CommonHTMLStrings.BEGIN_COMMENT, sbLocal, response);
          WriteNugget(sComment, sbLocal, response);
          WriteNugget(CommonHTMLStrings.END_COMMENT, sbLocal, response);
        }
        //   concrete subclasses implement the following
        AppendBeginningOfOpeningTag(false, sbLocal, response);
        //   This method adds all attributes common to all html elements,
        //   such as class="" id="", etc.
        AppendCommonAttributes(sbLocal, response);
        //   close the opening tag
        WriteNugget(CLOSE_BRACKET, sbLocal, response);
        GetInnerHTML(sbLocal, response, false);
        //   append the closing tag
        AppendClosingTag(false, sbLocal, response);
        sComment = GetPostComment();
        if ((sComment != null) && bPrintComments)
        {
          WriteNugget(CommonHTMLStrings.BEGIN_COMMENT, sbLocal, response);
          WriteNugget(sComment, sbLocal, response);
          WriteNugget(CommonHTMLStrings.END_COMMENT, sbLocal, response);
        }
      } else
      {
        // sHref is null, and not allow null href
        GetInnerHTML(sbLocal, response, false);
      }
    }
    
    /// <summary>
    /// Draw the beginning of the anchor tag.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendBeginningOfOpeningTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(BEGINTAG, sb, response);
      WriteNugget(sHRef, sb, response);
      WriteNugget(QUOTE, sb, response);
      if (sTabIndex != null)
      {
        WriteNugget(TAB_ATTR, sb, response);
        WriteNugget(sTabIndex, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sTarget != null)
      {
        WriteNugget(TARGET_ATTR, sb, response);
        WriteNugget(sTarget, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sAccessKey != null)
      {
        WriteNugget(ACCESSKEY_ATTR, sb, response);
        WriteNugget(sAccessKey, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sTitle != null)
      {
        WriteNugget(TITLE_ATTR, sb, response);
        WriteNugget(sTitle, sb, response);
        WriteNugget(QUOTE, sb, response);
      } else
      {
        if (bEnforceTitle || GetDo508CheckingForThisInstance())
        {
        }
      }
      //                myLogger.Error("You must set the title on the anchor with href: " + sHRef  );
      if (sOnContextMenu != null)
      {
        if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
        {
        }
        //                if (bError)
        //                {
        //                    myLogger.Error("You are not allowed to set the sOnContextMenu attribute value because the client of this html tree is attempting to render a 508-compliant page.");
        //                    myLogger.Error("this: " + this);
        //                }
        WriteNugget(ONCONTEXTMENU_ATTR, sb, response);
        WriteNugget(sOnContextMenu, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
    }
    
    /// <summary>
    /// Append the closing anchor tag
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendClosingTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(ENDTAG, sb, response);
    }
    
    /// <summary>
    /// Used to determine whether or not to display a closing tag for this
    /// concret subclass of HTMLElement.
    /// </summary>
    /// <returns>boolean true.</returns>
    protected override bool ProvidesClosingTag()
    {
      return true;
    }
    
    /// <summary>
    /// Get the next id value.  This is used by the GetClone method to ensure
    /// that cloned object have new ID attribute values.
    /// </summary>
    /// <returns>A valid id string.</returns>
    internal override String GetNextID()
    {
      String s = "anchor" + iNextID;
      iNextID++;
      return s;
    }
  }
}