using System;
//import com.plumtree.openfoundation.logging.*;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// Use this class to render an html area element. <br />
  /// <br />
  /// <a target="new"
  /// href="http://www.w3.org/TR/html4/struct/objects.html#h-13.2">Spec</a><br /><br />
  /// </summary>
  public class HTMLArea : HTMLElement
  {
    internal static int iNextID = 0;
    private static readonly byte[] BEGIN_TAG = HTMLElement.ToByteArray("<area");
    private const String RECT = "rect";
    private const String RECTANGLE = "rectangle";
    private const String CIRC = "circ";
    private const String CIRCLE = "circle";
    private const String POLY = "poly";
    private const String POLYGON = "polygon";
    private const String BLANK = "_blank";
    private const String PARENT = "_parent";
    private const String SELF = "_self";
    private const String TOP = "_top";
    private static readonly byte[] ALT_ATTR = HTMLElement.ToByteArray(" alt=\"");
    private static readonly byte[] COORDS_ATTR = HTMLElement.ToByteArray(" coords=\"");
    private static readonly byte[] HREF_ATTR = HTMLElement.ToByteArray(" href=\"");
    private static readonly byte[] NOHREF_ATTR = HTMLElement.ToByteArray(" nohref=\"true\" ");
    private static readonly byte[] SHAPE_ATTR = HTMLElement.ToByteArray(" shape=\"");
    private static readonly byte[] TARGET_ATTR = HTMLElement.ToByteArray(" target=\"");
    private static readonly byte[] ENDTAG = HTMLElement.ToByteArray("</area>");
    internal String sCoords = null;
    internal String sAlt;
    internal String sHref = null;
    internal bool bNohref = false;
    internal String sShape = null;
    internal String sTarget = null;
    
    /// <summary>
    /// Constructs a new HTMLArea
    /// </summary>
    /// <param name="alt">Pass the alt string of this area.</param>
    public HTMLArea(String alt)
    {
      sAlt = alt;
    }
    
    /// <summary>
    /// Constructs a new HTMLArea
    /// </summary>
    /// <param name="alt">Pass the alt string of this area.</param>
    /// <param name="shape">Pass the shape string of this area.</param>
    /// <param name="coords">Pass the coordinates string of this area.</param>
    /// <param name="href">Pass the target URL string of this area.</param>
    public HTMLArea(String shape, String alt, String coords, String href)
    {
      sAlt = alt;
      sShape = shape;
      sCoords = coords;
      sHref = href;
    }
    
    /// <summary>
    /// Get the value of the alt attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetAlt()
    {
      return sAlt;
    }
    
    /// <summary>
    /// Get the value of the nohref attribute
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool GetNohref()
    {
      return bNohref;
    }
    
    /// <summary>
    /// Get the value of the coords attribute
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetCoords()
    {
      return sCoords;
    }
    
    /// <summary>
    /// Get the value of the href attribute
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetHref()
    {
      return sHref;
    }
    
    /// <summary>
    /// Get the value of the Shape attribute
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetShape()
    {
      return sShape;
    }
    
    /// <summary>
    /// Get the value of the target attribute
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetTarget()
    {
      return sTarget;
    }
    
    /// <summary>
    /// Sets the value of the nohref attribute.
    /// </summary>
    /// <param name="bNohref"></param>
    public virtual void SetNohref(bool bNohref)
    {
      this.bNohref = bNohref;
    }
    
    /// <summary>
    /// Sets the value of the coords attribute.
    /// </summary>
    /// <param name="sCoords">The coordinates to set</param>
    public virtual void SetCoords(String sCoords)
    {
      this.sCoords = sCoords;
    }
    
    /// <summary>
    /// Sets the value of the href attribute.
    /// </summary>
    /// <param name="sHref">The href to set</param>
    public virtual void SetHref(String sHref)
    {
      this.sHref = sHref;
    }
    
    /// <summary>
    /// Sets the value of the target attribute.
    /// </summary>
    /// <param name="sTarget">The target to set</param>
    public virtual void SetTarget(String sTarget)
    {
      if (!((sTarget.Equals(BLANK)) || (sTarget.Equals(PARENT)) || (sTarget.Equals(SELF)) || (sTarget.Equals(TOP))))
      {
        throw new HTMLException("The target on an HTMLArea must be \"_blank\", \"_parent\", \"_self\", or \"_top\",");
        }
      this.sTarget = sTarget;
    }
    
    /// <summary>
    /// Set the value of the alt atribute.
    /// </summary>
    /// <param name="s">String value.</param>
    public virtual void SetAlt(String s)
    {
      sAlt = s;
    }
    
    /// <summary>
    /// Set the value of the shape attribute.
    /// </summary>
    /// <param name="strShape">Either "rect", "rectangle", "circ", "circle", "poly", or "polygon".</param>
    /// <throws>HTMLException</throws>
    public virtual void SetShape(String strShape)
    {
      if (!((strShape.Equals(RECT)) || (strShape.Equals(RECTANGLE)) || (strShape.Equals(CIRC)) || (strShape.Equals(CIRCLE)) || (strShape.Equals(POLY)) || (strShape.Equals(POLYGON))))
      {
        throw new HTMLException("The shape on an HTMLArea must be \"rect\", \"rectangle\", \"circ\",\"circle\",\"poly\", or \"polygon\",");
        }
      this.sShape = strShape;
    }
    
    /// <summary>
    /// This method generates the beginning of the opening tag.  The abstract
    /// HTMLElement superclass will provide the remainder of the opening tag,
    /// repleate with attributes common to all html elements, such as name, and
    /// id, and class.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    /// <throws>HTMLException</throws>
    protected override void AppendBeginningOfOpeningTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(BEGIN_TAG, sb, response);
      if (sShape != null)
      {
        WriteNugget(SHAPE_ATTR, sb, response);
        WriteNugget(sShape, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sAlt != null)
      {
        WriteNugget(ALT_ATTR, sb, response);
        WriteNugget(sAlt, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sCoords != null)
      {
        WriteNugget(COORDS_ATTR, sb, response);
        WriteNugget(sCoords, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sHref != null)
      {
        WriteNugget(HREF_ATTR, sb, response);
        WriteNugget(sHref, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sTarget != null)
      {
        WriteNugget(TARGET_ATTR, sb, response);
        WriteNugget(sTarget, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (bNohref)
      {
        WriteNugget(NOHREF_ATTR, sb, response);
      }
    }
    
    /// <summary>
    /// This method generates the closing tag of this html element.  If the
    /// providesClosingTag method returns false, then this method will never be
    /// called.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendClosingTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(ENDTAG, sb, response);
    }
    
    /// <summary>
    /// Indicates whether or not this HTML element provides a closing tag. Some
    /// elements only provide opening tags, such as the &lt;br&gt; element.
    /// </summary>
    /// <returns>boolean false.</returns>
    protected override bool ProvidesClosingTag()
    {
      return true;
    }
    
    /// <summary>
    /// Get the next id value.  This is used by the GetClone method to ensure
    /// that cloned object have new ID attribute values.
    /// </summary>
    /// <returns>A valid id string.</returns>
    internal override String GetNextID()
    {
      String s = "img" + iNextID;
      iNextID++;
      return s;
    }
  }
}