using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// <br /><br />
  /// <a target="new"
  /// href="http://www.w3.org/TR/html4/struct/dirlang.html#h-8.2.4">Spec</a><br /><br />
  /// </summary>
  public class HTMLBdo : HTMLElement
  {
    private const String LTR = "ltr";
    private const String RTL = "rtl";
    private const String BDO = "bdo";
    private static readonly byte[] BEGIN_TAG = HTMLElement.ToByteArray("<bdo");
    private static readonly byte[] DIR_ATTR = HTMLElement.ToByteArray(" dir=\"");
    private static readonly byte[] CLOSE_BRACKET = HTMLElement.ToByteArray(">");
    private static readonly byte[] END_TAG = HTMLElement.ToByteArray("</bdo>");
    private static int iNextID = 0;
    private String sDir;
    private String sLang;
    
    /// <summary>
    /// Creates a new HTMLBdo object.
    /// </summary>
    public HTMLBdo()
    {
      sDir = LTR;
      sLang = null;
    }
    
    /// <summary>
    /// Get the value of the dir attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetDir()
    {
      return sDir;
    }
    
    /// <summary>
    /// Call this method to set the value of the dir attribute.
    /// </summary>
    /// <param name="s">Pass the String value.</param>
    /// <throws>
    /// HTMLException Passing an invalid dir value with cause an
    /// exception     to be thrown.
    /// </throws>
    public virtual void SetDir(String s)
    {
      if (!(s.Equals(LTR)) && !(s.Equals(RTL)))
      {
        throw new HTMLException("The dir attribute on HTMLBdo must be \'ltr\' or \'rtl\'.");
        }
      sDir = s;
    }
    
    /// <summary>
    /// This method generates the beginning of the opening tag.  The abstract
    /// HTMLElement superclass will provide the remainder of the opening tag,
    /// repleate with attributes common to all html elements, such as name, and
    /// id, and class.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendBeginningOfOpeningTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(BEGIN_TAG, sb, response);
      if (sDir != null)
      {
        WriteNugget(DIR_ATTR, sb, response);
        WriteNugget(sDir, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      WriteNugget(CLOSE_BRACKET, sb, response);
    }
    
    /// <summary>
    /// This method generates the closing tag of this html element.  If the
    /// providesClosingTag method returns false, then this method will never be
    /// called.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendClosingTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(END_TAG, sb, response);
    }
    
    /// <summary>
    /// Indicates whether or not this HTML element provides a closing tag. Some
    /// elements only provide opening tags, such as the &lt;br&gt; element.
    /// </summary>
    /// <returns>boolean true.</returns>
    protected override bool ProvidesClosingTag()
    {
      return true;
    }
    
    /// <summary>
    /// Get the next id value.  This is used by the GetClone method to ensure
    /// that cloned object have new ID attribute values.
    /// </summary>
    /// <returns>A valid id string.</returns>
    internal override String GetNextID()
    {
      String s = BDO + iNextID;
      iNextID++;
      return s;
    }
  }
}