using System;
using com.plumtree.openlog;
using com.plumtree.openfoundation.io;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// All object representing HTML elements should subclass this class.
  /// <br /><br />
  /// Defined within this abstract class are methods for getting and setting
  /// attributes common to all HTML elements, such as ID, NAME, CLASS, and
  /// STYLE.
  /// <br /><br />
  /// Other methods, such as appendInnerHTML and appendTabs, will help
  /// subclasses display themselves.  Most all of the code for displaying HTML
  /// and generating HTML strings is located in this class.  For the most part,
  /// subclasses will not need to concern themselves with this functionality, but
  /// only with implementing the abstract methods of this class.  Exceptions to this
  /// are those subclasses that need to modify their display in some way, such
  /// as the HTMLElementCollection class.  This class displays much like any other
  /// HTMLElement object, except it does not display any HTML itself, but only that
  /// HTML generated by its children.  For this reason, the HTMLElementCollection
  /// class needs to override the display methods of this class.  Other classes
  /// may not support HTML formatting, such as the HTMLAnchor and HTMLTextArea
  /// class.  Formatting of the HTML generated by these classes affects the 
  /// display in a bad way, so formatting is not supported, and display methods
  /// need to be overridden.
  /// <br /><br />
  /// This class also provides methods and data structures for managing nested 
  /// HTML objects, such as addInnerHTMLElement and getInnerHTMLElements.
  /// <br /><br />
  /// </summary>
  public abstract class HTMLElement : ICloneable
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.xpshared.htmlelements.HTMLElement");
    //  some attributes common to most HTML elements.  These attributes are not
    //  strictly global, and in cases where a specific element should not
    //  display a given attribute it is the responsibility of the subclass to
    //  throw an exception if the client attempts to display said attribute.
    private String sClass;
    private String sStyle;
    private String sID;
    private String sName;
    private String sTitle;
    protected bool bShowLang;
    private bool bDisabled;
    private String sOnClick;
    private String sOnMouseOver;
    private String sOnMouseOut;
    private String sOnKeyUp;
    private String sOnKeyDown;
    private String sOnKeyPress;
    // Since the Display Options are set on every HTMLElement, if someone
    // caches an HTMLElement on the session or application, this object
    // will be leaked (retained) until the session or application is cleared.
    // There may also be some multi-threading issues when using a cached
    // HTMLElement for multiple users (the login page Auth Source dropdown
    // is stored on the application), since each user will attempt to set
    // their own Display Options on the cached HTMLElement.
    //
    // A good solution for this problem is to convert the DisplayOptions
    // helper class into a Visitor Pattern that traverses the HTMLTree
    // without changing anything.  Since nothing is changed and all the
    // state is kept in the Visitor, there are no multi-threading issues
    // when sharing HTMLElements across users.
    private DisplayOptions oDisplayOptions;
    private bool bHTMLElementReadOnly = false;
    //	Display pausing variables
    //	These members are used to record the display state of the html tree
    //	when one or more of the elements pauses after display.  This feature
    //	can be used to output the beginning of the html tree to the browser
    //	when other nested pieces are still being generated.
    //	This is the last child of this node that fully rendered.  This is set
    //	if the child itself pauses after display.
    private HTMLElement eLastChildFullyRendered = null;
    //	This is the child that paused in mid-render.  This is used if a child's 
    //	child has paused, and our child is not finished rendering.
    private HTMLElement eChildInMidRender = null;
    //	This memeber indicates that this element must pause after it displays 
    //	itself
    protected bool bPauseAfterDisplay = false;
    //	This member indicates that display has paused.
    protected bool bDisplayDidPause = false;
    //	This member indicates that display is resuming, after a previous pause, 
    //	and a child of this node
    //	must be called on to render once again.
    protected bool bResumingAndChildInMidRender = false;
    //	This member indicates that display is resuming, after a previous pause, 
    //	and one of this elements immmediate children paused, and display should
    //	resume with the next child.
    protected bool bResumingAndChildFinished = false;
    internal static readonly byte[] BEGIN_COMMENT = ToByteArray("<!--");
    internal static readonly byte[] END_COMMENT = ToByteArray("-->");
    internal static readonly byte[] CLOSE_BRACKET = ToByteArray(">");
    internal static readonly byte[] SLASH_BRACKET = ToByteArray("/>");
    internal static readonly byte[] CLASS_ATTR = ToByteArray(" class=\"");
    internal static readonly byte[] STYLE_ATTR = ToByteArray(" style=\"");
    internal static readonly byte[] NAME_ATTR = ToByteArray(" name=\"");
    internal static readonly byte[] TITLE_ATTR = ToByteArray(" title=\"");
    internal static readonly byte[] ID_ATTR = ToByteArray(" id=\"");
    internal static readonly byte[] LANG_ATTR = ToByteArray(" lang=\"");
    internal static readonly byte[] ONCLICK_ATTR = ToByteArray(" onclick=\"");
    internal static readonly byte[] ONMOUSEOVER_ATTR = ToByteArray(" onmouseover=\"");
    internal static readonly byte[] ONMOUSEOUT_ATTR = ToByteArray(" onmouseout=\"");
    internal static readonly byte[] ONKEYUP_ATTR = ToByteArray(" onkeyup=\"");
    internal static readonly byte[] ONKEYDOWN_ATTR = ToByteArray(" onkeydown=\"");
    internal static readonly byte[] ONKEY_PRESS_ATTR = ToByteArray(" onkeypress=\"");
    internal static readonly byte[] QUOTE = ToByteArray("\"");
    //   This is used to identify this element, this does not affect html output.
    private String sElementName;
    //  This is the index of the level at which this element is nested within
    //  the HTML page.  The HTMLPage class itself should always have a level of
    //  zero.  HTMLElement objects nested immediately within the HTMLPage should
    //  have a level of 1.  Level is calculated at display-time.
    private int iLevel;
    //   This is the index of the level at which this element is tabbed.
    //   Generally, a nested element is tabed at a level one greater than the
    //   element within which it's nested.  The exceptions for this are
    //   the <head> and <body> elements.
    //   TabLevel is calculated at display-time.
    private int iTabLevel;
    private bool bFlushAfterDisplay = false;
    //   This is the number of <table> elements nesting this element.  This is
    //   only calculated if a maximum table depth is being enforced.  This too
    //   is calculated at display-time.
    private int iTableLevel;
    
    /// <summary>
    /// This int member specifies the maximum depth of table nesting.  This 
    /// is not enforced if bEnforceMaxTableDepth is false.
    /// </summary>
    public const int iMaxTableDepth = 5;
    
    /// <summary>
    /// Set this member variable to specify whether or not a maximum table 
    /// depth is enforced.  If this member is true, then the display methods 
    /// will throw exceptions if there are tables nested too deeply.  See 
    /// iMaxTableDepth.
    /// </summary>
    public static bool bEnforceMaxTableDepth = true;
    
    /// <summary>
    /// Set this member variable to specify whether or not comments are 
    /// printed by the display methods.
    /// </summary>
    public static bool bPrintComments = true;
    
    /// <summary>
    /// Set this member variable to specify whether or not the SetStyleClass
    /// method throws an HTMLException when called with a String parameter.
    /// If this member is set false, SetStyleClass( String ) will always throw
    /// an HTMLException.  This is intended to force use of the corresponding 
    /// method SetStyleClass( StyleClass ), to ensure valid style classes.
    /// </summary>
    public static bool bAllowSetStyleClassWithString = false;
    //public static final String sNewLine = System.getProperty( "line.separator" );
    
    /// <summary>
    /// Set this member to specify the newline character.  If output is being 
    /// formatted, this character is appended frequently.  If output is not 
    /// being formatted, this character is not used.
    /// </summary>
    public static String sNewLine = "\n";
    public static byte[] aNewLine = ToByteArray(sNewLine);
    // The code that uses this is commented out.
    
    /// <summary>
    /// Set this boolean true to cause HTMLElements to write warnings to the
    /// log file if nested strings contain HTML.
    /// </summary>
    public static bool bCheckNestedStringForHTML = false;
    
    /// <summary>
    /// Set this boolean true to force formatting of for this element as
    /// well as nested elements.
    /// </summary>
    public bool bForceFormattingOff = false;
    public bool bForceFormattingOn = false;
    
    /// <summary>
    /// This member variable to specify the tab character.  If output is
    /// being formatted, this character is appended zero or more times to the 
    /// beginning of each line.
    /// </summary>
    public static String sTab = "  ";
    //   public static String sTab = "\t";
    //   Each element of this array contains a string representing the number of
    //   tabs corresponding to the index of the array element.  This is used to
    //   prevent unnecessary String concatenations.
    private static readonly String[] sTabArray;
    //   This is the maximum depth to which html can be tabbed.
    private const int iMaxTabDepth = 20;
    static HTMLElement()
    //   Here we initialize the array of tab strings.
    {
      sTabArray = new String[iMaxTabDepth];
      XPStringBuilder sbTemp = new XPStringBuilder(iMaxTabDepth * sTab.Length);
      sbTemp.Append(sTab);
      sTabArray[0] = "";
      for (int x = 1;x < iMaxTabDepth;x++)
      {
        sbTemp.Append(sTab);
        sTabArray[x] = sbTemp.ToString();
      }
    }
    //  whether or not to format the HTML output for greater readability
    private bool bFormat;
    // Wrap the inner elements in a private class for performance optimizations.
    private InnerElementList innerElements = new InnerElementList();
    //    //   We share the logger for efficiency.
    //    private static XPLogger myLogger = XPLogManager.GetLogger(HTMLElement.class);
    //
    //    //   pre-determining which levels are enabled allows the jvm to avoid
    //    //   concatenating parameters of the logging methods.
    //    private static boolean bDebug            = myLogger.IsDebugEnabled();
    //    private static boolean bError            = myLogger.IsErrorEnabled();
    //    private static boolean bWarn             = myLogger.IsWarnEnabled();
    //    private static boolean bInfo             = myLogger.IsInfoEnabled();
    //    private static boolean bFatal            = myLogger.IsFatalEnabled();
    //private static boolean bLogClone         = true;
    //   comment displayed before the begining tag
    private String sPreComment;
    //   comment displayed after the ending tag
    private String sPostComment;
    internal static ValidationLevel myValidationLevel = HTMLPage.ValidationLevel.MEDIUM;
    
    /// <summary>
    /// Toggle this false to log errors when clients attempt to
    /// set values for attributes such as onclick, and onmouseover.
    /// </summary>
    public static bool bAllowEvents = true;
    
    /// <summary>
    /// sLangLocal is used when an element's lang attribute is different from the global
    /// lang attribute. For example a search result page may display results in languages
    /// different from the page itself.
    /// </summary>
    protected String sLangLocal = null;
    
    /// <summary>
    /// Creates a new HTMLElement object.
    /// </summary>
    internal HTMLElement()
    {
      sClass = null;
      sStyle = null;
      sID = null;
      sName = null;
      sTitle = null;
      bShowLang = false;
      bDisabled = false;
      sOnClick = null;
      sOnMouseOver = null;
      sOnMouseOut = null;
      sOnKeyUp = null;
      sOnKeyDown = null;
      sOnKeyPress = null;
      sPreComment = null;
      sPostComment = null;
      iLevel = 0;
      iTabLevel = 0;
      iTableLevel = 0;
      bFormat = false;
      sElementName = null;
    }
    protected virtual void SetHTMLElementReadOnly(bool b)
    {
      bHTMLElementReadOnly = b;
    }
    protected virtual bool GetHTMLElementReadOnly()
    {
      return bHTMLElementReadOnly;
    }
    
    /// <summary>
    /// This method controls whether or not to use line breaks
    /// after elements where it does not affect the HTML output.
    /// This can only be used on a top-level HTMLElement (usually
    /// an HTMLPage).
    /// </summary>
    /// <param name="b">boolean true implies use line breaks.</param>
    public virtual void SetForceLineBreaksAfterSafeElements(bool b)
    {
      GetDisplayOptions().SetForceLineBreaksAfterSafeElements(b);
    }
    
    /// <summary>
    /// This method returns whether or not to use line breaks
    /// after elements where it does not affect the HTML output.
    /// </summary>
    /// <returns>boolean true implies use line breaks.</returns>
    public virtual bool GetForceLineBreaksAfterSafeElements()
    {
      return GetDisplayOptions().GetForceLineBreaksAfterSafeElements();
    }
    public virtual void SetTitle(String s)
    {
      sTitle = s;
    }
    public virtual String GetTitle()
    {
      return sTitle;
    }
    
    /// <summary>
    /// Set the default language available for all child elements -- not necessarily
    /// displayed unless the element is set to display the language attribute. 
    /// </summary>
    /// <param name="s">String value</param>
    public virtual void SetLangGlobal(String s)
    {
      GetDisplayOptions().SetLang(s);
    }
    
    /// <summary>
    /// Returns the language for use in the attribute, either the local language
    /// if available, or the global language.
    /// </summary>
    /// <returns>String value</returns>
    public virtual String GetLang()
    {
      if (null != sLangLocal)
      {
        return sLangLocal;
      }
      return GetDisplayOptions().GetLang();
    }
    
    /// <summary>
    /// Sets the value of the language attribute used by this element
    /// not the parent or child element(s).
    /// </summary>
    /// <param name="s">String value</param>
    public virtual void SetLangLocal(String s)
    {
      bShowLang = true;
      sLangLocal = s;
    }
    
    /// <summary>
    /// This method sets whether or not the browser is Netscape.
    /// Certain HTMLElements will display differently on Netscape
    /// versus other browsers.
    /// This can only be used on a top-level HTMLElement (usually
    /// an HTMLPage).
    /// </summary>
    /// <param name="b">boolean true implies the browser is Netscape</param>
    public virtual void SetIsNetscape(bool b)
    {
      GetDisplayOptions().SetIsNetscape(b);
    }
    
    /// <summary>
    /// This method returns whether or not the browser is Netscape.
    /// </summary>
    /// <returns>boolean true implies the browser is Netscape.</returns>
    public virtual bool GetIsNetscape()
    {
      return GetDisplayOptions().GetIsNetscape();
    }
    
    /// <summary>
    /// Call this method to cause the IXPResponse.Flush() method to be
    /// called after this element displays itself.  This will only
    /// cause a flushing of the response buffer if the IXPResponse object
    /// is passed to the display method.
    /// </summary>
    /// <param name="b">boolean</param>
    public virtual void SetFlushAfterDisplay(bool b)
    {
      bFlushAfterDisplay = b;
    }
    
    /// <summary>
    /// Call this method to specify whether or not this element pauses after display.
    /// If you pass true, after this html element fully renders, display will
    /// stop.  The resulting HTML will not be complete.  One of the display methods
    /// must be called again, and rendering will resume with the html immediately
    /// after this element.
    /// Call GetDisplayPaused() to determine whether or not the previous call to 
    /// one of the display methods resulted in a pause.
    /// </summary>
    /// <param name="b">boolean indicating whether or not this node pauses.</param>
    public virtual void SetPauseAfterDisplay(bool b)
    {
      bPauseAfterDisplay = b;
    }
    
    /// <summary>
    /// Call this method to determine whether or not this element pauses after
    /// display.
    /// </summary>
    /// <returns>boolean value</returns>
    public virtual bool GetPauseAfterDisplay()
    {
      return bPauseAfterDisplay;
    }
    
    /// <summary>
    /// Call this method to obtain a reference to the last child that fully rendered.
    /// </summary>
    /// <returns>HTMLElement child</returns>
    protected virtual HTMLElement GetLastChildFullyRendered()
    {
      return eLastChildFullyRendered;
    }
    
    /// <summary>
    /// Call this method to set the member reference to the last child fully 
    /// rendered.
    /// </summary>
    /// <param name="e">HTMLElement child</param>
    protected virtual void SetLastChildFullyRendered(HTMLElement e)
    {
      eLastChildFullyRendered = e;
    }
    
    /// <summary>
    /// Call this method to obtain a reference to the child that is in mid render.
    /// </summary>
    /// <returns>HTMLElement child</returns>
    protected virtual HTMLElement GetChildInMidRender()
    {
      return eChildInMidRender;
    }
    
    /// <summary>
    /// Call this method to set the member reference to the child that is in
    /// mid render.
    /// </summary>
    /// <param name="e">HTMLElement child</param>
    protected virtual void SetChildInMidRender(HTMLElement e)
    {
      eChildInMidRender = e;
    }
    
    /// <summary>
    /// Call this method to determine whether or not the most recent call to one
    /// of the display methods paused.  If this method returns true, a display
    /// method must be called again to render a complete HTML page. 
    /// </summary>
    /// <returns></returns>
    public virtual bool GetDisplayDidPause()
    {
      return bDisplayDidPause;
    }
    
    /// <summary>
    /// This method sets whether or not to do 508 validity
    /// checking for HTMLElements.  If this is true, HTMLElements
    /// will try to enforce that the developer does not make HTML
    /// that is not 508 compliant.
    /// This can only be used on a top-level HTMLElement (usually
    /// an HTMLPage).
    /// </summary>
    /// <param name="b">boolean true implies enforce 508 validity.</param>
    public virtual void SetDo508Checking(bool b)
    {
      GetDisplayOptions().SetDo508Checking(b);
    }
    
    /// <summary>
    /// This method returns whether or not to do 508 validity
    /// checking for HTMLElements.  If this is true, HTMLElements
    /// will try to enforce that the developer does not make HTML
    /// that is not 508 compliant.
    /// </summary>
    /// <returns>boolean true implies enforce 508 validity.</returns>
    public virtual bool GetDo508Checking()
    {
      return GetDisplayOptions().GetDo508Checking();
    }
    internal virtual bool GetDo508CheckingForThisInstance()
    {
      bool bDoCheck = false;
      DisplayOptions options = this.GetDisplayOptions();
      if (options != null)
      {
        bDoCheck = options.GetDo508Checking();
      }
      return bDoCheck;
    }
    
    /// <summary>
    /// Call this method to obtain the base url used for all HTMLImg objects
    /// being displayed
    /// </summary>
    /// <returns>String base url.</returns>
    public virtual String GetImgBaseUrl()
    {
      return GetDisplayOptions().GetImgBaseUrl();
    }
    
    /// <summary>
    /// Call this method to set the base url used for all HTMLImg objects being
    /// displayed.
    /// <br /><br />
    /// Call this method only on the root HTMLElement object.  If you are building
    /// an HTMLPage, for example, call myPage.SetImgBaseUrl( "http://blah/" );.
    /// All nested HTMLImg objects may use the base url specified on the HTMLPage
    /// object.
    /// <br /><br />
    /// HTMLImg objects may or may not use this base url specified.
    /// This can only be used on a top-level HTMLElement (usually
    /// an HTMLPage).
    /// </summary>
    /// <param name="s">
    /// Pass a base url string.  It should look something like: 
    /// "http://blah/".
    /// </param>
    public virtual void SetImgBaseUrl(String s)
    {
      GetDisplayOptions().SetImgBaseUrl(s);
    }
    
    /// <summary>
    /// This method returns the style class prefix used for all
    /// HTMLElement objects being displayed.
    /// </summary>
    /// <returns>The base URL used for style classes.</returns>
    public virtual String GetStyleClassPrefix()
    {
      return GetDisplayOptions().GetStyleClassPrefix();
    }
    
    /// <summary>
    /// Call this method to set the prefix used for all HTMLElement objects being
    /// displayed.
    /// <br /><br />
    /// Call this method only on the root HTMLElement object.  If you are building
    /// an HTMLPage, for example, call myPage.SetStyleClassPrefix( "PREFIX" );.
    /// All nested objects may use the prefix specified on the HTMLPage
    /// object.
    /// <br /><br />
    /// This can only be used on a top-level HTMLElement (usually
    /// an HTMLPage).
    /// </summary>
    /// <param name="s">
    /// Pass a base url string.  It should look something like: 
    /// "http://blah/".
    /// </param>
    public virtual void SetStyleClassPrefix(String s)
    {
      GetDisplayOptions().SetStyleClassPrefix(s);
    }
    
    /// <summary>
    /// Call this method to set the display options for this HTML tree.
    /// <br /><br />
    /// This method is called internally during the execution of display 
    /// methods.
    /// </summary>
    /// <param name="options">DisplayOptions object.</param>
    protected virtual void SetDisplayOptions(DisplayOptions options)
    {
      oDisplayOptions = options;
    }
    
    /// <summary>
    /// Helper method to do lazy initialization of display options.
    /// Call this method to get the display options for display of this
    /// HTML tree.  This method will only return a meaningful value
    /// during the execution of the display methods.  This value will be
    /// cleared when display is finished.
    /// </summary>
    /// <returns>DisplayOptions the display options for this HTML tree.</returns>
    protected virtual DisplayOptions GetDisplayOptions()
    {
      // NOTE: Lazy initialization of this object is pretty important,
      // although we should probably just remove it entirely.  It's
      // called 1500 times for an average empty mypage, and if we do
      // lazy initialization, it's only created 4 times, so even though it's
      // a small object, it was taking up about .5% of total page render
      // time.
      if (oDisplayOptions == null)
      {
        // TODOCR: DONH: This probably doesn't need to be here.  The comments
        // about this variable state that it should only be valid during display.
        // Also, the displayoptions from the HTMLPage override all lower ones,
        // so if someone changes the displayoptions of this element, their changes
        // could be lost.
        oDisplayOptions = new DisplayOptions();
      }
      return oDisplayOptions;
    }
    
    /// <summary>
    /// Call this method to give this element a name.  The name does not 
    /// affect the HTML output of this element, the name is purely for 
    /// identification puposes.
    /// </summary>
    /// <param name="s">Pass a String.</param>
    public virtual void SetElementName(String s)
    {
      sElementName = s;
    }
    
    /// <summary>
    /// Call this method to determine the name of this element.
    /// </summary>
    public virtual String GetElementName()
    {
      return sElementName;
    }
    
    /// <summary>
    /// Calling this method sets a private static field that determines
    /// whether or not a maximum table depth is enforced.  This is either
    /// true for all html elements, or false for all html elements.  
    /// Calling this method affects a global change.
    /// </summary>
    /// <param name="b">Pass a boolean.</param>
    internal virtual void SetEnforceMaxTableDepth(bool b)
    {
      bEnforceMaxTableDepth = b;
    }
    
    /// <summary>
    /// Concrete subclasses are responsible for providing a valid 
    /// element-type-specific id.
    /// </summary>
    internal abstract String GetNextID();
    
    /// <summary>
    /// Do not call this method.  This is implemented only so that internal
    /// clone methods can call clone() on HTMLElement objects.
    /// </summary>
    public virtual Object Clone()
    {
      return base.MemberwiseClone();
    }
    
    /// <summary>
    /// Call this method to obtain a clone of this HTMLElement object.
    /// </summary>
    public virtual Object GetClone()
    {
      //        if (bInfo && bLogClone)
      //        {
      //            myLogger.Info("Entering GetClone(), this: " + this);
      //        }
      HTMLElement eNewClone = null;
      try
      {
        eNewClone = (HTMLElement) base.MemberwiseClone();
      }
      catch (Exception ex)
      {
        log.Error(ex, "Unable to clone element.");
      }
      //        if (bDebug && bLogClone)
      //        {
      //            myLogger.Debug("e after clone: " + e);
      //        }
      if (eNewClone == null)
      {
        return null;
      }
      if (eNewClone.GetID() != null)
      {
        try
        {
          eNewClone.SetID(eNewClone.GetNextID());
        }
        catch (HTMLException he)
        {
          log.Error(he, "Unable to clone element.");
        }
      }
      // Make sure the clone doesn't have any direct references to our inner elements.
      // Right now the new clone has a link to the original inner elements list, so we
      // need to make sure we don't modify that.
      eNewClone.innerElements = new InnerElementList();
      // If we only have a single inner element
      if (innerElements.HasSingleElement())
      {
        // Just clone that
        eNewClone.innerElements.AddInnerHTMLElement((HTMLElement) innerElements.GetSingleElement().GetClone());
        } else
      {
        // Otherwise we need to clone the entire contents of the inner list
        // into a new list.
        // Get our inner elements
        XPArrayList al = GetInner();
        if (al != null)
        {
          // Create a new array list and clone all the elements of the
          // old list into this new list.
          XPArrayList newInner = new XPArrayList();
          eNewClone.SetInner(newInner);
          IXPEnumerator xpEnumerator = al.GetEnumerator();
          HTMLElement eInner;
          HTMLElement eChildClone;
          // Clone our inner elements into a new list for the clone.
          while (xpEnumerator.MoveNext())
          {
            eInner = (HTMLElement) xpEnumerator.GetCurrent();
            //                if (bDebug && bLogClone)
            //                {
            //                    myLogger.Debug("eInner: " + eInner);
            //                }
            eChildClone = null;
            try
            {
              eChildClone = (HTMLElement) eInner.GetClone();
            }
            catch (Exception ex)
            {
              log.Error(ex, "Unable to clone inner element.");
            }
            //                if (bDebug && bLogClone)
            //                {
            //                    myLogger.Debug("eClone: " + eClone);
            //                }
            if (eChildClone != null)
            {
              //                    if (bDebug && bLogClone)
              //                    {
              //                        myLogger.Debug("about to add eClone to newInner. i: " + i);
              //                    }
              try
              {
                newInner.Add(eChildClone);
              }
              catch (Exception ex)
              {
                log.Error(ex, "Unable to add inner clone.");
              }
            }
          }
        }
      }
      //   eClone != null
      //   while enum.MoveNext()
      //   al != null
      //        if (bDebug && bLogClone)
      //        {
      //            myLogger.Debug("e before return: " + e);
      //        }
      return eNewClone;
    }
    
    /// <summary>
    /// Call this method to obtain an XPArrayList containing all nested
    /// HTMLElement objects.  This method returns null if there are no
    /// nested HTMLElement objects.
    /// </summary>
    internal virtual XPArrayList GetInner()
    {
      // NOTE: This will create an XPArrayList if one doesn't already exist.
      return innerElements.GetInnerHTMLElements();
    }
    
    /// <summary>
    /// Call this method to set the internal XPArrayList of nested HTMLElement
    /// objects.
    /// Warning: This will erase all elements previously added to this element
    /// through calls to AddInnerHTMLElement.
    /// </summary>
    /// <throws>HTMLException if this element is read only.</throws>
    protected virtual void SetInner(XPArrayList al)
    {
      VerifyWritability();
      innerElements.SetInnerHTMLElements(al);
    }
    
    /// <summary>
    /// Concrete subclasses must implement this method.  This method is 
    /// responsible for appending the begining of the opening tag, 
    /// for example: &lt;tagname attribute="blah" 
    /// <br /><br />
    /// Common attributes will be automatically appended to the begninning
    /// of the opening tag.
    /// </summary>
    /// <param name="bFormat">Pass a boolean indicating whether or not to format.</param>
    /// <param name="sb">
    /// Pass an XPStringBuilder.  If this is not null, HTML will 
    /// be appended to this string builder.
    /// </param>
    /// <param name="response">
    /// Pass an IXPResponse.  If this is not null, HTML will
    /// be written to this response.
    /// </param>
    protected abstract void AppendBeginningOfOpeningTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response);
    
    /// <summary>
    /// Concrete subclasses must implement this method.  This method is
    /// responsible for appending the closing tag.
    /// </summary>
    /// <param name="bFormat">Pass a boolean indicating whether or not to format.</param>
    /// <param name="sb">
    /// Pass an XPStringBuilder.  If this is not null, HTML will 
    /// be appended to this string builder.
    /// </param>
    /// <param name="response">
    /// Pass an IXPResponse.  If this is not null, HTML will
    /// be written to this response.
    /// </param>
    protected abstract void AppendClosingTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response);
    
    /// <summary>
    /// Implement this method to indicate whether a concrete subclass of 
    /// HTMLElement provides a closing tag.  Elements such as BR do not
    /// provide closing tags, they are self-closing.
    /// </summary>
    protected abstract bool ProvidesClosingTag();
    
    /// <summary>
    /// This method saves the display string of this object to a temporary
    /// file.
    /// <br /><br />
    /// </summary>
    /// <returns>The absolute path of the file saved.</returns>
    public virtual String SaveToTempFile()
    {
      return SaveToTempFile("temp.html");
    }
    
    /// <summary>
    /// This method saves the display string of this object to a temporary
    /// file.
    /// <br /><br />
    /// </summary>
    /// <param name="sFileName">
    /// Pass the name of the file within which you wish to 
    /// save this HTMLPage.
    /// <br /><br />
    /// </param>
    /// <returns>The absolute path of the file saved.</returns>
    public virtual String SaveToTempFile(String sFileName)
    {
      //        if (bInfo)
      //        {
      //            myLogger.Info("Entering saveToTempFile()");
      //        }
      String sPage = null;
      try
      {
        sPage = GetDisplayString();
      }
      catch (HTMLException he)
      {
        log.Error(he, "Unable to save to temp file.");
      }
      XPFile htmlFile = null;
      XPFileWriter myWriter;
      try
      {
        //  create a temporary file for the HTML.
        htmlFile = new XPFile(sFileName);
        myWriter = htmlFile.CreateText();
        //  write HTML to the temp file.
        myWriter.Write(sPage);
        myWriter.Close();
      }
      //            if (bDebug)
      //            {
      //                myLogger.Debug("PATH FOR FILE IS: " + htmlFile.GetFullPath());
      //            }
      catch (Exception e)
      {
        log.Error(e, "Unable to write element to temp file.");
      }
      return htmlFile.GetFullPath();
    }
    
    /// <summary>
    /// Call this method to set the setonclick attribute
    /// </summary>
    /// <param name="s">Pass the value of this attribute.</param>
    public virtual void SetOnClick(String s)
    {
      if (!HTMLElement.bAllowEvents)
      {
      }
      //    		if( bError )
      //    		{
      //    			String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
      //    			
      //				Exception e = new Exception( sDesc );
      //				
      //				myLogger.Error( sDesc, e );
      //								
      //    		}	
      sOnClick = s;
    }
    
    /// <summary>
    /// Call this method to obtain the value of the setonclick attribute.
    /// </summary>
    public virtual String GetOnClick()
    {
      return sOnClick;
    }
    
    /// <summary>
    /// Call this method to set the onmouseover attribute
    /// </summary>
    /// <param name="s">Pass the value of this attribute.</param>
    public virtual void SetOnMouseOver(String s)
    {
      //    	if( !HTMLElement.bAllowEvents )
      //    	{
      //    		if( bError )
      //    		{
      //    			String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
      //    			
      //				Exception e = new Exception( sDesc );
      //				
      //				myLogger.Error( sDesc, e );
      //								
      //    		}	
      //    	}
      sOnMouseOver = s;
    }
    
    /// <summary>
    /// Call this method to obtain the value of the onmouseover attribute.
    /// </summary>
    public virtual String GetOnMouseOver()
    {
      return sOnMouseOver;
    }
    
    /// <summary>
    /// Call this method to set the onmouseout attribute
    /// </summary>
    /// <param name="s">Pass the value of this attribute.</param>
    public virtual void SetOnMouseOut(String s)
    {
      //    	if( !HTMLElement.bAllowEvents  )
      //    	{
      //    		if( bError )
      //    		{
      //    			String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
      //    			
      //				Exception e = new Exception( sDesc );
      //				
      //				myLogger.Error( sDesc, e );
      //								
      //    		}	
      //    	}
      sOnMouseOut = s;
    }
    
    /// <summary>
    /// Call this method to obtain the value of the onmouseout attribute.
    /// </summary>
    public virtual String GetOnMouseOut()
    {
      return sOnMouseOut;
    }
    
    /// <summary>
    /// Call this method to set the onkeyup attribute
    /// </summary>
    /// <param name="s">Pass the value of this attribute.</param>
    public virtual void SetOnKeyUp(String s)
    {
      //    	if( !HTMLElement.bAllowEvents )
      //    	{
      //    		if( bError )
      //    		{
      //    			String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
      //    			
      //				Exception e = new Exception( sDesc );
      //				
      //				myLogger.Error( sDesc, e );
      //								
      //    		}	
      //    	}
      sOnKeyUp = s;
    }
    
    /// <summary>
    /// Call this method to obtain the value of the onkeyup attribute.
    /// </summary>
    public virtual String GetOnKeyUp()
    {
      return sOnKeyUp;
    }
    
    /// <summary>
    /// Call this method to set the onkeydown attribute
    /// </summary>
    /// <param name="s">Pass the value of this attribute.</param>
    public virtual void SetOnKeyDown(String s)
    {
      //    	if( !HTMLElement.bAllowEvents )
      //    	{
      //    		if( bError )
      //    		{
      //    			String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
      //    			
      //				Exception e = new Exception( sDesc );
      //				
      //				myLogger.Error( sDesc, e );
      //								
      //    		}	
      //    	}
      sOnKeyDown = s;
    }
    
    /// <summary>
    /// Call this method to obtain the value of the onkeydown attribute.
    /// </summary>
    public virtual String GetOnKeyDown()
    {
      return sOnKeyDown;
    }
    
    /// <summary>
    /// Call this method to set the onkeypress attribute
    /// </summary>
    /// <param name="s">Pass the value of this attribute.</param>
    public virtual void SetOnKeyPress(String s)
    {
      //    	if( !HTMLElement.bAllowEvents  )
      //    	{
      //    		if( bError )
      //    		{
      //    			String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
      //    			
      //				Exception e = new Exception( sDesc );
      //				
      //				myLogger.Error( sDesc, e );
      //								
      //    		}	
      //    	}
      sOnKeyPress = s;
    }
    
    /// <summary>
    /// Call this method to obtain the value of the onkeypress attribute.
    /// </summary>
    public virtual String GetOnKeyPress()
    {
      return sOnKeyPress;
    }
    
    /// <summary>
    /// Use this method to set a comment that will be written before the
    /// opening tag of this element.
    /// </summary>
    public virtual void SetPreComment(String s)
    {
      sPreComment = s;
    }
    
    /// <summary>
    /// Call this method to obtain the comment that is written before the
    /// opening tag of this element.
    /// </summary>
    public virtual String GetPreComment()
    {
      return sPreComment;
    }
    
    /// <summary>
    /// Use this method to set a comment that will be written after the
    /// closing tag of this element.
    /// </summary>
    public virtual void SetPostComment(String s)
    {
      sPostComment = s;
    }
    
    /// <summary>
    /// Call this method to obtain the comment that is written after the
    /// closing tag of this element.
    /// </summary>
    public virtual String GetPostComment()
    {
      return sPostComment;
    }
    
    /// <summary>
    /// Call this method to explicitly set the index of the level at which this
    /// element is nested within the HTML page.
    /// <br /><br />
    /// For HTMLElement objects added to the collection of inner HTML objects
    /// throgh calls to addInnerHTMLElement() and addInnerHTMLString(), it is
    /// not necessary to call setLevel( ... ).  The setLevel() method is called
    /// automagically by appendInnerHTML( ... ) at display time.  If elements
    /// can be added to an HTMLElement
    /// object in other ways, such as through HTMLPage.setBody(), it will be
    /// necessary to explicitly call HTMLElement.setLevel() on the added inner
    /// HTML object.
    /// <br /><br />
    /// </summary>
    /// <param name="i">
    /// This parameter indicates the index of the nesting level of this
    /// HTMLElement object.  The top level HTML element, HTMLPage, should have
    /// an index of 0.
    /// </param>
    internal virtual void SetLevel(int i)
    {
      //        if (bInfo)
      //        {
      //            myLogger.Info("SetLevel - i: " + i);
      //        }
      iLevel = i;
    }
    
    /// <summary>
    /// This package private method is used to set the tab level for an 
    /// HTMLElement object.  The tab level determines how many tabs are 
    /// printed before the element tag if this HTMLElement object is formatting
    /// its output.
    /// </summary>
    /// <param name="i">Pass an int indicating how many tabs to display.</param>
    internal virtual void SetTabLevel(int i)
    {
      //        if (bInfo)
      //        {
      //            myLogger.Info("SetTabLevel - i: " + i);
      //        }
      //
      //        if (bDebug)
      //        {
      //            myLogger.Debug("SetTabLevel - this: " + this);
      //        }
      iTabLevel = i;
    }
    
    /// <summary>
    /// This package private method is used to set the table level of a
    /// given HTMLElement object.  The table level indicates how many tables
    /// contain the given HTML object.
    /// </summary>
    /// <param name="i">
    /// Pass an int indicating how many tables contain this
    /// object.
    /// </param>
    internal virtual void SetTableLevel(int i)
    {
      //        if (bInfo)
      //        {
      //            myLogger.Info("SetTableLevel - i: " + i);
      //        }
      iTableLevel = i;
    }
    
    /// <summary>
    /// Call this method to determine the nesting level of this HTML element.
    /// The outer page should have a level index of 0, the &lt;body&gt; within this
    /// page should have an index of 1, and so on, etc.
    /// <br /><br />
    /// Note that this index is calculated at display time.  So if you create a
    /// structure of HTMLElement objects, calling the getLevel() method will not
    /// necessarily return meaning information until one of the display methods
    /// is called.
    /// </summary>
    /// <returns>int indicating the nesting level</returns>
    internal virtual int GetLevel()
    {
      return iLevel;
    }
    
    /// <summary>
    /// This package private method returns the tab level of this HTMLElement
    /// object.  The tab level indicates how many tabs are printed before this
    /// element tag.
    /// </summary>
    /// <returns>An int indicating the tab level of this object.</returns>
    internal virtual int GetTabLevel()
    {
      return iTabLevel;
    }
    
    /// <summary>
    /// Call this method to determine the number of tables containing this
    /// element.
    /// <br /><br />
    /// Elements not within any tables will return 0.  Elements within a single
    /// table will return 1, etc.
    /// <br /><br />
    /// Table level is calculated at display time.  The value returned by this 
    /// method will only be meaningful after the containg HTMLPage has been 
    /// displayed.
    /// </summary>
    internal virtual int GetTableLevel()
    {
      return iTableLevel;
    }
    
    /// <summary>
    /// Call this method to determine whether or not this HTML element generates
    /// HTML text formatted for readabilty.
    /// </summary>
    public virtual bool GetDoFormat()
    {
      return bFormat;
    }
    
    /// <summary>
    /// Call this method to specify whether or not this HTML element generates
    /// HTML text formatted for readabilty.
    /// </summary>
    public virtual void SetDoFormat(bool b)
    {
      bFormat = b;
    }
    
    /// <summary>
    /// Call this method to force formatting off for this HTMLElement object.
    /// <br /><br />
    /// Whether or not HTML is formatted generally percolates down through an
    /// HTMLElement structure.  
    /// </summary>
    public virtual void ForceFormattingOff()
    {
      bForceFormattingOff = true;
    }
    protected virtual void AppendClosingLineBreak(XPStringBuilder sb, IXPResponse response)
    {
      bool b = true;
      DisplayOptions options = this.GetDisplayOptions();
      if (options != null)
      {
        if (!options.GetForceLineBreaksAfterSafeElements())
        {
          b = false;
        }
      }
      if (b)
      {
        WriteNugget(HTMLElement.sNewLine, sb, response);
      }
    }
    
    /// <summary>
    /// This method is used to get a reasonable sized
    /// String Builder.  It should be overridden by larger
    /// HTMLElements such as HTMLPage.
    /// </summary>
    /// <returns>XPStringBuilder</returns>
    protected virtual 
    // TODO cvn this could return a byte array and speed everything up
    XPStringBuilder GetStringBuilder()
    {
      // Default string size for smaller HTMLElements.
      // Larger Elements, such as HTMLPage, need to override
      // this to provide a more reasonable default.
      // Using a good default value for the size of this StringBuilder is
      // very important.  Here are some sample values for a quick test of
      // a mypage with one gadget and click through all the options (approx 10 hits)
      // 10000 = 3,761,480 bytes of temporary memory (+ 5,440 for appending)
      // 1000  =   874,432 bytes of temporary memory
      // 500   =   721,480 bytes of temporary memory
      // 250   =   640,840 bytes of temporary memory (+ 229,112 for appending)
      // 100   =   593,480 bytes of temporary memory (+ 261,848 for appending)
      // 50    =   576,840 bytes of temporary memory (+ 290,016 for appending)
      //
      // Here are some sample values for a quick test of the directory
      // (opening a folder, going up, and opening it again, going up, and
      // logging out):
      // 10000 = 4,010,512 bytes of temporary memory (+ 212,516 for appending)
      // 500   =   609,512 bytes of temporary memory
      // 250   =   519,296 bytes of temporary memory (+ 220,664 for appending)
      // 100   =   260,136 bytes of temporary memory (+ 181,876 for appending)
      // 50    =   447,696 bytes of temporary memory (+ 300,224 for appending)
      //
      // Be warned that these were manual tests, and the testing sequence
      // varied slightly between runs (one or two hit difference).  DevPartner
      // was used for these tests.
      //
      // This default value was originally set to 10000 on the assumption that
      // an HTMLPage element would be around 30k or so.  This works well, but
      // not all top level HTMLElements are HTMLPages.  There are many HTMLImgs
      // and other HTMLElements constructed separately in the navigation.  These
      // are more on the order of 100 characters.  In fact, HTMLPages are displayed
      // by passing the IXPResponse around so a string builder is never actually
      // needed.
      //
      // Setting the default to 100 is a good solution, but a better solution
      // is to have certain elements override this default (HTMLPage should still
      // use a default value of around 10000).
      return new XPStringBuilder(100);
    }
    
    /// <summary>
    /// Call this method to obtain a String of HTML markup that represents this
    /// HTML element, and all HTML elements nested within.
    /// </summary>
    /// <throws>HTMLException</throws>
    public virtual String GetDisplayString()
    {
      return GetDisplayString(this.GetDoFormat());
    }
    public virtual String GetInnerHTML()
    {
      // Get a reasonably sized String Builder for this element.
      XPStringBuilder sb = GetStringBuilder();
      this.GetInnerHTML(sb, null, false);
      return sb.ToString();
    }
    public virtual String GetInnerHTMLDisplayString()
    {
      InitializeRootForRendering();
      String sReturn = GetInnerHTML();
      CleanupHTMLSubtreeAfterRendering();
      return sReturn;
    }
    
    /// <summary>
    /// Call this method to display this object's HTML to an IXPResponse object.
    /// </summary>
    /// <param name="response">Pass an IXPResponse object.</param>
    /// <param name="bFormat">
    /// Pass a boolean indicating whether or not you wish this
    /// element's HTML to be formatted.
    /// </param>
    /// <throws>HTMLException If display fails, an HTMLException will be thrown.</throws>
    public virtual void Display(IXPResponse response, bool bFormat)
    {
      SetHTMLElementReadOnly(true);
      InitializeRootForRendering();
      InternalWrite(bFormat, null, response);
      CleanupHTMLSubtreeAfterRendering();
    }
    
    /// <summary>
    /// Call this method to display the HTML of this object to the 
    /// IXPResponse object.
    /// <br /><br />
    /// All HTML of this object, as well as the HTML of any nested 
    /// HTMLElement objects will be displayed.
    /// </summary>
    /// <param name="response">Pass an IXPResponse object.</param>
    public virtual void Display(IXPResponse response)
    {
      SetHTMLElementReadOnly(true);
      Display(response, this.GetDoFormat());
    }
    
    /// <summary>
    /// This protected method is used by this class an all subclasses to
    /// write the smallest divisible units of HTML.  These nuggets of HTML
    /// are either appended to an XPStringBuilder, written to the IXPResponse,
    /// both, or neither.
    /// <br /><br />
    /// </summary>
    /// <param name="s">Pass a String you wish appended or written.</param>
    /// <param name="sb">
    /// Pass an XPStringBuilder to which you wish the
    /// String appended.  Pass null if you do not wish the string appended to
    /// an XPStringBuilder.
    /// </param>
    /// <param name="response">
    /// Pass an IXPResponse to which you wish the String
    /// written.  Pass null if you do not wish the string written to an
    /// IXPResponse.
    /// </param>
    protected virtual void WriteNugget(String s, XPStringBuilder sbLocal,
     IXPResponse response)
    {
      if (response != null)
      {
        response.GetOutputStream().Write(ToByteArray(s));
      }
      if (sbLocal != null)
      {
        sbLocal.Append(s);
      }
    }
    
    /// <summary>
    /// This protected method is used by this class an all subclasses to
    /// write the smallest divisible units of HTML.  These nuggets of HTML
    /// are either appended to an XPStringBuilder, written to the IXPResponse,
    /// both, or neither.
    /// <p />
    /// This method is similar to the WriteNugget method with the String argument,
    /// but different in that it does not require the response to convert the String
    /// argument to a byte array.
    /// <br /><br />
    /// </summary>
    /// <param name="s">Pass a byte array you wish appended or written.</param>
    /// <param name="sbLocal">
    /// Pass an XPStringBuilder to which you wish the
    /// String appended.  Pass null if you do not wish the string appended to
    /// an XPStringBuilder.
    /// </param>
    /// <param name="response">
    /// Pass an IXPResponse to which you wish the String
    /// written.  Pass null if you do not wish the string written to an
    /// IXPResponse.
    /// </param>
    protected virtual void WriteNugget(byte[] b, XPStringBuilder sbLocal,
     IXPResponse response)
    {
      if (response != null)
      {
        response.GetOutputStream().Write(b);
      }
      if (sbLocal != null)
      {
        // TODO cvn - find a faster way to do this
        for (int i = 0;i < b.Length;i++)
        {
          sbLocal.Append(XPConvert.ToChar(b[i]));
        }
      }
    }
    
    /// <summary>
    /// This method contains the bulk of the logic for writting an HTMLElement.
    /// For subclasses needing to modify their display behavior, this is the
    /// primary method to override.
    /// </summary>
    /// <param name="bFormat">
    /// Pass a boolean indicating whether or not to format
    /// the output.
    /// </param>
    /// <param name="sb">
    /// Pass an XPStringBuilder to which you wish the
    /// String appended.  Pass null if you do not wish the string appended to
    /// an XPStringBuilder.
    /// </param>
    /// <param name="response">
    /// Pass an IXPResponse to which you wish the String
    /// written.  Pass null if you do not wish the string written to an
    /// IXPResponse.
    /// </param>
    protected virtual void InternalWrite(bool bFormat, XPStringBuilder sbLocal,
     IXPResponse response)
    {
      bool bMethodFormat = bFormat;
      if (bForceFormattingOff)
      {
        bMethodFormat = false;
      }
      if (bForceFormattingOn)
      {
        bMethodFormat = true;
      }
      //	These three need to be initialized each time this object
      //	displays itself.  
      bDisplayDidPause = false;
      bResumingAndChildInMidRender = false;
      bResumingAndChildFinished = false;
      if (this.eChildInMidRender != null)
      {
        //	if we have a ref to a child in mid render, then we are resuming
        //	display and a child is in mid render.
        bResumingAndChildInMidRender = true;
      }
      if (this.eLastChildFullyRendered != null)
      {
        //	if we have a ref to a child that has fully finished rendering, then
        //	we are resuming display an a child has finished.
        bResumingAndChildFinished = true;
      }
      if (!bResumingAndChildInMidRender && !bResumingAndChildFinished)
      {
        //	we're not resuming display at all.  None of this elements children
        //	are paused.
        //	PRE COMMENT
        String sComment = GetPreComment();
        if ((sComment != null) && bPrintComments)
        {
          if (bMethodFormat)
          {
            appendTabs(sbLocal, response);
          }
          WriteNugget(BEGIN_COMMENT, sbLocal, response);
          WriteNugget(sComment, sbLocal, response);
          WriteNugget(END_COMMENT, sbLocal, response);
          if (bMethodFormat)
          {
            WriteNugget(aNewLine, sbLocal, response);
          }
        }
        if (bMethodFormat)
        {
          appendTabs(sbLocal, response);
        }
        //   concrete subclasses implement the following
        AppendBeginningOfOpeningTag(bMethodFormat, sbLocal, response);
        //   This method adds all attributes common to all html elements,
        //   such as class="" id="", etc.
        AppendCommonAttributes(sbLocal, response);
        if (this.ProvidesClosingTag())
        {
          //   Concrete subclasses representing elements that have opening
          //   and closing tags return true for the above method.
          //   close the opening tag
          WriteNugget(CLOSE_BRACKET, sbLocal, response);
          if (bMethodFormat)
          {
            WriteNugget(aNewLine, sbLocal, response);
          }
          GetInnerHTML(sbLocal, response, bMethodFormat);
          //	a child paused, we return and perform none of the remaining
          //	display.
          if (bDisplayDidPause)
          {
            return;
          }
          if (bMethodFormat)
          {
            WriteNugget(aNewLine, sbLocal, response);
            appendTabs(sbLocal, response);
          }
          //   append the closing tag
          AppendClosingTag(bMethodFormat, sbLocal, response);
        } else
        {
          //   this particular html element doesn't have innards and a
          //   closing tag, such as "<img>"
          //   just close it.
          WriteNugget(SLASH_BRACKET, sbLocal, response);
        }
        if (bMethodFormat)
        {
          WriteNugget(aNewLine, sbLocal, response);
        }
        //	POST COMMENT
        sComment = GetPostComment();
        if ((sComment != null) && bPrintComments)
        {
          if (bMethodFormat)
          {
            appendTabs(sbLocal, response);
          }
          WriteNugget(BEGIN_COMMENT, sbLocal, response);
          WriteNugget(sComment, sbLocal, response);
          WriteNugget(END_COMMENT, sbLocal, response);
          if (bMethodFormat)
          {
            WriteNugget(aNewLine, sbLocal, response);
          }
        }
      } else
      //	not resuming
      {
        // We're resuming display, either one of our children finished,
        //	and the next child must be displayed, or one of our children's
        //	children finished, and that children must resume.
        //	This code is basically like the above code executed when we're
        //	not resuming, except the beginning of the tag is not rendered.
        GetInnerHTML(sbLocal, response, bMethodFormat);
        // 	one of our children paused, do not perform any of the remaining
        //	display	
        if (bDisplayDidPause)
        {
          return;
        }
        if (bMethodFormat)
        {
          WriteNugget(aNewLine, sbLocal, response);
          appendTabs(sbLocal, response);
        }
        //   append the closing tag
        AppendClosingTag(bMethodFormat, sbLocal, response);
        if (bMethodFormat)
        {
          WriteNugget(aNewLine, sbLocal, response);
        }
        //	POST COMMENT
        String sComment = GetPostComment();
        if ((sComment != null) && bPrintComments)
        {
          if (bMethodFormat)
          {
            appendTabs(sbLocal, response);
          }
          WriteNugget(BEGIN_COMMENT, sbLocal, response);
          WriteNugget(ToByteArray(sComment), sbLocal, response);
          WriteNugget(END_COMMENT, sbLocal, response);
          if (bMethodFormat)
          {
            WriteNugget(aNewLine, sbLocal, response);
          }
        }
      }
      //	resuming
      if (bFlushAfterDisplay)
      {
        if (response != null)
        {
          response.Flush();
        }
      }
    }
    
    /// <summary>
    /// This is only called if this element is the root of the display tree.
    /// </summary>
    private void InitializeRootForRendering()
    {
      bResumingAndChildInMidRender = false;
      bResumingAndChildFinished = false;
    }
    
    /// <summary>
    /// This is only called if this element is the root of the display tree.
    /// </summary>
    public virtual void CleanupHTMLSubtreeAfterRendering()
    {
    }
    // This was designed to allow cleanup of the HTMLElements underneat
    // the root, but never did this. -- DonH
    
    /// <summary>
    /// This method is the internal String generator for all HTMLElement
    /// classes.  This method may be overridden by subclasses, but this 
    /// shouldn't be necessary.
    /// <br /><br />
    /// </summary>
    /// <param name="bFormat">
    /// This parameter toggles whether or not all formating
    /// text is included in the display string.
    /// </param>
    protected virtual void GetDisplayString(bool bFormat, XPStringBuilder sbLocal)
    {
      InitializeRootForRendering();
      InternalWrite(bFormat, sbLocal, null);
      CleanupHTMLSubtreeAfterRendering();
    }
    
    /// <summary>
    /// This method is the internal String generator for all HTMLElement
    /// classes.  This method may be overridden by subclasses, but this 
    /// shouldn't be necessary.
    /// <br /><br />
    /// </summary>
    /// <param name="bFormat">
    /// This parameter toggles whether or not all formating
    /// text is included in the display string.
    /// </param>
    protected virtual String GetDisplayString(bool bFormat)
    {
      //        if (bInfo)
      //        {
      //            myLogger.Info("Entering GetDisplayString() - bFormat: " + bFormat);
      //        }
      // Get a reasonably sized String Builder for this element.
      XPStringBuilder sb = GetStringBuilder();
      this.GetDisplayString(bFormat, sb);
      return sb.ToString();
    }
    
    /// <summary>
    /// Call this method to Set the style class for this HTML element.  Calling
    /// this method with the parameter "gContentSection", for example, will cause
    /// this element to display something like: &lt;td class="gContentSection"...
    /// </summary>
    public virtual void SetStyleClass(String s)
    {
      sClass = s;
    }
    
    /// <summary>
    /// Call this method to Set the style class for this HTML element.  Calling
    /// this method with the parameter "gContentSection", for example, will cause
    /// this element to display something like: &lt;td class="gContentSection"...
    /// </summary>
    public virtual void SetStyleClass(StyleClass s)
    {
      sClass = ((s == null) ? null : s.GetStyleName());
    }
    
    /// <summary>
    /// Call this method to determine the style class assigned to this HTML
    /// element.  This style class will be rendered as an attribute of this
    /// element, like: &lt;a class="styleclass"...
    /// </summary>
    public virtual String GetStyleClass()
    {
      return sClass;
    }
    
    /// <summary>
    /// Call this method to set the style attribute for this html element.
    /// Calling this method with the parameter "text-decoration:none"
    /// for example, will cause this element to display something like:
    /// &lt;a style="text-decoration:none"...
    /// </summary>
    /// <param name="s"></param>
    public virtual void SetStyle(String s)
    {
      sStyle = s;
    }
    
    /// <summary>
    /// Call this method to obtain the value of the style attribute of this
    /// HTML element.
    /// </summary>
    /// <returns>String indicating the style attribute value.</returns>
    public virtual String GetStyle()
    {
      return sStyle;
    }
    
    /// <summary>
    /// Call this method to set the ID attribute for this HTML element.
    /// <br /><br />
    /// ID attributes may not contain spaces or dollar signs or anything weird.
    /// ID attributes must be unique to each element on a given HTML page.
    /// This is currently not enforced by this method.
    /// </summary>
    /// <param name="s">Pass the ID you wish to use for this element.</param>
    public virtual void SetID(String s)
    {
      bool bInvalidID = false;
      //  todo - better validation?
      if (XPStringUtility.IndexOf(s, " ") != -1)
      {
        bInvalidID = true;
      }
      if (XPStringUtility.IndexOf(s, "$") != -1)
      {
        bInvalidID = true;
      }
      if (bInvalidID)
      {
        throw new HTMLException("An element id may not contain spaces or dollar signs.  You were attempting to set the id to: " + s);
        }
      sID = s;
    }
    
    /// <summary>
    /// Call this method to determine the value of the id attribute of this
    /// HTML element.
    /// </summary>
    public virtual String GetID()
    {
      return sID;
    }
    
    /// <summary>
    /// Call this method to set the value of the name attribute of this HTML
    /// element.
    /// </summary>
    public virtual void SetName(String s)
    {
      sName = s;
    }
    
    /// <summary>
    /// Call this method to determine the value of the name attribute of this
    /// HTML element.
    /// </summary>
    public virtual String GetName()
    {
      return sName;
    }
    
    /// <summary>
    /// Call this method to toggle the "disabled" attribute for this element.
    /// </summary>
    /// <param name="b"></param>
    /// <throws>HTMLException</throws>
    public virtual void SetDisabled(bool b)
    {
      bDisabled = b;
    }
    
    /// <summary>
    /// Call this method to determine whether or not the disabled attribute is
    /// displayed for this HTML element.
    /// </summary>
    public virtual bool GetDisabled()
    {
      return bDisabled;
    }
    
    /// <summary>
    /// This method checks whether or not this tree is read only and throws an
    /// exception if it is not.
    /// </summary>
    /// <throws>HTMLException if this element is read only.</throws>
    protected virtual void VerifyWritability()
    {
      if (this.GetHTMLElementReadOnly())
      {
        throw new HTMLException("The HTMLElement (" + this.GetName() + ") which the client is attempting to change (i.e. add an HTMLElement) has already previously displayed.  This tree is Read-Only and may not be modified.");
        }
    }
    
    /// <summary>
    /// Call this method to append an additional HTML element to be displayed
    /// within this HTML element.  For example, if this is a &lt;body&gt; element,
    /// and you append an &lt;img&gt; element, and then you append an &lt;a&gt; element,
    /// the body element will display something like: 
    /// "&lt;body&gt;&lt;img&gt;&lt;a&gt;&lt;/body&gt;"
    /// <br /><br />
    /// Not all HTML elements allow you to append inner HTML elements.  Such
    /// elements will throw an exception if you attempt to do this.
    /// </summary>
    /// <throws>HTMLException if this element is read only.</throws>
    public virtual void AddInnerHTMLElement(HTMLElement e)
    {
      //   WARNING: Don't add any code to this method that needs to
      //   know anything about the structure of the HTML tree involved.
      //   This method can be called by the client at any time, and there is
      //   no guarantee that this object has been added to its intended parent.
      //   Perform such checking at display time.
      //        if (bInfo)
      //        {
      //            myLogger.Info("Entering addInnerHTMLElement()");
      //            myLogger.Info("this: " + this);
      //        }
      VerifyWritability();
      innerElements.AddInnerHTMLElement(e);
    }
    
    /// <summary>
    /// Adds an innerHTMLElement to the specified element, while also returning the 
    /// element added.
    /// </summary>
    /// <throws>HTMLException if this element is read only.</throws>
    public virtual HTMLElement InsertInnerHTMLElement(HTMLElement e)
    {
      AddInnerHTMLElement(e);
      return e;
    }
    
    /// <summary>
    /// Call this method to remove all nested html elements from this element.
    /// </summary>
    /// <throws>HTMLException if this element is read only.</throws>
    public virtual void ClearInnerHTMLElements()
    {
      VerifyWritability();
      innerElements.ClearInnerHTMLElements();
    }
    
    /// <summary>
    /// Call this method to get an XPArrayList containing all nested
    /// HTMLElement objects.
    /// </summary>
    /// <param name="bRecursive">
    /// Pass a boolean indicating whether or not you wish this
    /// list to be recursive.
    /// </param>
    public virtual XPArrayList GetInnerHTMLElementsAsArrayList(bool bRecursive)
    {
      //        if (bInfo)
      //        {
      //            myLogger.Info("Entering GetInnerHTMLElementsAsArrayList() - bRecursive: " + bRecursive);
      //            myLogger.Info("this: " + this);
      //        }
      if (innerElements.IsEmpty())
      {
        return null;
      }
      // NOTE: This will create an XPArrayList if one doesn't already exist, so we
      // need to do it after the check if the list is empty.
      XPArrayList alReturn = innerElements.GetInnerHTMLElements();
      //   todo - this might not be as efficient as possible.  Perhaps
      //   using Object arrays would be better.  This would yield less
      //   superfluous construction of classes, but would be error prone,
      //   and require dynamic resizing of arrays and stuff.
      if (bRecursive)
      {
        if (alReturn != null)
        {
          // Keep an iterator for the current inner elements so we can
          // iterate over them while we're filling in the new return
          // array below.
          IXPEnumerator e = alReturn.GetEnumerator();
          // NOTE: We don't really need to worry about this array list
          // because it is only called by straggler HTMLElements (i.e.
          // elements created to build a small piece of HTML and
          // immediately display it), rather than the main HTMLElement
          // display method (which does not display recursively like
          // this).
          alReturn = new XPArrayList();
          HTMLElement eTemp;
          HTMLElement eTemp2;
          IXPEnumerator eInner;
          while (e.MoveNext())
          {
            //   for each inner element
            eTemp = (HTMLElement) e.GetCurrent();
            alReturn.Add(eTemp);
            //   Get the inner elements recursively
            eInner = eTemp.GetInnerHTMLElements(true);
            if (eInner != null)
            {
              while (eInner.MoveNext())
              {
                //   Add all recursively gathered inner elements
                //   to the new arraylist.
                eTemp2 = (HTMLElement) eInner.GetCurrent();
                alReturn.Add(eTemp2);
              }
            }
          }
        }
      }
      //   for each recursive inner element
      //   eInner != null
      //   for each inner element
      //   alInner != null
      //   if bRecursive
      return alReturn;
    }
    
    /// <summary>
    /// Call this method to obtain an enumeration of the HTMLElement objects
    /// nested within this HTMLElement.
    /// <br /><br />
    /// </summary>
    /// <param name="bRecursive">
    /// Pass true to obtain an enumeration of all nested
    /// elements.  Pass false to obtain an enumeration of only those elements
    /// nested immediately within this element.
    /// </param>
    public virtual IXPEnumerator GetInnerHTMLElements(bool bRecursive)
    {
      //        if (bInfo)
      //        {
      //            myLogger.Info("Entering GetInnerHTMLElements() - bRecursive: " + bRecursive);
      //            myLogger.Info("this: " + this);
      //        }
      XPArrayList eTemp = this.GetInnerHTMLElementsAsArrayList(bRecursive);
      IXPEnumerator eReturn = null;
      if (eTemp != null)
      {
        eReturn = eTemp.GetEnumerator();
      }
      return eReturn;
    }
    
    /// <summary>
    /// Call this method to append an additional HTML String to be displayed
    /// within this HTML element.  For example, if this is a &lt;body> element,
    /// and you append the "&lt;img&gt;" String, and then you append the "&lt;a&gt;" String,
    /// the body element will display something like: "&lt;body&gt;&lt;img&gt;&lt;a&gt;&lt;/body&gt;"
    /// <br /><br />
    /// Not all HTML elements allow you to append inner HTML Strings.  Such
    /// elements will throw an exception if you attempt to do this.
    /// <br /><br />
    /// The String appended is not validated.  You are free to append garbage.
    /// This String will not be HTMLEncoded, therefore this method will be
    /// faster than addInnerHTMLEncodedString().  All localized strings should
    /// be safe and should therefore use this method.
    /// If a string contains some text that should be encoded, and some that should
    /// not, it should be manually encoded and this method should be used instead of
    /// addInnerHTMLEncodedString().
    /// </summary>
    /// <param name="s">Pass a String representation of a valid HTML element.</param>
    /// <throws>HTMLException</throws>
    public virtual void AddInnerHTMLString(String s)
    {
      //        if (bCheckNestedStringForHTML)
      //        {
      //            if (CheckStringForHTML(s))
      //            {
      //                if (bError)
      //                {
      //                    myLogger.Error("The string being added to this element contains HTML.");
      //                    myLogger.Error("If possible, this HTML should be generated using HTMLElement objets.");
      //                    myLogger.Error("s: " + s);
      //                    myLogger.Error("this: " + this);
      //                }
      //
      //            }
      //        }
      HTMLGenericElement eTemp = new HTMLGenericElement(s);
      AddInnerHTMLElement(eTemp);
    }
    
    /// <summary>
    /// Call this method to append an additional HTML String to be displayed
    /// within this HTML element.  For example, if this is a &lt;body> element,
    /// and you append the "&lt;img&gt;" String, and then you append the "&lt;a&gt;" String,
    /// the body element will display something like: "&lt;body&gt;&lt;img&gt;&lt;a&gt;&lt;/body&gt;"
    /// <br /><br />
    /// Not all HTML elements allow you to append inner HTML Strings.  Such
    /// elements will throw an exception if you attempt to do this.
    /// <br /><br />
    /// The String appended is not validated.  You are free to append garbage.
    /// This String will be automatically HTMLEncoded for you.  This method will
    /// be significantly slower than addInnerHTMLString(), so it should only be
    /// used for data that really needs to be HTMLEncoded (things entered by users).
    /// This method should not be used for localized strings, which should be safe.
    /// If a string contains some text that should be encoded, and some that should
    /// not, it should be manually encoded and addInnerHTMLString() should be used.
    /// </summary>
    /// <param name="s">Pass a String representation of a valid HTML element.</param>
    /// <throws>HTMLException if this element is read only</throws>
    public virtual void AddInnerHTMLEncodedString(String s)
    {
      VerifyWritability();
      // Warning: this call is expensive when it is used too much.
      // No need to check string for HTML, since it has been encoded.
      HTMLGenericElement eTemp = new HTMLGenericElement(XPHttpUtility.HtmlEncode(s));
      AddInnerHTMLElement(eTemp);
    }
    
    /// <summary>
    /// Call this method to obtain the HTML nested within this element.  This
    /// HTML does not contain the HTML for this element.
    /// </summary>
    /// <param name="sbLocal">
    /// Pass an XPStringBuilder.  Pass null if you do not wish
    /// for the HTML string to be added to a buffer.
    /// </param>
    /// <param name="IXPResponse">
    /// Pass an IXPResponse.  Pass null if you do not wish 
    /// for the HTML string to be written to the response.
    /// </param>
    /// <param name="bFormat">
    /// Pass a boolean indicating whether or not to format this
    /// HTML.
    /// </param>
    public virtual void GetInnerHTML(XPStringBuilder sbLocal, IXPResponse response,
     bool bFormatLocal)
    {
      if (innerElements.IsEmpty())
      {
        // Nothing to display, so avoid creating an extra XPArrayList.
        return;
      }
      if (innerElements.HasSingleElement())
      {
        // NOTE: By writing the single element out directly rather than
        // getting an array list with only 1 item, we drop our count of
        // XPArrayLists created in HTMLElement from 800 to 141 for a single
        // empty mypage (saving us 250k of temporary memory).  The tested
        // empty mypage had 143 HTMLElements with multiple inner elements,
        // 658 HTMLElements with only a single inner element, and 156
        // HTMLElements with no inner elements.  Items with no inner
        // HTMLElements already do lazy initialization of the
        // XPArrayList so we don't need to optimize them further.
        // False & false just means we didn't pause earlier in rendering,
        // which is appropriate because this is the first and only sub-element
        // of this element.
        WriteElement(innerElements.GetSingleElement(), false, false, sbLocal,
         response, bFormatLocal);
        // Only a single item, so no need to loop
        return;
      }
      IXPEnumerator e = this.GetInnerHTMLElements(false);
      HTMLElement element;
      bool bFoundChildInMidRender = false;
      bool bFoundLastChildFullyRendered = false;
      if (e != null)
      {
        while (e.MoveNext())
        {
          element = (HTMLElement) e.GetCurrent();
          bool bResult = WriteElement(element, bFoundChildInMidRender, bFoundLastChildFullyRendered,
           sbLocal, response, bFormatLocal);
          if (!bResult)
          {
            // We are paused, so stop displaying items.
            break;
          }
        }
      }
    }
    //   while( e.MoveNext() )
    //   e != null
    //   display is over, reset this to null.
    
    /// <param name="element"></param>
    /// <param name="bFoundChildInMidRender"></param>
    /// <param name="bFoundLastChildFullyRendered"></param>
    /// <param name="sbLocal"></param>
    /// <param name="response"></param>
    /// <param name="bFormatLocal"></param>
    /// <returns>false means stop writing elements</returns>
    private bool WriteElement(HTMLElement element, bool bFoundChildInMidRender,
     bool bFoundLastChildFullyRendered, XPStringBuilder sbLocal, IXPResponse response,
     bool bFormatLocal)
    {
      if (element != null)
      {
        //	This element may no longer be modified
        element.SetHTMLElementReadOnly(true);
        if (bResumingAndChildInMidRender && !bFoundChildInMidRender)
        {
          //	We're resuming display, one of our children is in mid
          //	render.  Elements before this child must be skipped, and display
          //	should resume with this child.
          if (element == this.eChildInMidRender)
          {
            bFoundChildInMidRender = true;
            this.eChildInMidRender = null;
          }
          if (!bFoundChildInMidRender)
          {
            return true;
          }
        }
        if (this.bResumingAndChildFinished && !bFoundLastChildFullyRendered)
        {
          //	We're resuming display, one of our children is fully rendered.
          //	display must resume with the child immediately following that
          //	child.
          if (element == this.eLastChildFullyRendered)
          {
            this.eLastChildFullyRendered = null;
            bFoundLastChildFullyRendered = true;
          }
          return true;
        }
        if (!bPrintComments)
        {
          if (element is HTMLComment)
          {
            return true;
          }
        }
        //   for each nested element, the display options are the
        //   same as our display options.
        element.SetDisplayOptions(GetDisplayOptions());
        //   If the inner element is an HTMLHead or HTMLBody, then
        //   the tab level is 0.  Otherwise, the tab level is the
        //   tab level of this, plus 1.
        if (!(element is HTMLHead) && !(element is HTMLBody))
        {
          element.SetTabLevel(iTabLevel + 1);
        }
        //   The real level is always the level of this plus 1.
        element.SetLevel(iLevel + 1);
        //   We don't set the table level unless we're going to enforce a maximum.
        //   This is purely for performance purposes.
        if (bEnforceMaxTableDepth)
        {
          if (element is HTMLTable)
          {
            //   If the child element is an HTMLTable, then it's
            //   one table-level deeper.
            element.SetTableLevel(iTableLevel + 1);
          } else
          {
            //   If it's anything else, it's at the same table level.
            element.SetTableLevel(iTableLevel);
          }
        }
        element.InternalWrite(bFormatLocal, sbLocal, response);
        if (bFormatLocal)
        {
          WriteNugget(aNewLine, sbLocal, response);
        }
        if (element.GetDisplayDidPause())
        {
          // The child did pause, a subchild of this child paused, and
          //	our child is in mid-render.  We too are paused.	
          this.SetChildInMidRender(element);
          bDisplayDidPause = true;
          return false;
        }
        if (element.GetPauseAfterDisplay())
        {
          //	Display must pause after the element just
          //	rendered.
          this.SetLastChildFullyRendered(element);
          bDisplayDidPause = true;
          return false;
        }
      }
      return true;
    }
    
    /// <summary>
    /// This method may be used by subclasses of HTMLElement to append
    /// common attributes to a StringBuffer containing a String describing an
    /// HTML element.  Common attributes are those attributes set and get through
    /// this class, the HTMLElement class.  All common attributes that are
    /// set to non-null values will be appended to the passed StringBuffer.
    /// <br /><br />
    /// It is presumed that the StringBuffer passed in already contains the
    /// opening of an HTML tag, such as "&lt;form ".  Note that the tag opening
    /// ends with a space, making the StringBuffer ready to accept the
    /// appendage of any common attributes.
    /// </summary>
    /// <param name="sbLocal">
    /// Pass an XPStringBuilder.  Pass null if you do not wish
    /// for the HTML string to be added to a buffer.
    /// </param>
    /// <param name="IXPResponse">
    /// Pass an IXPResponse.  Pass null if you do not wish 
    /// for the HTML string to be written to the response.
    /// </param>
    internal virtual void AppendCommonAttributes(XPStringBuilder sb, IXPResponse response)
    {
      if (sClass != null)
      {
        String sFinalStyle = sClass;
        DisplayOptions options = this.GetDisplayOptions();
        if (options != null)
        {
          String s = options.GetStyleClassPrefix();
          if (s != null)
          {
            sFinalStyle = s + sFinalStyle;
          }
        }
        WriteNugget(CLASS_ATTR, sb, response);
        WriteNugget(sFinalStyle, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sStyle != null)
      {
        WriteNugget(STYLE_ATTR, sb, response);
        WriteNugget(sStyle, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sName != null)
      {
        WriteNugget(NAME_ATTR, sb, response);
        WriteNugget(sName, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sTitle != null)
      {
        WriteNugget(TITLE_ATTR, sb, response);
        WriteNugget(sTitle, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sID != null)
      {
        WriteNugget(ID_ATTR, sb, response);
        WriteNugget(sID, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (bShowLang && (GetLang() != null))
      {
        WriteNugget(LANG_ATTR, sb, response);
        WriteNugget(GetLang(), sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sOnClick != null)
      {
        //        	if( !HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance() )
        //        	{
        //        		if( bError )
        //        		{
        //        			myLogger.Error( "You are not allowed to set the onclick attribute value because the client of this html tree is attempting to render a 508-compliant page." );
        //        			myLogger.Error( "this: " + this );
        //        		}	
        //        	}
        WriteNugget(ONCLICK_ATTR, sb, response);
        WriteNugget(sOnClick, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sOnMouseOver != null)
      {
        //        	if( !HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance() )
        //        	{
        //        		if( bError )
        //        		{
        //        			myLogger.Error( "You are not allowed to set the onmouseover attribute value because the client of this html tree is attempting to render a 508-compliant page." );
        //        			myLogger.Error( "this: " + this );
        //        		}	
        //        	}
        WriteNugget(ONMOUSEOVER_ATTR, sb, response);
        WriteNugget(sOnMouseOver, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sOnMouseOut != null)
      {
        //        	if( !HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance() )
        //        	{
        //        		if( bError )
        //        		{
        //        			myLogger.Error( "You are not allowed to set the onmouseout attribute value because the client of this html tree is attempting to render a 508-compliant page." );
        //        			myLogger.Error( "this: " + this );
        //        		}	
        //        	}
        WriteNugget(ONMOUSEOUT_ATTR, sb, response);
        WriteNugget(sOnMouseOut, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sOnKeyUp != null)
      {
        //        	if( !HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance() )
        //        	{
        //        		if( bError )
        //        		{
        //        			myLogger.Error( "You are not allowed to set the onkeyup attribute value because the client of this html tree is attempting to render a 508-compliant page." );
        //        			myLogger.Error( "this: " + this );
        //        		}	
        //        	}
        WriteNugget(ONKEYUP_ATTR, sb, response);
        WriteNugget(sOnKeyUp, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sOnKeyDown != null)
      {
        //        	if( !HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance() )
        //        	{
        //        		if( bError )
        //        		{
        //        			myLogger.Error( "You are not allowed to set the onkeydown attribute value because the client of this html tree is attempting to render a 508-compliant page." );
        //        			myLogger.Error( "this: " + this );
        //        		}	
        //        	}
        WriteNugget(ONKEYDOWN_ATTR, sb, response);
        WriteNugget(sOnKeyDown, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sOnKeyPress != null)
      {
        //        	if( !HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance() )
        //        	{
        //        		if( bError )
        //        		{
        //        			myLogger.Error( "You are not allowed to set the onkeypress attribute value because the client of this html tree is attempting to render a 508-compliant page." );
        //        			myLogger.Error( "this: " + this );
        //        		}	
        //        	}
        WriteNugget(ONKEY_PRESS_ATTR, sb, response);
        WriteNugget(sOnKeyPress, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
    }
    
    /// <summary>
    /// Call this method to append the appropriate number of tab characters to
    /// the passed XPStringBuilder.
    /// <br /><br />
    /// If HTML is being formatted, subclasses should call this method before
    /// displaying an HTML element tag.
    /// <br /><br />
    /// </summary>
    /// <param name="sbLocal">
    /// Pass an XPStringBuilder.  Pass null if you do not wish
    /// for the HTML string to be added to a buffer.
    /// </param>
    /// <param name="IXPResponse">
    /// Pass an IXPResponse.  Pass null if you do not wish 
    /// for the HTML string to be written to the response.
    /// </param>
    internal virtual void appendTabs(XPStringBuilder sb, IXPResponse response)
    {
      AppendTabs(iTabLevel, sb, response);
    }
    
    /// <summary>
    /// Call this method to append the some number of tab characters to
    /// the passed XPStringBuilder.
    /// <br /><br />
    /// You may pass negative numbers, and no tabs will be appended.
    /// <br /><br />
    /// </summary>
    /// <param name="sbLocal">
    /// Pass an XPStringBuilder.  Pass null if you do not wish
    /// for the HTML string to be added to a buffer.
    /// </param>
    /// <param name="IXPResponse">
    /// Pass an IXPResponse.  Pass null if you do not wish 
    /// for the HTML string to be written to the response.
    /// </param>
    internal virtual void AppendTabs(int iNumTabs, XPStringBuilder sb, IXPResponse response)
    {
      if (iNumTabs > (iMaxTabDepth - 1))
      {
        iNumTabs = iMaxTabDepth - 1;
      }
      WriteNugget(sTabArray[iNumTabs], sb, response);
    }
    
    /// <summary>
    /// Call this method to specify the level at which html validation will 
    /// occur.  Each available level will toggle each of the methods of 
    /// validation that can occur.
    /// <br /><br />
    /// Methods of validation currently include:
    /// <br /><br />
    /// <ul>
    /// <li>Enforcement of maximum table depth.</li>
    /// <li>Validation of HTML on the display of an HTMLPage.</li>
    /// </ul>
    /// <br /><br />
    /// This method is offered as a convenience.  Individual validation methods
    /// can be toggled individually.
    /// <br /><br />
    /// Validation methods are enabled on a package-level-scope.  For example,
    /// if maximum table depth is being enforced, it's enforced for all 
    /// HTMLPage objects created.   Validation methods cannot be toggled on
    /// an object-level-scope.
    /// </summary>
    public static void SetValidationLevel(ValidationLevel l)
    {
      myValidationLevel = l;
      if (l == ValidationLevel.NONE)
      {
        HTMLElement.bEnforceMaxTableDepth = false;
        HTMLPage.bAlwaysValidateHTMLOnDisplay = false;
        HTMLImg.bEnforceAlt = false;
      } else
      if (l == ValidationLevel.LOW)
      {
        HTMLElement.bEnforceMaxTableDepth = false;
        HTMLPage.bAlwaysValidateHTMLOnDisplay = false;
        HTMLImg.bEnforceAlt = true;
      } else
      if (l == ValidationLevel.MEDIUM)
      {
        HTMLElement.bEnforceMaxTableDepth = true;
        HTMLPage.bAlwaysValidateHTMLOnDisplay = false;
        HTMLImg.bEnforceAlt = true;
      } else
      if (l == ValidationLevel.HIGH)
      {
        HTMLElement.bEnforceMaxTableDepth = true;
        HTMLPage.bAlwaysValidateHTMLOnDisplay = true;
        HTMLImg.bEnforceAlt = true;
      } else
      if (l == ValidationLevel.CHECK_508)
      {
        HTMLElement.bAllowEvents = false;
        HTMLImg.bEnforceAlt = true;
        HTMLTable.bEnforceSummary = true;
        HTMLAnchor.bEnforceTitle = true;
      }
    }
    
    /// <summary>
    /// Convert a String to a byte array using XPStringUtility's default encoding, UTF-8.
    /// If the string is null return an array of zero length.
    /// </summary>
    /// <param name="_String">String to convert to a byte array.</param>
    /// <returns>	a byte array derived from the String argument.</returns>
    public static byte[] ToByteArray(String _String)
    {
      if (null != _String)
      {
        return XPConvert.ToByteArray(_String, XPStringUtility.DEFAULT_ENCODING);
        } else
      {
        return new byte[0];
      }
    }
    
    /// <summary>
    /// Call this method to obtain the ValidationLevel.
    /// </summary>
    public static ValidationLevel GetValidationLevel()
    {
      return myValidationLevel;
    }
    public class ValidationLevel
    {
      
      /// <summary>
      /// No validation will occur.  This is the lickety-splitest option.
      /// </summary>
      public static ValidationLevel NONE = new ValidationLevel("none");
      
      /// <summary>
      /// All possible validation is enabled.  This is very slow. This is for
      /// development purposes only.  Don't ship any product with HIGH 
      /// validation enabled.
      /// </summary>
      public static ValidationLevel HIGH = new ValidationLevel("high");
      public static ValidationLevel MEDIUM = new ValidationLevel("medium");
      public static ValidationLevel LOW = new ValidationLevel("low");
      public static ValidationLevel CHECK_508 = new ValidationLevel("508");
      private static String sLevel;
      
      /// <summary>
      /// Creates a new ValidationLevel object.
      /// </summary>
      /// <param name="s">Pass the String name of this validation level.</param>
      public ValidationLevel(String s)
      {
        sLevel = s;
      }
    }
    
    /// <summary>
    /// This class is used to simplify access to the inner HTML elements,
    /// since they have been optimized for performance.
    /// NOTE: By only storing a single item instead of an array list where
    /// possible, we drop our count of XPArrayLists created in HTMLElement
    /// from 800 to 141 for a single empty mypage (saving us 250kb of
    /// temporary memory).  The tested empty mypage had 143 HTMLElements
    /// with multiple inner elements, 658 HTMLElements with only a single
    /// inner element, and 156 HTMLElements with no inner elements.  Items
    /// with no inner HTMLElements already do lazy initialization of the
    /// XPArrayList so we don't need to optimize them further.
    /// </summary>
    private class InnerElementList
    {
      // NOTE: Either heSingleInner or alInnerList should be non-null,
      // but never both at the same time.  Either we have a single inner
      // element, or we have multiple inner elements, but not both.
      // If they are both null, then there are no inner elements.
      // The single inner element if that's all we have, null otherwise.
      protected HTMLElement heSingleInner = null;
      // The list of inner elements, if we have multiple items, null otherwise.
      protected XPArrayList alInnerList = null;
      
      /// <summary>
      /// WARNING: This will cause creation of an XPArrayList
      /// if one doesn't already exist, unless there are no inner
      /// elements.
      /// </summary>
      /// <returns>the list of inner HTML Elements.</returns>
      public virtual XPArrayList GetInnerHTMLElements()
      {
        if (heSingleInner != null)
        {
          // If we already have a single element, then upgrade our
          // data storage to a list.
          alInnerList = new XPArrayList();
          // Move the previous single inner element to the new list
          alInnerList.Add(heSingleInner);
          // Null out the old single item since it is now in the list.
          heSingleInner = null;
        }
        return alInnerList;
      }
      
      /// <summary>
      /// Set the inner HTML elements, overriding any previous
      /// elements.
      /// </summary>
      /// <param name="_alInner">the new inner elements.</param>
      public virtual void SetInnerHTMLElements(XPArrayList _alInner)
      {
        heSingleInner = null;
        alInnerList = _alInner;
      }
      
      /// <summary>
      /// Add a new item to the inner elements.
      /// </summary>
      /// <param name="_element">the new element to add to the list.</param>
      public virtual void AddInnerHTMLElement(HTMLElement _element)
      {
        if (_element == null)
        {
          return;
        }
        if ((heSingleInner == null) && (alInnerList == null))
        {
          // If we don't have any current inner elements, then just
          // store the single new inner element as an element without
          // creating a new array list. This gives us improved memory
          // performance (temp objects).
          heSingleInner = _element;
        } else
        if ((heSingleInner != null) && (alInnerList == null))
        {
          // If we already have a single element, then upgrade our
          // data storage to a list.  This moves the old single
          // element into the list for us.
          alInnerList = GetInnerHTMLElements();
          // Add the new item to the list.
          alInnerList.Add(_element);
        } else
        {
          // We already have a list, so just add the new item to the list.
          alInnerList.Add(_element);
        }
      }
      
      /// <summary>
      /// Call this method to remove all nested html elements from this element.
      /// </summary>
      public virtual void ClearInnerHTMLElements()
      {
        heSingleInner = null;
        if (alInnerList != null)
        {
          alInnerList.Clear();
          alInnerList = null;
        }
      }
      
      /// <returns>whether or not this list is empty.</returns>
      public virtual bool IsEmpty()
      {
        // Check if we have a single element or list, and the list is
        // empty.
        return (((heSingleInner == null) && (alInnerList == null)) || ((alInnerList != null) && (alInnerList.IsEmpty())));
        }
      
      /// <returns>whether or not this list has only one item.</returns>
      public virtual bool HasSingleElement()
      {
        return (heSingleInner != null);
      }
      
      /// <returns>
      /// the single item in the list. Will return null if
      /// the list is empty or has more than one item.
      /// </returns>
      public virtual HTMLElement GetSingleElement()
      {
        return heSingleInner;
      }
    }
  }
}