using System;
//import com.plumtree.openfoundation.logging.*;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// This class can be used to display an HTML form element. <br />
  /// <br />
  /// <a target="new"
  /// href="http://www.w3.org/TR/html4/interact/forms.html#h-17.3">Spec</a>
  /// </summary>
  public class HTMLForm : HTMLElement
  {
    internal static int iNextID = 0;
    private const String FORM = "form";
    private const String POST = "post";
    private const String GET = "get";
    private static readonly byte[] BEGIN_TAG = HTMLElement.ToByteArray("<form");
    private static readonly byte[] METHOD_ATTR = HTMLElement.ToByteArray(" method=\"");
    private static readonly byte[] ACTION_ATTR = HTMLElement.ToByteArray(" action=\"");
    private static readonly byte[] ONSUBMIT_ATTR = HTMLElement.ToByteArray(" onsubmit=\"");
    private static readonly byte[] ENCTYPE_ATTR = HTMLElement.ToByteArray(" enctype=\"");
    private static readonly byte[] END_TAG = HTMLElement.ToByteArray("</form>");
    private HTMLTable myParentTable = null;
    private String sAction;
    private String sEncType;
    private String sMethod;
    private String sOnSubmit;
    
    /// <param name="sID">Pass the id string of this form.</param>
    /// <throws>HTMLException</throws>
    public HTMLForm(String sID)
    {
      SetID(sID);
      sEncType = null;
      sMethod = null;
      sOnSubmit = null;
      sAction = null;
    }
    
    /// <summary>
    /// Get the value of the action attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetAction()
    {
      return sAction;
    }
    
    /// <summary>
    /// Get the value of the enctype attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetEncType()
    {
      return sEncType;
    }
    
    /// <summary>
    /// Get the value of the method attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetMethod()
    {
      return sMethod;
    }
    public virtual HTMLTable GetParentTable()
    {
      return myParentTable;
    }
    
    /// <summary>
    /// Set the value of the action attribute.
    /// </summary>
    /// <param name="s">A valid String value.</param>
    public virtual void SetAction(String s)
    {
      sAction = s;
    }
    
    /// <summary>
    /// Set the value of the enctype attribute.
    /// </summary>
    /// <param name="s">A valid String value.</param>
    public virtual void SetEncType(String s)
    {
      sEncType = s;
    }
    
    /// <summary>
    /// Set the value of the method attribute.
    /// </summary>
    /// <param name="s">Either "post" or "get".</param>
    /// <throws>HTMLException</throws>
    public virtual void SetMethod(String s)
    {
      if (!((s.Equals(POST)) || (s.Equals(GET))))
      {
        throw new HTMLException("The method on an HTMLForm must be \"get\" or \"post\".");
        }
      sMethod = s;
    }
    
    /// <summary>
    /// Set the value of the onsubmit attribute.
    /// </summary>
    /// <param name="s">String value.</param>
    public virtual void SetOnSubmit(String s)
    {
      if (!HTMLElement.bAllowEvents)
      {
      }
      //            if (bError)
      //            {
      //                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
      //
      //                Exception e = new Exception(sDesc);
      //
      //                myLogger.Error(sDesc, e);
      //            }
      sOnSubmit = s;
    }
    
    /// <summary>
    /// An HTMLForm can be added to an HTMLTable.
    /// </summary>
    /// <param name="table"></param>
    public virtual void SetParentTable(HTMLTable table)
    {
      myParentTable = table;
    }
    
    /// <summary>
    /// Call this method to add a HTMLTableRow object to this table.  The
    /// HTMLTableRow passed as a parameter must have been constructed with a
    /// reference to this very HTMLTable.
    /// </summary>
    /// <param name="e"></param>
    /// <throws>HTMLException</throws>
    public override void AddInnerHTMLElement(HTMLElement e)
    {
      if (this.GetParentTable() != null)
      {
        // this form has a parent table
        //	the following was copied from HTMLTable.
        //	if this form is nested in a table, then adding an
        //	element is just like adding an element to a table.
        if (!(e is HTMLTableRow) && !(e is HTMLElementCollection))
        {
          throw new HTMLException("You may only add HTMLRow or HTMLElementCollection elements to an HTMLForm in an HTMLTable.");
          }
        if (e is HTMLElementCollection)
        {
          IXPEnumerator enumItems = e.GetInnerHTMLElements(false);
          if (enumItems != null)
          {
            HTMLElement eTemp;
            while (enumItems.MoveNext())
            {
              eTemp = (HTMLElement) enumItems.GetCurrent();
              if (!(eTemp is HTMLTableRow))
              {
                throw new HTMLException("You may only add an HTMLElementCollection containing HTMLTableRow elements to an HTMLTable element.  You are attempting to add an: " + eTemp);
                }
              ProcessRowForAdding((HTMLTableRow) eTemp);
            }
          }
        } else
        //   for each inner element
        //   got inner elements
        // e instanceof HTMLElementCollection
        {
          // it's just a row being added
          ProcessRowForAdding((HTMLTableRow) e);
        }
        //   we must validate anew.
        this.myParentTable.Invalidate();
      } else
      {
        // this form has no parent table
        base.AddInnerHTMLElement(e);
      }
    }
    
    /// <summary>
    /// This method adds an HTMLRow to this table.
    /// </summary>
    /// <param name="r">Pass an HTMLRow object.</param>
    /// <throws>
    /// HTMLException This method throws an exception if for any reason
    /// the row passed isn't appropriate.
    /// </throws>
    internal virtual void ProcessRowForAdding(HTMLTableRow r)
    {
      if (r.GetParentTable() == null)
      {
        r.SetParentTable(this.GetParentTable());
      } else
      {
        if (r.GetParentTable() != this.GetParentTable())
        {
          throw new HTMLException("You may only add a given HTMLTableRow to a single given HTMLTable.");
          }
      }
      base.AddInnerHTMLElement(r);
    }
    
    /// <summary>
    /// This method generates the beginning of the opening tag.  The abstract
    /// HTMLElement superclass will provide the remainder of the opening tag,
    /// repleate with attributes common to all html elements, such as name, and
    /// id, and class.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendBeginningOfOpeningTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(BEGIN_TAG, sb, response);
      if (sMethod != null)
      {
        WriteNugget(METHOD_ATTR, sb, response);
        WriteNugget(sMethod, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sAction != null)
      {
        WriteNugget(ACTION_ATTR, sb, response);
        WriteNugget(sAction, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sOnSubmit != null)
      {
        //            if (!HTMLElement.bAllowEvents || this.GetDo508CheckingForThisInstance())
        //            {
        //                if (bError)
        //                {
        //                    myLogger.Error("You are not allowed to set the onsubmit attribute value because the client of this html tree is attempting to render a 508-compliant page.");
        //                    myLogger.Error("this: " + this);
        //                }
        //            }
        WriteNugget(ONSUBMIT_ATTR, sb, response);
        WriteNugget(sOnSubmit, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sEncType != null)
      {
        WriteNugget(ENCTYPE_ATTR, sb, response);
        WriteNugget(sEncType, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
    }
    
    /// <summary>
    /// This method generates the closing tag of this html element.  If the
    /// providesClosingTag method returns false, then this method will never be
    /// called.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendClosingTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(END_TAG, sb, response);
      AppendClosingLineBreak(sb, response);
    }
    
    /// <summary>
    /// Indicates whether or not this HTML element provides a closing tag. Some
    /// elements only provide opening tags, such as the &lt;br&gt; element.
    /// </summary>
    /// <returns>boolean true.</returns>
    protected override bool ProvidesClosingTag()
    {
      return true;
    }
    
    /// <summary>
    /// Get the next id value.  This is used by the GetClone method to ensure
    /// that cloned object have new ID attribute values.
    /// </summary>
    /// <returns>A valid id string.</returns>
    internal override String GetNextID()
    {
      String s = FORM + iNextID;
      iNextID++;
      return s;
    }
  }
}