using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// <br /><br />
  /// <a target="new"
  /// href="http://www.w3.org/TR/html4/present/frames.html#h-16.2.2">Spec</a><br /><br />
  /// </summary>
  public class HTMLFrame : HTMLElement
  {
    private static int iNextID = 0;
    private static readonly byte[] BEGIN_TAG = HTMLElement.ToByteArray("<frame");
    private static readonly byte[] SCROLLING_ATTR = HTMLElement.ToByteArray(" scrolling=\"");
    private static readonly byte[] NORESIZE_ATTR = HTMLElement.ToByteArray(" noresize=\"noresize\" ");
    private static readonly byte[] MARGIN_HEIGHT_ATTR = HTMLElement.ToByteArray(" marginheight=\"");
    private static readonly byte[] SRC_ATTR = HTMLElement.ToByteArray(" src=\"");
    private static readonly byte[] LONGDESC_ATTR = HTMLElement.ToByteArray(" longdesc\"");
    private static readonly byte[] TITLE_ATTR = HTMLElement.ToByteArray(" title\"");
    private static readonly byte[] FRAME_BORDER_ATTR = HTMLElement.ToByteArray(" frameborder=\"");
    private static readonly byte[] MARGIN_WIDTH_ATTR = HTMLElement.ToByteArray(" marginwidth=\"");
    private static readonly byte[] END_TAG = HTMLElement.ToByteArray("</frame>");
    private String sFrameBorder;
    private String sMarginHeight;
    private String sMarginWidth;
    private String sTitle;
    private String sLongDesc;
    private String sScrolling;
    private String sSrc;
    private bool bNoResize;
    
    /// <summary>
    /// Creates a new HTMLFrame object.
    /// </summary>
    public HTMLFrame()
    {
      sSrc = null;
      sFrameBorder = null;
      sMarginWidth = null;
      sMarginHeight = null;
      bNoResize = false;
      sScrolling = null;
      sTitle = null;
      sLongDesc = null;
    }
    public virtual void SetLongDesc(String s)
    {
      sLongDesc = s;
    }
    public virtual String GetLongDesc()
    {
      return sLongDesc;
    }
    public override void SetTitle(String s)
    {
      sTitle = s;
    }
    public override String GetTitle()
    {
      return sTitle;
    }
    
    /// <summary>
    /// Get the value of the frameborder attribute.
    /// </summary>
    /// <returns>String value</returns>
    public virtual String GetFrameBorder()
    {
      return sFrameBorder;
    }
    
    /// <summary>
    /// Get the value of the marginheight attribute.
    /// </summary>
    /// <returns>String value</returns>
    public virtual String GetMarginHeight()
    {
      return sMarginHeight;
    }
    
    /// <summary>
    /// Get the value of the marginwidth attribute.
    /// </summary>
    /// <returns>String value</returns>
    public virtual String GetMarginWidth()
    {
      return sMarginWidth;
    }
    
    /// <summary>
    /// Get the value of the noresize attribute.
    /// </summary>
    /// <returns>boolean value</returns>
    public virtual bool GetNoResize()
    {
      return bNoResize;
    }
    
    /// <summary>
    /// Get the value of the scrolling attribute.
    /// </summary>
    /// <returns>String value</returns>
    public virtual String GetScrolling()
    {
      return sScrolling;
    }
    
    /// <summary>
    /// Get the value of the src attribute.
    /// </summary>
    /// <returns>String value</returns>
    public virtual String GetSrc()
    {
      return sSrc;
    }
    
    /// <summary>
    /// Set the value of the frameborder attribute.
    /// </summary>
    /// <param name="s">A valid String value.</param>
    public virtual void SetFrameBorder(String s)
    {
      // todo enforce 1 | 0
      sFrameBorder = s;
    }
    
    /// <summary>
    /// Set the value of the marginheight attribute.
    /// </summary>
    /// <param name="s">A valid String value.</param>
    public virtual void SetMarginHeight(String s)
    {
      sMarginHeight = s;
    }
    
    /// <summary>
    /// Set the value of the marginwidth attribute.
    /// </summary>
    /// <param name="s">A valid String value.</param>
    public virtual void SetMarginWidth(String s)
    {
      sMarginWidth = s;
    }
    
    /// <summary>
    /// Set the value of the noresize attribute.
    /// </summary>
    /// <param name="b">boolean value.</param>
    public virtual void SetNoResize(bool b)
    {
      bNoResize = b;
    }
    
    /// <summary>
    /// Set the value of the scrolling attribute.
    /// </summary>
    /// <param name="s">A valid String value.</param>
    public virtual void SetScrolling(String s)
    {
      sScrolling = s;
    }
    
    /// <summary>
    /// Set the value of the src attribute.
    /// </summary>
    /// <param name="s">A valid String value.</param>
    public virtual void SetSrc(String s)
    {
      sSrc = s;
    }
    
    /// <summary>
    /// This method generates the beginning of the opening tag.  The abstract
    /// HTMLElement superclass will provide the remainder of the opening tag,
    /// repleate with attributes common to all html elements, such as name, and
    /// id, and class.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendBeginningOfOpeningTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(BEGIN_TAG, sb, response);
      if (sScrolling != null)
      {
        WriteNugget(SCROLLING_ATTR, sb, response);
        WriteNugget(sScrolling, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (bNoResize)
      {
        WriteNugget(NORESIZE_ATTR, sb, response);
      }
      if (sMarginHeight != null)
      {
        WriteNugget(MARGIN_HEIGHT_ATTR, sb, response);
        WriteNugget(sMarginHeight, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sTitle != null)
      {
        WriteNugget(TITLE_ATTR, sb, response);
        WriteNugget(sTitle, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sLongDesc != null)
      {
        WriteNugget(LONGDESC_ATTR, sb, response);
        WriteNugget(sLongDesc, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sSrc != null)
      {
        WriteNugget(SRC_ATTR, sb, response);
        WriteNugget(sSrc, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sFrameBorder != null)
      {
        WriteNugget(FRAME_BORDER_ATTR, sb, response);
        WriteNugget(sFrameBorder, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sMarginWidth != null)
      {
        WriteNugget(MARGIN_WIDTH_ATTR, sb, response);
        WriteNugget(sMarginWidth, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
    }
    
    /// <summary>
    /// This method generates the closing tag of this html element.  If the
    /// providesClosingTag method returns false, then this method will never be
    /// called.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendClosingTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(END_TAG, sb, response);
    }
    
    /// <summary>
    /// Indicates whether or not this HTML element provides a closing tag. Some
    /// elements only provide opening tags, such as the &lt;br&gt; element.
    /// </summary>
    /// <returns>boolean true.</returns>
    protected override bool ProvidesClosingTag()
    {
      return true;
    }
    
    /// <summary>
    /// Get the next id value.  This is used by the GetClone method to ensure
    /// that cloned object have new ID attribute values.
    /// </summary>
    /// <returns>A valid id string.</returns>
    internal override String GetNextID()
    {
      String s = "frame" + iNextID;
      iNextID++;
      return s;
    }
  }
}