using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// Classes that implement objects that can be used to obtain Strings that will
  /// render HTML elements that can be used to send name/value pairs in an HTTP
  /// request to a server may extend this abstract class. <br />
  /// <br />
  /// This class implements some functionality common to all such html elements.
  /// <br /><br />
  /// </summary>
  public abstract class HTMLGenericInputElement : HTMLElement
  {
    private static readonly byte[] VALUE_ATTR = HTMLElement.ToByteArray(" value=\"");
    private static readonly byte[] ONBLUR_ATTR = HTMLElement.ToByteArray(" onblur=\"");
    private static readonly byte[] ONCHANGE_ATTR = HTMLElement.ToByteArray(" onchange=\"");
    private static readonly byte[] ONFOCUS_ATTR = HTMLElement.ToByteArray(" onfocus=\"");
    internal String sValue;
    internal String sOnBlur;
    internal String sOnChange;
    internal String sOnFocus;
    private bool bValidateName = true;
    
    /// <summary>
    /// Creates a new HTMLGenericInputElement object.
    /// </summary>
    public HTMLGenericInputElement()
    {
      sOnChange = null;
      sOnFocus = null;
      sOnBlur = null;
      sValue = null;
    }
    public virtual void SetValidateName(bool b)
    {
      bValidateName = b;
    }
    
    /// <summary>
    /// Get the value of the onblur attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetOnBlur()
    {
      return sOnBlur;
    }
    
    /// <summary>
    /// Get the value of the onchange attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetOnChange()
    {
      return sOnChange;
    }
    
    /// <summary>
    /// Get the value of the onfocus attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetOnFocus()
    {
      return sOnFocus;
    }
    
    /// <summary>
    /// Get the value of the value attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetValue()
    {
      return sValue;
    }
    //  override HTMLElement.SetName()
    
    /// <summary>
    /// Override HTMLElement.SetName.
    /// </summary>
    /// <param name="s">String value</param>
    /// <throws>HTMLException Throws Exception if name isn't valid.</throws>
    public override void SetName(String s)
    {
      if (bValidateName)
      {
        ValidateName(s);
      }
      base.SetName(s);
    }
    
    /// <summary>
    /// This method must be implemented by concrete subclasses to throw an
    /// HTMLException if the name attribute of this element does not meet the
    /// standards of this particular concrete class. <br />
    /// <br />
    /// For example, the HTMLInput class implements this method to throw an
    /// HTMLException if the name attribute is not set to begin with "in_"
    /// followed by a two-letter code designating the type of the input. <br />
    /// <br />
    /// The validation that occurs is at the discretion of the subclass, but
    /// should enforce some level of quality. <br /><br />
    /// </summary>
    /// <param name="s"></param>
    /// <throws>HTMLException</throws>
    public abstract void ValidateName(String s);
    
    /// <summary>
    /// Set the value of the onblur attribute.
    /// </summary>
    /// <param name="s">String value</param>
    public virtual void SetOnBlur(String s)
    {
      //        if (!HTMLElement.bAllowEvents)
      //        {
      //            if (bError)
      //            {
      //                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
      //
      //                Exception e = new Exception(sDesc);
      //
      //                myLogger.Error(sDesc, e);
      //            }
      //        }
      sOnBlur = s;
    }
    
    /// <summary>
    /// Set the value of the onchange attribute.
    /// </summary>
    /// <param name="s">String value</param>
    public virtual void SetOnChange(String s)
    {
      //        if (!HTMLElement.bAllowEvents)
      //        {
      //            if (bError)
      //            {
      //                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
      //
      //                Exception e = new Exception(sDesc);
      //
      //                myLogger.Error(sDesc, e);
      //            }
      //        }
      sOnChange = s;
    }
    
    /// <summary>
    /// Set the value of the onfocus attribute.
    /// </summary>
    /// <param name="s">String value</param>
    public virtual void SetOnFocus(String s)
    {
      //        if (!HTMLElement.bAllowEvents)
      //        {
      //            if (bError)
      //            {
      //                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
      //
      //                Exception e = new Exception(sDesc);
      //
      //                myLogger.Error(sDesc, e);
      //            }
      //        }
      sOnFocus = s;
    }
    
    /// <summary>
    /// Set the value of the value attribute.  This value value will be
    /// HTMLEncoded.
    /// </summary>
    /// <param name="s">String value</param>
    /// <throws>HTMLException</throws>
    public virtual void SetValue(String s)
    {
      sValue = XPHttpUtility.HtmlEncode(s);
    }
    /* (non-Javadoc)
     * @see com.plumtree.xpshared.htmlelements.HTMLElement#AppendCommonAttributes(com.plumtree.openfoundation.util.XPStringBuilder, com.plumtree.openfoundation.web.IXPResponse)
     */
     
    internal override void AppendCommonAttributes(XPStringBuilder sb, IXPResponse response)
    {
      base.AppendCommonAttributes(sb, response);
      if (sValue != null)
      {
        WriteNugget(VALUE_ATTR, sb, response);
        WriteNugget(sValue, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sOnBlur != null)
      {
        WriteNugget(ONBLUR_ATTR, sb, response);
        WriteNugget(sOnBlur, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sOnChange != null)
      {
        WriteNugget(ONCHANGE_ATTR, sb, response);
        WriteNugget(sOnChange, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sOnFocus != null)
      {
        WriteNugget(ONFOCUS_ATTR, sb, response);
        WriteNugget(sOnFocus, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
    }
  }
}