using System;
//import com.plumtree.openfoundation.logging.*;
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// This class can be used to render an html input element. <br />
  /// <br />
  /// <a target="new" href="http://www.w3.org/TR/html4/interact/forms.html#h-17.4">Spec</a><br />
  /// <br />
  /// This class extends HTMLGeneralInputElement.  This means that objects of
  /// this class can be used to set name/value pairs within a form. <br />
  /// <br />
  /// This class also implements TabableHTMLElement.  This means that this class
  /// can generate a tabindex property that helps to guide the clients browser as
  /// the client tabs through form elements. <br /><br />
  /// </summary>
  public class HTMLInput : HTMLGenericInputElement,TabableHTMLElement
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.xpshared.htmlelements.HTMLInput");
    internal static int iNextID = 0;
    private static readonly byte[] BEGIN_TAG = HTMLElement.ToByteArray("<input type=\"");
    private static readonly byte[] END_TAG = HTMLElement.ToByteArray("</input>");
    private static readonly byte[] ALT_ATTR = HTMLElement.ToByteArray(" alt=\"");
    private static readonly byte[] TAB_INDEX_ATTR = HTMLElement.ToByteArray(" tabindex=\"");
    private static readonly byte[] ONCLICK_ATTR = HTMLElement.ToByteArray(" onclick=\"");
    private static readonly byte[] ONKEYDOWN_ATTR = HTMLElement.ToByteArray(" onkeydown=\"");
    private static readonly byte[] ONCHANGE_ATTR = HTMLElement.ToByteArray(" onchange=\"");
    private static readonly byte[] ALIGN_ATTR = HTMLElement.ToByteArray(" align=\"");
    private static readonly byte[] TITLE_ATTR = HTMLElement.ToByteArray(" title=\"");
    private static readonly byte[] CHECKED_ATTR = HTMLElement.ToByteArray(" checked");
    private static readonly byte[] READ_ONLY_ATTR = HTMLElement.ToByteArray(" readonly=\"readonly\"");
    private static readonly byte[] DISABLED_ATTR = HTMLElement.ToByteArray(" disabled=\"disabled\"");
    private static readonly byte[] SIZE_ATTR = HTMLElement.ToByteArray(" size=\"");
    private static readonly byte[] MAXLENGTH_ATTR = HTMLElement.ToByteArray(" maxlength=\"");
    private static readonly byte[] SRC_ATTR = HTMLElement.ToByteArray(" src=\"");
    private static readonly byte[] ACCESSKEY_ATTR = HTMLElement.ToByteArray(" accesskey=\"");
    private static readonly byte[] AUTOCOMPLETE_OFF_ATTR = HTMLElement.ToByteArray(" autocomplete=\"off\"");
    private HTMLInputTypes type;
    private String sAlign;
    private String sAlt;
    private String sAccessKey;
    private String sMaxLength;
    private String sOnClick;
    private String sOnKeyDown;
    private String sSize;
    private String sSrc;
    private String sTabIndex;
    private bool bChecked;
    private bool bReadOnly;
    private bool bSupportsSubmit;
    private bool bSupportsAccessKey;
    private bool bAutoComplete;
    
    /// <summary>
    /// Set this to toggle whether or not input sizes are converted to
    /// netscape comparable sizes.  If true, the SetSize() method will
    /// look to the root of the display tree to determine whether the
    /// HTML output is intended for an IE browser or a Netscape browser.
    /// </summary>
    public static bool bForceTextInputSizing = true;
    
    /// <param name="t">Pass an HTMLInputTypes object.</param>
    /// <param name="sTagName">Pass the name of this input element.</param>
    /// <param name="sID">Pass the ID of this input element</param>
    /// <param name="alt">Pass the alt attribute value.</param>
    /// <throws>HTMLException</throws>
    public HTMLInput(HTMLInputTypes t, String sTagName, String sID, String alt)
    {
      //        if (bDebug)
      //        {
      //            myLogger.Debug("constructor: t:" + t);
      //            myLogger.Debug("constructor: sTagName:" + sTagName);
      //            myLogger.Debug("constructor: alt:" + alt);
      //        }
      type = t;
      SetName(sTagName);
      SetID(sID);
      SetAlt(alt);
      //  onFocus, onBlur, and onChange inherited from HTMLGenericInputElement
      sAlign = null;
      bChecked = false;
      bAutoComplete = true;
      bReadOnly = false;
      sSize = null;
      sMaxLength = null;
      sSrc = null;
      sValue = "";
      bSupportsSubmit = false;
      bSupportsAccessKey = false;
      sOnClick = null;
      sOnKeyDown = null;
      sTabIndex = null;
      if ((t == HTMLInputTypes.SUBMIT_TYPE) || (t == HTMLInputTypes.BUTTON_TYPE))
      {
        bSupportsSubmit = true;
        bSupportsAccessKey = true;
      }
      // rob: removing the validation checks. main reason is to avoid the validation check on textbox for banner search with friendly url
      // (bug #70204) but it is also not necessary anymore since we are moving to layouts
      SetValidateName(false);
    }
    
    /// <summary>
    /// Returns the HTMLInputTypes object associated with this.
    /// </summary>
    /// <returns>HTMLInputTypes the typ of input.</returns>
    public virtual HTMLInputTypes GetHTMLInputType()
    {
      return type;
    }
    
    /// <param name="iHTMLInputType">Pass an int from the HTMLInputTypes enumeration.</param>
    /// <param name="sTagName">Pass the name of this input element.</param>
    /// <param name="sID">Pass the ID of this input element</param>
    /// <param name="alt">Pass the alt attribute value.</param>
    /// <throws>HTMLException</throws>
    public HTMLInput(int iHTMLInputType, String sTagName, String sID, String alt)
    {
      SetID(sID);
      SetAlt(alt);
      //  onFocus, onBlur, and onChange inherited from HTMLGenericInputElement
      sAlign = null;
      bChecked = false;
      bAutoComplete = true;
      bReadOnly = false;
      sSize = null;
      sMaxLength = null;
      sSrc = null;
      sValue = "";
      bSupportsSubmit = false;
      bSupportsAccessKey = false;
      sOnClick = null;
      sOnKeyDown = null;
      sTabIndex = null;
      switch (iHTMLInputType){
      case HTMLInputTypes.SUBMIT:
      bSupportsSubmit = true;
      bSupportsAccessKey = true;
      type = HTMLInputTypes.SUBMIT_TYPE;
      break;
      case HTMLInputTypes.BUTTON:
      bSupportsSubmit = true;
      bSupportsAccessKey = true;
      type = HTMLInputTypes.BUTTON_TYPE;
      break;
      case HTMLInputTypes.CHECKBOX:
      type = HTMLInputTypes.CHECKBOX_TYPE;
      break;
      case HTMLInputTypes.FILE:
      type = HTMLInputTypes.FILE_TYPE;
      break;
      case HTMLInputTypes.HIDDEN:
      type = HTMLInputTypes.HIDDEN_TYPE;
      break;
      case HTMLInputTypes.IMAGE:
      type = HTMLInputTypes.IMAGE_TYPE;
      break;
      case HTMLInputTypes.PASSWORD:
      type = HTMLInputTypes.PASSWORD_TYPE;
      break;
      case HTMLInputTypes.RADIO:
      type = HTMLInputTypes.RADIO_TYPE;
      break;
      case HTMLInputTypes.RESET:
      type = HTMLInputTypes.RESET_TYPE;
      break;
      case HTMLInputTypes.TEXT:
      type = HTMLInputTypes.TEXT_TYPE;
      break;
      }
      // rob: removing the validation checks. main reason is to avoid the validation check on textbox for banner search with friendly url
      // (bug #70204) but it is also not necessary anymore since we are moving to layouts
      // BT: Bug 70204 - Need to call this before SetName since it calls ValidateName.
      SetValidateName(false);
      SetName(sTagName);
    }
    
    /// <param name="t">Pass an HTMLInputTypes object.</param>
    /// <param name="alt">Pass the alt attribute value.</param>
    /// <throws>HTMLException</throws>
    public HTMLInput(HTMLInputTypes t, String sAlt)
    {
      type = t;
      SetID(this.GetNextID());
      SetAlt(sAlt);
      //  onFocus, onBlur, and onChange inherited from HTMLGenericInputElement
      sAlign = null;
      bChecked = false;
      bAutoComplete = true;
      bReadOnly = false;
      sSize = null;
      sMaxLength = null;
      sSrc = null;
      sValue = "";
      bSupportsSubmit = false;
      bSupportsAccessKey = false;
      sOnClick = null;
      sOnKeyDown = null;
      sTabIndex = null;
      if ((t == HTMLInputTypes.SUBMIT_TYPE) || (t == HTMLInputTypes.BUTTON_TYPE))
      {
        bSupportsSubmit = true;
        bSupportsAccessKey = true;
      }
      // rob: removing the validation checks. main reason is to avoid the validation check on textbox for banner search with friendly url
      // (bug #70204) but it is also not necessary anymore since we are moving to layouts
      SetValidateName(false);
    }
    
    /// <param name="t">Pass an HTMLInputTypes object.</param>
    /// <param name="sTagName">Pass the name of this input element.</param>
    /// <param name="alt">Pass the alt attribute value.</param>
    /// <throws>HTMLException</throws>
    public HTMLInput(HTMLInputTypes t, String sTagName, String alt)
    {
      //        if (bDebug)
      //        {
      //            myLogger.Debug("constructor: t:" + t);
      //            myLogger.Debug("constructor: sTagName:" + sTagName);
      //            myLogger.Debug("constructor: alt:" + alt);
      //        }
      type = t;
      SetName(sTagName);
      SetID(this.GetNextID());
      SetAlt(alt);
      //  onFocus, onBlur, and onChange inherited from HTMLGenericInputElement
      sAlign = null;
      bChecked = false;
      bAutoComplete = true;
      bReadOnly = false;
      sSize = null;
      sMaxLength = null;
      sSrc = null;
      sValue = "";
      bSupportsSubmit = false;
      bSupportsAccessKey = false;
      sOnClick = null;
      sOnKeyDown = null;
      sTabIndex = null;
      if ((t == HTMLInputTypes.SUBMIT_TYPE) || (t == HTMLInputTypes.BUTTON_TYPE))
      {
        bSupportsSubmit = true;
        bSupportsAccessKey = true;
      }
      // rob: removing the validation checks. main reason is to avoid the validation check on textbox for banner search with friendly url
      // (bug #70204) but it is also not necessary anymore since we are moving to layouts
      SetValidateName(false);
    }
    
    /// <summary>
    /// Call this method to create a new HTMLInput object. <br /><br />
    /// </summary>
    /// <param name="iHTMLInputType">
    /// Pass an int describing the type of this input
    /// element.
    /// </param>
    /// <param name="sTagName">Pass the name of this element.</param>
    /// <param name="alt">Pass the alt attribute of this element.</param>
    /// <throws>HTMLException</throws>
    /// <seealso cref="HTMLInputTypes"></seealso>
    public HTMLInput(int iHTMLInputType, String sTagName, String alt)
    {
      SetID(this.GetNextID());
      SetAlt(alt);
      //  onFocus, onBlur, and onChange inherited from HTMLGenericInputElement
      sAlign = null;
      bChecked = false;
      bAutoComplete = true;
      bReadOnly = false;
      sSize = null;
      sMaxLength = null;
      sSrc = null;
      sValue = "";
      bSupportsSubmit = false;
      bSupportsAccessKey = false;
      sOnClick = null;
      sOnKeyDown = null;
      sTabIndex = null;
      switch (iHTMLInputType){
      case HTMLInputTypes.SUBMIT:
      bSupportsSubmit = true;
      bSupportsAccessKey = true;
      type = HTMLInputTypes.SUBMIT_TYPE;
      break;
      case HTMLInputTypes.BUTTON:
      bSupportsSubmit = true;
      bSupportsAccessKey = true;
      type = HTMLInputTypes.BUTTON_TYPE;
      break;
      case HTMLInputTypes.CHECKBOX:
      type = HTMLInputTypes.CHECKBOX_TYPE;
      break;
      case HTMLInputTypes.FILE:
      type = HTMLInputTypes.FILE_TYPE;
      break;
      case HTMLInputTypes.HIDDEN:
      type = HTMLInputTypes.HIDDEN_TYPE;
      break;
      case HTMLInputTypes.IMAGE:
      type = HTMLInputTypes.IMAGE_TYPE;
      break;
      case HTMLInputTypes.PASSWORD:
      type = HTMLInputTypes.PASSWORD_TYPE;
      break;
      case HTMLInputTypes.RADIO:
      type = HTMLInputTypes.RADIO_TYPE;
      break;
      case HTMLInputTypes.RESET:
      type = HTMLInputTypes.RESET_TYPE;
      break;
      case HTMLInputTypes.TEXT:
      type = HTMLInputTypes.TEXT_TYPE;
      break;
      }
      // rob: removing the validation checks. main reason is to avoid the validation check on textbox for banner search with friendly url
      // (bug #70204) but it is also not necessary anymore since we are moving to layouts
      // BT: Bug 70204 - Need to call this before SetName since it calls ValidateName.
      SetValidateName(false);
      SetName(sTagName);
    }
    
    /// <summary>
    /// Get the value of the align attribute.
    /// </summary>
    /// <returns>String value</returns>
    public virtual String GetAlign()
    {
      return sAlign;
    }
    
    /// <summary>
    /// Get the value of the alt attribute.
    /// </summary>
    /// <returns>String value</returns>
    public virtual String GetAlt()
    {
      return sAlt;
    }
    
    /// <summary>
    /// Get the value of the checked attribute.
    /// </summary>
    /// <returns>boolean value</returns>
    public virtual bool GetChecked()
    {
      return bChecked;
    }
    
    /// <summary>
    /// Get the value of the onclick attribute.
    /// </summary>
    /// <returns>String value</returns>
    public override String GetOnClick()
    {
      return sOnClick;
    }
    public virtual bool GetAutoComplete()
    {
      return bAutoComplete;
    }
    
    /// <summary>
    /// Get the value of the onkeydown attribute.
    /// </summary>
    /// <returns>String value</returns>
    public override String GetOnKeyDown()
    {
      return sOnKeyDown;
    }
    
    /// <summary>
    /// Get the value of the readonly attribute.
    /// </summary>
    /// <returns>boolean value</returns>
    public virtual bool GetReadOnly()
    {
      return bReadOnly;
    }
    public virtual String GetTabIndex()
    {
      return sTabIndex;
    }
    
    /// <summary>
    /// Call this method to convert a size in IE to an equivalent size in
    /// Netscape.
    /// </summary>
    /// <param name="iSize">int</param>
    /// <returns>int size in Netscape units.</returns>
    public static int IESizeToNSSize(int iSize)
    {
      return (((2 * iSize) / 3) + 1);
    }
    
    /// <summary>
    /// Get the value of the maxlength attribute.
    /// </summary>
    /// <returns>String value</returns>
    public virtual String GetMaxLength()
    {
      return sMaxLength;
    }
    
    /// <summary>
    /// Get the value of the size attribute.
    /// </summary>
    /// <returns>String value</returns>
    public virtual String GetSize()
    {
      return sSize;
    }
    
    /// <summary>
    /// Get the value of the src attribute.
    /// </summary>
    /// <returns>String value</returns>
    public virtual String GetSrc()
    {
      return sSrc;
    }
    
    /// <summary>
    /// Set the value of the align attribute.
    /// </summary>
    /// <param name="s">String value.</param>
    public virtual void SetAlign(String s)
    {
      sAlign = s;
    }
    
    /// <summary>
    /// Set the value of the alt attribute.
    /// </summary>
    /// <param name="s">String value.</param>
    public virtual void SetAlt(String s)
    {
      //  needs to be encoded because it's displayed
      sAlt = XPHttpUtility.HtmlEncode(s);
    }
    
    /// <summary>
    /// Set the value of the checked attribute.
    /// </summary>
    /// <param name="bInputChecked">boolean value.</param>
    public virtual void SetChecked(bool bInputChecked)
    {
      bChecked = bInputChecked;
    }
    
    /// <summary>
    /// Set the value of the autocomplete attribute.
    /// </summary>
    /// <param name="_bAutoComplete">boolean value (false = disable autocomplete)</param>
    public virtual void SetAutoComplete(bool _bAutoComplete)
    {
      bAutoComplete = _bAutoComplete;
    }
    
    /// <summary>
    /// Set the value of the maxlength attribute.
    /// </summary>
    /// <param name="i">size</param>
    public virtual void SetMaxLength(int i)
    {
      sMaxLength = XPConvert.ToString(i);
    }
    
    /// <summary>
    /// Set the value of the maxlength attribute.
    /// </summary>
    /// <param name="s">String value.</param>
    public virtual void SetMaxLength(String s)
    {
      sMaxLength = s;
    }
    
    /// <summary>
    /// Set the value of the onclick attribute.
    /// </summary>
    /// <param name="s">String value.</param>
    public override void SetOnClick(String s)
    {
      //        if (!HTMLElement.bAllowEvents)
      //        {
      //            if (bError)
      //            {
      //                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
      //
      //                Exception e = new Exception(sDesc);
      //
      //                myLogger.Error(sDesc, e);
      //            }
      //        }
      sOnClick = s;
    }
    
    /// <summary>
    /// Set the value of the onkeydown attribute.
    /// </summary>
    /// <param name="s">String value.</param>
    public override void SetOnKeyDown(String s)
    {
      //        if (!HTMLElement.bAllowEvents)
      //        {
      //            if (bError)
      //            {
      //                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
      //
      //                Exception e = new Exception(sDesc);
      //
      //                myLogger.Error(sDesc, e);
      //            }
      //        }
      sOnKeyDown = s;
    }
    
    /// <summary>
    /// Set the value of the readonly attribute.
    /// </summary>
    /// <param name="bInputReadOnly">value</param>
    public virtual void SetReadOnly(bool bInputReadOnly)
    {
      bReadOnly = bInputReadOnly;
    }
    
    /// <summary>
    /// Set the value of the size attribute.
    /// </summary>
    /// <param name="i">value.</param>
    public virtual void SetSize(int i)
    {
      if (bForceTextInputSizing)
      {
        bool bNS = false;
        DisplayOptions options = this.GetDisplayOptions();
        if (options != null)
        {
          options.GetIsNetscape();
        }
        SetSize(i, bNS);
      } else
      {
        sSize = XPConvert.ToString(i);
      }
    }
    
    /// <summary>
    /// Call this method to set the size on this input.  This method can be used
    /// to convert an IE size to an NS size.  Using this method, you can pass a
    /// single size value, and the input resulting will look mostly the same in
    /// Netscape and IE. <br /><br />
    /// </summary>
    /// <param name="i">
    /// Pass an int.  This int  should make this input appear the
    /// correct size when viewed in IE.<br /><br />
    /// </param>
    /// <param name="bConvertIEToNS">
    /// Pass a boolean indicating whether or not the first
    /// parameter should be converted to an Netscap size.
    /// </param>
    public virtual void SetSize(int i, bool bConvertIEToNS)
    {
      int iSet = i;
      if (bConvertIEToNS)
      {
        iSet = HTMLInput.IESizeToNSSize(i);
      }
      sSize = XPConvert.ToString(iSet);
    }
    
    /// <summary>
    /// Set the value of the size attribute.
    /// </summary>
    /// <param name="s">String value.</param>
    public virtual void SetSize(String s)
    {
      sSize = s;
    }
    
    /// <summary>
    /// Call this method to set the size on this input.  This method can be used
    /// to convert an IE size to an NS size.  Using this method, you can pass a
    /// single size value, and the input resulting will look mostly the same in
    /// Netscape and IE. <br /><br />
    /// </summary>
    /// <param name="s">
    /// Pass a String that is parsable to an int.  This String  should
    /// make this input appear the correct size when viewed in IE.
    /// <br /><br />
    /// </param>
    /// <param name="bConvertIEToNS">
    /// Pass a boolean indicating whether or not the first
    /// parameter should be converted to an Netscap size.
    /// </param>
    public virtual void SetSize(String s, bool bConvertIEToNS)
    {
      int i;
      int iNS;
      String sSet = s;
      if (bConvertIEToNS)
      {
        try
        {
          i = XPConvert.ToInteger(s);
          iNS = HTMLInput.IESizeToNSSize(i);
          sSet = XPConvert.ToString(iNS);
        }
        catch (Exception e)
        {
          sSet = s;
        }
      }
      sSize = sSet;
    }
    
    /// <summary>
    /// Set the value of the src attribute.
    /// </summary>
    /// <param name="sUrl">String value.</param>
    public virtual void SetSrc(String sUrl)
    {
      sSrc = sUrl;
    }
    public virtual void SetTabIndex(String s)
    {
      sTabIndex = s;
    }
    
    /// <summary>
    /// Call this method to set the value of the accesskey attribute.
    /// </summary>
    /// <param name="s">Pass the String value.</param>
    public virtual void SetAccessKey(String s)
    {
      if (bSupportsAccessKey)
      {
        sAccessKey = s;
      } else
      {
        throw new HTMLException("Access keys may only be set on Button or Submit Inputs.");
        }
    }
    
    /// <summary>
    /// Call this method to get the value of the accesskey attribute.
    /// </summary>
    /// <returns>The String value of the accesskey attribute.</returns>
    public virtual String GetAccessKey()
    {
      return sAccessKey;
    }
    /*
     The following methods are necessary to implement TabableHTMLElement
     */
     
    public virtual void ClearTabIndex()
    {
      sTabIndex = null;
    }
    
    /// <summary>
    /// Check to see if a name string is valid.
    /// </summary>
    /// <param name="s">Name string.</param>
    /// <throws>
    /// HTMLException Throws an exception if the name passed isn't
    /// valid.
    /// </throws>
    public override void ValidateName(String s)
    {
      int typeInt = type.ToInt();
      //
      //        if (bDebug)
      //        {
      //            myLogger.Debug("validateName: type: " + type.toString());
      //        }
      switch (typeInt){
      case HTMLInputTypes.BUTTON:
      if (!(XPStringUtility.StartsWith(s, "in_bu_")))
      {
        throw new HTMLException("Input button names must begin with \'in_bu_\'.  " + s + " is an invalid name.");
        }
      break;
      case HTMLInputTypes.CHECKBOX:
      if (!(XPStringUtility.StartsWith(s, "in_cb_")))
      {
        throw new HTMLException("Input checkbox names must begin with \'in_cb_\'.  " + s + " is an invalid name.");
        }
      break;
      case HTMLInputTypes.FILE:
      if (!(XPStringUtility.StartsWith(s, "in_fi_")))
      {
        throw new HTMLException("Input file names must begin with \'in_fi_\'.  " + s + " is an invalid name.");
        }
      break;
      case HTMLInputTypes.HIDDEN:
      if (!(XPStringUtility.StartsWith(s, "in_hi_")))
      {
        throw new HTMLException("Input hidden names must begin with \'in_hi_\'.  " + s + " is an invalid name.");
        }
      break;
      case HTMLInputTypes.IMAGE:
      if (!(XPStringUtility.StartsWith(s, "in_im_")))
      {
        throw new HTMLException("Input image names must begin with \'in_im_\'.  " + s + " is an invalid name.");
        }
      break;
      case HTMLInputTypes.PASSWORD:
      if (!(XPStringUtility.StartsWith(s, "in_pw_")))
      {
        throw new HTMLException("Input password names must begin with \'in_pw_\'.  " + s + " is an invalid name.");
        }
      break;
      case HTMLInputTypes.RADIO:
      if (!(XPStringUtility.StartsWith(s, "in_ra_")))
      {
        throw new HTMLException("Input radio names must begin with \'in_ra_\'.  " + s + " is an invalid name.");
        }
      break;
      case HTMLInputTypes.RESET:
      if (!(XPStringUtility.StartsWith(s, "in_re_")))
      {
        throw new HTMLException("Input reset names must begin with \'in_re_\'.  " + s + " is an invalid name.");
        }
      break;
      case HTMLInputTypes.SUBMIT:
      if (!(XPStringUtility.StartsWith(s, "in_su_")))
      {
        throw new HTMLException("Input submit names must begin with \'in_su_\'.  " + s + " is an invalid name.");
        }
      break;
      case HTMLInputTypes.TEXT:
      if (!(XPStringUtility.StartsWith(s, "in_tx_")))
      {
        throw new HTMLException("Input text names must begin with \'in_tx_\'.  " + s + " is an invalid name.");
        }
      break;
      }
    }
    
    /// <summary>
    /// This method generates the beginning of the opening tag.  The abstract
    /// HTMLElement superclass will provide the remainder of the opening tag,
    /// repleate with attributes common to all html elements, such as name, and
    /// id, and class.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendBeginningOfOpeningTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(BEGIN_TAG, sb, response);
      WriteNugget(type.ToString(), sb, response);
      WriteNugget(QUOTE, sb, response);
      if (sAlt != null)
      {
        WriteNugget(ALT_ATTR, sb, response);
        WriteNugget(sAlt, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sTabIndex != null)
      {
        WriteNugget(TAB_INDEX_ATTR, sb, response);
        WriteNugget(sTabIndex, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sOnClick != null)
      {
        //            if (!HTMLElement.bAllowEvents || this.GetDo508CheckingForThisInstance())
        //            {
        //                if (bError)
        //                {
        //                    myLogger.Error("You are not allowed to set the onclick attribute value because the client of this html tree is attempting to render a 508-compliant page.");
        //                    myLogger.Error("this: " + this);
        //                }
        //            }
        WriteNugget(ONCLICK_ATTR, sb, response);
        WriteNugget(sOnClick, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sAccessKey != null)
      {
        WriteNugget(ACCESSKEY_ATTR, sb, response);
        WriteNugget(sAccessKey, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sOnKeyDown != null)
      {
        //            if (!HTMLElement.bAllowEvents || this.GetDo508CheckingForThisInstance())
        //            {
        //                if (bError)
        //                {
        //                    myLogger.Error("You are not allowed to set the onkeydown attribute value because the client of this html tree is attempting to render a 508-compliant page.");
        //                    myLogger.Error("this: " + this);
        //                }
        //            }
        WriteNugget(ONKEYDOWN_ATTR, sb, response);
        WriteNugget(sOnKeyDown, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sAlign != null)
      {
        WriteNugget(ALIGN_ATTR, sb, response);
        WriteNugget(sAlign, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (bChecked)
      {
        WriteNugget(CHECKED_ATTR, sb, response);
      }
      if (bReadOnly)
      {
        WriteNugget(READ_ONLY_ATTR, sb, response);
      }
      if (!bAutoComplete)
      {
        WriteNugget(AUTOCOMPLETE_OFF_ATTR, sb, response);
      }
      if (GetDisabled())
      {
        WriteNugget(DISABLED_ATTR, sb, response);
      }
      if (sSize != null)
      {
        String sSizeToWrite = sSize;
        if (bForceTextInputSizing)
        {
          bool bNS = false;
          DisplayOptions options = this.GetDisplayOptions();
          if (options != null)
          {
            bNS = options.GetIsNetscape();
          }
          if (bNS)
          {
            try
            {
              int i = XPConvert.ToInteger(sSize);
              sSizeToWrite = XPConvert.ToString(HTMLInput.IESizeToNSSize(i));
              }
            catch (Exception ex)
            {
              log.Info(ex, "Unable to calculate input size for netscape.");
              }
          }
        }
        WriteNugget(SIZE_ATTR, sb, response);
        WriteNugget(sSizeToWrite, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sMaxLength != null)
      {
        WriteNugget(MAXLENGTH_ATTR, sb, response);
        WriteNugget(sMaxLength, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sSrc != null)
      {
        WriteNugget(SRC_ATTR, sb, response);
        WriteNugget(sSrc, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
    }
    
    /// <summary>
    /// This method generates the closing tag of this html element.  If the
    /// providesClosingTag method returns false, then this method will never be
    /// called.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendClosingTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(END_TAG, sb, response);
    }
    
    /// <summary>
    /// Indicates whether or not this HTML element provides a closing tag. Some
    /// elements only provide opening tags, such as the &lt;br&gt; element.
    /// </summary>
    /// <returns>boolean false.</returns>
    protected override bool ProvidesClosingTag()
    {
      return true;
    }
    
    /// <summary>
    /// Get the next id value.  This is used by the GetClone method to ensure
    /// that cloned object have new ID attribute values.
    /// </summary>
    /// <returns>A valid id string.</returns>
    internal override String GetNextID()
    {
      String s = "input" + iNextID;
      iNextID++;
      return s;
    }
  }
}