using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// <br /><br />
  /// <a target="new"
  /// href="http://www.w3.org/TR/html4/struct/links.html#h-12.3">Spec</a>
  /// </summary>
  public class HTMLLink : HTMLElement
  {
    internal static int iNextID = 0;
    private static readonly byte[] BEGIN_TAG = HTMLElement.ToByteArray("<link");
    private static readonly byte[] CHARSET_ATTR = HTMLElement.ToByteArray(" charset=\"");
    private static readonly byte[] TYPE_ATTR = HTMLElement.ToByteArray(" type=\"");
    private static readonly byte[] HREF_ATTR = HTMLElement.ToByteArray(" href=\"");
    private static readonly byte[] REL_ATTR = HTMLElement.ToByteArray(" rel=\"");
    private static readonly byte[] REV_ATTR = HTMLElement.ToByteArray(" rev=\"");
    private static readonly byte[] END_TAG = HTMLElement.ToByteArray("</link>");
    internal String sCharSet;
    internal String sHRef;
    internal String sRel;
    internal String sRev;
    internal String sType;
    
    /// <summary>
    /// Creates a new HTMLLink object.
    /// </summary>
    public HTMLLink()
    {
      sCharSet = null;
      sHRef = null;
      sType = null;
      sRel = null;
      sRev = null;
      // This is turned off by default, but the Link element needs
      // to use the lang attribute to localize the style sheets.
      bShowLang = true;
    }
    
    /// <summary>
    /// Get the value of the charset attribute.
    /// </summary>
    /// <returns>String value</returns>
    public virtual String GetCharSet()
    {
      return sCharSet;
    }
    
    /// <summary>
    /// Get the value of the href attribute.
    /// </summary>
    /// <returns>String value</returns>
    public virtual String GetHRef()
    {
      return sHRef;
    }
    
    /// <summary>
    /// Get the value of the rel attribute.
    /// </summary>
    /// <returns>String value</returns>
    public virtual String GetRel()
    {
      return sRel;
    }
    
    /// <summary>
    /// Get the value of the rev attribute.
    /// </summary>
    /// <returns>String value</returns>
    public virtual String GetRev()
    {
      return sRev;
    }
    
    /// <summary>
    /// Get the value of the type attribute.
    /// </summary>
    /// <returns>String value</returns>
    public virtual String GetType()
    {
      return sType;
    }
    
    /// <summary>
    /// Set the value of the charset attribute.
    /// </summary>
    /// <param name="s">String value</param>
    public virtual void SetCharSet(String s)
    {
      sCharSet = s;
    }
    
    /// <summary>
    /// Set the value of the href attribute.
    /// </summary>
    /// <param name="s">String value</param>
    public virtual void SetHRef(String s)
    {
      sHRef = s;
    }
    
    /// <summary>
    /// Set the value of the rel attribute.
    /// </summary>
    /// <param name="s">String value</param>
    public virtual void SetRel(String s)
    {
      sRel = s;
    }
    
    /// <summary>
    /// Set the value of the rev attribute.
    /// </summary>
    /// <param name="s">String value</param>
    public virtual void SetRev(String s)
    {
      sRev = s;
    }
    
    /// <summary>
    /// Set the value of the type attribute.
    /// </summary>
    /// <param name="s">String value</param>
    public virtual void SetType(String s)
    {
      sType = s;
    }
    
    /// <summary>
    /// This method generates the beginning of the opening tag.  The abstract
    /// HTMLElement superclass will provide the remainder of the opening tag,
    /// repleate with attributes common to all html elements, such as name, and
    /// id, and class.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendBeginningOfOpeningTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(BEGIN_TAG, sb, response);
      if (sCharSet != null)
      {
        WriteNugget(CHARSET_ATTR, sb, response);
        WriteNugget(sCharSet, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sType != null)
      {
        WriteNugget(TYPE_ATTR, sb, response);
        WriteNugget(sType, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sHRef != null)
      {
        WriteNugget(HREF_ATTR, sb, response);
        WriteNugget(sHRef, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sRel != null)
      {
        WriteNugget(REL_ATTR, sb, response);
        WriteNugget(sRel, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sRev != null)
      {
        WriteNugget(REV_ATTR, sb, response);
        WriteNugget(sRev, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
    }
    
    /// <summary>
    /// This method generates the closing tag of this html element.  If the
    /// providesClosingTag method returns false, then this method will never be
    /// called.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendClosingTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(END_TAG, sb, response);
    }
    
    /// <summary>
    /// Indicates whether or not this HTML element provides a closing tag. Some
    /// elements only provide opening tags, such as the &lt;br&gt; element.
    /// </summary>
    /// <returns>boolean false.</returns>
    protected override bool ProvidesClosingTag()
    {
      return true;
    }
    
    /// <summary>
    /// Get the next id value.  This is used by the GetClone method to ensure
    /// that cloned object have new ID attribute values.
    /// </summary>
    /// <returns>A valid id string.</returns>
    internal override String GetNextID()
    {
      String s = "link" + iNextID;
      iNextID++;
      return s;
    }
  }
}