using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// <br /><br />
  /// <a target="new"
  /// href="http://www.w3.org/TR/html4/interact/forms.html#h-17.6">Spec</a>
  /// </summary>
  public class HTMLOption : HTMLElement
  {
    private static readonly byte[] BEGIN_TAG = HTMLElement.ToByteArray("<option");
    private static readonly byte[] VALUE_ATTR = HTMLElement.ToByteArray(" value=\"");
    private static readonly byte[] SELECTED_ATTR = HTMLElement.ToByteArray(" selected");
    private static readonly byte[] END_TAG = HTMLElement.ToByteArray("</option>");
    internal static int iNextID = 0;
    private String sValue;
    private bool bSelected;
    
    /// <param name="sTagValue">Pass the value string of this option.</param>
    public HTMLOption(String sTagValue)
    {
      bSelected = false;
      SetValue(sTagValue);
    }
    
    /// <param name="sTagValue">value attribute</param>
    /// <param name="sDisplayText">text node to be inserted inside <option> tags</param>
    public HTMLOption(String sTagValue, String sDisplayText) : this(sTagValue)
    {
      AddInnerHTMLString(sDisplayText);
    }
    
    /// <summary>
    /// Call this method to determine whether or not this option is selected.
    /// </summary>
    /// <returns>boolean value</returns>
    public virtual bool GetSelected()
    {
      return bSelected;
    }
    
    /// <summary>
    /// Call this method to obtain the value of the value attribute.
    /// </summary>
    /// <returns>String value</returns>
    public virtual String GetValue()
    {
      return sValue;
    }
    
    /// <summary>
    /// Method rewritten to prevent usage.
    /// </summary>
    /// <param name="s">String</param>
    /// <throws>HTMLException This method throws an exception.</throws>
    public override void SetName(String s)
    {
      throw new HTMLException("The \"name\" attribute is not valid for the <option> element.");
      }
    
    /// <summary>
    /// Call this method to set this option selected.
    /// </summary>
    /// <param name="b">boolean value.</param>
    public virtual void SetSelected(bool b)
    {
      bSelected = b;
    }
    
    /// <summary>
    /// Call this method to set the value attribute.
    /// </summary>
    /// <param name="s">String value</param>
    public virtual void SetValue(String s)
    {
      sValue = s;
    }
    
    /// <summary>
    /// This method generates the beginning of the opening tag.  The abstract
    /// HTMLElement superclass will provide the remainder of the opening tag,
    /// repleate with attributes common to all html elements, such as name, and
    /// id, and class.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendBeginningOfOpeningTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(BEGIN_TAG, sb, response);
      WriteNugget(VALUE_ATTR, sb, response);
      WriteNugget(GetValue(), sb, response);
      WriteNugget(QUOTE, sb, response);
      if (bSelected)
      {
        WriteNugget(SELECTED_ATTR, sb, response);
      }
    }
    
    /// <summary>
    /// This method generates the closing tag of this html element.  If the
    /// providesClosingTag method returns false, then this method will never be
    /// called.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendClosingTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(END_TAG, sb, response);
    }
    
    /// <summary>
    /// Indicates whether or not this HTML element provides a closing tag. Some
    /// elements only provide opening tags, such as the &lt;br&gt; element.
    /// </summary>
    /// <returns>boolean true.</returns>
    protected override bool ProvidesClosingTag()
    {
      return true;
    }
    
    /// <summary>
    /// Get the next id value.  This is used by the GetClone method to ensure
    /// that cloned object have new ID attribute values.
    /// </summary>
    /// <returns>A valid id string.</returns>
    internal override String GetNextID()
    {
      String s = "option" + iNextID;
      iNextID++;
      return s;
    }
  }
}