using System;
//import com.plumtree.openfoundation.logging.*;
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// This class can be used to obtain a String describing an HTML SELECT element. <br />
  /// <br />
  /// <a target="new" href="http://www.w3.org/TR/html4/interact/forms.html#h-17.6">Spec</a><br />
  /// <br />
  /// This class extends HTMLGeneralInputElement.  This means that objects of
  /// this class can be used to set name/value pairs within an HTML FORM. <br />
  /// <br />
  /// This class also implements TabableHTMLElement.  This means that this class
  /// can generate a tabindex property that helps to guide the clients browser as
  /// the client tabs through form elements.
  /// </summary>
  public class HTMLSelect : HTMLGenericInputElement,TabableHTMLElement
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.xpshared.htmlelements.HTMLSelect");
    internal static int iNextID = 0;
    private static readonly byte[] BEGIN_TAG = HTMLElement.ToByteArray("<select");
    private static readonly byte[] MULTIPLE_ATTR = HTMLElement.ToByteArray(" multiple=\"multiple\"");
    private static readonly byte[] SIZE_ATTR = HTMLElement.ToByteArray(" size=\"");
    private static readonly byte[] END_TAG = HTMLElement.ToByteArray("</select>");
    private static readonly byte[] DISABLED_ATTR = HTMLElement.ToByteArray(" disabled=\"disabled\"");
    private HTMLOption oOldSelected;
    private String sSize;
    private String sTabIndex;
    private XPArrayList alOptionCollections;
    private bool bMultiple;
    private bool bSort = true;
    
    /// <summary>
    /// This method sets the element to read only and prepares for display,
    /// if bReadOnly is true.
    /// </summary>
    protected override void SetHTMLElementReadOnly(bool bReadOnly)
    {
      try
      {
        if ((bReadOnly) && (!base.GetHTMLElementReadOnly()))
        {
          // If we are going to set this element to read only, then
          // we need to prepare the inner HTMLOptions for display
          // before we set it read only.
          PrepareForDisplay();
        }
        base.SetHTMLElementReadOnly(bReadOnly);
      }
      catch (HTMLException he)
      {
        // Due to possible multi-threaded access to this method,
        // it is possible that PrepareForDisplay could get called
        // when the element is already read only.  It is fine if
        // this happens, because that means that another thread
        // successfully called PrepareForDisplay().  Therefore
        // we just catch the exception and ignore it.
        // We will log the exception in case it is for another reason.
        log.Warn(he, "HTMLSelect.SetHTMLElementReadOnly failed.  This error should be ignored if it is because the Element is not writable, as this is a possible expected result.");
        }
    }
    
    /// <param name="sTagName">Pass the name of this select.</param>
    /// <param name="sTagID">Pass the id of this select.</param>
    /// <throws>HTMLException</throws>
    public HTMLSelect(String sTagName, String sTagID)
    {
      alOptionCollections = new XPArrayList();
      //  onFocus, onBlur, and onChange inherited from HTMLGenericInputElement
      oOldSelected = null;
      SetName(sTagName);
      SetID(sTagID);
      sSize = null;
      bMultiple = false;
      // This is turned off by default, but the Select element needs
      // to use the lang attribute for Mac IE 5.2 in asian languages.
      bShowLang = true;
      // rob: removing the validation checks. main reason is to avoid the validation check on textbox for banner search with friendly url
      // (bug #70204) but it is also not necessary anymore since we are moving to layouts
      SetValidateName(false);
    }
    
    /// <summary>
    /// Call this method to specify whether or not this select will sort its nested
    /// options before each display.
    /// </summary>
    /// <param name="b">boolean To sort or not</param>
    public virtual void SetDoSort(bool b)
    {
      bSort = b;
    }
    
    /// <summary>
    /// Call this method to determine whether or not this select sorts its contents
    /// immediately before display.
    /// </summary>
    /// <returns>boolean to sort or not</returns>
    public virtual bool GetDoSort()
    {
      return bSort;
    }
    
    /// <returns></returns>
    public virtual String GetSize()
    {
      return sSize;
    }
    
    /// <returns></returns>
    public virtual String GetTabIndex()
    {
      return sTabIndex;
    }
    
    /// <summary>
    /// Call this method to set the multiple attribute.
    /// </summary>
    /// <param name="bIsMultiple">boolean value</param>
    public virtual void SetMultiple(bool bIsMultiple)
    {
      bMultiple = bIsMultiple;
    }
    
    /// <summary>
    /// Call this method to set the size attribute value.
    /// </summary>
    /// <param name="i">int size.</param>
    public virtual void SetSize(int i)
    {
      sSize = XPConvert.ToString(i);
    }
    
    /// <summary>
    /// Call this method to set the size attribute value.
    /// </summary>
    /// <param name="s">size</param>
    public virtual void SetSize(String s)
    {
      sSize = s;
    }
    
    /// <param name="s"></param>
    public virtual void SetTabIndex(String s)
    {
      sTabIndex = s;
    }
    //  **********************************
    //  Implements InputItem
    //  This method overrides HTMLGeneralInputElement.SetValue(), adding
    //  some functionality.
    
    /// <summary>
    /// Call this method to set the value attribute value.  The nested option
    /// having this value will be set selected.  If no nested option has this
    /// value, an exception will be thrown.
    /// </summary>
    /// <param name="s">String value</param>
    /// <throws>
    /// HTMLException This method throws an exception if there is no
    /// nested HTMLOption object having the specified value, or if this
    /// element is read only.
    /// </throws>
    public override void SetValue(String s)
    {
      base.VerifyWritability();
      //  todo rewrite using a hashmap for quickness.
      HTMLOption oSelected = null;
      base.SetValue(s);
      //	todo this could be more efficient... value setting could
      //	be done by ProcessOptionCollections, but I don't believe
      //	that HTMLSelect.SetValue() is called often.  I believe
      //	clients generally set the value when adding children.
      // Get the list of options
      XPArrayList alOptions = ProcessOptionCollections();
      IXPEnumerator xpEnumerator = alOptions.GetEnumerator();
      HTMLOption oTemp;
      // Find the option to select.
      while (xpEnumerator.MoveNext())
      {
        oTemp = (HTMLOption) xpEnumerator.GetCurrent();
        if (s.Equals(oTemp.GetValue()))
        {
          oSelected = oTemp;
        }
      }
      if (oSelected == null)
      {
        throw new HTMLException("There is no <option> in this <select> having the value: " + s);
        }
      oSelected.SetSelected(true);
      // Unselect the previously selected option
      if ((oOldSelected != null) && (oOldSelected != oSelected))
      {
        oOldSelected.SetSelected(false);
      }
      // Store the selected option so we can unset it later if this method is called again.
      oOldSelected = oSelected;
    }
    
    /// <summary>
    /// This method converts the HTMLOptions and HTMLElementCollections
    /// into an array list.
    /// </summary>
    /// <throws>
    /// HTMLException if there are non HTMLOption elements in the
    /// list or in any of the collections.
    /// </throws>
    private XPArrayList ProcessOptionCollections()
    {
      XPArrayList alOptions = new XPArrayList();
      HTMLElement e;
      HTMLOption option;
      IXPEnumerator xpEnumerator = alOptionCollections.GetEnumerator();
      while (xpEnumerator.MoveNext())
      {
        e = (HTMLElement) xpEnumerator.GetCurrent();
        if (e is HTMLElementCollection)
        {
          //	a collection of options
          IXPEnumerator eOptions = e.GetInnerHTMLElements(false);
          HTMLElement eTemp;
          // NOTE: This was here, but I can't see any reason why we would
          // need to re-add this collection, since we never clear the list
          // and we just got it from the collection.
          //alOptionCollections.Add( e );
          while (eOptions.MoveNext())
          {
            eTemp = (HTMLElement) eOptions.GetCurrent();
            // We have to check this here and not just when the collection is
            // added, because the collection could be added to the select list
            // before it has been populated.
            // Verify that the HTMLElementCollection only contains HTMLOptions.
            if (!(eTemp is HTMLOption))
            {
              throw new HTMLException("An HTMLElementCollection being added to an HTMLSelect must exclusively contain HTMLOption elements.  " + eTemp.GetName() + " is not an HTMLOption.");
              }
            option = (HTMLOption) eTemp;
            alOptions.Add(option);
          }
        } else
        {
          // just an option
          // No need to verify this here because we verified this when the
          // option was added.
          option = (HTMLOption) e;
          alOptions.Add(option);
        }
      }
      return alOptions;
    }
    
    /// <summary>
    /// HTMLElement.addInnerHTMLElement rewritten.
    /// </summary>
    /// <param name="e">
    /// Pass an HTMLOption object or an HTMLElementCollection object.  If this
    /// HTMLSelect sorts its nested children, then each nested HTMLElementCollection
    /// will be independantly sorted.  These HTMLElementCollection objects must contain
    /// exclusively HTMLOption objects.  HTMLOption objects nested directly will not
    /// be sorted under any circumstances.
    /// </param>
    /// <throws>
    /// HTMLException This method throws an exception if you don't pass
    /// an HTMLOption object or a collection of HTMLOptions.
    /// </throws>
    public override void AddInnerHTMLElement(HTMLElement e)
    {
      // Verify that this is the correct type of HTMLElement.
      if (!(e is HTMLOption) && !(e is HTMLElementCollection))
      {
        throw new HTMLException("You may only add HTMLOption, or HTMLElementCollection elements to an HTMLSelect.");
        }
      // Verify that the HTMLElementCollection only contains HTMLOptions.
      if (e is HTMLElementCollection)
      {
        //	a collection of options
        IXPEnumerator eOptions = e.GetInnerHTMLElements(false);
        HTMLElement eTemp;
        // We can't really check this here, because the collection could
        // be added to the select list before it has been populated, but
        // we'll try anyway so we can produce a more useful error message.
        // We also need to check this at display time.
        if (null != eOptions)
        {
          while (eOptions.MoveNext())
          {
            eTemp = (HTMLElement) eOptions.GetCurrent();
            if (!(eTemp is HTMLOption))
            {
              throw new HTMLException("An HTMLElementCollection being added to an HTMLSelect must exclusively contain HTMLOption elements.");
              }
          }
        }
      }
      alOptionCollections.Add(e);
    }
    /*
     The following methods are necessary to implement TabableHTMLElement
     */
     
    public virtual void ClearTabIndex()
    {
      sTabIndex = null;
    }
    
    /// <summary>
    /// Check to see if a name string is valid.
    /// </summary>
    /// <param name="s">Name string.</param>
    /// <throws>
    /// HTMLException Throws an exception if the name passed isn't
    /// valid.
    /// </throws>
    public override void ValidateName(String s)
    {
      if (!XPStringUtility.StartsWith(s, "in_se_"))
      {
        throw new HTMLException("The name of a <select> element must begin with \"in_se_\".  " + s + " is an invalid name.");
        }
    }
    
    /// <summary>
    /// This method is called each time this select is called upon itself to display.
    /// This is where sorting is performed when necessary.
    /// </summary>
    private void PrepareForDisplay()
    {
      XPArrayList alProcessedInner = new XPArrayList();
      if (bSort)
      {
        //	we sort
        IXPEnumerator enumCollections = alOptionCollections.GetEnumerator();
        HTMLElement eTemp;
        // NOTE: We do not need to clear this table between different collections,
        // because we access it through a unique String[] for each collection.
        XPHashtable htTextToOption = new XPHashtable();
        while (enumCollections.MoveNext())
        {
          eTemp = (HTMLElement) enumCollections.GetCurrent();
          if (eTemp is HTMLElementCollection)
          {
            //	this option-collection is a collection.
            //	collections are sorted, options aren't.
            HTMLElementCollection tempCollection = (HTMLElementCollection) eTemp;
            XPArrayList alCollectionOptions = tempCollection.GetInnerHTMLElementsAsArrayList(false);
            int iSize = 0;
            IXPEnumerator xpEnumerator = null;
            if (alCollectionOptions != null)
            {
              iSize = alCollectionOptions.GetSize();
              xpEnumerator = alCollectionOptions.GetEnumerator();
            }
            if (iSize == 0)
            {
              return;
            }
            String[] saText = new String[iSize];
            HTMLOption oTemp;
            String sText;
            int i = 0;
            //	iterate through our options, adding the nested strings to an
            //	array, which will be sorted, and a hash table which maps strings
            //	to the object containing the strings.
            while (xpEnumerator.MoveNext())
            {
              oTemp = (HTMLOption) xpEnumerator.GetCurrent();
              sText = oTemp.GetInnerHTML();
              htTextToOption.PutElement(sText, oTemp);
              saText[i] = sText;
              i++;
            }
            //	WARNING: This call needs to be translated into .NET manually.
            XPArray.Sort(saText);
            for (int x = 0;x < iSize;x++)
            {
              sText = saText[x];
              oTemp = (HTMLOption) htTextToOption.GetElement(sText);
              alProcessedInner.Add(oTemp);
            }
          } else
          {
            //	this option-collection is an option.
            //	these aren't sorted, so we just nest them.
            alProcessedInner.Add(eTemp);
          }
        }
      } else
      //	for each option collection
      //	sorting
      {
        //	we don't sort
        //	We'll just add each of our inner options.  This cannot be done
        //	at the time this select is being built because it's possible that the
        //	select will be told to sort itself immediately before display.
        //	This iterates through all nested elements, HTMLOptions and
        //	HTMLElementCollections, and returns an XPArrayList containing each of them.
        alProcessedInner = ProcessOptionCollections();
      }
      // Store the processed options as the new inner HTML elements.
      base.SetInner(alProcessedInner);
    }
    
    /// <summary>
    /// This method generates the beginning of the opening tag.  The abstract
    /// HTMLElement superclass will provide the remainder of the opening tag,
    /// repleate with attributes common to all html elements, such as name, and
    /// id, and class.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendBeginningOfOpeningTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      if (!base.GetHTMLElementReadOnly())
      {
        // This method creates the underlying element list, so
        // we can only call this if the element is writable.
        PrepareForDisplay();
      }
      WriteNugget(BEGIN_TAG, sb, response);
      if (bMultiple)
      {
        WriteNugget(MULTIPLE_ATTR, sb, response);
      }
      if (sSize != null)
      {
        WriteNugget(SIZE_ATTR, sb, response);
        WriteNugget(sSize, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (GetDisabled())
      {
        WriteNugget(DISABLED_ATTR, sb, response);
      }
    }
    
    /// <summary>
    /// This method generates the closing tag of this html element.  If the
    /// providesClosingTag method returns false, then this method will never be
    /// called.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendClosingTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(END_TAG, sb, response);
    }
    
    /// <summary>
    /// Indicates whether or not this HTML element provides a closing tag. Some
    /// elements only provide opening tags, such as the &lt;br&gt; element.
    /// </summary>
    /// <returns>boolean true.</returns>
    protected override bool ProvidesClosingTag()
    {
      return true;
    }
    
    /// <summary>
    /// Get the next id value.  This is used by the GetClone method to ensure
    /// that cloned object have new ID attribute values.
    /// </summary>
    /// <returns>A valid id string.</returns>
    internal override String GetNextID()
    {
      String s = "select" + iNextID;
      iNextID++;
      return s;
    }
  }
}