using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// Use this HTMLElement object to generate an html table tag. <br />
  /// <br />
  /// <a target="new"
  /// href="http://www.w3.org/TR/html4/struct/tables.html#h-11.2.1">Spec</a>
  /// </summary>
  public class HTMLTable : HTMLElement
  {
    internal static int iNextID = 0;
    public static bool bEnforceSummary = false;
    private static readonly byte[] BEGINTAG = HTMLElement.ToByteArray("<table");
    private static readonly byte[] ALIGN_ATTR = HTMLElement.ToByteArray(" align=\"");
    private static readonly byte[] BGCOLOR_ATTR = HTMLElement.ToByteArray(" bgcolor=\"");
    private static readonly byte[] BORDERCOLOR_ATTR = HTMLElement.ToByteArray(" bordercolor=\"");
    private static readonly byte[] CELLPADDING_ATTR = HTMLElement.ToByteArray(" cellpadding=\"");
    private static readonly byte[] CELLSPACING_ATTR = HTMLElement.ToByteArray(" cellspacing=\"");
    private static readonly byte[] HEIGHT_ATTR = HTMLElement.ToByteArray(" height=\"");
    private static readonly byte[] WIDTH_ATTR = HTMLElement.ToByteArray(" width=\"");
    private static readonly byte[] BORDER_ATTR = HTMLElement.ToByteArray(" border=\"");
    private static readonly byte[] SUMMARY_ATTR = HTMLElement.ToByteArray(" summary=\"");
    private static readonly byte[] ON_SELECT_START_ATTR = HTMLElement.ToByteArray(" onselectstart=\"");
    private static readonly byte[] ENDTAG = HTMLElement.ToByteArray("</table>");
    private String sAlign;
    private String sBGColor;
    private String sBorder;
    private String sBorderColor;
    private String sCellPadding;
    private String sCellSpacing;
    private String sHeight;
    private String sOnSelectStart;
    private String sSummary;
    private String sWidth;
    private bool bErrorCaught = false;
    private bool bHousingForm = false;
    private bool bValidated;
    private int iNumColsSpanned;
    private int iNumRows;
    
    /// <summary>
    /// Creates a new HTMLTable object.
    /// </summary>
    public HTMLTable()
    {
      sAlign = null;
      sBGColor = null;
      sBorderColor = null;
      sCellPadding = null;
      sCellSpacing = null;
      sHeight = null;
      sWidth = null;
      sBorder = null;
      iNumColsSpanned = -1;
      iNumRows = 0;
      bValidated = false;
      sOnSelectStart = null;
      sSummary = null;
    }
    
    /// <summary>
    /// Call this method to get the value of the align attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetAlign()
    {
      return sAlign;
    }
    
    /// <summary>
    /// Call this method to get the value of the bgcolor attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetBGColor()
    {
      return sBGColor;
    }
    
    /// <summary>
    /// Call this method to get the value of the border attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetBorder()
    {
      return sBorder;
    }
    
    /// <summary>
    /// Call this method to get the value of the bordercolor attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetBorderColor()
    {
      return sBorderColor;
    }
    
    /// <summary>
    /// Call this method to get the value of the cellpadding attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetCellPadding()
    {
      return sCellPadding;
    }
    
    /// <summary>
    /// Call this method to get the value of the cellspacing attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetCellSpacing()
    {
      return sCellSpacing;
    }
    
    /// <summary>
    /// Call this method to obtain an clone of this table.
    /// </summary>
    /// <returns>Object that is an HTMLTable.</returns>
    public override Object GetClone()
    {
      //        if (bInfo)
      //        {
      //            myLogger.Info("Entering GetClone()");
      //        }
      HTMLTable tClone = (HTMLTable) base.GetClone();
      IXPEnumerator xpEnumerator = tClone.GetInnerHTMLElements(false);
      HTMLTableRow r;
      while (xpEnumerator.MoveNext())
      {
        //            if (bDebug)
        //            {
        //                myLogger.Debug("Setting parent table on inner row.");
        //            }
        r = (HTMLTableRow) xpEnumerator.GetCurrent();
        r.SetParentTable(this);
      }
      return tClone;
    }
    
    /// <summary>
    /// Call this method to get the value of the height attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetHeight()
    {
      return sHeight;
    }
    
    /// <summary>
    /// Call this method to determine the number of columns spanned by this
    /// table.  This value may change if rows are added or removed. <br />
    /// <br />
    /// Calling this method may be expensive, as the table needs to be
    /// evaluated to determine this information.
    /// </summary>
    /// <returns></returns>
    /// <throws>HTMLException</throws>
    public virtual int GetNumColsSpanned()
    {
      Validate();
      return iNumColsSpanned;
    }
    
    /// <summary>
    /// Call this method to get the value of the onselectstart attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetOnSelectStart()
    {
      return sOnSelectStart;
    }
    
    /// <summary>
    /// Call this method to get the value of the summary attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetSummary()
    {
      return sSummary;
    }
    
    /// <summary>
    /// Call this method to get the value of the width attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetWidth()
    {
      return sWidth;
    }
    
    /// <summary>
    /// Call this method to set the align attribute value.
    /// </summary>
    /// <param name="s">String value.</param>
    public virtual void SetAlign(String s)
    {
      sAlign = s;
    }
    
    /// <summary>
    /// Call this method to set the bgcolor attribute value.
    /// </summary>
    /// <param name="s">String value.</param>
    public virtual void SetBGColor(String s)
    {
      sBGColor = s;
    }
    
    /// <summary>
    /// Call this method to set the border attribute value.
    /// </summary>
    /// <param name="s">String value.</param>
    public virtual void SetBorder(String s)
    {
      sBorder = s;
    }
    
    /// <summary>
    /// Call this method to set the bordercolor attribute value.
    /// </summary>
    /// <param name="s">String value.</param>
    public virtual void SetBorderColor(String s)
    {
      sBorderColor = s;
    }
    
    /// <summary>
    /// Call this method to set the cellpadding attribute value.
    /// </summary>
    /// <param name="s">String value.</param>
    public virtual void SetCellPadding(String s)
    {
      sCellPadding = s;
    }
    
    /// <summary>
    /// Call this method to set the cellspacing attribute value.
    /// </summary>
    /// <param name="s">String value.</param>
    public virtual void SetCellSpacing(String s)
    {
      sCellSpacing = s;
    }
    
    /// <summary>
    /// Call this method to set the height attribute value.
    /// </summary>
    /// <param name="s">String value.</param>
    public virtual void SetHeight(String s)
    {
      sHeight = s;
    }
    
    /// <summary>
    /// Call this method to set the onselectstart attribute value.
    /// This is IE specific, and has been added so that developers
    /// can turn off text selection in tables as this can cause
    /// problems with fancier javascript code.
    /// </summary>
    /// <param name="s">String value.</param>
    public virtual void SetOnSelectStart(String s)
    {
      sOnSelectStart = s;
    }
    
    /// <summary>
    /// Call this method to set the summary attribute value.
    /// </summary>
    /// <param name="s">String value.</param>
    public virtual void SetSummary(String s)
    {
      sSummary = s;
    }
    
    /// <summary>
    /// Call this method to set the width attribute value.
    /// </summary>
    /// <param name="s">String value.</param>
    public virtual void SetWidth(String s)
    {
      sWidth = s;
    }
    
    /// <summary>
    /// Call this method to add a HTMLTableRow object to this table.  The
    /// HTMLTableRow passed as a parameter must have been constructed with a
    /// reference to this very HTMLTable.
    /// </summary>
    /// <param name="e"></param>
    /// <throws>HTMLException</throws>
    public override void AddInnerHTMLElement(HTMLElement e)
    {
      if (!(e is HTMLTableRow) && !(e is HTMLElementCollection) && !(e is HTMLForm))
      {
        throw new HTMLException("You may only add HTMLRow, HTMLForm, or HTMLElementCollection elements to an HTMLTable.");
        }
      if (e is HTMLForm)
      {
        this.bHousingForm = true;
        HTMLForm f = (HTMLForm) e;
        f.SetParentTable(this);
        base.AddInnerHTMLElement(f);
      } else
      if (e is HTMLElementCollection)
      {
        IXPEnumerator enumItems = e.GetInnerHTMLElements(false);
        if (enumItems != null)
        {
          HTMLElement eTemp;
          while (enumItems.MoveNext())
          {
            eTemp = (HTMLElement) enumItems.GetCurrent();
            //                    if (bDebug)
            //                    {
            //                        myLogger.Debug("eTemp: " + eTemp);
            //                    }
            if (!(eTemp is HTMLTableRow))
            {
              throw new HTMLException("You may only add an HTMLElementCollection containing HTMLTableRow elements to an HTMLTable element.  You are attempting to add an: " + eTemp);
              }
            ProcessRowForAdding((HTMLTableRow) eTemp);
          }
        }
      } else
      //   for each inner element
      //   got inner elements
      // e instanceof HTMLElementCollection
      {
        // it's just a row being added
        ProcessRowForAdding((HTMLTableRow) e);
      }
      //   we must validate anew.
      Invalidate();
    }
    
    /// <summary>
    /// This method generates the beginning of the opening tag.  The abstract
    /// HTMLElement superclass will provide the remainder of the opening tag,
    /// repleate with attributes common to all html elements, such as name, and
    /// id, and class.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    /// <throws>HTMLException</throws>
    protected override void AppendBeginningOfOpeningTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      Validate();
      WriteNugget(BEGINTAG, sb, response);
      if (sAlign != null)
      {
        WriteNugget(ALIGN_ATTR, sb, response);
        WriteNugget(sAlign, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sBGColor != null)
      {
        WriteNugget(BGCOLOR_ATTR, sb, response);
        WriteNugget(sBGColor, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sBorderColor != null)
      {
        WriteNugget(BORDERCOLOR_ATTR, sb, response);
        WriteNugget(sBorderColor, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sCellPadding != null)
      {
        WriteNugget(CELLPADDING_ATTR, sb, response);
        WriteNugget(sCellPadding, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sCellSpacing != null)
      {
        WriteNugget(CELLSPACING_ATTR, sb, response);
        WriteNugget(sCellSpacing, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sHeight != null)
      {
        WriteNugget(HEIGHT_ATTR, sb, response);
        WriteNugget(sHeight, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sWidth != null)
      {
        WriteNugget(WIDTH_ATTR, sb, response);
        WriteNugget(sWidth, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sBorder != null)
      {
        WriteNugget(BORDER_ATTR, sb, response);
        WriteNugget(sBorder, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sOnSelectStart != null)
      {
        WriteNugget(ON_SELECT_START_ATTR, sb, response);
        WriteNugget(sOnSelectStart, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sSummary != null)
      {
        WriteNugget(SUMMARY_ATTR, sb, response);
        WriteNugget(sSummary, sb, response);
        WriteNugget(QUOTE, sb, response);
      } else
      {
        if (bEnforceSummary || GetDo508CheckingForThisInstance())
        {
          if (!bErrorCaught)
          {
            //	this prevents infinite loop.
            bErrorCaught = true;
            //                    if (bError)
            //                    {
            //                        myLogger.Error("You must set the summary attribute on this HTMLTable.");
            //                    }
            bErrorCaught = false;
          }
        }
      }
    }
    
    /// <summary>
    /// This method generates the closing tag of this html element.  If the
    /// providesClosingTag method returns false, then this method will never be
    /// called.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendClosingTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(ENDTAG, sb, response);
      AppendClosingLineBreak(sb, response);
    }
    
    /// <summary>
    /// Indicates whether or not this HTML element provides a closing tag. Some
    /// elements only provide opening tags, such as the &lt;br&gt; element.
    /// </summary>
    /// <returns>boolean true.</returns>
    protected override bool ProvidesClosingTag()
    {
      return true;
    }
    
    /// <summary>
    /// Call to determine whether or not this HTMLTable is currently validated.
    /// </summary>
    /// <returns>boolean value.</returns>
    internal virtual bool IsValidated()
    {
      return bValidated;
    }
    
    /// <summary>
    /// Get the next id value.  This is used by the GetClone method to ensure
    /// that cloned object have new ID attribute values.
    /// </summary>
    /// <returns>A valid id string.</returns>
    internal override String GetNextID()
    {
      String s = "table" + iNextID;
      iNextID++;
      return s;
    }
    
    /// <summary>
    /// This method adds an HTMLRow to this table.
    /// </summary>
    /// <param name="r">Pass an HTMLRow object.</param>
    /// <throws>
    /// HTMLException This method throws an exception if for any reason
    /// the row passed isn't appropriate.
    /// </throws>
    internal virtual void ProcessRowForAdding(HTMLTableRow r)
    {
      if (r.GetParentTable() == null)
      {
        r.SetParentTable(this);
      } else
      {
        if (r.GetParentTable() != this)
        {
          throw new HTMLException("You may only add a given HTMLTableRow to a single given HTMLTable.");
          }
      }
      base.AddInnerHTMLElement(r);
    }
    //   package-private access level
    
    /// <summary>
    /// This method should be called whenever a change has been made to any
    /// member HTMLTableCell objects that would cause this  HTMLTableRow to
    /// need to validate.
    /// </summary>
    internal virtual void Invalidate()
    {
      //        if (bInfo)
      //        {
      //            myLogger.Info("HTMLTable::invalidate() ");
      //        }
      bValidated = false;
      //   we know not how many cols we span.
      iNumColsSpanned = -1;
    }
    
    /// <summary>
    /// Call this method to prepare this HTMLTable for display. <br />
    /// <br />
    /// If this HTMLTable is already validated, then this method will not do
    /// anything. <br />
    /// <br />
    /// <br />
    /// this method will iterate through this table's constituent rows.   Each
    /// row will first be validated. <br />
    /// <br />
    /// The number of columns spanned by this table will be determined. <br />
    /// <br />
    /// This HTMLTable will then enter a validated state, and be ready for
    /// display.
    /// </summary>
    /// <throws>HTMLException</throws>
    private void Validate()
    {
      //        if (bInfo)
      //        {
      //            myLogger.Info("Entering validate()");
      //        }
      //   If this table is already validated, or we're not validating tables,
      //   just return.
      if (bValidated)
      {
        //            if (bDebug)
        //            {
        //                myLogger.Debug("Already validated, returning.");
        //            }
        return;
      }
      //
      //        if (bInfo)
      //        {
      //            myLogger.Info("validate() - proceeding");
      //        }
      //        if (HTMLElement.bEnforceMaxTableDepth)
      //        {
      //            if (bDebug)
      //            {
      //                myLogger.Debug("validate() - about to test for excessive table nesting depth.");
      //            }
      //
      //            if (this.GetTableLevel() > iMaxTableDepth)
      //            {
      //                if (bError)
      //                {
      //                    myLogger.Error("This table is nested too deeply.");
      //                }
      //
      //                //                throw new HTMLTable.TableNestingTooDeepException("The maximum table depth is: " + this.iMaxTableDepth);
      //            }
      //        }
      IXPEnumerator e;
      e = GetInnerHTMLElements(false);
      if (this.bHousingForm)
      {
        e.MoveNext();
        HTMLForm f = (HTMLForm) e.GetCurrent();
        e = f.GetInnerHTMLElements(false);
      }
      HTMLTableRow r;
      int iMaxColsSpanned = 0;
      int iRowIndex = 0;
      if (e == null)
      {
      } else
      //            if (bError)
      //            {
      //                myLogger.Error(s);
      //            }
      //			throw new HTMLException( s );
      {
        while (e.MoveNext())
        {
          HTMLElement eTemp = (HTMLElement) e.GetCurrent();
          if (!(eTemp is HTMLTableRow))
          {
            continue;
          }
          r = (HTMLTableRow) e.GetCurrent();
          r.SetIndexInTable(iRowIndex);
          r.Validate();
          if (!r.GetHasAutoFitElement())
          {
            //  the row being added does not have an autofit element.
            //                    if (bDebug)
            //                    {
            //                        myLogger.Debug("validate() - Found a row without an autofit element.");
            //                    }
            int iRowColsSpanned = r.GetNumColsSpanned();
            //
            //                    if (bDebug)
            //                    {
            //                        myLogger.Debug("validate() - iNumColsSpanned: " + iNumColsSpanned);
            //                        myLogger.Debug("validate() - iRowColsSpanned: " + iRowColsSpanned);
            //                    }
            if (iNumColsSpanned == -1)
            {
              iNumColsSpanned = iRowColsSpanned;
            } else
            if (iRowColsSpanned != iNumColsSpanned)
            {
            }
            //  the row being added does not span the
            //  the appropriate number of columns.
            //String sErrorDescription = "The row at index " + iRowIndex + " spans " + r.GetNumColsSpanned() + " columns, where it should span " + iNumColsSpanned;
            //                        if (bError)
            //                        {
            //                            myLogger.Error(sErrorDescription);
            //                        }
            //	                    throw new InvalidColumnSpanException(sErrorDescription);
            iMaxColsSpanned = iNumColsSpanned;
          } else
          //  the row being added does not have an autofit element.
          {
            //  the row being added has an autofit element.
            //                    if (bDebug)
            //                    {
            //                        myLogger.Debug("validate() - Found a row WITH an autofit element.");
            //                    }
            int iRowColSpan = r.GetNumColsSpanned();
            //
            //                    if (bDebug)
            //                    {
            //                        myLogger.Debug("validate() - iRowColSpan: " + iRowColSpan);
            //                    }
            if (iRowColSpan > iMaxColsSpanned)
            {
              //   We need to determine the maximum number of columns
              //   spanned by any autofit row.  In the event that this table
              //   consists only of autofit rows, the rows with fewer elements will
              //   need to autofit themselves to match the row with the most
              //   elements.  Remember, when asked how many columns they span, rows
              //   with autofit elements assume that
              //   the autofit element spans only 1 column.
              iMaxColsSpanned = iRowColSpan;
            }
            //  If we haven't yet determined how many columns we span, then
            //  we shouldn't perform a check.
            if (iNumColsSpanned != -1)
            {
              //  We know how many rows we span.
              if (iRowColSpan > iNumColsSpanned)
              {
              }
            }
          }
          //  the row being added spans more columns than
          //   a row without an autofit element.
          //String sErrorDescription = "The row at index " + iRowIndex + " spans " + iRowColSpan + " columns, where it should span " + iNumColsSpanned;
          //                            if (bError)
          //                            {
          //                                myLogger.Error(sErrorDescription);
          //                            }
          //	                        throw new InvalidColumnSpanException(sErrorDescription);
          //  the row added has an autofit element.
          iRowIndex++;
        }
      }
      // for each row
      // e != null
      if (iNumColsSpanned == -1)
      {
        //   we were unable to determine how many columns we should span.
        //   This might be because all of the rows had autofit elements.
        //   In this case, this table must span the number of columns
        //   of the row spanning the greatest number of columns.
        iNumColsSpanned = iMaxColsSpanned;
      }
      bValidated = true;
    }
  }
}