using System;
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// Use this object to generate a &lt;td&gt;&lt;/td&gt; tag. <br />
  /// <br />
  /// <a target="new"
  /// href="http://www.w3.org/TR/html4/struct/tables.html#h-11.2.6">Spec</a>
  /// </summary>
  public class HTMLTableCell : HTMLElement
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.xpshared.htmlelements.HTMLTableCell");
    internal static int iNextID = 0;
    private static readonly byte[] BEGINTAG = HTMLElement.ToByteArray("<td");
    private static readonly byte[] ALIGN_ATTR = HTMLElement.ToByteArray(" align=\"");
    private static readonly byte[] VALIGN_ATTR = HTMLElement.ToByteArray(" valign=\"");
    private static readonly byte[] BGCOLOR_ATTR = HTMLElement.ToByteArray(" bgcolor=\"");
    private static readonly byte[] HEADERS_ATTR = HTMLElement.ToByteArray(" headers=\"");
    private static readonly byte[] HEIGHT_ATTR = HTMLElement.ToByteArray(" height=\"");
    private static readonly byte[] WIDTH_ATTR = HTMLElement.ToByteArray(" width=\"");
    private static readonly byte[] NOWRAP_ATTR = HTMLElement.ToByteArray(" nowrap=\"nowrap\"");
    private static readonly byte[] COLSPAN_ATTR = HTMLElement.ToByteArray(" colspan=\"");
    private static readonly byte[] ROWSPAN_ATTR = HTMLElement.ToByteArray(" rowspan=\"");
    private static readonly byte[] ENDTAG = HTMLElement.ToByteArray("</td>");
    private const String ONE = "1";
    private HTMLTable myParentTable;
    private HTMLTableRow myParentRow;
    private String sAlign;
    private String sBGColor;
    private String sColSpan;
    private String sRowSpan;
    private String sHeaders;
    private String sHeight;
    private String sVAlign;
    private String sWidth;
    private bool bAutoFit;
    private bool bNoWrap;
    private bool bValidated;
    
    /// <summary>
    /// Call this method to create a new HTMLTableCell object.
    /// </summary>
    /// <param name="rParent">
    /// Pass the HTMLTableRow to which this HTMLTableCell will be
    /// added.  A given instance of an HTMLTableCell may only be added
    /// to the HTMLTableRow specified to this constructor.<br /><br />
    /// </param>
    /// <deprecated>
    /// It's no longer necessary to pass the parent row, use
    /// constructor with no parameters.
    /// </deprecated>
    public HTMLTableCell(HTMLTableRow rParent)
    {
      sAlign = null;
      sVAlign = null;
      sColSpan = ONE;
      // no logic for rowspan
      sRowSpan = null;
      sBGColor = null;
      sHeaders = null;
      sHeight = null;
      sWidth = null;
      bNoWrap = false;
      bValidated = false;
      bAutoFit = false;
      myParentTable = rParent.GetParentTable();
      myParentRow = rParent;
    }
    
    /// <summary>
    /// Creates a new HTMLTableCell object.
    /// </summary>
    public HTMLTableCell()
    {
      sAlign = null;
      sVAlign = null;
      sColSpan = ONE;
      sRowSpan = null;
      sHeaders = null;
      sHeight = null;
      sWidth = null;
      bNoWrap = false;
      bValidated = false;
      bAutoFit = false;
      myParentTable = null;
      myParentRow = null;
    }
    
    /// <summary>
    /// Call this method to get the value of the align attribute.
    /// </summary>
    /// <returns>String value</returns>
    public virtual String GetAlign()
    {
      return sAlign;
    }
    
    /// <summary>
    /// Call this method to determine whether or not this HTMLCell object will
    /// automatically span the number of columns necessary.
    /// </summary>
    /// <returns></returns>
    public virtual bool GetAutoFit()
    {
      return bAutoFit;
    }
    
    /// <returns></returns>
    public virtual String GetBGColor()
    {
      return sBGColor;
    }
    
    /// <summary>
    /// Call this method to get the value of the colspan attribute.  This value
    /// is calculated at display time.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetColSpan()
    {
      return sColSpan;
    }
    
    /// <summary>
    /// Call this method to get the value of the rowspan attribute.  This value
    /// is calculated at display time.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetRowSpan()
    {
      return sRowSpan;
    }
    
    /// <summary>
    /// Call this method to get the value of the colspan attribute.  This value
    /// is calculated at display time.
    /// </summary>
    /// <returns>int value.</returns>
    public virtual int GetColSpanAsInt()
    {
      int i = 0;
      try
      {
        if (null != sColSpan)
        {
          i = XPConvert.ToInteger(sColSpan);
        }
      }
      catch (Exception e)
      {
        log.Info(e, "Unable to convert ColSpanAsInt.");
      }
      return i;
    }
    
    /// <summary>
    /// Call this method to get the value of the rowspan attribute.  This value
    /// is calculated at display time.
    /// </summary>
    /// <returns>int value.</returns>
    public virtual int GetRowSpanAsInt()
    {
      int i = 0;
      try
      {
        if (null != sRowSpan)
        {
          i = XPConvert.ToInteger(sRowSpan);
        }
      }
      catch (Exception e)
      {
        log.Info(e, "Unable to convert RowSpanAsInt.");
      }
      return i;
    }
    
    /// <returns></returns>
    public virtual String GetHeaders()
    {
      return sHeaders;
    }
    
    /// <returns></returns>
    public virtual String GetHeight()
    {
      return sHeight;
    }
    
    /// <returns></returns>
    public virtual bool GetNoWrap()
    {
      return bNoWrap;
    }
    
    /// <summary>
    /// Call this method to get the value of the valign attribute.
    /// </summary>
    /// <returns>String value</returns>
    public virtual String GetVAlign()
    {
      return sVAlign;
    }
    
    /// <returns></returns>
    public virtual String GetWidth()
    {
      return sWidth;
    }
    
    /// <summary>
    /// Call this method to set the value of the align attribute.
    /// </summary>
    /// <param name="s">String value</param>
    public virtual void SetAlign(String s)
    {
      //   This such a change does not invalidate.
      sAlign = s;
    }
    
    /// <summary>
    /// Call this method to toggle whether or not this HTMLTableCell is an
    /// autofit cell.  An autofit cell will take up as many columns as is
    /// necessary to complete the row within which it resides.  When asked to
    /// display, a given HTMLTableRow may only have one  single autofit cell.
    /// This check is executed at display time, and an HTMLException will be
    /// thrown if there are multiple autofit cells within a given row.
    /// Note that autofit does not apply to rowspan, which must be set manually.
    /// </summary>
    /// <param name="b"></param>
    public virtual void SetAutoFit(bool b)
    {
      //   If a change is made to this state, then this cell is no longer
      //   validated.  Nor is this cell's parent row, nor parent table.
      //   Everything will need to be re-validated upon display.
      if (b != bAutoFit)
      {
        Invalidate();
      }
      bAutoFit = b;
      if (b)
      {
        sColSpan = ONE;
      }
    }
    
    /// <param name="s"></param>
    public virtual void SetBGColor(String s)
    {
      sBGColor = s;
    }
    
    /// <summary>
    /// Call this method to set the value of the colspan attribute.
    /// </summary>
    /// <param name="s">String value.</param>
    /// <throws>HTMLException</throws>
    public virtual void SetColSpan(String s)
    {
      //   If a change is made to this state, then this cell is no longer
      //   validated.  Nor is this cell's parent row, nor parent table.
      //   Everything will need to be re-validated upon display.
      if (!sColSpan.Equals(s))
      {
        Invalidate();
      }
      sColSpan = s;
    }
    
    /// <summary>
    /// Call this method to set the value of the rowspan attribute.
    /// </summary>
    /// <param name="s">String value.</param>
    /// <throws>HTMLException</throws>
    public virtual void SetRowSpan(String s)
    {
      //   If a change is made to this state, then this cell is no longer
      //   validated.  Nor is this cell's parent row, nor parent table.
      //   Everything will need to be re-validated upon display.
      if ((sRowSpan == null) || (!sRowSpan.Equals(s)))
      {
        Invalidate();
      }
      sRowSpan = s;
    }
    //   To Be Deprecated
    
    /// <summary>
    /// Call this method to set the value of the headers attribute.
    /// </summary>
    /// <param name="s">String value.</param>
    public virtual void SetHeaders(String s)
    {
      sHeaders = s;
    }
    
    /// <summary>
    /// Call this method to set the value of the height attribute.
    /// </summary>
    /// <param name="s">String value.</param>
    public virtual void SetHeight(String s)
    {
      sHeight = s;
    }
    //   To Be Deprecated
    
    /// <summary>
    /// Call this method to toggle the nowrap attribute.
    /// </summary>
    /// <param name="b">value</param>
    public virtual void SetNoWrap(bool b)
    {
      bNoWrap = b;
    }
    
    /// <summary>
    /// Call this method to set the value of the valign attribute.
    /// </summary>
    /// <param name="s">String value</param>
    public virtual void SetVAlign(String s)
    {
      //   This such a change does not invalidate.
      sVAlign = s;
    }
    //   To Be Deprecated
    
    /// <summary>
    /// Call this method to set the value of the width attribute.
    /// </summary>
    /// <param name="s">String value.</param>
    public virtual void SetWidth(String s)
    {
      sWidth = s;
    }
    
    /// <summary>
    /// This method generates the beginning of the opening tag.  The abstract
    /// HTMLElement superclass will provide the remainder of the opening tag,
    /// repleate with attributes common to all html elements, such as name, and
    /// id, and class.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    /// <throws>HTMLException</throws>
    protected override void AppendBeginningOfOpeningTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      Validate();
      int iNumColsInParentTable = myParentTable.GetNumColsSpanned();
      int iNumColsInParentRow = myParentRow.GetNumColsSpanned();
      //        if (bDebug)
      //        {
      //            myLogger.Debug("getDisplayString - iNumColsInParentTable: " + iNumColsInParentTable);
      //            myLogger.Debug("getDisplayString - iNumColsInParentRow: " + iNumColsInParentRow);
      //        }
      WriteNugget(BEGINTAG, sb, response);
      if (sAlign != null)
      {
        WriteNugget(ALIGN_ATTR, sb, response);
        WriteNugget(sAlign, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sVAlign != null)
      {
        WriteNugget(VALIGN_ATTR, sb, response);
        WriteNugget(sVAlign, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sBGColor != null)
      {
        WriteNugget(BGCOLOR_ATTR, sb, response);
        WriteNugget(sBGColor, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sHeaders != null)
      {
        WriteNugget(HEADERS_ATTR, sb, response);
        WriteNugget(sHeaders, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sHeight != null)
      {
        WriteNugget(HEIGHT_ATTR, sb, response);
        WriteNugget(sHeight, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sWidth != null)
      {
        WriteNugget(WIDTH_ATTR, sb, response);
        WriteNugget(sWidth, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (bNoWrap)
      {
        WriteNugget(NOWRAP_ATTR, sb, response);
      }
      if (!bAutoFit)
      {
        // this cell doesn not auto fit.
        // note: can't have column and row spanning in the same element
        if (sRowSpan != null)
        {
          WriteNugget(ROWSPAN_ATTR, sb, response);
          WriteNugget(sRowSpan, sb, response);
          WriteNugget(QUOTE, sb, response);
        } else
        // sColSpan isn't NULL unless set so by user; default is ONE
        if (sColSpan != null)
        {
          WriteNugget(COLSPAN_ATTR, sb, response);
          WriteNugget(sColSpan, sb, response);
          WriteNugget(QUOTE, sb, response);
        }
      } else
      {
        //  This cell auto fits.
        int iTemp = (iNumColsInParentTable - iNumColsInParentRow) + 1;
        // note that it's either column or row spanning, not both
        if (sRowSpan != null)
        {
          WriteNugget(ROWSPAN_ATTR, sb, response);
          WriteNugget(sRowSpan, sb, response);
          WriteNugget(QUOTE, sb, response);
        } else
        // sColSpan isn't NULL unless set so by user; default is ONE
        if (sColSpan != null)
        {
          // note use of autofit calculated colspan
          WriteNugget(COLSPAN_ATTR, sb, response);
          WriteNugget(XPConvert.ToString(iTemp), sb, response);
          WriteNugget(QUOTE, sb, response);
        }
      }
    }
    
    /// <summary>
    /// This method generates the closing tag of this html element.  If the
    /// providesClosingTag method returns false, then this method will never be
    /// called.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendClosingTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(ENDTAG, sb, response);
      AppendClosingLineBreak(sb, response);
    }
    
    /// <summary>
    /// Indicates whether or not this HTML element provides a closing tag. Some
    /// elements only provide opening tags, such as the &lt;br&gt; element.
    /// </summary>
    /// <returns>boolean true.</returns>
    protected override bool ProvidesClosingTag()
    {
      return true;
    }
    
    /// <summary>
    /// Get the next id value.  This is used by the GetClone method to ensure
    /// that cloned object have new ID attribute values.
    /// </summary>
    /// <returns>A valid id string.</returns>
    internal override String GetNextID()
    {
      String s = "tabcell" + iNextID;
      iNextID++;
      return s;
    }
    
    /// <summary>
    /// Call this method to get the HTMLRow to which this HTMLTableCell belongs.
    /// </summary>
    /// <returns>HTMLTableRow parent.</returns>
    internal virtual HTMLTableRow GetParentRow()
    {
      return myParentRow;
    }
    
    /// <summary>
    /// Call this method to obtain the HTMLTable to which this cell belongs.
    /// </summary>
    /// <returns>HTMLTable parent table.</returns>
    internal virtual HTMLTable GetParentTable()
    {
      return myParentTable;
    }
    
    /// <summary>
    /// Call this method to set the HTMLTableRow to which this cell belongs.
    /// </summary>
    /// <param name="r">HTMLTableRow parent row.</param>
    internal virtual void SetParentRow(HTMLTableRow r)
    {
      myParentRow = r;
    }
    
    /// <summary>
    /// Call this method to set the HTMLTable to which this cell belongs.
    /// </summary>
    /// <param name="t">HTMLTable parent table.</param>
    internal virtual void SetParentTable(HTMLTable t)
    {
      myParentTable = t;
    }
    internal virtual void Invalidate()
    {
      //        if (bInfo)
      //        {
      //            myLogger.Info("Entering invalidate()");
      //        }
      if (myParentTable != null)
      {
        myParentTable.Invalidate();
      }
      if (myParentRow != null)
      {
        myParentRow.Invalidate();
      }
      bValidated = false;
    }
    
    /// <summary>
    /// Call this method at display time to cause this HTMLCell to prepare
    /// inself for display.
    /// </summary>
    /// <throws>HTMLException</throws>
    internal virtual void Validate()
    {
      //        if (bInfo)
      //        {
      //            myLogger.Info("Entering validate");
      //        }
      if (bValidated)
      {
        //            if (bDebug)
        //            {
        //                myLogger.Debug("Already validated, returning");
        //            }
        return;
      }
      if (myParentRow == null)
      {
        throw new HTMLException("The parent row of this cell has not been set.");
        }
      //   set the parent table.  This may have already been set, but we'll
      //   just set it again to be sure.
      myParentTable = myParentRow.GetParentTable();
    }
  }
}