using System;
//import com.plumtree.openfoundation.logging.*;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// This class can be used to generate <tr> html elements.
  /// <br /><br />
  /// <a target="new" href="http://www.w3.org/TR/html4/struct/tables.html#h-11.2.5">Spec</a>
  /// </summary>
  public class HTMLTableRow : HTMLElement
  {
    private String sAlign;
    private String sBGColor;
    private String sBorderColor;
    private String sVAlign;
    private int iNumColsSpanned;
    internal static int iNextID = 0;
    private int iIndexInTable = -1;
    private bool bHasAutoFitElement;
    private HTMLTable myParentTable;
    private bool bValidated;
    private static readonly byte[] BEGINTAG = HTMLElement.ToByteArray("<tr");
    private static readonly byte[] ALIGN_ATTR = HTMLElement.ToByteArray(" align=\"");
    private static readonly byte[] BGCOLOR_ATTR = HTMLElement.ToByteArray(" bgcolor=\"");
    private static readonly byte[] BORDERCOLOR_ATTR = HTMLElement.ToByteArray(" bordercolor=\"");
    private static readonly byte[] VALIGN_ATTR = HTMLElement.ToByteArray(" valign=\"");
    private static readonly byte[] ENDTAG = HTMLElement.ToByteArray("</tr>");
    
    /// <summary>
    /// Call this method to create a new HTMLTableRow object.
    /// </summary>
    /// <param name="tParent">
    /// Pass the HTMLTable to which this HTMLTableRow will be
    /// added.  A given instance of an HTMLTableRow may only be added to the
    /// HTMLTable specified to this constructor.
    /// <br /><br />
    /// </param>
    /// <deprecated>
    /// It's no longer necessary to pass the parent table, use
    /// constructor with no parameters.
    /// </deprecated>
    public HTMLTableRow(HTMLTable tParent)
    {
      sAlign = null;
      sBGColor = null;
      sBorderColor = null;
      sVAlign = null;
      iNumColsSpanned = 0;
      bHasAutoFitElement = false;
      bValidated = false;
      myParentTable = tParent;
    }
    
    /// <summary>
    /// Creates a new HTMLTableRow object.
    /// </summary>
    public HTMLTableRow()
    {
      sAlign = null;
      sBGColor = null;
      sBorderColor = null;
      sVAlign = null;
      iNumColsSpanned = 0;
      bHasAutoFitElement = false;
      bValidated = false;
      myParentTable = null;
    }
    
    /// <summary>
    /// Call this method to obtain an clone of this table row.
    /// </summary>
    /// <returns>Object that is an HTMLTableRow. </returns>
    public override Object GetClone()
    {
      HTMLTableRow rClone = (HTMLTableRow) base.GetClone();
      IXPEnumerator xpEnumerator = rClone.GetInnerHTMLElements(false);
      HTMLTableCell c;
      while (xpEnumerator.MoveNext())
      {
        c = (HTMLTableCell) xpEnumerator.GetCurrent();
        c.SetParentRow(this);
      }
      return rClone;
    }
    
    /// <summary>
    /// Get the next id value.  This is used by the GetClone method to ensure
    /// that cloned object have new ID attribute values.
    /// </summary>
    /// <returns>A valid id string.</returns>
    internal override String GetNextID()
    {
      String s = "tabrow" + iNextID;
      iNextID++;
      return s;
    }
    //   package-private method
    
    /// <summary>
    /// Call this method to obtain the index of this row in its parent table.
    /// This is calculated at display time.
    /// </summary>
    /// <returns>int index.</returns>
    internal virtual int GetIndexInTable()
    {
      return iIndexInTable;
    }
    //   package-private method
    
    /// <summary>
    /// Call this method to set the index of this row in its parent table.
    /// </summary>
    /// <param name="i">int index.</param>
    internal virtual void SetIndexInTable(int i)
    {
      iIndexInTable = i;
    }
    
    /// <summary>
    /// This method generates the beginning of the opening tag.  The abstract
    /// HTMLElement superclass will provide the remainder of the opening tag,
    /// repleate with attributes common to all html elements, such as name, and
    /// id, and class.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendBeginningOfOpeningTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      Validate();
      WriteNugget(BEGINTAG, sb, response);
      if (sAlign != null)
      {
        WriteNugget(ALIGN_ATTR, sb, response);
        WriteNugget(sAlign, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sBGColor != null)
      {
        WriteNugget(BGCOLOR_ATTR, sb, response);
        WriteNugget(sBGColor, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sBorderColor != null)
      {
        WriteNugget(BORDERCOLOR_ATTR, sb, response);
        WriteNugget(sBorderColor, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sVAlign != null)
      {
        WriteNugget(VALIGN_ATTR, sb, response);
        WriteNugget(sVAlign, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
    }
    
    /// <summary>
    /// This method generates the closing tag of this html element.  If the
    /// providesClosingTag method returns false, then this method will never
    /// be called.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendClosingTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(ENDTAG, sb, response);
      AppendClosingLineBreak(sb, response);
    }
    
    /// <summary>
    /// Indicates whether or not this HTML element provides a closing tag.
    /// Some elements only provide opening tags, such as the &lt;br&gt; element.
    /// </summary>
    /// <returns>boolean true.</returns>
    protected override bool ProvidesClosingTag()
    {
      return true;
    }
    
    /// <summary>
    /// Call this method to obtain a reference to the HTMLTable to which
    /// this HTMLTableRow may be added.
    /// </summary>
    public virtual HTMLTable GetParentTable()
    {
      return myParentTable;
    }
    // package-private
    
    /// <summary>
    /// Call this method to set the table to which this row belongs.
    /// </summary>
    /// <param name="t">HTMLTable parent table.</param>
    internal virtual void SetParentTable(HTMLTable t)
    {
      myParentTable = t;
    }
    
    /// <summary>
    /// Call this method to determine whether or not this HTMLTableRow
    /// has any autofit cells.
    /// </summary>
    public virtual bool GetHasAutoFitElement()
    {
      //        if (bInfo)
      //        {
      //            myLogger.Info("Entering GetHasAutoFitElement()");
      //        }
      Validate();
      //        if (bDebug)
      //        {
      //            myLogger.Debug("GetHasAutoFitElement() - bHasAutoFitElement: " + bHasAutoFitElement);
      //        }
      return bHasAutoFitElement;
    }
    
    /// <summary>
    /// Call this method to set the value of the align attribute.
    /// </summary>
    /// <param name="s">String value</param>
    public virtual void SetAlign(String s)
    {
      sAlign = s;
    }
    
    /// <summary>
    /// Call this method to obtain the value of the align attribute.
    /// </summary>
    /// <returns>String value</returns>
    public virtual String GetAlign()
    {
      return sAlign;
    }
    
    /// <summary>
    /// Call this method to set the value of the bgcolor attribute.
    /// </summary>
    /// <param name="s">String value</param>
    public virtual void SetBGColor(String s)
    {
      sBGColor = s;
    }
    
    /// <summary>
    /// Call this method to obtain the value of the bgcolor attribute.
    /// </summary>
    /// <returns>String value</returns>
    public virtual String GetBGColor()
    {
      return sBGColor;
    }
    
    /// <summary>
    /// Call this method to set the value of the bordercolor attribute.
    /// </summary>
    /// <param name="s">String value</param>
    public virtual void SetBorderColor(String s)
    {
      sBorderColor = s;
    }
    
    /// <summary>
    /// Call this method to obtain the value of the bordercolor attribute.
    /// </summary>
    /// <returns>String value</returns>
    public virtual String GetBorderColor()
    {
      return sBorderColor;
    }
    
    /// <summary>
    /// Call this method to set the value of the valign attribute.
    /// </summary>
    /// <param name="s">String value</param>
    public virtual void SetVAlign(String s)
    {
      sVAlign = s;
    }
    
    /// <summary>
    /// Call this method to obtain the value of the valign attribute.
    /// </summary>
    /// <returns>String value</returns>
    public virtual String GetVAlign()
    {
      return sVAlign;
    }
    
    /// <summary>
    /// This method returns the number of columns spanned by all of the cells
    /// in this row.  An AutoFit cell, if present, is assumed to span 1 column.
    /// </summary>
    /// <returns></returns>
    /// <throws>HTMLException</throws>
    public virtual int GetNumColsSpanned()
    {
      Validate();
      return iNumColsSpanned;
    }
    
    /// <summary>
    /// This method should be called whenever a change has been made
    /// to any member HTMLTableCell objects that would cause this 
    /// HTMLTableRow to need to validate.  
    /// For instance, a change to the colspan attribute of an HTMLTableCell
    /// should call the invalidate() method of that cell's parent HTMLTablRow.
    /// This way, when the HTMLTable is called to display itself, the column
    /// spans will be recalculated.
    /// </summary>
    internal virtual void Invalidate()
    //   package-private access level
    {
      //        if (bInfo)
      //        {
      //            myLogger.Info("Entering invalidate()");
      //        }
      bValidated = false;
      if (myParentTable != null)
      {
        myParentTable.Invalidate();
      }
    }
    
    /// <summary>
    /// Call this method to validate an HTMLTableCell.
    /// </summary>
    /// <param name="c">HTMLTableCell</param>
    /// <throws>
    /// HTMLException This method throws an exception if the HTMLTableCell
    /// passed cannot be added to this row for any reason.
    /// </throws>
    private void ProcessCellForValidation(HTMLTableCell c)
    {
      if (c.GetAutoFit())
      {
        //            if (bDebug)
        //            {
        //                myLogger.Debug("validate - found an autofit cell.");
        //            }
        if (bHasAutoFitElement)
        {
        }
        //            	String s =  "Only one autofit element may exist in each row.  Row at index " + this.GetIndexInTable() + " has multiple autofit cells.";
        //                if (bError)
        //                {
        //                    myLogger.Error( s );
        //                }
        //                throw new HTMLException(s);
        //  set the class member var
        bHasAutoFitElement = true;
      }
      //
      //        if (bDebug)
      //        {
      //            myLogger.Debug("validate - c.GetColSpanAsInt(): " + c.GetColSpanAsInt());
      //        }
      iNumColsSpanned = iNumColsSpanned + c.GetColSpanAsInt();
    }
    
    /// <summary>
    /// Calling this method ensures that there is only one autofit cell
    /// added to this HTMLTableRow.  Calling this method also calculates
    /// the number of columns spanned by this row.
    /// It is intended for this method to be called only when this object
    /// is actually displaying.
    /// Once this method is called, this object will be in a validated state.
    /// Calling this method subsequently will not do anything.  Only after a
    /// call to the invalidate() method will a call to the validate() method
    /// perform any significant action.
    /// </summary>
    internal virtual void Validate()
    {
      //        if (bInfo)
      //        {
      //            myLogger.Info("Entering validate()");
      //        }
      if (bValidated)
      {
        //            if (bDebug)
        //            {
        //                myLogger.Debug("Already validated, returning.");
        //            }
        return;
      }
      IXPEnumerator e = GetInnerHTMLElements(false);
      if (e == null)
      {
      } else
      //String s = "This row has no cells.  All rows must have at least one cell.";
      //			if( bError )
      //			{
      //				myLogger.Error( s );
      //			}
      //			throw new HTMLException( s );
      {
        HTMLTableCell c;
        iNumColsSpanned = 0;
        while (e.MoveNext())
        {
          c = (HTMLTableCell) e.GetCurrent();
          ProcessCellForValidation(c);
        }
      }
      bValidated = true;
    }
    
    /// <summary>
    /// Call this method to add an HTMLTableCell to this HTMLTableRow.
    /// The HTMLTableCell being added to this row must have been
    /// constructed with a reference to this particular HTMLTableRow.
    /// In this way, a given instance of an HTMLTableCell "belongs" to
    /// a given instance of an HTMLTableRow.
    /// </summary>
    public override void AddInnerHTMLElement(HTMLElement e)
    {
      if (!(e is HTMLTableCell) && !(e is HTMLElementCollection))
      {
        throw new HTMLException("You may only add HTMLCell or HTMLElementCollection elements to an HTMLTableRow.");
        }
      if (e is HTMLElementCollection)
      {
        IXPEnumerator enumItems = e.GetInnerHTMLElements(false);
        if (enumItems != null)
        {
          HTMLElement eTemp;
          while (enumItems.MoveNext())
          {
            eTemp = (HTMLElement) enumItems.GetCurrent();
            //                    if (bDebug)
            //                    {
            //                        myLogger.Debug("eTemp: " + eTemp);
            //                    }
            if (!(eTemp is HTMLTableCell))
            {
              throw new HTMLException("You may only add an HTMLElementCollection containing HTMLTableCell elements to an HTMLTableRow element.  You are attempting to add an: " + eTemp);
              }
            ProcessCellForAdding((HTMLTableCell) eTemp);
          }
        }
      } else
      //   for each inner element
      //   got inner elements
      // e instanceof HTMLElementCollection
      {
        ProcessCellForAdding((HTMLTableCell) e);
      }
      Invalidate();
    }
    
    /// <summary>
    /// Call this method to add an HTMLTableCell to this row
    /// </summary>
    /// <param name="c">HTMLTableCell</param>
    /// <throws>
    /// HTMLException This method throws an exception if the HTMLTableCell
    /// passed cannot be added to this row for any reason.
    /// </throws>
    private void ProcessCellForAdding(HTMLTableCell c)
    {
      if (c.GetParentRow() == null)
      {
        //   the cell's parent hasn't been set.
        //   set it to this row
        c.SetParentRow(this);
      } else
      {
        //   the cell's parent has been set, verify that it's set to
        //   this row.
        if (c.GetParentRow() != this)
        {
          throw new HTMLException("You may only add HTMLTableCell objects constructed with a reference to this instance of an HTMLTableRow.");
          }
      }
      base.AddInnerHTMLElement(c);
    }
  }
}