using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// <br /><br />
  /// <a target="new"
  /// href="http://www.w3.org/TR/html4/interact/forms.html#h-17.7">Spec</a>
  /// </summary>
  public class HTMLTextArea : HTMLGenericInputElement
  {
    internal static int iNextID = 0;
    private static readonly byte[] BEGIN_TAG = HTMLElement.ToByteArray("<textarea");
    private static readonly byte[] TAB_INDEX_ATTR = HTMLElement.ToByteArray(" tabindex=\"");
    private static readonly byte[] COLS_ATTR = HTMLElement.ToByteArray(" cols=\"");
    private static readonly byte[] ROWS_ATTR = HTMLElement.ToByteArray(" rows=\"");
    private static readonly byte[] READ_ONLY_ATTR = HTMLElement.ToByteArray(" readonly=\"readonly\"");
    private static readonly byte[] DISABLED_ATTR = HTMLElement.ToByteArray(" disabled=\"disabled\"");
    private static readonly byte[] END_TAG = HTMLElement.ToByteArray("</textarea>");
    private String sCols;
    private String sRows;
    private String sTabIndex;
    private bool bDisabled = false;
    private bool bReadOnly = false;
    
    /// <param name="cols">Pass the cols string of this textarea.</param>
    /// <param name="rows">Pass the rows string of this textarea.</param>
    /// <throws>HTMLException</throws>
    public HTMLTextArea(String cols, String rows)
    {
      try
      {
        XPConvert.ToInteger(cols);
        XPConvert.ToInteger(rows);
      }
      catch (Exception nfe)
      {
        throw new HTMLException("You must pass String representations of integers to the HTMLTextArea constructor.");
        }
      //   by default, textarea elements do not format.
      SetDoFormat(false);
      sRows = rows;
      sCols = cols;
      sTabIndex = null;
      // rob: removing the validation checks. main reason is to avoid the validation check on textbox for banner search with friendly url
      // (bug #70204) but it is also not necessary anymore since we are moving to layouts
      SetValidateName(false);
    }
    
    /// <summary>
    /// Call this method to get the value of the cols attribute.
    /// </summary>
    /// <returns>String value</returns>
    public virtual String GetCols()
    {
      return sCols;
    }
    
    /// <summary>
    /// Get the value of the readonly attribute.
    /// </summary>
    /// <returns>boolean value</returns>
    public virtual bool GetReadOnly()
    {
      return bReadOnly;
    }
    
    /// <summary>
    /// Call this method to get the value of the rows attribute.
    /// </summary>
    /// <returns>String value</returns>
    public virtual String GetRows()
    {
      return sRows;
    }
    
    /// <summary>
    /// Set the value of the readonly attribute.
    /// </summary>
    /// <param name="bInputReadOnly">value</param>
    public virtual void SetReadOnly(bool bInputReadOnly)
    {
      bReadOnly = bInputReadOnly;
    }
    
    /// <summary>
    /// Set the value of the Disabled attribute.
    /// </summary>
    /// <param name="bInputDisabled">value</param>
    public override void SetDisabled(bool bInputDisabled)
    {
      bDisabled = bInputDisabled;
    }
    
    /// <summary>
    /// This method rewritten to prevent usage.
    /// </summary>
    /// <param name="s">String value</param>
    /// <throws>HTMLException This method throws an exception.</throws>
    public override void SetValue(String s)
    {
      throw new HTMLException("The <textarea> element does not support the value attribute, use addInnerHTMLString().");
      }
    
    /// <summary>
    /// Call this method to validate the name of this HTMLTextArea.
    /// </summary>
    /// <param name="s">String name.</param>
    /// <throws>
    /// HTMLException This method throws an exception if the name passed
    /// is not valid.
    /// </throws>
    public override void ValidateName(String s)
    {
      if (!XPStringUtility.StartsWith(s, "in_ta_"))
      {
        throw new HTMLException("The name of a <textarea> element must begin with \"in_ta_\".  " + s + " is an invalid name.");
        }
    }
    
    /// <summary>
    /// HTMLElement.InternalWrite rewritten to prevent formatting.
    /// </summary>
    /// <param name="bFormat">
    /// Pass a boolean indicating whether or not to format the
    /// output.
    /// </param>
    /// <param name="sb">
    /// Pass an XPStringBuilder to which you wish the String appended.
    /// Pass null if you do not wish the string appended to an
    /// XPStringBuilder.
    /// </param>
    /// <param name="response">
    /// Pass an IXPResponse to which you wish the String written.
    /// Pass null if you do not wish the string written to an
    /// IXPResponse.
    /// </param>
    /// <throws>HTMLException</throws>
    protected override void InternalWrite(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      base.InternalWrite(false, sb, response);
    }
    
    /// <summary>
    /// This method generates the beginning of the opening tag.  The abstract
    /// HTMLElement superclass will provide the remainder of the opening tag,
    /// repleate with attributes common to all html elements, such as name, and
    /// id, and class.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendBeginningOfOpeningTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(BEGIN_TAG, sb, response);
      if (sTabIndex != null)
      {
        WriteNugget(TAB_INDEX_ATTR, sb, response);
        WriteNugget(sTabIndex, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      WriteNugget(COLS_ATTR, sb, response);
      WriteNugget(sCols, sb, response);
      WriteNugget(QUOTE, sb, response);
      WriteNugget(ROWS_ATTR, sb, response);
      WriteNugget(sRows, sb, response);
      WriteNugget(QUOTE, sb, response);
      if (bReadOnly)
      {
        WriteNugget(READ_ONLY_ATTR, sb, response);
      }
      if (bDisabled)
      {
        WriteNugget(DISABLED_ATTR, sb, response);
      }
    }
    
    /// <summary>
    /// Returns the tab index.
    /// </summary>
    /// <returns>String the tab index.</returns>
    public virtual String GetTabIndex()
    {
      return sTabIndex;
    }
    
    /// <summary>
    /// Sets the tab index.
    /// </summary>
    public virtual void SetTabIndex(String _sTabIndex)
    {
      sTabIndex = _sTabIndex;
    }
    
    /// <summary>
    /// Sets sTabIndex to null.
    /// </summary>
    public virtual void ClearTabIndex()
    {
      sTabIndex = null;
    }
    
    /// <summary>
    /// This method generates the closing tag of this html element.  If the
    /// providesClosingTag method returns false, then this method will never be
    /// called.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendClosingTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(END_TAG, sb, response);
    }
    
    /// <summary>
    /// Indicates whether or not this HTML element provides a closing tag. Some
    /// elements only provide opening tags, such as the &lt;br&gt; element.
    /// </summary>
    /// <returns>boolean true.</returns>
    protected override bool ProvidesClosingTag()
    {
      return true;
    }
    
    /// <summary>
    /// Get the next id value.  This is used by the GetClone method to ensure
    /// that cloned object have new ID attribute values.
    /// </summary>
    /// <returns>A valid id string.</returns>
    internal override String GetNextID()
    {
      String s = "textarea" + iNextID;
      iNextID++;
      return s;
    }
  }
}