package com.plumtree.uiinfrastructure.activityspace;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.uitasks.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.login.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.web.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * AActivitySpaces contain the MVC setup for a set of Display Pages, as well
 * as the Display Pages themselves.  All objects should come to their parent
 * AActivitySpace for access to other objects, such as IModels, IViews, and
 * user objects like IPTSession and UserPreferences.
 *
 * Activity spaces setup and manage the interaction between the different MVC
 * objects and the IDisplayPage.  They initialize all of the objects, and can
 * get and set data on the Models if necessary.  Activity Spaces are not subject
 * to the strict data/logic/presentation separation that MVC components are.
 *
 * This class provides helper implementations of the methods to get MVC Objects and
 * manages them automatically (IModel, IView, IControl, IDisplayPage).
 * Just register things and then call GetModel, etc... to access them.
 *
 * It also provides base implementations of some of the common AActivitySpace
 * functionality.
 * 
 * @author Don Hayler
 */
public abstract class AActivitySpace implements IMVCObject, IManagedObject, IEnvironment
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.activityspace.AActivitySpace");

	/** constants */

	/**
	 * NOTE: This number needs to be tuned.  Should probably be linked to cache
	 * size in Interpreter somehow.
	 */
	private static final int m_nSUBSPACE_WARNING_MAX = 5;

	/** Security mode on this activity space */
	private boolean m_bIsSecuredSpace = false;

	/** Application */
	private IApplication m_apApplication   = null;

	/** The current Display Page. */
	private IDisplayPage m_asdpCurrentPage = null;

	/** Sub-session (cleared on logout) */
	private ISessionManager m_smSession = null;

	/** Persistent Sub-session (NOT cleared on logout) */
	private ISessionManager m_smPersistentSession = null;

	/** generic user session. */
	private Object m_UserSession = null;

	/** sub-space key.  If this is null, not a sub-space (default) */
	private String m_sSubSpaceKey = null;
	private String m_strParentSpaceID = null;

	/** The name and ID of the parent space (i.e. the one that launched this one) */
	private String m_strParentSpaceName = null;

	/** the full url of the current request */
	private String m_strCurrentRequestUrl = null;

	/** the normalized base url of the current request */
	private String m_strCurrentNormalizedBaseUrl = null;

	/** The HTTP Request object for the current request - should be released at end of request. */
	private IXPRequest m_xpRequest = null;

	/** The HTTP Response object for the current request - should be released at end of request. */
	private IXPResponse m_xpResponse = null;

	/** Hastable with the name/value pairs from XPRequest (This for PEI use only.) */
	private XPHashtable m_htFormData = null;

	/** The current access style (cached). */
	private AccessStyles m_Style = null;

	/**
	 * The Space ID is the unique key used to locate spaces in the AS cache
	 * (in the Interpreter)
	 */
	private String m_strSpaceID = null;
	private XPHashtable m_mControlMap;

	/** Registered objects */
	private XPHashtable m_mModelMap;
	private XPHashtable m_mPageMap;
	private XPHashtable m_mSubSpaceMap;
	private XPHashtable m_mViewMap;

	private String m_sHelpTopic = null;

	/**
	 * overwrite sub-spaces?
	 * This needs to be true by default because someone can pop-up a sub-space
	 * and close it before it gets a chance to run it's javascript, which can
	 * lead to the opening page getting confused and opening a new pop-up,
	 * which then re-registers and fails (if this is set to false).  Therefore,
	 * it needs to be set to true.  This may hide bugs in someone's use of the
	 * sub-space code, but I can't see any way around this.
	 */
	private boolean m_bOverWriteSubSpaces = true;

	/** Current Display Page */
	private boolean m_bTrackCurrentPage = false;

    /**
     * This hashtable is used to store references to extremely long query string parameters.
     * 
     * e.g. hide lists to tree control
     * 
     * Only certain parts of the tree control know how to use this.
     * @author olegs
     */
    protected XPHashtable m_stringReferences;

	/** True to enable this AS for in place refresh. */
	private boolean m_bInPlaceRefreshEnabled = false;
	
	/** True to have Display Pages show in place content. */
	private boolean m_bInPlaceRefreshDisplayOn = false;

	///////////////////
	// Abstract methods
	///////////////////

	/**
	 * This method is called by the Interpreter at the beginning of every HTTP
	 * request to determine whether or not the current user has the security
	 * privileges necessary to view the requested page.
	 *
	 * This method is designed for checking whether or not a particular user has
	 * any access to the space, page, and control.  More detailed checking (of
	 * query string parameters, for instance), should be done in the
	 * IControl.CheckActionSecurityAndExecute() method.
	 *
	 * For Plumtree portal applications (IPTActivitySpaces), since the IPTSession
	 * is guaranteed to be set before Init() & CheckBasicAccess() are called, you
	 * can check access as follows:
	 *
	 * return (GetIPTSession().GetSessionInfo().CheckForActivityRight
	 *         (new Integer(PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_ACCESS_ADMINISTRATION))
	 *         == 1);
	 *
	 * @param String the requested IDisplayPage name (may be null for default)
	 * @param String the requested IControl name (may be null if not used)
	 * @param boolean true implies this is from an internal redirect from the same space
	 *                    false implies this is an URL from the user, or a redirect from
	 *                         another space.
	 *
	 * @return boolean true implies access is allowed, false implies access is denied
	 */
	abstract public boolean CheckBasicAccess(String strPage, String strControl,
		boolean bSameSpace);

	/**
	 * This method is used by the ASManager to return new instances of managed
	 * objects.
	 *
	 * @return A new instance of the managed class (i.e. return new Foo();)
	 */
	abstract public Object Create();

	/**
	 * This method is used to identify the type of this space.
	 *
	 * @return String the name of this AActivitySpace
	 */
	abstract public String GetName();

	///////////////////////
	// Non-abstract methods
	///////////////////////


	public void SetHelpTopic( String s )
	{
		// Toward the end of the release, help filenames all became
		// lowercase.
		m_sHelpTopic = XPStringUtility.ToLowerCase(s);
	}
	
	public String GetHelpTopic()
	{
		return m_sHelpTopic;
	}
	

	/**
	 * Changes the key of a sub space.
	 *
	 * @param String the key of the sub-space to change the key
	 * @param String  the new key for that sub space
	 */
	public void ChangeSubSpaceKey(String subSpaceKey, String subSpaceNewKey)
	{
		if (m_mSubSpaceMap.ContainsKey(subSpaceKey))
		{
			if (!m_mSubSpaceMap.ContainsKey(subSpaceNewKey))
			{
				AActivitySpace asSubSpace = (AActivitySpace) m_mSubSpaceMap.RemoveElement(subSpaceKey);
				m_mSubSpaceMap.PutElement(subSpaceNewKey, asSubSpace);
				asSubSpace.SetSubSpaceKey(subSpaceNewKey);

				
					log.Info(GetName() + " ChangeSubSpaceKey key for sub space: " +
						asSubSpace.GetName() + " has been changed from: " +
						subSpaceKey + " to: " + subSpaceNewKey);
				
			}
			else
			{
				
					log.Error(GetName() + " ChangeSubSpaceKey failed: New key " +
						subSpaceNewKey + " is already used.");
				
			}
		}
		else
		{
			
				log.Error(GetName() +
					" ChangeSubSpaceKey failed: Could not find sub-space " +
					subSpaceKey);
			
		}
	}

	/**
	 * Delete the specified Sub-space and remove it from the Activity
	 * Space cache.  This is permanent!
	 *
	 * The sub-space key is different from the space ID.
	 *
	 * @param String the sub-space key of the Sub-space to delete
	 */
	public void DeleteSubSpace(String subSpaceKey)
	{
		if ((null == subSpaceKey) || ("".equals(subSpaceKey)))
		{
			
				log.Error(GetName() +
					" DeleteSubSpace failed: Called with empty subspacekey.");
			

			return;
		}

		AActivitySpace asDelete = (AActivitySpace) m_mSubSpaceMap.RemoveElement(subSpaceKey);

		if (null == asDelete)
		{
			
				log.Error(GetName() +
					" DeleteSubSpace failed: Could not find sub-space " +
					subSpaceKey);
			

			return;
		}

		ASCache myCache = (ASCache) m_smSession.GetAttribute(ASConstants.SANDBOXASCACHE);
		myCache.RemoveSubSpace(asDelete.GetName(), asDelete.GetSpaceID());
	}

	/**
	 * This method returns the current access style (i.e. Section
	 * 508 compliance, low bandwidth, or standard).
	 *
	 * @see com.plumtree.uiinfrastructure.constants.AccessStyles
	 *
	 * @return AccessStyles The access style
	 */
	public AccessStyles GetAccessStyle()
	{
		// NOTE: This method gets called approximately 30 times per
		// page hit.  XPConvert is relatively cheap, so we probably
		// don't need to optimize this (by storing a style object
		// rather than a String or an int), but we need to be careful
		// not to do expensive things here, like throw exceptions.

		// NEWER NOTE: With the new page layouts and tag engine, this 
		// gets called up to 1000 times per page, so we should cache the
		// result.

		// NOTE: the style is nulled out at the end of every request so
		// we can pick up new styles if it changes.
		if (m_Style != null)
		{
			return m_Style;
		}

		AccessStyles style = AccessStyles.STANDARD;

		Object accessStyle = PersonalSettingsHelper.GetPersonalSettingValue
			(PersonalSettingsHelper.ACCESS_STYLE, this);
		
		if (null != accessStyle)
		{
			try
			{
				String strAccessStyle = XPConvert.ToString(accessStyle);
				if (!"".equals(strAccessStyle))
				{
					int val = XPConvert.ToInteger(strAccessStyle);
					style = AccessStyles.GetAccessStyle(val);
				}
			}
			catch (XPFormatException e)
			{
				// goes to default
				
					log.Error(e, GetName() + " could not convert access style " +
						accessStyle + " correctly.");
				

				style = AccessStyles.STANDARD;
			}
		}

		m_Style = style;

		return m_Style;
	}
	
	/**
	 * Return true if the current access style is STANDARD otherwise
	 * return false.
	 * 
	 * @return true if the access style is STANDARD
	 */
	public boolean GetIsAccessStyleStandard()
	{
		AccessStyles style = GetAccessStyle();		
		if (style == AccessStyles.STANDARD)
		{
			return true;	
		}
		return false;
	}
	
	/**
	 * Return true if the current access style is not STANDARD (it is
	 * SECTION_508 or LOW_BANDWIDTH) otherwise return false.
	 * 
	 * @return true if the access style is not STANDARD
	 */
	public boolean GetIsAccessStyleNonStandard()
	{
		AccessStyles style = GetAccessStyle();		
		if (style != AccessStyles.STANDARD)
		{
			return true;	
		}
		return false;
	}	

	/**
	 * This method is used to return whether or not
	 * this space is secured (SSL) or not (regular HTTP).
	 * 
	 * @return boolean true implies secured
	 */
	public boolean GetIsSecuredSpace()
	{
		return m_bIsSecuredSpace;
	}

	/**
	 * Return the Application.
	 *
	 * @return the application
	 */
	public IApplication GetApplication()
	{
		return m_apApplication;
	}

	/**
	 * Return the requested Control.  This method should probably
	 * wrap control objects in aggregating control objects to do page specific tasks.
	 * In other words, the default package control should have no page specific
	 * knowledge.
	 *
	 * @param String the name of the control to get
	 *
	 * @return IControl the requested control
	 */
	public IControl GetControl(String controlName)
	{
		IControl result;

		if (controlName == null)
		{
			result = (IControl) m_mControlMap.GetElement("");
		}
		else
		{
			result = (IControl) m_mControlMap.GetElement(controlName);
		}

		if ((result == null) && (controlName != null))
		{
			
				log.Error(GetName() + " GetControl failed: Could not find control " +
					controlName);
			
		}

		// SE: Bug 62787 - Add diagnostics for tracking MVC registration and displaying
		log.Debug(GetName() + " GetControl succeeded: Found " + controlName);
		
		return result;
	}

	/**
	 * This method checks whether or not a specific control exists in this AActivitySpace.
	 *
	 * @param String the name of the control to check
	 *
	 * @return boolean whether or not the control exists
	 */
	public boolean GetControlExists(String controlName)
	{
		return m_mControlMap.ContainsKey(controlName);
	}

	/**
	 * Return an Enumerator for the registered controls.
	 *
	 * @return IXPEnumerator the registered controls
	 */
	public IXPEnumerator GetControlsEnumerator()
	{
		return m_mControlMap.GetElements();
	}

	/**
	 * Return the most recent DisplayPage.
	 *
	 * @return the most recent page
	 */
	public IDisplayPage GetCurrentPage()
	{
		return m_asdpCurrentPage;
	}

	/**
	 * Return the full url of the current request.
	 *
	 * @return the url of the current request
	 */
	public String GetCurrentRequestURL()
	{
		if (m_strCurrentRequestUrl == null)
		{
			throw new XPException(GetName() +
				" GetCurrentRequestURL called when URL has not been set.");
		}

		return m_strCurrentRequestUrl;
	}

	/**
	 * Return the normalized base url of the current request.
	 * The normalized base url is essentially the request url
	 * up to and including the '?'.
	 *
	 * @see com.plumtree.uiinfrastructure.statichelpers.ConfigHelpers#GetNormalizedBaseURL(String)
	 * 
	 * @return The normalized base url of the current request
	 */
	public String GetCurrentNormalizedBaseURL()
	{
		if (m_strCurrentNormalizedBaseUrl == null)
		{
			
				log.Error(GetName() + " GetCurrentNormalizedBaseURL called when URL has not been set.");
			
		}

		return m_strCurrentNormalizedBaseUrl;
	}

	/**
	 * Return the HTTP Request for the current request.
	 *
	 * @return the current request
	 */
	public IXPRequest GetCurrentHTTPRequest()
	{
		return m_xpRequest;
	}

	/**
	 * Return the HTTP Response for the current request.
	 *
	 * @return the current response
	 */
	public IXPResponse GetCurrentHTTPResponse()
	{
		return m_xpResponse;
	}

	/**
	 * JF- Returns the current name/value pairs from the Request.
	 * (This for PEI use only.)
	 * 
	 * @return XPHashtable
	 */
	public XPHashtable GetCurrentFormData()
	{
		if (m_htFormData == null)
		{
			
				log.Error(GetName() + " GetCurrentFormData called when form data has not been set.");
			
		}		
		
		return m_htFormData;
	}

	/**
	 * Gets the current language of the user.
	 * 
	 * I.E. en, fr, jp
	 * 
	 * @return The current language of the user
	 */
	public String GetLanguage()
	{
		return (String) PersonalSettingsHelper.GetPersonalSettingValue
			(PersonalSettingsHelper.LANGUAGE, this);
	}

	/**
	 * Gets the current locale of the user.
	 * 
	 * I.E. en-us, en-gb, fr-fr, jp-jp
	 * 
	 * @return The current locale of the user.
	 */
	public String GetLocale()
	{
		return (String) PersonalSettingsHelper.GetPersonalSettingValue
			(PersonalSettingsHelper.LOCALE, this);
	}

	/**
	 * Return the requested Model.
	 *
	 * @param String the name of the model to get
	 *
	 * @return IModel the requested model
	 */
	public IModel GetModel(String modelName)
	{
		IModel result = (IModel) m_mModelMap.GetElement(modelName);

		if (result == null)
		{
			
				log.Error(GetName() + " GetModel failed: Could not find model " +
					modelName);
			
		}

		// SE: Bug 62787 - Add diagnostics for tracking MVC registration and displaying
		log.Debug(GetName() + " GetModel succeeded: Found " + modelName);

		return result;
	}

	/**
	 * This method checks whether or not a specific model exists in this AActivitySpace.
	 *
	 * @param String the name of the model to check
	 *
	 * @return boolean whether or not the model exists
	 */
	public boolean GetModelExists(String modelName)
	{
		return m_mModelMap.ContainsKey(modelName);
	}

	/**
	 * Return an Enumerator for the registered models.
	 *
	 * @return IXPEnumerator the registered models
	 */
	public IXPEnumerator GetModelsEnumerator()
	{
		return m_mModelMap.GetElements();
	}
	
	/**
	 * Return the requested DisplayPage.
	 *
	 * @param String the name of the IDisplayPage to get
	 *
	 * @return IDisplayPage the requested page
	 */
	public IDisplayPage GetPage(String pageName)
	{
		IDisplayPage result = (IDisplayPage) m_mPageMap.GetElement(pageName);

		if (result == null)
		{
			
				log.Error(GetName() + " GetPage failed: Could not find page " +
					pageName);
			
		}
		
		// SE: Bug 62787 - Add diagnostics for tracking MVC registration and displaying
		log.Debug(GetName() + " GetPage succeeded: Found " + pageName);

		return result;
	}

	/**
	 * This method checks whether or not a specific page exists in this AActivitySpace.
	 *
	 * @param String the name of the page to check
	 *
	 * @return boolean whether or not the page exists
	 */
	public boolean GetPageExists(String pageName)
	{
		return m_mPageMap.ContainsKey(pageName);
	}
	
	/**
	 * Return an Enumerator for the registered pages.
	 *
	 * @return IXPEnumerator the registered pages
	 */
	public IXPEnumerator GetPagesEnumerator()
	{
		return m_mPageMap.GetElements();
	}
	
	/**
	 * Return the parent AActivitySpace.  If there is no parent space,
	 * this will return null.
	 *
	 * @return AActivitySpace the parent AActivitySpace.  null if there
	 *             is no parent space.
	 */
	public AActivitySpace GetParentSpace()
	{
		if ((m_strParentSpaceName == null) || (m_strParentSpaceID == null))
		{
			// No parent space
			return null;
		}

		ASCache myCache = (ASCache) m_smSession.GetAttribute(ASConstants.SANDBOXASCACHE);

		return myCache.GetSpace(m_strParentSpaceName, m_strParentSpaceID);
	}

	/**
	 * Return the space ID of the parent AActivitySpace.  If there is no parent
	 * space, this will return null.
	 *
	 * @return String the space ID of the parent AActivitySpace.  null if there
	 *             is no parent space.
	 */
	public String GetParentSpaceID()
	{
		return m_strParentSpaceID;
	}

	/**
	 * Return the name of the parent AActivitySpace.  If there is no parent space,
	 * this will return null.
	 *
	 * @return String the name of the parent AActivitySpace.  null if there
	 *             is no parent space.
	 */
	public String GetParentSpaceName()
	{
		return m_strParentSpaceName;
	}

	/**
	 * Return the XPResourceManager
	 *
	 * AActivitySpace.GetString() and the static method
	 * XPResourceManager.FormatString() should be used
	 * instead.
	 * 
	 * @return the Resource manager
	 */
	public XPResourceManager GetResourceManager()
	{
		XPResourceManager rm = (XPResourceManager) m_apApplication.GetCachingManager()
			.GetEntry("ResourceManager");

		return rm;
	}

	/**
	 * Return the unique space ID for this AActivitySpace.
	 *
	 * @return String the unique space ID for this AActivitySpace
	 */
	public String GetSpaceID()
	{
		return m_strSpaceID;
	}

	/**
	 * This method gets localized strings for an
	 * internationalized string file based on the user's
	 * locale.
	 * 
	 * Available string files:
	 * @see com.plumtree.uiinfrastructure.constants.ResourceConstants
	 * Custom string files can also be added to the portal.
	 * 
	 * @param _nID The ID of the string to retrieve
	 * @param _strStringFile The name of the string file to use
	 * @return String The localized string
	 */
	public String GetString(int _nStringID, String _strResourceFileName)
	{
		String strLanguage;

		try
		{
			strLanguage = XPConvert.ToString(PersonalSettingsHelper.GetPersonalSettingValue
				(PersonalSettingsHelper.LANGUAGE, this));
		} catch (Exception e)
		{
			
				log.Error(e, GetName() + " GetString() unable to get language for user " +
					LoginHelper.INSTANCE.GetUserSessionName(m_UserSession) +
					" with ID " + LoginHelper.INSTANCE.GetUserSessionID(m_UserSession) +
					".");
			

			throw XPException.GetInstance(e);
		}

		String strResult;

		try
		{
			XPResourceManager rm = GetResourceManager();

			strResult = rm.GetString(_nStringID, _strResourceFileName, strLanguage);
		} catch (Exception e)
		{
			
				log.Error(e, GetName() + " GetString() unable to get string with ID " +
					_nStringID + " from resource file " + _strResourceFileName +
					" in language " + strLanguage + ".");
			

			throw XPException.GetInstance(e);
		}

		return strResult;
	}

	/**
	 * Figure out the style sheet; this method can be overriden by classes that
	 * require a special style sheet.
	 *
	 * @return An HTMLCollection that contains one or more PTStyleSheetLink objects
	 * (almost always just one)
	 */
	public HTMLElementCollection GetStyleSheetLinks()
	{
		return LoginHelper.INSTANCE.GetStyleSheetLinks(GetUserSession(), this);
	}

	/**
	 * Gets the time zone of the current user.
	 * 
	 * I.E. TODO
	 * 
	 * @return The time zone of the user.
	 */
	public String GetTimeZone()
	{
		return (String) PersonalSettingsHelper.GetPersonalSettingValue
			(PersonalSettingsHelper.TIMEZONE, this);
	}

	//////////////////////////
	// Internal helper methods
	//////////////////////////

	/**
	 * This method is called at the end of every HTTP request by the Interpreter.
	 * This method then calls IModel.CleanupTempData() on all IModels registered
	 * using RegisterModel().
	 *
	 * This method is intended for cleaning up temporary data that is only used
	 * for the lifetime of a single request, like DB query results.
	 */
	public void CleanupTempData()
	{
		IXPEnumerator myIter = m_mModelMap.GetElements();

		while (myIter.MoveNext() != false)
		{
			((IModel) myIter.GetCurrent()).CleanupTempData();
		}

		m_strCurrentRequestUrl = null;
		m_strCurrentNormalizedBaseUrl = null;
		m_xpRequest = null;
		m_xpResponse = null;
		m_htFormData = null;

		// Since we now cache the access style for performance reasons,
		// we need to clear this every request so that any updates to
		// the access style will be visible.
		m_Style = null;
	}

	/**
	 * This method is called when an ActivitySpace is removed from the cache.
	 * This method then calls IModel.CleanupAllData() on all IModels registered
	 * using RegisterModel().
	 *
	 * This method is intended for releasing data stored in the server when
	 * the activity space is no longer needed, such as search objects that
	 * need to be maintained as long as the user is clicking through the
	 * search, but should be freed when the space can no longer be used.
	 * 
	 * This can be used to prevent Java references to COM memory from hanging
	 * around in the long lived Java heap without cleaning up the underlying
	 * COM memory.  The Java reference will still stay in the long lived heap
	 * until it gets garbage collected (which could be a while), but the COM
	 * memory should get released here.
	 * 
	 * This method will not be called if an HTTPSession times out.
	 * 
	 * CleanupTempData will be called before CleanupAllData.
	 */
	public void CleanupAllData()
	{
		IXPEnumerator myIter = m_mModelMap.GetElements();

		while (myIter.MoveNext() != false)
		{
			((IModel) myIter.GetCurrent()).CleanupAllData();
		}
	}

	/**
	 * Returns the subsession that the AActivitySpace can use to store cross-space data.
	 *
	 * This also provides access to the ASCache object (ASConstants.SANDBOXASCACHE).
	 *
	 * This SubSession is cleared of all data on logout.
	 * 
	 * @return ISessionManager the subsession that the AActivitySpace can use to store
	 *             cross-space data.
	 */
	public ISessionManager GetSubSession()
	{
		return m_smSession;
	}

	/**
	 * Returns the persistent subsession that the AActivitySpace can use to store cross-space data
	 * for use across multiple login/logout cycles.
	 *
	 * This SubSession is not cleared of its data on logout, so be sure not to cache
	 * anything on this session that could be considered a security risk if it was
	 * leaked to another user.
	 * 
	 * Most developers should use the normal subsession (GetSubSession()) for cross-space
	 * data storage.  This is currently only used when editing the default profiles.
	 * 
	 * @return ISessionManager the persistent subsession that the AActivitySpace can use to store
	 *             cross-space data across multiple login/logout cycles.
	 */
	public ISessionManager GetPersistentSubSession()
	{
		return m_smPersistentSession;
	}

	/**
	 * Return the requested Sub-space.
	 *
	 * The sub-space key is different from the space ID.
	 *
	 * @param String the sub-space key of the Sub-space to get
	 *
	 * @return AActivitySpace the requested Sub-space
	 */
	public AActivitySpace GetSubSpace(String subSpaceKey)
	{
		AActivitySpace result = (AActivitySpace) m_mSubSpaceMap.GetElement(subSpaceKey);

		if (result == null)
		{
			
				log.Error(GetName() +
					" GetSubSpace failed: Could not find sub-space " +
					subSpaceKey);
			
		}

		return result;
	}

	/**
	 * This method checks whether or not a specific sub-space exists in this AActivitySpace.
	 *
	 * @param String the name of the sub-space to check
	 *
	 * @return boolean whether or not the sub-space exists
	 */
	public boolean GetSubSpaceExists(String subSpaceKey)
	{
		return m_mSubSpaceMap.ContainsKey(subSpaceKey);
	}

	/**
	 * Returns the sub-space key.  If this is null, this is not a sub-space.
	 *
	 * This key is used by the parent AActivitySpace to identify sub-spaces
	 * (in case there are multiple instances of the same type of sub-space).
	 *
	 * The sub-space key is different from the space ID.
	 *
	 * @return String the sub-space key
	 */
	public String GetSubSpaceKey()
	{
		return m_sSubSpaceKey;
	}

	/**
	 * Return an Enumerator for the registered sub-spaces.
	 *
	 * @return IXPEnumerator the registered sub-spaces
	 */
	public IXPEnumerator GetSubSpacesEnumerator()
	{
		return m_mSubSpaceMap.GetElements();
	}

	/**
	 * Return the user's Session.  This is a generic Object that
	 * can be used for different kinds of sessions.  This will
	 * have to be cast to the correct type of session (IPTSession
	 * for the portal).
	 *
	 * @return Object the user's session
	 */
	public Object GetUserSession()
	{
		return m_UserSession;
	}

	/**
	 * Helper method to access Variable Packages.
	 *
	 * @return IVarPack the requested VarPack
	 */
	public IVarPack GetVarPack(String strVarPack)
	{
		return m_apApplication.GetVarPackManager().GetVariablePackage(strVarPack);
	}

	/**
	 * Return the requested View.
	 *
	 * @param String the name of the view to get
	 *
	 * @return IView the requested view
	 */
	public IView GetView(String viewName)
	{
		IView result = null;

		if (GetAccessStyle() != AccessStyles.STANDARD)
		{
			// Try to get the 508 version of this view, if it exists.
			result = (IView) m_mViewMap.GetElement(viewName +
					ASConstants.STR_508_VIEW_SUFFIX);
		}

		if (result == null)
		{
			result = (IView) m_mViewMap.GetElement(viewName);
		}

		if (result == null)
		{
			
				log.Error(GetName() + " GetView failed: Could not find view " +
					viewName);
			
		}
		
		// SE: Bug 62787 - Add diagnostics for tracking MVC registration and displaying
		log.Debug(GetName() + " GetView succeeded: Found " + viewName);

		return result;
	}

	/**
	 * This method checks whether or not a specific view exists in this AActivitySpace.
	 *
	 * @param String the name of the view to check
	 *
	 * @return boolean whether or not the view exists
	 */
	public boolean GetViewExists(String viewName)
	{
		return m_mViewMap.ContainsKey(viewName);
	}

	/**
	 * Return an Enumerator for the registered views.
	 *
	 * @return IXPEnumerator the registered views
	 */
	public IXPEnumerator GetViewsEnumerator()
	{
		return m_mViewMap.GetElements();
	}

	/////////////////////////////////////////////////////////////////
	// Base methods that should be overridden and called as super.*()
	/////////////////////////////////////////////////////////////////

	/**
	 * Initialize the Activity Space.
	 *
	 * This method is called after common attributes like ID, parent, session,
	 * application, etc... are set.
	 *
	 * Get methods are not valid until this method is called.
	 *
	 * This method should be overridden and called as super.Init();
	 */
	public void Init()
	{
		// Create all the maps
		m_mModelMap		   = new XPHashtable();
		m_mViewMap		   = new XPHashtable();
		m_mControlMap	   = new XPHashtable();
		m_mPageMap		   = new XPHashtable();
		m_mSubSpaceMap     = new XPHashtable();

		// Check initialization prerequisites
		if (m_smSession == null)
		{
			
				log.Error(GetName() + " Init called before sub-session set. Sub-spaces will not be registered.");
			

			return;
		}

		if (m_apApplication == null)
		{
			
				log.Error(GetName() + " Init called before application set. Sub-spaces will not be registered.");
			

			return;
		}

		if ((m_strSpaceID == null) || ("".equals(m_strSpaceID)))
		{
			
				log.Error(GetName() + " Init called and Space ID is invalid: " +
					m_strSpaceID + ". Sub-spaces will not be registered.");
			

			return;
		}

		if ((m_sSubSpaceKey != null) && (!"".equals(m_sSubSpaceKey)))
		{
			ASCache myCache		  = (ASCache) m_smSession.GetAttribute(ASConstants.SANDBOXASCACHE);
			AActivitySpace parent = myCache.GetSpace(m_strParentSpaceName,
					m_strParentSpaceID);

			if (parent == null)
			{
				
					log.Error(GetName() +
						" No parent space found when attempting to initialize sub-space " +
						GetName() + ". Sub-spaces will not be registered.");
				
			}
			else
			{
				parent.RegisterSubSpace(this, m_sSubSpaceKey);
			}
		}
	}
	
	/**
	 * Re-Initialize the Activity Space in case models are holding on to an old session.
	 * 
	 */
	public void ReInit() {
		
	}

	///////////////////////////////////////
	// Common implementations of basic code
	///////////////////////////////////////
	///////////////////////////////
	// General Get Accessor Methods
	///////////////////////////////

	/**
	 * Returns whether or not this space is a sub-space.
	 *
	 * @return boolean true implies this is a sub-space
	 */
	public boolean IsSubSpace()
	{
		return m_sSubSpaceKey != null;
	}

	/**
	 * This method registers the control so the AActivitySpace can manage it
	 * automatically.
	 *
	 * It also initializes it with the IModel argument.
	 *
	 * This operation will fail and return false if the control is null or if
	 * there is already a control with the same name registered with the
	 * AActivitySpace.
	 *
	 * @param String controlName the name of the IControl to register
	 * @param IModel the model to initialize this IControl with
	 *
	 * @return boolean true implies success
	 *                     false implies failure
	 */
	public boolean RegisterControl(String controlName, IModel model)
	{
		return RegisterControl(controlName, controlName, model);
	}

	/**
	 * This method registers the control so the AActivitySpace can manage it
	 * automatically.  This method allows the client to store the control under
	 * a name other than the one returned by IControl.GetName().  This allows
	 * the client to use more than one control of the same type in a single
	 * AActivitySpace.
	 *
	 * It also initializes it with the IModel argument.
	 *
	 * This operation will fail and return false if the control is null or if
	 * there is already a control with the same name registered with the
	 * AActivitySpace.
	 *
	 * If you register a control with a storage name of "" (the empty string),
	 * this will be used as the default control in your space.  This means that
	 * anytime an URL does not have a control specified in it, this default
	 * control will be used.
	 *
	 * @param String controlName the name of the IControl to register
	 * @param String storageName the name to store this IControl under
	 * @param IModel the model to initialize this IControl with
	 *
	 * @return boolean true implies success
	 *                     false implies failure
	 */
	public boolean RegisterControl(String controlName, String storageName,
		IModel model)
	{
		IControl myControl = ASManager.INSTANCE.GetControl(controlName);

		if (myControl == null)
		{
			
				log.Error(GetName() +
					" RegisterControl failed: Could not get control " +
					controlName);
			

			return false;
		}

		// Put the control into the map
		Object mOldControl = m_mControlMap.PutElement(storageName, myControl);

		// If there was already a control there, put it back and fail
		if (mOldControl != null)
		{
			
				log.Error(GetName() +
					" RegisterControl failed: Control already registered " +
					storageName);
			

			m_mControlMap.PutElement(storageName, mOldControl);

			return false;
		}

		// Initialize the Control
		myControl.Init(model, this);

		// SE: Bug 62787 - Add diagnostics for tracking MVC registration and displaying
		log.Debug(GetName() +
				" RegisterControl succeeded: " + controlName + " registered");
		
		return true;
	}

	/**
	 * This method registers and initializes the model so the AActivitySpace can
	 * manage it automatically.
	 *
	 * This operation will fail and return false if the model is null or if
	 * there is already a model with the same name registered with the
	 * AActivitySpace.
	 *
	 * @param String modelName the name of the IModel to register
	 *
	 * @return boolean true implies success
	 *                     false implies failure
	 */
	public boolean RegisterModel(String modelName)
	{
		return RegisterModel(modelName, modelName);
	}

	/**
	 * This method registers and initializes the model so the AActivitySpace can
	 * manage it automatically.  This method allows the user to use a name other
	 * than the model name returned by IModel.GetName(), which allows the user to
	 * have multiple models of the same type in one AActivitySpace.
	 *
	 * This operation will fail and return false if the model is null or if
	 * there is already a model with the same name registered with the
	 * AActivitySpace.
	 *
	 * @param String modelName the name of the IModel to register
	 * @param String storageName the name to store the IModel under
	 *
	 * @return boolean true implies success
	 *                     false implies failure
	 */
	public boolean RegisterModel(String modelName, String storageName)
	{
		IModel myModel = ASManager.INSTANCE.GetModel(modelName);

		if (myModel == null)
		{
			
				log.Error(GetName() + " RegisterModel failed: Could not get model  " +
					modelName);
			

			return false;
		}

		// Put the model into the map
		Object mOldModel = m_mModelMap.PutElement(storageName, myModel);

		// If there was already a model there, put it back and fail
		if (mOldModel != null)
		{
			
				log.Error(GetName() +
					" RegisterModel failed: model already registered " +
					storageName);
			

			m_mModelMap.PutElement(storageName, mOldModel);

			return false;
		}

		// Initialize the Model
		myModel.Init(this);
		
		// SE: Bug 62787 - Add diagnostics for tracking MVC registration and displaying
		log.Debug(GetName() +
				" RegisterModel succeeded: " + modelName + " registered");

		return true;
	}

	/**
	 * Registers the Sub-space.
	 *
	 * The sub-space key is different from the space ID.
	 *
	 * @param String the key of the Sub-space to register
	 * @param AActivitySpace the sub-space to register
	 *
	 * @return true if registration succesful
	 */
	public boolean RegisterSubSpace(AActivitySpace subSpace, String storageName)
	{
		if (subSpace == null)
		{
			
				log.Error(GetName() +
					" RegisterSubSpace failed: Null subSpace with key " +
					storageName);
			

			return false;
		}

		// Put the subspace into the map
		AActivitySpace mOldSubSpace = (AActivitySpace) m_mSubSpaceMap.PutElement(storageName,
				subSpace);

		// If there was already a subspace there,
		if (mOldSubSpace != null)
		{
			if (!m_bOverWriteSubSpaces)
			{
				// put it back and fail
				
					log.Error(GetName() +
						" RegisterSubSpace failed: Subspace already registered " +
						storageName);
				

				m_mSubSpaceMap.PutElement(storageName, mOldSubSpace);

				return false;
			}
			else
			{
				// or delete it
				ASCache myCache = (ASCache) m_smSession.GetAttribute(ASConstants.SANDBOXASCACHE);
				myCache.RemoveSubSpace(mOldSubSpace.GetName(),
					mOldSubSpace.GetSpaceID());

				
					log.Info(GetName() +
						"RegisterSubSpace: Identical storage name - deleting old subspace with name " +
						mOldSubSpace.GetName() + " and ID " + mOldSubSpace.GetSpaceID());
				
			}
		}

		if (m_mSubSpaceMap.GetSize() >= m_nSUBSPACE_WARNING_MAX)
		{
			
				log.Warn(GetName() +
					" has an excessively large number of sub-spaces: " +
					m_mSubSpaceMap.GetSize());
			
		}

		return true;
	}

	/**
	 * This method registers the view so the AActivitySpace can manage it
	 * automatically.
	 *
	 * It also initializes it with the IModel and AActivitySpace arguments.
	 *
	 * This operation will fail and return false if the view is null or if
	 * there is already a view with the same name registered with the
	 * AActivitySpace.
	 *
	 * @param String viewName the name of the IView to register
	 * @param IModel the model to initialize this IView with
	 *
	 * @return boolean true implies success
	 *                     false implies failure
	 */
	public boolean RegisterView(String viewName, IModelRO model)
	{
		return RegisterView(viewName, viewName, model);
	}

	/**
	 * This method registers the view so the AActivitySpace can manage it
	 * automatically.  This method allows the user to store the view with a name
	 * other than the one returned by IView.GetName().  This allows the user to
	 * use multiple views of the same type in one AActivitySpace.
	 *
	 * It also initializes it with the IModel and AActivitySpace arguments.
	 *
	 * This operation will fail and return false if the view is null or if
	 * there is already a view with the same name registered with the
	 * AActivitySpace.
	 *
	 * @param String viewName the name of the IView to register
	 * @param String storageName the name to store this IView under
	 * @param IModel the model to initialize this IView with
	 *
	 * @return boolean true implies success
	 *                     false implies failure
	 */
	public boolean RegisterView(String viewName, String storageName,
		IModelRO model)
	{
		IView myView = ASManager.INSTANCE.GetView(viewName);

		if (myView == null)
		{
			
				log.Error(GetName() + " RegisterView failed: Could not get view " +
					viewName);
			

			return false;
		}

		// Put the view into the map
		Object mOldView = m_mViewMap.PutElement(storageName, myView);

		// If there was already a view there, put it back and fail
		if (mOldView != null)
		{
			
				log.Error(GetName() +
					" RegisterView failed: View already registered " +
					storageName);
			

			m_mViewMap.PutElement(storageName, mOldView);

			return false;
		}

		// Initialize the View
		myView.Init(model, this);

		// SE: Bug 62787 - Add diagnostics for tracking MVC registration and displaying
		log.Debug(GetName() +
				" RegisterView succeeded: " + viewName + " registered");
		
		return true;
	}

	/**
	 * Sets the Application.
	 *
	 * This method must be called before Init();
	 *
	 * @param the application
	 */
	public void SetApplication(IApplication application)
	{
		m_apApplication = application;
	}

	/**
	 * Sets the current Request URL and computes the normalized
	 * base url.
	 *
	 * This method must be called at the beginning of every request.
	 *
	 * @param the current request url
	 */
	public void SetCurrentRequestURL(String strCurrentRequestURL)
	{
		m_strCurrentRequestUrl = strCurrentRequestURL;
		m_strCurrentNormalizedBaseUrl = ConfigHelper.GetNormalizedBaseURL(strCurrentRequestURL);
	}
	
	/**
	 * Sets the HTTP Request and Response Objects for the current request.
	 *
	 * This method must be called at the beginning of every request.
	 *
	 * @param The current XPRequest
	 * @param The current XPResponse
	 */
	public void SetCurrentHTTPRequestResponseObjects(IXPRequest xpRequest, IXPResponse xpResponse)
	{
		m_xpRequest = xpRequest;
		m_xpResponse = xpResponse;
	}
	
	/**
	 * JF- Sets the hastable with the name/value pairs from XPRequest.
	 * (This for PEI use only.)
	 *
	 * This method must be called at the beginning of every request.
	 *
	 * @param _htFormData
	 */
	public void SetCurrentFormData(XPHashtable _htFormData)
	{
		m_htFormData = _htFormData;
	}

	/**
	 * Set the current page for this AActivitySpace.  If this AActivitySpace only
	 * has one IDisplayPage, the current page will automatically be set to that
	 * IDisplayPage.
	 *
	 * If current page tracking is on (off by default), the Interpeter will set the
	 * current page to be the most recently accessed page.
	 *
	 * @param IDisplayPage the current Display Page
	 */
	public void SetCurrentPage(IDisplayPage page)
	{
		if (m_bTrackCurrentPage && (page != null))
		{
			m_asdpCurrentPage = page;
		}
	}

	/**
	 * Should be set to the request secure flag (IsSecureConnection).
	 *
	 * This method must be called before Init();
	 *
	 * @param _bIsSecured
	 */
	public void SetIsASSecured(boolean _bIsSecured)
	{
		m_bIsSecuredSpace = _bIsSecured;
	}

	/**
	 * This method turns on and off overwriting of sub-spaces when they register
	 * with this AActivitySpace.
	 *
	 * If bOverWrite is false, the space will give an error and ignore requests
	 * to overwrite a sub-space.  This is the default state.
	 *
	 * @param boolean true implies allow overwriting of sub-space
	 *                    false implies ignore attempts to overwrite a sub-space (default)
	 */
	public void SetOverWriteSubSpaces(boolean bOverWrite)
	{
		m_bOverWriteSubSpaces = bOverWrite;
	}

	/**
	 * Set the name and ID of the parent AActivitySpace (i.e. the one that launched
	 * this space).  If there is no parent space, then this method does not need to
	 * be called.
	 *
	 * @param String spaceName the name of the parent space
	 * @param String spaceID the ID of the parent space
	 */
	public void SetParentSpace(String spaceName, String spaceID)
	{
		// Bug 66243 - Check against ASManager and set the space name if it exists. 
		// The ASManager holds all the ActivitySpaces of the portal. This is basically
		// any class that extends AActivitySpace and this is initialized when the portal starts up.
		if (null != spaceName && ASManager.INSTANCE.GetSpaceExists(spaceName)) 
		{
			m_strParentSpaceName     = spaceName;
		}
		
		// Case if space ID is "None" which is considered a valid value.
		if (ASCache.SPACEID_INVALID.equals(spaceID)) 
		{
			m_strParentSpaceID		 = spaceID;
		} 
		else 
		{
			// DCA - check to make sure parent space exist before setting it
			// Get the Activity Space cache from the user's session and check for the parent space.
			ASCache myCache = (ASCache) m_smSession.GetAttribute(ASConstants.SANDBOXASCACHE);
			if (null != myCache && null != spaceID) 
			{
				// Set the space ID if we can get the parent space from the cache. This should be
				// the usual case when a valid space ID is passed in.
				AActivitySpace parentSpace = myCache.GetSpace(spaceName, spaceID);
				if (null != parentSpace) 
				{
					m_strParentSpaceID = spaceID;
				} 
				else 
				{
					// If parent space is not valid or not in the cache, we shouldn't set the space ID.
					// Instead just set it to "None". This is to guard against junk getting passed in.
					m_strParentSpaceID = ASCache.SPACEID_INVALID;
				}
			}
		}
	}

	/**
	 * Sets the unique space ID for this AActivitySpace.
	 *
	 * This method must be called before Init();
	 *
	 * @param the unique space ID for this AActivitySpace
	 */
	public void SetSpaceID(String spaceID)
	{
		m_strSpaceID = spaceID;
	}

	/**
	 * Sets the subsession that the AActivitySpace can use to store cross-space data.
	 *
	 * This SubSession will be cleared on logout.
	 * 
	 * This method must be called before Init();
	 *
	 * @param the subsession that the AActivitySpace can use to store
	 *            cross-space data.
	 */
	public void SetSubSession(ISessionManager session)
	{
		m_smSession = session;
	}

	/**
	 * Sets the persistent subsession that the AActivitySpace can use to store
	 * cross-space data across multiple login/logout cycles.
	 * 
	 * This SubSession will not be cleared on logout.
	 *
	 * This method must be called before Init();
	 *
	 * @param the subsession that the AActivitySpace can use to store
	 *            cross-space data.
	 */
	public void SetPersistentSubSession(ISessionManager session)
	{
		m_smPersistentSession = session;
	}

	/////////////////////////////////////////////////////////////////////////////////
	// General Set Accessor Methods
	//
	// These methods will be used by the Interpreter to initialize the AActivitySpace
	// and will be called before Init()
	/////////////////////////////////////////////////////////////////////////////////

	/**
	 * Sets the sub-space key.  If this is null, this is
	 * not a sub-space.  By default, spaces are not sub-spaces.
	 *
	 * This key is used by the parent AActivitySpace to identify sub-spaces
	 * (in case there are multiple instances of the same type of sub-space).
	 *
	 * The sub-space key is different from the space ID.
	 *
	 * @param String sub-space key
	 */
	public void SetSubSpaceKey(String sSubSpaceKey)
	{
		m_sSubSpaceKey = sSubSpaceKey;

		if ("".equals(m_sSubSpaceKey))
		{
			m_sSubSpaceKey = null;
		}
	}

	/**
	 * This method turns on and off automatic tracking of the current IDisplayPage.
	 * The Interpreter calls SetCurrentPage at the end of every request with the
	 * most recently viewed IDisplayPage.
	 *
	 * If bTrack is false, MainAS will ignore this information.  This is the default
	 * state.
	 *
	 * @param boolean true implies track current page information
	 *                    false implies ignore page tracking information (default)
	 */
	public void SetTrackCurrentPage(boolean bTrack)
	{
		m_bTrackCurrentPage = bTrack;
	}

	/**
	 * Set the user's Session.  This is a generic object that can be used
	 * for different kinds of user sessions.  For the portal it is an
	 * IPTSession.
	 *
	 * This will be called before Init() and CheckBasicAccess() are called.
	 *
	 * @param Object userSession to be set
	 */
	public void SetUserSession(Object userSession)
	{
		m_UserSession = userSession;
	}

	/////////////////////////////////////////////
	// Register / Get methods for managed objects
	/////////////////////////////////////////////


	/**
	 * Un-registers the requested view (removes it from AActivitySpace)
	 *
	 * @param String the name of the view to un register
	 */
	public void UnRegisterPage(String pageName)
	{
		m_mPageMap.RemoveElement(pageName);

		// Set the current page if it's not set so AActivitySpaces with only one
		// IDisplayPage don't need to worry about this.
		if (m_asdpCurrentPage.GetName().equals(pageName))
		{
			IXPEnumerator temp = m_mPageMap.GetElements();
			temp.MoveNext();
			m_asdpCurrentPage = (IDisplayPage) temp.GetCurrent();
		}
	}

	/**
	 * Un-registers the requested view (removes it from AActivitySpace)
	 *
	 * @param String the name of the view to un register
	 */
	public void UnRegisterView(String viewName)
	{
		m_mViewMap.RemoveElement(viewName);
	}

	/**
	 * Un-registers the requested control (removes it from AActivitySpace)
	 *
	 * @param String the name of the control to unregister
	 */
	public void UnRegisterControl(String controlName)
	{
		m_mControlMap.RemoveElement(controlName);
	}

	/**
	 * Un-registers the requested model (removes it from AActivitySpace)
	 *
	 * @param String the name of the model to unregister
	 */
	public void UnRegisterModel(String modelName)
	{
		m_mModelMap.RemoveElement(modelName);
	}

	/**
	 * This method registers the IDisplayPage so the AActivitySpace can manage it
	 * automatically.
	 *
	 * It also initializes it with the AActivitySpace argument.
	 *
	 * This operation will fail and return false if the IDisplayPage is null or if
	 * there is already a IDisplayPage with the same name registered with the
	 * AActivitySpace.
	 *
	 * The first DisplayPage will be set as the current IDisplayPage.
	 *
	 * @param IDisplayPage page to register
	 *
	 * @return boolean true implies success
	 *                     false implies failure
	 */
	public boolean RegisterPage(IDisplayPage page)
	{
		if (page == null)
		{
			return false;
		}

		return RegisterPage(page, page.GetName());
	}

	/**
	 * This method registers the IDisplayPage so the AActivitySpace can manage it
	 * automatically.
	 *
	 * It also initializes it with the AActivitySpace argument.
	 *
	 * This operation will fail and return false if the IDisplayPage is null or if
	 * there is already a IDisplayPage with the same name registered with the
	 * AActivitySpace.
	 *
	 * The first DisplayPage will be set as the current IDisplayPage.
	 *
	 * @param IDisplayPage page to register
	 * @param String storageName the name to store the IDisplayPage under
	 *
	 * @return boolean true implies success
	 *                     false implies failure
	 */
	public boolean RegisterPage(IDisplayPage page, String strStorageName)
	{
		if (page == null)
		{
			return false;
		}

		// Put the page into the map
		Object mOldPage = m_mPageMap.PutElement(strStorageName, page);

		// If there was already a page there, put it back and fail
		if (mOldPage != null)
		{
			
				log.Error(GetName() +
					" RegisterPage failed: Page already registered " +
					strStorageName);
			

			m_mPageMap.PutElement(strStorageName, mOldPage);

			return false;
		}

		// Initialize the page
		page.Init(this);

		// Set the current page if it's not set so AActivitySpaces with only one
		// IDisplayPage don't need to worry about this.
		if (m_asdpCurrentPage == null)
		{
			m_asdpCurrentPage = page;
		}

		// SE: Bug 62787 - Add diagnostics for tracking MVC registration and displaying
		log.Debug(GetName() +
				" RegisterPage succeeded: " + page.GetName() + " registered");
		
		return true;
	}

	/////////////////////
	// Referenced Strings
	/////////////////////

    /**
     * This creates a reference to a long query string parameter.  Currently
     * only the tree knows how to unreference this for certain parameters.
     *
     * @param value
     *
     * @return returns the key that is used to unref the value - This needs
     * to be preceded with "@" for this to be used.
     */
    public String refString(String value)
    {
        if (value == null)
        {
            return null;
        }

        if (m_stringReferences == null)
        {
            m_stringReferences = new XPHashtable();
        }

        String key = XPConvert.ToString(value.hashCode());
        m_stringReferences.PutElement(key, value);

        return key;
    }

    /**
     * This unreferences a key to a long query string parameter
     *
     * @param key -- This must not have the "@" in front of it
     *
     * @return returns value or null if there was no value
     */
    public String unrefString(String key)
    {
        if ((key == null) || (m_stringReferences == null))
        {
            return null;
        }

        String result = (String) m_stringReferences.GetElement(key);

		// Currently, we don't remove this.  This is a minor temporary
		// memory leak, or at least overuse of memory.  This could be
		// improved.

        // (let it die with the AS) m_stringReferences.RemoveElement(key);
        return result;
    }
    
	/**
	 * This method is used to return whether this Activity Space
	 * has in place refresh enabled.  If it is enabled, then we 
	 * can, for example, include the JavaScript for JSXML.
	 *
	 * This method should be used carefully in shared components because 
	 * if shared components use this method, then any time you add any 
	 * in-place to an AS, you will have to implement it for all the 
	 * shared components. Shared components that require you to implement 
	 * a partial content display method should use another way to determine 
	 * if they should be enabled or not.
	 * 
	 * @return boolean True if this AS has enabled in place refresh.
	 */
	public boolean GetIsInPlaceRefreshEnabled()
	{
		return m_bInPlaceRefreshEnabled;
	}

	/**
	 * This method is used to determine whether this Activity Space
	 * has in place refresh enabled.  If it is enabled, then we 
	 * can, for example, include the JavaScript for JSXML.
	 * 
	 * This method should be used carefully in shared components because 
	 * if shared components use this method, then any time you add any 
	 * in-place to an AS, you will have to implement it for all the 
	 * shared components. Shared components that require you to implement 
	 * a partial content display method should use another way to determine 
	 * if they should be enabled or not.
	 * 
	 * @param boolean True to enabled this AS for in place refresh.
	 */
	public void SetIsInPlaceRefreshEnabled(boolean IsInPlaceRefreshEnabled)
	{
		m_bInPlaceRefreshEnabled = IsInPlaceRefreshEnabled;		
	}

	/**
	 * This method is used to return whether Display Pages
	 * should currently be displaying in place content instead 
	 * of their normal content.  
	 * 
	 * @return boolean True to show in place content.
	 */
	public boolean GetIsInPlaceRefreshDisplayOn()
	{
		return m_bInPlaceRefreshDisplayOn;
	}

	/**
	 * This method is used to determin whether Display Pages
	 * should currently be displaying in place content instead 
	 * of their normal content.  
	 * 
	 * @param boolean True to show in place content.
	 */
	public void SetIsInPlaceRefreshDisplayOn(boolean IsInPlaceRefreshDisplayOn)
	{
		m_bInPlaceRefreshDisplayOn = IsInPlaceRefreshDisplayOn;
	}
}

