package com.plumtree.uiinfrastructure.activityspace;

import com.plumtree.openlog.*;
import com.plumtree.openfoundation.util.*;

/**
 * This model is designed to have all of it's data stored in a temporary
 * data storage object.  This object will be automatically deleted at the end
 * of every HTTP Request and regenerated at the beginning of the next request.
 *
 * This is for browsing pages that have low-memory requirements.  This means
 * that data will only be stored during requests.  Admin and Editor pages should
 * probably not use this model scheme.
 *
 * All data should go on the temporary storage object, especially Query Results
 * and large arrays.
 *
 * The design of the temporary data object is left to the implementor's discretion.
 * 
 * @author Don Hayler
 */
public abstract class AClearModel implements IModel
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.activityspace.AClearModel");

	/** the parent AActivitySpace */
	public AActivitySpace m_asOwner = null;

	/** Temporary data object */
	private Object m_TempData = null;
	
	/** boolean to tell whether the temp data object is in the process of initialization */
	private boolean m_bBeingInitialized = false;

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#CleanupTempData()
	 */
	public void CleanupTempData()
	{
		m_TempData = null;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#CleanupAllData()
	 */
	public void CleanupAllData()
	{
		// This isn't really necessary, because CleanupTempData() gets called
		// right before this method.  Can't hurt, though.
		m_TempData = null;
	}

	/**
	 * This method is used by the ASManager to return new instances of managed
	 * objects.
	 *
	 * @return A new instance of the managed class (i.e. return new Foo();)
	 */
	abstract public Object Create();

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	abstract public String GetName();

	///////////////////
	// Abstract methods
	///////////////////

	/**
	 * This method returns a new temporary data object.  It is called at the beginning
	 * of a new HTTP Request.  This method can perform initialization on the data
	 * object if necessary.
	 *
	 * @return Object the new temporary data object
	 */
	abstract public Object GetNewTempData();

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#Init(AActivitySpace)
	 */
	public void Init(AActivitySpace parent)
	{
		m_asOwner = parent;
	}

	/**
	 * This method is used to access the temporary data object.  This ensures that
	 * a valid temporary data object can always be obtained.
	 *
	 * @return Object the current temporary data object
	 */
	protected Object GetTempData()
	{
		if (m_TempData == null)
		{
			if(m_bBeingInitialized) 
			{
				
					log.Error("Developer: Circular calls to GetTempData while creating tempdata Object.  Please check your code" +
						"because it could lead to infinite loops.  May also be illegal multi-threaded use of this model.");
				
				
				throw new XPException("Developer: Circular calls to GetTempData while creating tempdata Object.  Please check your code" +
										"because it could lead to infinite loops.  May also be illegal multi-threaded use of this model.");
			}

			// This is not threadsafe.  If multiple threads get into the same model at
			// the same time, this can easily produce a false positive.  This happened
			// with gateway login when it was retrieving the login space from the
			// cache rather than creating a new space for each thread.  Since this has
			// been fixed, this should not need to be threadsafe.
			m_bBeingInitialized = true;

			try
			{
				m_TempData = GetNewTempData();
			} catch (Exception e)
			{
				// We need to catch this, reset our state boolean, and then rethrow
				// the error otherwise this model will become unusable for subsequent
				// requests.
				m_bBeingInitialized = false;
				throw XPException.GetInstance(e);
			}

			m_bBeingInitialized = false;
		}

		return m_TempData;
	}
}

