package com.plumtree.uiinfrastructure.activityspace;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.xml.dom.*;
import com.plumtree.openlog.*;


/**
 * The ASManager is a singleton class that returns Activity Spaces,
 * Models, Views, & Controls. This uses dynamic discovery to import
 * all of the IActivitySpace, IModel, IView, and IControl objects
 * in certain locations, and then maintain a hashmap of clones,
 * with possible resource pooling.
 *
 * The maps are initialized at startup, and are then assumed to be
 * read only from that point on.  Therefore, they should not need
 * to be locked.
 *
 * @author Don Hayler
 */
public class ASManager
{
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
                                                             "com.plumtree.uiinfrastructure.activityspace.ASManager");

    // constants for interfaces to look for

    /** model interface */
    private static final String IMODEL = "com.plumtree.uiinfrastructure.activityspace.IModel";

    /** view interface */
    private static final String IVIEW = "com.plumtree.uiinfrastructure.activityspace.IView";

    /** control interface */
    private static final String ICONTROL = "com.plumtree.uiinfrastructure.activityspace.IControl";

    /** activity space interface */
    private static final String AACTIVITYSPACE = "com.plumtree.uiinfrastructure.activityspace.AActivitySpace";

    /** pagecontrol interface */
    private static final String IPAGECONTROL = "com.plumtree.uiinfrastructure.activityspace.IPageControl";

    /** name of the jar file that the above interfaces are in. */
    private static final String INFRASTRUCTURE = "uiinfrastructure";
    
	/** The exception that caused the system to fail startup. */
	public Exception m_StartupFailure = null;

    /**
     * Use this class static to access the one and only instance of
     * ActivitySpaceManager.
     */
    public static final ASManager INSTANCE = new ASManager();

    /** whether or not the system was able to start up correctly. */
    public boolean m_bSystemStartupSuccesful = false;

    // Internal arrays to store MVC objects in

    /** map of the controls */
    private XPHashtable m_xphControls = new XPHashtable();

    /** map of the models */
    private XPHashtable m_xphModels = new XPHashtable();

    /** map of the spaces */
    private XPHashtable m_xphSpaces = new XPHashtable();

    /** map of the views */
    private XPHashtable m_xphViews = new XPHashtable();

    /** map of the page controls */
    private XPHashtable m_xphPageControls = new XPHashtable();

    /** Whether or not this object has been initialized */
    private boolean m_bInitialized = false;

	/** Counters used in the diagnostic page */
	private int m_nNumCustomControls = 0;
	private int m_nNumCustomModels = 0;
	private int m_nNumCustomSpaces = 0;
	private int m_nNumCustomViews = 0;
	private int m_nNumCustomPageControls = 0;

    /**
     * Private constructor to ensure there is only one instance.
     */
    private ASManager()
    {
    }

    /**
     * Create the requested Control.
     *
     * This method should not be called by developers.  It is only for
     * AActivitySpace.
     *
     * @param controlName The name of the control to create
     *
     * @return The control, null on failure
     */
    public IControl GetControl(String controlName)
    {
        IControl myControl = null;

        try
        {
            try
            {
                Object temp = m_xphControls.GetElement(controlName);
                myControl = (IControl) temp;
            }
            catch (Exception ex)
            {
                throw XPException.GetInstance(ex);
            }
        }
        catch (XPClassCastException e)
        {
            log.Error(e,
                      "GetControl: XPClassCastException: The control " +
                      controlName + " is being miscast.");
        }
        catch (XPException e)
        {
            log.Error(e, "Couldn't retrieve control " + controlName + ".");
        }

        try
        {
            myControl = (IControl) myControl.Create();
        }
        catch (Exception e)
        {
            log.Error(e, "Couldn't create control " + controlName + ".");

            myControl = null;
        }

        return myControl;
    }

    /**
     * Checks to determine if specified control has been loaded by dynamic
     * discovery into the hashtable of controls.
     *
     * @param controlName Control name
     *
     * @return True if exists. Otherwise false.
     */
    public boolean GetControlExists(String controlName)
    {
        try
        {
            return m_xphControls.ContainsKey(controlName);
        }
        catch (Exception ex)
        {
            return false;
        }
    }

	/**
	 * Get the table containing the page controls
	 * @return XPHashtable
	 */
    public XPHashtable GetPageControlTable()
    {
        return m_xphPageControls;
    }

    /**
     * Create the Model specified by the model name.
     *
     * This method should not be called by developers.  It is only for
     * AActivitySpace.
     *
     * @param modelName The name of the model to be created
     *
     * @return the requested model, null on failure
     */
    public IModel GetModel(String modelName)
    {
        IModel myModel = null;

        try
        {
            try
            {
                Object temp = m_xphModels.GetElement(modelName);
                myModel = (IModel) temp;
            }
            catch (Exception ex)
            {
                throw XPException.GetInstance(ex);
            }
        }
        catch (XPClassCastException e)
        {
            log.Error(e,
                      "GetModel: XPClassCastException: The model " + modelName +
                      " is being miscast.");
        }
        catch (XPException e)
        {
            log.Error(e, "Couldn't retrieve model " + modelName + ".");
        }

        try
        {
            myModel = (IModel) myModel.Create();
        }
        catch (Exception e)
        {
            log.Error(e, "Couldn't create model " + modelName + ".");

            myModel = null;
        }

        return myModel;
    }

    /**
     * Checks to determine if specified model has been loaded by dynamic
     * discovery into the hashtable of models.
     *
     * @param modelName Model name
     *
     * @return True if exists. Otherwise false.
     */
    public boolean GetModelExists(String modelName)
    {
        try
        {
            return m_xphModels.ContainsKey(modelName);
        }
        catch (Exception ex)
        {
            return false;
        }
    }

    /**
     * Create the requested Activity Space.
     *
     * This method should not be called by developers.  It is only for
     * the Interpreter.
     *
     * @param spaceName The name of the space to create
     *
     * @return The requested space, null on failure
     */
    public AActivitySpace GetSpace(String spaceName)
    {
        AActivitySpace mySpace = null;

        try
        {
            try
            {
                Object temp = m_xphSpaces.GetElement(spaceName);
                mySpace = (AActivitySpace) temp;
            }
            catch (Exception ex)
            {
                throw XPException.GetInstance(ex);
            }
        }
        catch (XPClassCastException ex)
        {
            log.Error(ex,
                      "GetSpace: XPClassCastException: The space " + spaceName +
                      " is being miscast.");
        }
        catch (XPException e)
        {
            log.Error(e, "Couldn't retrieve space " + spaceName + ".");
        }

        try
        {
            mySpace = (AActivitySpace) mySpace.Create();
        }
        catch (Exception e)
        {
            log.Error(e, "Couldn't create space " + spaceName + ".");

            mySpace = null;
        }

        return mySpace;
    }

    /**
     * Checks to determine if specified activity space has been loaded by dynamic
     * discovery into the hashtable of activity spaces.
     *
     * @param spaceName Activity space name
     *
     * @return True if exists. Otherwise false.
     */
    public boolean GetSpaceExists(String spaceName)
    {
        try
        {
            return m_xphSpaces.ContainsKey(spaceName);
        }
        catch (Exception ex)
        {
            return false;
        }
    }

    /**
     * Create the view specified by the view name.
     *
     * This method should not be called by developers.  It is only for
     * AActivitySpace.
     *
     * @param viewName The name of the view to create
     *
     * @return the requested view, null on failure
     */
    public IView GetView(String viewName)
    {
        IView myView = null;

        try
        {
            try
            {
                Object temp = m_xphViews.GetElement(viewName);
                myView = (IView) temp;
            }
            catch (Exception ex)
            {
                throw XPException.GetInstance(ex);
            }
        }
        catch (XPClassCastException e)
        {
            log.Error(e,
                      "GetView: XPClassCastException: The view " + viewName +
                      " is being miscast.");
        }
        catch (XPException e)
        {
            log.Error(e, "Couldn't retrieve view " + viewName + ".");
        }

        try
        {
            myView = (IView) myView.Create();
        }
        catch (Exception e)
        {
            log.Error(e, "Couldn't create view " + viewName + ".");

            myView = null;
        }

        return myView;
    }

    /**
     * Checks to determine if specified view has been loaded by dynamic
     * discovery into the hashtable of views.
     *
     * @param viewName View name
     *
     * @return True if exists. Otherwise false.
     */
    public boolean GetViewExists(String viewName)
    {
        try
        {
            return m_xphViews.ContainsKey(viewName);
        }
        catch (Exception ex)
        {
            return false;
        }
    }

    /**
     * Create the page control specified by the pagecontrol name.
     *
     * This method should not be called by developers.  It is only for
     * AActivitySpace.
     *
     * @param pagecontrolName The name of the pagecontrol to create
     *
     * @return the requested pagecontrol, null on failure
     */
    public IPageControl GetPageControl(String pageControlName)
    {
        IPageControl myPageControl = null;

        try
        {
            try
            {
                Object temp = m_xphPageControls.GetElement(pageControlName);
                myPageControl = (IPageControl) temp;
            }
            catch (Exception ex)
            {
                throw XPException.GetInstance(ex);
            }
        }
        catch (XPClassCastException e)
        {
            log.Error(e,
                      "GetPageControl: XPClassCastException: The pagecontrol " +
                      pageControlName + " is being miscast.");
        }
        catch (XPException e)
        {
            log.Error(e, "Couldn't retrieve view " + pageControlName + ".");
        }

        try
        {
        	if (myPageControl != null){
            	myPageControl = (IPageControl) myPageControl.Create();
        	}
        }
        catch (Exception e)
        {
            log.Error(e, "Couldn't create pagecontrol " + pageControlName +
                      ".");

            myPageControl = null;
        }

        return myPageControl;
    }

    /**
     * Checks to determine if specified page control has been loaded by dynamic
     * discovery into the hashtable of pagecontrols.
     *
     * @param pageControlName pagecontrol name
     *
     * @return True if exists. Otherwise false.
     */
    public boolean GetPageControlExists(String pageControlName)
    {
        try
        {
            return m_xphPageControls.ContainsKey(pageControlName);
        }
        catch (Exception ex)
        {
            return false;
        }
    }

    /**
     * This method initializes the ASManager and should only be called by the
     * warmup servlet.  This method can only be called once.
     *
     * @param strLibDir The file directory where jars and dlls can be found
     * @param strConfigFile the name of the config file
     */
    public void LoadClasses(String strLibDir, String strConfigFile)
    {
        if (m_bInitialized)
        {
            log.Error("ASManager LoadClasses called twice.");

            return;
        }

        m_bInitialized = true;

        log.Info("Loading from library: " + strLibDir);
        log.Info("Loading config file: " + strConfigFile);

        XPDocument document = XPBuilder.LoadDocument(strConfigFile);

        InternalLoadClasses(strLibDir, document, false);
    }

    /**
     * This method initializes the ASManager with custom lib files
     * to overwrite the original ones and should only be called by the
     * warmup servlet.
     *
     * @param strLibDir The file directory where jars and dlls can be found
     * @param strConfigFile the name of the custom config file
     */
    public void LoadCustomClasses(String strLibDir, String strConfigFile)
    {
        log.Info("Loading from library: " + strLibDir);
        log.Info("Loading config file: " + strConfigFile);

        XPDocument document = XPBuilder.LoadDocument(strConfigFile);

        if (document != null)
        {
			// Reset Custom Object Counters;
			m_nNumCustomControls 		= 0;
			m_nNumCustomModels 			= 0;
			m_nNumCustomPageControls 	= 0;
			m_nNumCustomSpaces 			= 0;
			m_nNumCustomViews 			= 0;

            InternalLoadClasses(strLibDir, document, true);
        }
        else
        {
            log.Info("No custom lib file xml file found at " + strLibDir);
        }
    }

    /**
     * This method loads the lib files from an XML document into the
     * ASManager.
     *
     * @param strLibDir The file directory where jars and dlls can be found
     * @param strConfigFile the name of the config file to load from
     * @param boolean bOverWrite false implies error if an MVC component overwrite is attempted
     * 	true implies that the Custom Object Counters will be incremented for these classes.
     */
    private void InternalLoadClasses(String strLibDir, XPDocument document,
                                     boolean bOverwrite)
    {
        // Currently we store names to full package names and re-reflect.
        // We should store objects and clone off them.
        int i;
        int length;
        Object oldObject;
        String strObjectName;

        XPNodeList libFiles = document.GetElementsByTagName("libfile");

        if (libFiles == null)
        {
            // No library files were found
            return;
        }

        for (int j = 0; j < libFiles.GetLength(); j++)
        {
            String strLibFileName = ((XPElement) libFiles.Item(j)).GetAttributeNode("name")
                                     .GetNodeValue();

            log.Info("Loading lib file: " + strLibFileName);

            Object[] models = null;
            Object[] views = null;
            Object[] controls = null;
            Object[] activityspaces = null;
            Object[] pagecontrols = null;

            try
            {
                models = XPDynamicDiscovery.GetInstancesFromLib(strLibDir,
                                                                INFRASTRUCTURE,
                                                                strLibFileName,
                                                                IMODEL);
                views = XPDynamicDiscovery.GetInstancesFromLib(strLibDir,
                                                               INFRASTRUCTURE,
                                                               strLibFileName,
                                                               IVIEW);
                controls = XPDynamicDiscovery.GetInstancesFromLib(strLibDir,
                                                                  INFRASTRUCTURE,
                                                                  strLibFileName,
                                                                  ICONTROL);
                activityspaces = XPDynamicDiscovery.GetInstancesFromLib(strLibDir,
                                                                        INFRASTRUCTURE,
                                                                        strLibFileName,
                                                                        AACTIVITYSPACE);
                pagecontrols = XPDynamicDiscovery.GetInstancesFromLib(strLibDir,
                                                                      INFRASTRUCTURE,
                                                                      strLibFileName,
                                                                      IPAGECONTROL);
            }
            catch (Throwable ex)
            {
                log.Fatal(ex,
                          "Fatal Exception loading lib file: " +
                          strLibFileName);
            }

            log.Debug("Putting infrastructure objects from lib files into hash tables");

            if (models != null)
            {
                // Load Models
                IModel myModel;
                length = models.length;

                for (i = 0; i < length; i++)
                {
                    myModel = (IModel) models[i];
                    oldObject = null;

                    if (myModel != null)
                    {
                        strObjectName = myModel.GetName();
                        if ((strObjectName != null) &&
                                (!"".equals(strObjectName)))
                        {
                            oldObject = m_xphModels.PutElement(myModel.GetName(),
                                                               myModel);

                            if (!bOverwrite && (oldObject != null))
                            {
                                log.Warn("Loaded model " + strObjectName +
                                         " twice.  This is usually caused two Models sharing the same MVC object name.");
                            }
                        }
                        else
                        {
                            // Model has an invalid MVC object name.
                            log.Error("Model " + myModel +
                                      " does not have a valid MVC object name.  Ignoring.");
                        }
                    }
                }
            }

            if (views != null)
            {
                // Load Views
                IView myView;
                length = views.length;

                for (i = 0; i < length; i++)
                {
                    myView = (IView) views[i];
                    oldObject = null;

                    if (myView != null)
                    {
                        strObjectName = myView.GetName();
                        if ((strObjectName != null) &&
                                (!"".equals(strObjectName)))
                        {
                            oldObject = m_xphViews.PutElement(myView.GetName(),
                                                              myView);

                            if (!bOverwrite && (oldObject != null))
                            {
                                log.Warn("Loaded view " + strObjectName +
                                         " twice.  This is usually caused two Views sharing the same MVC object name.");
                            }
                        }
                        else
                        {
                            // View has an invalid MVC object name.
                            log.Error("View " + myView +
                                      " does not have a valid MVC object name.  Ignoring.");
                        }
                    }
                }
            }

            if (controls != null)
            {
                // Load Controls
                IControl myControl;
                length = controls.length;

                for (i = 0; i < length; i++)
                {
                    myControl = (IControl) controls[i];
                    oldObject = null;

                    if (myControl != null)
                    {
                        strObjectName = myControl.GetName();
                        if ((strObjectName != null) &&
                                (!"".equals(strObjectName)))
                        {
                            oldObject = m_xphControls.PutElement(myControl.GetName(),
                                                                 myControl);

                            if (!bOverwrite && (oldObject != null))
                            {
                                log.Warn("Loaded control " + strObjectName +
                                         " twice.  This is usually caused two Controls sharing the same MVC object name.");
                            }
                        }
                        else
                        {
                            // Control has an invalid MVC object name.
                            log.Error("Control " + myControl +
                                      " does not have a valid MVC object name.  Ignoring.");
                        }
                    }
                }
            }

            if (pagecontrols != null)
            {
                // Load PageControls
                IPageControl myPageControl;
                length = pagecontrols.length;

                for (i = 0; i < length; i++)
                {
                    myPageControl = (IPageControl) pagecontrols[i];
                    oldObject = null;

                    if (myPageControl != null)
                    {
                        strObjectName = myPageControl.GetName();
                        if ((strObjectName != null) &&
                                (!"".equals(strObjectName)))
                        {
                            oldObject = m_xphPageControls.PutElement(myPageControl.GetName(),
                                                                     myPageControl);

                            if (!bOverwrite && (oldObject != null))
                            {
                                log.Warn("Loaded pagecontrol " + strObjectName +
                                         " twice.  This is usually caused two pagecontrols sharing the same MVC object name.");
                            }
                        }
                        else
                        {
                            // PageControl has an invalid MVC object name.
                            log.Error("PageControl " + myPageControl +
                                      " does not have a valid MVC object name.  Ignoring.");
                        }
                    }
                }
            }

            if (activityspaces != null)
            {
                // Load Spaces
                AActivitySpace mySpace;
                length = activityspaces.length;

                for (i = 0; i < length; i++)
                {
                    mySpace = (AActivitySpace) activityspaces[i];
                    oldObject = null;

                    if (mySpace != null)
                    {
                        strObjectName = mySpace.GetName();
                        if ((strObjectName != null) &&
                                (!"".equals(strObjectName)))
                        {
                            oldObject = m_xphSpaces.PutElement(mySpace.GetName(),
                                                               mySpace);

                            if (!bOverwrite && (oldObject != null))
                            {
                                log.Warn("Loaded space " + strObjectName +
                                         " twice.  This is usually caused two Spaces sharing the same MVC object name.");
                            }
                        }
                        else
                        {
                            // Space has an invalid MVC object name.
                            log.Error("Space " + mySpace +
                                      " does not have a valid MVC object name.  Ignoring.");
                        }
                    }
                }
            }

			// Update Custom Object Counters;
			m_nNumCustomControls 		= (null != controls) ? controls.length : 0;
			m_nNumCustomModels 			= (null != models) ? models.length : 0;
			m_nNumCustomPageControls 	= (null != pagecontrols) ? pagecontrols.length : 0;
			m_nNumCustomSpaces 			= (null != activityspaces) ? activityspaces.length : 0;
			m_nNumCustomViews 			= (null != views) ? views.length : 0;
        }
    }

	/**
	 * This method returns the number of custom Controls
	 * currently loaded.  This number is reset every time
	 * custom objects are loaded.
	 *  
	 * @return The number of custom Controls currently loaded.
	 */
	public int GetNumCustomControls()
	{
		return m_nNumCustomControls;
	}

	/**
	 * This method returns the number of custom Models
	 * currently loaded.  This number is reset every time
	 * custom objects are loaded.
	 *  
	 * @return The number of custom Models currently loaded.
	 */
	public int GetNumCustomModels()
	{
		return m_nNumCustomModels;
	}

	/**
	 * This method returns the number of custom Page Controls
	 * currently loaded.  This number is reset every time
	 * custom objects are loaded.
	 *  
	 * @return The number of custom Page Controls currently loaded.
	 */
	public int GetNumCustomPageControls()
	{
		return m_nNumCustomPageControls;
	}

	/**
	 * This method returns the number of custom Activity Spaces
	 * currently loaded.  This number is reset every time
	 * custom objects are loaded.
	 *  
	 * @return The number of custom Activity Spaces currently loaded.
	 */
	public int GetNumCustomSpaces()
	{
		return m_nNumCustomSpaces;
	}

	/**
	 * This method returns the number of custom Views
	 * currently loaded.  This number is reset every time
	 * custom objects are loaded.
	 *  
	 * @return The number of custom Views currently loaded.
	 */
	public int GetNumCustomViews()
	{
		return m_nNumCustomViews;
	}
}
