package com.plumtree.uiinfrastructure.activityspace;

import com.plumtree.openlog.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.uiinfrastructure.helpers.ConfigService;
import com.plumtree.uiinfrastructure.login.*;
import com.plumtree.uiinfrastructure.opener.*;
import com.plumtree.uiinfrastructure.opener.types.classtypes.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This class creates an URL that points to MVC components.
 * 
 * This adds the session user ID to the query string for usage
 * tracking purposes (PTTracker).
 * 
 * @author Don Hayler
 * @author MichaelD
 */
public class ASURL implements ILinkElement
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.activityspace.ASURL");

	private HTMLAnchor m_haElement  = null;
	private String m_strControl 	= null;
	private String m_strLinkTitleLabel	= null;
	private String m_strPage    	= null;
	private String m_strSubSpaceKey = null;
	private boolean m_bIsSubSpace = false;

	/** This is used to get the proper base url when constructing the URL. */
	private String m_strTargetSpaceName = null;

	/** The URL is necessary to determine the proper base URL */
	private String m_strCurrentNormalizedBaseURL = null;

	/**
	 * Only used when the asurl is a container for a text label (not an anchor).
	 */
	private String m_strNoLinkLabel = null;

	// NOTE: The size for this XPStringBuilder needs to be tuned
	private XPStringBuilder m_sbQueryStringPairs 	= new XPStringBuilder(300);

	/** whether or not this url has been initialized. */
	private boolean m_bLinkSet					= false;

	private boolean m_bURLConstructed			 	= false;
	private boolean m_bXMLEncoded					= false;

	/** whether or not to use the main server URL */
	private boolean m_bUseMainServerURL = false;
	
	/** if we use the main server URL, whether or not to use the https one */
	private boolean m_bUseSecureMainServerURL = false;

	/** HACK: this is added to deal with the cases when old code passes ASURLs around, and updating it
	 * to use friendly urls is too involved. In these cases, this variable holds the friendly url so that
	 * we don't have to update to existing code to pass around another url object **/
	private String m_externalURL = null;
	
	/**
	 * Creates a new ASURL object.
	 * 
	 * @param AActivitySpace the owner activity space
	 */
	public ASURL(AActivitySpace asOwner)
	{
		// JF- The ASURL object cannot be created if the Application Base URL hasn't been set.
		// It has to be set from the WarmUp Servlet.
		if (!ConfigHelper.IsInitialized())
		{
			log.Error("Attempting to create an ASURL before the Application Base URL has been set on the config helper.");
		}

		m_strCurrentNormalizedBaseURL = asOwner.GetCurrentNormalizedBaseURL();
	}

	/**
	 * This method allows you to add a control argument to the HTML URL.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.ILink#AddQueryStringPair(String, String)
	 *
	 * This URL encodes the value for you
	 *
	 * @param strName The name of the argument
	 * @param strValue The value of the argument
	 */
	public void AddControlArgument(String strName, String strValue)
	{
		AddEncodedControlArgument(strName,
			XPHttpUtility.UrlEncode(strValue, LinkConstants.UTF_8_ENCODING));
	}

	/**
	 * This method allows you to add a control argument to the HTML URL.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.ILink#AddQueryStringPair(String, String[])
	 *
	 * This URL encodes the value for you
	 *
	 * @param strName The name of the argument
	 * @param strValues The array of values for the argument
	 */
	public void AddControlArgument(String strName, String[] strValues)
	{
		if (m_bURLConstructed)
		{
			log.Error("Ignoring attempt to change ASURL after URL has been constructed.");

			return;
		}

		if (strValues == null)
		{
			log.Warn("AddControlArgument called with null String[]");

			return;
		}

		for (int i = 0; i < strValues.length; i++)
		{
			AddControlArgument(strName, strValues[i]);
		}
	}

    /**
     * An addition that replaces long strings with references;
     * only certain parts of the tree know how to unref it
     * 
     * The strName will be preceded by "@" to mark it as a reference.
     * The "@" should not be used when unreferencing the data.
     * 
     * NOTE: Oleg needs to test this, make about 1000 (max is about 2000)
     * 
     * @author olegs
     *
     * @param strName The name of the query string parameter
     * @param strValue the long value to be passed as a reference
     * @param asOwner the parent AActivitySpace
     */
    public void AddControlArgument(String strName, String strValue, AActivitySpace asOwner)
    {
        if ((strValue != null) && (strValue.length() > 2))
        {
            strValue = "@" + asOwner.refString(strValue);
        }

        AddControlArgument(strName, strValue);
    }

	/**
	 * This method allows you to add a control argument to the HTML URL.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.ILink#AddControlArgument(String, String)
	 *
	 * @param strName The name of the argument
	 * @param nValue The int value of the argument
	 */
	public void AddControlArgument(String strName, int nValue)
	{
		AddEncodedControlArgument(strName, XPConvert.ToString(nValue));
	}

	/**
	 * This method allows you to add a control argument to the HTML URL.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.ILink#AddControlArgument(String, String)
	 *
	 * @param strName The name of the argument
	 * @param booleanValue The boolean value of the argument
	 */
	public void AddControlArgument(String strName, boolean bValue)
	{
		AddEncodedControlArgument(strName, XPConvert.ToString(bValue));
	}

	/**
	 * This method allows you to add a control argument to the HTML URL with no value.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.ILink#AddControlArgument(String, String)
	 *
	 * @param strName The name of the argument
	 */
	public void AddControlArgument(String strName)
	{
		if (m_bURLConstructed)
		{
			log.Error("Ignoring attempt to change ASURL after URL has been constructed.");

			return;
		}

		m_sbQueryStringPairs.Append(GetAmpersand() + strName + LinkConstants.EQUALS);
	}

	/**
	 * NOTE: If you only pass in an image with an alt tag, and no link label,
	 * you need to call the SetLinkTitle method with the alt string. Netscape 7
	 * does not display alt tags for images inside an anchor so you would need
	 * to use the anchor's title tag.
	 * 
	 * Calling AddInnerHTMLString 
	 *
	 * @param element the html element to be added inside the url
	 */
	public void AddInnerHTMLElement(HTMLElement element)
	{
		if (m_bURLConstructed)
		{
			log.Error("Ignoring attempt to change ASURL after URL has been constructed.");

			return;
		}

		m_haElement.AddInnerHTMLElement(element);
	}

	/**
	 * This adds a non-HTMLEncoded string.  This method
	 * should be used for localized strings, as they
	 * should be safe.
	 *
	 * @param string the string to be added to the url text
	 */
	public void AddInnerHTMLString(String innerStr)
	{
		if (m_bURLConstructed)
		{
			log.Error("Ignoring attempt to change ASURL after URL has been constructed.");

			return;
		}

		m_haElement.AddInnerHTMLString(innerStr);
		m_haElement.SetTitle(innerStr);
	}

	/**
	 * This adds an HTMLEncoded string.  This method
	 * should not be used for localized strings, as
	 * should not need to be encoded.
	 *
	 * @param string the pre-html encoded string to be added to the url text
	 */
	public void AddInnerHTMLEncodedString(String innerStr)
	{
		if (m_bURLConstructed)
		{
			log.Error("Ignoring attempt to change ASURL after URL has been constructed.");

			return;
		}
		m_haElement.SetTitle(XPHttpUtility.HtmlEncode(innerStr));
		m_haElement.AddInnerHTMLEncodedString(innerStr);
	}

	/**
	 * Creates an ASURL from an ASURLContainer. The ASURL is set to
	 * SetLinkGetSpaceIfCached, i.e. try to find the activityspace
	 * in the cache, create a new AS if it doesn't exist.
	 *
	 * @param ASURLContainer urlCont
	 */
	public static ASURL CreateFromURLContainer(ASURLContainer urlCont)
	{
		if (null == urlCont)
		{
			return null;
		}

		ASURL url = new ASURL(urlCont.GetASParent());
		url.SetLinkGetSpaceIfCached(urlCont.GetASSpace(), urlCont.GetASParent());

		url.SetControl(urlCont.GetControl());
		url.SetDisplayPage(urlCont.GetPage());
		url.AddQueryStringParams(urlCont.GetNonASQSPairs());

		try
		{
			url.AddInnerHTMLString(urlCont.GetURLName());
		}
		catch (HTMLException he)
		{
			log.Error(he, "Failed to set name on ASURL.");
		}

		return url;
	}

	/**
	 * Creates an ASURL from an ASURLContainer. The ASURL is set to
	 * SetLinkCreateNewSpace, i.e. always create a new AS.
	 *
	 * @param ASURLContainer urlCont
	 *
	 * @return ASURL
	 */
	public static ASURL CreateFromURLContainerNew(ASURLContainer urlCont)
	{
		if (null == urlCont)
		{
			return null;
		}

		ASURL url = new ASURL(urlCont.GetASParent());
		url.SetLinkCreateNewSpace(urlCont.GetASSpace(), urlCont.GetASParent());

		url.SetControl(urlCont.GetControl());
		url.SetDisplayPage(urlCont.GetPage());
		url.AddQueryStringParams(urlCont.GetNonASQSPairs());

		try
		{
			url.AddInnerHTMLString(urlCont.GetURLName());
		}
		catch (HTMLException he)
		{
			log.Error(he, "Failed to set name on ASURL.");
		}

		return url;
	}

	/**
	 * Create an HTML element from this url
	 *
	 * @return The html element that can display this url
	 */
	public HTMLElement GetURLAsHTMLElement()
	{
		// Quasi hack, if no haElement was created (AS never set) we know it
		// is an ASURL with only a label, return that label in a GenericHTMLElement.
		if (null == m_haElement)
		{
			return new HTMLGenericElement(m_strNoLinkLabel);
		}
		else
		{
			ConstructURL();

			if ( m_externalURL != null ) {
				// HACK: return external url if one is set, see comment on the variable definition
				// for when this is used
				m_haElement.SetHRef(m_externalURL);
			}
			
			return m_haElement;
		}
	}

	/**
	 * return the string that represents this url
	 *
	 * @return the string that represents this url
	 */
	public String GetURLAsString()
	{
		if ( m_externalURL != null ) {
			// HACK: return external url if one is set, see comment on the variable definition
			// for when this is used
			return m_externalURL;
		}
		
		ConstructURL();
		
		return m_haElement.GetHRef();
	}

	/**
	 * Set the control to use in this url
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.ILink#SetControl(String)
	 *
	 * @param strControl The name of the control to use in the url
	 */
	public void SetControl(String strControl)
	{
		if (m_bURLConstructed)
		{
			log.Error("Ignoring attempt to change ASURL after URL has been constructed.");

			return;
		}

		m_strControl = strControl;
	}

	/**
	 * Set the display page to be used in this url
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.ILink#SetDisplayPage(String)
	 *
	 * @param strPage the display page to show
	 */
	public void SetDisplayPage(String strPage)
	{
		if (m_bURLConstructed)
		{
			log.Error("Ignoring attempt to change ASURL after URL has been constructed.");

			return;
		}

		m_strPage = strPage;
	}

	/**
	 * This method allows you to set the image base url on the
	 * resulting HTMLElement.  This should only be used if you
	 * are using the element outside of the context of an HTMLPage.
	 *
	 * NOTE This should probably be removed, since you can just
	 * get the URL as an element and set it there.
	 * 
	 * @param strImageBaseURL The image base url
	 */
	public void SetImgBaseUrl(String strImageBaseURL)
	{
		m_haElement.SetImgBaseUrl(strImageBaseURL);
	}

	/**
	 * NOTE: This method is not for setting the display text of an anchor!
	 * Use SetLabelString to set the display text.
	 *
	 * This is used when the ASURL is used to only output a text label,
	 * i.e not a link.
	 */
	public void SetNoLinkLabelString(String label)
	{
		m_strNoLinkLabel = label;
	}
	
	/**
	 * Set the title of the html anchor. AddInnerHTMLString and
	 * AddInnerHTMLEncodeString will autoamitcally set the title of the anchor
	 * to the link label. Use this method to change this or pass in null
	 * to remove the title.
	 * 
	 * @param label	Text to be used in the title attribute of an html anchor.
	 * Null to remove title.
	 */
	public void SetLinkTitle(String label)
	{
		m_strLinkTitleLabel = label;	
	}

	/**
	 * Sets whether or not to use the main server URL.
	 * 
	 * @param boolean true implies use the main server URL
	 */
	public void SetUseMainServerURL(boolean bUse, boolean bIsSecure)
	{
		m_bUseMainServerURL = bUse;
		m_bUseSecureMainServerURL = bIsSecure;
	}

	/**
	 * Sets the URL to use the XML encoded ampersands rather than
	 * normal HTML ones (for WML devices)
	 * 
	 * @param boolean true implies use XML encoded ampersand
	 */
	public void SetXMLEncoding(boolean bXML)
	{
		m_bXMLEncoded = bXML;
	}

	//////////////////////////////////////////////////////
	// Set link methods.  One of these (and only one) must
	// be called before getting the result of this URL
	//////////////////////////////////////////////////////

	/**
	 * This method creates a brand new AActivitySpace.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.ILink#SetLinkCreateNewSpace(String, AActivitySpace)
	 *
	 * @param String strSpace
	 * @param AActivitySpace parent
	 *
	 * @return boolean true implies success, false implies failure
	 */
	public boolean SetLinkCreateNewSpace(String strSpace, AActivitySpace parent)
	{
		if (parent == null)
		{
			log.Error("Attempting to use SetLinkCreateNewSpace method with a null parent AActivitySpace.");

			return false;
		}

		if (m_bLinkSet)
		{
			log.Error("Attempting to use SetLinkCreateNewSpace method on an URL that has already had the space set");

			throw new XPException("Attempting to use SetLinkCreateNewSpace method on an URL that has already had the space set");
		}

		m_bLinkSet = true;

		if ((null == strSpace) || ("".equals(strSpace)))
		{
			log.Warn("Attempting to use SetLinkCreateNewSpace method with a null or empty space name.");
		}

		m_strTargetSpaceName = strSpace;

		// We cannot set the base URL here because we need to know if it is a sub-space
		// or not.  We prepend the base URL when we construct the URL.

		// Use the common opener to open Activity Spaces
		String sURL = IOpenerConstants.QS_OPEN + LinkConstants.EQUALS + UIActivitySpaces.UICLASSTYPEDESC_CLASSNAME + GetAmpersand() +
			UIActivitySpaces.QS_ACTIVITY_SPACE_NAME + LinkConstants.EQUALS + strSpace + GetAmpersand() +
			UIActivitySpaces.QS_PARENT_SPACE_NAME + LinkConstants.EQUALS + parent.GetName() + GetAmpersand() +
			UIActivitySpaces.QS_PARENT_SPACE_ID + LinkConstants.EQUALS + parent.GetSpaceID();

		// Get a cached version of the opener space if possible
		sURL += GetAmpersand() + ASConstants.CACHED + LinkConstants.EQUALS + XPConvert.ToString(true);

		// Add the user ID to the URL for usage tracking
		sURL += GetAmpersand() +
			ASConstants.USERID_INPUT + LinkConstants.EQUALS + LoginHelper.INSTANCE.GetUserSessionID(parent.GetUserSession());

		m_haElement = new HTMLAnchor(sURL);

		return true;
	}

	/**
	 * This constructor takes a specific space and returns a link to that (including space ID)
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.ILink#SetLinkGetCachedSpace(AActivitySpace)
	 *
	 * @param AActivitySpace space
	 * @param AActivitySpace parent
	 *
	 * @return boolean true implies success, false implies failure
	 */
	public boolean SetLinkGetCachedSpace(AActivitySpace space)
	{
		if (space == null)
		{
			log.Error("Attempting to use SetLinkGetCachedSpace method with a null AActivitySpace.");

			return false;
		}

		if (m_bLinkSet)
		{
			log.Error("Attempting to use SetLinkGetCachedSpace method on an URL that has already had the space set");

			throw new XPException("Attempting to use SetLinkGetCachedSpace method on an URL that has already had the space set");
		}

		m_bLinkSet = true;

		AActivitySpace parent = space.GetParentSpace();

		m_strTargetSpaceName = space.GetName();

		// Note that even though we now know that this is a sub-space,
		// we still don't set the base URL for consistency.
		m_bIsSubSpace = space.IsSubSpace();

		// We cannot set the base URL here because we need to know if it is a sub-space
		// or not.  We prepend the base URL when we construct the URL.

		// Use the Common Opener to open the space
		String sURL = IOpenerConstants.QS_OPEN + LinkConstants.EQUALS + UIActivitySpaces.UICLASSTYPEDESC_CLASSNAME + GetAmpersand() +
			UIActivitySpaces.QS_ACTIVITY_SPACE_NAME + LinkConstants.EQUALS + space.GetName() + GetAmpersand() +
			UIActivitySpaces.QS_ACTIVITY_SPACE_ID + LinkConstants.EQUALS + space.GetSpaceID();

		if (parent != null)
		{
			sURL = sURL + GetAmpersand() + UIActivitySpaces.QS_PARENT_SPACE_NAME + LinkConstants.EQUALS +
			parent.GetName() + GetAmpersand() + UIActivitySpaces.QS_PARENT_SPACE_ID + LinkConstants.EQUALS +
			parent.GetSpaceID();
		}

		// Get a cached version of the opener space if possible
		sURL += GetAmpersand() + ASConstants.CACHED + LinkConstants.EQUALS + XPConvert.ToString(true);

		sURL += GetAmpersand() + ASConstants.USERID_INPUT + LinkConstants.EQUALS +
			LoginHelper.INSTANCE.GetUserSessionID(space.GetUserSession());

		m_haElement = new HTMLAnchor(sURL);

		return true;
	}

	/**
	 * This method gets a named AActivitySpace from the cache if it is there, or
	 * creates a new one if it is not in the cache.
	 * This method should be used to get common AActivitySpaces such as MyPage and
	 * Community, not minor spaces like Tree and Editors.
	 * The SetSpaceGetCached(AActivitySpace) should be used instead of this one when
	 * you can.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.ILink#SetLinkGetSpaceIfCached(String, AActivitySpace)
	 *
	 * @param String strSpace
	 * @param AActivitySpace parent
	 *
	 * @return boolean true implies success, false implies failure
	 */
	public boolean SetLinkGetSpaceIfCached(String strSpace, AActivitySpace parent)
	{
		if (parent == null)
		{
			log.Error("Attempting to use SetLinkGetSpaceIfCached method with a null parent AActivitySpace.");

			return false;
		}

		if (m_bLinkSet)
		{
			log.Error("Attempting to use SetLinkGetSpaceIfCached method on an URL that has already had the space set");

			throw new XPException("Attempting to use SetLinkGetSpaceIfCached method on an URL that has already had the space set");
		}

		m_bLinkSet = true;

		if ((null == strSpace) || ("".equals(strSpace)))
		{
			log.Warn("Attempting to use SetLinkGetSpaceIfCached method with a null or empty space name.");
		}

		m_strTargetSpaceName = strSpace;

		// We cannot set the base URL here because we need to know if it is a sub-space
		// or not.  We prepend the base URL when we construct the URL.

		// Note: This will not recognize that this space is a sub-space if the parent
		// is null.

		// Use the Common Opener to open the space
		String sURL = IOpenerConstants.QS_OPEN + LinkConstants.EQUALS + UIActivitySpaces.UICLASSTYPEDESC_CLASSNAME + GetAmpersand() +
			UIActivitySpaces.QS_ACTIVITY_SPACE_NAME + LinkConstants.EQUALS + strSpace + GetAmpersand() +
			// This tells the opener that this space should be cached
			UIActivitySpaces.QS_ACTIVITY_SPACE_ID + LinkConstants.EQUALS + UIActivitySpaces.QS_SPACE_ID_CACHED;

		if (null != parent)
		{
			// Use common opener
			sURL += GetAmpersand() +
				UIActivitySpaces.QS_PARENT_SPACE_NAME + LinkConstants.EQUALS + parent.GetName() + GetAmpersand() +
				UIActivitySpaces.QS_PARENT_SPACE_ID + LinkConstants.EQUALS + parent.GetSpaceID();

			sURL += GetAmpersand() +
				ASConstants.USERID_INPUT + LinkConstants.EQUALS +
				LoginHelper.INSTANCE.GetUserSessionID(parent.GetUserSession());

			// If we have a valid parent space, then this space may be a sub-space.
			ASCache myCache = (ASCache) parent.GetSubSession()
				.GetAttribute(ASConstants.SANDBOXASCACHE);

			if (null != myCache)
			{
				// Use this cache method, rather than getting the space directly,
				// so that we don't reset the timestamp on the space.
				// This is needed for navigation type links which may be displayed
				// on almost every page, but rarely clicked.  Not resetting the
				// timestamp keeps this space from floating at the head of the cache,
				// even though it may not be being used.
				m_bIsSubSpace = myCache.GetSpaceIsSubSpace(strSpace);
			}
		}

		// Get a cached version of the opener space if possible
		sURL += GetAmpersand() + ASConstants.CACHED + LinkConstants.EQUALS + XPConvert.ToString(true);

		m_haElement = new HTMLAnchor(sURL);

		return true;
	}

	///////////////////////////
	// Optional methods on URL.
	///////////////////////////

	/**
	 * This method sets the sub space key.  If this is not called, or is empty,
	 * the space will not be a sub-space.  Otherwise it will be.
	 *
	 * The sub-space key is different from the space ID.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.ILink#SetSubSpaceKey(String)
	 *
	 * @param String sSubSpaceKey
	 */
	public void SetSubSpaceKey(String sSubSpaceKey)
	{
		if (m_bURLConstructed)
		{
			log.Error("Ignoring attempt to change ASURL after URL has been constructed.");

			return;
		}

		m_strSubSpaceKey = sSubSpaceKey;

		if (sSubSpaceKey == null)
		{
			m_strSubSpaceKey = "";
		}

		if (!"".equals(m_strSubSpaceKey))
		{
			m_bIsSubSpace = true;
		}
	}
	
	/**
	 * Sets the member variable m_bIsSubSpace to true (i.e. this ASURL is 
	 * creating a link to a sub space). 
	 * 
	 * There is one special case with TreeAS when we need to manually tell ASURL 
	 * that we are going to a sub space. This is when we create a link to a cached tree 
	 * before the tree has actually been cached (happens in SelectPTObject for example).  
	 * Otherwise it won't work in security mode 1, see bug 26851. In all other cases, 
	 * ASURL will be able to determine dynamically if the link is going to a sub space 
	 * or not. 
	 */
	public void IsSubSpace()
	{
		m_bIsSubSpace = true;
	}

	/**
	 * This is a helper method to add qs arguments.  This assumes that you
	 * have already URL encoded the values if necessary.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.ILink#AddQueryStringPair(String, String)
	 *
	 * @param strName The name of the query string argument
	 * @param strValue The previously HTML encoded value of the argument
	 */
	private void AddEncodedControlArgument(String strName, String strValue)
	{
		if (m_bURLConstructed)
		{
			log.Error("Ignoring attempt to change ASURL after URL has been constructed.");

			return;
		}

		m_sbQueryStringPairs.Append(GetAmpersand()).Append(strName).Append
			(LinkConstants.EQUALS).Append(strValue);
	}

	/**
	 * Internal only add query string parameter string. Used by CreateFromURLContainer
	 *
	 */
	private void AddQueryStringParams(String queryStrParams)
	{
		m_sbQueryStringPairs.Append(queryStrParams);
	}

	/**
	 * This internal method constructs the url into the element
	 */
	private String getBaseURL(String strTargetSpaceName, boolean bIsSubSpace)
	{
		if (m_bUseMainServerURL)
		{
			return ConfigService.getInstance().GetMainServerApplicationURL(m_bUseSecureMainServerURL);
		} else
		{
			return ConfigService.getInstance().GetApplicationBaseURLToAS(m_strCurrentNormalizedBaseURL, strTargetSpaceName, bIsSubSpace);
		}
	}

	/**
	 * This internal method constructs the url into the element
	 */
	private void ConstructURL()
	{
		if (!m_bLinkSet)
		{
			log.Error("Attempting to get URL without properly constructing it. You must call a SetSpace* method before getting the result of this URL.");
		}

		if (m_bURLConstructed)
		{
			return;
		}

		m_bURLConstructed = true;

		// We need to prepend the base url to the URL.  We cannot set it earlier
		// because we need to know if it is a sub-space or not.
		if ((m_strSubSpaceKey != null) && (!"".equals(m_strSubSpaceKey)))
		{
			m_haElement.SetHRef(getBaseURL(m_strTargetSpaceName, m_bIsSubSpace) + m_haElement.GetHRef() +
				GetAmpersand() + ASConstants.SUBSPACE + LinkConstants.EQUALS +
				m_strSubSpaceKey);
		} else
		{
			m_haElement.SetHRef(getBaseURL(m_strTargetSpaceName, m_bIsSubSpace) + m_haElement.GetHRef());
		}

		if (m_strControl != null)
		{
			m_haElement.SetHRef(m_haElement.GetHRef() + GetAmpersand() +
				ASConstants.CONTROL + LinkConstants.EQUALS + m_strControl);
		}

		if (m_strPage != null)
		{
			m_haElement.SetHRef(m_haElement.GetHRef() + GetAmpersand() +
				ASConstants.PAGE + LinkConstants.EQUALS + m_strPage);
		}

		m_haElement.SetHRef(m_haElement.GetHRef() +
			m_sbQueryStringPairs.ToString());

		// Set the title on the anchor if one exists.
		if ( null != m_strLinkTitleLabel ) {
			m_haElement.SetTitle(m_strLinkTitleLabel);	
		}

		String strURL = m_haElement.GetHRef();

		if (StringContainsMarkUpBraces(strURL))
		{
			log.Error("URL could not be constructed due to presence of a '<' or a '>' character in the URL: "
				+ strURL);

			m_haElement.SetHRef("");

			throw new XPException("URL could not be constructed due to presence of a '<' or a '>' character in the URL: "
				+ strURL);
		}
	}

	/**
	 * Helper method to return the correct ampersand.
	 * 
	 * WML and XML require a special encoded ampersand.
	 * 
	 * @return String the ampersand to use
	 */
	private String GetAmpersand()
	{
		if (m_bXMLEncoded)
		{
			return LinkConstants.XML_AMPERSAND;
		} else
		{
			return LinkConstants.AMPERSAND;
		}
	}

	/**
	 * Helper method that checks whether or not a string contains angle braces.
	 * 
	 * This is useful for determining if someone is trying to do a javascript hack.
	 * 
	 * @param String the string to process
	 * @return boolean true implies contains braces and may be unsafe.
	 */
	private static final boolean StringContainsMarkUpBraces(String _strStringIn)
	{
		if (XPStringUtility.IndexOf(_strStringIn, '<') != -1 || XPStringUtility.IndexOf(_strStringIn, '>') != -1)
		{
			return true;
		}
		
		return false;
	}

	/**
	 * HACK:
	 * Sets a full URL. ASURL shouldn't need this but this method is needed so that 
	 * we don't have to rip up too much existing code to facilitate friendly URLs   
	 * 
	 * @see com.plumtree.uiinfrastructure.activityspace.ILink#SetLinkToExternalURL(java.lang.String)
	 */
	public boolean SetLinkToExternalURL(String m_externalurl) {
		m_externalURL = m_externalurl;
		return true;
	}
}