package com.plumtree.uiinfrastructure.activityspace;

import com.plumtree.openfoundation.util.*;

/**
 * Controls process the events generated by users.  The Interpreter takes
 * the user events and fills out the fields in the control.  The control is then
 * executed and returns the next DisplayPage (usually the parent page, unless the
 * control is a go to url control).
 *
 * Controls should encapsulate the business logic of the AActivitySpace.  As an MVC
 * component, Views are subject to the strict separation between data, logic, and
 * presentation.
 *
 * Obviously, there should be a good way to refer to controls without having to
 * manually type in a text string.  There should be some sort of package identifier
 * used to get these.
 * 
 * @author Don Hayler
 */
public interface IControl extends IMVCObject, IManagedObject
{
	/**
	 * Execute the control business logic and optionally return a Redirect object
	 * pointing to another AActivitySpace.  Returning null means that the user
	 * should return to the parent AActivitySpace's current page.
	 *
	 * This method is designed to check that the user has access to perform the
	 * actions with the given parameters.  If the user does not, a redirect object
	 * which has had Redirect.SetRedirectToLogin(true) called on it.  If the user
	 * is the Guest, this will bounce to the login page and back to the Redirect
	 * after the user has logged on.  If the user is already logged on, they will
	 * be redirected to an error page.
	 *
	 * In general, the error redirect that is returned should contain all of the
	 * query string parameters that made up the URL that originally led to the
	 * control.
	 *
	 * Checking whether or not a particular user ever has any access to the space,
	 * page, and control should be done in IPTActivitySpace.CheckBasicAccess().
	 *
	 * @param XPHashtable this contains all of the name value pairs for the control
	 *             in String, String[] format.  This may contain information the control
	 *             doesn't need.  The control should not modify this argument.
	 *
	 * @return Redirect where to redirect to after the control is finished executing.
	 *         This will go to the login page or an error page if Redirect.SetRedirectToLogin(true)
	 *         has been called.
	 */
	public Redirect CheckActionSecurityAndExecute(XPHashtable arguments);

	/**
	 * Initialize the control with the model it should use and it's parent
	 * AActivitySpace.  This method needs to be called before any other methods.
	 * This method also clears all internal data so the control can be used again
	 * without fear of mixing data.
	 */
	public void Init(IModel model, AActivitySpace space);
}

