package com.plumtree.uiinfrastructure.activityspace;

/**
 * This interface provides a common way to create an object that links to
 * an AActivitySpace.  It also provides methods to specify optional Page,
 * Controls, and Query String arguments.
 *
 * To use this interface, first create an object that implements the
 * interface, then call one of the SetSpace* methods to set what kind of
 * link you want.  Than add optional arguments (page, etc...)
 *
 * @author Don Hayler
 */
public interface ILink
{
	/**
	 * This method allows you to add a control argument to the HTML URL.
	 *
	 * Do not use this method to set arguments that already have their own
	 * methods, such as sub-space key, display page, and control.
	 *
	 * For HTTP URLs, strValue should be automatically URL encoded.
	 *
	 * The behavior when arguments are added with the same name is
	 * undefined.  Sometimes it will overwrite the first name/value
	 * pair, and sometimes it will append the second name/value pair
	 * after the first.  Arguments should not be added twice.
	 *
	 * @param strName the name of the argument
	 * @param strValue the value of the argument
	 */
	public void AddControlArgument(String strName, String strValue);

	/**
	 * This method allows you to add a control argument to the HTML URL.
	 *
	 * Do not use this method to set arguments that already have their own
	 * methods, such as sub-space key, display page, and control.
	 *
	 * For HTTP URLs, strValues should be automatically URL encoded.
	 *
	 * The behavior when arguments are added with the same name is
	 * undefined.  Sometimes it will overwrite the first name/value
	 * pair, and sometimes it will append the second name/value pair
	 * after the first.  Arguments should not be added twice.
	 *
	 * @param strName the name of the argument
	 * @param strValues the value of the argument
	 */
	public void AddControlArgument(String strName, String[] strValues);

	/**
	 * This method allows you to add a control argument to the HTML URL.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.ILink#AddControlArgument(String, String)
	 *
	 * @param strName
	 * @param nValue
	 */
	public void AddControlArgument(String strName, int nValue);

	/**
	 * This method allows you to add a control argument to the HTML URL.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.ILink#AddControlArgument(String, String)
	 *
	 * @param strName
	 * @param booleanValue
	 */
	public void AddControlArgument(String strName, boolean bValue);

	/**
	 * This method allows you to add a control argument to the HTML URL with no value.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.ILink#AddControlArgument(String, String)
	 *
	 * @param strName
	 * @param booleanValue
	 */
	public void AddControlArgument(String strName);

	/**
	 *
	 *
	 * @param strControl the name of the control to link to
	 */
	public void SetControl(String strControl);

	/**
	 *
	 *
	 * @param strPage the name of the page to link to
	 */
	public void SetDisplayPage(String strPage);

	//////////////////////////////////////////////////////////
	// Set space methods.  One of these (and only one) must be
	// called before getting the result of this URL
	//////////////////////////////////////////////////////////

	/**
	 * This method creates a brand new AActivitySpace.
	 *
	 * @param strSpace The name of the space to create
	 * @param parent the parent space
	 *
	 * @return boolean true implies success, false implies failure
	 */
	public boolean SetLinkCreateNewSpace(String strSpace, AActivitySpace parent);

	/**
	 * This method takes a specific space and returns a link to that (including space ID).
	 *
	 * @param space the space to create a link to
	 *
	 * @return boolean true implies success, false implies failure
	 */
	public boolean SetLinkGetCachedSpace(AActivitySpace space);

	/**
	 * This method gets a named AActivitySpace from the cache if it is there, or
	 * creates a new one if it is not in the cache.
	 * This method should be used to get common AActivitySpaces such as MyPage and
	 * Community, not minor spaces like Tree and Editors.
	 * The SetLinkGetCachedSpace(AActivitySpace) should be used instead of this one
	 * when you can.
	 *
	 * @param strSpace the name of the space to get or create (if it can't be found)
	 * @param parent the parent space
	 *
	 * @return boolean true implies success, false implies failure
	 */
	public boolean SetLinkGetSpaceIfCached(String strSpace, AActivitySpace parent);

	///////////////////////////
	// Optional methods on URL.
	///////////////////////////

	/**
	 * This method sets the sub space key.  If this is not called, or is empty,
	 * the space will not be a sub-space.  Otherwise it will be.
	 *
	 * The sub-space key is different from the space ID.
	 *
	 * @param sSubSpaceKey the key for the sub-space
	 */
	public void SetSubSpaceKey(String sSubSpaceKey);
	
	/**
	 */
	public boolean SetLinkToExternalURL(String url);
}

