package com.plumtree.uiinfrastructure.activityspace;

/**
 * Models encapsulate the data in the MVC framework.  Views have access to the read
 * only model interface and should only read data from it.  Controls get and set data,
 * and therefore use the base IModel interface.
 *
 * Models should encapsulate the data handling of the AActivitySpace.  Temporary UI
 * data that you want to store for each user should go here.  As an MVC component,
 * Models are subject to the strict separation between data, logic, and presentation.
 *
 * It is a requirement that Models not reference HTMLElements or HTMLConstructs.
 * Models can return text Strings for display in HTML, but they are not allowed to use
 * HTML themselves.
 *
 * It is also a requirement that Models not return server structures to Views.  For
 * instance, rather than returning an IPTQueryResult to a View, the model should
 * provide accessor methods to it.  Views can use server constants such as
 * PT_PROPIDS.PT_PROPID_NAME.
 *
 * Obviously, there should be a good way to refer to models without having to
 * manually type in a text string.  There should be some sort of package identifier
 * used to get these.
 *
 * When writing a model class, the Get methods (read-only methods) should be split
 * out into an IMyModelRO interface, which MyModel then implements.  MyModel would
 * also include all of the Set methods (write methods) and any helper methods.  There
 * does not need to be an IMyModel interface for the Set methods, unless you are
 * expecting someone to implement their own version of your model.
 * 
 * @author Don Hayler
 */
public interface IModel extends IModelRO, IManagedObject
{
	/**
	 * This method is called at the end of every HTTP request by the Interpreter.
	 * The Interpreter calls AActivitySpace.CleanupTempData(), which calls this
	 * method on all IModels registered using AActivitySpace.RegisterModel().
	 *
	 * This method is intended for cleaning up temporary data that is only used
	 * for the lifetime of a single request, like DB query results.
	 */
	public void CleanupTempData();

	/**
	 * This method is called when an ActivitySpace is removed from the cache.
	 * The Interpreter calls AActivitySpace.CleanupAllData(), which calls this
	 * method on all IModels registered using AActivitySpace.RegisterModel().
	 *
	 * This method is intended for releasing data stored in the server when
	 * the activity space is no longer needed, such as search objects that
	 * need to be maintained as long as the user is clicking through the
	 * search, but should be freed when the space can no longer be used.
	 * 
	 * This can be used to prevent Java references to COM memory from hanging
	 * around in the long lived Java heap without cleaning up the underlying
	 * COM memory.  The Java reference will still stay in the long lived heap
	 * until it gets garbage collected (which could be a while), but the COM
	 * memory should get released here.
	 * 
	 * This method will not be called if an HTTPSession times out.
	 * 
	 * CleanupTempData will be called before CleanupAllData.
	 */
	public void CleanupAllData();

	/**
	 * This method initializes the model.  This is called automatically when
	 * AActivitySpace.RegisterModel() is called.
	 *
	 * @param parent the parent Activity Space
	 */
	public void Init(AActivitySpace parent);
}

