package com.plumtree.uiinfrastructure.activityspace;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This object wraps HTMLPages and binary data for output to the browser.
 * There will be methods for setting HTTP Headers for the browser.
 *
 * This interface is intended for use in IDisplayPages.
 * 
 * @author Don Hayler
 */
public interface IWebData
{
	/**    Call this method to add a cookie to the response.
	 *
	 *     @param XPCookie Pass an XPCookie
	 *
	 */
	public void AddCookie(XPCookie c);

	/**
	 * This optional method flushes all current data in the HTMLPage to the
	 * browser.  This should only be called when the banner has been fully added
	 * to the HTMLPage.
	 *
	 * This method requires SetData to have been called first.
	 *
	 * This means that the HTMLHead element must be completed and that all
	 * HTMLObjects (like tables) that are currently in the HTMLBody element
	 * need to be complete.  Data will not be able to be added to them after
	 * flushing.  Data will be able to be added to the HTMLBody only.
	 *
	 * This is not currently implemented.
	 */
	public void FlushData();

	/**
	 * Sends a true 302 redirect immediatly.
	 *
	 * @param _strLocation Where to go
	 */
	public void SendRedirect(String _strLocation);

	///////////////////////////////////
	// SetData methods
	// Only one of these may be called.
	///////////////////////////////////

	/**
	 *
	 * This method sets the HTMLElement as the data to be sent to the browser.
	 * The element should be an HTMLPage element in most cases - the only time
	 * that the element will not be an HTMLPage is when you want to display 
	 * partial page content.
	 *
	 * Only one SetData method should be called.
	 *
	 * @param HTMLElement to output to the browser
	 */
	public void SetData(HTMLElement element);

	/**
	 * This method sets array of bytes as the data to be sent to the browser.
	 *
	 * Only one SetData method should be called.
	 *
	 * @param byte[] binary data to output to the browser
	 */
	public void SetData(byte[] arrBodyBinary);

	/**
	 * This method sets string as the data to be sent to the browser.
	 *
	 * Only one SetData method should be called.
	 *
	 * @param String string to output to the browser
	 */
	public void SetData(String strData);

	//////////////////
	// General methods
	//////////////////    

	/**
	 * This method specifies whether the text data for this WebData object
	 * is HTML or not.  This method is not relevant for binary data.
	 * 
	 * Text data is assumed to be HTML by default.
	 *
	 * @param boolean whether or not the text data is HTML
	 */
	public void SetIsHTMLText(boolean _bIsHTMLText);

	/**
	 * JF- Sets the value of the content language.
	 * The current value of the content language will be set on every response.
	 * 
	 * @param _sLocale (e.g.: en-US)
	 */
	public void SetContentLanguage(String _sLocale);

	/**
	 * This method sets a specified header on the response.
	 *
	 * @param    String    that specifies the name of the HTTP header to set
	 *
	 * @param    String    that specifies the value of the HTTP header
	 */
	public void SetHeader(String strHeaderName, String strHeaderValue);

	/**
	 * Sets a response date header with the given name and date-value.
	 * 
	 * @param String strHeaderName - a String containing the name of the Date header
	 * @param XPDateTime xpDate - an XPDateTime containing the date.
	 */
	public void SetDateHeader(String strHeaderName, XPDateTime xpDate);

	/**
	 * Sets a response date header with the given name and date-value.
	 * 
	 * @param String strHeaderName - a String containing the name of the Date header
	 * @param long iDate - a long containing the date in milliseconds since EPOCH.
	 * Use XPDateTime.GetTimeInMilliseconds() to produce a commensurate value.
	 */
	public void SetDateHeader(String strHeaderName, long iDate);

	/**
	 * This method sets a status of the response.,
	 *
	 * @param    iStatus    an integer corresponding to the HTTP status code
	 */
	public void SetStatus(int iStatus);

	/**
	 * This method determines if WebData object will throw an exception in case of 
	 * sending an empty body of not.
	 * 
	 * @param bRequireNonEmptyBody - TRUE if you want WebData object to throw an exception
	 * 								 in case of sending an emptry body, FALSE otherwise.
	 *  
	 * @return previous value of the flag
	 */
	public boolean RequireNonEmptyBody(boolean bRequireNonEmptyBody);
}

