package com.plumtree.uiinfrastructure.activityspace;

import com.plumtree.uiinfrastructure.login.*;
import com.plumtree.openlog.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.XPHttpUtility;

/**
 * This object allows IControls to specify an AActivitySpace, IDisplayPage, or
 * IControl to redirect to.
 *
 * Currently, only AActivitySpaces and IDisplayPages are implemented.
 *
 * @author Don Hayler
 */
public class Redirect implements ILink
{ 
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.activityspace.Redirect");

	private XPHashtable m_htControlArguments;
	private boolean m_bIsErrorRedirect = false;
	private boolean m_bIsHTTPRedirect = false;
	private boolean m_bExternalURL = false;

	/** Whether or not this object has been initialized. */
	private boolean m_bLinkSet = false;

	/** external URL for use with SetLinkToExternalURL method. */
	private String m_strExternalURL = null;

	/** Indicates whether or not the parent space and ID should be stripped 
	 * from the control arguments in order to make a shorter URL. */
	private boolean m_bStripParentSpace = false;
	
	/**
	 * Creates a new Redirect object.
	 */
	public Redirect()
	{
		m_htControlArguments = new XPHashtable();
	}

	/**
	 * Optionally set single String arguments for the IControl.
	 */
	public void AddControlArgument(String name, String value)
	{
		if (name != null && value != null)
		{
			String[] values = new String[] { value };
			m_htControlArguments.PutElement(name, values);
		}
	}

	/**
	 * Optionally set String[] arguments for the IControl.
	 */
	public void AddControlArgument(String name, String[] values)
	{
		if (name != null && null != values)
		{
			m_htControlArguments.PutElement(name, values);
		}
	}

	/**
	 * This method allows you to add a control argument to the HTML URL.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.ILink#AddControlArgument(String, String)
	 *
	 * @param strName
	 * @param nValue
	 */
	public void AddControlArgument(String strName, int nValue)
	{
		AddControlArgument(strName, XPConvert.ToString(nValue));
	}

	/**
	 * This method allows you to add a control argument to the HTML URL.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.ILink#AddControlArgument(String, String)
	 *
	 * @param strName
	 * @param booleanValue
	 */
	public void AddControlArgument(String strName, boolean bValue)
	{
		AddControlArgument(strName, XPConvert.ToString(bValue));
	}

	/**
	 * This method allows you to add a control argument to the HTML URL with no value.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.ILink#AddControlArgument(String, String)
	 *
	 * @param strName
	 * @param booleanValue
	 */
	public void AddControlArgument(String strName)
	{
		m_htControlArguments.PutElement(strName, null);
	}

	/**
	 * Adds a map of String name/String[] values pairs to the redirect (overwriting previous versions)
	 *
	 * @param XPHashtable the control arguments in String name/String[] values pairs
	 */
	public void AddControlArguments(XPHashtable htArgs)
	{
		if (htArgs == null)
		{
			
				log.Error("AddControlArguments called with null hashtable.");
			

			return;
		}

		XPArrayList duplicateKeys = new XPArrayList();

		IXPEnumerator keysIter  = htArgs.GetKeys();
		while (keysIter.MoveNext())
		{
			duplicateKeys.Add(keysIter.GetCurrent());
		}

		// We need to iterate over a separate collection of keys
		// because .NET is very picky about not modifying a collection
		// while iterating over it.  This means that you cannot
		// iterate over the keys and get the associated elements at
		// the same time.
		IXPEnumerator duplicateKeysIter = duplicateKeys.GetEnumerator();
		while (duplicateKeysIter.MoveNext())
		{
			m_htControlArguments.PutElement(duplicateKeysIter.GetCurrent(),
				htArgs.GetElement(duplicateKeysIter.GetCurrent()));
		}
	}

	///////////////////////////////////////////////
	// Get Accessor Methods
	// These should only be used by the Interpreter
	// The space accessors will not return useful
	// values if the redirect is pointing to an
	// external URL.
	///////////////////////////////////////////////

	/**
	 * This method returns whether this is an internal redirect
	 * to an Activity Space (potentially a server redirect or
	 * a 302 redirect) or an external 302 redirect to another
	 * web site.
	 *
	 * @return boolean true implies external URL.
	 */
	public boolean GetIsExternalURL()
	{
		return m_bExternalURL;
	}

	/**
	 * This method returns the external URL to do a 302 redirect
	 * on.  This will not return useful information if the
	 * redirect does not point to an external URL.
	 *
	 * @return String the external URL to do a 302 redirect to.
	 */
	public String GetExternalURL()
	{
		return m_strExternalURL;
	}

	/**
	 * Returns whether to look in the cache for an instance of
	 * this space (not a particular instance).
	 *
	 * This should only be true for common Spaces, like myPage,
	 * where a user should only have one instance, unlike Editors,
	 * where a user could easily have multiple instances of an
	 * Editor open simultaneously.
	 *
	 * @return boolean true implies look in the cache for an
	 *                     instance of this space
	 */
	public boolean GetCachedNoID()
	{
		String strCached = GetControlArgument(ASConstants.CACHED);

		if (strCached == null)
		{
			strCached = GetControlArgument(ASConstants.CACHED_INPUT);
		}

		if (strCached == null)
		{
			return false;
		}
		else
		{
			return XPConvert.ToBoolean(strCached);
		}
	}

	/**
	 * Return the name of the IControl.
	 */
	public String GetControl()
	{
		String strControlName = GetControlArgument(ASConstants.CONTROL);

		if (null == strControlName)
		{
			strControlName = GetControlArgument(ASConstants.CONTROL_INPUT);
		}

		return strControlName;
	}

	/**
	 * Return the value of the specified control argument.
	 *
	 * @param String the name of the argument to get
	 */
	public String GetControlArgument(String strName)
	{
		String[] strValues = (String[]) m_htControlArguments.GetElement(strName);

		if (strValues != null)
		{
			return strValues[0];
		}

		return null;
	}

	/**
	 * Return a map of the IControl arguments in String name/String[] values pairs.
	 *
	 * @return XPHashtable the control arguments in String name/String[] values pairs
	 */
	public XPHashtable GetControlArguments()
	{
		return m_htControlArguments;
	}

	/**
	 * Return the name of the IDisplayPage.
	 */
	public String GetDisplayPage()
	{
		String strPageName = GetControlArgument(ASConstants.PAGE); 

		if (null == strPageName)
		{
			strPageName = GetControlArgument(ASConstants.PAGE_INPUT);
		}

		return strPageName; 
	}

	/**
	 * Returns the ID of the parent activity space of this space.
	 *
	 * @return String the ID of the parent of this space
	 */
	public String GetParentSpaceID()
	{
		String strParentSpaceID = GetControlArgument(ASConstants.PARENTSPACEID); 

		if (null == strParentSpaceID)
		{
			strParentSpaceID = GetControlArgument(ASConstants.PARENTSPACEID_INPUT);
		}

		return strParentSpaceID;
	}

	/**
	 * Returns the name of the parent activity space of this space.
	 *
	 * @return String the name of the parent of this space
	 */
	public String GetParentSpaceName()
	{
		String strParentSpaceName = GetControlArgument(ASConstants.PARENTSPACENAME); 

		if (null == strParentSpaceName)
		{
			strParentSpaceName = GetControlArgument(ASConstants.PARENTSPACENAME_INPUT);
		}

		return strParentSpaceName;
	}

	/**
	 * Returns a query string for the redirect containing all the control arguments.
	 *
	 * @return String the query string, excluding the question mark
	 */
	public String GetQueryString()
	{
		XPStringBuilder sb = new XPStringBuilder();
		IXPEnumerator enumerator = m_htControlArguments.GetKeys();

		// mj: iterate over all control arguments
		while (enumerator.MoveNext())
		{
			String strName     = XPHttpUtility.UrlEncode((String) enumerator.GetCurrent(),
				LinkConstants.UTF_8_ENCODING);
			String[] arrValues = (String[]) m_htControlArguments.GetElement(strName);

			// mj: check whether there are any values
			if ((arrValues != null) && (arrValues.length > 0))
			{
				for (int i = 0; i < arrValues.length; i++)
				{
					// mj: append the attribute name
					sb.Append(strName).Append('=');

					// mj: if this value is null, use an empty string
					if (arrValues[i] != null)
					{
						sb.Append(XPHttpUtility.UrlEncode(arrValues[i],
							LinkConstants.UTF_8_ENCODING));
					}

					sb.Append('&');
				}
			}
			else
			{
				// mj: no values; append an empty string
				sb.Append(strName).Append('=').Append('&');
			}
		}

		// mj: remove the trailing ampersand, if any
		if (sb.GetLength() > 0)
		{
			sb.SetLength(sb.GetLength() - 1);
		}

		return sb.ToString();
	}

	/**
	 * This method returns whether or not this redirect should be handled
	 * as an error bounce by the Interpreter.  If it is, and the user is
	 * not logged on, they will be redirected to the login page, and then
	 * be redirected back to this redirect after a succesful login.  If
	 * they are not logged in, they will see an error page and then be
	 * redirected back.
	 *
	 * @return boolean true implies is an error redirect
	 */
	public boolean GetIsErrorRedirect()
	{
		return m_bIsErrorRedirect;
	}

	/**
	 * This method returns whether or not this redirect should be handled
	 * as a true 302 redirect or an internal server redirect.
	 *
	 * 302 redirects will cause a roundtrip to the server, and the URL
	 * will show up in the browser window.  This will allow you to safely
	 * refresh the page.
	 *
	 * Server redirects don't incur a roundtrip to the server, but the
	 * new URL will not show up in the browser.  This can cause form
	 * data to be submitted twice if the user refreshes the page.
	 *
	 * In general, we should use internal server redirects to save time.
	 * However, destructive/constructive actions (saving an object, etc...)
	 * should use real 302 redirects.
	 *
	 * @return boolean true implies is a 302 redirect, and will cause a
	 * 	roundtrip from the browser to redirect.
	 */
	public boolean GetIsHTTPRedirect()
	{
		return m_bIsHTTPRedirect;
	}

	/**
	 * Return the ID of the AActivitySpace.
	 */
	public String GetSpaceID()
	{
		String strSpaceID = GetControlArgument(ASConstants.SPACEID); 

		if (null == strSpaceID)
		{
			strSpaceID = GetControlArgument(ASConstants.SPACEID_INPUT);
		}

		return strSpaceID;
	}

	/**
	 * Return the name of the AActivitySpace.
	 */
	public String GetSpaceName()
	{
		// This should probably be checked and enforced everywhere,
		// but I'm just going to do it here for now.
		if (!m_bLinkSet)
		{
			log.Error("Attempting to read from redirect before the space has been set.");
		}

		String strSpaceName = GetControlArgument(ASConstants.SPACENAME); 

		if (null == strSpaceName)
		{
			strSpaceName = GetControlArgument(ASConstants.SPACENAME_INPUT);
		}

		return strSpaceName;
	}

	/**
	 * Returns the Sub-space key.  If this is null or empty, this
	 * is not a sub-space.
	 *
	 * @return String the sub-space key
	 */
	public String GetSubSpaceKey()
	{
		String strSubSpaceKey = GetControlArgument(ASConstants.SUBSPACE); 

		if (null == strSubSpaceKey)
		{
			strSubSpaceKey = GetControlArgument(ASConstants.SUBSPACE_INPUT);
		}

		return strSubSpaceKey;
	}

	///////////////////////////////////////////////
	// Set Accessor Methods
	///////////////////////////////////////////////

	/**
	 * This method sets the sub space key.  If this is not called, or is empty,
	 * the space will not be a sub-space.  Otherwise it will be.
	 *
	 * This is null by default (meaning not a sub-space).
	 *
	 * @param String the sub space key
	 */
	public void SetSubSpaceKey(String sSubSpaceKey)
	{
		if (!"".equals(sSubSpaceKey))
		{
			AddControlArgument(ASConstants.SUBSPACE, sSubSpaceKey);
		}
	}

	/**
	 * Optionally set the name of an IControl to redirect to.
	 *
	 * @param String the name of the control to redirect to.
	 */
	public void SetControl(String control)
	{
		AddControlArgument(ASConstants.CONTROL, control);
	}

	/**
	 * Optionally set the name of an IDisplayPage to redirect to.
	 */
	public void SetDisplayPage(String page)
	{
		AddControlArgument(ASConstants.PAGE, page);
	}

	/**
	 * This method sets whether or not this redirect should be handled
	 * as an error bounce by the Interpreter.  If it is, and the user is
	 * not logged on, they will be redirected to the login page, and then
	 * be redirected back to this redirect after a succesful login.  If
	 * they are not logged in, they will see an error page and then be
	 * redirected back.
	 *
	 * @param boolean true implies is an error redirect
	 */
	public void SetIsErrorRedirect(boolean bRedirect)
	{
		m_bIsErrorRedirect = bRedirect;
	}

	/**
	 * This method sets whether or not this redirect should be handled
	 * as a true 302 redirect or an internal server redirect.
	 *
	 * 302 redirects will cause a roundtrip to the server, and the URL
	 * will show up in the browser window.  This will allow you to safely
	 * refresh the page.
	 *
	 * Server redirects don't incur a roundtrip to the server, but the
	 * new URL will not show up in the browser.  This can cause form
	 * data to be submitted twice if the user refreshes the page.
	 *
	 * In general, we should use internal server redirects to save time.
	 * However, destructive/constructive actions (saving an object, etc...)
	 * should use real 302 redirects.
	 *
	 * The default is internal server redirect.
	 *
	 * @param boolean true implies is a 302 redirect, and will cause a
	 * 	roundtrip from the browser to redirect.
	 */
	public void SetIsHTTPRedirect(boolean bHTTPRedirect)
	{
		m_bIsHTTPRedirect = bHTTPRedirect;
	}

	///////////////////////////////////////////////////////
	// Set link methods.  One of these (and only one)
	// must be called before getting the result of this URL
	///////////////////////////////////////////////////////

	/**
	 * This method creates a brand new AActivitySpace.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.ILink#SetLinkCreateNewSpace(String, AActivitySpace)
	 *
	 * @param String strSpace
	 * @param AActivitySpace parent
	 *
	 * @return boolean true implies success, false implies failure
	 */
	public boolean SetLinkCreateNewSpace(String strSpace, AActivitySpace parent)
	{
		if (m_bLinkSet)
		{
			
				log.Error("Attempting to use SetLinkCreateNewSpace method on a Redirect that has already been initialized.");
			

			return false;
		}

		if (parent == null)
		{
			//NOTE Developers should not get this error (i.e. pass in null activity space),
			//but on certain cases the framework (i.e. Interpreter) cause it,
			//(for example, on redirect after login), so the trace level is Info.
			
			log.Info("Attempting to use SetSpaceCreateNew method with a null parent AActivitySpace.");
			
		}
		else
		{
			AddControlArgument(ASConstants.PARENTSPACENAME, parent.GetName());
			AddControlArgument(ASConstants.PARENTSPACEID, parent.GetSpaceID());
			AddControlArgument(ASConstants.USERID_INPUT,
				LoginHelper.INSTANCE.GetUserSessionID(parent.GetUserSession()));
		}

		m_bLinkSet = true;

		if ((null == strSpace) || ("".equals(strSpace)))
		{
			log.Warn("Attempting to use SetSpaceCreateNew method with a null or empty space name.");
		}

		AddControlArgument(ASConstants.SPACENAME, strSpace);
		AddControlArgument(ASConstants.CACHED, XPConvert.ToString(false));

		return true;
	}

	/**
	 * This constructor takes a specific space and returns a link to that (including space ID)
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.ILink#SetLinkGetCachedSpace(AActivitySpace)
	 *
	 * @param AActivitySpace space
	 *
	 * @return boolean true implies success, false implies failure
	 */
	public boolean SetLinkGetCachedSpace(AActivitySpace space)
	{
		if (m_bLinkSet)
		{
			
				log.Error("Attempting to use SetLinkGetCachedSpace method on a Redirect that has already been initialized.");
			

			return false;
		}

		if (space == null)
		{
			
				log.Error("Attempting to use SetSpaceGetCached method with a null AActivitySpace.");
			

			return false;
		}

		m_bLinkSet = true;

		AddControlArgument(ASConstants.SPACENAME, space.GetName());
		AddControlArgument(ASConstants.SPACEID, space.GetSpaceID());

		AddControlArgument(ASConstants.PARENTSPACENAME, space.GetParentSpaceName());
		AddControlArgument(ASConstants.PARENTSPACEID, space.GetParentSpaceID());

		AddControlArgument(ASConstants.USERID_INPUT,
			LoginHelper.INSTANCE.GetUserSessionID(space.GetUserSession()));

		return true;
	}

	/**
	 * This method gets a named AActivitySpace from the cache if it is there, or
	 * creates a new one if it is not in the cache.
	 * This method should be used to get common AActivitySpaces such as MyPage and
	 * Community, not minor spaces like Tree and Editors.
	 * The SetSpaceGetCached(AActivitySpace) should be used instead of this one when
	 * you can.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.ILink#SetLinkGetSpaceIfCached(String, AActivitySpace)
	 *
	 * @param String strSpace
	 * @param AActivitySpace parent
	 *
	 * @return boolean true implies success, false implies failure
	 */
	public boolean SetLinkGetSpaceIfCached(String strSpace, AActivitySpace parent)
	{
		if (m_bLinkSet)
		{
			
				log.Error("Attempting to use SetLinkGetSpaceIfCached method on a Redirect that has already been initialized.");
			

			return false;
		}

		if (parent == null)
		{
			
				log.Warn("Attempting to use SetSpaceGetIfCached method with a null parent AActivitySpace.");
			
		}
		else
		{
			AddControlArgument(ASConstants.PARENTSPACENAME, parent.GetName());
			AddControlArgument(ASConstants.PARENTSPACEID, parent.GetSpaceID());
			AddControlArgument(ASConstants.USERID_INPUT,
				LoginHelper.INSTANCE.GetUserSessionID(parent.GetUserSession()));
		}

		m_bLinkSet = true;

		if ((null == strSpace) || ("".equals(strSpace)))
		{
			log.Warn("Attempting to use SetLinkGetSpaceIfCached method with a null or empty space name.");
		}

		AddControlArgument(ASConstants.SPACENAME, strSpace);
		AddControlArgument(ASConstants.CACHED, XPConvert.ToString(true));

		return true;
	}

	////////////////////////////////////////////
	// non-ILink SetLink methods (Redirect only)
	////////////////////////////////////////////

	/**
	 * This method sets the Redirect to do a true HTTP 302 redirect to an
	 * external URL.  This cannot be used to redirect to another Activity
	 * Space.  That should be done through the normal SetLink* methods.
	 *
	 * The URL parameter is assumed to be properly URL encoded, and this
	 * method will not do any encoding.  It is the responsibility of the
	 * method caller to properly encode the URL.
	 *
	 * @param String strExternalURL
	 *
	 * @return boolean true implies success, false implies failure
	 */
	public boolean SetLinkToExternalURL(String strExternalURL)
	{
		if (m_bLinkSet)
		{
			
				log.Error("Attempting to use SetLinkToExternalURL method on a Redirect that has already been initialized.");
			

			return false;
		}

		if ((strExternalURL == null) || ("".equals(strExternalURL)))
		{
			
				log.Warn("Attempting to use SetLinkToExternalURL method with a null or empty URL.");
			

			return false;
		}

		m_bLinkSet = true;

		m_bIsHTTPRedirect = true;
		m_bExternalURL = true;
		m_strExternalURL = strExternalURL;

		// No usage tracking user session ID in QS, because this doesn't point to
		// an ActivitySpace.

		return true;
	}

	/**
	 * Optionally remove single arguments for the IControl.
	 */
	public void RemoveControlArgument(String name)
	{
		if (name != null)
		{
			m_htControlArguments.RemoveElement(name);
		}
	}
	
	/**
	 * Sets whether or not to strip the parent space arguments from the URL.
	 *  
	 * @return
	 */
	public void SetStripParentSpace(boolean bStrip)
	{
	    m_bStripParentSpace = bStrip;
	}
	
	/**
	 * Gets whether or not to strip the parent space arguments from the URL.
	 *  
	 * @return
	 */
	public boolean GetStripParentSpace()
	{
	    return m_bStripParentSpace;
	}
}

