package com.plumtree.uiinfrastructure.activityspace;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.io.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openkernel.http.*;
import com.plumtree.openlog.*;
import com.plumtree.server.impl.core.*;
import com.plumtree.uiinfrastructure.activityspace.utils.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This object wraps HTMLPages and binary data for output to the browser.
 * There will be methods for setting HTTP Headers for the browser.
 *
 * This implementation of the IWebData interface is intended for use by the Interpreter.
 *
 * @author Don Hayler
 * @author DaniilK
 */
public class WebData implements IWebData
{ 
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.activityspace.WebData");

	/** Data to be output (only one can be non-null at a time) */
	private byte[] m_arrBinaryBody = null;
	private HTMLElement m_DisplayElement = null;
	private String m_strDisplayString = null;

	// True by default.
	private boolean m_bIsHTMLData = true;

	/** Private variables */
	private IXPResponse m_xpResponse = null;
	private String m_strImageBaseURL  = null;
	private String m_strLang = null;

	/** All header keys in this table must be lower case to ensure there are no duplicates. */
	private XPHashtable m_htHeaders = null;

	private boolean m_bSuppressAutomaticHeaders = false;
	// Default OK status
	private int m_iStatus					    = 200;
	private boolean m_bRequireNonEmptyBody = true;
	private int m_nSecurityMode;
	private boolean m_bIsSecure;
	private boolean m_bSendRedirect = false;
	private BrowserType m_ptBrowserType;
	private boolean m_bDataSent = false;
	private int m_iGatewayDownloadStreamID = -1;

	/**
	 * Creates a new WebData object.
	 *
	 * This should only be created by the Interpreter.
	 * 
	 * @param _response IXPResponse assigned to the member variable m_xpResponse
	 * @param _strImageBaseURL String assigned to the member variable m_strImageBaseURL
	 * @param _nSecurityMode int assigned to the member variable _nSecurityMode
	 * @param _isSecure boolean assigned to the member variable m_bIsSecure
	 * @param _ptBrowserType BrowserType assigned to the member variable m_ptBrowserType
	 */
	public WebData(IXPResponse _response,
	        String _strImageBaseURL,
	        int _nSecurityMode,
	        boolean _isSecure,
			BrowserType _ptBrowserType)
	{
		m_htHeaders			= new XPHashtable();
		m_xpResponse		= _response;
		m_strImageBaseURL	= _strImageBaseURL;
		m_nSecurityMode		= _nSecurityMode;
		m_bIsSecure			= _isSecure;
		m_ptBrowserType		= _ptBrowserType;
		
		if (m_xpResponse == null)
		{
			
				log.Error("Response not set in constructor.");
			
		}
	}

    public void setResponse(IXPResponse _response) {
        m_xpResponse = _response;
    }
    
	/**
	 * This method determines if WebData object will throw an exception in case of 
	 * sending an empty body of not.
	 * 
	 * @param bRequireNonEmptyBody - TRUE if you want WebData object to throw an exception
	 * 								 in case of sending an emptry body, FALSE otherwise.
	 *  
	 * @return previous value of the flag
	 */
	public boolean RequireNonEmptyBody(boolean bRequireNonEmptyBody)
	{
		boolean bPrevValue = m_bRequireNonEmptyBody; 
		m_bRequireNonEmptyBody = bRequireNonEmptyBody;
		return bPrevValue;
	}
	
	/**
	 * JF- Helper method to set the content language header from the locale.
	 * 
	 * @param _sLocale (e.g.: en-US)
	 */
	public void SetContentLanguage(String _sLocale)
	{
		try
		{
			XPLocale xpLocale = new XPLocale(_sLocale);
			m_strLang = xpLocale.GetLanguage();
			m_htHeaders.PutElement(XPStringUtility.ToLowerCase(HTTPHeaderConstants.CONTENT_LANGUAGE),
				new HTTPHeader(HTTPHeaderConstants.CONTENT_LANGUAGE, m_strLang));
		}
		catch (Exception e)
		{
			
				log.Error(e, "Unable to get the current language from the current locale: " + _sLocale 
					+ ". The content language (Content-Language) will not be set in the HTTP header");
			
		}
	}
	
	/**
	 * Call this method to add a cookie to the response.
	 *
	 * @param XPCookie Pass an XPCookie
	 */
	public void AddCookie(XPCookie c)
	{
		if ( c.GetName().equalsIgnoreCase( "PT-Stream-ID" ) )
		{
			log.Debug( "AddCookie() - Trying to set special cookie indicating streaming " 
						+ "gateway download. The value of the cookie is [{0}].",
						c.GetValue() );
			m_iGatewayDownloadStreamID = XPConvert.ToInteger( c.GetValue() );
		}
		else
		{
			this.m_xpResponse.AddCookie(c);
		}
	}

	/**
	 * This method is used to clear all of the underlying data from this
	 * object to ensure that no memory is "leaked" if a Display Page
	 * mistakenly maintains a reference to this object.
	 */
	public void ClearAllData()
	{
		// Clear data from this object.
		m_xpResponse		   = null;
		m_arrBinaryBody		   = null;
		m_htHeaders			   = new XPHashtable();
		m_DisplayElement				   = null;
		m_strDisplayString     = null;
	}

	/**
	 * This clears the data to be output (text or binary) so that
	 * this object can be used again.  Some of this data can still
	 * be transmitted if it has already been written to the response
	 * by the time this method is called.
	 */
	public void ClearOutputData()
	{
		// Clear data from this object.
		m_arrBinaryBody		   = null;
		m_htHeaders			   = new XPHashtable();
		m_DisplayElement				   = null;
		m_strDisplayString     = null;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IWebData#FlushData()
	 *
	 * This method flushes all current data in the HTMLPage to the browser.
	 * This should only be called when the banner has been fully added to the
	 * HTMLPage.
	 *
	 * This method requires SetData to have been called first.
	 *
	 * This means that the HTMLHead element must be completed and that all
	 * HTMLObjects (like tables) that are currently in the HTMLBody element
	 * need to be complete.  Data will not be able to be added to them after
	 * flushing.  Data will be able to be added to the HTMLBody only.
	 */
	public void FlushData()
	{
		if ((m_DisplayElement == null) && (m_arrBinaryBody == null) &&
				(m_strDisplayString == null) &&
				ResponseRequiresBody(m_iStatus))
		{
			
				log.Error("Attempted to flush data before setting it.");
			

			return;
		}

		// Flush headers
		// flush HTMLPage
	}

	/**
	 * This method tells you whether or not the data on this object is binary.
	 * If it is binary, you can't add text data after the binary data.
	 *
	 * @return boolean true implies binary data
	 */
	public boolean IsBinaryData()
	{
		return m_arrBinaryBody != null;
	}

	/**
	 * This method tells you whether or not the data on this object is HTML text.
	 *
	 * @return boolean true implies HTML text data
	 */
	public boolean IsHTMLData()
	{
		if (IsBinaryData())
		{
			// Binary data cannot be HTML data.
			return false;
		}

		return m_bIsHTMLData;
	}

	/**
	 * This method tells you whether or not the data on this object is already 
	 * set using either of the SetData() methods.
	 *
	 * @return boolean true implies data set
	 */
	public boolean IsDataSet()
	{
		return ((m_DisplayElement != null) || (m_arrBinaryBody != null) ||
			(m_strDisplayString != null) || m_bSendRedirect);
	}

	/**
	 * This method tells you whether or not the data on this object has already 
	 * been sent using the OutputData() method.
	 *
	 * @return boolean true implies data sent
	 */
	public boolean HasSentData()
	{
		return m_bDataSent;
	}

	/**
	 * This method marks the data as being sent so that this page cannot
	 * be redisplayed.
	 */
	public void SetDataSent()
	{
		m_bDataSent = true;
	}

	/**
	 * This method outputs the data to the browser.  This should only be called
	 * by the Interpreter after data has been set on this object by the IDisplayPage.
	 */
	public void OutputData()
	{
		// if we do not have a body and we really need it - well, lets complain
		if ((m_DisplayElement == null) && (m_arrBinaryBody == null) &&
				(m_strDisplayString == null) &&
				ResponseRequiresBody(m_iStatus) && m_bRequireNonEmptyBody)
		{
			throw new XPException("Attempted to output data before setting it.");
		}

		// if we do not have response object, that is a problem - lets complain
		if (m_xpResponse == null)
		{
			throw new XPException("Response not set.");
		}

		if (m_bDataSent)
		{
			throw new XPException("Attempted to re-send data.");
		}

		// set status on the response
		log.Debug("Setting HTTP status code on response to [" + m_iStatus + "]");

		m_xpResponse.SetStatus(m_iStatus);

		SetDataSent();

		// set body (which sets the headers)
		if (null != m_arrBinaryBody)
		{
			SendBinaryData();
		}
		else
		{
			SendTextData();
		}
	}

	/**
	 * Sends a true 302 redirect immediatly.
	 *
	 * @param _strLocation Where to go
	 */
	public void SendRedirect(String _strLocation)
	{
		SetHeader("LOCATION", _strLocation);
		SetStatus(302);
		m_bSendRedirect = true;
		FlushData();
	}

	/* (non-Javadoc)
	 * @see com.plumtree.uiinfrastructure.activityspace.IWebData#SetData(com.plumtree.xpshared.htmlelements.HTMLElement)
	 */
	public void SetData(HTMLElement element)
	{
	    m_DisplayElement = element;
	    
		if ((m_strDisplayString != null) || (m_arrBinaryBody != null))
		{			
			log.Error("Only one SetData method can be called on WebData object.");		
		}

		m_DisplayElement.SetImgBaseUrl(m_strImageBaseURL);
		m_DisplayElement.SetIsNetscape(m_ptBrowserType.IsNetscape());
		
		if (null != m_strLang)
		{
		    m_DisplayElement.SetLangGlobal(m_strLang);
		}
	}
	
	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IWebData#SetData()
	 *
	 * This method sets the string for output to the browser.
	 *
	 * Only one SetData method should be called.
	 *
	 * @param String text to output to the browser
	 */
	public void SetData(String strData)
	{
		if ((m_DisplayElement != null) || (m_arrBinaryBody != null))
		{
			
				log.Error("Only one SetData method can be called on WebData object.");
			
		}

		m_strDisplayString = strData;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IWebData#SetData()
	 *
	 * This method sets the HTMLPage for output to the browser.
	 *
	 * Only one SetData method should be called.
	 *
	 * @param HTMLPage page to output to the browser
	 */
	public void SetData(byte[] arrBodyBinary)
	{
		if ((m_strDisplayString != null) || (m_DisplayElement != null))
		{
			
				log.Error("Only one SetData method can be called on WebData object.");
			
		}

		m_arrBinaryBody = arrBodyBinary;
	}

	/* (non-Javadoc)
	 * @see com.plumtree.uiinfrastructure.activityspace.IWebData#SetIsHTMLText(boolean)
	 */
	public void SetIsHTMLText(boolean _bIsHTMLText)
	{
		m_bIsHTMLData = _bIsHTMLText;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IWebData#SetHeader(String, String)
	 *
	 * This method sets a HTTP header to a specified value
	 *
	 * @param String header name
	 *
	 * @param String header value
	 */
	public void SetHeader(String strHeaderName, String strHeaderValue)
	{
		m_htHeaders.PutElement(XPStringUtility.ToLowerCase(strHeaderName),
			new HTTPHeader(strHeaderName, strHeaderValue));
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IWebData#SetDateHeader(String, XPDateTime)
	 */
	public void SetDateHeader(String strHeaderName, XPDateTime xpDate)
	{
		SetDateHeader(strHeaderName, xpDate.GetTimeInMilliSeconds());
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IWebData#SetDateHeader(String, long)
	 */
	public void SetDateHeader(String strHeaderName, long iDate)
	{
		m_htHeaders.PutElement(XPStringUtility.ToLowerCase(strHeaderName),
			new HTTPHeader(strHeaderName, iDate));
	}

	/**
	 *
	 *
	 * @param iStatus
	 */
	public void SetStatus(int iStatus)
	{
		m_iStatus = iStatus;
	}

	/**
	 * This method turns on or off automatic HTTP headers
	 * (such as the automatic expires header set to now).
	 *
	 * Default is to add automatic HTTP headers.
	 *
	 * @param boolean true implies suppress automatic headers
	 *                     false implies show automatic headers
	 */
	public void SetSuppressAutomaticHeaders(boolean bSuppress)
	{
		m_bSuppressAutomaticHeaders = bSuppress;
	}

	/**
	 *
	 *
	 * @param iStatus
	 *
	 * @return
	 */
	private boolean ResponseRequiresBody(int iStatus)
	{
		return ((iStatus > 199) && (iStatus < 300)); // iStatus is 2xx
	}

	/**
	 *
	 */
	private void SendBinaryData()
	{
		// WE WANT TO SEND HEADERS EVEN IF THERE IS NO BODY!
		SetHTTPHeadersOnResponse(true);
		// On .NET we need to call Flush() to ensure that OnPreSendRequestHeaders event
		// is triggered. Since this call does not really do anything on Java, i put it 
		// here regardless of what platform we are running on.
		m_xpResponse.Flush();		
		log.Debug( "SendBinaryData() - Set headers on response." );

		if ( m_iGatewayDownloadStreamID > -1 )
		{
			SendStreamData();
		}
		else
		{		
			// In order for the binary gateway to work, we need to call Write()
			// method on the response object, even if m_arrBinaryBody array is 
			// empty (length = 0). So, here we only check for m_arrBinaryBody being
			// NULL, which actually should never happen.
			if (null == m_arrBinaryBody)
			{
				return;
			}
	
			/* Get output stream */
			XPOutputStream xpStream = m_xpResponse.GetOutputStream();
			log.Debug( "SendBinaryData() - Got output stream." );
	
			/* write out binary body into the output stream */
			try
			{
				if (null != m_arrBinaryBody && m_arrBinaryBody.length != 0) {
					xpStream.Write(m_arrBinaryBody);			
					xpStream.Flush();
				}
				m_xpResponse.Flush();			
			}
			catch (XPIOException e)
			{
				log.Warn("There was an error writing data to the socket. ");
				log.Warn(e, "It usually happens when the browser window was closed before " +
						"all of the data was sent to the client.");
			}		
		}

		return;
	}

	private void SendStreamData()
	{
		long lBytesSent = 0;
		long lStartTime = XPSystem.GetCurrentTimeMilliSec();
		long lPerfTime = log.PerformanceBegin();
		IOKHttpInputStream streamRemoteServerToPortal = 
			(IOKHttpInputStream)Global.GATEWAY_STREAM_MANAGER.GetAndClear( m_iGatewayDownloadStreamID );
		XPOutputStream streamPortalToBrowser = m_xpResponse.GetOutputStream();
		
		try
		{
			while ( streamRemoteServerToPortal.IsOpen() )
			{
				byte[] barrData = streamRemoteServerToPortal.Read( 5000 );
				
				try
				{					
					if ( null != barrData )
					{
						log.Debug( "SendStreamData(): Sending [{0}] bytes to the browser.", 
									barrData.length );
						
						lBytesSent += barrData.length;
						streamPortalToBrowser.Write( barrData );
						streamPortalToBrowser.Flush();
						m_xpResponse.Flush();
					}
					else
					{
						if ( streamRemoteServerToPortal.IsOpen() )
						{
							log.Warn( "SendStreamData(): Could not read data from the remote server for 5 seconds." );
						}
						else
						{
							log.Info( "SendStreamData(): Stream was closed by the sending side." );
						}
						return;
					}
				}
				catch (Exception e)
				{
					log.Warn( e, "SendStreamData(): There was an exception writing to the output stream. " +
						"Reading rest of data from input stream to ensure it closes correctly.");

					// NOTE: HACK: Due to bug#67622 in OpenHTTP, we need to continue reading
					// all of the data out of the stream before closing it, otherwise
					// it will leak the socket.  This happens if the browser window is
					// closed while we are in the middle of streaming.  This code should be
					// removed when the OpenHTTP bug is fixed.
					while(barrData != null)
					{
						barrData = streamRemoteServerToPortal.Read(5000);
					}

					return;
				}
			}
		}
		catch(Exception e)
		{
			log.Warn( e, "SendStreamData(): There was an exception trying to read data from the remote server.");
		}
		finally
		{
			log.PerformanceEnd( "SendStreamData(): Sent " + lBytesSent + " bytes in " 
					+ (XPSystem.GetCurrentTimeMilliSec() - lStartTime ) + "ms.", lPerfTime );
			streamRemoteServerToPortal.Close();
		}		
	}
	
	/**
	 *
	 */
	private void SendTextData()
	{
		/* WE WANT TO SEND HEADERS EVEN IF THERE IS NOT BODY! */
		SetHTTPHeadersOnResponse(false);

		if (null != m_DisplayElement)
		{
			try
			{
				m_DisplayElement.Display(m_xpResponse);
			}
			catch (HTMLException he)
			{
				log.Error(he, "Error displaying page.");
				throw new XPException(he);
			}
		}
		else if (null != m_strDisplayString)
		{
			// Output as bytes as the rest of the code does (cannot mix bytes and String;
			// XPResponse will throw an exception.
			// m_xpResponse.Write(m_strDisplayString);
			m_xpResponse.GetOutputStream().Write(HTMLElement.ToByteArray(m_strDisplayString)); 
		}

		return; /* do nothing if none of the cases above succeeded */
	}

	/**
	 *
	 *
	 * @param strContentTypeArg
	 */
	private void SetContentTypeIfNotAlreadySpecified(String strContentTypeArg)
	{
		/* Get content type header, since it's a binary data it might not be text/html */
		HTTPHeader header = (HTTPHeader) m_htHeaders.GetElement(XPStringUtility.ToLowerCase(HTTPHeaderConstants.CONTENT_TYPE));

		/* if the header was not set specifically, we use the default one "text/html" */
		if (null == header)
		{
			
				log.Debug("Setting HTTP Content-Type header to [" + strContentTypeArg + "]");
			
			m_xpResponse.SetContentType(strContentTypeArg);
		}
		else
		{
			String strContentType = header.m_strValue;
			
				log.Debug("Setting HTTP Content-Type header to [" + strContentType + "]");
			
			m_xpResponse.SetContentType(strContentType);
		}
	}

	/**
	 * Sets various headers on the response for both binary and text data. This method
	 * always sets the Content-Type header. If m_bSuppressAutomaticHeaders is false it
	 * sets the following headers: Expires, Cache-control, Last-Modified and Pragma (
	 * however Cache-control, Last-Modified and Pragma are not set if the browser is IE 
	 * and either the security mode is 3 or the session is secure.) It also sets whatever
	 * headers are present in m_htHeaders.
	 * 
	 * @param bSendingBinaryData
	 *
	 * @return
	 */
	private int SetHTTPHeadersOnResponse(boolean bSendingBinaryData)
	{
		int iHeadersAdded = 0;

		/* Set content type first, since it requires a special call on the interface */
		if (bSendingBinaryData)
		{
			SetContentTypeIfNotAlreadySpecified("text/plain");
		}
		else
		{
			SetContentTypeIfNotAlreadySpecified("text/html");
		}
		
		// These headers should only be sent for portal pages and never for gateway pages.
		//		-- MOOSE
		if (!m_bSuppressAutomaticHeaders)
		{
			// JF- Set "Expires" to the day before (this is to avoid any problems in case the client 
			// and the server don't have exactly the same time)
			log.Debug("Setting HTTP Expires header to [" + (XPSystem.GetCurrentTimeMilliSec() - 24*60*60*1000) + "]"); 
			m_xpResponse.SetDateHeader(HTTPHeaderConstants.EXPIRES, XPSystem.GetCurrentTimeMilliSec() - 24*60*60*1000);

			if(!m_ptBrowserType.IsNetscape4())
			{
				// Set UTF-8 internationalization header
				SetContentTypeIfNotAlreadySpecified("text/html; charset=utf-8");
			} else {
				// If it's not netscape 4.x, use utf8
				// If it is netscape 4.x, use 8859_1, because NS4
				// can't handle utf-8 chars
				SetContentTypeIfNotAlreadySpecified("text/html; charset=8859_1");
			}
			
			// these headers s/b set unless the the browser type is ie & the session is secure or
			// the security mode is 3. this is to resolve ie's problem with downloading binaries
			// with these headers in https (bug 39333)
			if (bSendingBinaryData && m_ptBrowserType.IsIE() && ((m_bIsSecure) || (m_nSecurityMode == 3)))
			{
                log.Debug("Not Setting default HTTP headers");
			}
			else
			{					
			    log.Debug("Setting HTTP headers \r\n"
						  + "["+HTTPHeaderConstants.CACHE_CONTROL+": no-store, no-cache, must-revalidate, max-age=0, post-check=0, pre-check=0]\r\n"
						  + "["+HTTPHeaderConstants.PRAGMA+": no-cache]\r\n");
			    
			    m_xpResponse.AddHeader(HTTPHeaderConstants.CACHE_CONTROL, "no-store, no-cache, must-revalidate, max-age=0, post-check=0, pre-check=0");
				m_xpResponse.AddHeader(HTTPHeaderConstants.PRAGMA, "no-cache");
				m_xpResponse.SetDateHeader(HTTPHeaderConstants.LAST_MODIFIED, XPSystem.GetCurrentTimeMilliSec());
			}
		}
		/* Set the rest of the headers */
		for (IXPEnumerator xpEnumHeaders = m_htHeaders.GetElements();
				xpEnumHeaders.MoveNext();)
		{
			HTTPHeader header = (HTTPHeader) xpEnumHeaders.GetCurrent();

			if (null != header)
			{				
				log.Debug("Setting HTTP header [" + header.m_strName + ": " + header.m_strValue + "]");

				header.SetHeader(m_xpResponse);
				iHeadersAdded += 1;
			}
		}

		return iHeadersAdded;
	}

	/** Internal helper class that represents HTTP Headers */
	static class HTTPHeader
	{
		/**  */
		public String m_strName;

		/**  */
		public String m_strValue;

		public long m_nValue;

		public boolean m_bDateHeader = false;

		/**
		 * Creates a new HTTPHeader object.
		 *
		 * @param name
		 * @param value
		 */
		public HTTPHeader(String name, String value)
		{
			m_strName		= name;
			m_strValue		= value;
			m_bDateHeader	= false;
		}

		/**
		 * Creates a new HTTPHeader object for a date.
		 *
		 * @param name
		 * @param value
		 */
		public HTTPHeader(String name, long value)
		{
			m_strName		= name;
			m_nValue		= value;
			m_bDateHeader	= true;
		}
		
		/**
		 * Adds the header to the response.
		 * 
		 * @param _response
		 */
		public void SetHeader(IXPResponse _response)
		{
			if (m_bDateHeader)
			{
				_response.SetDateHeader(m_strName, m_nValue);
			} 
			else
			{
				_response.AddHeader(m_strName, m_strValue);
			}
		}
	}
}