/*
 * Created on Dec 22, 2003
 */
package com.plumtree.uiinfrastructure.activityspace.utils;

import com.plumtree.openlog.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;


/**
 * Utility class for client browser type information.
 * 
 * @author DavidP
 * @author NaveenP
 */
public class BrowserType 
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.activityspace.utils.BrowserType");

	/** The browser name specified in the user agent string. */
    private String m_strBrowserName = "";

    /**
     * The browser version specified in the user agent string.  If we
     * can't parse the version just assume an old browser.
     * 
     * NOTE: This only includes the major and minor versions of the
     * browser.  I.E. "1.3.1" will be stored as "1.3".
     */
    private float m_nBrowserVersion = 1.0f;

    /**
     * The gecko version specified in a Netscape6/Mozilla user agent string.
     */
    private int m_nGeckoVersion = 0;

    /**
     * The browser platform specified in the user agent string.
     */
    private String m_strBrowserPlatform = "unknown";

    /** Constants used by this class. */
    public static final String MSIE 		= "MSIE";
    public static final String OPERA 		= "Opera";
    public static final String MOZILLA 		= "Mozilla";
    public static final String NETSCAPE 	= "Netscape";
    public static final String FIREFOX 		= "Firefox";
    public static final String SAFARI 		= "Safari";
    public static final String WINDOWS 		= "Windows";
    public static final String UNIX 		= "Unix";
    public static final String SUNOS 		= "SunOS";
    public static final String LINUX 		= "Linux";
    public static final String MACINTOSH 	= "Macintosh";
    public static final String UNKNOWN 		= "unknown";


	/**
     * Constructor
     */
	public BrowserType( IXPRequest request )
    {
	    this(request.GetHeader("User-Agent"));
    }
	
	/**
     * Constructor
     */
	public BrowserType( String userAgentString )
    {
    	try
    	{
		    if (null == userAgentString) {
		        m_strBrowserName = UNKNOWN;
		        log.Warn("Unable to find the browser's name or version number from the User-Agent header. Dropping to lowest supported browser type.");
		        return;
		    }
	        int versionStartIndex = userAgentString.indexOf("/");
	        int versionEndIndex = userAgentString.indexOf(" ");
	
	        // Get the browser name.
	        try {
	            m_strBrowserName = userAgentString.substring( 0, versionStartIndex );
	        } catch (Exception e) {
	            m_strBrowserName = UNKNOWN;
	        }
	
			// Get the browser version.
	        try
	        {
	            // Not all user agents will have a space in the reported
	            // string.
	            String agentSubstring = null;
	            if ( versionEndIndex < 0 )
	                agentSubstring = userAgentString.substring( versionStartIndex+1 );
	            else
	                agentSubstring = userAgentString.substring( versionStartIndex+1, versionEndIndex );
	
				m_nBrowserVersion = GetVersionNumber(agentSubstring);
	        }
	        catch (Exception e)
	        {
	            // Just use the default value.
	        }
	
	        // MSIE lies about its name.  Of course...
	        if ( userAgentString.indexOf(MSIE) != -1 )
	        {
	            // Ex: Mozilla/4.0 (compatible; MSIE 5.0; Windows NT; DigExt)
	            versionStartIndex = (userAgentString.indexOf(MSIE) +
	                                 MSIE.length() + 1);
	            versionEndIndex = userAgentString.indexOf( ";",
	                                                       versionStartIndex );
	
	            m_strBrowserName = MSIE;
	            try
	            {
	            	m_nBrowserVersion = GetVersionNumber( userAgentString.substring(versionStartIndex, versionEndIndex) );
	            }
	            catch (Exception e)
	            {
	                // Just use the default value.
	            }
	        }
	
	        // Netscape 6+ and Mozilla have a host of complexities, and
			// browser version isn't the whole story here...
			// A Netscape 6+ user-agent string looks something like this:
			//
			// Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.0.1) Gecko/20020823 Netscape/7.0
			//
			// Netscape 6 & Netscape 7 are both classified as Mozilla/5.0
			// But technically the codebase underneath them is the Gecko build number.
			// In this case, Gecko/20020823 means that this was built off of code from
			// the Gecko project on 8/23/2002.
			//
			// Netscape versions    Gecko versions    Mozilla versions
			// -----------------    --------------    ----------------
			// 6.0                  20001108          0.6
			// 6.0.1                20010131
			// 6.1                  20010726          0.9.1
			// 6.2                  20011019          0.9.4
			// 6.2.1                20011128          0.9.4.1
			// 6.2.2                20020314          0.9.4.2
			// 6.2.3                20020508          0.9.4.3
			// 7.0 (Mac)            20020730          1.0.1 (fictional, used by Netscape 7.0 & 7.1 Mac browsers)
			// 7.0 (Win)            20020823
			//                      20020826          1.1
			// 7.1                  20030624		  1.4
			//
			// Patrick Corcoran <patrickc@plumtree.com>   12/19/2003
			if ( userAgentString.indexOf(NETSCAPE) != -1 )
			{
				m_strBrowserName = NETSCAPE;
				// Netscape example: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.0.1) Gecko/20020823 Netscape/7.0
	            versionStartIndex = (userAgentString.indexOf(NETSCAPE) +
	            		NETSCAPE.length() + 1);
	            versionEndIndex = userAgentString.indexOf( " ", versionStartIndex );
	
	            if (versionEndIndex == -1) { versionEndIndex = userAgentString.length(); }
	
	            try
	            {
	            	m_nBrowserVersion = GetVersionNumber( userAgentString.substring(versionStartIndex, versionEndIndex) );
	            }
	            catch (Exception e)
	            {
	                // Just use the default value.
	            }
			}
	
			if ( userAgentString.indexOf(FIREFOX) != -1 )
			{
				m_strBrowserName = FIREFOX;
				// Firefox example: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.7.5) Gecko/20041107 Firefox/1.0
	            versionStartIndex = (userAgentString.indexOf(FIREFOX) +
	            		FIREFOX.length() + 1);
	            versionEndIndex = userAgentString.indexOf( " ", versionStartIndex );
	            
	            if (versionEndIndex == -1) { versionEndIndex = userAgentString.length(); }
	            
	            try
	            {
	            	m_nBrowserVersion = GetVersionNumber( userAgentString.substring(versionStartIndex, versionEndIndex) );
	            }
	            catch (Exception e)
	            {
	                // Just use the default value.
	            }
			}
	
	        // Opera isn't completely honest, either...
	        // Modificaton by Chris Mospaw <mospaw@polk-county.com>
	        if ( userAgentString.indexOf(OPERA) != -1 )
	        {
	            //Ex: Mozilla/4.0 (Windows NT 4.0;US) Opera 3.61  [en]
	            versionStartIndex = (userAgentString.indexOf(OPERA) +
	                                 OPERA.length() + 1);
	            versionEndIndex = userAgentString.indexOf( " ",
	                                                       versionStartIndex );
	
	            m_strBrowserName = OPERA;
	            try
	            {
	            	m_nBrowserVersion = GetVersionNumber( userAgentString.substring(versionStartIndex, versionEndIndex) );
	            }
	            catch (Exception e)
	            {
	                // Just use the default value.
	            }
	        }
	
	        // Safari
	        if ( userAgentString.indexOf(SAFARI) != -1 )
	        {
	        	// Ex: 
	        	// Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-us) AppleWebKit/XX (KHTML, like Gecko) Safari/YY
	        	// Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/312.5.1 (KHTML, like Gecko) Safari/312.3.1
	        	// NOTE: Safari can have non-float version numbers such as 312.3.1
	            versionStartIndex = (userAgentString.indexOf(SAFARI) +
	                                 SAFARI.length() + 1);       	
	            m_strBrowserName = SAFARI;	
	
				try
				{
					float browserBuild = GetVersionNumber( userAgentString.substring(versionStartIndex) );
				
					if (browserBuild < 100) m_nBrowserVersion = 1.0f;
					else if (browserBuild >= 100) m_nBrowserVersion = 1.1f;
				} catch (Exception e)
				{
					// Just use the default value
				}
	        }
	
	        // Try to figure out what platform.
	        if ( (userAgentString.indexOf("Windows") != -1) ||
	             (userAgentString.indexOf("WinNT") != -1) ||
	             (userAgentString.indexOf("Win98") != -1) ||
	             (userAgentString.indexOf("Win95") != -1) )
	        {
	        	m_strBrowserPlatform = WINDOWS;
	        }
	
	        if ( userAgentString.indexOf("Mac") != -1 )
	        {
	        	m_strBrowserPlatform = MACINTOSH;
	        }
	
	        if ( userAgentString.indexOf("X11") != -1 )
	        {
	        	if ( userAgentString.indexOf(SUNOS) != -1 )
	            {
	            	m_strBrowserPlatform =  SUNOS;
	            } else if ( userAgentString.indexOf(LINUX) != -1 )
	            {
	            	m_strBrowserPlatform =  LINUX;
	            } else {
	            	m_strBrowserPlatform =  UNIX;
	            }
	        }
    	} catch (Exception e)
    	{
    		log.Error(e, "Unable to parse browser information to determine " +
    			"browser type, version, and platform.");

			throw new XPException("Unable to parse browser information to determine " +
				"browser type, version, and platform.", e);
    	}
    }

	/**
	 * This helper function attempts to convert the user agent
	 * string into a float major/minor version number.
	 * 
	 * @param _strAgent The browser user agent string.
	 * @return The major/minor version number as a float.
	 * @throws XPFormatException If the agent string cannot be
	 * converted to a float.
	 */
	protected float GetVersionNumber(String _strAgent)
	{
		int nFirstDecimal = XPStringUtility.IndexOf(_strAgent, '.');
		int nLastDecimal = XPStringUtility.LastIndexOf(_strAgent, '.');

		// Some agent substrings may have more than just major and minor version numbers.
		// For instance, Safari 1.3.1
		if (nFirstDecimal != nLastDecimal)
		{
			// Remove the last decimal and try again
			return GetVersionNumber(XPStringUtility.Substring(_strAgent, 0, nLastDecimal));
		}

		return XPConvert.ToFloat( _strAgent );
	}

	/**
	 * @return true if the current client browser can handle javascript, else false
	 */
	public boolean IsECMAScriptOK()
	{
		if (IsIE() &&  (GetBrowserVersion() >= 5.0f)) {
			return true;
		} else if ((IsFirefox() || IsMozilla()) && (GetBrowserVersion() >= 1.0f)) {
			return true;
		} else if (IsNetscape() && (GetBrowserVersion() >= 7.0f)) {
			return true;
		} else if (IsOpera() && (GetBrowserVersion() >= 7.0f)) {
			return true;
		}
		return  false;
	}

	/**
	 * @return true if the current client browser can handle javascript, else false
	 */
	public boolean IsJSPortalMenuCompliant()
	{
		if ((IsNetscape() && (GetBrowserVersion() >= 4.0f)) 
				|| (IsIE() && !IsMac()) || IsFirefox() || IsMozilla() || (IsSafari() && GetBrowserVersion() >= 2.0f)) {
			return true;
		} else {
			return  false;
		}
	}

	/**
	 * @return true if the current client browser can handle javascript, else false
	 */
	public boolean IsJSPortletCompliant()
	{
		if (IsIE() || IsNetscape() || IsFirefox() || IsMozilla()) {
			return true;
		} else {
			return  false;
		}
	}

	/**
	 * @return true if the current client browser can handle javascript, else false
	 */
	public boolean IsDragDropLayoutCompliant()
	{
		if ( (IsIE()|| IsNetscape4()) && !IsMac() ) {
			return true;
		} else {
			return  false;
		}
	}

	/**
	 * @return true if the current client browser is an Internet Explorer browser, else false
	 */
	public boolean IsIE()
	{
		return  MSIE == m_strBrowserName;
	}

	/**
	 * @return true if the current client browser is a Safari browser, else false
	 */
	public boolean IsSafari()
	{
		return SAFARI == m_strBrowserName;
	}

	/**
	 * @return true if the current client browser is a Firefox browser, else false
	 */
	public boolean IsFirefox()
	{
		return FIREFOX == m_strBrowserName;
	}

	/**
	 * @return true if the current client browser is a Mozilla browser, else false
	 */
	public boolean IsMozilla()
	{
		return MOZILLA == m_strBrowserName;
	}

	/**
	 * @return true if the current client browser is a Mozilla browser, else false
	 */
	public boolean IsOpera()
	{
		return OPERA == m_strBrowserName;
	}

	/**
	 * @return true if the current client browser is a Netscape browser, else false
	 */
	public boolean IsNetscape()
	{
		return NETSCAPE == m_strBrowserName;
	}

	/**
	 * @return true if the current client browser is a Netscape 4.x browser, else false
	 */
	public boolean IsNetscape4()
	{
		return IsMozilla() && (m_nBrowserVersion < 5.0);
	}

	/**
	 * @return true if the current client browser is a Netscape 6.x or 7.x browser, else false
	 */
	public boolean IsNetscape67()
	{
		return IsNetscape() && (m_nBrowserVersion > 6.0);
	}

	/**
	 * @return true if the current client browser is a browser for Windows OS, else false
	 */
	public boolean IsWindows()
	{
		return  WINDOWS == m_strBrowserPlatform;
	}

	/**
	 * @return true if the current client browser is a browser for Mac OS, else false
	 */
	public boolean IsMac()
	{
		return MACINTOSH == m_strBrowserPlatform;
	}

	/**
	 * 
	 * @return true if the current client browser is running on Solaris
	 */
	public boolean IsSunOS()
	{
		return SUNOS == m_strBrowserName;	
	}

	/**
	 * 
	 * @return the browser's name
	 */
	public String GetBrowserName()
	{
		return m_strBrowserName;	
	}

	/**
     * NOTE: This only includes the major and minor versions of the
     * browser.  I.E. "1.3.1" will be returned as "1.3".
	 * 
	 * @return the browser's version number, returns 1.0 if it was not found
	 */
	public float GetBrowserVersion()
	{
		return m_nBrowserVersion;	
	}
}