package com.plumtree.uiinfrastructure.application;

import com.plumtree.uiinfrastructure.application.notification.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.web.*;

/**
 * A class implementing this interface can be used to do custom
 * initialization of the Activity Spaces frameworks.
 */
public interface IAppWarmUpHelper
{
    /**
     * This method loads a custom config var pack that will be used to
     * initialize the rest of the application.  To use the standard
     * config var pack, simply return null from this method.  In order
     * for the config varpack to be recognized correctly by the application,
     * it must use the following varpack ID:
     * 
     * com.plumtree.uiinfrastructure.application.varpacks.ConfigVarPack.VARPACK_ID
     * 
     * If the config var pack fails to load, throw an XPException.  This
     * will cause the App WarmUp to fail and disable the portal.
     *
     * @param String strSettingsConfigFolder The folder location of the
     * 			settings config folder where the configuration xml files
     * 			can be found.
     * @param String strPlatform the current platform.  Either
     * 			ConfigVarPack.APP_PLATFORM_DOT_NET or ConfigVarPack.APP_PLATFORM_JAVA
     * @return ConfigVarPack the ConfigVarPack to use
     */
    public ConfigVarPack LoadConfigVarPack(String strSettingsConfigFolder, String strPlatform);

    /**
     * This method is used to load custom var packs.  This method will be
     * called along with the other initialization methods.  At this point,
     * the base VarPacks and ResourceManager will have been loaded.  The 
     * difference between this method and the LoadVarPacks method is that 
     * this method will only be called once, so put non-reloaded VarPacks
     * here.
     *
     * Add varpacks to the varpack manager as follows:
     * 
     * varPackManager.AddVariablePackage(vpConfig);
     * 
     * If any of the var packs fail to load, throw an XPException.  This
     * will cause the App WarmUp to fail and disable the portal.
     *
     * @param VarPackManager the varpack manager to add the config var
     * 			pack to.
     * @param String strSettingsConfigFolder The folder location of the
     * 			settings config folder where the configuration xml files
     * 			can be found.
     * @param String strPlatform the current platform.  Either
     * 			ConfigVarPack.APP_PLATFORM_DOT_NET or ConfigVarPack.APP_PLATFORM_JAVA
     */
    public ReturnValue InitializeVarPacks(VarPackManager vpmVarPackManager,
    	String strSettingsConfigFolder, String strPlatform);

    /**
     * This method is used to load custom var packs.  The difference 
     * between this method and the Initialize VarPacks method is that 
     * by this point all of the non-customized initialization has 
     * been completed.  Also, this method will be used to reload 
     * VarPacks.
     *
     * Add varpacks to the varpack manager as follows:
     * 
     * varPackManager.AddVariablePackage(vpConfig);
     * 
     * If any of the var packs fail to load, throw an XPException.  This
     * will cause the App WarmUp to fail and disable the portal.
     *
     * @param VarPackManager the varpack manager to add the config var
     * 			pack to.
     * @param String strSettingsConfigFolder The folder location of the
     * 			settings config folder where the configuration xml files
     * 			can be found.
     * @param String strPlatform the current platform.  Either
     * 			ConfigVarPack.APP_PLATFORM_DOT_NET or ConfigVarPack.APP_PLATFORM_JAVA
     */
    public ReturnValue LoadVarPacks(VarPackManager vpmVarPackManager,
    	String strSettingsConfigFolder, String strPlatform);

    /**
     * This method is used to initialize miscellaneous objects.  This
     * method will be called along with the other initialization methods.
     * The difference between this method and the LoadObjects method is 
     * that this method will only be called once, so put non-reloadable
     * objects here.  Also, customized spaces will not be available yet.
     *
     * If any of the objects fail to initialize properly, throw an
     * XPException.  This will cause the App WarmUp to fail and disable
     * the portal.
     * 
     * @param ConfigVarPack
     * @param String strApplicationName
     * @param String strImageServerBaseURL
     * @param String strSecureImageServerBaseURL
     * @param CachingManager
     */
    public ReturnValue InitializeObjects(ConfigVarPack vpConfig, String strApplicationName,
		String strImageServerBaseURL, String strSecureImageServerBaseURL,
		CachingManager cmCachingManager);

    /**
     * This method is used to initialize miscellaneous objects.  The 
     * difference between this method and the InitializeObjects method
     * that by this point customized activity spaces will have been 
     * loaded.  Also, this method can be used reload objects.
     *
     * If any of the objects fail to initialize properly, throw an
     * XPException.  This will cause the App WarmUp to fail and disable
     * the portal.
     * 
     * @param ConfigVarPack
     * @param String strApplicationName
     * @param String strImageServerBaseURL
     * @param String strSecureImageServerBaseURL
     * @param CachingManager
     */
    public ReturnValue LoadObjects(ConfigVarPack vpConfig, String strApplicationName,
		String strImageServerBaseURL, String strSecureImageServerBaseURL,
		CachingManager cmCachingManager);

    /**
     * This method is used to initialize any additional managers.  This
     * method will be called along with the other initialization methods.
     * The difference between this method and the LoadManagers method is 
     * that by this method occurs before customized code is loaded, and 
     * will only be run once.
     *
     * If any manager fails to initialize properly, throw an
     * XPException.  This will cause the App WarmUp to fail and disable
     * the portal.
     * 
     * @param String strLibHomePath
     * @param String strBaseConfigDirectory
     */
    public ReturnValue InitializeManagers(String strLibHomePath, String strBaseConfigDirectory);

    /**
     * This method is used to initialize any additional managers.  The 
     * difference between this method and the InitializeManagers method
     * is that by this point customized activity spaces will have been 
     * loaded.  Also, this method can be used to reload managers.
     *
     * If any manager fails to initialize properly, throw an
     * XPException.  This will cause the App WarmUp to fail and disable
     * the portal.
     * 
     * @param String strLibHomePath
     * @param String strBaseConfigDirectory
     */
    public ReturnValue LoadManagers(String strLibHomePath, String strBaseConfigDirectory);
}
