/*
 * Purpose:	Wraps the primary initialization task for appwarmup.
 * 
 * Author: 	D. Phipps Apr 7, 2005
 */
package com.plumtree.uiinfrastructure.application.startup;

import com.plumtree.openfoundation.io.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openkernel.factory.*;
import com.plumtree.openlog.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.application.*;
import com.plumtree.uiinfrastructure.application.notification.*;
import com.plumtree.uiinfrastructure.application.startup.exceptions.*;
import com.plumtree.uiinfrastructure.application.startup.messages.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.util.*;
import com.plumtree.xpshared.config.*;

/**
 * Wraps openkernel and portalobjectsfactory initialization in a multistep
 * startup class with detailed recommendations on failures.
 * 
 * @author DavidP
 */
public class InitPortalObjects implements IStartupTask
{
	private static OpenLogger log = OpenLogService.GetLogger(
			OpenLogService.GetComponent("UI_Infrastructure"),
			"com.plumtree.uiinfrastructure.application.startup.InitPortalObjects");
    
    private String m_applicationName = ""; 
    
    private String m_pthome = "";
    
    // array of messages
    private static StartupTaskMessage m_diag[] = 
    {
        // steps are one-based; step zero is the init call
        // recommender picks error most appropriate for step, platform, errlevel 
        new StartupTaskMessage( 
                0, Platform.DOTNET,PT_STARTUP_RESULTCODES.FATAL,
                "PT_HOME Environment Variable Failure (.NET)", 
                "RECOMMENDATION: The environment variable PT_HOME is not set or is set to an invalid path.  " +
                "Please correct it."
                ),
        new StartupTaskMessage( 
                0, Platform.JAVA,PT_STARTUP_RESULTCODES.FATAL,
                "PT_HOME Environment Variable Failure (Java)", 
                "RECOMMENDATION: The environment variable PT_HOME is not set or is set to an invalid path.  " +
                "Please correct it."
                ),
        new StartupTaskMessage( 
                1, Platform.DOTNET,PT_STARTUP_RESULTCODES.FATAL,
                "Basic Path Failure (.NET)", 
                "RECOMMENDATION: The directory that the environment variable PT_HOME refers to is not a " +
                "valid installation, since key directories and files are missing.  This usually means that " +
                "your PT_HOME environment variable or pthome.xml file points to an invalid or old location. " +
                "Please correct it."
                ),
        new StartupTaskMessage( 
                1, Platform.JAVA,PT_STARTUP_RESULTCODES.FATAL,
                "Basic Path Failure (Java)", 
                "RECOMMENDATION: The directory that the environment variable PT_HOME refers to is not a " +
                "valid installation, since key directories and files are missing.  This usually means that " +
                "your PT_HOME environment variable or pthome.xml file points to an invalid or old location. " +
                "Please correct it."
                ),
        new StartupTaskMessage( 
                2, Platform.DOTNET,PT_STARTUP_RESULTCODES.FATAL,
                "OpenKernel Initialization Failure (.NET)", 
                "RECOMMENDATION: Review the exception listed below for more details.  " +
                "Check that pthome.xml is in the correct place, check that the PT_HOME environment variable " +
                "points to it, and check that your database settings by running ptconfig.  " +
				"Check that the portalconfig.xml file is accessible by the web application server."
                ),
        new StartupTaskMessage( 
                2, Platform.JAVA,PT_STARTUP_RESULTCODES.FATAL,
                "OpenKernel Initialization Failure (Java)",
                "RECOMMENDATION: Review the detailed exception listed below for a specific analysis.  " +
                "Check that pthome.xml is in the correct place, check that the PT_HOME environment variable " +
                "points to it, and check that your database settings by running ptconfig. " +
                "Check that the portalconfig.xml file is accessible by the web application server."
                ),
        new StartupTaskMessage( 
                3, Platform.DOTNET,PT_STARTUP_RESULTCODES.FATAL,
                "PortalObjectsFactory Initialization Failure (.NET)", 
                "RECOMMENDATION: Review the exception listed below for more details.  " +
                "Since OpenKernel initialized correctly, this usually means that the portal database " +
                "is not scripted or properly configured, or there is an installation " +
                "problem related to .NET assemblies.  This may mean the search libraries are " +
                "not on your path. Try running ptverify."                
                ),
        new StartupTaskMessage( 
                3, Platform.JAVA,PT_STARTUP_RESULTCODES.FATAL,
                "PortalObjectsFactory Initialization Failure (Java)", 
                "RECOMMENDATION: Review the exception listed below for details.  " +
                "Since OpenKernel initialized correctly, this usually means that the portal database " +
                "is not scripted or properly configured, or there is an installation " +
                "problem related to JAR files.  This may also mean the search libraries are " +
                "not on your PATH or LD_LIBRARY_PATH. Try running ptverify."                
                ),
    };

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskName()
     */
    public String getTaskName()
    {
        return "InitPortalObjects";
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskID()
     */
    public int getTaskID()
    {
        return PTStartupTasks.INITBASE.getOrdinal();
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getNumSteps()
     */
    public int getNumSteps()
    {
        // init open kernel plus init portal objects factory
        return 4;
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#init(java.lang.String)
     */
    public ReturnValue init(String _strApplicationName)
    {
        m_applicationName = _strApplicationName;
        return new ReturnValue(PT_STARTUP_RESULTCODES.OK,getTaskName()+" init successful",null);
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#executeStep(int)
     */
    public ReturnValue executeStep(int sequentialStepID)
    {
		ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,
		        "Incorrect step request on init",
		        StackTracer.MakeXPException("(internal stacktrace)"));
		switch( sequentialStepID )
		{
			case 0:
			    // check the env vars and paths
				retVal = CheckPTHome();
				break;
			case 1:
			    // check the env vars and paths
				retVal = CheckPaths();
				break;
			case 2:
			    // open config file
				retVal = InitializeOpenKernel();
				break;
			case 3:
			    // load config file
				retVal = InitPortalObjectsStep();
				break;
		}
		return retVal;
    }
    
	/**
	 * Initializes our connection to OpenKernel.  Before this point you cannot use the logger 
	 * because it will not have been initialized yet.  To display errors 
	 * before now, output directly to the console.
	 */
    public ReturnValue CheckPTHome()
    {
	    ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK,"Environment variables are properly set",null);
		try
		{
		    // is the var set?
		    String str = XPEnvironment.GetEnvironmentVariable(AppWarmUp.PARAM_PTHOME_ENVVAR_NAME);
		    if ((str == null)||(str.length() < 1))
		    {
		        StackTracer.ThrowNewXPException("PT_HOME environment variable is not set or is empty");
		    }
		    
		    // does the var point to a valid directory?
		    boolean isValid = false;
		    try
		    {
		        XPDirectory fileDir = new XPDirectory(str);
		        isValid = fileDir.Exists();
		    }
		    catch( XPException e )
		    {
		        throw XPFileNotFoundException.GetInstance("PT_HOME environment variable points to invalid directory: "+str,e);
		    }
		    
		    if (!isValid)
		    {
		        StackTracer.ThrowNewXPException("PT_HOME environment variable points to invalid directory: "+str );
		    }
		    
		    // save the value
		    m_pthome = str;
		    
		    // check for search library in path
		    
		    // check again in LD_LIBRARY_PATH if that fails
		}
		catch (Throwable te)
		{
		    String reason = "";
		    if (te.getMessage() != null)
		        reason = te.getMessage();
		    else
		        reason = RecommendationHelper.ConvertExceptionToRecommendation( XPException.GetInstance(te) );
			retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,reason,XPException.GetInstance(te));				
		}
		
		return retVal;
    }
    
	/**
	 * Initializes our connection to OpenKernel.  Before this point you cannot use the logger 
	 * because it will not have been initialized yet.  To display errors 
	 * before now, output directly to the console.
	 */
    public ReturnValue CheckPaths()
    {
	    ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK,"Basic (most primitive) paths are properly set",null);
		try
		{
		    // does pthome.xml exist?
		    boolean isValid = false;
		    String szPath = m_pthome + XPFile.GetPathSeparator() + "pthome.xml";
		    try
		    {
		        XPFile filePthome = new XPFile(szPath);
		        isValid = filePthome.Exists();
		    }
		    catch( XPException e )
		    {
		        throw XPException.GetInstance("The file pthome.xml does not exist in the directory pointed to by the PT_HOME environment variable: "+szPath,e);
		    }
		    
		    if (!isValid)
		    {
		        throw new XPFileNotFoundException("The file pthome.xml does not exist in the directory pointed to by the PT_HOME environment variable: "+szPath);
		    }
		    
		    // does pthome's code work?
		    String szOKPath = ConfigPathResolver.GetOpenConfigPath();
		    
		    // does the openkernel settings directory exist?
		    isValid = false;
		    try
		    {
		        XPDirectory fileDir = new XPDirectory(szOKPath);
		        isValid = fileDir.Exists();
		    }
		    catch( XPException e )
		    {
		        throw XPException.GetInstance("The OpenKernel settings directory, which is under the directory defined by the PT_HOME environment variable, does not exist: "+szOKPath,e );
		    }
		    if (!isValid)
		    {
		        throw new XPFileNotFoundException("The OpenKernel settings directory, which is under the directory defined by the PT_HOME environment variable, does not exist: "+szOKPath );
		    }

		    // does the portal settings file exist?
		    isValid = false;
		    String szPortalConfig = szOKPath + XPFile.GetPathSeparator() + "portal" + XPFile.GetPathSeparator() + "portalconfig.xml";
		    try
		    {
		        XPFile fileSettings = new XPFile(szPortalConfig);
		        isValid = fileSettings.Exists();
		    }
		    catch( XPException e )
		    {
		        throw XPException.GetInstance("The portal configuration file, which is under the directory defined by the PT_HOME environment variable, does not exist: "+szPortalConfig,e );
		    }
		    if (!isValid)
		    {
		        throw new XPFileNotFoundException("The portal configuration file, which is under the directory defined by the PT_HOME environment variable, does not exist: "+szPortalConfig );
		    }
		}
		catch (Throwable te)
		{
		    String reason = RecommendationHelper.ConvertExceptionToRecommendation( XPException.GetInstance(te) );
			retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,reason,XPException.GetInstance(te));				
		}
		
		return retVal;
    }
    
	/**
	 * Initializes our connection to OpenKernel.  Before this point you cannot use the logger 
	 * because it will not have been initialized yet.  To display errors 
	 * before now, output directly to the console.
	 */
	public ReturnValue InitializeOpenKernel()
	{
	    ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK,"OpenKernel initialized successfully",null);
		try
		{
		    // retrieve portal home path from pthome.xml.
			// DAP Note: that sApplicationName *must* be the one used in pthome.xml and serverconfig.xml
			// I have reduced many instances of hardcoded magic strings 
			if ((m_applicationName == null) || (m_applicationName.compareTo(AppConstants.MAIN_APPLICATION_NAME.ToString()) != 0))
			{
			    // return value
			    if (m_applicationName == null)
			        StackTracer.ThrowNewXPException("Error!  Application name is NULL.  Please check web.xml or web.config and correct this immediately");
		        else
		            StackTracer.ThrowNewXPException("Error!  Application name '" + m_applicationName + "' does not match the internal constant name '" + AppConstants.MAIN_APPLICATION_NAME + "'.  Please check web.xml or web.config and correct this immediately");
			}
		    String strPortalHome = GeneralHelpers.FormatPath(ConfigPathResolver.
		        GetProductPath(m_applicationName), AppWarmUp.PATH_SEPARATOR, true);

		    // create the path to serverconfig.xml's directory
		    // WilliamA - path is PT_HOME\settings\common\serverconfig.xml file.
			// DAP trying another method
			//String strPTHome = strPortalHome + ".." + AppWarmUp.PATH_SEPARATOR + "..x" + AppWarmUp.PATH_SEPARATOR;
			String strPTHome = GeneralHelpers.FormatPath(
				XPEnvironment.GetEnvironmentVariable(AppWarmUp.PARAM_PTHOME_ENVVAR_NAME), 
				AppWarmUp.PATH_SEPARATOR, true);
			String strPlumtreeSettingsDir = strPTHome + AppWarmUp.FOLDER_SETTINGS + AppWarmUp.PATH_SEPARATOR;
		    
	        //MES- Initialize OpenKernel, telling OpenConfig where to get it's stuff
	    	AppWarmUp.m_configContext = OKConfigFactory.createInstance(strPlumtreeSettingsDir, m_applicationName);
	    	
	    	//MES- Set up the log service
	    	//	NOTE: We are using the "backwards compatible" mode to do this
	    	OpenLogService.initialize(AppWarmUp.m_configContext);
	    	
			log.Info("Logging initialized");
		}
		catch (Throwable te)
		{
		    String reason = RecommendationHelper.ConvertExceptionToRecommendation( XPException.GetInstance(te) );
			retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,reason,XPException.GetInstance(te));				
		}
		
		return retVal;
	}
	
	/**
	 * Initializes the portal objects factory, which inits each singleton 
	 * manager for openkernel.
	 * @return
	 */
	public ReturnValue InitPortalObjectsStep()
	{
	    ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK,"PortalObjectsFactory initialized successfully.",null);
	    try
	    {
	        PortalObjectsFactory.Init(AppWarmUp.m_configContext);
	        
	        /** steps:
				todo: can I do each of these in turn right here?  will that work?  
				it is slower but perhaps in verbose mode?
			InternalSession.Init(context);
			ObjectManagedObjectCache.InitSingletons();
			GadgetCache.InitSingletons();
			WebServiceCache.InitSingletons();
			PTUserInterfaceCache.InitSingletons();
			PTAdminFolderCache.InitSingletons();
			PTPropertyCache.InitSingletons();
			CommunityInfoCache.InitSingletons();
			PageInfoCache.InitSingletons();
			PrefsCache.InitSingletons();
			PTCardCache.InitSingletons();
			PTFolderCache.InitSingletons();
			PTDocumentTypeCache.InitSingletons();
			CollabInfoCache.InitSingletons();
			PTSnapshotQueryCache.InitSingletons();
			ExternalInfoCache.InitSingletons();
			SubPortalInfoCache.InitSingletons();
			GuestLoginInfoCache.InitSingletons();
			FederatedPortalCache.InitSingletons();
			SiteMapFolderInfoCache.InitSingletons();
			ExpressionInfoCache.InitSingletons();
			SSOAuthProvider.StaticInit();
			PTSQLCallbackListener.StaticInit();
			GatewayConfig.GetSingleton().InitSingleton();
			
			ExpressionEngine.GetInstance().InitSingleton();
			**/
	    }
	    catch( Throwable te )
	    {
		    String reason = RecommendationHelper.ConvertExceptionToRecommendation( XPException.GetInstance(te) );
			retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,reason,XPException.GetInstance(te));				
	    }
		
		return retVal;	    
	}
    

    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsg(int)
     */
    public XPArrayList getMsg( int stepid, Platform platform, ReturnValue ret )
    {
        // Use the diagnostic helper to find the right message        
        return RecommendationHelper.getList(stepid,platform,ret.getResultCode(),m_diag);
    }
    
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsgArray()
     */
    public StartupTaskMessage[] getMsgArray()
    {
        return m_diag;
    }
    
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getNumMsgs()
     */
    public int getNumMsgs()
    {
        return m_diag.length;
    }

}
