/*
 * Created on Mar 30, 2005
 */
package com.plumtree.uiinfrastructure.application.startup;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.server.*;
import com.plumtree.server.impl.core.*;
import com.plumtree.uiinfrastructure.application.*;
import com.plumtree.uiinfrastructure.application.notification.*;
import com.plumtree.uiinfrastructure.application.startup.exceptions.*;
import com.plumtree.uiinfrastructure.application.startup.messages.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.util.*;
import com.plumtree.uiinfrastructure.web.*;
import com.plumtree.jsincluder.*;
import com.plumtree.openkernel.http.*;

/**
 * Initializes the ImageServerHelper, Part II.  Logs in as guest,
 * then tests the imageserver.
 * 
 * @author DavidP
 */
public class InitializeImageServerHelperPartII implements IStartupTask
{
	private static OpenLogger log = OpenLogService.GetLogger(
			OpenLogService.GetComponent("UI_Infrastructure"),
			"com.plumtree.uiinfrastructure.application.startup.InitializeImageServerHelperPartII");
	
    private String m_applicationName = "";
    private ConfigVarPack m_vpConfig = null;
    private String m_strImageServerConnectionURL = null;
    private IApplication m_application = null;     
    private IPTSession m_session = null;
    
	private static String DEFAULT_HEADERS = 
		"User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.7) Gecko/20040707 Firefox/1.0.1\r\n" +
		"Accept: text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5\r\n" +
		"Accept-Language: en-us\r\n" +
		"Accept-Charset: windows-1251,utf-8;q=0.7,*;q=0.7\r\n" +
		"\r\n";
	
	private int m_timeout = 30; // thirty second timeout
    
    // array of messages
    private static StartupTaskMessage m_diag[] = 
    {
        // steps are one-based; step zero is the init call
        // DAP TODO this really needs to be in an external file
        new StartupTaskMessage( 
                0, Platform.ALL,PT_STARTUP_RESULTCODES.FATAL,
                "Imageserver Initialization Problem", 
                "RECOMMENDATION: This indicates previous startup steps have generated warnings that have been ignored.  " +
                "Either the imageserver has not initialized or the configuration manager has encountered a problem."
                ),
        new StartupTaskMessage( 
                1, Platform.ALL,PT_STARTUP_RESULTCODES.FATAL,
                "Session Initialization Problem", 
                "RECOMMENDATION: When the guest user session cannot be initialized, this usually indicates corrupt, empty, or " +
                "old tables in the portal database.  It can also indicate an improperly configured portal server, so you " +
                "should check each entry in serverconfig.xml."
                ),
        new StartupTaskMessage( 
                2, Platform.DOTNET,PT_STARTUP_RESULTCODES.FATAL,
                "Imageserver Inaccessible, or Missing Folder", 
                "RECOMMENDATION: This error usually indicates your imageserver is not running, or is inaccessible due to network " +
                "issues.<P>It may also indicate you are running against an old imageserver.  Note that you cannot " +
                "upgrade only the portal; the imageserver must also be upgraded to the same version.  Check the specific URL " +
                "listed below.<P>Note: You can disable this check by setting ImageServerConnectionURLTimeout to -1 in portalconfig.xml."
                ),
        new StartupTaskMessage( 
                2, Platform.JAVA,PT_STARTUP_RESULTCODES.FATAL,
                "Imageserver Inaccessible, or Missing Folder", 
                "RECOMMENDATION: This error usually indicates your imageserver is not running, or is inaccessible due to network " +
                "issues.<P>It may also indicate you are running against an old imageserver.  Note that you cannot " +
                "upgrade only the portal; the imageserver must also be upgraded to the same version.  Check the specific URL " +
                "listed below.<P>Note 1: You can disable this check by setting ImageServerConnectionURLTimeout to -1 in " +
                "portalconfig.xml.<P>Note 2: If you are serving your imageserver on the same instance of the Java web application " +
                "server as the portal, you must disable this check (I.E. set ImageServerConnectionURLTimeout to -1), because the " +
                "imageserver will not be available until all webapps on the application server have started up."
                ),
    };    
    
    /**
     * 
     */
    public InitializeImageServerHelperPartII()
    {
        super();
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskName()
     */
    public String getTaskName()
    {
        return "InitializeImageServerHelperPartII";
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskID()
     */
    public int getTaskID()
    {
        return PTStartupTasks.INITIMGSERVERII.getOrdinal();
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getNumSteps()
     */
    public int getNumSteps()
    {
        return 3;
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#init(java.lang.String)
     */
    public ReturnValue init(String _strApplicationName)
    {
    	m_applicationName = _strApplicationName;
    	return new ReturnValue(PT_STARTUP_RESULTCODES.OK,getTaskName()+" init successful",null);
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#executeStep(int)
     */
    public ReturnValue executeStep(int sequentialStepID)
    {
		ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,"Incorrect step request on init",
		        StackTracer.MakeXPException("(internal stacktrace)"));
		switch( sequentialStepID )
		{
			case 0:
			    // log in as guest
			    retVal = executeStepOne();
			    break;
			case 1:
			    // log in as guest
			    retVal = executeLogInAsGuest();
			    break;
			case 2:
			    // check the urls
			    retVal = executeCheckImageserverJsincluderUrl();
			    break;
		}
		return retVal;
    }

    /**
     * Load config file, get imageserver URL, form related urls
     * @return
     */
    private ReturnValue executeStepOne()
    {
        ReturnValue retVal  = new ReturnValue(PT_STARTUP_RESULTCODES.OK,"InitializeImageServerHelper successful.",null);
	   
		log.Info("Initializing ImageServerHelper.");
	
	    // form the URL
		try
		{
			// retrieve Application and ConfigVarPack.
		    String strApplicationName = XPConvert.ToString(AppWarmUp.s_htInitSettings.
		        GetElement(AppWarmUp.KEY_STR_APPLICATION_NAME));
		    m_application = ApplicationManager.GetInstance().
		    	GetApplication(strApplicationName);
		    m_vpConfig = (ConfigVarPack) m_application.GetVarPackManager().
		    	GetVariablePackage(ConfigVarPack.VARPACK_ID);
		    
		    
		    m_strImageServerConnectionURL = m_vpConfig.GetConfigValueAsString("MainURLs",
					ConfigConstants.IMAGE_SERVER_CONNECTION_URL, ConfigConstants.VALUE_ATTRIBUTE);
		}
		catch (Throwable ex)
		{
		    String errorMessage = "Failed to get ImageServerConnectionURL from ConfigVarPack: ";
			retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,
			        errorMessage,
			        XPException.GetInstance(ex));
			return retVal;
		}		
		
		return retVal;
    }	
    
    /**
     * Log in as guest, to get the session, as a test and also to prep for an http request
     * 
     * @return ReturnValue
     */
    private ReturnValue executeLogInAsGuest()
    {
        ReturnValue retVal  = new ReturnValue(PT_STARTUP_RESULTCODES.OK,"InitializeImageServerHelper successful.",null);
        
        try
        {
            // Log in as guest (this also serves to validate the entire session)
            IPTSession session = PortalObjectsFactory.GetGuestReadOnlySession();
            m_session = session;
        }
		catch (Throwable ex)
		{
		    String errorMessage = "Could not log in as default guest: ";
			retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,
			        errorMessage,
			        XPException.GetInstance(ex));
			return retVal;
		}
		
        return retVal;
    }
    
    /**
     * Validate the imageserver URL
     * 
     * @return
     */
    private ReturnValue executeCheckImageserverJsincluderUrl()
    {
        ReturnValue retVal  = new ReturnValue(PT_STARTUP_RESULTCODES.OK,"InitializeImageServerHelper successful.",null);
    	
        // form the connection url
		try
		{
		    // check that the config helper is up
		    if (!ConfigHelper.IsInitialized())
		    {
			    return new ReturnValue(PT_STARTUP_RESULTCODES.OK,
			            "The ConfigHelper is not initialized. There is a problem " +
			            "with the order of execution of the startup tasks.  Startup cannot continue. " +
			            "Please contact BEA Support.",null);
		    }
		    
		    // get the connect timeout
		    m_timeout = ConfigHelper.GetImageserverConnectTimeout();
		    
		    // bypass if less than 1
		    if (m_timeout > 0)
		    {
			    // check the PCC
			    JSIncluder includer = JSIncluder.GetIncluder(m_strImageServerConnectionURL);
			    String strVersion = ConfigHelper.GetJSIncluderComponentVersion(ConfigHelper.JSCOMPONENT_JSPORTLET);
			    String strUrl = includer.GetDiagnosticURL(ConfigHelper.JSCOMPONENT_JSPORTLET, strVersion, m_strImageServerConnectionURL );
				
				// form the HTTP request and execute it
				IOKHttpService service = InternalSession.GetHttpService();
				IOKHttpSession session = m_session.GetHttpUserSession();
				IOKRequestWaiter waiter = service.GetWaiterObject();
				IOKUrl url = service.CreateUrl( strUrl );
				IOKHttpRequest request = session.CreateRequest( url );
				waiter.AddRequest( request );
	
				request.GetRequestHeaders().SetRawHeader( DEFAULT_HEADERS );
				request.GetRequestHeaders().SetProtocolVersion( "HTTP/1.0" );
				request.SetTimeout( m_timeout * 1000 );
				request.Submit();
				
				waiter.Wait( m_timeout * 1000 );
				
				if (request.GetRequestState() == OKHttpRequestState.DONE)
				{			    
					IOKHttpResponse response = request.GetResponse();
					if ((response != null) && (response.GetBodyAsString() != null) && (response.GetBodyAsString().length() > 0))
					{
					    // check http response codes
					    int code = response.GetResponseCode();
					    if (code != 200)
					    {
					        // bad response; fail
					        String strCode = (new Integer(code)).toString();
						    retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,
						            "FAILURE: bad HTTP response code (HTTP "+strCode+").  Check URL '"+strUrl+"'",
						            new XPException("Bad response from URL: " + strUrl + " HTTP Code: " + strCode));
					    }
					    
					    // it works!
					    retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK,
					            "The imageserver responded with a valid HTTP response " +
					            "reflecting proper javascript structure.",null);
					}
					else if (response == null)
					{
					    // dead response; can't resolve server
					    retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,
					            "FAILURE: dead response; cannot resolve server.  Check URL '"+strUrl+"'",
					            new XPException("No bytes returned from url: " + strUrl));
					}
					else
					{
					    // empty response; no file or empty file
					    retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,
					            "FAILURE: empty response; key imageserver file is missing or empty.  Check URL '"+strUrl+"'",
					            new XPException("No bytes returned from url: " + strUrl));
					}
				}
				else
				{
				    // timeout; dead imageserver
				    String strSec = (new Integer(m_timeout)).toString();
				    retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,
				            "FAILURE: timeout ("+strSec+" seconds) on key imageserver request.  Check URL '"+strUrl+"'",
				            new XPException("No response from url: " + strUrl));
				}
		    }
		    else
		    {
		        retVal  = new ReturnValue(PT_STARTUP_RESULTCODES.OK,"InitializeImageServerHelper successful (Warning: Bypassed imageserver connect checking)",null);
		    }
		}
		catch (Throwable ex)
		{
		    String errorMessage = "Primary failure testing ImageServerConnectionURL: '" + m_strImageServerConnectionURL + "'";
			retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,
			        errorMessage,
			        XPException.GetInstance(ex));
			return retVal;
		}
		
        return retVal;
    }
    
    
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsg(int)
     */
    public XPArrayList getMsg( int stepid, Platform platform, ReturnValue ret )
    {
        // Use the diagnostic helper to find the right message        
        return RecommendationHelper.getList(stepid,platform,ret.getResultCode(),m_diag);
    }
    
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsgArray()
     */
    public StartupTaskMessage[] getMsgArray()
    {
        return m_diag;
    }
    
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getNumMsgs()
     */
    public int getNumMsgs()
    {
        return m_diag.length;
    }
	
}
