/*
 * Purpose:	Initializes appwarmup helper objects
 * Author: 	D. Phipps May 18, 2005
 */
package com.plumtree.uiinfrastructure.application.startup;

import com.plumtree.openfoundation.io.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.application.*;
import com.plumtree.uiinfrastructure.application.notification.*;
import com.plumtree.uiinfrastructure.application.startup.exceptions.*;
import com.plumtree.uiinfrastructure.application.startup.messages.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.tempfile.*;
import com.plumtree.uiinfrastructure.util.*;
import com.plumtree.uiinfrastructure.web.*;
import com.plumtree.xpshared.jsutil.*;

/**
 * Initializes Objects.  This method will initialize any additional
 * objects specified here, and any additional objects as specified 
 * by the AppWarmUpHelper.  After this point, the Portal's TempFile,
 * JSRegistry, and Secure JSRegistry will be available, as well as
 * the objects defined in AppWarmUpHelper.
 * 
 * @author DavidP
 */
public class InitializeObjects implements IStartupTask
{
	private static OpenLogger log = OpenLogService.GetLogger(
			OpenLogService.GetComponent("UI_Infrastructure"),
			"com.plumtree.uiinfrastructure.application.startup.InitializeObjects");

    private String m_applicationName = "";
	private IAppWarmUpHelper m_ptAppWarmUpHelper=null; 
	private IApplication m_application=null;
	
	private String m_strImageServerBaseURL = null;
	private String m_strSecureImageServerBaseURL = null;
	private String m_strImageServerConnectionURL = null;
	
	private ConfigVarPack m_vpConfig = null;
	
    
    // array of messages
    private static StartupTaskMessage m_diag[] = 
    {
        // steps are one-based; step zero is the init call
        // DAP TODO this really needs to be in an external file
        new StartupTaskMessage( 
                0, Platform.ALL,PT_STARTUP_RESULTCODES.FATAL,
                "InitializeObjects: TempFile or JSRegistry problem.", 
                "RECOMMENDATION: Check that the TempHome setting in portalconfig.xml points to a valid directory that " +
                "the appserver has rights to read and write to.  Also check that the imageserver is running properly and " +
                "that the correct JSRegistry version has been installed.  For more details, review the secondary message " +
                "below."
                                ),
        new StartupTaskMessage( 
                1, Platform.ALL,PT_STARTUP_RESULTCODES.FATAL,
                "InitializeObjects: AppWarmupHelper objects load problem.", 
                "RECOMMENDATION: This can be caused by many different dynamic loading issues, including classtype descriptors " +
                "expression engine loading issues, and custom conditions loading.  For more details, review the secondary " +
                "message below." )
    };    
    /**
     * 
     */
    public InitializeObjects()
    {
        super();
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskName()
     */
    public String getTaskName()
    {
        return "InitializeObjects";
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskID()
     */
    public int getTaskID()
    {
        return PTStartupTasks.INITOBJECTS.getOrdinal();
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getNumSteps()
     */
    public int getNumSteps()
    {
        return 2;
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#init(java.lang.String)
     */
    public ReturnValue init(String _strApplicationName)
    {
    	m_applicationName = _strApplicationName;
    	return new ReturnValue(PT_STARTUP_RESULTCODES.OK,getTaskName()+" init successful",null);
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#executeStep(int)
     */
    public ReturnValue executeStep(int sequentialStepID)
    {
		ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,"Incorrect step request on init",
		        StackTracer.MakeXPException("(internal stacktrace)"));
		switch( sequentialStepID )
		{
			case 0:
			    // open config file
				retVal = executeStepOne();
				break;
			case 1:
			    // load config file
				retVal = executeStepTwo();
				break;
		}
		return retVal;
    }
    
    /**
     * Inits jsregistry and tempfile
     * @return
     */
    public ReturnValue executeStepOne()
    {
		ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK,"Initialize objects step one successful",
		        null);
		
		try
		{
		    log.Info("Initializing Objects.");
		    
		    ////////////////////////////////
		    // INITIALIZE TEMPFILE OBJECT //
		    ////////////////////////////////

		    String strApplicationName = XPConvert.ToString(AppWarmUp.s_htInitSettings.
		        GetElement(AppWarmUp.KEY_STR_APPLICATION_NAME));
		    m_application = ApplicationManager.GetInstance().
		    	GetApplication(strApplicationName);
		    m_vpConfig = (ConfigVarPack) m_application.GetVarPackManager().
	    		GetVariablePackage(ConfigVarPack.VARPACK_ID);

			try 
			{
			    // tempfile class provides access to temp directory.
				TempFile.StaticInit(m_vpConfig.GetConfigValueAsString(
				    "MainURLs", "TempHome", "value"));
				String strTempPath = TempFile.BASE_TEMP_DIR;
				    
 			    // check that it works		
			    checkDirectoryExists(strTempPath,"The TempHome setting in portalconfig.xml points to an invalid directory.");
			} 
			catch (Throwable ex) 
			{
			    String errorMessage = "Error initializing Tempfile: ";
			    retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,errorMessage,XPException.GetInstance(ex));
			    return retVal;
			}

			/////////////////////////////////////////////////
			// INITIALIZE JSREGISTRY AND SECURE JSREGISTRY //
			/////////////////////////////////////////////////
			
		    m_strImageServerBaseURL = XPConvert.ToString(AppWarmUp.s_htInitSettings.
		        GetElement(AppWarmUp.KEY_STR_IMAGESERVER_BASE_URL));
		    m_strSecureImageServerBaseURL = XPConvert.ToString(AppWarmUp.s_htInitSettings.
		        GetElement(AppWarmUp.KEY_STR_SECURE_IMAGESERVER_BASE_URL));
		    m_strImageServerConnectionURL = XPConvert.ToString(AppWarmUp.s_htInitSettings.
		        GetElement(AppWarmUp.KEY_STR_IMAGESERVER_CONNECTION_URL));

			try 
			{
				JSRegistry ptJSReg = JSRegistry.GetRegistry(m_strImageServerBaseURL, 
				    m_strImageServerConnectionURL);
				m_application.GetCachingManager().SetEntry(AppWarmUp.KEY_CACHEMAN_JS_REGISTRY, 
				    ptJSReg);
			
			} 
			catch (Throwable ex) 
			{
			    String errorMessage = "Failed to get put the JSRegistry (NON-SECURE) " +
			    	"onto the Aqualogic Interaction application. This is usually due to the " +
			    	"Aqualogic Interaction not being able to talk to the Image Service. Please " +
			    	"make sure that the URL specified by the ImageServerURLConnection " +
			    	"variable in your portalconfig.xml file can reach the Image Service FROM " +
			    	"the Portal Server. The value the Aqualogic Interaction is using is " + 
			    	m_strImageServerConnectionURL + ". Extended error: ";			    
			    retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,errorMessage,XPException.GetInstance(ex));
			    return retVal;
			}
			
			
			try 
			{				
				JSRegistry ptSecureJSReg = JSRegistry.GetRegistry(m_strSecureImageServerBaseURL, 
				    m_strImageServerConnectionURL);
				m_application.GetCachingManager().SetEntry(AppWarmUp.KEY_CACHEMAN_SECURE_JS_REGISTRY, 
				    ptSecureJSReg);			
			} 
			catch (Throwable ex) 
			{
			    String errorMessage = "Failed to get put the JSRegistry (SECURE) " +
		    		"onto the Aqualogic Interaction application. This is usually due to the " +
		    		"Aqualogic Interaction not being able to talk to the Image Service. Please " +
		    		"make sure that the URL specified by the ImageServerURLConnection " +
		    		"variable in your portalconfig.xml file can reach the Image Service FROM " +
		    		"the Portal Server. The value the Portal Service is using is " + 
		    		m_strImageServerConnectionURL + ". Extended error: ";			    
			    retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,errorMessage,XPException.GetInstance(ex));
			}
		}
		catch (Throwable ex)
		{
		    String errorMessage = "Error initializing Objects: ";
		    retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,errorMessage,XPException.GetInstance(ex));
		}
		
		return retVal;
    }
    
    /**
     * Initializes the appwarmup helper objects
     * @return
     */
    public ReturnValue executeStepTwo()
    {
		ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK,"Initialize objects step two successful",
		        StackTracer.MakeXPException("(internal stacktrace)"));
		
		try
		{

			////////////////////////////////////////
			// INITIALIZE APPWARMUPHELPER OBJECTS //
			////////////////////////////////////////
			
		    // retrieving necessary elements.
		    m_ptAppWarmUpHelper = (IAppWarmUpHelper) 
		    	AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_APP_WARMUP_HELPER);
		
		    if (null != m_ptAppWarmUpHelper)
		    {
		        retVal = m_ptAppWarmUpHelper.InitializeObjects(m_vpConfig, m_applicationName, m_strImageServerBaseURL,
						m_strSecureImageServerBaseURL, m_application.GetCachingManager());
		    }
		    
		    if (retVal.getResultCode() == PT_STARTUP_RESULTCODES.OK)
		        log.Info("Objects initialized.");
		}
		catch (Throwable ex)
		{
		    String errorMessage = "Error initializing Objects in the appwarmup helper: ";
		    retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,errorMessage,
		            XPException.GetInstance(ex));
		}
		
		return retVal;
    }

    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsg(int)
     */
    public XPArrayList getMsg( int stepid, Platform platform, ReturnValue ret )
    {
        // Use the diagnostic helper to find the right message        
        return RecommendationHelper.getList(stepid,platform,ret.getResultCode(),m_diag);
    }
    
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsgArray()
     */
    public StartupTaskMessage[] getMsgArray()
    {
        return m_diag;
    }
    
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getNumMsgs()
     */
    public int getNumMsgs()
    {
        return m_diag.length;
    }    
    
    /**
     * Check that the directory exists
     * @param szPath
     * @param szMessage
     */
    private void checkDirectoryExists(String szPath, String szMessage)
    {
	    boolean isValid = false;
	    try
	    {
	        XPDirectory dirItself = new XPDirectory(szPath);
	        isValid = dirItself.Exists();
	    }
	    catch( XPException e )
	    {
	        throw XPException.GetInstance(szMessage+": "+szPath,e);
	    }
	    if (!isValid)
	    {
	        StackTracer.ThrowNewXPFileNotFoundException(szMessage+": "+szPath);
	    }						        
    }
    
    /**
     * Check that the file exists
     * @param szPath
     * @param szMessage
     */
    private void checkFileExists(String szPath, String szMessage)
    {
	    boolean isValid = false;
	    XPFile fileItself = null;
	    try
	    {
	        fileItself = new XPFile(szPath);
	        isValid = fileItself.Exists();
	    }
	    catch( XPException e )
	    {
	        throw XPException.GetInstance(szMessage+": "+szPath,e);
	    }
	    if (!isValid)
	    {
	        StackTracer.ThrowNewXPFileNotFoundException(szMessage+": "+szPath);
	    }						        
    }
}
