package com.plumtree.uiinfrastructure.application.varpacks;

import com.plumtree.openlog.*;
import com.plumtree.openfoundation.io.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.xml.dom.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.web.*;
import com.plumtree.uiinfrastructure.application.*;
import com.plumtree.openkernel.config.*;

/**
 * This var pack handles all of the different devices and
 * markup formats that we support.
 *
 * @author SudhaS
 */
public class DevicesVarPack extends BaseVarPack
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.application.varpacks.DevicesVarPack");

	/** The string ID of this variable package. */
	public static final String VARPACK_ID = "DevicesVarPack";
	
	// OpenConfig component type
	public static final String PORTAL_DEVICES_COMPONENT_TYPE = "http://www.plumtree.com/config/component/types/portal/devices";
	
	// Internal constants
	private static final String SECTION_SUBSECTION_SEPARATOR = ".";

	/** Cache the Application Base URL to make it easy to get to. */
	private String m_strApplicationBaseURL = "";

	private XPHashtable m_htDevicesXMLSettings;

	/**
	 * This method returns the Application Base URL. This is for easy access.
	 * You could just call GetConfigValueAsString with the appropriate
	 * parameters.
	 *
	 * @return
	 */
	public String GetApplicationBaseURL()
	{
		return m_strApplicationBaseURL;
	}

	/**
	 * This method determines the proper markup type to
	 * use for whatever device the user is using.  This
	 * uses the accepts header and locates the first
	 * supported markup format that the device prefers.
	 *
	 * @param String the accepts header for the device
	 *
	 * @return String the code for the markup that should]
	 *             be displayed.  null implies standard HTML.
	 */
	public String GetDevice(String strAcceptsHeader, String strUserAgentHeader)
	{
		if (m_htDevicesXMLSettings == null)
		{
			
				log.Error("Unable to get devices xml file.  Returning standard HTML Device.");
			
			return null;
		}


		if (strUserAgentHeader != null)
		{
			//Iterate through the Devices and return the "_PDA" postfix if the header
			//contains a PDA browser
			Object[] oDevices = InternalGetDeviceValues("HTMLDevice");

			for (int i = 0; i < oDevices.length; i++)
			{
				String strAgent = XPConvert.ToString(oDevices[i]);
				if ((strAgent != null) &&
					(strUserAgentHeader.indexOf(strAgent) > -1))
				{
					return ("_HTML32");
				}
			}
		}


		if (strAcceptsHeader != null)
		{
			String strAttrValue = InternalGetDeviceValueAsString("WMLDevice", "Accept");

			if (strAttrValue != null)
			{
				if (strAcceptsHeader.indexOf(strAttrValue) != -1)
				{
					return ("_WML");
				}
			}
		}


		return null;
	}


	/**
	 * This is an internal helper method.
	 *
	 * @param xmlDoc the base xml doc to read the values from
	 * @param _strSection
	 * @param _strSubSection
	 * @param _strAttribute
	 * @param bReportErrors true implies report errors
	 *
	 * @return String null if not found
	 */
	protected String InternalGetDeviceValueAsString(String _strSection, String _strSubSection)
	{
		
			log.Debug("Retrieving device Value from xml doc: " + _strSection + ", " +
				_strSubSection + ".");
		

		String strRetVal = null;

		try
		{
			String strKey = _strSection + SECTION_SUBSECTION_SEPARATOR + _strSubSection;
			strRetVal = XPConvert.ToString(m_htDevicesXMLSettings.GetElement(strKey));
		}
		catch (Exception e)
		{
			// Default error
			
				log.Error(e, "GetDeviceValueAsString() failed.  The section is " + _strSection + " and the subsection is " + _strSubSection + ".");
			

			// Just return null string if we fail to get the value
		}

		return strRetVal;
	}

	/**
	 * This is an internal helper method
	 *
	 * @param xmlDoc the base xml doc to read the values from
	 * @param _strSection
	 * @param _strSubSection
	 * @param _strAttribute
	 * @param bReportErrors true implies report errors
	 *
	 * @return
	 */

	protected Object[] InternalGetDeviceValues(String _strSection)
	{
		
			log.Debug("Retrieving device Values from xml doc: " + _strSection + ".");
		

		XPArrayList alElements = new XPArrayList();
		//Object[] oaReturn = null;

		try
		{
			IXPEnumerator eKeys = m_htDevicesXMLSettings.GetKeys();

			while(eKeys.MoveNext())
			{
				String strKey = XPConvert.ToString(eKeys.GetCurrent());

				if (XPStringUtility.StartsWith(strKey, _strSection))
				{
					alElements.Add(m_htDevicesXMLSettings.GetElement(strKey));
				}
			}
		}
		catch (Exception e)
		{
			/*
			if (xmlRoot == null)
			{
				if (bReportErrors && (log.IsErrorEnabled()))
				{
					log.Error("GetConfigValues() - Couldn't get xmlRoot: " +
						_strSection + ", " + _strSubSection + ", " +
						_strAttribute + ".");
				}
			}
			else if (xmlList == null)
			{
				if (bReportErrors && (log.IsErrorEnabled()))
				{
					log.Error("GetConfigValues() - Couldn't get xmlList.  Category not correct: " +
						_strSection + ", " + _strSubSection + ", " +
						_strAttribute + ".");
				}
			}
			else if (xmlSection == null)
			{
				if (bReportErrors && (log.IsErrorEnabled()))
				{
					log.Error("GetConfigValues() - Couldn't get xmlSection. Category not correct: " +
						_strSection + ", " + _strSubSection + ", " +
						_strAttribute + ".");
				}
			}
			else if (xmlSubList == null)
			{
				if (bReportErrors && (log.IsErrorEnabled()))
				{
					log.Error("GetConfigValues() - Couldn't get xmlSublist.  Subsection not correct: " +
						_strSection + ", " + _strSubSection + ", " +
						_strAttribute + ".");
				}
			}
			else if (xmlAttrElem == null)
			{
				if (bReportErrors && (log.IsErrorEnabled()))
				{
					log.Error("GetConfigValues() - Couldn't get xmlAttrElem: " +
						_strSection + ", " + _strSubSection + ", " +
						_strAttribute + ".");
				}
			}
			*/

				// Default error
			
				log.Error(e, "GetDeviceValues() failed.  The Section was " + _strSection + ".");
			


			// Just return empty string if we fail to get the value
		}

		return alElements.ToArray();
	}
	/**
	 * @see com.plumtree.uiinfrastructure.web.IVarPack#GetVarPackID()
	 */
	public String GetVarPackID()
	{
		return VARPACK_ID;
	}

	/**
	 * This method will return an enumeration of the top level nodes in the XML
	 * tree that is x_config.xml.
	 *
	 * @see com.plumtree.uiinfrastructure.web.IVarPack#GetVarPackKeys()
	 */
	public IXPEnumerator GetVarPackKeys()
	{
		return m_htDevicesXMLSettings.GetKeys();
	}

	/**
	 * You cannot expire the x_config.xml content. You can only reload it. This
	 * method does nothing.
	 *
	 * @see com.plumtree.xpsupport.web.BaseVarPack#ExpireContent()
	 */
	protected void ExpireContent()
	{
	}

	/**
	 * This method will return a chunk of from one level under the PTConfig
	 * node.
	 *
	 * @see com.plumtree.xpsupport.web.BaseVarPack#GetValue(String)
	 */
	protected Object GetValue(String arg0)
	{
		return m_htDevicesXMLSettings.GetElement(arg0);
	}

	/**
	 * This method loads the x_config.xml file into memory.
	 *
	 * @see com.plumtree.xpsupport.web.BaseVarPack#Load()
	 */
	protected void Load()
	{
		/*
		String strFileSeparator = XPFileSystemObject.GetPathSeparator();

		String strDeviceFilePath = GetConfigPath() + strFileSeparator + "device.xml";
		strDeviceFilePath = GeneralHelpers.FormatPath(strDeviceFilePath, strFileSeparator, false);

		 XPDocument xmlDoc = XPBuilder.LoadDocument(strDeviceFilePath);

		 if (xmlDoc == null)
		 {
			 
				 log.Error("Could not load Config XML document from " + strDeviceFilePath);
			 

			 throw new XPException("Could not load device XML document from " + strDeviceFilePath);
		 }
		 */

		 // NOTE DonH: Reloading this will not change the base URLS, because
		 // they are stored in ConfigHelper.  Reloading wouldn't be
		 // threadsafe anyways.

		LoadSettingsIntoXPHashtable();
		 
		log.Info("Finished loading device config XML document.");
	}

	/**
	 * Helper method to load the settings in the portal:Devices component
	 * from portalconfig.xml and put them in a XPHashtable.
	 */
	private void LoadSettingsIntoXPHashtable()
	{
		m_htDevicesXMLSettings = new XPHashtable();

		// Use OpenConfig to read the settings
		IOKContext configContext = AppWarmUp.GetOKConfigContext();
		IOKComponent component = null;
		
		component = configContext.getComponentByType(PORTAL_DEVICES_COMPONENT_TYPE);

		if (null == component)
		{
			log.Warn("Cannot retrieve component type: {0}", PORTAL_DEVICES_COMPONENT_TYPE);
			return;
		}			
			
		IOKSetting[] settings = component.querySettings();
		for(int j=0;j<settings.length;j++)
		{
			String strName = settings[j].getName();
			String strValue = null;
			
			//log.Info("Setting Loaded: " + strName);		        
			//log.Info("Setting type: " + settings[j].getDataType());
		        
			int nSettingType = settings[j].getDataType();
			switch(nSettingType)
			{
				case OKConfigDataTypes.TYPE_BOOLEAN:
					strValue = XPConvert.ToString(settings[j].getBooleanValue());
					break;
				case OKConfigDataTypes.TYPE_STRING:
					strValue = settings[j].getStringValue();
					break;
				case OKConfigDataTypes.TYPE_INTEGER:
					strValue = XPConvert.ToString(settings[j].getIntValue());
					break;
				case OKConfigDataTypes.TYPE_FLOAT:
					strValue = XPConvert.ToString(settings[j].getFloatValue());
					break;
				case OKConfigDataTypes.TYPE_DATE:
					strValue = XPConvert.ToString(settings[j].getDateValue());
					break;
				default:
					throw new XPException("Unknown data type:" + nSettingType);
			}

			//log.Info("Setting value: " + strValue);
				
			if ((null != strName) && (null != strValue))
			{			
				m_htDevicesXMLSettings.PutElement(strName, strValue);
			}				
		}
	}
	
	/**
	 * Helper method to load the settings from x_config.xml and put them in a XPHashtable.
	 * This Hashtable will be used for the Memory Debug page where we need to be able
	 * to iterate through the keys and display all the name/value pairs.
	 * 
	 * @deprecated Use LoadSettingsIntoXPHashtable() instead that uses OpenConfig API
	 */
	private void LoadSettingsIntoXPHashtable(XPDocument xmlDoc)
	{
		// JF- x_config.xml is two levels deep.
		// We need to iterate through both levels before we can actually get the name/value pairs.

		if (xmlDoc == null)
		{
			throw new XPException("Config XML document not found.");
		}

		//XPNodeList nlTopLevel = xmlDoc.GetChildNodes();
		/*
		if (nlTopLevel == null)
		{
			throw new XPException("Config XML document top level nodes not found.");
		}*/

		XPElement nTopNode = xmlDoc.GetDocumentElement();

		
			log.Info("The top root node of devices.xml is " + nTopNode.GetNodeName());
		

		if (nTopNode == null)
		{
			throw new XPException("Config XML document top node not found.");
		}

		XPNodeList nlFirstLevel = nTopNode.GetChildNodes();
		if (nlFirstLevel == null)
		{
			throw new XPException("Config XML document first level nodes not found.");
		}

		m_htDevicesXMLSettings = new XPHashtable();
		XPNodeList nlSecondLevel;
		String sName, sValue, sFirstLevelName;

		for (int i = 0; i < nlFirstLevel.GetLength(); i ++)
		{
			// JF- Make sure this is a valid node and not a comment
			if (!nlFirstLevel.Item(i).GetNodeName().startsWith("#"))
			{
				nlSecondLevel = nlFirstLevel.Item(i).GetChildNodes();
				sFirstLevelName = nlFirstLevel.Item(i).GetNodeName();

				for (int k = 0; k < nlSecondLevel.GetLength(); k ++)
				{
					sName = nlSecondLevel.Item(k).GetNodeName();

					// JF- Make sure this is a valid node and not a comment
					if (!sName.startsWith("#"))
					{
						String sKey = sFirstLevelName + sName;
						sValue = ((XPElement) nlSecondLevel.Item(k)).GetAttributeNode(ConfigConstants.VALUE_ATTRIBUTE).GetNodeValue();
						m_htDevicesXMLSettings.PutElement(sKey, sValue);
					}
				}
			}
		}
	}

	/**
	 * This method reloads the x_config.xml file into memory. It delegates to
	 * Load().
	 *
	 * @see com.plumtree.xpsupport.web.BaseVarPack#Reload()
	 */
	protected void Reload()
	{
	}
}

