package com.plumtree.uiinfrastructure.application.varpacks;

import com.plumtree.openlog.*;
import com.plumtree.openfoundation.io.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.xml.dom.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.web.*;
import com.plumtree.uiinfrastructure.application.*;
import com.plumtree.openkernel.config.*;

/**
 * This VarPack is used to store the list of secure ASs. The list is loaded by
 * the WarmUp Servlet from SecureActivitySpaces.xml. In Security Mode 1,
 * Activity Spaces in this list will be automatically redirected to https.
 * 
 * @author John Osborne
 */

public class SecureASVarPack extends BaseVarPack
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.application.varpacks.SecureASVarPack");

	/** The string ID of this variable package. */
	public static final String VARPACK_ID = "SecureASVarPack";

	// OpenConfig component type
	public static final String PORTAL_SECURESPACES_COMPONENT_TYPE = "http://www.plumtree.com/config/component/types/portal/secureactivityspaces";
	
	/** Constant for the entry name in the xml file. */
	private static final String XML_ENTRY_SPACE = "space";

	/** Constant for the entry name in the xml file. */
	private static final String XML_ENTRY_NAME = "name";

	/** Holds a hashtable of the secure Activity Spaces. */
	private XPHashtable m_htSecureASList;

	/**
	 * @see com.plumtree.uiinfrastructure.web.IVarPack#GetVarPackID()
	 */
	public String GetVarPackID()
	{
		return VARPACK_ID;
	}

	/**
	 * This method will return an enumeration of the secure Activity Spaces.
	 * 
	 * @see com.plumtree.uiinfrastructure.web.IVarPack#GetVarPackKeys()
	 */
	public IXPEnumerator GetVarPackKeys()
	{
		if (m_htSecureASList == null)
		{
			return null;
		}
		else
		{
			return m_htSecureASList.GetKeys();
		}
	}

	/**
	 * Returns true if the AS identified by the name in argument is in the list
	 * and should be secured.
	 *
	 * @param _sASName
	 * 
	 * @return boolean
	 */
	public boolean IsSpaceSecured(String _sASName)
	{
		if (m_htSecureASList == null)
		{
			throw new XPException("The Secure AS has not been initialized.");
		}

		if (m_htSecureASList.ContainsKey(_sASName))
		{
			return true;
		}
 
		return false;
	}

	/**
	 * You cannot the secure space list.
	 *
	 * @see com.plumtree.xpsupport.web.BaseVarPack#ExpireContent()
	 */
	protected void ExpireContent()
	{

	}

	/**
	 * This method will return the name of the secure space if it is in this
	 * variable package.
	 * 
	 * @see com.plumtree.xpsupport.web.BaseVarPack#GetValue(String)
	 */
	protected Object GetValue(String arg0)
	{
		try {
			if (m_htSecureASList == null)
			{
				return null;
			}

			return m_htSecureASList.GetElement(arg0);
		} catch (XPException e)
		{
			
			   log.Error("SecureASVarPack::GetValue Unable to get the value for key: " + arg0 + " The following error ocurred: " + e);
    
			 
			
		}
		
		return null;
		
	}

	/**
	 * This method loads the secure space list into memory.
	 *
	 * @see com.plumtree.xpsupport.web.BaseVarPack#Load()
	 */
	protected void Load()
	{
		/*
		String strFileSeparator = XPFileSystemObject.GetPathSeparator();
		String strSecurityFilePath = GetConfigPath() + strFileSeparator + "SecureActivitySpaces.xml";
		
		strSecurityFilePath = GeneralHelpers.FormatPath(strSecurityFilePath, strFileSeparator, false);
		XPDocument xmlDoc = XPBuilder.LoadDocument(strSecurityFilePath);
		*/
		
		LoadASIntoXPHashtable();
	}

	/**
	 * This method reloads the x_config.xml file into memory.  It clears the
	 * current data rather than delegating to Load().
	 * 
	 * @see com.plumtree.xpsupport.web.BaseVarPack#Reload()
	 */
	protected void Reload()
	{
		// JF- Reset member variables
		m_htSecureASList = null;

		// JF- Reload the values
		Load();
	}

	/**
	 * Helper method to load the Activity Space list from the XML DOM into an
	 * XPHashtable.
	 * 
	 * @throws XPException
	 */
	private void LoadASIntoXPHashtable()
	{
		m_htSecureASList = new XPHashtable();
		
		// Use OpenConfig to read the settings
		IOKContext configContext = AppWarmUp.GetOKConfigContext();
		IOKComponent component = null;
		
		component = configContext.getComponentByType(PORTAL_SECURESPACES_COMPONENT_TYPE);

		if (null == component)
		{
			log.Warn("Cannot retrieve component type: {0}", PORTAL_SECURESPACES_COMPONENT_TYPE);
			return;
		}			
			
		IOKSetting[] settings = component.querySettings();
		for(int j=0;j<settings.length;j++)
		{
			String strName = settings[j].getName();
			
			log.Debug("Secure activity space: " + strName);		        			
		        				
			if (null != strName)
			{
				m_htSecureASList.PutElement(strName, strName);
			}				
		}

	}
	
	/**
	 * Helper method to load the Activity Space list from the XML DOM into an
	 * XPHashtable.
	 * 
	 * @param xmlDoc The document that contains the secure space list.
	 * 
	 * @throws XPException
	 * 
	 * @deprecated Use LoadASIntoXPHashtable() instead, which uses OpenConfig API.
	 */
	private void LoadASIntoXPHashtable(XPDocument xmlDoc)
	{
	  if (xmlDoc == null)
	  {
		throw new XPException("LoadASIntoXPHashtable called with null parameter.");
        
	  }
	  XPElement nTopNode = xmlDoc.GetDocumentElement();
	  
		log.Info("The top root node is " + nTopNode.GetNodeName());
        
	  
	  if (nTopNode == null)
	  {
		throw new XPException("The Secure AS list XML document top node not found.");
	  }
     
	  XPElement xpElem = xmlDoc.GetDocumentElement();
	  XPNodeList nlFirstLevel = xpElem.GetElementsByTagName(XML_ENTRY_SPACE);
	  if (nlFirstLevel == null)
	  {
		throw new XPException("The Secure AS list XML document first level nodes not found.");
        
	  }
	  m_htSecureASList = new XPHashtable();
	  for (int i = 0;i < nlFirstLevel.GetLength();i++)
	  {
		if (!XPStringUtility.StartsWith(nlFirstLevel.Item(i).GetNodeName(),
		 "#"))
		{
		  String strKey = ((XPElement) nlFirstLevel.Item(i)).GetAttributeNode(XML_ENTRY_NAME).GetNodeValue();
		  String strValue = strKey;
		  m_htSecureASList.PutElement(strKey, strValue);
		}
	  }
	}
}

