package com.plumtree.uiinfrastructure.application.varpacks;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.application.*;
import com.plumtree.uiinfrastructure.web.*;
import com.plumtree.openkernel.config.*;

/**
 * This VarPack is used to determine which Activity Space to go
 * to if the space is not on the URL, based on what parameters
 * are available on the URL. 
 * 
 * @author Ricky Wong
 */
public class URLASMappingsVarPack extends BaseVarPack
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.application.varpacks.URLASMappingsVarPack");

	/** The String ID of this VarPack. */
	public static final String VARPACK_ID = "URLASMappingsVarPack";

	/** The XML file this VarPack loads. */
	public static final String VARPACK_FILE = "URLASMappings.xml";

	// OpenConfig component type
	public static final String PORTAL_URLASMAPPINGS_COMPONENT_TYPE = "http://www.plumtree.com/config/component/types/portal/urlasmappings";

	/** Parameter specific mappings. */
	private XPHashtable m_htParameterMap;
	
	/** Node names from our XML file. */
	private static final String NODE_PARAMETER = "ParameterMaps";

	/* (non-Javadoc)
	 * @see com.plumtree.uiinfrastructure.web.IVarPack#GetVarPackID()
	 */
	public String GetVarPackID()
	{
		return VARPACK_ID;
	}

	/* (non-Javadoc)
	 * @see com.plumtree.uiinfrastructure.application.varpacks.XMLBaseVarPack#GetVarPackXMLFileName()
	 */
	public String GetVarPackXMLFileName()
	{
	    return VARPACK_FILE;
	}

	/**
	 * Loads in the same manner as XMLBaseVarPack, but then we
	 * organize the data for URLASMapping specific purposes.
	 */
	protected void Load()
	{  	
		// initialize storage members.
		m_htParameterMap = new XPHashtable();

		// Use OpenConfig to read the settings
		IOKContext configContext = AppWarmUp.GetOKConfigContext();
		IOKComponent component = null;
		
		component = configContext.getComponentByType(PORTAL_URLASMAPPINGS_COMPONENT_TYPE);

		if (null == component)
		{
			log.Warn("Cannot retrieve component type: {0}", PORTAL_URLASMAPPINGS_COMPONENT_TYPE);
			return;
		}			
			
		IOKSetting[] settings = component.querySettings();
		for(int j=0;j<settings.length;j++)
		{
			String strName = settings[j].getName();
			String strValue = null;
			
			// log.Info("Setting Loaded: " + strName);		        
		        
			int nSettingType = settings[j].getDataType();
			
			switch(nSettingType)
			{
				// the value is a space name, so only TYPE_STRING should be supported
				case OKConfigDataTypes.TYPE_STRING:
					strValue = settings[j].getStringValue();
					break;
				default:
					throw new XPException("Unknown data type:" + nSettingType);
			}

			// log.Info("Setting value: " + strValue);
				
			if ((null != strName) && (null != strValue))
			{			
				m_htParameterMap.PutElement(strName, strValue);
			}
		}		
	}
	
	/**
	 * Returns an enumeration of the Parameter map keys.
	 * 
	 * @return IXPEnumerator of the Parameter map keys.
	 */
	public IXPEnumerator GetParameterMapKeys()
	{
	    if (null != m_htParameterMap)
	    {
	        return m_htParameterMap.GetKeys();
	    }
	    else
	    {
	        return null;
	    }
	}
	
	/**
	 * Returns the Parameter map value for the given key.
	 * 
	 * @param strKey The Parameter key.
	 * @return The Parameter value.
	 */
	public String GetParameterMapValue(String strKey)
	{
	    if (null != m_htParameterMap)
	    {
	        return XPConvert.ToString(m_htParameterMap.GetElement(strKey));	        
	    }
	    else
	    {
	        return null;
	    }
	}
	
	/**
	 * This method will return an enumeration of the top level nodes in the XML
	 * tree that is x_config.xml.
	 *
	 * @see com.plumtree.uiinfrastructure.web.IVarPack#GetVarPackKeys()
	 */
	public IXPEnumerator GetVarPackKeys()
	{
		if (null != m_htParameterMap)
		{
			return m_htParameterMap.GetKeys();
		}
		else
		{
			return null;
		}
	}

	/**
	 * This method will return a chunk of from one level under the PTConfig
	 * node.
	 *
	 * @see com.plumtree.xpsupport.web.BaseVarPack#GetValue(String)
	 */
	protected Object GetValue(String arg0)
	{
		if (null != m_htParameterMap)
		{
			return XPConvert.ToString(m_htParameterMap.GetElement(arg0));	        
		}
		else
		{
			return null;
		}
	}
	
	/**
	 * You cannot expire the x_config.xml content. You can only reload it. This
	 * method does nothing.
	 *
	 * @see com.plumtree.xpsupport.web.BaseVarPack#ExpireContent()
	 */
	protected void ExpireContent()
	{
	}
	
	/**
	 * This method reloads the XML file into memory. It delegates to
	 * Load().  This may not be entirely threadsafe.
	 *
	 * @see com.plumtree.uiinfrastructure.web.BaseVarPack#Reload()
	 */
	protected void Reload()
	{
		Load();
	}
}

