package com.plumtree.uiinfrastructure.application.varpacks;

import com.plumtree.openlog.*;
import com.plumtree.openfoundation.io.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.xml.dom.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.web.*;

/**
 * This class is designed to read in Name/Value pairs for an xml file and store them
 * in a hashtable for easy lookup.  The XML file is assumed to have a single root
 * node with multiple section nodes underneath it.  The section nodes contain multiple
 * Name/Value pair nodes where the name of the node is the name and the value
 * attribute is the value (i.e. <NodeName value="NodeValue"/>).  These will be read
 * into an internal hashtable and will be accessible by calling
 * VarPack.GetValue("SectionNameNodeName").  For example, if there is a section called
 * "Timeouts" and a Node in that section that looks like this: <DocumentTimeout value="17"/>
 * that value should be retrieved by calling VarPack.GetValueAsInt("Timeout", "DocumentTimeout"),
 * which returns an int.  You can also use VarPack.GetValue("TimeoutDocumentTimeout")
 * which will return an Object, but that is not the preferred method.
 */
abstract public class XMLBaseVarPack extends BaseVarPack
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.application.varpacks.XMLBaseVarPack");

	/** This hashtable holds the Name/Value pairs loaded from the XML file. */
	private XPHashtable m_htXMLNameValuePairs;

	/** The marker for XML comments */
	protected static final String STR_COMMENT_PREFIX = "#";

	/** The value attribute */
	protected static final String VALUE_ATTRIBUTE = "value";

	/**
	 * This method returns the name that the VarPack should be stored under on the application.
	 * 
	 * @see com.plumtree.uiinfrastructure.web.IVarPack#GetVarPackID()
	 */
	abstract public String GetVarPackID();

	/**
	 * This method returns the name of the XML file that this VarPack should load.
	 * This should be the full name including the ".xml" suffix.  The file will be
	 * loaded from the the portal config directory (ptportal\5.0\settings\config)
	 * and should be placed there.
	 * 
	 * @return String the full name of the xml file to load, without directory path.
	 */
	abstract public String GetVarPackXMLFileName();

	/**
	 * This method will return the String value for a named node in a section
	 * under the root XML node.
	 *
	 * @param String strSection the name of the section that the Name/Value pair
	 * 	is located in.
	 * @param String strName the name of the Name/Value pair.
	 * @return String the String value of the Name/Value pair.
	 * 
	 * @throws XPException if the Value cannot be converted to a String.
	 */
	public String GetValueAsString(String strSection, String strName)
	{
		return XPConvert.ToString(GetValue(strSection + strName));
	}

	/**
	 * This method will return the int value for a named node in a section
	 * under the root XML node.
	 *
	 * @param String strSection the name of the section that the Name/Value pair
	 * 	is located in.
	 * @param String strName the name of the Name/Value pair.
	 * @return int the int value of the Name/Value pair.
	 * 
	 * @throws XPException if the Value cannot be converted to an int.
	 */
	public int GetValueAsInt(String strSection, String strName)
	{
		return XPConvert.ToInteger(GetValue(strSection + strName));
	}

	/**
	 * This method will return the boolean value for a named node in a section
	 * under the root XML node.
	 *
	 * @param String strSection the name of the section that the Name/Value pair
	 * 	is located in.
	 * @param String strName the name of the Name/Value pair.
	 * @return boolean the boolean value of the Name/Value pair.
	 * 
	 * @throws XPException if the Value cannot be converted to a boolean.
	 */
	public boolean GetValueAsBoolean(String strSection, String strName)
	{
		return XPConvert.ToBoolean(GetValue(strSection + strName));
	}

	/**
	 * This method will return the String value for a named node in a section
	 * under the root XML node.
	 * 
	 * GetValueAsString, GetValueAsInt, and GetValueAsBoolean are the preferred
	 * methods of accessing the data in this VarPack, and should be used instead
	 * of this method.
	 * 
	 * @param String Concatenation of the Section name and the Name/Value pair name.
	 *
	 * @see com.plumtree.uiinfrastructure.web.BaseVarPack#GetValue(String)
	 */
	protected Object GetValue(String strSectionAndName)
	{
		return m_htXMLNameValuePairs.GetElement(strSectionAndName);
	}

	/**
	 * This method will return an enumeration of the top level nodes in the XML
	 * tree that was loaded by this VarPack.  This is used by the Memory Debug
	 * page to query for the contents of the VarPack.
	 *
	 * @see com.plumtree.uiinfrastructure.web.IVarPack#GetVarPackKeys()
	 */
	public IXPEnumerator GetVarPackKeys()
	{
		return m_htXMLNameValuePairs.GetKeys();
	}

	/**
	 * You cannot expire the XML content. You can only reload it. This
	 * method does nothing.
	 *
	 * @see com.plumtree.uiinfrastructure.web.BaseVarPack#ExpireContent()
	 */
	protected void ExpireContent()
	{
	}

	/**
	 * This method loads the XML file into the hashtable in memory.
	 *
	 * @see com.plumtree.uiinfrastructure.web.BaseVarPack#Load()
	 */
	protected void Load()
	{
		
			log.Debug("Loading " + GetVarPackID() + " VarPack.");
		

		String strFileSeparator = XPFileSystemObject.GetPathSeparator();

		String strXMLFilePath = GetConfigPath() + strFileSeparator + GetVarPackXMLFileName();

		strXMLFilePath = GeneralHelpers.FormatPath(strXMLFilePath, strFileSeparator, false);

		XPDocument xmlDoc = XPBuilder.LoadDocument(strXMLFilePath);

		if (xmlDoc == null)
		{
			
				log.Error("VarPack " + GetVarPackID() + " could not load XML document from " +
					strXMLFilePath);
			

			throw new XPException("VarPack " + GetVarPackID() +
				" could not load XML document from " + strXMLFilePath);
		}

		LoadSettingsIntoXPHashtable(xmlDoc);

		
			log.Debug("Finished loading " + GetVarPackID() + " XML document from " +
				strXMLFilePath + ".");
		
	}

	/**
	 * Helper method to load the settings from the XML file and put them in a XPHashtable.
	 * This Hashtable will be used for the Memory Debug page where we need to be able
	 * to iterate through the keys and display all the name/value pairs.
	 * 
	 * This can be overridden if the XML file has a different format.
	 */
	protected void LoadSettingsIntoXPHashtable(XPDocument xmlDoc)
	{
		// XML file is assumed to be two levels deep.
		// We need to iterate through both levels before we can actually get the name/value pairs.

		if (xmlDoc == null)
		{
			throw new XPException("VarPack " + GetVarPackID() + " XML document (" +
				GetVarPackXMLFileName() + ") not found.");
		}

		// Get the document root node
		XPElement nTopNode = xmlDoc.GetDocumentElement();

		if (nTopNode == null)
		{
			throw new XPException("VarPack " + GetVarPackID() + " XML document (" +
				GetVarPackXMLFileName() + ") root node not found.");
		}

		// Get the section nodes
		XPNodeList nlFirstLevel = nTopNode.GetChildNodes();
		if (nlFirstLevel == null)
		{
			throw new XPException("VarPack " + GetVarPackID() + " XML document (" +
				GetVarPackXMLFileName() + ") first level nodes not found.");
		}

		// Temporary hashtable for the Name/Value pairs.
		XPHashtable htXMLNameValuePairs = new XPHashtable();

		XPNodeList nlSecondLevel;
		String sName, sValue, sKey, sFirstLevelName;

		// Go through the first level nodes (sections) and load all of the Name/Value pairs.
		for (int i = 0; i < nlFirstLevel.GetLength(); i ++)
		{
			// Make sure this is a valid node and not a comment.
			if (!nlFirstLevel.Item(i).GetNodeName().startsWith(STR_COMMENT_PREFIX))
			{
				nlSecondLevel = nlFirstLevel.Item(i).GetChildNodes();
				sFirstLevelName = nlFirstLevel.Item(i).GetNodeName();

				// Go through the Name/Value pairs and add them into the hashtable.
				for (int j = 0; j < nlSecondLevel.GetLength(); j ++)
				{
					sName = nlSecondLevel.Item(j).GetNodeName();

					// Make sure this is a valid node and not a comment.
					if (!sName.startsWith(STR_COMMENT_PREFIX))
					{
						// Construct the key and store the value in the hashtable.
						sKey = sFirstLevelName + sName;
						sValue = ((XPElement) nlSecondLevel.Item(j)).GetAttributeNode(VALUE_ATTRIBUTE).GetNodeValue();
						htXMLNameValuePairs.PutElement(sKey, sValue);
					}
				}
			}
		}

		// Move the temporary hashtable into the VarPack.
		// NOTE: This may not be entirely threadsafe.
		m_htXMLNameValuePairs = htXMLNameValuePairs;
	}

	/**
	 * This method reloads the XML file into memory. It delegates to
	 * Load().  This may not be entirely threadsafe.
	 *
	 * @see com.plumtree.uiinfrastructure.web.BaseVarPack#Reload()
	 */
	protected void Reload()
	{
		Load();
	}
}

