package com.plumtree.uiinfrastructure.approval;

import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.openlog.*;
import com.plumtree.openfoundation.util.*;

/**
 * @author Ben Geislinger
 *
 * This abstarct class represents an item awaiting approval.
 * <br><br>
 * Knows concrete subclasses are DocumentItemDescriptor and
 *     ObjectItemDescriptor.
 *
 */
public abstract class AItemDescriptor
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.approval.AItemDescriptor");

	protected static int iPrimaryID  = 0;
	protected AActivitySpace mySpace = null;
	private ApprovalItemType myType = null;
	private Redirect editRedirect   = null;
	private String sComment     = "";
	private String sDescription;
	private String sEditHRef = null;
	private String sImgUUID = null;
	private String sID;
	private String sName;
	private String sPreviewHRef = null;
	private String sSubmittedBy;
	private XPArrayList alInstances;
	private XPDateTime dSubmitted;
	private XPHashtable mapInstances;
	private boolean bApprove = false;
	private boolean bClear  = false;
	private boolean bHold   = true;
	private boolean bReject = false;

	/**
	 * Creates a new AItemDescriptor object.
	 *
	 * @param name String name of the object awaiting approval
	 * @param description String description of the object awaiting approval
	 * @param submittedBy String
	 * @param submittedDate XPDateTime
	 */
	protected AItemDescriptor(String name, String description,
		String submittedBy, XPDateTime submittedDate, AActivitySpace space,
		ApprovalItemType type)
	{
		sDescription     = description;
		sName			 = name;
		sSubmittedBy     = submittedBy;
		dSubmitted		 = submittedDate;
		mySpace			 = space;
		myType			 = type;

		sID = type.GetName() + "_IT_" + XPConvert.ToString(iPrimaryID);
		iPrimaryID++;
	}

	/**
	 * Call this method to add an AInstanceDescriptor to this AItemDescriptor
	 *
	 * @param instance AInstanceDescriptor
	 */
	public void AddInstance(AInstanceDescriptor instance)
	{
		if (alInstances == null)
		{
			alInstances		 = new XPArrayList();
			mapInstances     = new XPHashtable();
		}

		
			log.Debug("AddInstance() - My ID: " + sID);
			log.Debug("AddInstance() - Instance ID: " + instance.GetID());
		

		alInstances.Add(instance);

		mapInstances.PutElement(instance.GetID(), instance);
	}

	/**
	 *
	 */
	public abstract void Execute();

	/**
	 *
	 */
	public void Approve()
	{
		
			log.Debug("ENTERING Approve()");
		

		bApprove     = true;
		bReject		 = false;
		bHold		 = false;

		IXPEnumerator eInstances = this.GetInstances();
		AInstanceDescriptor inst;

		if (eInstances != null)
		{
			while (eInstances.MoveNext())
			{
				inst = (AInstanceDescriptor) eInstances.GetCurrent();

				inst.Approve();
			}
		}
	}

	/**
	 *
	 */
	public void ClearMigrationRequest()
	{
		
			log.Debug("ClearMigrationRequest()");
		

		bApprove     = false;
		bReject		 = false;
		bHold		 = false;
		bClear		 = true;
	}

	/**
	 *
	 *
	 * @return
	 */
	public boolean GetClear()
	{
		return bClear;
	}


	public void SetClear( boolean b )
	{
		bClear = b;
	}


	/**
	 *
	 *
	 * @return
	 */
	public String GetComment()
	{
		return sComment;
	}

	/**
	 *
	 *
	 * @return
	 */
	public XPDateTime GetDateSubmitted()
	{
		return dSubmitted;
	}

	/**
	 * Call this method to obtain the description of this item
	 *
	 * @return String
	 */
	public String GetDescription()
	{
		return sDescription;
	}

	/**
	 * Call this method to get the HREF that will allow users to edit
	 *     the object represented by this AItemDescriptor.
	 *
	 * @return String
	 */
	public String GetEditHRef()
	{
		return sEditHRef;
	}

	/**
	 *
	 *
	 * @return
	 */
	public Redirect GetEditRedirect()
	{
		return editRedirect;
	}

	/**
	 *
	 */
	public boolean GetHold()
	{
		return bHold;
	}

	/**
	 * Call this method to get the ID of this item.
	 *
	 * @return String id
	 */
	public String GetID()
	{
		return sID;
	}


	public String GetImgUUID()
	{
		return sImgUUID;
	}

	public void SetImgUUID( String s )
	{
		sImgUUID = s;
	}


	/**
	 * Call this method to obtain a reference to the specified AInstance
	 *     descriptor.
	 *
	 * @param sID String ID
	 * @return AInstanceDescriptor
	 */
	public AInstanceDescriptor GetInstance(String sID)
	{
		AInstanceDescriptor iTemp = null;

		if (mapInstances != null)
		{
			iTemp = (AInstanceDescriptor) mapInstances.GetElement(sID);
		}

		return iTemp;
	}

	/**
	 * Call this method to obtain an IXPEnumerator of all AInstanceDescriptors
	 *     added to this item descriptor.
	 *
	 * @return IXPEnumerator of AInstanceDescriptors
	 */
	public IXPEnumerator GetInstances()
	{
		IXPEnumerator enumerator = null;

		if (alInstances != null)
		{
			enumerator = alInstances.GetEnumerator();
		}

		return enumerator;
	}

	/**
	 * Call this method to obtain the name of this item.
	 *
	 * @return String
	 */
	public String GetName()
	{
		return sName;
	}



	/**
	 * Call this method to get the HREF that will allow users to preview
	 *     the object represented by this AItemDescriptor.
	 *
	 * @return String
	 */
	public String GetPreviewHRef()
	{
		return sPreviewHRef;
	}

	/**
	 *
	 *
	 * @return
	 */
	public abstract String GetStatus();

	/**
	 *
	 */
	public boolean GetApprove()
	{
		return bApprove;
	}

	/**
	 *
	 */
	public boolean GetReject()
	{
		return bReject;
	}

	/**
	 *
	 *
	 * @return
	 */
	public String GetSubmittedBy()
	{
		return sSubmittedBy;
	}

	/**
	 *
	 *
	 * @return
	 */
	public ApprovalItemType GetType()
	{
		return myType;
	}

	/**
	 *
	 */
	public void Hold()
	{
		
			log.Debug("ENTERING Hold()");
		

		bApprove     = false;
		bReject		 = false;
		bHold		 = true;

		IXPEnumerator eInstances = this.GetInstances();
		AInstanceDescriptor inst;

		if (eInstances != null)
		{
			while (eInstances.MoveNext())
			{
				inst = (AInstanceDescriptor) eInstances.GetCurrent();

				inst.Hold();
			}
		}
	}

	/**
	 *
	 */
	public void Reject()
	{
		
			log.Debug("ENTERING Reject()");
		

		bApprove     = false;
		bReject		 = true;
		bHold		 = false;

		IXPEnumerator eInstances = this.GetInstances();
		AInstanceDescriptor inst;

		if (eInstances != null)
		{
			while (eInstances.MoveNext())
			{
				inst = (AInstanceDescriptor) eInstances.GetCurrent();

				inst.Reject();
			}
		}
	}

	/**
	 *
	 *
	 * @param s
	 */
	public void SetComment(String s)
	{
		sComment = s;
	}


	public void SetName( String s )
	{
		sName = s;
	}

	/**
	 *
	 *
	 * @param s
	 */
	public void SetDescription(String s)
	{
		sDescription = s;
	}

	/**
	 * Call this method to set the HREF that will allow users to edit
	 *     the object represented by this AItemDescriptor.
	 *
	 * @param s String
	 */
	public void SetEditHRef(String s)
	{
		sEditHRef = s;
	}

	/**
	 *
	 *
	 * @param r
	 */
	public void SetEditRedirect(Redirect r)
	{
		editRedirect = r;
	}

	/**
	 * Call this method to set the HREF that will allow users to preview
	 *     the object represented by this AItemDescriptor.
	 *
	 * @param s String
	 */
	public void SetPreviewHRef(String s)
	{
		sPreviewHRef = s;
	}
}

