package com.plumtree.uiinfrastructure.common;

import com.plumtree.openfoundation.util.XPHashtable;
import com.plumtree.openfoundation.util.XPSystem;

/**
 * The TimeTracker Class is used to measure time differences, in milliseconds.
 * 
 * @author scottc
 * Created on May 3, 2003
 */
public class TimeTracker
{
	private long m_startTime;

	private XPHashtable m_checkpoints;

	public TimeTracker()
	{
		m_startTime = XPSystem.GetCurrentTimeMilliSec();
	}

	/**
	 * The TimeTracker is reset.  Alternative to creating a new one.
	 */
	public void Reset()
	{
		m_startTime = XPSystem.GetCurrentTimeMilliSec();
		m_checkpoints = null;
	}
	
	/**
	 * The time in milliseconds since the TimeTracker creation or the last Reset()
	 * is returned.
	 * 
	 * @return long
	 */
	public long GetTotalTime()
	{
		return XPSystem.GetCurrentTimeMilliSec() - m_startTime;
	}

	/**
	 * SetCheckpoint is used to create a time checkpiont for later retrieval.
	 * The checkpoint is identified by a string, and stored in an internal
	 * XPHashtable with that string as a key.
	 * 
	 * @param checkpoint
	 */
	public void SetCheckpoint(String checkpoint)
	{
		if (m_checkpoints == null)
		{
			m_checkpoints = new XPHashtable();
		}

		m_checkpoints.PutElement(checkpoint,
			new TimeContainer(XPSystem.GetCurrentTimeMilliSec()));
	}
	
	/**
	 * GetCheckpointTimeFromStart attempts to retrieve a checkpoint with the key given.
	 * It returns -1 if there is no checkpoint by the given name.
	 * It returns the time in milliseconds between the creation of the TimeTracker 
	 * or its last Reset() and the checkpoint time.
	 * 
	 * @param checkpoint
	 * @return long the time between the start of the time and the checkpoint
	 */
	public long GetCheckpointTimeFromStart(String checkpoint)
	{
		if (m_checkpoints == null)
		{
			return -1;
		}
		else
		{

			TimeContainer myTimeContainer =
				(TimeContainer) m_checkpoints.GetElement(checkpoint);

			if (myTimeContainer == null)
			{
				return -1;
			}
			else
			{
				return myTimeContainer.GetTime() - m_startTime;
			}
		}
	}

	/**
	 * GetCheckpointTimeFromStart attempts to retrieve the two checkpoints with
	 * the keys given.
	 * It returns -1 if either of the checkpoints do not exist.
	 * It returns the time in milliseconds between the start checkpoint and the
	 * end checkpoint time.
	 * 
	 * @param startCheckpoint
	 * @param endCheckpoint
	 * @return long the time between the two checkpoints
	 */
	public long GetCheckpointTimeFromCheckpoint(String startCheckpoint, String endCheckpoint)
	{
		if (m_checkpoints == null)
		{
			return -1;
		}
		else
		{

			TimeContainer myStartTimeContainer =
				(TimeContainer) m_checkpoints.GetElement(startCheckpoint);

			if (myStartTimeContainer == null)
			{
				return -1;
			}

			TimeContainer myEndTimeContainer =
				(TimeContainer) m_checkpoints.GetElement(endCheckpoint);

			if (myEndTimeContainer == null)
			{
				return -1;
			}

			return myEndTimeContainer.GetTime() - myStartTimeContainer.GetTime();
		}
	}
}

