package com.plumtree.uiinfrastructure.constants;

/**
 * AccessStyles is an emumeration of the various access types allowed.
 *
 * @author John Osborne
 */
public class AccessStyles
{
	/**
	 * internal constants for our conversion function.  These correspond to
	 * the values used in the portal.
	 */
	private static final int nSection508   = 1;
	private static final int nLowBandwidth = 2;
	private static final int nStandard     = 3;

	/** The standard portal style */
	public static final AccessStyles STANDARD = new AccessStyles(nStandard);

	/** The style that is 508 complant */
	public static final AccessStyles SECTION_508 = new AccessStyles(nSection508);

	/** A 508-like style UI that allows for regular gadgets to be displayed */
	public static final AccessStyles LOW_BANDWIDTH = new AccessStyles(nLowBandwidth);

	/** The integer ID that specifies the access style */
	private int m_nAccessID;

	/**
	 * Creates a new AccessStyles object.
	 *
	 * @param _nAccessID
	 */
	private AccessStyles(int _nAccessID)
	{
		m_nAccessID = _nAccessID;
	}

	/**
	 * This converts an int into an Access Style.  This is necessary because
	 * the access style comes from the portal, where it is stored as an int,
	 * rather than an enumerator.
	 *
	 * @return AccessStyles the enumerator value of the int.  null if there was an error
	 */
	public static AccessStyles GetAccessStyle(int nAccessStyle)
	{
		switch (nAccessStyle)
		{
		case nStandard:
			return STANDARD;

		case nSection508:
			return SECTION_508;

		case nLowBandwidth:
			return LOW_BANDWIDTH;

		default:

			// Couldn't determine the correct style, return standard as default.
			return STANDARD;
		}
	}

	/**
	 * Gives back the integer representation of the access style.
	 * NOTE: This is the right way to work; the above method is for debug
	 * purposes.
	 *
	 * @return An integer version of the access style.
	 */
	public int GetAccessID()
	{
		return m_nAccessID;
	}

	/**
	 * Gives back the string representation of the access style.
	 *
	 * @return A string version of the access style. For example, if the mode
	 *         is SECTION_508, you get back the string: "SECTION_508". In the
	 *         event of an error, null is returned.
	 */
	public String ToString()
	{
		switch (m_nAccessID)
		{
		case nStandard:
			return "STANDARD";

		case nSection508:
			return "SECTION_508";

		case nLowBandwidth:
			return "LOW_BANDWIDTH";

		default:
			return null;
		}
	}
	
	/**
	 * JF- Helper method to validate an Access ID.
	 * 
	 * @param _nAccessID ID to validate
	 * @return boolean
	 */
	public static boolean IsValidAccessID(int _nAccessID)
	{
		boolean bIsValid = true;
		
		if ((_nAccessID < 1) || (_nAccessID > 3))
		{
			bIsValid = false;
		}
		
		return bIsValid;
	}
	
	/**
	 * Returns the default access ID.
	 * 
	 * @return int
	 */
	public static int GetDefaultAccessID()
	{
		return nStandard;
	}
}

