/*
 * Created on Mar 11, 2005
 */
package com.plumtree.uiinfrastructure.diagnostic.reporter;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.application.notification.*;
import com.plumtree.uiinfrastructure.application.startup.messages.*;
import com.plumtree.uiinfrastructure.util.*;

/**
 * Manages error reporting. Handles line wrapping for command consoles, and 
 * manages event logs for UI reporting.   
 * 
 * @author DavidP
 */
public class Reporter
{
	private static OpenLogger logger = OpenLogService.GetLogger(
			OpenLogService.GetComponent("UI_Infrastructure"),
			"com.plumtree.uiinfrastructure.diagnostic.reporter.Reporter");
    
    public static final int LINEWIDTH	= 60;
    public static final int INDENT1	= 4;
    
    // straight string log of issues
    private static XPArrayList m_reportLog = new XPArrayList();
    
    // more intelligent structure: event reports
    private static XPArrayList m_eventLog = new XPArrayList();
    
    /**
     * Outputs recommendations with descriptions to the console.
     * 
     * @param bSystemOut: if true, logs more verbosely 
     */
    public static void report( boolean bSystemOut, int taskID, int stepID, String taskName, Platform platform, ReturnValue retval, IMessage msgClass)
    {
        String errorMessage = "";
        String errorDescription = "";
        String exceptionDetail = "";
        
        if (msgClass != null)
        {
	        XPArrayList list = msgClass.getMsg(stepID,platform,retval);
	        if (list != null)
	        {
	            IXPEnumerator walker = list.GetEnumerator();

				String msg = "Recommendations follow:";	            

	            // at this time we just grab the first one
	            if (walker.MoveNext())
	            {
					logNoIndent(bSystemOut,msg);
                
					// Simple writeln for now
					// Possibly log to file?
					StartupTaskMessage elt = (StartupTaskMessage)walker.GetCurrent();
					errorMessage = elt.getErrMessage();		        
					linewrap( true, errorMessage, LINEWIDTH, INDENT1 );
		        
					errorDescription = elt.getErrDescription();	
					linewrap( true, errorDescription, LINEWIDTH, INDENT1 );
	            }
			    
			    // is there a more accurate answer?
			    if (retval.getException() != null)
			    {
			        exceptionDetail = RecommendationHelper.ConvertExceptionToRecommendation( retval.getException() );
		            msg = "More Detail:";
		            logNoIndent(bSystemOut,msg);
			        linewrap( bSystemOut, exceptionDetail, LINEWIDTH, INDENT1 );
			    }
	        }
	        else
	        {
	            Integer i = new Integer(stepID);
	            
	            errorMessage = "No further details for step #" + i.toString();
	            logNoIndent(bSystemOut,errorMessage);
	        }
        }
        
        // Only log events if they are errors or worse
        if (retval.getResultCode().getID() >= PT_STARTUP_RESULTCODES.ERROR.getID())
        {
	        Platform thisPlatform = Platform.getCurPlatform();
	        logEvent( taskID, stepID, taskName, thisPlatform.getName(), errorMessage, 
	                errorDescription, exceptionDetail, retval);
        }
    }
    
    /**
     * Simply logs the string to the output.
     * 
     * @param bSystemOut: if true, logs to stdout as well
     * @param line
     */
    public static void logNoIndent( boolean bSystemOut, String line )
    {
        // Simple writeln for now
        // Also adds to a global array
        linewrap( bSystemOut, line, LINEWIDTH, 0 );
    }
    
    /**
     * Simply logs the string to the output.
     * 
     * @param bSystemOut: if true, logs to stdout as well
     * @param line
     */
    public static void log( boolean bSystemOut, String line )
    {
        // Simple writeln for now
        // Also adds to a global array
        linewrap( bSystemOut, line, LINEWIDTH, INDENT1 );
    }
    
    /**
     * Event logger.  Tracks errors for later display.
     */
    public static void logEvent( int taskID, int stepID, String taskName, String platformName, String errorMessage, 
            String errorDescription, String exceptionRecommendation, ReturnValue retVal)            
    {
        EventLog eventItem = new EventLog();
        eventItem.taskID = taskID;
        eventItem.taskStep = stepID;
        eventItem.taskName = taskName;
        eventItem.platformName = platformName;
        eventItem.errorMessage = errorMessage;
        eventItem.errorDescription = errorDescription;
        eventItem.exceptionRecommendation = exceptionRecommendation;
        eventItem.retVal = retVal;
        
        m_eventLog.Add(eventItem);
    }
    
	/**
	 * Formats strings into N-char chunks with indent; breaks on word boundary
	 * 
	 * @param line
	 */
	public static void linewrap( boolean bSystemOut, String line, int width, int indent )
	{
	    int linePos = 0;
	    String currentLine = "";	    
	    String indentStr = "";
	
	    if (line == null || line.length() < 2)
	        return;

	    // to spy
	    if (bSystemOut)
	        logger.Info(line);
	 
	    // break up into 80 char chunks for systemout
	    for (int loop = 0; loop < indent; ++loop)
	    {
	        indentStr += " ";
	    }
	    
	    if (bSystemOut)
	        XPSystem.Out.Write(indentStr);
	        
	    currentLine += indentStr;
	    
	    XPStringTokenizer strTok = new XPStringTokenizer(line," ",false);
	    if (strTok == null)
	    {
		    if (bSystemOut)
		        XPSystem.Out.WriteLine(line);
		    m_reportLog.Add(line);	        
	        return;
	    }
	    
	    // Loop through word tokens	    
	    while(strTok.HasMoreTokens())
	    {
	        String word = strTok.GetNextToken();
	        if (word != null && word.length() > 1)
	        {
		        linePos += word.length();
		        if (linePos > width)
		        {
		            if (bSystemOut)
		                XPSystem.Out.WriteLine(" ");
		            m_reportLog.Add(currentLine);
		            currentLine = "";
		            linePos = word.length() + 1;
		            if (bSystemOut)
		            {
		                XPSystem.Out.Write(indentStr);	            
		                linePos += indentStr.length();
	                }
		            currentLine += indentStr;
		        }
		        if (bSystemOut)
		            XPSystem.Out.Write(word+" ");
		        currentLine += word;
		        currentLine += " ";
	        }
	    }
	    if (bSystemOut)
	        XPSystem.Out.WriteLine(" ");
	    
	    m_reportLog.Add(currentLine);
	}
    
	/**
	 * Formats strings into N-char chunks with indent; breaks on word boundary
	 * 
	 * @param line
	 */
	public static void SystemReport( String line, int width, int indent, String frontMsg )
	{
	    boolean bSystemOut = true;
	    String currentLine = "";
	    String indentStr = "";
	    int linePos = 0;
    
	    if (line == null || line.length() < 2)
	        return;
	    
	    // to spy
	    logger.Info(line);
	 
	    // break into 80 char chunks for systemout
	    for (int loop = 0; loop < indent; ++loop)
	    {
	        indentStr += " ";
	    }
	    
	    if (bSystemOut)
	    {
	        if (frontMsg != null)
	        {
	            XPSystem.Out.Write(frontMsg);
	            linePos += frontMsg.length();
	            currentLine += frontMsg;
	        }
	        else
	        {
	            XPSystem.Out.Write(indentStr);
	            linePos += indentStr.length();
	            currentLine += indentStr;
	        }
	    }
	    
	    XPStringTokenizer strTok = new XPStringTokenizer(line," ",false);
	    if (strTok == null)
	    {
		    if (bSystemOut)
		        XPSystem.Out.WriteLine(line);
		    
		    currentLine += line;
	        return;
	    }

	    // Loop through word tokens
	    while(strTok.HasMoreTokens())
	    {
	        String word = strTok.GetNextToken();
	        if (word != null && word.length() > 1)
	        {
		        linePos += word.length() + 1;
		        if (linePos > width)
		        {
		            if (bSystemOut)
		                XPSystem.Out.WriteLine(" ");
		            m_reportLog.Add(currentLine);
		            currentLine = "";
		            linePos = word.length() + 1;
		            if (bSystemOut)
		            {
		                XPSystem.Out.Write(indentStr);
		                linePos += indentStr.length();
		            }
		            currentLine += indentStr;
		        }
		        if (bSystemOut)
		            XPSystem.Out.Write(word+" ");
		        currentLine += word;
		        currentLine += " ";
	        }
	    }
	    if (bSystemOut)
	        XPSystem.Out.WriteLine(" ");
	}
	
	/**
	 * Returns current report as a string array
	 * @return String[] lines of report
	 */
	public static String[] getReport()
	{
	    try
	    {
	        Object[] arr = m_reportLog.ToArray();
	        int loop = 0;
	        String[] resultList = new String[arr.length+1];
	        for (loop = 0; loop < arr.length; ++loop)
	            resultList[loop] = (String)arr[loop];
	        return resultList;
	    }
	    catch( Exception e )
	    {
	        LogAndWriteError("Could not convert report log to array.");
	    }
	    return null;
	}
	
	/**
	 * Returns current report as an array list of event objects
	 * @return EventLog[] lines of report
	 */
	public static EventLog[] getEventLogReport()
	{
	    try
	    {
	        Object[] arr = m_eventLog.ToArray();
	        int loop = 0;
	        EventLog[] resultList = new EventLog[arr.length+1];
	        for (loop = 0; loop < arr.length; ++loop)
	            resultList[loop] = (EventLog)arr[loop];
	        return resultList;
	    }
	    catch( Exception e )
	    {
	        LogAndWriteError("Could not convert event log to array.");
	    }
	    return null;
	}
	
	/**
	 * Both write to system out and log to spy.
	 * @param msg
	 */
	public static void LogAndWrite( String msg )
	{
	    LogNoCarriageReturns(msg);
	    XPSystem.Out.WriteLine(msg);
	}
	
	/**
	 * Both write to system out and log to spy (error version).
	 * @param msg
	 */
	public static void LogAndWriteError( String msg )
	{
	    LogNoCarriageReturnsError( msg );
	    XPSystem.Out.WriteLine(msg);
	}	
	
	/**
	 * Both write to system out and log to spy (error version).
	 * @param msg
	 */
	public static void LogAndWriteFatal( String msg )
	{
	    LogNoCarriageReturnsFatal( msg );
	    XPSystem.Out.WriteLine(msg);
	}	
	
	/**
	 * Both write to system out and log to spy (error version).
	 * @param msg
	 */
	private static void LogNoCarriageReturns( String msg )
	{
	    logger.Action(StripCarriageReturns(msg));
	}	
	
	/**
	 * Both write to system out and log to spy (error version).
	 * @param msg
	 */
	private static void LogNoCarriageReturnsError( String msg )
	{
	    logger.Error(StripCarriageReturns(msg));
	}	
	
	/**
	 * Both write to system out and log to spy (error version).
	 * @param msg
	 */
	private static void LogNoCarriageReturnsFatal( String msg )
	{
	    logger.Fatal(StripCarriageReturns(msg));
	}	
	
	/**
	 * Both write to system out and log to spy (error version).
	 * @param msg
	 */
	private static String StripCarriageReturns( String msg )
	{
	    String stripped = "";
	    for (int loop = 0; loop < msg.length(); ++loop)
	    {
	        char c = (char)msg.charAt(loop);
	        if (!((c == 0x0d)||(c == 0x0a)))
	            stripped += c;
	    }
	    return stripped;
	}
	
	/**
	 * Overrides existing event and report logs and replaces them with new ones.
	 * 
	 * @param eventVector
	 */
	public static void setEventLogReport( EventLog[] eventVector, XPArrayList reportLog )
	{
		m_eventLog = new XPArrayList();
	    try
	    {
	    	// store the new event log
	        int loop = 0;
	        for (loop = 0; loop < eventVector.length; ++loop)
	        	m_eventLog.Add((EventLog)eventVector[loop]);
	        
	        // save the report log
	        m_reportLog = reportLog;
	    }
	    catch( Exception e )
	    {
	        LogAndWriteError("Could not convert event log vector to array list.");
	    }
	}
}

