package com.plumtree.uiinfrastructure.editor;

import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.tips.*;

// Framework Imports
import com.plumtree.openlog.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.xpshared.htmlconstructs.*;

/**
 * This is the Activity Space class for the Editor Framework.
 * Editor Activity Space classes should extend this class.
 *
 * @author Julie Fournier
 */
public abstract class EditorAS extends AFormAS implements IFormAS
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.editor.EditorAS");


	/** JF- Associated model name. */
	public String m_strModelName = null;

	/** DH- The actual MVC name of the model to use if we're going to be storing it with a different name. */
	public String m_strModelMVCName = null;

	/** JF - This is where we store the Editor left navigation. */
	protected LeftNavMenu m_mEditorMenu;
	
	/**
	 * JF- Keep track of the previous page.
	 */
	private IDisplayPage m_dpPrevPage = null; 

	/**
	 * @see com.plumtree.portaluiinfrastructure.activityspace.AActivitySpace#CheckBasicAccess(String, String, boolean)
	 */
	public boolean CheckBasicAccess(String strPage, String strControl,
		boolean bSameSpace)
	{
		// JF- If this is a redirect inside the same AS, just return true, we know the access has already been checked.
		if (bSameSpace)
		{
			return true;
		}
		
		// JF- If the user is returning to a cached Editor.
		IModel myModel = GetModel(m_strModelName);
		
		if (null != myModel)
		{
			try
			{
				if (((EditorModel) myModel).GetUserHasAccessToEditor())
				{
					return true;	
				}
			}	
			catch (Exception e)
			{
				
					log.Debug("Unable to cast the Model: " + myModel + " to EditorModel." +  
						"The access to the Editor for the current user has to be checked.");
					
			}
			
		}

		// JF- Otherwise make sure this is either the start or the repost control. The access will be checked by those controls.
		if ((null != strControl) &&
			(strControl.equals(GetRepostControlName()) ||
			strControl.equals(EditorStartControl.STR_MVC_CLASS_NAME)))
		{
			return true;
		}

		// JF- Either the current user does not have access or the access will be checked by the Editor itself.
		return false;
	}

	/**
	 * Returns the Editor main title.
	 * It will get printed out in EditorHeaderView.
	 *
	 * @return String
	 */
	public abstract String GetEditorTitle();

	/**
	 * Optional method to implement a sub banner.
	 *
	 * @return PTConstruct
	 */
	public PTConstruct GetSubBanner()
	{
		return null;
	}

	/**
	 * Returns the Editor left nav menu.
	 *
	 * @return LeftNavMenu
	 */
	public LeftNavMenu GetLeftMenu()
	{
		return (m_mEditorMenu);
	}
	
	/**
	 * Returns the previous Display Page.
	 * Return null if there was no previous display page.
	 *
	 * @return IDisplayPage
	 */
	public IDisplayPage GetPreviousPage()
	{
		return (m_dpPrevPage);
	}

	/**
	 * Returns the name of the repost control class.
	 * E.g.: PropRepostControl.GetName()
	 *
	 * @return String
	 */
	public abstract String GetRepostControlName();

	/**
	 * Deletes a previously created sub Editor from the sub Editor
	 * list contained in the main Editor AS class.
	 *
	 * @param _sSubEditorKey
	 */
	public void DeleteSubEditorFromList(String _sSubEditorKey)
	{
		DeleteSubSpace(_sSubEditorKey);
	}

	// **************** Helper methods to start, edit and delete sub Editors ***************

	/**
	 * Creates and returns a ASURL object going to the
	 * Editor whose name and key were passed in argument.
	 *
	 * @param _sSubEditorName
	 * @param _sSubEditorKey - The 1st time you access that Editor,
	 * you can decide the value of the key. Next time you want to
	 * access that Editor you need to pass in the same key.
	 * @param _nSubEditorType - Sub Editor Type from EditorStartControl.
	 * @param _sControlKey Optional - only set it if you want
	 * to set an additional setting for the Control. If not,
	 * should be null. (E.g.: Can be used to set an Object class ID.)
	 * @param _sControlValue Optional - Should be the value associated
	 * to the ControlKey.
	 * @return ASURL
	 */
	public ASURL GoToSubEditor(String _sSubEditorName, String _sSubEditorKey,
		int _nSubEditorType, String _sControlKey, String _sControlValue)
	{
		ASURL subEditor = new ASURL(this);

		if (!GetSubSpaceExists(_sSubEditorKey))
		{
			subEditor = GoToNonCachedSubEditor(_sSubEditorName, _sSubEditorKey,
					_nSubEditorType, _sControlKey, _sControlValue);
		}
		else
		{
			subEditor = GoToCachedSubEditor(_sSubEditorKey);
		}

		return (subEditor);
	}

	/**
	 * Registers Views, Controls and Models used by the Editor Framework.
	 *
	 * Warning: m_myModel must be set before calling Init().
	 */
	public void Init()
	{
		super.Init();

		if (m_strModelName == null)
		{
			
				log.Error("Attempting to Init() before m_strModelName has been set.");
			
		}

		// Turn on automatic tracking of current page (off by default).
		SetTrackCurrentPage(true);

		// JF- Register Model and then get it 
		// because we need it to initialize Views and Controls.
		if (m_strModelMVCName != null)
		{
			// We're registering a model and storing it under a different name.
			RegisterModel(m_strModelMVCName, m_strModelName);			
		} else
		{
			RegisterModel(m_strModelName);
		}

		IModel myModel = GetModel(m_strModelName);

		// JF- Register Views
		RegisterView(EditorHeaderView.STR_MVC_CLASS_NAME, myModel);
		RegisterView(EditorLeftNavView.STR_MVC_CLASS_NAME, myModel);

		// JF- Register Controls
		RegisterControl(EditorStartControl.STR_MVC_CLASS_NAME, myModel);
		RegisterControl(EditorRepostControl.STR_MVC_CLASS_NAME, myModel);

		// JF- Register Display Pages
		RegisterPage(new ClosePopUpDP());

		// Error Model
		RegisterModel(ErrorModel.STR_MVC_CLASS_NAME);

		IModel errorModel = GetModel(ErrorModel.STR_MVC_CLASS_NAME);

		RegisterView(ErrorView.STR_MVC_CLASS_NAME, errorModel);
		RegisterControl(ErrorControl.STR_MVC_CLASS_NAME, errorModel);

		RegisterModel(TipsModel.STR_MVC_CLASS_NAME);

		IModel tipsModel = GetModel(TipsModel.STR_MVC_CLASS_NAME);

		RegisterView(TipsView.STR_MVC_CLASS_NAME, tipsModel);

		RegisterControl(TipsControl.STR_MVC_CLASS_NAME, tipsModel);

		// JF- Init Editor left nav menu.
		m_mEditorMenu = new LeftNavMenu(GetName());
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.AActivitySpace#SetCurrentPage()
	 *
	 * @param IDisplayPage the current Display Page
	 */
	public void SetCurrentPage(IDisplayPage page)
	{
		// JF- Keep track of the previous page
		if ((null != page) && (null != GetCurrentPage()))
		{
			m_dpPrevPage = GetCurrentPage();
		}
			 
		// JF- ClosePopUpDP should never become the current display page. 
		// This is not a real display page.
		if (page.GetName() != ClosePopUpDP.STR_MVC_CLASS_NAME)
		{
			super.SetCurrentPage(page);
		}
	}

	/**
	 * Sets a new current page from any Display Page name.
	 *
	 * @param _sNewCurrentPage
	 */
	public void SetCurrentPage(String _sNewCurrentPage)
	{
		EditorDP dpNewPage = null;

		// JF- This is a helper method to be able to set the current display page 
		// with only the name of the DP whereas the AS requires to pass in the DP 
		// itself.
		// JF- Classes that extend this one use that method in their init method 
		// to set the page selected by default the 1st time the user accesses the Editor.
		try
		{		
			try
			{
				dpNewPage = (EditorDP) GetPage(_sNewCurrentPage);
			}
			catch(Exception e)
			{
				throw XPException.GetInstance(e);
			}
		}		
		catch (XPClassCastException cce)
		{
			
				log.Error("EditorAS::SetCurrentPage Unable to cast the Object " +
					dpNewPage +
					" to EditorDP. The following exception occured: " +
					cce);
			

			// JF- No need to extract, this is an unexpected error.
			throw new XPException("Unable to cast the Object " + dpNewPage +
				" to EditorDP.", cce);
		}

		if (null != dpNewPage)
		{
			SetCurrentPage(dpNewPage);
		}
		else
		{
			
				log.Error("EditorAS::SetCurrentPage Unable to set current page to: " +
					_sNewCurrentPage + ". The current has not been changed.");
			
		}
	}

	/**
	 * Helper method to build the ASURL Object for a cached
	 * Sub Editor.
	 *
	 * @param _sSubEditorKey Key of the sub Editor you
	 * want to redirect to.
	 * @return ASURL
	 */
	private ASURL GoToCachedSubEditor(String _sSubEditorKey)
	{
		ASURL subEditor = new ASURL(this);

		if (null != _sSubEditorKey)
		{
			// JF- First, change "DeleteOnCancel" flag to false.
			EditorAS mySubEditor = (EditorAS) GetSubSpace(_sSubEditorKey);
			subEditor.SetLinkGetCachedSpace(mySubEditor);
		}

		return (subEditor);
	}

	/**
	 * Helper method to build the ASURL Object for a new
	 * Sub Editor.
	 *
	 * @param _sSubEditorName
	 * @param _sSubEditorKey - The 1st time you access that Editor,
	 * you can decide the value of the key. Next time you want to
	 * access that Editor you need to pass in the same key.
	 * @param _nSubEditorType - Sub Editor Type from EditorStartControl.
	 * @param _sControlKey Optional - only set it if you want
	 * to set an additional setting for the Control. If not,
	 * should be null. (E.g.: Can be used to set an Object class ID.)
	 * @param _sControlValue Optional - Should be the value associated
	 * to the ControlKey.
	 * @return ASURL
	 */
	private ASURL GoToNonCachedSubEditor(String _sSubEditorName,
		String _sSubEditorKey, int _nEditorType, String _sControlKey,
		String _sControlValue)
	{
		ASURL subEditor = new ASURL(this);

		subEditor.SetLinkCreateNewSpace(_sSubEditorName, this);
		subEditor.SetControl(EditorStartControl.STR_MVC_CLASS_NAME);
		
		subEditor.AddControlArgument(EditorStartControl.QS_EDITOR_TYPE,
			XPConvert.ToString(_nEditorType));
			
		subEditor.AddControlArgument(EditorStartControl.QS_DELETE_ON_CANCEL,
			XPConvert.ToString(true));
		subEditor.SetSubSpaceKey(_sSubEditorKey);

		if ((null != _sControlKey) && (null != _sControlValue))
		{
			subEditor.AddControlArgument(_sControlKey, _sControlValue);
		}

		return (subEditor);
	}

}

