package com.plumtree.uiinfrastructure.editor;

import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.openlog.*;

// Framework Imports
import com.plumtree.openfoundation.util.*;

/**
 * This is the Model class for the Editor Framework.
 * Model classes should extend this class.
 *
 * @author Julie Fournier
 */
public abstract class EditorModel implements IRepostModel, IEditorModelRO
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.editor.EditorModel");


	/** Associated Activity Space */
	public AActivitySpace m_asOwner;

	/** 
	 * JF- This is for sub editors. 
	 * The first time we access the Editor, if the user clicks on cancel, we want to delete
	 * it from the sub Editor list.
	 */
	protected boolean m_bDeleteOnCancel = false;
	
	/** 
	 * JF- If this is true, the framework will close the window on finish and cancel.
	 */
	protected boolean m_bCloseOnDone = false;

	/** JF- This is to hide the left navigation */
	protected boolean m_bHideLeftNav = false;

	/** JF- If the user only has read access to the Editor. */
	protected boolean m_bIsEditorReadOnly = false;

	/** JF- Is true if the current user has access to this Editor. */
	protected boolean m_bUserHasAccessToEditor = false;
	
	/** JF- The Apply button next to the finish button is optional */
	protected boolean m_bEnableApplyButton = false;

	/** 
	 * JF- This is the Editor type. Constants for that variable are in EditorStartControl.
	 * This is mainly to know if it's create or edit mode.
	 */
	protected int m_nEditorType = EditorStartControl.EDITOR_START_FLAG_INVALID;

	/**
	 * This method is used by the ASManager to return new instances of managed
	 * objects.
	 *
	 * @return A new instance of the managed class (i.e. return new Foo();)
	 */
	abstract public Object Create();

	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public abstract String GetName();

	/**
	 * Implement some code to execute when the user clicks on "Cancel".
	 *
	 */
	public void Cancel()
	{
	}

	/**
	 * Implements some actions to perform on start.
	 * Use the Editor Type if needed.
	 *
	 * @param _htFormData
	 *
	 * @return boolean Is true if the current user has the rights to access this Editor.
	 *
	 * @return boolean Security flag: if false user will be redirected
	 */
	public abstract boolean CheckEditorAccessAndStart(XPHashtable _htFormData);

	/**
	 * Implement some code to execute when the user clicks on "Finish".
	 *
	 */
	public abstract void Finish();

	/**
	 * That method contains the code that should be executed just
	 * after the object has been stored in the DB.
	 *
	 */
	public void DoTaskAfterStore()
	{
	}

	/**
	 * That method contains the code that should be executed just
	 * before the object is stored in the DB.
	 *
	 */
	public void DoTaskBeforeStore()
	{
	}

	/**
	 * Returns the current value of the repost action.
	 *
	 * @return int
	 */
	public int GetCurrentRepostAction()
	{
		RepostControl myRepostControl;

		// JF- Retrieve the repost control because this is where we store the repost action
		myRepostControl = (RepostControl) m_asOwner.GetControl(((EditorAS) m_asOwner).GetRepostControlName());

		return myRepostControl.GetRepostAction();
	}

	/**
	 * Returns the DeleteOnCancel flag.
	 * For sub Editors only:
	 * this flag will determine if the Editor
	 * should be deleted from the list of sub Editors
	 * on cancel.
	 *
	 * @return boolean
	 */
	public boolean GetDeleteOnCancel()
	{
		return (m_bDeleteOnCancel);
	}
	
	/**
	 * Returns the CloseOnDone flag.
	 * 
	 * @return boolean
	 */
	public boolean GetCloseOnDone()
	{
		return (m_bCloseOnDone);
	}

	/**
	 * Returns the Editor Type
	 * (see EditorStartControl for constant names).
	 *
	 * @return int
	 */
	public int GetEditorType()
	{
		return (m_nEditorType);
	}
	
	/**
	 * Returns true if the apply button should be displayed.
	 *
	 * @return boolean
	 */
	public boolean GetEnableApplyButton()
	{
		return m_bEnableApplyButton;
	}
	
	/**
	 * Enables or disables the apply button next to the finish button.
	 *
	 * @param _bEnableApplyButton
	 */
	public void SetEnableApplyButton(boolean _bEnableApplyButton)
	{
		m_bEnableApplyButton = _bEnableApplyButton;
	}

	/**
	 * Returns the DeleteOnCancel flag.
	 * For sub Editors only:
	 * this flag will determine if the Editor
	 * should be deleted from the list of sub Editors
	 * on cancel.
	 *
	 * @return boolean
	 */
	public boolean GetHideLeftNav()
	{
		return (m_bHideLeftNav);
	}

	/**
	 * Returns true if the current user only has read access to this Editor.
	 * (can be true in Edit mode only)
	 *
	 * @return boolean
	 */
	public boolean GetIsEditorReadOnly()
	{
		return (m_bIsEditorReadOnly);
	}

	/**
	 * Returns the Redirect object
	 * (this is were we need to go at the end of the Editor).
	 * By default this is going to the previous AS
	 * (the one that opened this one). But Editors can override
	 * that method if they need to redirect somewhere else.
	 *
	 * @return Redirect
	 */
	public Redirect GetRedirectAfterFinish()
	{
		Redirect result = new Redirect();

		// JF- If the parent space is already in the cache, we redirect to it.
		if (null != m_asOwner.GetParentSpace())
		{
			result.SetLinkGetCachedSpace(m_asOwner.GetParentSpace());
		}
		// JF- If not, we redirect to a new one.
		else
		{	
			result.SetLinkCreateNewSpace(m_asOwner.GetParentSpaceName(), m_asOwner);
		}

		return (result);
	}

	/**
	 * Returns true if the current user has access to this Editor.
	 * (If false the user will be redirected.)
	 *
	 * @return boolean
	 */
	public boolean GetUserHasAccessToEditor()
	{
		return (m_bUserHasAccessToEditor);
	}

	/**
	 * Inits the Model with the Activity Space.
	 *
	 * @param parent
	 */
	public void Init(AActivitySpace parent)
	{
		m_asOwner = parent;
	}

	/**
	 * Calls the right save function according to the page name passed in argument.
	 *
	 * @param _sPageName
	 * @param _htFormData
	 * @return int Page status for server side validation
	 */
	public abstract int SavePage(String _sPageName, XPHashtable _htFormData);

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#CleanupTempData()
	 */
	public void CleanupTempData()
	{
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#CleanupAllData()
	 */
	public void CleanupAllData()
	{
	}

	/**
	 * This method can be implemented by classes extending this one.
	 * It will be executed each time the user changes page on the left
	 * navigation menu.
	 *
	 * @param _sNewSpaceName
	 * @param _sNewPageName
	 */
	public void DoTaskOnChangePage(String _sNewSpaceName, String _sNewPageName)
	{
	}

	/**
	 * This method will be called when you come back to the main Editor from a sub Editor.
	 *
	 */
	public void DoTaskOnComeBackFromSubEditor()
	{
	}

	/**
	 * That method contains the code that should be executed just
	 * after the object has been initialized when the new editor
	 * is started.
	 *
	 * @param _htFormData
	 */
	public void DoTaskOnStartEditor(XPHashtable _htFormData)
	{
	}

	/**
	 * Sets the DeleteOnCancel flag.
	 * For sub Editors only:
	 * this flag will determine if the Editor
	 * should be deleted from the list of sub Editors
	 * on cancel.
	 *
	 * @param _bDeleteOnCancel
	 */
	public void SetDeleteOnCancel(boolean _bDeleteOnCancel)
	{
		m_bDeleteOnCancel = _bDeleteOnCancel;
	}
	
	/**
	 * To tell the framework to close the window on finish or cancel.
	 *
	 * @param _bCloseOnDone
	 */
	public void SetCloseOnDone(boolean _bCloseOnDone)
	{
		m_bCloseOnDone = _bCloseOnDone;
	}

	/**
	 * Sets the Editor Type.
	 * (see EditorStartControl for constant names)
	 *
	 * @param _nEditorType
	 */
	public void SetEditorType(int _nEditorType)
	{
		m_nEditorType = _nEditorType;
	}

	/**
	 * Sets the m_bHideLeftNav flag.
	 * If this flag is set to true, the left navigation will not be displayed.
	 *
	 * @param _bHideLeftNav
	 */
	public void SetHideLeftNav(boolean _bHideLeftNav)
	{
		m_bHideLeftNav = _bHideLeftNav;
	}

	/**
	 * Sets a new Page status.
	 *
	 * @param _nNewPageStatus
	 */
	public void SetPageStatus(int _nNewPageStatus)
	{
		RepostControl myRepostControl;

		// JF- Retrieve the repost control because this is where we store the page status 
		myRepostControl = (RepostControl) m_asOwner.GetControl(((EditorAS) m_asOwner).GetRepostControlName());

		// JF- Set the new value
		myRepostControl.SetPageStatus(_nNewPageStatus);
		
		
  			log.Debug("EditorModel::SetPageStatus Page status has been changed to :" + _nNewPageStatus);
  		
	}

	/**
	 * Sets a new value for the m_bUserHasAccessToEditor flag.
	 *
	 * @param _bUserHasAccessToEditor
	 */
	public void SetUserHasAccessToEditor(boolean _bUserHasAccessToEditor)
	{
		m_bUserHasAccessToEditor = _bUserHasAccessToEditor;
	}
}

