package com.plumtree.uiinfrastructure.editor;

import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.login.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.tips.*;

// Framework Imports
import com.plumtree.openlog.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.xpshared.htmlelements.*;
import com.plumtree.xpshared.htmlconstructs.*;

/**
 * This repost control implements the repost actions for Editor pages.
 *
 * @author Julie Fournier
 */
public class EditorRepostControl extends RepostControl
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.editor.EditorRepostControl");

	// ***WARNING*** if you extend this class you need to override the ObjectName.
	// ***WARNING*** you also need to override the Create() method at the bottom of
	// this page.

	/** Object Name for AS */
	public static final String STR_MVC_CLASS_NAME = "EditorRepost";

	/**
	 * Repost action constant: This action is called when the user
	 * goes back to a main Editor after finishing a sub Editor
	 * (the main Editor form is posted).
	 */
	public final static int POSTTOSELF_ACTION_ONRETURN_FROM_SUBEDITOR = 6;

	/**
	 * @see com.plumtree.uiinfrastructure.form.RepostControl#CheckActionSecurityAndExecute
	 */
	public Redirect CheckActionSecurityAndExecute(XPHashtable arguments)
	{
		// JF- We need to override this method to implement the read-only mode.
		if (!m_bInitialized)
		{
			
				log.Error("Repost control has not been initialized: likely cause - Init() overridden but super.Init() not called.");
			
		}
		
		// Initialize member variables
		m_htFormData     = arguments;
		m_rRedirect		 = null;
		m_nAction		 = POSTTOSELF_ACTION_UNDEFINED;

		// Set the repost action mode in the member variable.
		SetRepostAction();

		// JF- On each repost we check if the user has the rights to be in this Editor.
		if (!((EditorModel) m_model).GetUserHasAccessToEditor())
		{
			m_htFormData = null;
			
			if (POSTTOSELF_ACTION_CANCEL == m_nAction)
			{
				// JF- Redirect to the default space
				return (Redirect) LoginHelper.INSTANCE.GetDefaultPageRedirect
					(new Redirect(), m_asOwner.GetUserSession(), m_asOwner);
			} 
			else 
			{
				String strErrorMessage = m_asOwner.GetString(161, ResourceConstants.INFRASTRUCTURE);
				// JF- For some reasons, the Editor has not been initialized, redirect to the Error page.
				GeneralHelpers.SetUIError(m_asOwner, OpenLogService.GetComponent("UI_Infrastructure"), 
					"EditorRepostControl::CheckActionSecurityAndExecute", 
					new XPException(strErrorMessage), true, UIErrorLevel.SERIOUS, 
					strErrorMessage);
			}
		}

		// JF- Pages are not saved in read-only mode
		if (!((EditorModel) m_model).GetIsEditorReadOnly())
		{
			try 
			{
				// Save the page
				SaveCurrentPage();
			}
			catch (Exception e)
			{
				m_nPageStatus = PAGE_STATUS_INVALID;
				
				IXPEnumerator arrErrors = UIErrorHandler.GetErrors(m_asOwner.GetSubSession(), m_asOwner.GetApplication(), m_asOwner, true, true);
				
				if (!arrErrors.MoveNext())
				{
					GeneralHelpers.SetUIError(m_asOwner, OpenLogService.GetComponent("UI_Infrastructure"), 
						"EditorRepostControl::CheckActionSecurityAndExecute", 
						e, false, UIErrorLevel.SERIOUS, m_asOwner.GetString(127, ResourceConstants.INFRASTRUCTURE));
				}
				// JF- An error has already been raised by the Editor. Just log the Exception.
				else 
  					log.Error(e, "Error saving current page.");
  				
				
				m_htFormData = null;
				return null;
			}
			
			// JF- Only enable server validation when the user is changing page or finishing the Editor.
  			if ((POSTTOSELF_ACTION_GOTOPAGE != m_nAction) &&
  				(POSTTOSELF_ACTION_FINISH != m_nAction) &&
  				(POSTTOSELF_ACTION_APPLY_CHANGES != m_nAction))
  			{
  				UIErrorHandler.ClearErrors(m_asOwner.GetSubSession(),
  					m_asOwner.GetApplication());
  
  				
  					log.Info("EditorRepostControl::SaveCurrentPage Just cleared all errors on UIErrorHandler " +
  						"because this is not a Finish or Change Page action.");
  				
  			}

		}

		try
		{
			// JF- In read-only mode, you can only change page and cancel.
			if ((!((EditorModel) m_model).GetIsEditorReadOnly()) ||
				(POSTTOSELF_ACTION_GOTOPAGE == m_nAction) ||
				(POSTTOSELF_ACTION_CANCEL == m_nAction))
			{
				// perform action
				PerformAction(m_nAction);
			}
			else if (((EditorModel) m_model).GetIsEditorReadOnly())
			{
				PerformReadOnlyAction(m_nAction);	
			}
			
		}
		catch (Exception e)
		{
			
	  			log.Error(e, "Error performing repost action.");
	  		
	  			
			if (POSTTOSELF_ACTION_CANCEL != m_nAction)
			{
				m_nPageStatus = PAGE_STATUS_INVALID;
					
				IXPEnumerator arrErrors = UIErrorHandler.GetErrors(m_asOwner.GetSubSession(), m_asOwner.GetApplication(), m_asOwner, true, true);
					
				if (!arrErrors.MoveNext())
				{
					GeneralHelpers.SetUIError(m_asOwner, OpenLogService.GetComponent("UI_Infrastructure"), 
						"EditorRepostControl::CheckActionSecurityAndExecute", 
						e, false, UIErrorLevel.SERIOUS, 
						m_asOwner.GetString(128, ResourceConstants.INFRASTRUCTURE));
				}	
			}
		}

		// clear hashtable of form params
		m_htFormData = null;

		return m_rRedirect;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new EditorRepostControl();
	}

	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Helper method to implement the Finish action.
	 * If this is a sub Editor we don't want to call
	 * the Finish method. If this is the main Editor
	 * and it has some sub Editors we also want to
	 * call the Finish method on each sub Editor.
	 *
	 */
	public void PerformFinishAction()
	{
		CheckMandatoryPages();
		
		if (PAGE_STATUS_VALID == m_nPageStatus)
		{
			m_model.Finish();
			
			// JF- Make sure the status is still valid (it could have been set to 
			// invalid inside the Finish method).
			if (PAGE_STATUS_VALID == m_nPageStatus)
			{
				if (m_asOwner.IsSubSpace())
				{
					SetRedirectToCloseSubEditor();
				}
				else 
				{
					// JF- Redirect (default implementation of GetRedirectAfterFinish 
					// redirects to the previous AS).
					Redirect myRedirect = m_model.GetRedirectAfterFinish();
					boolean bCloseOnDone = false;
					try
					{
						bCloseOnDone = ((EditorModel) m_model).GetCloseOnDone();
					}
					catch (Exception e)
					{
						
							log.Info("Unable to retrieve the CloseOnCancel flag from the EditorModel.");
							
					}
						
				//	if ("Gateway".equalsIgnoreCase(myRedirect.GetSpaceName()) || bCloseOnDone)
					String rSpaceName= myRedirect.GetSpaceName();
					if ((rSpaceName!=null && XPStringUtility.EqualsIgnoreCase("Gateway",rSpaceName)) || bCloseOnDone)
					{
						// JF- If the parent space was the Gateway then just close the window
						myRedirect = new Redirect();
						myRedirect.SetLinkGetCachedSpace(m_asOwner);
						myRedirect.SetDisplayPage(ClosePopUpDP.STR_MVC_CLASS_NAME);		
					}
					else
					{
						// Set this redirect to be a true 302 HTTP redirect so that
						// refreshing the page doesn't finish the editor twice.
						myRedirect.SetIsHTTPRedirect(true);
					}
					
					SetRedirect(myRedirect);
				}
			}
		}
		
		
	}
	
	/**
	 * Checks if any mandatory page has not been visited.
	 * If there are any, raises an error and sets the page status to invalid.
	 * 
	 */
	protected void CheckMandatoryPages()
	{
		// JF- Only check Mandatory pages if there is no other errors
		if (PAGE_STATUS_VALID == m_nPageStatus)
		{
			EditorAS myAS = null;
			
			try 
			{	
				myAS = (EditorAS) m_asOwner;
			} 
			catch (Exception e)
			{
				
					log.Debug("The following exception was thrown when trying to cast the current Activity Space to EditorAS." 
						+ " The Editor Framework won't be able to check the Mandatory pages. " + e);
						
			}
			
			if ((null != myAS) && (myAS.GetLeftMenu().HasUnvisitedMandatoryPage()))
			{
				try 
				{
					XPArrayList alMandatoryPages = myAS.GetLeftMenu().GetArrayOfNonVisitedMandatoryPages();
				
					m_nPageStatus = PAGE_STATUS_INVALID;	
						
					UIError myError = new UIError(UIErrorLevel.SERIOUS,
							UIErrorScope.CURRENT_USER, null, null,
							null);
								
					HTMLElementCollection myHTML = new HTMLElementCollection();
					HTMLSpan mySpan;
					HTMLAnchor myAnchor = null;
					
					mySpan = new HTMLSpan();
					myHTML.AddInnerHTMLElement(mySpan);
					mySpan.SetStyleClass(PTStyleClass.ALERT_SUBTITLE);
					mySpan.AddInnerHTMLString(m_asOwner.GetString(123, ResourceConstants.INFRASTRUCTURE));
					
					LeftNavLink myLink;
					String sPageName;
					EditorDP dpPage;
					
					for (int i = 0; i < alMandatoryPages.GetSize(); i ++)
					{
						myLink = (LeftNavLink) alMandatoryPages.GetElement(i);
						sPageName = (String) myLink.GetDPName();
						dpPage = (EditorDP) m_asOwner.GetPage(sPageName);
						
						myHTML.AddInnerHTMLElement(new HTMLBr());
						myHTML.AddInnerHTMLString("&bull;");
						myHTML.AddInnerHTMLString(CommonHTMLStrings.SPACE);
						myAnchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
						myAnchor.SetOnClick("goToPage('" + myLink.GetASName() +
							"', '" + sPageName + "','" + myLink.GetJSValidationName() +
							"'); return false;");
						myAnchor.AddInnerHTMLString(XPHttpUtility.HtmlEncode(dpPage.GetLeftNavString()));
						myHTML.AddInnerHTMLElement(myAnchor);
					}	
						
					myError.SetHTMLMessage(myHTML);			
					
					UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), 
						m_asOwner.GetApplication());
				}
				catch (Exception e)
				{
					
						log.Error("EditorRepostControl::CheckMandatoryPages The following error occured" +
							" when trying to display the mandatory pages: " + e);
						
				}
			}
		}				
	}
	
	/**
	 * Performs repost actions in Read-Only mode.
	 *
	 * @param _nAction
	 */
	protected void PerformReadOnlyAction(int _nAction)
	{
		// JF- Implement this method if you need to execute some 
		// code on repost when the Editor is Read-Only.

		EditorHelperRepostControl tipsControl = (EditorHelperRepostControl) this.m_asOwner.GetControl(TipsControl.STR_MVC_CLASS_NAME);
		tipsControl.PerformRepostAction(_nAction, m_htFormData);

			
	}

	/**
	 * Performs repost actions.
	 *
	 * @param _nAction
	 */
	protected void PerformAction(int _nAction)
	{
		//JF- We don't use the super class because the implementation of 
		// PerformAction is very different because of sub Editors.
		
		//	Call PerformRepostAction on the TipsControl.
		//	This control may intercept tip actions.
		EditorHelperRepostControl tipsControl = (EditorHelperRepostControl) this.m_asOwner.GetControl(TipsControl.STR_MVC_CLASS_NAME);
		tipsControl.PerformRepostAction(_nAction, m_htFormData);
		
		

		switch (_nAction)
		{
		case POSTTOSELF_ACTION_FINISH:
			try
			{
				PerformFinishAction();
			} 
			catch (Exception e)
			{
				m_nPageStatus = PAGE_STATUS_INVALID;
				
				IXPEnumerator arrErrors = UIErrorHandler.GetErrors(m_asOwner.GetSubSession(), m_asOwner.GetApplication(), m_asOwner, true, true);
				
				if (!arrErrors.MoveNext())
				{
					GeneralHelpers.SetUIError(m_asOwner, OpenLogService.GetComponent("UI_Infrastructure"), 
						"EditorRepostControl::PerformAction", 
						e, false, UIErrorLevel.SERIOUS, m_asOwner.GetString(129, ResourceConstants.INFRASTRUCTURE));
				}
				else 
  					log.Error(e, "Error performing finish action.");
  						
			}
			break;

		case POSTTOSELF_ACTION_CANCEL:
			try
			{
				PerformCancelAction();
			} 
			catch (Exception e)
			{
				// JF- Don't raise any error here because the Editor has been canceled. Just log the exception.
				
  					log.Error(e, "Error performing cancel action.");
  					
			}
			break;

		case POSTTOSELF_ACTION_GOTOPAGE:
			try
			{
				PerformChangePageAction();
			} 
			catch (Exception e)
			{
				m_nPageStatus = PAGE_STATUS_INVALID;
				
				IXPEnumerator arrErrors = UIErrorHandler.GetErrors(m_asOwner.GetSubSession(), m_asOwner.GetApplication(), m_asOwner, true, true);
				
				if (!arrErrors.MoveNext())
				{
					GeneralHelpers.SetUIError(m_asOwner, OpenLogService.GetComponent("UI_Infrastructure"), 
						"EditorRepostControl::PerformAction", 
						e, false, UIErrorLevel.SERIOUS, m_asOwner.GetString(130, ResourceConstants.INFRASTRUCTURE));
				}
				else 
  					log.Error(e, "Error performing change page action.");
  						
			}

			break;

		case POSTTOSELF_ACTION_PASSTHRU:
			PerformPassthruAction();

			break;

		case POSTTOSELF_ACTION_ONRETURN_FROM_SUBEDITOR:
			((EditorModel) m_model).DoTaskOnComeBackFromSubEditor();

			break;

			// JF: Do not add a default case because sub classes will extend this one and add their own action modes.
		}
	}

	/**
	 * @see com.plumtree.uiinfrastructure.form.RepostControl#SaveCurrentPage
	 *
	 */
	protected void SaveCurrentPage()
	{
		if (!((EditorModel) m_model).GetIsEditorReadOnly())
		{
			super.SaveCurrentPage();
		}
	}

	/**
	 * This method is called when we finish a sub Editor.
	 * It takes care of closing the sub Editor pop-up winodws
	 * and repost the main Editor window.
	 *
	 */
	protected void SetRedirectToCloseSubEditor()
	{
		((EditorModel) m_model).SetDeleteOnCancel(false);

		HTMLScript script = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);

		// BT: Bug 7826931 - This postToSelf call seems to cause JS errors when the script debugger in IE7 is
		// turned on. IE6 probably just ignored it. In Firefox - Firebug, it reports that this function is not defined.
		// This issue has existed since 6.0.1 (I've tested all the various 6.x versions). Issue is resolved if
		// I commented the following out and didn't see any problems with closing popups in various admin editors.
		// ClosePopUpDP already calls close on itself so I think it is safe without the following.
		// BT: Seems like SubEditors in edit mode (EDITOR_START_FLAG_EDIT) need this postToSelf call so that it refreshes 
		// the parent editor (i.e. when creating a Experience Rule defintion) but portlets that goes into edit mode
		// (i.e. Search portlet - edit preferences) has a different mode (EDITOR_START_FLAG_MISC). For those types,
		// we should skip adding postToSelf to avoid the JS error.
		if (EditorStartControl.EDITOR_START_FLAG_MISC != ((EditorModel) m_model).GetEditorType())
		{
			try
			{
				script.AddInnerHTMLString("window.opener.postToSelf(" +
					POSTTOSELF_ACTION_ONRETURN_FROM_SUBEDITOR + ");");
			}
			catch (HTMLException he)
			{
				log.Error(he, "EditorRepostControl::SetRedirectToCloseSubEditor An error occurred" +
					" when trying to display the JavaScript.");
			}
		}

		ClosePopUpDP myClosePopUp = (ClosePopUpDP) m_asOwner.GetPage(ClosePopUpDP.STR_MVC_CLASS_NAME);
		myClosePopUp.SetScript(script);

		Redirect myRedirect = new Redirect();
		myRedirect.SetLinkGetCachedSpace(m_asOwner);
		myRedirect.SetDisplayPage(ClosePopUpDP.STR_MVC_CLASS_NAME);

		SetRedirect(myRedirect);
	}

	/**
	 * Helper method to implement the Cancel action.
	 * On cancel, we just redirect to whatever is
	 * returned by GetRedirectAfterFinish.
	 * If this is a sub Editor in Create mode we also
	 * delete the sub Editor from the list. And then we
	 * close the pop-up.
	 *
	 */
	protected void PerformCancelAction()
	{
		((EditorModel) m_model).Cancel();

		if (m_asOwner.IsSubSpace())
		{
			if (((EditorModel) m_model).GetDeleteOnCancel())
			{
				m_asOwner.GetParentSpace().DeleteSubSpace(m_asOwner.GetSubSpaceKey());
			}

			Redirect myRedirect = new Redirect();
			myRedirect.SetLinkGetCachedSpace(m_asOwner);
			
			// JF- Before we redirect to the close pop up DP we clear all the scripts.
			// This DP can be used for different reasons inside the same editor so it's 
			// safer to clear any old script.
			ClosePopUpDP myClosePopUpDP = (ClosePopUpDP) m_asOwner.GetPage(ClosePopUpDP.STR_MVC_CLASS_NAME);
			myClosePopUpDP.SetScript(null);
			
			myRedirect.SetDisplayPage(ClosePopUpDP.STR_MVC_CLASS_NAME);

			SetRedirect(myRedirect);
		}
		else
		{
			Redirect myRedirect = m_model.GetRedirectAfterFinish();
			boolean bCloseOnDone = false;
			try
			{
				bCloseOnDone = ((EditorModel) m_model).GetCloseOnDone();
			}
			catch (Exception e)
			{
				
					log.Info("Unable to retrieve the CloseOnCancel flag from the EditorModel.");
					
			}
			
			//if ("Gateway".equalsIgnoreCase(myRedirect.GetSpaceName()) || bCloseOnDone)
			String rSpaceName= myRedirect.GetSpaceName();
			if ((rSpaceName!=null && XPStringUtility.EqualsIgnoreCase("Gateway",rSpaceName)) || bCloseOnDone)
			{
				// JF- If the parent space was the Gateway then just close the window
				myRedirect = new Redirect();
				myRedirect.SetLinkGetCachedSpace(m_asOwner);
				myRedirect.SetDisplayPage(ClosePopUpDP.STR_MVC_CLASS_NAME);		
			} else
			{
				myRedirect.SetIsHTTPRedirect(true);
			}

			SetRedirect(myRedirect);
		}
	}

	/**
	 * Helper method to implement the Change page action.
	 *
	 */
	private void PerformChangePageAction()
	{
		
		if (PAGE_STATUS_VALID == m_nPageStatus)
		{
			// JF- We get back the new AS Name and new page name from the QS and redirect there.
			String sNewSpaceName;
			String sNewPageName;

			// JF- We get back the new AS name
			String[] data = (String[]) m_htFormData.GetElement(EditorLeftNavView.HTMLHIDDEN_EDITOR_REDIRECT_SPACENAME);

			if ((null == data) || (0 == data.length) || (null == data[0]))
			{
				
					log.Error("EditorRepostControl::PerformChangePageAction The new Space name is invalid: " +
						data);
				

				throw new XPException(m_asOwner.GetString(43,
						ResourceConstants.INFRASTRUCTURE));
			}

			// JF- New AS name
			sNewSpaceName     = data[0];

			// JF- We get back the new DP name
			data = (String[]) m_htFormData.GetElement(EditorLeftNavView.HTMLHIDDEN_EDITOR_REDIRECT_PAGENAME);

			if ((null == data) || (0 == data.length) || (null == data[0]))
			{
				
					log.Error("EditorRepostControl::PerformChangePageAction The new Page name is invalid: " +
						data);
				

				throw new XPException(m_asOwner.GetString(43,
						ResourceConstants.INFRASTRUCTURE));
			}

			// JF- New page name
			sNewPageName     = data[0];

			m_rRedirect = new Redirect();

			if (m_asOwner.GetName().equals(sNewSpaceName))
			{
				// JF- We are just changing page in the same AS
				m_rRedirect.SetLinkGetCachedSpace(m_asOwner);
				m_rRedirect.SetDisplayPage(sNewPageName);
			}
			else
			{
				// JF- We are redirecting to another AS
				m_rRedirect.SetLinkCreateNewSpace(sNewSpaceName, m_asOwner);
				m_rRedirect.SetDisplayPage(sNewPageName);
			}

			((EditorModel) m_model).DoTaskOnChangePage(sNewSpaceName,
				sNewPageName);
		}
	}
}


