package com.plumtree.uiinfrastructure.editor;

import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.statichelpers.*;

// Framework Imports
import com.plumtree.openlog.*;
import com.plumtree.openfoundation.util.*;

/**
 * This start control is called anytime
 * an Editor is started.
 * Start Control classes should extend this class.
 *
 * @author Julie Fournier
 */
public class EditorStartControl implements IControl
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.editor.EditorStartControl");

	/** Object Name for AS */
	public static final String STR_MVC_CLASS_NAME = "EditorStart";

	/** JF- Query String constants: Editor type */
	public static final String QS_EDITOR_TYPE		  = "editorType";

	/** JF- Query String constants: Param to tell the Activity space to delete it if the user clicks on cancel. */
	public static final String QS_DELETE_ON_CANCEL    = "deleteOnCancel";
	
	/** JF- Query String constants: Param to tell the framework to close the window on finish and cancel. */
	public static final String QS_CLOSE_WINDOW_ON_DONE    = "closeOnDone";

	/** JF- Query String constants: Param to tell the Framework to hide the left navigation */
	public static final String QS_EDITOR_HIDE_LEFTNAV = "hideLeftNav";

	/** JF- Editor type: Invalid */
	public static final int EDITOR_START_FLAG_INVALID = -1;

	/** JF- Editor type: Create mode */
	public static final int EDITOR_START_FLAG_CREATE = 1;

	/** JF- Editor type: Edit mode */
	public static final int EDITOR_START_FLAG_EDIT = 2;

	/** JF- Editor type: Other */
	public static final int EDITOR_START_FLAG_MISC = 10;

	/** JF- Associated Activity Space */
	public AActivitySpace m_asOwner;

	/** JF- Associated Model */
	protected EditorModel m_model;

	/** JF- Contains all the form params from the request. */
	protected XPHashtable m_htFormData;
	

	/**
	 * This code is called by the AS Servlet each time there is an Editor is started.
	 * It calls the startEditor method in the Model if a valid Editor Type was in the QS.
	 *
	 * If the Redirect object returned is not null,
	 *  the Servlet will the page specified by the Redirect.
	 *
	 * If the Redirect is null, the Servlet will get the current page
	 * from the AS and display it.
	 *
	 * @return Redirect
	 */
	public Redirect CheckActionSecurityAndExecute(XPHashtable arguments)
	{
		m_htFormData = arguments;

		// JF- Get data from the query String and set them in the Model.
		SetModelVariables();

		if ((AccessStyles.LOW_BANDWIDTH == m_asOwner.GetAccessStyle()) ||
				(AccessStyles.SECTION_508 == m_asOwner.GetAccessStyle()))
		{
			// JF- Disable the left nav for 508 pages
			m_model.SetHideLeftNav(true);
		}

		// JF- Call the Start Editor method on the Model.
		if (EDITOR_START_FLAG_INVALID != m_model.GetEditorType())
		{
			try {
				m_model.SetUserHasAccessToEditor(m_model.CheckEditorAccessAndStart(
					m_htFormData));
			} 
			catch (Exception e) 
			{
				IXPEnumerator arrErrors = UIErrorHandler.GetErrors(m_asOwner.GetSubSession(), m_asOwner.GetApplication(), m_asOwner, true, true);
				
				if (!arrErrors.MoveNext())
				{
					GeneralHelpers.SetUIError(m_asOwner, OpenLogService.GetComponent("UI_Infrastructure"), 
						"EditorStartControl::CheckActionSecurityAndExecute", e, true, UIErrorLevel.SERIOUS, 
						m_asOwner.GetString(131, ResourceConstants.INFRASTRUCTURE));
				}
				else
				{
					throw XPException.GetInstance(e);
				}
			}

			if (!m_model.GetUserHasAccessToEditor())
			{
				IXPEnumerator arrErrors = UIErrorHandler.GetErrors(m_asOwner.GetSubSession(), m_asOwner.GetApplication(), m_asOwner, true, true);
				
				if (!arrErrors.MoveNext())
				{
					GeneralHelpers.SetUIError(m_asOwner, OpenLogService.GetComponent("UI_Infrastructure"), 
						"EditorStartControl::CheckActionSecurityAndExecute", 
						new XPException(m_asOwner.GetString(132, ResourceConstants.INFRASTRUCTURE)), true, UIErrorLevel.SERIOUS, 
						m_asOwner.GetString(132, ResourceConstants.INFRASTRUCTURE));
				}
				else
				{
					throw new XPException(m_asOwner.GetString(132, ResourceConstants.INFRASTRUCTURE));
				}
			}
		}
		else
		{
			
				log.Error("EditorStartControl::Execute The Editor was not started because the Editor Type is invalid.");
			
			
			IXPEnumerator arrErrors = UIErrorHandler.GetErrors(m_asOwner.GetSubSession(), m_asOwner.GetApplication(), m_asOwner, true, true);
				
			if (!arrErrors.MoveNext())
			{
				GeneralHelpers.SetUIError(m_asOwner, OpenLogService.GetComponent("UI_Infrastructure"), 
						"EditorStartControl::CheckActionSecurityAndExecute", 
						new XPException(m_asOwner.GetString(44, ResourceConstants.INFRASTRUCTURE)), true, UIErrorLevel.SERIOUS, 
						m_asOwner.GetString(44, ResourceConstants.INFRASTRUCTURE));
			}
			else
			{
				throw new XPException(m_asOwner.GetString(44,
					ResourceConstants.INFRASTRUCTURE));
			}
		}

		// JF- Remove form params from hashtable. 
		// So we start with a clean one, next time this Control is called.
		m_htFormData = null;

		// JF- return null because we are not redirecting anywhere, 
		// We just want to see the current DP in the current AS.
		return null;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new EditorStartControl();
	}

	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Inits the class with the associated model and activity space.
	 *
	 * @param model
	 * @param space
	 */
	public void Init(IModel model, AActivitySpace space)
	{
		try
		{
			try
			{
				m_model = (EditorModel) model;
			}
			catch(Exception e)
			{
				throw XPException.GetInstance(e);
			}
		}
		catch (XPClassCastException e)
		{
			log.Error(e, "EditorStartControl::Init Unable to cast the Object " +
				model + " to EditorModel.");

			// JF- No need to extract, this is an unexpected error.
			throw new XPException("Unable to cast the Object " + model +
				" to EditorModel.", e);
		}

		m_asOwner = space;
	}

	/**
	 * Helper method to set the Model member
	 * variables from the form params.
	 *
	 */
	private void SetModelVariables()
	{
		// JF- Get and set the Editor type.
		String[] data = (String[]) m_htFormData.GetElement(QS_EDITOR_TYPE);

		if ((null != data) && (0 != data.length) && (!"".equals(data[0])))
		{
			m_model.SetEditorType(XPConvert.ToInteger(data[0]));

			
				log.Debug("EditorStartControl::SetModelVariables Editor Type: " +
					data[0]);
			
		}
		else
		{
			
				log.Error("EditorStartControl::SetModelVariables Editor Type ID was not in Query String.");
			

			throw new XPException("Unable to start the Editor.");
		}

		// JF- Get and set the DeleteOnCancel flag.
		data = (String[]) m_htFormData.GetElement(QS_DELETE_ON_CANCEL);

		if ((null != data) && (0 != data.length) && (!"".equals(data[0])))
		{
			m_model.SetDeleteOnCancel(XPConvert.ToBoolean(data[0]));

			
				log.Debug("EditorStartControl::SetModelVariables DeleteOnCancel: " +
					data[0]);
			
		}
		else
		{
			// JF- This should happen each time this is NOT a sub Editor (= most of the time).
			
				log.Debug("EditorStartControl::SetModelVariables DeleteOnCancel was not in Query String.");
			
		}
		
		// JF- Get and set the CloseOnDone flag.
		data = (String[]) m_htFormData.GetElement(QS_CLOSE_WINDOW_ON_DONE);

		if ((null != data) && (0 != data.length) && (!"".equals(data[0]))) 
		{
			m_model.SetCloseOnDone(XPConvert.ToBoolean(data[0]));

			
				log.Debug("EditorStartControl::SetModelVariables CloseOnDone: " +
					data[0]);
			
		}
		else
		{
			// JF- This should happen each time this is NOT a sub Editor (= most of the time).
			
				log.Debug("EditorStartControl::SetModelVariables DeleteOnCancel was not in Query String.");
			
		}

		// JF- Get and set the hideLeftNav flag.
		data = (String[]) m_htFormData.GetElement(QS_EDITOR_HIDE_LEFTNAV);

		if ((null != data) && (0 != data.length) && (!"".equals(data[0])))
		{
			m_model.SetHideLeftNav(XPConvert.ToBoolean(data[0]));

			
				log.Debug("EditorStartControl::SetModelVariables hideLeftNav: " +
					data[0]);
			
		}
		else
		{
			// JF- This should happen each time we don't want to hide the left navigation
			
				log.Debug("EditorStartControl::SetModelVariables hideLeftNav was not in Query String.");
			
		}
	}
}


