package com.plumtree.uiinfrastructure.editor;

// Framework Imports
import com.plumtree.openlog.*;
import com.plumtree.openfoundation.util.*;

/**
 * This is helper class that contains the implementation
 * of the left navigation menu as well as  two inner classes.
 * One for the sub menus and one the links.
 * This class is mainly used by EditorAS and classes extending it.
 * The left navigation menu is printed out in EditorHeaderView.
 *
 * @author Julie Fournier
 */
public class LeftNavMenu
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.editor.LeftNavMenu");

	/** JF- Current AS name (name of the AS you create that menu from). */
	private String m_sCurrentASName;
	
	/** JF- Array of Sub Menus. */
	private XPArrayList m_alMenus;
	
	/** JF- Array of Mandatory Pages. */
	private XPArrayList m_alMandatoryPages;

	/**
	 * Creates a new Left Nav Menu object.
	 * Should have only one menu per Editor.
	 *
	 * @param _sCurrentASName name of the AS you create that menu from
	 */
	public LeftNavMenu(String _sCurrentASName)
	{
		m_alMenus			 = new XPArrayList();
		m_alMandatoryPages	 = new XPArrayList();
		m_sCurrentASName     = _sCurrentASName;
	}
	
	/**
	 * Removes the page in argument from the array of mandatory pages.
	 * This is to keep track of the mandatory pages. When the page is 
	 * being visited by the user it should be removed from the array.
	 *
	 * @param _dpName Name of the Display Page to remove
	 */
	public void RemovePageFromMandatoryArray(String _dpName)
	{
		LeftNavLink link = GetLinkGivenDPName(_dpName);
		
		if (null != link)
		{
			if (m_alMandatoryPages.Contains(link))	
			{
				m_alMandatoryPages.Remove(link);	
			}	
		}
		else
		{
			
				log.Debug("LeftNavMenu::RemovePageFromMandatoryArray " +
					"The link corresponding to " + _dpName +
					" was not found in the left navigation menu.");
			
		}
	}
	
	/**
	 * Changes the status of the page in argument to mandatory or to non-mandatory.
	 *
	 * @param _bIsMandatory New status
	 * @param _dpName Name of the Display Page
	 */
	public void SetMandatoryPage(boolean _bIsMandatory, String _dpName)
	{
		LeftNavLink link = GetLinkGivenDPName(_dpName);
		
		if (null != link)
		{
			if (_bIsMandatory)
			{
				link.SetMandatory(true);
				
				if (!m_alMandatoryPages.Contains(link))	
				{
					m_alMandatoryPages.Add(link);	
				}	
			}
			else
			{
				link.SetMandatory(false);
				
				if (m_alMandatoryPages.Contains(link))	
				{
					m_alMandatoryPages.Remove(link);	
				}		
			}
		}
		else
		{
			
				log.Debug("LeftNavMenu::AddPageToMandatoryArray " +
					"The link corresponding to " + _dpName +
					" was not found in the left navigation menu.");
			
		}
	}

	/**
	 * To set a JavaScript to be called when the user tries to go to a particular Display Page. 
	 *
	 * @param _sDPName Name of the Display Page to modify
	 * @param _sJSName Name of the JavaScript function
	 */
	public void AddJavaScriptValidationToLink(String _sDPName, String _sJSName)
	{
		LeftNavLink link = GetLinkGivenDPName(_sDPName);

		if (null != link)
		{
			link.SetJSValidationName(_sJSName);
		}
		else
		{
			
				log.Error("LeftNavMenu::AddJavaScriptValidationToLink " +
					"The link corresponding to " + _sDPName +
					" was not found in the left navigation menu.");
			
		}
	}
	
	/**
	 * Returns the array of non visited mandatory pages.
	 *
	 * @return XPArrayList
	 */
	public XPArrayList GetArrayOfNonVisitedMandatoryPages()
	{
		return m_alMandatoryPages;
	}
	
	/**
	 * Returns true if some of the mandatory pages have not been visited
	 *
	 * @return boolean
	 */
	public boolean HasUnvisitedMandatoryPage()
	{
		return !m_alMandatoryPages.IsEmpty();
	}

	/**
	 * Creates a new link and adds it to the sub menu identified
	 * by the ID passed in argument.
	 * That link is going to the DP passed in argument in the current AS.
	 *
	 * @param _nSubMenuID ID of the sub menu you want to add that link to
	 * @param _dpName name of the display page this link should go to
	 * 
	 * @return int ID of the newly created link. You need that ID only if
	 * you want to set optional settings on the link.
	 */
	public int AddLinkToSubMenu(int _nSubMenuID, String _dpName)
	{
		int nNewLinkID = AddLinkToSubMenu(_nSubMenuID, m_sCurrentASName, _dpName, false);

		return (nNewLinkID);
	}
	
	/**
	 * Creates a new link and adds it to the sub menu identified
	 * by the ID passed in argument.
	 * That link is going to the DP passed in argument in the current AS.
	 *
	 * @param _nSubMenuID ID of the sub menu you want to add that link to
	 * @param _dpName name of the display page this link should go to
	 * @param _bIsMandatory true if the link is mandatory
	 * 
	 * @return int ID of the newly created link. You need that ID only if
	 * you want to set optional settings on the link.
	 */
	public int AddLinkToSubMenu(int _nSubMenuID, String _dpName, boolean _bIsMandatory)
	{
		int nNewLinkID = AddLinkToSubMenu(_nSubMenuID, m_sCurrentASName, _dpName, _bIsMandatory);

		return (nNewLinkID);
	}

	/**
	 * Returns the array of sub menus.
	 *
	 * @return XPArrayList
	 */
	public XPArrayList GetArraySubMenus()
	{
		return m_alMenus;
	}

	/**
	 * Will display or hide the link identified by the ID in argument.
	 *
	 * @param _bHide true to hide it, false to not hide it
	 * @param _nSubMenuID ID of the sub menu containing the link
	 * @param _nSubMenuID ID of the link in that sub menu
	 */
	public void HideLink(boolean _bHide, int _nSubMenuID, int _nLinkID)
	{
		LeftNavLink link = GetSubMenu(_nSubMenuID).GetLink(_nLinkID);
		
		link.SetHide(_bHide);
		RemovePageFromMandatoryArray(link.GetDPName());
	}

	/**
	 * Will display or hide the link identified by the Display Page name in argument.
	 * If multiple instances of that DP are found, they will all be changed.
	 *
	 * @param _bHide true to hide it, false to not hide it
	 * @param _sDPName Name of the Display Page to hide
	 */
	public void HideLink(boolean _bHide, String _sDPName)
	{
		LeftNavLink link = GetLinkGivenDPName(_sDPName);

		if (null != link)
		{
			link.SetHide(_bHide);
		}
		else
		{
			
				log.Warn("LeftNavMenu::HideLink " + "The link corresponding to " +
					_sDPName + " was not found in the left navigation menu.");
			
		}
		
		RemovePageFromMandatoryArray(_sDPName);
	}

	/**
	 * Will display or hide the sub menu identified by the ID in argument.
	 *
	 * @param _bHide true to hide it, false to not hide it
	 * @param _nSubMenuID ID of the sub menu
	 */
	public void HideSubMenu(boolean _bHide, int _nSubMenuID)
	{
		GetSubMenu(_nSubMenuID).SetHide(_bHide);
	}

	/**
	 * Creates a new sub menu, adds it to the array of
	 * menus and returns the ID of that new sub menu.
	 * The ID is the position of the sub menu in the array.
	 *
	 * @param _sSubMenuTitle
	 * 
	 * @return int
	 */
	public int StartNewSubMenu(String _sSubMenuTitle)
	{
		int nNewSubMenuID;

		LeftNavSubMenu smNewSubMenu = new LeftNavSubMenu(_sSubMenuTitle);
		nNewSubMenuID = m_alMenus.Add(smNewSubMenu);
		smNewSubMenu.SetID(nNewSubMenuID);

		
			log.Debug("LeftNavMenu::StartNewSubMenu Started following menu: " +
				_sSubMenuTitle + ", and returned following menu ID: " +
				nNewSubMenuID + ".");
		

		return (nNewSubMenuID);
	}

	/**
	 * Creates a new link and adds it to the sub menu identified
	 * by the ID passed in argument.
	 * That link is going to the AS and the DP passed in argument.
	 *
	 * @param _nSubMenuID ID of the sub menu you want to add that link to
	 * @param _sASName Name of the AS where the link should go.
	 * @param _dpName Name of the display page this link should go to.
	 * @param _bIsMandatory true if the link is mandatory
	 * 
	 * @return int ID of the newly created link. You need that ID only if
	 * you want to set optional settings on the link.
	 */
	private int AddLinkToSubMenu(int _nSubMenuID, String _sASName,
		String _dpName, boolean _bIsMandatory)
	{
		int nNewLinkID;

		LeftNavLink lNewLink	    = new LeftNavLink(_dpName, _sASName);
		lNewLink.SetMandatory(_bIsMandatory);
		LeftNavSubMenu smNewSubMenu = (LeftNavSubMenu) m_alMenus.GetElement(_nSubMenuID);
		nNewLinkID				    = smNewSubMenu.GetArrayLinks().Add(lNewLink);
		
		if (_bIsMandatory)
		{
			m_alMandatoryPages.Add(lNewLink);	
		}

		
			log.Debug("LeftNavMenu::AddLinkToSubMenu Added following link: Activity Space: " +
				_sASName + " - Display Page: " + _dpName +
				", to following menu ID: " + _nSubMenuID + ", with link ID: " +
				nNewLinkID + ".");
		

		return (nNewLinkID);
	}

	/**
	 * Helper method that returns the Link Object given the Display Page name.
	 * Returns null if the Link was not found
	 *
	 * @param _sDPName Name of the Display Page
	 */
	private LeftNavLink GetLinkGivenDPName(String _sDPName)
	{
		XPArrayList alLinks;
		LeftNavLink link;

		// JF- We loop through the sub menus
		for (int i = 0; i < m_alMenus.GetSize(); i++)
		{
			alLinks = ((LeftNavSubMenu) m_alMenus.GetElement(i)).GetArrayLinks();

			// JF- Then for each sub menu, we loop through the links
			for (int j = 0; j < alLinks.GetSize(); j++)
			{
				link = (LeftNavLink) alLinks.GetElement(j);

				if (_sDPName.equals(link.GetDPName()))
				{
					return link;
				}
			}
		}

		return null;
	}

	/**
	 * Helper method that returns the sub menu
	 * corresponding to the ID passed in argument.
	 *
	 * @param _nSubMenuID
	 * 
	 * @return LeftNavSubMenu
	 */
	private LeftNavSubMenu GetSubMenu(int _nSubMenuID)
	{
		return (LeftNavSubMenu) m_alMenus.GetElement(_nSubMenuID);
	}
}


/**
 * Represents a Left Navigation sub menu.
 *
 * @author Julie Fournier
 */
class LeftNavSubMenu
{
	/** JF- Title of the sub menu. */
	private String m_sTitle;
	
	/** JF- Array of links to display in that sub menu. */
	private XPArrayList m_alLinks;
	
	/** JF- To hide this sub menu */
	private boolean m_bHide;
	
	/** JF- ID of the sub menu. */
	private int m_nID;

	/**
	 * Creates a new sub menu object.
	 *
	 * @param _sTitle
	 */
	public LeftNavSubMenu(String _sTitle)
	{
		m_alLinks     = new XPArrayList();
		m_sTitle	  = _sTitle;
		m_nID		  = -1;
		m_bHide		  = false;
	}

	/**
	 * Returns the array of links contained in that sub menu.
	 *
	 * @return XPArrayList
	 */
	public XPArrayList GetArrayLinks()
	{
		return m_alLinks;
	}

	/**
	 * Returns true if this sub menu should not be displayed
	 * in the left navigation.
	 *
	 * @return boolean
	 */
	public boolean GetHide()
	{
		return m_bHide;
	}

	/**
	 * Returns the ID of the sub menu.
	 *
	 * @return int
	 */
	public int GetID()
	{
		return m_nID;
	}

	/**
	 * Returns the link corresponding to the ID
	 * passed in argument.
	 *
	 * @param _nLinkID
	 * @return LeftNavLink
	 */
	public LeftNavLink GetLink(int _nLinkID)
	{
		return (LeftNavLink) m_alLinks.GetElement(_nLinkID);
	}

	/**
	 * Returns the title of the sub menu.
	 *
	 * @return String
	 */
	public String GetTitle()
	{
		return m_sTitle;
	}

	/**
	 * Set it to true if this sub menu should not be displayed
	 * in the left navigation.
	 *
	 * @param _bHide
	 */
	public void SetHide(boolean _bHide)
	{
		m_bHide = _bHide;
	}

	/**
	 * Sets the ID of the sub menu.
	 * Has to be the position of that sub menu in the array
	 * of the main menu.
	 *
	 * @param _nID
	 */
	public void SetID(int _nID)
	{
		m_nID = _nID;
	}

	/**
	 * Sets a new title for that sub menu.
	 *
	 * @param _sTitle
	 */
	public void SetTitle(String _sTitle)
	{
		m_sTitle = _sTitle;
	}
}


/**
 * Represents a left navigation link.
 *
 * @author Julie Fournier
 */
class LeftNavLink
{
	/** JF- Name of the AS where this link should go to. */
	private String m_sASName;
	
	/** JF- Name of the DP where this link should go to. */
	private String m_sDPName;
	
	/** JF- Name of a JavaScript to call before changing page when the user clicks on that link. */
	private String m_sValidationJS = "";
	
	/** JF- To hide this link */
	private boolean m_bHide;
	
	/** JF- If this is a mandatory page, we display an red star next to the link. */
	private boolean m_bIsMandatory;

	/**
	 * Create a new left nav link with the
	 * name of the DP the link is going to and the
	 * name of the AS that DP belongs to.
	 *
	 * @param _sDPName
	 * @param _sASName
	 */
	public LeftNavLink(String _sDPName, String _sASName)
	{
		m_sDPName     	= _sDPName;
		m_sASName     	= _sASName;
		m_bHide		  	= false;
		m_bIsMandatory 	= false;
	}

	/**
	 * Returns the AS name.
	 *
	 * @return String
	 */
	public String GetASName()
	{
		return m_sASName;
	}

	/**
	 * Returns the DP name.
	 *
	 * @return String
	 */
	public String GetDPName()
	{
		return m_sDPName;
	}

	/**
	 * Returns true if this link should not be displayed
	 * in the left navigation.
	 *
	 * @return boolean
	 */
	public boolean GetHide()
	{
		return m_bHide;
	}
	
	/**
	 * Returns true if this link is mandatory.
	 *
	 * @return boolean
	 */
	public boolean GetMandatory()
	{
		return m_bIsMandatory;
	}

	/**
	 * Returns the name of the validation JavaScript for this link.
	 *
	 * @return String
	 */
	public String GetJSValidationName()
	{
		return m_sValidationJS;
	}

	/**
	 * Sets a new AS name.
	 *
	 * @param _sASName
	 */
	public void SetASName(String _sASName)
	{
		m_sASName = _sASName;
	}

	/**
	 * Sets a new DP name.
	 *
	 * @param _sDPName
	 */
	public void SetDPName(String _sDPName)
	{
		m_sDPName = _sDPName;
	}

	/**
	 * Set it to true if this link should not be displayed
	 * in the left navigation.
	 *
	 * @param _bHide
	 */
	public void SetHide(boolean _bHide)
	{
		m_bHide = _bHide;
	}
	
	/**
	 * Set it to true if this link is mandatory.
	 *
	 * @param _bMandatory
	 */
	public void SetMandatory(boolean _bMandatory)
	{
		m_bIsMandatory = _bMandatory;
	}

	/**
	 * Sets the name of the JavaScript to call before changing
	 * page when the user clicks on this link.
	 *
	 * @param _sValidationJS
	 */
	public void SetJSValidationName(String _sValidationJS)
	{
		m_sValidationJS = _sValidationJS;
	}
}

