package com.plumtree.uiinfrastructure.errordisplay;

import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.web.*;
import com.plumtree.openlog.*;
import com.plumtree.openfoundation.util.*;

/**
 * This is an abstract model extended by various concrete error model.
 *
 * @author Beng
 */
public abstract class AErrorModel implements IModel, IErrorModelRO
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.errordisplay.AErrorModel");

	// Parent Activity Space
	protected AActivitySpace m_asOwner;

	// TODOCR JF 12/10/02 - What is this?
	private String sCurrentASKey = "currentAS";

	/**
	 * Implement to remove the specified error.
	 *     Concrete implementations should differ
	 *     to the UIErrorHandler class, or one of its subclasses.
	 * <br><br>
	 * @param sKey String key used to store the error to be removed.
	 * @param mySessionManager ISessionManager
	 * @param myApplication IApplication
	 */
	public abstract void ClearError(String sKey,
		ISessionManager mySessionManager, IApplication myApplication);

	/**
	 * Implement to remove all errors available to the current user.
	 *     Concrete implementations should differ
	 *     to the UIErrorHandler class, or one of its subclasses.
	 * <br><br>
	 * @param mySessionManager ISessionManager
	 * @param myApplication IApplication
	 */
	public abstract void ClearErrors(ISessionManager mySessionManager,
		IApplication myApplication);

	/**
	 * Implement to clear those errors that need to be cleared when
	 *     the user changes activity space.  Concrete implementations should differ
	 *     to the UIErrorHandler class, or one of its subclasses.
	 * <br><br>
	 * @param sASName String name of the new activity space.
	 * @param mySessionManager ISessionManager
	 * @param myApplication IApplication
	 */
	public abstract void ClearErrorsForASChange(String sASName,
		ISessionManager mySessionManager, IApplication myApplication);

	/**
	 * This method is used by the ASManager to return new instances of managed
	 * objects.
	 *
	 * @return A new instance of the managed class (i.e. return new Foo();)
	 */
	abstract public Object Create();

	/**
	 * Implement to return a boolean indicating whether this MVC
	 *     should display errors requiring a separate display page.
	 *
	 * @return boolean value
	 */
	public abstract boolean GetDisplaySeparatePageErrors();

	/**
	 * Implement to obtain an enumeration of errors appropriate for the current
	 *     user.  Concrete implementations should differ
	 *     to the UIErrorHandler class, or one of its subclasses.
	 * <br><br>
	 * @param session ISessionManager
	 * @param myApplication IApplication
	 * @param as AActivitySpace
	 * @param bGetErrorsAssociatedWithAnAS boolean value
	 * @return IXPEnumerator of UIError objects
	 */
	public abstract IXPEnumerator GetErrors(ISessionManager session,
		IApplication myApplication, AActivitySpace activitySpace,
		boolean bGetErrorsAssociatedWithAnAS);

	/**
	 * Implement to return a boolean indicating whether this MVC
	 *     should display only those errors associated with the
	 *     current MVC.
	 * <br><br>
	 *
	 * @return boolean value
	 */
	public abstract boolean GetErrorsAssociatedWithAnAS();

	/**
	 * Return the name of the MVC object.
	 *
	 * @return the name of the object.
	 */
	abstract public String GetName();

	/**
	 * This method initializes the model.  This is called automatically when
	 * AActivitySpace.RegisterModel() is called.
	 *
	 * @param parent the parent Activity Space
	 */
	abstract public void Init(AActivitySpace parent);

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#CleanupTempData()
	 */
	public void CleanupTempData()
	{
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#CleanupAllData()
	 */
	public void CleanupAllData()
	{
	}

	/**
	 * Call this method to clear the specified error.
	 *
	 * @param sKey String key used to store the error.  Available from
	 *     UIError.GetSessionKey().
	 */
	public void ClearError(String sKey)
	{
		ISessionManager mySessionManager = m_asOwner.GetSubSession();
		IApplication myApplication		 = m_asOwner.GetApplication();

		try
		{
			this.ClearError(sKey, mySessionManager, myApplication);
		}
		catch (XPException ex)
		{
			
				log.Error(ex, "Unable to clear error");
			
		}
	}

	/**
	 * Call this method to clear all errors exposed to the current user.
	 */
	public void ClearErrors()
	{
		ISessionManager mySessionManager = m_asOwner.GetSubSession();
		IApplication myApplication		 = m_asOwner.GetApplication();

		this.ClearErrors(mySessionManager, myApplication);
	}

	/**
	 * Call this method to obtain an enumeration of all errors that
	 *     are appropriate for the current user.
	 * <br><br>
	 * @return IXPEnumerator containing UIError objects.
	 */
	public IXPEnumerator GetErrors()
	{
		ISessionManager mySessionManager = m_asOwner.GetSubSession();
		IApplication myApplication		 = m_asOwner.GetApplication();

		String sCurrentAS = null;

		//   what is the current AS
		String sASName = this.m_asOwner.GetName();

		//   what we think to be the current AS
		sCurrentAS = (String) mySessionManager.GetAttribute(sCurrentASKey);

		//   correct the session
		mySessionManager.SetAttribute(sCurrentASKey, sASName);

		if (!sASName.equals(sCurrentAS))
		{
			
				log.Debug("Activity Space Change.");
			

			this.ClearErrorsForASChange(sASName, mySessionManager, myApplication);
		}

		IXPEnumerator eTemp;

		eTemp = this.GetErrors(mySessionManager, myApplication, this.m_asOwner,
			this.GetErrorsAssociatedWithAnAS());

		return eTemp;
	}
}

