package com.plumtree.uiinfrastructure.errordisplay;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.htmlconstructs.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This helper class allows you to retrieve and display UI error
 * messages.
 * 
 * It is implemented as an object rather than static methods so
 * that it can be extended and re-used if necessary.
 */
public class ErrorDisplayHelper
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.errordisplay.ErrorDisplayHelper");

	private AActivitySpace m_currentAS;

	private boolean m_bDisplaySeparatePageErrors = false;

	private IXPEnumerator m_Errors;

	/**
	 * This helper object displays UI errors.
	 * 
	 * @param _currentAS The current Activity Space.
	 * @param _errors The current UI errors.
	 */
	public ErrorDisplayHelper(AActivitySpace _currentAS,
		IXPEnumerator _errors)
	{
		m_currentAS = _currentAS;
		m_Errors = _errors;
	}

	/**
	 * This method sets whether or not this helper method will attempt to display
	 * errors that are configured to display on a separate page.  This setting
	 * only applies to the full HTML display, not the simple GetFirstErrorText
	 * method.
	 *  
	 * @param _bDisplaySeparatePageErrors whether or not this helper object
	 * should display errors that are configured to display on a separate page.
	 */
	public void SetDisplaySeparatePageErrors(boolean _bDisplaySeparatePageErrors)
	{
		m_bDisplaySeparatePageErrors = _bDisplaySeparatePageErrors;
	}

	/**
	 * This method gets the base error string for the first UI error
	 * and it does not clears all errors.  It does not handle error 
	 * exceptions or extended error messages.
	 * 
	 * @return String the first error string.  Empty string if there
	 * are no errors.
	 */
	public String GetFirstErrorText()
	{
		if (m_Errors.MoveNext())
		{
			UIError error = (UIError) m_Errors.GetCurrent();

			String sErrorMessage = error.GetMessage();
			
			// Clear all errors
			// RT: Moving this to Error Tag and ErrorText Tag since they should
			// Handle clearing errors after displaying the Errors
//			UIErrorHandler.ClearErrors(m_currentAS.GetSubSession(),
//				m_currentAS.GetApplication());

			return sErrorMessage;
		}

		// There were no error messages.
		return "";
	}
	
	/**
	 * This method gets the extended error message for the first UI error.
	 * It does not handle error exceptions.
	 * 
	 * @return String the first extended error string.  Empty string if there
	 * are no errors.
	 */
	public String GetFirstErrorExtendedText()
	{		
		if (m_Errors.MoveNext())
		{
			UIError error = (UIError) m_Errors.GetCurrent();			
			String strExtendedMsg = error.GetExtendedMessage();

			return strExtendedMsg;
		}
		// There were no error messages.
		return "";

	}

	/**
	 * This method displays multiple errors with HTML formatting and images.
	 * It also displays error exceptions and extended error messages.
	 * 
	 * After displaying the errors, it will clear all errors.
	 * 
	 * @return HTMLElement the formatted error HTML.
	 */
	public HTMLElement DisplayErrors()
	{
		HTMLElement result    = null;

		HTMLTable myTable     = null;
		HTMLTableRow myRow    = null;
		HTMLTableCell myCell  = null;

		HTMLSpan mySpan		  = null;

		UIError error;
		String sErrorTitle    = null;
		String sErrorMessage  = null;
		HTMLElement errorHTML = null;

		HTMLImg myErrorImg    = null;

		PTConstruct myConstruct;

		log.Debug("Entering Display() - bDisplaySeparatePageErrors: " +
			XPConvert.ToString(m_bDisplaySeparatePageErrors));
		log.Debug("AS Name: " + m_currentAS.GetName());

		try
		{
			myConstruct = UIConstructFactory.GetConstruct(PTImageType.ICON_ERROR,
				true, m_currentAS);

			myErrorImg = (HTMLImg) myConstruct.GetHTMLElement();
			myErrorImg.SetAlt(m_currentAS.GetString( 1592, ResourceConstants.PORTALBROWSINGMSGS ));
		}
		catch (Exception ex)
		{
			log.Error(ex, "Error displaying errors.");
		}

		try
		{
			myConstruct = UIConstructFactory.GetConstruct(PTImageType.SPACER,
				true, m_currentAS);
		}
		catch (Exception ex)
		{
			log.Error(ex, "Error getting spacer construct.");
		}

		boolean bGotErrors = false;

		boolean bEq;
		Exception exError;
		XPStringBuilder sbStackTrace = null;

		while (m_Errors.MoveNext())
		{
			error = (UIError) m_Errors.GetCurrent();

			sErrorMessage = null;
			errorHTML = null;

			exError = error.GetException();

			bEq = (error.GetDisplayOnSeparatePage() == m_bDisplaySeparatePageErrors);

			log.Debug("Display() - error.GetDisplayOnSeparatePage(): " +
				error.GetDisplayOnSeparatePage());
			log.Debug("Display() - bEq: " + XPConvert.ToString(bEq));

			if (bEq)
			{
				bGotErrors = true;

				// either:
				// 1) we're displaying seperate page errors, and this error is
				//    displayed on a separate page
				// or
				// 2) we're not, and it's not.
				//
				//	If this error is to be cleared immediately, clear it now that
				//	it's been displayed.  Since the view is really the final
				//	arbiter of what gets displayed, the view knows when to clear
				//	errors that should be cleared immediately upon display.
				if (error.GetImmediateClear())
				{
					UIErrorHandler.ClearError(error,
						m_currentAS.GetSubSession(),
						m_currentAS.GetApplication());
				}

				if (myTable == null)
				{
					myTable = new HTMLTable();

					myTable.SetBorder(CommonHTMLStrings.ZERO);
					myTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
					myTable.SetCellPadding("5");
					myTable.SetCellSpacing(CommonHTMLStrings.ZERO);
				}

				try
				{
					myRow = new HTMLTableRow();

					if( sbStackTrace != null )
					{
						myRow.SetPreComment( sbStackTrace.ToString() );
					}

					myRow.SetStyleClass(PTStyleClass.ALERT_BG);

					myTable.AddInnerHTMLElement(myRow);

					myCell = new HTMLTableCell();
					myRow.AddInnerHTMLElement(myCell);

					myCell.SetStyleClass(PTStyleClass.ALERT_ERROR_TITLE);
					myCell.SetVAlign(CommonHTMLStrings.TOP);
					myCell.SetAlign(CommonHTMLStrings.CENTER);

					myCell.SetWidth("80");

					myCell.AddInnerHTMLElement(myErrorImg);

					myCell = new HTMLTableCell();
					myRow.AddInnerHTMLElement(myCell);

					myCell.SetStyleClass(PTStyleClass.ALERT_ERROR_TITLE);
					myCell.SetAlign(CommonHTMLStrings.LEFT);
					myCell.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);

					mySpan = new HTMLSpan();
					mySpan.SetStyleClass(PTStyleClass.ALERT_ERROR_TITLE);
					myCell.AddInnerHTMLElement(mySpan);

					// retrieve error title and message.
					sErrorTitle = error.GetTitle();
					sErrorMessage = error.GetMessage();
						
					if (sErrorTitle != null)
					{
						// this is a user created title - we can't just append the
						// message to the title because it may look bad, so we'll 
						// just revert to the original way here.
						mySpan.AddInnerHTMLString(sErrorTitle);

						// now add the message if there is one.  the message will 
						// be just the next line down.
						if (sErrorMessage != null)
						{
							mySpan = new HTMLSpan();
						
							//NOTE - We may want to add a new style definition so that
							//when viewing source, the title is separated from the message.
							//mySpan.SetStyleClass(PTStyleClass.ALERT_ERROR_DESCRIPTION);
							mySpan.SetStyleClass(PTStyleClass.ALERT_ERROR_TITLE);
							myCell.AddInnerHTMLElement(mySpan);
					
							mySpan.AddInnerHTMLEncodedString(sErrorMessage);					
						}
					}
					else
					{
						// no title was given, so we are using the standard error 
						// titles.  in this case we can append the message to the
						// end of the title, if there is a message.  
						if (sErrorMessage != null)
						{
							// an error with additional message.
							if (error.GetLevel() == UIErrorLevel.WARNING)
							{
								sErrorTitle = XPResourceManager.FormatMessage(
									m_currentAS.GetString(146, ResourceConstants.INFRASTRUCTURE),
									sErrorMessage);
							}
							else
							{
								sErrorTitle = XPResourceManager.FormatMessage(
								m_currentAS.GetString(145, ResourceConstants.INFRASTRUCTURE),
									sErrorMessage);
							}
						}
						else
						{
							// an error that doesn't have additional messages.
							if (error.GetLevel() == UIErrorLevel.WARNING)
							{
								sErrorTitle = m_currentAS.GetString(49, ResourceConstants.INFRASTRUCTURE);
							}
							else
							{
								sErrorTitle = m_currentAS.GetString(48, ResourceConstants.INFRASTRUCTURE);
							}							
						}
						
						// add the default title and possibly message.
						mySpan.AddInnerHTMLEncodedString(sErrorTitle);
					}

					myCell = new HTMLTableCell();
					myCell.SetAlign(CommonHTMLStrings.RIGHT);
					myCell.SetWidth(CommonHTMLStrings.ZERO);

					myRow.AddInnerHTMLElement(myCell);

					HTMLElement eClearButton = null;

					if (error.GetManualClear())
					{
						//   This button can just be a redirect because the form
						//   needn't be submitted.
						ASURL myHRef = new ASURL(m_currentAS);
						myHRef.SetLinkGetCachedSpace(m_currentAS);

						myHRef.SetControl(ErrorControl.STR_MVC_CLASS_NAME);
						myHRef.AddControlArgument(ErrorControl.PARAM_CLEAR_ERROR,
							error.GetSessionKey());

						try
						{
							myConstruct = UIConstructFactory.GetConstruct(PTButtonType.CLOSE,
									true, m_currentAS);

							HTMLAnchor myAnchor = (HTMLAnchor) myConstruct.GetNestedElement(PTButtonType.CLOSE.GetElementName(
										PTButtonType.ELEMENT_NAME_ID_ANCHOR));

							myAnchor.SetHRef(myHRef.GetURLAsString());

							myConstruct.ReplaceNestedElementsWithHTMLString(PTButtonType.CLOSE.TITLE_ELEMENT_NAME,
								m_currentAS.GetString(50, ResourceConstants.INFRASTRUCTURE));

							HTMLImg myImg = (HTMLImg) myConstruct.GetNestedElement(PTButtonType.CLOSE.GetElementName(
										PTButtonType.ELEMENT_NAME_ID_IMG));

							if (myImg != null)
							{
								myImg.SetAlt(m_currentAS.GetString(51, ResourceConstants.INFRASTRUCTURE));
							}

							eClearButton = myConstruct.GetHTMLElement();
						}
						catch (Exception ex)
						{
							log.Error(ex, "Unable to create clear button.");
						}

						myCell.AddInnerHTMLElement(eClearButton);
					}
					 // this error requires manual clearing

					myRow = new HTMLTableRow();
					myRow.SetStyleClass(PTStyleClass.ALERT_BG);
					myTable.AddInnerHTMLElement(myRow);

					myCell = new HTMLTableCell();
					myRow.AddInnerHTMLElement(myCell);
					myCell.SetStyleClass(PTStyleClass.ALERT_ERROR_TITLE);
					myCell.SetVAlign(CommonHTMLStrings.TOP);
					myCell.SetAlign(CommonHTMLStrings.CENTER);
					myCell.SetWidth("80");
					myCell = new HTMLTableCell();
					myRow.AddInnerHTMLElement(myCell);
					myCell.SetAutoFit(true);
					
					errorHTML = error.GetHTMLMessage();

					if (errorHTML != null)
					{
						myCell.AddInnerHTMLElement(new HTMLBr());

						mySpan = new HTMLSpan();
						//NOTE - We may want to add a new style definition so that
						//when viewing source, the title is separated from the message.
						//mySpan.SetStyleClass(PTStyleClass.ALERT_ERROR_DESCRIPTION);
						mySpan.SetStyleClass(PTStyleClass.ALERT_ERROR_TITLE);
						myCell.AddInnerHTMLElement(mySpan);

						mySpan.AddInnerHTMLElement(errorHTML);
					}

					String sExtended = error.GetExtendedMessage();

					if (sExtended != null)
					{
						sExtended = XPHttpUtility.HtmlEncode( sExtended );
	
						myCell.SetPostComment("Extended Error Message: " +
							sExtended );
					}

					if( exError != null )
					{
						myRow.SetPostComment( "Exception Message: " +
							exError.getMessage() );
					}

				}
				catch (HTMLException he)
				{
					log.Error(he, "Unable to display error message.");
				}

				myRow = new HTMLTableRow();
				
				try
				{
					myTable.AddInnerHTMLElement(myRow);

					myCell = new HTMLTableCell();
					myRow.AddInnerHTMLElement(myCell);

					myCell.SetAutoFit(true);

					myCell.SetHeight("10");
				}
				catch (HTMLException he)
				{
					log.Error(he, "Unable to add spacer cell.");
				}
			}
			// we're display seperate page errors, and this error is displayed
			//   on a separate page, or we're not, and it's not.
		}
		 // for each error

		if (bGotErrors)
		{
			// Clear all errors
			UIErrorHandler.ClearErrors(m_currentAS.GetSubSession(),
				m_currentAS.GetApplication());

			result = myTable;
		}
		else
		{
			result = null;
		}

		return result;
	}
}