package com.plumtree.uiinfrastructure.errordisplay;

import com.plumtree.xpshared.htmlelements.*;

/**
 * @author Ben Geislinger
 *
 * UIError objects describe an error condition that needs to be communicated
 *     to one or more users of the portal.
 * <br><br>
 * UI code encountering an error condition should create a new UIError object,
 *     configure that UIError as appropriate, and raise it using the
 *     PTErrorHandler class.  Error messages will then be generated and displayed
 *     as appropriate.
 */
public class UIError
{
	private Exception m_myException     = null;
	private HTMLElement m_myHTMLMessage = null;
	private String m_myExtendedMessage  = null;

	// TODOCR JF 12/10/02 - Can you comment each of those member variables?
	private String m_myMessage = null;
	private String m_myTitle = null;
	private String m_strASName		 = null;
	private String m_strKey			 = null;
	private UIErrorLevel m_myLevel = null;
	private UIErrorScope m_myScope = null;
	private boolean m_bClearWithASChange     = false;
	private boolean m_bDisplayOnSeparatePage = false;
	private boolean m_bImmediateClear = true;
	private boolean m_bManualClear    = false;

	/**
	 * Creates a new UIError object.
	 *
	 * @param level UIErrorLevel enumeration indicates the severity of this error.
	 * @param scope UIErrorScope enumeration indicates the intended audience of this error.
	 * @param exception Previously caught exception might provide valuable clues to debuggers.
	 * @param message This message will be displayed to the user.
	 * @param extendedMessage This message will be embedded in HTML comments.
	 */
	public UIError(UIErrorLevel level, UIErrorScope scope, Exception exception,
		String message, String extendedMessage)
	{
		m_myLevel			= level;
		m_myScope			= scope;
		m_myException		= exception;
		m_myMessage			= message;
		m_myExtendedMessage = extendedMessage;
	}

	/**
	 * Creates a new UIError object.
	 *
	 * @param level UIErrorLevel enumeration indicates the severity of this error.
	 * @param scope UIErrorScope enumeration indicates the intended audience of this error.
	 * @param message This message will be displayed to the user.
	 * @param extendedMessage This message will be embedded in HTML comments.
	 */
	public UIError(UIErrorLevel level, UIErrorScope scope, String message,
		String extendedMessage)
	{
		m_myLevel			= level;
		m_myScope			= scope;
		m_myMessage			= message;
		m_myExtendedMessage = extendedMessage;
	}

	/**
	 * Call this method to determine the activity space with which this error
	 *     is associated.  This method returns null if this error is not associated
	 *     with an activity space.
	 *
	 * @return String name of the activity space.
	 */
	public String GetActivitySpace()
	{
		return m_strASName;
	}

	/**
	 * Call this method to determine whether or not this error should be cleared
	 *     when the user changes activity spaces.
	 *
	 * @return boolean value
	 */
	public boolean GetClearWithActivitySpaceChange()
	{
		return m_bClearWithASChange;
	}

	/**
	 * Call this method to determine whether or not the error message for this
	 * UIError object should be displayed on a separate ui page.
	 * <br><br>
	 * @return boolean value
	 */
	public boolean GetDisplayOnSeparatePage()
	{
		return m_bDisplayOnSeparatePage;
	}

	/**
	 * Call this method to obtain the Exception associated with this error.
	 * <br><br>
	 * @return Exception
	 */
	public Exception GetException()
	{
		return m_myException;
	}

	/**
	 * Call this method to obtain the extended message of this error.
	 * <br><br>
	 * @return String extended message
	 */
	public String GetExtendedMessage()
	{
		return m_myExtendedMessage;
	}

	/**
	 * Call this method to obtain the HTMLElement message displayed with this
	 *     error.
	 * <br><br>
	 * @return HTMLElement message
	 */
	public HTMLElement GetHTMLMessage()
	{
		return m_myHTMLMessage;
	}

	/**
	 *
	 *
	 * @return
	 */
	public boolean GetImmediateClear()
	{
		return m_bImmediateClear;
	}

	/**
	 *
	 *
	 * @return
	 */
	public UIErrorLevel GetLevel()
	{
		return m_myLevel;
	}

	/**
	 *
	 *
	 * TODOCR JF 12/10/02 - Javadoc?
	 *
	 * @return
	 */
	public boolean GetManualClear()
	{
		return m_bManualClear;
	}

	/**
	 * Call this method to obtain the String message displayed along with this
	 *     error.
	 * <br><br>
	 * @return String message.
	 */
	public String GetMessage()
	{
		return m_myMessage;
	}

	/**
	 *
	 *
	 * @return
	 */
	public UIErrorScope GetScope()
	{
		return m_myScope;
	}

	/**
	 * Call this method to obtain the key with which this error was stored on
	 *     the user's session or application.
	 * <br><br>
	 * @return String value.
	 */
	public String GetSessionKey()
	{
		return m_strKey;
	}

	/**
	 * Call this method to obtain the title of this error.
	 * <br><br>
	 * @return String title
	 */
	public String GetTitle()
	{
		return m_myTitle;
	}

	/**
	 * Call this method to associate this error with an activity space.  This
	 *     error will only be seen by the user when the user is on the specified
	 *     activity space.
	 * <br><br>
	 * Note that it does not make sense to associate this error with an activity
	 *     space if this error gets its own display page, because the error display
	 *     page is its own activity space.
	 * <br><br>
	 *
	 * @param sName Pass the String name of the activity space.
	 */
	public void SetActivitySpace(String sName)
	{
		m_strASName = sName;
	}

	/**
	 * Call this method to toggle whether or not this error should be cleared
	 *     from memory when the user changes activity spaces.  This clearing will
	 *     only take affect if you also call the SetActivitySpace( String ) method
	 *     to tie this error to an activity space.
	 *
	 * @param b boolean value
	 */
	public void SetClearWithActivitySpaceChange(boolean b)
	{
		m_bClearWithASChange = b;
	}

	/**
	 * Call this method to toggle whether or not the error message for this
	 *     UIError object should be displayed on a separate ui page.
	 * <br><br>
	 * Pass false to have this error displayed on the main portal or editor
	 *     page.  Pass true to have this error displayed on a separate page.
	 *
	 * @param b boolean value.
	 */
	public void SetDisplayOnSeparatePage(boolean b)
	{
		m_bDisplayOnSeparatePage = b;
	}

	/**
	 * Call this method to set the Exception included within this error.
	 * <br><br>
	 * Information from this exception will be logged, but it will not
	 *     be displayed to the user.
	 * <br><br>
	 * @param ex Exception
	 */
	public void SetException(Exception ex)
	{
		m_myException = ex;
	}

	/**
	 * Call this method to set the message displayed along with this error.
	 * <br><br>
	 * An error can contain both a String message and an HTMLElement message,
	 *     and both will be displayed along with the error.
	 * <br><br>
	 * @param e HTMLElement message.
	 */
	public void SetHTMLMessage(HTMLElement e)
	{
		m_myHTMLMessage = e;
	}

	/**
	 *
	 *
	 * @param b
	 */
	public void SetImmediateClear(boolean b)
	{
		m_bImmediateClear     = b;
	}

	/**
	 *
	 *
	 * TODOCR JF 12/10/02 - What is this doing? No Javadoc?
	 *
	 * @param b
	 */
	public void SetManualClear(boolean b)
	{
		m_bManualClear	    = b;
	}

	/**
	 * Call this method to set the title of this error.
	 * <br><br>
	 * This title will be displayed above the error description.
	 * <br><br>
	 * @param title String title
	 */
	public void SetTitle(String title)
	{
		m_myTitle = title;
	}

	/**
	 * Call this method to set the session key that was used to store this
	 *     error on the user's session or application.
	 * <br><br>
	 * @param s String key.
	 */
	public void SetSessionKey(String s)
	{
		m_strKey = s;
	}
}

