package com.plumtree.uiinfrastructure.expandablelist;


// Framework Imports
import com.plumtree.openlog.*;

import com.plumtree.openfoundation.util.*;

import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.arraywrapper.*;
import com.plumtree.uiinfrastructure.common.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.form.pagination.*;

import com.plumtree.xpshared.htmlconstructs.*;

/**
 * This class contains the state of the Expandable List. The data contained in
 * that Model are printed out by  the associated View: ExpListView. The repost
 * actions will be  handled by the ExpListControl class.  That class extends
 * the Editor Framework plug-in section  classes so it can be added to any
 * Editor page pretty easily. (see template 7)  The Expandable List Framework
 * is abstract and has to be  extended in order to be used.  Note that the
 * Expandable List uses the Array Wrapper  Framework to actually store the
 * data and facilitate the  implementation of the sorting requirements.
 *
 * @author Julie Fournier
 */
abstract public class ExpListModel extends EditorHelperModel implements IExpListModelRO, IPaginationModel
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.expandablelist.ExpListModel");

	/**
	 * Key (as an Integer) to get back the column ID  from m_arrExpListColInfo
	 */
	public static final int EXPLIST_COLUMN_ID = 0;

	/**
	 * Key (as a String) to get back the column title  from m_arrExpListColInfo
	 */
	public static final int EXPLIST_COLUMN_TITLE = 1;

	/**
	 * Key (as a String) to get back the column width  from m_arrExpListColInfo
	 */
	public static final int EXPLIST_COLUMN_WIDTH = 2;

	/** Colspan for the HTML cell. */
	public static final int EXPLIST_COLUMN_COLSPAN = 3;

	/** Flag to know if the link to sort by that column should be enabled. */
	public static final int EXPLIST_COLUMN_ENABLESORT = 4;

	/** MES- Flag to know how many flags there are- how many columns */
	public static final int EXPLIST_COLUMN_INVALID = 5;

	/**
	 * We are defining here a column ID for the Growable Sorted Array objects.
	 * This column has to reflect the position of the row in the list. Methods
	 * to implement the move up, down, first and last actions are already
	 * implemented in this class and are ready to be used. However they will
	 * work only if the column ID below exists in the object array.
	 */
	public static final int EXPLIST_SORTEDARRAY_PROPID_INDEX = 0;

	/**
	 * The move action value is posted by the form as HTMLHIDDEN_MOVEACTION. It
	 * can have one of the four values below.
	 */
	/** Will move the corresponding item to the top of the list. */
	public static final int MOVEACTION_FIRST = 1;

	/** Will move the corresponding item one row up. */
	public static final int MOVEACTION_UP = 2;

	/** Will move the corresponding item one row down. */
	public static final int MOVEACTION_DOWN = 3;

	/** Will move the corres ponding item to the bottom of the list. */
	public static final int MOVEACTION_LAST = 4;

	/**
	 * To keep track of the next available index.  This index is incremented
	 * each time a row is added  to the list.
	 */
	public int m_nNextIndex = 0;

	/** Associated AS class */
	protected AActivitySpace parent;

	/**
	 * This is where we store the data displayed on the list. We are taking
	 * advantage of the sorting features offered  by the Array Wrapper
	 * framework.
	 */
	protected PTGrowableSortedArrayWrapper m_saData;

	/**
	 * JF- Wrap attribute for the column headers
	 */
	protected boolean m_bWrapColumnHeaders = true;

	/** The add icon. */
	protected PTImageType m_imgAddIcon;

	/** The delete icon. */
	protected PTImageType m_imgDeleteIcon;

	/** Alt text for the add icon. */
	protected String m_sAddIconAlt;

	/** Text to display next to the add icon. */
	protected String m_sAddLabel;

	/** Alt text for the delete icon. */
	protected String m_sDeleteIconAlt;

	/** Message to display as a JavaScript alert if the list is empty. */
	protected String m_sEmptyListAlert;

	/** Instructions to display when the list is empty. */
	protected String m_sInstructions = "";

	/** Main message to display when the list is empty (in red). */
	protected String m_sMainMessage = "";

	/** The section main title. */
	protected String m_sSectionTitle = "";

	/** Array of column titles, associated column IDs and column width */
	protected Object[][] m_arrExpListColInfo;

	/**
	 * DCA - Variable to show add button or not.
	 */
	protected boolean m_bAddButtonDisabled = false;

	/**
	 * DCA - Variable to show check boxes or not.
	 */
	protected boolean m_bCheckBoxesDisabled = false;

	/**
	 * Set this flag to true if you want to display the move up and  down
	 * buttons at the end of each row.  Warning: if this is set to true, then
	 * the sort by column will  NOT be available anymore. You can't have both
	 * it doesn't make sense.
	 */
	protected boolean m_bEnableMoveRowActions = false;

	/**
	 * Special mode to have a read-only expandable list: all actions are hidden.
	 */
	protected boolean m_bIsReadOnly;

	/**
	 * JF- List instructions.
	 */
	protected String m_sTableInstructions;

	/**
	 * JF- COnfirmation message to display when the user deletes a row.
	 */
	protected String m_sDelRowConfirmMsg;

	/**
	 * This key is used when there is more than one expandable  list on the
	 * same Display Page.
	 */
	protected int m_nListKey;

	/**
	 * Will be called each time the add action is performed. Should take care
	 * of adding the row(s) to the list. Use the GetItems() method to get the
	 * list.
	 *
	 * @param _htFormData Form data
	 */
	abstract public void AddRowsOnRepost(XPHashtable _htFormData);

	/**
	 * Gets the array of checked rows and delete  them from the list.
	 *
	 * @param _htFormData
	 */
	public void DeleteRowsOnRepost(XPHashtable _htFormData)
	{
		if (!m_bIsReadOnly)
		{
			String sDataKey = GetUniqueString(ExpListView.HTMLCHECKBOX_DELROW);

			// JF- We get back the list of index of rows to delete.
			if (_htFormData.ContainsKey(sDataKey))
			{
				int[] aRowsToDelete = GetArrayOfObjToDelete(_htFormData);

				// JF- We can only delete rows if the array is not empty
				if (m_saData.GetCount() > 0)
				{
					m_saData.RemoveRows(aRowsToDelete);
				}
			}
		}
	}

	/**
	 * DCA - Choose not to display add button
	 *
	 */
	public void DisableAddButton()
	{
		m_bAddButtonDisabled = true;
	}

	/**
	 * DCA - Disable check boxes.
	 */
	public void DisableCheckBoxes()
	{
		m_bCheckBoxesDisabled = true;
	}

	/**
	 * JF- Sets the wrap cell attribute for the column headers.
	 * Is true by default.
	 *
	 * @param _bWrapColumnHeaders
	 */
	public void SetWrapColumnHeaders(boolean _bWrapColumnHeaders)
	{
		m_bWrapColumnHeaders = _bWrapColumnHeaders;
	}

	/**
	 * JF- Returns the wrap attribute for the column headers.
	 *
	 * @return boolean
	 */
	public boolean GetWrapColumnHeaders()
	{
		return m_bWrapColumnHeaders;
	}

	/**
	 * JF- Returns table intructions
	 *
	 * @return String
	 */
	public String GetTableInstructions()
	{
		return m_sTableInstructions;
	}

	/**
	 * JF- Sets table intructions.
	 *
	 * @param _sTableInstructions
	 */
	public void SetTableInstructions(String _sTableInstructions)
	{
		m_sTableInstructions = _sTableInstructions;
	}

	/**
	 * JF- Returns the confirmation message to display when the user tries to delete a row.
	 *
	 * @return String
	 */
	public String GetDelRowConfirmMsg()
	{
		return m_sDelRowConfirmMsg;
	}

	/**
	 * JF- Sets the confirmation message to display when the user tried to delete a row.
	 *
	 * @param m_sDelRowConfirmMsg
	 */
	public void SetDelRowConfirmMsg(String _sDelRowConfirmMsg)
	{
		m_sDelRowConfirmMsg = _sDelRowConfirmMsg;
	}

	/**
	 * Returns icon for the add action.
	 *
	 * @return PTImageType
	 */
	public PTImageType GetAddIcon()
	{
		return m_imgAddIcon;
	}

	/**
	 * Returns the alt text for the add icon.
	 *
	 * @return String
	 */
	public String GetAddIconAlt()
	{
		return m_sAddIconAlt;
	}

	/**
	 * Returns the String to display next to the add icon.
	 *
	 * @return String
	 */
	public String GetAddLabel()
	{
		return m_sAddLabel;
	}

	/**
	 * Returns the message to display by the JavaScript when the list is empty.
	 *
	 * @return String
	 */
	public String GetAlertMessageForEmptyList()
	{
		return m_sEmptyListAlert;
	}

	/**
	 * Returns the array of column titles,  associated column IDs and column
	 * width.
	 *
	 * @return Object[][]
	 */
	public Object[][] GetArrExpListColInfo()
	{
		return m_arrExpListColInfo;
	}

	/**
	 * ROB: Make the available for other models.
	 *
	 * @param _htFormData
	 * @return int[]
	 */
	public int[] GetArrayOfObjToDelete(XPHashtable _htFormData)
	{
		String sDataKey = GetUniqueString(ExpListView.HTMLCHECKBOX_DELROW);
		String[] aDataValue = (String[]) _htFormData.GetElement(sDataKey);

		int[] aRowsToDelete = new int[aDataValue.length];
		int nRowIndex;

		for (int i = 0; i < aDataValue.length; i++)
		{
			try
			{
				nRowIndex = XPConvert.ToInteger(aDataValue[i]);

				if (nRowIndex < m_saData.GetCount())
				{
					aRowsToDelete[i] = nRowIndex;
				}
			}
			catch (Exception e)
			{
				
					log.Error("ExpListModel::GetArrayOfObjToDelete Unbale to retrieve row index.");
				
			}
		}

		return aRowsToDelete;
	}

	/**
	 * Returns icon for the delete action.
	 *
	 * @return PTImageType
	 */
	public PTImageType GetDeleteIcon()
	{
		return m_imgDeleteIcon;
	}

	/**
	 * Returns the alt text for the delete icon.
	 *
	 * @return String
	 */
	public String GetDeleteIconAlt()
	{
		return m_sDeleteIconAlt;
	}

	/**
	 * Returns the secondary String to display when the String is empty.
	 * (Should give instructions to the user on what to do).
	 *
	 * @return String
	 */
	public String GetEmptyListInstructions()
	{
		return m_sInstructions;
	}

	/**
	 * Returns the main String to diplay when the list is empty.
	 *
	 * @return String
	 */
	public String GetEmptyListMainMessage()
	{
		return m_sMainMessage;
	}

	/**
	 * Returns true if the move up and down buttons should be displayed. (use
	 * GetHTMLForMoveRowActions in ExpListView to dsiplay the HTMl  on each
	 * row)
	 *
	 * @return boolean
	 */
	public boolean GetEnableMoveRowActions()
	{
		return m_bEnableMoveRowActions;
	}

	/**
	 * Returns the value of the m_bIsReadOnly member variable.
	 * If this is true, the user will not be able to perform any action on the list.
	 *
	 * @return boolean
	 */
	public boolean GetIsListReadOnly()
	{
		return m_bIsReadOnly;
	}

	/**
	 * Returns the array of data.
	 *
	 * @return PTGrowableSortedArrayWrapper
	 */
	public PTGrowableSortedArrayWrapper GetItems()
	{
		return m_saData;
	}

	/**
	 * Returns the key of the expandable list. (This key is used to avoid
	 * conflicts when  there are multiple lists on the same page.)
	 *
	 * @return int
	 */
	public int GetListKey()
	{
		return m_nListKey;
	}

	/**
	 * Sets a new value for this list key.
	 *
	 * @param _nListKey
	 */
	public void SetListKey(int _nListKey)
	{
		m_nListKey = _nListKey;
	}

	/**
	 * Returns the column ID of the primary sorted column. (the array can be
	 * sorted by multiple columns)
	 *
	 * @return int
	 */
	public int GetMainColumnSort()
	{
		int[][] arrSort = m_saData.GetSort();

		return arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_PROPID];
	}

	/**
	 * Returns the sort direction of the primary sorted column.
	 *
	 * @return int
	 */
	public int GetMainColumnSortDirection()
	{
		int[][] arrSort = m_saData.GetSort();

		return arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION];
	}

	/**
	 * Returns a read-only interface of the data.
	 *
	 * @return IPTGrowableSortedArrayWrapperRO
	 */
	public IPTGrowableSortedArrayWrapperRO GetReadOnlyItems()
	{
		return (IPTGrowableSortedArrayWrapperRO) m_saData;
	}

	/**
	 * Returns the section title.
	 *
	 * @return String
	 */
	public String GetSectionTitle()
	{
		return m_sSectionTitle;
	}

	/**
	 * Creates dynamically a unique int and returns it. This method can be used
	 * for repost actions for example. This is to make sure that if there are
	 * more than one expandable  list on the same dislpay page, there won't be
	 * any conflict  between the two lists.
	 *
	 * @param _nValue
	 *
	 * @return int
	 */
	public int GetUniqueInt(int _nValue)
	{
		return XPConvert.ToInteger(XPConvert.ToString(_nValue) + XPConvert.ToString(m_nListKey));
	}

	/**
	 * Creates dynamically a unique name with the list key and returns it. This
	 * is to make sure that if there are more than one expandable  list on the
	 * same dislpay page, there won't be any conflict  between the two lists.
	 * This method should be used to create unique input names.  It can also
	 * be used for JavaScript function names (other than  postToSelf - for
	 * this one you just need to use the GetUniqueInt  method to make sure
	 * your repost actions and other int values  are unique).
	 *
	 * @param _sName
	 *
	 * @return String
	 */
	public String GetUniqueString(String _sName)
	{
		return (_sName + m_nListKey);
	}

	/**
	 * Initializes the expandable list.
	 *
	 * @param _sSectionTitle Main title.
	 * @param _arrExpListColInfo Information about columns: title, ID and
	 *        width.
	 * @param _imgAddIcon Icon for the add action.
	 * @param _sAddLabel Text to display next to the add icon.
	 * @param _nListKey In case you have multiple Expandable Lists on the same
	 *        DP, they should have a different key (otherwise, just set it to
	 *        1).
	 *
	 */
	public void InitializeExpandableList(String _sSectionTitle, Object[][] _arrExpListColInfo, PTImageType _imgAddIcon, String _sAddLabel, int _nListKey)
	{
		if (5 > _arrExpListColInfo[0].length)
		{
			throw new XPException("ExpListModel::InitializeExpandableList _arrExpListColInfo doesn't have 5 columns.");
		}

		m_sSectionTitle		    = _sSectionTitle;
		m_arrExpListColInfo     = _arrExpListColInfo;
		m_nListKey			    = _nListKey;
		m_imgAddIcon		    = _imgAddIcon;
		m_sAddLabel			    = _sAddLabel;

		m_saData			 = new PTGrowableSortedArrayWrapper();
		m_imgDeleteIcon		 = PTImageType.ACTION_REMOVE;
		m_sAddIconAlt		 = "";
		m_sDeleteIconAlt     = "";

		InitializeSortedArray(m_saData);
		
		// Register Pagination Controls
		if (!m_asOwner.GetControlExists(PaginationControl.STR_MVC_CLASS_NAME))
		{
			m_asOwner.RegisterControl(PaginationControl.STR_MVC_CLASS_NAME, this);
		}
		
		if (!m_asOwner.GetControlExists(PassthruControl.STR_MVC_CLASS_NAME))
		{
			m_asOwner.RegisterControl(PassthruControl.STR_MVC_CLASS_NAME, this);
		}
	}

	/**
	 * Initializes the read-only expandable list.
	 *
	 * @param _sSectionTitle Main title.
	 * @param _arrExpListColInfo Information about columns: title, ID and
	 *        width.
	 * @param _nListKey In case you have multiple Expandable Lists on the same
	 *        DP, they should have a different key (otherwise, just set it to
	 *        1).
	 */
	public void InitializeReadOnlyExpandableList(String _sSectionTitle, Object[][] _arrExpListColInfo, int _nListKey)
	{
		if (5 > _arrExpListColInfo[0].length)
		{
			throw new XPException("ExpListModel::InitializeExpandableList _arrExpListColInfo doesn't have 5 columns.");
		}

		m_sSectionTitle		    = _sSectionTitle;
		m_arrExpListColInfo     = _arrExpListColInfo;
		m_nListKey			    = _nListKey;
		m_bIsReadOnly		    = true;

		m_saData			 = new PTGrowableSortedArrayWrapper();
		m_imgDeleteIcon		 = PTImageType.ACTION_REMOVE;
		m_sAddIconAlt		 = "";
		m_sDeleteIconAlt     = "";

		InitializeSortedArray(m_saData);
	}

	/**
	 * Will be called when this class is initalized.  Should take care of
	 * initializing _saData (calling Initialize).
	 *
	 * @param _saData
	 */
	abstract public void InitializeSortedArray(PTGrowableSortedArrayWrapper _saData);

	/**
	 * DCA - Check if add button is to be displayed
	 *
	 */
	public boolean IsAddButtonDisabled()
	{
		return m_bAddButtonDisabled;
	}

	/**
	 * DCA - Check to see if check boxes has been disabled.
	 *
	 * @return boolean
	 */
	public boolean IsCheckBoxesDisabled()
	{
		return m_bCheckBoxesDisabled;
	}

	/**
	 * Action for modifying a row
	 *
	 * @param _htFormData
	 */
	public void ModifyRow(XPHashtable _htFormData)
	{
		// This should be implemented by child classes if it is needed.
	}

	/**
	 * Gets the row index back and the move action back. Then perform the
	 * associated action.
	 *
	 * @param _htFormData
	 */
	public void MoveRow(XPHashtable _htFormData)
	{
		// JF- The move rows action is supported only if the EXPLIST_SORTEDARRAY_INDEX 
		// column is in the object array.
		if (m_saData.ContainsPropID(EXPLIST_SORTEDARRAY_PROPID_INDEX))
		{
			int nRowIndex = GetRowIndex(_htFormData);
			int nMoveAction = GetMoveAction(_htFormData);

			
				log.Debug("ExpListModel::MoveRow The move action value is: " + nMoveAction);
			

			if ((-1 != nRowIndex) && (-1 != nMoveAction))
			{
				switch (nMoveAction)
				{
				case MOVEACTION_FIRST:
					MoveFirst(nRowIndex);

					break;

				case MOVEACTION_UP:
					MoveUp(nRowIndex);

					break;

				case MOVEACTION_DOWN:
					MoveDown(nRowIndex);

					break;

				case MOVEACTION_LAST:
					MoveLast(nRowIndex);

					break;

				default:

					
						log.Warn("ExpListModel::MoveRow Move action: " + nMoveAction + " is invalid.");
					
				}
			}
		}
	}

	/**
	 * Reverses the sort of the primary sorted column.
	 */
	public void ReverseCurrentSort()
	{
		int[][] arrSort = m_saData.GetSort();

		// JF- Get the current sort.
		int nCurrentDirection = arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION];

		// JF- 
		if (PTSortedArrayWrapper.PTSAW_SORT_ASCENDING == nCurrentDirection)
		{
			arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION] = PTSortedArrayWrapper.PTSAW_SORT_DESCENDING;
		}
		else
		{
			arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION] = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;
		}

		m_saData.Sort(arrSort);
	}

	/**
	 * Saves the data when a repost occurs. Editors using this framework need
	 * to call that method from  the savePage method of the main Model.
	 *
	 * @param _htFormData
	 *
	 * @return int Status for server side validation
	 */
	public int SaveSection(XPHashtable _htFormData)
	{
		int nSaveStatus = RepostControl.PAGE_STATUS_VALID;
		
		// JF- For each row in the list we call the abstract saveRow method.
		for (int nRowIndex = 0; nRowIndex < m_saData.GetCount(); nRowIndex++)
		{
			if (RepostControl.PAGE_STATUS_INVALID == SaveRow(m_saData, nRowIndex, _htFormData))
			{
				nSaveStatus = RepostControl.PAGE_STATUS_INVALID;
			}
		}
		
		return nSaveStatus;
	}

	/**
	 * The delete icon will be set by default to add.gif.  Use that method
	 * if you want to change it.
	 *
	 * @param __imgAddIcon
	 */
	public void SetAddIcon(PTImageType _imgAddIcon)
	{
		m_imgAddIcon = _imgAddIcon;
	}

	/**
	 * Sets the value of the alt text for the add icon. This is optional, by
	 * default this is set to empty String.
	 *
	 * @param _sAddIconAlt
	 */
	public void SetAddIconAlt(String _sAddIconAlt)
	{
		m_sAddIconAlt = _sAddIconAlt;
	}

	/**
	 * The delete icon will be set by default to delete.gif.  Use that method
	 * if you want to change it.
	 *
	 * @param _imgDeleteIcon
	 */
	public void SetDeleteIcon(PTImageType _imgDeleteIcon)
	{
		m_imgDeleteIcon = _imgDeleteIcon;
	}

	/**
	 * Sets the value of the alt text for the delete icon. This is optional, by
	 * default this is set to empty String.
	 *
	 * @param _sDeleteIconAlt
	 */
	public void SetDeleteIconAlt(String _sDeleteIconAlt)
	{
		m_sDeleteIconAlt = _sDeleteIconAlt;
	}

	/**
	 * Sets messages to display when the list is empty.
	 *
	 * @param _sMainMessage Main message to display in red.
	 * @param _sInstructions Instructions on what to do.
	 */
	public void SetEmptyListMessages(String _sMainMessage, String _sInstructions)
	{
		m_sMainMessage	    = _sMainMessage;
		m_sInstructions     = _sInstructions;
	}

	/**
	 * Enable the JavaScript validation to ensure that the list is not empty.
	 * The String in argument will be displayed by the JavaScript alert. And
	 * the String returned is the JavaScript name that should be  registered
	 * for validation in the Editor. The name returned will be  prefixed with
	 * the Expandable List key to avoid conflicts.
	 *
	 * @param _sEmptyListAlert
	 *
	 * @return String JavaScript function name
	 */
	public String SetEnableEmptyListClientValidation(String _sEmptyListAlert)
	{
		m_sEmptyListAlert = _sEmptyListAlert;

		return GetUniqueString("CheckEmptyList");
	}

	/**
	 * Set this flag to true if you want to display the move up  and down
	 * buttons at the end of each row.  Warning: if this is set to true, then
	 * the sort  by column will NOT be available anymore.
	 *
	 * @param _bEnableMoveRowActions
	 */
	public void SetEnableMoveRowActions(boolean _bEnableMoveRowActions)
	{
		m_bEnableMoveRowActions = _bEnableMoveRowActions;

		// JF- Disable the sort on each column
		if (m_bEnableMoveRowActions)
		{
			for (int i = 0; i < m_arrExpListColInfo.length; i++)
			{
				m_arrExpListColInfo[i][EXPLIST_COLUMN_ENABLESORT] = Boolean.FALSE;
			}
		}

		// JF- Resort the array by index
		int[][] arrSort = new int[1][2];
		arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_PROPID]		  = EXPLIST_SORTEDARRAY_PROPID_INDEX;
		arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION]     = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;
		m_saData.Sort(arrSort);
	}

	/**
	 * Changes the value of the m_bIsReadOnly member variable.
	 * If the list is read only, no action will be displayed in the action bar.
	 *
	 * @param _bIsReadOnly
	 */
	public void SetIsListReadOnly(boolean _bIsReadOnly)
	{
		m_bIsReadOnly = _bIsReadOnly;
	}

	/**
	 * Sets the section title.
	 *
	 * @param _sSectionTitle
	 */
	public void SetSectionTitle(String _sSectionTitle)
	{
		m_sSectionTitle = _sSectionTitle;
	}

	/**
	 * Gets the column ID back from the form data and resort  the array by that
	 * column ID, ascending.
	 *
	 * @param _htFormData
	 */
	public void SortByColumn(XPHashtable _htFormData)
	{
		String sDataKey = GetUniqueString(ExpListView.HTMLHIDDEN_COLUMNID);

		// JF- First we need to get back the column ID from the form data.
		if (_htFormData.ContainsKey(sDataKey))
		{
			String[] aDataValue = (String[]) _htFormData.GetElement(sDataKey);

			if ((null != aDataValue) && (0 != aDataValue.length))
			{
				try
				{
					int nColumnID = XPConvert.ToInteger(aDataValue[0]);

					// JF- Then we get back the sort and update and modify the 1st row.
					// If the array was sorted by more than 1 column we want to keep the other columns.
					int[][] arrSort = m_saData.GetSort();
					arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_PROPID]		  = nColumnID;
					arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION]     = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;

					// JF- Finally we resort the array
					m_saData.Sort(arrSort);
				}
				catch (Exception e)
				{
					
						log.Error("ExpListModel::SortByColumn An exception occured" + " when trying to resort the array by column with ID: " + aDataValue[0]);
					
				}
			}
		}
	}

	/**
	 * Helper method to retrieve the move action type from the form data.
	 *
	 * @param _htFormData
	 *
	 * @return int
	 */
	protected int GetMoveAction(XPHashtable _htFormData)
	{
		int nMoveAction = -1;

		String sDataKey = GetUniqueString(ExpListView.HTMLHIDDEN_MOVEACTION);

		if (_htFormData.ContainsKey(sDataKey))
		{
			String[] aDataValue = (String[]) _htFormData.GetElement(sDataKey);

			if ((null != aDataValue) && (0 != aDataValue.length))
			{
				try
				{
					nMoveAction = XPConvert.ToInteger(aDataValue[0]);
				}
				catch (Exception e)
				{
					
						log.Error("ExpListModel::GetMoveAction An exception occured" + " when trying to get the move action back from the form data. -1 will be returned instead.");
					
				}
			}
		}

		return nMoveAction;
	}

	/**
	 * Helper method to retrieve the row index from the form data.
	 *
	 * @param _htFormData
	 *
	 * @return int
	 */
	protected int GetRowIndex(XPHashtable _htFormData)
	{
		int nRowIndex = -1;

		String sDataKey = GetUniqueString(ExpListView.HTMLHIDDEN_ROWINDEX);

		if (_htFormData.ContainsKey(sDataKey))
		{
			String[] aDataValue = (String[]) _htFormData.GetElement(sDataKey);

			if ((null != aDataValue) && (0 != aDataValue.length))
			{
				try
				{
					nRowIndex = XPConvert.ToInteger(aDataValue[0]);
				}
				catch (Exception e)
				{
					
						log.Error("ExpListModel::GetRowIndex An exception occured" + " when trying to get the row index back from the form data. -1 will be returned instead.");
					
				}
			}
		}

		return nRowIndex;
	}

	/**
	 * Moves the row identified by the index in argument, one row down. The
	 * idea is that we are going to change the value of the index column.  We
	 * just need to swap that value with the one of the row just after.
	 *
	 * @param _nRowIndex Index of the row to move.
	 */
	protected void MoveDown(int _nRowIndex)
	{
		// Make sure the row is not already in last position.
		if (m_saData.GetCount() > (_nRowIndex + 1))
		{
			try
			{
				// JF- We get back the index values of the row to move and the one just before.
				Object oCurrentRow = m_saData.GetItem(_nRowIndex, EXPLIST_SORTEDARRAY_PROPID_INDEX);
				Object oNextRow = m_saData.GetItem(_nRowIndex + 1, EXPLIST_SORTEDARRAY_PROPID_INDEX);

				Object[][] arrNewItem = new Object[2][3];
				arrNewItem[0][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_INDEX] = new Integer(_nRowIndex);
				arrNewItem[0][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_PROPERTYID] = new Integer(EXPLIST_SORTEDARRAY_PROPID_INDEX);
				arrNewItem[0][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_NEWVALUE] = oNextRow;
				
				arrNewItem[1][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_INDEX] = new Integer(_nRowIndex + 1);
				arrNewItem[1][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_PROPERTYID] = new Integer(EXPLIST_SORTEDARRAY_PROPID_INDEX);
				arrNewItem[1][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_NEWVALUE] = oCurrentRow;

				// JF- Then we swap the values.
				m_saData.SetItems(arrNewItem);

				// JF- The object array is sorted automatically when a value is set.
			}
			catch (Exception e)
			{
				
					log.Error("ExpListModel::MoveDown An exception occured" + " when trying to move down the row: " + _nRowIndex + ". " + e);
				
			}
		}
	}

	/**
	 * Moves the row identified by the index in argument,  to the top of the
	 * list.
	 *
	 * @param _nRowIndex Index of the row to move.
	 */
	protected void MoveFirst(int _nRowIndex)
	{
		// Make sure the row is not already in first position.
		if (0 != _nRowIndex)
		{
			try
			{
				Object[][] arrNewItem = new Object[_nRowIndex + 1][3];
				
				arrNewItem[0][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_INDEX] = new Integer(_nRowIndex);
				arrNewItem[0][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_PROPERTYID] = new Integer(EXPLIST_SORTEDARRAY_PROPID_INDEX);
				arrNewItem[0][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_NEWVALUE] = new Integer(0);

				int i = 1;
				
				// JF- Then, one by one, we move the row to the top.
				while (i <= _nRowIndex)
				{					
					arrNewItem[i][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_INDEX] = new Integer(i - 1);
					arrNewItem[i][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_PROPERTYID] = new Integer(EXPLIST_SORTEDARRAY_PROPID_INDEX);
					arrNewItem[i][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_NEWVALUE] = new Integer(i);
					
					i++;
				}
				
				m_saData.SetItems(arrNewItem);

				// JF- The object array is sorted automatically when a value is set.
			}
			catch (Exception e)
			{
				
					log.Error("ExpListModel::MoveFirst An exception occured" + " when trying to move first the row: " + _nRowIndex + ". " + e);
				
			}
		}
	}

	/**
	 * Moves the row identified by the index in argument,  to the last position
	 * of the list.
	 *
	 * @param _nRowIndex Index of the row to move.
	 */
	protected void MoveLast(int _nRowIndex)
	{
		int nCount = m_saData.GetCount();
		
		// Make sure the row is not already in last position.
		if (nCount > (_nRowIndex + 1))
		{
			try
			{
				Object[][] arrNewItem = new Object[nCount - _nRowIndex][3];
				
				arrNewItem[0][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_INDEX] = new Integer(_nRowIndex);
				arrNewItem[0][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_PROPERTYID] = new Integer(EXPLIST_SORTEDARRAY_PROPID_INDEX);
				arrNewItem[0][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_NEWVALUE] = new Integer(nCount - 1);

				int i = _nRowIndex + 1;
				
				// JF- Then, one by one, we move the row to the bottom.
				while (i < nCount)
				{					
					arrNewItem[i - _nRowIndex][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_INDEX] = new Integer(i);
					arrNewItem[i - _nRowIndex][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_PROPERTYID] = new Integer(EXPLIST_SORTEDARRAY_PROPID_INDEX);
					arrNewItem[i - _nRowIndex][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_NEWVALUE] = new Integer(i - 1);
					
					i++;
				}
				
				m_saData.SetItems(arrNewItem);
				
				// JF- The object array is sorted automatically when a value is set.
			}
			catch (Exception e)
			{
				
					log.Error("ExpListModel::MoveLast An exception occured" + " when trying to move last the row: " + _nRowIndex + ". " + e);
				
			}
		}
	}

	/**
	 * Moves the row identified by the index in argument, one row up. The idea
	 * is that we are going to change the value of the index column.  We just
	 * need to swap that value with the one of the previous row.
	 *
	 * @param _nRowIndex Index of the row to move.
	 */
	protected void MoveUp(int _nRowIndex)
	{
		// Make sure the row is not already in first position.
		if (0 != _nRowIndex)
		{
			try
			{
				// JF- We get back the index values of the row to move and the one just before.
				Object oCurrentRow = m_saData.GetItem(_nRowIndex, EXPLIST_SORTEDARRAY_PROPID_INDEX);
				Object oPreviousRow = m_saData.GetItem(_nRowIndex - 1, EXPLIST_SORTEDARRAY_PROPID_INDEX);

				Object[][] arrNewItem = new Object[2][3];
				arrNewItem[0][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_INDEX] = new Integer(_nRowIndex);
				arrNewItem[0][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_PROPERTYID] = new Integer(EXPLIST_SORTEDARRAY_PROPID_INDEX);
				arrNewItem[0][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_NEWVALUE] = oPreviousRow;
				
				arrNewItem[1][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_INDEX] = new Integer(_nRowIndex - 1);
				arrNewItem[1][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_PROPERTYID] = new Integer(EXPLIST_SORTEDARRAY_PROPID_INDEX);
				arrNewItem[1][PTSortedArrayWrapper.PTSAW_NEWITEMSINFO_NEWVALUE] = oCurrentRow;

				// JF- Then we swap the values.
				m_saData.SetItems(arrNewItem);

				// JF- The object array is sorted automatically when a value is set.
			}
			catch (Exception e)
			{
				
					log.Error("ExpListModel::MoveUp An exception occured" + " when trying to move up the row: " + _nRowIndex + ". " + e);
				
			}
		}
	}

	/**
	 * Will be called each ime there is a repost to save  the data on the list.
	 * Should contain the code to update the data on the  row _nRowIndex.Use
	 * the SetItem method to update _saData and  use _htFormData to retrieve
	 * the posted data.
	 *
	 * @param _saData
	 * @param _nRowIndex
	 * @param _htFormData
	 *
	 * @return int
	 */
	abstract protected int SaveRow(PTGrowableSortedArrayWrapper _saData, int _nRowIndex, XPHashtable _htFormData);
	
	// ************** Pagination methods *************
	
	/**
	 * Max number of items per page.
	 */
	public static final int MAX_ROWS = 1000;
	
	/** Storage key of the current class (used for pagination). */
	private String m_sModelStorageName = null;
	
	/** Pagination Current Lower Bound */	
	private int m_nPaginationLowerBound = -1;
	
	/** Pagination Current Upper Bound */
	private int m_nPaginationUpperBound = -1;
	
	/**
	 * @see com.plumtree.uiinfrastructure.form.pagination.IPaginationModel#SetPagination()
	 */
	public void SetPagination(int nPaginationID, int nPaginationLowerBound,
			int nPaginationUpperBound)
	{
		if (m_nListKey == nPaginationID)
		{
			m_nPaginationLowerBound = nPaginationLowerBound;
			m_nPaginationUpperBound = nPaginationUpperBound;
		}
		else
		{
			
				log.Warn("Trying to set Pagination settings with an invalid Pagination ID: " + nPaginationID 
					+ ". The expected pagination ID was: " + m_nListKey);
			
		}
	}
		
	/**
	 * @see com.plumtree.uiinfrastructure.form.pagination.IPaginationModelRO#GetCurrentLowerObjectCount()
	 */				
	public int GetCurrentLowerObjectCount()
	{
		int nCount = m_nPaginationLowerBound;
		
		if (m_nPaginationLowerBound > m_saData.GetCount())
		{
			nCount = m_saData.GetCount() - GetPaginationSize() + 1;
		}
				
		if (m_nPaginationLowerBound < 1)
		{
			nCount = 1;
		}
		
		return nCount;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.form.pagination.IPaginationModelRO#GetCurrentUpperObjectCount()
	 */		
	public int GetCurrentUpperObjectCount()
	{
		int nCount = m_saData.GetCount();
		
		if (IsPaginationEnabled())
		{
			if (nCount > GetPaginationSize())
			{
				nCount = GetPaginationSize();
			}
			
			if (-1 != m_nPaginationUpperBound)
			{
				if (m_nPaginationUpperBound <= m_saData.GetCount())
				{
					nCount = m_nPaginationUpperBound;
				}
				else
				{
					nCount = m_saData.GetCount();
				}
			}
		}
		
		return nCount;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.form.pagination.IPaginationModelRO#GetNumObjects()
	 */
	public int GetNumObjects()
	{
		return m_saData.GetCount();
	}

	/**
	 * @see com.plumtree.uiinfrastructure.form.pagination.IPaginationModelRO#GetPaginationSize()
	 */	
	public int GetPaginationSize()
	{
		return MAX_ROWS;
	}
	
	/**
	 * Returns the storage key of this Model.
	 * 
	 * @return String
	 */
	public String GetModelStorageName()
	{
		return m_sModelStorageName;
	}
	
	/**
	 * Enables the pagination.
	 * 
	 * @param _sModelStorageName Storage name of the Expandable List Model.
	 */
	public void EnablePagination(String _sModelStorageName)
	{
		m_sModelStorageName = _sModelStorageName;
	}
	
	/**
	 * Returns true if the pagination is enabled for this expandable list.
	 * 
	 * @return boolean
	 */
	public boolean IsPaginationEnabled()
	{
		if (null == m_sModelStorageName)
		{
			return false;
		}		
		return true;
	}
	
	/**
	 * 
	 *
	 * @return 
	 */
	public XPSortedDictionary GetUnusedProperties()
	{
		throw new XPUnsupportedOperationException();
	}
}

