package com.plumtree.uiinfrastructure.expandablelist.growablelist;

import com.plumtree.uiinfrastructure.arraywrapper.*;
import com.plumtree.uiinfrastructure.expandablelist.*;
import com.plumtree.uiinfrastructure.form.*;

// Framework Imports
import com.plumtree.openfoundation.util.*;
import com.plumtree.xpshared.htmlconstructs.*;

/**
 * This is the implementation of the Expandable List
 * framework for the Growable Array.
 * The Growable Array is a list of input texts.
 * On add, a new input text populated with the default value
 * will be added at the bottom of the list.
 *
 * @author Julie Fournier
 */
public class GrowableListModel extends ExpListModel
	implements IGrowableListModelRO
{
	/**
	 * Class name for the activity space */
	public static final String STR_MVC_CLASS_NAME = "GrowableList";

	/**
	 * Number total of columns */
	public static final int EXPLIST_COL_MAX = 2;

	/**
	 * Second column: the value of the input text. */
	public static final int EXPLIST_SORTEDARRAY_PROPID_INPUTTEXT = 1;

	/**
	 * Client validation type: no validation */
	public static final int CLIENT_VALIDATION_TYPE_NONE = 1;

	/**
	 * Client validation type: no String can be empty */
	public static final int CLIENT_VALIDATION_TYPE_NON_EMPTY_STRING = 2;

	/**
	 * Client validation type: Strings can't have spaces */
	public static final int CLIENT_VALIDATION_TYPE_STRING_NO_SPACE = 3;

	/**
	 * Client validation type: each String must be a valid URL */
	public static final int CLIENT_VALIDATION_TYPE_VALID_URL = 4;

	/**
	 * Icon to display on each row, just before the input text. */
	protected PTImageType m_imgRowIcon;

	/**
	 * Message to display in the JavaScript validation.
	 */
	protected String m_sAlertMessage;

	/**
	 * This is used on add to populate the input
	 * text of the newly added row.
	 */
	protected String m_sDefaultValue = "";

	/**
	 * Type of Client validation for the String in the input text of each row.
	 */
	protected int m_nClientValidationType = CLIENT_VALIDATION_TYPE_NONE;

	/**
	 * Adds one row at the bottom of the list and
	 * sets the following values:
	 * First column: m_nNextIndex
	 * Second column: m_sDefaultValue
	 *
	 * @param _htFormData Form data
	 */
	public void AddRowsOnRepost(XPHashtable _htFormData)
	{
		boolean bCanAddNewRow = true;

		// JF- Before we add a new row at the end of the list, we check 
		// if you don't already have a "new row" in the list.
		// The user doesn't need to have more than one empty row (or row with the default value).
		// JF- Check the array see if we have the default value in it.
		for (int i = 0; i < m_saData.GetCount(); i++)
		{
			if (m_sDefaultValue.equals(m_saData.GetItem(i,
							EXPLIST_SORTEDARRAY_PROPID_INPUTTEXT)))
			{
				// JF- We already have a new row no need to add one more.
				bCanAddNewRow = false;

				break;
			}
		}

		if (bCanAddNewRow)
		{
			AddRowsToList(new String[] { m_sDefaultValue });
		}
	}

	/**
	 * Adds an array of new rows to the list.
	 *
	 * @param _aData
	 */
	public void AddRowsToList(String[] _aData)
	{
		Object[][] oRow;

		// fix for #51045: build an array of the data to add all the data at once 
		oRow	= new Object[_aData.length][EXPLIST_COL_MAX];
		for (int i = 0; i < _aData.length; i++)
		{
			oRow[i][EXPLIST_SORTEDARRAY_PROPID_INDEX] = new Integer(m_nNextIndex);
			m_nNextIndex++;
			oRow[i][EXPLIST_SORTEDARRAY_PROPID_INPUTTEXT] = _aData[i];
			
		}
		// add the 2d array
		super.GetItems().AddRows(oRow);
		
		
		
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new GrowableListModel();
	}

	/**
	 * Returns the alert message to display in the JavaScript.
	 *
	 * @return String
	 */
	public String GetAlertMessage()
	{
		return m_sAlertMessage;
	}

	/**
	 * Returns the client validation type.
	 *
	 * @return int
	 */
	public int GetClientValidationType()
	{
		return m_nClientValidationType;
	}

	/**
	 * This method should ge used by the client to
	 * get back the array of values at the end of
	 * the Editor and store them.
	 *
	 * @return String[]
	 */
	public String[] GetListValues()
	{
		String[] aListValues = new String[m_saData.GetCount()];

		for (int i = 0; i < m_saData.GetCount(); i++)
		{
			aListValues[i] = (String) m_saData.GetItem(i,
					EXPLIST_SORTEDARRAY_PROPID_INPUTTEXT);
		}

		return aListValues;
	}

	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Returns the icon to display on each row before the
	 * input text.
	 * If the value is null, no icon is displayed.
	 *
	 * @return PTImageType
	 */
	public PTImageType GetRowIcon()
	{
		return m_imgRowIcon;
	}

	/**
	 * Initializes the expandable list.
	 *
	 * @param _sSectionTitle Main title.
	 * @param _sSectionSubTitle Sub title
	 * @param _imgAddIcon Icon for the add action.
	 * @param _sAddLabel Text to display next to the add icon.
	 * @param _nListKey In case you have multiple Expandable Lists on
	 * the same DP, they should have a different key (otherwise, just set it to 1).
	 */
	public void InitializeGrowableList(String _sSectionTitle,
		String _sSectionSubTitle, PTImageType _imgAddIcon, String _sAddLabel,
		int _nListKey)
	{
		Object[][] arrExpListColInfo = new Object[1][5];
		arrExpListColInfo[0][EXPLIST_COLUMN_ID]			    = new Integer(EXPLIST_SORTEDARRAY_PROPID_INPUTTEXT);
		arrExpListColInfo[0][EXPLIST_COLUMN_TITLE]		    = _sSectionSubTitle;
		arrExpListColInfo[0][EXPLIST_COLUMN_WIDTH]		    = "100%";
		arrExpListColInfo[0][EXPLIST_COLUMN_COLSPAN]	    = "1";
		arrExpListColInfo[0][EXPLIST_COLUMN_ENABLESORT]     = Boolean.FALSE;

		super.InitializeExpandableList(_sSectionTitle, arrExpListColInfo,
			_imgAddIcon, _sAddLabel, _nListKey);
	}

	/**
	 * Initalizes the list (PTGrowableSortedArrayWrapper)
	 * with a column mapping and an array of sort.
	 * The growable array will always be sorted by the 1st
	 * column ascending.
	 *
	 * @param _saData
	 */
	public void InitializeSortedArray(PTGrowableSortedArrayWrapper _saData)
	{
		// JF- Column mapping
		int[] arrColumns = new int[]
			{
				EXPLIST_SORTEDARRAY_PROPID_INDEX,
				EXPLIST_SORTEDARRAY_PROPID_INPUTTEXT
			};

		// JF- Sort array
		// The growable array will always be sorted by the 1st column.
		int[][] arrSort = new int[1][2];
		arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_PROPID]		  = EXPLIST_SORTEDARRAY_PROPID_INDEX;
		arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION]     = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;

		// JF- We don't initalize the list itself (null below)
		// At first it's empty and we will be initalized if needed using the 
		// AddRowsToList method.
		_saData.Initialize(arrColumns, null, arrSort, true);
	}

	/**
	 * Sets the type of JavaScript validation for the rows.
	 * The type should be a CLIENT_VALIDATION_TYPE_ constant.
	 *
	 * @param _nClientValidationType
	 * @param _sAlertMessage
	 *
	 * @return String This is the JavaScript function name that should be added to
	 * the Editor validation functions.
	 */
	public String SetClientValidationType(int _nClientValidationType,
		String _sAlertMessage)
	{
		m_nClientValidationType     = _nClientValidationType;
		m_sAlertMessage			    = _sAlertMessage;

		return GetUniqueString("ValidateRows");
	}

	/**
	 * Sets a new default value.
	 * By default this is an empty string.
	 * This value is used on add to populate the input
	 * text of the newly added row.
	 *
	 * @param _sDefaultValue New value
	 */
	public void SetDefaultValue(String _sDefaultValue)
	{
		m_sDefaultValue = _sDefaultValue;
	}

	/**
	 * Sets the icon to display on each row before the
	 * input text.
	 * By default, no icon is displayed.
	 *
	 * @param _imgRowIcon
	 */
	public void SetRowIcon(PTImageType _imgRowIcon)
	{
		m_imgRowIcon = _imgRowIcon;
	}

	/**
	 * Saves the data on the row _nRowIndex.
	 *
	 * @param _saData List to update the data on row _nRowIndex
	 * @param _nRowIndex Index of the row to save.
	 * @param _htFormData Data posted by the form.
	 * @return int Status for server side validation.
	 */
	protected int SaveRow(PTGrowableSortedArrayWrapper _saData, int _nRowIndex,
		XPHashtable _htFormData)
	{
		String sDataKey     = GetUniqueString(GrowableListView.HTMLINPUT_ITEM) +
			_nRowIndex;
		String[] aDataValue;

		// JF- For each input text posted by the form, we get the value back 
		// and set the new value in the array.
		if (_htFormData.ContainsKey(sDataKey))
		{
			aDataValue = (String[]) _htFormData.GetElement(sDataKey);

			if ((null != aDataValue) && !(0 == aDataValue.length))
			{
				_saData.SetItem(_nRowIndex,
					EXPLIST_SORTEDARRAY_PROPID_INPUTTEXT, aDataValue[0]);
			}
		}

		return RepostControl.PAGE_STATUS_VALID;
	}
	
	
}

