package com.plumtree.uiinfrastructure.expandablelist.multicolumnlist;

import com.plumtree.uiinfrastructure.arraywrapper.*;
import com.plumtree.uiinfrastructure.expandablelist.*;
import com.plumtree.uiinfrastructure.form.*;

// Framework Imports
import com.plumtree.openfoundation.util.*;
import com.plumtree.xpshared.htmlconstructs.*;

/**
 * This is the implementation of the Expandable List framework for the Growable
 * Array. The Growable Array is a list of input texts. On add, a new input
 * text populated with the default value will be added at the bottom of the
 * list.
 *
 * @author JohnH
 */
public class MultiColumnListModel extends ExpListModel
	implements IMultiColumnListModelRO
{
	/** Class name for the activity space */
	public static final String STR_MVC_CLASS_NAME = "MultiColumnList";

	/** Number total of columns */
	public int m_intNumColumns = 2;

	/** Icon to display on each row, just before the input text. */
	protected PTImageType m_imgRowIcon;

	/**
	 * This is used on add to populate the input text of the newly added row.
	 * Note, it MUST be initialized explicitly by the client.
	 */
	protected String[] m_arrDefaultValue;

	/**
	 * The names of the form items for each column.
	 */
	protected String[] m_arrFormDataKeys;

	/**
	 * Adds one row at the bottom of the list and sets the following values:
	 * First column: m_nNextIndex Second column: m_sDefaultValue
	 *
	 * @param _htFormData Form data
	 */
	public void AddRowsOnRepost(XPHashtable _htFormData)
	{
		String[][] arrDefault = new String[m_intNumColumns][1];

		for (int i = 0; i < m_intNumColumns; i++)
		{
			arrDefault[i][0] = m_arrDefaultValue[i];
		}

		AddRowsToList(arrDefault);
	}

	/**
	 * Adds an array of new rows to the list.
	 *
	 * @param _aData
	 */
	public void AddRowsToList(Object[][] _aData)
	{
		// rob: the 2d data array passed in and the 2d array we passing in to the addrows method
		// is inversed: oRow[row][col], _aData[col][row]
		Object[][] oRow	= new Object[_aData[0].length][m_intNumColumns + 1];

		for (int rows = 0; rows < _aData[0].length; rows++)
		{
			oRow[rows][EXPLIST_SORTEDARRAY_PROPID_INDEX]     = new Integer(m_nNextIndex);
			m_nNextIndex++;

			for (int cols = 0; cols < m_intNumColumns; cols++)
			{
				oRow[rows][cols + 1] = _aData[cols][rows];
			}
		}
		super.GetItems().AddRows(oRow);
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new MultiColumnListModel();
	}

	/**
	 * This method should ge used by the client to get back the array of values
	 * at the end of the Editor and store them.
	 *
	 * @return Object[][]
	 */
	public Object[][] GetListValues()
	{
		Object[][] aListValues = new Object[m_intNumColumns][m_saData.GetCount()];

		for (int i = 0; i < m_saData.GetCount(); i++)
		{
			for (int j = 1; j <= m_intNumColumns; j++)
			{
				aListValues[j - 1][i] = (String) m_saData.GetItem(i, j);
			}
		}

		return aListValues;
	}

	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Returns the numColumns.
	 *
	 * @return int
	 */
	public int GetNumColumns()
	{
		return m_intNumColumns;
	}

	/**
	 * Returns the icon to display on each row before the input text. If the
	 * value is null, no icon is displayed.
	 *
	 * @return PTImageType
	 */
	public PTImageType GetRowIcon()
	{
		return m_imgRowIcon;
	}

	/**
	 * Initializes the expandable list.
	 *
	 * @param _sSectionTitle Main title.
	 * @param _sSectionSubTitle Sub title
	 * @param _imgAddIcon Icon for the add action.
	 * @param _sAddLabel Text to display next to the add icon.
	 * @param _nListKey In case you have multiple Expandable Lists on the same
	 *        DP, they should have a different key (otherwise, just set it to
	 *        1).
	 */
	public void InitializeMultiColumnList(String _sSectionTitle,
		String[] _sSectionSubTitle, PTImageType _imgAddIcon, String _sAddLabel,
		int _nListKey)
	{
		//Set the width based on the number of columns
		int intColWidth    = 100 / m_intNumColumns;
		XPStringBuilder sb = new XPStringBuilder(5);
		sb.Append(intColWidth).Append("%");

		Object[][] arrExpListColInfo = new Object[m_intNumColumns][5];

		for (int i = 0; i < m_intNumColumns; i++)
		{
			arrExpListColInfo[i][EXPLIST_COLUMN_ID]			    = new Integer(i +
					1);
			arrExpListColInfo[i][EXPLIST_COLUMN_TITLE]		    = _sSectionSubTitle[i];
			arrExpListColInfo[i][EXPLIST_COLUMN_WIDTH]		    = sb.ToString();
			arrExpListColInfo[i][EXPLIST_COLUMN_COLSPAN]	    = "1";
			arrExpListColInfo[i][EXPLIST_COLUMN_ENABLESORT]     = Boolean.FALSE;
		}

		super.InitializeExpandableList(_sSectionTitle, arrExpListColInfo,
			_imgAddIcon, _sAddLabel, _nListKey);
	}

	/**
	 * Initalizes the list (PTGrowableSortedArrayWrapper) with a column mapping
	 * and an array of sort. The growable array will always be sorted by the
	 * 1st column ascending.
	 *
	 * @param _saData
	 */
	public void InitializeSortedArray(PTGrowableSortedArrayWrapper _saData)
	{
		// JF- Column mapping
		int[] arrColumns = new int[m_intNumColumns + 1];
		arrColumns[0] = EXPLIST_SORTEDARRAY_PROPID_INDEX;

		for (int i = 1; i <= m_intNumColumns; i++)
		{
			arrColumns[i] = i;
		}

		// JF- Sort array
		// The growable array will always be sorted by the 1st column.
		int[][] arrSort = new int[1][2];
		arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_PROPID]		  = EXPLIST_SORTEDARRAY_PROPID_INDEX;
		arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION]     = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;

		// JF- We don't initalize the list itself (null below)
		// At first it's empty and we will be initalized if needed using the
		// AddRowsToList method.
		_saData.Initialize(arrColumns, null, arrSort, true);
	}

	/**
	 * Sets a new default value. By default this is an array of empty strings.
	 * This value is used on add to populate the input text of the newly added
	 * row.
	 *
	 * @param _sDefaultValue New value
	 */
	public void SetDefaultValue(String[] _sDefaultValue)
	{
		m_arrDefaultValue = _sDefaultValue;
	}

	/**
	 * Sets the base name for the form data items in each column.
	 *
	 * @param _sDefaultValue New value
	 */
	public void SetFormDataKeys(String[] _arrFormDataKeys)
	{
		m_arrFormDataKeys = new String[_arrFormDataKeys.length];
		for (int i = 0; i < _arrFormDataKeys.length; i++)
		{
			m_arrFormDataKeys[i] = GetUniqueString(_arrFormDataKeys[i]);
		}
	}

	/**
	 * Sets the numColumns.
	 *
	 * @param numColumns The numColumns to set
	 */
	public void SetNumColumns(int numColumns)
	{
		m_intNumColumns = numColumns;
	}

	/**
	 * Sets the icon to display on each row before the input text. By default,
	 * no icon is displayed.
	 *
	 * @param _imgRowIcon
	 */
	public void SetRowIcon(PTImageType _imgRowIcon)
	{
		m_imgRowIcon = _imgRowIcon;
	}

	/**
	 * Saves the data on the row _nRowIndex.
	 *
	 * @param _saData List to update the data on row _nRowIndex
	 * @param _nRowIndex Index of the row to save.
	 * @param _htFormData Data posted by the form.
	 *
	 * @return int Status for server side validation.
	 */
	protected int SaveRow(PTGrowableSortedArrayWrapper _saData, int _nRowIndex,
		XPHashtable _htFormData)
	{
		XPStringBuilder sb = new XPStringBuilder(100);
		sb.Append(MultiColumnListView.HTMLINPUT_ITEM);

		for (int i = 1; i <= m_intNumColumns; i++)
		{
			String sDataKey;
			if (m_arrFormDataKeys != null)
			{
				// use the client-specified key names -jas
				sDataKey = m_arrFormDataKeys[i - 1];
			}
			else
			{
				// generate the default name -jas
				sDataKey = GetUniqueString(sb.Append(i).ToString());
			}
			sDataKey += _nRowIndex;
			String[] aDataValue;

			// JF- For each input text posted by the form, we get the value back
			// and set the new value in the array.
			if (_htFormData.ContainsKey(sDataKey))
			{
				aDataValue = (String[]) _htFormData.GetElement(sDataKey);

				if ((null != aDataValue) && !(0 == aDataValue.length))
				{
					_saData.SetItem(_nRowIndex, i, aDataValue[0]);
				}
			}
			else
			{
				_saData.SetItem(_nRowIndex, i, "");
			}
		}

		return RepostControl.PAGE_STATUS_VALID;
	}
	
	
}

