package com.plumtree.uiinfrastructure.form;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.activityspace.utils.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.form.status.*;
import com.plumtree.uiinfrastructure.interpreter.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

//Put explicit import for Interpreter (to avoid ambiguity of using the deprecated class)
import com.plumtree.uiinfrastructure.interpreter.Interpreter;

/**
 * All AActivitySpaces that contain IDisplayPages that extend AFormDP must
 * also implement IRepostAS.  The exception to this rule is that PlumtreeDP
 * extends AFormDP, but by default does not include the form code.  Therefore,
 * when you are using PlumtreeDP, you only need to implement IRepostAS if you
 * have called SetAddMainForm(true);
 * 
 * @author Don Hayler
 */
abstract public class AFormDP implements IDisplayPage
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.form.AFormDP");

	/** The name of the main form (for use in Javascript, etc...) */
	public static final String HTMLFORM_NAME = "mainform";

	/** The name of the main form javascript submission function */
	public static final String JAVASCRIPT_SUBMIT_FORM = "postToSelf";

	/** Name of helper javascript function to set up for
	 *  postToPassthru function */
	public static final String JAVASCRIPT_SETUP_FOR_SUBMIT_TO_CONTROL = "setupForPassthru";

	/** Name of helper javascript function to post the form, then
	 *  pass through to another control */
	public static final String JAVASCRIPT_SUBMIT_TO_CONTROL = "postToPassthru";

	/** multi-part form data encoding constant */
	public static final String MULTIPART_ENCODING = "multipart/form-data";

	/** Default Javascript functions */
	private static final String JAVASCRIPT_VALIDATE		    = "validatePage()";
	private static final String DEFAULT_VALIDATION_FUNCTION = "function " +
		JAVASCRIPT_VALIDATE + " { return true; }\n";

	/** Form submission types */
	public static final String POST = "post";
	public static final String GET = "get";

	/**
	 * The parent activity space.
	 * 
	 * Warning: EditorDP overrides this to provide an editor specific version,
	 * so GetOwner() must be used in this class file.
	 */
	protected AActivitySpace m_asOwner;

	/** Validation functions to call in Javascript before submitting the form */
	private XPArrayList m_xpValidationFunctions;

	/** Whether or not to turn on multipart form encoding */
	private boolean m_bMultiPart         = false;

	/** Which parts of the form to display */
	private boolean m_bShowFormHeader    = false;
	private boolean m_bShow508FormFooter = false;
	private boolean m_bShowFormFooter    = false;
	private boolean m_bShowOKButton	   	 = false;
	
	/** Arguments for the status view. */
	private boolean m_bStatusViewEnabled 	= true;
	private int[] m_aStatusViewRepostValues	= null;
	
	public AFormDP()
	{
		m_xpValidationFunctions = new XPArrayList();
		
		// default status view to show up only on FINISH.
		m_aStatusViewRepostValues = new int[] { RepostControl.POSTTOSELF_ACTION_FINISH };
	}

	/**
	 * This is a helper method that adds a function to the client side page validation.
	 * If this is not called, then a blank default validation function will be used.
	 *
	 * AddValidationFunction() should be called when the DisplayPage is first being
	 * constructed in the Activity Space.  It should not be called each time the
	 * page is being displayed.
	 *
	 * The function name should be just the name of the function without parentheses
	 * or semicolons.  I.E.  myValidateFunction, which will be called by AFormDP as
	 * myValidateFunction().  These functions should return true if the page is valid,
	 * or display an error message and return false if the page is invalid.
	 *
	 * @param String the name of the Javascript function to call for validation
	 */
	public final void AddToValidationFunction(String functionName)
	{
		if (functionName == null)
		{
			
				log.Error("Tried to add null Validation Function.");
			

			return;
		}

		if ("".equals(functionName))
		{
			
				log.Error("Tried to add blank Validation Function name.");
			

			return;
		}

		if ((functionName.indexOf("()") != -1) ||
				(functionName.indexOf(';') != -1))
		{
			
				log.Error("Cannot add Validation Function name with () or ; in it: " +
					functionName);
			

			return;
		}

		if (m_xpValidationFunctions.Contains(functionName))
		{
			
				log.Error("Cannot add Validation Function more than once: " +
					functionName);
			

			return;
		}

		m_xpValidationFunctions.Add(functionName);
	}

	/**
	 * This is a helper method that removes a function from the client side page
	 * validation.
	 *
	 * The function name should be just the name of the function without parentheses
	 * or semicolons.  I.E.  myValidateFunction, which will be called by AFormDP as
	 * myValidateFunction().
	 *
	 * @param String the name of the Javascript function to remove from the list
	 * of functions to call for validation
	 */
	public final void RemoveFromValidationFunction(String functionName)
	{
		m_xpValidationFunctions.Remove(functionName);
	}

	/**
	 * This method sets whether or not the form should use multipart encoding.
	 *
	 * @param boolean true implies multipart encoding in the form.
	 */
	public final void SetMultiPartEncoding(boolean bMultiPart)
	{
		m_bMultiPart = bMultiPart;
	}

	/**
	 * This method sets whether or not the form should display the done and cancel
	 * buttons at the bottom of the page in 508 mode.  This variable is ignored
	 * in normal mode.  It is only valid in 508 and low bandwidth modes.
	 *
	 * This is for developers who provide their own buttons in standard mode, but
	 * want to use the AFormDP buttons in 508 mode (such as the editor).
	 * 
	 * @param boolean true implies buttons in footer, no header is displayed (in 508 mode).
	 */
	public final void SetShow508FormFooter(boolean bShow508Footer)
	{
		m_bShow508FormFooter = bShow508Footer;
	}

	/**
	 * This method helps to set the attributes of the page correctly.
	 * 
	 * This should only be called at page init time.
	 * 
	 * @param FormPageTypes the type of the current page.
	 */
	private void InitializePageType(FormPageTypes _pageType)
	{
		if (FormPageTypes.NORMAL_PAGE.equals(_pageType))
		{
			m_bShowFormHeader     = true;
			m_bShowFormFooter     = false;
		} else if (FormPageTypes.MEDIUM_POPUP.equals(_pageType))
		{
			m_bShowFormHeader     = true;
			m_bShowFormFooter     = false;
			m_bShowOKButton		  = true;

		} else if (FormPageTypes.SMALL_POPUP.equals(_pageType))
		{
			m_bShowFormHeader     = false;
			m_bShowFormFooter     = true;
			m_bShowOKButton		  = true;
		} else if (FormPageTypes.NO_BUTTONS.equals(_pageType))
		{
			m_bShowFormHeader	  = false;
			m_bShowFormFooter	  = false;
		}
	}

	///////////////
	// Form Options
	///////////////

	/**
	 * This method returns the current page type.  Override this
	 * method to display a different page type (small pop-up, etc...)
	 * 
	 * This gets called at Init() time.
	 * 
	 * Default has no buttons on the form.
	 * 
	 * @return FormPageTypes the type of the page to display
	 */
	public FormPageTypes GetFormPageType()
	{
		// default implementation
		return FormPageTypes.NO_BUTTONS;
	}

	/**
	 * This is a method that classes extending this one can over write.
	 * The HTML returned by that method will get printed out before
	 * the FORM tag and AFTER the BODY tag.
	 * E.g.: DIV tags should go in that method.
	 *
	 * Override this method to display HTML after the body and before the form.
	 *
	 * @return HTMLElement
	 */
	public HTMLElementCollection DisplayHTMLBeforeForm()
	{
		return null;
	}
	
	/**
	 * This is a method that classes extending this one can over write.
	 * The HTML returned by that method will get printed out after the FORM tag
	 *  E.  g.: If you have other forms, like with portlets.
	 *
	 * Override this method to display HTML after the form.
	 *
	 * @return HTMLElement
	 */
	public HTMLElementCollection DisplayHTMLAfterForm()
	{
		return null;
	}

	/**
	 * This method returns the repost values that we will turn
	 * the status view on for.  The status view replaces the body
	 * of the existing view.  Be default, AFormDP is initialized 
	 * to show the status view only after a FINISH repost.
	 * 
	 * @return Repost values for which you want the status view on.
	 */
	public int[] GetStatusViewRepostValues()
	{
		return m_aStatusViewRepostValues;
	}

	/**
	 * This method sets the repost values that we will turn the 
	 * status view on for.  The status view replaces the body of
	 * the existing view.  Be default, AFormDP is initialized to 
	 * show the status view only after a FINISH repost.
	 * 
	 * Set this to NULL to allow all values to display the view.
	 * 
	 * @param aRepostValues Values for which you want the status view on.
	 */
	public void SetStatusViewRepostValues(int[] aRepostValues)
	{
		m_aStatusViewRepostValues = aRepostValues;
	}

	/**
	 * This method returns whether the status view is enabled.
	 * 
	 * @return True if the status view is enabled, false otherwise.
	 */
	public boolean GetStatusViewEnabled()
	{
		return m_bStatusViewEnabled;
	}

	/**
	 * This method sets whether the status view is enabled.
	 * 
	 * @param bEnabled True to enable the status view, false otherwise.
	 */
	public void SetStatusViewEnabled(boolean bEnabled)
	{
		m_bStatusViewEnabled = bEnabled;
	}

	/////////////////////////////////
	// Methods to optionally override
	/////////////////////////////////

	/**
	 * This is a method that classes extending this one can over write.
	 * The HTML returned by that method will get printed out in the head.
	 * Warning: Only TITLE LINK and META can be added to the head with that
	 * method (use DisplayJavaScriptFromChild for SCRIPT)
	 *
	 * Override this method to display HTML in the head.
	 *
	 * @return HTMLElement
	 */
	public HTMLElementCollection DisplayHTMLinHEAD()
	{
		return null;
	}

	/**
	 * This is a method that classes extending this one can overwrite
	 * to set an onload handler for the page.
	 */
	public String GetOnLoad()
	{
		return null;
	}

	/**
	 * This is a helper method that determines whether or not cancels should close
	 * the window immediately.  This is useful in the case of pop up windows, where
	 * you don't need to know anything unless the user presses finish.  In this case,
	 * cancel should just call window.close();
	 *
	 * Override this method to immediately close the pop ups.
	 *
	 * @return boolean true implies close the window immediately on cancel
	 */
	public boolean GetCloseOnCancel()
	{
		return false;
	}

	////////////////////
	// Abstract methods.
	////////////////////

	/**
	 * This is a helper method that all classes that inherit from AFormDP need to
	 * implement.  The method should return the title that should be displayed in
	 * the window TITLE element as well as on the form header bar, if displayed.
	 * 
	 * The String returned by this method needs to already be HTML Encoded for
	 * safe display.
	 *
	 * @return String the HTML Encoded title for the form header
	 */
	abstract public String GetFormTitle();

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	abstract public String GetName();

	/**
	 * This is a helper method that all classes that inherit from AFormDP need
	 * to implement.  The method should add whatever HTML the child class wants to
	 * display to the body argument.  The parent class will then add that HTML to
	 * the HTMLPage after filling in the banner and navigation.
	 *
	 * @return HTMLElement the HTML to put inside the form
	 */
	abstract public HTMLElement PageDisplay();

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IDisplayPage#Display()
	 *
	 * Warning: PlumtreeDP overrides this, so if you change things here, you probably
	 * need to change it there as well.
	 */
	public void Display(IWebData pageData)
	{
	    if (m_asOwner.GetIsInPlaceRefreshDisplayOn())
	    {
	        try
	        {
	        	HTMLElement myElement = PartialContentDisplay();
	        	pageData.SetData(myElement);
	        }
	        catch (Exception ex)
	        {
	            log.Error(ex, "Error displaying text: " + ex.getMessage());
	            throw new XPException("Error displaying text: " + ex.getMessage(), ex);
	        }
	        finally 
	        {
	            // reset the in place refresh flag so that future hits
	            // will be displayed normally.  this flag is normally 
	            // set to true via an in place refresh repost.
	            m_asOwner.SetIsInPlaceRefreshDisplayOn(false);
	        }	       
	    }
	    else
	    {	    
		    // Calculate whether or not to display javascript.
			boolean bNoJavascript = !(GetUseJavascript());
	
			HTMLPage myPage = new HTMLPage();
	
			// Set the HTMLPage as the data to be sent to the browser.
			pageData.SetData(myPage);
	
			if (bNoJavascript)
			{
				myPage.SetDo508Checking(true);
			}
	
			HTMLHead myHead = new HTMLHead();
			myPage.SetHead(myHead);
	
			// GetFormTitle should already be encoded.
			HTMLTitle myTitle = new HTMLTitle(GetFormTitle());
	
			HTMLBody myBody = new HTMLBody();
			myPage.SetBody(myBody);
			myBody.SetMarginHeight("0");
			myBody.SetMarginWidth("0");
			myBody.SetLeftMargin("0");
			myBody.SetTopMargin("0");
			if (!bNoJavascript)
			{
				String strPCCfunction = "document.PCC.RaiseWindowEvent(event);";
				myBody.SetOnActivate(strPCCfunction);
				myBody.SetOnAfterPrint(strPCCfunction);
				myBody.SetOnBeforeActivate(strPCCfunction);
				myBody.SetOnBeforeUnload(strPCCfunction);
				myBody.SetOnBlur(strPCCfunction);
				myBody.SetOnControlSet(strPCCfunction);
				myBody.SetOnDeactivate(strPCCfunction);
				myBody.SetOnError(strPCCfunction);
				myBody.SetOnFocus(strPCCfunction);
				myBody.SetOnHelp(strPCCfunction);
				myBody.SetOnLoad(strPCCfunction);
				myBody.SetOnResize(strPCCfunction);
				myBody.SetOnResizeEnd(strPCCfunction);
				myBody.SetOnResizeStart(strPCCfunction);
				myBody.SetOnUnload(strPCCfunction);
			}
			 
			try
			{
				// Set to dark background color if this is a small form.
				if (m_bShowFormFooter)
				{
					myBody.SetStyleClass(PTStyleClass.DIALOGUE_BG);
				}
	
				myHead.AddInnerHTMLElement(myTitle);
	
				myHead.AddInnerHTMLElement(GetOwner().GetStyleSheetLinks());
	
				// Add javascript if we are in standard access style (i.e. not 508)
				if (!bNoJavascript)
				{
					// Add javascript if we are in standard access style (i.e. not 508)
					String strOnLoad = GetOnLoad();
	
					// The Common JS Includes need to come before any functions are registered with
					// the PCC.  This should come before both the PCCHelper and the page and view
					// Javascript (DisplayJavaScriptFromChild, etc...).	
					
					// Includes jsportlet javascript components files on the page, the version of the jsportlet to use is defined in /settings/config/JSComponentRegistry.xml
					HTMLScriptCollection jsportletScript = new HTMLScriptCollection();
					jsportletScript.AddInnerHTMLElement( ConfigHelper.GetJSIncluderJSComponentInclude(ConfigHelper.JSCOMPONENT_JSPORTLET, GetOwner()) );
					myHead.AddInnerHTMLElement(jsportletScript);

					// Display the common javascript first so it can be used in child javascript onLoads.
					myHead.AddInnerHTMLElement(((IFormAS) GetOwner()).DisplayCommonJavaScript());

					myHead.AddInnerHTMLElement(DisplayJavascript());
					myHead.AddInnerHTMLElement(DisplayJavaScriptFromChild());
	
					HTMLScript scriptPCCHelper = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);
					scriptPCCHelper.AddInnerHTMLString
					   ("var windowOnloadFunction = (window.onload) ? window.onload : new Function;\n" +
					   "document.PCC.RegisterForWindowEvent(\"onload\", windowOnloadFunction);\n" +
					   "var windowOnBlurFunction = (window.onblur) ? window.onblur : new Function;\n" +
					   "document.PCC.RegisterForWindowEvent(\"onblur\", windowOnBlurFunction);\n" +
					   "var windowOnFocusFunction = (window.onfocus) ? window.onfocus : new Function;\n" +
					   "document.PCC.RegisterForWindowEvent(\"onfocus\", windowOnFocusFunction);\n");
			
	
					if (null != strOnLoad)
					{
						scriptPCCHelper.AddInnerHTMLString("document.PCC.RegisterForWindowEvent(\"onload\", " + strOnLoad + ");\n");
					}
					myHead.AddInnerHTMLElement(scriptPCCHelper);
				}
	
				// JF- Helper method so that Activity Spaces can add HTML into the head
				// (like meta and link tags for example).           
				HTMLElementCollection myHTMLinHead = DisplayHTMLinHEAD();
	
				if (null != myHTMLinHead)
				{
					myHead.AddInnerHTMLElement(myHTMLinHead);
				}
	
				// JF- Helper method so that Activity Spaces can add HTML after the body 
				// tag and before the form tag (like div tags for example).            
				HTMLElementCollection myHTMLBeforeForm = DisplayHTMLBeforeForm();
	
				if (null != myHTMLBeforeForm)
				{
					myBody.AddInnerHTMLElement(myHTMLBeforeForm);
				}
	
				myBody.AddInnerHTMLElement(DisplayForm());
				
				// PAI - Helper method so that Activity Spaces can add HTML after the form
				// tag and before the form tag (like for pages with other forms).
				HTMLElementCollection myHTMLAfterForm = DisplayHTMLAfterForm();
				
				if (null != myHTMLAfterForm)
				{
					myBody.AddInnerHTMLElement(myHTMLAfterForm);
				}
				
				BrowserType browser = (BrowserType) GetOwner().GetSubSession().GetAttribute(Interpreter.BROWSERTYPE);
			
				if(!bNoJavascript && browser.IsJSPortletCompliant())
				{
					HTMLScript script = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);
					script.AddInnerHTMLString("InitializeEvents();"); //InitializeEvents is defined in "jsportlet" code included above
					myBody.AddInnerHTMLElement(script);
				}
				
			}
			catch (HTMLException he)
			{
				
					log.Error(he, "Display: The following HTML error occured when trying to display the HTML:");
				
	
				throw new XPException(he);
			}
			catch (Exception e)
			{
				
					log.Error(e, "Display: The following error occured when trying to display the HTML:");
				
	
				throw XPException.GetInstance(e);
			}	
	    }
	}
	
	/////////////////
	// Helper Methods
	/////////////////


	/**
	 * This is a helper method that displays the javascript in the page.
	 *
	 * SetValidationFunction() and AddJavascript() should be called before this
	 * method.
	 *
	 * @return HTMLScriptCollection the Javascript for this DisplayPage (child javascript
	 *             excluded)
	 */
	public final HTMLScriptCollection DisplayJavascript()
	{
		HTMLScriptCollection myScriptCollection = new HTMLScriptCollection();
		
		// Number of client side page validation functions
		int nSize = m_xpValidationFunctions.GetSize();

		// Repost function
		// This javascript function reposts the page based upon hidden HTML inputs in
		// the form.
		XPStringBuilder sb = new XPStringBuilder(300 + (nSize * 40));
		sb.Append("\n");
		
		// JF- Set the focus on the current window if it's a pop-up.
		// (This is only useful if the window had been minimized.)
        if(GetWindowFocusEnabled())
        {
		sb.Append("if ((parent) && (parent.opener))\n");
		sb.Append("{\n");		
		sb.Append("  window.focus();\n");
		sb.Append("}\n");
        }

		sb.Append("function ").Append(JAVASCRIPT_SUBMIT_FORM).Append("(varPostToSelf)\n");
		sb.Append("{\n");
		sb.Append("  if (((varPostToSelf != ")
		.Append(RepostControl.POSTTOSELF_ACTION_FINISH)
		.Append("  ) && (varPostToSelf != ")
		.Append(RepostControl.POSTTOSELF_ACTION_GOTOPAGE).Append(")) || (validatePage() == true)) {\n");
		if (GetCloseOnCancel())
		{
			// Automatically close the window on cancel
			sb.Append("    if (varPostToSelf == ").Append(RepostControl.POSTTOSELF_ACTION_CANCEL)
			.Append(") {\n");
			sb.Append("      window.close();\n");
			sb.Append("      return;\n");
			sb.Append("    }\n");
		}
		sb.Append("    document.").Append(HTMLFORM_NAME).Append(".")
		.Append(RepostControl.HTMLINPUT_POSTTOSELF).Append(".value= varPostToSelf;\n");
		// Insert custom javascript processing code before form submit.
		InsertJavascriptBeforeFormSubmit(sb);
		sb.Append("    document.").Append(HTMLFORM_NAME).Append(".submit();\n");
		if (GetStatusViewEnabled())
		{
			// Insert JavaScript code to display Status View on FINISH.
			sb.Append(StatusView.GetJSShowStatusInBodyRestricted("varPostToSelf", GetStatusViewRepostValues(), "    "));
		}
		sb.Append("  }\n");
		sb.Append("}\n");

		sb.Append("\n");
		sb.Append("function ").Append(JAVASCRIPT_SETUP_FOR_SUBMIT_TO_CONTROL)
		.Append("(varPassthru, varArgs)\n");
		sb.Append("{\n");
		sb.Append("  document.").Append(HTMLFORM_NAME).Append(".")
		.Append(RepostControl.HTMLINPUT_PASSTHRU_CONTROL).Append(".value = varPassthru;\n");
		sb.Append("  document.").Append(HTMLFORM_NAME).Append(".")
		.Append(RepostControl.HTMLINPUT_PASSTHRU_ARGS).Append(".value = varArgs;\n");
		sb.Append("  document.").Append(HTMLFORM_NAME).Append(".")
		.Append(RepostControl.HTMLINPUT_POSTTOSELF).Append(".value = ")
		.Append(RepostControl.POSTTOSELF_ACTION_PASSTHRU).Append(";\n");
		sb.Append("}\n");

		sb.Append("\n");
		sb.Append("function ").Append(JAVASCRIPT_SUBMIT_TO_CONTROL).Append("(varPassthru, varArgs)\n");
		sb.Append("{\n");
		sb.Append("  ").Append(JAVASCRIPT_SETUP_FOR_SUBMIT_TO_CONTROL).Append("(varPassthru, varArgs);\n");
		sb.Append("  postToSelf(")
		.Append(RepostControl.POSTTOSELF_ACTION_PASSTHRU).Append(");\n");
		sb.Append("}\n");

		// Client side page validation function
		if (nSize > 0)
		{
			sb.Append("function ").Append(JAVASCRIPT_VALIDATE).Append("\n");
			sb.Append("{\n");
			for (int i = 0; i < nSize; i++)
			{
				sb.Append("  if (");
				String validationMethod = (String) m_xpValidationFunctions.GetElement(i);
				if (XPStringUtility.EndsWith(validationMethod,")"))
				{
					sb.Append((String) m_xpValidationFunctions.GetElement(i)).Append(" == false)\n");
				}
				else
				{
					sb.Append((String) m_xpValidationFunctions.GetElement(i)).Append("() == false)\n");
				}
				sb.Append("    return false;\n");
			}

			sb.Append("  return true;\n");
			sb.Append("}\n");
		}
		else
		{
			sb.Append(DEFAULT_VALIDATION_FUNCTION);
		}

		HTMLScript myScript;
		
		// Post To Self JavaScript.
		myScript = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);
		myScript.AddInnerHTMLString(sb.ToString());
		myScriptCollection.AddInnerHTMLElement(myScript);

		// Status bar Javascript.
		myScriptCollection.AddInnerHTMLElement(StatusView.
			GetDisplayJavascript(m_asOwner.GetIsSecuredSpace()));
		
		return myScriptCollection;
	}

    /**
     * @return if true, DisplayJavascript will inject a window.focus into the AFormDP javascript
     */
    public boolean GetWindowFocusEnabled()
    {
        return true;
    }

	/**
	 * Helper method to insert custom javascript in the form submit
	 * method right before the form is submitted.
	 * 
	 * javascript should have 4 spaces indenting each line to make it
	 * line up with the code it is being inserted into.
	 * 
	 * @param sb StringBuilder to add your custom javascript to.
	 */
	public void InsertJavascriptBeforeFormSubmit(XPStringBuilder sb)
	{
		// Add your custom javascript here.
	}

	/**
	 * This is a helper method that adds the main form to the page.  It also calls
	 * PageDisplay, so AddForm and PageDisplay should not be used in the same page.
	 *
	 * @return HTMLForm the HTML for the main form
	 */
	public HTMLForm DisplayForm()
	{
		// Calculate whether or not to display javascript.
		boolean bNoJavascript = GetOwner().GetAccessStyle() == AccessStyles.SECTION_508;

		HTMLForm myForm;
		HTMLInput myInput;

		// Form and hidden inputs
		myForm = new HTMLForm("");

		myForm.AddInnerHTMLElement(new HTMLComment(
				"Begin: Display Form and hidden inputs"));

		myForm.SetAction(ConfigHelper.GetApplicationBaseURLToAS(m_asOwner.GetCurrentNormalizedBaseURL(), 
			m_asOwner.GetName(), m_asOwner.IsSubSpace()));
		myForm.SetMethod(GetRepostTypeIsPost() ? POST : GET);
		myForm.SetName(HTMLFORM_NAME);

		// Hidden Inputs
		// If we can't use javascript, we need to do form posts differently than we do here
		if (!bNoJavascript)
		{
			// The action for the repost control to perform (repost, finish, cancel, etc...)
			myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
					RepostControl.HTMLINPUT_POSTTOSELF, "");
			myForm.AddInnerHTMLElement(myInput);
			myInput.SetValue(XPConvert.ToString(
					RepostControl.POSTTOSELF_ACTION_UNDEFINED));

			// Name of passthru control to run when PASSTHRU action is selected
			myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
					RepostControl.HTMLINPUT_PASSTHRU_CONTROL, "");
			myForm.AddInnerHTMLElement(myInput);

			// Arguments to pass to passthru control when PASSTHRU action is selected
			myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
					RepostControl.HTMLINPUT_PASSTHRU_ARGS, "");
			myForm.AddInnerHTMLElement(myInput);
		}

		// Name of space to POST this form to.
		myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
				ASConstants.SPACENAME_INPUT, "");
		myForm.AddInnerHTMLElement(myInput);
		myInput.SetValue(GetOwner().GetName());

		// ID of space to POST this form to.
		myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
				ASConstants.SPACEID_INPUT, "");
		myForm.AddInnerHTMLElement(myInput);
		myInput.SetValue(GetOwner().GetSpaceID());

		// Name of control to POST this form to.
		myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
				ASConstants.CONTROL_INPUT, "");
		myForm.AddInnerHTMLElement(myInput);
		myInput.SetValue(GetRepostControlName());

		// Name of display page to display after POST.
		myInput = new HTMLInput(HTMLInputTypes.HIDDEN, ASConstants.PAGE_INPUT,
				"");
		myForm.AddInnerHTMLElement(myInput);
		myInput.SetValue(GetOwner().GetCurrentPage().GetName());

		myForm.AddInnerHTMLElement(new HTMLComment(
				"End: Display Form and hidden inputs"));

		if (GetOwner().GetAccessStyle() != AccessStyles.STANDARD)
		{
			myForm.AddInnerHTMLElement(PageDisplay());

			// If we are in 508 or low bandwidth mode, we can only display buttons at
			// the bottom.  FormFooterView can handle no javascript, but the header
			// view cannot.
			if (m_bShowFormHeader || m_bShowFormFooter || m_bShow508FormFooter)
			{
				myForm.AddInnerHTMLElement(GetOwner()
					.GetView(FormFooterView.STR_MVC_CLASS_NAME).Display());
			}
		}
		else
		{
			if (m_bShowFormHeader)
			{
				if (m_bShowOKButton)
				{
					myForm.AddInnerHTMLElement(GetOwner()
						.GetView(MediumFormHeaderView.STR_MVC_CLASS_NAME).Display());
				} else
				{
					myForm.AddInnerHTMLElement(GetOwner()
						.GetView(FormHeaderView.STR_MVC_CLASS_NAME).Display());
				}
			}

			myForm.AddInnerHTMLElement(PageDisplay());

			if (m_bShowFormFooter)
			{
				myForm.AddInnerHTMLElement(GetOwner()
					.GetView(FormFooterView.STR_MVC_CLASS_NAME).Display());
			}
			
			String strLang = (String) PersonalSettingsHelper.
				GetPersonalSettingValue(PersonalSettingsHelper.LANGUAGE, GetOwner());

			// Add status bar view.
			myForm.AddInnerHTMLElement(StatusView.GetDisplay(strLang));
		}

		// Set the encoding at the end, just in case one of the Views wants to
		// change the encoding (not the recommended way to change the encoding;
		// should be done in ActivitySpace.Init() or somewhere like that).
		if (m_bMultiPart)
		{
			myForm.SetEncType(MULTIPART_ENCODING);
		}

		return myForm;
	}

	/**
	 * Method Get508RepostURL.
	 * @param nRepostAction
	 * @return ASURL
	 */
	public ASURL Get508RepostURL(int nRepostAction)
	{
		ASURL myAsurl = new ASURL(m_asOwner);
		myAsurl.SetLinkGetCachedSpace(GetOwner());
		myAsurl.SetDisplayPage(this.GetName());
		myAsurl.SetControl(GetRepostControlName());
		myAsurl.AddControlArgument(EditorRepostControl.HTMLSUBMIT_508_POSTTOSELF +
			nRepostAction, nRepostAction);

		return myAsurl;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IDisplayPage#GetOwner()
	 */
	public AActivitySpace GetOwner()
	{
		return m_asOwner;
	}

	////////////////
	// Class Methods
	////////////////

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IDisplayPage#Init(AActivitySpace)
	 */
	public void Init(AActivitySpace parent)
	{
		m_asOwner = parent;

		InitializePageType(GetFormPageType());
	}

	///////////////////
	// Override Methods
	///////////////////

	/**
	 * This is a helper method that all classes that inherit from AFormDP can override
	 * if they like.  The method provides a hook into the head so that DP's can add javascript.
	 * Simply override and return and HTMLScript.  Also, it is best to return the results of the
	 * DisplayJavaScript functions from the appropriate views.
	 *
	 * Override this method to Display javascript from child IViews
	 *
	 * @return HTMLScriptCollection the Javascript from the Child IViews
	 */
	protected HTMLScriptCollection DisplayJavaScriptFromChild()
	{
		HTMLScriptCollection scripts = new HTMLScriptCollection();

		/*
		* SAMPLE
		*
		* scripts.AddInnerHTMLElement(GetOwner().GetView(ObjectView.STR_MVC_CLASS_NAME).DisplayJavascript());
		*/
		return scripts;
	}

	/**
	 * This method is provided so that the repost control name can be overridden
	 * on a per DP basis.
	 *
	 * @return String the name of the repost control to use
	 */
	protected String GetRepostControlName()
	{
		return ((IFormAS) GetOwner()).GetRepostControlName();
	}

	/**
	 * Override this method to set the form submission type.
	 *
	 * @return boolean true implies POST, false implies GET
	 */
	protected boolean GetRepostTypeIsPost()
	{
		return true;
	}
	
	/*
	 * 
	 * This method determines whether to use Javascript.  On most pages,
	 * only the STANDARD portal uses JS.  However, on portal, community, and
	 * the default profile pages, the low-bandwidth portal can display JS.  In these
	 * cases, this method is overridden to allow JS for Standard and LB (which is the
	 * same as NOT 5_08).
	 * 
	 * @return boolean
	 */
	
	public boolean GetUseJavascript()
	{
		return(GetOwner().GetAccessStyle() == AccessStyles.STANDARD);
	}
	
	/**
	 * This is a helper method that all classes that inherit from AFormDP can 
	 * override.  When displaying in place refresh content, this method is 
	 * called instead of PageDisplay. In place refresh can be enabled via 
	 * setting a flag in the activity space.  
	 * 
	 * @return String Partial page content to be displayed.
	 */
	public HTMLElement PartialContentDisplay()
	{
	    HTMLElementCollection myCollection = new HTMLElementCollection();
	    
	    myCollection.AddInnerHTMLString(m_asOwner.GetString(159, ResourceConstants.INFRASTRUCTURE));
	    
	    return myCollection;
	}
}
