package com.plumtree.uiinfrastructure.form;

import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;

// Framework Imports
import com.plumtree.openlog.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;

/**
 * This class is used along with IRepostModel and IRepostAS
 * to partially implement the logic of the repost action.
 * Those two classes are used by the Editor Framework to help
 * implementing the repost mechanism on any Editor page.
 *
 * They can also be used by any other Activity Space.
 * To do so here is what you need to do:
 * - Extend this class in your activity space and add any repost
 *   action (e.g.: com.plumtree.portalpagessample.editor.template1.PropRepostControl).
 * - Your model must implement IRepostModel.
 * - Your AS must implement IRepostAS.
 * - In your view, you can implement any repost action using the postToSelf
 *   JavaScript method.
 * - If you extend PlumtreeDP and call SetAddMainForm(true) in the init
 *   method of your DP, the form and the JS postToSelf will be printed out
 *   automatically.
 *
 * @author Julie Fournier
 */
public class RepostControl implements IControl
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.form.RepostControl");

	/** Object Name for AS */
	public static final String STR_MVC_CLASS_NAME = "RepostControl";

	/** Form element names */
	public static final String HTMLINPUT_POSTTOSELF		 = "in_hi_PostToSelf";

	/** HTML Element: 508 submit button */
	public static final String HTMLSUBMIT_508_POSTTOSELF = "in_su_508PostToSelf_";

	/** The pass thru inputs need to be manually added in 508 mode if they are needed. */
	public static final String HTMLINPUT_PASSTHRU_CONTROL = "in_hi_Passthru";

	/** HTML Element: Pass thru arguments */
	public static final String HTMLINPUT_PASSTHRU_ARGS = "in_hi_PassthruArgs";

	/**
	 * Repost action constants
	 * 
	 * Actions from -1 to 9 are reserved for the Framework.
	 * 
	 * TODO - All the reserved numbers have been used - may want to reserve a new batch.
	 */
	
	/** JF- Repost Action: Undefined */
	public final static int POSTTOSELF_ACTION_UNDEFINED    = -1;

	/** RW- Repost Action: In place refresh */
	public final static int POSTTOSELF_ACTION_IN_PLACE_REFRESH = 0;

	/** JF- Repost Action: Simple repost */
	public final static int POSTTOSELF_ACTION_SIMPLEREPOST = 1;

	/** JF- Repost Action: Finish */
	public final static int POSTTOSELF_ACTION_FINISH   = 2;

	/** JF- Repost Action: Cancel */
	public final static int POSTTOSELF_ACTION_CANCEL   = 3;

	/** JF- Repost Action: Change page */
	public final static int POSTTOSELF_ACTION_GOTOPAGE = 4;
	
	/** JF- Repost Action: Apply changes */
	public final static int POSTTOSELF_ACTION_APPLY_CHANGES = 5;

	/** JF- Repost Action: Pass thru */
	public final static int POSTTOSELF_ACTION_PASSTHRU = 9;

	/** JF- Page status: Valid */
	public static final int PAGE_STATUS_VALID   = 1;

	/** JF- Page status: Invalid */
	public static final int PAGE_STATUS_INVALID = 2;
	
	/** JF- Associated Activity Space. */
	protected AActivitySpace m_asOwner;

	/** JF- Associated Model */
	protected IRepostModel m_model;
	
	/** JF- Redirect object. Used to redirect at the end of the Form. */
	protected Redirect m_rRedirect = null;

	/** 
	 * Variables for the lifetime of the execute().  Just for convenience. 
	 * (Contains all the form params from the request.)
	 */
	protected XPHashtable m_htFormData;
	
	/** 
	 * has Init() been called properly?
	 */
	protected boolean m_bInitialized = false;
	
	/** 
	 * JF- Value of the repost action
	 */
	protected int m_nAction = POSTTOSELF_ACTION_UNDEFINED;

	/** 
	 * JF- Server side validation. Current state.
	 *  ***Note that if the state is set to invalid, all it does is 
	 *  it will not perform the changePage and finish actions.
	 *  ***Also note that this framework does not provide any way to 
	 *  set the error/status message to display to the end user. 
	 *  Another framework is responsible for it.
	 *  ***Finaly note that multiple save methods might return invalid on the same DP 
	 *  but we don't want to keep an array of status returned, 
	 *  we just need to know if invalid was returned at least once. */
	protected int m_nPageStatus = PAGE_STATUS_VALID;

	/**
	 * This code is called by the AS Servlet each time there is a repost.
	 * First it saves the page and then performs any repost action.
	 *
	 * If the Redirect object returned is not null,
	 *  the Servlet will go to the page specified by the Redirect.
	 *
	 * If the Redirect is null, the Servlet will get the current page
	 * from the AS and display it.
	 *
	 * @return Redirect
	 */
	public Redirect CheckActionSecurityAndExecute(XPHashtable arguments)
	{
		if (!m_bInitialized)
		{
			
				log.Error("Repost control has not been initialized: likely cause - Init() overridden but super.Init() not called.");
			
		}

		// Initialize control
		m_htFormData     = arguments;
		m_rRedirect		 = null;
		m_nAction		 = POSTTOSELF_ACTION_UNDEFINED;

		// Set the repost action mode in the member variable.
		SetRepostAction();

		// Save the page
		SaveCurrentPage();

		// perform action
		PerformAction(m_nAction);

		// clear hashtable of form params
		m_htFormData = null;

		return m_rRedirect;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new RepostControl();
	}

	/**
	 * Return the class name.
	 *
	 * @return String
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Returns the current page status.
	 *
	 * @return int
	 */
	public int GetPageStatus()
	{
		return m_nPageStatus;
	}

	/**
	 * Returns the current value of the repost action.
	 *
	 * @return int
	 */
	public int GetRepostAction()
	{
		return m_nAction;
	}

	/**
	 * Inits the class with the associated model and activity space.
	 *
	 * If this method is overridden in a sub-class, this version of
	 * Init must be called as super.Init();
	 *
	 * @param model
	 * @param space
	 */
	public void Init(IModel model, AActivitySpace space)
	{
		try
		{		
			try
			{
				m_model = (IRepostModel) model;
			}
			catch (Exception e)
			{
				throw XPException.GetInstance(e);
			}
		}catch(XPClassCastException cce)
		{
			
				log.Error(cce, "RepostControl::Init Unable to cast the Object " + model +
					" to IRepostModel. The following exception occured.");
			

			// JF- No need to extract, this is an unexpected error.
			throw new XPException("Unable to cast the Object " + model +
				" to IRepostModel.", cce);
		}

		m_asOwner     = space;

		m_bInitialized = true;
	}

	/**
	 * Sets a new page status.
	 *
	 * @param _nPageStatus
	 */
	public void SetPageStatus(int _nPageStatus)
	{
		m_nPageStatus = _nPageStatus;
	}

	/**
	 * Given the repost action mode, calls the associated method in the model.
	 *
	 * @param _nAction
	 */
	protected void PerformAction(int _nAction)
	{
		switch (_nAction)
		{
			case POSTTOSELF_ACTION_FINISH:	
				if (PAGE_STATUS_VALID == m_nPageStatus)
				{
					m_model.Finish();
					SetRedirect(m_model.GetRedirectAfterFinish());
				}	
				break;	
				
			case POSTTOSELF_ACTION_CANCEL:
				SetRedirect(m_model.GetRedirectAfterFinish());	
				break;
	
			case POSTTOSELF_ACTION_PASSTHRU:
				PerformPassthruAction();	
				break;
	
			case POSTTOSELF_ACTION_IN_PLACE_REFRESH:
			    // set a flag so that display pages will show in place 
			    // content instead of normal content on the next pass.
			    m_asOwner.SetIsInPlaceRefreshDisplayOn(true);
				break;
				
			// JF: Do not add a default case because sub classes will 
		    // extend this one and add their own action modes.
		}
	}

	/**
	 * Sets the Pass thru arguments from the pass thru control into m_htFormData.
	 * And executes the Pass thru control.
	 */
	protected void PerformPassthruAction()
	{
		if (m_htFormData.ContainsKey(HTMLINPUT_PASSTHRU_CONTROL) &&
				m_htFormData.ContainsKey(HTMLINPUT_PASSTHRU_ARGS))
		{
			// Get the passthru control
			String[] ctlName  = (String[]) m_htFormData.GetElement(HTMLINPUT_PASSTHRU_CONTROL);
			IControl passthru = m_asOwner.GetControl(ctlName[0]);

			// Get the passthru arguments
			String[] ctlArgs = (String[]) m_htFormData.GetElement(HTMLINPUT_PASSTHRU_ARGS);
      if (null != ctlArgs[0]) {

      // URL-decode the arguments (since they were submitted through an onclick they
      // were not decoded automatically)
      ctlArgs[0] = XPHttpUtility.UrlDecode(ctlArgs[0], LinkConstants.UTF_8_ENCODING);

			// Add arguments to the form data
			XPStringTokenizer tokenizer = new XPStringTokenizer(ctlArgs[0], "&");

			while (tokenizer.HasMoreTokens())
			{
				String token = tokenizer.GetNextToken();

				int equals = token.indexOf('=');

				if (-1 != equals)
				{
					m_htFormData.PutElement(token.substring(0, equals),
						new String[] { token.substring(equals + 1) });
				}
			}

			passthru.CheckActionSecurityAndExecute(m_htFormData);
      }
		}
	}

	/**
	 * Get the current page name and call the save method on the model.
	 *
	 */
	protected void SaveCurrentPage()
	{
		// We always save the data on the page except on cancel.
		if (POSTTOSELF_ACTION_CANCEL != m_nAction)
		{
			// JF- Get back the page name from the form.
			String[] data = (String[]) m_htFormData.GetElement(ASConstants.PAGE_INPUT);

			if (data == null)
			{
				data = (String[]) m_htFormData.GetElement(ASConstants.PAGE);
			}

			if (data == null)
			{
				
					log.Error("RepostControl::SaveCurrentPage The page name has not been posted by the form.");
				

				throw new XPException(
					"Unexpected error when trying to save the current page.");
			}

			if ((0 != data.length) && (null != data[0]))
			{
				// JF- Save the page
				m_nPageStatus = m_model.SavePage(data[0], m_htFormData);

				
					log.Debug("RepostControl::SaveCurrentPage Saving page: " +
						data[0]);
				
			}
			else
			{
				
					log.Warn("RepostControl::SaveCurrentPage The page name value is invalid. The page will not be saved.");
				
			}
		}
	}

	/**
	 * Sets the redirect member variable.
	 * (Used to redirect at the end of the Editor.)
	 *
	 * @param _rRedirect
	 */
	protected void SetRedirect(Redirect _rRedirect)
	{
		m_rRedirect = _rRedirect;
	}

	/**
	 * Get the repost action mode from the form data
	 * hashtable and set it in the member variable.
	 *
	 */
	protected void SetRepostAction()
	{
		if ((AccessStyles.LOW_BANDWIDTH == m_asOwner.GetAccessStyle()) ||
				(AccessStyles.SECTION_508 == m_asOwner.GetAccessStyle()))
		{
			// JF- If we are in 508 mode, we get the repost action from the submit button name.
			// E.g.: if the button that submitted the form was:
			//       <input type="submit" name="in_su_508PostToSelf_2" value="Done">
			//       The repost action will be 2.
			IXPEnumerator enumFormKeys = m_htFormData.GetKeys();
			String sKey;

			while (enumFormKeys.MoveNext())
			{
				sKey = (String) enumFormKeys.GetCurrent();

				if (sKey.startsWith(HTMLSUBMIT_508_POSTTOSELF))
				{
					try
					{
						String sRepostAction = sKey.substring(HTMLSUBMIT_508_POSTTOSELF.length());
						m_nAction = XPConvert.ToInteger(sRepostAction);

						
							log.Debug("RepostControl::SetRepostAction Repost action ID value: " +
								m_nAction);
						

						break;
					}
					catch (Exception e)
					{
						
							log.Error("RepostControl::SetRepostAction Unable to retrieve the 508 repost " +
								"action from the submit button: " + sKey);
						
					}
				}
			}
		}
		else if (m_htFormData.ContainsKey(HTMLINPUT_POSTTOSELF))
		{
			String[] data = (String[]) m_htFormData.GetElement(HTMLINPUT_POSTTOSELF);

			if ((0 != data.length) && (null != data[0]))
			{
				m_nAction = XPConvert.ToInteger(data[0]);

				
					log.Debug("RepostControl::SetRepostAction Repost action ID value: " +
						m_nAction);
				
			}
			else
			{
				
					log.Warn("RepostControl::SetRepostAction Repost action ID was not in Query String.");
				
			}
		}
	}
}

