package com.plumtree.uiinfrastructure.form.pagination;

import com.plumtree.openlog.*;
import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.common.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.htmlconstructs.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This control sets the current pagination that the user has requested.
 * 
 * @author Don Hayler
 */
public class PaginationControl implements IControl
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.form.pagination.PaginationControl");

	/** Object Name for AS */
	public static final String STR_MVC_CLASS_NAME = "PaginationControl";

	/** Form element names */
	/** The hidden inputs ARE NOT added by the CreatePaginationBar method below. */
	/** The id of the objects to paginate (hidden input) */
	public static final String HTMLINPUT_PAGINATION_ID = "in_hi_PaginationID";

	/** The object count to paginate from (hidden input) */
	public static final String HTMLINPUT_PAGINATE_FROM = "in_hi_PaginateFrom";

	/** The object count to paginate to (hidden input) */
	public static final String HTMLINPUT_PAGINATE_TO = "in_hi_PaginateTo";
	
	/** The storage name of the associated Model (hidden input) */
	public static final String HTMLINPUT_MODEL_STORAGE_NAME = "in_hi_ModelStorageName";
	
	/** The text inputs ARE added by the CreatePaginationBar method below. */
	/** The object count to paginate from (text input) */
	public static final String HTMLINPUT_PAGINATE_FROM_TEXT = "in_tx_PaginateFrom";

	/** The object count to paginate to (text input) */
	public static final String HTMLINPUT_PAGINATE_TO_TEXT = "in_tx_PaginateTo";

	/** the name of the javascript method that pages that use the pagination bar need
	 * to implement.  This javascript method should set the html inputs and post the page to
	 * this control (potentially as a pass-thru control).
	 */
	public static final String JAVASCRIPT_PAGINATE = "Paginate";

	// NOTE: some of these should be moved to .js files
	/** internal javascript function names. */
	private static final String JAVASCRIPT_CHECK_NUMERIC_TO_FROM_VALUES = "CheckNumericToFrom";

	/** the bar that separates different sections in the pagination links. */
	public static final String PAGINATION_LINK_SEPARATOR = "&nbsp;|&nbsp;";

	/** the number of links to pagination sections that we will show at the bottom. */
	private static final int NUM_PAGINATION_LINKS = 10;

	/** model */
	private IPaginationModel m_asModel;
	
	/** Activity Space */
	private AActivitySpace m_asOwner;

	/**
	 * Static method to create a pagination bar.
	 *
	 * DisplayPages that include this pagination bar (through their views) must ensure that the
	 * JAVASCRIPT_PAGINATE method is implemented on the page.
	 *
	 * warning: in 508 mode, this will add the RepostControl.HTMLINPUT_PASSTHRU_CONTROL
	 * elements (since they won't be in the form), so if another view tries to use them
	 * there will be problems.  The pagination bar should also not be added to a page
	 * more than once in 508 mode.
	 *
	 * This bar does pagination filtering (view from X to Y).
	 * 
	 * @param AActivitySpace the owner space for this HTML
	 * @param int the pagination ID (usually the class ID)
	 * @param String the name of the HTML form
	 *
	 * @return PTConstruct a construct for the pagination bar of type PTBannerType.OBJECT_ACTION_BAR
	 */
	public static PTConstruct CreatePaginationFilterBar(AActivitySpace asOwner,
		int nPaginationID, int nObjectCount)
	{
		boolean bNoJavascript = (asOwner.GetAccessStyle() != AccessStyles.STANDARD);

		HTMLInput myInput;

		PTConstruct bannerConstruct = UIConstructFactory.GetConstruct(PTBannerType.OBJECT_ACTION_BAR,
				true, asOwner);

		try
		{
			// View Items
			HTMLSpan mySpan = new HTMLSpan();
			mySpan.SetStyleClass(PTStyleClass.ACTIONBAR_TEXT);
			mySpan.AddInnerHTMLString(asOwner.GetString(88, ResourceConstants.INFRASTRUCTURE)); //_ui_ViewItems

			bannerConstruct.AddElementToNestedElement(PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL_ELEMENT_NAME,
				mySpan);
			bannerConstruct.AddHTMLStringToNestedElement(PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL_ELEMENT_NAME,
				CommonHTMLStrings.SPACE);

			if (bNoJavascript)
			{
				// Just use normal paginate input name in 508 mode
				myInput = new HTMLInput(HTMLInputTypes.TEXT,
						HTMLINPUT_PAGINATE_FROM_TEXT, "");
			}
			else
			{
				myInput = new HTMLInput(HTMLInputTypes.TEXT,
						HTMLINPUT_PAGINATE_FROM_TEXT + nPaginationID, "");
			}

			myInput.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);
			myInput.SetMaxLength(8);
			myInput.SetSize(5);

			bannerConstruct.AddElementToNestedElement(PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL_ELEMENT_NAME,
				myInput);
			bannerConstruct.AddHTMLStringToNestedElement(PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL_ELEMENT_NAME,
				CommonHTMLStrings.SPACE);

			mySpan = new HTMLSpan();
			mySpan.SetStyleClass(PTStyleClass.ACTIONBAR_TEXT);
			mySpan.AddInnerHTMLString(asOwner.GetString(89, ResourceConstants.INFRASTRUCTURE)); //_ui_To
			bannerConstruct.AddElementToNestedElement(PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL_ELEMENT_NAME,
				mySpan);
			bannerConstruct.AddHTMLStringToNestedElement(PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL_ELEMENT_NAME,
				CommonHTMLStrings.SPACE);

			if (bNoJavascript)
			{
				// Just use normal paginate input name in 508 mode
				myInput = new HTMLInput(HTMLInputTypes.TEXT,
						HTMLINPUT_PAGINATE_TO_TEXT, "");
			}
			else
			{
				myInput = new HTMLInput(HTMLInputTypes.TEXT,
						HTMLINPUT_PAGINATE_TO_TEXT + nPaginationID, "");
			}

			myInput.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);
			myInput.SetMaxLength(8);
			myInput.SetSize(5);

			bannerConstruct.AddElementToNestedElement(PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL_ELEMENT_NAME,
				myInput);
			bannerConstruct.AddHTMLStringToNestedElement(PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL_ELEMENT_NAME,
				CommonHTMLStrings.SPACE);

			if (bNoJavascript)
			{
				// Name of passthru control to run when PASSTHRU action is selected
				myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
						RepostControl.HTMLINPUT_PASSTHRU_CONTROL, "");
				myInput.SetValue(STR_MVC_CLASS_NAME);
				bannerConstruct.AddElementToNestedElement(PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL_ELEMENT_NAME,
					myInput);

				myInput = new HTMLInput(HTMLInputTypes.SUBMIT,
						RepostControl.HTMLSUBMIT_508_POSTTOSELF +
						RepostControl.POSTTOSELF_ACTION_PASSTHRU, "");
			}
			else
			{
				myInput = new HTMLInput(HTMLInputTypes.BUTTON, "in_bu_GOTO", "");
				myInput.SetOnClick(JAVASCRIPT_PAGINATE + "(" + nPaginationID +
					", " + HTMLINPUT_PAGINATE_FROM_TEXT + nPaginationID +
					".value, " + HTMLINPUT_PAGINATE_TO_TEXT + nPaginationID +
					".value, " + nObjectCount + "); return false;");
			}

			myInput.SetValue(">>");
			myInput.SetStyleClass(PTStyleClass.FORM_BTN_TEXT);

			bannerConstruct.AddElementToNestedElement(PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL_ELEMENT_NAME,
				myInput);
			bannerConstruct.AddHTMLStringToNestedElement(PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL_ELEMENT_NAME,
				CommonHTMLStrings.SPACE);
		}
		catch (Exception e)
		{
			
				log.Error(e, "Unable to build pagination bar.");
			

			bannerConstruct = null;
		}

		return bannerConstruct;
	}
	
	/**
	 * @see com.plumtree.uiinfrastructure.form.pagination.PaginationControl#CreatePaginationLinks()
	 */
	public static HTMLElement CreatePaginationLinks(AActivitySpace asOwner,
		int nPaginationID, int nObjectCount, int nCurrentStart, int nCurrentFinish, int nPaginationSize)
	{
		return PaginationControl.CreatePaginationLinks(asOwner, nPaginationID, nObjectCount, 
			nCurrentStart, nCurrentFinish, nPaginationSize, "");	
	}

	/**
	 * Static method to create the pagination links that go at the bottom of the display.
	 *
	 * DisplayPages that include this pagination links (through their views) must ensure that the
	 * JAVASCRIPT_PAGINATE method is implemented on the page.
	 * 
	 * This bar does pagination links (prev, 1-50, 50-100, etc... next).
	 * 
	 * @param AActivitySpace the owner space for this HTML
	 * @param int the pagination ID (usually the class ID)
	 * @param String the name of the HTML form
	 *
	 * @return HTMLElement the links for the pagination at the bottom of the display
	 */
	public static HTMLElement CreatePaginationLinks(AActivitySpace asOwner,
		int nPaginationID, int nObjectCount, int nCurrentStart, int nCurrentFinish, int nPaginationSize, 
		String sModelStorageKey)
	{
		HTMLElement result = new HTMLElementCollection();
		HTMLSpan mySpan = new HTMLSpan();
		mySpan.SetStyleClass(PTStyleClass.OBJECT_TEXT);
		result.AddInnerHTMLElement(mySpan);

		boolean bNoJavascript = (asOwner.GetAccessStyle() != AccessStyles.STANDARD);

		try
		{
			// DCA - check also if number of objects is equal to number of objects being shown
			if (nObjectCount == nCurrentFinish - (nCurrentStart - 1))
			{
				// No pagination necessary.
				return result;
			} else
			{
				// if we have more than 2 sets of pagination and we're not on the first one ...
				if ((nObjectCount > (2 * nPaginationSize * NUM_PAGINATION_LINKS)) &&
					(nCurrentFinish > (nPaginationSize * NUM_PAGINATION_LINKS)))
				{
					// first link

					// Display link
					mySpan.AddInnerHTMLElement(PaginationControl.GetPaginationAnchor(1, nPaginationSize,
						nPaginationID, asOwner.GetString(139, ResourceConstants.INFRASTRUCTURE), bNoJavascript,
						sModelStorageKey, asOwner));

					// Display spacer					
					mySpan.AddInnerHTMLString(PAGINATION_LINK_SEPARATOR);
				}

				// prev link
				if (nCurrentStart > 1)
				{
					// link
					int nLinkLowerLimit = 1;
					int nLinkUpperLimit = nPaginationSize;

					for (int i = 1; i <= nCurrentStart; i += nPaginationSize)
					{
						// Calculate the upper and lower bounds of the previous section
						// for previous link
						nLinkLowerLimit = i - nPaginationSize;
						if (nLinkLowerLimit < 1)
						{
							nLinkLowerLimit = 1;
						}
						nLinkUpperLimit = nLinkLowerLimit + nPaginationSize - 1;
					}

					// Display link
					mySpan.AddInnerHTMLElement(PaginationControl.GetPaginationAnchor(nLinkLowerLimit, nLinkUpperLimit,
						nPaginationID, asOwner.GetString(207, ResourceConstants.PORTALINFRASTRUCTURE), bNoJavascript,
						sModelStorageKey, asOwner));

					// Display spacer					
					mySpan.AddInnerHTMLString(PAGINATION_LINK_SEPARATOR);
				}

				// Calculate beginning and end of current pagination sections to display

				// integer division will truncate remainder to give us current section.
				int nPaginationSectionStart = nCurrentStart / (nPaginationSize * NUM_PAGINATION_LINKS);
				// get start of current section
				nPaginationSectionStart = (nPaginationSectionStart * nPaginationSize * NUM_PAGINATION_LINKS) + 1;

				int nPaginationSectionEnd = nPaginationSectionStart + (nPaginationSize * NUM_PAGINATION_LINKS) - 1;


				if (nPaginationSectionStart > 1)
				{
					// link to previous set of pagination links

					int nPreviousPaginationSectionStart = nPaginationSectionStart - (nPaginationSize * NUM_PAGINATION_LINKS);
					int nPreviousPaginationSectionEnd = nPreviousPaginationSectionStart + nPaginationSize - 1;

					// Display link
					mySpan.AddInnerHTMLElement(PaginationControl.GetPaginationAnchor(nPreviousPaginationSectionStart,
						nPreviousPaginationSectionEnd, nPaginationID,
						asOwner.GetString(137, ResourceConstants.INFRASTRUCTURE), bNoJavascript,
						sModelStorageKey, asOwner));

					mySpan.AddInnerHTMLString(PAGINATION_LINK_SEPARATOR);
				}

				int nCurrentUpperLimit;

				// object group links
				for (int i = 1; i < nObjectCount; i += nPaginationSize)
				{
					nCurrentUpperLimit = i + nPaginationSize - 1;
					if (nCurrentUpperLimit > nObjectCount)
					{
						nCurrentUpperLimit = nObjectCount;
					}

					// Only display pagination sections within the pagination section limits (10 sections)
					if ((i >= nPaginationSectionStart) && (nCurrentUpperLimit <= nPaginationSectionEnd))
					{
						// m - n string
						String strCurrentRange = XPResourceManager.FormatMessage(
							asOwner.GetString(138, ResourceConstants.INFRASTRUCTURE),
							new Integer(i), new Integer(nCurrentUpperLimit));
	
						if ((nCurrentStart >= i) && (nCurrentFinish <= nCurrentUpperLimit))
						{
							// Display as text without a link
							mySpan.AddInnerHTMLString(strCurrentRange);
						} else
						{
							// Display link
							mySpan.AddInnerHTMLElement(PaginationControl.GetPaginationAnchor(
								i, nCurrentUpperLimit, nPaginationID, strCurrentRange, bNoJavascript,
								sModelStorageKey, asOwner));
						}
	
						mySpan.AddInnerHTMLString(PAGINATION_LINK_SEPARATOR);
					}
				}

				if (nPaginationSectionEnd < nObjectCount)
				{
					// link to next set of pagination links

					int nNextPaginationSectionStart = nPaginationSectionStart + (nPaginationSize * NUM_PAGINATION_LINKS);
					int nNextPaginationSectionEnd = nNextPaginationSectionStart + nPaginationSize - 1;
					if (nNextPaginationSectionEnd > nObjectCount)
					{
						nNextPaginationSectionEnd = nObjectCount;
					}

					// Display link
					mySpan.AddInnerHTMLElement(PaginationControl.GetPaginationAnchor(
						nNextPaginationSectionStart, nNextPaginationSectionEnd,
						nPaginationID, asOwner.GetString(137, ResourceConstants.INFRASTRUCTURE),
						bNoJavascript, sModelStorageKey, asOwner));

					mySpan.AddInnerHTMLString(PAGINATION_LINK_SEPARATOR);
				}

				// next link
				if (nCurrentFinish < nObjectCount)
				{
					// link
				    // DCA - set new upper limit to 1 more than the current last item being shown
					int nNewLowerLimit = nCurrentFinish + 1;
					int nNewUpperLimit = nObjectCount;

					// Calculate the upper and lower bounds of the next section
					// for next link
					for (int i = 1; i < nObjectCount; i += nPaginationSize)
					{
						if (i >= nCurrentFinish)
						{
							nNewLowerLimit = i;
							nNewUpperLimit = i + nPaginationSize - 1;
	
							break;
						}
					}

					if (nNewUpperLimit > nObjectCount)
					{
						nNewUpperLimit = nObjectCount;
					}
					if (nNewLowerLimit < 1)
					{
						nNewLowerLimit = 1;
					}

					// Display link
					mySpan.AddInnerHTMLElement(PaginationControl.GetPaginationAnchor(
						nNewLowerLimit, nNewUpperLimit, nPaginationID,
						asOwner.GetString(208, ResourceConstants.PORTALINFRASTRUCTURE),
						bNoJavascript, sModelStorageKey, asOwner));
				}

				// if we have more than 2 sets of pagination and we're not on the last one ...
				if ((nObjectCount > (2 * nPaginationSize * NUM_PAGINATION_LINKS)) &&
					(nPaginationSectionEnd < nObjectCount))
				{
					// last link

					// integer division truncates, leaving us with number of pagination groups
					int nLastLowerLimit = nObjectCount / nPaginationSize;
					// Get start of last pagination group
					nLastLowerLimit = (nLastLowerLimit * nPaginationSize) + 1;

					if (nLastLowerLimit > nObjectCount)
					{
						nLastLowerLimit = nLastLowerLimit - nPaginationSize;
					}

					// Display separator
					mySpan.AddInnerHTMLString(PAGINATION_LINK_SEPARATOR);

					// Display link
					mySpan.AddInnerHTMLElement(PaginationControl.GetPaginationAnchor(nLastLowerLimit, nObjectCount,
						nPaginationID, asOwner.GetString(140, ResourceConstants.INFRASTRUCTURE), bNoJavascript,
						sModelStorageKey, asOwner));
				}
			}
		}
		catch (Exception e)
		{
			
				log.Error(e, "Unable to create pagination links.");
			

			result = new HTMLElementCollection();
		}

		return result;
	}

	/** Helper method to generate links to new paginations. */
	private static HTMLAnchor GetPaginationAnchor(int nLowerLimit, int nUpperLimit, int nPaginationID,
		String strText, boolean bNoJavascript, String sModelStorageKey, AActivitySpace asOwner)
	{
		HTMLAnchor result;

		// Display link
		if (bNoJavascript)
		{
			ASURL myURL = new ASURL(asOwner);
			myURL.SetLinkGetCachedSpace(asOwner);
			myURL.SetControl(PassthruControl.STR_MVC_CLASS_NAME);
			myURL.AddControlArgument(PassthruControl.QS_PASSTHRU_CONTROL, STR_MVC_CLASS_NAME);
			myURL.AddControlArgument(PassthruControl.QS_MAIN_CONTROL, ((IFormAS) asOwner).GetRepostControlName());
			myURL.AddControlArgument(ASConstants.PAGE, asOwner.GetCurrentPage().GetName());
			myURL.AddControlArgument(HTMLINPUT_PAGINATION_ID, nPaginationID);
			myURL.AddControlArgument(HTMLINPUT_PAGINATE_FROM, nLowerLimit);
			myURL.AddControlArgument(HTMLINPUT_PAGINATE_TO, nUpperLimit);
			result = (HTMLAnchor) myURL.GetURLAsHTMLElement();
			result.AddInnerHTMLString(strText);
		} else
		{
			result = ConstructStaticHelpers.GetJavascriptAnchor
				(strText, JAVASCRIPT_PAGINATE + "(" + nPaginationID +
				", " + nLowerLimit + ", " + nUpperLimit + ",'','" +
				sModelStorageKey + "')");
		}

		return result;
	}

	/**
	 * This method creates the default javascript for the pagination control.
	 *
	 * This assumes that there is an AFormDP that should be reposted via a
	 * pass-thru control.
	 *
	 * @return String the javascript to display.
	 */
	public static String CreatePaginationJavascript(AActivitySpace asOwner)
	{
		XPStringBuilder sb = new XPStringBuilder(1250);

		sb.Append("function ").Append(JAVASCRIPT_CHECK_NUMERIC_TO_FROM_VALUES)
		.Append("(strBegin, strEnd)\n");
		sb.Append("{\n");
		// Check if the strings are numeric or empty
		sb.Append("  if ((false == PTNumberUtil.isInteger(strBegin)) ||\n");
		sb.Append("      (false == PTNumberUtil.isInteger(strEnd)) ||\n");
		// Check that the strings are not empty.
		sb.Append("      (true == PTStringUtil.isAllWhitespace(strBegin)) ||\n");
		sb.Append("      (true == PTStringUtil.isAllWhitespace(strEnd)))\n");
		sb.Append("  {\n");
		sb.Append("      alert(\"").Append(asOwner.GetString(160, ResourceConstants.INFRASTRUCTURE)) //_ui_JavaScriptAlert_OutOfRangeValue)
		.Append("\");\n");
		sb.Append("      return false;\n");
		sb.Append("  }\n");
		sb.Append("  return true;\n");
		sb.Append("}\n");

		sb.Append("function ").Append(JAVASCRIPT_PAGINATE)
		.Append("(nID,strBegin,strEnd,strMax,strModelStorageKey)\n").Append("{\n")
		.Append("  if (true == ").Append(JAVASCRIPT_CHECK_NUMERIC_TO_FROM_VALUES)
		.Append("(strBegin, strEnd))\n").Append("  {\n")
		.Append("     var nBegin = parseInt(strBegin);\n")
		.Append("     var nEnd = parseInt(strEnd);\n")
		.Append("     var nMax = parseInt(strMax);\n")
		.Append("     if (nBegin < 1)\n").Append("     {\n")
		.Append("        nBegin = 1;\n")
		.Append("     }\n")
		.Append("     if (nEnd < 1)\n").Append("     {\n")
		.Append("        nEnd = 1;\n")
		.Append("     }\n")
		.Append("     if (nBegin > nMax)\n").Append("     {\n")
		.Append("        nBegin = nMax;\n")
		.Append("     }\n")
		.Append("     if (nEnd > nMax)\n").Append("     {\n")
		.Append("        nEnd = nMax;\n")
		.Append("     }\n")
		.Append("     if (nEnd < nBegin)\n").Append("     {\n")
		.Append("        nEnd = nBegin;\n")
		.Append("     }\n")
		.Append("     frm = document.").Append(AFormDP.HTMLFORM_NAME).Append(";\n")
		.Append("     frm.").Append(HTMLINPUT_PAGINATION_ID).Append(".value = nID;\n")
		.Append("     frm.").Append(HTMLINPUT_PAGINATE_FROM).Append(".value = nBegin;\n")
		.Append("     frm.").Append(HTMLINPUT_PAGINATE_TO).Append(".value = nEnd;\n")
		.Append("     frm.").Append(HTMLINPUT_MODEL_STORAGE_NAME).Append(".value = strModelStorageKey;\n")
		.Append("     ").Append(AFormDP.JAVASCRIPT_SUBMIT_TO_CONTROL)
		.Append("('").Append(STR_MVC_CLASS_NAME).Append("');\n")
		.Append("  }\n").Append("}\n");

		return sb.ToString();
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IControl#CheckActionSecurityAndExecute(XPHashtable)
	 */
	public Redirect CheckActionSecurityAndExecute(XPHashtable arguments)
	{
		int nPaginationID;
		int nPaginationUpperBound;
		int nPaginationLowerBound;

		// Find pagination ID
		String[] sValues = (String[]) arguments.GetElement(HTMLINPUT_PAGINATION_ID);

		if (sValues != null)
		{
			nPaginationID = XPConvert.ToInteger(sValues[0]);
		}
		else
		{
			
				log.Warn("Unable to find pagination ID.  Using default.");
			

			nPaginationID = 1;
		}

		// Find Pagination lower bound
		sValues = (String[]) arguments.GetElement(HTMLINPUT_PAGINATE_FROM);

		if (sValues != null)
		{
			nPaginationLowerBound = XPConvert.ToInteger(sValues[0]);
		}
		else
		{
			sValues = (String[]) arguments.GetElement(HTMLINPUT_PAGINATE_FROM_TEXT);

			if (sValues != null)
			{
				nPaginationLowerBound = XPConvert.ToInteger(sValues[0]);
			}
			else
			{
				
					log.Error("Unable to find pagination lower bound.  Using default.");
				

				nPaginationLowerBound = 1;
			}
		}

		// Find Pagination upper bound
		sValues = (String[]) arguments.GetElement(HTMLINPUT_PAGINATE_TO);

		if (sValues != null)
		{
			nPaginationUpperBound = XPConvert.ToInteger(sValues[0]);
		}
		else
		{
			sValues = (String[]) arguments.GetElement(HTMLINPUT_PAGINATE_TO_TEXT);

			if (sValues != null)
			{
				nPaginationUpperBound = XPConvert.ToInteger(sValues[0]);
			}
			else
			{
				
					log.Error("Unable to find pagination upper bound.  Using default.");
				

				nPaginationUpperBound = 1;
			}
		}
		
		// Find Model storage key
		String sModelStorageKey = "";
		sValues = (String[]) arguments.GetElement(HTMLINPUT_MODEL_STORAGE_NAME);

		if ((sValues != null) && (null != sValues[0]))
		{
			sModelStorageKey = XPConvert.ToString(sValues[0]);
		}
		
		if ((("").equals(sModelStorageKey)) || (!m_asOwner.GetModelExists(sModelStorageKey)))
		{
			
				log.Debug("Unable to find Model storage name.  Using default Model.");
			
			
			m_asModel.SetPagination(nPaginationID, nPaginationLowerBound,
						nPaginationUpperBound);
		}
		else
		{
			try
			{
				IPaginationModel myModel = (IPaginationModel) m_asOwner.GetModel(sModelStorageKey);
				myModel.SetPagination(nPaginationID, nPaginationLowerBound,
					nPaginationUpperBound);
			}
			catch (Exception e)
			{
				
					log.Error("Retrieved wrong Model type with storage key: " + sModelStorageKey + ". Expecting IPaginationModel");
				
			}
		}
		
		return null;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new PaginationControl();
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IControl#Init(IModel, AActivitySpace)
	 */
	public void Init(IModel model, AActivitySpace space)
	{
		m_asOwner = space;
		
		try
		{
			m_asModel = (IPaginationModel) model;
		}
		catch (Exception e)
		{
			
				log.Error(e, "Initialized with wrong type of model.  Expecting IPaginationModel.");
			
		}
	}
}

