package com.plumtree.uiinfrastructure.form.status;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.htmlconstructs.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This class contains the HTML and Javascript for placing a status
 * bar into your HTML.  While it implements IView, this class does 
 * not need to be loaded into an AS in the traditional manner. 
 * Plenty of static methods allow you to insert this code into your
 * existing DPs and Views as an optional/external component.
 * 
 * @author RickyW
 */
public class StatusView implements IView 
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.form.status.StatusView");

	public static final String STR_MVC_CLASS_NAME = "StatusView";
	
	private static final String ELEMID_TABLE = "StatusViewTable";
	private static final String ELEMID_CELL = "StatusViewCell";
	
	private static final String FILENAME_JS_STATUSPAGE = "ptstatuspage.js";	
	
	private static final String JS_METHOD_SHOW_IN_BODY = "PTStatusPage.replaceBodyWithStatusAndShow";

	protected AActivitySpace m_asOwner;
	
	/* (non-Javadoc)
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#Init(com.plumtree.uiinfrastructure.activityspace.IModelRO, com.plumtree.uiinfrastructure.activityspace.AActivitySpace)
	 */
	public void Init(IModelRO model, AActivitySpace parent) 
	{
		m_asOwner = parent;
	}

	/* (non-Javadoc)
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName() 
	{
		return STR_MVC_CLASS_NAME;
	}

	/* (non-Javadoc)
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create() 
	{
		return new StatusView();
	}

	/* (non-Javadoc)
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#Display()
	 */
	public HTMLElement Display() 
	{
		String strLangID = (String) PersonalSettingsHelper.
			GetPersonalSettingValue(PersonalSettingsHelper.LANGUAGE, m_asOwner);

		return StatusView.GetDisplay(strLangID);
	}

	/* (non-Javadoc)
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()
	 */
	public HTMLScript DisplayJavascript() 
	{
		return StatusView.GetDisplayJavascript(m_asOwner.GetIsSecuredSpace());
	}

	/**
	 * Alternative way to get the display from this view 
	 * without the need to create an instance of it.
	 * 
	 * @return This method returns the HTMLElement for the view.
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#Display()
	 */
	public static HTMLElement GetDisplay(String strLang)
	{
		HTMLElementCollection result;
		HTMLTable myTable;
		HTMLTableRow myRow;
		HTMLTableCell myCell;
		HTMLImg myImage;
		HTMLSpan mySpan;
		
		// new collection.
		result = new HTMLElementCollection();
		
		try
		{
			// SPAN - Main span.
			mySpan = new HTMLSpan();
			mySpan.SetStyleClass(PTStyleClass.DIALOGUE_BG);
			result.AddInnerHTMLElement(mySpan);
			
			// TABLE - Main table.
			myTable = new HTMLTable();
			myTable.SetID(StatusView.ELEMID_TABLE);
			myTable.SetBorder("0");
			myTable.SetWidth("100%");
			myTable.SetHeight("90%");
			myTable.SetCellPadding("0");
			myTable.SetCellSpacing("10");
			myTable.SetStyle("display:none");
			mySpan.AddInnerHTMLElement(myTable);

			// ROW - Main row.
			myRow = new HTMLTableRow();
			myTable.AddInnerHTMLElement(myRow);
			myCell = new HTMLTableCell();
			myCell.SetID(StatusView.ELEMID_CELL);
			myCell.SetAlign("center");
			myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
			myRow.AddInnerHTMLElement(myCell);
			
			// CELL 1 - Status Text.
			myCell.AddInnerHTMLString(ConfigHelper.GetResourceManager()
				.GetString(157, "ptmsgs_infrastructure", strLang));
			myCell.AddInnerHTMLElement(new HTMLBr());
			myCell.AddInnerHTMLElement(new HTMLBr());
			
			// CELL 2 - Status Image.
			PTConstruct myConstruct = UIConstructFactory.GetConstruct(PTImageType.ICON_STATUS_BAR);
			myImage = (HTMLImg) myConstruct.GetHTMLElement();
			myImage.SetAlt(ConfigHelper.GetResourceManager().GetString(158, "ptmsgs_infrastructure", strLang));
			myCell.AddInnerHTMLElement(myImage);
		}
		catch (Exception ex)
		{
			log.Error(ex, "Error displaying the HTML for this view.");
			throw new XPException("Error displaying the HTML for this view.", ex);
		}

		return result;
	}

	/**
	 * Alternative way to get the Javascript from this view 
	 * without the need to create an instance of it.
	 * 
	 * @return This method returns the HTMLScript for the view.
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()
	 */
	public static HTMLScript GetDisplayJavascript(boolean bIsSecureSpace) 
	{
		HTMLScript myScript = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);

		myScript.SetSrc(ConfigHelper.GetPortalImageServerJavaScriptURL(bIsSecureSpace) +
			StatusView.FILENAME_JS_STATUSPAGE);

		return myScript;
	}
	
	/**
	 * Gets the Javascript method that shows the status bar in place of the body.
	 * 
	 * @return The Javascript method that shows the status bar in place of the body.
	 */
	public static String GetJSShowStatusInBody()
	{
		return StatusView.JS_METHOD_SHOW_IN_BODY + "('" + StatusView.ELEMID_TABLE + "');\n";
	}
	
	/**
	 * Returns the method from GetJSShowStatusInBody but additionally wraps it in a
	 * check that makes sure the the repost argument is one of the values in the 
	 * repost values.
	 * 
	 * @param strRepostArgument The argument that should contain your value.
	 * @param aRepostValues The values to check your argument against.
	 * @param strSpacing Optional spacing for formatting purposes.
	 * 
	 * @return Check-Wrapped Javascript method that shows the status bar in place of the body.
	 */
	public static String GetJSShowStatusInBodyRestricted(String strRepostArgument, int[] aRepostValues, String strSpacing)
	{
		if (null == aRepostValues)
		{
			return strSpacing + GetJSShowStatusInBody();
		}
		else
		{
			if (null == strSpacing)
				strSpacing = "";
				
			XPStringBuilder sb = new XPStringBuilder(300);
		
			sb.Append(strSpacing).Append("if (");
			
			for (int ii = 0; ii < aRepostValues.length; ii ++)
			{
				sb.Append("(")
				.Append(strRepostArgument)
				.Append(" == ")
				.Append(aRepostValues[ii])
				.Append(") || ");
			}
			
			sb.Append("false)\n");
			
			sb.Append(strSpacing).Append("{\n");
			sb.Append(strSpacing).Append(strSpacing).Append(StatusView.GetJSShowStatusInBody());
			sb.Append(strSpacing).Append("}\n");
			
			return sb.ToString();
		}
	}
}
