package com.plumtree.uiinfrastructure.interpreter;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.activityspace.utils.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.interpreter.filter.utils.*;
import com.plumtree.uiinfrastructure.login.*;
import com.plumtree.uiinfrastructure.opener.*;
import com.plumtree.uiinfrastructure.opener.types.classtypes.*;
import com.plumtree.uiinfrastructure.pei.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This is the main MVC controller for invoking the application's logic.  
 * It performs the execution of activityspace related control flow.
 *  
 * @author Don Hayler
 * @author Julie Fournier (Security Modes)
 * @author WilliamA  (Writing this new class)
 */
public class MVCController implements IMVCController {

	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.interpreter.MVCController");

	// VARIABLES
	private static MVCController m_Instance = new MVCController();
	private final static String strSessionPrefsSpaceName = "SessionPrefs";
	
	// CONSTRUCTORS
	private MVCController()
	{		
	}
	
	// METHODS (static)
	public static MVCController GetInstance() {
		return m_Instance;
	}
	
	// METHODS
	/**
	 * This method calls the IPageActions OnPageStart() method.
	 * 
	 * @see com.plumtree.uiinfrastructure.activityspace.IPageActions
	 * @see com.plumtree.uiinfrastructure.interpreter.IMVCController#DoOnPageStartActions(com.plumtree.uiinfrastructure.application.RequestData, java.lang.String)
	 */
	public void DoOnPageStartActions(Redirect redirect, RequestData requestData, String hostname)
	{
		if (redirect!=null) 
		{		
			// Now set to the new redirect
			requestData.SetRedirect(redirect);
		}
		
		// Try to get the Activity Space from the cache
		LocateActivitySpace(requestData);

		// Create new space if could not retrieve a cached space.
		if (requestData.m_asSpace == null)
		{
			// Create new space
			CreateSpace(requestData);

			Redirect secureRedirect = CheckSecureActivitySpace(requestData);
			if (secureRedirect!=null)
			{
				// Switched security modes through a 302 redirect
				throw new UnauthorizedAccessException(secureRedirect);
			}
		}

		Object[] oa;
		Object o;
		IPageActions iActions;
	
		oa = (Object[]) requestData.m_Application.GetCachingManager().GetEntry("PageActions");
		
		//	ON PAGE START
		if (oa != null)
		{
			for (int x = 0; x < oa.length; x++)
			{
				o = oa[x];

				log.Debug("DoOnPageStartActions() - About to call OnPageStart on an: " + o);			
			
				if(o != null)
				{
					iActions = (IPageActions) o;
					iActions.OnPageStart(requestData.m_asSpace, requestData.m_asSpace.GetUserSession(), hostname);
				 }
			}
		}
	}

	/**
	 * This method performs the core control flow execution of activityspace
	 * related logics. It processes the given redirect (if any) in the context
	 * of the requestData. This method may return a final Redirect object,
	 * which typically is a Redirect containing a DisplayPage object, or
	 * a flag to indicate an HTTP redirect.
	 */
	public Redirect HandleRedirect(Redirect redirect, RequestData tempData)
	{
		Redirect prevRedirect = null;
		XPHashtable prevHtArgs = null;
		Redirect secureRedirect = null;

		if (redirect!=null) 
		{
			prevRedirect = tempData.GetRedirect();
			
			// Query string sometimes contains more params than the prevRedirect contains 
			// so keep this (for redirect after login)
			prevHtArgs = tempData.GetQueryString();
			
			// Now set to the new redirect
			tempData.SetRedirect(redirect);
		}
		
		secureRedirect = CheckSecureActivitySpace(tempData);

		// Try to get the Activity Space from the cache
		LocateActivitySpace(tempData);

		// Create new space if could not retrieve a cached space.
		if (tempData.m_asSpace == null)
		{
			// Create new space
			CreateSpace(tempData);

			secureRedirect = CheckSecureActivitySpace(tempData);
			if (secureRedirect!=null)
			{
				// Switched security modes through a 302 redirect
				throw new UnauthorizedAccessException(secureRedirect);
			}
		}

		// WilliamA - Move the throw of UnauthorizedAccessException to after locating
		// or creating the activityspace so that requestData.m_asSpace is not null,
		// which is used by the OnPageFinish PEIs. Otherwise user session and space will 
		// be both null inside the OnPageFinish PEIs.
		if (secureRedirect!=null)
		{
			// Switched security modes through a 302 redirect
			throw new UnauthorizedAccessException(secureRedirect);
		}
			
		log.Info("Current space is " + tempData.m_asSpace.GetName());		

		// Redirect to pass to the login control if the guest user gets bounced
		// from a page.  Default to null (which the login control will interpret
		// as go the the default mypage).
		Redirect redirectAfterLogin = null;

		boolean bLoginOccurredCookiePresent = LoginHandlers.IsLoginOccurredCookiePresent(tempData.m_xpRequest);
				
		boolean bIsLoginSpace = LoginHandlers.LOGINSPACE.equals(tempData.GetRootSpaceName());

		// WilliamA - Check one more thing since we support MyPage as a login page now
		String loginPageSpaceName = (String)tempData.m_SessionManager.GetAttribute(
			Authenticator.LOGINPAGESPACE_ATTR_KEY);
			
		if (!bIsLoginSpace && (null != loginPageSpaceName) && 
			(loginPageSpaceName.equals(tempData.GetRootSpaceName())))
		{
			bIsLoginSpace = true;
			// don't mark this as access denied in this case
		}
		
		// Bug 39028 - special case for "CreateAccount" space:
		// That is, allow guest access to "CreateAccount" space even if AllowGuestAccess
		// setting is set to 0 (disabled) for the portal.
		boolean bIsCreateAccountGuestAccess = false;
		
		if (tempData.m_bGuest)
		{
			// On submit the create account page does not use ASURL, but uses "space=CreateAccountAS"
			if (tempData.GetSpaceName().equals("CreateAccountAS"))
			{
				bIsCreateAccountGuestAccess	= true;
			}
			// The "create account" link uses ASURL, however, and ASURL goes through the 
			// common opener AS, so check for this as well.
			else if (IsCommonOpenerRedirectingTo("CreateAccountAS", tempData))
			{
				bIsCreateAccountGuestAccess	= true;
			}
		}
		
		
		boolean bPageAccessDenied = false;
		boolean bIsCustomLoginRedirect = false;
		
		// Check whether user has rights to see this page
		if (!tempData.m_asSpace.CheckBasicAccess(tempData.m_sPage, tempData.m_sControl, false))
		{
			bPageAccessDenied = true;
				
			log.Info("User does not have rights to see page " +
			tempData.m_asSpace.GetName() + ".  Redirecting.");
				
		} else if (tempData.m_bGuest && !LoginHandlers.s_bGuestAccessAllowed &&
			!bIsLoginSpace && !tempData.GetSpaceName().equals("RulesDebugMSGAS") &&
			!bIsCreateAccountGuestAccess)
		{
			// mj: also check whether guest access is enabled
			bPageAccessDenied = true;
				
			log.Info("Guest cannot see non-login pages.  Redirecting to login page.");				

		} else if (bLoginOccurredCookiePresent && tempData.m_bGuest &&
			!bIsLoginSpace && !tempData.GetSpaceName().equals("RulesDebugMSGAS") &&
			!bIsCreateAccountGuestAccess)
		{
			// bg: also check whether a login has occurred within this browser, 
			//	and the guest session is in use, indicating a session timeout has
			//	occurred.
		
			// Fix bug 42543: forward port Pfizer fixes in Ellington to Martok
			// regarding login PEI execution when session times out should not
			// be diverted.
									
			// We're blowing away the actual redirect we got from the PEI 
			// here because of a timeout.  If the redirect was to an ILogin space
			// which implements an ILoginControl then
			// we must be careful to use *it* rather than the default
			// to support multiple guest users. DAP feb 05
			Redirect loginPEIRedirect = tempData.GetRedirect();
			if (null != loginPEIRedirect)
			{
				// Note that the space + control don't exist yet, so we'll temporarily 
				// bypass the activityspace work to simply instantiate the thing; we're 
				// checking if it is an instance of an interface only
				String possibleLoginControlName = loginPEIRedirect.GetControl();
				IControl possibleLoginControl = null;
				if (possibleLoginControlName != null)
				{
					possibleLoginControl = (IControl)ASManager.INSTANCE.GetControl(possibleLoginControlName);
					if (possibleLoginControl != null)
					{
						bIsCustomLoginRedirect = (possibleLoginControl instanceof ILoginControl);
						if (bIsCustomLoginRedirect)
						{
							String sRedirectToTimeoutOriginal = tempData.GetRequestFullURL();
							
							// Tell the control to do a 302 to the original destination after login
							Redirect redir = new Redirect();
							redir.SetIsHTTPRedirect(true);
							redir.SetLinkToExternalURL(sRedirectToTimeoutOriginal);
							redirectAfterLogin = redir;
						}
					}
				}							
			}
			
			// Don't set page access denied if we have a custom login PEI,
			// so that we can invoke that login PEI.
			if (!bIsCustomLoginRedirect)
			{			
				bPageAccessDenied = true;
				
				//If this is a page that sets session prefs, we need to add a special header.
				//This header will tell JSXML that the user's portal session has expired.
				if (strSessionPrefsSpaceName.equals(tempData.GetRootSpaceName()))
				{
					// Verify thatthis is a JSXML Request
					boolean bJSXMLRequest = GatewayHandlers.IsJSXMLRequest(tempData);

					// Create a redirect that will take us to the login page for
					// this guest user.
					Redirect loginRedirect = Authenticator.GetGuestLoginRedirect(tempData);
					loginRedirect.SetIsHTTPRedirect(true);

					// Redirect us back to the current URL after login.
					loginRedirect.AddControlArgument(ASConstants.REDIRECT_AFTER_LOGIN, GatewayHandlers.GetRedirectToURL(tempData, bJSXMLRequest));

					// TODO: This may not be the correct place to go if there is
					// a custom login pei that redirects to a custom login page.
					tempData.SetRedirect(loginRedirect);

					if (bJSXMLRequest) 
					{
						//Set everything up on the response
						GatewayHandlers.HandleJSXMLSessionTimeout(tempData, loginRedirect);
					}

				}
			}
			
			log.Info("User session timed out.  Redirecting to login page.");
		}

		//	This should be done the same way as HandleGatewayRequest -- Commenting this out for now.
		//	if (bLoginOccurredCookiePresent && (bPageAccessDenied || bIsLoginSpace))
		//	{
		//		Interpreter.ClearLoginOccurredCookiePresent( response );
		//	}

		if (bPageAccessDenied)
		{
			throw new UnauthorizedAccessException();
		}

		IControl myControl = null;

		// null means default control, which not all spaces have
		myControl = tempData.m_asSpace.GetControl(tempData.m_sControl);

		if ((myControl == null) && (tempData.m_sControl != null))
		{
			log.Error("Couldn't find control: " + tempData.m_sControl);
		}

		// Special case for IHTTPControl
		if (myControl instanceof IHTTPControl)
		{
			// NOTE: this method should pass in a wrapper that
			// is cleared at the end of the request so the params
			// can't be leaked.
			((IHTTPControl) myControl).SetHTTPItems(tempData.m_xpRequest,
				tempData.m_wdPageData);
		}

		// mj: declare args hashtable here so we can add something it for sso
		XPHashtable htArgs = null;

		// Fill out the arguments from the request
		if (myControl != null)
		{
			htArgs = tempData.GetQueryString();
		}

		// set the redirect on the login space if we are redirecting there
		if (LoginHandlers.LOGINSPACE.equals(tempData.GetRootSpaceName()))
		{
			redirectAfterLogin = prevRedirect;
			
			if (htArgs==null && redirectAfterLogin != null)
			{
				htArgs = tempData.GetQueryString();
			}
			
			// We need to check if there is a redirect URL in the QS.
			String sRedirectFromQS = tempData.GetRedirectURLFromQS();
			
			// Fix bug 42543: forward port Pfizer fixes in Ellington to Martok		
			if (null == sRedirectFromQS )
			{
				// check for secondary redirect request for PEI
				sRedirectFromQS = GetSecondaryRedirectURLFromQS(tempData);
			}
						
			if (null != sRedirectFromQS)
			{
				redirectAfterLogin = new Redirect();
				redirectAfterLogin.SetLinkToExternalURL(sRedirectFromQS);
				log.Info("Redirect after login QS param: {0}", sRedirectFromQS);
			}

			if (redirectAfterLogin != null)
			{
				// For RequestData's that have had a redirect set on them, this will
				// overwrite the args of that redirect with these args, which should
				// be identical.
				if (!redirectAfterLogin.GetIsExternalURL())
				{
					redirectAfterLogin.AddControlArguments(prevHtArgs);
				}
					
				ILoginControl loginControl = (ILoginControl) tempData.m_asSpace.GetControl(LoginHandlers.LOGINCONTROL);

				if (loginControl != null)
				{
					loginControl.SetRedirectForUseAfterLogin(redirectAfterLogin);
				}
			}
		} 
		else if (bIsLoginSpace)
		{
			// WilliamA - This means guest MyPage as login page:
			// Set the redirect after login into the sandbox to be consumed
			// by the login portlet during its Display(). Right after successful
			// login, LoginModel will remove this entry from the sandbox.
			// Note that it is removed after successful login such that on failed
			// login this redirect after login entry continues to exist.
			redirectAfterLogin = prevRedirect;
			
			if (null != redirectAfterLogin)
			{
				String redirectAfterLoginURL = tempData.GetRequestFullURL();
				// DCA - make sure we have the correct http entry mode
				redirectAfterLoginURL = HttpUtils.GetNormalizedURL(redirectAfterLoginURL);
				tempData.m_smSandbox.SetAttribute("PT_" + ASConstants.REDIRECT_AFTER_LOGIN,
						redirectAfterLoginURL);
			}
			else
			{
				// Fix bug 44591 - if redirect after login is a gatewayed URL and guest MyPage
				// is the login page, the redirect after login is not preserved.
				// - Check the redirect after login QS param, if any
				String sRedirectFromQS = tempData.GetRedirectURLFromQS();
			
				// - Include fix for bug 42543: forward port Pfizer fixes in Ellington to Martok		
				if (null == sRedirectFromQS)
				{
					// check for secondary redirect request for PEI
					sRedirectFromQS = GetSecondaryRedirectURLFromQS(tempData);
				}
				
				if (null != sRedirectFromQS)
				{
					tempData.m_smSandbox.SetAttribute("PT_" + ASConstants.REDIRECT_AFTER_LOGIN,
						sRedirectFromQS);
				}
			}
		}

		if (htArgs != null)
		{
			// These two arguments are added here instead of above because we
			// don't want them in the login redirect.
	
			// The following special entries in the hashtable are required by
			// MyPage and Communities to correctly initialize AppData object
			// that is later passed to the HTTPGP.
			//		-- moose
			// NOTE: This may no longer be necessary because of
			// AActivitySpace.GettCurrentRequestURL()

			// Add URL to the arguments
			String[] sTempValues = new String[1];
			sTempValues[0]     = tempData.m_asSpace.GetCurrentRequestURL();
			htArgs.PutElement(HttpUtils.HASHTABLE_CONTROL_KEY_URL, sTempValues);
	
			// Add raw headers to the arguments
			sTempValues		   = new String[1];
			sTempValues[0]     = HttpUtils.GetRawHeaders(tempData.m_xpRequest);
			htArgs.PutElement(HttpUtils.HASHTABLE_CONTROL_KEY_HTTP_HEADERS,
				sTempValues);
		}

		Redirect myRedirect = null;

		// Handle optional Control argument and redirects
		if (myControl != null)
		{	
			log.Info("current control is " + myControl.GetName());

			tempData.m_ttTimer.SetCheckpoint(Interpreter.CHECKPOINT_CONTROL_START);

            try {
    			// Be sure to set the redirect on the proper control object  
    			// when there's a custom login PEI.
    			if (bIsCustomLoginRedirect)
    			{
    				((ILoginControl)myControl).SetRedirectForUseAfterLogin(redirectAfterLogin);
    			}
       
    			// Execute the control
    			myRedirect = myControl.CheckActionSecurityAndExecute(htArgs);
    
    			// Special case for ILoginControl
    			if (myControl instanceof ILoginControl)
    			{
    				Redirect tempRedirect = null;
    					
    				tempRedirect = LoginHandlers.DoLogin(myControl, tempData);
    					
    				if( tempRedirect != null )
    				{
    					//	If the login action returned a valid redirect object
    					//	then we'll use this instead of the redirect provided
    					//	by the CheckActionSecurityAndExecute method.  This
    					//	is because DoTasksAfterLogin() may be implemented
    					//	to return a redirect, allowing clients to modify this
    					//	behavior without modifying our code.
    						
    					myRedirect = tempRedirect;
    				}
    
    				if (myRedirect != null)
    				{
    					// The login space has not been updated with the new user session yet,
    					// so this needs to be updated manually.
    						
    					if (!myRedirect.GetIsExternalURL())
    					{
    						myRedirect.AddControlArgument(ASConstants.USERID_INPUT,
    							LoginHelper.INSTANCE.GetUserSessionID(tempData.GetUserSession()));
    					}
    					else
    					{
    						// JF- This is an external URL but it could still be a Portal URL 
    						// so we might have to append the user id to the URL.
    							
    						// JF- This happens in Security Mode 1 when the user tries to access 
    						// a Portal Page without being logged in, he will be redirected to the login page, 
    						// then once logged in, back to the original page. To redirect back to the 
    						// original page, we do a external URL to the orginal requested URL. 
    						// That's why even if it's an external URL it might still be a Portal URL.     
    							
    						String sURL = myRedirect.GetExternalURL();
    						String sHTTPEntryPoint = ConfigHelper.GetHTTPEntryPoint();
    							
    						if (null != sURL)
    						{
    							// JF- Only append the user id if this is a portal URL and not a Gateway URL
    							if ((-1 != XPStringUtility.IndexOf(sURL, sHTTPEntryPoint)) 
    								&& (-1 == XPStringUtility.IndexOf(sURL, GatewayHandlers.GATEWAY_UNIQUE))
    								&& (-1 == XPStringUtility.IndexOf(sURL, sHTTPEntryPoint + "/")))
    							{
    								// JF- Make sure there is a ? or a & before we append the user id
    								if (XPStringUtility.EndsWith(sURL, sHTTPEntryPoint))
    								{
    									sURL += "?";
    								}
    								else if (!XPStringUtility.EndsWith(sURL, "&") && !XPStringUtility.EndsWith(sURL, "?"))
    								{
    									sURL += "&";
    								}
    									
    								// JF- Append the user id and create a Redirect object with the new URL
    								sURL += ASConstants.USERID_INPUT + "=";
    								sURL += LoginHelper.INSTANCE.GetUserSessionID(tempData.GetUserSession());
    								myRedirect = new Redirect();
    								myRedirect.SetLinkToExternalURL(sURL);
    							}
    						}
    					}
    				}
    			}
    
    			// Loop through the redirects until we get to a final display page.
    			myRedirect = HandleRedirectChain(myRedirect, tempData);
            } finally {
                tempData.m_ttTimer.SetCheckpoint(Interpreter.CHECKPOINT_CONTROL_END);
            }
		}

		return myRedirect;
	}

	/**
	 * This method calls the IPageActions OnPageFinish() method and
	 * adds the resulting HTMLComments (if there are any) to the 
	 * XPResponse.
	 * 
	 * @see com.plumtree.uiinfrastructure.activityspace.IPageActions
	 * @see com.plumtree.uiinfrastructure.interpreter.IMVCController#DoOnPageFinishActions(com.plumtree.uiinfrastructure.application.RequestData, java.lang.String) 
	 */
	public HTMLElement DoOnPageFinishActions(RequestData requestData, String hostname)
	{

		Object[] oa;
		Object o;
		IPageActions iActions;
		HTMLComment comment;
		HTMLElementCollection result = new HTMLElementCollection();
	
		oa = (Object[]) requestData.m_Application.GetCachingManager().GetEntry("PageActions");

		//	ON PAGE FINISH
		if (oa != null)
		{
			for (int x = 0; x < oa.length; x++)
			{
				o = oa[x];
		
				log.Debug("DoOnPageFinishActions() - About to call OnPageFinish on an: " + o);
			
				if(o != null)
				{
					iActions = (IPageActions) o;
					comment = iActions.OnPageFinish(requestData.m_asSpace, requestData.m_asSpace.GetUserSession(), hostname);
					if (comment != null)
					{
						result.AddInnerHTMLElement(comment);
					}
				 }
			}
		}

		return result;
	}

	//////////////////////////////////
	// Private internal helper methods
	//////////////////////////////////

	/**
	 * This method attempts to get a localized string.
	 * 
	 * @return String
	 */
	private String GetString(int _nStringID, String _strResourceFileName, RequestData tempData)
	{
		String strLanguage = XPConvert.ToString(PersonalSettingsHelper.GetPersonalSettingValue
			(PersonalSettingsHelper.LANGUAGE, tempData.m_smSandbox, tempData.m_Application));

		XPResourceManager rm = (XPResourceManager) tempData.m_Application.GetCachingManager()
			.GetEntry("ResourceManager");

		return rm.GetString(_nStringID, _strResourceFileName, strLanguage);
	}
	
	private boolean IsCommonOpenerRedirectingTo(String strToSpaceName, RequestData tempData) 
	{
		if (tempData.GetSpaceName().equals(BaseOpenerControl.STR_MVC_CLASS_NAME))
		{
			// Check for query string "name=[content of strToSpaceName var]"
			XPHashtable htQS = tempData.GetQueryString();
			if (null != htQS)
			{
				String[] paramValues = (String[])htQS.GetElement(UIActivitySpaces.QS_ACTIVITY_SPACE_NAME);
				if ((null != paramValues) && (paramValues.length>0))
				{
					String sSpaceNameOpened = paramValues[0];
					if ((null != sSpaceNameOpened) && sSpaceNameOpened.equals(strToSpaceName))
					{
						return true;
					}
				}
			}
		}
		
		return false;
	}
	
	/**
	 * Helper method that creates a new AActivitySpace and stores
	 * it on the tempData object.
	 *
	 * This method should be thread safe for multiple
	 * simultaneous requests per user if tempData
	 * has a null m_smCache variable (this is the case
	 * when this method is called from the HandleGatewayRequest
	 * method.
	 *
	 * @param RequestData the data for this HTTPRequest
	 */
	public void CreateSpace(RequestData tempData)
	{	 
		// Create new space
		AActivitySpace newSpace = ASManager.INSTANCE.GetSpace(tempData.GetSpaceName());

		if (newSpace == null)
		{			
			log.Error("Could not create space named " + tempData.GetSpaceName());			

			String strError = GetString(98, "ptmsgs_infrastructure", tempData) // _ui_CreateSpaceError
				+ tempData.GetSpaceName();
			UIError myError = new UIError(UIErrorLevel.FRIENDLY,
					UIErrorScope.CURRENT_USER,
					strError, strError);

			myError.SetDisplayOnSeparatePage(true);
			myError.SetImmediateClear(true);

			UIErrorHandler.RaiseError(myError, tempData.m_smSandbox,
				tempData.m_Application);

			return;
		}
		
		// COM Recycling code, if the process recycled we want to migrate current session to new process.
		// Makes a new session that will be created in the new com process and swap out the old session. 
		Object userSession = LoginHelper.INSTANCE.HandleCOMProcessRecycled(tempData.GetUserSession());
		if ( null != userSession ) {
			// replace the current session with a new one in the new com process.
			tempData.SetUserSession(userSession);	
		} 

		// Store the space and initialize it with the generated cache key
		if (tempData.m_smCache == null)
		{
			// This is used for gatewayed spaces
			newSpace.SetSpaceID(ASCache.SPACEID_INVALID);
		}
		else
		{
			// We need to check here to make sure that spaces that claim to
			// be subspaces have a valid parent space that is still in the cache,
			// or the invalid spaceID key so that the cache can discount the space.
			// Otherwise we can corrupt the ASCache.  We need to leave the
			// sub-space key in the URL for spaces launched from the gateway, otherwise
			// we will not be able to determine if they should be http or https.
			if ((null != tempData.m_sSubSpaceKey) &&
				(null != tempData.m_sParentID) &&
				(null == tempData.m_smCache.GetAttribute(tempData.m_sParentID)) &&
				(!ASCache.SPACEID_INVALID.equals(tempData.m_sParentID)))
			{
				// If they do not, then they should not be subspaces.
				tempData.m_sSubSpaceKey = null;

				log.Warn("Space named " + tempData.GetSpaceName() +
					" did not have a valid parent space; Removing subspace key.");				
			}

			newSpace.SetSpaceID(tempData.m_smCache.SetAttribute(newSpace));
			tempData.m_xpmCacheList.PutElement(newSpace.GetSpaceID(),
				new CacheListItem(tempData.GetSpaceName(),
					((tempData.m_sSubSpaceKey != null) ? true : false),
					tempData.m_sParentID));
		}

		newSpace.SetSubSession(tempData.m_smSandbox);
		newSpace.SetPersistentSubSession(tempData.m_smPersistentSandbox);
		newSpace.SetParentSpace(tempData.m_sParentName, tempData.m_sParentID);
		newSpace.SetSubSpaceKey(tempData.m_sSubSpaceKey);
		newSpace.SetApplication(tempData.m_Application);

		newSpace.SetIsASSecured(ConfigHelper.GetIsSpaceSecure(tempData.m_xpRequest.IsSecureConnection()));

		newSpace.SetUserSession(tempData.GetUserSession());

		newSpace.SetCurrentRequestURL(tempData.GetRequestFullURL());
		newSpace.SetCurrentHTTPRequestResponseObjects(tempData.m_xpRequest, tempData.m_xpResponse);
		newSpace.SetCurrentFormData(tempData.GetQueryString());

		newSpace.Init();
		
		if (tempData.m_smCache == null)
		{
			if ( log.IsDebugEnabled() ) {
				log.Debug("Created space " + tempData.GetSpaceName() + " with key " +
					newSpace.GetSpaceID() + " (not stored in cache).");
			}
		}
		else
		{
			if ( log.IsDebugEnabled() ) {
				log.Debug("Created space " + tempData.GetSpaceName() +
					", stored in AS cache with key " + newSpace.GetSpaceID() +
					".");
			}
		}		

		tempData.m_asSpace = newSpace;
		tempData.m_sSpaceID = newSpace.GetSpaceID();
		tempData.m_sParentName = newSpace.GetParentSpaceName();
		tempData.m_sParentID = newSpace.GetParentSpaceID();
	}

	/**
	 * Tries to get a space from the cache by name, or, if it
	 * can't find it, creates it.
	 *
	 * @param RequestData the temporary data for the request.
	 */
	private void GetOrCreateSpace(RequestData tempData)
	{
		// Try to find the space if it is cached
		IXPEnumerator spaceKeys = tempData.m_xpmCacheList.GetKeys();

		while (spaceKeys.MoveNext())
		{
			if (tempData.GetSpaceName().equals(((CacheListItem) tempData.m_xpmCacheList.GetElement(
							spaceKeys.GetCurrent())).m_strName))
			{
				tempData.m_asSpace = (AActivitySpace) tempData.m_smCache.GetAttribute((String) spaceKeys.GetCurrent());
				tempData.m_sSpaceID = tempData.m_asSpace.GetSpaceID();
				tempData.m_sParentName = tempData.m_asSpace.GetParentSpaceName();
				tempData.m_sParentID = tempData.m_asSpace.GetParentSpaceID();

				break;
			}
		}

		// Create a new space if could not retrieve the cached space.
		if (tempData.m_asSpace == null)
		{
			// Create new space
			CreateSpace(tempData);
		} else
		{
			tempData.m_asSpace.SetCurrentRequestURL(tempData.GetRequestFullURL());
			tempData.m_asSpace.SetCurrentHTTPRequestResponseObjects(tempData.m_xpRequest, tempData.m_xpResponse);
			tempData.m_asSpace.SetCurrentFormData(tempData.GetQueryString());
		}
	}
	
	///////////////////////
	// Cache helper methods
	///////////////////////

	/**
	 * Attempt to find the cached activity space.
	 *
	 * @param RequestData the temporary data for the request.
	 */
	private void LocateActivitySpace(RequestData tempData)
	{
		// Try to find previously cached AActivitySpace using ID
		if ((tempData.m_sSpaceID != null) && (tempData.GetSpaceName() != null))
		{
			tempData.m_asSpace = (AActivitySpace) tempData.m_smCache.GetAttribute(tempData.m_sSpaceID);

			if (tempData.m_asSpace == null)
			{
				
					log.Info("Cached space with name " + tempData.GetSpaceName() +
						" and ID " + tempData.m_sSpaceID +
						" no longer in cache.");
				
			}
			else
			{
				if (!tempData.GetSpaceName().equals(tempData.m_asSpace.GetName()))
				{		
					log.Error("Cached space with name " +
						tempData.m_asSpace.GetName() + " and ID " +
						tempData.m_sSpaceID +
						" does not match name given in QS: " +
						tempData.GetSpaceName());
					

					tempData.m_asSpace = null;
				}
				else 
					if ( log.IsDebugEnabled() ) {
						log.Debug("Retrieved cached space with name " +
							tempData.m_asSpace.GetName() + " and ID " +
							tempData.m_asSpace.GetSpaceID());
					}
				
			}
		}

		// Try to find previously cached AActivitySpace without using ID
		if ((null == tempData.m_asSpace) && tempData.m_bGetCacheNoID && (tempData.GetSpaceName() != null))
		{
			IXPEnumerator spaceKeys = tempData.m_xpmCacheList.GetKeys();

			tempData.m_asSpace = null;

			while (spaceKeys.MoveNext())
			{
				if (tempData.GetSpaceName().equals(((CacheListItem) tempData.m_xpmCacheList.GetElement(
								spaceKeys.GetCurrent())).m_strName))
				{
					tempData.m_asSpace = (AActivitySpace) tempData.m_smCache.GetAttribute((String) spaceKeys.GetCurrent());

					break;
				}
			}

			
			if (tempData.m_asSpace == null)
			{
				if ( log.IsDebugEnabled() ) {
					log.Debug("Cached space with name " + tempData.GetSpaceName() +
						" not in cache.");
				}
			}
			else
			{
				if ( log.IsDebugEnabled() ) {
					log.Debug("Retrieved cached space with name " +
						tempData.m_asSpace.GetName() + " and ID " +
						tempData.m_asSpace.GetSpaceID() + " without using ID.");
				}
			}			
		}

		if (tempData.m_asSpace != null)
		{
			tempData.m_asSpace.SetCurrentRequestURL(tempData.GetRequestFullURL());
			tempData.m_asSpace.SetCurrentHTTPRequestResponseObjects(tempData.m_xpRequest, tempData.m_xpResponse);
			tempData.m_asSpace.SetCurrentFormData(tempData.GetQueryString());

			tempData.m_sSpaceID = tempData.m_asSpace.GetSpaceID();
			tempData.m_sParentName = tempData.m_asSpace.GetParentSpaceName();
			tempData.m_sParentID = tempData.m_asSpace.GetParentSpaceID();
		}
	}

	/**
	 * 
	 *
	 * @param myRedirect 
	 * @param tempData 
	 * 
	 * @return myRedirect the last redirect
	 */
	private Redirect HandleRedirectChain(Redirect myRedirect,
		RequestData tempData) throws UnauthorizedAccessException
	{
		// tempData.m_asSpace can be null if we have an OnAfterLogin redirect
		// while logging back in as guest user

		boolean bSameSpace;
		IControl myControl		    = null;

		// Handle internal server redirects
		while ((myRedirect != null) && (!myRedirect.GetIsHTTPRedirect()))
		{
			bSameSpace = true;

			String strSpaceName = null;
			String strSpaceID = null;

			if (null != tempData.m_asSpace)
			{
				strSpaceName = tempData.m_asSpace.GetName();
				strSpaceID = tempData.m_asSpace.GetSpaceID();
			}

			// Handle Space redirect
			if (!myRedirect.GetSpaceName().equals(strSpaceName) || // space name different
					((myRedirect.GetSpaceID() != null) && // space ID exists in redirect and
					(!myRedirect.GetSpaceID().equals(strSpaceID)))) // space ID different
			{
				if (null != tempData.m_asSpace)
				{
					// Clean up the temporary data on the old space
					tempData.m_asSpace.CleanupTempData();
				}

				// Clean out old information and set new information
				tempData.SetRedirect(myRedirect);

				bSameSpace = false;

				boolean bIsErrorPageRequired = Interpreter.IsErrorPageRequired(tempData);
				
				Redirect secureRedirect = CheckSecureActivitySpace(tempData);

				// Don't get the error space from the cache.
				// Otherwise it will break.  Must create from scratch
				// each time.
				if (!bIsErrorPageRequired)
				{
					// Try to get the Activity Space from the cache
					LocateActivitySpace(tempData);
				}

				// Create new space if could not retrieve a cached space.
				if (tempData.m_asSpace == null)
				{
					// Create a new space
					CreateSpace(tempData);
				}

				// Fix bug 46308 - In security mode 1 with no logout PEI's, a NullPointerException 
				// occurs on MVCController.DoOnPageFinishActions. 
				// WilliamA - Move the throw of UnauthorizedAccessException to after locating
				// or creating the activityspace so that requestData.m_asSpace is not null,
				// which is used by the OnPageFinish PEIs. Otherwise user session and space will 
				// be both null inside the OnPageFinish PEIs.
				if (secureRedirect!=null)
				{
					// Switched security modes through a 302 redirect
					throw new UnauthorizedAccessException(secureRedirect);
				}				

					log.Info("Switching space to " + tempData.GetSpaceName() +
						" from Control Redirect.");
			}

			tempData.m_sControl = myRedirect.GetControl();

			tempData.m_sPage = myRedirect.GetDisplayPage();

			boolean bAccessAllowed   = tempData.m_asSpace.CheckBasicAccess(tempData.m_sPage,
					tempData.m_sControl, bSameSpace);
			boolean bIsErrorRedirect = myRedirect.GetIsErrorRedirect();

			// Check whether user has rights to see this page
			if (bIsErrorRedirect || !bAccessAllowed)
			{
				if (bIsErrorRedirect)
				{
					log.Info("User does not have rights for operation.  Redirecting from page " +
						tempData.m_asSpace.GetName() + ".");
				}
				else
				{
					log.Info("User does not have rights to see redirected page " +
						tempData.m_asSpace.GetName() + ".  Redirecting.");
				}
				
				throw new UnauthorizedAccessException();
			}

			// Get Control
			myControl = tempData.m_asSpace.GetControl(tempData.m_sControl);

			// control name as null implies default control, which not all spaces have
			if ((myControl == null) && (tempData.m_sControl != null))
			{			
				log.Error("Couldn't find control from redirect: " +
					tempData.m_sControl);

				myRedirect = null;
			}
			else if (myControl != null)
			{			
				log.Info("Executing redirect to control " + tempData.m_sControl + ".");
	
				// Special case for IHTTPControl
				if (myControl instanceof IHTTPControl)
				{
					((IHTTPControl) myControl).SetHTTPItems(tempData.m_xpRequest,
						tempData.m_wdPageData);
				}

				myRedirect = myControl.CheckActionSecurityAndExecute(myRedirect.GetControlArguments());

				// Special case for ILoginControl
				if (myControl instanceof ILoginControl)
				{
					Redirect tempRedirect = null;
					
					tempRedirect = LoginHandlers.DoLogin(myControl, tempData);
					
					if( tempRedirect != null )
					{
						//	If the login action returned a valid redirect object
						//	then we'll use this instead of the redirect provided
						//	by the CheckActionSecurityAndExecute method.  This
						//	is because DoTasksAfterLogin() may be implemented
						//	to return a redirect, allowing clients to modify this
						//	behavior without modifying our code.
												
						myRedirect = tempRedirect;	
					}

					if (myRedirect != null)
					{
						// The login space has not been updated with the new user session yet,
						// so this needs to be updated manually.
						
						if (!myRedirect.GetIsExternalURL())
						{
							myRedirect.AddControlArgument(ASConstants.USERID_INPUT,
								LoginHelper.INSTANCE.GetUserSessionID(tempData.GetUserSession()));
						}
						else
						{
							// JF- If this is an external redirect, append the session ID to the URL
							String sURL = myRedirect.GetExternalURL();
							
							// WilliamA - Bug 50994 and 50863 (Login PEI redirecting to external URL fails)
							// Limit the session ID QS param addition to "internal" URL only 
							String portalURL = ConfigHelper.GetMainServerApplicationURL(false);
							portalURL = XPStringUtility.Substring(portalURL, "http://".length());
							if (-1 != XPStringUtility.IndexOf(sURL, portalURL))
							{
								if (!XPStringUtility.EndsWith(sURL, "&"))
								{
									sURL += "&";
								}
								sURL += ASConstants.USERID_INPUT + "=" + LoginHelper.INSTANCE.GetUserSessionID(tempData.GetUserSession());
								myRedirect = new Redirect();
								myRedirect.SetLinkToExternalURL(sURL);								
							}
							
						}
					}
				}
			}
			else
			{
				myRedirect = null;
			}

			// Handle DisplayPage redirect
			if ((tempData.m_sPage != null) && (!"".equals(tempData.m_sPage)))
			{
				tempData.m_dpNextDisplay = tempData.m_asSpace.GetPage(tempData.m_sPage);
			}
		}

		return myRedirect;
	}

	/**
	 * JF- Checks if we need to do a 302 Redirect to the Activity Space in the request data.
	 * This ignores QS args from the request (as they shouldn't be relevant here).
	 * 
	 * WilliamA - Modify this to return a Redirect object instead of sending 
	 * a HTTP redirect directly.
	 *
	 * @param RequestData all of the data associated with this request
	 *
	 * @return a Redirect object if the Activity Space is a secure one and the  
	 * security mode is 1 and we are not on secure connection. Otherwise, null
	 * is returned.
	 */
	private Redirect CheckSecureActivitySpace(RequestData tempData)
	{	
		int nSecurityMode = ConfigHelper.GetSecurityMode();
		
		if (ConfigVarPack.SECURITY_MODE_SSL_CRITICAL_AS_ONLY == nSecurityMode)
		{
			// JF- If there is an external redirect in tempData, we need to do a 302 redirect to it
			if (tempData.GetHasExternalRedirect())
			{
				// -- tempData.m_xpResponse.Redirect(tempData.GetRedirect().GetExternalURL());
				return tempData.GetRedirect();
			}
			
			SecureASVarPack vpSecureASs = (SecureASVarPack) tempData.m_Application.GetVarPackManager()
				.GetVariablePackage(SecureASVarPack.VARPACK_ID);

			if ((null != tempData.GetRootSpaceName()) 
				&& !tempData.GetRootSpaceName().equals("") 
				&& vpSecureASs.IsSpaceSecured(tempData.GetRootSpaceName()))
			{
				if (!tempData.m_xpRequest.IsSecureConnection())
				{
					// Use the redirect, because it doesn't include QS args from the
					// request, which we don't want here.
					Redirect temp = tempData.GetRedirect();
	
					String sURL = ConfigHelper.GetApplicationBaseURL(ConfigHelper.
						GetNormalizedBaseURL(tempData.m_xpRequest.GetRequestURL()),	true);
					sURL += temp.GetQueryString();
					sURL = tempData.AppendRedirectURLTo302URL(temp.GetSpaceName(), sURL);						 

					Redirect secureRedirect = new Redirect();
					secureRedirect.SetLinkToExternalURL(sURL);
					secureRedirect.SetIsHTTPRedirect(true);
					
					if (LoginHandlers.LOGINSPACE.equals(temp.GetSpaceName()))
					{
						// JF- Special flag to remember that we need to check the auto login cookie
						// when the request comes back
						// (In mode 1, the auto login cookie is secure and http pages don't have access to it, 
						// we need to make sure the auto login still happens when we redirect to the login page)
						
						// WilliamA - "ModeOneRedirectToLogin" may no longer be needed if we can
						// use the LoginHelper.GetIsGuestUser() to indicate guest or not in the
						// Authenticator module.
						tempData.m_smSandbox.SetAttribute("ModeOneRedirectToLogin", "");
					}
					
					return secureRedirect;
				}
				
				// JF- We don't check if the space is not secure and the connection is secure 
				// because if it's a sub space we might have force a secure connection on a 
				// non secure space (see bug # 20486 for more info)
			}
		}
		
		return null;
	}
	
	public IDisplayPage GetFinalDisplayPage(RequestData tempData)
	{
		Redirect secureRedirect = CheckSecureActivitySpace(tempData);
		if (secureRedirect!=null)
		{
			throw new UnauthorizedAccessException(secureRedirect);
		}

		// See if we need to get the new page
		if ((tempData.m_asSpace == null) ||
			(!tempData.m_asSpace.GetName().equals(tempData.GetSpaceName())))
		{
			GetOrCreateSpace(tempData);
		}

		// If there was no redirect to a IDisplayPage ...
		if (tempData.m_dpNextDisplay == null)
		{
			if (tempData.m_sPage != null)
			{
				// Use optional page argument
				return tempData.m_asSpace.GetPage(tempData.m_sPage);
			}
			else
			{
				// otherwise get default Display page
				return tempData.m_asSpace.GetCurrentPage();
			}
		} else 
		{
			return tempData.m_dpNextDisplay;
		}
	}

	// The following method is for bug 42543: forward port Pfizer fixes in Ellington to Martok.
	/**
	 * Returns the second redirect URL (ASConstants.REDIRECT_AFTER_LOGIN_SECONDARY) from the QS. 
	 * (Returns null if it was not there.)
	 * 
	 * @return String
	 */
	public static String GetSecondaryRedirectURLFromQS(RequestData rq)
	{
		String sRedirectURL = null;
		
		try
		{
			XPHashtable htArgs = rq.GetQueryString();
	    	
			if (htArgs.ContainsKey(ASConstants.REDIRECT_AFTER_LOGIN_SECONDARY))
			{
				// JF- Get the redirect URL from the QS and make sure it looks like a valid URL
				String[] aData = (String[]) htArgs.GetElement(ASConstants.REDIRECT_AFTER_LOGIN_SECONDARY);
				 
				sRedirectURL = aData[0];
				
				if (!XPStringUtility.StartsWith(sRedirectURL, "http"))
				{
					sRedirectURL = null;
				}
			}
		}
		catch (Exception e)
		{
			log.Error(e, "An unexpected error occured when trying to retrieve the *secondary* redirect URL (login PEI) from the Query String.");
		}
    	
		return sRedirectURL;
	}
		
	/*
	public static void ClearLoginOccurredCookiePresent( IXPResponse response )
	{
		// Not setting an expires value on a cookie uses the default behaviour in both
		// Java and .NET (deleted when browser is closed).
		// Perhaps this should delete the cookie (0 in Java, time in the past on .NET)
		XPCookie cookie = new XPCookie(LoginHandlers.LOGIN_OCCURED_COOKIE_NAME,
				XPConvert.ToString(false));
		
		cookie.SetPath("/");
	
		response.AddCookie( cookie );
	}
	*/
}
