package com.plumtree.uiinfrastructure.interpreter.filter;

import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.interpreter.*;

/**
 * This filter will load up the environment data for the expression engine 
 * and run the expression engine. It will figure out the right subportal to 
 * use for this request.
 * 
 * The actual implementation of the expression engine filter is in the 
 * portaluiinfrastructure layer (i.e., class
 * portaluiinfrastructure.condition.interpreterfilter.ExpressionEngineFilter).
 * This class is created to avoid mixing of dependency between uiinfrastructure
 * and portaluiinfrastructure layers. The Expression Engine component is part of
 * portaluiinfrastructure layer, while interpreter filter is part of 
 * uiinfrastructure layer. Hence, static initializer is used in this class 
 * to init the actual implementation that calls and runs the expression engine. 
 * 
 * @author William Adjie-Winoto
 */
public class ExpressionEngineFilter extends BaseInterpreterFilter {

	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.login.LoginHelper");
		
	/** The class that implements the actual logic for loading and running
	 *  the expression engine. */
	private static IInterpreterFilter m_Filter = null;

	/** Whether or not this class has already been initialized. */
	private static boolean m_bInitialized = false;

	/**
	 * This method load the necessary environment data and run the 
	 * Expression Engine for set-GuestLogin action. 
	 */
	public boolean PreFilter(RequestData requestData) {
		
		return RunExpressionEngine(requestData);
	}

	/**
	 * Static initialization method.  Use this method to set the actual
	 * class that implements the Expression Engine interpreter filter
	 * logic.
	 * 
	 * @param IInterpreterFilter the filter implementation
	 */
	public static final void StaticInit(IInterpreterFilter filter)
	{
		if (m_bInitialized)
		{
			log.Error("ExpressionEngineFilter StaticInit attempted twice.  Ignoring second attempt.");	

			return;
		}

		m_Filter = filter;
		m_bInitialized = true;
	}
	
	/**
	 * Run the Expression Engine interpreter filter to load up
	 * the environment data for the engine and run the expression
	 * engine. It will figure out the right subportal to use for
	 * this request.
	 * 
	 * @param requestData
	 * @return
	 */
	public static boolean RunExpressionEngine(RequestData requestData)
	{
		if (null != m_Filter)
		{
			return m_Filter.PreFilter(requestData);
		}
		
		return false;
	}
}
