package com.plumtree.uiinfrastructure.interpreter.filter;

import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.application.AppWarmUp;
import com.plumtree.uiinfrastructure.interpreter.*;
import com.plumtree.uiinfrastructure.interpreter.filter.utils.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.util.*;
import com.plumtree.uiinfrastructure.util.XPProcess;
import com.plumtree.openkernel.config.*;
import com.plumtree.openfoundation.util.*;

/**
 * A filter to handle ping requests.  Clients can call this
 * URL to find out if the Portal thinks it is healthy.
 * 
 * @author MichaelS
 */
public class HealthFilter extends BaseInterpreterFilter {
	
	public static final String PT_HEALTH = "PT_HEALTH";
	public static final int PT_DEFAULT_ERROR_CODE = 503;

	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.interpreter.filter.HealthFilter");
	
	private static int m_siErrorResponseCode = PT_DEFAULT_ERROR_CODE;
	private static int m_siMaxVM_MB = 0;
	private static int m_siAttemptToAllocateObjects = 0;
	private static boolean m_sbStaticsInitialized = false;
	private static long m_maxVirtMem = 0L;
	
	// messages
	private static String m_exceedMemory;
	
	private static String m_goodMemory; 
		
	/**
	 * MES- This method detects if the incoming request is a health request.
	 * If it is, it will take over the processing of the request, and return
	 * a boolean true to indicate that Interpreter shall not process the
	 * request further. 
	 */
	public boolean PreFilter(RequestData requestData)
	{
		//MES- NOTE: This code is supposed to report the health of the 
		//	portal.  Therefore, it's expected to work reasonably well even
		//	if the portal is sick (e.g. low on memory, out of disk, whatever.)
		//	To try to help this be true, we try not to allocate any memory we
		//	don't have to (e.g. no logging.)  We also surround the whole thing
		// 	with a try/catch, so "any" error will be handled. 
		try
		{
			//MES- Is this a health request?  We only handle health requests.
			if (isHealthRequest(requestData))
			{
				//MES- This is a ping request, we should handle it
				
				//MES- Initialize our static info, if we haven't done so already
				InitializeIfNeeded();
				
				//MES- Check the virtual memory
				if (checkVMSize(m_siMaxVM_MB))
				{
					//MES- Yup, we're using too much memory
					setResponse(requestData, m_siErrorResponseCode, m_exceedMemory );
					
					return true;
				}
						
				//MES- In this memory check, we'll allocate some memory- if we
				//	get an out of memory exception, then we'll assume that we're...
				//	out of memory!
				try
				{
					//MES- Allocate memory. This should throw an exception if
					//	unable to allocate memory. 
					tryAllocatingMemory(m_siAttemptToAllocateObjects);
				}
				catch (Exception e)
				{
					//MES- We seem to be out of memory (or similar error)!
					//MES- It'd be nice to include the error message from
					//	the exception here, but we don't want to do 
					//	string manipulation, since that might cause another
					//	out of memory error
					setResponse(requestData, m_siErrorResponseCode, m_exceedMemory );
					return true;	
				}
				
				
				//MES- TODO: We might want to check if there have been any System.OutOfMemory type errors thrown by 
				//	the UI- maybe check a global that records that info?
			
				//MES- Everything is cool- either all tests passed, or there were no tests
				//	to perform.
				setResponse(requestData, 200, m_goodMemory );
				return true;
			}

			//MES- We don't want to handle this request, it's not a ping request
			return false;
		}
		catch (Exception e)
		{
			//MES- An unknown error occurred.  We want to do as LITTLE work as possible to
			//	report this- if it's an out of memory exception, we might raise ANOTHER
			//	exception if we try to do any real work.
			setResponse(requestData, 503, null);
			return true;
		}
	}
	
	/**
	 * MES- Initialize static members if they're not already initialized.
	 *	This should be called before relying on values in static members.
	 */
	private void InitializeIfNeeded()
	{
		//MES- Are we already initialized?
		if (!m_sbStaticsInitialized)
		{
			//MES- No, cache the config info.  This way, we don't have to deal
			//	with config info on each hit (and therefore don't have to deal with
			//	any potential errors- e.g. out of memory- on each hit)
			IOKComponent comp = AppWarmUp.GetOKConfigContext().getComponentByType("http://www.plumtree.com/config/component/types/portal/failover");
	
			m_siErrorResponseCode = comp.getSettingValue("ErrorResponseCode", PT_DEFAULT_ERROR_CODE);
			m_siMaxVM_MB = comp.getSettingValue("MaxVmMb", 0); //MES- Default to 0 (which means don't do the check)
			m_siAttemptToAllocateObjects = comp.getSettingValue("AttemptToAllocateObjects", 0); //MES- Default to 0 (which means don't do the check)
		
			//MES- We're initialized now, don't need to do it again
			m_sbStaticsInitialized = true;
			
			
			// DAP init strings
			m_exceedMemory = "<HTML><HEAD><TITLE>Aqualogic Interaction Health Check</TITLE><meta http-equiv='Content-Type' content='text/html; charset=iso-8859-1'></HEAD>"+
				"<BODY style='FONT-SIZE: 9pt; FONT-FAMILY: Verdana, Helvetica, Arial, sans-serif; BACKGROUND-COLOR: #ffffff'>"+
				"<TABLE BORDER='0' WIDTH='95%' CELLPADDING='1' CELLSPACING='0' ID='Table1'><HR style='WIDTH: 95%; COLOR: #999; HEIGHT: 1px; TEXT-ALIGN: left' align='left'><H2 style='FONT-WEIGHT: bold; FONT-SIZE: 18px'>Test Failed</H2>"+
				"<PRE>Test #1 <br>   |<br>   +--<b>Memory</b><br></PRE><HR style='WIDTH: 95%; COLOR: #999; HEIGHT: 1px; TEXT-ALIGN: left' align='left'>"+
				"<DL><DT>error: <B>exceeded system memory limit</B></DT></DL><P><BR></P><HR style='WIDTH: 95%; COLOR: #999; HEIGHT: 1px; TEXT-ALIGN: left' align='left'></TABLE>"+
				"<TABLE style='solid: ' BORDER='1' CELLPADDING='3' CELLSPACING='0' WIDTH='95%' ID='Table2'><TR BGCOLOR='#ccccff' style='BACKGROUND: #dde4d9' width='95%'>"+
				"<TD COLSPAN='2'><H1 style='PADDING-RIGHT: 3px; DISPLAY: inline-block; PADDING-LEFT: 3px; FONT-WEIGHT: bold; FONT-SIZE: 15px; PADDING-BOTTOM: 3px; PADDING-TOP: 3px'>Recommendation Summary</H1></TD></TR>";
			m_exceedMemory += 
				"<TR BGCOLOR='white' style='BACKGROUND: #ffffff' width='95%'><TD ALIGN='right' VALIGN='top' WIDTH='22%'><FONT SIZE='-1'><CODE style=\"FONT-SIZE: 12px; FONT-FAMILY: 'courier new', courier, monospace\">&nbsp;"+
				"<SPAN style='COLOR: #007711'>failure</SPAN></CODE></FONT></TD><TD>An application error occurred on the portal.<P>Virutal Memory limit has been exceeded for this process.</P></TD></TR>"+
				"<TR BGCOLOR='white' style='BACKGROUND: #ffffff' width='95%'><TD ALIGN='right' VALIGN='top' WIDTH='22%'><FONT SIZE='-1'><CODE style=\"FONT-SIZE: 12px; FONT-FAMILY: 'courier new', courier, monospace\">&nbsp;"+
				"<SPAN style='COLOR: #007711'>recommendation</SPAN></CODE></FONT></TD><TD>"+
				"<P>Reduce load or isolate the cause of memory consumption and restart your portal, or increase the current value of MaxVmMb in your settings file.</P>"+
				"</TD></TR></TABLE></TABLE></BODY></HTML>";	
			
			m_goodMemory = 
				"<HTML><HEAD><TITLE>Aqualogic Interaction Health Check</TITLE><meta http-equiv='Content-Type' content='text/html; charset=iso-8859-1'></HEAD>"+
				"<BODY style='FONT-SIZE: 9pt; FONT-FAMILY: Verdana, Helvetica, Arial, sans-serif; BACKGROUND-COLOR: #ffffff'>"+
				"<TABLE BORDER='0' WIDTH='95%' CELLPADDING='1' CELLSPACING='0' ID='Table1'><HR style='WIDTH: 95%; COLOR: #999; HEIGHT: 1px; TEXT-ALIGN: left' align='left'><H2 style='FONT-WEIGHT: bold; FONT-SIZE: 18px'>Test Passed</H2>"+
				"<PRE>Test #1 <br>   |<br>   +--<b>Memory</b><br></PRE><HR style='WIDTH: 95%; COLOR: #999; HEIGHT: 1px; TEXT-ALIGN: left' align='left'>"+
				"<DL><DT>success: <B>within system memory limit</B></DT></DL><P><BR></P><HR style='WIDTH: 95%; COLOR: #999; HEIGHT: 1px; TEXT-ALIGN: left' align='left'></TABLE></BODY></HTML>";
		}
	}
	
	/**
	 * MES- A helper that determines if a request should be handled by this
	 */
	private boolean isHealthRequest(RequestData requestData)
	{
		//MES- Get the URL
		IXPRequest request = requestData.m_xpRequest;
		String strURL1 = request.GetRequestURL();
		String strURL2 = request.GetQueryString();
		
		//MES- NOTE: We allow the "signature" string to be in the
		//	querystring, or in the main URL.  This is for convenience-
		//	trimming out the querystring might require us to make substrings,
		//	etc., which might cause an out of memory exception if the Portal
		//	is not healthy.
		
		//MES- Is this a health request?  They include PT_HEALTH in the URL.
		if (strURL1 != null)
			if (XPStringUtility.IndexOf(strURL1, PT_HEALTH) >= 0)
				return true;
		if (strURL2 != null)
			if (XPStringUtility.IndexOf(strURL2, PT_HEALTH) >= 0)
				return true;
		
		//MES- Not a ping URL
		return false;
	}
	 
	/**
	 * MES- A helper to set response code and text.  The body
	 * is optional- it can be null.
	 */
	private void setResponse(RequestData requestData, int iResponseCode, String sBody)
	{
		//MES- Get the response object
		IXPResponse resp = requestData.m_xpResponse;
		//MES- And set the data
		resp.SetStatus(iResponseCode);
		//MES- Don't set the body if it wasn't passed in
		if (null != sBody)
		{
			resp.Write(sBody);
		}
		
		//MES- Abandon the appserver session, if it exists...
		if (null != requestData.m_SessionManager)
			requestData.m_SessionManager.Abandon(); 
	}
	
	/**
	 * MES- Checks if the virtual memory used by this process is more than should be allowed
	 * @param iMaxVM_MB The maximum megabytes of virtual memory that should be allowed
	 * @return true if the process is using too much memory, false if not or if it cannot
	 * be determined
	 */
	private boolean checkVMSize(int iMaxVM_MB)
	{
		//MES- Should we check at all?  If the max is 0 or less, we're
		//	not checking
		if (0 < iMaxVM_MB)
		{
			//MES- Check that our virtual memory isn't more than the indicated number of megabytes
			long lVM_Bytes = XPProcess.GetCurrentProcess().GetVirtualMemoryBytes();
			
			//MES- Did we get any info?  The call returns 0 if it fails (this call
			//	is not supported on all platforms.)
			if (0 < lVM_Bytes)
			{
				//MES- Are we over memory?
				long lVM_MB = lVM_Bytes / (1024 * 1024);
				if (lVM_MB > (long)iMaxVM_MB)
				{
					//MES- Yup, we're using too much memory
					m_maxVirtMem = lVM_MB; 
					return true;
				}
			}
		}
		
		//MES- Either we're fine, or we don't know, return false
		return false;
	}
	
	
	/**
	 * MES- Allocate the indicated number of objects in memory.  This function does NOT
	 * do error handling, so it will throw an error if the memory cannot be allocated.
	 * @param iAttemptToAllocateObjects the number of objects to allocate
	 */
	private void tryAllocatingMemory(int iAttemptToAllocateObjects)
	{
		//MES- Were we asked to allocate any objects?
		if (0 < iAttemptToAllocateObjects)
		{
			//MES- Allocate an array to hold strong references to memory
			//	we allocate
			Object [] arrObj = new Object[iAttemptToAllocateObjects];
	        
			//MES- Allocate the objects
			for (int iOuter = 0; iOuter < iAttemptToAllocateObjects; ++iOuter)
			{
				//MES- Make an object and shove it into the array
				Object objInner = new Object();
				arrObj[iOuter] = objInner;
			}
	        
	        //MES- We successfully allocated, release all the strong references
	        //	so that they can easily be garbage collected
	        for (int iOuter = 0; iOuter < iAttemptToAllocateObjects; ++iOuter)
	        {
				//MES- Reference the object, to fool the compiler into thinking we
	        	//	care about it- we don't want it to be optimized away!
				Object objInner = arrObj[iOuter];
				arrObj[iOuter] = null;
	        }
	        //MES- We don't HAVE to be explicit about destroying arrObj, but
	        //	let's do it for cleanliness.
	        arrObj = null;
		}
	}
}
