package com.plumtree.uiinfrastructure.interpreter.filter;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.interpreter.*;
import com.plumtree.uiinfrastructure.web.*;

/**
 * A filter to perform a lock on the session. The lock will
 * be released in the PostFilter method. 
 * 
 * @author DonH
 */
public class RequestLockFilter extends BaseInterpreterFilter
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.interpreter.filter.RequestLockFilter");

	/**
	 * This method will perform a lock on the session, since the
	 * activityspace related code is not thread safe.
	 */
	public boolean PreFilter(RequestData requestData) 
	{
		try 
		{
			LockSession(requestData.m_SessionManager);
		}
		catch (XPThreadInterruptedException e)
		{
			// Unable to acquire lock

			log.Info(e, "Unable to acquire lock for user.");

			throw new InterpreterException(99, ResourceConstants.INFRASTRUCTURE, 
				"The server is busy processing your previous request.  Please try again later.", 503);
		}

		return false;
	}

	/**
	 * This method releases the session lock.
	 */
	public void PostFilter(RequestData requestData) 
	{
		UnlockSession(requestData.m_SessionManager);
	}

	/**
	 * Helper method to lock the session.
	 * 
	 * This method is public so the code can be shared with the gateway.
	 * 
	 * @param session The current session manager.
	 */
	public static void LockSession(ISessionManager session)
	{
		// Get the lock from the session
		// NOTE: DonH -- This is potentially not threadsafe, as it's possible
		// the lock returned by GetAttribute could only be partially constructed.
		// See NOTE below.  However, this should be okay because we are in a try/catch
		// block, and all this means is that this request will fail, but the system
		// will continue to operate normally and subsequent requests should work.
		XPLock myLock = null;

		myLock = (XPLock) session.GetAttribute(InterpreterAttributes.LOCKATTRIBUTE);

		if (myLock == null)
		{
			// store the lock on the session if it doesn't exist
			myLock = new XPLock();
			session.SetAttribute(InterpreterAttributes.LOCKATTRIBUTE, myLock);

			// NOTE: DonH -- Double checked locking does not work in Java.
			// http://www.cs.umd.edu/~pugh/java/memoryModel/DoubleCheckedLocking.html
			// There are many complicated reasons, but basically the compiler could
			// reorder things in such a way that we could try to acquire the lock
			// before it has been fully constructed.
			// It's possible that this is not relevant here, but since most JVM's
			// don't even implement the Java Memory Model correctly, better safe than
			// sorry.
			// Therefore, we cannot attempt to get the lock again (in case some has
			// written over it since we created it).  We can, however, use our lock,
			// which is guaranteed to be safe for us.  This means that if 2 simultaneous
			// threads enter this code before the HTTPSession has been initialized, we
			// can get two threads with two different locks accessing the same code.
			// I can't come up with any case where this would happen, so I am going
			// assume that we are safe with this.  All of the code that caches objects
			// on the HTTP session is synchronized, so I believe we should be okay.

			// This is not truly thread safe, but shouldn't be a problem as this will only
			// occur the first time a user hits the portal.  The only common case where we
			// have multiple requests is in frames, and the initial frameset should
			// correctly initialize the lock before the multiple frame requests occur.
		}

		// NOTE: DonH -- We want this lock timeout to be long because we are trying to optimize
		// this for the end user experience, not for server performance or to stop denial of
		// service attacks.
		//
		// If this lock times out on a user, they will see an annoying 'server busy processing
		// your previous request' message that won't endear them to Plumtree.  Then will then
		// have to refresh their page, which may time out again, compounding the problem.
		// We want the user to eventually get a valid page without seeing the server busy
		// message, if at all possible.
		//
		// A sidebar to this is that we can optimize the usage of this lock for the end user
		// experience by cancelling identical page refreshes.  If the user hits their mypage,
		// which takes 30 seconds to load, and then hits refresh several times before the
		// page has had a chance to finish, we know that all of the refreshes between the
		// currently executing page and the last request are not going to be displayed.  This
		// is only true for identical URLs from the same user waiting on the same lock.  This
		// does mean that if a user has two browser windows open on their mypage and they are
		// refreshing both many times, one of the windows could display a server busy message.
		//
		// This can be implemented by having threads register their URL with a per session
		// multi-threaded queue before attempting to get the lock.  Then, when a thread registers,
		// it can check if any of the threads before it are the same as it.  If they are, it can
		// mark those as cancelled.  Then, when one of those threads gets the lock, it can check
		// its status and return a server busy message rather than trying to display the page.
		// When the last request (which has not been cancelled) finally gets the lock, it will
		// remove itself from the queue and display the requested page.

		// Make sure we release this lock later.
		// Attempt to get a lock.  This will throw an exception if it fails or cannot obtain
		// the lock.
		myLock.Attempt(60000);
	}

	/**
	 * A helper method to unlock the session.
	 * 
	 * This method is public so the code can be shared with the gateway.
	 * 
	 * @param session The current session manager.
	 */
	public static void UnlockSession(ISessionManager session)
	{
		XPLock myLock = (XPLock) session.GetAttribute(InterpreterAttributes.LOCKATTRIBUTE);

		if (myLock != null)
		{
			// NOTE: If an exception occurred when we were locking the lock,
			// this may double release the lock, which should be fine.
			myLock.Release();
		}	
	}
}