package com.plumtree.uiinfrastructure.interpreter.filter;

import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.activityspace.utils.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.interpreter.*;
import com.plumtree.uiinfrastructure.interpreter.filter.utils.*;
import com.plumtree.uiinfrastructure.web.*;

/**
 * A filter to check if there are too many requests queued up on 
 * this session already. If it is, the PreFilter method will thrown an
 * InterpreterException with message: The server is busy processing your 
 * previous request. Otherwise, the PreFilter method will put this request 
 * into the request queue. 
 * 
 * @author Don Hayler
 * @author WilliamA  (Writing this new class)
 */
public class RequestQueueFilter extends BaseInterpreterFilter {

	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.interpreter.filter.RequestQueueFilter");

	/**
	 * This method prevents too many request on the session. It uses a queue
	 * to keep track how many pending request currently on the session.
	 */
	public boolean PreFilter(RequestData requestData) 
	{
		ISessionManager session = requestData.m_SessionManager;

		// NOTE: If a request hangs or the thread dies, the appserver threadpool
		// will be permanently depleted.  How do we deal with this?  Create our
		// own duplicate thread pool?

		// NOTE: To better handle a single user refreshing their page constantly
		// and using up all of the available threads on the app server we need to
		// implement a queue of threads waiting on the lock.  When a new request
		// adds itself to the queue, the queue should check for identical previous
		// requests and cancel them.  When those requests get the lock, they will
		// check their status and return immediately if they have been cancelled.
		// Only 100% identical requests will be cancelled.  This means that if a
		// user refreshes their mypage 5 times, the middle 3 requests will be
		// cancelled, and only the last (relevant) request will be processed.

		// Check if there are too many requests queued up on this session
		// and return immediately if there are.

		ASQueue myQueue = (ASQueue)session.GetAttribute(InterpreterAttributes.QUEUEATTRIBUTE);
		boolean bRegisterSuccessful = false;

		try
		{
			if (myQueue == null)
			{
				// store the queue on the session if it doesn't exist
				myQueue = new ASQueue(requestData.m_Application);
				session.SetAttribute(InterpreterAttributes.QUEUEATTRIBUTE, myQueue);
			}

			bRegisterSuccessful = myQueue.Register();
		} catch (Exception e)
		{
			
			log.Error(e, "Unable to check request queue length for this user.");			

			// TODOCR DONH: If we cannot access the queue, should we fail, or
			// just proceed as if it worked?
			bRegisterSuccessful = false;
		}

		if (!bRegisterSuccessful)
		{			
			log.Info("Too many identical requests detected for this user, returning server busy message.");

			// Server temporarily unavailable HTTP status code.

			throw new InterpreterException(99, ResourceConstants.INFRASTRUCTURE, 
			"The server is busy processing your previous request.  Please try again later.", 503);			
		}

		return false;
	}

	/**
	 * This method removes the current request from the
	 * pending-request queue.
	 */
	public void PostFilter(RequestData requestData) 
	{
		ASQueue myQueue = (ASQueue)requestData.m_SessionManager.GetAttribute(InterpreterAttributes.QUEUEATTRIBUTE);

		if (myQueue != null)
		{
			myQueue.RemoveFromQueue();
		}
	}
}