package com.plumtree.uiinfrastructure.login;

import com.plumtree.openlog.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.web.*;

/**
 * This class is used to access some of the application specific data that would
 * normally be available through the AActivitySpace.
 */
public class ApplicationData
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.login.ApplicationData");

	private IXPRequest m_Request;
	private IWebData m_WebData;
	private IApplication m_Application;
	private ISessionManager m_SubSession;
	private ISessionManager m_PersistentSubSession;

	public ApplicationData(IXPRequest _Request, IWebData _WebData,
		IApplication _Application, ISessionManager _SubSession, 
		ISessionManager _PersistentSubSession)
	{
		m_Request = _Request;
		m_WebData = _WebData;
		m_Application = _Application;
		m_SubSession = _SubSession;
		m_PersistentSubSession = _PersistentSubSession;
	}

	/**
	 * This method returns the application.
	 * 
	 * @return IApplication
	 */
	public IApplication GetApplication()
	{
		return m_Application;
	}

	/**
	 * This method returns the sub session.
	 * 
	 * This sub session will be cleared on logout,
	 * and therefore is a good place for storing variables
	 * needed for the lifetime of the user session.
	 * 
	 * @return ISessionManager
	 */
	public ISessionManager GetSubSession()
	{
		return m_SubSession;
	}

	/**
	 * This method returns the persistent sub session.
	 * 
	 * This persistent sub session will not be cleared on logout.
	 * You should use the regular SubSession unless you have a 
	 * specific reason to use this persistent sub session.
	 * 
	 * @return ISessionManager
	 */
	public ISessionManager GetPersistentSubSession()
	{
		return m_PersistentSubSession;
	}

	/**
	 * This method adds the specified cookie.
	 * 
	 * @param String the cookie name.
	 * @param String the cookie value.
	 */
	public void SetCookie(String strCookieName, String strCookieValue)
	{
		m_WebData.AddCookie(new XPCookie(strCookieName, strCookieValue));
	}
	
	/**
	 * This method adds the specified cookie.
	 * 
	 * @param _xpCookie the XPCookie to add.
	 */
	public void SetCookie(XPCookie _xpCookie)
	{
	    m_WebData.AddCookie(_xpCookie);
	}

	/**
	 * This method returns the requested cookie.
	 * 
	 * @return String the cookie value. null if cookie was not found.
	 */
	public String GetCookie(String strCookieName)
	{
		if ((strCookieName == null) || ("".equals(strCookieName)))
		{
			return null;
		}

		XPCookie[] ca = m_Request.GetCookies();
		String sCookieVal = null;

		if (ca != null)
		{
			int iNum   = ca.length;
			XPCookie c;

			for (int x = 0; x < iNum; x++)
			{
				c = ca[x];

				
					log.Debug("ApplicationData.GetCookie() examining cookie: " +
						c.GetName());
				

				if (strCookieName.equals(c.GetName()))
				{
					
						log.Debug("Found cookie.");
					

					sCookieVal = c.GetValue();

					return sCookieVal;
				}

			}	//	for each cookie

		}	//	got cookies

		return sCookieVal;
	}

	/**
	 * This method returns the requested header.
	 * 
	 * @return String the header value. null if header was not found.
	 */
	public String GetHeader(String strHeaderName)
	{
		if ((strHeaderName == null) || ("".equals(strHeaderName)))
		{
			return null;
		}

		return m_Request.GetHeader(strHeaderName);
	}
	
	/**
	 * JF- Sets the content language on the WebData.
	 * 
	 * @param _sLocale
	 */
	public void SetContentLanguage(String _sLocale)
	{
		m_WebData.SetContentLanguage(_sLocale);
	}

	/**
	 * This method sets the requested header.
	 * 
	 * @param String the header name
	 * @param String the header value
	 */
	public void SetHeader(String strHeaderName, String strHeaderValue)
	{
		m_WebData.SetHeader(strHeaderName, strHeaderValue);
	}
	
	/**
	 * This method sets the status code on the Response.
	 * 
	 * @param _nStatus Status code
	 */
	public void SetStatus(int _nStatus)
	{
		m_WebData.SetStatus(_nStatus);
	}	
	
	/**
	 * Returns the current request.
	 * 
	 * @return IXPRequest
	 */
	public IXPRequest GetRequest()
	{
		return m_Request;	
	}

	/**
	 * JF- Returns the current request URL.
	 * 
	 * @return String
	 */
	public String GetRequestURL()
	{
		return m_Request.GetRequestURL();	
	}
	
	/**
	 * JF- Returns the current server name from XPRequest.
	 * 
	 * @return String
	 */
	public String GetServerName()
	{
		return m_Request.GetServerName();	
	}
	
	/**
	 * JF- Returns the current port number from XPRequest.
	 * 
	 * @return int
	 */
	public int GetServerPort()
	{
		return m_Request.GetServerPort();	
	}
	
	/**
	 * JF- Returns the value of the paramter in argument.
	 * 
	 * @return String
	 */
	public String GetParameterValue(String _sParamName)
	{
		return m_Request.GetParameterValue(_sParamName);	
	}
}

