package com.plumtree.uiinfrastructure.login;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.activityspace.utils.*;
import com.plumtree.uiinfrastructure.interpreter.*;

/**
 * The authentication module for login-token authentication. It handles both the
 * admin token authentication (where the token comes from the URL query string) 
 * and the remember-my-password token authentication (where the token comes from 
 * the cookie).
 * 
 * @author Don Hayler
 * @author Julie Fournier (Security Modes)
 * @author William Adjie-Winoto (Writing this new class)
 */
public class TokenAuthenticator implements IAuthenticator {

	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.login.TokenAuthenticator");

	private static TokenAuthenticator m_Instance = new TokenAuthenticator();
    
	private TokenAuthenticator()
	{    			
	}
    
	public static TokenAuthenticator GetInstance() {
		return m_Instance;
	}
	
	/**
	 * Performs the login-token authentication. The status of successful/failed
	 * login will be conveyed in the returned LoginResult object. If no token
	 * can be found in the request, this method returns null.
	 * 
	 * @see com.plumtree.uiinfrastructure.login.IAuthenticator#Login(com.plumtree.uiinfrastructure.application.RequestData)
	 */
	public LoginResult Login(RequestData requestData) {
		LoginResult loginResult = null;
		String strToken = null;
		
		log.Info("Inside TokenAuthenticator::Login");
		
		//SE: Bug 54224 - consult all the login tokens we have available
		//	to us.  If any have expired, try the next one.  In particular,
		//	the replication token may have expired - use the remember
		//	my password token instead.
		XPArrayList alTokens = new XPArrayList();
		
		//ES - see if it's using replication token
		XPCookie[] cookieArr = requestData.m_xpRequest.GetCookies();
		for (int i = 0; i < cookieArr.length; i++) {
			if (XPStringUtility.EqualsIgnoreCase(cookieArr[i].GetName(), LoginHelper.REPLICATION_COOKIE_NAME)) {
				strToken = cookieArr[i].GetValue();
				alTokens.Add(strToken);
				log.Info("TokenAuthenticator::Login: Found a rep token: " + strToken);
				break;
			}
		}
		
		// Try admin Query String token login
		strToken = requestData.m_xpRequest.GetParameterValue(LoginHelper.LOGIN_TOKEN_COOKIE_NAME);
		if (null != strToken)
			alTokens.Add(strToken);
					
		// Try remember my password login token login
		if (LoginHandlers.GetIsAutoConnectEnabled(requestData))
		{
			strToken = LoginHelper.INSTANCE.GetAutologinToken(requestData.m_xpRequest);
			if (null != strToken)
			{
				alTokens.Add(strToken);
			}
		}
		
		if (!alTokens.IsEmpty()) 
		{
			// This clears the AS cache and other data.
			LoginHandlers.DoLogout(requestData);
				
			log.Info("Attempting to log in using " + alTokens.GetSize() + " login tokens.");

			//MES- Try each token, if one doesn't work, try the next.  If one DOES
			//	work, we're done.
			for (int iTokenIndex = 0; iTokenIndex < alTokens.GetSize(); ++iTokenIndex)
			{
				strToken = (String)alTokens.GetElement(iTokenIndex);
				
				loginResult = LoginHelper.INSTANCE.AttemptLogin(strToken,
					requestData.m_smSandbox, requestData.m_smPersistentSandbox, requestData.m_Application, 
					requestData.GetRequestURL(), requestData.m_xpRequest, requestData.m_wdPageData);
				
				//MES- If we were successful, break the loop
				if (loginResult.m_bSuccess)
					break;
			}
				
			if (!loginResult.m_bSuccess) 
			{
				// If there was a redirect, then a before login PEI redirected us
				// away before we could login, so it's not an error.
				if (loginResult.m_Redirect == null)
				{
					log.Error("Unable to log in using login token from cookie: " +
						loginResult.m_strError);
					log.Info("Clearing invalid login token in cookie.");			
		
					// We need to clear the invalid cookie here so that we don't get
					// into an infinite redirect in Security Mode 1
					// (https token login fails, login as guest, 302 redirect to http mypage, repeat).
					LoginHelper.INSTANCE.ClearAutologinToken(
						requestData.m_xpRequest.GetRequestURL(), requestData.m_wdPageData);
				}
			}
			
			// JF- Now that we are logged in, checked if there is a redirect URL in the QS
			// (this is in mode 1 only).  This must be done after login, otherwise the
			// redirect from the query string could get overwritten.
			String sRedirectFromQS = requestData.GetRedirectURLFromQS();
				
			if (null != sRedirectFromQS)
			{
				Redirect redirect = new Redirect();
				redirect.SetLinkToExternalURL(sRedirectFromQS);
					
				// WilliamA - We now use LoginResult object as a way to communicate
				// with the Interpreter (instead of modifying the requestData
				// directly)
				// -- requestData.SetRedirect(redirect);
				loginResult.m_Redirect = redirect;
				loginResult.m_bRedirectedBeforeLogin = true;
			}			
		}
		
		return loginResult;
	}

	/**
	 * Login token authentication method does not create challenge.
	 * 
	 * @see com.plumtree.uiinfrastructure.login.IAuthenticator#GetOrSendLoginChallenge(com.plumtree.uiinfrastructure.application.RequestData)
	 */
	public Redirect GetOrSendLoginChallenge(RequestData requestData) {
		// Login Token Auth method does not create challenge
		return null;
	}

	/**
	 * Logout is handled by the ILoginControl in the Login activityspace.
	 * 
	 * @see com.plumtree.uiinfrastructure.login.IAuthenticator#Logout(com.plumtree.uiinfrastructure.application.RequestData)
	 */
	public boolean Logout(RequestData requestData) {
		// Logout is handled by the ILoginControl in the Login AS
		return false;
	}

	/**
	 * Returns a boolean indicating whether a login token exists in the request.
	 * 	 
	 * @see com.plumtree.uiinfrastructure.login.IAuthenticator#ShouldUseAuth(com.plumtree.uiinfrastructure.application.RequestData)
	 */
	public boolean UseThisAuth(RequestData requestData) {
		String strToken = null;

		//ES: Check for replication cookie token
		log.Info("TokenAuthenticator::UseThisAuth: checking if there's a replication token.");
		String strRepToken = null;
		XPCookie[] cookieArr = requestData.m_xpRequest.GetCookies();
		for (int i = 0; i < cookieArr.length; i++) {
			if (XPStringUtility.EqualsIgnoreCase(cookieArr[i].GetName(), LoginHelper.REPLICATION_COOKIE_NAME)) {
				strRepToken = cookieArr[i].GetValue();
				if (null != strRepToken)
				{
					log.Info("TokenAuthenticator::UseThisAuth: Found a replication token: " + strRepToken);
					return true; 
				}
			}
		}
		
		// Try admin Query String token login		
		strToken = requestData.m_xpRequest.GetParameterValue(LoginHelper.LOGIN_TOKEN_COOKIE_NAME);

		if (null != strToken)
		{
			return true;
		}
		else
		{
			// Try remember my password login token login	
			strToken = LoginHelper.INSTANCE.GetAutologinToken(requestData.m_xpRequest);

			if ((null != strToken) && (LoginHandlers.GetIsAutoConnectEnabled(requestData)))
			{
				return true;
			}			
			else
			{			
				return false;
			}
		}
	}

}
