package com.plumtree.uiinfrastructure.opener;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.interpreter.*;
import com.plumtree.uiinfrastructure.opener.types.*;
import com.plumtree.uiinfrastructure.pei.*;
import com.plumtree.uiinfrastructure.statichelpers.*;

/**
 * This control redirects to the appropriate editor, file, etc.
 *
 * @author John Osborne
 */
public class BaseOpenerControl implements IControl
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.opener.BaseOpenerControl");

	public static final String STR_MVC_CLASS_NAME = "Opener";

	/** The AS Cache on the session */
	private ASCache m_smCache;

	/** The parent activity space (opener space) */
	private BaseOpenerAS m_asOwner;

	/**
	 * This method is the gateway to the Common Opener.  We inspect the
	 * URL and depending on what arguments are given, we direct to the 
	 * correct place.  
	 * 
	 * The first step in this process is running any plugins that are
	 * registered in OpenerPlugins.xml.  The plugins will be able to
	 * change the QS arguments as well as specify a redirect.
	 * 
	 * After the plugins are run, we know exactly where we want to go.  
	 * If there's a redirect, we will be going there.  If not, we will
	 * be going to the ClassID/ObjectID pair.
	 * 
	 * Next, we run any PEIs that are registered in OpenerActions.xml.
	 * This is a good place to do tracking because we have already 
	 * deteremined where we want to go.  Optionally, we allow the PEIs
	 * to change flow by returning a redirect.
	 * 
	 * If there's a redirect, we go there.  If not, we do our normal
	 * processing of going to the ClassID/ObjectID via the class type
	 * descriptor.
	 * 
	 * Note that we have changed our QS arguments (Tiny URL project).
	 * i.e. "in_hi_classID" is now "clsID".  However, we will still
	 * support the older format, at least in this release.
	 * 
	 * @param _htArguments These are our QS arguments.
	 * @return Redirect to the appropriate object handler.
	 * @see com.plumtree.uiinfrastructure.activityspace.IControl#Execute()
	 */
	public Redirect CheckActionSecurityAndExecute(XPHashtable _htArguments)
	{	
		try
		{
		    /////////////////////////////
		    // PROCESS CUSTOMIZED CODE //
		    /////////////////////////////

		    // process each dynamically loaded plugin.  the plugins will
		    // have access to and be able to modify the QS arguments.
		    RedirectData redirectData = new RedirectData();
		    redirectData.SetQSArguments(_htArguments);
		    redirectData = ProcessOpenerPlugins(redirectData);
		    _htArguments = redirectData.GetQSArguments();

			// at this point, we know where we want to go.  if a redirect
		    // exists, we will be going there.  if not, we will open the
		    // object via the ClassID/ObjectID pair from the QS arguments.
			int nClassID = HTTPHelpers.GetArgumentAsInt(_htArguments, 
					IOpenerConstants.QS_CLASSID, HTTPHelpers.GetArgumentAsInt(
					_htArguments, IOpenerConstants.STR_QS_CLASSID, -1));	
			int nObjectID = HTTPHelpers.GetArgumentAsInt(_htArguments, 
					IOpenerConstants.QS_OBJECTID, HTTPHelpers.GetArgumentAsInt(
					_htArguments, IOpenerConstants.STR_QS_OBJECTID, -1));

		    // retrieve the classkey.  if the classkey is numeric, that
		    // means we will be opening a PT class type.  if the key is
		    // a string, then we will be opening a UI class type.
		    String strClassKey = HTTPHelpers.GetArgumentAsString(
		        _htArguments, IOpenerConstants.QS_OPEN, null);

			if (null == strClassKey)
			{
				// If we don't have a class key, just use the class ID
				strClassKey = XPConvert.ToString(nClassID);
			}
		    
			// process any on before open PEIs.  since the PEI knows where
			// we are going, it can track data. the PEI will also have one
			// last chance to change flow by specifying a new redirect. we
			// could open this up by allowing them access to change the 
			// QS arguments (like the Plugins), but for now let's limit it.
			Redirect redirect = ProcessOnBeforeOpen(redirectData.GetRedirect(), 
			    nClassID, nObjectID, strClassKey, _htArguments);

			if (null != redirect)
			{
				// if the PEI chooses to redirect, we want
			    // to halt processing and go straight there.
				return redirect;
			}

			////////////////////////////
			//  PROCESS REDIRECT FLOW //
			////////////////////////////

			if (redirectData.HasRedirect())
			{
			    // based on a plugin's interpretation of the 
			    // QS arguments, we are going to a redirect.
			    return redirectData.GetRedirect();
			}

			////////////////////////////////////
			//  PROCESS CLASSID/OBJECTID PAIR //
			////////////////////////////////////

			// get the parent AS.  when we call into the code in the 
			// class type descriptor, we want it to think that we are
			// on the space that we came from - the opener should be
			// transparent.  if there is no parent, like in the case 
			// of the gateway space, then we have to use the Opener.
			AActivitySpace asParent = m_asOwner.GetParentSpace();
			if (null == asParent)
			{
				asParent = m_asOwner;
			}

			// get the class type descriptor manager.
			IClassTypeManager classTypeDescManager = 
				ConfigHelper.GetTypeDescrManager();

			// get the Opener mode from the querystring.  if it  
			// doesn't exist, then use the class default mode.
			int nOpenerMode = HTTPHelpers.GetArgumentAsInt(_htArguments, 
			    IOpenerConstants.QS_MODE, HTTPHelpers.GetArgumentAsInt(
			        _htArguments, IOpenerConstants.STR_QS_OPENER_MODE, 
			        classTypeDescManager.GetDefaultOpenerMode(strClassKey)));				

			// call into the specified class/mode implementation.9
			Redirect ObjectRedirect = classTypeDescManager.GetOpenerRedirect(
			    strClassKey, nOpenerMode, _htArguments, asParent, 
			    (IPTSession) m_asOwner.GetUserSession());
			
			if (!ObjectRedirect.GetStripParentSpace())
			{
				// try to get the parent space name and id attributes from the URL.  
				// if it is not on the URL, try getting the parentspace from the 
				// space itself.  we try the URL first because it may have the most
				// recent information, as opposed to the opener space's parent, 
				// which may just be the first space that linked to the opener.
				String strParentSpaceName = HTTPHelpers.GetArgumentAsString(_htArguments, 
					ASConstants.PARENTSPACENAME, HTTPHelpers.GetArgumentAsString(_htArguments, 
					ASConstants.PARENTSPACENAME_INPUT, m_asOwner.GetParentSpaceName()));
				String strParentSpaceID = HTTPHelpers.GetArgumentAsString(_htArguments, 
					ASConstants.PARENTSPACEID, HTTPHelpers.GetArgumentAsString(_htArguments, 
					ASConstants.PARENTSPACEID_INPUT, m_asOwner.GetParentSpaceID()));
	
				// again, we don't want the parent of whatever we go to to be the Opener 
				// (since it's not a space that we can return to).  we want to set this
				// space to be whatever the user specified, or at the very least whatever
				// space invoked the opener this last time.
				ObjectRedirect.AddControlArgument(ASConstants.PARENTSPACENAME, strParentSpaceName);
				ObjectRedirect.AddControlArgument(ASConstants.PARENTSPACEID, strParentSpaceID);
			}
			else
			{
			    ObjectRedirect.RemoveControlArgument(ASConstants.PARENTSPACENAME);
			    ObjectRedirect.RemoveControlArgument(ASConstants.PARENTSPACEID);
			}

			return ObjectRedirect;
		}
		//  DCA - there is also a System.UnauthorizedAccessException in dotNet
		catch (com.plumtree.uiinfrastructure.interpreter.UnauthorizedAccessException uae) 
		{
			throw uae;
		}
		catch (Exception ex)
		{
			String errorMessage = "The Common Opener could not open this object. ";

			// the interpreter will show the standard server error page, and 
			// if the user views source, they will see a detailed error. 
			throw new XPException(errorMessage, ex);			
		}
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new BaseOpenerControl();
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IControl#Init(IModel, AActivitySpace)
	 */
	public void Init(IModel _model, AActivitySpace _asSpace)
	{
		m_smCache = (ASCache) _asSpace.GetSubSession().GetAttribute(ASConstants.SANDBOXASCACHE);
		m_asOwner = (BaseOpenerAS) _asSpace;
	}
	
	/**
	 * This method calls the IOpenerPlugins ProcessPlugIn() method.
	 * 
	 * @see com.plumtree.portaluiinfrastructure.opener.IOpenerPlugins.
	 * @param redirectData
	 * @return
	 */
	private RedirectData ProcessOpenerPlugins(RedirectData redirectData)
	{
		try
		{
			Object myObject;
			Object[] myObjectArray;

			IOpenerPlugins iPlugin;

			myObjectArray = (Object[]) m_asOwner.GetApplication().
				GetCachingManager().GetEntry("OpenerPlugins");
	
			if (null != myObjectArray)
			{
				for (int x = 0; x < myObjectArray.length; x++)
				{
					myObject = myObjectArray[x];
	
					if (null != myObject)
					{
						iPlugin = (IOpenerPlugins) myObject;
	
						redirectData = iPlugin.ProcessPlugIn(redirectData,
						    m_asOwner, (IPTSession) m_asOwner.GetUserSession());
							
					    if (redirectData.GetHaltPlugins())
					        return redirectData;
					}
				}
			}
		}
		//  DCA - there is also a System.UnauthorizedAccessException in dotNet
		catch (com.plumtree.uiinfrastructure.interpreter.UnauthorizedAccessException uae) 
		{
			throw uae;
		}
		catch (Exception ex)
		{
			// the interpreter will show the standard server error page, and 
			// if the user views source, they will see a detailed error. 
		    String errorMessage = "Error processing Opener Plugins";
			throw new XPException(errorMessage, ex);
		}
				
		return redirectData;
	}
	
	/**
	 * This method calls the IOpenerActions OnBeforeOpen() method.
	 * 
	 * @see com.plumtree.portaluiinfrastructure.pei.IOpenerActions
	 */
	private Redirect ProcessOnBeforeOpen(Redirect rRedirect,
		int nClassID, int nObjectID, String strClassKey,
		XPHashtable htArguments)
	{		
		try
		{
			Object myObject;
			Object[] myObjectArray;

			Redirect actionReturn;
			IOpenerActions iActions;

			myObjectArray = (Object[]) m_asOwner.GetApplication().
				GetCachingManager().GetEntry("OpenerActions");
	
			if (null != myObjectArray)
			{
				for (int x = 0; x < myObjectArray.length; x++)
				{
				    myObject = myObjectArray[x];
	
					log.Debug("ProcessOnBeforeOpen() - About to call OnBeforeOpen on an: " + myObject);
	
					if (null != myObject)
					{
						iActions = (IOpenerActions) myObject;
	
						// this PEI returns a redirect.  If there are multiple 
						// implementations of this PEI, they will be processed 
						// in the order that they appear.  
						actionReturn = iActions.OnBeforeOpen(nClassID, nObjectID,
							strClassKey, htArguments, rRedirect,
							(AActivitySpace) m_asOwner, m_asOwner.GetUserSession());
							
						if (null != actionReturn)
						{
						    // processing stops when a valid 
						    // redirect is returned, however.
						    return actionReturn;
						}
					}
				}
			}
		}
		catch (Exception ex)
		{
			// the interpreter will show the standard server error page, and 
			// if the user views source, they will see a detailed error. 
		    String errorMessage = "Error processing OnBeforeOpen PEI";
			throw new XPException(errorMessage, ex);
		}

		// return the original redirect
		// if the PEIs don't change it.
		return rRedirect;
	}
}

