package com.plumtree.uiinfrastructure.opener;

import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.login.*;
import com.plumtree.uiinfrastructure.opener.types.classtypes.*;
import com.plumtree.uiinfrastructure.statichelpers.*;

/**
 * This helper object enables client programmers to
 * easily and generically open objects throughout the system.
 */
public class OpenerLinks
{
	public static final int	INT_PARAM_EXCLUDEVALUE	= -1;

	/**
	 * Returns an URL that opens the specified Activity Space.  If you do not 
	 * specify the mode to open the object in, the default mode for that class 
	 * will be used.
	 * 
	 * By default, the Application Base URL and User ID will be retrieved from 
	 * the AS owner.  If you want to use alternate values, or remove the User
	 * ID from the URL, you need to use the base GetOpenerURL method.
	 * 
	 * Additionally, the ParentSpace Name and ID will also be placed onto the 
	 * URL and these too will be retrieved from the AS.  If you want to change 
	 * the location of the Parent or remove it from the URL, you can use the 
	 * overloaded method of the same name.
	 * 
	 * @param _strASName Name of the Activity Space to open.
	 * @param _nOpenerMode Mode to open object in, use OpenerAs.INT_PARAM_EXCLUDEVALUE constan to exclude.
	 * @param _asOwner AS to retrieve information from.
	 * 
	 * @return URL that opens the specified Activity Space.
	 */
	public static final String GetOpenerURLOpenAS(String _strASName, int _nOpenerMode, AActivitySpace _asOwner)
	{
		return GetOpenerURLOpenAS(_strASName, _nOpenerMode, _asOwner,
			_asOwner.GetName(), _asOwner.GetSpaceID(), null);
	}

	/**
	 * Returns an URL that opens the specified Activity Space.  If you do not 
	 * specify the mode to open the object in, the default mode for that class 
	 * will be used.
	 * 
	 * By default, the Application Base URL and User ID will be retrieved from 
	 * the AS owner.  If you want to use alternate values, or remove the User
	 * ID from the URL, you need to use the base GetOpenerURL method.
	 * 
	 * @param _strASName Name of the Activity Space to open.
	 * @param _nOpenerMode Mode to open object in, use OpenerAs.INT_PARAM_EXCLUDEVALUE constan to exclude.
	 * @param _asOwner AS to retrieve information from.
	 * @param _strOverrideParentSpaceName Alternative ParentSpaceName to use, NULL to exclude.
	 * @param _strOverrideParentSpaceID Alternative ParentSpaceID to use, NULL to exclude.
	 * @param _strOptQSArgs Additional arguments to add to the URL, NULL to exclude.
	 * 
	 * @return URL that opens the specified Activity Space.
	 */
	public static final String GetOpenerURLOpenAS(String _strASName, int _nOpenerMode, AActivitySpace _asOwner,
		String _strOverrideParentSpaceName, String _strOverrideParentSpaceID, String _strOptQSArgs)
	{
		XPStringBuilder sbQSArgs = new XPStringBuilder(256);

		HTTPHelpers.AppendQSArgument(sbQSArgs, IOpenerConstants.QS_OPEN, UIActivitySpaces.UICLASSTYPEDESC_CLASSNAME, true);
		HTTPHelpers.AppendQSArgument(sbQSArgs, UIActivitySpaces.QS_ACTIVITY_SPACE_NAME, _strASName);

		if (null != _strOptQSArgs)
		{
			sbQSArgs.Append(_strOptQSArgs);
		}

		if ((null != _strOverrideParentSpaceName) &&
			(null != _strOverrideParentSpaceID))
		{
			// only add space and id if both of them are valid.
			HTTPHelpers.AppendQSArgument(sbQSArgs, ASConstants.PARENTSPACENAME, _strOverrideParentSpaceName);
			HTTPHelpers.AppendQSArgument(sbQSArgs, ASConstants.PARENTSPACEID, _strOverrideParentSpaceID);
		}

		return GetOpenerURL(sbQSArgs.ToString(), _nOpenerMode, _asOwner);
	}

	/**
	 * Returns an URL that opens the specified Activity Space.  If you do not 
	 * specify the mode to open the object in, the default mode for that class 
	 * will be used.
	 * 
	 * This method takes in optional QS arguments.  Therefore no arguments
	 * (i.e. ParentSpaceName/ParentSpaceID), etc will be set for you.  You must
	 * specify all arguments using the QSArgs argument.
	 * 
	 * @param _strASName Name of the Activity Space to open.
	 * @param _strOptQSArgs Arguments to use for opening the object, NULL to exclude.
	 * @param _nOpenerMode Mode to open object in, use OpenerAs.INT_PARAM_EXCLUDEVALUE constant to exclude.
	 * @param _strApplicationBaseURL The Application Base URL.
	 * @param _nUserID User ID to append to the URL, use OpenerAs.INT_PARAM_EXCLUDEVALUE constant to exclude.
	 * 
	 * @return URL that opens the specified object, generically.
	 */
	public static final String GetOpenerURLOpenAS(String _strASName,  String _strOptQSArgs, 
			int _nOpenerMode, String _strApplicationBaseURL, int _nUserID)
	{   
		XPStringBuilder sbQSArgs = new XPStringBuilder(256);

		HTTPHelpers.AppendQSArgument(sbQSArgs, IOpenerConstants.QS_OPEN, UIActivitySpaces.UICLASSTYPEDESC_CLASSNAME, true);
		HTTPHelpers.AppendQSArgument(sbQSArgs, UIActivitySpaces.QS_ACTIVITY_SPACE_NAME, _strASName);

		if (null != _strOptQSArgs)
		{
			sbQSArgs.Append(_strOptQSArgs);
		}

		return GetOpenerURL(sbQSArgs.ToString(), _nOpenerMode, _strApplicationBaseURL, XPConvert.ToString(_nUserID));
	}

	/**
	 * Returns an URL that opens the specified object.  If you do not specify
	 * the mode to open the object in, the default mode for that class will
	 * be used.  Note that you need to provide both the QS name and value pair
	 * in the QSArgs argument (the extended GetOpenerURLXXX methods do this 
	 * for you automatically).    
	 * 
	 * By default, the Application Base URL and User ID will be retrieved from 
	 * the AS owner.  If you want to use alternate values, or remove the User
	 * ID from the URL, you need to use the base GetOpenerURL method.
	 * 
	 * See the base GetOpenerURL method for what else is added to the URL.
	 * 
	 * @param _strQSArgs Arguments to use for opening the object.
	 * @param _nOpenerMode Mode to open object in, use OpenerAs.INT_PARAM_EXCLUDEVALUE constan to exclude.
	 * @param _asOwner AS to retrieve information from.
	 * 
	 * @return URL that opens the specified object, generically.
	 */
	public static final String GetOpenerURL(String _strQSArgs, int _nOpenerMode, AActivitySpace _asOwner)
	{
		return GetOpenerURL(_strQSArgs, _nOpenerMode, ConfigHelper.GetApplicationBaseURL(_asOwner),
			LoginHelper.INSTANCE.GetUserSessionID(_asOwner.GetUserSession()));
	}

	/**
	 * Returns an URL that opens the specified object.  If you do not specify
	 * the mode to open the object in, the default mode for that class will
	 * be used.  Note that you need to provide both the QS name and value pair
	 * in the QSArgs argument (the extended GetOpenerURLXXX methods do this 
	 * for you automatically).    
	 * 
	 * This is the base GetOpenerURL method.  You will need to provide the 
	 * Application Base URL and the User ID, as well as any other information 
	 * that you want to add to the URL, such as the ParentSpace arguments.
	 * Nothing else is added to the URL, you must specify all arguments using
	 * the QSArgs argument.
	 * 
	 * @param _strQSArgs Arguments to use for opening the object.
	 * @param _nOpenerMode Mode to open object in, use OpenerAs.INT_PARAM_EXCLUDEVALUE constan to exclude.
	 * @param _strApplicationBaseURL The Application Base URL.
	 * @param _strUserID User ID to append to the URL, use OpenerAs.INT_PARAM_EXCLUDEVALUE constant to exclude.
	 * 
	 * @return URL that opens the specified object, generically.
	 */
	public static final String GetOpenerURL(String _strQSArgs, int _nOpenerMode, 
		String _strApplicationBaseURL, String _strUserID)
	{   
		// deal with sub space security.
		if ((null != _strQSArgs) && 
			(INT_PARAM_EXCLUDEVALUE != XPStringUtility.IndexOf(_strQSArgs, ASConstants.SUBSPACE))) 
		{
			if (XPStringUtility.StartsWith(_strApplicationBaseURL, ConfigHelper.HTTPS_SCHEME))
			{
				// JF- this link is going to a sub space and the parent space is HTTPS,
				// so the sub space should also be HTTPS. i.e. in modes 2 and 3, links
				// are HTTPS, and in mode 1, the sub and parent should be the same.
				_strApplicationBaseURL = ConfigHelper.GetApplicationBaseURL(_strApplicationBaseURL, true); 	
			}
			else
			{
				// conversly, if the parent is HTTP, then sub should be too (at least 
				// in modes 0 and 1, in modes 2 and 3 the links are always HTTPS).
				_strApplicationBaseURL = ConfigHelper.GetApplicationBaseURL(_strApplicationBaseURL); 	
			}
		}
        
		// ensure that our base url is ready for arguments.
		_strApplicationBaseURL = XPStringUtility.ForceEndsWith(_strApplicationBaseURL, "?");

		// construct our opener url.
		XPStringBuilder sbOpenerURL = new XPStringBuilder(256);
		sbOpenerURL.Append(_strApplicationBaseURL);

		// place the space name here, note that this will no longer be necessary.
		// sbOpenerURL.Append(ASConstants.SPACENAME).Append("=").Append(BaseOpenerAS.STR_MVC_CLASS_NAME);

		if (null != _strQSArgs)
		{
			// this should not be null, may want to throw exception.
			sbOpenerURL.Append(_strQSArgs);
		}

		HTTPHelpers.AppendQSArgument(sbOpenerURL, IOpenerConstants.QS_MODE, _nOpenerMode);
		if ( !"-1".equals(_strUserID))
		{
			HTTPHelpers.AppendQSArgument(sbOpenerURL, ASConstants.USERID_INPUT, _strUserID);
		}

		// Always try to get a cached opener space.  There is no need to have more than
		// one opener space in the cache.
		HTTPHelpers.AppendQSArgument(sbOpenerURL, ASConstants.CACHED, XPConvert.ToString(true));
        
		return sbOpenerURL.ToString();
	}    

	/**
	 * Helper method to generate the window name based on the specified
	 * key.  The key is generally a string that can uniquely identify the
	 * object you are opening the window for, such as the UUID, or the
	 * ClassID/ObjectID pair.
	 * 
	 * @param _strNameKey Key to use to generate the window name.
	 * 
	 * @return A window name based on the specified key.
	 */
	public static final String GetWindowName(String _strNameKey)
	{
		if ((_strNameKey != null) && (INT_PARAM_EXCLUDEVALUE != _strNameKey.compareTo(""))) 
		{
			return BaseOpenerAS.STR_MVC_CLASS_NAME + "_"  + _strNameKey;            
		}
		else
		{
			return BaseOpenerAS.STR_MVC_CLASS_NAME + "_UNKNOWN";
		}
	}
}