package com.plumtree.uiinfrastructure.opener;

import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.activityspace.*;

/**
 * This object is used to encapsulate the destination
 * information for the Common Opener.  The destination
 * can be a redirect (allowing us to go to Activity 
 * Spaces), or there can be information in the QS 
 * arguments.  Plugins can use the QS information to 
 * perform different actions, such as converting a UUID
 * into a ClassID/ObjectID pair.  
 * 
 * This object also has a method that allows the Plugin
 * process to be ended prematurely (if multiple Plugins
 * are installed).
 * 
 * @author RickyW
 */
public class RedirectData
{
    /** Contains an optional Redirect object. */
    private Redirect m_rRedirect;
    
    /** Contains our Querystring arguments. */
    private XPHashtable m_htArguments;
    
    /** Member to prematurely stop Plugin processing. */
    private boolean m_bHaltPlugins;
    
    /**
     * Constructor simply initializes members. 
     */
    public RedirectData()
    {
        m_rRedirect = null;
        m_htArguments = new XPHashtable();
        m_bHaltPlugins = false;
    }
    
    /**
     * @return True if there is a Redirect.
     */
    public boolean HasRedirect()
    {
        return (null != m_rRedirect);
    }

    /**
     * Get the Redirect destination.
     * @return Redirect destination.
     */
    public Redirect GetRedirect()
    {
        return m_rRedirect;
    }

    /**
     * Set the Redirect destination.
     * @param _rRedirect Redirect destination.
     */
    public void SetRedirect(Redirect _rRedirect)
    {    
        m_rRedirect = _rRedirect;
    }
    
    /**
     * Get the Querystring arguments.
     * @return Querystring arguments.
     */
    public XPHashtable GetQSArguments()
    {
        return m_htArguments;
    }

    /**
     * Set the Querystring arguments.
     * @param _htArguments Querystring arguments.
     */
    public void SetQSArguments(XPHashtable _htArguments)
    {    
        m_htArguments = _htArguments;
    }

    /**
     * Adds a querystring argument to the Querystring arguments
     * hashtable.  If the argument is already there, then it is
     * replaced.  If the key or hashtable are null, do nothing.
     * 
     * @param _strKey The key to add into the Querystring hashtable.
     * @param _strValue The value to add into the Querystring hashtable.
     */
    public void AddQSArgument(String _strKey, String _strValue)
    {
        if (null != m_htArguments)
        {
    		if (null != _strKey)
    		{
    			String[] aValues = null;
    			
    			if (null != _strValue)
    			{
    				aValues = new String[] {_strValue};
    			}
    			
                m_htArguments.PutElement(_strKey, aValues);
    		}

        }
    }

    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.opener.RedirectData#AddQSArgument(String, String)
     */
    public void AddQSArgument(String _strKey, int _nValue)
    {
        AddQSArgument(_strKey, XPConvert.ToString(_nValue));
    }

    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.opener.RedirectData#AddQSArgument(String, String)
     */
    public void AddQSArgument(String _strKey, boolean _bValue)
    {
        AddQSArgument(_strKey, XPConvert.ToString(_bValue));
    }

    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.opener.RedirectData#AddQSArgument(String, String)
     */
    public void AddQSArgument(String _strKey, Object _oValue)
    {
        AddQSArgument(_strKey, XPConvert.ToString(_oValue));
    }

    /**
     * Get whether to halt plugin processing.
     * @return True to halt plugin processing.
     */
    public boolean GetHaltPlugins()
    {
        return m_bHaltPlugins;
    }
    
    /**
     * Set whether to halt plugin processing.
     * @param _bHaltPlugins True to halt plugin processing.
     */
    public void SetHaltPlugins(boolean _bHaltPlugins)
    {
        m_bHaltPlugins = _bHaltPlugins;
    }
}
