package com.plumtree.uiinfrastructure.pagecontrols.framework;

import com.plumtree.openfoundation.util.*;
import com.plumtree.xpshared.htmlelements.*;


/**
 * This is the abstract class for all page controls. A page control is a single
 * input/output component on an web page. One example of a control is the
 * datepicker. For each instance of the datepicker on a single page, you need
 * an instance of an AComponent. Components of like controls are managed
 * through a single component manger. So, on a single page you might have 3
 * datepickers (3 instances of AComponent) all managed by a single datepicker
 * manager. If you were to have other types of controls on that same page, you
 * would need an instance of the appropriate componetent manager for those
 * controls. The Page manager aggregates all instances of component mangers
 * and provides access to page specific information that might be relevant to
 * controls (i.e. locale and timezone).  In summary: Component - Might have
 * many on a page (i.e. datepicker). Note: There is really no difference
 * between a "component" and a "control." Component Manager - One per type of
 * component on a page. Page Manager - Only one per page. Aggregates all the
 * component managers.  How is the component manager differnt from the page
 * manager? Why have both? The component manager serves two funtions. First,
 * it aggregates like components. Second, it handles the printing of
 * page-level content (contrast with component-level content.) Consider the
 * datepicker example again. If you have 2 datepickers on one page, there is
 * data that needs to be outputted once (page-level) and data that needs to be
 * outputted once for each datepicker (component-level). One special note
 * about the page-controls framework... It assumes that the client will be
 * using the HTTP Session for state management. Given this, the client needs
 * to have implemented a good HTTP Session mangment strategy that will release
 * old or unused objects. See the documentation on Activity Spaces for such a
 * framework.
 *
 * @author John Osborne
 */
public abstract class AComponent
{
    /** The page manager associated with this component. */
    protected PageMgr m_ptPageMgr;

    /**
     * Each component needs an special string tag to allow it to be uniquely
     * identified on a page. This tage is appended to the end of the HTML name
     * of the component's input elements.
     */
    private String m_strUniqueInstanceID;

    /**
     * Creates a new AComponent
     *
     */
    public AComponent()
    {
    }

    /**
     * Initializes a new AComponent()
     *
     * @param _ptComponentMgr The component manager for this component.
     * @param _strUniqueInstanceID The unique ID for this component.
     */
    public void Init(PageMgr _ptPageMgr, String _strUniqueInstanceID)
    {
        m_ptPageMgr = _ptPageMgr;
        m_strUniqueInstanceID = _strUniqueInstanceID;
    }

    /**
     * This method serves as a getter for component HTML. Sub-classes must
     * implement this method.
     *
     * @return The HTML for the component itself.
     */
    public abstract HTMLElementCollection GetComponentHTML();

    /**
     * This method returns the page manager that houses this component manager.
     *
     * @return The page manager.
     */
    public PageMgr GetPageMgr()
    {
        return m_ptPageMgr;
    }

    /**
     * This method serves as a getter for "sub-component" HTML. The breakdown
     * of sub-components is decided by the component author. One example is
     * the date/time-picker. It is made up of two sub-components: the
     * date-picker and the time-picker. The constants for component sub-index
     * should be specified in the component file and prefixed with
     * "N_SUB_COMPONENT_INDEX." If the specified index is not valid, null
     * should be returned. This method returns null in the parent instead of
     * being abstract becuase many components will not need to use this
     * method.
     *
     * @param _nSubComponentIndex The sub-component index specified in the the
     *                   componet file itself.
     *
     * @return The HTML for the sub-component requested.
     */
    public HTMLElementCollection GetSubComponentHTMLByIndex(int _nSubComponentIndex)
    {
        return null;
    }

    /**
     * This abstract method stores the input submitted by the user. It should
     * be called after a POST or GET. In the case of the date picker, it would
     * store the date.
     *
     * @param _htRequestParameters A hashtable containing the request's name-
     *                   value pairs.
     */
    public abstract int StoreUserInput(XPHashtable _htRequestParameters);

    /**
     * This method returns script that needs to be printed within the HTML head
     * tag. This method could have been abstract except that most components
     * do not have a use for it. It therefore returns null and should be
     * overridden as needed. (Note: there is an anologous method for printing
     * page- level content in ComponentMgr.)
     *
     * @return HTML that goes within the head tag.
     */
    public HTMLElementCollection GetComponentHeadContent()
    {
        return null;
    }

    /**
     * This method returns script that needs to be printed after the component
     * itself has been printed. This method could have been abstract except
     * that most components do not have a use for it. It therefore returns
     * null and should be overridden as needed. (Note: there is an anologous
     * method for printing page- level content in ComponentMgr.)
     *
     * @return HTML that should be printed after the component itself.
     */
    public HTMLElementCollection GetComponentPostContent()
    {
        return null;
    }

    /**
     * This method returns script that needs to be printed before the component
     * itself has been printed. This method could have been abstract except
     * that most components do not have a use for it. It therefore returns
     * null and should be overridden as needed. (Note: there is an anologous
     * method for printing page- level content in ComponentMgr.)
     *
     * @return HTML that should be printed before the component itself.
     */
    public HTMLElementCollection GetComponentPreContent()
    {
        return null;
    }

    /**
     * This method returns script that needs to be printed before the form in
     * which this component is embedded. It therefore returns null and should
     * be overridden as needed. (Note: there is an anologous method for
     * printing page- level content in ComponentMgr.)
     *
     * @return
     */
    public HTMLElementCollection GetComponentPreFormContent()
    {
        return null;
    }

    /**
     * This method returns the string used to uniquely idenify this component
     * on a page.
     *
     * @return
     */
    public String GetUniqueInstanceID()
    {
        return m_strUniqueInstanceID;
    }

    /**
     * This method should be used to add the unique identifier to HTML inputs
     * related to this component. If my element name is "in_cb_MyCheckBox" and
     * my unique ID is "one", then this method will return
     * "in_cb_MyCheckBox_one".
     *
     * @param _strElementName The name of the element that needs to have the
     *                   unique ID appended to it.
     *
     * @return The element name plus the unique ID.
     */
    protected final String AppendUniqueIDToElementName(String _strElementName)
    {
        return _strElementName + "_" + m_strUniqueInstanceID;
    }
    
	/**
	 * Return the name or this component.
	 * 
	 * @return the element name
	 */
    public abstract String GetName();
	
    /**
     * Create method for the object returns an instance of the object.
     * 
     * @return and object that extends AComponent
     */	
	public abstract Object Create();
	
	/**
	 * For every Validator associated with this object execute it's Validate method.
	 * Return an Arraylist of UIError objects generated from executing the Validators.
	 * Return and empty Arraylist if no problems were found.
	 * 
	 * @return Arraylist of UIErrors.
	 */
	public abstract XPArrayList Validate();
	
	/**
	 * Return an Arraylist of validators associated with this object.
	 * @return
	 */
	public abstract XPArrayList GetValidators();
}
