package com.plumtree.uiinfrastructure.pagecontrols.framework;

import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;


/**
 * JF- This is the abstract class for all HTML Element page controls.
 *
 * @author Julie Fournier
 */
public abstract class AHTMLComponent extends AComponent
{
    /** JF- HTML Component Name */
    private String m_sHTMLComponentName;

    /** JF- HTML Component Value */
    private String m_sHTMLComponentValue;

    /** HTML Component Size */
    private String m_sHTMLComponentSize;
    
    /** HTML Component Style Class */
    private PTStyleClass m_HTMLComponentStyleClass;
    
    /** HTML Component Label */
    private String m_sLabel;
    
    /** HTML Component Validators */
    private XPArrayList m_Validators;

	/**
	 * Creates an AHTML Component
	 */
    public AHTMLComponent()
    {
    }

    /**
     * JF- Initializes a new AHTMLComponent object.
     *
     * @param _ptComponentMgr The component manager for this component.
     * @param _strUniqueInstanceID The unique ID for this component.
     */
    public void Init(PageMgr _ptPageMgr, String _strUniqueInstanceID)
    {
        super.Init(_ptPageMgr, _strUniqueInstanceID);
        m_sHTMLComponentName 		= AppendUniqueIDToElementName(GetHTMLComponentBaseName());
        m_sHTMLComponentSize 		= "";
        m_sHTMLComponentValue 		= "";
        m_Validators = new XPArrayList(0);
        
    }
    
    /**
     * JF- Store the new value of this HTML component.
     *
     * @param _htRequestParameters
     */
    public int StoreUserInput(XPHashtable _htRequestParameters)
    {
        Object oElement = _htRequestParameters.GetElement(m_sHTMLComponentName);

        if (oElement != null)
        {
            String[] aStrValues = (String[]) oElement;

            if (aStrValues.length > 0)
            {
                m_sHTMLComponentValue = aStrValues[0];
            }
        }

        return RepostControl.PAGE_STATUS_VALID;
    }
    
    /**
     * Add a validator to this component's collection of validators.
     * 
     * @param _ptValidator a validator to add to the list of validators
     * 			owned by this component
     */
    public void AddValidator(IValidator _ptValidator)
    {
    	m_Validators.Add(_ptValidator);
    }
    
    /**
     * Return an XPArrayList of UIError messages gathered by calling the 
     * validate() method on all Validators in this Component's collection
     * of Validators.
     */
	public XPArrayList Validate()
	{
		XPArrayList myErrors = new XPArrayList();
		IXPEnumerator enumerator = m_Validators.GetEnumerator();
		while (enumerator.MoveNext())
		{
			IValidator myValidator = (IValidator) enumerator.GetCurrent();
			UIError myError = myValidator.Validate(GetHTMLComponentValue());
			if (null != myError)
			{
				myErrors.Add(myError);
			}
		}
		return myErrors;
	}
	
	/**
	 * Return the list of Validators associated with this object.
	 * 
	 * @return XPArrayList of validators
	 */
	public XPArrayList GetValidators()
	{
		return m_Validators;
	}
	
	/**
	 * Return a string made up of all the JSValidationCode for each of the
	 * Validator's associated with this component.
	 * 
	 * @return JavaScript validation code
	 */
	public String GetJSValidationCode()
	{
		XPStringBuilder sb = new XPStringBuilder(1250);
		for (int i = 0; i < m_Validators.GetSize(); i++)
		{
			IValidator myValidator = (IValidator) m_Validators.GetElement(i);
			sb.Append(myValidator.GetJSValidationCode());
		}
		return sb.toString();
	}

    /**
     * JF- Returns the base name of this HTML Component.
     * E.g.: in_tx_pc
     *
     * @return String
     */
    protected abstract String GetHTMLComponentBaseName();

    /**
     * JF- Returns the name of this HTML Component.
     *
     * @return String
     */
    public String GetHTMLComponentName()
    {
        return m_sHTMLComponentName;
    }

    /**
     * JF- Returns the size of this HTML Component.
     *
     * @return String
     */
    public String GetHTMLComponentSize()
    {
        return m_sHTMLComponentSize;
    }

    /**
     * JF- Returns the style class of this HTML Component.
     *
     * @return PTStyleClass
     */
    public PTStyleClass GetHTMLComponentStyleClass()
    {
        return m_HTMLComponentStyleClass;
    }

    /**
     * JF- Returns the value of this HTML Component.
     *
     * @return String
     */
    public String GetHTMLComponentValue()
    {
        return m_sHTMLComponentValue;
    }

    /**
     * JF- Sets the name of this HTML Component.
     *
     * @param _sHTMLComponentName
     */
    public void SetHTMLComponentName(String _sHTMLComponentName)
    {
        m_sHTMLComponentName = _sHTMLComponentName;
    }

    /**
     * JF- Sets the size of this HTML Component.
     *
     * @param _sHTMLComponentSize
     */
    public void SetHTMLComponentSize(String _sHTMLComponentSize)
    {
        m_sHTMLComponentSize = _sHTMLComponentSize;
    }

    /**
     * JF- Sets the style class of this HTML Component.
     *
     * @param _HTMLComponentStyleClass
     */
    public void SetHTMLComponentStyleClass(PTStyleClass _HTMLComponentStyleClass)
    {
        m_HTMLComponentStyleClass = _HTMLComponentStyleClass;
    }


	/**
	 * Transfer the properties set to this object to the associated properties
	 * on the HTMLInput object. For example, Set the HTMLInput's styleClass 
	 * with this object's GetHTMLComponentStyleClass.
	 * 
	 * @param _ptHTMLInput
	 * @return
	 */
	protected HTMLInput SetParameters(HTMLInput _ptHTMLInput)
	{
		if (null != GetHTMLComponentStyleClass())
		{
			_ptHTMLInput.SetStyleClass(GetHTMLComponentStyleClass());
		}		
		if (null != GetHTMLComponentSize())
		{
			_ptHTMLInput.SetSize(GetHTMLComponentSize());
		}
		if (null != GetHTMLComponentValue())
		{
			_ptHTMLInput.SetValue(GetHTMLComponentValue());
		}
		return _ptHTMLInput;
	}


    /**
     * JF- Sets the value of this HTML Component.
     *
     * @param _sHTMLComponentValue
     */
    public void SetHTMLComponentValue(String _sHTMLComponentValue)
    {
        m_sHTMLComponentValue = _sHTMLComponentValue;
    }
		
	/**
	 * Returns the Label of this component
	 * 
	 * @return Returns the m_sLabel member variable
	 */
	public String GetLabel() {
		return m_sLabel;
	}
	
	/**
	 * Sets the Label of this component
	 * 
	 * @param label The m_sLabel to set
	 */
	public void SetLabel(String label) {
		m_sLabel = label;
	}
	
	/**
	 * Returns a ComponentMgr for this component. By default classes that extend
	 * AHTMLComponent may use this method to return the defualt ComponentMgr
	 * HTMLComponentMgr. If a component needs another ComponentMgr it should override
	 * this method.
	 * 
	 * @param _ptPageMgr the PageMgr that the ComponentMgr is registered.
	 * @return the default ComponentMgr, HTMLComponentMgr.
	 */
	public ComponentMgr GetComponentMgr(PageMgr _ptPageMgr)
	{
		return new HTMLComponentMgr(_ptPageMgr);
	}
	
	/**
	 * Returns the name of the  ComponentMgr for this component. By default classes 
	 * that extend AHTMLComponent may use this method to return the defualt ComponentMgr
	 * name, HTMLComponentMgr.STR_CLASS_NAME. If a component needs another ComponentMgr 
	 * it should override this method.
	 * 
	 * @return the name of the default ComponentMgr, HTMLComponentMgr.STR_CLASS_NAME.
	 */
	public String GetComponentMgrName()
	{
		return HTMLComponentMgr.STR_CLASS_NAME;
	}
}