package com.plumtree.uiinfrastructure.pagecontrols.framework;

import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.xpshared.htmlelements.*;


/**
 * This class manages instances of like components that all appear on a single
 * page. For more information on the page controls framework in general,
 * please consult the documentation for AComponent.  There is one important
 * concept to understand when it comes to the ComponentMgr class: for page
 * controls, there is script to be outputted that has "component scope" and
 * script to be outputted that has "page scope." Component scope means that
 * the data needs to be printed once per component. Page scope means that the
 * data needs to be printed once per page, independent of the number of times
 * a component of that type is written to the page. The component manager
 * allows you to print data for either or both scopes through a series of
 * methods.  Components may be registered with the component manager and then
 * obtained either using their unique instance ID or by way of an int index.
 * The index is determined by the order in which you registered the
 * components.
 *
 * @author John Osborne
 */
public class ComponentMgr
{
    /** The page manager that aggregates all component managers on this page. */
    private PageMgr m_ptPageMgr;

    /** A list of all the registered components on this page. */
    private XPArrayList m_alRegisteredComponents;

	/** class name constant */	
	public final String STR_CLASS_NAME = "ComponentMgr";
	
	/** a boolean to track whether jsfunctions have been added for this componentmgr */
	private boolean m_bAddedFunctions = false; 
    
    /**
     * Creates a new ComponentMgr object. Clients need one of these per
     * component type (i.e. datepicker).
     *
     * @param _ptPageMgr The page manager for this component manager.
     */
    public ComponentMgr(PageMgr _ptPageMgr)
    {
        m_ptPageMgr = _ptPageMgr;
        m_alRegisteredComponents = new XPArrayList();
    }

    /**
     * This method returns ALL HTML--page scope and component scope for all
     * components under this manger--that belongs between the HTML head tags.
     *
     * @return All HTML content that should go between the head tags for
     *                    components of this type.
     */
    public HTMLElementCollection GetAllHeadContent()
    {
        HTMLElementCollection retVal = new HTMLElementCollection();

        HTMLElementCollection htmlContent1 = GetPageHeadContent();

        if (htmlContent1 != null)
        {
            retVal.AddInnerHTMLElement(htmlContent1);
        }

        for (int i = 0; i < m_alRegisteredComponents.GetSize(); i++)
        {
            HTMLElementCollection htmlContent2 = ((AComponent) m_alRegisteredComponents.GetElement(i)).GetComponentHeadContent();

            if (htmlContent2 != null)
            {
                retVal.AddInnerHTMLElement(htmlContent2);
            }
        }

        return retVal;
    }

    /**
     * This method returns ALL HTML--page scope and component scope for all
     * components under this manger--that should be printed after the
     * component itself.
     *
     * @return All HTML content that should be printed after the component
     *                    itself.
     */
    public HTMLElementCollection GetAllPostContent()
    {
        HTMLElementCollection retVal = new HTMLElementCollection();

        HTMLElementCollection htmlContent1 = GetPagePostContent();

        if (htmlContent1 != null)
        {
            retVal.AddInnerHTMLElement(htmlContent1);
        }

        for (int i = 0; i < m_alRegisteredComponents.GetSize(); i++)
        {
            HTMLElementCollection htmlContent2 = ((AComponent) m_alRegisteredComponents.GetElement(i)).GetComponentPostContent();

            if (htmlContent2 != null)
            {
                retVal.AddInnerHTMLElement(htmlContent2);
            }
        }

        return retVal;
    }

    /**
     * This method returns ALL HTML--page scope and component scope for all
     * components under this manger--that should be printed before the
     * component itself.
     *
     * @return All HTML content that should be printed before the component
     *                    itself.
     */
    public HTMLElementCollection GetAllPreContent()
    {
        HTMLElementCollection retVal = new HTMLElementCollection();
        HTMLElementCollection htmlContent1 = GetPagePreContent();

        if (htmlContent1 != null)
        {
            retVal.AddInnerHTMLElement(htmlContent1);
        }

        for (int i = 0; i < m_alRegisteredComponents.GetSize(); i++)
        {
            HTMLElementCollection htmlContent2 = ((AComponent) m_alRegisteredComponents.GetElement(i)).GetComponentPreContent();

            if (htmlContent2 != null)
            {
                retVal.AddInnerHTMLElement(htmlContent2);
            }
        }

        return retVal;
    }

    /**
     * This method returns ALL HTML--page scope and component scope for all
     * components under this manger--that should be printed before the form in
     * which this group of components is embedded.
     *
     * @return All HTML content that should be printed before the form.
     */
    public HTMLElementCollection GetAllPreFormContent()
    {
        HTMLElementCollection retVal = new HTMLElementCollection();
        HTMLElementCollection htmlContent1 = GetPagePreFormContent();

        if (htmlContent1 != null)
        {
            retVal.AddInnerHTMLElement(htmlContent1);
        }

        for (int i = 0; i < m_alRegisteredComponents.GetSize(); i++)
        {
            HTMLElementCollection htmlContent2 = ((AComponent) m_alRegisteredComponents.GetElement(i)).GetComponentPreFormContent();

            if (htmlContent2 != null)
            {
                retVal.AddInnerHTMLElement(htmlContent2);
            }
        }

        return retVal;
    }

    /**
     * This method returns an AComponent object given its unique instance ID.
     * Null is returned if the component is not found.
     *
     * @param _strUniqueInstanceID The ID that uniquely represents the
     *                   component that you want.
     *
     * @return The AComponent instance requested (or null).
     */
    public AComponent GetComponent(String _strUniqueInstanceID)
    {
        // This is not very efficient, but the assumption is that there are not many instances per page.
        for (int i = 0; i < m_alRegisteredComponents.GetSize(); i++)
        {
            AComponent oInstance = (AComponent) m_alRegisteredComponents.GetElement(i);

            if (null != oInstance 
                    && null != oInstance.GetUniqueInstanceID() 
                    	&& oInstance.GetUniqueInstanceID().equals(_strUniqueInstanceID))
            {
                return oInstance;
            }
        }

        return null;
    }

    /**
     * Returns an AComponent given an index. The index is determined by the
     * registration order. This method can help you loop through all
     * registered AComponents.
     *
     * @param _nIndex The index of the component you want.
     *
     * @return The AComponent instance requested (or null).
     */
    public AComponent GetComponent(int _nIndex)
    {
        Object oElement = null;

        try
        {
            oElement = m_alRegisteredComponents.GetElement(_nIndex);
        }
        catch (Exception e)
        {
            oElement = null;

            // Do nothing... just return null
        }

        if (oElement != null)
        {
            AComponent oComponent = (AComponent) oElement;

            return oComponent;
        }

        return null;
    }

    /**
     * Returns the HTML for the designated component.
     *
     * @param _strUniqueInstanceID The unique instance ID by which to identify
     *                   the component.
     *
     * @return The HTML for the component itself.
     */
    public HTMLElementCollection GetComponentHTML(String _strUniqueInstanceID)
    {
        AComponent oComponent = GetComponent(_strUniqueInstanceID);

        if (oComponent != null)
        {
            return oComponent.GetComponentHTML();
        }

        return null;
    }

    /**
     * Returns the HTML for the designated component.
     *
     * @param _nIndex The index by which to identify the component.
     *
     * @return The HTML for the component itself.
     */
    public HTMLElementCollection GetComponentHTML(int _nIndex)
    {
        Object oElement = m_alRegisteredComponents.GetElement(_nIndex);

        if (oElement != null)
        {
            AComponent oComponent = (AComponent) oElement;

            return oComponent.GetComponentHTML();
        }

        return null;
    }

    /**
     * Returns the HTML for the designated component that is should be written
     * out within the HTML head tags.
     *
     * @param _strUniqueInstanceID The unique instance ID by which to identify
     *                   the component.
     *
     * @return The script to write into the HTML head.
     */
    public HTMLElementCollection GetComponentHeadContent(String _strUniqueInstanceID)
    {
        AComponent oComponent = GetComponent(_strUniqueInstanceID);

        if (oComponent != null)
        {
            oComponent.GetComponentHeadContent();
        }

        return null;
    }

    /**
     * Returns the HTML for the designated component that is should be written
     * out within the HTML head tags.
     *
     * @param _nIndex The index by which to identify the component.
     *
     * @return The script to write into the HTML head.
     */
    public HTMLElementCollection GetComponentHeadContent(int _nIndex)
    {
        Object oElement = m_alRegisteredComponents.GetElement(_nIndex);

        if (oElement != null)
        {
            AComponent oComponent = (AComponent) oElement;

            return oComponent.GetComponentHeadContent();
        }

        return null;
    }

    /**
     * Returns the HTML for the designated component that should be written out
     * after the component itself.
     *
     * @param _strUniqueInstanceID The unique instance ID by which to identify
     *                   the component.
     *
     * @return The script to write after the component.
     */
    public HTMLElementCollection GetComponentPostContent(String _strUniqueInstanceID)
    {
        AComponent oComponent = GetComponent(_strUniqueInstanceID);

        if (oComponent != null)
        {
            oComponent.GetComponentPostContent();
        }

        return null;
    }

    /**
     * Returns the HTML for the designated component that should be written out
     * after the component itself.
     *
     * @param _nIndex The index by which to identify the component.
     *
     * @return The script to write after the component.
     */
    public HTMLElementCollection GetComponentPostContent(int _nIndex)
    {
        Object oElement = m_alRegisteredComponents.GetElement(_nIndex);

        if (oElement != null)
        {
            AComponent oComponent = (AComponent) oElement;

            return oComponent.GetComponentPostContent();
        }

        return null;
    }

    /**
     * Returns the HTML for the designated component that should be written out
     * before the component itself.
     *
     * @param _strUniqueInstanceID The unique instance ID by which to identify
     *                   the component.
     *
     * @return The script to write before the component.
     */
    public HTMLElementCollection GetComponentPreContent(String _strUniqueInstanceID)
    {
        AComponent oComponent = GetComponent(_strUniqueInstanceID);

        if (oComponent != null)
        {
            return oComponent.GetComponentPreContent();
        }

        return null;
    }

    /**
     * Returns the HTML for the designated component that should be written out
     * before the component itself.
     *
     * @param _nIndex The index by which to identify the component.
     *
     * @return The script to write before the component.
     */
    public HTMLElementCollection GetComponentPreContent(int _nIndex)
    {
        Object oElement = m_alRegisteredComponents.GetElement(_nIndex);

        if (oElement != null)
        {
            AComponent oComponent = (AComponent) oElement;

            return oComponent.GetComponentPreContent();
        }

        return null;
    }

    /**
     * Returns the HTML for the designated component that should be written out
     * before the form in which the component is embedded.
     *
     * @param _nIndex The index by which to identify the component.
     *
     * @return The script to write before the form in which the component is
     *                    embedded.
     */
    public HTMLElementCollection GetComponentPreFormContent(int _nIndex)
    {
        Object oElement = m_alRegisteredComponents.GetElement(_nIndex);

        if (oElement != null)
        {
            AComponent oComponent = (AComponent) oElement;

            return oComponent.GetComponentPreFormContent();
        }

        return null;
    }

    /**
     * Returns the HTML for the designated component that should be written out
     * before the form in which the component is embedded.
     *
     * @param _strUniqueInstanceID The unique instance ID by which to identify
     *                   the component.
     *
     * @return The script to write before the form in which the component is
     *                    embedded.
     */
    public HTMLElementCollection GetComponentPreFormContent(String _strUniqueInstanceID)
    {
        AComponent oComponent = GetComponent(_strUniqueInstanceID);

        if (oComponent != null)
        {
            return oComponent.GetComponentPreFormContent();
        }

        return null;
    }

    /**
     * Returns the number of components registered with this component manger.
     *
     * @return An int representing the number of registered components.
     */
    public int GetNumComponents()
    {
        return m_alRegisteredComponents.GetSize();
    }

    /**
     * This method is meant to return the page-level head content for
     * components of this type. This method just returns null. It could have
     * been abstract, but many components do need to use it. If you need this,
     * you should override it.
     *
     * @return The HTML and script that should go into the head tag.
     */
    public HTMLElementCollection GetPageHeadContent()
    {
        return null;
    }

    /**
     * Returns the page manger that is associated with this component manager.
     *
     * @return The page manger.
     */
    public PageMgr GetPageMgr()
    {
        return m_ptPageMgr;
    }

    /**
     * This method is meant to return the page-level "post" content for
     * components of this type. This method just returns null. It could have
     * been abstract, but many components do need to use it. If you need this,
     * you should override it.
     *
     * @return The HTML and script that should be written after the component.
     */
    public HTMLElementCollection GetPagePostContent()
    {
        return null;
    }

    /**
     * This method is meant to return the page-level head "pre" for components
     * of this type. This method just returns null. It could have been
     * abstract, but many components do need to use it. If you need this, you
     * should override it.
     *
     * @return The HTML and script that should be written before the component.
     */
    public HTMLElementCollection GetPagePreContent()
    {
        return null;
    }

    /**
     * This method is meant to return the page-level head content for
     * components of this type. This method just returns null. It could have
     * been abstract, but many components do need to use it. If you need this,
     * you should override it.
     *
     * @return The HTML and script that should be written before the form in
     *                    which this component is embedded.
     */
    public HTMLElementCollection GetPagePreFormContent()
    {
        return null;
    }

    /**
     * This method allows you to register a component with the component
     * manager. The order in which you register components is the order in
     * which calls will be made when doing any looping operation--storing or
     * printing. If you attempt to re-register the same component, it will
     * overwrite the one that existed there before.
     *
     * @param _ptComponent The component that you wish to register.
     */
    public void RegisterComponent(IPageControl _ptComponent)
    {
        if (_ptComponent != null)
        {
            if (m_alRegisteredComponents.Contains(_ptComponent))
            {
                int nIndex = m_alRegisteredComponents.IndexOf(_ptComponent);
                m_alRegisteredComponents.Insert(nIndex, _ptComponent);
            }
            else
            {
                m_alRegisteredComponents.Add(_ptComponent);
            }
        }
    }

    /**
     * Stores the user input for all componets registered with this component
     * manager.
     *
     * @param _htRequestParameters A hashtable of name-value pairs passed in
     *                   with the request.
     */
    public int StoreAllUserInput(XPHashtable _htRequestParameters)
    {
    	int nError = RepostControl.PAGE_STATUS_VALID, nStatus;
        for (int i = 0; i < m_alRegisteredComponents.GetSize(); i++)
        {
            nStatus = StoreUserInput(i, _htRequestParameters);
            if (nStatus ==RepostControl.PAGE_STATUS_INVALID){
            	nError = nStatus;
            }
        }
        return nError;
    }
    
    
    /**
     * Call the validation method on all registered componenets. Returns an XPArrayList of all
     * UIErrors generated by the componenets. 
     * 
     * @return an XPArrayList of UIError objects
     */
    public XPArrayList Validate()
    {
    	XPArrayList myErrors = new XPArrayList(0);
        for (int i = 0; i < m_alRegisteredComponents.GetSize(); i++)
        {
            AComponent myComponent = GetComponent(i);
            if (null != myComponent.Validate())
            {
            	myErrors.AddRange(myComponent.Validate());
            }
        }
        return myErrors;
    }

    /**
     * Return JavaScript validation code for each of the validators associated with
     * all components. This function returns the validation code once for each validator.
     * Although the validator may be used several times it is assumed that the JavaScript
     * code is only required one time.
     * 
     * @return String JavaScript code for validators
     */
	public String GetJSValidationCode()
	{
		XPStringBuilder sb = new XPStringBuilder(1250);
        XPArrayList allValidators = new XPArrayList();
		for (int i = 0; i < m_alRegisteredComponents.GetSize(); i++)
        {
            AComponent myComponent = GetComponent(i);
            XPArrayList myValidators = myComponent.GetValidators();
            if (null != myValidators) 
            {
	        	for (int j = 0; j < myValidators.GetSize(); j++)
	        	{
	        		IValidator myValidator = (IValidator)myValidators.GetElement(j);
	        		if (!allValidators.Contains(myValidator.GetName()))
	        		{
	        			sb.Append(myValidator.GetJSValidationCode());
	        			allValidators.Add(myValidator.GetName());
	        		}
	        	}
            }
        }
        return sb.ToString();
	}
	
	/**
	 * For all registered components, for every Validator associated with that
	 * component call AFormDP.AddToValidationFunction for the validator. This inserts
	 * the JavaScript Validation function name from the Validator to the page's 
	 * JavaScript Validate method.
	 *  
	 * @param _ptAFormDP AFormDP from the activity space where this ComponentMgr
	 * 				is used.
	 */
	public void AddToValidationFunction(AFormDP _ptAFormDP)
	{
		//if they've been added already no need to do it again.
		if (m_bAddedFunctions)
		{
			return;
		}
		for (int i = 0; i < m_alRegisteredComponents.GetSize(); i++)
        {
            AComponent myComponent = GetComponent(i);
            XPArrayList myValidators = myComponent.GetValidators();
            if (null != myValidators) 
            {
	        	for (int j = 0; j < myValidators.GetSize(); j++)
	        	{
	        		IValidator myValidator = (IValidator)myValidators.GetElement(j);
	        		_ptAFormDP.AddToValidationFunction(myValidator.GetJSFunction(null));
	        	}
            }
        }
		// track that we've added the functions
		m_bAddedFunctions = true;
	}
    
    /**
     * This method stores the user input for a particular component given its
     * unique ID.
     *
     * @param _strUniqueInstanceID The unique instance ID by which to identify
     *                   the component.
     * @param _htRequestParameters A hashtable of name-value pairs passed in
     *                   with the request.
     */
    public void StoreUserInput(String _strUniqueInstanceID,
                               XPHashtable _htRequestParameters)
    {
        AComponent oComponent = GetComponent(_strUniqueInstanceID);
        oComponent.StoreUserInput(_htRequestParameters);
    }

    /**
     * This method stores the user input for a particular component given its
     * index.
     *
     * @param _nIndex The index by which to identify the component.
     * @param _htRequestParameters A hashtable of name-value pairs passed in
     *                   with the request.
     */
    public int StoreUserInput(int _nIndex, XPHashtable _htRequestParameters)
    {
        AComponent oComponent = GetComponent(_nIndex);
        return oComponent.StoreUserInput(_htRequestParameters);
    }

    /**
     * This method unregisters a component given its unique instance ID.
     *
     * @param _strUniqueInstanceID The unique instance ID.
     */
    public void UnRegisterComponent(String _strUniqueInstanceID)
    {
        // This is not very efficient, but the assumption is that there are not many instances per page.
        int nIndexToRemove = -1;

        for (int i = 0; i < m_alRegisteredComponents.GetSize(); i++)
        {
            AComponent oInstance = (AComponent) m_alRegisteredComponents.GetElement(i);

            if (oInstance.GetUniqueInstanceID().equals(_strUniqueInstanceID))
            {
                nIndexToRemove = i;

                break;
            }
        }

        if (nIndexToRemove != -1)
        {
            m_alRegisteredComponents.RemoveAt(nIndexToRemove);
        }
    }
}
