package com.plumtree.uiinfrastructure.pagecontrols.framework;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;

import com.plumtree.xpshared.htmlelements.*;
import com.plumtree.xpshared.jsutil.*;

/**
 * This class is meant to manage JS components. Most of the work is done by the
 * parent class. However, there are some things that are specific to the JS
 * classes that are handled by the JS registry. For many JS components (like
 * the date-picker), you will want to override this class.
 *
 * @author John Osborne
 */
public class JSComponentMgr extends ComponentMgr
{
	/** The JS Registry that handles versioning for JS components. */
	private JSRegistry m_ptJSRegistry;

	/** class name constant */	
	public static final String STR_CLASS_NAME = "JSComponentMgr";
	
	/**
	 * The name for JS controls of this type (ie.e jsdatepicker"). It is this
	 * name that will be used to retreive the version number from the
	 * IJSComponentVersionMgr.
	 */
	private String m_strJSControlName;

	/** The version (as an String) to use for JS controls of this type. */
	private String m_strVersionID;

	/** The version (as an int) to use for JS controls of this type. */
	private int m_nVersionID;

	/**
	 * Creates a new JSComponentMgr object.
	 *
	 * @param _strControlName The name for JS controls of this type (ie.e
	 * 		  jsdatepicker").
	 * @param _ptJSRegistry The JS Registry that handles versioning for JS
	 * 		  components.
	 * @param _ptJSCompVerMgr An object that can tell you the version number to
	 * 		  use for a particular JS component (given a string name).
	 * @param _ptPageMgr The page manager for this component manager.
	 */
	public JSComponentMgr(String _strControlName, JSRegistry _ptJSRegistry, IJSComponentVersionMgr _ptJSCompVerMgr, PageMgr _ptPageMgr)
	{
		super(_ptPageMgr);

		m_ptJSRegistry		   = _ptJSRegistry;
		m_strJSControlName     = _strControlName;
		m_nVersionID		   = _ptJSCompVerMgr.GetJSComponentVersionNumber(_strControlName);
		m_strVersionID		   = "LATEST";
	}

	/**
	 * Returns the name for JS controls of this type (ie.e jsdatepicker").
	 *
	 * @return The name for JS controls of this type.
	 */
	public String GetJSControlName()
	{
		return m_strJSControlName;
	}

	/**
	 * This method returns the HTML script include tags required by JS
	 * components of this type. If none are found, it will return null.
	 *
	 * @return The HTML script includes.
	 */
	public HTMLElementCollection GetJSIncludes()
	{
		HTMLElementCollection retVal = null;

		try
		{
			
			XPLocale locale = GetPageMgr().GetLocale();
			String strHTMLLinks = m_ptJSRegistry.GetJSComponent(m_strJSControlName, m_nVersionID).GenIncludes(locale);

			retVal = new HTMLElementCollection();
			retVal.AddInnerHTMLString(strHTMLLinks);
		}
		catch (Exception e)
		{
			// TODOCR: DONH - This should have better error handling.  Trace error.
			return null;
		}

		return retVal;
	}

	/**
	 * Returns the version number of the JS component that is being used.
	 *
	 * @return The version number of the JS component that is being used (as an
	 * 		   int).
	 */
	public int GetVersionIDAsInt()
	{
		return m_nVersionID;
	}

	/**
	 * Returns the version number of the JS component that is being used.
	 *
	 * @return The version number of the JS component that is being used (as a
	 * 		   String).
	 */
	public String GetVersionIDAsString()
	{
		return m_strVersionID;
	}
}

