package com.plumtree.uiinfrastructure.pagecontrols.framework;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.form.*;

/**
 * The page manager (PageMgr) class aggregates all of the comoponent managers for 
 * a given page. It also page-scope information such as locale and time zone. 
 * It has some convenience methods that act on all ComponenteMgrs registered with 
 * the PageMgr and all their Page Controls -- StoreAllUserInput and Validate. 
 *
 * @author John Osborne
 */
public class PageMgr
{
	/** logger of errors */
	private static OpenLogger log = OpenLogService.GetLogger(
			OpenLogService.GetComponent("UI_Infrastructure"),
			"com.plumtree.uiinfrastructure.pagecontrols.framework.PageMgr");
	
	/** The base URL to the image server. */
	private String m_strImageServerBaseURL;
	
	/** The connection URL to the image server */
	private String m_strImageServerConnectionURL;

	/** The language ID for this page (i.e. "en") */
	private String m_strLangID;

	/** A list of all the registered component managers on this page. */
	private XPArrayList m_alRegisteredComponentMgrs;
	
	/** hashtable associating page controls with their component mgrs */
	private XPHashtable	m_hashControlsManagers;

	/** The locale for this page. */
	private XPLocale m_xpLocale;

	/**
	 * The timezone to be used by date-pickers under the control of this
	 * manager
	 */
	private XPTimeZone m_xpTimeZone;
	
	/** array list of error messages */
	private XPArrayList m_xpErrorMsgs = new XPArrayList();
	
	/** The access mode for the page */
	private AccessStyles m_asAccessStyle;
	
	/** The table of component managers */
	private XPHashtable m_xpComponentManagers = new XPHashtable();

	/**
	 * Creates a new PageMgr object. At the time of creation, the page manager
	 * has no component managers; they must be registered. If the locale or
	 * time zone is null, the default is used. The image server base URL
	 * cannot be null or emppty string.
	 *
	 * @param _asAccessStyle The access mode for this page
	 * @param _xpLocale The locale for this page.
	 * @param _xpTimeZone The time-zone for this page.
	 * @param _strImageServerBaseURL The base URL to the image server.
	 * @param _strImageConnectionBaseURL The connection URL to the image server
	 * 
	 */
	public PageMgr(AccessStyles _asAccessStyle,
			XPLocale _xpLocale, 
			XPTimeZone _xpTimeZone,
			String _strImageServerBaseURL,
			String _strImageServerConnectionURL)
	{
		m_asAccessStyle = _asAccessStyle; 
		m_strImageServerConnectionURL = _strImageServerConnectionURL;
		m_alRegisteredComponentMgrs = new XPArrayList();

		if (null == _xpLocale)
		{
			m_xpLocale = XPLocale.GetDefault();
		}
		else
		{
			m_xpLocale = _xpLocale;
		}

		if (null != _xpTimeZone)
		{
			m_xpTimeZone = _xpTimeZone;
		}
		else
		{
			m_xpTimeZone = XPTimeZone.GetCurrentTimeZone();
		}

		m_strLangID = m_xpLocale.GetLanguage();

		if ((_strImageServerBaseURL == null) || _strImageServerBaseURL.equals(""))
		{
			throw new XPException("You must pass in a valid Image Server URL to the Page Manger.");
		}

		m_strImageServerBaseURL = _strImageServerBaseURL;
		m_hashControlsManagers = new XPHashtable();
	}

	/**
	 * Returns the image server base URL.
	 *
	 * @return The image server base url.
	 */
	public String GetImgSvrBaseURL()
	{
		return m_strImageServerBaseURL;
	}

	/**
	 * Returns the language ID for this page. (It is derived from the locale
	 * the client used when the
	 *
	 * @return The language ID (i.e. "en").
	 */
	public String GetLangID()
	{
		return m_strLangID;
	}

	/**
	 * Returns the locale that is being used by this page manager.
	 *
	 * @return The XPLocale for this page.
	 */
	public XPLocale GetLocale()
	{
		return m_xpLocale;
	}

	/**
	 * Returns the number of component managers currently registered with this
	 * page manager.
	 *
	 * @return An int that tells the clinent how many component managers are
	 * 		   registered.
	 */
	public int GetNumComponentMgrs()
	{
		return m_alRegisteredComponentMgrs.GetSize();
	}

	/**
	 * Returns the timezone being used by this page manager.
	 *
	 * @return The timezone.
	 */
	public XPTimeZone GetTimeZone()
	{
		return m_xpTimeZone;
	}

	/**
	 * This method allows you to register a component manager with the page
	 * manager. The order in which you register components is the order in
	 * which calls will be made when doing any looping operation--storing or
	 * printing. If you attempt to re-register the same component, it will
	 * overwrite the one that existed there before.
	 *
	 * @param _ptComponentMgr The component manger that you wish to register.
	 */
	public void RegisterComponentMgr(ComponentMgr _ptComponentMgr)
	{
		if (_ptComponentMgr != null)
		{
			if (m_alRegisteredComponentMgrs.Contains(_ptComponentMgr))
			{
				int nIndex = m_alRegisteredComponentMgrs.IndexOf(_ptComponentMgr);
				//m_alRegisteredComponentMgrs.Insert(nIndex, _ptComponentMgr);
				m_alRegisteredComponentMgrs.SetElement(nIndex, _ptComponentMgr);
			}
			else
			{
				m_alRegisteredComponentMgrs.Add(_ptComponentMgr);
			}
		}
	}

	/**
	 * This method stores all data entered by users to controls on this page.
	 * It does so by looping through the registered component mangers and
	 * asking them to save themselves. (The component managers, in turn, loop
	 * through the components and ask them to save themselves. The order in
	 * which the save is called is the same as the order in which the
	 * component managers were registered.
	 *
	 * @param _htRequestParameters A hashtable of the request parameters.
	 */
	public int StoreAllUserInput(XPHashtable _htRequestParameters)
	{
		int nError = RepostControl.PAGE_STATUS_VALID, nStatus;
		for (int i = 0; i < m_alRegisteredComponentMgrs.GetSize(); i++)
		{
			ComponentMgr ptComponentMgr = GetComponentMgr(i);
			nStatus = ptComponentMgr.StoreAllUserInput(_htRequestParameters);
			if (nStatus == RepostControl.PAGE_STATUS_INVALID){
				nError = nStatus;
			}
		}
		return nError;
	}
	
	/**
	 * This method validates all Page Controls owned by this PageMgr. It does so by
	 * looping through all registered ComponentMgrs which in turn loop through all their
	 * registered components (Page Controls). It returns an XPArrayList of all UIError
	 * messages generated by the various Page Controls.
	 * 
	 * @return XPArrayList of UIError objects.
	 */
	public XPArrayList Validate()
	{
		XPArrayList myErrors = new XPArrayList(0);
		for (int i = 0; i < m_alRegisteredComponentMgrs.GetSize(); i++)
		{
			ComponentMgr ptComponentMgr = GetComponentMgr(i);
			myErrors.AddRange(ptComponentMgr.Validate());
		}
		return myErrors;
	}
	
	/**
	 * This method provides js validation code for all Page Controls owned by this PageMgr.
	 * It does so by looping through all registered ComponentMgrs which in turn loop through
	 * all their registered components (Page Controls) to return their validation code. 
	 * 
	 * @return XPArrayList of UIError objects.
	 */
	public String GetJSValidationCode()
	{
		XPStringBuilder sb = new XPStringBuilder(1250);
		for (int i = 0; i < m_alRegisteredComponentMgrs.GetSize(); i++)
		{
			ComponentMgr ptComponentMgr = GetComponentMgr(i);
			sb.Append(ptComponentMgr.GetJSValidationCode());
		}
		return sb.ToString();
	}
	
	public void AddToValidationFunction(AFormDP _ptAFormDP)
	{
		for (int i = 0; i < m_alRegisteredComponentMgrs.GetSize(); i++)
		{
			ComponentMgr ptComponentMgr = GetComponentMgr(i);
			ptComponentMgr.AddToValidationFunction(_ptAFormDP);
		}
	}

	/**
	 * Returns a component manager given a valid index. This is useful if the
	 * client wants to loop through all the component managers on the page and
	 * perform some operation. To do this, please refer to the
	 * GetNumComponentMgrs method. If you pass in an invalid index, null is
	 * returned.
	 *
	 * @param _nIndex The index for a component manager registered with this
	 * 		  page manager.
	 *
	 * @return A component manager (or null).
	 */
	private ComponentMgr GetComponentMgr(int _nIndex)
	{
		Object oElement = null;

		try
		{
			oElement = m_alRegisteredComponentMgrs.GetElement(_nIndex);
		}
		catch (Exception e)
		{
			oElement = null;

			// Do nothing, just return null
		}

		if (oElement != null)
		{
			ComponentMgr oComponentMgr = (ComponentMgr) oElement;

			return oComponentMgr;
		}

		return null;
	}
	
	/**
     * Adds the error message to the array containing the rest of the 
     * error messages.
     * 
     * @param _strErrorMsg
     */
    public void AddErrorMessage(String _strErrorNum, String _strFileName)
    {
    	if (!m_xpErrorMsgs.Contains( _strErrorNum + " " + _strFileName)){
       	 	m_xpErrorMsgs.Add( _strErrorNum + " " + _strFileName);
    	}
    }

	 /**
     * Returns all the error messages for the Activity Apace
     * @return String - collection of error messages
     */
    public String GetErrorMessages(AActivitySpace _asOwner)
    {
    	XPStringBuilder strErrorMsgs = new XPStringBuilder();
        XPStringTokenizer strMsg;
        
		IXPEnumerator myXPEnumerator = m_xpErrorMsgs.GetEnumerator();
		
		// extract all the error number and file name pairs from the m_xpErrorMsgs hashtable
		// and append them together to return as a String
        while (myXPEnumerator.MoveNext()){
       		try{
				strMsg = new XPStringTokenizer ((String) myXPEnumerator.GetCurrent());
        		strErrorMsgs.Append(_asOwner.GetString(XPConvert.ToInteger(strMsg.GetNextToken()), strMsg.GetNextToken()) + "\n");
       		}
       		catch (Exception e){
				log.Error(e, e.getMessage());
       		}
        }
        // clear the collection of error msgs
        m_xpErrorMsgs.Clear();

        return strErrorMsgs.ToString();
    }
	
	
	/**
	 * Retreives an instance of a Page Control specified by _strMVCPageControlName 
	 * and returns it. Registers that Page Control with the ComponentMgr appropriate
	 * for it which is registered with this PageMgr instance. If no ComponentMgr of
	 * that type exists the Page Control creates a ComponentMgr of the correct type
	 * and that ComponentMgr is registered with the PageMgr.
	 *
	 * @param _strMVCPageControlName the name of the Page Control (i.e. HTMLTextBox)
	 * @return IPageControl an instance of the Page Control Object
	 */
	public IPageControl AddPageControl(String _strMVCPageControlName)
	{
		IPageControl myPageControl;
		ComponentMgr myComponentMgr;
		// if in standard mode, retrive component from PageControl table
		if (m_asAccessStyle.equals(AccessStyles.STANDARD)){
			myPageControl = (IPageControl) ASManager.INSTANCE.GetPageControl(_strMVCPageControlName);
		}
		// in 508 mode, retrivie the component with the MVC name appended with "_D"
		else
		{
			if ((myPageControl = (IPageControl) ASManager.INSTANCE.GetPageControl(_strMVCPageControlName + "_D")) == null)
			{
				myPageControl = (IPageControl) ASManager.INSTANCE.GetPageControl(_strMVCPageControlName);			
			}
		}
		
		// associate this type of page control with this type of ComponentMgr
		m_hashControlsManagers.PutElement(_strMVCPageControlName, myPageControl.GetComponentMgrName());
		
		// retrieve component manager and register page control
		//if (m_xpComponentManagers.ContainsKey(_strMVCPageControlName))
		if (m_xpComponentManagers.ContainsKey(myPageControl.GetComponentMgrName()))
		{
			//myComponentMgr = (ComponentMgr) m_xpComponentManagers.GetElement(_strMVCPageControlName);
			myComponentMgr = (ComponentMgr) m_xpComponentManagers.GetElement(myPageControl.GetComponentMgrName());
			myComponentMgr.RegisterComponent(myPageControl);
		}
		// create component manager, store in hashtable, and register page control
		else
		{
			myComponentMgr = myPageControl.GetComponentMgr(this);
			myComponentMgr.RegisterComponent(myPageControl);
			//m_xpComponentManagers.PutElement(_strMVCPageControlName, myComponentMgr);
			m_xpComponentManagers.PutElement(myPageControl.GetComponentMgrName(), myComponentMgr);
		}
		
		// register the component manager with the page
		RegisterComponentMgr(myComponentMgr);
		return myPageControl;
	}
	
	/**
	 * Return the Page Control of the type specified by _strMVCPageControlName (i.e. 
	 * HTMLTextBox) with the unique instance ID specified by _strUniqueInstanceID.
	 * 
	 * @param _strMVCPageControlName the name of the Page Control (i.e. HTMLTextBox)
	 * @param _strUniqueInstanceID
	 * 
	 *@return IPageControl
	 */
	public IPageControl GetPageControl(String _strMVCPageControlName, String _strUniqueInstanceID){
		ComponentMgr myComponentMgr;
		IPageControl myPageControl  = null;
		try
		{
			// retrieve the component manager
			String myComponentMgrName = (String) m_hashControlsManagers.GetElement(_strMVCPageControlName);
			myComponentMgr =  (ComponentMgr) m_xpComponentManagers.GetElement(myComponentMgrName);
			// retrieve the component using the unique instance id
			myPageControl =  (IPageControl) myComponentMgr.GetComponent(_strUniqueInstanceID);	
		}
		catch (Exception e)
		{
			log.Error(e, e.getMessage());
		}
		return myPageControl;
	}
	
	/**
	 * Return a ComponentMgr of the correct type for the given Page Control. 
	 * 
	 * @param _strMVCPageControlName the name of the Page Control (i.e. HTMLTextBox)
	 * @return The ComponentMgr of the correct type for the Page Control 
	 */
	public ComponentMgr GetComponentMgr(String _strMVCPageControlName){
		// retrieve the component manager with the MVC name
		String myComponentMgrName = (String) m_hashControlsManagers.GetElement(_strMVCPageControlName);
		return (ComponentMgr) m_xpComponentManagers.GetElement(myComponentMgrName);
	}
	
	/**
	 * Return the ImageServerConnectionURL member variable.
	 * 
	 * @return The image server connectin URL as a string 
	 */
	public String GetImageServerConnectionURL()
	{
		return m_strImageServerConnectionURL;
	}
}

