package com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.pagecontrols.framework.*;
import com.plumtree.uiinfrastructure.validators.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * HTMLDuration Page Control handles display and values of a page control
 * made up of a text box and a select dropdown. A bitwise operators (with
 * values defined in PTDurationCombo) dictates that time increments to display
 * in the select.
 * 
 * @author CaiusV
 */
public class HTMLDuration extends AHTMLComponent implements IPageControl
{

	/** for logging */
	private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
				"com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols.HTMLDuration");
				
	/** base name constant */
	private static final String STR_HTML_INPUT_BASE_NAME = "in_tx_pc";
	
	/** class name constant */
	public static final String STR_MVC_CLASS_NAME = "HTMLDuration";

	/** the textbox for this object */
	public HTMLTextBox m_TextBox;
	
	/** the selectdropdown for this object */
	public HTMLSelectDropDown m_SelectDropDown;
	
	/** the defautl value of the textbox */
	private String m_sDefaultValue;
	
	private String m_sTextBoxErrorMessage;
	
	/**
	 * Default constructor. Creates a new HTMLTextAndDropDown object.
	 */
	public HTMLDuration()
	{	
	}

	/**
	 * Initializes a new HTMLTextAndDropDown object. Creates and initializes
	 * the HTMLTextBox and HTMLSelectDropdown objects contained by this object.
	 * A bitwise operators (with values defined in PTDurationCombo) dictates 
	 * that time increments to display in the select.
	 * 
	 * @param _ptPageMgr	PageManager
	 * @param _strUniqueInstanceID	Unique Instance ID
	 * @param _intDisplayFlags bitwise operator
	 * @param _asOwner    AActivitySpace where this appears
	 * @param _strFormName name of the form on which this appears
	 * @param _strDefaultValue default value for textbox (must be an int)
	 */
	public void Init(PageMgr _ptPageMgr, 
				String _strUniqueInstanceID,
				int _intDisplayFlags,
				AActivitySpace _asOwner,
				String _strFormName,
				String _strDefaultValue)
	{
		try
		{
			XPConvert.ToInteger(_strDefaultValue);
		}
		catch(XPException exception)
		{
			log.Error(exception, "default value " + _strDefaultValue + " is not an int");
		}
		super.Init(_ptPageMgr, _strUniqueInstanceID);
		m_TextBox = (HTMLTextBox) _ptPageMgr.AddPageControl(HTMLTextBox.STR_MVC_CLASS_NAME);
		m_TextBox.Init(_ptPageMgr, _strUniqueInstanceID);
		m_TextBox.SetHTMLComponentSize("3");
		IValidator myIntValidator = new PositiveIntValidator(
				_strFormName,
				m_TextBox.GetHTMLComponentName(),
				_strDefaultValue);
		myIntValidator.SetErrorMessage("HTMLDuration must be a positive int");
		m_TextBox.AddValidator(myIntValidator);
		if (_asOwner.GetIsAccessStyleStandard())
		{
			m_TextBox.m_ptHTMLInput.SetOnChange(myIntValidator.GetJSFunction("this"));
		}
		m_TextBox.SetHTMLComponentValue(_strDefaultValue);
        XPArrayList myValues = new XPArrayList();
        XPArrayList myLabels = new XPArrayList();

        //Determine which options to display through bitwise comparison
        if ((_intDisplayFlags & PTDurationCombo.DISPLAYMILLISECONDS) != 0)
        {
            myValues.Add(XPConvert.ToString(PTDurationCombo.DISPLAYMILLISECONDS));
            myLabels.Add(_asOwner.GetString(113, ResourceConstants.INFRASTRUCTURE));
        }

        if ((_intDisplayFlags & PTDurationCombo.DISPLAYSECONDS) != 0)
        {
            myValues.Add(XPConvert.ToString(PTDurationCombo.DISPLAYSECONDS));
            myLabels.Add(_asOwner.GetString(114, ResourceConstants.INFRASTRUCTURE));
        }

        if ((_intDisplayFlags & PTDurationCombo.DISPLAYMINUTES) != 0)
        {
            myValues.Add(XPConvert.ToString(PTDurationCombo.DISPLAYMINUTES));
            myLabels.Add(_asOwner.GetString(115, ResourceConstants.INFRASTRUCTURE));
        }

        if ((_intDisplayFlags & PTDurationCombo.DISPLAYHOURS) != 0)
        {
            myValues.Add(XPConvert.ToString(PTDurationCombo.DISPLAYHOURS));
            myLabels.Add(_asOwner.GetString(116, ResourceConstants.INFRASTRUCTURE));
        }

        if ((_intDisplayFlags & PTDurationCombo.DISPLAYDAYS) != 0)
        {
            myValues.Add(XPConvert.ToString(PTDurationCombo.DISPLAYDAYS));
            myLabels.Add(_asOwner.GetString(117, ResourceConstants.INFRASTRUCTURE));
        }

        if ((_intDisplayFlags & PTDurationCombo.DISPLAYWEEKS) != 0)
        {
            myValues.Add(XPConvert.ToString(PTDurationCombo.DISPLAYWEEKS));
            myLabels.Add(_asOwner.GetString(118, ResourceConstants.INFRASTRUCTURE));
        }

        if ((_intDisplayFlags & PTDurationCombo.DISPLAYMONTHS) != 0)
        {
            myValues.Add(XPConvert.ToString(PTDurationCombo.DISPLAYMONTHS));
            myLabels.Add(_asOwner.GetString(119, ResourceConstants.INFRASTRUCTURE));
        }

        if ((_intDisplayFlags & PTDurationCombo.DISPLAYYEARS) != 0)
        {
            myValues.Add(XPConvert.ToString(PTDurationCombo.DISPLAYYEARS));
            myLabels.Add(_asOwner.GetString(120, ResourceConstants.INFRASTRUCTURE));
        }
        String[][] nameValuePairs = new String[myValues.GetSize()][2];
        for (int i = 0; i < myValues.GetSize(); i++ )
        {
        	nameValuePairs[i][0] = (String)myValues.GetElement(i);
        	nameValuePairs[i][1] = (String)myLabels.GetElement(i);
        }
		m_SelectDropDown = (HTMLSelectDropDown) _ptPageMgr.AddPageControl(HTMLSelectDropDown.STR_MVC_CLASS_NAME);
		m_SelectDropDown.Init(_ptPageMgr, _strUniqueInstanceID, nameValuePairs, "foo");
	}
	
	/**
	 * Returns the base name of this HTML Component.
	 *
	 * @return String
	 */
	protected String GetHTMLComponentBaseName()
	{
		return STR_HTML_INPUT_BASE_NAME;
	}


	/**
	 * Returns the HTML for this component.
	 *
	 * @return HTML for this object
	 */
	public HTMLElementCollection GetComponentHTML()
	{
		HTMLElementCollection retValue = new HTMLElementCollection();
		retValue.AddInnerHTMLElement(m_TextBox.GetComponentHTML());
		retValue.AddInnerHTMLString(CommonHTMLStrings.SPACE);
		if (null != GetHTMLComponentStyleClass())
		{
			m_TextBox.SetHTMLComponentStyleClass(GetHTMLComponentStyleClass());
			m_SelectDropDown.SetHTMLComponentStyleClass(GetHTMLComponentStyleClass());
		}
		retValue.AddInnerHTMLElement(m_SelectDropDown.GetComponentHTML());		
		return retValue;
	}
	
	/**
	 * Return the component values of the TextBox and SelectDropdown
	 * separated by a space.
	 */
	public String GetHTMLComponentValue()
	{
		return m_TextBox.GetHTMLComponentValue() + " " +
			m_SelectDropDown.GetHTMLComponentValue();
	}
	
	/**
	 * Return the value of this page contrl in seconds. so if the units is set to 
	 * minutes and the value of the textbox is 5 return 5 * 60 or 300.
	 * 
	 * @return the value in seconds
	 */
	public int GetValueInSeconds()
	{
		try {
			int returnValue = XPConvert.ToInteger(m_TextBox.GetHTMLComponentValue());
			int nDurationType = XPConvert.ToInteger(m_SelectDropDown.GetHTMLComponentValue());
			
			switch (nDurationType)
			{
			case PTDurationCombo.DISPLAYYEARS:
				returnValue = 12 * returnValue;
			
			case PTDurationCombo.DISPLAYMONTHS:
				returnValue = 4 * returnValue;
			
			case PTDurationCombo.DISPLAYWEEKS:
				returnValue = 7 * returnValue;
			
			case PTDurationCombo.DISPLAYDAYS:
				returnValue = 24 * returnValue;
			
			case PTDurationCombo.DISPLAYHOURS:
				returnValue = 60 * returnValue;
			
			case PTDurationCombo.DISPLAYMINUTES:
				returnValue = 60 * returnValue;
			}
			return returnValue;
		}
		catch (XPException e)
		{
			log.Warn(e, e.getMessage());
			return 0;
		}
	}
	
	/**
	 * This method is used by the ASManager to return new instances of managed
	 * objects.
	 *
	 * @return A new instance of the managed class (i.e. return new Foo();)
	 */
	public Object Create()
	{
		return new HTMLDuration();
	}
	
	/**
	 * Return the name of the MVC object.
	 *
	 * @return the name of the object.
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * For every Validator associated with this object execute it's Validate method.
	 * Return an Arraylist of UIError objects generated from executing the Validators.
	 * Return and empty Arraylist if no problems were found.
	 * 
	 * @return Arraylist of UIErrors.
	 */
	public XPArrayList Validate()
	{
		XPArrayList myErrorList = super.Validate();
		if (m_TextBox.Validate().GetSize() > 0) {
			myErrorList.Add(m_TextBox.Validate().GetElement(0));
		}
		return myErrorList;
	}

	/**
	 * Return a string made up of all the JSValidationCode for each of the
	 * Validator's associated with this component.
	 * 
	 * @return JavaScript validation code
	 */
	public String GetJSValidationCode()
	{
		XPStringBuilder sb = new XPStringBuilder(1250);
		sb.Append(super.GetJSValidationCode());
		sb.Append(m_TextBox.GetJSValidationCode());
		return sb.toString();
	}
		
	/**
	 * @return Returns the m_sTextBoxErrorMessage.
	 */
	public String GetTextBoxErrorMessage() {
		return m_sTextBoxErrorMessage;
	}
	/**
	 * @param textBoxErrorMessage The m_sTextBoxErrorMessage to set.
	 */
	public void SetTextBoxErrorMessage(String textBoxErrorMessage) {
		m_sTextBoxErrorMessage = textBoxErrorMessage;
	}
}
