package com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols;

import com.plumtree.openfoundation.internationalization.XPResourceManager;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.arraywrapper.IPTGrowableSortedArrayWrapperRO;
import com.plumtree.uiinfrastructure.pagecontrols.framework.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * HTMLNameDescLang page control handles display and values of a page control
 * made up of a text box text area and a select dropdown. The select is
 * populated with default language options. This is a common construct
 * throughout the admin ui.
 * 
 * @author CaiusV
 */
public class HTMLNameDescLang extends AHTMLComponent implements IPageControl
{

	/** for logging */
	private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
				"com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols.HTMLNameDescLang");
				
	/** base name constant */
	private static final String STR_HTML_INPUT_BASE_NAME = "in_tx_pc";
	
	/** class name constant */
	public static final String STR_MVC_CLASS_NAME = "HTMLNameDescLang";

	/** the textbox for this object */
	public HTMLTextBox m_TextBox;
	
	/** the selectdropdown for this object */
	public HTMLSelectDropDown m_SelectDropDown;
	
	/** the textarea for this object */
	public HTMLTextArea m_TextArea;
	
	/** primary language */
	private String m_strPrimaryLang;
	
	/** stores already selected langs */
	private IPTGrowableSortedArrayWrapperRO m_ptGSArrayWrapper;
	
	/** array of language options */
	private XPArrayList m_alOptions;
	
	/**
	 * Default constructor. Creates a new HTMLTextAndDropDown object.
	 */
	public HTMLNameDescLang()
	{	
	}

	/**
	 * Initializes a new HTMLNameDescLang object. Creates and initializes
	 * the HTMLTextBox, HTMLTextArea and HTMLSelectDropdown objects 
	 * contained by this object. Populates the default languages for the
	 * select.
	 * 
	 * @param _ptPageMgr The component manager for this component
	 * @param _strUniqueInstanceID The unique ID for this component.
	 * @param _arrDropDownValueText The dropdown values
	 */
	public void Init(PageMgr _ptPageMgr, 
				String _strUniqueInstanceID,
				XPResourceManager _ptResourceManager,
				String _strPrimaryLang)
	{
		super.Init(_ptPageMgr, _strUniqueInstanceID);
		// the primary language
		m_strPrimaryLang = _strPrimaryLang;
		// set up the textbox page control
		m_TextBox = (HTMLTextBox) _ptPageMgr.AddPageControl(HTMLTextBox.STR_MVC_CLASS_NAME);
		m_TextBox.Init(_ptPageMgr, _strUniqueInstanceID);
		m_SelectDropDown = (HTMLSelectDropDown) _ptPageMgr.AddPageControl(HTMLSelectDropDown.STR_MVC_CLASS_NAME);
		// set up the dropdown page control	
		SetLangChoices(_ptResourceManager);
		m_SelectDropDown.Init(_ptPageMgr, _strUniqueInstanceID, m_alOptions, "");
		// set up the htmltextarea page control
		m_TextArea =
			(HTMLTextArea) m_ptPageMgr.AddPageControl(HTMLTextArea.STR_MVC_CLASS_NAME);
		m_TextArea.Init(m_ptPageMgr, _strUniqueInstanceID, 50, 3);
	}
	
	/**
	 * Returns the base name of this HTML Component.
	 *
	 * @return String
	 */
	protected String GetHTMLComponentBaseName()
	{
		return STR_HTML_INPUT_BASE_NAME;
	}


	/**
	 * Returns the HTML for this component.
	 *
	 * @return HTML for this object
	 */
	public HTMLElementCollection GetComponentHTML()
	{
		HTMLElementCollection retValue = new HTMLElementCollection();
		if (null != GetHTMLComponentStyleClass())
		{
			m_TextBox.SetHTMLComponentStyleClass(GetHTMLComponentStyleClass());
			m_SelectDropDown.SetHTMLComponentStyleClass(GetHTMLComponentStyleClass());
			m_TextArea.SetHTMLComponentStyleClass(GetHTMLComponentStyleClass());
		}
		retValue.AddInnerHTMLElement(m_TextBox.GetComponentHTML());
		retValue.AddInnerHTMLString(CommonHTMLStrings.SPACE);
		retValue.AddInnerHTMLElement(m_TextArea.GetComponentHTML());
		retValue.AddInnerHTMLString(CommonHTMLStrings.SPACE);
		retValue.AddInnerHTMLElement(m_SelectDropDown.GetComponentHTML());		
		return retValue;
	}
	
	/**
	 * Return the component values of the TextBox and SelectDropdown
	 * separated by a space.
	 */
	public String GetHTMLComponentValue()
	{
		return m_TextBox.GetHTMLComponentValue() + " " +
			m_SelectDropDown.GetHTMLComponentValue() + " " + m_TextArea.GetHTMLComponentValue();
	}
	
	/**
	 * This method is used by the ASManager to return new instances of managed
	 * objects.
	 *
	 * @return A new instance of the managed class (i.e. return new Foo();)
	 */
	public Object Create()
	{
		return new HTMLNameDescLang();
	}
	
	/**
	 * Return the name of the MVC object.
	 *
	 * @return the name of the object.
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}
	
	/**
	 * For every Validator associated with this object execute it's Validate method.
	 * Return an Arraylist of UIError objects generated from executing the Validators.
	 * Return and empty Arraylist if no problems were found.
	 * 
	 * @return Arraylist of UIErrors.
	 */	
	public XPArrayList Validate()
	{
		XPArrayList myErrorList = super.Validate();
		if (m_TextBox.Validate().GetSize() > 0) {
			myErrorList.Add(m_TextBox.Validate().GetElement(0));
		}
		if (m_TextArea.Validate().GetSize() > 0) {
			myErrorList.Add(m_TextArea.Validate().GetElement(0));
		}
		if (m_SelectDropDown.Validate().GetSize() > 0) {
			myErrorList.Add(m_SelectDropDown.Validate().GetElement(0));
		}
		return myErrorList;
	}

	/**
	 * Return a string made up of all the JSValidationCode for each of the
	 * Validator's associated with this component.
	 * 
	 * @return JavaScript validation code
	 */
	public String GetJSValidationCode()
	{
		XPStringBuilder sb = new XPStringBuilder(1250);
		sb.Append(super.GetJSValidationCode());
		sb.Append(m_TextBox.GetJSValidationCode());
		sb.Append(m_TextArea.GetJSValidationCode());
		sb.Append(m_SelectDropDown.GetJSValidationCode());
		return sb.toString();
	}
	
	/**
	 * Set the options of the member variable m_SelectDropDown with the appropriate
	 * languages.
	 * 
	 * @param _ptResourceManager XPResourceManager with language options
	 */
	public void SetLangChoices(
			XPResourceManager _ptResourceManager)
	{
		XPArrayList usedLangs = new XPArrayList();
		m_alOptions = new XPArrayList();
		usedLangs.Add(m_strPrimaryLang);

		if (null != m_ptGSArrayWrapper)
		{
			for (int i = 0; i < m_ptGSArrayWrapper.GetCount(); ++i)
			{
				String strDisallowedLang = (String) m_ptGSArrayWrapper.GetItem(i, 3);
				strDisallowedLang = XPStringUtility.ToLowerCase(strDisallowedLang);
				usedLangs.Add(strDisallowedLang);
			}
		}
		
		String[] asKnownLangs = _ptResourceManager.GetSupportedLanguages();
		
		int size = asKnownLangs.length;
		
		for (int i = 0; i < size; i++) {
			if (!usedLangs.Contains(asKnownLangs[i]))
			{
				HTMLOption myOption = new HTMLOption(asKnownLangs[i]);
	            myOption.AddInnerHTMLString(
	            		_ptResourceManager.GetDisplayLocaleFromLocaleID(asKnownLangs[i],
	            		m_ptPageMgr.GetLangID()));
	            m_alOptions.Add(myOption);
			}
		}
		m_SelectDropDown.SetOptions(m_alOptions);
	}

	/**
	 * Returns member variable m_ptGSArrayWrapper.
	 * 
	 * @return m_ptGSArrayWrapper member variable
	 */
	public IPTGrowableSortedArrayWrapperRO GetData() {
		return m_ptGSArrayWrapper;
	}
	
	/**
	 * Sets the member variable m_ptGSArrayWrapper
	 * 
	 * @param data
	 */
	public void SetData(IPTGrowableSortedArrayWrapperRO data) {
		m_ptGSArrayWrapper = data;
	}
	
	/**
	 * Set the primary language member variable.
	 * 
	 * @param _sPrimaryLang the primary language
	 */
	public void SetPrimaryLanguage(String _sPrimaryLang) {
		m_strPrimaryLang = _sPrimaryLang;
	}
}
