package com.plumtree.uiinfrastructure.pagecontrols.jsdatepicker;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.OpenLogService;
import com.plumtree.openlog.OpenLogger;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.pagecontrols.datepicker.*;
import com.plumtree.uiinfrastructure.pagecontrols.framework.*;
import com.plumtree.uiinfrastructure.pagecontrols.jsdatetimepicker.JSDateTimePickerMgr;
import com.plumtree.xpshared.htmlelements.*;
import com.plumtree.xpshared.jsutil.JSRegistry;

/**
 *
 * @author John Osborne, John Hogan
 */
public class JSDatePicker extends ADatePickerComponent implements IPageControl {
	
	/** The name for the HTML div tag that the datepicker uses. */
	private static final String STR_DP_DIV_NAME = "dpDiv";

	/** The base name for the JS variable name of the date-picker. */
	private static final String STR_DP_VAR_NAME = "varName";

	/** The base name of the date-picker input textbox. */
	public static final String STR_DP_DATE_INPUT_NAME = "in_tx_DPInput";

	/** The base name of the date-picker button. */
	private static final String STR_DP_BUTTON_INPUT_NAME = "in_bu_DPButton";

	/** The name for the calendar button. */
	private String m_strButtonName;

	/** The name for the date input text box */
	private String m_strDateInputName;

	/** The name of the form in which the datepicker resides */
	private String m_strFormName;

	/** The String value of the currently selected date. */
	private String m_strSelectedDate;

	/** The name of the date-picker variables. */
	private String m_strVarName;

	/** A date formatter to help with formatting and parsing. */
	private XPDateFormat m_dfDateFormat;

	/** MVC class name for page control */
	public static final String STR_MVC_CLASS_NAME = "DatePicker";
	
	/** Variable to store a unique ID for the DatePickMgr associated with this DatePicker **/
	public String m_strDatePickerMgrID = "";
	
	/** logger */
	private static OpenLogger log = 
		OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
			"com.plumtree.uiinfrastructure.pagecontrols.jsdatepicker.JSDatePicker");
	
	/** Static EN-us locale used for formatting dates exchanged with client side javascript */
	protected static final XPLocale m_oUSLocale = new XPLocale("EN","us");

	/** Create a new instance of JSDatePicker */
	public JSDatePicker()
	{
	}

	/**
	 * Initialize the JSDatePicker.
	 * 
	 * @param _strFormName The name of the form in which the datepicker resides
	 * @param _ptPageMgr The datepicker page manager for this instance of
	 *                   the datepicker
	 * @param _strUniqueInstanceID The string that will be appended to
	 *                   variables to make them unique
	 */
	public void Init(
		String _strFormName,
		PageMgr _ptPageMgr,
		String _strUniqueInstanceID) 
	{
		super.Init(_ptPageMgr, _strUniqueInstanceID);

		if (null == _strUniqueInstanceID)
		{
			_strUniqueInstanceID = EMPTY;
		}

		m_strFormName = _strFormName;
		
		m_dfDateFormat = XPDateFormat.GetDateInstance(
				XPDateFormat.MEDIUM,
				m_oUSLocale,
				_ptPageMgr.GetTimeZone());

		m_strSelectedDate 	= EMPTY;
		m_strDateInputName 	= AppendUniqueIDToElementName(STR_DP_DATE_INPUT_NAME);
		m_strButtonName 	= AppendUniqueIDToElementName(STR_DP_BUTTON_INPUT_NAME);
		m_strVarName 		= AppendUniqueIDToElementName(STR_DP_VAR_NAME);
	}

	/**
	 * Initialize a new JSDatePicker object.
	 *
	 * @param _strFormName The name of the form in which the datepicker resides
	 * @param _strDefaultDate The initial date selected
	 * @param _ptPageMgr The datepicker page manager for this instance of
	 *                   the datepicker
	 * @param _strUniqueInstanceID The string that will be appended to
	 *                   variables to make them unique
	 */
	public void Init(
		String _strFormName,
		String _strDefaultDate,
		PageMgr _ptPageMgr,
		String _strUniqueInstanceID)
	{
		super.Init(_ptPageMgr, _strUniqueInstanceID);

		if (null == _strUniqueInstanceID) 
		{
			_strUniqueInstanceID = EMPTY;
		}
		m_strFormName = _strFormName;
		m_dfDateFormat = XPDateFormat.GetDateInstance(
				XPDateFormat.MEDIUM,
				m_oUSLocale,
				_ptPageMgr.GetTimeZone());

		if ((_strDefaultDate == null) || _strDefaultDate.equals(EMPTY))
		{
			m_strSelectedDate = EMPTY;
		}
		else 
		{
			try 
			{
				m_dfDateFormat.Parse(_strDefaultDate);
			}
			catch (XPException e)
			{
				log.Error(e, "invalid date format " + _strDefaultDate);
				m_strSelectedDate = EMPTY;
			}
		}

		m_strDateInputName =
			AppendUniqueIDToElementName(STR_DP_DATE_INPUT_NAME);
		m_strButtonName = AppendUniqueIDToElementName(STR_DP_BUTTON_INPUT_NAME);
		m_strVarName = AppendUniqueIDToElementName(STR_DP_VAR_NAME);
	}

	/**
	 * Initailize a new JSDatePicker object.
	 *
	 * @param _strFormName The name of the form in which the datepicker resides
	 * @param _dtDefaultTime The initial date selected
	 * @param _ptPageMgr The page manager where this control is registered
	 * @param _strUniqueInstanceID The string that will be appended to
	 *                   variables to make them unique
	 */
	public void Init(
		String _strFormName,
		XPDateTime _dtDefaultTime,
		PageMgr _ptPageMgr,
		String _strUniqueInstanceID)
	{
		super.Init(_ptPageMgr, _strUniqueInstanceID);
		if (null == _strUniqueInstanceID)
		{
			_strUniqueInstanceID = EMPTY;
		}

		m_strFormName = _strFormName;

		m_dfDateFormat = XPDateFormat.GetDateInstance(
				XPDateFormat.MEDIUM,
				m_oUSLocale,
				_ptPageMgr.GetTimeZone());

		if (null != _dtDefaultTime) 
		{
			try 
			{
				m_strSelectedDate = m_dfDateFormat.Format(_dtDefaultTime);
			} 
			catch (XPException e) 
			{
				log.Error(e, "invalid date " + _dtDefaultTime,  _dtDefaultTime);
				m_strSelectedDate = EMPTY;
			}
		} 
		else
		{
			m_strSelectedDate = EMPTY;
		}

		m_strDateInputName 	= AppendUniqueIDToElementName(STR_DP_DATE_INPUT_NAME);
		m_strButtonName 	= AppendUniqueIDToElementName(STR_DP_BUTTON_INPUT_NAME);
		m_strVarName	 	= AppendUniqueIDToElementName(STR_DP_VAR_NAME);
	}

	/**
	 * This method returns the datepicker HTML.
	 *
	 * @return The HTMLElementCollection that is the datepicker.
	 */
	public HTMLElementCollection GetComponentHTML() 
	{
		HTMLElementCollection retHTMLCollection = new HTMLElementCollection();
		HTMLScript myScript;
		XPCalendar xpCalendar;
		int nFirstDayOfWeek;

		xpCalendar = XPCalendar.GetInstance(
				m_ptPageMgr.GetTimeZone(),
				m_ptPageMgr.GetLocale());
		nFirstDayOfWeek = xpCalendar.GetFirstDayOfWeek();

		//Add the script elements
		myScript = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);

		XPStringBuilder sb = new XPStringBuilder(200);
		sb.Append("var " + m_strVarName + " = new PTDatepicker(");
		sb.Append("\"" + STR_DP_DIV_NAME + m_strDatePickerMgrID +  "\","); // div name
		sb.Append("\"" + m_strVarName + "\",");

		// var name - this needs to be unique
		sb.Append("\"" + m_strFormName + "\","); // form name
		sb.Append("\"" + m_strDateInputName + "\",");

		// name for the input text box
		sb.Append("\"" + m_strButtonName + "\","); // the name for the button
		sb.Append("\"" + m_strSelectedDate + "\","); // the default value
		sb.Append("\"" + m_dfDateFormat.ToPattern() + "\",");

		// Date format used to communicate with the Date Picker
		sb.Append(nFirstDayOfWeek).Append(","); // first day of the week
		sb.Append(
			"\""
				+ m_ptPageMgr.GetImgSvrBaseURL()
				+ "plumtree/common/private/js/"
				+ JSDatePickerMgr.STR_JS_COMPONENT_NAME
				+ "/"
				+ "LATEST"
				+ "/images/"
				+ "\",");
		sb.Append("false,"); // No Validation
		sb.Append("\"\",");
		sb.Append("\"\",");
		sb.Append("\"\",");
		sb.Append("\"\",");
		sb.Append("\"\",");
		sb.Append("\"\",");
		sb.Append(
			"\""
				+ m_ptPageMgr.GetLocale().GetLanguage()
				+ "_"
				+ m_ptPageMgr.GetLocale().GetCountry()
				+ "\"");

		//Locale
		sb.Append(");");

		myScript.AddInnerHTMLString(sb.ToString());
		retHTMLCollection.AddInnerHTMLElement(myScript);

		myScript = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);
		myScript.AddInnerHTMLString(m_strVarName + ".writeControl();");
		retHTMLCollection.AddInnerHTMLElement(myScript);

		return retHTMLCollection;
	}

	/**
     * This method returns the datepicker HTML.
     *
	 * @param _strPickerType the picker type.
     * @return The HTMLElementCollection that is the datepicker.
	 */
	public HTMLElementCollection GetComponentHTML(String _strPickerType) {
		HTMLElementCollection retHTMLCollection = new HTMLElementCollection();
		HTMLScript myScript;
		XPCalendar xpCalendar;
		int nFirstDayOfWeek;

		xpCalendar = XPCalendar.GetInstance(m_ptPageMgr.GetTimeZone(),
				m_ptPageMgr.GetLocale());
		nFirstDayOfWeek = xpCalendar.GetFirstDayOfWeek();

		//Add the script elements
		myScript = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);

		XPStringBuilder sb = new XPStringBuilder(200);
		sb.Append("var " + m_strVarName + " = new PTDatepicker(");
		sb.Append("\"" + STR_DP_DIV_NAME + m_strDatePickerMgrID + "\","); // div name
		sb.Append("\"" + m_strVarName + "\",");

		// var name - this needs to be unique
		sb.Append("\"" + m_strFormName + "\","); // form name
		sb.Append("\"" + m_strDateInputName + "\",");

		// name for the input text box
		sb.Append("\"" + m_strButtonName + "\","); // the name for the button
		sb.Append("\"" + m_strSelectedDate + "\","); // the default value
		sb.Append("\"" + m_dfDateFormat.ToPattern() + "\",");

		// Date format used to communicate with the Date Picker
		sb.Append(nFirstDayOfWeek).Append(","); // first day of the week
		sb.Append(
			"\""
				+ m_ptPageMgr.GetImgSvrBaseURL()
				+ "plumtree/common/private/js/"
				+ JSDatePickerMgr.STR_JS_COMPONENT_NAME
				+ "/"
				+ "LATEST"
				+ "/images/"
				+ "\",");
		sb.Append("false,"); // No Validation
		sb.Append("\"\",");
		sb.Append("\"\",");
		sb.Append("\"\",");
		sb.Append("\"\",");
		sb.Append("\"\",");
		sb.Append("\"\",");
		sb.Append(
			"\""
				+ m_ptPageMgr.GetLocale().GetLanguage()
				+ "_"
				+ m_ptPageMgr.GetLocale().GetCountry()
				+ "\"");

		//Locale
		sb.Append(");");

		myScript.AddInnerHTMLString(sb.ToString());
		retHTMLCollection.AddInnerHTMLElement(myScript);

		myScript = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);
		myScript.AddInnerHTMLString(m_strVarName + ".writeControl();");
		retHTMLCollection.AddInnerHTMLElement(myScript);

		return retHTMLCollection;
	}

	/**
	 * Provides a means of retrieving the HTML Input name for the date portion
	 * of the date picker.  This allows for using javascript to
	 * programmatically set input values, for example.
	 *
	 * @return String
	 */
	public String GetDateInputName() {
		return m_strDateInputName;
	}

	/**
	 * Returns the name of the PTDatepicker JavaScript variable name
	 *
	 * @return String
	 */
	public String GetPTDatepickerVarName() {
		return m_strVarName;
	}

	/**
	 * Returns the date selected in the date picker as a string
	 *
	 * @return String
	 */
	public String GetSelectedDateAsString() {
		return m_strSelectedDate;
	}

	/**
	 * Stores the user input after the request is sent back.
	 *
	 * @param _htRequestParameters The XPHashtable of request name/value pairs.
	 */
	public int StoreUserInput(XPHashtable _htRequestParameters)
	{
		String[] data = (String[]) _htRequestParameters.GetElement(m_strDateInputName);
		if ((null != data) && !(data[0].equals(EMPTY))) 
		{
			m_strSelectedDate = data[0];
		} 
		else 
		{
			m_strSelectedDate = EMPTY;
		}
		return RepostControl.PAGE_STATUS_VALID;
	}

	/**
	 * This function returns an XPDateTime object that represents the date and
	 * time the user has chosen using the date/time-picker HTML control.
	 *
	 * @return XPDateTime
	 */
	public XPDateTime GetDateAsDateTime() {
		XPDateTime xpdtResult = null;

		try 
		{
		    XPDateFormat myDateFormat = XPDateFormat.GetDateInstance(
					XPDateFormat.MEDIUM,
					m_oUSLocale,
					m_ptPageMgr.GetTimeZone());
		    String myDateString = GetSelectedDateAsString();
		    xpdtResult = myDateFormat.Parse(myDateString);
		} 
		catch (XPException e) 
		{
		    log.Error(e, "selected date: "+GetSelectedDateAsString()+" is invalid");
			return null;
		}

		return xpdtResult;
	}
	
	public void SetDatePickerMgrID(String strDatePickerMgrID)
	{
		m_strDatePickerMgrID = strDatePickerMgrID;
	}

	/**
	 * This method is used by the ASManager to return new instances of managed
	 * objects.
	 *
	 * @return A new instance of the managed class (i.e. return new Foo();)
	 */
	public Object Create()
	{
		return new JSDatePicker();
	}

	/**
	 * Return the name of the MVC object.
	 *
	 * @return the name of the object.
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}
	
	public ComponentMgr GetComponentMgr(PageMgr _ptPageMgr)
	{
		JSRegistry ptJSRegistry = JSRegistry.GetRegistry(_ptPageMgr.GetImgSvrBaseURL(),
					_ptPageMgr.GetImageServerConnectionURL());
		return new JSDateTimePickerMgr(ptJSRegistry, new JSDatePickerVersionMgr(), _ptPageMgr);
	}
	
	/**
	 * Return the name of the component manager responsible for this page control
	 */
	public String GetComponentMgrName()
	{
		return JSDateTimePickerMgr.STR_CLASS_NAME;
	}
	
	/**
	 * Currently validation on the date/time picker page controls is not implemented.
	 * This method returns null.
	 */
	public XPArrayList Validate()
	{
		return null;
	}
	
	/**
	 * Currently validation on the date/time picker page controls is not implemented.
	 * This method returns null.
	 */
	public XPArrayList GetValidators()
	{
		return null;
	}
}
