package com.plumtree.uiinfrastructure.statichelpers;

import com.plumtree.xpshared.htmlelements.*;

/**
 * This class is used to insert anchors in localized strings using placeholders.
 * Strings should have placeholders {0} and {1} around the section that should be
 * turned into a link.  This class does not handle escaped placeholders (\\{0}).
 */
public class AnchorInLocalizedString
{
    private static final String PLACEHOLDER0 = "{0}";
	private static final String PLACEHOLDER1 = "{1}";

	/**
	 * This method returns the section of the string before the place holder.
	 * 
	 * @param String the entire localized string
	 * @return String the section before the first placeholder.  "" If there
	 * is no place holder.
	 */
	public static String GetStringBeforeAnchor(String strBaseText)
	{
		int nPlaceHolderPosition = strBaseText.indexOf(PLACEHOLDER0);
		if (nPlaceHolderPosition == -1)
		{
			return "";
		}

		return strBaseText.substring(0, nPlaceHolderPosition);
	}

	/**
	 * This method returns the section of the string after the place holder.
	 * 
	 * @param String the entire localized string
	 * @return String the section before the first placeholder.
	 */
	public static String GetStringAfterAnchor(String strBaseText)
	{
		int nPlaceHolderPosition = strBaseText.indexOf(PLACEHOLDER1);
		if (nPlaceHolderPosition == -1)
		{
			return "";
		}

		return strBaseText.substring(nPlaceHolderPosition + PLACEHOLDER1.length());
	}

	/**
	 * This method returns the HTMLAnchor generated from the section of the
	 * string between the {0} and {1} placeholders.
	 * 
	 * @param String the entire localized string
	 * @param String the URL of the anchor
	 * @return HTMLAnchor the anchor generated from the text inbetween {0} and {1}
	 */
	public static HTMLAnchor GetAnchor(String strBaseText, String strURL)
	{
		int nFirstPlaceHolderPosition = strBaseText.indexOf(PLACEHOLDER0);
		int nSecondPlaceHolderPosition = strBaseText.indexOf(PLACEHOLDER1);
		if ((nFirstPlaceHolderPosition == -1) || (nFirstPlaceHolderPosition == -1))
		{
			return null;
		}

		String strAnchorText = strBaseText.substring(nFirstPlaceHolderPosition + PLACEHOLDER0.length(),
			nSecondPlaceHolderPosition);

		HTMLAnchor result = new HTMLAnchor(strURL);
		result.AddInnerHTMLEncodedString(strAnchorText);
		result.SetTitle(strAnchorText);

		return result;
	}

	/**
	 * This method returns an HTMLElementCollection containing the string with the
	 * anchor element inserted between the {0} and {1} placeholders.
	 * 
	 * @param String the entire localized string
	 * @param String the URL of the anchor
	 * @return HTMLElementCollection the String with the anchor generated from the text inbetween {0} and {1}
	 */
	public static HTMLElementCollection GetAnchorInString(String strBaseText, String strURL)
	{
		HTMLElementCollection result = new HTMLElementCollection();

		result.AddInnerHTMLEncodedString(GetStringBeforeAnchor(strBaseText));

		result.AddInnerHTMLElement(GetAnchor(strBaseText, strURL));

		result.AddInnerHTMLEncodedString(GetStringAfterAnchor(strBaseText));

		return result;
	}

	/**
	 * This method takes the string with the anchor element inserted between
	 * the {0} and {1} placeholders and adds it to the given element.
	 * 
	 * @param String the entire localized string
	 * @param String the URL of the anchor
	 * @param HTMLElement the element to add the String with the anchor generated from the text inbetween {0} and {1}
	 */
	public static void AddAnchorInStringToElement(String strBaseText, String strURL, HTMLElement element)
	{
		element.AddInnerHTMLEncodedString(GetStringBeforeAnchor(strBaseText));

		element.AddInnerHTMLElement(GetAnchor(strBaseText, strURL));

		element.AddInnerHTMLEncodedString(GetStringAfterAnchor(strBaseText));
	}
}
