package com.plumtree.uiinfrastructure.statichelpers;

import com.plumtree.openlog.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.errordisplay.*;

/**
 * @author robertz
 * 
 * Static helper functions
 */
public class GeneralHelpers
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.statichelpers.GeneralHelpers");

	public final static String FormatPath(String _strPath, String _strDelimiter, boolean _bForceEndsWith)
	{
		
		XPStringBuilder buf = new XPStringBuilder(_strPath);
		
		int i = 0;
		int nStart = 0;
		int nEnd;
		boolean bAlreadyEndsWith = false;
		
		//SS - Iterate through our string
		while (i < buf.GetLength()) 
		{
			//SS - We see the start of our replaceable tokens ('\' or '/')
			if ((buf.CharAt(i) == '\\') || (buf.CharAt(i) == '/')) {
				nStart = i;
				//SS - Iterate through our replaceable tokens
				while ((buf.CharAt(i) == '\\') || (buf.CharAt(i) == '/')) {
					if (++i == buf.GetLength()) break;
				} 
				nEnd = i;                                                                                                                                                       ;
				//SS - Replace the tokens with our delimiter and start our
				//counter back at the new buffer position (because we
				//have changed our string)
				buf.Replace(nStart, nEnd, _strDelimiter);
				i = nStart + _strDelimiter.length() - 1;
			}
			i++;
		}
		
		//SS - Now we check to correct for ends-with functionality
		String strIntermediate = buf.ToString();
		XPStringBuilder bufIntermediate = new XPStringBuilder(strIntermediate);
		nEnd = strIntermediate.length();
		nStart = strIntermediate.length() - _strDelimiter.length();
		
		//SS - Check to see if the string already ends with our delimiter
		if (XPStringUtility.CompareIgnoreCase(bufIntermediate.Substring(nStart, nEnd), _strDelimiter) == 0) bAlreadyEndsWith = true;
		//SS - If we have to end with the delimiter and it's already there, return the string.
		//Otherwise add it and return the string.
		if(_bForceEndsWith) {
			if(bAlreadyEndsWith) return strIntermediate;
			buf.Append(_strDelimiter);
			return(buf.ToString());
		}
		//SS - We do not want the string to return with the delimiter so
		//return the string if it doesn't end with the delimiter,
		//or remove the delimiter if the string does end with it (the delimiter)
		if(!bAlreadyEndsWith) return strIntermediate;
		buf.Remove(nStart, nEnd);
		return(buf.ToString());

	}
	
    /**
     * replace substrings within string.
     *
     * @param s     string to be modified
     * @param sub   sub-string to search for
     * @param with  string to replace sub-string
     * @return string replaced with new sub-string or the unmodified
     *         string if no match could be found
     */
    public final static String Replace(String s, String sub, String with)
    {
        int c = 0;
        int i = s.indexOf(sub, c);

        if (i == -1)
        {
            return s;
        }

        XPStringBuilder buf = new XPStringBuilder(s.length() + with.length());

        do
        {
            buf.Append(s.substring(c, i));
            buf.Append(with);
            c = i + sub.length();
        }
        while ((i = s.indexOf(sub, c)) != -1);

        if (c < s.length())
        {
            buf.Append(s.substring(c, s.length()));
        }

        return buf.ToString();
    }

    /**
     * Adds spaces into long strings so that it can be displayed with wrapping
     * in HTML without having to truncate.
     */
    public static String BreakDownLongWordsInString(String strIn, int maxCharsPerWord)
    {
        XPStringTokenizer tokenizer = new XPStringTokenizer(strIn, " ");
        strIn = "";

        String strToken;
        String strTokenSubstring;

        while (tokenizer.HasMoreTokens())
        {
            strToken = tokenizer.GetNextToken();

            if (strToken.length() <= maxCharsPerWord)
            {
                strIn += (strToken + " ");
            }
            else
            {
                while (strToken.length() > maxCharsPerWord)
                {
                    strTokenSubstring = strToken.substring(0, maxCharsPerWord);
                    strIn += (strTokenSubstring + " ");
                    strToken = strToken.substring(maxCharsPerWord, strToken.length());
                }

                strIn += (strToken + " ");
            }
        }

        return strIn;
    }

    /**
     * Description: This function returns a shortened version of a string that
     * is too long. It will shorten the string to intMaxNumberOfCharacters and
     * append a "..."
     *
     * @param strInput           String to check.
     * @param nMaxChars          Maximum number of characters in a string before
     *                            truncation.  The length truncated string will
     *                            be no longer than this number plus an additional
     *                            3 characters for the '...'.
     * @param bMaximumWordLength variable is to specify that you want to
     *                            shorten the string only if it has a word in it
     *                            longer than nMaxChars.
     *
     * bMaximumWordLength is used to determine whether or not any word in the
     * string will be too long and cause the HTML to look bad.
     * 
     * Warning: If the string is going to be HTML Encoded, remember to truncate it
     * first, and then HTML encode it.  Otherwise you can get truncated international
     * language codes such as &#... instead of an actual character.
     * 
     * Examples
     * --------
     *      TruncateLongString("123456789 123456 12345678 123", 10, True)
     *          Returns: "123456789 123456 12345678 123"
     *      TruncateLongString("123456789 123456 12345678 123", 10, False)
     *          Returns: "123456789..."
     *      TruncateLongString("123456789 123456 1234567890123 123", 12, True)
     *          Returns: "123456789 12..."
     *      TruncateLongString("123456789 123456 1234567890123 123", 10, False)
     *          Returns: "123456789..."
     *      TruncateLongString("123456789 123456 1234567890123 123", 70, False)
     *          Returns: "123456789 123456 1234567890123 123"
     *      TruncateLongString("1234", 2, False)
     *          Returns: "12..."
     */
    public static String TruncateLongString(String strInput, int nMaxChars, boolean bMaximumWordLength)
    {
        if (!(strInput.length() > nMaxChars))
        {
        	// If the string is already small enough, just return it.
            return strInput;
        }

        if (bMaximumWordLength)
        {
        	// We are going to search the string for any words over maxChars
        	// and only truncate if we find long words.
            int nWordLength = 0;

            for (int i = 0; i < strInput.length(); i++, nWordLength++)
            {
                if (' ' == strInput.charAt(i))
                {
                	// We are at a new word.  Reset the word length counter.
                    nWordLength = 0;
                }

                if (nWordLength >= nMaxChars)
                {
                	// There is a word somewhere in the string that is too
                	// long, so just truncate the string
                    return (strInput.substring(0, nMaxChars).trim() + "...");
                }
            }

            return strInput;
        } else
        {
	    	// If we're not worried about long words, just truncate the string.
    	    return (strInput.substring(0, nMaxChars).trim() + "...");
        }
    }

    /**
     * This method escapes a String for use in Javascript.
     * 
     * This code was adapted from Studio's code (Patrick Corcoran)
     * 
     * @param String the String to be escaped
     * @return String the escaped version of the input
     */
    public static String EscapeJavascript(String strInput)
    {
        if (strInput == null)
        {
            return "";
        }

		int n = strInput.length();

        XPStringBuilder sb = new XPStringBuilder(n);

        for (int i = 0; i < n; i++)
        {
            char c = strInput.charAt(i);

            switch (c)
            {
            case '\n':
                sb.Append("\\n");

                break;

            case '\r':
                sb.Append("");

                break;

            case '\t':
                sb.Append(" ");

                break;

            case '\'':
                sb.Append("\\'");

                break;

            case '"':
                sb.Append("&quot;");

                break;

            //case '"':
            //	sb.Append("\\u0022");
            //
            //	break;

            case '\\':
                sb.Append("\\\\");

                break;

            case '\u00a5':
                sb.Append("&yen;");

                break;

            case '<':
                sb.Append("\\u003c");

                break;

            case '>':
                sb.Append("\\u003e");

                break;

            default:
                sb.Append(c);

                break;
            }
        }

        return sb.ToString();
    }

	/**
	 * This method takes a String with numbers separated by ','s and splits it into
	 * an int array.  If there is an error, the rest of the array is filled with 0s.
	 * 
	 * @param String the comma separated string of numbers to convert
	 * @return int[] the result
	 */
	public static int[] ConvertToInts(String data)
	{
		if (data == null)
		{
			return new int[0];
		}

		XPStringTokenizer st = new XPStringTokenizer(data, ",");
		int[] result		 = new int[st.GetTokenCount()];
		int i				 = 0;

		try
		{
			for (i = 0; st.HasMoreTokens(); i++)
			{
				result[i] = XPConvert.ToInteger(st.GetNextToken());
			}
		}
		catch (Exception e)
		{
			
				log.Warn(e, "ConvertToInts: Bad data - expected numeric data separated by ','s, got : " +
					data);
			

			// null out rest of array
			for (; i < result.length; i++)
			{
				result[i] = 0;
			}
		}

		return result;
	}

	/**
	 * This method converts the expanded object types int[] into a comma
	 * separated String
	 * 
	 * @param int[]
	 * @return String
	 */
	public static final String ConvertToString(int[] nInts)
	{
		if ((nInts == null) || (nInts.length < 1))
		{
			return "";
		}

		// Build the string
		XPStringBuilder sb = new XPStringBuilder(nInts.length * 3);

		for (int i = 0; i < nInts.length; i++)
		{
			sb.Append(nInts[i]);

			if (i < (nInts.length - 1))
			{
				sb.Append(",");
			}
		}

		return sb.ToString();
	}
	
	/**
	 * This helper function sets a UI Error with the messages specified, and optionally
	 * throws an exception.  The UI Error will contain the strings specified only, but the
	 * Debug log entry will also contain the text of the exception (e.getMessage())
	 * 
	 * @author - JohnH
	 * @param asOwner - The activity space in which the error was thrown
	 * @param strFunctionName - The function that trapped the error (e.g. MyModel::CheckEditorAccessAndStart)
	 * @param uiComponent - The UI component in which the error occurred, e.g. OpenLogService.GetComponent("Portal_UI_Infrastructure") 
	 * @param e - The exception that has been thrown
	 * @param bThrow - Throw an XPException in addition to setting the error?
	 * @param uiErrorLevel - The error level (e.g. UIErrorLevel.SERIOUS)
	 * @param strErrorMessage - The error message string to be displayed
	 */
	public static void SetUIError(AActivitySpace asOwner, OpenLogComponent uiComponent, String strFunctionName, Exception e, boolean bThrow, UIErrorLevel uiErrorLevel, String strErrorMessage)
	{
		XPStringBuilder sbShortMsg = new XPStringBuilder(100);
		XPStringBuilder sbLongMsg = new XPStringBuilder(150);
		//Build the error string
		sbShortMsg.Append(strErrorMessage);
		
		if (null != e)
		{
			sbLongMsg.Append(strErrorMessage).Append(e.getMessage());
		}
		else
		{
			sbLongMsg.Append(strErrorMessage);
		}
		
		//Raise the UI error
		UIError myError = new UIError(uiErrorLevel,
			UIErrorScope.CURRENT_USER, null, sbShortMsg.ToString(),
				sbLongMsg.ToString());
		UIErrorHandler.RaiseError(myError,
			asOwner.GetSubSession(), asOwner.GetApplication());

		//Add the exception text and trace it
		
			if (null != e )
				log.Error(e, strFunctionName + sbShortMsg.ToString());
			else 
				log.Error(strFunctionName + sbShortMsg.ToString());
		

		//Optionally throw the exception
		if (bThrow)
		{
			if ( null != e )
				throw XPException.GetInstance(strErrorMessage, e);
			else
				throw new XPException(strErrorMessage);
		}
	}

	/**
	 * Helper method that checks whether or not a string contains unsafe
	 * characters, such as angle braces for HTML or curly braces for
	 * javascript.
	 *
	 * This is useful for determining if someone is trying to do a javascript hack.
	 *
	 * @param String the string to process
	 * @return boolean true implies log a message in PTSpy if the string contains unsafe characters.
	 */
	public static final boolean ContainsUnsafeCharacters(String _strStringIn, boolean bLogErrors)
	{
		if (null == _strStringIn)
		{
			return false;
		}

		if ((XPStringUtility.IndexOf(_strStringIn, '<') != -1) ||
			(XPStringUtility.IndexOf(_strStringIn, '>') != -1) ||
			(XPStringUtility.IndexOf(_strStringIn, '{') != -1) ||
			(XPStringUtility.IndexOf(_strStringIn, '}') != -1) ||
			(XPStringUtility.IndexOf(_strStringIn, ';') != -1) ||
			(XPStringUtility.IndexOf(_strStringIn, '\'') != -1) ||
			(XPStringUtility.IndexOf(_strStringIn, '"') != -1) ||
			(XPStringUtility.IndexOf(_strStringIn, '\\') != -1))
		{
			if (bLogErrors)
			{
				
					log.Error("Query string contained unsafe characters, such as angle braces or curly braces.  Possible hack attempt: "
						+ _strStringIn);
				
			}

			return true;
		}

		return false;
	}
}

