package com.plumtree.uiinfrastructure.statichelpers;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.activityspace.*;

/**
 * This class contains helpers for dealing with HTTP, QS,
 * JS, and other URL related issues.
 * 
 * @author RickyW
 */
public class HTTPHelpers
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.statichelpers.HTTPHelpers");

	/**
	 * This helper method retrieves a single argument from the
	 * argument hashtable.  If the argument does not exist or
	 * is null, the default value will be used.  
	 *
	 * @param _htArguments Arguments to reach the new space.
	 * @param _strKey The key for the argument you want. 
	 * @param _nDefault Value to return if argument is null.
	 *
	 * @return The specified argument from the argument hashtable. 
	 */
	public static final int GetArgumentAsInt(XPHashtable _htArguments, String _strKey, int _nDefault)
	{
		int nRetVal = _nDefault;
		String[] aStrValues = null;
		
		try
		{
			aStrValues = (String[]) _htArguments.GetElement(_strKey);
			
			if ((null != aStrValues) && 
				(!aStrValues[0].equals("")))
			{
				nRetVal = XPConvert.ToInteger(aStrValues[0]);
			}
		}
		catch(Exception ex)
		{
			log.Warn("Error retrieving int from arguments, using default value.");			
		}		

		return nRetVal;
	}

	/**
	 * This helper method retrieves a single argument from the
	 * argument hashtable.  If the argument does not exist or
	 * is null, the default value will be used.  
	 *
	 * @param _htArguments Arguments to reach the new space.
	 * @param _strKey The key for the argument you want. 
	 * @param _strDefault Value to return if argument is null.
	 *
	 * @return The specified argument from the argument hashtable. 
	 */
	public static final String GetArgumentAsString(XPHashtable _htArguments, String _strKey, String _strDefault)
	{
		String strRetVal = _strDefault;
		String[] aStrValues = null;
		
		try
		{
			aStrValues = (String[]) _htArguments.GetElement(_strKey);
			
			if (null != aStrValues)
			{
				strRetVal = aStrValues[0];
			}
		}
		catch(Exception ex)
		{
			log.Warn("Error retrieving string from arguments, using default value.");
		}
		
			
		return strRetVal;
	}

	/**
	 * This helper method retrieves a single argument from the
	 * argument hashtable.  If the argument does not exist or
	 * is null, the default value will be used.  
	 *
	 * @param _htArguments Arguments to reach the new space.
	 * @param _strKey The key for the argument you want. 
	 * @param _bDefault Value to return if argument is null.
	 *
	 * @return The specified argument from the argument hashtable. 
	 */
	public static final boolean GetArgumentAsBoolean(XPHashtable _htArguments, String _strKey, boolean _bDefault)
	{
		boolean bRetVal = _bDefault;
		String[] aStrValues = null;
		
		try
		{
			aStrValues = (String[]) _htArguments.GetElement(_strKey);
			
			if (null != aStrValues)
			{
				bRetVal = XPConvert.ToBoolean(aStrValues[0]);
			}
		}
		catch(Exception ex)
		{
			log.Warn("Error retrieving boolean from arguments, using default value.");
		}		

		return bRetVal;
	}
	

    /**
     * This helper adds a QS name to your XPStringBuilder.  If the 
     * argument name is null, nothing is added.
     * 
     * @param _sbArgs XPStringBuilder to append to.
     * @param _strArgName Name of name/value pair.
     */
    public static final void AppendQSArgument(XPStringBuilder _sbArgs, String _strArgName)
    {
        if (null != _strArgName) 
        {
            _sbArgs.Append("&");
            _sbArgs.Append(_strArgName);
        }    
    }

    /**
     * This helper adds a QS name/value pair to your XPStringBuilder.  
     * If the argument name or value is null or empty, nothing is added.
     * 
     * @param _sbArgs XPStringBuilder to append to.
     * @param _strArgName Name of name/value pair.
     * @param _strArgValue Value of name/value pair.
     */
    public static final void AppendQSArgument(XPStringBuilder _sbArgs, String _strArgName, String _strArgValue)
    {
        AppendQSArgument(_sbArgs, _strArgName, _strArgValue, false);
    }
    
    /**
     * This helper adds a QS name/value pair to your XPStringBuilder.  
     * If the argument name or value is null or empty, nothing is added.
     * 
     * @param _sbArgs XPStringBuilder to append to.
     * @param _strArgName Name of name/value pair.
     * @param _strArgValue Value of name/value pair.
     * @param _bLeaveOutAmpersand True to leave out the leading "&".
     */
    public static final void AppendQSArgument(XPStringBuilder _sbArgs, String _strArgName, String _strArgValue, boolean _bLeaveOutAmpersand)
    {
        if ((null != _strArgName) && (null != _strArgValue) && (!_strArgValue.equalsIgnoreCase("")))
        {
            if (!_bLeaveOutAmpersand)
            {
                _sbArgs.Append("&");
            }
            _sbArgs.Append(_strArgName);
            _sbArgs.Append("=");
            _sbArgs.Append(_strArgValue);
        }
    }

    /**
     * @param _sbArgs
     * @param _strArgName
     * @param _nArgValue
     */
    public static final void AppendQSArgument(XPStringBuilder _sbArgs, String _strArgName, int _nArgValue)
    {
        AppendQSArgument(_sbArgs, _strArgName, _nArgValue, false);
    }
    
    /**
     * This helper adds a QS name/value pair to your XPStringBuilder.  If 
     * the argument name is null, or if the argument value is -1, nothing 
     * is added.
     * 
     * @param _sbArgs XPStringBuilder to append to.
     * @param _strArgName Name of name/value pair.
     * @param _nArgValue Value of name/value pair.
     * @param _bLeaveOutAmpersand True to leave out the leading "&".
     */
    public static final void AppendQSArgument(XPStringBuilder _sbArgs, String _strArgName, int _nArgValue, boolean _bLeaveOutAmpersand)
    {
        if ((null != _strArgName) && (-1 != _nArgValue))
        {
            if (!_bLeaveOutAmpersand)
            {
                _sbArgs.Append("&");
            }
            _sbArgs.Append(_strArgName);
            _sbArgs.Append("=");
            _sbArgs.Append(_nArgValue);
        }
    }

    /**
     * This helper generates the QS argument string from the given
     * argument name.  If the argument name is null, empty string
     * is returned.
     * 
     * @param _strArgName Name of name/value pair.
     * 
     * @return Argument string to add to your QS.
     */
    public static final String GetQSArgString(String _strArgName)
    {
        if (null != _strArgName) 
        {
            return "&" + _strArgName;
        }    
        else
        {
            return "";
        }
    }
    
    /**
     * This helper adds a QS name/value pair to your XPStringBuilder.  
     * If the argument name or value is null or empty, nothing is added.
     * 
     * @param _strArgName Name of name/value pair.
     * @param _strArgValue Value of name/value pair.
     * 
     * @return Argument string to add to your QS.
     */
    public static final String GetQSArgString(String _strArgName, String _strArgValue)
    {
        if ((null != _strArgName) && (null != _strArgValue) && (!_strArgValue.equalsIgnoreCase("")))
        {
            return "&" + _strArgName + "=" + _strArgValue;
        }
        else
        {
            return "";
        }
    }

    /**
     * This helper adds a QS name/value pair to your XPStringBuilder.  If 
     * the argument name is null, or if the argument value is -1, nothing 
     * is added.
     * 
     * @param _strArgName Name of name/value pair.
     * @param _nArgValue Value of name/value pair.
     * 
     * @return Argument string to add to your QS.
     */
    public static final String GetQSArgString(String _strArgName, int _nArgValue)
    {
        if ((null != _strArgName) && (-1 != _nArgValue))
        {
            return "&" + _strArgName + "=" + _nArgValue;
        }
        else
        {
            return "";
        }
    }

    /**
     * Helper method to determine if the given string contains characters
     * that we have deemed to be unsafe for URLs, indicating possible 
     * hack attempts.
     *
     * @author JohnO
     *  
     * @param __strInput 
     * 
     * @return True if string contains unsafe characters.
     */
    public static final boolean StringHasUnsafeURLCharacters(String _strInput)
    {
    	if (null == _strInput)
    	{
    		return false;
    	}

		if ((XPStringUtility.IndexOf(_strInput, '<') != -1) ||
			(XPStringUtility.IndexOf(_strInput, '>') != -1) ||
			(XPStringUtility.IndexOf(_strInput, ';') != -1) ||
			(XPStringUtility.IndexOf(_strInput, '"') != -1) ||
			(XPStringUtility.IndexOf(_strInput, '\'') != -1) ||
			(XPStringUtility.IndexOf(_strInput, '\\') != -1))
		{	
			log.Error("Unsafe URL characters detected, possible hack attempt: " + _strInput);
			
			return true;
		}

		return false;
    }
    
	/**
	 * Returns a query string for the redirect containing all the control arguments.
	 * Copied from com.plumtree.uiinfrastructure.activityspace.Redirect.
	 *
	 * @return String The querystring, excluding the question mark.
	 */
	public static String GetQueryString(XPHashtable _htArguments)
	{
		XPStringBuilder sb = new XPStringBuilder();
		IXPEnumerator enumerator = _htArguments.GetKeys();

		// mj: iterate over all control arguments
		while (enumerator.MoveNext())
		{
			String strName     = XPHttpUtility.UrlEncode((String) enumerator.GetCurrent(),
				LinkConstants.UTF_8_ENCODING);
			String[] arrValues = (String[]) _htArguments.GetElement(strName);

			// mj: check whether there are any values
			if ((arrValues != null) && (arrValues.length > 0))
			{
				for (int i = 0; i < arrValues.length; i++)
				{
					// mj: append the attribute name
					sb.Append(strName).Append('=');

					// mj: if this value is null, use an empty string
					if (arrValues[i] != null)
					{
						sb.Append(XPHttpUtility.UrlEncode(arrValues[i],
							LinkConstants.UTF_8_ENCODING));
					}

					sb.Append('&');
				}
			}
			else
			{
				// mj: no values; append an empty string
				sb.Append(strName).Append('=').Append('&');
			}
		}

		// mj: remove the trailing ampersand, if any
		if (sb.GetLength() > 0)
		{
			sb.SetLength(sb.GetLength() - 1);
		}

		return sb.ToString();
	}
	
	/**
	 * 
	 * @return
	 */
	public static String GetParameterArrayAsString(String[] listParamNames, String[] listParamValues)
	{
		CheckIntegrityOfParameterArrays(listParamNames, listParamValues);
		
		if ( null == listParamNames && null == listParamValues ) {
			return null;
		}
		
		XPStringBuilder buf = new XPStringBuilder(listParamNames.length * 8 * 2 );
		buf.Append("&");
		for ( int i = 0; i < listParamNames.length; i++ ) {
			buf.Append(listParamNames[i] + "=" + listParamValues[i] + "&");
			
		}
		// remove last & char.
		buf.RemoveCharAt(buf.GetLength()-1);
		
		return buf.ToString();
	}
	
	/**
	 * 
	 * 
	 * @param strParamNames
	 * @param strParamValues
	 * @return
	 */
	public static void CheckIntegrityOfParameterArrays(String[] strParamNames,  String[] strParamValues)
	{
		String errorMsg = null;
		
		if ( null == strParamNames && null == strParamValues ) {
			return;
		}
		
		// They both can't be null so one of the arguments are null, error 
		if ( null == strParamNames )
		{
			errorMsg = "Length of strParamNames and strParamValues arrays does not match,  strParamNames argument has a null value.";
		} else if ( null == strParamValues )
		{
			errorMsg = "Length of strParamNames and strParamValues arrays does not match,  strParamValues argument has a  null value.";
		}
		
		// Length of arrays don't match
		if ((strParamNames != null) && ( strParamNames.length != strParamValues.length)) {
			errorMsg = "Length of strParamNames and strParamValues arrays does not match,  strParamNames.length = "+strParamNames.length+" strParamValues.length = "+strParamValues.length;
		}
		if ( null != errorMsg ) {
			throw new XPIllegalArgumentException(errorMsg);
		}
	}
	
	/**
	 * 
	 * @return
	 */
	public static String GetParameterListAsString(XPArrayList listParamNames, XPArrayList listParamValues)
	{
		CheckIntegrityOfParameterArrays(listParamNames, listParamValues);
		
		if ( null == listParamNames && null == listParamValues ) {
			return null;
		}
		
		XPStringBuilder buf = new XPStringBuilder(listParamNames.GetSize() * 8 * 2 );
		buf.Append("&");
		for ( int i = 0; i < listParamNames.GetSize(); i++ ) {
			buf.Append(listParamNames.GetElement(i) + "=" + listParamValues.GetElement(i) + "&");
		}
		// remove last & char.
		buf.RemoveCharAt(buf.GetLength()-1);
		
		return buf.ToString();
	}
	
	/**
	 * 
	 * 
	 * @param strParamNames
	 * @param strParamValues 
	 * @return
	 */
	public static void CheckIntegrityOfParameterArrays(XPArrayList strParamNames,  XPArrayList strParamValues)
	{
		String errorMsg = null;
		
		if ( null == strParamNames && null == strParamValues ) {
			return;
		}
		// They both can be null so one of the arguments are null, error 
		if ( null == strParamNames )
		{
			errorMsg = "Length of strParamNames and strParamValues arrays does not match,  strParamNames argument has a null value.";
		} else if ( null == strParamValues)
		{
			errorMsg = "Length of strParamNames and strParamValues arrays does not match,  strParamValues argument has a  null value.";
		}
		
		// Length of arrays don't match
		if ((strParamNames != null) && ( strParamNames.GetSize() != strParamValues.GetSize() )) {
			errorMsg = "Length of strParamNames and strParamValues arrays does not match,  strParamNames.length = "+strParamNames.GetSize()+" strParamValues.length = "+strParamValues.GetSize();
		}
		if ( null != errorMsg ) {
			throw new XPIllegalArgumentException(errorMsg);
		}
	}
	
}

