package com.plumtree.uiinfrastructure.statichelpers;

import com.plumtree.openfoundation.util.*;

/**
 * This class helps Plumtree products communicate with the Image Server. Mainly
 * its purpose is to build URLs that enforce the rules of the Image Server.
 *
 * Special note: The LX characters in the folder name constants represent the
 * level. So L3 means that the folder is three levels deep on the Image Server.
 *
 * @author John Osborne
 */
public class ImgSvrURLHelper
{
	/** This String constant represents the images folder. */
	public static final String STR_L4_FILE_TYPE_IMG = "img";

	/** This String constant represents the style sheets folder. */
	public static final String STR_L4_FILE_TYPE_CSS = "css";

	/** This String constant represents the JavaScript folder. */
	public static final String STR_L4_FILE_TYPE_JS = "js";

	/** This String constant represents the help files folder. */
	public static final String STR_L4_FILE_TYPE_HELP = "help";

	/** This String constant represents the private folder. The private folder is
	 * intended to be a place where products can place files that they DO NOT
	 * wish to share with PSO and cstomers. */
	public static final String STR_L3_SCOPE_PRIVATE = "private";

	/** This String constant represents the public folder. The public folder is
	 * intended to be a place where products can place files that they DO wish
	 * to share with PSO and cstomers. */
	public static final String STR_L3_SCOPE_PUBLIC = "public";

	/** This String constant represents the custom folder. The custom folder is
	 * intended to be a place where PSO and customers can place their static
	 * content files. */
	public static final String STR_L3_SCOPE_CUSTOM = "custom";

	/** This String constant represents the common folder. The public lives at
	 * the  same level as the product folders (i.e. portal, content, collab,
	 * etc.). It is a place where Plumtree products can share static content. */
	public static final String STR_L2_PRODUCT_NAME_COMMON = "common";

	/** This String constant represents the Peoplesoft Portlet Suite folder. This is
	 * where the Peoplesoft Portlet Suite places its static content. */
	public static final String STR_L1_DOMAIN_PLUMTREE = "plumtree";
	
	// Private Static Variables
	private static final String 		STR_FORWARD_SLASH 							= "/";
	private static ImgSvrURLHelper 	s_oImageServerHelper 						= null;

	// Private Member Variables
	private String 					m_strImageServerBaseURL 					= null;
	private String 					m_strSecureImageServerBaseURL 				= null;
	private String 					m_strDomainName 							= null;
	private String 					m_strProductName 							= null;
	private boolean 					m_bUseTrailingSlashByDefault 				= false;

	/**
	 * Creates a new ImgSvrURLHelper object.
	 *
	 * The constuctor is private. Clients should use the Init method and the
	 * GetInstance method.
	 *
	 * @param _strImageServerBaseURL -- The base Image Server URL (http://)
	 * @param _strSecureImageServerBaseURL -- The secure base Image Server URL
	 * (https://)
	 * @param _strDomainName -- The Domain namespace for the product (i.e. "plumtree")
	 * @param _strProductName -- The name of the product using the Image
	 * Server URL heloer. Examples include "portal" and "collab."
	 * @param _bUseTrailingSlashByDefault -- Mark tbis as true if you want your
	 * URLs to end with a "/".
	 */
	private ImgSvrURLHelper(String _strImageServerBaseURL, String _strSecureImageServerBaseURL, String _strDomainName, String _strProductName, boolean _bUseTrailingSlashByDefault)
	{
		m_strImageServerBaseURL = _strImageServerBaseURL;

		if (!m_strImageServerBaseURL.endsWith(STR_FORWARD_SLASH))
		{
			m_strImageServerBaseURL = m_strImageServerBaseURL + STR_FORWARD_SLASH;
		}

		m_strSecureImageServerBaseURL = _strSecureImageServerBaseURL;

		if (!m_strSecureImageServerBaseURL.endsWith(STR_FORWARD_SLASH))
		{
			m_strSecureImageServerBaseURL = m_strSecureImageServerBaseURL + STR_FORWARD_SLASH;
		}

		m_strDomainName					 = _strDomainName;
		m_strProductName				 = _strProductName;
		m_bUseTrailingSlashByDefault     = _bUseTrailingSlashByDefault;
	}

	/**
	 * This method returns an instance of the ImgSvrURLHelper. You will get back
	 * the same instance each time you call this method. You must also call the
	 * Init method first.
	 *
	 * @return An instance of the ImgSvrURLHelper.
	 *
	 */
	public static final ImgSvrURLHelper GetInstance()
	{
		if (s_oImageServerHelper == null)
		{
			throw new XPException("You must first initialize the ImgSvrURLHelper before you can call GetInstance.");
		}

		return s_oImageServerHelper;
	}

	/**
	 * Clients need to call this method only once per application. This method
	 * essentially allows you to set your Image Server default preferences. For
	 * example, by setting the product name, you don't have to specify it again
	 * each time you call GetURL.
	 *
	 * @param _strImageServerBaseURL -- The base Image Server URL (http://)
	 * @param _strSecureImageServerBaseURL -- The secure base Image Server URL
	 * (https://)
	 * @param _strDomainName -- The Domain namespace for the product (i.e. "plumtree")
	 * @param _strProductName -- The name of the product using the Image
	 * Server URL heloer. Examples include "portal" and "collab."
	 * @param _bUseTrailingSlashByDefault -- Mark tbis as true if you want your
	 * URLs to end with a "/".	 */
	public static final void Init(String _strImageServerBaseURL, String _strSecureImageServerBaseURL, String _strDomainName, String _strProductName, boolean _bUseTrailingSlashByDefault)
	{
		if (s_oImageServerHelper == null)
		{
			s_oImageServerHelper = new ImgSvrURLHelper(_strImageServerBaseURL, _strSecureImageServerBaseURL, _strDomainName, _strProductName, _bUseTrailingSlashByDefault);
		}
	}

	/**
	 * This method passes back the root Image Server URL. For example, "http:
	 * //myserver:80/ptimages/".
	 *
	 * @param _bIsSecure -- Do you want the secure (https://) URL back or the
	 * regular Image Server URL back?
	 *
	 * @return The root URL.
	 */
	public String GetRootURL(boolean _bIsSecure)
	{
		String strImageServerBaseURL;
		
		// Secure or not?
		if (_bIsSecure)
		{
			strImageServerBaseURL = m_strSecureImageServerBaseURL;
		}
		else
		{
			strImageServerBaseURL = m_strImageServerBaseURL;
		}
		
		// Trailing slash or not?
		if (!m_bUseTrailingSlashByDefault)
		{
			strImageServerBaseURL = strImageServerBaseURL.substring(0, strImageServerBaseURL.length() - 1);
		}
		
		return strImageServerBaseURL;
	}

	/**
	 * This method passes back the root Image Server URL. For example, "http:
	 * //myserver:80/ptimages/".
	 *
	 * @param _bIsSecure -- Do you want the secure (https://) URL back or the
	 * regular Image Server URL back?
	 *
	 * @return The root URL.
	 */
	public String GetDomainRootURL(boolean _bIsSecure)
	{
		String strImageServerBaseURL;
		
		// Secure or not?
		if (_bIsSecure)
		{
			strImageServerBaseURL = m_strSecureImageServerBaseURL;
		}
		else
		{
			strImageServerBaseURL = m_strImageServerBaseURL;
		}
		
		strImageServerBaseURL = strImageServerBaseURL + m_strDomainName;
		
		// Trailing slash or not?
		if (m_bUseTrailingSlashByDefault)
		{
			strImageServerBaseURL = strImageServerBaseURL + STR_FORWARD_SLASH;
		}
		
		return strImageServerBaseURL;
	}

	/**
	 * This method is for PSO and customers. It passes back a URL to the folder
	 * where they should be placing their static content. For a particular product.
	 * Here is a sample:  "http://myserver:80/ptimages/portal/custom/".
	 * 
	 * @param _bIsSecure -- Do you want the secure (https://) URL back or the
	 * regular Image Server URL back?
	 * @param _strProductName -- The product whose static content you want to
	 * access (i.e. "portal" or "collab").
	 * @return The root URL.
	 */
	public String GetCustomURL(boolean _bIsSecure, String _strProductName)
	{
		XPStringBuilder sbRetURL = new XPStringBuilder(256);

		if (_bIsSecure)
		{
			sbRetURL.Append(m_strSecureImageServerBaseURL);
		}
		else
		{
			sbRetURL.Append(m_strImageServerBaseURL);
		}

		sbRetURL.Append(m_strDomainName).Append(STR_FORWARD_SLASH).Append(_strProductName);
		sbRetURL.Append(STR_FORWARD_SLASH).Append(STR_L3_SCOPE_CUSTOM);

		if (m_bUseTrailingSlashByDefault)
		{
			sbRetURL.Append(STR_FORWARD_SLASH);
		}

		return sbRetURL.ToString();
	}

	/**
	 * This method is for PSO and customers. It passes back a URL to the folder
	 * where they should be placing their static content. For a particular product.
	 * Here is a sample:  "http://myserver:80/ptimages/portal/custom/".
	 * 
	 * @param _bIsSecure -- Do you want the secure (https://) URL back or the
	 * regular Image Server URL back?
	 * @param _strDomainName -- The Domain namespace for the product (i.e. "plumtree")
	 * @param _strProductName -- The product whose static content you want to
	 * access (i.e. "portal" or "collab").
	 * @return The root URL.
	 */
	public String GetCustomURL(boolean _bIsSecure, String _strDomainName, String _strProductName)
	{
		XPStringBuilder sbRetURL = new XPStringBuilder(256);

		if (_bIsSecure)
		{
			sbRetURL.Append(m_strSecureImageServerBaseURL);
		}
		else
		{
			sbRetURL.Append(m_strImageServerBaseURL);
		}

		sbRetURL.Append(_strDomainName).Append(STR_FORWARD_SLASH).Append(_strProductName);
		sbRetURL.Append(STR_FORWARD_SLASH).Append(STR_L3_SCOPE_CUSTOM);

		if (m_bUseTrailingSlashByDefault)
		{
			sbRetURL.Append(STR_FORWARD_SLASH);
		}

		return sbRetURL.ToString();
	}

	/**
	 * Returns an Image Server URL.
	 *
	 * @param _bIsSecure -- Do you want the secure (https://) URL back or the
	 * regular Image Server URL back?
	 * @param _strFileType -- The kind of static content you want to access (i.
	 * e. Images, JavaScript files, Style Sheets. Seet the string constant
	 * enumerated at the top).
	 *
	 * @return An Image Server URL
	 */
	public String GetURL(boolean _bIsSecure, String _strFileType)
	{
		return GetURL(_bIsSecure, m_strDomainName, m_strProductName, false, _strFileType);
	}

	/**
	 * Returns an Image Server URL.
	 *
	 * @param _bIsSecure -- Do you want the secure (https://) URL back or the
	 * regular Image Server URL back?
	 * @param _bIsPublic -- The scope of the folder you want to access (i.e.
	 * "public", "private", "custom".
	 * @param _strFileType -- The kind of static content you want to access (i.
	 * e. Images, JavaScript files, Style Sheets. Seet the string constant
	 * enumerated at the top).
	 *
	 * @return An Image Server URL
	 */
	public String GetURL(boolean _bIsSecure, boolean _bIsPublic, String _strFileType)
	{
		return GetURL(_bIsSecure, m_strDomainName, m_strProductName, _bIsPublic, _strFileType);
	}

	/**
	 * Returns an Image Server URL.
	 *
	 * @param _bIsSecure -- Do you want the secure (https://) URL back or the
	 * regular Image Server URL back?
	 * @param _strProductName -- The product whose static content you want to
	 * access (i.e. "portal" or "collab").
	 * @param _bIsPublic -- The scope of the folder you want to access (i.e.
	 * "public", "private", "custom".
	 * @param _strFileType -- The kind of static content you want to access (i.
	 * e. Images, JavaScript files, Style Sheets. Seet the string constant
	 * enumerated at the top).
	 *
	 * @return An Image Server URL
	 */
	public String GetURL(boolean _bIsSecure, String _strProductName, boolean _bIsPublic, String _strFileType)
	{
		return GetURL(_bIsSecure, m_strDomainName, _strProductName, _bIsPublic, _strFileType);
	}

	/**
	 * Returns an Image Server URL.
	 *
	 * @param _bIsSecure -- Do you want the secure (https://) URL back or the
	 * regular Image Server URL back?
	 * @param _strDomainName -- The Domain namespace for the product (i.e. "plumtree")
	 * @param _strProductName -- The product whose static content you want to
	 * access (i.e. "portal" or "collab").
	 * @param _bIsPublic -- The scope of the folder you want to access (i.e.
	 * "public", "private", "custom".
	 * @param _strFileType -- The kind of static content you want to access (i.
	 * e. Images, JavaScript files, Style Sheets. Seet the string constant
	 * enumerated at the top).
	 *
	 * @return An Image Server URL
	 */
	public String GetURL(boolean _bIsSecure, String _strDomainName, String _strProductName, boolean _bIsPublic, String _strFileType)
	{
		XPStringBuilder sbRetURL = new XPStringBuilder(256);

		if (_bIsSecure)
		{
			sbRetURL.Append(m_strSecureImageServerBaseURL);
		}
		else
		{
			sbRetURL.Append(m_strImageServerBaseURL);
		}

		sbRetURL.Append(_strDomainName).Append(STR_FORWARD_SLASH).Append(_strProductName);

		if (_bIsPublic)
		{
			sbRetURL.Append(STR_FORWARD_SLASH).Append(STR_L3_SCOPE_PUBLIC);
		}
		else
		{
			sbRetURL.Append(STR_FORWARD_SLASH).Append(STR_L3_SCOPE_PRIVATE);
		}

		sbRetURL.Append(STR_FORWARD_SLASH).Append(_strFileType);

		if (m_bUseTrailingSlashByDefault && !(_strFileType.equals("")))
		{
			sbRetURL.Append(STR_FORWARD_SLASH);
		}

		return sbRetURL.ToString();
	}
}
