package com.plumtree.uiinfrastructure.statichelpers;

import com.plumtree.openfoundation.util.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * Javascript helper functions
 * 
 * @author adolfoc
 * @author robertz
 *
 */
public class JSHelper
{
	/** Name of the safe javascript var container variable **/
	public static final String SAFEJSVAR_CONTAINER_NAME = "gSafeJSVarContainer";
	/** rob: Initialized in common-jsdojo build bea/dnd/dnd.js, probably should move to init.js **/
	public static final String ALIDOJO_OBJECT = "bea.dojoObj";
	
	/**
	 * Escape single and double quotes, neccessary for javascript variable values.
	 *
	 * @param strIn 			String to escape
	 *
	 * @return 					String with escaped single and double quotes
	 */
	public static String JSEncodeString(String strIn)
	{
		strIn     = GeneralHelpers.Replace(strIn, "\'", "\\\'");
		strIn     = GeneralHelpers.Replace(strIn, "\"", "\\\"");

		return strIn;
	}
	
	/**
	 * Helper function to define a safe javascript variable.
	 * The safe variable will be defined in its on javascript block, so it won't
	 * affect any other javascript if the assigned value is invalid.
	 * 
	 * Reference it by using the javascript getSafeVariable or getSafeVariableWarn
	 * helper functions or GetSafeJSVariable or GetSafeJSVariableWarn java helper
	 * functions to generate the javascript calls.
	 * 
	 * NOTE: The strValue is the string with the javascript value so don't surround
	 * it with quotes unless you want the value to be a string.
	 * E.g. 
	 * strValue = "new Object()" - will assign a new object to the variable while
	 * strValue = "'new Object()'" - will assign the string 'new Object()'   
	 * 
	 * @param strName
	 * @param strValue
	 * @return HTMLScript
	 */
	public static HTMLScript AddSafeJSVariable(String strName, String strValue)
	{
		HTMLScript script = new HTMLScript("text/javascript");
		script.AddInnerHTMLString("\naddSafeVar('"+strName+"', "+strValue+");");
		
		return script;
	}
	
	/**
	 * Helper function to retrieve a safe javascript variable.
	 * The javascript function will return false if it is unable
	 * to get the variable.
	 * 
	 * @param strName
	 * @return String
	 */
	public static String GetSafeJSVariable(String strName)
	{
		return "getSafeVar('"+strName+"')";
	}
	
	/**
	 * Helper function to retrieve a safe javascript variable.
	 * The javascript function will show a javascript alert 
	 * error with the name of the variable. 
	 * Use for critical variables that absolutely need to be defined.
	 * 
	 * @param strName
	 * @return String
	 */
	public static String GetSafeJSVariableWarn(String strName)
	{
		return "getSafeVarWarn('"+strName+"')";
	}
	
	/**
	 * Create the container to hold the safe variables.
	 * The container is just a javascript object and we but
	 * the vars on it as object properties.
	 * 
	 * 
	 * @return HTMLScript
	 */
	public static HTMLScript InitializeSafeJSVarElements()
	{
		HTMLScript script = new HTMLScript("text/javascript");
		script.AddInnerHTMLString(
		"/** The object that holds the safe variables. Store the vars"+
		" as properties on the object, like a hashtable. **/"+
		"\n"+"var "+SAFEJSVAR_CONTAINER_NAME+" = new Object();\n"+
		"/** Define a new safe variable, pass the in the name and the value."+
		"Returns true if successful, method call will fail if the value is invalid. **/"+
		"\nfunction addSafeVar(strName, oValue) {\n"+
		"\t"+SAFEJSVAR_CONTAINER_NAME+"[strName] = oValue;\n"+
		"\t"+"return true;\n"+
		"}\n"+
		"/** Retrieve a safe var. Returns false if the variable is undefined "+
		" or if the value is actually false, but safe vars shouldn't be used"+
		"to store simple boolean values anyway. **/"+
		"\nfunction getSafeVar(strName) {\n"+
		"\t"+"var temp = "+SAFEJSVAR_CONTAINER_NAME+"[strName];\n"+
		"\t"+"if (temp) { return temp; }\n"+
		"\t"+"else { return false;}\n"+
		"}\n"+
		"/** Retrieve a safe var, if the variable is undefined an alert will show"+
		"with the name of the variable. Use this method to retrieve critical"+
		"variables. **/"+
		"\nfunction getSafeVarWarn(strName) {\n"+
		"\t"+"var temp = "+SAFEJSVAR_CONTAINER_NAME+"[strName];\n"+
		"\t"+"if (temp) { return temp; }\n"+
		"\t"+"else { alert('Safe Variable: '+strName+' is not defined!'); }\n"+
		"}\n"		
		);
		return script;
	}
	
	
	/**
	  * Escape JavaScript strings.
	  */
	public static final String escapeJS(String s)
	{
		if (s == null) s = "";
 
		XPStringBuilder sb = new XPStringBuilder();
		int n = s.length();
		for (int i = 0; i < n; i++) {
			char c = s.charAt(i);
			switch (c) {
				case '\n':  sb.Append("\\n"); break;
				case '\r':  sb.Append(""); break;
				case '\t':  sb.Append(" "); break;
				case '\'':  sb.Append("\\'"); break;
				case '"':  sb.Append("\\u0022"); break;
				case '\\':  sb.Append("\\\\"); break;
				case '\u00a5': sb.Append("&yen;"); break;
				case '<':  sb.Append("\\u003c"); break;
				case '>':  sb.Append("\\u003e"); break;
				default:  sb.Append(c); break;
			}
		}
		return sb.ToString();
	}
	
}

