package com.plumtree.uiinfrastructure.statichelpers;

import com.plumtree.openlog.*;
import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.web.*;

/**
 * Helper to get and set key values on the HTTPSession sandbox.  Please refer
 * to AActivitySpace to see usage.  Includes things originally in LoginHelper;
 * intent is to support core personal settings applicable to any product:
 * access style, locale, language.
 *
 * @author DavidP
 *
 */
public class PersonalSettingsHelper
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.statichelpers.PersonalSettingsHelper");

	/** key session setting names used by defaults */
	public static final String ACCESS_STYLE = "strAccessStyle";

	public static final String LOCALE   = "strLocale";

	public static final String LANGUAGE = "strLanguage";

	public static final String TIMEZONE = "strTimeZone";

	public static final String BASE_URL = "strBaseURL";

	/**
	 * name of the personal settings sub-session.  This should only be used
	 * by the interpreter.
	 * */
	public static final String PREFS_SUBSESSION_NAME = "userprefsss";

	/**
	 *     Call this method to determine if the subsession for caching personal
	 *     settings exists in the specified ISessionManager.
	 *     <br><br>
	 *     @param AActivitySpace
	 */
	public static boolean DoesPrefsSubSessionExist(AActivitySpace activitySpace)
	{
		// This method simply insures you're using the right HTTPSession "sandbox" off of the AActivitySpace
		ISessionManager sessionManager = activitySpace.GetSubSession();

		return DoesPrefsSubSessionExist(sessionManager);
	}

	/**
	 *     Use this version of DoesPrefsSubSessionExist if all you have is
	 *     the  session manager.
	 *     <br><br>
	 *     @param ISessionManager
	 */
	public static boolean DoesPrefsSubSessionExist(
		ISessionManager sessionManager)
	{
		boolean b = false;

		if (sessionManager.ContainsSubSession(
					PersonalSettingsHelper.PREFS_SUBSESSION_NAME))
		{
			b = true;
		}

		return b;
	}

	/**
	 *     Call this method to find the value associated with given setting name.
	 *     <br><br>
	 *     @param oaSettings Pass an Object[][] obtained from LookupPersonalSettings
	 *     @param String name of setting
	 *
	 */
	public static Object FindPersonalSetting(Object[][] oaSettings, String sName)
	{
		Object oVal = null;

		for (int iSettingIndex = 0; iSettingIndex < oaSettings[0].length;
				++iSettingIndex)
		{
			String sSettingName = (String) oaSettings[0][iSettingIndex];

			if (0 == sSettingName.compareTo(sName))
			{
				//oVal = (String) oaSettings[1][iSettingIndex]; // DAP commented out because cast throws exception
				oVal = oaSettings[1][iSettingIndex];

				break;
			}
		}

		return oVal;
	}

	/**
	 * Only use this version if all you have is the subsession
	 * @param sSettingName String name of personal setting
	 * @param sessionManager ISessionManager
	 * @throws XPException This method will throw an XPException if you do not
	 *     pass an ISessionManager already containing the correct subSession.  The
	 *     ISessionManager obtained from your Activity Space will contain the correct
	 *     subsession.
	 */
	public static Object GetPersonalSettingValue(String sSettingName,
		ISessionManager sessionManager, IApplication application)
	{
		
			log.Debug("Entering GetPersonalSettingValue() - sSettingsName: " +
				sSettingName);
		

		// Try to retrieve the requested setting directly from the low-level HTTPSession
		Object oVal				   = null;
		ISessionManager subSession = sessionManager.GetSubSession(PersonalSettingsHelper.PREFS_SUBSESSION_NAME);

		oVal					   = subSession.GetAttribute(sSettingName);

		if (oVal == null)
		{
			
				log.Debug("GetPersonalSettingValue() - Couldn't get from the session cache, checking defaults.");
			

			// On fail, fall through to hardcoded default values in certain cases
			if (sSettingName.equals(ACCESS_STYLE))
			{
				oVal = ConfigHelper.GetConfigVariableAsString(ConfigConstants.PERSONAL_SETTINGS_SECTION,
					ConfigConstants.ACCESSIBILITY_SUBSECTION, ConfigConstants.VALUE_ATTRIBUTE, application);
			}
			else if (sSettingName.equals(LOCALE))
			{
				oVal = ConfigHelper.GetConfigVariableAsString(ConfigConstants.INTERNATIONAL_SECTION,
					ConfigConstants.LOCALE_SUBSECTION, ConfigConstants.VALUE_ATTRIBUTE, application);
			}
			else if (sSettingName.equals(LANGUAGE))
			{
				oVal = ConfigHelper.GetConfigVariableAsString(ConfigConstants.INTERNATIONAL_SECTION,
					ConfigConstants.LOCALE_SUBSECTION, ConfigConstants.VALUE_ATTRIBUTE, application);

				XPResourceManager rm = (XPResourceManager) application
					.GetCachingManager().GetEntry("ResourceManager");

				// Convert the default locale to a language
				oVal = rm.MapLocaleToSupportedUILangID(XPConvert.ToString(oVal));
			}
		}

		
			log.Debug("GetPersonalSettingValue() - oVal: " + oVal);
		

		return oVal;
	}

	/**
	 * Call this method to obtain the value of a personal setting.
	 * <br><br>
	 *     This value will be retrieved from the user's web session if possible,
	 *     otherwise the portal server will be queried.
	 * <br><br>
	 * @param sSettingName String name of personal setting
	 * @param AActivitySpace as
	 * @throws XPException This method will throw an XPException if you do not
	 *     pass an ISessionManager already containing the correct subSession.  The
	 *     ISessionManager obtained from your Activity Space will contain the correct
	 *     subsession.
	 */
	public static Object GetPersonalSettingValue(String sSettingName,
		AActivitySpace activitySpace)
	{
		if (!PersonalSettingsHelper.DoesPrefsSubSessionExist(activitySpace))
		{
			throw new XPException(
				"SubSession for prefs cache does not exist.  Make sure AActivitySpace is valid.");
		}

		// This method simply insures you're using the right HTTPSession "sandbox" off of the AActivitySpace
		ISessionManager sessionManager = activitySpace.GetSubSession();
		IApplication application = activitySpace.GetApplication();

		return GetPersonalSettingValue(sSettingName, sessionManager, application);
	}

	/**
	 * Call this method to set a personal setting name value pair on the
	 *     session (infrastructure HttpSession sandbox)
	 *     <br><br>
	 * @param sSettingName String name of personal setting
	 * @param oSettingVal Object value of personal setting
	 * @param as AActivitySpace Pass an AActivitySpace implementing
	 *     IXPActivitySpace.
	 * @throws XPException This method may throw an XPException.
	 *
	 */
	public static void SetPersonalSettingValue(String sSettingName,
		Object oSettingVal, AActivitySpace activitySpace)
	{
		// This method simply insures you're using the right HTTPSession "sandbox" off of the AActivitySpace
		ISessionManager sessionManager = activitySpace.GetSubSession();
		SetPersonalSettingValue(sSettingName, oSettingVal, sessionManager);
	}

	/**
	 * Call this method to set a personal setting name value pair on the
	 *     session (infrastructure HttpSession sandbox)
	 *     <br><br>
	 * @param sSettingName String name of personal setting
	 * @param oSettingVal Object value of personal setting
	 * @param sessionManager ISessionManager Pass the correct subsession (AS.
	 *             GetSubSession)
	 * @throws XPException This method may throw an XPException.
	 *
	 */
	public static void SetPersonalSettingValue(String sSettingName,
		Object oSettingVal, ISessionManager sessionManager)
	{
		
			log.Debug("Entering SetAndCachePersonalSetting() - sSettingsName: " +
				sSettingName);
		

		ISessionManager subSession = sessionManager.GetSubSession(PersonalSettingsHelper.PREFS_SUBSESSION_NAME);

		subSession.SetAttribute(sSettingName, oSettingVal);
	}
	
	/**
	 * Call this method to remove a personal setting name on the
	 *     session (infrastructure HttpSession sandbox)
	 *     <br><br>
	 * @param sSettingName String name of personal setting
	 * @param sessionManager ISessionManager Pass the correct subsession (AS.
	 *             GetSubSession)
	 * @throws XPException This method may throw an XPException.
	 *
	 */
	public static void RemovePersonalSetting(String sSettingName, ISessionManager sessionManager)
	{
		
			log.Debug("Entering RemovePersonalSetting() - sSettingsName: " +
				sSettingName);
		

		ISessionManager subSession = sessionManager.GetSubSession(PersonalSettingsHelper.PREFS_SUBSESSION_NAME);

		
		subSession.RemoveAttribute(sSettingName);
	}
}

