package com.plumtree.uiinfrastructure.tempfile;

import com.plumtree.openlog.*;
import com.plumtree.openfoundation.io.*;
import com.plumtree.openfoundation.util.*;

/**
 * @author adolfoc
 *
 * Created on Dec 27, 2002
 */
public class TempFile
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.tempfile.TempFile");

	/**
	 * The location of our temp directory. The current setting is a bogus default value.
	 * This should be reset from x_config.xml in StaticInit (which should be called in the
	 * warmup servlet).
	 */
	public static String BASE_TEMP_DIR = null;

	/**
	 * The maximum number of temp files before we think that there's an error. This is a safegaurd
	 * value and should be set pretty high, but lower than the expected number of available handles
	 * on the system.
	 */
	public static final int MAX_TEMP_FILES = 1028;

	/**
	 * Error checking to make sure we're only initialized once.
	 */
	private static boolean s_bInitialized = false;

	/**
	 * This is how we create a temp file.
	 *
	 * Step #1: Use the sub-directory name supplied to find or create a subdirectory of the BASE_TEMP_DIR.
	 * Step #2: Using the file prefix and extension, we attempt to create a new file. If there is an
	 *             existing file with the same name, we will append a number after the file prefix in order
	 *             to make it unique.
	 * Step #3: We walk through all of the files in the supplied subdirectory and delete any whose
	 *             last modified date is older than the expiration time.
	 *
	 * NOTE*: The expiration time is a minimum only and does not "stick" with the file created. It is used
	 *           against all files in the sub-directory to determine when they should be deleted. The file created
	 *           by this function will be deleted based on the expiration time passed into subsequent calls to this
	 *           function - not based on this expiration time.
	 *
	 * NOTE**: The expiration time is measured from the last modified time of the created temp file - not the create
	 *             time.
	 *
	 * Returns the full path of the created temp file. The created temp file may have some data in it. It is expected
	 * that the caller overwrite the created tempfile with their own data.
	 */
	public static String CreateTempFile(String strSubDirectory,
		String strFilePrefix, String strFileExtension,
		int nExpireFilesOlderThanMinutes)
	{
		String strAsciiFilePrefix = AsciiEncode(strFilePrefix);
		
		if (!s_bInitialized)
		{
			throw new XPException(
				"TempFile.StaticInit has not been called. StaticInit must be called before any other method on TempFile.");
		}

		try
		{
			// Verify the existence of the base sub directory.
			XPDirectory xpBaseTempDir = new XPDirectory(BASE_TEMP_DIR);

			if (!xpBaseTempDir.Exists())
			{
				
					log.Error("CreateTempFile could not find the BASE temp directory. Please ensure that the TempHome variable in x_config.xml is set properly.");
				

				return null;
			}

			ExpireTempFiles(strSubDirectory, nExpireFilesOlderThanMinutes);

			XPDirectory xpSubTempDir = null;
			xpSubTempDir = new XPDirectory(xpBaseTempDir.GetFullPath() +
					XPFileSystemObject.GetPathSeparator() + strSubDirectory);

			if (!xpSubTempDir.Exists())
			{
				xpSubTempDir.Create();
			}

			XPFile xpTempFile   = null;
			
			// Now try to create the file.				

			try
			{
				xpTempFile = GenerateNewFile(xpSubTempDir.GetFullPath(), strFilePrefix, strFileExtension);
					
			}
			catch (Exception e)
			{
				// AGC: Bug# 25461 - With java versions < 1.4 you cannot save a file to the filesystem with a non-ASCII name
				// that doesn't match the system code-page. Let's try this again with an ASCII name, shall we?				
				xpTempFile = GenerateNewFile(xpSubTempDir.GetFullPath(), strAsciiFilePrefix, strFileExtension);
						
			}
			return xpTempFile.GetFullPath();
		}
		catch (Exception e)
		{
			String strException =
				"CreateTempFile: Error attempting to create temp file. e: " +
				e.getMessage();

			
				log.Error(e, strException);
			

			throw new XPException(strException, e);
		}
	}

	/**
	 * This function expires (read deletes) files that were last modified more than nExpireFilesOlderThanMinutes
	 * ago from the strSubDirectory of the base temp directory.
	 */
	public static void ExpireTempFiles(String strSubDirectory,
		int nExpireFilesOlderThanMinutes)
	{
		if (!s_bInitialized)
		{
			throw new XPException(
				"TempFile.StaticInit has not been called. StaticInit must be called before any other method on TempFile.");
		}

		try
		{
			// Verify the existence of the base sub directory.
			XPDirectory xpBaseTempDir = new XPDirectory(BASE_TEMP_DIR);

			if (!xpBaseTempDir.Exists())
			{
				
					log.Error("CreateTempFile could not find the BASE temp directory. Please ensure that the TempHome variable in x_config.xml is set properly.");
				

				return;
			}

			XPDirectory xpSubTempDir = null;
			xpSubTempDir = new XPDirectory(xpBaseTempDir.GetFullPath() +
					XPFileSystemObject.GetPathSeparator() + strSubDirectory);

			if (!xpSubTempDir.Exists())
			{
				// Nothing to do... Go home.
				return;
			}

			XPFileSystemObject[] xparFSOChildren = xpSubTempDir.GetChildren();

			for (int nIndex = 0; nIndex < xparFSOChildren.length; nIndex++)
			{
				XPFileSystemObject xpfsoCurrent = xparFSOChildren[nIndex];

				XPDateTime xpdtModified = xpfsoCurrent.GetLastModifiedTime();
				XPDateTime xpdtNow	    = XPCalendar.GetInstance().GetTime();

				long lNowMilliseconds = xpdtNow.GetTimeInMilliSeconds();

				long lExpiresMilliseconds = lNowMilliseconds -
					(nExpireFilesOlderThanMinutes * 1000L * 60L);

				XPDateTime xpdtExpires = new XPDateTime(lExpiresMilliseconds);

				if (xpdtModified.Before(xpdtExpires))
				{
					xpfsoCurrent.Delete();
				}

				try
				{
				}
				catch (Exception e)
				{
					
						log.Error(e, "ExpireTempFiles: Unknown error attempting to expire temp file " +
							xpfsoCurrent.GetFullPath());
					
				}
			}
		}
		catch (Exception e)
		{
			String strException =
				"ExpireTempFiles: Unknown error attempting to expire old temp files. e: " +
				e.getMessage();

			
				log.Error(e, strException);
			
		}
	}

	/**
	 * Initializes our static variables for future use.
	 */
	public static void StaticInit(String strTempHome)
	{
		if (s_bInitialized)
		{
			throw new XPException(
				"TempFile.StaticInit can only be called once.");
		}

		s_bInitialized = true;

		try
		{
			String strBaseTempDir = strTempHome;

			XPDirectory xpdBaseTempDir = new XPDirectory(strBaseTempDir);

			if (!xpdBaseTempDir.Exists())
			{
				// This will bail if the directory is bogus. Leaving the BASE_TEMP_DIR unset.
				xpdBaseTempDir.Create();
			}

			BASE_TEMP_DIR = xpdBaseTempDir.GetFullPath();

			
				log.Info("Temp Directory: " + BASE_TEMP_DIR);
			
		}
		catch (Exception e)
		{
			log.Error("Temp Directory could not be found/created. Please ensure that the TempHome variable in x_config.xml is set properly.");
		}
	}
	
	/**
	* This function walks a string and re-encodes any non-Ascii charcters it finds with a suitable
	* (though not pretty) substitute.
	**/
	private static String AsciiEncode (String strIn)
	{
		XPStringBuilder xpstrOut = new XPStringBuilder();
		int nMaxLen = strIn.length();
		
		for (int nIndex = 0; nIndex < nMaxLen; nIndex++)
		{
			char chCurrent = XPStringUtility.CharAt(strIn,nIndex);
			if (InASCIIRange(chCurrent))
			{
				xpstrOut.Append(chCurrent);
			} else
			{
				xpstrOut.Append('#');
				xpstrOut.Append((int)chCurrent);
			}
		}
		
		return xpstrOut.toString();
	}

	/**
	* This function is actually in XPHttpUtility.java, but it is private and I am not going to update 
	* openfoundation right now.
	**/	
	private static boolean InASCIIRange(char ch) {
		if (ch >= 0 && ch <= 127) {
			return true;
		} else {
			return false;
		}
	}

	/**
	* This function attempts to create a file with a specific prefix and path.
	* This will return null if the file already exists. Otherwise, it will create
	* a new file and fill it with some bogus text.
	**/
	private static XPFile GenerateNewFile(String strParentDir, String strFilePrefix, String strFileExtension)
	{
		XPFile xpTempFile = null;
		boolean bNewFileFound = false;
		int nDuplicateCount = 0;
		
		while (!bNewFileFound)
		{			
			nDuplicateCount++;
						
			// AGC- just a test to make sure we don't recurse forever
			if (nDuplicateCount > MAX_TEMP_FILES)
			{
				String strException =
					"CreateTempFile: Too many temp files (over " +
					XPConvert.ToString(MAX_TEMP_FILES) +
					"). Assuming an error.";

				
					log.Error(strException);
				

				throw new XPException(strException);
			}

			xpTempFile = new XPFile(strParentDir +
				XPFileSystemObject.GetPathSeparator() + strFilePrefix +
				XPConvert.ToString(nDuplicateCount) + strFileExtension);
				
			if (!xpTempFile.Exists())
			{
				bNewFileFound = true;
			}		
		}
		
		XPFileWriter xpFW = xpTempFile.CreateText();

		// AGC- Let's just dump some random text in there for kicks. The file will be overwritten soon, so this text will go away.
		xpFW.Write("This file was created by Plumtree and can be safely deleted after reboot. (TempFile::CreateTempFile)");

		// AGC- Now we close up the temp file - we don't need it anymore and holding it open will
		// interfere with future operations on the file.
		xpFW.Close();
		
		return xpTempFile;
	}
}

