package com.plumtree.uiinfrastructure.tree;

import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.openlog.*;
import com.plumtree.openfoundation.util.*;

/**
 * This abstract class handles setting most of the portal independent variables
 * on the ITreeModel when a new tree is started.
 * 
 * @author Don Hayler
 */
abstract public class AStartTreeControl implements IControl
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.tree.AStartTreeControl");

	/** Query String arguments for this control */

	/**
	 * true = check boxes, false = radio buttons
	 * (multi select vs. single select of objects)
	 */
	public static final String MULTI_SELECT = "Multi";

	/** See TreeConstants for select modes (leaves vs. folders vs. both) */
	public static final String SELECT_MODE = "SelectMd";

	/** See TreeConstants for select options (strict leaves). Optional. */
	public static final String SELECT_OPTIONS = "SelectOptions";

	/**
	 * The class IDs of the types of objects you want to display.
	 * If you don't want to display any objects (folders only), don't use this.
	 * Make sure to include catalogfolder if you want catalogfolders, otherwise
	 * only folders will appear.
	 */
	public static final String CLASS_IDS_TO_DISPLAY = "Class";

	/** classID1,objectID1,classID2,objectID2,etc... */
	public static final String SELECTED_ITEMS = "Select";

	/**
	 * classID1,objectID1,classID2,objectID2,etc...
	 * This will display all objects except for the following
	 */
	public static final String HIDDEN_ITEMS = "Hide";

	/**
	 * classID1,objectID1,classID2,objectID2,etc...
	 * This will only display the selected objects.
	 * This does not work for IPTFolders.  The class IDs
	 * still need to be specified and the root folder ID
	 * needs to be specified, even though it will be
	 * ignored.
	 * 
	 * Normal folder browsing will be ignored
	 */
	public static final String DISPLAYED_ITEMS = "Display";

	/**
	 * This determines whether or not the root folder will be displayed.
	 * By default, it will be displayed.  Pass in true or false.
	 */
	public static final String SHOW_ROOT = "ShowRoot";

	/** See TreeConstants for select modes (server vs. Javascript) */
	public static final String SUBMIT_MODE = "SubmitMd";

	/**
	 * These two should be used together or not at all.
	 * The name of the parent form that you would like the data passed back to.
	 */
	public static final String PARENT_FORM_NAME = "Form";

	/**
	 * The name of the input in the parent form that you would like the
	 * AActivitySpace ID of the tree space put into.  This is used to
	 * reopen the tree after it has been created.
	 */
	public static final String PARENT_FORM_SPACE_ID = "Input";

	/**
	 * The name of the javascript function in the parent page to call when the tree
	 * is submitted.  This should not include the '();'.
	 * In Javascript submit mode this method should take an array argument which will
	 * be made up of objects with name, ObjectID, and ClassID member variables.
	 * see ATreeAS for details.  Warning: this array will dissapear after the function
	 * call is over.  If you want to save the items, you must copy them somewhere in
	 * your page.
	 * 
	 * The names of the objects will be HTMLEncoded and truncated to 50 characters.
	 */
	public static final String PARENT_SUBMIT_CALL = "Submit";

	/**
	 * The name of the javascript function in the parent page to call when tree
	 * elements are clicked.  This should include the whole path to the function
	 * (i.e. parent.myframe.myfunction) without the ending parentheses or semicolon
	 * '();'.  This should not include the '();'.  This function will be called
	 * with the clicked input as the only argument in the onclick method of the
	 * tree items.  It should return true or false depending on whether or not the
	 * button should be clicked (true implies it will be clicked, false implies it
	 * will not).  The function name should also not contain {,},',",\
	 * 
	 * Using this method will turn off dynamic updating of the 'you have selected'
	 * frame, which uses the same mechanism.
	 * 
	 * This should be used extremely sparingly (only in the save page currently).
	 */
	public static final String ONCLICK_CALL = "OnClick";

	/** The title of the tree pop-up */
	public static final String TITLE = "Title";

	/** The action sub-title of the tree */
	public static final String ACTIONSUBTITLE = "SubTitle";

	/** Allow users to click finish on a tree with nothing selected. default is false */
	public static final String ALLOW_EMPTY_SUBMISSION = "AllowEmpty";

	/** The help topic. */
	public static final String HELP_TOPIC = "Help";

	protected ATreeAS m_asOwner;

	/** Internal data **/
	protected ITreeModel m_asModel;

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IControl#Execute()
	 */
	public Redirect CheckActionSecurityAndExecute(XPHashtable arguments)
	{
		String[] sValues;

		// Reset tree
		m_asModel.ResetTree();

		// Multiple select
		sValues = (String[]) arguments.GetElement(MULTI_SELECT);

		if (sValues != null)
		{
			m_asModel.SetMultipleSelect(XPConvert.ToBoolean(sValues[0]));
		}

		// Selection mode
		sValues = (String[]) arguments.GetElement(SELECT_MODE);

		if (sValues != null)
		{
			try
			{
				int nSelectMode = XPConvert.ToInteger(sValues[0]);

				if ((nSelectMode == TreeConstants.N_COMPOSITESELECT) ||
					(nSelectMode == TreeConstants.N_LEAFCOMPOSITESELECT) ||
					(nSelectMode == TreeConstants.N_LEAFSELECT) ||
					(nSelectMode == TreeConstants.N_NOSELECT))
				{
					m_asModel.SetSelectMode(nSelectMode);
				}
				else
				{
					log.Warn("Attempted to set Select Mode to an invalid value: " +
						nSelectMode);
				}
			}
			catch (XPFormatException e)
			{
				
					log.Error("could not parse Select Mode data in query string: " +
						sValues[0]);
				
			}
		}

		// Selection options
		sValues = (String[]) arguments.GetElement(SELECT_OPTIONS);

		if (sValues != null)
		{
			try
			{
				int nSelectOptions = XPConvert.ToInteger(sValues[0]);

				if ((nSelectOptions == TreeConstants.N_LOOSELEAF) ||
					(nSelectOptions == TreeConstants.N_STRICTLEAF))
				{
					m_asModel.SetSelectOptions(nSelectOptions);
				}
				else
				{
					log.Warn("Attempted to set Select Options to an invalid value: " +
						nSelectOptions);	
				}
			}
			catch (XPFormatException e)
			{
				log.Error("could not parse Select Options data in query string: " +
					sValues[0]);
			}
		}

		// Submit mode
		sValues = (String[]) arguments.GetElement(SUBMIT_MODE);

		if (sValues != null)
		{
			try
			{
				int nSubmitMode = XPConvert.ToInteger(sValues[0]);

				if ((nSubmitMode == TreeConstants.N_SERVERSUBMIT) ||
						(nSubmitMode == TreeConstants.N_JAVASCRIPTSUBMIT))
				{
					m_asModel.SetSubmitMode(nSubmitMode);
				}
				else
				{
					log.Warn("Attempted to set Submit Mode to an invalid value: " +
						nSubmitMode);
				}
			}
			catch (XPFormatException e)
			{
				log.Error("could not parse Submit Mode data in query string: " +
					sValues[0]);
			}
		}

		// parent form name
		sValues = (String[]) arguments.GetElement(PARENT_FORM_NAME);

		if (sValues != null)
		{
			m_asModel.SetParentFormName(sValues[0]);
		}

		// parent form space ID
		sValues = (String[]) arguments.GetElement(PARENT_FORM_SPACE_ID);

		if (sValues != null)
		{
			m_asModel.SetParentFormSpaceID(sValues[0]);
		}

		// parent form submit call
		sValues = (String[]) arguments.GetElement(PARENT_SUBMIT_CALL);

		if (sValues != null)
		{
			if ((sValues[0].indexOf('(') == -1) &&
					(sValues[0].indexOf(')') == -1) &&
					(sValues[0].indexOf(';') == -1))
			{
				m_asModel.SetParentFormSubmitFunction(sValues[0]);
			}
			else
			{
				
					log.Error("Submit method cannot contain ();");
				
			}
		}

		// onclick call
		sValues = (String[]) arguments.GetElement(ONCLICK_CALL);

		if (sValues != null)
		{
			if ((sValues[0].indexOf('(') == -1) &&
					(sValues[0].indexOf(')') == -1) &&
					(sValues[0].indexOf(';') == -1))
			{
				if (GeneralHelpers.ContainsUnsafeCharacters(sValues[0], true))
				{
					
						log.Error("Onclick method cannot characters such as <,>,{,},;, etc... " + sValues[0]);
					
				} else
				{
					m_asModel.SetOnClickFunction(sValues[0]);
				}
			}
			else
			{
				
					log.Error("Onclick method cannot contain ();");
				
			}
		}

		// Allow empty tree submission
		sValues = (String[]) arguments.GetElement(ALLOW_EMPTY_SUBMISSION);

		if (sValues != null)
		{
			m_asModel.SetAllowEmptySubmission(XPConvert.ToBoolean(sValues[0]));
		}

		// Title for the tree pop-up
		sValues = (String[]) arguments.GetElement(TITLE);

		if (sValues != null)
		{
			m_asModel.SetTitle(sValues[0]);
		}

		// Action sub-title for the tree pop-up
		sValues = (String[]) arguments.GetElement(ACTIONSUBTITLE);

		if (sValues != null)
		{
			m_asModel.SetActionSubTitle(sValues[0]);
		}

		// help topic for the tree pop-up
		sValues = (String[]) arguments.GetElement(HELP_TOPIC);

		if (sValues != null)
		{
			m_asModel.SetHelpTopic(sValues[0]);
		}

		// Class ids to display
		sValues = (String[]) arguments.GetElement(CLASS_IDS_TO_DISPLAY);

		if (sValues != null)
		{
			m_asModel.SetClassIDsToDisplay(GeneralHelpers.ConvertToInts(sValues[0]));
		}

		// Object ids to select
		sValues = (String[]) arguments.GetElement(SELECTED_ITEMS);

		if (sValues != null)
		{
			// Check for referenced long strings
			String strSelectedObjects = TreeDataHelpers.UnReferenceQueryStringData
				(sValues, m_asOwner);

			if (strSelectedObjects == null)
			{
				// Use the normal data if there were no long strings referenced
				strSelectedObjects = sValues[0];
			}

			m_asModel.SetObjectIDsToSelect(GeneralHelpers.ConvertToInts(strSelectedObjects));
		}

		// Can only use one of these two arguments (HIDDEN_ITEMS & DISPLAYED_ITEMS)
		// Object ids to hide
		sValues = (String[]) arguments.GetElement(HIDDEN_ITEMS);

		if (sValues != null)
		{
			// Check for referenced long strings
			String strHiddenObjects = TreeDataHelpers.UnReferenceQueryStringData
				(sValues, m_asOwner);

			if (strHiddenObjects == null)
			{
				// Use the normal data if there were no long strings referenced
				strHiddenObjects = sValues[0];
			}

			m_asModel.SetObjectIDsToHide(GeneralHelpers.ConvertToInts(strHiddenObjects));
		}
		else
		{
			// Object ids to display
			sValues = (String[]) arguments.GetElement(DISPLAYED_ITEMS);

			if (sValues != null)
			{
				m_asModel.SetObjectIDsToDisplay(GeneralHelpers.ConvertToInts(sValues[0]));
			}
		}

		// Whether or not to display the root folder.  Default true.
		sValues = (String[]) arguments.GetElement(SHOW_ROOT);

		if (sValues != null)
		{
			m_asModel.SetShowRoot(XPConvert.ToBoolean(sValues[0]));
		}

		// default DP is TreeFrameDP
		return null;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IControl#Init(IModel, AActivitySpace)
	 */
	public void Init(IModel model, AActivitySpace space)
	{
		m_asModel     = (ITreeModel) model;
		m_asOwner     = (ATreeAS) space;
	}

	// Abstract Methods

	/**
	 * This method is used by the ASManager to return new instances of managed
	 * objects.
	 *
	 * @return A new instance of the managed class (i.e. return new Foo();)
	 */
	abstract public Object Create();

	/**
	 * @see com.plumtree.xpshared.activityspace.IMVCObject#GetName()
	 */
	abstract public String GetName();
}

