package com.plumtree.uiinfrastructure.tree;

import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.openlog.*;
import com.plumtree.openfoundation.util.*;

/**
 * This model handles generating a tree for admin objects.
 *
 * @author Don Hayler
 */
public abstract class ATreeModel implements ITreeModel, ITreeModelRO,
	IRepostModel
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.tree.ATreeModel");

	private static int m_nINITIAL_UNIQUE_ID = 1;
	protected AActivitySpace m_asOwner;
	protected String m_strActionSubTitle  = null;
	protected String m_strOnClickFunction = null;
	protected String m_strParentFormItems = null;
	protected String m_strParentFormName  = null;
	protected String m_strParentFormSpaceID = null;
	protected String m_strParentFormSubmit  = null;
	protected String m_strTitle = null;
	protected TreeCompositeNode m_Root;
	protected XPHashtable m_xpmSelectedItems;
	protected boolean m_bFlatList = false;
	protected boolean m_bAllChecked = false;
	protected boolean m_bMultiSelect = false;
	protected boolean m_bShowRoot = true;
	protected boolean m_bEmptyList = false;
	protected boolean m_bAllowEmptyTreeSubmission = false;
	protected boolean m_bHelpEnabled = false;
	protected int m_nSelectMode = TreeConstants.N_LEAFSELECT;
	protected int m_nSelectOptions = TreeConstants.N_LOOSELEAF;
	protected int m_nSubmitMode = TreeConstants.N_SERVERSUBMIT;
	protected int m_nLastExpandedNodeID = -1;

	// This counter provides the next unique ID for internal tree IDs.
	private int m_nNextUniqueID = m_nINITIAL_UNIQUE_ID;

	// This counter provides the next unique ID for internal tree IDs.
	private static final String UNIQUE_NAME_PREFIX = ",UniqueTreeName";

	// AGC- A simple method to reset the SelectedItems hash map.
	// It doesn't seem trivial to allow an external list of selected
	// items to be added. This may have to be done in a subclass.
	// For now, I'm going to assume that clearing the selected items
	// might be good enough for my purposes.
	public void ClearSelectedItems()
	{
		m_xpmSelectedItems = new XPHashtable();
	}

	/**
	 * Expand or collapse a node in the tree.  This method is responsible
	 * for updating the last expanded node member (m_nLastExpandedNodeID).
	 *
	 * @param boolean bExpand
	 * @param int nNodeID the unique node of the ID in the tree
	 */
	public void ExpandNode(boolean bExpand, int nNodeID)
	{
		// Set the last expanded node id so we can go to its html anchor
		// when we display the page.
		m_nLastExpandedNodeID = nNodeID;

		// If we are expanding, there should be new nodes that may not be checked
		if (bExpand)
		{
			m_bAllChecked = false;
		}
	}

	/**
	 * Returns the most recently expanded or collapsed node in the tree.
	 *
	 * @param int nNodeID the unique node of the ID in the tree that was
	 * most recently expanded or collapsed.  -1 implies no node was expanded
	 * or collapsed in the last repost.
	 */
	public int GetLastExpandedNodeID()
	{
		return m_nLastExpandedNodeID;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#CleanupTempData()
	 */
	public void CleanupTempData()
	{
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#CleanupAllData()
	 */
	public void CleanupAllData()
	{
	}

	/**
	 * This method is used by the ASManager to return new instances of managed
	 * objects.
	 *
	 * @return A new instance of the managed class (i.e. return new Foo();)
	 */
	abstract public Object Create();

	/**
	 * @see com.plumtree.uiinfrastructure.form.IRepostModel#Finish()
	 */
	public void Finish()
	{
	}

	/**
	 * @see com.plumtree.uiinfrastructure.tree.ITreeModelRO#GetActionSubTitle()
	 */
	public String GetActionSubTitle()
	{
		if (m_strActionSubTitle == null)
		{
			// This means a developer missed setting the title.  Should we
			// report an error here?
			return "";
		}
		return m_strActionSubTitle;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.tree.ITreeModelRO#GetHelpEnabled()
	 */
	public boolean GetHelpEnabled()
	{
		return m_bHelpEnabled;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.tree.ITreeModelRO#GetAllowEmptySubmission()
	 */
	public boolean GetAllowEmptySubmission()
	{
		return m_bAllowEmptyTreeSubmission;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.tree.ITreeModelRO#GetDisplayingFlatList()
	 */
	public boolean GetDisplayingFlatList()
	{
		return m_bFlatList;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.tree.ITreeModelRO#GetDisplayingFlatList()
	 */
	public boolean GetAllChecked()
	{
		return m_bAllChecked;
	}

	/**
	 *
	 *
	 * @return
	 */
	public boolean GetMultipleSelect()
	{
		return m_bMultiSelect;
	}

	///////////////////////////
	// RO Methods from IModelRO
	///////////////////////////

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	abstract public String GetName();

	/**
	 *
	 *
	 * @return
	 */
	public String GetOnClickFunction()
	{
		return m_strOnClickFunction;
	}

	/**
	 *
	 *
	 * @return
	 */
	public String GetParentFormName()
	{
		return m_strParentFormName;
	}

	/**
	 *
	 *
	 * @return
	 */
	public String GetParentFormSpaceID()
	{
		return m_strParentFormSpaceID;
	}

	/**
	 *
	 *
	 * @return
	 */
	public String GetParentFormSubmitFunction()
	{
		return m_strParentFormSubmit;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.form.IRepostModel#GetRedirectAfterFinish()
	 */
	public Redirect GetRedirectAfterFinish()
	{
		// Close the Tree pop-up
		Redirect result = new Redirect();
		result.SetLinkGetCachedSpace((AActivitySpace) m_asOwner);
		result.SetDisplayPage(ATreeAS.TREECLOSEPOPUP);

		return result;
	}

	///////////////////////////////
	// RO Methods from ITreeModelRO
	///////////////////////////////

	/**
	 * @see com.plumtree.uiinfrastructure.tree.ITreeModelRO#GetTreeIsEmpty()
	 */
	public boolean GetTreeIsEmpty()
	{
		return m_bEmptyList;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.tree.ITreeModelRO#GetRootTreeIterator()
	 */
	public TreeCompositeIterator GetRootTreeIterator()
	{
		// Make sure the tree accurately displays the selected items.
		RefreshTree();

		return new TreeCompositeIterator(m_Root, m_asOwner);
	}

	/**
	 *
	 *
	 * @return
	 */
	public int GetSelectMode()
	{
		return m_nSelectMode;
	}

	/**
	 *
	 *
	 * @return
	 */
	public int GetSelectOptions()
	{
		return m_nSelectOptions;
	}

	///////////////////
	// For parent space
	///////////////////

	/**
	 * @see com.plumtree.uiinfrastructure.tree.ITreeModelRO#GetSelectedItems()
	 */
	public XPArrayList GetSelectedItems()
	{
		// Add the elements to a sorted dictionary so they come out
		// in a reasonable order
		XPSortedDictionary tempDictionary = new XPSortedDictionary();
		TreeLeafNode tempNode;

		IXPEnumerator temp = m_xpmSelectedItems.GetElements();
		int i = 0;

		while (temp.MoveNext())
		{
			tempNode = (TreeLeafNode) temp.GetCurrent();
			// We need to make the names unique so we don't collapse multiple nodes with the same name
			// to the same element and lose objects.
			tempDictionary.PutElement(tempNode.GetName() + UNIQUE_NAME_PREFIX + i,
				new TreeLeafNode(tempNode));
			i++;
		}

		XPArrayList result = new XPArrayList();

		temp = tempDictionary.GetElements();

		while (temp.MoveNext())
		{
			result.Add(temp.GetCurrent());
		}

		return result;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.tree.ITreeModelRO#GetShowRoot()
	 */
	public boolean GetShowRoot()
	{
		return m_bShowRoot;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.tree.ITreeModelRO#GetSubmitMode()
	 */
	public int GetSubmitMode()
	{
		return m_nSubmitMode;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.tree.ITreeModelRO#GetTitle()
	 */
	public String GetTitle()
	{
		if (m_strTitle == null)
		{
			// This means a developer forgot to set the title.  Should
			// we report this somehow?
			return "";
		}
		return m_strTitle;
	}

	////////////////////////////
	// Write Methods from IModel
	////////////////////////////

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#Init(AActivitySpace)
	 */
	public void Init(AActivitySpace parent)
	{
		m_asOwner = parent;
	}

	/////////////////
	// Helper methods
	/////////////////

	/**
	 * This method goes through the selected items list and makes sure
	 * that all nodes (including multiply instanced nodes) are correctly selected.
	 */
	public void RefreshTree()
	{
		IXPEnumerator temp = m_xpmSelectedItems.GetElements();

		while (temp.MoveNext())
		{
			TreeLeafNode curNode = (TreeLeafNode) temp.GetCurrent();

			if (curNode.m_bMultiplyInstanced)
			{
				// For groups-in-groups (users can be in multiple groups)
				m_Root.SetSelected(curNode.m_nClassID, curNode.m_nObjectID);
			}
			else
			{
				curNode.m_bSelected = true;
			}
		}
	}

	////////////////////////////////
	// Write Methods from ITreeModel
	////////////////////////////////

	public void ResetTree()
	{
		m_Root = null;
		ResetUniqueIDs();
		m_bShowRoot				= true;
		m_bFlatList				= false;
		m_bAllChecked			= false;
		m_bEmptyList			= false;
		m_bAllowEmptyTreeSubmission = false;
		m_bHelpEnabled			= false;
		m_xpmSelectedItems		= new XPHashtable();
		m_nLastExpandedNodeID	= -1;
	}

	/**
	 *
	 */
	public void ResetUniqueIDs()
	{
		m_nNextUniqueID = m_nINITIAL_UNIQUE_ID;
	}

	//////////////////////////////////
	// Write Methods from IRepostModel
	//////////////////////////////////

	/**
	 * @see com.plumtree.uiinfrastructure.form.IRepostModel#SavePage(String, XPHashtable)
	 *
	 * @return int Page status for server side validation
	 */
	public int SavePage(String _sPageName, XPHashtable _htFormData)
	{
		
			IXPEnumerator keys   = _htFormData.GetKeys();
			IXPEnumerator values = _htFormData.GetElements();

			while (keys.MoveNext())
			{
				values.MoveNext();
				log.Debug("Save Page: key = " + keys.GetCurrent() + " value = " +
					((String[]) values.GetCurrent())[0]);
			}
		

		String strSelectedInputName = null;

		if (_htFormData.GetElement(TreeView.HTMLBUTTON_RADIO) != null)
		{
			strSelectedInputName = TreeView.HTMLBUTTON_RADIO;
		}
		else if (_htFormData.GetElement(TreeView.HTMLBUTTON_CHECKBOX) != null)
		{
			strSelectedInputName = TreeView.HTMLBUTTON_CHECKBOX;
		}

		XPHashtable xpmTempSelected = new XPHashtable();

		// Get pre-selected nodes (tree only, not search)
		String[] strTemp = (String[]) _htFormData.GetElement(TreeView.HTMLHIDDEN_SELECTEDIDS);

		if (strTemp != null)
		{
			// tree mode
			int[] nPreSelectedNodeIDs = GeneralHelpers.ConvertToInts(strTemp[0]);
			int nNode;
			String[] strSelectedItems = (String[]) _htFormData.GetElement(strSelectedInputName);

			if (strSelectedItems != null)
			{
				// Find and set the new selected elements
				for (int i = 0; i < strSelectedItems.length; i++)
				{
					nNode = -1;

					try
					{
						try
						{
							nNode = XPConvert.ToInteger(strSelectedItems[i]);
						}
						catch (Exception e)
						{
							throw XPException.GetInstance(e);
						}
					}
					catch (XPFormatException fe)
					{
						
							log.Error("Save Page: ID of selected item in invalid format: " +
								strSelectedItems[i]);
							log.Error(fe, "Format error.");
						

						nNode = -1;
					}

					if (nNode >= m_nINITIAL_UNIQUE_ID)
					{
						TreeLeafNode node = m_Root.GetNode(nNode);

						if (node != null)
						{
							node.m_bSelected = true;
							xpmTempSelected.PutElement(GetKeyFromNode(node), node);
						}
						else 
							log.Warn("Save Page: bad selected element: node ID = " +
								nNode);
						
					}
				}
			}

			// Find items that have been un-selected and remove them from the list.
			for (int i = 0; i < nPreSelectedNodeIDs.length; i++)
			{
				if (nPreSelectedNodeIDs[i] >= m_nINITIAL_UNIQUE_ID)
				{
					TreeLeafNode node = m_Root.GetNode(nPreSelectedNodeIDs[i]);

					if (node != null)
					{
						if (!xpmTempSelected.ContainsKey(GetKeyFromNode(node)))
						{
							// Item was un-selected, remove it
							m_xpmSelectedItems.RemoveElement(GetKeyFromNode(node));

							// make sure we unselect it so it doesn't show up selected in the tree
							node.m_bSelected = false;
						}
					}
					else 
						log.Warn("Save Page: bad pre-selected element: node ID = " +
							nPreSelectedNodeIDs[i]);
					
				}
			}
		}

		IXPEnumerator temp = xpmTempSelected.GetElements();

		// Clear the selected list if we have a selection in single select mode
		if ((!m_bMultiSelect) && (xpmTempSelected.GetSize() > 0))
		{
			m_xpmSelectedItems.Clear();

			// Add the single selected item
			if (temp.MoveNext())
			{
				TreeLeafNode curNode = (TreeLeafNode) temp.GetCurrent();
				m_xpmSelectedItems.PutElement(GetKeyFromNode(curNode), curNode);
			} else
			{
				
						log.Error("Tree Save Page: Couldn't access single selected item.");
					
			}
		}
		else
		{
			// Add all selected items to the list
			while (temp.MoveNext())
			{
				TreeLeafNode curNode = (TreeLeafNode) temp.GetCurrent();
				m_xpmSelectedItems.PutElement(GetKeyFromNode(curNode), curNode);
			}
		}

		// Check if the user checked the check all box
		m_bAllChecked = false;

		String[] strCheckAll = (String[]) _htFormData.GetElement(TreeView.HTMLINPUT_CHECK_ALL);

		if (strCheckAll != null)
		{
			// This will only be here if it is checked.
			m_bAllChecked = true;
		}

		return RepostControl.PAGE_STATUS_VALID;
	}

    /**
     * @see com.plumtree.uiinfrastructure.tree.ITreeModel#RemoveFromSelection(int classID, int objectID)
     */
    public void RemoveFromSelection(int classID, int objectID)
    {
        // mel: This method was added to assist in fixing 42066
        try
        {
            // Ignore remove if there are no elements in the tree
            if(m_Root == null)
            {
                return;
            }

            TreeLeafNode node = m_Root.GetNode(classID, objectID);

            // Ignore remove if no element exists identified by classID,objectID
            if(node == null)
            {
                return;
            }

            String nodeKey = GetKeyFromNode(node);

            if(m_xpmSelectedItems.ContainsKey(nodeKey))
            {
                m_xpmSelectedItems.RemoveElement(nodeKey);
                node.m_bSelected = false;
            }
        }
        catch(Exception anyException)
        {
            log.Error(anyException, "ATreeModel.RemoveFromSelection Failed to remove classID,objectID from the tree selection:" + classID + "," + objectID);
        }
    }

	/**
	 * @see com.plumtree.uiinfrastructure.tree.ITreeModel#SetActionSubTitle(String)
	 */
	public void SetActionSubTitle(String strSubTitle)
	{
		m_strActionSubTitle = strSubTitle;
	}


	/**
	 * @see com.plumtree.uiinfrastructure.tree.ITreeModel#SetAllowEmptySubmission(boolean)
	 */
	public void SetAllowEmptySubmission(boolean bAllow)
	{
		m_bAllowEmptyTreeSubmission = bAllow;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.tree.ITreeModel#SetHelpTopic(String)
	 */
	public void SetHelpTopic(String strHelpTopic)
	{
		m_asOwner.SetHelpTopic(strHelpTopic);
		m_bHelpEnabled = true;
	}

	/**
	 *
	 *
	 * @param nIDs
	 */
	abstract public void SetClassIDsToDisplay(int[] nIDs);

	/**
	 *
	 *
	 * @param bMultiSelect
	 */
	public void SetMultipleSelect(boolean bMultiSelect)
	{
		m_bMultiSelect = bMultiSelect;
	}

	/**
	 *
	 *
	 * @param nIDs
	 */
	abstract public void SetObjectIDsToDisplay(int[] nIDs);

	/**
	 *
	 *
	 * @param nIDs
	 */
	abstract public void SetObjectIDsToHide(int[] nIDs);

	/**
	 *
	 *
	 * @param nIDs
	 */
	abstract public void SetObjectIDsToSelect(int[] nIDs);

	/**
	 *
	 *
	 * @return
	 */
	public int GetNextUniqueID()
	{
		return m_nNextUniqueID++;
	}

	/**
	 *
	 *
	 * @param name
	 */
	public void SetOnClickFunction(String name)
	{
		m_strOnClickFunction = name;
	}

	/**
	 *
	 *
	 * @param name
	 */
	public void SetParentFormName(String name)
	{
		m_strParentFormName = name;
	}

	/**
	 *
	 *
	 * @param name
	 */
	public void SetParentFormSpaceID(String name)
	{
		m_strParentFormSpaceID = name;
	}

	/**
	 *
	 *
	 * @param name
	 */
	public void SetParentFormSubmitFunction(String name)
	{
		m_strParentFormSubmit = name;
	}

	/**
	 *
	 *
	 * @param nSelectMode
	 */
	public void SetSelectMode(int nSelectMode)
	{
		m_nSelectMode = nSelectMode;
	}

	/**
	 *
	 *
	 * @param nSelectOptions
	 */
	public void SetSelectOptions(int nSelectOptions)
	{
		m_nSelectOptions = nSelectOptions;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.tree.ITreeModel#SetShowRoot(boolean)
	 */
	public void SetShowRoot(boolean bShowRoot)
	{
		m_bShowRoot = bShowRoot;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.tree.ITreeModel#SetSubmitMode(int)
	 */
	public void SetSubmitMode(int nMode)
	{
		m_nSubmitMode = nMode;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.tree.ITreeModel#SetTitle(String)
	 */
	public void SetTitle(String strTitle)
	{
		m_strTitle = strTitle;
	}

	/**
	 *
	 *
	 * @param strKey
	 *
	 * @return
	 */
	protected TreeCompositeNode GetCompositeIfAlreadySelected(String strKey)
	{
		TreeCompositeNode myFolder = (TreeCompositeNode) m_xpmSelectedItems.GetElement(strKey);

		if (myFolder == null)
		{
			return null;
		}

		if (myFolder.m_nTreeNodeID != -1)
		{
			
				log.Warn("attempting to create a new tree folder node that is already selected.  Key " +
					strKey);
			
		}
		else
		{
			TreeCompositeNode temp = new TreeCompositeNode(GetNextUniqueID());
			temp.SetNameAndDescription(myFolder);
			temp.m_nObjectID	   = myFolder.m_nObjectID;
			temp.m_nClassID		   = myFolder.m_nClassID;
			temp.m_strLocation     = myFolder.m_strLocation;
			temp.m_bSelected	   = myFolder.m_bSelected;
			temp.m_Image		   = myFolder.m_Image;
			temp.m_OpenImage	   = myFolder.m_OpenImage;

			return temp;
		}

		return null;
	}

	/**
	 *
	 *
	 * @param strKey
	 *
	 * @return
	 */
	protected TreeLeafNode GetLeafIfAlreadySelected(String strKey)
	{
		TreeLeafNode myLeaf = (TreeLeafNode) m_xpmSelectedItems.GetElement(strKey);

		if (myLeaf == null)
		{
			return null;
		}

		if (myLeaf.m_nTreeNodeID != -1)
		{
			
				log.Warn("attempting to create a new tree node that is already selected.  Key " +
					strKey);
			
		}
		else
		{
			// Should this copy everything?  expanded, etc...?
			TreeLeafNode temp = new TreeLeafNode(GetNextUniqueID());
			temp.SetNameAndDescription(myLeaf);
			temp.m_nObjectID	   = myLeaf.m_nObjectID;
			temp.m_nClassID		   = myLeaf.m_nClassID;
			temp.m_strLocation     = myLeaf.m_strLocation;
			temp.m_bSelected	   = myLeaf.m_bSelected;
			temp.m_Image		   = myLeaf.m_Image;

			return temp;
		}

		return null;
	}

	/**
	 * This function creates a unique key given a node.
	 * It is used to store items in XPHashtables.
	 * Override this function if your tree's nodes
	 * can't be differentiated by "classid, objectid"
	 *
	 * @param node
	 * @return String
	 */
	protected String GetKeyFromNode(TreeLeafNode node)
	{
		return node.m_nClassID + "," + node.m_nObjectID;
	}
}

