package com.plumtree.uiinfrastructure.tree;

/**
 * This interface contains all of the writable (set) methods that a tree model
 * must implement.
 *
 * @author Don Hayler
 */
public interface ITreeModel extends ITreeModelRO
{
	/**
	 * Expand or collapse a node in the tree
	 */
	public void ExpandNode(boolean bExpand, int nNodeID);

	/**
	 * Accessor for managed series of unique tree ids.  All objects
	 * in a folder should have sequential tree ids.
	 */
	public int GetNextUniqueID();

	/**
	 * This method initializes tree variables.  sub-classes can extend this
	 * method.
	 */
	public void ResetTree();

	/**
	 * This resets the unique tree ids.  This should only be called
	 * when resetting the entire tree.
	 */
	public void ResetUniqueIDs();

	/**
	 * Set the action sub-title for the tree popup
	 * I.E. select a user
	 *
	 * @param String the action sub-title
	 */
	public void SetActionSubTitle(String strSubTitle);

	/**
	 * Set the help topic for the tree popup
	 *
	 * @param String the help topic
	 */
	public void SetHelpTopic(String strHelpTopic);

	/**
	 * Allow users to click finish on trees without selecting
	 * anything.
	 *
	 * @param boolean true implies allow
	 */
	public void SetAllowEmptySubmission(boolean bAllow);

	/**
	 * This method will set the class IDs to be displayed.
	 *
	 * This must be run before SetObjectIDsToHide() and
	 * SetObjectIDsToSelect().
	 *
	 * @param int[] nIDs the class IDs to show
	 */
	public void SetClassIDsToDisplay(int[] nIDs);

	/**
	 *
	 *
	 * @param bMultiSelect
	 */
	public void SetMultipleSelect(boolean bMultiSelect);

	/**
	 * This method will display only the given objects.
	 *
	 * Only this method or SetObjectIDsToHide() can be called,
	 * not both.
	 *
	 * @param int[] nIDs classID1,objectID1,classID2,objectID2,etc...
	 */
	public void SetObjectIDsToDisplay(int[] nIDs);

	/**
	 * This method will hide the given objects.
	 *
	 * Only this method or SetObjectIDsToDisplay() can be called,
	 * not both.
	 *
	 * @param int[] nIDs classID1,objectID1,classID2,objectID2,etc...
	 */
	public void SetObjectIDsToHide(int[] nIDs);

	/**
	 * This method will select the given objects.
	 *
	 * @param int[] nIDs classID1,objectID1,classID2,objectID2,etc...
	 */
	public void SetObjectIDsToSelect(int[] nIDs);


    /**
     * Removes an object from the selected items if it exists and is selected.
     * The pair of classID,objectID uniquely identify each object.
     *
     * This method has no effect if the object does not exist or is not selected.
     * This method does not throw exceptions but does log errors.
     *
     * @param classID The classID from PT_CLASSIDS of the object to remove
     * @param objectID The objectID of the object to remove
     */
    public void RemoveFromSelection(int classID, int objectID);

	/**
	 *
	 *
	 * @param name
	 */
	public void SetOnClickFunction(String name);

	/**
	 *
	 *
	 * @param name
	 */
	public void SetParentFormName(String name);

	/**
	 *
	 *
	 * @param name
	 */
	public void SetParentFormSpaceID(String name);

	/**
	 *
	 *
	 * @param name
	 */
	public void SetParentFormSubmitFunction(String name);

	/**
	 *
	 *
	 * @param nSelectMode
	 */
	public void SetSelectMode(int nSelectMode);

	/**
	 *
	 *
	 * @param nSelectOptions
	 */
	public void SetSelectOptions(int nSelectOptions);

	/**
	 * This method sets whether or not to show the root folder.
	 *
	 * @param boolean true implies show root (default)
	 */
	public void SetShowRoot(boolean bShowRoot);

	/**
	 *
	 *
	 * @param nSubmitMode
	 */
	public void SetSubmitMode(int nSubmitMode);

	/**
	 * Set the title for the tree popup
	 *
	 * @param String the title
	 */
	public void SetTitle(String strTitle);
}

