package com.plumtree.uiinfrastructure.tree;

import com.plumtree.openlog.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.htmlconstructs.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This iterator is used for walking through the tree.  The
 * GetBase* methods should always be valid, but the GetCurrent*
 * methods are only valid after MoveToNextCompositeChild() has
 * been called and has returned true.
 *
 * @author Don Hayler
 */
public class TreeCompositeIterator implements ITreeIterator
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.tree.TreeCompositeIterator");

	private AActivitySpace m_asOwner;
	private HTMLImg m_DefaultClosedImage	    = null;
	private HTMLImg m_DefaultOpenImage		    = null;
	private IXPEnumerator m_CompositeEnumerator;
	private IXPEnumerator m_LeafEnumerator;
	private TreeCompositeNode m_CurrentComposite;

	private TreeCompositeNode m_Root;

	/**
	 * Constructor for iterator
	 *
	 * @param TreeCompositeNode the root node of this iterator
	 * @param AActivitySpace asOwner the space that this tree is in
	 *             (used to get default localized strings)
	 */
	public TreeCompositeIterator(TreeCompositeNode root, AActivitySpace asOwner)
	{
		m_asOwner     = asOwner;

		m_Root     = root;

		m_LeafEnumerator     = root.GetLeafChildIterator();

		m_CompositeEnumerator     = root.GetCompositeChildIterator();
		m_CurrentComposite		  = null;
	}

	/**
	 * Private constructor so no one can use it.
	 */
	private TreeCompositeIterator()
	{
	}

	/**
	 *
	 *
	 * @return
	 */
	public int GetBaseClassID()
	{
		if (m_Root != null)
		{
			return m_Root.m_nClassID;
		}

		return -1;
	}

	/**
	 *
	 *
	 * @return
	 */
	public boolean GetBaseDisabled()
	{
		if (m_Root != null)
		{
			return m_Root.m_bDisabled;
		}

		return false;
	}

	/**
	 *
	 *
	 * @return
	 */
	public boolean GetBaseBrowsableOnly()
	{
		if (m_Root != null)
		{
			return m_Root.m_bBrowsableOnly;
		}

		return false;
	}

	/**
	 *
	 *
	 * @return
	 */
	public boolean GetBaseExpanded()
	{
		if (m_Root != null)
		{
			return m_Root.m_bExpanded;
		}

		return false;
	}

	/**
	 *
	 *
	 * @return
	 */
	public boolean GetBaseForceSelectable()
	{
		if (m_Root != null)
		{
			return m_Root.m_bForceSelectable;
		}

		return false;
	}

	/**
	 * This method returns the current image.  This will
	 * automatically handle open and closed images.
	 *
	 * @return HTMLImg the current image
	 */
	public HTMLImg GetBaseImage()
	{
		HTMLImg result = null;

		if (m_Root != null)
		{
			if (m_Root.m_bExpanded)
			{
				result = m_Root.m_OpenImage;
			}
			else
			{
				result = m_Root.m_Image;
			}
		}

		if (result == null)
		{
			result = GetDefaultImage(m_Root.m_bExpanded);
		}

		return result;
	}

	/**
	 * This method returns an iterator to the leaf child nodes
	 * of the root node.
	 *
	 * @return iterator to the leaf children of the root node.
	 */
	public TreeLeafIterator GetBaseLeafChildIterator()
	{
		return new TreeLeafIterator(this.m_LeafEnumerator, m_asOwner);
	}

	/**
	 * This method returns the number of leaf child nodes
	 * of the root node.
	 *
	 * @return the number of leaf children of the root node.
	 */
	public int GetBaseLeafChildCount()
	{
		if (m_Root != null)
		{
			return m_Root.GetLeafChildCount();
		}

		return 0;
	}

	/**
	 *
	 *
	 * @return
	 */
	public String GetBaseLocation()
	{
		if (m_Root != null)
		{
			return m_Root.m_strLocation;
		}

		return null;
	}

	/**
	 *
	 *
	 * @return
	 */
	public String GetBaseEncodedName()
	{
		if (m_Root != null)
		{
			return m_Root.GetEncodedName();
		}

		return null;
	}

	/**
	 *
	 *
	 * @return
	 */
	public String GetBaseDescription()
	{
		if (m_Root != null)
		{
			return m_Root.GetDescription();
		}

		return null;
	}

	/**
	 *
	 *
	 * @return
	 */
	public int GetBaseObjectID()
	{
		if (m_Root != null)
		{
			return m_Root.m_nObjectID;
		}

		return -1;
	}

	/**
	 *
	 *
	 * @return
	 */
	public boolean GetBaseSelected()
	{
		if (m_Root != null)
		{
			return m_Root.m_bSelected;
		}

		return false;
	}

	/**
	 *
	 *
	 * @return
	 */
	public int GetBaseTreeNodeID()
	{
		if (m_Root != null)
		{
			return m_Root.m_nTreeNodeID;
		}

		return -1;
	}

	/**
	 *
	 *
	 * @return
	 */
	public int GetCurrentClassID()
	{
		if (m_CurrentComposite != null)
		{
			return m_CurrentComposite.m_nClassID;
		}

		return -1;
	}

	/**
	 *
	 *
	 * @return
	 */
	public TreeCompositeIterator GetCurrentCompositeChildIterator()
	{
		if (m_CurrentComposite != null)
		{
			return new TreeCompositeIterator(m_CurrentComposite, m_asOwner);
		}

		return null;
	}

	/**
	 *
	 *
	 * @return
	 */
	public boolean GetCurrentDisabled()
	{
		if (m_CurrentComposite != null)
		{
			return m_CurrentComposite.m_bDisabled;
		}

		return false;
	}

	/**
	 *
	 *
	 * @return
	 */
	public boolean GetCurrentExpanded()
	{
		if (m_CurrentComposite != null)
		{
			return m_CurrentComposite.m_bExpanded;
		}

		return false;
	}

	/**
	 *
	 *
	 * @return
	 */
	public boolean GetCurrentForceSelectable()
	{
		if (m_CurrentComposite != null)
		{
			return m_CurrentComposite.m_bForceSelectable;
		}

		return false;
	}

	/**
	 * This method returns the current image.  This will
	 * automatically handle open and closed images.
	 *
	 * @return HTMLImg the current image
	 */
	public HTMLImg GetCurrentImage()
	{
		HTMLImg result = null;

		if (m_CurrentComposite != null)
		{
			if (m_CurrentComposite.m_bExpanded)
			{
				result = m_CurrentComposite.m_OpenImage;
			}
			else
			{
				result = m_CurrentComposite.m_Image;
			}
		}

		if (result == null)
		{
			result = GetDefaultImage(m_CurrentComposite.m_bExpanded);
		}

		return result;
	}

	/**
	 *
	 *
	 * @return
	 */
	public String GetCurrentLocation()
	{
		if (m_CurrentComposite != null)
		{
			return m_CurrentComposite.m_strLocation;
		}

		return null;
	}

	/**
	 *
	 *
	 * @return
	 */
	public String GetCurrentEncodedName()
	{
		if (m_CurrentComposite != null)
		{
			return m_CurrentComposite.GetEncodedName();
		}

		return null;
	}

	/**
	 *
	 *
	 * @return
	 */
	public int GetCurrentObjectID()
	{
		if (m_CurrentComposite != null)
		{
			return m_CurrentComposite.m_nObjectID;
		}

		return -1;
	}

	/**
	 *
	 *
	 * @return
	 */
	public boolean GetCurrentSelected()
	{
		if (m_CurrentComposite != null)
		{
			return m_CurrentComposite.m_bSelected;
		}

		return false;
	}

	/**
	 *
	 *
	 * @return
	 */
	public int GetCurrentTreeNodeID()
	{
		if (m_CurrentComposite != null)
		{
			return m_CurrentComposite.m_nTreeNodeID;
		}

		return -1;
	}

	/**
	 * Moves to the next composite child.
	 *
	 * @return boolean false if there are no more composite children.
	 */
	public boolean MoveToNextCompositeChild()
	{
		if (m_CompositeEnumerator.MoveNext())
		{
			m_CurrentComposite = (TreeCompositeNode) m_CompositeEnumerator.GetCurrent();
			return true;
		}

		m_CurrentComposite = null;
		return false;
	}

	/////////////////
	// Helper methods
	/////////////////
	private HTMLImg GetDefaultImage(boolean bExpanded)
	{
		HTMLImg result = null;

		if (bExpanded)
		{
			if (m_DefaultOpenImage == null)
			{
				try
				{
					m_DefaultOpenImage = (HTMLImg) UIConstructFactory.GetConstruct(PTImageType.ICON_FOLDER_OPEN,
							true, m_asOwner).GetHTMLElement();
					m_DefaultOpenImage.SetBorder("0");


					m_DefaultOpenImage.SetAlt(m_asOwner.GetString(78,
							ResourceConstants.INFRASTRUCTURE));
				}
				catch (Exception e)
				{
					
						log.Error(e, "Unable to construct generic open folder image.");
					

					m_DefaultOpenImage = null;
				}
			}

			result = m_DefaultOpenImage;
		}
		else
		{
			if (m_DefaultClosedImage == null)
			{
				try
				{
					m_DefaultClosedImage = (HTMLImg) UIConstructFactory.GetConstruct(PTImageType.ICON_FOLDER,
							true, m_asOwner).GetHTMLElement();
					m_DefaultClosedImage.SetBorder("0");


					m_DefaultClosedImage.SetAlt(m_asOwner.GetString(78,
							ResourceConstants.INFRASTRUCTURE));
				}
				catch (Exception e)
				{
					
						log.Error(e, "Unable to construct generic closed folder image.");
					

					m_DefaultClosedImage = null;
				}
			}

			result = m_DefaultClosedImage;
		}

		return result;
	}
}

