package com.plumtree.uiinfrastructure.tree;

import com.plumtree.openfoundation.util.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This class represents composite nodes in the tree such
 * as folders, or browseable groups.
 *
 * @author Don Hayler
 */
public class TreeCompositeNode extends TreeLeafNode
{
	/** The image when the container is open */
	public HTMLImg m_OpenImage		 = null;

	/** Have the children of this node been loaded? */
	public boolean m_bChildrenLoaded = false;

	/** Neither browsable nor selectable */
	public boolean m_bDisabled = false;

	/** true implies browsable but not selectable */
	public boolean m_bBrowsableOnly = false;

	/** Is this folder expanded */
	public boolean m_bExpanded = false;

	/** children arrays */
	private XPArrayList m_CompositeChildren = new XPArrayList();
	private XPArrayList m_LeafChildren	    = new XPArrayList();

	/**
	 * Creates a new TreeCompositeNode object.
	 * 
	 * All objects in a folder should have sequential tree ids.
	 *
	 * @param int internal ID of this node
	 */
	public TreeCompositeNode(int nTreeNodeID)
	{
		super(nTreeNodeID);
	}

	// This should never be called.
	private TreeCompositeNode()
	{
		super(-1);
	}

	/**
	 *
	 *
	 * @param child
	 */
	public void AddCompositeChild(TreeCompositeNode child)
	{
		if (child != null)
		{
			m_CompositeChildren.Add(child);
		}
	}

	/**
	 *
	 *
	 * @param child
	 */
	public void RemoveCompositeChild(TreeCompositeNode child)
	{
		if (child != null)
		{
			m_CompositeChildren.Remove(child);
		}
	}

	/**
	 *
	 *
	 * @param child
	 */
	public void AddLeafChild(TreeLeafNode child)
	{
		if (child != null)
		{
			m_LeafChildren.Add(child);
		}
	}

	/**
	 *
	 *
	 * @param child
	 */
	public void RemoveLeafChild(TreeLeafNode child)
	{
		if (child != null)
		{
			m_LeafChildren.Remove(child);
		}
	}

	// This method recursively clears the selected bits on all nodes and leaves
	// underneath this node
	public void ClearSelected(boolean bVisibleOnly, boolean bRecursive)
	{
		m_bSelected = false;

		// Only clear the children if this node is expanded or we are clearing all children
		if (m_bExpanded || !bVisibleOnly)
		{
			IXPEnumerator enumerator = m_CompositeChildren.GetEnumerator();

			while (enumerator.MoveNext())
			{
				if (bRecursive)
				{
					((TreeCompositeNode) enumerator.GetCurrent()).ClearSelected(bVisibleOnly,
						bRecursive);
				}
				else
				{
					((TreeCompositeNode) enumerator.GetCurrent()).m_bSelected = false;
				}
			}

			enumerator = m_LeafChildren.GetEnumerator();

			while (enumerator.MoveNext())
			{
				((TreeLeafNode) enumerator.GetCurrent()).m_bSelected = false;
			}
		}
	}

	/**
	 *
	 *
	 * @return
	 */
	public IXPEnumerator GetCompositeChildIterator()
	{
		return m_CompositeChildren.GetEnumerator();
	}

	/**
	 *
	 *
	 * @return
	 */
	public IXPEnumerator GetLeafChildIterator()
	{
		return m_LeafChildren.GetEnumerator();
	}

	/**
	 * This method calculates how many leaf children this node contains.
	 *
	 * @return the number of leaf children in this node.
	 */
	public int GetLeafChildCount()
	{
		return m_LeafChildren.GetSize();
	}

	/**
	 *
	 *
	 * @param nNodeID
	 *
	 * @return
	 */
	public TreeLeafNode GetNode(int nNodeID)
	{
		// Search this node
		if (nNodeID == m_nTreeNodeID)
		{
			return this;
		}

		// Search child leaf nodes
		IXPEnumerator enumerator = m_LeafChildren.GetEnumerator();

		while (enumerator.MoveNext())
		{
			if (((TreeLeafNode) enumerator.GetCurrent()).m_nTreeNodeID == nNodeID)
			{
				return (TreeLeafNode) enumerator.GetCurrent();
			}
		}

		// Search child composite nodes
		TreeLeafNode result = null;
		enumerator = m_CompositeChildren.GetEnumerator();

		while ((enumerator.MoveNext()) && (result == null))
		{
			result = ((TreeCompositeNode) enumerator.GetCurrent()).GetNode(nNodeID);
		}

		return result;
	}

	/**
	 *
	 *
	 * @param nClassID
	 * @param nObjectID
	 *
	 * @return
	 */
	public TreeLeafNode GetNode(int nClassID, int nObjectID)
	{
		// Search this node
		if ((nClassID == m_nClassID) && (nObjectID == m_nObjectID))
		{
			return this;
		}

		TreeLeafNode current;

		// Search child leaf nodes
		IXPEnumerator enumerator = m_LeafChildren.GetEnumerator();

		while (enumerator.MoveNext())
		{
			current = (TreeLeafNode) enumerator.GetCurrent();

			if ((current.m_nClassID == nClassID) &&
					(current.m_nObjectID == nObjectID))
			{
				return current;
			}
		}

		// Search child composite nodes
		enumerator = m_CompositeChildren.GetEnumerator();

		while (enumerator.MoveNext())
		{
			current = ((TreeCompositeNode) enumerator.GetCurrent()).GetNode(nClassID,
					nObjectID);

			if (current != null)
			{
				return current;
			}
		}

		return null;
	}

	/**
	 * Note: This function compares the location to
	 * get the node
	 *
	 * @param nClassID
	 * @param nObjectID
	 *
	 * @return
	 */
	public TreeLeafNode GetNode(String strLocation)
	{
		// Search this node
		if (m_strLocation.equals(strLocation))
		{
			return this;
		}

		TreeLeafNode current;

		// Search child leaf nodes
		IXPEnumerator enumerator = m_LeafChildren.GetEnumerator();

		while (enumerator.MoveNext())
		{
			current = (TreeLeafNode) enumerator.GetCurrent();

			if (current.m_strLocation.equals(strLocation))
			{
				return current;
			}
		}

		// Search child composite nodes
		enumerator = m_CompositeChildren.GetEnumerator();

		while (enumerator.MoveNext())
		{
			current = ((TreeCompositeNode) enumerator.GetCurrent()).GetNode(strLocation);

			if (current != null)
			{
				return current;
			}
		}

		return null;
	}

	/**
	 * This method searches the entire tree for nodes with the classID
	 * and objectID and sets them to selected.  This should only be used
	 * for multiply instanced nodes.
	 */
	public void SetSelected(int nClassID, int nObjectID)
	{
		// Search this node
		if ((m_nClassID == nClassID) && (m_nObjectID == nObjectID))
		{
			m_bSelected = true;
		}

		// Set child leaf nodes
		IXPEnumerator enumerator = m_LeafChildren.GetEnumerator();
		TreeLeafNode curLeafNode;

		while (enumerator.MoveNext())
		{
			curLeafNode = (TreeLeafNode) enumerator.GetCurrent();

			if ((curLeafNode.m_nClassID == nClassID) &&
					(curLeafNode.m_nObjectID == nObjectID))
			{
				curLeafNode.m_bSelected = true;
			}
		}

		// Set child composite nodes
		enumerator = m_CompositeChildren.GetEnumerator();

		while (enumerator.MoveNext())
		{
			((TreeCompositeNode) enumerator.GetCurrent()).SetSelected(nClassID,
				nObjectID);
		}
	}
}

