package com.plumtree.uiinfrastructure.tree;

import com.plumtree.openlog.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.htmlconstructs.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This iterator is used for walking through the tree.  The
 * GetGetCurrent* methods are only valid after MoveToNextChild()
 * has been called and has returned true.
 *
 * @author Don Hayler
 */
public class TreeLeafIterator implements ITreeIterator
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.tree.TreeLeafIterator");

	private AActivitySpace m_asOwner;
	private IXPEnumerator m_ChildEnumerator;
	private TreeLeafNode m_CurrentChild;

	/**
	 * Constructor for iterator
	 *
	 * @param IXPEnumerator the child nodes for this iterator
	 * @param AActivitySpace asOwner the space that this tree is in
	 *             (used to get default localized strings)
	 */
	public TreeLeafIterator(IXPEnumerator children, AActivitySpace asOwner)
	{
		m_asOwner			  = asOwner;
		m_ChildEnumerator     = children;
		m_CurrentChild		  = null;
	}

	/**
	 * Private constructor so no one can use it.
	 */
	private TreeLeafIterator()
	{
	}

	/**
	 *
	 *
	 * @return
	 */
	public int GetCurrentClassID()
	{
		if (m_CurrentChild != null)
		{
			return m_CurrentChild.m_nClassID;
		}

		return -1;
	}

	/**
	 *
	 *
	 * @return
	 */
	public boolean GetCurrentForceSelectable()
	{
		if (m_CurrentChild != null)
		{
			return m_CurrentChild.m_bForceSelectable;
		}

		return false;
	}

	/**
	 * This method returns the current image.  This will
	 * automatically handle open and closed images.
	 *
	 * @return HTMLImg the current image
	 */
	public HTMLImg GetCurrentImage()
	{
		HTMLImg result = null;

		if (m_CurrentChild != null)
		{
			result = m_CurrentChild.m_Image;
		}

		if (result == null)
		{
			try
			{
				result = (HTMLImg) UIConstructFactory.GetConstruct(PTImageType.ICON_DOCUMENT,
						true, m_asOwner).GetHTMLElement();
				result.SetBorder("0");


				result.SetAlt(m_asOwner.GetString(79, ResourceConstants.INFRASTRUCTURE));
			}
			catch (Exception e)
			{
				
					log.Error(e, "Unable to construct generic image.");
				

				return null;
			}
		}

		return result;
	}

	/**
	 *
	 *
	 * @return
	 */
	public String GetCurrentLocation()
	{
		if (m_CurrentChild != null)
		{
			return m_CurrentChild.m_strLocation;
		}

		return null;
	}

	/**
	 *
	 *
	 * @return
	 */
	public String GetCurrentEncodedName()
	{
		if (m_CurrentChild != null)
		{
			return m_CurrentChild.GetEncodedName();
		}

		return null;
	}

	/**
	 *
	 *
	 * @return
	 */
	public String GetCurrentDescription()
	{
		if (m_CurrentChild != null)
		{
			return m_CurrentChild.GetDescription();
		}

		return null;
	}

	/**
	 *
	 *
	 * @return
	 */
	public int GetCurrentObjectID()
	{
		if (m_CurrentChild != null)
		{
			return m_CurrentChild.m_nObjectID;
		}

		return -1;
	}

	/**
	 *
	 *
	 * @return
	 */
	public boolean GetCurrentSelected()
	{
		if (m_CurrentChild != null)
		{
			return m_CurrentChild.m_bSelected;
		}

		return false;
	}

	/**
	 *
	 *
	 * @return
	 */
	public int GetCurrentTreeNodeID()
	{
		if (m_CurrentChild != null)
		{
			return m_CurrentChild.m_nTreeNodeID;
		}

		return -1;
	}

	/**
	 * Moves to the next child.
	 *
	 * @return boolean false if there are no more children.
	 */
	public boolean MoveToNextChild()
	{
		if (m_ChildEnumerator.MoveNext())
		{
			m_CurrentChild = (TreeLeafNode) m_ChildEnumerator.GetCurrent();
			return true;
		}

		m_CurrentChild = null;
		return false;
	}
}

