package com.plumtree.uiinfrastructure.tree;

import com.plumtree.openfoundation.web.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This class represents a node in the tree.
 *
 * @author Don Hayler
 */
public class TreeLeafNode
{
	/** unique internal ID for this node */
	public int m_nTreeNodeID;

	/**
	 * The default image for this object.  For a composite node, this will be
	 * the closed image (the composite node will store the open image).
	 */
	public HTMLImg m_Image	    = null;

	public String m_strLocation = null;

	/** Node data */
	/** The name is private so we can truncate it properly. */
	private String m_strName = null;

	/** The description is private so we can truncate it properly. */
	private String m_strDescription = null;

	/** Whether or not to override the normal selectable logic and
	 * force this node to be selectable.
	 * true implies this node should be selectable no matter what.
	 */
	public boolean m_bForceSelectable = false;

	/** Whether or not this node can be simultaneously located
	 * in multiple places.  true implies there can be more
	 * than one instance of the object this node points to in
	 * the same tree.
	 *
	 * N.B.  Multiply instanced items must use unique class ID /
	 * object ID pairs (per item, not per instance).
	 */
	public boolean m_bMultiplyInstanced = false;

	/** Whether or not this node is selected. */
	public boolean m_bSelected = false;

	public int m_nClassID  = -1;

	public int m_nObjectID = -1;

	/**
	 * Creates a new TreeLeafNode object.
	 *
	 * All objects in a folder should have sequential tree ids.
	 * 
	 * @param int internal ID of this node
	 */
	public TreeLeafNode(int nTreeNodeID)
	{
		m_nTreeNodeID = nTreeNodeID;
	}

	/**
	 * Creates a new TreeLeafNode object.
	 *
	 * @param node
	 */
	public TreeLeafNode(TreeLeafNode node)
	{
		m_nTreeNodeID			 = node.m_nTreeNodeID;
		m_strName				 = node.m_strName;
		m_strDescription		 = node.m_strDescription;
		m_nObjectID				 = node.m_nObjectID;
		m_nClassID				 = node.m_nClassID;
		m_strLocation			 = node.m_strLocation;
		m_Image					 = node.m_Image;
		m_bSelected				 = node.m_bSelected;
		m_bMultiplyInstanced     = node.m_bMultiplyInstanced;
		m_bForceSelectable		 = node.m_bForceSelectable;
	}

	// No default constructor - this should never be called
	private TreeLeafNode()
	{
		m_nTreeNodeID = -1;
	}

	/**
	 * This method returns the name of the node (truncated to
	 * 50 characters).
	 *
	 * This method unencodes the stored
	 * encoded name, so it should only be used when retrieving
	 * the final tree selections.  Tracing should use the
	 * GetEncodedName method.
	 *
	 * @return String the name of the node.
	 */
	public String GetName()
	{
		return XPHttpUtility.HtmlDecode(m_strName);
	}

	/**
	 * This method returns the name of the node (truncated to
	 * 50 characters and HTMLEncoded).
	 *
	 * The name is stored truncated and encoded, so this method
	 * is fast.  Tracing should use this method, even though it
	 * does not need to encode the name.
	 *
	 * @return String the name of the node.
	 */
	public String GetEncodedName()
	{
		return m_strName;
	}

	/**
	 * This method sets the name of the node, truncates it to
	 * 50 characters, and HTMLEncodes it.  This also sets the
	 * description to the full name, if there is no description
	 * set.
	 *
	 * @param String the name of the node.
	 */
	public void SetName(String strName)
	{
		// true means only truncate if there is a word longer
		// than the max length.
		m_strName = GeneralHelpers.TruncateLongString(strName,
			ATreeAS.MAX_DISPLAYED_NAME_LENGTH, true);
		m_strName = XPHttpUtility.HtmlEncode(m_strName);

		if ((m_strDescription == null) || ("".equals(m_strDescription)))
		{
			SetDescription(strName);
		}
	}

	/**
	 * This method sets the name and description
	 * of the node by copying the truncated and
	 * encoded versions from the node.
	 *
	 * @param TreeLeafNode the node to take the info from.
	 */
	public void SetNameAndDescription(TreeLeafNode node)
	{
		m_strName = node.m_strName;
		m_strDescription = node.m_strDescription;
	}

	/**
	 * This method returns the description of the node (HTMLEncoded).
	 * If there is no description, it returns the name.
	 *
	 * @return String the description of the node.
	 */
	public String GetDescription()
	{
		if (m_strDescription == null)
		{
			// Return the name if there is no description
			return GetEncodedName();
		}

		return m_strDescription;
	}

	/**
	 * This method sets the description of the node and HTMLEncodes it.
	 * This will ignore the description if it is null or the empty string.
	 *
	 * @param String the description of the node.
	 */
	public void SetDescription(String strDescription)
	{
		if ((strDescription != null) && (!"".equals(strDescription)))
		{
			m_strDescription = XPHttpUtility.HtmlEncode(strDescription);
		}
	}
}

