package com.plumtree.uiinfrastructure.uitasks;

import com.plumtree.openfoundation.web.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.web.*;

/**
 * This class provides access to the environment that a
 * tag is executed in.
 */
public interface IEnvironment
{
	/**
	 * This method returns the current access style (i.e. Section
	 * 508 compliance, low bandwidth, or standard).
	 *
	 * @see com.plumtree.uiinfrastructure.constants.AccessStyles
	 *
	 * @return AccessStyles The access style
	 */
	public AccessStyles GetAccessStyle();

	/**
	 * Return the HTTP Request for the current request.
	 *
	 * @return the current request
	 */
	public IXPRequest GetCurrentHTTPRequest();

	/**
	 * Return the HTTP Response for the current request.
	 *
	 * @return the current response
	 */
	public IXPResponse GetCurrentHTTPResponse();

	/**
	 * Return the normalized base url of the current request.
	 * The normalized base url is essentially the request url
	 * up to and including the '?'.
	 * <p>
	 * @see com.plumtree.uiinfrastructure.statichelpers.ConfigHelpers#GetNormalizedBaseURL(String)
	 * 
	 * @return The normalized base url of the current request
	 */
	public String GetCurrentNormalizedBaseURL();

	/**
	 * Return the full url of the current request.
	 *
	 * @return the url of the current request
	 */
	public String GetCurrentRequestURL();

	/**
	 * Gets the current language of the user.
	 * <p>
	 * E.G. en, fr, jp
	 * 
	 * @return The current language of the user
	 */
	public String GetLanguage();

	/**
	 * Gets the current locale of the user.
	 * <p>
	 * E.G. en-us, en-gb, fr-fr, jp-jp
	 * 
	 * @return The current locale of the user.
	 */
	public String GetLocale();

	/**
	 * Gets the time zone of the current user.
	 * <p>
	 * E.G. America/Los_Angeles
	 * 
	 * @return The time zone of the user.
	 */
	public String GetTimeZone();

	/**
	 * This method gets localized strings for an
	 * internationalized string file based on the user's
	 * locale.
	 * <p>
	 * Available string files:
	 * <br>
	 * @see com.plumtree.uiinfrastructure.constants.ResourceConstants
	 * <p>
	 * Custom string files can also be added to the portal.
	 * 
	 * @param _nID The ID of the string to retrieve
	 * @param _strStringFile The name of the string file to use
	 * @return String The localized string
	 */
	public String GetString(int _nID, String _strStringFile);

	/**
	 * Return the user's Session.  This is a generic Object that
	 * can be used for different kinds of sessions.  This will
	 * have to be cast to the correct type of session (IPTSession
	 * for the portal).
	 *
	 * @return Object the user's session
	 */
	public Object GetUserSession();

	/**
	 * This method gets Variable Packages.
	 *
	 * @deprecated Variable packages are no longer available directly in tags.  In the portal, the Environment object can be cast to an Activity Space and VarPacks can be retrieved that way.
	 * 
	 * @param String _strVarPack The ID of the requested VarPack
	 * @return IVarPack The requested VarPack
	 */
	public IVarPack GetVarPack(String _strVarPack);
}
