/*
 * Created on Jan 6, 2005
 */
package com.plumtree.uiinfrastructure.utils;

import com.plumtree.openfoundation.util.*;

/**
 * A data structure that is equivalent to a list of hashtables or a 2D staggered array but simpler interface and with
 * convenience methods to retrieve String and integers.
 * 
 * Logical image of the data structure:
 * 
 * [0][key][key2][key3]...[keyn]
 * [1][key][key2][key3]...[keyn]
 * [2][key][key2][key3]...[keym]
 * 
 * 
 * Uses one hashtable internally for efficiency. 
 * 
 * Example:
 * Setting data
 * <code>
 * * PTListMap foo = new PTListMap();
 * int nIndex = foo.AddNewElement();
 * foo.AddElementToIndex(nIndex, "fooKey1", "fooValue");
 * foo.AddElementToIndex(nIndex, "fooKey2", new Integer(100) );
 * foo.AddElementToIndex(nIndex, "fooKey3", new FooClass() );
 * 
 * nIndex = foo.AddNewElement();
 * foo.AddElementToIndex(nIndex, "fooKey1", "fooValue");
 * foo.AddElementToIndex(nIndex, "fooKey2", new Integer(100) );
 * foo.AddElementToIndex(nIndex, "fooKey3", new FooClass() );
 *</code>
 * Getting data
 *<code>
 * for ( int i = 0; i < foo.GetSize(); i++ )
 * {
 *   String strVal = foo.GetElementAsString(i, "fooKey1");
 *   int nVal = foo.GetElementAsInt(i, "fooKey2");
 *   Object oVal = int nVal = foo.GetElementAs(i, "fooKey3");
 * }
 * </code>
 * 
 * @author robertz
 */
public class PTListMap
{
	//
	private XPHashtable m_hashtable;
	
	//
	private int m_nListEntries; 
	
	/**
	 * Creates an empty PTListMap
	 */
	public PTListMap()
	{
		m_hashtable = new XPHashtable();
		m_nListEntries = 0;
	}
	
	/**
	 * Get the number of rows.
	 * 
	 * @return
	 */
	public int GetSize()
	{
		return m_nListEntries;
	}
	
	/**
	 * Gets the index of the next row
	 * 
	 * @return The index of the new row
	 */
	public int AddNewElement()
	{
		
		m_nListEntries++;
		
		return m_nListEntries-1;
	}
	
	/**
	 * Adds a key/value pair to an existing row.
	 * 
	 * @param nIndex Row index
	 * @param strKey 
	 * @param Value
	 */
	public void AddElementToIndex(int nIndex, String strKey, Object Value)
	{
		m_hashtable.PutElement(CreateInternalKeyName(nIndex, strKey), Value);
	}
	
	/**
	 * Get an entry as Object
	 * 
	 * @param nIndex
	 * @param strKey
	 * @return
	 */
	public Object GetElement(int nIndex, String strKey)
	{
		return m_hashtable.GetElement(CreateInternalKeyName(nIndex, strKey));
	}
	
	/**
	 * Get an entry as Int
	 * 
	 * @param nIndex
	 * @param strKey
	 * @return
	 */
	public int GetElementAsInt(int nIndex, String strKey)
	{
		Object o = m_hashtable.GetElement(CreateInternalKeyName(nIndex, strKey)); 
		int nVal = XPConvert.ToInteger(o);
		return nVal;
	}
	
	/**
	 * Get an entry as String
	 * 
	 * @param nIndex
	 * @param strKey
	 * @return
	 */
	public String GetElementAsString(int nIndex, String strKey)
	{
		Object o = m_hashtable.GetElement(CreateInternalKeyName(nIndex, strKey)); 
		String strVal = XPConvert.ToString(o);
		return strVal;
	}
	
	
	/**
	 * 
	 * 
	 * @param nIndex
	 * @param strKey
	 * @return
	 */
	private InternalListTableEntry  CreateInternalKeyName(int nIndex, String strKey)
	{
		return new InternalListTableEntry(nIndex, strKey);
	}
	
	/**
	 * 
	 * 
	 * @param oInternalKey
	 * @return
	 */
	private int GetIndexFromInternalKeyName(Object oInternalKey)
	{
		
		return  ((InternalListTableEntry)oInternalKey).GetIndex();
	}
	
	/**
	 * 
	 * 
	 * @param oInternalKey
	 * @return
	 */
	private String GetKeyNameFromInternalKeyName(Object oInternalKey)
	{
		return  ((InternalListTableEntry)oInternalKey).GetKey();
	}
	
	
	/**
	 * Internal class for entries
	 * 
	 * @author robertz
	 */
	private static class InternalListTableEntry
	{
		private int m_nIndex;
		private String m_strKey;
		
		public InternalListTableEntry(int nIndex, String strKey)
		{
			m_nIndex = nIndex;
			m_strKey = strKey;
		}
		
		public int GetIndex()
		{
			return m_nIndex;
		}
		
		public String GetKey()
		{
			return m_strKey;
		}
		
		/**
		 * @see java.lang.Object#hashCode()
		 */
		public int hashCode()
		{
			return (m_nIndex+m_strKey).hashCode();
		}
		
		public boolean equals(Object o )
		{
			InternalListTableEntry entry = null;
			try {
				entry = (InternalListTableEntry)o;
			} catch (XPException e) {
				return false;
			}
			return  ( entry.m_nIndex == m_nIndex && entry.m_strKey.equals(m_strKey) );
		}
		
	}
	
}
