package com.plumtree.uiinfrastructure.validators;

import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.pagecontrols.framework.AValidator;

/**
 * Validates that an object is within a given int range bounded by m_intMax & m_intMin.
 * 
 * @author caiusv
 */
public class IntRangeValidator extends AValidator implements IValidator {

	public String VALIDATOR_NAME = "IntRangeValidator";
	
	/** name of the form where this validator appears (used by JavaScript validation)*/
	public String m_strFormName;
	
	/** name of the input which this validator works on (used by JavaScript validation) */
	public String m_strInputName;
	
	/** the defualt value of the input */
	private String m_strDefaultValue;
	
	private int m_intMax;
	private int m_intMin;
	
	/**
	 * Creates a new IntRangeValidator.
	 * 
	 * @param _sFormName name of the form whre this appears
	 * @param _sInputName	name of the input element
	 * @param _sDefaultValue	default value of the input element
	 * @param _iMax	max value for the int validated
	 * @param _iMin min value for the int validated
	 */
	public IntRangeValidator(String _sFormName, 
			String _sInputName,
			String _sDefaultValue,
			int _iMax,
			int _iMin)
	{
		m_strFormName 		= _sFormName;
		m_strInputName 		= _sInputName;
		m_strDefaultValue 	= _sDefaultValue;
		m_intMax			= _iMax;
		m_intMin			= _iMin;
	}
	
	/**
	 * Return a UIError object if the object passed in is not an int within the range 
	 * specified by the m_intMax & m_intMin.
	 * 
	 * @return a UIError containing the error message set with SetErrorMessage()
	 */
	public UIError Validate(Object _oObject)
	{
		String strErrorMsg = GetErrorMessage();
		int myInt = 0;
		UIError myError =
			new UIError(
				UIErrorLevel.FRIENDLY,
				UIErrorScope.CURRENT_USER,
				strErrorMsg,
				strErrorMsg);
		if (null == _oObject)
		{
			return myError;
		}
		try {
			myInt = XPConvert.ToInteger(_oObject);
		}
		catch (Exception e)
		{
			return myError;
		}
		if (myInt > m_intMax || myInt < m_intMin)
		{
			return myError;
		}
		return null;
	}

	/**
	 * Returns the JavaScript Validation method that may be used on a page to validate
	 * associated form input.
	 * 
	 * @see com.plumtree.uiinfrastructure.activityspace.IValidator#GetJSValidationCode()
	 */
	public String GetJSValidationCode() {
		XPStringBuilder sb = new XPStringBuilder(1250);
		sb.Append("<script type=\"text/javascript\">\n");
		sb.Append("function isIntInRange(inputName)\n");
		sb.Append("{\n");
		sb.Append("    if (PTNumberUtil.isInteger(parseInt(inputName.value)) " +
				"&& inputName.value < "+m_intMax+" " +
						"&& inputName.value > "+m_intMin+")\n");
		sb.Append("    {\n");
		sb.Append("        return true;\n");
		sb.Append("    }\n");
		sb.Append("    else");
		sb.Append("    {\n");
		sb.Append("        alert(\""+GetErrorMessage()+"\");\n");
		sb.Append("        inputName.value = \""+m_strDefaultValue+"\";\n");
		sb.Append("        return false;\n");
		sb.Append("    }\n");
		sb.Append("}\n");
		sb.Append("</script>\n");
		return sb.toString();
	}
	
	/**
	 * Return the name of the JavaScript validation method along with whatever
	 * parameters the script needs.
	 */
	public String GetJSFunction(String _sParameters)
	{
		if(null != _sParameters)
		{
			return "isIntInRange("+_sParameters+")";
		}
		else
		{
			return "isIntInRange("+m_strFormName+"."+m_strInputName+")";
		}
	}
	
	/**
	 * Return the name of this Validator
	 */
	public String GetName()
	{
		return VALIDATOR_NAME;
	}
}
