package com.plumtree.xpshared.htmlconstructs;

import com.plumtree.xpshared.htmlelements.*;

/**
 * This class contains static methods that generate HTMLElements.
 */
public class ConstructStaticHelpers
{
	/**
	 * This should be used whenever you need a blank URL (such as when you have
	 * a javascript onClick and the URL should never be called).  This should
	 * always be used because it may redirect to a no-javascript error page in
	 * the future.
	 */
	public static final String BLANK_URL = "#";

	/** static strings for performance */
	private static final String STR_RETURN = "return";
	private static final String STR_SEMICOLON = ";";
	private static final String STR_SPACE = " ";
	private static final String STR_TRUE = "true";
	private static final String STR_FALSE = "false";

	/**
	 * Static method to create a javascript onclick anchor.  This version
	 * returns false, and adds no text to be displayed in the anchor.  
	 * This is the most common case.
	 * 
	 * @param String strJavascript the text for the javascript OnClick method
	 * 			without trailing semicolon or return (will be added)
	 */
	public static HTMLAnchor GetJavascriptAnchor(String strJavascript)
	{
		// If this does have return
		if (strJavascript.indexOf(STR_RETURN) != -1)
		{
			throw new HTMLException("Javascript parameter in GetJavascriptAnchor has return, but it will be added automatically.");
		}

		// If this does have trailing semi-colon
		if (strJavascript.indexOf(STR_SEMICOLON) != -1)
		{
			throw new HTMLException("Javascript parameter in GetJavascriptAnchor has trailing semi-colon, but it will be added automatically.");
		}

		HTMLAnchor result = new HTMLAnchor(BLANK_URL);
		result.SetOnClick(strJavascript + STR_SEMICOLON + STR_SPACE + STR_RETURN + STR_SPACE +
			STR_FALSE + STR_SEMICOLON);

		return result;
	}

	/**
	 * Static method to create a javascript onclick anchor.  This version
	 * returns false.  This is the most common case.
	 * 
	 * @param String strText the text to be displayed in the Anchor
	 * @param String strJavascript the text for the javascript OnClick method
	 * 			without trailing semicolon or return (will be added)
	 */
	public static HTMLAnchor GetJavascriptAnchor(String strText, String strJavascript)
	{
		// If this does have return
		if (strJavascript.indexOf(STR_RETURN) != -1)
		{
			throw new HTMLException("Javascript parameter in GetJavascriptAnchor has return, but it will be added automatically. (" + strText + ")");
		}

		// If this does have trailing semi-colon
		if (strJavascript.indexOf(STR_SEMICOLON) != -1)
		{
			throw new HTMLException("Javascript parameter in GetJavascriptAnchor has trailing semi-colon, but it will be added automatically. (" + strText + ")");
		}

		HTMLAnchor result = new HTMLAnchor(BLANK_URL);
		result.AddInnerHTMLString(strText);
		result.SetOnClick(strJavascript + STR_SEMICOLON + STR_SPACE + STR_RETURN + STR_SPACE +
			STR_FALSE + STR_SEMICOLON);

		return result;
	}

	/**
	 * Static method to create a javascript onclick anchor.  This version returns true.
	 * 
	 * @param String strText the text to be displayed in the Anchor
	 * @param String strJavascript the text for the javascript OnClick method
	 * 			without trailing semicolon or return (will be added)
	 */
	public static HTMLAnchor GetJavascriptAnchorReturnTrue(String strText, String strJavascript)
	{
		// If this does have return
		if (strJavascript.indexOf(STR_RETURN) != -1)
		{
			throw new HTMLException("Javascript parameter in GetJavascriptAnchor has return, but it will be added automatically. (" + strText + ")");
		}

		// If this does have trailing semi-colon
		if (strJavascript.indexOf(STR_SEMICOLON) != -1)
		{
			throw new HTMLException("Javascript parameter in GetJavascriptAnchor has trailing semi-colon, but it will be added automatically. (" + strText + ")");
		}

		HTMLAnchor result = new HTMLAnchor(BLANK_URL);
		result.AddInnerHTMLString(strText);
		result.SetOnClick(strJavascript + STR_SEMICOLON + STR_SPACE + STR_RETURN + STR_SPACE +
			STR_TRUE + STR_SEMICOLON);

		return result;
	}

	/**
	 * Static method to create a javascript onclick anchor.  This version
	 * returns the return value of the javascript function.
	 * 
	 * @param String strText the text to be displayed in the Anchor
	 * @param String strJavascript the text for the javascript OnClick method
	 * 			without trailing semicolon or return (will be added)
	 */
	public static HTMLAnchor GetJavascriptAnchorReturnFunctionValue(String strText, String strJavascript)
	{
		// If this does have return
		if (strJavascript.indexOf(STR_RETURN) != -1)
		{
			throw new HTMLException("Javascript parameter in GetJavascriptAnchor has return, but it will be added automatically. (" + strText + ")");
		}

		// If this does have trailing semi-colon
		if (strJavascript.indexOf(STR_SEMICOLON) != -1)
		{
			throw new HTMLException("Javascript parameter in GetJavascriptAnchor has trailing semi-colon, but it will be added automatically. (" + strText + ")");
		}

		HTMLAnchor result = new HTMLAnchor(BLANK_URL);
		result.AddInnerHTMLString(strText);
		result.SetOnClick(STR_RETURN + STR_SPACE + strJavascript + STR_SEMICOLON);

		return result;
	}
}
