package com.plumtree.xpshared.htmlconstructs;

//import com.plumtree.xpfoundation.logging.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.xpshared.htmlelements.*;


/**
 *
 *
 * @author beng  An objects of this class contains a structure of HTML
 *         elements.   PTConstruct objects    cannot be created by client
 *         code, but rather  they are generated by the PTConstructFactory
 *         class. <br>
 *         <br>
 *         A construct contains a nugget of HTML that is commonly used by the
 *         various ui components, such as a button, a banner, an image. <br>
 *         <br>
 *         The HTMLElements within a PTConstruct can be navigated using
 *         various PTConstruct methods, such as GetHTMLElement,
 *         GetNestedElement, and GetAllElements. <br>
 *         <br>
 *         Various PTConstruct methods allow client code to instert additional
 *         HTML elements into the construct, such as
 *         ReplaceNestedElementsWithHTMLString,  and
 *         AddHTMLStringToNestedElement.
 */
public class PTConstruct
{
//
//    private static XPLogger myLogger = XPLogManager.GetLogger(PTConstruct.class);
//
//
//    private static boolean bDebug = myLogger.IsDebugEnabled();
//
//
//    private static boolean bError = myLogger.IsErrorEnabled();
//
//
//    private static boolean bWarn = myLogger.IsWarnEnabled();
//
//
//    private static boolean bInfo = myLogger.IsInfoEnabled();
//
//
//    private static boolean bFatal = myLogger.IsFatalEnabled();


    private HTMLElement myRootElement;


    private PTConstructType myType;


    private String sDescription = null;


    private XPArrayList alElements;


    private XPHashtable myHash;


    private XPHashtable myStringMap;

    /**
     * Constructor creates a new PTConstruct object. Construct objects are
     * created by the PTConstructFactory object.
     *
     * @param type Pass PTConstructType
     * @param e Pass HTMLElement structure for this construct.
     */
    public PTConstruct(PTConstructType type, HTMLElement e)
    {
//        if (bInfo)
//        {
//            myLogger.Info("Entering PTConstruct()");
//            myLogger.Info("type: " + type);
//        }

        myType        = type;
        myRootElement = e;

        sDescription = type.GetDescription();

        myStringMap = type.GetStringMap();

        update();
    }

    /**
     * Call this method to insert an HTMLElement into an HTMLElement object
     * nested within this construct.  Any existing HTML nested within the
     * specified container element will be preserved. <br><br>
     *
     * @param enContainer Pass an ElementName object specifying an HTMLElement
     *        object nested within this construct.  The HTMLElement passed to
     *        this  method will be appended to any HTML existing within the
     *        container  element.  If no    HTMLElement object having the
     *        specified ElementName  can be found, an Exception will be
     *        thrown. <br><br>
     * @param element Pass an HTMLElement.  This element will be appended to
     *        any html existing within the container HTMLElement object
     *        specified  by the enContainer parameter.
     *
     * @throws Exception
     */
    public void AddElementToNestedElement(ElementName enContainer, HTMLElement element)
    {
        HTMLElement eTemp = null;

        eTemp = this.GetNestedElement(enContainer);

        if (eTemp != null)
        {
            try
            {
                eTemp.AddInnerHTMLElement(element);
            }
            catch (HTMLException he)
            {
				// do nothing
            }
        }
    }

    /**
     * Call this method to insert an HTML string into an HTMLElement object
     * nested within this construct.  Any existing HTML nested within the
     * specified container element will be preserved. <br><br>
     *
     * This will not HTMLEncode the string, and therefore should be used for
     * localized strings, which should be safe.
     *
     * @param enContainer Pass an ElementName object specifying an HTMLElement
     *        object nested within this construct.  The HTML string passed to
     *        this  method will be appended to any HTML existing within this
     *        element.  If no HTMLElement object having the specified
     *        ElementName can be found, an Exception will be thrown. <br><br>
     * @param s Pass an HTML string.  This html will be appended to any html
     *        existing within the container HTMLElement object specified by
     *        the enContainer parameter.
     *
     * @throws Exception
     */
    public void AddHTMLStringToNestedElement(ElementName enContainer, String s)
	{
//        if (bInfo)
//        {
//            myLogger.Info("Entering AddHTMLStringToNestedElement");
//            myLogger.Info("eName.GetName(): " + enContainer.GetName());
//            myLogger.Info("s: " + s);
//            myLogger.Info("GetType(): " + this.GetType());
//        }

        HTMLElement eTemp = null;

        eTemp = this.GetNestedElement(enContainer);

        if (eTemp != null)
        {
            eTemp.AddInnerHTMLString(s);
        }
    }

    /**
     * Call this method to insert an HTML string into an HTMLElement object
     * nested within this construct.  Any existing HTML nested within the
     * specified container element will be preserved. <br><br>
     *
     * This will HTMLEncode the string, and therefore should not be used for
     * localized strings, which should be safe.
     *
     * @param enContainer Pass an ElementName object specifying an HTMLElement
     *        object nested within this construct.  The HTML string passed to
     *        this  method will be appended to any HTML existing within this
     *        element.  If no HTMLElement object having the specified
     *        ElementName can be found, an Exception will be thrown. <br><br>
     * @param s Pass an HTML string.  This html will be appended to any html
     *        existing within the container HTMLElement object specified by
     *        the enContainer parameter.
     *
     * @throws Exception
     */
    public void AddHTMLEncodedStringToNestedElement(ElementName enContainer, String s)
	{
//        if (bInfo)
//        {
//            myLogger.Info("Entering AddHTMLEncodedStringToNestedElement");
//            myLogger.Info("eName.GetName(): " + enContainer.GetName());
//            myLogger.Info("s: " + s);
//            myLogger.Info("GetType(): " + this.GetType());
//        }

        HTMLElement eTemp = null;

        eTemp = this.GetNestedElement(enContainer);

        if (eTemp != null)
        {
            eTemp.AddInnerHTMLEncodedString(s);
        }
    }

    /**
     * Call this method to obtain an XPArrayList containing all of the
     * HTMLElement objects constituting this construct.   <br>
     * <br>
     * This list is recursively genererated, so it is likely that some  of the
     * HTMLElements    on this list will internally include  references to
     * other HTMLElements on this list.  For example, if this construct were a
     * table, with a row, with a column, with an image,  this method would
     * return an XPArrayList with 4 items: an HTMLTable, an HTMLTableRow, an
     * HTMLTableCell, and an HTMLImg.
     *
     * @return
     */
    public XPArrayList GetAllElements()
    {
        return alElements;
    }

    /**
     * Get the description of this construct.
     *
     * @return String description
     */
    public String GetDescription()
    {
        return sDescription;
    }

    /**
     * Call this method to obtain the root HTMLElement of this construct. <br>
     * <br>
     * For example, if this construct were a table, with a row, with a cell,
     * with an image, this method would return a reference to an HTMLTable.
     *
     * @return
     */
    public HTMLElement GetHTMLElement()
    {
        return myRootElement;
    }

    /**
     * Call this method to obtain an HTMLElement nested within this construct.
     * Specify the ElementName identifying the HTMLElement you wish to obtain.
     * ElementName objects can be obtained by calling the
     * PTConstructType.GetElementName( ) method.
     *
     * @param eName
     *
     * @return
     *
     * @throws Exception
     */
    public HTMLElement GetNestedElement(ElementName eName)
    {
//        if (bInfo)
//        {
//            myLogger.Info("Entering GetNestedElement()");
//            myLogger.Info("this type: " + this.GetType());
//            myLogger.Info("eName: " + eName.GetName());
//        }

        Object o          = null;
        HTMLElement eTemp = null;

        try
        {
            o = myHash.GetElement(eName.GetName());
        }
        catch (Exception e)
        {
			// do nothing
        }

        if (o == null)
        {
            throw new XPException("GetNestedElement() - The " + eName.GetName() + " element does not exist in the " + GetType() + " construct.");
        }

        eTemp = (HTMLElement) o;

        return eTemp;
    }

    /**
     * Call this method to obtain an XPStringDictionary describing those
     * HTMLElements of this construct containing strings that can be updated. <br>
     * <br>
     * This XPStringDictionary contains a mapping of element names to string
     * ids.  The element name indicates an HTMLElement object nested within
     * this construct.  The corresponding string id indicates which
     * localized     string is to be nested within.
     *
     * @return
     */
    public XPHashtable GetStringMap()
    {
        return myStringMap;
    }

    /**
     * Call this method to obtain a PTConstructType specifying the specific
     * type of this construct.
     *
     * @return
     */
    public PTConstructType GetType()
    {
        return myType;
    }

    /**
     * Call this method to set the inner HTML element of a specific element
     * within this construct. <br>
     * <br>
     * All previous inner html elements and strings will be removed,   and
     * replaced with the passed HTMLElement. <br><br>
     *
     * @param enContainer Pass an ElementName indicating an HTMLElement nested
     *        within this construct.  All HTMLElements nested within the
     *        HTMLElement indicated by this ElementName will be replaced.
     *        ElementName objects are available from the specific
     *        PTConstructType used to obtain this construct. <br><br>
     * @param element Pass an HTMLElement.  This HTMLElement will be nested
     *        within the HTMLElement indicated by the ElementName passed.
     *
     * @throws Exception
     */
    public void ReplaceNestedElementsWithElement(ElementName enContainer, HTMLElement element)
    {
        HTMLElement eTemp = null;

        eTemp = this.GetNestedElement(enContainer);

        if (eTemp != null)
        {
            eTemp.ClearInnerHTMLElements();

            eTemp.AddInnerHTMLElement(element);
        }
    }

    /**
     * Call this method to set the inner HTML string of a specific element
     * within this construct. <br>
     * <br>
     * All previous inner html elements and strings will be removed,   and
     * replaced with the passed string. <br><br>
     *
     * This will not HTMLEncode the string, and therefore should be used for
     * localized strings, which should be safe.
     *
     * @param enContainer Pass an ElementName indicating an HTMLElement nested
     *        within this construct.  All HTMLElements nested within the
     *        HTMLElement indicated by this ElementName will be replaced.
     *        ElementName objects are available from the specific
     *        PTConstructType used to obtain this construct. <br><br>
     * @param s Pass an HTML string.  This string will be nested within the
     *        HTMLElement indicated by the ElementName passed.
     *
     * @throws Exception
     */
    public void ReplaceNestedElementsWithHTMLString(ElementName enContainer, String s)
    {
//        if (bInfo)
//        {
//            myLogger.Info("Entering ReplaceNestedElementsWithHTMLString");
//            myLogger.Info("eName.GetName(): " + enContainer.GetName());
//            myLogger.Info("s: " + s);
//            myLogger.Info("GetType(): " + this.GetType());
//        }

        HTMLElement eTemp = null;

        eTemp = this.GetNestedElement(enContainer);

        if (eTemp != null)
        {
            eTemp.ClearInnerHTMLElements();

            eTemp.AddInnerHTMLString(s);
        }
    }

    /**
     * Call this method to set the inner HTML string of a specific element
     * within this construct. <br>
     * <br>
     * All previous inner html elements and strings will be removed,   and
     * replaced with the passed string. <br><br>
     *
     * This will HTMLEncode the string, and therefore should not be used with
     * localized strings, which shouldn't need to be encoded.
     *
     * @param enContainer Pass an ElementName indicating an HTMLElement nested
     *        within this construct.  All HTMLElements nested within the
     *        HTMLElement indicated by this ElementName will be replaced.
     *        ElementName objects are available from the specific
     *        PTConstructType used to obtain this construct. <br><br>
     * @param s Pass an HTML string.  This string will be nested within the
     *        HTMLElement indicated by the ElementName passed.
     *
     * @throws Exception
     */
    public void ReplaceNestedElementsWithHTMLEncodedString(ElementName enContainer, String s)
    {
//        if (bInfo)
//        {
//            myLogger.Info("Entering ReplaceNestedElementsWithHTMLEncodedString");
//            myLogger.Info("eName.GetName(): " + enContainer.GetName());
//            myLogger.Info("s: " + s);
//            myLogger.Info("GetType(): " + this.GetType());
//        }

        HTMLElement eTemp = null;

        eTemp = this.GetNestedElement(enContainer);

        if (eTemp != null)
        {
            eTemp.ClearInnerHTMLElements();

            eTemp.AddInnerHTMLEncodedString(s);
        }
    }

    /**
     * Call this method to set the inner HTML element of a specific element
     * within this construct. <br>
     * <br>
     * All previous inner html elements and strings will be removed,   and
     * replaced with the passed HTMLElement.
     *
     * @param eName
     * @param element
     *
     * @deprecated Use ReplaceNestedElementsWithElement
     */
    public void SetElementInnerHTMLElement(ElementName eName, HTMLElement element) //   todo - remove this method, it does the same thing as the better throws Exception
    {
        HTMLElement eTemp = null;

        try
        {
            eTemp = this.GetNestedElement(eName);
        }
        catch (Exception ex)
        {
			// do nothing
        }

        if (eTemp != null)
        {
            eTemp.ClearInnerHTMLElements();

            try
            {
                eTemp.AddInnerHTMLElement(element);
            }
            catch (HTMLException he)
            {
				// do nothing
            }
        }
    }

    /**
     * Call this method to set the inner HTML string of a specific element
     * within this construct. <br>
     * <br>
     * All previous inner html elements and strings will be removed,   and
     * replaced with the passed string.
     *
     * This will not HTMLEncode the string, and therefore should be used
     * for localized strings, which don't need encoding.
     *
     * @param eName
     * @param s
     *
     * @throws Exception
     *
     * @deprecated Use ReplaceNestedElementsWithHTMLString
     */
    public void SetElementInnerHTMLString(ElementName eName, String s)
    {
//        if (bInfo)
//        {
//            myLogger.Info("Entering SetElementInnerHTMLString");
//            myLogger.Info("eName.GetName(): " + eName.GetName());
//            myLogger.Info("s: " + s);
//            myLogger.Info("GetType(): " + this.GetType());
//        }

        Object o          = null;
        HTMLElement eTemp = null;

        o                 = myHash.GetElement(eName.GetName());

        if (o == null)
        {
//            if (bError)
//            {
//                myLogger.Error("SetElementInnerHTMLString() - The element specified does not exist in this construct.");
//                myLogger.Error("SetElementInnerHTMLString() - Container element name: " + eName.GetName());
//                myLogger.Error("SetElementInnerHTMLString() - String being nested: " + s);
//                myLogger.Error("SetElementInnerHTMLString() - This construct type: " + this.GetType());
//
//                return;
//            }
        }

        eTemp = (HTMLElement) o;

        if (eTemp != null)
        {
            eTemp.ClearInnerHTMLElements();

            try
            {
                eTemp.AddInnerHTMLString(s);
            }
            catch (HTMLException he)
            {
				// do nothing
            }
        }
    }

    /**
     * Call this method to set the inner HTML string of a specific element
     * within this construct. <br>
     * <br>
     * All previous inner html elements and strings will be removed,   and
     * replaced with the passed string.
     *
     * This will HTMLEncode the string, and therefore should not be used for
     * localized strings, which don't need to be encoded.
     *
     * @param eName
     * @param s
     *
     * @throws Exception
     *
     * @deprecated Use ReplaceNestedElementsWithHTMLString
     */
    public void SetElementInnerHTMLEncodedString(ElementName eName, String s)
    {
//        if (bInfo)
//        {
//            myLogger.Info("Entering SetElementInnerHTMLEncodedString");
//            myLogger.Info("eName.GetName(): " + eName.GetName());
//            myLogger.Info("s: " + s);
//            myLogger.Info("GetType(): " + this.GetType());
//        }

        Object o          = null;
        HTMLElement eTemp = null;

        o                 = myHash.GetElement(eName.GetName());

        if (o == null)
        {
//            if (bError)
//            {
//                myLogger.Error("SetElementInnerHTMLEncodedString() - The element specified does not exist in this construct.");
//                myLogger.Error("SetElementInnerHTMLEncodedString() - Container element name: " + eName.GetName());
//                myLogger.Error("SetElementInnerHTMLEncodedString() - String being nested: " + s);
//                myLogger.Error("SetElementInnerHTMLEncodedString() - This construct type: " + this.GetType());
//
//                return;
//            }
        }

        eTemp = (HTMLElement) o;

        if (eTemp != null)
        {
            eTemp.ClearInnerHTMLElements();

            try
            {
                eTemp.AddInnerHTMLEncodedString(s);
            }
            catch (HTMLException he)
            {
				// do nothing
            }
        }
    }

    /**
     * Package-private update method.  Stores all named HTMLElement objects in
     * a member hash table for quick lookup.  Currently only called by the
     * constructor, but this functionality could potentially be needed after
     * object creation.
     */
    void update()
    {
//        if (bInfo)
//        {
//            myLogger.Info("Entering update()");
//            myLogger.Info("this type: " + this.GetType());
//        }

        alElements = myRootElement.GetInnerHTMLElementsAsArrayList(true);

        if (alElements != null)
        {
            //   Here we store a hash map of element names to HTMLElement objects.
            //   this allows for quick lookup based on name.
            myHash = new XPHashtable(alElements.GetSize());

            IXPEnumerator enumElements = alElements.GetEnumerator();

            HTMLElement eTemp;
            String sTemp;

            sTemp = myRootElement.GetElementName();

            if (sTemp != null)
            {
                myHash.PutElement(sTemp, myRootElement);
            }

            while (enumElements.MoveNext())
            {
                eTemp = (HTMLElement) enumElements.GetCurrent();

                sTemp = eTemp.GetElementName();

                if (sTemp != null)
                {
//                    if (bDebug)
//                    {
//                        myLogger.Debug("Found element named: " + sTemp);
//                    }

                    myHash.PutElement(sTemp, eTemp);
                }
            }
        } //   alElements != null
    }
}
