package com.plumtree.xpshared.htmlconstructs;

import com.plumtree.openfoundation.util.*;

import com.plumtree.xpshared.htmlelements.*;


/**
 * @author beng
 *	
 * 	Use this class to build tables for which each cell spans exactly 1 column.
 * 	<br><br>
 * 	This class is intended to simplify the creation of such tables.
 * 
 * 
 */
public class PTTableBuilder
{
    int iCols = -1;
    HTMLTable myTable  = null;
    XPArrayList alRows = null;
    PTStyleClass scTable = null;
    PTStyleClass scRow   = null;
    PTStyleClass scCell  = null;

    /**
     * Creates a new PTTableBuilder object.
     * <br><br>
     * The table produced by this builder constructed in this way will
     * contain no rows, will be "100%" wide, with cellspacing and cellpadding
     * each "0", no style classes will be set.
     * 
     * @param cols Pass the number of columns which this table spans.
     */
    public PTTableBuilder(int cols)
    {
        InternalConstructor(0, cols, "100%", "0", "0", null, null, null);
    }

    /**
     * Creates a new PTTableBuilder object.
     * <br><br>
     * The table produced by this builder constructed in this way will
     * contain the specified number of rows, will be "100%" wide, with 
     * cellspacing and cellpadding
     * each "0", no style classes will be set.
     * 
     * @param rows Pass the number of rows this table spans.
     * @param cols Pass the number of columns this table spans.
     */
    public PTTableBuilder(int rows, int cols)
    {
        InternalConstructor(rows, cols, "100%", "0", "0", null, null, null);
    }

    /**
     * Creates a new PTTableBuilder object.
     * <br><br>
     * No style classes will be set for the table produced by this 
     * builder constructed in this way,
     * 
     * @param rows Pass the number of rows this table spans.
     * @param cols Pass the number of columns this table spans.
     * @param sWidth Pass the width of this table.
     * @param sCellPadding Pass the cell padding.
     * @param sCellSpacing Pass the cell spacing.
     */
    public PTTableBuilder(int rows, int cols, String sWidth, String sCellPadding, String sCellSpacing)
    {
        InternalConstructor(rows, cols, sWidth, sCellPadding, sCellSpacing, null, null, null);
    }

    /**
     * Creates a new PTTableBuilder object.
     * <br><br>
     * The table produced by this builder constructed in this way will
     * contain no rows, will be "100%" wide, with cellspacing and cellpadding
     * each "0", style classes will be set as specified.
     * 
     * @param cols Pass the number of columns this table spans.
     * @param tableClass Pass the style class of this table.
     * @param rowClass Pass the style class for the rows of this table.
     * @param cellClass Pass the style class for the cell sof this table.
     */
    public PTTableBuilder(int cols, PTStyleClass tableClass, PTStyleClass rowClass, PTStyleClass cellClass)
    {
        InternalConstructor(0, cols, "100%", "0", "0", tableClass, rowClass, cellClass);
    }

    /**
     * Creates a new PTTableBuilder object.
     * <br><br>
     * The table produced by this builder constructed in this way will
     * contain the specified number of rows, will be "100%" wide, with 
     * cellspacing and cellpadding each "0", style classes will be set 
     * as specified.
     * 
     * @param rows Pass the number of rows this table spans.
     * @param cols Pass the number of columns this table spans.
     * @param tableClass Pass the style class of this table.
     * @param rowClass Pass the style class for the rows of this table.
     * @param cellClass Pass the style class for the cell sof this table.
     */
    public PTTableBuilder(int rows, int cols, PTStyleClass tableClass, PTStyleClass rowClass, PTStyleClass cellClass)
    {
        InternalConstructor(rows, cols, "100%", "0", "0", tableClass, rowClass, cellClass);
    }

    /**
     * Creates a new PTTableBuilder object.
     * <br><br>
     * 
     * @param rows Pass the number of rows this table spans.
     * @param cols Pass the number of columns this table spans.
     * @param sWidth Pass the width of this table.
     * @param sCellPadding Pass the cell padding of this table.
     * @param sCellSpacing Pass the cell spacing of this table.
     * @param tableClass Pass the style class of this table.
     * @param rowClass Pass the style class of this table's rows.
     * @param cellClass Pass the style class of this table's cells.
     */
    public PTTableBuilder(int rows, int cols, String sWidth, String sCellPadding, String sCellSpacing, PTStyleClass tableClass, PTStyleClass rowClass, PTStyleClass cellClass)
    {
        InternalConstructor(rows, cols, sWidth, sCellPadding, sCellSpacing, tableClass, rowClass, cellClass);
    }

    /**
     * Internal constructor used by all public constructors.
     * 
     * @param rows Pass the number of rows this table spans.
     * @param cols Pass the number of columns this table spans.
     * @param sWidth Pass the width of this table.
     * @param sCellPadding Pass the cell padding of this table.
     * @param sCellSpacing Pass the cell spacing of this table.
     * @param tableClass Pass the style class of this table.
     * @param rowClass Pass the style class of this table's rows.
     * @param cellClass Pass the style class of this table's cells.
     */
    private void InternalConstructor(int rows, int cols, String sWidth, String sCellPadding, String sCellSpacing, PTStyleClass cTable, PTStyleClass cRow, PTStyleClass cCell)
    {
        iCols = cols;

        scTable = cTable;
        scRow  = cRow;
        scCell = cCell;

        alRows = new XPArrayList();

        myTable = new HTMLTable();

        myTable.SetWidth(sWidth);
        myTable.SetCellPadding(sCellPadding);
        myTable.SetCellSpacing(sCellSpacing);

        if (cTable != null)
        {
            try
            {
                myTable.SetStyleClass(cTable);
            }
            catch (Exception ex)
            {
				// do nothing            	
            }
        }

        for (int x = 0; x < rows; x++)
        {
            InternalAddRow();

        }
    }

    /**
     * Call this method to obtain a reference to the table created by this
     * table builder.
     * 
     * @return HTMLTable table
     */
    public HTMLTable GetTable()
    {
        return myTable;
    }

    /**
     * Call this method to obtain the number of rows in the table built by this
     * builder.
     * 
     * @return int rows
     */
    public int GetNumRows()
    {
        XPArrayList alTemp = myTable.GetInnerHTMLElementsAsArrayList( false );
        
        int i = 0;
        
        if( alTemp != null )
        {
        	i = alTemp.GetSize();
        }
        
        return i;
    }

    /**
     * Call this method to determine the number of columns in the table
     * 	built by this builder.
     * 
     * @return int columns
     */
    public int GetNumCols()
    {
        return iCols;
    }

    /**
     * Call this method to add a row to the table built by this builder.
     * <br><br>
     * The appropriate number of cells will be added to this row.
     * 
     * @return HTMLTablRow row added
     */
    public HTMLTableRow AddRow()
    {
        return InternalAddRow();
    }

    /**
     * Internal row adding method.
     * 
     * @return HTMLTableRow row added
     */
    private HTMLTableRow InternalAddRow()
    {
        HTMLTableRow myRow;
        HTMLTableCell myCell;

        myRow = new HTMLTableRow();

        try
        {
            myTable.AddInnerHTMLElement(myRow);

            if (scRow != null)
            {
                myRow.SetStyleClass(scRow);
            }

            alRows.Add(myRow);

        }
        catch (Exception ex)
        {
			// do nothing        	
        }

        for (int y = 0; y < iCols; y++)
        {
            myCell = new HTMLTableCell();

            try
            {
                if (scCell != null)
                {
                    myCell.SetStyleClass(scCell);
                }

                myRow.AddInnerHTMLElement(myCell);

            }
            catch (Exception ex)
            {
				// do nothing            	
            }

        }
        
        return myRow;

    }

    /**
     * Call this method to obtain an enumeration of the rows in the table
     * 	built by this builder.
     * 
     * @return IXPEnumerator rows
     */
    public IXPEnumerator GetRows()
    {
        return myTable.GetInnerHTMLElements( false );
    }

    /**
     * Call this method to obtain an enumeration of the cells within a given
     * row within the table built by this builder.
     * 
     * 
     * @param row Pass the index of the row you wish to obtain.  If this row 
     * 	doesnt exist in the table, you will receive null;
     * 
     * @return IXPEnumerator cells
     */
    public IXPEnumerator GetCells(int row)
    {
    	
    	XPArrayList alTemp = myTable.GetInnerHTMLElementsAsArrayList( false );

        HTMLTableRow r = (HTMLTableRow) alTemp.GetElement(row);

        IXPEnumerator eTemp = null;

        if (r != null)
        {
            eTemp = r.GetInnerHTMLElements(false);
        }

        return eTemp;
    }
}
