package com.plumtree.xpshared.htmlelements;

/**
 * This helper class contains the display options for displaying
 * HTMLElements.
 * 
 * These are used throughout the entire HTML tree (usually from HTMLPage
 * down, although sometimes the root of the tree will be another type of
 * HTMLElement).
 * 
 * Since the Display Options are set on every HTMLElement, if someone
 * caches an HTMLElement on the session or application, this object
 * will be leaked (retained) until the session or application is cleared.
 * There may also be some multi-threading issues when using a cached
 * HTMLElement for multiple users (the login page Auth Source dropdown
 * is stored on the application), since each user will attempt to set
 * their own Display Options on the cached HTMLElement.
 * 
 * @author DonH
 */
public class DisplayOptions
{
    private String sImgBaseUrl;
    private String sStyleClassPrefix;
    private String sLang = null;
    private boolean bIsNetscape                       = false;
    private boolean bForceLineBreaksAfterSafeElements = true;
    private boolean bDo508Checking                    = false;

    /**
     * Call this method to obtain the base url used for all HTMLImg objects
     * being displayed
     * 
     * @return String base url.
     */
    public String GetImgBaseUrl()
    {
        return sImgBaseUrl;
    }

    /**
     * Call this method to set the base url used for all HTMLImg objects being
     * displayed.
     * <br><br>
     * Call this method only on the root HTMLElement object.  If you are building
     *     an HTMLPage, for example, call myPage.SetImgBaseUrl( "http://blah/" );.
     *     All nested HTMLImg objects may use the base url specified on the HTMLPage
     *     object.
     * <br><br>
     * HTMLImg objects may or may not use this base url specified.
     * 
     * @param s Pass a base url string.  It should look something like: 
     *     "http://blah/".
     */
    public void SetImgBaseUrl(String s)
    {
        sImgBaseUrl = s;
    }

	/**
	 * Set the value of the lang attribute.
	 *
	 * @param s String value
	 */
	public void SetLang(String s)
	{
		sLang = s;
	}

	/**
	 * Get the value of the lang attribute.
	 *
	 * @return String value (default null)
	 */
	public String GetLang()
	{
		return sLang;
	}

    /**
     * This method sets whether or not the browser is Netscape.
     * 
     * Certain HTMLElements will display differently on Netscape
     * versus other browsers.
     *
     * @param b boolean true implies the browser is Netscape
     */
    public void SetIsNetscape(boolean b)
    {
        bIsNetscape = b;
    }

    /**
     * This method returns whether or not the browser is Netscape.
     *
     * @return boolean true implies the browser is Netscape.
     */
    public boolean GetIsNetscape()
    {
        return bIsNetscape;
    }

    /**
     * This method controls whether or not to use line breaks
     * after elements where it does not affect the HTML output.
     *
     * @param b boolean true implies use line breaks.
     */
    public void SetForceLineBreaksAfterSafeElements(boolean b)
    {
        this.bForceLineBreaksAfterSafeElements = b;
    }

    /**
     * This method returns whether or not to use line breaks
     * after elements where it does not affect the HTML output.
     *
     * @return boolean true implies use line breaks.
     */
    public boolean GetForceLineBreaksAfterSafeElements()
    {
        return this.bForceLineBreaksAfterSafeElements;
    }

    /**
     * This method returns the style class prefix used for all
     * HTMLElement objects being displayed.
     *
     * @return The base URL used for style classes.
     */
    public String GetStyleClassPrefix()
    {
        return sStyleClassPrefix;
    }

    /**
     * Call this method to set the style class prefix used for all
     * HTMLElement objects being displayed.
     * <br><br>
     * Call this method only on the root HTMLElement object.  If you are building
     *     an HTMLPage, for example, call myPage.SetStyleClassPrefix( "PREFIX" );.
     *     All nested objects may use the prefix specified on the HTMLPage
     *     object.
     * <br><br>
     *
     * @param s Pass a base url string.  It should look something like: 
     *     "http://blah/".
     */
    public void SetStyleClassPrefix(String s)
    {
        sStyleClassPrefix = s;
    }

    /**
     * This method sets whether or not to do 508 validity
     * checking for HTMLElements.  If this is true, HTMLElements
     * will try to enforce that the developer does not make HTML
     * that is not 508 compliant.
     *
     * @param b boolean true implies enforce 508 validity.
     */
    public void SetDo508Checking(boolean b)
    {
        bDo508Checking = b;
    }

    /**
     * This method returns whether or not to do 508 validity
     * checking for HTMLElements.  If this is true, HTMLElements
     * will try to enforce that the developer does not make HTML
     * that is not 508 compliant.
     *
     * @return boolean true implies enforce 508 validity.
     */
    public boolean GetDo508Checking()
    {
        return bDo508Checking;
    }
}
