package com.plumtree.xpshared.htmlelements;

//import com.plumtree.openfoundation.logging.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;


/**
 * This class can be used to display an HTML anchor element. <br>
 * <br>
 * <a target="new"
 * href="http://www.w3.org/TR/html4/struct/links.html#h-12.2">Spec</a><br><br>
 */
public class HTMLAnchor extends HTMLElement implements TabableHTMLElement
{
    private static final byte[] BEGINTAG = HTMLElement.ToByteArray("<a href=\"");
    private static final byte[] TAB_ATTR = HTMLElement.ToByteArray(" tabindex=\"");
    private static final byte[] TARGET_ATTR = HTMLElement.ToByteArray(" target=\"");
    private static final byte[] TITLE_ATTR = HTMLElement.ToByteArray(" title=\"");
    private static final byte[] ACCESSKEY_ATTR = HTMLElement.ToByteArray(" accesskey=\"");
    private static final byte[] ONCONTEXTMENU_ATTR = HTMLElement.ToByteArray(" oncontextmenu=\"");
    private static final byte[] ENDTAG = HTMLElement.ToByteArray("</a>");

    static int iNextID                      = 0;

    public static boolean bEnforceTitle = false;

    private String sAccessKey;

    private String sHRef;

    private String sOnContextMenu;

    private String sTabIndex;

    private String sTarget;

    private String sTitle;

    private boolean bAllowNullHref = false;

    /**
     * Creates a new HTMLAnchor object.
     *
     * @param sAnchorHRef Pass the value of the href parameter.
     */
    public HTMLAnchor(String sAnchorHRef)
    {
        sHRef     = sAnchorHRef;
        sTarget   = null;
        sTitle    = null;
        sTabIndex = null;


        //   by default, anchor elements do not format.
        SetDoFormat(false);
    }

    /**
     * Call this method to get the value of the accesskey attribute.
     *
     * @return The String value of the accesskey attribute.
     */
    public String GetAccessKey()
    {
        return sAccessKey;
    }

    /**
     * Get the value of the href attribute
     *
     * @return The String value of the href attribute.
     */
    public String GetHRef()
    {
        return sHRef;
    }

    /**
     * Call this method to get the value of the tabindex attribute.
     *
     * @return The String value of the tabindex attribute.
     */
    public String GetTabIndex()
    {
        return sTabIndex;
    }

    /**
     * Get the value of the target attribute.
     *
     * @return The String value of the target attribute.
     */
    public String GetTarget()
    {
        return sTarget;
    }

    /**
     * Get the value of the title attribute.
     *
     * @return The String value of the title attribute.
     */
    public String GetTitle()
    {
        return sTitle;
    }

    /**
     * Call this method to set the value of the accesskey attribute.
     *
     * @param s Pass the String value.
     */
    public void SetAccessKey(String s)
    {
        sAccessKey = s;
    }

    /**
     * Call this method with true to allow for the absence of an href
     * attribute.  By default, a null href value causes the anchor to not
     * display at all, and only display nested HTMLElement objects.
     *
     * @param b
     */
    public void SetAllowNullHRef(boolean b)
    {
        bAllowNullHref = b;
    }

    /**
     * Use this method to set the value of the href attribute.
     *
     * @param sAnchorHRef Pass the value of the href attribute.
     */
    public void SetHRef(String sAnchorHRef)
    {
        sHRef = sAnchorHRef;
    }




	/**
	 *
	 * @param s
	 */
	public String GetOnContextMenu(String s)
	{
		return sOnContextMenu;
	}



    /**
     *
     * @param s
     */
    public void SetOnContextMenu(String s)
    {
        if (!HTMLElement.bAllowEvents)
        {
//            if (bError)
//            {
//                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
//
//                Exception e = new Exception(sDesc);
//
//                myLogger.Error(sDesc, e);
//            }
        }

        sOnContextMenu = s;
    }

    /**
     * Call this method to set the value of the tabindex attribute.
     *
     * @param s Pass the String value.
     */
    public void SetTabIndex(String s)
    {
        sTabIndex = s;
    }

    /**
     * Set the value of the target attribute.
     *
     * @param s The String value of the target attribute.
     */
    public void SetTarget(String s)
    {
        sTarget = s;
    }

    /**
     * Set the value of the title attribute.
     *
     * @param s The String value of the title attribute.
     */
    public void SetTitle(String s)
    {
        sTitle = s;
    }

    /*
        The following methods are necessary to implement TabableHTMLElement
    */

    /**
     * Call this method to remove the tab index from this HTMLElement.
     */
    public void ClearTabIndex()
    {
        sTabIndex = null;
    }

    /**
     * This method contains the bulk of the logic for writting an HTMLElement.
     * For subclasses needing to modify their display behavior, this is the
     * primary method to override.
     *
     * @param bFormat Pass a boolean indicating whether or not to format the
     *        output.
     * @param sbLocal Pass an XPStringBuilder to which you wish the String
     *        appended.  Pass null if you do not wish the string appended to
     *        an XPStringBuilder.
     * @param response Pass an IXPResponse to which you wish the String written.
     *        Pass null if you do not wish the string written to an
     *        IXPResponse.
     *
     * @throws HTMLException
     */
    protected void InternalWrite(boolean bFormat, XPStringBuilder sbLocal, IXPResponse response)
    {
        if ((sHRef != null) || bAllowNullHref)
        {
            String sComment = GetPreComment();

            if ((sComment != null) && bPrintComments)
            {
                WriteNugget(CommonHTMLStrings.BEGIN_COMMENT, sbLocal, response);
                WriteNugget(sComment, sbLocal, response);
                WriteNugget(CommonHTMLStrings.END_COMMENT, sbLocal, response);
            }

            //   concrete subclasses implement the following
            AppendBeginningOfOpeningTag(false, sbLocal, response);

            //   This method adds all attributes common to all html elements,
            //   such as class="" id="", etc.
            AppendCommonAttributes(sbLocal, response);

            //   close the opening tag
            WriteNugget(CLOSE_BRACKET, sbLocal, response);

            GetInnerHTML(sbLocal, response, false);

            //   append the closing tag
            AppendClosingTag(false, sbLocal, response);

            sComment = GetPostComment();

            if ((sComment != null) && bPrintComments)
            {
                WriteNugget(CommonHTMLStrings.BEGIN_COMMENT, sbLocal, response);
                WriteNugget(sComment, sbLocal, response);
                WriteNugget(CommonHTMLStrings.END_COMMENT, sbLocal, response);
            }
        }
        else
        {
            // sHref is null, and not allow null href
            GetInnerHTML(sbLocal, response, false);
        }
    }

    /**
     * Draw the beginning of the anchor tag.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendBeginningOfOpeningTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(BEGINTAG, sb, response);
        WriteNugget(sHRef, sb, response);
        WriteNugget(QUOTE, sb, response);

        if (sTabIndex != null)
        {
            WriteNugget(TAB_ATTR, sb, response);
            WriteNugget(sTabIndex, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sTarget != null)
        {
            WriteNugget(TARGET_ATTR, sb, response);
            WriteNugget(sTarget, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sAccessKey != null)
        {
            WriteNugget(ACCESSKEY_ATTR, sb, response);
            WriteNugget(sAccessKey, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sTitle != null)
        {
            WriteNugget(TITLE_ATTR, sb, response);
            WriteNugget(sTitle, sb, response);
            WriteNugget(QUOTE, sb, response);
        }
        else
        {
            if (bEnforceTitle || GetDo508CheckingForThisInstance())
            {
//                myLogger.Error("You must set the title on the anchor with href: " + sHRef  );
            }
        }

        if (sOnContextMenu != null)
        {
            if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
            {
//                if (bError)
//                {
//                    myLogger.Error("You are not allowed to set the sOnContextMenu attribute value because the client of this html tree is attempting to render a 508-compliant page.");
//                    myLogger.Error("this: " + this);
//                }
            }

            WriteNugget(ONCONTEXTMENU_ATTR, sb, response);
            WriteNugget(sOnContextMenu, sb, response);
            WriteNugget(QUOTE, sb, response);
        }
    }

    /**
     * Append the closing anchor tag
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendClosingTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(ENDTAG, sb, response);
    }

    /**
     * Used to determine whether or not to display a closing tag for this
     * concret subclass of HTMLElement.
     *
     * @return boolean true.
     */
    protected boolean ProvidesClosingTag()
    {
        return true;
    }

    /**
     * Get the next id value.  This is used by the GetClone method to ensure
     * that cloned object have new ID attribute values.
     *
     * @return A valid id string.
     */
    String GetNextID()
    {
        String s = "anchor" + iNextID;
        iNextID++;

        return s;
    }
}
