package com.plumtree.xpshared.htmlelements;

//import com.plumtree.openfoundation.logging.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;


/**
 * Use this class to render an html area element. <br>
 * <br>
 * <a target="new"
 * href="http://www.w3.org/TR/html4/struct/objects.html#h-13.2">Spec</a><br><br>
 */
public class HTMLArea extends HTMLElement
{
	static int iNextID = 0;

	private static final byte[] BEGIN_TAG = HTMLElement.ToByteArray("<area");

	private static final String RECT = "rect";
	private static final String RECTANGLE = "rectangle";
	private static final String CIRC = "circ";
	private static final String CIRCLE = "circle";
	private static final String POLY = "poly";
	private static final String POLYGON = "polygon";

	private static final String BLANK = "_blank";
	private static final String PARENT = "_parent";
	private static final String SELF = "_self";
	private static final String TOP = "_top";

	private static final byte[] ALT_ATTR = HTMLElement.ToByteArray(" alt=\"");
	private static final byte[] COORDS_ATTR = HTMLElement.ToByteArray(" coords=\"");
	private static final byte[] HREF_ATTR = HTMLElement.ToByteArray(" href=\"");
	private static final byte[] NOHREF_ATTR = HTMLElement.ToByteArray(" nohref=\"true\" ");
	private static final byte[] SHAPE_ATTR = HTMLElement.ToByteArray(" shape=\"");
	private static final byte[] TARGET_ATTR = HTMLElement.ToByteArray(" target=\"");
	private static final byte[] ENDTAG = HTMLElement.ToByteArray("</area>");


	String sCoords = null;

	String sAlt;

	String sHref = null;

	boolean bNohref = false;

	String sShape = null;

	String sTarget = null;


	/**
	 * Constructs a new HTMLArea
	 * 
	 * @param alt Pass the alt string of this area.
	 */
	public HTMLArea(String alt)
	{
		sAlt   = alt;
	}

	/**
	 * Constructs a new HTMLArea
	 * 
	 * @param alt Pass the alt string of this area.
	 * @param shape Pass the shape string of this area.
	 * @param coords Pass the coordinates string of this area.
	 * @param href Pass the target URL string of this area.
	 */
	public HTMLArea(String shape, String alt, String coords, String href)
	{
		sAlt = alt;
		sShape = shape;
		sCoords = coords;
		sHref  = href;
	}

	/**
	 * Get the value of the alt attribute.
	 *
	 * @return String value.
	 */
	public String GetAlt()
	{
		return sAlt;
	}

	/**
	 * Get the value of the nohref attribute
	 * 
	 * @return boolean
	 */
	public boolean GetNohref()
	{
		return bNohref;
	}

	/**
	 * Get the value of the coords attribute
	 * 
	 * @return String
	 */
	public String GetCoords()
	{
		return sCoords;
	}

	/**
	 * Get the value of the href attribute
	 * 
	 * @return String
	 */
	public String GetHref()
	{
		return sHref;
	}

	/**
	 * Get the value of the Shape attribute
	 * 
	 * @return String
	 */
	public String GetShape()
	{
		return sShape;
	}

	/**
	 * Get the value of the target attribute
	 * @return String
	 */
	public String GetTarget()
	{
		return sTarget;
	}

	/**
	 * 
	 * Sets the value of the nohref attribute
	 * .
	 * @param bNohref
	 */
	public void SetNohref(boolean bNohref)
	{
		this.bNohref = bNohref;
	}

	/**
	 * Sets the value of the coords attribute.
	 * @param sCoords The coordinates to set
	 */
	public void SetCoords(String sCoords)
	{
		this.sCoords = sCoords;
	}

	/**
	 * Sets the value of the href attribute.
	 * @param sHref The href to set
	 */
	public void SetHref(String sHref)
	{
		this.sHref = sHref;
	}

	/**
	 * Sets the value of the target attribute.
	 * @param sTarget The target to set
	 */
	public void SetTarget(String sTarget)
	{
		if (!((sTarget.equals(BLANK)) || (sTarget.equals(PARENT)) || (sTarget.equals(SELF)) ||  (sTarget.equals(TOP))))
		{
			throw new HTMLException("The target on an HTMLArea must be \"_blank\", \"_parent\", \"_self\", or \"_top\",");
		}

		this.sTarget = sTarget;
	}

	/**
	 * Set the value of the alt atribute.
	 *
	 * @param s String value.
	 */
	public void SetAlt(String s)
	{
		sAlt = s;
	}

	/**
	 * Set the value of the shape attribute.
	 *
	 * @param strShape Either "rect", "rectangle", "circ", "circle", "poly", or "polygon".
	 *
	 * @throws HTMLException
	 */
	public void SetShape(String strShape)
	{
		if (!((strShape.equals(RECT)) || (strShape.equals(RECTANGLE)) || (strShape.equals(CIRC)) || 
			(strShape.equals(CIRCLE)) || (strShape.equals(POLY)) || (strShape.equals(POLYGON))))
		{
			throw new HTMLException("The shape on an HTMLArea must be \"rect\", \"rectangle\", \"circ\",\"circle\",\"poly\", or \"polygon\",");
		}
		
		this.sShape = strShape;
	}


	/**
	 * This method generates the beginning of the opening tag.  The abstract
	 * HTMLElement superclass will provide the remainder of the opening tag,
	 * repleate with attributes common to all html elements, such as name, and
	 * id, and class.
	 *
	 * @param bFormat boolean indicating whether or not to format.
	 * @param sb XPStringBuilder
	 * @param response IXPResponse
	 *
	 * @throws HTMLException
	 */
	protected void AppendBeginningOfOpeningTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
	{
		WriteNugget(BEGIN_TAG, sb, response);

		if (sShape != null)
		{
			WriteNugget(SHAPE_ATTR, sb, response);
			WriteNugget(sShape, sb, response);
			WriteNugget(QUOTE, sb, response);
		}

		if (sAlt != null)
		{
			WriteNugget(ALT_ATTR, sb, response);
			WriteNugget(sAlt, sb, response);
			WriteNugget(QUOTE, sb, response);
		}

		if (sCoords != null)
		{
			WriteNugget(COORDS_ATTR, sb, response);
			WriteNugget(sCoords, sb, response);
			WriteNugget(QUOTE, sb, response);
		}

		if (sHref != null)
		{
			WriteNugget(HREF_ATTR, sb, response);
			WriteNugget(sHref, sb, response);
			WriteNugget(QUOTE, sb, response);
		}

		if (sTarget != null)
		{
			WriteNugget(TARGET_ATTR, sb, response);
			WriteNugget(sTarget, sb, response);
			WriteNugget(QUOTE, sb, response);
		}

		if (bNohref)
		{
			WriteNugget(NOHREF_ATTR, sb, response);
		}

	}

	/**
	 * This method generates the closing tag of this html element.  If the
	 * providesClosingTag method returns false, then this method will never be
	 * called.
	 *
	 * @param bFormat boolean indicating whether or not to format.
	 * @param sb XPStringBuilder
	 * @param response IXPResponse
	 */
	protected void AppendClosingTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
	{
		WriteNugget(ENDTAG, sb, response);
	}

	/**
	 * Indicates whether or not this HTML element provides a closing tag. Some
	 * elements only provide opening tags, such as the &lt;br&gt; element.
	 *
	 * @return boolean false.
	 */
	protected boolean ProvidesClosingTag()
	{
		return true;
	}

	/**
	 * Get the next id value.  This is used by the GetClone method to ensure
	 * that cloned object have new ID attribute values.
	 *
	 * @return A valid id string.
	 */
	String GetNextID()
	{
		String s = "img" + iNextID;
		iNextID++;

		return s;
	}

}
