package com.plumtree.xpshared.htmlelements;

//import com.plumtree.openfoundation.logging.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;


/**
 * This class can be used to display an HTML form element. <br>
 * <br>
 * <a target="new"
 * href="http://www.w3.org/TR/html4/interact/forms.html#h-17.3">Spec</a>
 */
public class HTMLForm extends HTMLElement
{
    static int iNextID = 0;

    private static final String FORM = "form";

    private static final String POST = "post";
    private static final String GET = "get";

    private static final byte[] BEGIN_TAG = HTMLElement.ToByteArray("<form");
    private static final byte[] METHOD_ATTR = HTMLElement.ToByteArray(" method=\"");
    private static final byte[] ACTION_ATTR = HTMLElement.ToByteArray(" action=\"");
    private static final byte[] ONSUBMIT_ATTR = HTMLElement.ToByteArray(" onsubmit=\"");
    private static final byte[] ENCTYPE_ATTR = HTMLElement.ToByteArray(" enctype=\"");
    private static final byte[] END_TAG = HTMLElement.ToByteArray("</form>");

    private HTMLTable myParentTable = null;

    private String sAction;
    private String sEncType;
    private String sMethod;
    private String sOnSubmit;

    /**
     *
     * @param sID Pass the id string of this form.
     *
     * @throws HTMLException
     */
    public HTMLForm(String sID)
    {
        SetID(sID);

        sEncType  = null;
        sMethod   = null;
        sOnSubmit = null;
        sAction   = null;
    }

    /**
     * Get the value of the action attribute.
     *
     * @return String value.
     */
    public String GetAction()
    {
        return sAction;
    }

    /**
     * Get the value of the enctype attribute.
     *
     * @return String value.
     */
    public String GetEncType()
    {
        return sEncType;
    }

    /**
     * Get the value of the method attribute.
     *
     * @return String value.
     */
    public String GetMethod()
    {
        return sMethod;
    }

    /**
     *
     */
    public HTMLTable GetParentTable()
    {
        return myParentTable;
    }

    /**
     * Set the value of the action attribute.
     *
     * @param s A valid String value.
     */
    public void SetAction(String s)
    {
        sAction = s;
    }

    /**
     * Set the value of the enctype attribute.
     *
     * @param s A valid String value.
     */
    public void SetEncType(String s)
    {
        sEncType = s;
    }

    /**
     * Set the value of the method attribute.
     *
     * @param s Either "post" or "get".
     *
     * @throws HTMLException
     */
    public void SetMethod(String s)
    {
        if (!((s.equals(POST)) || (s.equals(GET))))
        {
            throw new HTMLException("The method on an HTMLForm must be \"get\" or \"post\".");
        }

        sMethod = s;
    }

    /**
     * Set the value of the onsubmit attribute.
     *
     * @param s String value.
     */
    public void SetOnSubmit(String s)
    {
        if (!HTMLElement.bAllowEvents)
        {
//            if (bError)
//            {
//                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
//
//                Exception e = new Exception(sDesc);
//
//                myLogger.Error(sDesc, e);
//            }
        }

        sOnSubmit = s;
    }

    /**
     * An HTMLForm can be added to an HTMLTable.
     *
     * @param table
     */
    public void SetParentTable(HTMLTable table)
    {
        myParentTable = table;
    }

    /**
     * Call this method to add a HTMLTableRow object to this table.  The
     * HTMLTableRow passed as a parameter must have been constructed with a
     * reference to this very HTMLTable.
     *
     * @param e
     *
     * @throws HTMLException
     */
    public void AddInnerHTMLElement(HTMLElement e) throws HTMLException
    {
        if (this.GetParentTable() != null)
        {
            // this form has a parent table
            //	the following was copied from HTMLTable.
            //	if this form is nested in a table, then adding an
            //	element is just like adding an element to a table.
            if (!(e instanceof HTMLTableRow) && !(e instanceof HTMLElementCollection))
            {
                throw new HTMLException("You may only add HTMLRow or HTMLElementCollection elements to an HTMLForm in an HTMLTable.");
            }

            if (e instanceof HTMLElementCollection)
            {
                IXPEnumerator enumItems = e.GetInnerHTMLElements(false);

                if (enumItems != null)
                {
                    HTMLElement eTemp;

                    while (enumItems.MoveNext())
                    {
                        eTemp = (HTMLElement) enumItems.GetCurrent();

                        if (!(eTemp instanceof HTMLTableRow))
                        {
                            throw new HTMLException("You may only add an HTMLElementCollection containing HTMLTableRow elements to an HTMLTable element.  You are attempting to add an: " + eTemp);
                        }

                        ProcessRowForAdding((HTMLTableRow) eTemp);

                    } //   for each inner element
                } //   got inner elements
            } // e instanceof HTMLElementCollection
            else
            {
                // it's just a row being added
                ProcessRowForAdding((HTMLTableRow) e);
            }


            //   we must validate anew.
            this.myParentTable.Invalidate();
        }
        else
        {
            // this form has no parent table
            super.AddInnerHTMLElement(e);
        }
    }

	/**
	 * This method adds an HTMLRow to this table.
	 *
	 * @param r Pass an HTMLRow object.
	 *
	 * @throws HTMLException This method throws an exception if for any reason
	 *         the row passed isn't appropriate.
	 */
	void ProcessRowForAdding(HTMLTableRow r)
	{
		if (r.GetParentTable() == null)
		{
			r.SetParentTable(this.GetParentTable());
		}
		else
		{
			if (r.GetParentTable() != this.GetParentTable())
			{
				throw new HTMLException("You may only add a given HTMLTableRow to a single given HTMLTable.");
			}
		}

		super.AddInnerHTMLElement(r);
	}

    /**
     * This method generates the beginning of the opening tag.  The abstract
     * HTMLElement superclass will provide the remainder of the opening tag,
     * repleate with attributes common to all html elements, such as name, and
     * id, and class.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendBeginningOfOpeningTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(BEGIN_TAG, sb, response);

        if (sMethod != null)
        {
            WriteNugget(METHOD_ATTR, sb, response);
            WriteNugget(sMethod, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sAction != null)
        {
            WriteNugget(ACTION_ATTR, sb, response);
            WriteNugget(sAction, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sOnSubmit != null)
        {
//            if (!HTMLElement.bAllowEvents || this.GetDo508CheckingForThisInstance())
//            {
//                if (bError)
//                {
//                    myLogger.Error("You are not allowed to set the onsubmit attribute value because the client of this html tree is attempting to render a 508-compliant page.");
//                    myLogger.Error("this: " + this);
//                }
//            }

            WriteNugget(ONSUBMIT_ATTR, sb, response);
            WriteNugget(sOnSubmit, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sEncType != null)
        {
            WriteNugget(ENCTYPE_ATTR, sb, response);
            WriteNugget(sEncType, sb, response);
            WriteNugget(QUOTE, sb, response);
        }
    }

    /**
     * This method generates the closing tag of this html element.  If the
     * providesClosingTag method returns false, then this method will never be
     * called.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendClosingTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(END_TAG, sb, response);

        AppendClosingLineBreak(sb,response);
    }

    /**
     * Indicates whether or not this HTML element provides a closing tag. Some
     * elements only provide opening tags, such as the &lt;br&gt; element.
     *
     * @return boolean true.
     */
    protected boolean ProvidesClosingTag()
    {
        return true;
    }

    /**
     * Get the next id value.  This is used by the GetClone method to ensure
     * that cloned object have new ID attribute values.
     *
     * @return A valid id string.
     */
    String GetNextID()
    {
        String s = FORM + iNextID;
        iNextID++;

        return s;
    }
}
