package com.plumtree.xpshared.htmlelements;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;


/**
 * <br><br>
 * <a target="new"
 * href="http://www.w3.org/TR/html4/present/frames.html#h-16.2.2">Spec</a><br><br>
 */
public class HTMLFrame extends HTMLElement
{
    private static int iNextID = 0;
    
    private static final byte[] BEGIN_TAG = HTMLElement.ToByteArray("<frame");
    private static final byte[] SCROLLING_ATTR = HTMLElement.ToByteArray(" scrolling=\"");
    private static final byte[] NORESIZE_ATTR = HTMLElement.ToByteArray(" noresize=\"noresize\" ");
    private static final byte[] MARGIN_HEIGHT_ATTR = HTMLElement.ToByteArray(" marginheight=\"");
    private static final byte[] SRC_ATTR = HTMLElement.ToByteArray(" src=\"");
	private static final byte[] LONGDESC_ATTR = HTMLElement.ToByteArray(" longdesc\"");
	private static final byte[] TITLE_ATTR = HTMLElement.ToByteArray(" title\"");
    private static final byte[] FRAME_BORDER_ATTR = HTMLElement.ToByteArray(" frameborder=\"");
    private static final byte[] MARGIN_WIDTH_ATTR = HTMLElement.ToByteArray(" marginwidth=\"");
    private static final byte[] END_TAG = HTMLElement.ToByteArray("</frame>");

    private String sFrameBorder;
    private String sMarginHeight;
    private String sMarginWidth;
	private String sTitle;
	private String sLongDesc;
    private String sScrolling;
    private String sSrc;

    private boolean bNoResize;

    /**
     * Creates a new HTMLFrame object.
     */
    public HTMLFrame()
    {
        sSrc          = null;
        sFrameBorder  = null;
        sMarginWidth  = null;
        sMarginHeight = null;

        bNoResize  = false;
        sScrolling = null;
        sTitle = null;
        sLongDesc = null;
    }


	public void SetLongDesc( String s )
	{
		sLongDesc = s;
	}

	public String GetLongDesc()
	{
		return sLongDesc;
	}

	public void SetTitle( String s )
	{
		sTitle = s;
	}

	public String GetTitle()
	{
		return sTitle;
	}


    /**
     * Get the value of the frameborder attribute.
     *
     * @return String value
     */
    public String GetFrameBorder()
    {
        return sFrameBorder;
    }

    /**
     * Get the value of the marginheight attribute.
     *
     * @return String value
     */
    public String GetMarginHeight()
    {
        return sMarginHeight;
    }

    /**
     * Get the value of the marginwidth attribute.
     *
     * @return String value
     */
    public String GetMarginWidth()
    {
        return sMarginWidth;
    }

    /**
     * Get the value of the noresize attribute.
     *
     * @return boolean value
     */
    public boolean GetNoResize()
    {
        return bNoResize;
    }

    /**
     * Get the value of the scrolling attribute.
     *
     * @return String value
     */
    public String GetScrolling()
    {
        return sScrolling;
    }

    /**
     * Get the value of the src attribute.
     *
     * @return String value
     */
    public String GetSrc()
    {
        return sSrc;
    }

    /**
     * Set the value of the frameborder attribute.
     *
     * @param s A valid String value.
     */
    public void SetFrameBorder(String s)
    {
        // todo enforce 1 | 0
        sFrameBorder = s;
    }

    /**
     * Set the value of the marginheight attribute.
     *
     * @param s A valid String value.
     */
    public void SetMarginHeight(String s)
    {
        sMarginHeight = s;
    }

    /**
     * Set the value of the marginwidth attribute.
     *
     * @param s A valid String value.
     */
    public void SetMarginWidth(String s)
    {
        sMarginWidth = s;
    }

    /**
     * Set the value of the noresize attribute.
     *
     * @param b boolean value.
     */
    public void SetNoResize(boolean b)
    {
        bNoResize = b;
    }

    /**
     * Set the value of the scrolling attribute.
     *
     * @param s A valid String value.
     */
    public void SetScrolling(String s)
    {
        sScrolling = s;
    }

    /**
     * Set the value of the src attribute.
     *
     * @param s A valid String value.
     */
    public void SetSrc(String s)
    {
        sSrc = s;
    }

    /**
     * This method generates the beginning of the opening tag.  The abstract
     * HTMLElement superclass will provide the remainder of the opening tag,
     * repleate with attributes common to all html elements, such as name, and
     * id, and class.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendBeginningOfOpeningTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(BEGIN_TAG, sb, response);

        if (sScrolling != null)
        {
            WriteNugget(SCROLLING_ATTR, sb, response);
            WriteNugget(sScrolling, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (bNoResize)
        {
            WriteNugget(NORESIZE_ATTR, sb, response);
        }

        if (sMarginHeight != null)
        {
            WriteNugget(MARGIN_HEIGHT_ATTR, sb, response);
            WriteNugget(sMarginHeight, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

		if( sTitle != null )
		{
			WriteNugget(TITLE_ATTR, sb, response);
			WriteNugget(sTitle, sb, response);
			WriteNugget(QUOTE, sb, response);
		}

		if( sLongDesc != null )
		{
			WriteNugget(LONGDESC_ATTR, sb, response);
			WriteNugget(sLongDesc, sb, response);
			WriteNugget(QUOTE, sb, response);
		}

        if (sSrc != null)
        {
            WriteNugget(SRC_ATTR, sb, response);
            WriteNugget(sSrc, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sFrameBorder != null)
        {
            WriteNugget(FRAME_BORDER_ATTR, sb, response);
            WriteNugget(sFrameBorder, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sMarginWidth != null)
        {
            WriteNugget(MARGIN_WIDTH_ATTR, sb, response);
            WriteNugget(sMarginWidth, sb, response);
            WriteNugget(QUOTE, sb, response);
        }
    }

    /**
     * This method generates the closing tag of this html element.  If the
     * providesClosingTag method returns false, then this method will never be
     * called.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendClosingTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(END_TAG, sb, response);
    }

    /**
     * Indicates whether or not this HTML element provides a closing tag. Some
     * elements only provide opening tags, such as the &lt;br&gt; element.
     *
     * @return boolean true.
     */
    protected boolean ProvidesClosingTag()
    {
        return true;
    }

    /**
     * Get the next id value.  This is used by the GetClone method to ensure
     * that cloned object have new ID attribute values.
     *
     * @return A valid id string.
     */
    String GetNextID()
    {
        String s = "frame" + iNextID;
        iNextID++;

        return s;
    }
}
