package com.plumtree.xpshared.htmlelements;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;


/**
 * Classes that implement objects that can be used to obtain Strings that will
 * render HTML elements that can be used to send name/value pairs in an HTTP
 * request to a server may extend this abstract class. <br>
 * <br>
 * This class implements some functionality common to all such html elements.
 * <br><br>
 */
public abstract class HTMLGenericInputElement extends HTMLElement
{
    private static final byte[] VALUE_ATTR 		= HTMLElement.ToByteArray(" value=\"");
    private static final byte[] ONBLUR_ATTR 	= HTMLElement.ToByteArray(" onblur=\"");
    private static final byte[] ONCHANGE_ATTR 	= HTMLElement.ToByteArray(" onchange=\"");
    private static final byte[] ONFOCUS_ATTR 	= HTMLElement.ToByteArray(" onfocus=\"");
    
    String sValue;
    String sOnBlur;
    String sOnChange;
    String sOnFocus;
    
    private boolean bValidateName = true;

    /**
     * Creates a new HTMLGenericInputElement object.
     */
    public HTMLGenericInputElement()
    {
        sOnChange    = null;
        sOnFocus     = null;
        sOnBlur      = null;
        sValue 		 = null;
    }


	public void SetValidateName( boolean b )
	{
		bValidateName = b;
	}


    /**
     * Get the value of the onblur attribute.
     *
     * @return String value.
     */
    public String GetOnBlur()
    {
        return sOnBlur;
    }

    /**
     * Get the value of the onchange attribute.
     *
     * @return String value.
     */
    public String GetOnChange()
    {
        return sOnChange;
    }

    /**
     * Get the value of the onfocus attribute.
     *
     * @return String value.
     */
    public String GetOnFocus()
    {
        return sOnFocus;
    }

    /**
     * Get the value of the value attribute.
     *
     * @return String value.
     */
    public String GetValue()
    {
        return sValue;
    }

    //  override HTMLElement.SetName()

    /**
     * Override HTMLElement.SetName.
     *
     * @param s String value
     *
     * @throws HTMLException Throws Exception if name isn't valid.
     */
    public void SetName(String s)
    {
		if( bValidateName )
	        ValidateName(s);


        super.SetName(s);
    }

    /**
     * This method must be implemented by concrete subclasses to throw an
     * HTMLException if the name attribute of this element does not meet the
     * standards of this particular concrete class. <br>
     * <br>
     * For example, the HTMLInput class implements this method to throw an
     * HTMLException if the name attribute is not set to begin with "in_"
     * followed by a two-letter code designating the type of the input. <br>
     * <br>
     * The validation that occurs is at the discretion of the subclass, but
     * should enforce some level of quality. <br><br>
     *
     * @param s
     *
     * @throws HTMLException
     */
    public abstract void ValidateName(String s) ;

    /**
     * Set the value of the onblur attribute.
     *
     * @param s String value
     */
    public void SetOnBlur(String s)
    {
//        if (!HTMLElement.bAllowEvents)
//        {
//            if (bError)
//            {
//                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
//
//                Exception e = new Exception(sDesc);
//
//                myLogger.Error(sDesc, e);
//            }
//        }

        sOnBlur = s;
    }

    /**
     * Set the value of the onchange attribute.
     *
     * @param s String value
     */
    public void SetOnChange(String s)
    {
//        if (!HTMLElement.bAllowEvents)
//        {
//            if (bError)
//            {
//                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
//
//                Exception e = new Exception(sDesc);
//
//                myLogger.Error(sDesc, e);
//            }
//        }

        sOnChange = s;
    }

    /**
     * Set the value of the onfocus attribute.
     *
     * @param s String value
     */
    public void SetOnFocus(String s)
    {
//        if (!HTMLElement.bAllowEvents)
//        {
//            if (bError)
//            {
//                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
//
//                Exception e = new Exception(sDesc);
//
//                myLogger.Error(sDesc, e);
//            }
//        }

        sOnFocus = s;
    }

    /**
     * Set the value of the value attribute.  This value value will be
     * HTMLEncoded.
     *
     * @param s String value
     *
     * @throws HTMLException
     */
    public void SetValue(String s)
    {
        sValue = XPHttpUtility.HtmlEncode(s);
    }
    
    /* (non-Javadoc)
     * @see com.plumtree.xpshared.htmlelements.HTMLElement#AppendCommonAttributes(com.plumtree.openfoundation.util.XPStringBuilder, com.plumtree.openfoundation.web.IXPResponse)
     */
    void AppendCommonAttributes(XPStringBuilder sb, IXPResponse response)
    {
        super.AppendCommonAttributes(sb, response);
        
        if (sValue != null)
        {
	        WriteNugget(VALUE_ATTR, sb, response);
	        WriteNugget(sValue, sb, response);
	        WriteNugget(QUOTE, sb, response);
        }
                
        if (sOnBlur != null)
        {
            WriteNugget(ONBLUR_ATTR, sb, response);
            WriteNugget(sOnBlur, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sOnChange != null)
        {
            WriteNugget(ONCHANGE_ATTR, sb, response);
            WriteNugget(sOnChange, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sOnFocus != null)
        {
            WriteNugget(ONFOCUS_ATTR, sb, response);
            WriteNugget(sOnFocus, sb, response);
            WriteNugget(QUOTE, sb, response);
        }
    }
}
