package com.plumtree.xpshared.htmlelements;

//import com.plumtree.openfoundation.logging.*;
import com.plumtree.openlog.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;



/**
 * This class can be used to render an html input element. <br>
 * <br>
 * <a target="new" href="http://www.w3.org/TR/html4/interact/forms.html#h-17.4">Spec</a><br>
 * <br>
 * This class extends HTMLGeneralInputElement.  This means that objects of
 * this class can be used to set name/value pairs within a form. <br>
 * <br>
 * This class also implements TabableHTMLElement.  This means that this class
 * can generate a tabindex property that helps to guide the clients browser as
 * the client tabs through form elements. <br><br>
 */
public class HTMLInput extends HTMLGenericInputElement implements TabableHTMLElement
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.xpshared.htmlelements.HTMLInput");

    static int iNextID = 0;

    private static final byte[] BEGIN_TAG = HTMLElement.ToByteArray("<input type=\"");
	private static final byte[] END_TAG = HTMLElement.ToByteArray("</input>");
    private static final byte[] ALT_ATTR = HTMLElement.ToByteArray(" alt=\"");
    private static final byte[] TAB_INDEX_ATTR = HTMLElement.ToByteArray(" tabindex=\"");
    private static final byte[] ONCLICK_ATTR = HTMLElement.ToByteArray(" onclick=\"");
    private static final byte[] ONKEYDOWN_ATTR = HTMLElement.ToByteArray(" onkeydown=\"");
    private static final byte[] ONCHANGE_ATTR = HTMLElement.ToByteArray(" onchange=\"");
    private static final byte[] ALIGN_ATTR = HTMLElement.ToByteArray(" align=\"");
    private static final byte[] TITLE_ATTR = HTMLElement.ToByteArray(" title=\"");
    private static final byte[] CHECKED_ATTR = HTMLElement.ToByteArray(" checked");
    private static final byte[] READ_ONLY_ATTR = HTMLElement.ToByteArray(" readonly=\"readonly\"");
    private static final byte[] DISABLED_ATTR = HTMLElement.ToByteArray(" disabled=\"disabled\"");
    private static final byte[] SIZE_ATTR = HTMLElement.ToByteArray(" size=\"");
    private static final byte[] MAXLENGTH_ATTR = HTMLElement.ToByteArray(" maxlength=\"");
    private static final byte[] SRC_ATTR = HTMLElement.ToByteArray(" src=\"");
	private static final byte[] ACCESSKEY_ATTR = HTMLElement.ToByteArray(" accesskey=\"");
	private static final byte[] AUTOCOMPLETE_OFF_ATTR = HTMLElement.ToByteArray(" autocomplete=\"off\"");

    private HTMLInputTypes type;

    private String sAlign;
    private String sAlt;
	private String sAccessKey;
    private String sMaxLength;
    private String sOnClick;
    private String sOnKeyDown;
    private String sSize;
    private String sSrc;
    private String sTabIndex;
    
    private boolean bChecked;
    private boolean bReadOnly;
    private boolean bSupportsSubmit;
    private boolean bSupportsAccessKey;
    private boolean bAutoComplete;

	/**
	 * 	Set this to toggle whether or not input sizes are converted to
	 * 	netscape comparable sizes.  If true, the SetSize() method will
	 * 	look to the root of the display tree to determine whether the
	 * 	HTML output is intended for an IE browser or a Netscape browser.
	 *
	 */
	public static boolean bForceTextInputSizing = true;



    /**
     *
     * @param t Pass an HTMLInputTypes object.
     * @param sTagName Pass the name of this input element.
     * @param sID Pass the ID of this input element
     * @param alt Pass the alt attribute value.
     *
     * @throws HTMLException
     */
    public HTMLInput(HTMLInputTypes t, String sTagName, String sID, String alt)
    {
//        if (bDebug)
//        {
//            myLogger.Debug("constructor: t:" + t);
//            myLogger.Debug("constructor: sTagName:" + sTagName);
//            myLogger.Debug("constructor: alt:" + alt);
//        }

        type = t;

        SetName(sTagName);
        SetID(sID);

        SetAlt(alt);


        //  onFocus, onBlur, and onChange inherited from HTMLGenericInputElement
        sAlign          = null;
        bChecked        = false;
		bAutoComplete 	= true;
        bReadOnly       = false;
        sSize           = null;
        sMaxLength      = null;
        sSrc            = null;
        sValue          = "";
        bSupportsSubmit = false;
		bSupportsAccessKey = false;
        sOnClick        = null;
        sOnKeyDown      = null;
        sTabIndex       = null;

        if ((t == HTMLInputTypes.SUBMIT_TYPE) || (t == HTMLInputTypes.BUTTON_TYPE))
        {
            bSupportsSubmit = true;
			bSupportsAccessKey = true;
        }
        
        // rob: removing the validation checks. main reason is to avoid the validation check on textbox for banner search with friendly url
    	// (bug #70204) but it is also not necessary anymore since we are moving to layouts
        SetValidateName(false);
    }
    
    /**
     * Returns the HTMLInputTypes object associated with this.
     * 
     * @return HTMLInputTypes the typ of input.
     */
    public HTMLInputTypes GetHTMLInputType()
    {
        return type;
    }

    /**
     *
     * @param iHTMLInputType Pass an int from the HTMLInputTypes enumeration.
     * @param sTagName Pass the name of this input element.
     * @param sID Pass the ID of this input element
     * @param alt Pass the alt attribute value.
     *
     * @throws HTMLException
     */
    public HTMLInput(int iHTMLInputType, String sTagName, String sID, String alt)
    {
        SetID(sID);

        SetAlt(alt);


        //  onFocus, onBlur, and onChange inherited from HTMLGenericInputElement
        sAlign          = null;
        bChecked        = false;
		bAutoComplete 	= true;
        bReadOnly       = false;
        sSize           = null;
        sMaxLength      = null;
        sSrc            = null;
        sValue          = "";
        bSupportsSubmit = false;
		bSupportsAccessKey = false;
        sOnClick        = null;
        sOnKeyDown      = null;
        sTabIndex       = null;

        switch (iHTMLInputType)
        {
        case HTMLInputTypes.SUBMIT:
            bSupportsSubmit = true;
			bSupportsAccessKey = true;
            type            = HTMLInputTypes.SUBMIT_TYPE;

            break;

        case HTMLInputTypes.BUTTON:
            bSupportsSubmit = true;
			bSupportsAccessKey = true;
            type            = HTMLInputTypes.BUTTON_TYPE;

            break;

        case HTMLInputTypes.CHECKBOX:
            type = HTMLInputTypes.CHECKBOX_TYPE;

            break;

        case HTMLInputTypes.FILE:
            type = HTMLInputTypes.FILE_TYPE;

            break;

        case HTMLInputTypes.HIDDEN:
            type = HTMLInputTypes.HIDDEN_TYPE;

            break;

        case HTMLInputTypes.IMAGE:
            type = HTMLInputTypes.IMAGE_TYPE;

            break;

        case HTMLInputTypes.PASSWORD:
            type = HTMLInputTypes.PASSWORD_TYPE;

            break;

        case HTMLInputTypes.RADIO:
            type = HTMLInputTypes.RADIO_TYPE;

            break;

        case HTMLInputTypes.RESET:
            type = HTMLInputTypes.RESET_TYPE;

            break;

        case HTMLInputTypes.TEXT:
            type = HTMLInputTypes.TEXT_TYPE;

            break;
        }

        // rob: removing the validation checks. main reason is to avoid the validation check on textbox for banner search with friendly url
    	// (bug #70204) but it is also not necessary anymore since we are moving to layouts
        // BT: Bug 70204 - Need to call this before SetName since it calls ValidateName.
        SetValidateName(false);
        
        SetName(sTagName);
    }



	/**
	 *
	 * @param t Pass an HTMLInputTypes object.
	 * @param alt Pass the alt attribute value.
	 *
	 * @throws HTMLException
	 */
	public HTMLInput( HTMLInputTypes t, String sAlt )
	{
		type = t;
		SetID( this.GetNextID() );

		SetAlt( sAlt );

		//  onFocus, onBlur, and onChange inherited from HTMLGenericInputElement
		sAlign          = null;
		bChecked        = false;
		bAutoComplete 	= true;
		bReadOnly       = false;
		sSize           = null;
		sMaxLength      = null;
		sSrc            = null;
		sValue          = "";
		bSupportsSubmit = false;
		bSupportsAccessKey = false;
		sOnClick        = null;
		sOnKeyDown      = null;
		sTabIndex       = null;

		if ((t == HTMLInputTypes.SUBMIT_TYPE) || (t == HTMLInputTypes.BUTTON_TYPE))
		{
			bSupportsSubmit = true;
			bSupportsAccessKey = true;
		}
		
		// rob: removing the validation checks. main reason is to avoid the validation check on textbox for banner search with friendly url
    	// (bug #70204) but it is also not necessary anymore since we are moving to layouts
        SetValidateName(false);
	}

    /**
     *
     * @param t Pass an HTMLInputTypes object.
     * @param sTagName Pass the name of this input element.
     * @param alt Pass the alt attribute value.
     *
     * @throws HTMLException
     */
    public HTMLInput(HTMLInputTypes t, String sTagName, String alt)
    {
//        if (bDebug)
//        {
//            myLogger.Debug("constructor: t:" + t);
//            myLogger.Debug("constructor: sTagName:" + sTagName);
//            myLogger.Debug("constructor: alt:" + alt);
//        }

        type = t;
        SetName(sTagName);
        SetID(this.GetNextID());

        SetAlt(alt);


        //  onFocus, onBlur, and onChange inherited from HTMLGenericInputElement
        sAlign          = null;
        bChecked        = false;
		bAutoComplete 	= true;
        bReadOnly       = false;
        sSize           = null;
        sMaxLength      = null;
        sSrc            = null;
        sValue          = "";
        bSupportsSubmit = false;
		bSupportsAccessKey = false;
        sOnClick        = null;
        sOnKeyDown      = null;
        sTabIndex       = null;

        if ((t == HTMLInputTypes.SUBMIT_TYPE) || (t == HTMLInputTypes.BUTTON_TYPE))
        {
            bSupportsSubmit = true;
			bSupportsAccessKey = true;
        }
        
        // rob: removing the validation checks. main reason is to avoid the validation check on textbox for banner search with friendly url
    	// (bug #70204) but it is also not necessary anymore since we are moving to layouts
        SetValidateName(false);
    }

    /**
     * Call this method to create a new HTMLInput object. <br><br>
     *
     * @param iHTMLInputType Pass an int describing the type of this input
     *        element.
     * @param sTagName Pass the name of this element.
     * @param alt Pass the alt attribute of this element.
     *
     * @throws HTMLException
     *
     * @see HTMLInputTypes
     */
    public HTMLInput(int iHTMLInputType, String sTagName, String alt)
    {
        SetID(this.GetNextID());

        SetAlt(alt);


        //  onFocus, onBlur, and onChange inherited from HTMLGenericInputElement
        sAlign          = null;
        bChecked        = false;
		bAutoComplete 	= true;
        bReadOnly       = false;
        sSize           = null;
        sMaxLength      = null;
        sSrc            = null;
        sValue          = "";
        bSupportsSubmit = false;
		bSupportsAccessKey = false;
        sOnClick        = null;
        sOnKeyDown      = null;
        sTabIndex       = null;

        switch (iHTMLInputType)
        {
        case HTMLInputTypes.SUBMIT:
            bSupportsSubmit = true;
			bSupportsAccessKey = true;
            type            = HTMLInputTypes.SUBMIT_TYPE;

            break;

        case HTMLInputTypes.BUTTON:
            bSupportsSubmit = true;
			bSupportsAccessKey = true;
            type            = HTMLInputTypes.BUTTON_TYPE;

            break;

        case HTMLInputTypes.CHECKBOX:
            type = HTMLInputTypes.CHECKBOX_TYPE;

            break;

        case HTMLInputTypes.FILE:
            type = HTMLInputTypes.FILE_TYPE;

            break;

        case HTMLInputTypes.HIDDEN:
            type = HTMLInputTypes.HIDDEN_TYPE;

            break;

        case HTMLInputTypes.IMAGE:
            type = HTMLInputTypes.IMAGE_TYPE;

            break;

        case HTMLInputTypes.PASSWORD:
            type = HTMLInputTypes.PASSWORD_TYPE;

            break;

        case HTMLInputTypes.RADIO:
            type = HTMLInputTypes.RADIO_TYPE;

            break;

        case HTMLInputTypes.RESET:
            type = HTMLInputTypes.RESET_TYPE;

            break;

        case HTMLInputTypes.TEXT:
            type = HTMLInputTypes.TEXT_TYPE;

            break;
        }

        // rob: removing the validation checks. main reason is to avoid the validation check on textbox for banner search with friendly url
    	// (bug #70204) but it is also not necessary anymore since we are moving to layouts
        // BT: Bug 70204 - Need to call this before SetName since it calls ValidateName.
        SetValidateName(false);
        
        SetName(sTagName);
    }

    /**
     * Get the value of the align attribute.
     *
     * @return String value
     */
    public String GetAlign()
    {
        return sAlign;
    }

    /**
     * Get the value of the alt attribute.
     *
     * @return String value
     */
    public String GetAlt()
    {
        return sAlt;
    }

    /**
     * Get the value of the checked attribute.
     *
     * @return boolean value
     */
    public boolean GetChecked()
    {
        return bChecked;
    }

    /**
     * Get the value of the onclick attribute.
     *
     * @return String value
     */
    public String GetOnClick()
    {
        return sOnClick;
    }
    
    public boolean GetAutoComplete()
    {
    	return bAutoComplete;
    }

    /**
     * Get the value of the onkeydown attribute.
     *
     * @return String value
     */
    public String GetOnKeyDown()
    {
        return sOnKeyDown;
    }

    /**
     * Get the value of the readonly attribute.
     *
     * @return boolean value
     */
    public boolean GetReadOnly()
    {
        return bReadOnly;
    }

    /**
     */
    public String GetTabIndex()
    {
        return sTabIndex;
    }

    /**
     * Call this method to convert a size in IE to an equivalent size in
     * Netscape.
     *
     * @param iSize int
     *
     * @return int size in Netscape units.
     */
    public static int IESizeToNSSize(int iSize)
    {
        return (((2 * iSize) / 3) + 1);
    }

    /**
     * Get the value of the maxlength attribute.
     *
     * @return String value
     */
    public String GetMaxLength()
    {
        return sMaxLength;
    }

    /**
     * Get the value of the size attribute.
     *
     * @return String value
     */
    public String GetSize()
    {
        return sSize;
    }

    /**
     * Get the value of the src attribute.
     *
     * @return String value
     */
    public String GetSrc()
    {
        return sSrc;
    }

    /**
     * Set the value of the align attribute.
     *
     * @param s String value.
     */
    public void SetAlign(String s)
    {
        sAlign = s;
    }

    /**
     * Set the value of the alt attribute.
     *
     * @param s String value.
     */
    public void SetAlt(String s)
    {
        //  needs to be encoded because it's displayed
        sAlt = XPHttpUtility.HtmlEncode(s);
    }

    /**
     * Set the value of the checked attribute.
     *
     * @param bInputChecked boolean value.
     */
    public void SetChecked(boolean bInputChecked)
    {
        bChecked = bInputChecked;
    }
    
    
	/**
	 * Set the value of the autocomplete attribute.
	 *
	 * @param _bAutoComplete boolean value (false = disable autocomplete)
	 */
    
    public void SetAutoComplete(boolean _bAutoComplete)
	{
		bAutoComplete = _bAutoComplete;
			
	}
    /**
     * Set the value of the maxlength attribute.
     *
     * @param i size
     */
    public void SetMaxLength(int i)
    {
        sMaxLength = XPConvert.ToString(i);
    }

    /**
     * Set the value of the maxlength attribute.
     *
     * @param s String value.
     */
    public void SetMaxLength(String s)
    {
        sMaxLength = s;
    }

    /**
     * Set the value of the onclick attribute.
     *
     * @param s String value.
     */
    public void SetOnClick(String s)
    {
//        if (!HTMLElement.bAllowEvents)
//        {
//            if (bError)
//            {
//                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
//
//                Exception e = new Exception(sDesc);
//
//                myLogger.Error(sDesc, e);
//            }
//        }

        sOnClick = s;
    }

    /**
     * Set the value of the onkeydown attribute.
     *
     * @param s String value.
     */
    public void SetOnKeyDown(String s)
    {
//        if (!HTMLElement.bAllowEvents)
//        {
//            if (bError)
//            {
//                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
//
//                Exception e = new Exception(sDesc);
//
//                myLogger.Error(sDesc, e);
//            }
//        }

        sOnKeyDown = s;
    }

    /**
     * Set the value of the readonly attribute.
     *
     * @param bInputReadOnly value
     */
    public void SetReadOnly(boolean bInputReadOnly)
    {
        bReadOnly = bInputReadOnly;
    }

    /**
     * Set the value of the size attribute.
     *
     * @param i value.
     */
    public void SetSize(int i)
    {
    	if( bForceTextInputSizing )
    	{
			boolean bNS = false;
			DisplayOptions options = this.GetDisplayOptions();

			if( options != null )
				options.GetIsNetscape();


    		SetSize( i, bNS );
    	}
    	else
    	{
	        sSize = XPConvert.ToString(i);
    	}
    }

    /**
     * Call this method to set the size on this input.  This method can be used
     * to convert an IE size to an NS size.  Using this method, you can pass a
     * single size value, and the input resulting will look mostly the same in
     * Netscape and IE. <br><br>
     *
     * @param i Pass an int.  This int  should make this input appear the
     *        correct size when viewed in IE.<br><br>
     * @param bConvertIEToNS Pass a boolean indicating whether or not the first
     *        parameter should be converted to an Netscap size.
     */
    public void SetSize(int i, boolean bConvertIEToNS)
    {
        int iSet = i;

        if( bConvertIEToNS )
	        iSet = HTMLInput.IESizeToNSSize(i);

        sSize = XPConvert.ToString(iSet);
    }

    /**
     * Set the value of the size attribute.
     *
     * @param s String value.
     */
    public void SetSize(String s)
    {
    	sSize = s;
    }

    /**
     * Call this method to set the size on this input.  This method can be used
     * to convert an IE size to an NS size.  Using this method, you can pass a
     * single size value, and the input resulting will look mostly the same in
     * Netscape and IE. <br><br>
     *
     * @param s Pass a String that is parsable to an int.  This String  should
     *        make this input appear the correct size when viewed in IE.
     *        <br><br>
     * @param bConvertIEToNS Pass a boolean indicating whether or not the first
     *        parameter should be converted to an Netscap size.
     */
    public void SetSize(String s, boolean bConvertIEToNS)
    {
        int i;
        int iNS;

        String sSet = s;

        if( bConvertIEToNS )
        {

	        try
	        {
	            i = XPConvert.ToInteger(s);

	            iNS = HTMLInput.IESizeToNSSize(i);

	            sSet = XPConvert.ToString(iNS);
	        }
	        catch (Exception e)
	        {
	            sSet = s;
	        }

        }

        sSize = sSet;
    }

    /**
     * Set the value of the src attribute.
     *
     * @param sUrl String value.
     */
    public void SetSrc(String sUrl)
    {
        sSrc = sUrl;
    }

    /**
     */
    public void SetTabIndex(String s)
    {
        sTabIndex = s;
    }
    
	/**
	 * Call this method to set the value of the accesskey attribute.
	 *
	 * @param s Pass the String value.
	 */
	public void SetAccessKey(String s)
	{
		if (bSupportsAccessKey) sAccessKey = s;
		else throw new HTMLException("Access keys may only be set on Button or Submit Inputs.");
			
	}
	
	/**
	 * Call this method to get the value of the accesskey attribute.
	 *
	 * @return The String value of the accesskey attribute.
	 */
	public String GetAccessKey()
	{
		
		return sAccessKey;
	}


    /*
        The following methods are necessary to implement TabableHTMLElement
    */

    /**
     */
    public void ClearTabIndex()
    {
        sTabIndex = null;
    }

    /**
     * Check to see if a name string is valid.
     *
     * @param s Name string.
     *
     * @throws HTMLException Throws an exception if the name passed isn't
     *         valid.
     */
    public void ValidateName(String s)
    {
        int typeInt = type.ToInt();
//
//        if (bDebug)
//        {
//            myLogger.Debug("validateName: type: " + type.toString());
//        }

        switch (typeInt)
        {
        case HTMLInputTypes.BUTTON:

            if (!(s.startsWith("in_bu_")))
            {
                throw new HTMLException("Input button names must begin with 'in_bu_'.  " + s + " is an invalid name.");
            }

            break;

        case HTMLInputTypes.CHECKBOX:

            if (!(s.startsWith("in_cb_")))
            {
                throw new HTMLException("Input checkbox names must begin with 'in_cb_'.  " + s + " is an invalid name.");
            }

            break;

        case HTMLInputTypes.FILE:

            if (!(s.startsWith("in_fi_")))
            {
                throw new HTMLException("Input file names must begin with 'in_fi_'.  " + s + " is an invalid name.");
            }

            break;

        case HTMLInputTypes.HIDDEN:

            if (!(s.startsWith("in_hi_")))
            {
                throw new HTMLException("Input hidden names must begin with 'in_hi_'.  " + s + " is an invalid name.");
            }

            break;

        case HTMLInputTypes.IMAGE:

            if (!(s.startsWith("in_im_")))
            {
                throw new HTMLException("Input image names must begin with 'in_im_'.  " + s + " is an invalid name.");
            }

            break;

        case HTMLInputTypes.PASSWORD:

            if (!(s.startsWith("in_pw_")))
            {
                throw new HTMLException("Input password names must begin with 'in_pw_'.  " + s + " is an invalid name.");
            }

            break;

        case HTMLInputTypes.RADIO:

            if (!(s.startsWith("in_ra_")))
            {
                throw new HTMLException("Input radio names must begin with 'in_ra_'.  " + s + " is an invalid name.");
            }

            break;

        case HTMLInputTypes.RESET:

            if (!(s.startsWith("in_re_")))
            {
                throw new HTMLException("Input reset names must begin with 'in_re_'.  " + s + " is an invalid name.");
            }

            break;

        case HTMLInputTypes.SUBMIT:

            if (!(s.startsWith("in_su_")))
            {
                throw new HTMLException("Input submit names must begin with 'in_su_'.  " + s + " is an invalid name.");
            }

            break;

        case HTMLInputTypes.TEXT:

            if (!(s.startsWith("in_tx_")))
            {
                throw new HTMLException("Input text names must begin with 'in_tx_'.  " + s + " is an invalid name.");
            }

            break;
        }
    }

    /**
     * This method generates the beginning of the opening tag.  The abstract
     * HTMLElement superclass will provide the remainder of the opening tag,
     * repleate with attributes common to all html elements, such as name, and
     * id, and class.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendBeginningOfOpeningTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(BEGIN_TAG, sb, response);
        WriteNugget(type.ToString(), sb, response);
        WriteNugget(QUOTE, sb, response);

        if (sAlt != null)
        {
            WriteNugget(ALT_ATTR, sb, response);
            WriteNugget(sAlt, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sTabIndex != null)
        {
            WriteNugget(TAB_INDEX_ATTR, sb, response);
            WriteNugget(sTabIndex, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sOnClick != null)
        {
//            if (!HTMLElement.bAllowEvents || this.GetDo508CheckingForThisInstance())
//            {
//                if (bError)
//                {
//                    myLogger.Error("You are not allowed to set the onclick attribute value because the client of this html tree is attempting to render a 508-compliant page.");
//                    myLogger.Error("this: " + this);
//                }
//            }

            WriteNugget(ONCLICK_ATTR, sb, response);
            WriteNugget(sOnClick, sb, response);
            WriteNugget(QUOTE, sb, response);
        }
        
		if (sAccessKey != null)
		{
			WriteNugget(ACCESSKEY_ATTR, sb, response);
			WriteNugget(sAccessKey, sb, response);
			WriteNugget(QUOTE, sb, response);
		}

        if (sOnKeyDown != null)
        {
//            if (!HTMLElement.bAllowEvents || this.GetDo508CheckingForThisInstance())
//            {
//                if (bError)
//                {
//                    myLogger.Error("You are not allowed to set the onkeydown attribute value because the client of this html tree is attempting to render a 508-compliant page.");
//                    myLogger.Error("this: " + this);
//                }
//            }

            WriteNugget(ONKEYDOWN_ATTR, sb, response);
            WriteNugget(sOnKeyDown, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sAlign != null)
        {
            WriteNugget(ALIGN_ATTR, sb, response);
            WriteNugget(sAlign, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (bChecked)
        {
            WriteNugget(CHECKED_ATTR, sb, response);
        }

        if (bReadOnly)
        {
            WriteNugget(READ_ONLY_ATTR, sb, response);
        }
        
        if (!bAutoComplete)
        {
        	WriteNugget(AUTOCOMPLETE_OFF_ATTR, sb, response);
        }

        if (GetDisabled())
        {
            WriteNugget(DISABLED_ATTR, sb, response);
        }

        if (sSize != null)
        {

			String sSizeToWrite = sSize;

			if( bForceTextInputSizing )
			{
				boolean bNS = false;
				DisplayOptions options = this.GetDisplayOptions();

				if( options != null )
					bNS = options.GetIsNetscape();

    			if( bNS )
    			{

    				try
    				{
	    				int i = XPConvert.ToInteger( sSize );
		    			sSizeToWrite = XPConvert.ToString( HTMLInput.IESizeToNSSize( i ) );
    				}
    				catch( Exception ex )
    				{
						log.Info(ex, "Unable to calculate input size for netscape.");
    				}
    			}

			}


            WriteNugget(SIZE_ATTR, sb, response);
            WriteNugget(sSizeToWrite, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sMaxLength != null)
        {
            WriteNugget(MAXLENGTH_ATTR, sb, response);
            WriteNugget(sMaxLength, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sSrc != null)
        {
            WriteNugget(SRC_ATTR, sb, response);
            WriteNugget(sSrc, sb, response);
            WriteNugget(QUOTE, sb, response);
        }
    }

    /**
     * This method generates the closing tag of this html element.  If the
     * providesClosingTag method returns false, then this method will never be
     * called.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendClosingTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
    	WriteNugget(END_TAG, sb, response);
    }

    /**
     * Indicates whether or not this HTML element provides a closing tag. Some
     * elements only provide opening tags, such as the &lt;br&gt; element.
     *
     * @return boolean false.
     */
    protected boolean ProvidesClosingTag()
    {
        return true;
    }

    /**
     * Get the next id value.  This is used by the GetClone method to ensure
     * that cloned object have new ID attribute values.
     *
     * @return A valid id string.
     */
    String GetNextID()
    {
        String s = "input" + iNextID;
        iNextID++;

        return s;
    }
}
