package com.plumtree.xpshared.htmlelements;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;


/**
 * <br><br>
 * <a target="new"
 * href="http://www.w3.org/TR/html4/struct/links.html#h-12.3">Spec</a>
 */
public class HTMLLink extends HTMLElement
{
    static int iNextID = 0;

    private static final byte[] BEGIN_TAG = HTMLElement.ToByteArray("<link");
    private static final byte[] CHARSET_ATTR = HTMLElement.ToByteArray(" charset=\"");
    private static final byte[] TYPE_ATTR = HTMLElement.ToByteArray(" type=\"");
    private static final byte[] HREF_ATTR = HTMLElement.ToByteArray(" href=\"");
    private static final byte[] REL_ATTR = HTMLElement.ToByteArray(" rel=\"");
    private static final byte[] REV_ATTR = HTMLElement.ToByteArray(" rev=\"");
    private static final byte[] END_TAG = HTMLElement.ToByteArray("</link>");

    String sCharSet;
    String sHRef;
    String sRel;
    String sRev;
    String sType;

    /**
     * Creates a new HTMLLink object.
     */
    public HTMLLink()
    {
        sCharSet = null;
        sHRef    = null;
        sType    = null;
        sRel     = null;
        sRev     = null;

		// This is turned off by default, but the Link element needs
		// to use the lang attribute to localize the style sheets.
		bShowLang = true;
    }

    /**
     * Get the value of the charset attribute.
     *
     * @return String value
     */
    public String GetCharSet()
    {
        return sCharSet;
    }

    /**
     * Get the value of the href attribute.
     *
     * @return String value
     */
    public String GetHRef()
    {
        return sHRef;
    }

    /**
     * Get the value of the rel attribute.
     *
     * @return String value
     */
    public String GetRel()
    {
        return sRel;
    }

    /**
     * Get the value of the rev attribute.
     *
     * @return String value
     */
    public String GetRev()
    {
        return sRev;
    }

    /**
     * Get the value of the type attribute.
     *
     * @return String value
     */
    public String GetType()
    {
        return sType;
    }

    /**
     * Set the value of the charset attribute.
     *
     * @param s String value
     */
    public void SetCharSet(String s)
    {
        sCharSet = s;
    }

    /**
     * Set the value of the href attribute.
     *
     * @param s String value
     */
    public void SetHRef(String s)
    {
        sHRef = s;
    }

    /**
     * Set the value of the rel attribute.
     *
     * @param s String value
     */
    public void SetRel(String s)
    {
        sRel = s;
    }

    /**
     * Set the value of the rev attribute.
     *
     * @param s String value
     */
    public void SetRev(String s)
    {
        sRev = s;
    }

    /**
     * Set the value of the type attribute.
     *
     * @param s String value
     */
    public void SetType(String s)
    {
        sType = s;
    }

    /**
     * This method generates the beginning of the opening tag.  The abstract
     * HTMLElement superclass will provide the remainder of the opening tag,
     * repleate with attributes common to all html elements, such as name, and
     * id, and class.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendBeginningOfOpeningTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(BEGIN_TAG, sb, response);

        if (sCharSet != null)
        {
            WriteNugget(CHARSET_ATTR, sb, response);
            WriteNugget(sCharSet, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sType != null)
        {
            WriteNugget(TYPE_ATTR, sb, response);
            WriteNugget(sType, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sHRef != null)
        {
            WriteNugget(HREF_ATTR, sb, response);
            WriteNugget(sHRef, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sRel != null)
        {
            WriteNugget(REL_ATTR, sb, response);
            WriteNugget(sRel, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sRev != null)
        {
            WriteNugget(REV_ATTR, sb, response);
            WriteNugget(sRev, sb, response);
            WriteNugget(QUOTE, sb, response);
        }
    }

    /**
     * This method generates the closing tag of this html element.  If the
     * providesClosingTag method returns false, then this method will never be
     * called.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendClosingTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(END_TAG, sb, response);
    }

    /**
     * Indicates whether or not this HTML element provides a closing tag. Some
     * elements only provide opening tags, such as the &lt;br&gt; element.
     *
     * @return boolean false.
     */
    protected boolean ProvidesClosingTag()
    {
        return true;
    }

    /**
     * Get the next id value.  This is used by the GetClone method to ensure
     * that cloned object have new ID attribute values.
     *
     * @return A valid id string.
     */
    String GetNextID()
    {
        String s = "link" + iNextID;
        iNextID++;

        return s;
    }
}
