package com.plumtree.xpshared.htmlelements;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;


/**
 * <br><br>
 * <a target="new"
 * href="http://www.w3.org/TR/html4/interact/forms.html#h-17.6">Spec</a>
 */
public class HTMLOption extends HTMLElement
{
    private static final byte[]  BEGIN_TAG = HTMLElement.ToByteArray("<option");
    private static final byte[]  VALUE_ATTR = HTMLElement.ToByteArray(" value=\"");
    private static final byte[]  SELECTED_ATTR = HTMLElement.ToByteArray(" selected");
    private static final byte[] END_TAG = HTMLElement.ToByteArray("</option>");

    static int iNextID                      = 0;

    private String sValue;

    private boolean bSelected;

    /**
     *
     * @param sTagValue Pass the value string of this option.
     */
    public HTMLOption(String sTagValue)
    {
        bSelected = false;
        SetValue(sTagValue);
    }

    /**
    *
    * @param sTagValue value attribute
    * @param sDisplayText text node to be inserted inside <option> tags
    */
    public HTMLOption(String sTagValue, String sDisplayText)
    {
        this(sTagValue);
        AddInnerHTMLString(sDisplayText);
    }

    /**
     * Call this method to determine whether or not this option is selected.
     *
     * @return boolean value
     */
    public boolean GetSelected()
    {
        return bSelected;
    }

    /**
     * Call this method to obtain the value of the value attribute.
     *
     * @return String value
     */
    public String GetValue()
    {
        return sValue;
    }

    /**
     * Method rewritten to prevent usage.
     *
     * @param s String
     *
     * @throws HTMLException This method throws an exception.
     */
    public void SetName(String s)
    {
        throw new HTMLException("The \"name\" attribute is not valid for the <option> element.");
    }

    /**
     * Call this method to set this option selected.
     *
     * @param b boolean value.
     */
    public void SetSelected(boolean b)
    {
        bSelected = b;
    }

    /**
     * Call this method to set the value attribute.
     *
     * @param s String value
     */
    public void SetValue(String s)
    {
        sValue = s;
    }

    /**
     * This method generates the beginning of the opening tag.  The abstract
     * HTMLElement superclass will provide the remainder of the opening tag,
     * repleate with attributes common to all html elements, such as name, and
     * id, and class.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendBeginningOfOpeningTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(BEGIN_TAG, sb, response);

        WriteNugget(VALUE_ATTR, sb, response);
        WriteNugget(GetValue(), sb, response);
        WriteNugget(QUOTE, sb, response);

        if (bSelected)
        {
            WriteNugget(SELECTED_ATTR, sb, response);
        }
    }

    /**
     * This method generates the closing tag of this html element.  If the
     * providesClosingTag method returns false, then this method will never be
     * called.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendClosingTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(END_TAG, sb, response);
    }

    /**
     * Indicates whether or not this HTML element provides a closing tag. Some
     * elements only provide opening tags, such as the &lt;br&gt; element.
     *
     * @return boolean true.
     */
    protected boolean ProvidesClosingTag()
    {
        return true;
    }

    /**
     * Get the next id value.  This is used by the GetClone method to ensure
     * that cloned object have new ID attribute values.
     *
     * @return A valid id string.
     */
    String GetNextID()
    {
        String s = "option" + iNextID;
        iNextID++;

        return s;
    }
}
