package com.plumtree.xpshared.htmlelements;

import com.plumtree.openlog.*;
import com.plumtree.openfoundation.io.*;
//import com.plumtree.openfoundation.logging.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;


/**    This object can display an entire page of HTML, consisting of 
 *     one &lt;html&gt; element, one &lt;head&gt; element and one &lt;body&gt; 
 *     element.
 *     <br><br>
 *     Use the SetHead() and SetBody() methods to define the contents of this
 *     HTML page.
 *     <br><br>
 *     Use the validateHTML() method to determine whether or not the html generated
 *     by this object complies to the HTML 4 specification.  This method requires
 *     that CSE HTML Validator 5.0 be installed on the client computer.
 */
public class HTMLPage extends HTMLElement
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.xpshared.htmlelements.HTMLPage");

    private HTMLHead myHead;
    private HTMLBody myBody;
    private HTMLFrameSet myFrameSet;

    public static final String DOCTYPE_4_0_STRICT       = "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\" \"http://www.w3.org/TR/html4/strict.dtd\">";
    public static final String DOCTYPE_4_0_TRANSITIONAL = "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">";
    public static final String DOCTYPE_4_0_FRAMESET = "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Frameset//EN\" \"<http://www.w3.org/TR/REC-html40/frameset.dtd>\">";
    public static final String DOCTYPE_3_2 = "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">";
    
    public static String DOCTYPE_RENDERED = null;
    
    private static final byte[] BEGINTAG = HTMLElement.ToByteArray("<html");
    private static final byte[] ENDTAG   = HTMLElement.ToByteArray("</html>");

    static int iNextID = 0;

//
//    //   We share the logger for efficiency.
//    private static XPLogger myLogger = XPLogManager.GetLogger(HTMLPage.class);
//
//    //   pre-determining which levels are enabled allows the jvm to avoid
//    //   concatenating parameters of the logging methods.
//    private static boolean bDebug = myLogger.IsDebugEnabled();
//    private static boolean bError = myLogger.IsErrorEnabled();
//    private static boolean bWarn  = myLogger.IsWarnEnabled();
//    private static boolean bInfo  = myLogger.IsInfoEnabled();
//    private static boolean bFatal = myLogger.IsFatalEnabled();

    /**    Set this member true to cause the page to validate every time 
     *     GetDisplayString() is called.
     */
    public static boolean bAlwaysValidateHTMLOnDisplay = false;



    /**
     * Creates a new HTMLPage object.
     */
    public HTMLPage()
    {
        myHead     = null;
        myBody     = null;
        myFrameSet = null;
    }

	/**
	 * This method is used to get a reasonable sized
	 * String Builder.  It should be overridden by larger
	 * HTMLElements such as HTMLPage.
	 * 
	 * @return XPStringBuilder
	 */
	protected XPStringBuilder GetStringBuilder()
	{
		// This is currently not strictly necessary, because the
		// content of an HTMLPage is not converted into a string,
		// but rather output directly to an IXPResponse that is
		// passed down the HTML tree.

		// Default string size for smaller HTMLElements.
		// Larger Elements, such as HTMLPage, need to override
		// this to provide a more reasonable default.

        // Using a good default value for the size of this StringBuilder is
        // very important.  Here are some sample values for a quick test of
        // a mypage with one gadget and click through all the options (approx 10 hits)
        // 10000 = 3,761,480 bytes of temporary memory (+ 5,440 for appending)
        // 1000  =   874,432 bytes of temporary memory
        // 500   =   721,480 bytes of temporary memory
        // 250   =   640,840 bytes of temporary memory (+ 229,112 for appending)
        // 100   =   593,480 bytes of temporary memory (+ 261,848 for appending)
        // 50    =   576,840 bytes of temporary memory (+ 290,016 for appending)
        //
        // Here are some sample values for a quick test of the directory
        // (opening a folder, going up, and opening it again, going up, and
        // logging out):
        // 10000 = 4,010,512 bytes of temporary memory (+ 212,516 for appending)
        // 500   =   609,512 bytes of temporary memory
        // 250   =   519,296 bytes of temporary memory (+ 220,664 for appending)
        // 100   =   260,136 bytes of temporary memory (+ 181,876 for appending)
        // 50    =   447,696 bytes of temporary memory (+ 300,224 for appending)
        //
        // Be warned that these were manual tests, and the testing sequence
        // varied slightly between runs (one or two hit difference). DevPartner
        // was used for these tests.
        //
        // This default value was originally set to 10000 on the assumption that
        // an HTMLPage element would be around 30k or so.  This works well, but
        // not all top level HTMLElements are HTMLPages.  There are many HTMLImgs
        // and other HTMLElements constructed separately in the navigation.  These
        // are more on the order of 100 characters.
        //
        // Setting the default to 100 is a good solution, but a better solution
        // is to have certain elements override this default (HTMLPage should still
        // use a default value of around 10000).

		return new XPStringBuilder(10000);
	}


    //   Override HTMLElement.GetDisplayString

    /**
     * HTMLElement.GetDisplayString rewritten.
     * 
     * @return String display string.
     * @throws HTMLException Throws an exception if the html isn't valid
     * 	or cannot be generated.
     */
    public String GetDisplayString() 
    {
        if (bAlwaysValidateHTMLOnDisplay)
        {
            try
            {
                this.ValidateHTML();
            }
            catch (Exception e)
			{
				log.Error(e, "Unable to validate HTML.");
			}
        }

        return super.GetDisplayString();
    }

    /**
     * Get the next id value.  This is used by the GetClone method to ensure
     *     that cloned object have new ID attribute values.
     * 
     * @return A valid id string.
     */
    String GetNextID()
    {
        String s = "page" + iNextID;
        iNextID++;

        return s;
    }

    /**
     * This method generates the beginning of the opening tag.  The abstract
     *     HTMLElement superclass will provide the remainder of the opening tag,
     *     repleate with attributes common to all html elements, such as name, and
        *    id, and class.
     * 
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendBeginningOfOpeningTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        if( DOCTYPE_RENDERED != null )
        {
            WriteNugget(DOCTYPE_RENDERED, sb, response);
        }

        if (bFormat)
        {
            WriteNugget(sNewLine, sb, response);
        }

        WriteNugget(BEGINTAG, sb, response);
    }

    /**
     * This method generates the closing tag of this html element.  If the
     *     providesClosingTag method returns false, then this method will never
     *     be called.
     * 
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendClosingTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(ENDTAG, sb, response);

    }

    /**
     * Indicates whether or not this HTML element provides a closing tag.
     * Some elements only provide opening tags, such as the &lt;br&gt; element.
     * 
     * @return boolean true.
     */
    protected boolean ProvidesClosingTag()
    {
        return true;
    }

	/**
	 * Call this method to have greater control over validation.
	 * You can decide whether to save the results file, and if
	 * so, what the name and path of the file is.
	 * 
	 * @param bSaveResults save our results? 
	 * @param bSaveOnlyErrors save only on errors? 
	 * @param sFileName name of results file.
	 * @param sPathName path to results file.
	 * @throws Exception
	 */
	public void ValidateHTML(boolean bSaveResults,
							 boolean bSaveOnlyErrors,
							 String sFileName, 
							 String sPathName) throws Exception
	{
//		  if (bInfo)
//		  {
//			  myLogger.Info("Entering validateHTML()");
//		  }
		boolean hasErrors = false;

		String sPage = null;

		sPage = GetDisplayString(true);

		XPFile htmlFile     = null;
		String htmlFileName;

		// RWW - create a file with the given name / path
		htmlFile = new XPFile(sPathName + sFileName + ".html");

		XPFileWriter myWriter = htmlFile.CreateText();

		//  write HTML to the temp file.
		myWriter.Write(sPage);
		myWriter.Close();

		String sPath = htmlFile.GetFullPath();
		XPSystem.Out.WriteLine("sPath:[" + sPath + "]");

//		  if (bDebug)
//		  {
//			  myLogger.Debug("validateHTML - sPath: " + sPath);
//		  }

		//  This command executes HTML Validator on the temp HTML file.
		String sCommand = "cmdlineprocessor.exe " + sPath;

		XPProcess myProcess = XPProcess.Start(sCommand);

		//  Wait for the validator to validate.
		myProcess.WaitForExit();

		//  get the name of the temp file.
		htmlFileName = htmlFile.GetFullPath();
		XPSystem.Out.WriteLine("htmlFileName:[" + htmlFileName + "]");

		//  get the ".val" file.  This file describes the errors found.
		XPFile resultFile = new XPFile(htmlFileName + ".val");
		String sResultPath = resultFile.GetFullPath();
		XPSystem.Out.WriteLine("sResultPath:[" + sResultPath + "]");

		XPFileReader myReader             = resultFile.OpenText();
		XPBufferedReader myBufferedReader = new XPBufferedReader(myReader);

		String sLine = null;

		//  read the first line.
		sLine = myBufferedReader.ReadLine();

		int i;

		while (sLine != null)
		{
			//  while there's a line.
			i = sLine.indexOf("Number of errors:");

			if (i != -1)
			{
				//  This is the line describing the number of errors found.
				i = sLine.indexOf("Number of errors: 0");

				if (i == -1)
				{
					myReader.Close();

					// RWW - we don't want to throw the exception
					// in here because our clean up won't work then.
					hasErrors = true;
					break;
				}

				//  no errors were found! break and finish.
				break;

			} //  this is the line describing the number of errors.

			// if there are errors don't need to check any more lines
			if (hasErrors)
				break; 
				
			//  get the next line.
			sLine = myBufferedReader.ReadLine();

		} //  we have a line.
		
		// close
		myReader.Close();
		
		// should we save results?
		if (bSaveResults)
		{
			// save only the error results
			if (bSaveOnlyErrors)
			{
				// no errors so delete
				if (!hasErrors)
				{
					htmlFile.Delete();					
					resultFile.Delete();		
				}
			}
		}
		else
		{
			// not saving results, delete
			htmlFile.Delete();					
			resultFile.Delete();		
		}
			
		// if there were errors throw an exception			
		if (hasErrors)
			throw new HTMLException("Invalid HTML!\nValidator output can be found at: " + sResultPath);
	}

	/** Call this method to determine whether or not the HTML generated by this
	 *  HTMLPage object is valid HTML.<br><br>CSE HTML Validator version 5.0 
	 *     must be installed on the computer running this code.  The path to the 
	 *     main folder for the HTML Validator must
	 *  be included in the "PATH" environment variable.<br><br>
	 *  <b>CSE HTML Validator must be configured in the following way:</b>
	 *    <br><br>
	 *     <ol>
	 *  <li>Lauch the Validator.</li>
	 *  <li>Choose Options | Validator Engine Options.</li>
	 *  <li>Go to the Classic tab.</li>
	 *  <li>CHECK "Don't use output file; use .val files." option.</li>
	 *  <li>UNCHECK "If no errors/warnings, don't view." option.</li>
	 *  <li>CHECK "Never open viewer with .val files." option.</li>
	 *  <li>Click OK, and exit the HTML Validator.</li>
	 *     </ol>
	 *     <br><br>
	 *  This method will throw an HTMLException if the HTML generated is not
	 *  valid.  This exception will present the location of the HTML Validator
	 *     output file.  There will be no output file for valid HTML.
	 *     <br><br>
	 *     The client of this method will probably wish to catch
	 *  this Exception and display it to the console.  In this way, the invalid
	 *  HTML can still be generated and debugged.
	 *    <br><br>
	 *  Like:
	 *    <code>
	 *  try<br>
	 *  {<br>
	 *  &nbsp;&nbsp;myPage.validateHTML();<br>
	 *  }<br>
	 *  catch( HTMLException he )<br>
	 *  {<br>
	 *  &nbsp;&nbsp;System.out.println( he );<br>
	 *  }<br>
	 *    myPage.display( myResponse );<br>
	 *    </code>
	 *    <br><br>
	 *
	 * @throws HTMLException This Exception will describe the location of a
	 *     file giving a complete rundown of all errors in the HTML.
	 */
	public void ValidateHTML() throws Exception
	{
		// RWW - this was the default, use a temp
		// file and don't save the results.
		ValidateHTML(false, false, "temp", "");
	}

    /**    Call this method to remove the HTMLBody object associated with this
     *     HTMLPage.
     * 
     */
    public void ClearBody()
    {
        myBody = null;

        //   This funny little bit here, it's so that HTMLElement.GetInnerHTMLElements()
        //   always functions correctly.  Each time Head or Body is set, the internal
        //   list of inner htmlelements must be correct.  It's also important to maintain
        //   the order, of head, then body, cause that's the way it is.
        this.ClearInnerHTMLElements();

//        if (bDebug)
//        {
//            myLogger.Debug("About to call addInnerHTMLElement() twice ");
//        }

        try
        {
            super.AddInnerHTMLElement(this.myHead);
        }
        catch (HTMLException he)
		{
			log.Error(he, "Unable to clear body.");
		}
    }

    /**    Do not call this method.  Use the SetHeadh() and SetBody() methods 
     *     to define the HTML displayed by this html page.
     * 
     */
    public void AddInnerHTMLElement(HTMLElement e) 
    {
        throw new HTMLException("Use the SetHead(), SetBody(), and SetFrameSet() methods instead " + "of the addInnerHTMLElement method on HTMLPage.");
    }

    /**    Call this method to set the head of this html page.  If this head
     *     is not set, this page will display with an empty head.
     * 
     * 
     */
    public void SetHead(HTMLHead h)
    {
//        if (bDebug)
//        {
//            myLogger.Debug("Entering SetHead()");
//        }

        myHead = h;

        //   Start keeping track of level inside the head.
        myHead.SetLevel(0);

        //   This funny little bit here, it's so that HTMLElement.GetInnerHTMLElements()
        //   always functions correctly.  Each time Head or Body is set, the internal
        //   list of inner htmlelements must be correct.  It's also important to maintain
        //   the order, of head, then body, cause that's the way it is.
        this.ClearInnerHTMLElements();

        try
        {
            super.AddInnerHTMLElement(h);
            super.AddInnerHTMLElement(this.myBody);
        }
        catch (HTMLException e)
		{
			log.Error(e, "Unable to set head.");
		}

    }

    /**    Call this method to obtain the HTMLHead object describing the head of
     *     this html page.
     * 
     */
    public HTMLHead GetHead()
    {
        return myHead;
    }

    /**
     * Call this method to get the HTMLFrameSet of this page.
     * 
     * @return HTMLFrameSet object.
     */
    public HTMLFrameSet GetFrameSet()
    {
        return myFrameSet;
    }

    /**    Call this method to set the frameset of this page.  This object will not
     *     display until this method is called with a valid HTMLFrameSet object,
     *     or SetBody() is called with valid HTMLBody object.
     *     <br><br>
     *     A valid HTMLPage can have either an HTMLFrameSet or an HTMLBody, it 
     *     can neither have both nor neither.
     * 
     * 
     */
    public void SetFrameSet(HTMLFrameSet fs)
    {
//        if (bDebug)
//        {
//            myLogger.Debug("Entering SetFrameSet()");
//            myLogger.Debug("this: " + this);
//        }

        myFrameSet = fs;

        //   Start keeping track of level inside the body.
        myFrameSet.SetLevel(0);

        //   This funny little bit here, it's so that HTMLElement.GetInnerHTMLElements()
        //   always functions correctly.  Each time Head or Body is set, the internal
        //   list of inner htmlelements must be correct.  It's also important to maintain
        //   the order, of head, then body, cause that's the way it is.
        this.ClearInnerHTMLElements();

//        if (bDebug)
//        {
//            myLogger.Debug("About to call addInnerHTMLElement() twice ");
//        }

        try
        {
            super.AddInnerHTMLElement(this.myHead);
            super.AddInnerHTMLElement(myFrameSet);
        }
        catch (HTMLException he)
        {
			log.Error(he, "Unable to set frameset.");
        }

    }

    /**    Call this method to set the body of this page.  This object will not
     *     display until this method is called with a valid HTMLBody object or 
     *     a valid HTMLFrameSet object.
     *     <br><br>
     *     A valid HTMLPage can have either an HTMLFrameSet or an HTMLBody, it 
     *     can neither have both nor neither.
     * 
     */
    public void SetBody(HTMLBody b)
    {
//        if (bDebug)
//        {
//            myLogger.Debug("Entering SetBody()");
//            myLogger.Debug("this: " + this);
//        }

        myBody = b;

        //   Start keeping track of level inside the body.
        myBody.SetLevel(0);

        //   This funny little bit here, it's so that HTMLElement.GetInnerHTMLElements()
        //   always functions correctly.  Each time Head or Body is set, the internal
        //   list of inner htmlelements must be correct.  It's also important to maintain
        //   the order, of head, then body, cause that's the way it is.
        this.ClearInnerHTMLElements();
//
//        if (bDebug)
//        {
//            myLogger.Debug("About to call addInnerHTMLElement() twice ");
//        }

        try
        {
            super.AddInnerHTMLElement(this.myHead);
            super.AddInnerHTMLElement(b);
        }
        catch (HTMLException he)
        {
			log.Error(he, "Unable to set body.");
        }
    }

    /**    Call this method to obtain the HTMLBody object describing the body of
     *     this html page.
     * 
     */
    public HTMLBody GetBody()
    {
        return myBody;
    }
}
