package com.plumtree.xpshared.htmlelements;

//import com.plumtree.openfoundation.logging.*;
import com.plumtree.openlog.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;



/**
 * This class can be used to obtain a String describing an HTML SELECT element. <br>
 * <br>
 * <a target="new" href="http://www.w3.org/TR/html4/interact/forms.html#h-17.6">Spec</a><br>
 * <br>
 * This class extends HTMLGeneralInputElement.  This means that objects of
 * this class can be used to set name/value pairs within an HTML FORM. <br>
 * <br>
 * This class also implements TabableHTMLElement.  This means that this class
 * can generate a tabindex property that helps to guide the clients browser as
 * the client tabs through form elements.
 */
public class HTMLSelect extends HTMLGenericInputElement implements TabableHTMLElement
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.xpshared.htmlelements.HTMLSelect");

    static int iNextID = 0;

    private static final byte[] BEGIN_TAG = HTMLElement.ToByteArray("<select");
    private static final byte[] MULTIPLE_ATTR = HTMLElement.ToByteArray(" multiple=\"multiple\"");
    private static final byte[] SIZE_ATTR = HTMLElement.ToByteArray(" size=\"");
    private static final byte[] END_TAG = HTMLElement.ToByteArray("</select>");
    private static final byte[] DISABLED_ATTR = HTMLElement.ToByteArray(" disabled=\"disabled\"");

    private HTMLOption oOldSelected;

    private String sSize;
    private String sTabIndex;

	private XPArrayList alOptionCollections;

    private boolean bMultiple;

    private boolean bSort = true;

	/**
	 * This method sets the element to read only and prepares for display,
	 * if bReadOnly is true.
	 */
	protected void SetHTMLElementReadOnly( boolean bReadOnly )
	{
		try
		{
			if ((bReadOnly) && (!super.GetHTMLElementReadOnly()))
			{
				// If we are going to set this element to read only, then
				// we need to prepare the inner HTMLOptions for display
				// before we set it read only.
	
				PrepareForDisplay();
			}
	
			super.SetHTMLElementReadOnly(bReadOnly);
		} catch (HTMLException he)
		{
			// Due to possible multi-threaded access to this method,
			// it is possible that PrepareForDisplay could get called
			// when the element is already read only.  It is fine if
			// this happens, because that means that another thread
			// successfully called PrepareForDisplay().  Therefore
			// we just catch the exception and ignore it.

			// We will log the exception in case it is for another reason.
			
				log.Warn(he, "HTMLSelect.SetHTMLElementReadOnly failed.  This error should be ignored if it is because the Element is not writable, as this is a possible expected result.");
			
		}
	}

    /**
     *
     * @param sTagName Pass the name of this select.
     * @param sTagID Pass the id of this select.
     *
     * @throws HTMLException
     */
    public HTMLSelect(String sTagName, String sTagID)
    {

		alOptionCollections = new XPArrayList();

        //  onFocus, onBlur, and onChange inherited from HTMLGenericInputElement
        oOldSelected = null;

        SetName(sTagName);
        SetID(sTagID);

        sSize     = null;
        bMultiple = false;

		// This is turned off by default, but the Select element needs
		// to use the lang attribute for Mac IE 5.2 in asian languages.
		bShowLang = true;
		
		// rob: removing the validation checks. main reason is to avoid the validation check on textbox for banner search with friendly url
    	// (bug #70204) but it is also not necessary anymore since we are moving to layouts
        SetValidateName(false);
    }


	/**
	 * Call this method to specify whether or not this select will sort its nested
	 * options before each display.
	 *
	 *
	 * @param b boolean To sort or not
	 */
	public void SetDoSort( boolean b )
	{
		bSort = b;
	}

	/**
	 * 	Call this method to determine whether or not this select sorts its contents
	 * 	immediately before display.
	 *
	 * @return boolean to sort or not
	 */
	public boolean GetDoSort()
	{
		return bSort;
	}



    /**
     *
     * @return
     */
    public String GetSize()
    {
        return sSize;
    }

    /**
     *
     * @return
     */
    public String GetTabIndex()
    {
        return sTabIndex;
    }

    /**
     * Call this method to set the multiple attribute.
     *
     * @param bIsMultiple boolean value
     */
    public void SetMultiple(boolean bIsMultiple)
    {
        bMultiple = bIsMultiple;
    }

    /**
     * Call this method to set the size attribute value.
     *
     * @param i int size.
     */
    public void SetSize(int i)
    {
        sSize = XPConvert.ToString(i);
    }

    /**
     * Call this method to set the size attribute value.
     *
     * @param s size
     */
    public void SetSize(String s)
    {
        sSize = s;
    }

    /**
     *
     * @param s
     */
    public void SetTabIndex(String s)
    {
        sTabIndex = s;
    }

    //  **********************************
    //  Implements InputItem
    //  This method overrides HTMLGeneralInputElement.SetValue(), adding
    //  some functionality.

    /**
     * Call this method to set the value attribute value.  The nested option
     * having this value will be set selected.  If no nested option has this
     * value, an exception will be thrown.
     *
     * @param s String value
     *
     * @throws HTMLException This method throws an exception if there is no
     *         nested HTMLOption object having the specified value, or if this
     * 			element is read only.
     */
    public void SetValue(String s)
    {
    	super.VerifyWritability();

        //  todo rewrite using a hashmap for quickness.
        HTMLOption oSelected = null;

        super.SetValue(s);

		//	todo this could be more efficient... value setting could
		//	be done by ProcessOptionCollections, but I don't believe
		//	that HTMLSelect.SetValue() is called often.  I believe
		//	clients generally set the value when adding children.

		// Get the list of options
		XPArrayList alOptions = ProcessOptionCollections();

		IXPEnumerator xpEnumerator = alOptions.GetEnumerator();
        HTMLOption oTemp;

		// Find the option to select.
		while (xpEnumerator.MoveNext())
		{
			oTemp = (HTMLOption) xpEnumerator.GetCurrent();
			if (s.equals(oTemp.GetValue()))
			{
				oSelected = oTemp;
			}
		}

        if (oSelected == null)
        {
            throw new HTMLException("There is no <option> in this <select> having the value: " + s);
        }

        oSelected.SetSelected(true);

		// Unselect the previously selected option
        if ((oOldSelected != null) && (oOldSelected != oSelected))
        {
            oOldSelected.SetSelected(false);
        }

		// Store the selected option so we can unset it later if this method is called again.
        oOldSelected = oSelected;
    }


	/**
	 * This method converts the HTMLOptions and HTMLElementCollections
	 * into an array list.
	 * 
	 * @throws HTMLException if there are non HTMLOption elements in the
	 * list or in any of the collections.
	 */
	private XPArrayList ProcessOptionCollections()
	{
		XPArrayList alOptions = new XPArrayList();

		HTMLElement e;
		HTMLOption option;

		IXPEnumerator xpEnumerator = alOptionCollections.GetEnumerator();

		while( xpEnumerator.MoveNext() )
		{
			e = (HTMLElement) xpEnumerator.GetCurrent();

			if( e instanceof HTMLElementCollection )
			{
				//	a collection of options
				IXPEnumerator eOptions = e.GetInnerHTMLElements( false );

				HTMLElement eTemp;

				// NOTE: This was here, but I can't see any reason why we would
				// need to re-add this collection, since we never clear the list
				// and we just got it from the collection.
				//alOptionCollections.Add( e );

				while( eOptions.MoveNext() )
				{
					eTemp = (HTMLElement) eOptions.GetCurrent();

					// We have to check this here and not just when the collection is
					// added, because the collection could be added to the select list
					// before it has been populated.

					// Verify that the HTMLElementCollection only contains HTMLOptions.
					if( !(eTemp instanceof HTMLOption) )
					{
						throw new HTMLException( "An HTMLElementCollection being added to an HTMLSelect must exclusively contain HTMLOption elements.  " +
							eTemp.GetName() + " is not an HTMLOption." );
					}

					option = (HTMLOption) eTemp;

					alOptions.Add(option);
				}

			}
			else
			{
				// just an option
				// No need to verify this here because we verified this when the
				// option was added.
				option = (HTMLOption) e;

				alOptions.Add(option);
			}
		}

		return alOptions;
	}

    /**
     * HTMLElement.addInnerHTMLElement rewritten.
     *
     * @param e Pass an HTMLOption object or an HTMLElementCollection object.  If this
     * 	HTMLSelect sorts its nested children, then each nested HTMLElementCollection
     * 	will be independantly sorted.  These HTMLElementCollection objects must contain
     *	exclusively HTMLOption objects.  HTMLOption objects nested directly will not
     *	be sorted under any circumstances.
     *
     * @throws HTMLException This method throws an exception if you don't pass
     *         an HTMLOption object or a collection of HTMLOptions.
     */
    public void AddInnerHTMLElement(HTMLElement e)
    {
    	// Verify that this is the correct type of HTMLElement.
        if (!(e instanceof HTMLOption) && !(e instanceof HTMLElementCollection) )
        {
            throw new HTMLException("You may only add HTMLOption, or HTMLElementCollection elements to an HTMLSelect.");
        }

		// Verify that the HTMLElementCollection only contains HTMLOptions.
		if (e instanceof HTMLElementCollection)
		{
			//	a collection of options

			IXPEnumerator eOptions = e.GetInnerHTMLElements( false );
			HTMLElement eTemp;

			// We can't really check this here, because the collection could
			// be added to the select list before it has been populated, but
			// we'll try anyway so we can produce a more useful error message.
			// We also need to check this at display time.
			if (null != eOptions)
			{
				while( eOptions.MoveNext() )
				{
					eTemp = (HTMLElement) eOptions.GetCurrent();
	
					if( !(eTemp instanceof HTMLOption) )
					{
						throw new HTMLException( "An HTMLElementCollection being added to an HTMLSelect must exclusively contain HTMLOption elements." );
					}
				}
			}
		}

		alOptionCollections.Add( e );
    }



    /*
        The following methods are necessary to implement TabableHTMLElement
    */

    public void ClearTabIndex()
    {
        sTabIndex = null;
    }

    /**
     * Check to see if a name string is valid.
     *
     * @param s Name string.
     *
     * @throws HTMLException Throws an exception if the name passed isn't
     *         valid.
     */
    public void ValidateName(String s)
    {
        if (!s.startsWith("in_se_"))
        {
            throw new HTMLException("The name of a <select> element must begin with \"in_se_\".  " + s + " is an invalid name.");
        }
    }


    /**
     *	This method is called each time this select is called upon itself to display.
     *	This is where sorting is performed when necessary.
     *
     */
    private void PrepareForDisplay()
    {
   		XPArrayList alProcessedInner = new XPArrayList();

    	if( bSort )
    	{
    		//	we sort

	    	IXPEnumerator enumCollections = alOptionCollections.GetEnumerator();

	    	HTMLElement eTemp;

			// NOTE: We do not need to clear this table between different collections,
			// because we access it through a unique String[] for each collection.
			XPHashtable htTextToOption = new XPHashtable();

	    	while( enumCollections.MoveNext() )
	    	{

	    		eTemp = (HTMLElement) enumCollections.GetCurrent();

	    		if( eTemp instanceof HTMLElementCollection )
	    		{
	    		    //	this option-collection is a collection.
	    		    //	collections are sorted, options aren't.

	    			HTMLElementCollection tempCollection = (HTMLElementCollection) eTemp;

					XPArrayList alCollectionOptions = tempCollection.GetInnerHTMLElementsAsArrayList( false );

					int iSize = 0;
					IXPEnumerator xpEnumerator = null;

					if( alCollectionOptions != null )
					{
						iSize = alCollectionOptions.GetSize();

						xpEnumerator = alCollectionOptions.GetEnumerator();
					}

	    			if( iSize == 0 )
	    				return;

					String[] saText = new String[ iSize ];

					HTMLOption oTemp;
					String sText;
					int i = 0;

					//	iterate through our options, adding the nested strings to an
					//	array, which will be sorted, and a hash table which maps strings
					//	to the object containing the strings.

					while( xpEnumerator.MoveNext() )
					{
						oTemp = (HTMLOption) xpEnumerator.GetCurrent();

						sText = oTemp.GetInnerHTML();

						htTextToOption.PutElement( sText, oTemp );

						saText[i] = sText;

						i++;
					}

					//	WARNING: This call needs to be translated into .NET manually.
					XPArray.Sort( saText );

					for( int x = 0; x < iSize; x++ )
					{

						sText = saText[ x ];

						oTemp = (HTMLOption) htTextToOption.GetElement( sText );

						alProcessedInner.Add(oTemp);
					}
	    		}
	    		else
	    		{
	    			//	this option-collection is an option.
	    			//	these aren't sorted, so we just nest them.

	    			alProcessedInner.Add(eTemp);
	    		}
	    	}	//	for each option collection
	    	//	sorting
    	}
    	else
    	{
    		//	we don't sort

			//	We'll just add each of our inner options.  This cannot be done
			//	at the time this select is being built because it's possible that the
			//	select will be told to sort itself immediately before display.

    		//	This iterates through all nested elements, HTMLOptions and
    		//	HTMLElementCollections, and returns an XPArrayList containing each of them.

			alProcessedInner = ProcessOptionCollections();
    	}

		// Store the processed options as the new inner HTML elements.
		super.SetInner(alProcessedInner);
    }



    /**
     * This method generates the beginning of the opening tag.  The abstract
     * HTMLElement superclass will provide the remainder of the opening tag,
     * repleate with attributes common to all html elements, such as name, and
     * id, and class.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendBeginningOfOpeningTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
		if (!super.GetHTMLElementReadOnly())
		{
			// This method creates the underlying element list, so
			// we can only call this if the element is writable.
	    	PrepareForDisplay();		
		}

        WriteNugget(BEGIN_TAG, sb, response);
        
        if (bMultiple)
        {
            WriteNugget(MULTIPLE_ATTR, sb, response);
        }

        if (sSize != null)
        {
            WriteNugget(SIZE_ATTR, sb, response);
            WriteNugget(sSize, sb, response);
            WriteNugget(QUOTE, sb, response);
        }
        if (GetDisabled())
        {
            WriteNugget(DISABLED_ATTR, sb, response);
        }
    }

    /**
     * This method generates the closing tag of this html element.  If the
     * providesClosingTag method returns false, then this method will never be
     * called.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendClosingTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(END_TAG, sb, response);
    }

    /**
     * Indicates whether or not this HTML element provides a closing tag. Some
     * elements only provide opening tags, such as the &lt;br&gt; element.
     *
     * @return boolean true.
     */
    protected boolean ProvidesClosingTag()
    {
        return true;
    }

    /**
     * Get the next id value.  This is used by the GetClone method to ensure
     * that cloned object have new ID attribute values.
     *
     * @return A valid id string.
     */
    String GetNextID()
    {
        String s = "select" + iNextID;
        iNextID++;

        return s;
    }
}
