package com.plumtree.xpshared.htmlelements;

import com.plumtree.openlog.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;


/**
 * Use this object to generate a &lt;td&gt;&lt;/td&gt; tag. <br>
 * <br>
 * <a target="new"
 * href="http://www.w3.org/TR/html4/struct/tables.html#h-11.2.6">Spec</a>
 */
public class HTMLTableCell extends HTMLElement
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.xpshared.htmlelements.HTMLTableCell");

    static int iNextID = 0;

    private static final byte[] BEGINTAG 		= HTMLElement.ToByteArray("<td");
    private static final byte[] ALIGN_ATTR 		= HTMLElement.ToByteArray(" align=\"");
    private static final byte[] VALIGN_ATTR 	= HTMLElement.ToByteArray(" valign=\"");
    private static final byte[] BGCOLOR_ATTR 	= HTMLElement.ToByteArray(" bgcolor=\"");
	private static final byte[] HEADERS_ATTR 	= HTMLElement.ToByteArray(" headers=\"");
    private static final byte[] HEIGHT_ATTR 	= HTMLElement.ToByteArray(" height=\"");
    private static final byte[] WIDTH_ATTR 		= HTMLElement.ToByteArray(" width=\"");
    private static final byte[] NOWRAP_ATTR 	= HTMLElement.ToByteArray(" nowrap=\"nowrap\"");
    private static final byte[] COLSPAN_ATTR 	= HTMLElement.ToByteArray(" colspan=\"");
	private static final byte[] ROWSPAN_ATTR 	= HTMLElement.ToByteArray(" rowspan=\"");
    private static final byte[] ENDTAG 			= HTMLElement.ToByteArray("</td>");

    private static final String ONE 			= "1";

    private HTMLTable myParentTable;

    private HTMLTableRow myParentRow;

    private String sAlign;
    private String sBGColor;
    private String sColSpan;
	private String sRowSpan;
	private String sHeaders;
    private String sHeight;
    private String sVAlign;
    private String sWidth;
    private boolean bAutoFit;
    private boolean bNoWrap;
    private boolean bValidated;

    /**
     * Call this method to create a new HTMLTableCell object.
     *
     * @param rParent Pass the HTMLTableRow to which this HTMLTableCell will be
     *        added.  A given instance of an HTMLTableCell may only be added
     *        to the HTMLTableRow specified to this constructor.<br><br>
     *
     * @deprecated It's no longer necessary to pass the parent row, use
     *             constructor with no parameters.
     */
    public HTMLTableCell(HTMLTableRow rParent)
    {
        sAlign   = null;
        sVAlign  = null;
        sColSpan = ONE;
		sRowSpan = null;  // no logic for rowspan
        sBGColor = null;
		sHeaders = null;
        sHeight  = null;
        sWidth   = null;
        bNoWrap  = false;

        bValidated    = false;
        bAutoFit      = false;
        myParentTable = rParent.GetParentTable();
        myParentRow   = rParent;
    }

    /**
     * Creates a new HTMLTableCell object.
     */
    public HTMLTableCell()
    {
        sAlign   = null;
        sVAlign  = null;
        sColSpan = ONE;
        sRowSpan = null;

		sHeaders = null;
        sHeight = null;
        sWidth  = null;
        bNoWrap = false;

        bValidated    = false;
        bAutoFit      = false;
        myParentTable = null;
        myParentRow   = null;
    }

    /**
     * Call this method to get the value of the align attribute.
     *
     * @return String value
     */
    public String GetAlign()
    {
        return sAlign;
    }

    /**
     * Call this method to determine whether or not this HTMLCell object will
     * automatically span the number of columns necessary.
     *
     * @return
     */
    public boolean GetAutoFit()
    {
        return bAutoFit;
    }

    /**
     *
     *
     * @return
     */
    public String GetBGColor()
    {
        return sBGColor;
    }

    /**
     * Call this method to get the value of the colspan attribute.  This value
     * is calculated at display time.
     *
     * @return String value.
     */
    public String GetColSpan()
    {
        return sColSpan;
    }

	/**
	 * Call this method to get the value of the rowspan attribute.  This value
	 * is calculated at display time.
	 *
	 * @return String value.
	 */
	public String GetRowSpan()
	{
		return sRowSpan;
	}

    /**
     * Call this method to get the value of the colspan attribute.  This value
     * is calculated at display time.
     *
     * @return int value.
     */
    public int GetColSpanAsInt()
    {
        int i = 0;
        
        try
        {
            if (null != sColSpan)
            {
                i = XPConvert.ToInteger(sColSpan);
            }
        }
        catch (Exception e)
        {
			log.Info(e, "Unable to convert ColSpanAsInt.");
        }

        return i;
    }

	/**
	 * Call this method to get the value of the rowspan attribute.  This value
	 * is calculated at display time.
	 *
	 * @return int value.
	 */
	public int GetRowSpanAsInt()
	{
        int i = 0;

		try
		{
			if (null != sRowSpan)
			{
			    i = XPConvert.ToInteger(sRowSpan);
			}
		}
		catch (Exception e)
		{
			log.Info(e, "Unable to convert RowSpanAsInt.");
		}

		return i;
	}

	/**
	 *
	 * @return
	 */
	public String GetHeaders()
	{
		return sHeaders;
	}

    /**
     *
     * @return
     */
    public String GetHeight()
    {
        return sHeight;
    }

    /**
     *
     * @return
     */
    public boolean GetNoWrap()
    {
        return bNoWrap;
    }

    /**
     * Call this method to get the value of the valign attribute.
     *
     * @return String value
     */
    public String GetVAlign()
    {
        return sVAlign;
    }

    /**
     *
     * @return
     */
    public String GetWidth()
    {
        return sWidth;
    }

    /**
     * Call this method to set the value of the align attribute.
     *
     * @param s String value
     */
    public void SetAlign(String s)
    {
        //   This such a change does not invalidate.
        sAlign = s;
    }

    /**
     * Call this method to toggle whether or not this HTMLTableCell is an
     * autofit cell.  An autofit cell will take up as many columns as is
     * necessary to complete the row within which it resides.  When asked to
     * display, a given HTMLTableRow may only have one  single autofit cell.
     * This check is executed at display time, and an HTMLException will be
     * thrown if there are multiple autofit cells within a given row.
     * 
     * Note that autofit does not apply to rowspan, which must be set manually.
     *
     * @param b
     */
    public void SetAutoFit(boolean b)
    {
        //   If a change is made to this state, then this cell is no longer
        //   validated.  Nor is this cell's parent row, nor parent table.
        //   Everything will need to be re-validated upon display.
        if (b != bAutoFit)
        {
            Invalidate();
        }

        bAutoFit = b;

        if (b)
        {
            sColSpan = ONE;
        }
    }

    /**
     *
     * @param s
     */
    public void SetBGColor(String s)
    {
        sBGColor = s;
    }

    /**
     * Call this method to set the value of the colspan attribute.
     *
     * @param s String value.
     *
     * @throws HTMLException
     */
    public void SetColSpan(String s)
    {
        //   If a change is made to this state, then this cell is no longer
        //   validated.  Nor is this cell's parent row, nor parent table.
        //   Everything will need to be re-validated upon display.
        if (!sColSpan.equals(s))
        {
            Invalidate();
        }

        sColSpan = s;
    }

	/**
	 * Call this method to set the value of the rowspan attribute.
	 *
	 * @param s String value.
	 *
	 * @throws HTMLException
	 */
	public void SetRowSpan(String s)
	{
		//   If a change is made to this state, then this cell is no longer
		//   validated.  Nor is this cell's parent row, nor parent table.
		//   Everything will need to be re-validated upon display.
		if ((sRowSpan == null) || (!sRowSpan.equals(s)))
		{
			Invalidate();
		}

		sRowSpan = s;
	}

    //   To Be Deprecated

	/**
	 * Call this method to set the value of the headers attribute.
	 *
	 * @param s String value.
	 */
	public void SetHeaders(String s)
	{
		sHeaders = s;
	}

	/**
	 * Call this method to set the value of the height attribute.
	 *
	 * @param s String value.
	 */
	public void SetHeight(String s)
	{
		sHeight = s;
	}

    //   To Be Deprecated

    /**
     * Call this method to toggle the nowrap attribute.
     *
     * @param b value
     */
    public void SetNoWrap(boolean b)
    {
        bNoWrap = b;
    }

    /**
     * Call this method to set the value of the valign attribute.
     *
     * @param s String value
     */
    public void SetVAlign(String s)
    {
        //   This such a change does not invalidate.
        sVAlign = s;
    }

    //   To Be Deprecated

    /**
     * Call this method to set the value of the width attribute.
     *
     * @param s String value.
     */
    public void SetWidth(String s)
    {
        sWidth = s;
    }

    /**
     * This method generates the beginning of the opening tag.  The abstract
     * HTMLElement superclass will provide the remainder of the opening tag,
     * repleate with attributes common to all html elements, such as name, and
     * id, and class.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     *
     * @throws HTMLException
     */
    protected void AppendBeginningOfOpeningTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        Validate();

        int iNumColsInParentTable = myParentTable.GetNumColsSpanned();
        int iNumColsInParentRow   = myParentRow.GetNumColsSpanned();

//        if (bDebug)
//        {
//            myLogger.Debug("getDisplayString - iNumColsInParentTable: " + iNumColsInParentTable);
//            myLogger.Debug("getDisplayString - iNumColsInParentRow: " + iNumColsInParentRow);
//        }

        WriteNugget(BEGINTAG, sb, response);

        if (sAlign != null)
        {
            WriteNugget(ALIGN_ATTR, sb, response);
            WriteNugget(sAlign, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sVAlign != null)
        {
            WriteNugget(VALIGN_ATTR, sb, response);
            WriteNugget(sVAlign, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sBGColor != null)
        {
            WriteNugget(BGCOLOR_ATTR, sb, response);
            WriteNugget(sBGColor, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

		if (sHeaders != null)
		{
			WriteNugget(HEADERS_ATTR, sb, response);
			WriteNugget(sHeaders, sb, response);
			WriteNugget(QUOTE, sb, response);
		}

        if (sHeight != null)
        {
            WriteNugget(HEIGHT_ATTR, sb, response);
            WriteNugget(sHeight, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sWidth != null)
        {
            WriteNugget(WIDTH_ATTR, sb, response);
            WriteNugget(sWidth, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (bNoWrap)
        {
            WriteNugget(NOWRAP_ATTR, sb, response);
        }

        if (!bAutoFit)
        {
            // this cell doesn not auto fit.
            // note: can't have column and row spanning in the same element
			if (sRowSpan != null)
			{
				WriteNugget(ROWSPAN_ATTR, sb, response);
				WriteNugget(sRowSpan, sb, response);
				WriteNugget(QUOTE, sb, response);
			}
			// sColSpan isn't NULL unless set so by user; default is ONE
            else if (sColSpan != null)
            {
                WriteNugget(COLSPAN_ATTR, sb, response);
                WriteNugget(sColSpan, sb, response);
                WriteNugget(QUOTE, sb, response);
            }
        }
        else
        {
            //  This cell auto fits.
            int iTemp = (iNumColsInParentTable - iNumColsInParentRow) + 1;

            // note that it's either column or row spanning, not both
			if (sRowSpan != null)
			{
				WriteNugget(ROWSPAN_ATTR, sb, response);
				WriteNugget(sRowSpan, sb, response);
				WriteNugget(QUOTE, sb, response);
			}
			// sColSpan isn't NULL unless set so by user; default is ONE
			else if (sColSpan != null)
			{
				// note use of autofit calculated colspan
    	        WriteNugget(COLSPAN_ATTR, sb, response);
        	    WriteNugget(XPConvert.ToString(iTemp), sb, response);
            	WriteNugget(QUOTE, sb, response);
	        }
        }
    }

    /**
     * This method generates the closing tag of this html element.  If the
     * providesClosingTag method returns false, then this method will never be
     * called.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendClosingTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(ENDTAG, sb, response);

		AppendClosingLineBreak(sb,response);
    }

    /**
     * Indicates whether or not this HTML element provides a closing tag. Some
     * elements only provide opening tags, such as the &lt;br&gt; element.
     *
     * @return boolean true.
     */
    protected boolean ProvidesClosingTag()
    {
        return true;
    }

    /**
     * Get the next id value.  This is used by the GetClone method to ensure
     * that cloned object have new ID attribute values.
     *
     * @return A valid id string.
     */
    String GetNextID()
    {
        String s = "tabcell" + iNextID;
        iNextID++;

        return s;
    }

    /**
     * Call this method to get the HTMLRow to which this HTMLTableCell belongs.
     *
     * @return HTMLTableRow parent.
     */
    HTMLTableRow GetParentRow()
    {
        return myParentRow;
    }

    /**
     * Call this method to obtain the HTMLTable to which this cell belongs.
     *
     * @return HTMLTable parent table.
     */
    HTMLTable GetParentTable()
    {
        return myParentTable;
    }

    /**
     * Call this method to set the HTMLTableRow to which this cell belongs.
     *
     * @param r HTMLTableRow parent row.
     */
    void SetParentRow(HTMLTableRow r)
    {
        myParentRow = r;
    }

    /**
     * Call this method to set the HTMLTable to which this cell belongs.
     *
     * @param t HTMLTable parent table.
     */
    void SetParentTable(HTMLTable t)
    {
        myParentTable = t;
    }

    void Invalidate()
    {
//        if (bInfo)
//        {
//            myLogger.Info("Entering invalidate()");
//        }

        if (myParentTable != null)
        {
            myParentTable.Invalidate();
        }

        if (myParentRow != null)
        {
            myParentRow.Invalidate();
        }

        bValidated = false;
    }

    /**
     * Call this method at display time to cause this HTMLCell to prepare
     * inself for display.
     *
     * @throws HTMLException
     */
    void Validate()
    {
//        if (bInfo)
//        {
//            myLogger.Info("Entering validate");
//        }

        if (bValidated)
        {
//            if (bDebug)
//            {
//                myLogger.Debug("Already validated, returning");
//            }

            return;
        }

        if (myParentRow == null)
        {
            throw new HTMLException("The parent row of this cell has not been set.");
        }


        //   set the parent table.  This may have already been set, but we'll
        //   just set it again to be sure.
        myParentTable = myParentRow.GetParentTable();
    }
}
