package com.plumtree.xpshared.htmlelements;

//import com.plumtree.openfoundation.logging.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;


/**    This class can be used to generate <tr> html elements.
 *
 *  <br><br>
 *  <a target="new" href="http://www.w3.org/TR/html4/struct/tables.html#h-11.2.5">Spec</a>
 *
 */
public class HTMLTableRow extends HTMLElement
{
    private String sAlign;
    private String sBGColor;
    private String sBorderColor;
    private String sVAlign;
    private int iNumColsSpanned;
    static int iNextID                 = 0;
    private int iIndexInTable          = -1;
    private boolean bHasAutoFitElement;
    private HTMLTable myParentTable;
    private boolean bValidated;

    private static final byte[] BEGINTAG         = HTMLElement.ToByteArray("<tr");
    private static final byte[] ALIGN_ATTR       = HTMLElement.ToByteArray(" align=\"");
    private static final byte[] BGCOLOR_ATTR     = HTMLElement.ToByteArray(" bgcolor=\"");
    private static final byte[] BORDERCOLOR_ATTR = HTMLElement.ToByteArray(" bordercolor=\"");
    private static final byte[] VALIGN_ATTR      = HTMLElement.ToByteArray(" valign=\"");
    private static final byte[] ENDTAG           = HTMLElement.ToByteArray("</tr>");

    /**    Call this method to create a new HTMLTableRow object.
     * 
     * @param tParent Pass the HTMLTable to which this HTMLTableRow will be
     * added.  A given instance of an HTMLTableRow may only be added to the
     * HTMLTable specified to this constructor.
     * <br><br>        
     * @deprecated It's no longer necessary to pass the parent table, use
     * constructor with no parameters.
     */
    public HTMLTableRow(HTMLTable tParent)
    {
        sAlign             = null;
        sBGColor           = null;
        sBorderColor       = null;
        sVAlign            = null;
        iNumColsSpanned    = 0;
        bHasAutoFitElement = false;
        bValidated = false;

        myParentTable = tParent;
    }

    /**
     * Creates a new HTMLTableRow object.
     */
    public HTMLTableRow()
    {
        sAlign             = null;
        sBGColor           = null;
        sBorderColor       = null;
        sVAlign            = null;
        iNumColsSpanned    = 0;
        bHasAutoFitElement = false;
        bValidated = false;

        myParentTable = null;
    }


    /**
     * Call this method to obtain an clone of this table row.
     * 
     * @return Object that is an HTMLTableRow. 
     */
    public Object GetClone()
    {
        HTMLTableRow rClone = (HTMLTableRow) super.GetClone();

        IXPEnumerator xpEnumerator = rClone.GetInnerHTMLElements(false);
        HTMLTableCell c;

        while (xpEnumerator.MoveNext())
        {
            c              = (HTMLTableCell) xpEnumerator.GetCurrent();

            c.SetParentRow(this);
        }

        return rClone;
    }

    /**
     * Get the next id value.  This is used by the GetClone method to ensure
     *     that cloned object have new ID attribute values.
     * 
     * @return A valid id string.
     */
    String GetNextID()
    {
        String s = "tabrow" + iNextID;
        iNextID++;

        return s;
    }



    //   package-private method
    /**
     * Call this method to obtain the index of this row in its parent table.
     * 	This is calculated at display time.
     * 
     * @return int index.
     */
    int GetIndexInTable()
    {
        return iIndexInTable;
    }


    //   package-private method
    /**
     * Call this method to set the index of this row in its parent table.
     * 
     * @param i int index.
     */
    void SetIndexInTable(int i)
    {
        iIndexInTable = i;
    }

    /**
     * This method generates the beginning of the opening tag.  The abstract
     *     HTMLElement superclass will provide the remainder of the opening tag,
     *     repleate with attributes common to all html elements, such as name, and
     *    id, and class.
     * 
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendBeginningOfOpeningTag(boolean bFormat, XPStringBuilder sb, IXPResponse response) 
    {
        Validate();

        WriteNugget(BEGINTAG, sb, response);

        if (sAlign != null)
        {
            WriteNugget(ALIGN_ATTR, sb, response);
            WriteNugget(sAlign, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sBGColor != null)
        {
            WriteNugget(BGCOLOR_ATTR, sb, response);
            WriteNugget(sBGColor, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sBorderColor != null)
        {
            WriteNugget(BORDERCOLOR_ATTR, sb, response);
            WriteNugget(sBorderColor, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sVAlign != null)
        {
            WriteNugget(VALIGN_ATTR, sb, response);
            WriteNugget(sVAlign, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

    }

    /**
     * This method generates the closing tag of this html element.  If the
     *     providesClosingTag method returns false, then this method will never
     *     be called.
     * 
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendClosingTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(ENDTAG, sb, response);

        AppendClosingLineBreak(sb,response);
    }

    /**
     * Indicates whether or not this HTML element provides a closing tag.
     * Some elements only provide opening tags, such as the &lt;br&gt; element.
     * 
     * @return boolean true.
     */
    protected boolean ProvidesClosingTag()
    {
        return true;
    }

    /**    Call this method to obtain a reference to the HTMLTable to which
     *     this HTMLTableRow may be added.
     * 
     */
    public HTMLTable GetParentTable()
    {
        return myParentTable;
    }

    // package-private


    /**
     * Call this method to set the table to which this row belongs.
     * 
     * @param t HTMLTable parent table.
     */
    void SetParentTable(HTMLTable t)
    {
        myParentTable = t;
    }

    /**    Call this method to determine whether or not this HTMLTableRow
     *     has any autofit cells.
     * 
     */
    public boolean GetHasAutoFitElement() 
    {
//        if (bInfo)
//        {
//            myLogger.Info("Entering GetHasAutoFitElement()");
//        }

        Validate();

//        if (bDebug)
//        {
//            myLogger.Debug("GetHasAutoFitElement() - bHasAutoFitElement: " + bHasAutoFitElement);
//        }

        return bHasAutoFitElement;
    }

    /**
     * Call this method to set the value of the align attribute.
     * 
     * @param s String value
     */
    public void SetAlign(String s)
    {
        sAlign = s;
    }

    /**
     * Call this method to obtain the value of the align attribute.
     * 
     * @return String value
     */
    public String GetAlign()
    {
        return sAlign;
    }

    /**
     * Call this method to set the value of the bgcolor attribute.
     * 
     * @param s String value
     */
    public void SetBGColor(String s)
    {
        sBGColor = s;
    }

    /**
     * Call this method to obtain the value of the bgcolor attribute.
     * 
     * @return String value
     */
    public String GetBGColor()
    {
        return sBGColor;
    }

    /**
     * Call this method to set the value of the bordercolor attribute.
     * 
     * @param s String value
     */
    public void SetBorderColor(String s)
    {
        sBorderColor = s;
    }

    /**
     * Call this method to obtain the value of the bordercolor attribute.
     * 
     * @return String value
     */
    public String GetBorderColor()
    {
        return sBorderColor;
    }

    /**
     * Call this method to set the value of the valign attribute.
     * 
     * @param s String value
     */
    public void SetVAlign(String s)
    {
        sVAlign = s;
    }

    /**
     * Call this method to obtain the value of the valign attribute.
     * 
     * @return String value
     */
    public String GetVAlign()
    {
        return sVAlign;
    }

    /** This method returns the number of columns spanned by all of the cells
     *  in this row.  An AutoFit cell, if present, is assumed to span 1 column.
     *
     *
     * @return
     * @throws HTMLException
     */
    public int GetNumColsSpanned() 
    {
        Validate();

        return iNumColsSpanned;
    }

    /**    This method should be called whenever a change has been made
     *     to any member HTMLTableCell objects that would cause this 
     *     HTMLTableRow to need to validate.  
     * 
     *     For instance, a change to the colspan attribute of an HTMLTableCell
     *     should call the invalidate() method of that cell's parent HTMLTablRow.
     *     This way, when the HTMLTable is called to display itself, the column
     *     spans will be recalculated.
     * 
     */


    //   package-private access level
    void Invalidate()
    {
//        if (bInfo)
//        {
//            myLogger.Info("Entering invalidate()");
//        }

        bValidated = false;

        if (myParentTable != null)
        {
            myParentTable.Invalidate();
        }

    }

    /**
     * Call this method to validate an HTMLTableCell.
     * 
     * @param c HTMLTableCell
     * @throws HTMLException This method throws an exception if the HTMLTableCell
     * 	passed cannot be added to this row for any reason.
     */
    private void ProcessCellForValidation(HTMLTableCell c) 
    {
    	
        if (c.GetAutoFit())
        {
//            if (bDebug)
//            {
//                myLogger.Debug("validate - found an autofit cell.");
//            }

            if (bHasAutoFitElement)
            {
            	
//            	String s =  "Only one autofit element may exist in each row.  Row at index " + this.GetIndexInTable() + " has multiple autofit cells.";
            	
//                if (bError)
//                {
//                    myLogger.Error( s );
//                }
				
//                throw new HTMLException(s);
            }

            //  set the class member var
            bHasAutoFitElement = true;
        }
//
//        if (bDebug)
//        {
//            myLogger.Debug("validate - c.GetColSpanAsInt(): " + c.GetColSpanAsInt());
//        }

        iNumColsSpanned = iNumColsSpanned + c.GetColSpanAsInt();

    }

    /**    Calling this method ensures that there is only one autofit cell
     *     added to this HTMLTableRow.  Calling this method also calculates
     *     the number of columns spanned by this row.
     * 
     *     It is intended for this method to be called only when this object
     *     is actually displaying.
     * 
     *     Once this method is called, this object will be in a validated state.
     *     Calling this method subsequently will not do anything.  Only after a
     *     call to the invalidate() method will a call to the validate() method
     *     perform any significant action.
     * 
     * 
     */
    void Validate() 
    {
//        if (bInfo)
//        {
//            myLogger.Info("Entering validate()");
//        }

        if (bValidated)
        {
//            if (bDebug)
//            {
//                myLogger.Debug("Already validated, returning.");
//            }

            return;
        }

        IXPEnumerator e = GetInnerHTMLElements(false);

		if( e == null )
		{
			//String s = "This row has no cells.  All rows must have at least one cell.";
			
//			if( bError )
//			{
//				myLogger.Error( s );
//			}
						
//			throw new HTMLException( s );
		}
		else
		{
	
	        HTMLTableCell c;
	        iNumColsSpanned = 0;
	
	        while (e.MoveNext())
	        {
	            c = (HTMLTableCell) e.GetCurrent();
	
	            ProcessCellForValidation(c);
	        }
		}
			
			
        bValidated = true;

    }

    /**    Call this method to add an HTMLTableCell to this HTMLTableRow.
     * 
     *     The HTMLTableCell being added to this row must have been
     *     constructed with a reference to this particular HTMLTableRow.
     *     In this way, a given instance of an HTMLTableCell "belongs" to
     *     a given instance of an HTMLTableRow.
     * 
     * 
     */
    public void AddInnerHTMLElement(HTMLElement e) 
    {
        if (!(e instanceof HTMLTableCell) && !(e instanceof HTMLElementCollection))
        {
            throw new HTMLException("You may only add HTMLCell or HTMLElementCollection elements to an HTMLTableRow.");
        }

        if (e instanceof HTMLElementCollection)
        {
            IXPEnumerator enumItems = e.GetInnerHTMLElements(false);

            if (enumItems != null)
            {
                HTMLElement eTemp;

                while (enumItems.MoveNext())
                {
                    eTemp = (HTMLElement) enumItems.GetCurrent();

//                    if (bDebug)
//                    {
//                        myLogger.Debug("eTemp: " + eTemp);
//                    }

                    if (!(eTemp instanceof HTMLTableCell))
                    {
                        throw new HTMLException("You may only add an HTMLElementCollection containing HTMLTableCell elements to an HTMLTableRow element.  You are attempting to add an: " + eTemp);
                    }

                    ProcessCellForAdding((HTMLTableCell) eTemp);

                } //   for each inner element

            } //   got inner elements

        } // e instanceof HTMLElementCollection
        else
        {
            ProcessCellForAdding((HTMLTableCell) e);

        }

        Invalidate();
    }


    /**
     * Call this method to add an HTMLTableCell to this row
     * 
     * @param c HTMLTableCell
     * @throws HTMLException This method throws an exception if the HTMLTableCell
     * 	passed cannot be added to this row for any reason.
     */
    private void ProcessCellForAdding(HTMLTableCell c) 
    {
        if (c.GetParentRow() == null)
        {
            //   the cell's parent hasn't been set.
            //   set it to this row
            c.SetParentRow(this);

        }
        else
        {
            //   the cell's parent has been set, verify that it's set to
            //   this row.
            if (c.GetParentRow() != this)
            {
                throw new HTMLException("You may only add HTMLTableCell objects constructed with a reference to this instance of an HTMLTableRow.");
            }
        }

        super.AddInnerHTMLElement(c);

    }
}
