package com.plumtree.xpshared.htmlelements;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;


/**
 * <br><br>
 * <a target="new"
 * href="http://www.w3.org/TR/html4/interact/forms.html#h-17.7">Spec</a>
 */
public class HTMLTextArea extends HTMLGenericInputElement
{
    static int iNextID = 0;

    private static final byte[] BEGIN_TAG = HTMLElement.ToByteArray("<textarea");
    private static final byte[] TAB_INDEX_ATTR = HTMLElement.ToByteArray(" tabindex=\"");
    private static final byte[] COLS_ATTR = HTMLElement.ToByteArray(" cols=\"");
    private static final byte[] ROWS_ATTR = HTMLElement.ToByteArray(" rows=\"");
    private static final byte[] READ_ONLY_ATTR = HTMLElement.ToByteArray(" readonly=\"readonly\"");
	private static final byte[] DISABLED_ATTR = HTMLElement.ToByteArray(" disabled=\"disabled\"");
	private static final byte[] END_TAG = HTMLElement.ToByteArray("</textarea>");

    private String sCols;
    private String sRows;
    private String sTabIndex;
    
    private boolean bDisabled= false;
    private boolean bReadOnly = false;

    /**
     *
     * @param cols Pass the cols string of this textarea.
     * @param rows Pass the rows string of this textarea.
     *
     * @throws HTMLException
     */
    public HTMLTextArea(String cols, String rows)
    {
        try
        {
            XPConvert.ToInteger(cols);
			XPConvert.ToInteger(rows);
        }
        catch (Exception nfe)
        {
            throw new HTMLException("You must pass String representations of integers to the HTMLTextArea constructor.");
        }


        //   by default, textarea elements do not format.
        SetDoFormat(false);

        sRows = rows;
        sCols = cols;
        sTabIndex = null;
        
        // rob: removing the validation checks. main reason is to avoid the validation check on textbox for banner search with friendly url
    	// (bug #70204) but it is also not necessary anymore since we are moving to layouts
        SetValidateName(false);
    }

    /**
     * Call this method to get the value of the cols attribute.
     *
     * @return String value
     */
    public String GetCols()
    {
        return sCols;
    }

    /**
     * Get the value of the readonly attribute.
     *
     * @return boolean value
     */
    public boolean GetReadOnly()
    {
        return bReadOnly;
    }

    /**
     * Call this method to get the value of the rows attribute.
     *
     * @return String value
     */
    public String GetRows()
    {
        return sRows;
    }

    /**
     * Set the value of the readonly attribute.
     *
     * @param bInputReadOnly value
     */
    public void SetReadOnly(boolean bInputReadOnly)
    {
        bReadOnly = bInputReadOnly;
    }
    
    
	/**
	 * Set the value of the Disabled attribute.
	 *
	 * @param bInputDisabled value
	 */
    public void SetDisabled(boolean bInputDisabled)
    {
    	bDisabled = bInputDisabled;
    	
    }

    /**
     * This method rewritten to prevent usage.
     *
     * @param s String value
     *
     * @throws HTMLException This method throws an exception.
     */
    public void SetValue(String s)
    {
        throw new HTMLException("The <textarea> element does not support the value attribute, use addInnerHTMLString().");
    }

    /**
     * Call this method to validate the name of this HTMLTextArea.
     *
     * @param s String name.
     *
     * @throws HTMLException This method throws an exception if the name passed
     *         is not valid.
     */
    public void ValidateName(String s)
    {
        if (!s.startsWith("in_ta_"))
        {
            throw new HTMLException("The name of a <textarea> element must begin with \"in_ta_\".  " + s + " is an invalid name.");
        }
    }

    /**
     * HTMLElement.InternalWrite rewritten to prevent formatting.
     *
     * @param bFormat Pass a boolean indicating whether or not to format the
     *        output.
     * @param sb Pass an XPStringBuilder to which you wish the String appended.
     *        Pass null if you do not wish the string appended to an
     *        XPStringBuilder.
     * @param response Pass an IXPResponse to which you wish the String written.
     *        Pass null if you do not wish the string written to an
     *        IXPResponse.
     *
     * @throws HTMLException
     */
    protected void InternalWrite(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        super.InternalWrite(false, sb, response);
    }

    /**
     * This method generates the beginning of the opening tag.  The abstract
     * HTMLElement superclass will provide the remainder of the opening tag,
     * repleate with attributes common to all html elements, such as name, and
     * id, and class.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendBeginningOfOpeningTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(BEGIN_TAG, sb, response);
        if (sTabIndex != null)
        {
            WriteNugget(TAB_INDEX_ATTR, sb, response);
            WriteNugget(sTabIndex, sb, response);
            WriteNugget(QUOTE, sb, response);
        }
        WriteNugget(COLS_ATTR, sb, response);
        WriteNugget(sCols, sb, response);
        WriteNugget(QUOTE, sb, response);
        WriteNugget(ROWS_ATTR, sb, response);
        WriteNugget(sRows, sb, response);
        WriteNugget(QUOTE, sb, response);

        if (bReadOnly)
        {
            WriteNugget(READ_ONLY_ATTR, sb, response);
        }
        
		if (bDisabled)
		{
			WriteNugget(DISABLED_ATTR, sb, response);
		}
    }
    
    /**
     * Returns the tab index.
     * 
     * @return String the tab index.
     */
    public String GetTabIndex()
    {
        return sTabIndex;
    }
    
    /**
     * Sets the tab index.
     */
    public void SetTabIndex(String _sTabIndex)
    {
        sTabIndex = _sTabIndex;
    }
    
    /**
     * Sets sTabIndex to null.
     */
    public void ClearTabIndex()
    {
        sTabIndex = null;
    }

    /**
     * This method generates the closing tag of this html element.  If the
     * providesClosingTag method returns false, then this method will never be
     * called.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendClosingTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(END_TAG, sb, response);
    }

    /**
     * Indicates whether or not this HTML element provides a closing tag. Some
     * elements only provide opening tags, such as the &lt;br&gt; element.
     *
     * @return boolean true.
     */
    protected boolean ProvidesClosingTag()
    {
        return true;
    }

    /**
     * Get the next id value.  This is used by the GetClone method to ensure
     * that cloned object have new ID attribute values.
     *
     * @return A valid id string.
     */
    String GetNextID()
    {
        String s = "textarea" + iNextID;
        iNextID++;

        return s;
    }
}
