package com.plumtree.uiinfrastructure.login;

import com.plumtree.openfoundation.web.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.web.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This interface defines the extensible login helper code.  This is used
 * by the Interpreter and SSO.
 */
public interface ILoginHelper
{
	/**
	 * Attempts to create and connect a new user session.
	 * 
	 * @return a valid userSession
	 */
	public Object AttemptLogin(String sUserName, String sPassword,
		Object token, boolean bUseNumericConnect, ISessionManager session,
		IApplication application, String sRequestURL);

	/**
	 * Attempts to create and connect a new default guest user session.
	 * 
	 * @return a valid guest userSession
	 */
	public Object AttemptGuestLogin(Object token, ISessionManager session,
		IApplication application, String sRequestURL);

	/**
	 * Multiple-guest-user support. Attempts to create a new user session
	 * and connect as the given guest user.
	 * 
	 * @return a valid guest userSession
	 */
	public Object AttemptGuestLogin(int nUserID, Object token, 
		ISessionManager session, IApplication application, String sRequestURL);
		
	/**
	 * Attempts to create and connect a new user session.
	 * Throws an exception if login fails.
	 *
	 * @return a valid user session
	 */
	//ES - add XPRequest
	public Object AttemptLogin(String sLoginToken,
		ISessionManager session, IApplication application, String sRequestURL, IXPRequest request);

	/**
	 * Attempts to create and connect a new user session.
	 * 
	 * This method is used for Basic Auth logins.  Unfortunately, in Basic Auth
	 * the user name and password are encoded in the character encoding of the
	 * client machine, and the server does not know what that is.  Therefore
	 * we have to guess the appropriate encoding.  The user credentials
	 * passed to this method need to be unencoded as win-1252, shift-jis, Big5,
	 * etc... before they can be used.  The credentials are basic auth credentials;
	 * user name and password are separated by a colon.
	 * 
	 * @return a valid userSession
	 */
	public Object AttemptBasicAuthLogin(String sEncodedCredentials);

	/**
	 * This method gets the user ID from the supplied user
	 * session object.
	 * 
	 * @param Object the user session
	 * @return String the user ID (return "" instead of null
	 * 	if necessary)
	 */
	public String GetUserSessionID(Object userSession);

	/**
	 * This method gets the user Name from the supplied user
	 * session object.
	 * 
	 * @param Object the user session
	 * @return String the user Name (return "" instead of null
	 * 	if necessary)
	 */
	public String GetUserSessionName(Object userSession);

	/**
	 * This method returns whether or not the user is a guest user.
	 *
	 * @param Object the user session
	 * @return boolean true implies the user is a guest user.
	 */
	public boolean GetIsGuestUser(Object userSession);
	
	/**
	 * This method gets an error message suitable for display to the user
	 * from the supplied exception (i.e. localized and without any details
	 * that a hacker could exploit, such as whether or not that particular
	 * user name is valid).
	 *
	 * @param Exception the login exception to interpret
	 * @param ISessionManager the current subsession to use
	 * @param IApplication the application
	 * @return String the error message for the user (return "" instead of null
	 * 	if necessary)
	 */
	public String GetDetailedFailureMessage(Exception exception, ISessionManager subSession, IApplication application);

	/**
	 * Call this method to obtain a redirect to the default space.
	 * 
	 * @param ILink user supplied link that will get filled in with the
	 * proper parameters. (Redirect or ASURL object)
	 * @param Object userSession
	 * @param AActivitySpace
	 * @return ILink to the default page for this user.
	 */
	public ILink GetDefaultPageRedirect(ILink suppliedLink, Object userSession,
		AActivitySpace asOwner);
	
	/**
	 * Call this method to obtain a redirect to the default space.
	 * 
	 * @param ILink user supplied link that will get filled in with the
	 * proper parameters. (Redirect or ASURL object)
	 * @param Object userSession
	 * @param IXPRequest request
	 * @return ILink to the default page for this user.
	 */
	public ILink GetDefaultPageRedirect(ILink suppliedLink, Object userSession,
		IXPRequest request);
		
	/**
	 * Call this method to set an autologin token for the user.
	 *
	 * @param oWebData
	 * @param oUserSession
	 * @param nExpires cookie expiration time in seconds.  -1 means use default value.
	 * @param bIsPersistentCookie false means don't set an expires time
	 * (expire at end of session).
	 * 	 * @param bIsSecure whether or not the login cookie is secure
	 */
	public void SetAutologinToken( IWebData oWebData, Object oUserSession,
		int nExpires, boolean bIsPersistentCookie, boolean bIsSecure );
	
	/**
	 * Checks if the current COM process has been recycled - retuns a new session in the new process if it has, otherwise returns null.
	 * 
	 * @param userSession Current user session 
	 * @return An IPTSession created in new COM process with the current user logged in if the current process has been recycled.
	 * null if no recycling happened.
	 */	
	public Object HandleCOMPlusProcessRecycled(Object userSession);

	/**
	 * Returns an HTMLElementCollection of style sheet links.  A style sheet is
	 * named "stylesheetname-language", for instance "mainstyle12-en".
	 *
	 * @param userSession
	 * @param AActivitySpace
	 */
	public HTMLElementCollection GetStyleSheetLinks(Object userSession, AActivitySpace asOwner);
}

